#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include "wonka.h"
#include "oswald.h"

#define swap_short(s) (((s & 0x00FF) << 8) | \
                       ((s & 0xFF00) >> 8))
#define swap_int(i)   (((i & 0x000000FF) << 24) | \
                       ((i & 0x0000FF00) << 8) | \
                       ((i & 0x00FF0000) >> 8) | \
                       ((i & 0xFF000000) >> 24))
 
w_int ulaw2linear(w_word ulaw) {
  static w_int exp_lut[8]={0,132,396,924,1980,4092,8316,16764};
  w_int sign, exponent, mantissa, sample;
  w_ubyte ulawbyte = (w_ubyte)ulaw;
  
  ulawbyte = ~ulawbyte;
  sign = (ulawbyte & 0x80);
  exponent = (ulawbyte >> 4) & 0x07;
  mantissa = ulawbyte & 0x0F;
  sample = exp_lut[exponent] + (mantissa << (exponent+3));
  if (sign != 0) sample = -sample;
  return(sample);
}

w_void play_au(w_ubyte *file_data) {

  w_int i;
  w_int temp;
  char refsig[] = {'.', 's', 'n', 'd'};
  w_int *testsig = (w_int *)&refsig;
  w_int dspfd;
  
  w_int sig;
  w_int header_size;
  w_int sample_data_size;
  w_int audio_encoding;
  w_int sample_rate;
  w_int channels;

  w_short *new_data;
  w_ubyte *sound_data;
  w_word sound_size;

  /*
  ** Get the header of the .au file.
  */

  sig = ((unsigned int *)file_data)[0];
  header_size = swap_int(((unsigned int *)file_data)[1]);
  sample_data_size = swap_int(((unsigned int *)file_data)[2]);
  audio_encoding = swap_int(((unsigned int *)file_data)[3]);
  sample_rate = swap_int(((unsigned int *)file_data)[4]);
  channels = swap_int(((unsigned int *)file_data)[5]);

  sound_size = sample_data_size << 1;

  /*
  ** Check the signature of the data.
  */
  
  if(sig != *testsig) {
    printf("Wrong signature\n");
    return;
  }

  /*
  ** Open the dsp device.
  */

  dspfd = open("/dev/dsp", O_WRONLY, 0);

  /*
  ** Try to set the dsp device to 16 bit signed (little endian)
  */
  
  temp = AFMT_S16_LE;
  if (ioctl(dspfd, SNDCTL_DSP_SETFMT, &temp) == -1) {
    printf("SNDCTL_DSP_SETFMT\n");
    return;
  }
  if (temp != AFMT_S16_LE) {
    printf("not supported format, instead use this:  %d\n", temp);
    return;
  }

  /*
  ** Set the number of channels (1 = mono, 2 = stereo).
  */

  temp = channels;
  if (ioctl(dspfd, SNDCTL_DSP_CHANNELS, &temp) == -1) {
    printf("SNDCTL_DSP_CHANNELS\n");
    return;
  }

  if (channels != temp) {
    printf("Oops, requested number of channels not available\n");
    return;
  }

  /*
  ** Set the sample rate.
  */

  temp = sample_rate;
  if (ioctl(dspfd, SNDCTL_DSP_SPEED, &temp)==-1) {
    printf("SNDCTL_DSP_SPEED\n");
    return;
  }

  /*
  ** Convert the original data from mu-law to 16 bit signed.
  */

  sound_data = &file_data[header_size];
  new_data = x_mem_alloc(sound_size);
  for(i=0; i < sample_data_size; i++) {
    new_data[i] = (w_short)(ulaw2linear((w_word)sound_data[i]));
  }
  
  write(dspfd, new_data, sound_size);
  memset(new_data, 0, sound_size);
  write(dspfd, new_data, sound_size);
  write(dspfd, new_data, sound_size);

  temp = 0;
  if(ioctl(dspfd, SNDCTL_DSP_POST, &temp) == -1) {
    printf("SNDCTL_DSP_POST\n");
  }
  if(ioctl(dspfd, SNDCTL_DSP_SYNC, &temp) == -1) {
    printf("SNDCTL_DSP_SYNC\n");
  }
  
  /*
  ** Release the arrays.
  */

  close(dspfd);

  x_mem_free(new_data);

}

