/* $Id: Font.h,v 1.4 2002/03/18 17:40:50 vandeneede Exp $ */

#ifndef _FONT_H
#define _FONT_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


#include "rudolph.h"

/* 
** Character structure:
*/

typedef struct r_Char {
//  char value;			/* translated name ("A", "B", "\033agrave;") */
  w_char value;			/* translated name ("A", "B", "\033agrave;") */
  char *data;		        /* character definition */
  int width;                    /* character width */
  int height;		        /* height (size of lines array) */
  int advance; 	                /* width with padding on left and right */
  int xoffset;			/* # pixels to move to the right before drawing */
  int yoffset;			/* # pixels to move up (>0) or down (<0) before drawing */
} r_Char;


/*
** Font structure: 
*/

typedef struct r_Font {
  char *name;			/* font name */
  char *family;			/* font family ("Times") */
  char *face_name;		/* face name ("Times Roman") */
  char *width_name;		/* face name ("Normal") */
  int proportional;		/* yes or no? (fixed/proportional font) */
  int style;                    /* 0, 1, 2 or 3: plain, bold, italic, bold-italic */
  int size;                     /* value identifying a font together with its family and style */
  int ascent;                   /* height of a font in pixels above the baseline*/
  int descent;                  /* height of a font in pixels below the baseline*/
  int leading;
  int maxAscent;
  int maxDescent;
  int maxAdvance;               /* maximum actual width of all characters in a font */
//140302
//  r_Char *chars[256];		/* list of chars */
  r_Char *chars[512];		/* list of chars */
} r_Font;

w_void Font_init(void);
w_void Font_readFile(char *name, char *path);
w_void Font_readData(char *name, char **lines);
r_font Font_getFont(char *name, int style, int size);
w_void Font_getSize(r_font font, unsigned int *height_return);
//140302
//w_void Font_getDimensions(r_font font, char *text, int len, int *width_return, int *height_return);
//int Font_getTextWidth(r_font font, char* text, int len);
w_void Font_getDimensions(r_font font, w_string text, int len, int *width_return, int *height_return);
int Font_getCStringWidth(r_font font, char* text, int len);
int Font_getStringWidth(r_font font, w_string text);
int Font_getHeight(r_font);
//140302
//w_void Font_getTextHeight(r_font font, char* text, int len, int* ascent, int* descent);
w_void Font_getStringHeight(r_font font, w_string text, int* ascent, int* descent);
w_void Font_drawStringAligned(r_buffer buffer, r_font font, int x, int y, int w, int h, w_string text, int col, unsigned int alignment);
w_void Font_drawStringUnAligned(r_buffer buffer, r_font font, int charx, int chary, int x, int y, int w, int h, w_string text, int col);
//140302
//w_void Font_drawCharString(r_buffer buffer, r_font font, int charx, int chary, int x, int y, int w, int h, char* charstring, int textlen, int c);
w_void Font_drawString(r_buffer buffer, r_font font, int charx, int chary, int x, int y, int w, int h, w_string string, int c);

//140302
//inline static const r_char Font_getChar(r_font font, int ch) {
//  return font->chars[(unsigned char)ch];
inline static const r_char Font_getChar(r_font font, int ch) {
  return font->chars[(ch > 0x2fff && ch < 0x3100) ? ch-0x2f00 : ch%512];
}

#endif
