/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

package com.acunia.wonka.rudolph;

import java.awt.image.*;
import java.awt.*;

public class Graphics extends java.awt.Graphics {

  private Color foreground = null;
  private Color background = null;
  private Font  font = null;
  
  private int ox = 0;   // offset x
  private int oy = 0;   // offset y
  
  private int cx = 0;   // clip x
  private int cy = 0;   // clip y
  private int cw = -1;  // clip width
  private int ch = -1;  // clip height
  private boolean clip = false;

  public Graphics() {
    cw = -1;
    ch = -1; 
    clip = false;
  }

  public void clearRect(int x, int y, int w, int h) {
    Color temp = foreground;
    foreground = background;
    fillRect(x, y, w, h);
    foreground = temp;
  }
  
  public void clipRect(int x, int y, int w, int h) {
  }
  
  native public void copyArea(int x, int y, int w, int h, int deltax, int deltay);

  public java.awt.Graphics create() {
    return this;
  }
  
  public java.awt.Graphics create(int x, int y, int w, int h) {
    return this;
  }
  
  public void dispose() {
  }

  native public void drawArc(int x, int y, int w, int h, int startAngle, int arcAngle);
  
  public boolean drawImage(java.awt.Image image, int dx1, int dy1, ImageObserver observer) {
    return drawImage(image, dx1, dy1, null, observer);
  }

  public boolean drawImage(java.awt.Image image, int dx1, int dy1, Color bg, ImageObserver observer) {
    int x2 = image.getWidth(null) - 1;
    int y2 = image.getHeight(null) - 1;
    return drawImage(image, dx1, dy1, dx1 + x2, dy1 + y2, 0, 0, x2, y2, bg, observer);
  }

  public boolean drawImage(java.awt.Image image, int dx1, int dy1, int w, int h, ImageObserver observer) {
    return drawImage(image, dx1, dy1, w, h, null, observer);
  }

  public boolean drawImage(java.awt.Image image, int dx1, int dy1, int w, int h, Color bg, ImageObserver observer) {
    if(w < 0 && h >= 0) w = (image.getWidth(null) * h) / image.getHeight(null); 
    else if(h < 0 && w >= 0) h = (image.getHeight(null) * w) / image.getWidth(null); 
    else if(w < 0 && h < 0) {
      w = image.getWidth(null);
      h = image.getHeight(null);
    }
    return drawImage(image, dx1, dy1, dx1 + w-1, dy1 + h-1, 0, 0, image.getWidth(null)-1, image.getHeight(null)-1, bg, observer);
  }

  public boolean drawImage(java.awt.Image image, int dx1, int dy1, int dx2, int dy2, int sx1, int sy1, int sx2, int sy2, ImageObserver observer)  {
    return drawImage(image, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2, null, observer);
  }

  public boolean drawImage(java.awt.Image image, int dx1, int dy1, int dx2, int dy2, int sx1, int sy1, int sx2, int sy2, Color bg, ImageObserver observer) {
    if(image instanceof com.acunia.wonka.rudolph.Image) {
      Image im = (Image)image;
      if(im.producer != null && ((im.flags & ImageObserver.ALLBITS) == 0)) {
        Toolkit.getDefaultToolkit().prepareImage(image, -1, -1, observer);
      }

      /*
      ** Change proportional values to real values.
      */

      if(sx2 < 0 && sy2 < 0) {
        sx2 = image.getWidth(null) - 1;
        sy2 = image.getHeight(null) - 1;
      }
      else if(sx2 < 0 && sy2 >= 0) {
        sx2 = (sy2 * image.getWidth(null)) / image.getHeight(null);
      }
      else if(sy2 < 0 && sx2 >= 0) {
        sy2 = (sx2 * image.getHeight(null)) / image.getWidth(null);
      }

      if(dx2 < 0 && dy2 < 0) {
        dx2 = dx1 + image.getWidth(null) - 1;
        dy2 = dy1 + image.getHeight(null) - 1;
      }
      else if(dx2 < 0 && dy2 >= 0) {
        dx2 = dx1 + ((dy2 - dy1) * image.getWidth(null)) / image.getHeight(null);
      }
      else if(dy2 < 0 && dx2 >= 0) {
        dy2 = dx2 + ((dx2 - dx1) * image.getHeight(null)) / image.getWidth(null);
      }

      if(sx2 > 0 && sy2 > 0) {
        return drawImageBuffer(im, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2, bg);
      }
      else {
        return false;
      }
    } else {
      System.out.println("[com.acunia.wonka.rudolph.Graphics.drawImage()] Unknown Image class");
      return false;
    }
  }

  native private boolean imageHasBuffer(java.awt.Image image);
  native private boolean drawImageBuffer(java.awt.Image image, int dx1, int dy1, int dx2, int dy2, 
                                         int sx1, int sy1, int sx2, int sy2, Color bg);
  native public void drawLine(int x1, int y1, int x2, int y2);
  native public void drawOval(int x, int y, int width, int height);
  native public void drawPolygon(int[] x, int[] y, int n);

  public void drawPolyline(int[] xPoints, int[] yPoints, int nPoints) {
    for(int i=0; i < nPoints - 1; i++) {
      drawLine(xPoints[i], yPoints[i], xPoints[i + 1], yPoints[i + 1]);
    }
  }
  
  native public void drawRoundRect(int x, int y, int width, int height, int arcWidth, int arcHeight);
  native public void drawString(String string, int x, int y);
  native public void fillArc(int x, int y, int width, int height, int startAngle, int arcAngle);
    
  public void fillOval(int x, int y, int w, int h) {
    System.out.println("[Graphics] fillOval not implemented");
  }
  
  native public void fillPolygon(int[] x, int[] y, int n);
  native public void fillRect(int x, int y, int w, int h);
  native public void fillRoundRect(int x, int y, int w, int h, int arcWidth, int arcHeight);
  
  public Shape getClip() {
    return (clip ? (Shape)(new Rectangle(cx, cy, cw, ch)) : null);
  }
  
  public Rectangle getClipBounds() {
    return (clip ? new Rectangle(cx, cy, cw, ch) : null);
  }
  
  public Rectangle getClipRect() {
    return (clip ? new Rectangle(cx, cy, cw, ch) : null);
  }

  public Color getColor() {
    return foreground;
  }
  
  public Font getFont() {
     return font;
  }

  public FontMetrics getFontMetrics() {
    return getFontMetrics(font);
  }
  
  public FontMetrics getFontMetrics(Font font) {
    return new FontMetrics((font == null) ? Component.DEFAULT_FONT : font);
  }
  
  public void setClip(int x, int y, int w, int h) {
    cx = x;
    cy = y;
    cw = w;
    ch = h;
    clip = true;
  }
  
  public void setClip(Shape shape) {
    Rectangle r = shape.getBounds();
    setClip(r.x, r.y, r.width, r.height);
  }
  
  public void setColor(Color color) {
    if (color != null) {
      this.foreground = color;
    }
  }
  
  public void setFont(Font font) {
    if (font != null) {
      this.font = font;
    }
  }
  
  public void setPaintMode() {
    System.out.println("[Graphics] setPaintMode not implemented");
  }
  
  public void setXORMode(Color color) {
    System.out.println("[Graphics] setXORMode not implemented");
  }
  
  public void translate(int x, int y) {
    ox += x;
    oy += y;
  }

}

