/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


package com.acunia.wonka.rudolph;

import java.awt.*;
import java.awt.image.*;
import java.util.*;

public class Image extends java.awt.Image {

  private String filename = null;
  protected ImageProducer producer = null;
  private Hashtable props;
  private char[] pixels;
  protected ImageConsumer consumer = null;
  protected int flags;
  protected int width = -1;
  protected int height = -1;
  private int key = 0;

  private static native int convertRGB(int red, int green, int blue);
  private native void setPixel(int x, int y, int scanline, int pix);
  private native void do_conversion(byte[] src, char[] dst, IndexColorModel model, int x, int y, int w, int h, int offset, int scansize);
  private native void do_conversion(int[] src, char[] dst, DirectColorModel model, int x, int y, int w, int h, int offset, int scansize);

  protected class Consumer implements ImageConsumer {

    protected Consumer() { }
    
    public void setColorModel(ColorModel model) { }
    public void setHints(int hints) { }
    
    public void imageComplete(int status) { 
      if(status == STATICIMAGEDONE) {
        createImage();
        pixels = null;
        flags |= ImageObserver.ALLBITS; 
      }
    }
    
    public void setProperties(Hashtable properties) { 
      flags |= ImageObserver.PROPERTIES;
      props = properties;
      pixels = new char[width * height];
    }

    public void setDimensions(int w, int h) { 
      width = w;
      height = h;
      flags |= ImageObserver.WIDTH | ImageObserver.HEIGHT;
    }

    public void setPixels(int x, int y, int w, int h, ColorModel model, byte[] pix, int offset, int scansize) {
      if(model instanceof IndexColorModel) {
        do_conversion(pix, pixels, (IndexColorModel)model, x, y, w, h, offset, scansize);
      }
      else {
        for(int j=0; j<h; j++) {
          for(int i=0; i<w; i++) {
            byte pixel = pix[j * scansize + i + offset];
            setPixel(i, j, width, convertRGB(model.getRed(pixel), model.getGreen(pixel), model.getBlue(pixel)));
            /*pixels[(j + y) * width + (i + x)] = convertRGB(model.getRed(pixel), model.getGreen(pixel), model.getBlue(pixel));*/
          }
        }
      }
      pix = null;
    }

    public void setPixels(int x, int y, int w, int h, ColorModel model, int[] pix, int offset, int scansize) {
      if(model instanceof DirectColorModel) {
        do_conversion(pix, pixels, (DirectColorModel)model, x, y, w, h, offset, scansize);
      } 
      else {
        for(int j=0; j<h; j++) {
          for(int i=0; i<w; i++) {
            int pixel = pix[j * scansize + i + offset];
            setPixel(i, j, width, convertRGB(model.getRed(pixel), model.getGreen(pixel), model.getBlue(pixel)));
            /*pixels[(j + y) * width + (i + x)] = convertRGB(model.getRed(pixel), model.getGreen(pixel), model.getBlue(pixel));*/
          }
        }
      }
      pix = null;
    }
  }

  public Image(ImageProducer producer) {
    this.consumer = new Consumer();
    this.producer = producer;
    producer.startProduction(consumer);
  }

  native private void createImage();
  native private int getBufferWidth();
  native private int getBufferHeight();

  public int getWidth(ImageObserver observer) {
    if(producer == null) {
      width = getBufferWidth();
      flags |= ImageObserver.WIDTH;
    }
    if(observer != null) {
      observer.imageUpdate(this, flags, 0, 0, width, height);
    }
    return width;
  }
  
  public int getHeight(ImageObserver observer) {
    if(producer == null) {
      height = getBufferHeight();
      flags |= ImageObserver.HEIGHT;
    }
    if(observer != null) {
      observer.imageUpdate(this, flags, 0, 0, width, height);
    }
    return height;
  }
  
  native public java.awt.Graphics getGraphics();
  
  public Object getProperty(String name, ImageObserver observer) {
    Object result;
    if(observer != null) {
      observer.imageUpdate(this, flags, 0, 0, width, height);
    }
    if(props != null) {
      result = props.get(name);
      if(result == null) result = UndefinedProperty;
    } else {
      result = UndefinedProperty;
    }
    return result;
  }
  
  public ImageProducer getSource() {
    return (producer != null ? producer : new BufferImageSource(this));
    // return producer;
  }
  
  public void flush() {
  }

  void setKey(int key) {
    this.key = key;
  }

  protected void finalize() {
    if(key != 0) {
      com.acunia.wonka.rudolph.Toolkit.t.imageCache.remove(key);
    }
    finalize0();
  }

  private native void finalize0();

}

