/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

package com.acunia.wonka.rudolph;

import java.awt.image.*;
import java.awt.*;
import java.net.URL;
import java.io.*;
import java.util.*;
import java.util.zip.*;

public class Toolkit extends java.awt.Toolkit {

  protected static Toolkit t;

  protected ImageCache imageCache = new ImageCache();

  public static Toolkit getInstance() {
    if(t == null) {
      t = new Toolkit();
    }
    return t;
  }

  public Toolkit() {
  }

  public void beep() {
    /* BEEP BEEP */
  }

  /**
   * @status  not implemented
   * @remark  not implemented
   */ 
  public int checkImage(java.awt.Image image, int w, int h, ImageObserver observer) {
    if(image instanceof com.acunia.wonka.rudolph.Image) {
      if(w != -1 || h != -1) {
        System.out.println("[com.acunia.wonka.rudolph.Toolkit.checkImage()] Rescaling image not supported");
        return 0;
      }
      return ((Image)image).flags;
    } else {
      return 0;
    }
  }

  /**
   * @status  implemented
   * @remark  implemented
   */ 
  public java.awt.Image createImage(ImageProducer producer) {
    com.acunia.wonka.rudolph.Image image = new com.acunia.wonka.rudolph.Image(producer);
    return (java.awt.Image)image;
  }

  /**
   * @status  implemented
   * @remark  only works for Portable Network Graphics (PNG) images and GIF, not for JPEG.
   */ 
  public synchronized java.awt.Image createImage(byte[] imageData) {
    
    CRC32 crc = new CRC32();
    com.acunia.wonka.rudolph.Image image = null;
    
    crc.update(imageData);
    image = (com.acunia.wonka.rudolph.Image)imageCache.get((int)crc.getValue());

    if(image == null) {

      /*
      ** Image is not yet in the cache.
      */
      
      if(imageData[1] == 'P' && imageData[2] == 'N' && imageData[3] == 'G') {
        PNGImageSource png = new PNGImageSource(imageData);
        image = new com.acunia.wonka.rudolph.Image((ImageProducer)png);
      }
      else if(imageData[0] == 'G' && imageData[1] == 'I' && imageData[2] == 'F') {
        GIFImageSource gif = new GIFImageSource(imageData);
        image = new com.acunia.wonka.rudolph.Image((ImageProducer)gif);
      }
      
      if(image != null) {
        imageCache.put((int)crc.getValue(), image);
        image.setKey((int)crc.getValue());
      }
    }
    
    return (java.awt.Image)image;
  }

  /**
   * @status  implemented
   * @remark  implemented
   */ 
  public java.awt.Image createImage(byte[] imageData, int offset, int count) {
    byte[] data = new byte[count];
    for(int i=0; i < count; i++) {
      data[i] = imageData[offset + i];
    }
    return createImage(data);
  }

  public java.awt.Image createImage(java.net.URL url) {
    if (url == null) {
      return null;
    }
    ByteArrayOutputStream bas = new ByteArrayOutputStream(2048);	
    try {
      InputStream in = url.openStream();
      byte[] bytes = new byte[1024];
      int len = in.read(bytes, 0, 1024);
      while (len != -1) {
        bas.write(bytes,0,len);
        len = in.read(bytes, 0, 1024);
      }
    }	
    catch (IOException e) {
      return null;
    }
    return createImage(bas.toByteArray());
  }

  public java.awt.Image createImage(String filename) {
    if (filename == null) {
      return null;
    }
    ByteArrayOutputStream bas = new ByteArrayOutputStream(2048);	
    try {
      InputStream in = new FileInputStream(filename);
      byte[] bytes = new byte[1024];
      int len = in.read(bytes, 0, 1024);
      while (len != -1) {
        bas.write(bytes,0,len);
        len = in.read(bytes, 0, 1024);
      }
    }	
    catch (IOException e) {
      return null;
    }
    return createImage(bas.toByteArray());
  }

  public java.awt.Image getImage(URL url) {
    return createImage(url);
  }
  
  /**
   * @status  implemented
   * @remark  implemented
   */
  public ColorModel getColorModel() {
    return new DirectColorModel(32, 0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000);
  }
  
  /**
   * @status  not implemented
   * @remark  not implemented
   */
  public String[] getFontList() {
    return null;
  }
  
  /**
   * @status  not implemented
   * @remark  not implemented
   */
  public FontMetrics getFontMetrics(Font font) {
    return new FontMetrics((font == null) ? Component.DEFAULT_FONT : font);
  }

  /**
   * @status  partly implemented
   * @remark  only works for Portable Network Graphics (PNG) images and not for JPEG or GIF.
   */  
  public java.awt.Image getImage(String filename) {
    try {
      RandomAccessFile raf = new RandomAccessFile(filename, "r");
      byte[] ba = new byte[(int)raf.length()];
      raf.readFully(ba);
      raf.close();
      return Toolkit.getDefaultToolkit().createImage(ba);
    }
    catch (IOException e) {
      return null;       
    }
  }

  /**
   * @status  not implemented
   * @remark  not implemented
   */  
  public int getMenuShortcutKeyMask() {
    return 0;
  }

  /**
   * @status  implemented
   * @remark  always returns 72 dpi.
   */
  public int getScreenResolution() {
    return 72;
  }
  
  private native int getScreenWidth();
  private native int getScreenHeight();

  /**
   * @status  implemented
   * @remark  implemented
   */
  public Dimension getScreenSize() {
    return new Dimension(getScreenWidth(), getScreenHeight());
  }
  
  /**
   * @status  not implemented
   * @remark  not implemented
   */  
  protected EventQueue getSystemEventQueueImpl() {
    return null;
  }

  /**
   * @status  implemented
   * @remark  not yet according to specs. The observer isn't used yet.
   */  
  public boolean prepareImage(java.awt.Image image, int w, int h, ImageObserver observer) {
    if(image instanceof com.acunia.wonka.rudolph.Image) {
      Image im = (com.acunia.wonka.rudolph.Image)image;  
      im.getSource().startProduction(im.consumer);
      if(w != -1 || h != -1) {
        System.out.println("[com.acunia.wonka.rudolph.Toolkit.prepareImage()] Rescaling image not supported");
        return false;
      }
      return true;
    } else {
      System.out.println("[com.acunia.wonka.rudolph.Toolkit.prepareImage()] Unknown Image class");
      return false;
    }
  }

  /**
   * @status  not implemented
   * @remark  not implemented
   */  
  public void sync() {
  }
  
  public final native EventQueue getSystemEventQueue();

}

