/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


package java.awt;

import java.awt.event.*;

public class Button extends Component {
  
  String label;
  String command;
 
  public Button() {
    this.label = "";
    this.command = null;
    createPeer();
  }
  
  public Button(String label) {
    this.label = (label != null) ? label : "";
    this.command = null;
    createPeer();
  }

  native private void createPeer();

  public Dimension getPreferredSize () {
    synchronized(getTreeLock()) {    
      if (preferredSize == null || valid == false) {
        Font f = getFont();
        FontMetrics fm = getFontMetrics((f != null) ? f : Component.DEFAULT_FONT);
      
        int cx = Math.max(30, 4 * fm.stringWidth(label) / 3);
        int cy = Math.max(20, 2 * fm.getHeight());
        
        preferredSize = new Dimension(cx, cy);
      }
      return preferredSize;
    }
  }
 
  /**
   * Action listener functions
   */
  /**
   * Assign the new listener or return an AWTEventMulticaster 
   * listener that contains it
   */
  public synchronized void addActionListener(java.awt.event.ActionListener newlistener) {
    actionListener = AWTEventMulticaster.add(actionListener, newlistener);
  }
  /**
   * Remove the old listener or return an AWTEventMulticaster
   * listener where it is removed
   */
  public synchronized void removeActionListener(java.awt.event.ActionListener oldlistener) {
    actionListener = AWTEventMulticaster.remove(actionListener, oldlistener);
  }

  /**
   * If overwrite processEvent so that a super or subclass requesting
   * to throw an ActionEvent throws it through the multicaster to all
   * subscribed listeners
   */
  protected void processEvent(AWTEvent event) {
    if (event instanceof ActionEvent) {
      processActionEvent(new ActionEvent(event.getSource(), event.getID(), (command == null) ? label : command));
    }
    else super.processEvent(event);
  }

  /**
   * Throw a specific action event
   */
  protected void processActionEvent(ActionEvent event) {
    if (actionListener != null) {
      actionListener.actionPerformed(event);
    }
  } 
  
  public void addNotify() {
    if (notified == false) {
      // createCheckboxPeer();
      super.addNotify();
    }
  }

  public String getActionCommand() {
    return (command == null) ? label : command;
  }

  public void setActionCommand(String command) {
    this.command = command;
  }
  
  public synchronized void setLabel(String label) {
    this.label = (label != null) ? label : "";

    if (valid) invalidate();

    refresh(Component.REFRESH_LOCAL, Component.CACHE_NONE);
  }

  public String getLabel() {
    return label;
  }

  public String toString() {
    return getClass().getName() +" - label: "+ label +", bounds: x = "+ x +", y = "+ y +", w = "+ width +", h = "+ height;
  }

  /**
   * @status  not implemented
   * @remark  not implemented
   */
  protected String paramString() {
    return getClass().getName() +" label: "+ label+" ActionCommand: "+ command +", bounds: ("+x +", "+ y +", "+ width +", "+ height+")";
  }



}
