/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


package java.awt;

import java.awt.event.*;

public class Checkbox extends Component implements java.io.Serializable, ItemSelectable {

  String label;
  CheckboxGroup group;
  boolean state;
  ItemListener itemListener;
  
  private boolean isGroup;
 
  public Checkbox() {
    this("", false, null);
  }
  
  public Checkbox(String label) {
    this(label, false, null);
  }

  public Checkbox(String label, boolean state) {
    this(label, state, null);
  }

  public Checkbox(String label, CheckboxGroup group, boolean state) {
    this(label, state, group);
  }

  public Checkbox(String label, boolean state, CheckboxGroup group) {
    this.label = label;
    this.state = state;
    this.group = group;

    if (group != null) {
      if (state == true || group.getSelectedCheckbox() == null) {
        group.setSelectedCheckbox(this);
      }

      isGroup = true;
    }
    else isGroup = false;

    createPeer();
  }

  native private void createPeer();

  native private void update();

  public Dimension getPreferredSize () {
    synchronized(getTreeLock()) {
      if (preferredSize == null || valid == false) { 
        FontMetrics fm = getFontMetrics((font != null) ? font : Component.DEFAULT_FONT);
        int cx = Math.max(60, 30 + fm.stringWidth(label));
        int cy = Math.max(20, 2 * fm.getHeight());
        preferredSize = new Dimension(cx, cy);
      }  
      return preferredSize;
    }
  }
    
  public synchronized void addItemListener(ItemListener listener) {
    itemListener = AWTEventMulticaster.add(itemListener, listener);
  }

  public synchronized void removeItemListener(ItemListener listener) {
    itemListener = AWTEventMulticaster.remove(itemListener, listener);
  }

  protected void processEvent(AWTEvent event) {
    if (event instanceof ItemEvent) {
      processItemEvent((ItemEvent) event);
    }
  }

  protected void processItemEvent(ItemEvent event) {
    if (this.itemListener != null) {
      itemListener.itemStateChanged(event);
    }
  }

  public void addNotify() {
    if (notified == false) {
      // createPeer()
      super.addNotify();
    }
  }

  public void setCheckboxGroup(CheckboxGroup group) {
    if (group != null) {
      this.group = group;
      this.isGroup = true;
    }
    else {
      this.group = null;
      this.isGroup = false;
    }

    if (valid) {
      invalidate();
    }

    refresh(Component.REFRESH_LOCAL, Component.CACHE_NONE);
  }

  public CheckboxGroup getCheckboxGroup() {
    return group;
  }

  public void setLabel(String label) {

    if (this.label != label) {
      this.label = label;

      if (valid) {
        invalidate();
      }

      refresh(Component.REFRESH_LOCAL, Component.CACHE_NONE);
    }

  }

  public String getLabel() {
    return label;
  }

  public void setState(boolean state) {
  
    CheckboxGroup group = this.group;
  
    if (group != null) {
      if (state == false) {
        if (group.getSelectedCheckbox() == this) {
          state = true;
        }
      }
      else {
        if (group.getSelectedCheckbox() != this) {
          group.setSelectedCheckbox(this);
        }
      }
    }

    this.state = state;

    refresh(Component.REFRESH_LOCAL, Component.CACHE_NONE);
  }

  public boolean getState() {
    return state;
  }

  public Object[] getSelectedObjects() {
    Object[] objects;

    if (state == true) {
      objects = new Object[1];
      objects[0] = label;
    }
    else {
      objects = null; 
    }

    return objects;
  }

  public String toString() {
    return getClass().getName() +" - label: "+ label +", bounds: x = "+ x +", y = "+ y +", w = "+ width +", h = "+ height;
  }

  protected String paramString(){
    return getClass().getName() +" label: "+ label +", bounds:("+ x +", "+ y +", "+ width +", "+ height+") group:"+group;
  }
}
