/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


package java.awt;

import java.util.*;
import java.io.*;
import javax.comm.*;
import com.acunia.wonka.*;

public class EventQueue {
  /******************************/
  /**
   * variables
   */

  private AWTEvent nativeAWTEvent;
  private Vector queue = new Vector();
  private EventDispatchThread dispatchThread;

  /*
   * We maintain one event queue; in future we might want to
   * use multiple priority queues along with an appropriate
   * scheduler.
   */
  static private SerialPort screen;
  static private InputStream screenInput;
  static private OutputStream screenOutput;

  /******************************/
  /**
   * Constructor
   * @status  Complient with java specs
   * @remark  Complient with java specs
   */
  public EventQueue() {
    String arch = System.getProperty("os.arch");

    if (arch.startsWith("OBIWAN")) {

      // Initialize touchscreen:
      String pointingDevicePort = System.getProperty("com.acunia.wonka.pointing-device.port");

      if (pointingDevicePort==null) {
        System.err.println("Could not find system property com.acunia.wonka.pointing-device.port!");
      }
      else {
       try {
          screen = (SerialPort)CommPortIdentifier.getPortIdentifier(pointingDevicePort).open("Touchscreen",1000);
          screenInput = screen.getInputStream();
          screenOutput = screen.getOutputStream();
        
          sendCommand("PN812");
          getResponse();
          sendCommand("FT");
          getResponse();
          sendCommand("PL");
          getResponse();

          giveInputStreamToGrx(screenInput);

        } catch (Exception e) {
          System.err.println("Unable to initialise touchscreen - got "+e.getClass().getName()+" "+e.getMessage());
        }
      }
    }

    // dispatchThread = new EventDispatchThread(this);
  }

  private void sendCommand(String c) {
    StringBuffer sb = new StringBuffer ();
    sb.append ( '\u0001'); // ctrl-A
    sb.append ( c);
    sb.append ( '\r'); // CR
    byte[] combuf = sb.toString().getBytes();
    try {
      screenOutput.write(combuf);
    } catch (Exception e) {
      System.err.println("sendCommand got exception "+e.getClass().getName()+e.getMessage());
    }
  }

  private String getResponse() {
    byte [] respbuf = new byte[32];
    int onebyte;
    int i = 0;

    try {
      onebyte = screenInput.read(); // skip over ctrl-A

      while (i<32) {
        onebyte = screenInput.read();
        if (onebyte<0 || onebyte==13) break; // exit on CR
        respbuf[i++] = (byte)onebyte;
      }
    } catch (IOException e) {
      System.err.println("IOException: "+e.getMessage());
    }

    return new String(respbuf,0,0,i);
  }

  protected void dispatchEvent(AWTEvent event) {
    Object source = event.getSource();
    if (source instanceof Component) {
      ((Component)source).dispatchEvent(event);
    } 
  }

  private native void giveInputStreamToGrx(InputStream s);

  /******************************/
  /**
   * scan events from queue
   * @status  Complient with java specs
   * @remark  Complient with java specs
   */
  public synchronized AWTEvent getNextEvent() {
    if (queue.isEmpty()) return null;
    else {
      AWTEvent event = (AWTEvent) queue.elementAt(queue.size() - 1);
      queue.removeElementAt(queue.size() - 1);
      return event;
    }
  }
  
  public synchronized AWTEvent peekEvent() {
    if (queue.isEmpty()) return null;
    else return (AWTEvent) queue.elementAt(queue.size() - 1);
  }
  
  public synchronized AWTEvent peekEvent(int id) {
    System.out.println("EventQueue.peekEvent(id): TODO!");
    return null;
  }

  /******************************/
  /**
   * post events
   * @status  Complient with java specs
   * @remark  Complient with java specs
   */
  public void postNativeEvent() {
    try {
      if (nativeAWTEvent != null) {
        postEvent(nativeAWTEvent);
        nativeAWTEvent = null;
      }
    }
    catch (Throwable t) {
      t.printStackTrace();
    }
  }
  
  public synchronized void postEvent(AWTEvent event) {
    dispatchEvent(event);  // temporary shortcut
    // queue.addElement(event);
    // dispatchThread.doCheck();
  }
}
