/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


package java.awt;

import java.awt.image.*;

public abstract class Image {

  static {
    Component.initRudolph();
  }

  public static final int SCALE_DEFAULT = 1;
  public static final int SCALE_FAST = 2;
  public static final int SCALE_SMOOTH = 4;
  public static final int SCALE_REPLICATE = 8;
  public static final int SCALE_AREA_AVERAGING = 16;

  /**
   * @status implemented
   * @remark compliant with specifications
   */
  public static final Object UndefinedProperty = new Object();

  /**
   * @status not implemented
   * @remark not implemented
   */
  public abstract void flush();

  /**
   * @status implemented
   * @remark not compliant with specification; instead of throwing a 'ClassCastException' a null pointer 
   * is returned if the image is not an off-screen image.
   */
  public abstract Graphics getGraphics();
  
  /**
   * @status implemented
   * @remark compliant with specifications
   */
  public abstract int getHeight(ImageObserver observer);

  /**
   * @status not implemented
   * @remark not implemented
   */
  public abstract ImageProducer getSource();

  /**
   * @status not implemented
   * @remark not implemented
   */
  public abstract Object getProperty(String name, ImageObserver observer);

  /**
   * @status dummy implementation
   * @remark returns null Image
   */
  public Image getScaledInstance(int width, int height, int hints) {
    return Toolkit.getDefaultToolkit().createImage(new FilteredImageSource(this.getSource(), new ReplicateScaleFilter(width, height)));
  }

  /**
   * @status implemented
   * @remark compliant with the specifications
   */
  public abstract int getWidth(ImageObserver observer);

}
