/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/



package java.awt;

public class Label extends Component {
  // Local variables:
  String text;
  int alignment = LEFT;

  // Alignment types:
  public static final int LEFT = 0;
  public static final int CENTER = 1;
  public static final int RIGHT = 2;

  native private void createPeer();

  public Label() {
    this("", LEFT);
  }
  
  public Label(String label) {
    this(label, LEFT);
  }
  
  public Label(String text, int alignment) {
    this.text = text;

    switch (alignment) {
      case LEFT: 
        this.alignment = LEFT;
        break;
      case CENTER:
        this.alignment = CENTER;
        break;
      case RIGHT:
        this.alignment = RIGHT;
        break;
      default:
         throw new IllegalArgumentException("invalid alignment: " + alignment);
    }

    createPeer();
  }

  public Dimension getPreferredSize() {
    synchronized(getTreeLock()) {
      if (preferredSize == null || valid == false) { 
        Font f = getFont();
        FontMetrics fm = getFontMetrics((f != null) ? f : Component.DEFAULT_FONT);
        int cx = Math.max(30, fm.stringWidth(text));
        int cy = Math.max(20, 3 * fm.getHeight() / 2);                            
        preferredSize = new Dimension(cx, cy);
      }
      return preferredSize;
    }
  }

  public void addNotify() {
    if (notified == false) {
      // createLabelPeer();
      super.addNotify();
    }
  }
  
  public int getAlignment() {
    return this.alignment;
  }
  
  public String getText() {
    return this.text;
  }
  
  public synchronized void setAlignment(int alignment) {
    switch (alignment) {
      case LEFT: 
        this.alignment = LEFT;
        break;
      case CENTER:
        this.alignment = CENTER;
        break;
      case RIGHT:
        this.alignment = RIGHT;
        break;
      default:
        throw new IllegalArgumentException("improper alignment: " + alignment);
    }
    
    if (valid) {
      invalidate();
    }

    refresh(Component.REFRESH_LOCAL, Component.CACHE_NONE);
  }
  
  public synchronized void setText(String text) {
    this.text = text;
    
    if (valid) {
      invalidate();
    }

    refresh(Component.REFRESH_LOCAL, Component.CACHE_NONE);
  }

  public String toString() {
    return getClass().getName() +" - text: "+ text +", bounds: x = "+ x +", y = "+ y +", w = "+ width +", h = "+ height;
  }

  protected String paramString() {
    return getClass().getName() +" text=["+ text +"] alignment="+alignment+", bounds:("+ x +", "+ y +", "+ width +", "+ height+")";
  }
}

