/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


// Author: n.oberfeld
// version: 1.0.00 2001/06
// => Full implementation using Strings
// version: 2.0.00 2001/08
// => Textbuffer instead of Strings
// => free selection of Fonts
// selective redraw and double buffering
// version: 2.00.01  2001/08/19
// =>cleanup code and removing functions no-longer needed
// version: 2.01.01  2001/10/01
// =>One continuous thread, simplified run, scrollbox dragging handled by thread just as well

package java.awt;

import java.awt.event.*;//listeners
import java.util.EventListener.*;  		// Listener definitions
import java.awt.peer.*; //RudolphPeer drawing algorithms

/**
@status: List is still a subclass of Panel instead of Component, as we found that the Component-version still crashes at initialisation
*/
public class List extends Panel implements MouseListener, MouseMotionListener, ItemSelectable {
  /****************************************************************************************************************************************/
  /**
  * Variables :
  * (variables that are declared protected are also used in the derived Choice's DropList)
  */

  /** minimum and preferred sizes */
  private int minimumLines;

  /** viewports & sizes */
  protected Dimension screenViewport; /* the component rectangle in which to display the complete list component */
  private Dimension listViewport;   /* the inner rectangle, without the scrollbars in which to display the list elements */

  /** text, start of textlines, start&length of visible parts  */
  private char[] textBuffer;
  private int[] textlineEnd;
  private int[] visibleLineStart;
  private int[] visibleLineLength;

  /** list selections  */
  private boolean[] itemSelected;
  private int currentSelected;
  private int lastSelected;
  private boolean multiple;


  /** visibility & offset vertically and horizontally */
  private int lineOffset;      //vertical scrollbar barPos
  private int viewportLabels;   //vertical scrollbar barSpan
  //(vertical scrollbar barRange = textlineEnd.length )

  private int charOffset;      //horizontal scrollbar barPos
  //(horizontal scrollbar barSpan = listViewport.width)
  private int maximumLineWidth;//horizontal scrollbar barRange

  /** colors & font */
  private Color[] listColors;
  private Font painterFont;
  protected FontMetrics painterMetrics;
  private int painterFixedWidth; // = fm.getMaxAdvance for fixed width characters, -1 for variable width

  /** scrollbars */
  private HListScrollPainter hScroll;
  private boolean	hScrollVisible;
  private VListScrollPainter vScroll;
  private boolean	vScrollVisible;
  /** scroll thread */
  final static ScrollRunner mouseEventThread = new ScrollRunner();

  //mouse and border scrolling */
  private Point lastMousePosition; // detecting mouse movements
  private Point lastMouseMoved;    // total of <unresolved> mouse movements to be followed by the scrollbox

  /** painting buffer */
  private Image backgroundImage;
  private Graphics backgroundGraphics;
  /** selective painting flags */
  private boolean repaintListArea;
  private boolean repaintHScroll;
  private boolean repaintVScroll;
  private boolean repaintBottomRight;
  private boolean repaintAll;

  /** listeners */
  public transient ActionListener aMultiListener;
  public transient ItemListener iMultiListener;

  /****************************************************************************************************************************************/
  /**
  * General List Constructor	
  *	@status implemented
  *	@remark
  */
  public List(int rows, boolean allowmultipleselections)
  {
    //sizes
    minimumLines = rows;
    maximumLineWidth = 0;
      	
    screenViewport = new Dimension();
    listViewport = new Dimension();
    viewportLabels=0;
      	
    //texts and selections
    textBuffer = null;
    textlineEnd = null;
    visibleLineStart = null;
    visibleLineLength = null;
      	
    //selections
    multiple = allowmultipleselections;
    currentSelected = -1;
    lastSelected = -1;
    itemSelected = null;//always, as well as for multiple selections as for one-line
      	
    // default colors, font
    listColors = RudolphPeer.getBarColors();

    painterFont = Component.DEFAULT_FONT; // previously RudolphListPeer.DEFAULT_FONT;
    painterMetrics = new FontMetrics(painterFont);
    painterFixedWidth = painterMetrics.getMaxAdvance();
    if(painterFixedWidth>painterMetrics.charWidth('i') ) {
      // the 'i' (smallest character) is smaller then the biggest character  => variable width
      painterFixedWidth=-1;
    }
    super.setFont(painterFont);
      	
    //scrollbars
    hScroll = new HListScrollPainter();
    hScrollVisible = false;
    vScroll = new VListScrollPainter();
    vScrollVisible = false;
    // hscroll advances just enough to ALWAYS shift one character
    hScroll.setLineStep(painterMetrics.getMaxAdvance());
    vScroll.setLineStep(1);
    //scrollbar colors
    hScroll.setBarColors(listColors);
    vScroll.setBarColors(listColors);
      	
    //listeners
    iMultiListener = null;
    aMultiListener = null;
    addMouseListener(this);
    addMouseMotionListener(this);
    //Runnable mouse click & mouse position
    lastMousePosition = new Point(0,0);
    lastMouseMoved = new Point(0,0);

    // painting buffer
    backgroundImage = null;
    backgroundGraphics = null;
    //no repainting untill screen calculations and screen buffer
    repaintListArea = false;
    repaintHScroll = false;
    repaintVScroll = false;
    repaintAll = false;
      	
    //System.out.println("Initialised List rows: "+minimumLines+" multiple selections: "+multiple);
  }

  /****************************************************************************************************************************************/
  /** List constructor variants : by default selection = single */
  public List(int rows) {
    this(rows, false);
  }
  /****************************************************************************************************************************************/
  /** List constructor variants : by default  minimum 4 rows and selection = single
  * @remark : compliant again with the java 1.2 specs again
  */
  public List() {
    this(4, false);
  }

  /****************************************************************************************************************************************/
  /**
  * Get the desired minimum rows/lines/visible elements (This is NOT the number of elements currently visible,
  * but the one we should strife for when calculating the lists dimensions
  */
  public int getRows() {
    return minimumLines;
  }
  /****************************************************************************************************************************************/
  /**
  * Get the minimum size: enough to display the maximum width, the DESIRED number of lines and a vertical scrollbar
  */
  public Dimension getMinimumSize() {
    return new Dimension( maximumLineWidth+ vScroll.getMinimumThickness(),RudolphListPeer.getHeight(minimumLines, painterMetrics) );
  }
  /****************************************************************************************************************************************/
  /**
  * Get the preferred size: enough to display the maximum width, the DESIRED number of lines and a vertical scrollbar
  * @note with no specific system List window present, this is equal to getMinimumSize()
  */
  public Dimension getPreferredSize()	{
    return new Dimension( maximumLineWidth+ vScroll.getMinimumThickness(),RudolphListPeer.getHeight(minimumLines, painterMetrics) );
  }

  /****************************************************************************************************************************************/
  /**
  * Overrides the Components functions to pass the colors to the painter
  */
  public void setBackground(Color cb) {
    super.setBackground(cb); // component
    listColors = RudolphPeer.getBarColors(cb,getForeground() );
    // copy new colors to scrollbars
    hScroll.setBarColors(listColors);
    vScroll.setBarColors(listColors);
    // repaint all in new colors
    repaintAll = true;
    repaint();
  }
  	
  public void setForeground(Color cf) {
    super.setForeground(cf); // component
    listColors[4] = cf;
    // copy new colors to scrollbars
    hScroll.setBarColors(listColors);
    vScroll.setBarColors(listColors);
    // repaint all in new colors
    repaintAll = true;
    repaint();
  }
  /****************************************************************************************************************************************/
  /**
  * Extends the Component's setFont function to pass the desired font towards the actual painting algorithm in the RudolphListPeer
  * Also recalculates the lists new width and height and sets viewports and scrollbars accordingly
  */
  public void setFont(Font f) {
    super.setFont(f);
    //get font, fontmetrics, fixed width charlength
    painterFont = (f==null)?RudolphListPeer.DEFAULT_FONT:f;
    painterMetrics = new FontMetrics(painterFont);
    painterFixedWidth = painterMetrics.getMaxAdvance();
    if(painterFixedWidth>painterMetrics.charWidth('i') ) {
    // the 'i' (smallest character) is smaller then the biggest character   => variable width
      painterFixedWidth=-1;
    }
      	
    //if text, recalculate screen, scrollbars & visibility
    if(textBuffer!=null) {
      // calculate new maximum length
      calculateMaximumWidth();
      // Calculate new scrollbars & set scrollbar step
      calculateScrollbarSettings(); //new scrollbar visibility, screen sizes, scrollbar sizes
      calculateVisibleTextlines();  //new start/stop for textlines viewport
      hScroll.setLineStep(painterMetrics.getMaxAdvance());
      // repaint all in new colors
      repaintAll = true;
      repaint();
    }
  }
  	

  /****************************************************************************************************************************************/
  /**
  * Listeners
  * @status: Instead of using the AWTEventMulticaster class as in handbook design patterns, we implement our own listener array
  */
  /****************************************************************************************************************************************/
  /**
  * add action listener handbook design is => aListener = AWTEventMulticaster.add(aListener, newalistener);
  */
  public void addActionListener(ActionListener newlistener) {
    aMultiListener = AWTEventMulticaster.add(aMultiListener,newlistener);
  }

  /****************************************************************************************************************************************/
  /**
  * remove action listener handbook design is => alistener = AWTEventMulticaster.remove(alistener, newalistener);
  */
  public void removeActionListener(ActionListener oldlistener) {
    aMultiListener = AWTEventMulticaster.remove(aMultiListener,oldlistener);
  }

  /****************************************************************************************************************************************/
  /**
  * add item listener (either the listener itself or an AWTEventMulticaster ItemListener handling it
  */
  public void addItemListener(ItemListener newlistener) {
    iMultiListener = AWTEventMulticaster.add(iMultiListener,newlistener);
  }

  /****************************************************************************************************************************************/
  /**
  * remove item listener. If the listener was in the  AWTEventMulticaster ItemListener list, return a new AWTEventMulticaster without it
  * if the listener was the only listener remaining, let AWTEventMulticaster delete it and return a null
  */
  public void removeItemListener(ItemListener oldlistener) {
    iMultiListener = AWTEventMulticaster.remove(iMultiListener,oldlistener);
  }

  /****************************************************************************************************************************************/
  /**
  * On a call to Component.processEvent, use our own listeners to handle all Item and action listeners
  * ( not that the list itself never calls this function, yet we reservr the option to receive action and Item events
  * from either the component superclass or from a custom-build derived list
  */
  protected void processEvent(AWTEvent e) {
    if(e instanceof  ActionEvent) {
      if(aMultiListener != null) {
        aMultiListener.actionPerformed((ActionEvent)e);
      }
    }
    else if(e instanceof  ItemEvent) {
      if(iMultiListener != null) {
        iMultiListener.itemStateChanged((ItemEvent)e);
      }
    }
    else {
      super.processEvent(e);
    }
  }


  /****************************************************************************************************************************************/
  /**
  * Process adjustmentevent as described in Adjustable interface
  */
  protected void processActionEvent(ActionEvent aet) {
    if(aMultiListener != null) {
      aMultiListener.actionPerformed(aet);
    }
  }

  protected void processItemEvent(ItemEvent iet){
    if(iMultiListener != null) {
      iMultiListener.itemStateChanged(iet);
    }
  }


  /****************************************************************************************************************************************/
  /**
  * alternative form using just the event data that isn't fixed already for List and ItemEvent classes
  * deprecated because it's just as eaasy to call the main form in the two or three cases where throwing Itemevents is needed
  private void processItemEvent(int index, int selected) {
    if(iMultiListener != null && textBuffer!=null) {
      int start = (index>0)?textlineEnd[index-1]:0;
      iMultiListener.itemStateChanged(new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED, new String(textBuffer,start,textlineEnd[index]-start), selected));
    }
  }
  /****************************************************************************************************************************************/
  /**
  * Alternative form: as we know all the Actionevent data except for the index of the label clicked, we only need this index to throw
  * an action event on our own
  * deprecated because it's just as eaasy to call the main form in the two or three cases where throwing Itemevents is needed
  * /
  private void throwActionEvent(int index) {
    if(aMultiListener != null && textBuffer!=null) {
      int start = (index>0)?textlineEnd[index-1]:0;
      aMultiListener.actionPerformed(new ActionEvent(this,ActionEvent.ACTION_PERFORMED,new String(textBuffer,start,textlineEnd[index]-start)));
    }
  }


  /****************************************************************************************************************************************/
  /**
  * Selection and multiple mode functions
  */
  /****************************************************************************************************************************************/

  /****************************************************************************************************************************************/
  /**
  * Select current item:
  * => single mode: current selection = index
  * => multiple mode: in addition element(index) is marked selected
  * according to java specifications (jdk 1.2 / 1998), an out-of-range index throws an ArrayIndexOutOfBoundsException
  */	
  public void select(int index) throws ArrayIndexOutOfBoundsException {
    if(textBuffer==null) {
      throw new ArrayIndexOutOfBoundsException("Attempt to select element "+index+" of list size 0");
    }
    else if (index<0 || index >= textlineEnd.length) {
      throw new ArrayIndexOutOfBoundsException("Attempt to select element "+index+" of list size "+textlineEnd.length);
    }
    //else
    currentSelected = index;
    lastSelected = index;
    if(multiple)
    itemSelected[index] = true;
  }

  /****************************************************************************************************************************************/
  /**
  * Deselect current item:
  * => single mode: if index equals current selected, current selected = void (-1)
  * => multiple mode: in addition if element(index) = selected: deselect
  * according to newest java specifications (jdk 1.2 / july 2001), an out-of-range index is simply disregarded
  */	
  public void deselect(int index) {
    if(textBuffer==null  || index <0 || index >= textlineEnd.length) {
      return;
    }
    if(currentSelected == index) {
      currentSelected = -1;
    }
    if(multiple) {
      itemSelected[index] = false;
    }
  }
  	
  /****************************************************************************************************************************************/
  /**
  * return true if current item is selected
  * => single mode: look if current selection == index
  * => multiple mode: look if element[index] is marked selected
  * @note The specifications remain slightly hazy as to wether an out-of-range index throws an ArrayIndexOutOfBoundsException or not
  * In analogy to Select(), the Rudolph/Wonka implementation throws an ArrayIndexOutOfBoundsException
  */	
  public boolean isIndexSelected(int index)  throws ArrayIndexOutOfBoundsException {
    if(textBuffer==null) {
      throw new ArrayIndexOutOfBoundsException("Attempt to select element "+index+" of list size 0");
    }
    else if (index<0 || index >= textlineEnd.length) {
      throw new ArrayIndexOutOfBoundsException("Attempt to select element "+index+" of list size "+textlineEnd.length);
    }
    else if(multiple) {
      return (itemSelected[index]);
    }
    else {
      return (currentSelected == index);
    }
  }

  /****************************************************************************************************************************************/
  /**
  * return last selected index
  */	
  public synchronized int getSelectedIndex() {
    return currentSelected;
  }
  /****************************************************************************************************************************************/
  /**
  * return the index of the last visible object (even if that object is no longer selected)
  */	
  public int getVisibleIndex() {
    return lastSelected;
  }
  		

  /****************************************************************************************************************************************/
  /**
  * return indexes of all selected elements
  * (single mode and selection is made: return array int[1] containing current selection)
  * (no selection or no elements in list: return int[0])
  */
  public synchronized int[] getSelectedIndexes() {
    int[] selection;
    if(textBuffer==null) {
      //no items at all
      selection = new int[0];
    }
    else if(!multiple && currentSelected<0) {
      //single mode, no selection
      selection = new int[0];
    }
    else if (!multiple) {
      // single mode, but a selected element: return int[1] containing that element's number
      selection = new int[1];
      selection[0] = currentSelected;
    }
    else {
      // multiple mode: return an array containing all selected items
      // 1) count number of selected items
      int selected = 0;
      for(int i=0; i<itemSelected.length; i++) {
        if(itemSelected[i]) {
          selected++;
        }
      }
      // 2) build an array to contain them
      selection = new int[selected];
      // 3) fill array
      selected = 0;
      for(int i=0; i<itemSelected.length; i++) {
        if(itemSelected[i]) {
          selection[selected] = i;
          selected++;
        }
      }	
    }
  return selection;
  }	

  /****************************************************************************************************************************************/
  /**
  * Check if multiple mode
  */	
  public boolean isMultipleMode() {
    return multiple;
  }
  /**deprecated*/
  public boolean AllowsMultipleSelections() {
    return multiple;
  }

  /****************************************************************************************************************************************/
  /**
  * Switch multiple selection mode on/off
  */	
  public void setMultipleMode(boolean newmode) {
    multiple = newmode;
    if(!newmode) {
      // no multiple selections
      itemSelected = null;
      //(currentSelected remains valid)
    }
    else if(textBuffer!= null) {
      // we have items, so make the itemVisible array just as well
      itemSelected = new boolean[textlineEnd.length];
      // Set all array elements to no-selected
      for(int i=0; i<itemSelected.length; i++) {
        itemSelected[i] = false;
      }
      //if we have a current selection, set this one's multiple array to be selected
      if(currentSelected >= 0) {
        itemSelected[currentSelected] = true;
      }
    }
  }

  /** deprecated*/
  public void setMultipleSelections(boolean mode) {
    setMultipleMode(mode);
  }



  /****************************************************************************************************************************************/
  /**
  * retrieving elements from list
  */
  /****************************************************************************************************************************************/

  /****************************************************************************************************************************************/
  /**
  * number of items in list
  */
  public int getItemCount() {
    return (textBuffer==null)?0:textlineEnd.length;
  }

  /****************************************************************************************************************************************/
  /**
  * Gets (A COPY OF) the item at the selected index, or throws an ArrayIndexOutOfBoundException
  */
  public String getItem(int index) throws ArrayIndexOutOfBoundsException {
    if(textBuffer==null) {
      throw new ArrayIndexOutOfBoundsException("Attempt to acces element "+index+" of list size 0");
    }
    else if (index<0 || index >= textlineEnd.length) {
      throw new ArrayIndexOutOfBoundsException("Attempt to acces element "+index+" of list size "+textlineEnd.length);
    }
    else if (index<1) {
      return new String(textBuffer,0,textlineEnd[0]);
    }
    else {
      return new String(textBuffer,textlineEnd[index-1],textlineEnd[index]-textlineEnd[index-1] );
    }
  }

  /****************************************************************************************************************************************/
  /**
  * returns an array containing A COPY of all the items in the list
  */
  public String[] getItems() {
    if(textBuffer==null) {
      //no items yet
      return new String[0];
    }
    //else
    //1) build array of size items.length (with textbuffer instead of item Strings, this is the size of textlineEnd
    // or any of the other line position arrays
    String[] copy = new String[textlineEnd.length];
     	
    //2) fill the array with strings recalculated from the textbuffer and line ends
    copy[0]= new String(textBuffer,0,textlineEnd[0]);
    int linestart = textlineEnd[0];
    for (int i=1; i<textlineEnd.length; i++) {
      copy[i] = new String(textBuffer,linestart,textlineEnd[i]-linestart);
      linestart=textlineEnd[i];
    }	
    return copy;
  }

  /****************************************************************************************************************************************/
  /**
  * return last selected element
  * By definition, in multiple mode, the function ALWAYS returns null
  */	
  public synchronized String getSelectedItem() {
    if(textBuffer==null || multiple || currentSelected<0) {
      // no elements, or no selection: return null
      // by definition, the function must return null in multiple mode
      return null;
    }
    if(currentSelected>0) {
      // return new String from textlineEnd[previous item] to textLineEnd[this]
      return new String(textBuffer,textlineEnd[currentSelected-1],textlineEnd[currentSelected] - textlineEnd[currentSelected-1]);
    }
    else {
      // special case for the first item selected:
      // the string for the first item goes from position 0 to position textlineEnd[ item==0]
      return new String(textBuffer,0,textlineEnd[0]);
    }
  }

  /****************************************************************************************************************************************/
  /**
  * return String array of all selected elements (single mode: return array[1] containing current selection)
  * ( as getSelectedIndexes, but this time returning Strings instead of numbers )
  */	
  public synchronized String[] getSelectedItems() {
  	int[] indexes = getSelectedIndexes();
  	if(indexes.length==0) {
  	  return new String[0];
    }
    //else
    //1) build String buffer for the elements
    String[] selection = new String[indexes.length];
    //2) fill the buffer
    int i=0;
    // if the first element is 0, we have to use the special case for string retrieval: stringbuffer range 0 to textlineEnd[0]
    if(indexes[0]==0) {
      selection[0]= new String(textBuffer,0,textlineEnd[0]);
      i++;
    }
    // copy all other elements by general case: range=textlineEnd[previous] to textlineEnd[this]
    while(i<indexes.length) {
      selection[i]= new String(textBuffer,textlineEnd[indexes[i]-1],textlineEnd[indexes[i]]-textlineEnd[indexes[i]-1]);
      i++;
    }
    return selection;
  }	

  /****************************************************************************************************************************************/
  /**
  * return array of all selected objects (as selectedItems but this time returning the Strings as Object instances)
  */	
  public synchronized Object[] getSelectedObjects() {
  	int[] indexes = getSelectedIndexes();
  	if(indexes.length==0) {
  	  return new Object[0];
    }
    //else
    //1) build String buffer for the elements
    Object[] selection = new Object[indexes.length];
    //2) fill the buffer
    int i=0;
    // if the first element is 0, we have to use the special case for string retrieval: stringbuffer range 0 to textlineEnd[0]
    if(indexes[0]==0) {
      selection[0]= new String(textBuffer,0,textlineEnd[0]);
      i++;
    }
    // copy all other elements by general case: range=textlineEnd[previous] to textlineEnd[this]
    while(i<indexes.length) {
      selection[i]= new String(textBuffer,textlineEnd[indexes[i]-1],textlineEnd[indexes[i]]-textlineEnd[indexes[i]-1]);
      i++;
    }
    return selection;
  }	

  		

  /****************************************************************************************************************************************/
  /**
  * adding, removing and replacing items.
  */
  /****************************************************************************************************************************************/

  /****************************************************************************************************************************************/
  /**
  * add to the end of the list
  */
  public void add(String newitem) {
    if(textBuffer== null) {
      newList(newitem);
    }
    else  {
      // add to end
      addElement(newitem, textlineEnd.length);	
    }
    //by default redraw everything...
    repaintAll = true;
    repaint();
  }

  /** deprecated */
  public void addItem(String newitem) {
    add(newitem);
  }

  /****************************************************************************************************************************************/
  /**
  * add to the list at desired position
  * @note according to newest java specifications (jdk 1.2 / july 2001),the function is no longer synchronised
  */
  public void add(String newitem, int pos)
  {
    if(textBuffer == null) {
      newList(newitem);
    }
    else if(pos<0 || pos>=textlineEnd.length) {
      addElement(newitem, textlineEnd.length);	
    }
    else {
      addElement(newitem, pos);
    }
    //by default redraw everything...
    repaintAll = true;
    repaint();
  }
  	
  /** deprecated */
  public void addItem(String newitem, int pos) {
    add(newitem, pos);
  }

  /****************************************************************************************************************************************/
  /**
  * replace an item at a given position with another given element.
  * @note: the jdk specifications don't prescribe what to do if the item is not within range. In analogy with the delete functions,
  *        Wonka/Rudolph will throw an ArrayIndexOutOfBoundsException
  */
  public void replaceItem(String newitem, int pos)  throws ArrayIndexOutOfBoundsException
  {
    if(textBuffer==null) {
      throw new ArrayIndexOutOfBoundsException("Attempt to delete element "+pos+" of list size 0");
    }
    else if (pos<0 || pos >= textlineEnd.length) {
      throw new ArrayIndexOutOfBoundsException("Attempt to delete element "+pos+" of list size "+textlineEnd.length);
    }
    else if(textlineEnd.length<1) {
      // replace the only element = make a new list with the new one
      newList(newitem);
      //by default redraw everything...
      repaintAll = true;
      repaint();
    }
    else {
      replaceElement(newitem,pos);
      //by default redraw everything...
      repaintAll = true;
      repaint();
    }
  }

  /****************************************************************************************************************************************/
  /**
  * remove an item at a given position of the list. (throws ArrayIndexOutOfBoundsException if the item is not within range)
  */
  public void remove(int index)  throws ArrayIndexOutOfBoundsException {

    if(textBuffer==null) {
      throw new ArrayIndexOutOfBoundsException("Attempt to delete element "+index+" of list size 0");
    }
    else if (index<0 || index >= textlineEnd.length) {
      throw new ArrayIndexOutOfBoundsException("Attempt to delete element "+index+" of list size "+textlineEnd.length);
    }
    else if(textlineEnd.length<2) {
      //remove the only element => remove the whole list
      removeAll();
    }
    else {
      removeElement(index, index+1);
      //by default redraw everything...
      repaintAll = true;
      repaint();
    }
  }

  /** deprecated */
  public void delItem(int index) throws ArrayIndexOutOfBoundsException {
    if(textBuffer==null) {
      throw new ArrayIndexOutOfBoundsException("Attempt to delete element "+index+" of list size 0");
    }
    else if (index<0 || index >= textlineEnd.length) {
      throw new ArrayIndexOutOfBoundsException("Attempt to delete element "+index+" of list size "+textlineEnd.length);
    }
    else if(textlineEnd.length<2) {
      //remove the only element => remove the whole list
      removeAll();
    }
    else {
      removeElement(index, index+1);
      //by default redraw everything...
      repaintAll = true;
      repaint();
    }
  }
  	
  /** deprecated */
  public void delItems(int start, int stop)throws ArrayIndexOutOfBoundsException { //{remove(pos);}
    if(textBuffer==null) {
      throw new ArrayIndexOutOfBoundsException("Attempt to delete element "+start+" of list size 0");
    }
    else if (start<0 || start >= textlineEnd.length) {
      throw new ArrayIndexOutOfBoundsException("Attempt to delete element "+start+" of list size "+textlineEnd.length);
    }
    else if (stop<0 || stop >= textlineEnd.length) {
      throw new ArrayIndexOutOfBoundsException("Attempt to delete element "+stop+" of list size "+textlineEnd.length);
    }
    else if(textlineEnd.length<2) {
      //remove the only element => remove the whole list
      removeAll();
    }
    else if(stop<start) {
      removeElement(stop, start+1);
      //by default redraw everything...
      repaintAll = true;
      repaint();
    }
    else {
      removeElement(start, stop+1);
      //by default redraw everything...
      repaintAll = true;
      repaint();
    }
  }
  	
  /****************************************************************************************************************************************/
  /**
  * remove a given item from the list. (throws IllegalArgumentException if the item is not found in the list)
  */
  public void remove(String element)  throws ArrayIndexOutOfBoundsException {
    if(textBuffer==null) {
      throw new IllegalArgumentException("Element "+element+" not found in empty list");
    }
    //else
    //get first element and compare to desired one
    String label= new String(textBuffer,0,textlineEnd[0]);
    int pos=(label.equals(element))?0:-1;
    int linestart=textlineEnd[0];
    //idem for rest of list
    for(int i=1;i<textlineEnd.length && pos<0;i++) {
      label=new String(textBuffer,linestart,textlineEnd[i]-linestart);
      pos=(label.equals(element))?i:-1;
      linestart=textlineEnd[i];
    }
       	
    if(pos<0) {
      throw new IllegalArgumentException("Element "+element+" not found in empty list");
    }
    else if(textlineEnd.length<2) {
      //remove the only element => remove the whole list
      removeAll();
    }
    else {
      removeElement(pos, pos+1);
      //by default redraw everything...
      repaintAll = true;
      repaint();
    }
  }

  /****************************************************************************************************************************************/
  /**
  * remove all items. (this is an explicit delete for all of the items/visibleItems/selected arrays
  */
  public void removeAll() {
    //delete arrays
    textBuffer = null;
    textlineEnd = null;
    visibleLineStart = null;
    visibleLineLength = null;
       	
    if(multiple) {
      itemSelected = null;
    }

    //selections invalid
    currentSelected = -1; //no selection
    lastSelected = -1; 	  // 		
    // no item length and scrollbar
    maximumLineWidth = 0;
    calculateScrollbarSettings(); //this also sets hScrollVisible = false;  and vScrollVisible = false;
      	
    //by default redraw everything...
    repaintAll = true;
    repaint();
  }

  /** deprecated*/
  public void clear() {
    removeAll();
  }

  /****************************************************************************************************************************************/
  /**
  * auxilliary function to calculate the maximum line width out of the text buffer and linestart array
  */
  private void calculateMaximumWidth() {
    maximumLineWidth = (painterFixedWidth>0)?painterFixedWidth*textlineEnd[0]:painterMetrics.charsWidth(textBuffer, 0, textlineEnd[0]);
    int len;
    int linestart= textlineEnd[0];
    // subsequent lines
    for(int i=1; i<textlineEnd.length;i++) {
      len=(painterFixedWidth>0)?painterFixedWidth*(textlineEnd[i]-linestart): painterMetrics.charsWidth(textBuffer, linestart, textlineEnd[i]-linestart);
      if(len>maximumLineWidth) {
        maximumLineWidth = len;
      }
      linestart=textlineEnd[i];
    }		
  }	
   	
  /****************************************************************************************************************************************/
  /**
  * auxilliary function to calculate the line width of a given line
  */
  private int calculateLineWidth(int line) {
    if(painterFixedWidth>0 && line>0) {
      return painterFixedWidth*(textlineEnd[line]-textlineEnd[line-1]);
    }
    else if(painterFixedWidth>0) {
      return painterFixedWidth*textlineEnd[0];
    }
    else if(line>0) {
      return painterMetrics.charsWidth(textBuffer,textlineEnd[line-1],textlineEnd[line]-textlineEnd[line-1]);
    }
    else {
      return painterMetrics.charsWidth(textBuffer,0,textlineEnd[0]);
    }
  }

  /****************************************************************************************************************************************/
  /**
  * Auxilliary function to add element into an existing list at an existing position. The
  */
  private void addElement(String newitem, int pos) {	
    int ilen=newitem.length();
    int i,j;
    //build bigger arrays to add items
    char[] newbuffer = new char[textBuffer.length+ilen];	
    // buffer copy (Off course we could use System.arrayCopy, but this algorithm's simple enough to do the same without the overlay)
    //copy buffer up to position
    j=(pos>0)?textlineEnd[pos-1]:0;
    for(i=0;i<j;i++) {
      newbuffer[i]=textBuffer[i];
    }
    // copy new string into position
    newitem.getChars(0,ilen,newbuffer,j);
    // copy all other chars <ilen> bytes to the right
    for(i=j+ilen;i<newbuffer.length;i++) {
      newbuffer[i]=textBuffer[j++];
    }
    		
    //assign buffer
    textBuffer = newbuffer;
    		
    //bigger line-end , visible start, visible length arrays
    j=textlineEnd.length +1;
    int[] newend = new int[j];
    int[] newstart = new int[j];
    int[] newlength = new int[j];
    //copy arrays up to position
    for(i=0;i<pos;i++) {
      newend[i]=textlineEnd[i];
      newstart[i]=visibleLineStart[i];
      newlength[i]=visibleLineLength[i];
    }
    //end of new line = previous+length
    newend[pos]=(pos<1)?ilen:textlineEnd[i-1]+ilen;
    j=pos;
      for(i=pos+1;i<newend.length;i++) {
      newend[i]=textlineEnd[j]+ilen;
      newstart[i]=visibleLineStart[j]+ilen;
      newlength[i]=visibleLineLength[j];
      j++;
    }
    		
    //assign new arays to the new list
    textlineEnd=newend;
    visibleLineStart = newstart;
    visibleLineLength = newlength;
    		
    // Adjust selected positions (if the new element is inserted before them, they should be augmented by one)
    if(currentSelected >=pos) {
      currentSelected++;
    }
    if(lastSelected >=pos) {
      lastSelected++; 	  		
    }
    // idem for multiple selections: By default, the new item is NOT selected
    if(multiple) {
      //assign a new selection array of size<old+1>
      boolean[] newselected = new boolean[textlineEnd.length];
      for(i=0; i<pos; i++) {
        newselected[i]=itemSelected[i];
      }
      newselected[pos]= false;
      j=pos;
      for(i=pos+1; i<textlineEnd.length; i++) {
        newselected[i]=itemSelected[j++];
      }				
      itemSelected = newselected;  		
    }
    		
    // new maximum if nessessary
    j=calculateLineWidth(pos);
    if(j>maximumLineWidth) {
      maximumLineWidth = j;
    }
    			
    //adjust scrollbars and set visible part of new item update visible parts of rest too if needed)
    if (calculateScrollbarSettings() ) {
      // adjust scrollbars and visible parts if needed
      calculateVisibleTextlines();  // if size changed, update all
    }
    else {
      calculateVisibleTextline(pos); //only get the new item
    }
  }

  /****************************************************************************************************************************************/
  /**
  * Auxilliary function to add the first element to a list, ergo to construct a whole new set of list arrays for this one element
  */
  private void newList(String newitem) {
    // new text buffer containing the new item as only line
    textBuffer = new char[newitem.length()];
    newitem.getChars(0, textBuffer.length, textBuffer, 0);
    //new textline and visibility arrays
    textlineEnd = new int[1];
    visibleLineStart = new int[1];
    visibleLineLength = new int[1];
       	
    // only line ends at buffer.length
    textlineEnd[0] = textBuffer.length;
       	
    // new maximum
    maximumLineWidth=(painterFixedWidth>0)?painterFixedWidth*textlineEnd[0]: painterMetrics.charsWidth(textBuffer,0,textlineEnd[0]);   	
    // calculate scrollbar visibility and ranges for the new element and element size
    calculateScrollbarSettings();
    calculateVisibleTextline(0);  //calculate new element's visibility
    //multiple selection
    if(multiple) {
      itemSelected = new boolean[1];
      itemSelected[0] = false;
    }
       	
    //selection and current item
    currentSelected = -1; //no selection
    lastSelected = -1; 	
  }


  /****************************************************************************************************************************************/
  /**
  * remove from the list from desired position
  */
  public void removeElement(int start, int stop) {
    int drange=stop-start;
    int dstart=(start>0)?textlineEnd[start-1]:0;
    //int dstop=textlineEnd(stop-1):0;
    int dlen=textlineEnd[stop-1]-dstart;
    int i,j;
    //system.out.println("=>list (size="+textlineEnd.length+" max="+maximumLineWidth+") : removing "+start+" to "+stop
    //+"(chars "+dstart+" to "+textlineEnd[stop-1]+")");
    	
    //check if one of the deleted items has the maximum line length (if so, we have to calculate a new one)
    boolean newmax = false;
    for(i=start; i<stop&&!newmax; i++) {
      newmax=(calculateLineWidth(i) >= maximumLineWidth);
    } 	
    //build smaller text array to contain textbuffer-chars from line start to line stop
    char[] newbuffer = new char[textBuffer.length - dlen];	
    // buffer copy (Off course we could use System.arrayCopy, but this algorithm's simple enough to do the same without the overlay)
    //copy buffer up to delete start
    for(i=0;i<dstart;i++) {
      newbuffer[i]=textBuffer[i];
    }
    // skip the chars between dstart and dlen
    j=	textlineEnd[stop-1];
    // copy all other chars <ilen> bytes to the left
    for(i=dstart;i<newbuffer.length;i++) {
      newbuffer[i]=textBuffer[j++];
    }		
    //assign buffer
    textBuffer = newbuffer;
    		
    //build smaller arrays for line-end , visible start, visible length
    j=textlineEnd.length-drange;
    int[] newend = new int[j];
    int[] newstart = new int[j];
    int[] newlength = new int[j];
    //copy arrays up to start position
    for(i=0;i<start;i++) {
      newend[i]=textlineEnd[i];
      newstart[i]=visibleLineStart[i];
      newlength[i]=visibleLineLength[i];
    }
    //skip lines up to <stop>, then copy the subsequent lines, their start positions shifted <dlen> to the left
    j=stop;
    for(i=start;i<newend.length;i++) {
      newend[i]=textlineEnd[j]-dlen;
      newstart[i]=visibleLineStart[j]-dlen;
      newlength[i]=visibleLineLength[j];
      j++;
    }
    		
    //assign new arays to the new list
    textlineEnd=newend;
    visibleLineStart = newstart;
    visibleLineLength = newlength;
    // Adjust selected positions (if the new element is inserted before them, they should be augmented by one)
    if(currentSelected >=start && currentSelected<stop) {
      //inside the deleted range
      currentSelected = -1; // no more selection, we just deleted it
    }
    else if(currentSelected>=stop) {
      //above the deleted range: shift drange lines down
      currentSelected-=drange;
    }

    if(lastSelected >=start && currentSelected<stop) {
      lastSelected = -1; 	  		
    }
    else if(lastSelected>=stop) {
      //above the deleted range: shift drange lines down
      lastSelected-=drange;
    }
    // idem for multiple selections:
    if(multiple) {
      //assign a new selection array of size<old-1>
      boolean[] newselected = new boolean[textlineEnd.length];
      for(i=0; i<start; i++) {
        newselected[i]=itemSelected[i];
      }
      j=stop;
      for(i=start; i<textlineEnd.length; i++) {
        newselected[i]=itemSelected[j++];
      }    				
      itemSelected = newselected;  		
    }
    		
    //adjust scrollbars (&& recalculate visibilities if needed)
    if(newmax) {
      calculateMaximumWidth();
    }
    if (calculateScrollbarSettings() ) {
      // adjust scrollbars and visible parts if needed
      calculateVisibleTextlines();
    }
  }

  /****************************************************************************************************************************************/
  /**
  * replace an item in the list
  */
  public void replaceElement(String newitem, int pos) {
    int ilen=newitem.length();
    int dlen =(pos>0)?ilen-textlineEnd[pos]+textlineEnd[pos-1]:ilen-textlineEnd[pos];
    int i,j;
    //check if one of the deleted items has the maximum line length (if so, we have to calculate a new one)
    i= calculateLineWidth(pos);
    j= (painterFixedWidth>0)?painterFixedWidth*ilen:painterMetrics.stringWidth(newitem);
    boolean newmax=false;
    if(j>=maximumLineWidth) {
      maximumLineWidth=j;
    }
    else if(i>=maximumLineWidth) {
      newmax = true;
    }
    //build bigger arrays to add items
    char[] newbuffer = new char[textBuffer.length+dlen];	
    // buffer copy (Off course we could use System.arrayCopy, but this algorithm's simple enough to do the same without the overlay)
    //copy buffer up to position
    j=(pos>0)?textlineEnd[pos-1]:0;
    for(i=0;i<j;i++) {
      newbuffer[i]=textBuffer[i];
    }
    // copy new string into position
    newitem.getChars(0,ilen,newbuffer,j);
    // copy all other chars <ilen> bytes to the right
    i=j+ilen;
    j=textlineEnd[pos];
    while(i<newbuffer.length) {
      newbuffer[i]=textBuffer[j];
      i++;
      j++;
    }
    		
    //assign buffer
    textBuffer = newbuffer;
    		
    //The size of the position arrays stays the same, all we have to do is adjust their values for the elements comming after pos
    //copy arrays up to position
    for(i=pos;i<textlineEnd.length;i++) {
      textlineEnd[i]+=dlen;
      visibleLineStart[i]+=dlen;
      //visibleLineLength[j]; stays the same
    }
    		
    		
    // By default, the new item is NOT selected => if needed, set the current and last selected element to -1
    if(currentSelected ==pos) {
      currentSelected=-1;
    }
    if(lastSelected ==pos) {
      lastSelected=-1; 	  		
    }
    // idem for multiple selections: By default, the new item is NOT selected
    if(multiple) {
      itemSelected[pos] = false;  		
    }			
    //adjust scrollbars and set visible part of new item update visible parts of rest too if needed)
    if(newmax) {
      calculateMaximumWidth();
      // adjust scrollbars and visible parts if needed
    }
    if (calculateScrollbarSettings() ) {
      // if size changed, update all
      calculateVisibleTextlines();  // if size changed, update all
    }
    else {
      //only get the new item
      calculateVisibleTextline(pos);
    }
  }
  /****************************************************************************************************************************************/
  /**
  * number of items, item width or panel size changed: recalculate the visible lines and scrollbar
  * returns true when the update demands a recalculation of the visible item parts
  * (this is when either itemViewport.width or charOffset has changed)
  * (Protected because this function is also used in the derived Choice's DropList)
  */
  protected boolean calculateScrollbarSettings() {
    int fullscreenlabels =  RudolphListPeer.getLabels(screenViewport.height, painterMetrics);
    int barredlabels = RudolphListPeer.getLabels(screenViewport.height - RudolphScrollbarPeer.HSCROLL_HEIGHT, painterMetrics );

    int innerwidth  = RudolphListPeer.getInnerLength(screenViewport.width);
    int barredwidth = innerwidth - RudolphScrollbarPeer.VSCROLL_WIDTH;
      	
    //look if overal width( in chars) will change
    int oldlinewidth = listViewport.width;
    int oldcharoffset = charOffset;
    // see which scrollbars will be needed and calculate the list viewport accordingly
    if(textBuffer == null) {
      // no text at all

      // no scrollbars needed
      hScrollVisible = false;
      charOffset = 0;
      vScrollVisible = false;
      lineOffset = 0;
      // tell new sizes to framework
      listViewport.setSize(screenViewport);
      viewportLabels = fullscreenlabels;
    }
    else if(innerwidth>= maximumLineWidth && fullscreenlabels >= textlineEnd.length) {
      // no scrollbars needed
      hScrollVisible = false;
      charOffset = 0;
      vScrollVisible = false;
      lineOffset = 0;
      // tell new sizes to framework
      listViewport.setSize(screenViewport);
      viewportLabels = fullscreenlabels;
    }
    else if(barredwidth >= maximumLineWidth) {
      //vertical scrollbar only

      //get space for vertical bar
      listViewport.setSize(screenViewport.width - RudolphScrollbarPeer.VSCROLL_WIDTH,screenViewport.height) ;
      viewportLabels = fullscreenlabels;
      // Vertical scrollbar
      hScrollVisible = false;
      charOffset = 0;
      // Vertical scrollbar
      vScrollVisible = true;
      vScroll.setOffset(listViewport.width);
      lineOffset = vScroll.setScreenHeight(listViewport.height, fullscreenlabels, textlineEnd.length);
    }
    else if(barredlabels >= textlineEnd.length) {
      //horizontal scrollbar only

      //get space for horizontal bar
      listViewport.setSize(screenViewport.width,screenViewport.height - RudolphScrollbarPeer.HSCROLL_HEIGHT) ;
      viewportLabels = barredlabels;
      // horizontal scrollbar
      hScrollVisible = true;
      hScroll.setOffset(listViewport.height);
      charOffset = hScroll.setScreenWidth(listViewport.width, maximumLineWidth);
      //no vertical
      vScrollVisible = false;
      lineOffset = 0;
    }
    else {
      //definitely need the two scrollbars

      //get space for horizontal and vertical bar
      listViewport.setSize(screenViewport.width-RudolphScrollbarPeer.VSCROLL_WIDTH, screenViewport.height-RudolphScrollbarPeer.HSCROLL_HEIGHT);  		
      viewportLabels = barredlabels;
      // horizontal scrollbar
      hScrollVisible = true;
      hScroll.setOffset(listViewport.height);
      charOffset = hScroll.setScreenWidth(listViewport.width, maximumLineWidth);
      //vertical
      vScrollVisible = true;
      vScroll.setOffset(listViewport.width);
      lineOffset = vScroll.setScreenHeight(listViewport.height, barredlabels, textlineEnd.length);
    }
    // always repaint the whole screen
    repaintAll = true;
    // return true if overal width has changed
    return(oldlinewidth!=listViewport.width || oldcharoffset!= charOffset);

  }

  /****************************************************************************************************************************************/
  /**
  * Auxilliary: calculate the visible part of an item string (from given char offset and viewport lines(in chars)
  */
  private void calculateVisibleTextline(int line) {
    int linestart = (line>0)? textlineEnd[line-1]:0;
    if(charOffset==0) {
      // no offset: simply start at linestart
      visibleLineStart[line] = linestart;
    }
    else if(painterFixedWidth>0) {
      //fixed width: divide offset by width to get the offset in chars
      visibleLineStart[line] = linestart+(charOffset/painterFixedWidth)+1;
    }
    else {
      //Variable width: use the RudolphPeer functions
      visibleLineStart[line] = RudolphPeer.getChars(charOffset,linestart,textBuffer,painterMetrics)+1;
      //( we start at the character AFTER THE LAST ONE that can be displayed in the offset )
    }
      	
    if(visibleLineStart[line]>=textlineEnd[line]) {
      // visible line should 'start' after end of current line => display nothing
      visibleLineStart[line] = textlineEnd[line];
      visibleLineLength[line] = 0;
    }
    else if(painterFixedWidth>0) {
      //fixed width: line length in chars = screen width / character width
      visibleLineLength[line] = RudolphListPeer.getInnerLength(listViewport.width)/painterFixedWidth;
      if((visibleLineStart[line]+visibleLineLength[line])>textlineEnd[line]) {
        // the allowed line part ends after the real liiine is already ended => show only the real part
        visibleLineLength[line] = textlineEnd[line]-visibleLineStart[line];
      }
    }
    else {
      //Variable width: use the RudolphPeer functions
      visibleLineLength[line]=RudolphPeer.getChars(RudolphListPeer.getInnerLength(listViewport.width),visibleLineStart[line],textBuffer,painterMetrics);
      if(visibleLineLength[line]>=textlineEnd[line]) {
        visibleLineLength[line] = textlineEnd[line]-visibleLineStart[line];
      }
      else {
        visibleLineLength[line]-=visibleLineStart[line];
      }
    }
  }

  private void calculateVisibleTextlines() {
    if(textBuffer!=null) {
      for(int i=0; i<textlineEnd.length; i++) {
        calculateVisibleTextline(i);
      }
    }
  }


  /****************************************************************************************************************************************/
  /****************************************************************************************************************************************/
  /****************************************************************************************************************************************/
  /****************************************************************************************************************************************/
  /**
  * Mouse implementations
  */
  /** Mouse entered,*/
  public void mouseEntered(MouseEvent e) {
  }
  /** Mouse moved,*/
  public void mouseMoved(MouseEvent e) {
  }

  /****************************************************************************************************************************************/
  /**
  * Mouse clicked:
  * when mouse DOUBLE_CLICKED, throw an action event
  * @remark: touchscreen-Wonka does not have a notion of double-clicking, therefore MouseEvent.getClickCount() is useles.
  * @remark: => instead fire an ActionEvent when clicked and mouse points at the (just made)active field
  */
  public void mouseClicked(MouseEvent e) {
    // get clicked item
    int click = RudolphListPeer.getLabel(e.getY(), painterMetrics)+lineOffset;
    if(click == currentSelected && textBuffer != null) {//&& currentSelected>=0)  	
      // the current item is clicked => send a <selected> action event
      int start = (click>0)?textlineEnd[click-1]:0;
      //fire action event
      processActionEvent(new ActionEvent(this,ActionEvent.ACTION_PERFORMED,new String(textBuffer,start,textlineEnd[click]-start)));
    }
  }

  /****************************************************************************************************************************************/
  /** Mouse down:
  => see if in list box: set selection
  => see if in scrollbars: set active field
  */
  public void mousePressed(MouseEvent e) {
    int x=e.getX();
    int y=e.getY();
    if(hScrollVisible && y>listViewport.height) {
      // lower then screen => in horizontal scrollbar
      // tell scrollbar where we clicked him
      int active = hScroll.setActive(x,y);
      if(active == AdjustmentEvent.UNIT_DECREMENT && hScroll.lineUp() ) {
      	// launch scroll thread
      	mouseEventThread.setRunner(hScroll,this);
      	//repaint scrollbar and text to latest settings
      	repaint(hScroll);
      }
      else if(active == AdjustmentEvent.UNIT_DECREMENT) {
        // simply paint the horizontal left arrow down
        repaintHScroll=true;
        repaint();
      }
      else if(active == AdjustmentEvent.UNIT_INCREMENT && hScroll.lineDn() ) {
      	// launch scroll thread
      	mouseEventThread.setRunner(hScroll,this);
      	//repaint scrollbar and text to latest settings
      	repaint(hScroll);
      }
      else if(active == AdjustmentEvent.UNIT_INCREMENT) {
        // simply paint the horizontal left arrow down
        repaintHScroll=true;
        repaint();
      }
      else if(active == AdjustmentEvent.BLOCK_DECREMENT && hScroll.pageUp() ) {
      	// launch scroll thread
      	mouseEventThread.setRunner(hScroll,this);
      	//repaint scrollbar and text to latest settings
      	repaint(hScroll);
      }
      else if(active == AdjustmentEvent.BLOCK_INCREMENT && hScroll.pageDn() ) {
      	// launch scroll thread
      	mouseEventThread.setRunner(hScroll,this);
      	//repaint scrollbar and text to latest settings
      	repaint(hScroll);
      }
      else if(active == AdjustmentEvent.TRACK) {
        lastMousePosition.setLocation(x,y);
        // also paint the box in <down> state
        repaintHScroll=true;
        repaint();
      }
    }
    else if(vScrollVisible && x>listViewport.width ){
      // right of screen => in vertical scrollbar
      int active = vScroll.setActive(x,y);
      if(active == AdjustmentEvent.UNIT_DECREMENT && vScroll.lineUp()) {
      	// launch scroll thread
      	mouseEventThread.setRunner(vScroll,this);
      	//repaint scrollbar and text to latest settings
      	repaint(vScroll);
      }
      else if(active == AdjustmentEvent.UNIT_DECREMENT) {
        // simply paint the horizontal left arrow down
        repaintVScroll=true;
        repaint();  	
      }
      else if(active == AdjustmentEvent.UNIT_INCREMENT && vScroll.lineDn()) {
      	// launch scroll thread
      	mouseEventThread.setRunner(vScroll,this);
      	//repaint scrollbar and text to latest settings
      	repaint(vScroll);
      }
      else if(active == AdjustmentEvent.UNIT_INCREMENT) {
        // simply paint the horizontal left arrow down
        repaintVScroll=true;
        repaint(); 	
      }
      else if(active == AdjustmentEvent.BLOCK_DECREMENT && vScroll.pageUp()) {
      	// launch scroll thread
      	mouseEventThread.setRunner(vScroll,this);
      	//repaint scrollbar and text to latest settings
      	repaint(vScroll);
      }
      else if(active == AdjustmentEvent.BLOCK_INCREMENT && vScroll.pageDn()) {
      	// launch scroll thread
      	mouseEventThread.setRunner(vScroll,this);
      	//repaint scrollbar and text to latest settings
      	repaint(vScroll);
      }  	
      else if(active == AdjustmentEvent.TRACK) {
        lastMousePosition.setLocation(x,y);
        repaintVScroll = true;				
        repaint();
      }

    }
    else {
      //inside list viewport
      lastMousePosition.setLocation(x,y);
      int click = RudolphListPeer.getLabel(y, painterMetrics)+lineOffset;
      if(textBuffer!=null && click <= textlineEnd.length) {
        //clicked on an item
        itemClicked (click);
        //repaint the list field only
        repaintListArea = true;				
        repaint();
      }
    }
 }
  	
  /****************************************************************************************************************************************/
  /** Mouse up:
  => inactivate scrollmode field and scrollbars selections
  */
  public void mouseReleased(MouseEvent e) {
    // multiple selections: current selected changes state from <currently selected> to <just one of the selections>
    if(multiple && currentSelected>=0) {
      currentSelected = -1;
      repaintListArea = true;
    }
    // horizontal scrollbar button pressed: all buttons are UP now (and scrollbar should be repainted to show this)
    if(hScrollVisible && hScroll.isSelected()) {
      mouseEventThread.stopRunner(hScroll);
      repaintHScroll = true;				
    }
    // vertical scrollbar button pressed: all buttons are UP now (and scrollbar should be repainted to show this)
    if(vScrollVisible && vScroll.isSelected()) {
      mouseEventThread.stopRunner(vScroll);
      repaintVScroll = true;				
    }
    // repaint if needed
    if(repaintListArea || repaintHScroll || repaintVScroll)
    repaint();
  }

  /** Mouse exiteded,
  => like mouse-up, when leaving the current component, inactivate scrollmode field and scrollbars selections
  */
  public void mouseExited(MouseEvent e) {
    // multiple selections: current selected changes state from <currently selected> to <just one of the selections>
    if(multiple && currentSelected>=0) {
      currentSelected = -1;
      repaintListArea = true;
    }
    // horizontal scrollbar button pressed: all buttons are UP now (and scrollbar should be repainted to show this)
    if(hScrollVisible && hScroll.isSelected()) {
      mouseEventThread.stopRunner(hScroll);
      repaintHScroll = true;				
    }
    // vertical scrollbar button pressed: all buttons are UP now (and scrollbar should be repainted to show this)
    if(vScrollVisible && vScroll.isSelected()) {
      mouseEventThread.stopRunner(vScroll);
      repaintVScroll = true;				
    }
    // repaint if needed
    if(repaintListArea || repaintHScroll || repaintVScroll)
    repaint();
  }

  /****************************************************************************************************************************************/
  /** Mouse dragged:
  => if scrollmode, see if scrollmode movement
  => if scrollbar scrollbox: move box and update positions
  */
  public void mouseDragged(MouseEvent e) {
    int x=e.getX();
    int y=e.getY();
    // vertical scrollbox
    if(vScrollVisible) {
      if(vScroll.getActive() == AdjustmentEvent.TRACK) {
        if(vScroll.moveBar(y-lastMousePosition.y)) {
          repaint(vScroll);
          lastMousePosition.y=y;
        }
        //else  do nothing and disregard all other cases
      }
      else if(x<0 || x>listViewport.width) {
        // no longer in list area, so stop
        mouseEventThread.stopRunner(vScroll);
      }
      else if(y>0 && y<RudolphListPeer.BORDER && y<lastMousePosition.y && vScroll.lineUp()) {
        // in upper area moved up => set scrollbar one up & start thread (moving up)
        mouseEventThread.setRunner(vScroll,this,ScrollRunner.SCROLL_UP);
        repaint(vScroll);
        lastMousePosition.y=y;
      }
      else if(y>RudolphListPeer.BORDER && y<lastMousePosition.y){
        //moved up when not in upper area => stop (moving down) thread
        mouseEventThread.stopRunner(vScroll);
        lastMousePosition.y=y;
      }
      else if(y<listViewport.height && y>(listViewport.height-RudolphListPeer.BORDER) && y>lastMousePosition.y && vScroll.lineDn()){
        // in lower area moved down: set scrollbar one down and start thread (moving down)
        mouseEventThread.setRunner(vScroll,this,ScrollRunner.SCROLL_DOWN);
        repaint(vScroll);
        lastMousePosition.y=y;
      }
      else if(y<(listViewport.height-RudolphListPeer.BORDER) &&  y>lastMousePosition.y){
        //moved down when not in lower area => stop (moving up) thread
        mouseEventThread.stopRunner(vScroll);
        lastMousePosition.y=y;
      }  	
    }
    // horizontal scrolling through scrollbox and mouse
    if(hScrollVisible) {
      if(hScroll.getActive() == AdjustmentEvent.TRACK) {
        // scrollbox moved (
        if(hScroll.moveBar(x-lastMousePosition.x)) {
          repaint(hScroll);
          lastMousePosition.x=x;
        }
        //else  do nothing and disregard all other cases
      }
      else if(y<0 || y>listViewport.height) {
        // no longer in list area, so stop
        mouseEventThread.stopRunner(hScroll);
      }
      else if(x>0 && x<RudolphListPeer.BORDER && x<lastMousePosition.x && hScroll.lineUp()) {
        // in upper area moved up => set scrollbar one up & start thread (moving up)
        mouseEventThread.setRunner(hScroll,this,ScrollRunner.SCROLL_UP);
        repaint(vScroll);
        lastMousePosition.x=x;
      }
      else if(x>RudolphListPeer.BORDER && x<lastMousePosition.x){
        //moved up when not in upper area => stop (moving down) thread
        mouseEventThread.stopRunner(hScroll);
        lastMousePosition.x=x;
      }
      else if(x<listViewport.width && x>(listViewport.width-RudolphListPeer.BORDER) && x>lastMousePosition.x && hScroll.lineDn()){
        // in lower area moved down: set scrollbar one down and start thread (moving down)
        mouseEventThread.setRunner(hScroll,this,ScrollRunner.SCROLL_DOWN);
        repaint(vScroll);
        lastMousePosition.x=x;
      }
      else if(x<(listViewport.width-RudolphListPeer.BORDER) &&  x>lastMousePosition.x){
        //moved down when not in lower area => stop (moving up) thread
        mouseEventThread.stopRunner(hScroll);
        lastMousePosition.x=x;
      }  	
    }
  }
  	
  	

  	
  /****************************************************************************************************************************************/
  /** auxilliary function: if item is selected, deselect it, otherwise select it */	
  private void itemClicked(int index) {
    if(textBuffer != null && index >=0 && index <textlineEnd.length) {
      //selection in list

      if(currentSelected == index) {
        //clicked on a selected item
        currentSelected = -1;       //deselect this
      }
      else {
        //clicked item is not selected
        currentSelected = index; //select this
        lastSelected = index;    //id
      }

      if(textBuffer!=null) {
        int start = (index>0)?textlineEnd[index-1]:0;
        int selected;
        if(multiple) {
          // if multiple lines, also select/deselect item in multiple buffer
          itemSelected[index] = !itemSelected[index];
          selected=(itemSelected[index])? ItemEvent.SELECTED: ItemEvent.DESELECTED; 	

        }
        else {
          //throw item event
          selected=(currentSelected>=0)? ItemEvent.SELECTED: ItemEvent.DESELECTED; 	
        }
        processItemEvent(new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED, new String(textBuffer,start,textlineEnd[index]-start), selected));
      }
    }
  }

  /****************************************************************************************************************************************/
  /**
  * 'manually' set the list to a certain position so that a certain item is visible
  * @note : java 1.2 defines makeVisible to be synchronised, Sun's online java reference defines it to be not.
  * As I see not much blocking variable access in the Wonka code for this function, I keep the wonka function unsynchronised
  */
  public void makeVisible(int target) {
    //safety on margins
    if(target <0) {
      target = 0;
    }
    else if(target>=textlineEnd.length) {
      target = textlineEnd.length-1;
    }

    if(target < lineOffset) {
      // target higher then upper screen marging => adjust lineOffset to make visible
      lineOffset = target;
      vScroll.setBarPos(lineOffset); //even if vScroll isn't visible, it still is there
      repaintListArea = true;
      repaintVScroll = true;
      repaint();
    }
    else if(target > (lineOffset+viewportLabels) ) {
      // target deeper then lower screen => move lineOffset down
      lineOffset = target-viewportLabels;
      vScroll.setBarPos(lineOffset); //even if vScroll isn't visible, it still is there
      repaintListArea = true;
      repaintVScroll = true;
      repaint();
    }
  }

  /****************************************************************************************************************************************/
  /**
  * Auxilliary: on a horizontal or vertical scrollbar position change, also update the list contents and redraw
  *  (this function is class protected so it can be called from the list runner thread just as well)
  */
  void repaint(ScrollPainter alignation) {
    if(alignation==hScroll) {
      // horizontal selection changed
      charOffset = hScroll.getBarPos();
      calculateVisibleTextlines();
      repaintHScroll = true;
    }
    else {
      // vertical selection changed
      lineOffset = vScroll.getBarPos();
      //calculateVisibleTextlines();
      repaintVScroll = true;
    }
    //in any case..
    repaintListArea = true;
    repaint();
  }

  /****************************************************************************************************************************************/
  /**
  * Paint the list. As both scrollbar and textfield cover the complete area of the list panel, we do not need
  * background clearing and can shortcut to 'update' right away
  */
  public void paint(Graphics g) {
    update(g);
  }

  public void update(Graphics g)
  {
    //if needed, set the screen sizes for list area and scrollbars and assign a background image buffer of the given length
    if(backgroundImage == null || !screenViewport.equals(getSize() ) ) {
      screenViewport.setSize(getSize());
      calculateScrollbarSettings();
      calculateVisibleTextlines();
      //new background image
      backgroundImage=createImage(screenViewport.width, screenViewport.height);
      if( backgroundImage!= null &&  backgroundGraphics!= null) {
        backgroundGraphics.dispose();
      }
      backgroundGraphics = backgroundImage.getGraphics();	
      //repaint all
      repaintAll = true;
    }
      	
    //check the colors
    if(getForeground()!=null && !getForeground().equals(listColors[4]) ) {
      listColors[4] = getForeground();
      hScroll.setBarColors(listColors);
      vScroll.setBarColors(listColors);
      repaintAll = true;
    }

    if(getBackground()!=null && !getBackground().equals(listColors[0]) ) {
      listColors = RudolphPeer.getBarColors(getBackground(), listColors[4]);
      hScroll.setBarColors(listColors);
      vScroll.setBarColors(listColors);
      repaintAll = true;
    }
    if(repaintAll) {
      backgroundGraphics.clearRect(1, 1, screenViewport.width-2, screenViewport.height-2);
    }

    // paint list area if needed
    if(repaintListArea||repaintAll) {
      //Clear the list window
      if(!repaintAll) {
        backgroundGraphics.clearRect(1, 1, listViewport.width-2, listViewport.height-2);
      }
      //paint the list items
      RudolphListPeer.paintList(
                                 1,1,listViewport.width-2, listViewport.height-2,
                                 textBuffer, visibleLineStart, visibleLineLength,
                                 lineOffset, itemSelected, currentSelected,
                                 painterFont, painterMetrics, listColors, backgroundGraphics
                               );
      repaintListArea = false;
    }

    //scrollbars
    if((repaintHScroll||repaintAll) && hScrollVisible) {
      hScroll.paint(backgroundGraphics);
      repaintHScroll = false;
    }

    if((repaintVScroll||repaintAll) && vScrollVisible) {
      vScroll.paint(backgroundGraphics);
      repaintVScroll = false;
    }

    repaintAll=false;
    // (finaly) paint background image
    g.drawImage(backgroundImage,0,0,this); 			
  }

  	
  /****************************************************************************************************************************************/
  /**
  * Debug information
  */
  public String toString() {
    Rectangle b=getBounds();
    String text = "List ("+b.x+","+b.y+","+b.width+","+b.height+")"+textlineEnd.length+" items";
    if(multiple) {
      text = "Multiple "+text;
    }
    else {
      text+=" selected:"+getSelectedIndex();
    }
    return text;
  }

  protected String paramString() {
    Rectangle b=getBounds();
    String text = "List "+getName()+" ("+b.x+","+b.y+","+b.width+","+b.height+") selected: "+textlineEnd.length+" items";
    if(multiple) {
      text = "Multiple "+text;
      String[] items = getSelectedItems();
      if(items.length>0){
        text+="["+items[0];
        for(int i=1; i<items.length; i++) {
          text+="],["+items[i];
        }
        text+="]";
      }
      else {
        text+="NONE";
      }
    }
    else {
      text+=") selected:"+getSelectedItem();
    }
    return text;
  }

  	
	
	
	
	
  /****************************************************************************************************************************************/
  //inner classes scrollbar painters
  /****************************************************************************************************************************************/
  /****************************************************************************************************************************************/
  /****************************************************************************************************************************************/
  class HListScrollPainter extends ScrollPainter
  {
    //	private int barOffset;
    /**
    * Constructor : set height, lineup/lineDn and minimum box width for horizontal scrollbar
    */
    public HListScrollPainter() {
      super(RudolphScrollbarPeer.HSCROLL_HEIGHT, RudolphScrollbarPeer.HSCROLL_LINEUPWIDTH,
      RudolphScrollbarPeer.HSCROLL_LINEDNWIDTH, RudolphScrollbarPeer.HSCROLL_MINIMUMBOXWIDTH);
    }
      	
    /**
    * scrollbar width and scrollbar position out of (x,y) dimension
    * the abstract ScrollPainter classes filled in for horizontal scrollbar
    */
    /** (horizontal bar) x- position in scrollbar from point (x,y) */
    public int getPos(int x, int y) {
      return x;
    }
    /** (horizontal bar) scrollbar height as thickness from dimension (width, height)*/
    public int getThickness(int width, int height) {
      return height;
    }
    /**scrollbar length and thickness to (width, height) dimension for horizontal and vertical scrollbar*/
    protected Dimension getSize(int scrollbarlength, int scrollbarthickness) {
      return new Dimension(scrollbarlength, scrollbarthickness);
    }
    /** replace <thickness> of given dimension by scrollban minimum thickness*/
    public Dimension getPreferredSize(int width, int height) {
      return new Dimension(width, minimumThickness);	
    }
      	
    /**
    * GetSize derived functions:specific overwrite for horizontal scrollbar */
    public Dimension getMinimumSize() {
      return new Dimension(lineUpSpan+lineDnSpan+minimumScreenSpan , minimumThickness);	
    }

    public Dimension getPreferredSize() {
      return new Dimension(lineUpSpan+lineDnSpan+screenRange , minimumThickness);	
    }

    public Dimension getCurrentSize() {
      return new Dimension(lineUpSpan+lineDnSpan+screenRange , minimumThickness);	
    }
      	
    public void setThickness(int width, int height) {
      currentThickness =(height>minimumThickness)?height:minimumThickness;
    }
      	
    //public int getRange(int x, int y) 	{ return x-2*RudolphScrollbarPeer.HSCROLL_LINEUPWIDTH;	}
    public boolean setRange(int width, int height) {
      return setRange(width);
    }
      	
    /**************************************************************************************************************************************/
    /**
    * basic functions : get field in which probing point exists
    * additional feature: next to the x-position(scrollbar length)
    * we also look if the y-position is within the range (offset , offset + height)
    **/
    protected int getField(int x, int y) {
      return getField(x);	
    }
    public synchronized int setActive(int x, int y) {
      currentActive = (y>barOffset)?getField(x):RudolphScrollbarPeer.FIELD_NONESELECTED;
      return currentActive;
    }

    /**
    set a new screen width and visibility after textarea rescaling
    To consider:
    -> the screen width relates both to screenRange(subtract line-up and line-dn buttons) and barSpan(divide by char width)
    -> the safety checks can also force changes in barPos and other variables
    The function returns the barPos value (changed or not, but since changing the width ALWAYS forces a redraw, this doesn't realy matter)
    */
    public int setScreenWidth(int totalwidth, int maximumlength)
    {
      //screenrange and barSpan from total width
      screenRange = totalwidth - lineUpSpan - lineDnSpan;
      barSpan = RudolphListPeer.getInnerLength(totalwidth);//part of the total screen width reserved for displaying
      barRange = maximumlength; //getScreenwidth(maximumlength);
      if (screenRange <=0) {
        // range too small for end boxes:  scrollbar becomes crippled
        screenRange = 0;
        screenSpan = 0;
        screenPos = 0;        			
        crippledSpan = totalwidth;			
      }
      else {
        //set new screen range and visibility bar span
        crippledSpan = -1;			
        //safety check
        if(barSpan > barRange) {
          barSpan = barRange;
          barPos = 0;
        }
        else if((barPos+barSpan)>barRange) {
          barPos = barRange - barSpan;
        }
        //calculate screen scrollbox span and -position
        setScreen();
        // one block = one screen width
        blockStep=(barSpan>lineStep)?barSpan-lineStep:lineStep;
      }
      //always return true for redrawing
      return barPos;
    }		
      	
    /**
    Equals lineUp/lineDn/pageUp/pageDn...but with specific names for horizontal text scrollbar
    * /
    public boolean charLeft() {
      return setBarPos(barPos - lineStep);
    }
    public boolean charRight() {
      return setBarPos(barPos + lineStep);
    }
    public boolean pageLeft() {
      return setBarPos(barPos - blockStep);
    }
    public boolean pageRight() {
      return setBarPos(barPos + blockStep);
    }
    /**************************************************************************************************************************************/
    /**
    * paint command
    */
    public void paint(Graphics g) {
      if(crippledSpan<0) {
        RudolphScrollbarPeer.paintHScrollbar(0,barOffset,minimumThickness,paintedScreenPos,screenSpan,screenRange, currentActive,barColors,g);	
      }
      else {
        RudolphScrollbarPeer.paintCrippledHScrollbar(0,barOffset,minimumThickness, crippledSpan, currentActive,barColors,g);	
      }
    }
  }









  /****************************************************************************************************************************************/
  //inner classes scrollbar painters : vertical
  /****************************************************************************************************************************************/
  /****************************************************************************************************************************************/
  class VListScrollPainter extends ScrollPainter {
    /**
    * Constructor : set height, lineup/lineDn and minimum box width for horizontal scrollbar
    */
    public VListScrollPainter(){
      super(RudolphScrollbarPeer.VSCROLL_WIDTH, RudolphScrollbarPeer.VSCROLL_LINEUPHEIGHT,
      RudolphScrollbarPeer.VSCROLL_LINEDNHEIGHT, RudolphScrollbarPeer.VSCROLL_MINIMUMBOXHEIGHT);
    }
      	
    /**
    * scrollbar width and scrollbar position out of (x,y) dimension
    * the abstract ScrollPainter classes filled in for horizontal scrollbar
    */
    /** (horizontal bar) x- position in scrollbar from point (x,y) */
    public int getPos(int x, int y) {
      return y;
    }
    /** (horizontal bar) scrollbar height as thickness from dimension (width, height)*/
    public int getThickness(int width, int height) {
      return width;
    }
    /**scrollbar length and thickness to (width, height) dimension for horizontal and vertical scrollbar*/
    protected Dimension getSize(int scrollbarlength, int scrollbarthickness) {
      return new Dimension(scrollbarthickness, scrollbarlength);
    }
    /** replace <thickness> of given dimension by scrollban minimum thickness*/
    public Dimension getPreferredSize(int width, int height) {
      return new Dimension(minimumThickness,height);	
    }
      	
    /**
    * GetSize derived functions:specific overwrite for horizontal scrollbar
    */
    public Dimension getMinimumSize() {
      return new Dimension(minimumThickness, lineUpSpan+lineDnSpan+minimumScreenSpan);	
    }
    public Dimension getPreferredSize() {
      return new Dimension(minimumThickness, lineUpSpan+lineDnSpan+screenRange);	
    }
    public Dimension getCurrentSize() {
      return new Dimension(minimumThickness, lineUpSpan+lineDnSpan+screenRange);	
    }
      	
    public void setThickness(int width, int height) {
      currentThickness =(width>minimumThickness)?width:minimumThickness;
    }
      	
    public boolean setRange(int width, int height) {
      return setRange(height);
    }
      	

    /**
    set a new screen height and visibility after textarea rescaling
    To consider:
    -> the screen width relates both to screenRange(subtract line-up and line-dn buttons) and barSpan(divide by char width)
    -> the safety checks can also force changes in barPos and other variables
    The function returns the barPos value (changed or not, but since changing the width ALWAYS forces a redraw, this doesn't realy matter)
    */
    public int setScreenHeight(int totalheight, int linespan, int linerange) {
      //screenrange and barSpan from total width
      screenRange = totalheight - lineUpSpan - lineDnSpan;
      barSpan = linespan;
      barRange = linerange;
      if (screenRange <=0) {
        // the scrollbar is 'crippled' : soo small to house at least a line-up and a line-down box
        screenRange = 0;
        screenSpan = 0;
        screenPos = 0;          			
        crippledSpan = totalheight;			
      }
      else {
        //set new screen range and visibility bar span
        crippledSpan = -1;			
        //safety check
        if(barSpan > barRange) {
          barSpan = barRange;
          barPos = 0;
        }
        else if((barPos+barSpan)>barRange) {
          barPos = barRange - barSpan;
        }
        //calculate screen scrollbox span and -position
        setScreen();
        //block width = visible lines
        blockStep=(linespan>1)?linespan-1:1;
      }
      //always return true for redrawing
      return barPos;
    }	

    /**************************************************************************************************************************************/
    /**
    * basic functions : get field in which probing point exists
    * Special case: we take into account the offset
    **/
    protected int getField(int x, int y) {
      return getField(y);	
    }

    public synchronized int setActive(int x, int y) {
      currentActive = (x>barOffset)?getField(y):RudolphScrollbarPeer.FIELD_NONESELECTED;	
      return currentActive;
    }

    /**
    * move the scrollbar (and recalculate barPos if needed)
    * /
    public boolean moveScreenPos(int dx, int dy) {	return (dy!=0)?moveBar(dy):false;	}
    //we use moveBar(dy) directly
    /**************************************************************************************************************************************/
    /**
    * paint command
    */
    public void paint(Graphics g) {
      if(crippledSpan<0) {
        RudolphScrollbarPeer.paintVScrollbar(barOffset,0,minimumThickness, paintedScreenPos, screenSpan, screenRange, currentActive, barColors, g);	
      }
      else {
        RudolphScrollbarPeer.paintCrippledVScrollbar(barOffset,0,minimumThickness, crippledSpan, currentActive, barColors, g);	
      }
    }
  }

}