/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


// Author: N.Oberfeld
// Created: 2001/03/28
// version: 1.01.01  2001/10/01
// =>One continuous thread, simplified run, scrollbox dragging handled by thread just as well


package java.awt;

import java.awt.event.*; //AWT event listeners
import java.awt.peer.*; //RudolphPeer drawing algorithms

/****************************************************************************************************************************************/
/**
* Scrollbar wrapper: handles existence, movement commands,events of the scrollbar.
* the main painting job is passed on tho the RectangleComponents and rectanglePainters
*/
/****************************************************************************************************************************************/

//public class Scrollbar extends Component  implements Runnable, MouseListener, MouseMotionListener, Adjustable
//wonka throws exception when deriving from a Component, dreive from a panel instead

public class Scrollbar extends Panel  implements MouseListener, MouseMotionListener, Adjustable
{
  /**
  * variables
  */
  final static int DEFAULTVALUE           = 0;
  final static int DEFAULTVISIBLEAMOUNT   = 10;
  final static int DEFAULTMINIMUM         = 0;
  final static int DEFAULTMAXIMUM         = 100;
  final static int DEFAULTUNITINCREMENT   = 1;
  final static int DEFAULTBLOCKINCREMENT  = 10;
  final static int DEFAULTORIENTATION     = Adjustable.VERTICAL;

  //all variables are already used by the scrollPainter, except for the minimum
  private int scrollbarMinimum;

  //our scrollpainter
  private ScrollPainter thePainter;

  //scrollbox pointer last position
  private Point lastMousePosition;
  //scrollbox mover thread
  final static ScrollRunner mouseEventRunner = new ScrollRunner();

  // adjustment listener
  public transient AdjustmentListener multiListener;
  	
  // painting buffer
  private Image backgroundImage;
  private Graphics backgroundGraphics;

  /****************************************************************************************************************************************/
  /**
  * Constructors
  */
  /** Default values: vertical scrollbar, range 0-100, visible scrollbox 1, */
  public Scrollbar() {
    this(DEFAULTORIENTATION, DEFAULTVALUE, DEFAULTVISIBLEAMOUNT, DEFAULTMINIMUM, DEFAULTMAXIMUM);   //default values
  }

  /** Default values but speccified orientation: horizontal or vertical */
  public Scrollbar(int orientation) {
    this(orientation, DEFAULTVALUE, DEFAULTVISIBLEAMOUNT, DEFAULTMINIMUM, DEFAULTMAXIMUM);   //default values
  }

  /** full initialisation (generous autochecks on min, max and value)**/
  public Scrollbar(int orientation, int value, int visible, int minimum, int maximum) {
    //build scrollbar
    if(orientation == Adjustable.HORIZONTAL) {
      thePainter = new HScrollPainter();
    }
    else {
      thePainter = new VScrollPainter();
    }

    // minimum value locally, other values to painter
    if(minimum<maximum) {
      scrollbarMinimum = minimum;
      thePainter.setBarValues(value-minimum, visible, maximum-minimum );
    }
    else {
      scrollbarMinimum = maximum;
      thePainter.setBarValues(value-maximum, visible, minimum-maximum );
    }
    // immediately set foreground and background colors
    thePainter.setBarColors(RudolphPeer.getBarColors());

    //mouse listener
    addMouseListener(this);
    addMouseMotionListener(this);
    //last mouse click point
    lastMousePosition = new Point(0,0);
    // painting buffer
    backgroundImage = null;
    backgroundGraphics = null;
    // Adjustment event listener to fire events to
    multiListener = null;
  }

  /****************************************************************************************************************************************/
  /**
  * add listener
  */
  public void addAdjustmentListener(AdjustmentListener newlistener) {
    multiListener = AWTEventMulticaster.add(multiListener, newlistener);
  }

  /****************************************************************************************************************************************/
  /**
  * Remove listener
  */
  public void removeAdjustmentListener(AdjustmentListener newlistener) {
    multiListener = AWTEventMulticaster.remove(multiListener, newlistener);
  }


  /****************************************************************************************************************************************/
  /**
  * overrides Component.processEvent so sub- and superclasses have their AdjustmentEvents dispatched through our (multi-)listener
  */
  protected void processEvent(AWTEvent e) {
    if(e instanceof  AdjustmentEvent) {
      if(multiListener != null) {
        multiListener.adjustmentValueChanged((AdjustmentEvent)e);
      }
    }
    else {
      super.processEvent(e);
    }
  }


  /****************************************************************************************************************************************/
  /**
  * Process adjustmentevent as described in Adjustable interface
  */
  protected void processAdjustmentEvent(AdjustmentEvent e) {
    if(multiListener != null) {
      multiListener.adjustmentValueChanged(e);
    }
  }



  /****************************************************************************************************************************************/
  /**
  * Our own (package-protected) version using the scrollbars own variables where possible
  */
  void processAdjustmentEvent(int type) {
    AdjustmentEvent e = new AdjustmentEvent(this, AdjustmentEvent.ADJUSTMENT_VALUE_CHANGED, type, thePainter.getBarPos()+scrollbarMinimum);
    if(multiListener != null) {
      multiListener.adjustmentValueChanged(e);
    }
    //tell the screen
    repaint();
  }

  /****************************************************************************************************************************************/
  /**
  * Extends the Component's setColor function to pass the colors further to the scrollPainter
  */
  public void setBackground(Color c) {
    super.setBackground(c); // component
    thePainter.setBarColors(c, getForeground());
  }
  	
  public void setForeground(Color c) {
    super.setForeground(c); // component
    thePainter.setBarColors(getBackground(), c);
  }

  /****************************************************************************************************************************************/
  /**
  * Get /set data items and mirror this to the barScreen
  */
  /**
  * Orientation:
  */
  public int  getOrientation() {
    return (thePainter instanceof HScrollPainter)?Adjustable.HORIZONTAL:Adjustable.VERTICAL;
  }

  public void setOrientation(int orientation) {
    //security : no need to rebuild a bar if already the right orientation
    if(orientation == getOrientation()) {
      return;
    }
    //get the data from the old bar	
    int pos = thePainter.getBarPos();
    int span = thePainter.getBarSpan();
    int range = thePainter.getBarRange();
    //(re)build the scrollbar barScreen for new orientation
    if(orientation == Adjustable.HORIZONTAL) {
      thePainter = new HScrollPainter();
    }
    else {
      thePainter = new VScrollPainter();
    }
    //(re)calculate position and size for scrollbox
    thePainter.setBarValues(pos,span, range);
    //repaint the scrollbox
    repaint();
  }

  /**
  * Maximum value
  */
  public int  getMaximum() {
    return (thePainter.getBarRange() + scrollbarMinimum);
  }

  public void setMaximum(int maximum) {
    thePainter.setBarRange(maximum - scrollbarMinimum);
    //repaint the scrollbox
    repaint();
  }

  /**
  * Minimum value
  */
  public int  getMinimum() {
    return scrollbarMinimum;
  }

  public void setMinimum(int minimum) {
   //recalculate maximum and position to new minimum
   int pos = thePainter.getBarPos()+ scrollbarMinimum - minimum;
   int span = thePainter.getBarSpan();// not affected
   int range = thePainter.getBarRange()+ scrollbarMinimum - minimum;
   //new minimum
   scrollbarMinimum = minimum;
   //new barPos, barRange and according screen values
   thePainter.setBarValues(pos,span, range);
   //repaint the scrollbox
   repaint();
  }

  /**
  * current value
  */
  public int  getValue() {
    return (thePainter.getBarPos() + scrollbarMinimum) ;
  }

  public void setValue(int value) {
    thePainter.setBarPos(value - scrollbarMinimum);
    //repaint the scrollbox
    repaint();
  }

  /**
  * visible amount of scrollbox
  */
  public int  getVisibleAmount() {
    return thePainter.getBarSpan();
  }
  public void setVisibleAmount(int visible) {
    thePainter.setBarSpan(visible);
    //repaint the scrollbox
    repaint();
  }


  /**
  * All settings in one
  */
  public void setValues(int value, int visible, int minimum, int maximum) {
    //new minimum
    if(maximum > minimum) {
      scrollbarMinimum = minimum;
      thePainter.setBarValues(value - minimum, visible, maximum - minimum);
    }
    else {
      scrollbarMinimum = maximum;
      thePainter.setBarValues(value - maximum, visible, minimum - maximum);
    }
    //in any case:
    repaint();
  }
  /**
  * unit increment (this does not affect the current scrollbox, so no update is necessary)
  */
  public int  getUnitIncrement() {
    return thePainter.getLineStep();
  }
  public void setUnitIncrement(int increment) {
    thePainter.setLineStep(increment);
  }

  /**
  * block increment (this does not affect the current scrollbox, so no update is necessary)
  */
  public int  getBlockIncrement() {
    return thePainter.getBlockStep();
  }
  public void setBlockIncrement(int increment) {
    thePainter.setBlockStep(increment);
  }


  /****************************************************************************************************************************************/
  /**
  * Get the scrollbar sizes for the screen build commands
  */
  /**
  * minimum size
  */
  public Dimension getMinimumSize() {
    return thePainter.getMinimumSize();	
  }

  /**
  * preferred size
  */
  public Dimension getPreferredSize() {
    return thePainter.getPreferredSize(getSize().width,getSize().height);	
  }

  /**
  * given screen size to ScrollbarPainter scrollbar size
  */
  public void setScrollbarSize(Dimension size) {
    //set range and recalculate position and span
    thePainter.setRange(size.width, size.height);
    //set new thickness
    thePainter.setThickness(size.width, size.height);
  }

  /*********************************************************
  *	Mouse listener forwarding to fire the Adjustment events
  */
  public  void mouseClicked(MouseEvent e) {
    //System.out.println("mouse clicked ("+e.getX()+", "+e.getY()+")");
  }

  public  void mousePressed(MouseEvent e) {
    //set the clicked field active
    int active = thePainter.setActive(e.getX(),e.getY());
    // if scrollbox, note the position for mouse moving
    if(active==AdjustmentEvent.UNIT_DECREMENT && thePainter.lineUp()) {
      processAdjustmentEvent(AdjustmentEvent.UNIT_DECREMENT);
      mouseEventRunner.setRunner(thePainter,this);
    }
    else if(active==AdjustmentEvent.UNIT_INCREMENT && thePainter.lineDn() ) {
      processAdjustmentEvent(AdjustmentEvent.UNIT_INCREMENT);
      mouseEventRunner.setRunner(thePainter,this);
    }
    else if(active==AdjustmentEvent.BLOCK_DECREMENT && thePainter.pageUp() ) {
      processAdjustmentEvent(AdjustmentEvent.BLOCK_DECREMENT);
      mouseEventRunner.setRunner(thePainter,this);
    }
    else if(active==AdjustmentEvent.BLOCK_INCREMENT && thePainter.pageDn() ) {
      processAdjustmentEvent(AdjustmentEvent.BLOCK_INCREMENT);
      mouseEventRunner.setRunner(thePainter,this);
    }
    else if(active==AdjustmentEvent.TRACK ) {
      lastMousePosition.setLocation(e.getX(),e.getY());
      //repaint to show the new active part
      repaint();
    }
  }

  public  void mouseReleased(MouseEvent e) {
    //no more fields active
    if(thePainter.isSelected() ) {
      mouseEventRunner.stopRunner(thePainter);
      repaint();
    }
  }

  public  void mouseEntered(MouseEvent e) {
    //System.out.println("mouse Entered ("+e.getX()+", "+e.getY()+")");
  }

  public  void mouseExited(MouseEvent e) {
    //mouse out of mouse panel counts as mouse released
    if(thePainter.isSelected() ) {
      mouseEventRunner.stopRunner(thePainter);
      repaint();
    }
  }


  public  void mouseDragged(MouseEvent e) {
    if(thePainter.getActive() == AdjustmentEvent.TRACK) {
      if(thePainter.moveScreenPos(e.getX()-lastMousePosition.x, e.getY()-lastMousePosition.y)) {
        // Ok, we were able to move the bar
        //tell the listeners  (this also repaints)
        processAdjustmentEvent(AdjustmentEvent.TRACK);
      }
      else if(thePainter.setActive(e.getX(),e.getY()) != AdjustmentEvent.TRACK) {
        thePainter.setNoSelected();
        repaint();
      }
      lastMousePosition.setLocation(e.getX(),e.getY());
    }
  }

  public  void mouseMoved(MouseEvent e) {
    //System.out.println("mouse Moved ("+e.getX()+", "+e.getY()+")");
  }


  /****************************************************************************************************************************************/
  /**
  * Paint the scrollbar
  */	
  public void paint(Graphics g) {
    // using double bufering, we don't need to clear the screen every time, so we can revert directly to Update and save the flickering
    update(g);
  }

  public void update(Graphics g) {
  //update the size if needed
  if(backgroundImage == null || !getSize().equals(thePainter.getCurrentSize() ) ) {
    // If the screen size has changed: calculate the scrollbar values for that size and initialise a new background screen
    thePainter.setRange(getSize().width, getSize().height);     // new length
    thePainter.setThickness(getSize().width, getSize().height); // new thickness
      		
    //new background image
    backgroundImage=createImage(getSize().width, getSize().height);
    if( backgroundImage!= null &&  backgroundGraphics!= null) {
      backgroundGraphics.dispose();
    }
    backgroundGraphics = backgroundImage.getGraphics();	
  }

  //update the colors if needed  	  	
  Color back = (getBackground() != null)? getBackground():SystemColor.scrollbar;
  Color font = (getForeground() != null)? getForeground():SystemColor.menuText;
  if(!back.equals(thePainter.getBackground()) || !font.equals(thePainter.getForeground())) {
    thePainter.setBarColors(getBackground(), getForeground() );			
  }

  //tell the painter to paint the bar in background image.
  thePainter.paint(backgroundGraphics);

  // paint background image on the screen
  g.drawImage(backgroundImage,0,0,this); 			
  }

  /****************************************************************************************************************************************/
  /**
  * Debug information
  */
  protected String paramString() {
    String param = "Scrollbar< orientation: "+getOrientation()+" min: "+scrollbarMinimum;
    param += "painter: <"+thePainter.toString()+"> ";
    param += "component <"+super.paramString()+">";
    return param;
  }

  public String toString() {
    String param = (thePainter instanceof HScrollPainter)?"Horizontal Scrollbar ":"Vertical SCrollbar";
    param+="Covering "+(thePainter.getBarPos()+scrollbarMinimum)+" to "+(thePainter.getBarPos()+thePainter.getBarSpan()+scrollbarMinimum);
    param+=" of span "+scrollbarMinimum+" to"+(thePainter.getBarRange()+scrollbarMinimum);
    return param;
  }






  /**************************************************************************************************************************************/
  /** internal scrollpainter class for horizontal scrollbar */
  /**************************************************************************************************************************************/
  /**************************************************************************************************************************************/
  class HScrollPainter extends ScrollPainter {
    /**
    * Constructor : set height, lineup/lineDn and minimum box width for horizontal scrollbar
    */
    public HScrollPainter() {
      super(RudolphScrollbarPeer.HSCROLL_HEIGHT, RudolphScrollbarPeer.HSCROLL_LINEUPWIDTH,
       RudolphScrollbarPeer.HSCROLL_LINEDNWIDTH, RudolphScrollbarPeer.HSCROLL_MINIMUMBOXWIDTH);
    }
      	
    /**
    * scrollbar width and scrollbar position out of (x,y) dimension
    * the abstract ScrollPainter classes filled in for horizontal scrollbar
    */
    /** (horizontal bar) x- position in scrollbar from point (x,y) */
    public int getPos(int x, int y) {
      return x;
    }
    /** (horizontal bar) scrollbar height as thickness from dimension (width, height)*/
    public int getThickness(int width, int height) {
      return height;
    }
    /**scrollbar length and thickness to (width, height) dimension for horizontal and vertical scrollbar*/
    protected Dimension getSize(int scrollbarlength, int scrollbarthickness) {
      return new Dimension(scrollbarlength, scrollbarthickness);
    }
    /** replace <thickness> of given dimension by scrollban minimum thickness*/
    public Dimension getPreferredSize(int width, int height) {
      return new Dimension(width, minimumThickness);	
    }
      	
    /**
    * GetSize derived functions:specific overwrite for horizontal scrollbar */
    public Dimension getMinimumSize() {
      return new Dimension(lineUpSpan+lineDnSpan+minimumScreenSpan , minimumThickness);	
    }
    public Dimension getPreferredSize() {
      return new Dimension(lineUpSpan+lineDnSpan+screenRange , minimumThickness);	
    }
    public Dimension getCurrentSize() {
      return new Dimension(lineUpSpan+lineDnSpan+screenRange , currentThickness);	
    }
      	
    public void setThickness(int width, int height) {
      currentThickness =(height>minimumThickness)?height:minimumThickness;
    }
      	
    //public int getRange(int x, int y) 	{ return x-2*RudolphScrollbarPeer.HSCROLL_LINEUPWIDTH;	}
    public boolean setRange(int width, int height) {
      return setRange(width);
    }
      	
    /**************************************************************************************************************************************/
    /**
    * basic functions : get field in which probing point exists
    * additional feature: next to the x-position(scrollbar length)
    * we also look if the y-position is within the range (offset , offset + height)
    **/
    protected int getField(int x, int y) {	
      return getField(x);
    }
    public synchronized int setActive(int x, int y) {
      currentActive = getField(x);
      return currentActive;
    }


    /**
    * move the scrollbar (and recalculate barPos if needed)
    */
    //	public boolean moveScreenPos(int dx  	{	 	return moveBar(dx;	}
    public boolean moveScreenPos(int dx, int dy) {
      return (dx!=0)?moveBar(dx):false;
    }

    /**************************************************************************************************************************************/
    /**
    * paint command
    */
    public void paint(Graphics g) {
      if(crippledSpan<0) {
        RudolphScrollbarPeer.paintHScrollbar(0,0,currentThickness, paintedScreenPos,screenSpan,screenRange, currentActive,barColors,g);	
      }
      else {
        RudolphScrollbarPeer.paintCrippledHScrollbar(0, 0, currentThickness, crippledSpan, currentActive, barColors, g);	
      }
    }
    //(end HScrollPainter)
  }








  /**************************************************************************************************************************************/
  /** internal scrollpainter class for vertical scrollbar */
  /**************************************************************************************************************************************/
  /**************************************************************************************************************************************/
  class VScrollPainter extends ScrollPainter {
    /**
    * Constructor : set height, lineup/lineDn and minimum box width for horizontal scrollbar
    */
    public VScrollPainter() {
      super(RudolphScrollbarPeer.VSCROLL_WIDTH, RudolphScrollbarPeer.VSCROLL_LINEUPHEIGHT,
       RudolphScrollbarPeer.VSCROLL_LINEDNHEIGHT, RudolphScrollbarPeer.VSCROLL_MINIMUMBOXHEIGHT);
    }
      	
    /**
    * scrollbar width and scrollbar position out of (x,y) dimension
    * the abstract ScrollPainter classes filled in for horizontal scrollbar
    */
    /** (horizontal bar) x- position in scrollbar from point (x,y) */
    public int getPos(int x, int y) {
      return y;
    }
    /** (horizontal bar) scrollbar height as thickness from dimension (width, height)*/
    public int getThickness(int width, int height) {
      return width;
    }
    /**scrollbar length and thickness to (width, height) dimension for horizontal and vertical scrollbar*/
    protected Dimension getSize(int scrollbarlength, int scrollbarthickness) {
      return new Dimension(scrollbarthickness, scrollbarlength);
    }
    /** replace <thickness> of given dimension by scrollban minimum thickness*/
    public Dimension getPreferredSize(int width, int height) {
      return new Dimension(minimumThickness,height);	
    }
      	
    /**
    * GetSize derived functions:specific overwrite for horizontal scrollbar */
    public Dimension getMinimumSize() {
      return new Dimension(minimumThickness, lineUpSpan+lineDnSpan+minimumScreenSpan);	
    }
    public Dimension getPreferredSize() {
      return new Dimension(minimumThickness, lineUpSpan+lineDnSpan+screenRange);	
    }
    public Dimension getCurrentSize() {
      return new Dimension(currentThickness, lineUpSpan+lineDnSpan+screenRange);	
    }
      	
    public void setThickness(int width, int height) {
      currentThickness =(width>minimumThickness)?width:minimumThickness;
    }
      	
    public boolean setRange(int width, int height) {
      return setRange(height);
    }
      	
    /************************************************************************************************************************************/
    /**
    * basic functions : get field in which probing point exists
    **/
    protected int getField(int x, int y) {
      return getField(y);	
    }
    public synchronized int setActive(int x, int y) {
      currentActive = getField(y);	
      return currentActive;
    }


    /**
    * move the scrollbar (and recalculate barPos if needed)
    */
    //	public boolean moveScreenPos(int dy) 	{	 	return moveBar(dy);
    public boolean moveScreenPos(int dx, int dy) {
      return (dy!=0)?moveBar(dy):false;	
    }
    /**************************************************************************************************************************************/
    /**
    * active part of screen
    */
    //	public void setActive(int x, int y) { CurrentActive = getField(x,y); }
      	
    /**
    * paint command
    */
    public void paint(Graphics g) {
      if(crippledSpan<0) {
        RudolphScrollbarPeer.paintVScrollbar(0,0,currentThickness,  paintedScreenPos,screenSpan,screenRange, currentActive,barColors,g);	
      }
      else {
        RudolphScrollbarPeer.paintCrippledVScrollbar(0, 0, currentThickness, crippledSpan, currentActive, barColors, g);	
      }
    }
    //(end VScrollPainter)
  }
  //end Scrollbar
}