/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


package java.awt;

import java.awt.event.*;

public class TextComponent extends Component implements KeyListener {
  
  /*
  ** common Variables for all derived classes
  */
  
  protected String text;
  protected int position;
  protected int selectionStart;
  protected int selectionStop;
  protected boolean editable = true;

  public transient TextListener textListener;

  public TextComponent() {
    super(); 
    addKeyListener(this);
  }

  /*
  ** Peer architecture
  ** @status not implemented
  ** @remark not implemented
  */
  
  /* 
  ** add/notify adding of system peer
  ** (addNotify is done by the specific subclasses TextField and TextArea)
  */

  /*
  ** remove/notify removal of system peer
  ** public void removeNotify();
  */

  /*
  ** set carret position
  */
  
  public void setCaretPosition(int newposition) {
  	if(newposition < 0) {
      position = 0;
  	}
  	else if(newposition > text.length()) {
  	  position = text.length();
  	}
  	else {
      position = newposition;
  	}
  }

  /*
  ** get carret position
  */
  
  public int getCaretPosition() {
    return position;
  }


  /*
  ** set selection start
  */

  public synchronized void setSelectionStart(int newstart) {
  	if(newstart >= 0 && newstart < text.length()) {
   	  selectionStart = newstart;
  	}
  }

  /*
  ** get selection start
  */
  
  public synchronized int getSelectionStart() {
  	return selectionStart;
  }

  /*
  ** set selection end
  */

  public synchronized void setSelectionEnd(int newstop) {
  	if(newstop >= 0 && newstop < text.length()) {
	    selectionStop = newstop;
	  }	
  }

  /*
  ** get selection end
  */
  
	public synchronized int getSelectionEnd() {
  	return selectionStop;
  }

  /*
  ** set selection start and end
  */
  
  public synchronized void select(int newstart, int newstop) {
  	setSelectionStart(newstart);
  	setSelectionEnd(newstop);
  }

  /*
  ** set selection start and end to select the whole text
  */
  
  public synchronized void selectAll() {
  	selectionStart = 0;
  	selectionStop = text.length();
  }

  /*
  ** Text, selected text
  * override setText in TextArea and TextField to show the desired text
  */
  public synchronized void setText(String newtext) {
    text = newtext;
    if(position > text.length()) position = text.length();
    if(selectionStart > text.length()) selectionStart = text.length();
    if(selectionStop > selectionStart) selectionStop = selectionStart;
  }

  public synchronized String getText() {
    return text;
  }

  public synchronized String getSelectedText()	{
  	if(selectionStart >= selectionStop) {
  		return "";
    }
  	else {
  		return text.substring(selectionStart, selectionStop);
    }
  }

  /*
  ** Editable
  */
  
  public synchronized void setEditable(boolean condition) {
    this.editable = condition;
  }

  public boolean isEditable() {
    return editable;
  }
  
  public String toString() {
  	String display = getClass().getName()+" text["+text;
  	display += (editable)?"] editable selection <":"] not editable selection <";
  	return display+selectionStart+", "+selectionStop+">";
  }

  protected String paramString() {
  	return(getClass().getName()+" text["+text+"], caret("+position+") selection<"+selectionStart+", "+selectionStop+">");
  }


  /*
  ** Handling and throwing TextEvents
  ** @status implemented

  /*
  ** Call on AWTEventMulticaster to return either the new listener or a Multicaster TextListener that contains it 
  */
  
  public synchronized void addTextListener(TextListener newlistener) {
    textListener = AWTEventMulticaster.add(textListener, newlistener);
  }

  /*
  ** Call on AWTEventMulticaster to remove the listener from the Multicaster TextListener that contains it 
  */
  
  public void removeTextListener(TextListener oldlistener) {
    textListener = AWTEventMulticaster.remove(textListener, oldlistener);
  }

  /*
  ** overrides Component.processEvent to forward TextEvents to our own listener or Multicaster  
  */
  
  protected void processEvent(AWTEvent e) {
    if(e instanceof  TextEvent) {
      if(textListener != null) {
        textListener.textValueChanged((TextEvent)e);
      }
    }
    else {
      super.processEvent(e);
    }
  }

  /*
  ** throw a textevent using the current textListener. 
  */
  
  protected void processTextEvent(TextEvent e) {
    if(textListener != null) {
      textListener.textValueChanged(e);
    }
  }

  /*
  ** Handle keyboard events.
  */

  public void keyPressed(KeyEvent ke) {
  }
  
  public void keyReleased(KeyEvent ke) {
  }
  
  public void keyTyped(KeyEvent evt) {
    switch(evt.getKeyCode()) {
      // key left: move carret left and set selection to carer
      case KeyEvent.VK_LEFT:
        if(evt.isShiftDown() && moveSelection(position-1)) {
          repaint();
        }
        else if(moveCaret(position-1)) {
          repaint();
        }
        break;

      // key right: move carret right and set selection to caret
      case KeyEvent.VK_RIGHT:
        if(evt.isShiftDown() && moveSelection(position+1)) {
          repaint();
        }
        else if(moveCaret(position+1)) {
          repaint();
        }
        break;

      // key right: move carret completely left
      case KeyEvent.VK_HOME:
        if(evt.isShiftDown() && moveSelection(0)) {
          repaint();
        }
        else if(moveCaret(0)) {
          repaint();
        }
        break;

      case KeyEvent.VK_END:
        if(evt.isShiftDown() && moveSelection(text.length())) {
          repaint();
          setCaretPosition(position); // triggers a repaint
        }
        else if(moveCaret(text.length())) {
          repaint();
        }
        break;

      case KeyEvent.VK_BACK_SPACE:
        if(selectionStart < selectionStop ){
          deleteSelection();
          repaint();
        }
        else if(deleteCaret(position-1)) {
          repaint();
        }
        break;

      case KeyEvent.VK_DELETE:
        if(selectionStart < selectionStop ){
          deleteSelection();
          repaint();
        }
        else if(deleteCaret(position)) {
          repaint();
        }
        break;

      case KeyEvent.VK_SHIFT:
      case KeyEvent.VK_CONTROL:
      case KeyEvent.VK_UP:
      case KeyEvent.VK_DOWN:
        break;

      default:
        if(selectionStart < selectionStop ){
          insertSelection(evt.getKeyChar());
          repaint();
        }
        else if(insertCaret(position, evt.getKeyChar()) ) {
          repaint();
        }
    }

    //repaint();=>setText or setCaret() should trigger that
  }

  /*
  ** Key to text helper classes
  */
  /** set caret to desired position, also collapse the selection */
  private boolean moveCaret(int newpos){
    if(newpos<0 || newpos>text.length() || newpos==position) {
      return false ;
    }
    //else {
    position = newpos;
    selectionStart = newpos;
    selectionStop = newpos;
    return true;
  }

  /** set selection start or stop to desired position, update the carret */
  private boolean moveSelection(int newpos){
    if(newpos<0 || newpos>text.length() || newpos==position) {
      return false ;
    }
    else if(position == selectionStart && newpos < selectionStop) {
      position = newpos;
      selectionStart = newpos;
      return true;
    }
    else if(position == selectionStart) { // && newpos > selectionStop) {
      position = newpos;
      selectionStart = selectionStop;
      selectionStop = newpos;
      return true;
    }
    else if(newpos > selectionStart) { //&& position == selectionStop)
      position = newpos;
      selectionStop = newpos;
      return true;
    }
    else { //if(newpos < selectionStart && position == selectionStop)
      position = newpos;
      selectionStop = selectionStart;
      selectionStart = newpos;
      return true;
    }
  }


  /** Delete the caracter after the given position, also set carret and zero selection to position */
  private boolean deleteCaret(int newpos){
    if(newpos<0 || newpos>=text.length()) {
      return false ;
    }
    //else {
    position = newpos;
    selectionStart = newpos;
    selectionStop = newpos;
    text = new String(new StringBuffer(text).deleteCharAt(newpos));

    return true;
  }

  /** Delete the caracter after the given position, also set carret and zero selection to position */
   private void deleteSelection(){
    text = new String(new StringBuffer(text).delete(selectionStart, selectionStop));
    position = selectionStart;
    //selectionStart = selectionStart;
    selectionStop = selectionStart;
  }

  /** Insert a new caracter after the given position, also set carret and zero selection just behind character added */
  private boolean insertCaret(int newpos, char c){
    if(newpos<0 || newpos>text.length()) {
      return false ;
    }
    //else {
    position = newpos+1;
    selectionStart = position;
    selectionStop = position;
    text = new String(new StringBuffer(text).insert(newpos,c));

    return true;
  }

  private void insertSelection(char c){
    StringBuffer buf =new StringBuffer(text);
    buf.delete(selectionStart, selectionStop);
    buf.insert(selectionStart,c);
    text = new String(buf);

    position = selectionStart;
    //selectionStart = selectionStart;
    selectionStop = selectionStart;
  }
}
