/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

package java.awt;

import java.awt.event.*;
import java.util.EventListener;

public class TextField extends TextComponent implements FocusListener, MouseListener, MouseMotionListener, KeyListener {

  private int      cols;
  private char     echo;
  private String   str;
  private boolean  focus;
  private int      xoffset = 0;
  private boolean  selecting = false;
  private int      selstart = 0;
  private int      selstop = 0;
  
  public TextField() {
    this("", 0);
  }
 
  public TextField(int cols) {
    this("", cols);
  }
  
  public TextField(String text) {
    this(text, 0);
  }
  
  public TextField(String text, int cols) {
    this.text = text;
    this.cols = cols;
    createPeer();
    addFocusListener(this);
    addMouseListener(this);
    addMouseMotionListener(this);
    addKeyListener(this);
  }

  private native void createPeer();
  

  /*
  ** @status As the action event of a textfield is triggered by the <return> key, not every functions is operational already
  */
   
  public synchronized void addActionListener(java.awt.event.ActionListener newlistener) {
    //AWTEventMulticaster.add(listeners, newlistener);
  }

  public synchronized void removeActionListener(java.awt.event.ActionListener oldlistener) {
    //AWTEventMulticaster.remove(listeners, oldlistener);
  }

  protected void processEvent(AWTEvent e) {
    if(e instanceof  ActionEvent) {
      processActionEvent((ActionEvent)e);
    }
    else {
      super.processEvent(e);
    }
  }

  protected void processActionEvent(java.awt.event.ActionEvent event) {
    // if(listeners != null) {
    //   listeners.processActionevent(event);
    // }
  }

  public void addNotify() {
    if (notified == false) {
      // createTextfieldPeer();
      super.addNotify();
    }
  }
 
  /*
  ** The echo character.
  */
  
  public boolean echoCharIsSet() {
    return (echo != 0) ? true : false;
  }
  
  public char getEchoChar() {
    return echo;
  }
  
  public void setEchoChar(char ch) {
    echo = ch;   
    setText(text);
  }  

  /*
  ** Columns.
  */
  
  public int getColumns() {
    return cols;
  }
  
  public void setColumns(int cols) {
    this.cols = cols;
  }

  /*
  ** Component Size.
  */
  
  public Dimension getMinimumSize() {
    return getMinimumSize(cols);
  }
  
  public Dimension getMinimumSize(int cols) {
    return getPreferredSize(cols);
  }
  
  public Dimension getPreferredSize() {
    return getPreferredSize(cols);
  }
  
  public Dimension getPreferredSize(int cols) {
    synchronized(getTreeLock()) {
      if (preferredSize == null || valid == false) {
        Font f = getFont();
        FontMetrics fm = getFontMetrics((f != null) ? f : Component.DEFAULT_FONT);

        int cx = Math.max(50, cols * 8);
        int cy = Math.max(30, 2 * fm.getHeight());

        preferredSize = new Dimension(cx, cy);
      }
    }
    return preferredSize;
  }

  /*
  ** Get/Set text
  */
  
  public synchronized String getText() {
    return (echo != 0) ? str : text;
  }
  
  public synchronized void setText(String text) {
    this.text = (text != null) ? text : "";
      // If the parameter is null then the text is set to the empty string "".

    if (echo != 0) {
      str = text;
      this.text = echo(str);
    }

    refresh(Component.REFRESH_LOCAL, Component.CACHE_NONE);
  }

  private String echo(String s) {
    StringBuffer sb = new StringBuffer(s.length());
    for (int i = 0; i < s.length(); i++) {
      sb.append(echo);
    }
    return new String(sb);
  }

  /*
  ** toString & paramString.
  */
  
  public String toString() {
    String display = getClass().getName() +" length = "+ cols +" cols text = ["+ text;
    display += (editable) ? "] editable selection <" : "] not editable selection <";
    return display+selectionStart+", "+selectionStop+">";
  }

  protected String paramString() {
    return (getClass().getName()+" cols="+ cols +" text["+ text +"], caret("+ position +") selection<"+ selectionStart +", "+ selectionStop +">");
  }


  /*
  ** paint.
  */

  public void paint(Graphics g) {
    int x = 0;
    int y = height / 2 - g.getFontMetrics().getHeight();
    int w = width - 5;
    int h = g.getFontMetrics().getHeight() + 9;
    int ystr = (h - g.getFontMetrics().getHeight()) / 2 + g.getFontMetrics().getAscent();

    int carretx = g.getFontMetrics().stringWidth(text.substring(0, getCaretPosition()));
   
    /*
    ** Make sure the caret is visible.
    */
    
    if(xoffset > x + w - carretx) {
      xoffset = x + w - carretx;
    }
    
    if(xoffset < x - carretx) {
      xoffset = x - carretx;
    }

    /*
    ** Draw the bounding box.
    */
      
    g.setColor(Color.white);
    g.fillRect(x, y, w, h);
      
    g.setColor(new Color(175, 175, 175));
    g.drawLine(x, y + h, x + w, y + h);
    g.drawLine(x + w, y, x + w, y + h);
    g.setColor(new Color(40, 40, 40));
    g.drawLine(x, y, x + w, y);
    g.drawLine(x, y, x, y + h);
     
    /*
    ** Draw the text.
    */
      
    g.setClip(x, y, w, h);
      
    g.setColor(Color.black);
    g.drawString(text, x + xoffset, y + ystr);

    /*
    ** Draw the selection (if there is any and we have focus)
    */

    if(getSelectionEnd() - getSelectionStart() > 0 && focus) {
      int sx = g.getFontMetrics().stringWidth(text.substring(0, getSelectionStart()));
      int ex = g.getFontMetrics().stringWidth(text.substring(0, getSelectionEnd()));

      g.setColor(new Color(80, 80, 80));
      g.fillRect(sx + xoffset, y + 3, ex - sx, h - 6);
        
      g.setColor(Color.white);
      g.setClip(sx + xoffset, y, ex - sx, h);
      g.drawString(text, x + xoffset, y + ystr);
        
      g.setClip(x, y, w, h);
      g.setColor(Color.black);
    }

    /*
    ** Draw the cursor (if we have focus).
    */

    if(focus) {
      g.drawLine(carretx + xoffset, y + 2, carretx + xoffset, y + h - 2);
      g.drawLine(carretx + xoffset + 1, y + 2, carretx + xoffset + 1, y + h - 2);
    }

  }

  /*
  ** Focus Events.
  */

  public void focusGained(FocusEvent fe) {
    focus = true;
    repaint();
  }
  
  public void focusLost(FocusEvent fe) {
    focus = false;
    repaint();
  }

  /*
  ** Mouse Events.
  */

  private int getNearestPos(int x) {
    int i = 0;
    Font f = getFont();
    FontMetrics fm = getFontMetrics((f != null) ? f : Component.DEFAULT_FONT);
    while(i <= text.length()) {
      if(x - xoffset < fm.stringWidth(text.substring(0, i))) {
        return i - 1;
      }
      i++;
    }
    return text.length();
  }

  private void doSelection() {
    if(selstart <= selstop) {
      select(selstart, selstop);
    }
    else {
      select(selstop, selstart);
    }
  }

  public void mouseEntered(MouseEvent me) {}
  public void mouseExited(MouseEvent me) {}
  public void mouseMoved(MouseEvent me) {}
  public void mouseClicked(MouseEvent me) {}
  
  public void mousePressed(MouseEvent me) {
    selstart = getNearestPos(me.getX());
    selstop = getNearestPos(me.getX());
    doSelection();
    setCaretPosition(selstart);
    repaint();
  }
  
  public void mouseReleased(MouseEvent me) {
    selstop = getNearestPos(me.getX());
    doSelection();
    setCaretPosition(selstop);
    repaint();
  }
  
  public void mouseDragged(MouseEvent me) {
    selstop = getNearestPos(me.getX());
    doSelection();
    setCaretPosition(selstop);
    repaint();
  }
  
  public void keyPressed(KeyEvent ke) {
  }
  
  public void keyReleased(KeyEvent ke) {
  }
  
  public void keyTyped(KeyEvent evt) {
    switch(evt.getKeyCode()) {
      case KeyEvent.VK_ENTER:
        break;

      default:
        super.keyTyped(evt);
    }
  }
  
}
