/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

package java.awt.image;

import java.awt.*;
import java.util.*;

public class PixelGrabber implements ImageConsumer {

  private int                status;
  private int                hints;
  private ColorModel         colorModel = null;
  private int                height = -1;
  private int                width = -1;
  private Hashtable          properties;
  private ImageProducer      producer;
  private int                sx, sy, sw, sh, off, scansize;
  private int[]              pixels;
  
  public PixelGrabber(Image img, int x, int y, int w, int h, int[] pix, int off, int scansize) {
    this(img.getSource(), x, y, w, h, pix, off, scansize);
  }

  public PixelGrabber(ImageProducer ip, int x, int y, int w, int h, int[] pix, int off, int scansize) {
    sx = x;
    sy = y;
    sw = w;
    sh = h;
    pixels = pix;
    this.off = off;
    this.scansize = scansize;
    producer = ip;
  }

  public PixelGrabber(Image img, int x, int y, int w, int h, boolean forceRGB) {
    sx = x;
    sy = y;
    sw = w;
    sh = h;
    pixels = null;
    off = 0;
    scansize = w;
    producer = img.getSource();
  }

  public synchronized void abortGrabbing() {
  }

  public synchronized ColorModel getColorModel() {
    return colorModel;
  }

  public synchronized int getHeight() {
    return height;
  }

  public synchronized int getWidth() {
    return width;
  }

  public synchronized Object getPixels() {
    return pixels;
  }

  public synchronized int getStatus() {
    return status;
  }

  public synchronized boolean grabPixels() throws InterruptedException {
    startGrabbing();
    while(status != STATICIMAGEDONE) {
      Thread.currentThread().sleep(1);
    }
    return true;
  }

  public synchronized boolean grabPixels(long ms) throws InterruptedException {
    startGrabbing();
    while(status != STATICIMAGEDONE) {
      Thread.currentThread().sleep(1);
    }
    return true;
  }

  public synchronized void startGrabbing() {
    producer.startProduction(this); 
  }

  /*
  ** The following are from the ImageConsumer interface.
  */

  public void imageComplete(int status) {
    this.status = status;
    if(status == STATICIMAGEDONE) {
    }
  }
  
  public void setColorModel(ColorModel model) {
    colorModel = model;
  }
  
  public void setDimensions(int w, int h) {
    width = w;
    height = h;
  }
  
  public void setHints(int hints) {
    this.hints = hints;
  }

  public void setPixels(int x, int y, int w, int h, ColorModel model, byte[] pixels, int offset, int scansize) {
    for(int j=0; j<h; j++) {
      for(int i=0; i<w; i++) {
        // byte pixel = pixels[j * scansize + i + offset];
        // this.pixels[(j + y) * width + (i + x)] = convertRGB(model.getRed(pixel), model.getGreen(pixel), model.getBlue(pixel));
      }
    }
  }

  public void setPixels(int x, int y, int w, int h, ColorModel model, int[] pixels, int offset, int scansize) {
    for(int j=0; j<h; j++) {
      for(int i=0; i<w; i++) {
        // int pixel = pixels[j * scansize + i + offset];
        // this.pixels[(j + y) * width + (i + x)] = convertRGB(model.getRed(pixel), model.getGreen(pixel), model.getBlue(pixel));
      }
    }
  }

  public void setProperties(Hashtable properties) {
    this.properties = properties;
  }

}
