/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


// Author: N.Oberfeld
// Created: 2001/04/11
// version: 2.02.00  2001/07
// paintList to buffer/starts/lengths format (using drawChars instead of drawString)
// version: 2.02.01  2001/08
// refining algorithms and rewriting getHeight/getLabel/getLength functions
// version: 2.02.02  2001/08/21
// Cleanup code


package java.awt.peer;

import java.awt.Graphics;
import java.awt.Color;
import java.awt.SystemColor;
import java.awt.Font;
import java.awt.FontMetrics;

/**
*
*LIST PAINT COMMANDS FOR ALL RUDOLPH LIST USERS
**/
public class RudolphListPeer extends RudolphPeer {
  /**
  *static variables
  **/
  public static final Font DEFAULT_FONT = new Font("courP14", 0, 14); //new Font("courP17", 0, 17);
  public final static int CHARHEIGHT = 14; //definition for fixed font
  public final static int CHARWIDTH  = 8;  //definition for fixed font

  public final static int BORDER = 15; //border around list in which mouse-drag scrolling
  public final static int LABELASCENT = 3; //definition for variable font: reserve X pixels between label upper line and font top
  public final static int LABELDESCENT = 4; //definition for variable font: reserve X pixels between font line and label lower line
  public final static int LABELOFFSET = 5;  //definition space between left border and start of text

  /****************************************************************************************************************************************/
  /**
  *	paint a text area, given  the lines to display as a number of arrays char[] text, int[]offsets, int[] length
  * whereas a line i is defined as being the characters text[offsets[i]] to text[offset[i]+length[i]]
  **/
  public static void paintList (
                                 int x0, int y0, int width, int height,
                                 char[] text, int[] offsets, int[] length,
                                 int lineoffset, boolean[] selections, int current,
                                 Font f, FontMetrics fm, Color[] peercolors, Graphics g
                               ) { 		
    // paint list box
    g.setColor(peercolors[1]); //darker
    g.drawLine(x0+1,y0+1,x0+width-1, y0+1);
    g.drawLine(x0+1,y0+1,x0+1, y0+height-1);
    g.setColor(peercolors[2]); //light
    g.drawLine(x0+width-1,y0+1,x0+width-1, y0+height-1);
    g.drawLine(x0+1,y0+height-1,x0+width-1, y0+height-1);
      	
    //calculate some screen variables we'll use later on...
    // height of a label: height of the text and ascent/descent variables
    int labelheight = LABELASCENT+fm.getHeight()+LABELDESCENT;

    //x,y coords for text
    int xt=x0+LABELOFFSET;
    int yt=y0+2+LABELASCENT+fm.getHeight()-fm.getDescent(); //start at y0+2, draw text under label+LABELASCENT

    //x,y coords for lines
    int xl = x0+2;
    int xr = x0+width-2;//right line end
    int yl=y0+2; //yo+2 for offset +1 for upper line
    //lowest line of a label stopping at lowest position y0+height-2 of the screen
    // as we compare with yl (which starts at y0+3), the lowest yl should start at (y0+3) + (height-2-2) - labelheight
    int ymax = y0+height-labelheight;

    // width & height for label rects
    int wl=width-4;
    int hl=labelheight-2;
    //draw the list
    if(text != null)
    {
      //font
      g.setFont(f);
      // line by line...
      for(int i=lineoffset; i<offsets.length && yl<=ymax; i++) {
        if(i==current) {
          //label to draw is currently selected :
          // fill label rectangle in deep dark
          g.setColor(peercolors[3]);
          g.fillRect(xl,yl,wl,hl);
          // draw upper line slightly lighter then label beckground (this is slightly lighter then deep dark... is slightly dark)
          // and advance current label start
          g.setColor(peercolors[1]); //slightly darker
          g.drawLine(xl,yl,xr, yl);
          yl+=labelheight;
          //draw text
          g.setColor(peercolors[2]); //light
          g.drawChars(text,offsets[i],length[i], xt, yt);
          yt+=labelheight;
          //draw lower line
          g.drawLine(xl,yl-2, xr,yl-2);	  		
          g.drawLine(xr,yl-2, xr,yl-labelheight);	  		
        }
        else if(selections != null && selections[i]) {
          //label to draw is one of the list selections, though not the current one
          //fill label rectangle in slightly darker selection color
          g.setColor(peercolors[1]); //slightly darker
          g.fillRect(xl,yl,wl,hl);
          // draw upper line deep dark... and advance current label start
          g.setColor(peercolors[3]);
          g.drawLine(xl,yl,xr, yl);
          g.drawLine(xl,yl, xl,yl-2+labelheight);	  		
          yl+=labelheight;
          //draw text
          g.setColor(peercolors[2]); //light
          g.drawChars(text,offsets[i],length[i], xt, yt);
          yt+=labelheight;
          //draw lower line
          g.drawLine(xl,yl-2, xr,yl-2);	  		
          g.drawLine(xr,yl-2, xr,yl-labelheight);	  		
        }
        else {
          //Simple line of text: foreground text on background label
          // no need to draw a selection box here: background is already in background color
          //draw upper line in light frame color
          g.setColor(peercolors[2]); //light
          g.drawLine(xl,yl, xr,yl);
          g.drawLine(xl,yl, xl,yl-2+labelheight);	  		
          yl+=labelheight;
          //draw text
          g.setColor(peercolors[4]); //foreground
          g.drawChars(text,offsets[i],length[i], xt, yt);
          yt+=labelheight;
          //draw lower line in dark frame color
          g.setColor(peercolors[3]); //deep dark
          g.drawLine(xl,yl-2, xr,yl-2);	  		
          g.drawLine(xr,yl-2, xr,yl-labelheight);	  		
        }
      }
    }
    		
  }

  /****************************************************************************************************************************************/
  /**
  * Get the height of a viewport containing <x> lines
  * (remember to update this calculation when changing the display & border definitions)
  */
  public static int getHeight(int lines, FontMetrics fm) {
    // inner viewport stretches from <y0+2> to <yo+height-2>, so the total viewport height will be (lines* total label height) +4
    return(lines*(LABELASCENT+fm.getHeight()+LABELDESCENT)+4);
  }
  /****************************************************************************************************************************************/
  /**
  * Get the offset to the viewport (in pixels) of line number <x>
  * (remember to update this calculation when changing the display & border definitions)
  */
  public static int getLabelOffset(int line, FontMetrics fm) {
    // inner viewport starts at <y0+2>, so the total label offset will be (lines* total label height) +2
    return(line*(LABELASCENT+fm.getHeight()+LABELDESCENT)+2);
  }

  /****************************************************************************************************************************************/
  /**
  * get the number of labels to fit in a given viewportheight
  * (remember to update this calculation when changing the display & border definitions)
  */
  public static int getLabels(int height, FontMetrics fm) {
    // inner viewport stretches from <y0+2> to <yo+height-2>, so the total number of labels will be (height-4)/ total label height
    return(height-4)/(LABELASCENT+fm.getHeight()+LABELDESCENT);
  }
  /****************************************************************************************************************************************/
  /**
  * get the number of the label on a certain position in the viewport
  * (remember to update this calculation when changing the display & border definitions)
  */
  public static int getLabel(int y, FontMetrics fm) {
    // inner viewport starts at <y0+2> so the number of the label at position y will be (y-2)/ total label height
    return(y-2)/(LABELASCENT+fm.getHeight()+LABELDESCENT);
  }

  /****************************************************************************************************************************************/
  /**
  * Calculating the line contents in char[]/start/length format, we do not need pos-to-string and string-to-pos calculations,
  * but can do everything using RudolphPeer.getCarretPosition() and fm.getCharsWidth(buffer,pos,len)/ len*fm.getMaxAdvance().
  * The only thing we need is tho calculate the 'inner' length the text can be displayed in out of the ListPeer screen width
  * and vice versa
  * (in THIS version, the effective inner length is the region from <x0+LABELOFFSET> to <x0+screenwidth-2>, if changing
  * the display of the List Peer, remenber to update this calculations to the new values)
  */
  public static int getInnerLength(int screenwidth) {
    // inner screen length stretches from <x0+LABELOFFSET> to <x0+width-2>
    return screenwidth-LABELOFFSET-2;
  }

  public static int getScreenwidth(int innerlength) {
    // inner screen length stretches from <x0+LABELOFFSET> to <x0+width-2>
    return innerlength+LABELOFFSET+2;
  }

}