/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/* $Id: GIFImageSource.c,v 1.4 2002/02/20 09:03:19 dfrederickx Exp $ */

#include <string.h>
#include <stdio.h>

#include "awt-classes.h"  // F_Component_*, F_Graphics_*
#include "realms.h"
#include "threads.h"       // currentWonkaThread
#include "ts-mem.h"       // allocMem(), releaseMem()
#include "stack.h"
#include "heap.h"
#include "arrays.h"        // Array stuff

#ifdef GIF_SUPPORT

#define FLAG_BITS        0x0007
#define FLAG_PALETTE     0x0080
#define FLAG_INTERLACED  0x0040

#define BLOCK_IMAGE      0x002c
#define BLOCK_EXT        0x0021
#define BLOCK_NULL       0x0000

#define EXT_TEXT         0x0001
#define EXT_CONTROL      0x00f9
#define EXT_COMMENT      0x00fe
#define EXT_APPLICATION  0x00ff

typedef struct gif_Params {
  w_ushort   width;
  w_ushort   height;
  w_ushort   bits;
  w_ubyte    flags;
  w_ubyte    back;
  w_ubyte    aspect;
  w_ubyte    pal[768];
} gif_Params;

typedef gif_Params  *gif_params;

static inline void gif_next_code(w_ubyte **p, w_ubyte **q, w_ubyte **data) {
  w_int block_size;
  (*p)++;
  if(*p >= *q) {
    block_size = *((*data)++);
    *p = *data;
    *q = *data + block_size;
    *data += block_size;
  }
}

w_int gif_unpack(w_ubyte *data, w_ubyte *image, w_word bits, gif_params params) {

  w_int    code_size = bits + 1;             // Current code size in bits.
  w_int    next_code = (1 << bits) + 2;      // Next entry.
  w_short  this_code;                        // Current entry.
  w_int    prev_code = -1;                   // Previous entry.
  w_int    prev_token = -1;                  // Last decoded entry.
  w_int    current_code;                     // Current code.
  w_int    bits_left = 8;                    // Bits left in *p;
  w_ubyte  *p = NULL;                        // Current byte;
  w_ubyte  *q = NULL;                        // Last byte + 1;
  w_ubyte  *u;
  w_word   current_line = 0;
  w_word   current_pixel = 0;
  w_word   current_pass = 0;

  w_ubyte  *first_stack;
  w_ubyte  *last_stack;
  w_int    *stack;
  w_int    result = 0;

  static w_int word_mask[] = { 0x0000, 0x0001, 0x0003, 0x0007, 0x000f, 0x001f, 0x003f, 0x007f,
                               0x00ff, 0x01ff, 0x03ff, 0x07ff, 0x0fff, 0x1fff, 0x3fff, 0x7fff };
                               
  static w_int interlaced[] = { 8, 8, 4, 2, 0 };
  static w_int start[] = { 0, 4, 2, 1, 0 };
  
  if(bits < 2 || bits > 8) return 0;

  first_stack = x_mem_calloc(4096);
  last_stack = x_mem_calloc(4096);
  stack = x_mem_calloc(4096 * sizeof(w_int));
  
  woempa(5, "bits: %d\n", bits);

  while(1) {
    woempa(5, "bits_left: %d\n", bits_left);
    if(bits_left == 8) {
      gif_next_code(&p, &q, &data);
      bits_left = 0;
    }

    this_code = *p;
    current_code = code_size + bits_left;
    
    woempa(5, "this_code: %d\n", this_code);
    woempa(5, "current_code: %d\n", current_code);
    
    /*
    ** Get the next code. (code_size bits wide)
    */
    
    if(current_code <= 8) {
      *p >>= code_size;
      bits_left = current_code;
    }
    else {
      gif_next_code(&p, &q, &data);
      this_code |= *p << (8 - bits_left);
      if(current_code <= 16) {
        bits_left = current_code - 8;
        *p >>= bits_left;
      }
      else {
        gif_next_code(&p, &q, &data);
        this_code |= *p << (16 - bits_left);
        bits_left = current_code - 16;
        *p >>= bits_left;
      }
    }

    /*
    ** Mask it so that we have only <code_size> bits
    */
    
    this_code &= word_mask[code_size];
    
    current_code = this_code;

    /*
    ** If we arrive at a code (1 << bits) + 1
    ** (e.g. 257 if we have 8 bits) we are at the end
    ** of the image block.
    */
    
    if(this_code == (1 << bits) + 1) {
      result = 1;
      break;
    }

    if(this_code > next_code) {

      /*
      ** Error, should not happen...
      */

      woempa(9, "Error\n");
      break;
    }

    /*
    ** If we arrive at a code (1 << bits)
    ** (e.g. 256 if we have 8 bits) we need to reinitialize
    ** the string table and start over with the original
    ** code_size.
    */
    
    if(this_code == (1 << bits)) {
      next_code = (1 << bits) + 2;
      code_size = bits + 1;
      prev_token = -1;
      prev_code = -1;
      continue;
    }

    u = first_stack;

    if(this_code == next_code) {
      if(prev_code == -1) {

        /*
        ** Bad code...
        */

        woempa(9, "Bad code\n");
        break;
      }

      *u++ = prev_token;
      this_code = prev_code;
    }

    /*
    ** Search the string table.
    */
    
    while(this_code >= (1 << bits)) {
      *u++ = last_stack[this_code];
      this_code = stack[this_code];
    }

    prev_token = this_code;

    while(1) {

      /*
      ** Store the decoded pixel in the image.
      */

      woempa(5, "line : %d,  pixel: %d\n", current_line, current_pixel);
      
      image[4 * (current_line * params->width + current_pixel) + 0]= 0xff;
      image[4 * (current_line * params->width + current_pixel) + 3] = params->pal[this_code * 3 + 0];
      image[4 * (current_line * params->width + current_pixel) + 2] = params->pal[this_code * 3 + 1];
      image[4 * (current_line * params->width + current_pixel) + 1] = params->pal[this_code * 3 + 2];
      
      current_pixel++;
      
      if(current_pixel >= params->width) {

        /*
        ** We crossed the line, go to the beginning of the next line.
        */
        
        current_pixel = 0;
        
        if(params->flags & FLAG_INTERLACED) {

          /*
          ** The image is interlaced. Go to the next interlaced line.
          */
          
          current_line += interlaced[current_pass];
          
          if(current_line >= params->height) {

            /*
            ** Jumped to far... Go to the next pass and start again
            ** from the top.
            */
            
            current_pass++;
            current_line = start[current_pass];
          }
        }
        else {

          /*
          ** Image is not interlaced, go to the next line.
          */
          
          current_line++;
        }
      }
            
      if(u <= first_stack) break;
      this_code = *--u;
    }

    if(next_code < 4096 && prev_code != -1) {

      stack[next_code] = prev_code;
      last_stack[next_code] = prev_token;
      next_code++;

      /*
      ** If the next code is larger than the current code_size,
      ** we need to increment the code_size.
      */
      
      if(next_code >= (1 << code_size) && code_size < 12) {
        code_size++;
      }
    }
    
    prev_code = current_code;
    
  }

  x_mem_free(first_stack);
  x_mem_free(last_stack);
  x_mem_free(stack);

  return result;
   
}

w_void GIFImageSource_readImage(JNIEnv *env, jobject thisObject, jbyteArray imageFile) {
  
  w_thread   thread = currentWonkaThread;
  w_ubyte    *image = instance2Array_byte(imageFile);
  w_int      length = instance2Array_length(imageFile);
  w_int      *result;
  w_word     len;
  w_word     position = 0;
  w_ubyte    keep_going = 1;
  gif_params params;
  w_ubyte    code_size;
  w_ubyte    block_size;
  w_ubyte    *image_data = NULL;
  w_int      unpack_result = 0;

  /*
  ** Allocate memory for the parameters
  */ 
  
  params = x_mem_alloc(sizeof(gif_Params));
  
  /*
  ** The header of GIF file looks like this :
  **
  **  char[6]  signature  (GIF87a or GIF89a)
  **  short    width
  **  short    height
  **  char     flags
  **  char     background
  **  char     aspect
  **
  ** We only read in the first frame of the image and
  ** since every frame has it's own header with width
  ** & height, we discard the ones from the header.
  **
  ** Jump over the signature, width & height. 
  */ 
 
  position += 6 + 2 + 2;

  /*
  ** Get the flags. We need these to know if the GIF
  ** file has a global palette for all it's frames.
  ** And also to known the number of bits/pixel.
  */

  params->flags = image[position++];
  params->bits = (params->flags & FLAG_BITS) + 1;

  /*
  ** Jump over background and aspect.
  */

  position += 1 + 1;
  
  /*
  ** Get the global palette.
  */
  
  if((params->flags & FLAG_PALETTE) == FLAG_PALETTE) {
    len = 3 * (1 << params->bits);
    woempa(5, "Palette length : %d\n", len);
    memcpy(&params->pal[0], &image[position], len);
    position += len;
  }

  /*
  ** Parse the different blocks in the file.
  ** We only need the first image block, the other
  ** blocks are discarded.
  */

  while(keep_going) {
    switch(image[position++]) {
      case BLOCK_IMAGE:

        woempa(5, "Image Block\n");

        /*
        ** Get parameters from the block header. 
        ** The header looks like this:
        **
        **  short   x
        **  short   y
        **  short   width
        **  short   height
        **  char    flags
        **
        ** We only need width, height and flags.
        */

        position += 2 + 2;
        
        memcpy(&params->width,   &image[position], 2);
        position += 2;
        memcpy(&params->height,  &image[position], 2);
        position += 2;

        params->flags = image[position++];
        params->bits = (params->flags & FLAG_BITS) + 1;
        
        /*
        ** Check if the image block has a local palette and read it.
        ** (We overwrite the global palette since we only support the first frame).
        */
        
        if(params->flags & FLAG_PALETTE) {
          len = 3 * (1 << params->bits);
          woempa(5, "Palette length : %d\n", len);
          memcpy(&params->pal[0], &image[position], len);
          position += len;
        }

        /*
        ** Get the code size with wich we need to start decompression.
        */
        
        code_size = image[position++];

        /*
        ** Allocate memory to hold the pixel data.
        */

        image_data = x_mem_alloc((w_word)(params->width * params->height * 4));
        
        /*
        ** Now we need to uncompress the image data.
        */

        unpack_result = gif_unpack(image + position, image_data, code_size, params);
        
        keep_going = 0;
        break;
      
      case BLOCK_EXT:
        woempa(5, "Extension Block\n");

        switch(image[position++]) {
          case EXT_TEXT:
          case EXT_CONTROL:
            
            /*
            ** We need to parse a graphical control block (if there's one)
            ** to know which is the transparant color (if there's one).
            **
            ** This block looks like this:
            **
            **  char   block_size
            **  char   flags
            **  short  delay
            **  char   transparent_color
            **
            ** The flags:
            **  0x01   transparant color
            **  0x02   user input
            **  0x1C   disposal method of frames
            **           0 - do nothing
            **           1 - leave frame in place
            **           2 - restore background color
            **           3 - restore original background
            **  0xE0   reserved
            **
            ** We are oly interested in the transparent flag & the transparant color.
            */

            position += 1;            // Jump over the block_size
            len = image[position++];  // Store the flags
            position += 2;            // Jump over the delay

            if(len & 0x01) {
              len = image[position++];

              /*
              ** Change the transparent color into white.
              */
              
              params->pal[3 * len + 0] = 0xFF; 
              params->pal[3 * len + 1] = 0xFF; 
              params->pal[3 * len + 2] = 0xFF; 
            }
            else {
              position++;
            }

            break;

            
          case EXT_COMMENT:
          case EXT_APPLICATION:
          default:

            /*
            ** Unknown extension. 
            ** Read in the block size and skip over it..
            */
            
            block_size = image[position++];
            position += block_size;
        }
        
        break;
      
      case BLOCK_NULL: 
        woempa(5, "NULL Block\n");
        break;
        
      default:
        woempa(9, "Block Error (%x, %c)\n", image[position - 1], image[position - 1]);
        keep_going = 0;
    }
  }

  if(image_data == NULL) {
    woempa(10, "WARNING: could not load GIF image.\n");
    return;
  }

  if(!unpack_result) {
    x_mem_free(image_data);
    woempa(10, "WARNING: error decoding GIF image.\n");
    return;
  }

  thisObject[F_GIFImageSource_width] = params->width;
  thisObject[F_GIFImageSource_height] = params->height;

  length = params->width * params->height;

  thisObject[F_GIFImageSource_pixels] = (w_int)allocArrayInstance(thread, atype2clazz[P_int], 1, &length); 

  if(thisObject[F_GIFImageSource_pixels]) {
    
    result = instance2Array_int((w_instance)thisObject[F_GIFImageSource_pixels]);
    memcpy(result, image_data, (w_word)(params->width * params->height * 4));

  }
  else {
    woempa(10, "WARNING: could not allocate array to hold image data.\n");
  }
  
  // Release memory:
  x_mem_free(image_data);
  x_mem_free(params);
  
  return;
}

#else 

/*
** No GIF support 
*/

w_void GIFImageSource_readImage(JNIEnv *env, jobject thisObject, jbyteArray imageFile) {
 
  return;

}

#endif
