/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/* $Id: Graphics.c,v 1.22 2002/03/28 14:04:24 dumon Exp $ */

#include <string.h>
#include "hashtable.h"    // ht_register()
#include "strings.h"      // cstring2String() 
#include "threads.h"	  // currentWonkaThread
#include "arrays.h"        // instace2Array_*()
#include "clazz.h"

#include "awt-classes.h"  // F_Component_*, F_Graphics_*
#include "registry.h"
#include "canvas.h"

#include "Component.h"
#include "Font.h"
#include "Graphics.h"
#include "Image.h"

w_void r_drawOvalQuarter(r_buffer buffer, w_int x, w_int y, w_int w, w_int h, w_int color, w_int q);
w_void r_fillOvalQuarter(r_buffer buffer, w_int x, w_int y, w_int w, w_int h, w_int color, w_int q);
  
void Graphics_preUpdate(r_canvas canvas) {

  woempa(5, "Graphics_preUpdate of '%k' (instance: %p, component: %p, refresh: %d)\n", instance2clazz(canvas->component->instance), canvas->component->instance, canvas->component, canvas->component->refresh);
    
  if (native_paint == 0) {
  
    /*
    ** Lock the component tree:
    */
    
    rudolph_lock();
  }

  if (canvas->buffer->status == 0) {

    /*
    ** Validate and reallocate the canvas buffer if necessary:
    */
      
    buffer_realloc(canvas);
  }
}

void Graphics_postUpdate(r_canvas canvas) {

  // Debug output:
  woempa(5, "Graphics_postUpdate of '%k' (instance: %p, component %p, refresh: %d)\n", instance2clazz(canvas->component->instance), canvas->component->instance, canvas->component, canvas->component->refresh);

  if (native_paint == 0) {
      
    /*
    ** Tag parent canvases as being invalid:
    */
    
    if (canvas->component->parent) {
      invalidateTreeUpwards(canvas->component->parent, 1);
    }

    /*
    ** Redraw current canvas:
    */
    
    canvas_canvas2screen(canvas);
 
    /*
    ** Invalidate this container (mandatory):
    */

    canvas->component->refresh = 1;
    
    /*
    ** Unlock the component tree:
    */
  
    rudolph_unlock();  
  }

}

w_int Graphics_verifyCoordinate(r_buffer buffer, w_int x, w_int y) {

  if (x < 0 || y < 0 || x > buffer->fw - 1 || y > buffer->fh - 1) {
    return 0;
  }
  else {
    return 1;
  }

}


void Graphics_copyArea(JNIEnv *env, jobject thisGraphics, jint x, jint y, jint width, jint height, jint deltax, jint deltay) {

  // Resolve required information:
  r_component component = (r_component)thisGraphics[F_Graphics_wotsit];

  x += thisGraphics[F_Graphics_ox];
  y += thisGraphics[F_Graphics_oy];
  deltax += thisGraphics[F_Graphics_ox];
  deltay += thisGraphics[F_Graphics_oy];

  if (component->tag == Z_IMAGE) {
    woempa(5, "  WARNING: copying in an image not supported; copy ignored\n");
  }
  else if (component->tag == Z_CONTAINER) {
    r_canvas canvas = (r_canvas) component->object;

    // copy canvas area:
    if (canvas) {

      w_int sx1 = x;
      w_int sy1 = y;
      w_int dx1 = sx1 + deltax;
      w_int dy1 = sy1 + deltay;
      w_int j;

      if (dx1 < 0) {
        sx1 = sx1 - dx1;
        width += dx1;
        dx1 = 0;
      }
      if (dy1 < 0) {
        sy1 = sy1 - dy1;
        height += dy1;
        dy1 = 0;
      }
      if (dx1 + width > canvas->buffer->fw) {
        width = canvas->buffer->fw - dx1;
      }
      if (dy1 + height > canvas->buffer->fh) {
        height = canvas->buffer->fh - dy1;
      }

      Graphics_preUpdate(canvas);

      // The actual copying, taking possible overlap of source and destination into account
      
      /* FIXME: Deal with copy / move operations that don't fall on byte boundaries */

      if (sx1 + width - 1 < dx1 || dx1 + width - 1 < sx1) {       // x-ranges of source ans destination do not overlap!
        for (j = 0; j < height; j++) {
	  /*pixelcopyline(
	    canvas->buffer->buffer + pixels2bytes((dy1+j)*canvas->buffer->fw), 
	    canvas->buffer->buffer + pixels2bytes((sy1+j)*canvas->buffer->fw),
	    dx1, sx1, width
	  );*/
	    
          memcpy(canvas_getPosition(canvas->buffer, dx1, dy1 + j), canvas_getPosition(canvas->buffer, sx1, sy1+j), (size_t)pixels2bytes(width));
        }
      }
      else {
        if (sy1 + height - 1 < dy1 || dy1 + height - 1 < sy1) {    // y-ranges of source ans destination do not overlap!
          for (j = 0; j < height; j++) {
	    /*pixelcopyline(
	      canvas->buffer->buffer + pixels2bytes((dy1+j)*canvas->buffer->fw), 
	      canvas->buffer->buffer + pixels2bytes((sy1+j)*canvas->buffer->fw),
	      dx1, sx1, width
	    );*/
            memcpy(canvas_getPosition(canvas->buffer, dx1, dy1 + j), canvas_getPosition(canvas->buffer, sx1, sy1+j), (size_t)pixels2bytes(width));
          }
        }
        else {
          if (sy1 < dy1) {
            for (j = height - 1; j >= 0; j--) {
              /*pixelcopyline(
	        canvas->buffer->buffer + pixels2bytes((dy1+j)*canvas->buffer->fw), 
	        canvas->buffer->buffer + pixels2bytes((sy1+j)*canvas->buffer->fw),
	        dx1, sx1, width
	      );*/
              memcpy(canvas_getPosition(canvas->buffer, dx1, dy1 + j), canvas_getPosition(canvas->buffer, sx1, sy1+j), (size_t)pixels2bytes(width));
            }
          }
          else {
            if (sy1 > dy1) {
              for (j = 0; j < height; j++) {
		/*pixelcopyline(
	          canvas->buffer->buffer + pixels2bytes((dy1+j)*canvas->buffer->fw), 
	          canvas->buffer->buffer + pixels2bytes((sy1+j)*canvas->buffer->fw),
	          dx1, sx1, width
	        );*/
                memcpy(canvas_getPosition(canvas->buffer, dx1, dy1 + j), canvas_getPosition(canvas->buffer, sx1, sy1+j), (size_t)pixels2bytes(width));
              }
            }
            else {  // sy1 == dy1
              /* FIXME: may result in some screen corruption */
              for (j = 0; j < height; j++) {
		/*pixelcopyline(
	          canvas->buffer->buffer + pixels2bytes((dy1+j)*canvas->buffer->fw), 
	          canvas->buffer->buffer + pixels2bytes((sy1+j)*canvas->buffer->fw),
	          dx1, sx1, width
	        );*/
                memmove(canvas_getPosition(canvas->buffer, dx1, dy1 + j), canvas_getPosition(canvas->buffer, sx1, sy1+j), (size_t)pixels2bytes(width));
              }
            }
          }
        }
      }    // end actual copying


      Graphics_postUpdate(canvas);

      // Debug output:
      woempa(5, "  Graphics_copyArea(x = %i, y = %i, width = %i, height = %i, deltax = %i, deltay = %i) 2 \n", x, y, width, height, deltax, deltay);
    }
    else {
      woempa(5, "  ERROR: Graphics_copyArea(): graphics component corrupt. \n");
    }
  }
  else {
    r_canvas canvas = (r_canvas) component->parent->object;

    // Copy component area:
    if (canvas ) {
      w_int sx1 = x + component->instance[F_Component_x];
      w_int sy1 = y + component->instance[F_Component_y];
      w_int dx1 = sx1 + deltax;
      w_int dy1 = sy1 + deltay;
      w_int j;

      if (dx1 < 0) {
        sx1 = sx1 - dx1;
        dx1 = 0;
      }
      if (dy1 < 0) {
        sy1 = sy1 - dy1;
        dy1 = 0;
      }
      if (dx1 + width > canvas->buffer->fw) {
        width = canvas->buffer->fw - dx1;
      }
      if (dy1 + height > canvas->buffer->fh) {
        height = canvas->buffer->fh - dy1;
      }

      Graphics_preUpdate(canvas);

      // The actual copying, taking possible overlap of source and destination into account
      
      /* FIXME: Deal with copy / move operations that don't fall on byte boundaries */

      if (sx1 + width - 1 < dx1 || dx1 + width - 1 < sx1) {       // x-ranges of source ans destination do not overlap!
        for (j = 0; j < height; j++) {
	    /*pixelcopyline(
	      canvas->buffer->buffer + pixels2bytes((dy1+j)*canvas->buffer->fw), 
	      canvas->buffer->buffer + pixels2bytes((sy1+j)*canvas->buffer->fw),
	      dx1, sx1, width
	    );*/
          memcpy(canvas_getPosition(canvas->buffer, dx1, dy1 + j), canvas_getPosition(canvas->buffer, sx1, sy1+j), (size_t)pixels2bytes(width));
        }
      }
      else {
        if (sy1 + height - 1 < dy1 || dy1 + height - 1 < sy1) {    // y-ranges of source ans destination do not overlap!
          for (j = 0; j < height; j++) {
	      /*pixelcopyline(
	      canvas->buffer->buffer + pixels2bytes((dy1+j)*canvas->buffer->fw), 
	      canvas->buffer->buffer + pixels2bytes((sy1+j)*canvas->buffer->fw),
	      dx1, sx1, width
	    );*/
            memcpy(canvas_getPosition(canvas->buffer, dx1, dy1 + j), canvas_getPosition(canvas->buffer, sx1, sy1+j), (size_t)pixels2bytes(width));
          }
        }
        else {
          if (sy1 < dy1) {
            for (j = height - 1; j >= 0; j--) {
		/*pixelcopyline(
	      canvas->buffer->buffer + pixels2bytes((dy1+j)*canvas->buffer->fw), 
	      canvas->buffer->buffer + pixels2bytes((sy1+j)*canvas->buffer->fw),
	      dx1, sx1, width
	    );*/
              memcpy(canvas_getPosition(canvas->buffer, dx1, dy1 + j), canvas_getPosition(canvas->buffer, sx1, sy1+j), (size_t)pixels2bytes(width));
            }
          }
          else {
            if (sy1 > dy1) {
              for (j = 0; j < height; j++) {
		  /*pixelcopyline(
	      canvas->buffer->buffer + pixels2bytes((dy1+j)*canvas->buffer->fw), 
	      canvas->buffer->buffer + pixels2bytes((sy1+j)*canvas->buffer->fw),
	      dx1, sx1, width
	    );*/
                memcpy(canvas_getPosition(canvas->buffer, dx1, dy1 + j), canvas_getPosition(canvas->buffer, sx1, sy1+j), (size_t)pixels2bytes(width));
              }
            }
            else {  // sy1 == dy1
              for (j = 0; j < height; j++) {
		  /*pixelcopyline(
	      canvas->buffer->buffer + pixels2bytes((dy1+j)*canvas->buffer->fw), 
	      canvas->buffer->buffer + pixels2bytes((sy1+j)*canvas->buffer->fw),
	      dx1, sx1, width
	    );*/
                memmove(canvas_getPosition(canvas->buffer, dx1, dy1 + j), canvas_getPosition(canvas->buffer, sx1, sy1+j), (size_t)pixels2bytes(width));
              }
            }
          }
        }
      }    // end actual copying


      Graphics_postUpdate(canvas);

    }
    else {
      woempa(10, "  ERROR: Graphics_copyArea(): graphics component corrupt. \n");
    }
  }

}

void Graphics_drawString(JNIEnv *env, jobject thisGraphics, jobject stringInstance, jint x, jint y) {
  
  r_component component = (r_component)thisGraphics[F_Graphics_wotsit];
  w_instance fontInstance = (w_instance)thisGraphics[F_Graphics_font];
  r_font font = (r_font)fontInstance[F_Font_wotsit];
  w_instance color = (w_instance)thisGraphics[F_Graphics_foreground];
  w_string string;
 
  w_int cx = thisGraphics[F_Graphics_cx];
  w_int cy = thisGraphics[F_Graphics_cy];
  w_int cw = thisGraphics[F_Graphics_cw];
  w_int ch = thisGraphics[F_Graphics_ch];

  if(stringInstance != NULL) {
    string = (w_string)stringInstance[F_String_wotsit];
  }
  else {
    return;
  }

  x += thisGraphics[F_Graphics_ox];
  y += thisGraphics[F_Graphics_oy];
  
  /*
  ** The given y-coordinate is suposed to specify the position of the text baseline.
  ** The x-coordinate specifies the position of the first character of the string.
  */
  
  // Debug output:
  woempa(5, "  [paint] called Graphics_drawString('%w', x = %i, y = %i)\n", string, x, y);

  // Draw string:
  //  call Font_DrawStringUnAligned
  //
  if (component->tag == Z_IMAGE) {
    r_image image = (r_image)component->object;
    cw = (cw <= 0 ? image->w : (cw > image->w ? image->w : cw));
    ch = (ch <= 0 ? image->h : (ch > image->h ? image->h : ch));
    Font_drawStringUnAligned(image->buffer, font, x, y,  cx, cy, cw, ch, string, color2pixel(color));
  }
  else if (component->tag == Z_CONTAINER) {
    r_canvas canvas = (r_canvas)component->object;
    Graphics_preUpdate(canvas);
    cw = (cw <= 0 ? canvas->buffer->fw : (cw > canvas->buffer->fw ? canvas->buffer->fw : cw));
    ch = (ch <= 0 ? canvas->buffer->fh : (ch > canvas->buffer->fh ? canvas->buffer->fh : ch));
    Font_drawStringUnAligned(canvas->buffer, font, x, y, cx, cy, cw, ch, string, color2pixel(color));
    Graphics_postUpdate(canvas);
  }
  else {
    r_canvas canvas = (r_canvas)component->parent->object;
    // x and y are assumed to be relative to the component's origin.
    Graphics_preUpdate(canvas);
    x += component->instance[F_Component_x];
    y += component->instance[F_Component_y];
    cx += component->instance[F_Component_x];
    cy += component->instance[F_Component_y];
    cw = (cw <= 0 ? (signed)component->instance[F_Component_width] : (cw > (signed)component->instance[F_Component_width] ?  (signed)component->instance[F_Component_width] : cw));
    ch = (ch <= 0 ? (signed)component->instance[F_Component_height] : (ch > (signed)component->instance[F_Component_height] ? (signed)component->instance[F_Component_height] : ch));
    Font_drawStringUnAligned(canvas->buffer, font, x, y, cx, cy, cw, ch, string, color2pixel(color));
    Graphics_postUpdate(canvas);
  }
}

void Graphics_fillArc(JNIEnv *env, jobject thisGraphics, jint x, jint y, jint w, jint h, jint startAngle, jint stopAngle) {
  woempa(10, "Not yet... Someday maybe...\n");
}

void Graphics_drawArc(JNIEnv *env, jobject thisGraphics, jint x, jint y, jint w, jint h, jint startAngle, jint angle) {

  r_component component = (r_component) thisGraphics[F_Graphics_wotsit];
  w_instance color_inst = (w_instance)thisGraphics[F_Graphics_foreground];
  r_pixel color = color2pixel(color_inst);
  r_buffer buffer = NULL;
  r_canvas canvas = NULL;

  w_int a = w / 2;
  w_int b = h / 2;
  w_int a2 = a * a;
  w_int b2 = b * b;
  w_int fa2 = 4 * a2;
  w_int fb2 = 4 * b2;
  w_int sigma = 2 * b2 + a2 * (1 - 2 * b);
  w_int xi, yi;
  
  x += thisGraphics[F_Graphics_ox] + w / 2;
  y += thisGraphics[F_Graphics_oy] + h / 2;
  
  // Debug output:
  woempa(5, "  [paint] called Graphics_drawArc(x = %i, y = %i, w = %i, h = %i, start = %i, stop = %i)\n", x, y, w, h, startAngle, angle);

  if (component->tag == Z_IMAGE) {
    buffer = ((r_image)component->object)->buffer;
  }
  else if (component->tag == Z_CONTAINER) {
    canvas = (r_canvas)component->object;
    buffer = canvas->buffer;
    Graphics_preUpdate(canvas);
  }
  else {
    canvas = (r_canvas)component->parent->object;
    buffer = canvas->buffer;
    x += component->instance[F_Component_x];
    y += component->instance[F_Component_y];
    Graphics_preUpdate(canvas);
  }

  for(xi = 0, yi = b; b2 * xi <= a2 * yi; xi++) {
    drawPixelClipped(buffer, x + xi, y + yi, color);
    drawPixelClipped(buffer, x - xi, y + yi, color);
    drawPixelClipped(buffer, x + xi, y - yi, color);
    drawPixelClipped(buffer, x - xi, y - yi, color);
    
    if(sigma >= 0) {
      sigma += fa2 * (1 - yi);
      yi--;
    }
    sigma += b2 * (4 * xi + 6);
  }
  
  sigma = 2 * a2 + b2 * (1 - 2 * a);
  
  for(xi = a, yi = 0; a2 * yi <= b2 * xi; yi++) {
    drawPixelClipped(buffer, x + xi, y + yi, color);
    drawPixelClipped(buffer, x - xi, y + yi, color);
    drawPixelClipped(buffer, x + xi, y - yi, color);
    drawPixelClipped(buffer, x - xi, y - yi, color);
    
    if(sigma >= 0) {
      sigma += fb2 * (1 - xi);
      xi--;
    }
    sigma += a2 * (4 * yi + 6);
  }
  
  if(component->tag != Z_IMAGE) {
    Graphics_postUpdate(canvas);
  }
}

void Graphics_drawLine(JNIEnv *env, jobject thisGraphics, jint x1, jint y1, jint x2, jint y2) {

  r_component component = (r_component) thisGraphics[F_Graphics_wotsit];
  w_instance color = (w_instance)thisGraphics[F_Graphics_foreground];
  
  x1 += thisGraphics[F_Graphics_ox];
  y1 += thisGraphics[F_Graphics_oy];
  x2 += thisGraphics[F_Graphics_ox];
  y2 += thisGraphics[F_Graphics_oy];
  
  // Debug output:
  woempa(5, "  [paint] called Graphics_drawLine(x1 = %i, y1 = %i, x2 = %i, y2 = %i)\n", x1, y1, x2, y2);

  if (component->tag == Z_IMAGE) {
    r_image image = (r_image)component->object;
    drawLine(image->buffer, x1, y1, x2, y2, color2pixel(color));
  }
  else if (component->tag == Z_CONTAINER) {
    r_canvas canvas = (r_canvas)component->object;
    Graphics_preUpdate(canvas);
    drawLine(canvas->buffer, x1, y1, x2, y2, color2pixel(color));
    Graphics_postUpdate(canvas);
  }
  else {
    r_canvas canvas = (r_canvas)component->parent->object;
    Graphics_preUpdate(canvas);
    drawLine(canvas->buffer, x1 + (signed)component->instance[F_Component_x], y1 + (signed)component->instance[F_Component_y], x2 + (signed)component->instance[F_Component_x], y2 + (signed)component->instance[F_Component_y], color2pixel(color));
    Graphics_postUpdate(canvas);
  }

}

/*
** Description:
**   Optimized oval algorithm using shift operations instead
**   of multiplications and divisions.
** Algorithm:
**   We avoid the usage of slow sinus and cosinus functions
**   and/or tables, by replacing them with a set of 2
**   lineair functions.  Secondly, by exploiting the symmetry
**   of a cirle we can cut down the number of mathematical
**   operations drastically.
**/

void Graphics_drawOval(JNIEnv *env, jobject thisGraphics, int x, int y, int w, int h) {
  r_component component = (r_component) thisGraphics[F_Graphics_wotsit];
  w_instance color_inst = (w_instance)thisGraphics[F_Graphics_foreground];
  r_pixel color = color2pixel(color_inst);
  r_buffer buffer = NULL;
  r_canvas canvas = NULL;

  w_int a = w / 2;
  w_int b = h / 2;
  w_int a2 = a * a;
  w_int b2 = b * b;
  w_int fa2 = 4 * a2;
  w_int fb2 = 4 * b2;
  w_int sigma = 2 * b2 + a2 * (1 - 2 * b);
  w_int xi, yi;
  
  x += thisGraphics[F_Graphics_ox] + w / 2;
  y += thisGraphics[F_Graphics_oy] + h / 2;
  
  // Debug output:
  woempa(5, "  [paint] called Graphics_drawOval(x = %i, y = %i, w = %i, h = %i)\n", x, y, w, h);

  if (component->tag == Z_IMAGE) {
    buffer = ((r_image)component->object)->buffer;
  }
  else if (component->tag == Z_CONTAINER) {
    canvas = (r_canvas)component->object;
    buffer = canvas->buffer;
    Graphics_preUpdate(canvas);
  }
  else {
    canvas = (r_canvas)component->parent->object;
    buffer = canvas->buffer;
    x += component->instance[F_Component_x];
    y += component->instance[F_Component_y];
    Graphics_preUpdate(canvas);
  }

  for(xi = 0, yi = b; b2 * xi <= a2 * yi; xi++) {
    drawPixelClipped(buffer, x + xi, y + yi, color);
    drawPixelClipped(buffer, x - xi, y + yi, color);
    drawPixelClipped(buffer, x + xi, y - yi, color);
    drawPixelClipped(buffer, x - xi, y - yi, color);
    
    if(sigma >= 0) {
      sigma += fa2 * (1 - yi);
      yi--;
    }
    sigma += b2 * (4 * xi + 6);
  }
  
  sigma = 2 * a2 + b2 * (1 - 2 * a);
  
  for(xi = a, yi = 0; a2 * yi <= b2 * xi; yi++) {
    drawPixelClipped(buffer, x + xi, y + yi, color);
    drawPixelClipped(buffer, x - xi, y + yi, color);
    drawPixelClipped(buffer, x + xi, y - yi, color);
    drawPixelClipped(buffer, x - xi, y - yi, color);
    
    if(sigma >= 0) {
      sigma += fb2 * (1 - xi);
      xi--;
    }
    sigma += a2 * (4 * yi + 6);
  }
  
  if(component->tag != Z_IMAGE) {
    Graphics_postUpdate(canvas);
  }
}

void r_drawCircle(r_buffer buffer, int x, int y, int w, int c) { 
  int cx = 0;
  int cy = w;
  int cd = 1 - w;

  while (cy >= cx) {
    
    /*
    ** Draw the pixels in eight different octants:
    */
    
    drawPixelClipped(buffer, x + cx, y + cy, c);
    drawPixelClipped(buffer, x + cx, y - cy, c);
    drawPixelClipped(buffer, x - cx, y + cy, c);
    drawPixelClipped(buffer, x - cx, y - cy, c);
    drawPixelClipped(buffer, x + cy, y + cx, c);
    drawPixelClipped(buffer, x + cy, y - cx, c);
    drawPixelClipped(buffer, x - cy, y + cx, c);
    drawPixelClipped(buffer, x - cy, y - cx, c);
   
    if (cd < 0) {
      cd += (cx << 1) + 3;
    }
    else { 
      cd += ((cx - cy) << 1) + 5;
      cy--;
    }     
    cx++;
  }
}

void r_drawFillCircle(r_buffer buffer, int x, int y, int w, int c) { 
  int cx = 0;
  int cy = w;
  int cd = 1 - w;

  while (cy >= cx) {
    
    /*
    ** Draw the pixels in eight different octants:
    */
    
    /*
    
    drawPixelClipped(buffer, x + cx, y + cy, c);
    drawPixelClipped(buffer, x + cx, y - cy, c);
    drawPixelClipped(buffer, x - cx, y + cy, c);
    drawPixelClipped(buffer, x - cx, y - cy, c);
    drawPixelClipped(buffer, x + cy, y + cx, c);
    drawPixelClipped(buffer, x + cy, y - cx, c);
    drawPixelClipped(buffer, x - cy, y + cx, c);
    drawPixelClipped(buffer, x - cy, y - cx, c);

    */

    /*
    ** Fill circle:
    */
    
    drawLine(buffer, x - cx, y + cy, x + cx + 1, y + cy, c);
    drawLine(buffer, x - cx, y - cy, x + cx + 1, y - cy, c);
    drawLine(buffer, x - cy, y + cx, x + cy + 1, y + cx, c);
    drawLine(buffer, x - cy, y - cx, x + cy + 1, y - cx, c);
   
    if (cd < 0) {
      cd += (cx << 1) + 3;
    }
    else { 
      cd += ((cx - cy) << 1) + 5;
      cy--;
    }     
    cx++;
  }
}

void Graphics_drawPolygon(JNIEnv *env, jobject thisGraphics, jobject xpoints, jobject ypoints, jint n) {

  if ((thisGraphics != nullInstance) && (xpoints != nullInstance) && (ypoints != nullInstance)) {

    if ((n <= instance2Array_length(xpoints)) && (n <= instance2Array_length(ypoints))) {

      // Resolve required information:
      r_component component = (r_component)thisGraphics[F_Graphics_wotsit];
      w_instance color = (w_instance)thisGraphics[F_Graphics_foreground];
      r_pixel col;
      w_int *x = instance2Array_int(xpoints);
      w_int *y = instance2Array_int(ypoints);
      w_int i;
      
      col = color2pixel(color);

      // Debug information:
      woempa(5, "  [paint] called drawPolygon(vertices = %i, edges = %i\n", n, n);

      // Draw polygon:
      if (component->tag == Z_IMAGE) {
        r_image image = (r_image)component->object;
        for (i = 0; i < n - 1; i++) {
          drawLine(image->buffer, *(x + i), *(y + i), *(x + i + 1), *(y + i + 1), col);
        }
        drawLine(image->buffer, *(x + n-1), *(y + n-1), *x, *y, col);
      }
      else if (component->tag == Z_CONTAINER) {
        r_canvas canvas = (r_canvas)component->object;
        Graphics_preUpdate(canvas);
        for (i = 0; i < n - 1; i++) {
          drawLine(canvas->buffer, *(x + i), *(y + i), *(x + i + 1), *(y + i + 1), col);
        }
        drawLine(canvas->buffer, *(x + n-1), *(y + n-1), *x, *y, col);
        Graphics_postUpdate(canvas);
      }
      else {
        r_canvas canvas = (r_canvas)component->parent->object;
        int xoffs;
        int yoffs;
        
        Graphics_preUpdate(canvas);
        
        xoffs = (signed)component->instance[F_Component_x];
        yoffs = (signed)component->instance[F_Component_y];
        
        for (i = 0; i < n - 1; i++) {
          drawLine(canvas->buffer, *(x + i)+xoffs, *(y + i)+yoffs, *(x + i + 1)+xoffs, *(y + i + 1)+yoffs, col);
        }
        drawLine(canvas->buffer, *(x + n-1)+xoffs, *(y + n-1)+yoffs, *x+xoffs, *y+yoffs, col);
        
        Graphics_postUpdate(canvas);
      }
    }
  }
}


/*
** An auxilliary function to build a linked list for a given array of values
** values: the buffer of element values
** order: here the ordered list will be stored
** returns: first element in linked list
*/
w_int r_sortIncreasing(w_int* values, w_int* order, w_int size) {
  w_int first, previous, i;
  first = 0;
  order[0]=-1;

  for(i=1; i<size; i++){
    //add next
    if(values[i]<=values[first]){
      order[i]=first;
      first = i;
    }
    else {
      previous = first;
      while(order[previous]>=0 && values[order[previous]]<=values[i]){
        previous = order[previous];
      }
      order[i]=order[previous];
      order[previous]=i;
    }
  }
  return first;
}


/*
** Description:
**   The function fills a polygon area area of the buffer of the given graphics'
**   component with the current foreground color as per java.awt.fillPolygon()
**
**	By default, this function should be completely aware of boundaries clipping
*/

void Graphics_fillPolygon(JNIEnv *env, jobject thisGraphics, jobject xpoints, jobject ypoints, jint size) {
  /* Check for null instances: */
  if ((thisGraphics != nullInstance) && (xpoints != nullInstance) && (ypoints != nullInstance)) {
    /* Check for buffer overflows: */
    if ((size <= instance2Array_length(xpoints)) && (size <= instance2Array_length(ypoints))) {

      /** Resolve required information: */
      /**********************************/
      //component
      r_component component = (r_component)thisGraphics[F_Graphics_wotsit];

      //screen buffer
      r_buffer buffer;
      r_canvas canvas;
      //screen color
      w_instance color = (w_instance)thisGraphics[F_Graphics_foreground];
      r_pixel pixelColor;
      //screen offsets
      w_int ox = 0;
      w_int oy = 0;

      // corner point arrays (point_x and point_y component
      w_int *pointx = instance2Array_int(xpoints);
      w_int *pointy = instance2Array_int(ypoints);

      // Our own buffers:
      // linked list for points in top-down order
      w_int *nextpoint =  (w_int*)allocMem(currentWonkaThread, (size_t)(size*sizeof(w_int)));
      // linepiece start and stop buffers
      w_int *linestarts =  (w_int*)allocMem(currentWonkaThread, (size_t)(size*sizeof(w_int)));
      w_int *linestops =  (w_int*)allocMem(currentWonkaThread, (size_t)(size*sizeof(w_int)));
      // data buffer for scanline intersection nodes, linked list buffer to order them
      w_int *nodes =  (w_int*)allocMem(currentWonkaThread, (size_t)(size*sizeof(w_int)));
      w_int *nextnode =  (w_int*)allocMem(currentWonkaThread, (size_t)(size*sizeof(w_int)));

      //variables for line-scan loop:
      w_int numberoflines;
      w_int currentpoint, previous, next;
      w_int scanline, scanstart, scanstop;
      w_int i, realline;

      woempa(5, "  [paint] called fillPolygon %i edges\n", size);
      /** initialise screen: color, canvas, buffer, offset*/
      // color
      pixelColor = color2pixel(color);

      if (component->tag == Z_IMAGE) {
        r_image image = (r_image)component->object;
        buffer = image->buffer;
        canvas = NULL;
      }
      else if (component->tag == Z_CONTAINER) {
        canvas = (r_canvas)component->object;
        Graphics_preUpdate(canvas);
        buffer = canvas->buffer;
      }
      else {
        canvas = (r_canvas)component->parent->object;
        Graphics_preUpdate(canvas);
        buffer = canvas->buffer;
        ox = (signed)component->instance[F_Component_x];
        oy = (signed)component->instance[F_Component_y];
      }

      /** Sort the polygon points and initialise the scanline loop */
      numberoflines = 0;
      currentpoint = r_sortIncreasing(pointy, nextpoint, size);
      scanstart = pointy[currentpoint];

      /** loop */
      do{
        // get previous and next linepieces
        previous = (currentpoint>0)? currentpoint-1: size-1;
        next =  (currentpoint<(size-1))? currentpoint+1: 0;

        // scan linepiece from previous point to current
        if(pointy[previous]>pointy[currentpoint]){
          // the linepiece starts here and goes down: add it to linepieces list
          linestarts[numberoflines]=currentpoint;
          linestops[numberoflines++]=previous;
        }
        else if(pointy[previous]<pointy[currentpoint]){
          // the linepiece comes from higher and ends here remove it from list
          for(i=0; (linestarts[i]!=previous || linestops[i]!=currentpoint); i++);
          linestarts[i]=linestarts[--numberoflines];
          linestops[i]=linestops[numberoflines];
        }

        // scan linepiece from current point to next
        if(pointy[next]>pointy[currentpoint]){
          // the linepiece starts here and goes down: add it to boundaries list
          linestarts[numberoflines]=currentpoint;
          linestops[numberoflines++]=next;
        }
        else if(pointy[next]<pointy[currentpoint]){
          // the linepiece comes from higher and ends here remove it from list
          for(i=0; (linestarts[i]!=next || linestops[i]!=currentpoint); i++);
          linestarts[i]=linestarts[--numberoflines];
          linestops[i]=linestops[numberoflines];
        }

        // get subset boundaries:
        scanstop = pointy[nextpoint[currentpoint]];
        realline=scanstart+oy;
        woempa(5, "  drawing polygon subset line %i to %i\n", scanstart, scanstop);
        // look if in screen before drawing
        if(realline<(buffer->fh) && (scanstop+oy)>0) {
          // okay, we start well under the buffers maximum and stop well over the buffers minimum
          // update the scan lines if necessary
          if(realline<0){
            realline = 0;
            scanstart = -oy;
          }
          if((scanstop+oy)>= (buffer->fh)){
            scanstop = (buffer->fh) -1 - oy;
          }
          // for all lines of this linepiece: calculate the intersections with the boundary lines, and paint
          for(scanline=scanstart; scanline<scanstop; scanline++) {
            // get the nodes for the intersections of the scanlines with the current page
            for(i=0; i<numberoflines; i++){
              // calculate the intersection (we directly include the offset)
              nodes[i] = pointx[linestarts[i]] + ox;  // if scanline==start of linepiece, we directly know what the x-offset is
              if(scanline > pointy[linestarts[i]]) {
                nodes[i] += (pointx[linestops[i]]-pointx[linestarts[i]]) * (scanline-pointy[linestarts[i]]) / (pointy[linestops[i]]-pointy[linestarts[i]]);
              }
            }

            // sort them in a linked list
            previous = r_sortIncreasing(nodes, nextnode, numberoflines);
            // draw from node to next node as long as there are
            while(previous>=0){
              next = nextnode[previous];
              // if in bounds, draw the line
              if(nodes[previous]>=buffer->fw || nodes[next]<0) {
                 woempa(5, "  Drawing scanline %i, real line %i line %i to %i out of screen (0->%i) \n", realline, scanline, nodes[previous], nodes[next], buffer->fw);
              }
              else if(nodes[previous]<0 && nodes[next]>=(buffer->fw)) {
                drawLine(buffer, 0, realline, buffer->fw, realline, pixelColor);      		
              }
              else if(nodes[previous]<0){
                drawLine(buffer, 0, realline, nodes[next], realline, pixelColor);      		
              }
              else if(nodes[next]>=(buffer->fw)){
                drawLine(buffer, nodes[previous], realline, (buffer->fw)-1, realline, pixelColor);      		
              }
              else {
                drawLine(buffer, nodes[previous], realline, nodes[next], realline, pixelColor);      		
              }
              previous = nextnode[next];
            }
            realline++;
          }
        }
        //next point, next scan area
        currentpoint = nextpoint[currentpoint];
        scanstart = scanstop;
      }
      while(nextpoint[currentpoint]>=0);

      // post-update if needed
      if (canvas != NULL) {
        Graphics_postUpdate(canvas);
      }

      /** Ok, we're done, now release the buffer memory again*/
      releaseMem(nextpoint);
      releaseMem(linestarts);
      releaseMem(linestops);
      releaseMem(nodes);
      releaseMem(nextnode);

    }
    /* done Check for buffer overflow: */
  }
  /* done Check for null instances: */
}

void Graphics_drawRect(r_buffer buffer, int x, int y, int w, int h, int c) {

  // Debug output:
  woempa(5, "called Graphics_drawRect: x = %i, y = %i, w = %i, h = %i, color = %i\n", x, y, w, h, c);

  // Vertical lines:
  drawLine(buffer, x, y, x, y, c);
  drawLine(buffer, x + w, y, x + w, y + h, c);
  
  // Horizontal lines:
  drawHLineClipped(buffer, x, y, w, c);
  drawHLineClipped(buffer, x, y + h, w, c);

}

void Graphics_drawRoundRect(JNIEnv *env, jobject thisGraphics, jint x, jint y, jint w, jint h, jint arcWidth, jint arcHeight) {
  r_component component = (r_component) thisGraphics[F_Graphics_wotsit];
  w_instance color_inst = (w_instance)thisGraphics[F_Graphics_foreground];
  r_pixel color = color2pixel(color_inst);
  r_buffer buffer = NULL;
  r_canvas canvas = NULL;

  x += thisGraphics[F_Graphics_ox];
  y += thisGraphics[F_Graphics_oy];
  
  // Debug output:
  woempa(5, "  [paint] called Graphics_drawRoundRect(x = %i, y = %i, w = %i, h = %i)\n", x, y, w, h);

  if (component->tag == Z_IMAGE) {
    buffer = ((r_image)component->object)->buffer;
  }
  else if (component->tag == Z_CONTAINER) {
    canvas = (r_canvas)component->object;
    buffer = canvas->buffer;
    Graphics_preUpdate(canvas);
  }
  else {
    canvas = (r_canvas)component->parent->object;
    buffer = canvas->buffer;
    x += component->instance[F_Component_x];
    y += component->instance[F_Component_y];
    Graphics_preUpdate(canvas);
  }

  // Vertical lines:
  drawLine(buffer, x, y + arcHeight / 2, x, y + h - arcHeight / 2, color);
  drawLine(buffer, x + w, y + arcHeight / 2, x + w, y + h - arcHeight / 2, color);
  
  // Horizontal lines:
  drawHLineClipped(buffer, x + arcWidth / 2, y, w - arcWidth + 1, color);
  drawHLineClipped(buffer, x + arcWidth / 2, y + h, w - arcWidth + 1, color);
  
  // Roundings:
  r_drawOvalQuarter(buffer, x, y, arcWidth, arcHeight, color, 2);  
  r_drawOvalQuarter(buffer, x + w - arcWidth + 1, y, arcWidth, arcHeight, color, 1);
  r_drawOvalQuarter(buffer, x, y + h - arcHeight + 1, arcWidth, arcHeight, color, 3);
  r_drawOvalQuarter(buffer, x + w - arcWidth + 1, y + h - arcHeight + 1, arcWidth, arcHeight, color, 4);
  
  if(component->tag != Z_IMAGE) {
    Graphics_postUpdate(canvas);
  }
}

void Graphics_fillRoundRect(JNIEnv *env, jobject thisGraphics, jint x, jint y, jint w, jint h, jint arcWidth, jint arcHeight) {
  r_component component = (r_component) thisGraphics[F_Graphics_wotsit];
  w_instance color_inst = (w_instance)thisGraphics[F_Graphics_foreground];
  r_pixel color = color2pixel(color_inst);
  r_buffer buffer = NULL;
  r_canvas canvas = NULL;
  w_int i;

  x += thisGraphics[F_Graphics_ox];
  y += thisGraphics[F_Graphics_oy];
  
  // Debug output:
  woempa(5, "  [paint] called Graphics_drawRoundRect(x = %i, y = %i, w = %i, h = %i)\n", x, y, w, h);

  if (component->tag == Z_IMAGE) {
    buffer = ((r_image)component->object)->buffer;
  }
  else if (component->tag == Z_CONTAINER) {
    canvas = (r_canvas)component->object;
    buffer = canvas->buffer;
    Graphics_preUpdate(canvas);
  }
  else {
    canvas = (r_canvas)component->parent->object;
    buffer = canvas->buffer;
    x += component->instance[F_Component_x];
    y += component->instance[F_Component_y];
    Graphics_preUpdate(canvas);
  }
  
  for(i = 0; i <= arcHeight / 2; i++) {
    drawHLineClipped(buffer, x + arcWidth / 2, y + i, w - arcWidth + 1, color);
    drawHLineClipped(buffer, x + arcWidth / 2, y + h - i, w - arcWidth + 1, color);
  }

  for(i = y + arcHeight / 2; i <= y + h - arcHeight / 2; i++) {
    drawHLineClipped(buffer, x, i, w, color);
  }
  
  // Roundings:
  r_fillOvalQuarter(buffer, x, y, arcWidth, arcHeight, color, 2);  
  r_fillOvalQuarter(buffer, x + w - arcWidth + 1, y, arcWidth, arcHeight, color, 1);
  r_fillOvalQuarter(buffer, x, y + h - arcHeight + 1, arcWidth, arcHeight, color, 3);
  r_fillOvalQuarter(buffer, x + w - arcWidth + 1, y + h - arcHeight + 1, arcWidth, arcHeight, color, 4);
  
  if(component->tag != Z_IMAGE) {
    Graphics_postUpdate(canvas);
  }
}

/*
** Description:
**   The function fills a rectangular area of the buffer of the given graphics'
**   component with a color specified by 'colorInstance'
**   Given coordinate (x,y) specifies the origin of the rectangle.
**   Its horizontal size is specified by w, and the vertical size is h.
**   In the vertical direction, only the intersection of the given rectangle with
**   the component buffer of the given graphics object is drawn.
**   For restrictions in the horizontal direction, see the docs of 'drawHLine'
*/

void Graphics_fillRect(JNIEnv *env, jobject thisGraphics, jint x, jint y, jint w, jint h) {

  r_component component = NULL;

  x += thisGraphics[F_Graphics_ox];
  y += thisGraphics[F_Graphics_oy];

  if  ((component = (r_component)thisGraphics[F_Graphics_wotsit])) {

    r_canvas canvas = NULL;
    r_buffer buffer = NULL;
    r_image  image  = NULL;
    r_pixel  pixelColor;
    w_int i;
    
    w_instance color = (w_instance)thisGraphics[F_Graphics_foreground];
    pixelColor = color2pixel(color);

    // Debug output:
    woempa(5, "  [paint] called Graphics_fillRect(x = %i, y = %i, w = %i, h = %i)\n", x, y, w, h);

    if (component->tag == Z_IMAGE) {
      image = (r_image) component->object;
      buffer = (r_buffer) image->buffer;
    }
    else if (component->tag == Z_CONTAINER){
      canvas = (r_canvas) component->object;
      Graphics_preUpdate(canvas);
      buffer = (r_buffer) canvas->buffer;
    }
    else {
      canvas = (r_canvas) component->parent->object;
      Graphics_preUpdate(canvas);
      buffer = (r_buffer) canvas->buffer;
    }

    // clip rectangle:
    if (x < 0) x = 0;
    if (y < 0) y = 0;
    w = (buffer->fw >= x + w) ? w : buffer->fw - x;
    h = (buffer->fh >= y + h) ? h : buffer->fh - y;
                
    if (component->tag == Z_IMAGE) {
      for (i = 0; i < h; i++) {
        drawHLineClipped(buffer, x, y + i, w, pixelColor);
      }
    }
    else if (component->tag == Z_CONTAINER) {
      for (i = 0; i < h; i++) {
        drawHLineClipped(buffer, x, y + i, w, pixelColor);
      }
      Graphics_postUpdate(canvas);
    }
    else {
      // extra clipping:
      w = ((signed)component->instance[F_Component_width] > x + w) ? w : (signed)component->instance[F_Component_width] - x;
      h = ((signed)component->instance[F_Component_height] > y + h) ? h : (signed)component->instance[F_Component_height] - y;
      for (i = 0; i < h; i++) {
        drawHLineClipped(buffer, x + (signed)component->instance[F_Component_x], y + (signed)component->instance[F_Component_y] + i, w, pixelColor);
      }
      Graphics_postUpdate(canvas);
    }

  }
}

void r_drawFillRect(r_buffer buffer, int x, int y, int w, int h, int c) {   

  int i;

  for (i = 0; i < h; i++) {
    drawHLine(buffer, x, y + i, w, c);
  }

}

jboolean Graphics_imageHasBuffer(JNIEnv *env, jobject thisGraphics, jobject imageInstance) {
  return ((r_image)imageInstance[F_Image_wotsit] != NULL);
}

jboolean Graphics_drawImage(JNIEnv *env, jobject thisGraphics, jobject imageInstance, jint dx1, jint dy1, jint dx2, jint dy2, jint sx1, jint sy1, jint sx2, jint sy2, jobject bgColor) {

  dx1 += thisGraphics[F_Graphics_ox];
  dy1 += thisGraphics[F_Graphics_oy];
  dx2 += thisGraphics[F_Graphics_ox];
  dy2 += thisGraphics[F_Graphics_oy];

  if (imageInstance != nullInstance) {

    // Resolve required information:
    r_component component = (r_component)thisGraphics[F_Graphics_wotsit];

    if (component->tag == Z_IMAGE) {
      r_image dst_image = (r_image)component->object;
      r_image image = (r_image)imageInstance[F_Image_wotsit];
   
      // Draw image on an image:
      if (dst_image && image) {

        w_boolean drawStatus = Image_drawImage(dst_image->buffer, image, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2);

        // Debug output:
        woempa(5, "  Graphics_drawImage('%s', dx1 = %i, dy1 = %i) 2 \n", image->filename, dx1, dy1);
        return drawStatus;
      }
      else {
        woempa(5, "  ERROR: Graphics_drawImage(): graphics component or image corrupt. \n");
        return (w_boolean)0;
      }
      return (w_boolean)0;
    }
    else if (component->tag == Z_CONTAINER) {
      r_canvas canvas = (r_canvas) component->object;
      r_image image = (r_image)imageInstance[F_Image_wotsit];
   
      // Draw image on canvas:
      if (canvas && canvas->buffer && image) {

        w_boolean drawStatus;
        Graphics_preUpdate(canvas);
        drawStatus = Image_drawImage(canvas->buffer, image, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2);
        Graphics_postUpdate(canvas);

        // Debug output:
        woempa(5, "  Graphics_drawImage('%s', dx1 = %i, dy1 = %i) 2 \n", image->filename, dx1, dy1);
        return drawStatus;
      }
      else {
        woempa(5, "  ERROR: Graphics_drawImage(): graphics component or image corrupt. \n");
        return (w_boolean)0;
      }
    }
    else {
      
      if (component->parent) {
        r_canvas canvas = (r_canvas) component->parent->object;
        r_image image = (r_image)imageInstance[F_Image_wotsit];

        // Draw image on component:
        if (canvas && canvas->buffer && image) {

          w_boolean drawStatus;
          Graphics_preUpdate(canvas);

          dx1 += component->instance[F_Component_x];
          dx2 += component->instance[F_Component_x];
          dy1 += component->instance[F_Component_y];
          dy2 += component->instance[F_Component_y];

          drawStatus = Image_drawImage(canvas->buffer, image, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2);
  
          Graphics_postUpdate(canvas);
 
          return drawStatus;
  
        }
        else {
          woempa(10, "  ERROR: Graphics_drawImage(): graphics component or image corrupt. \n");
          return 0;
        }
      }
      else {
        return 0;
      }
    }
  }
  else {
     woempa(10, "  ERROR: Graphics_drawImage(): input image is null. \n");
     return (w_boolean)0;
  }
}

void Graphics_drawBitmap(r_buffer buffer, unsigned char *bitmap, int x, int y, int w, int h, int c) {  

  int byte    = *bitmap;
  int shifted = 0;
  int bits = (w*h);
  int cx = x;
  int cy = y;
  
  while (--bits) {
      
    if (byte & 1) 
      pixelset(c, buffer->buffer, buffer->fw, cx, cy);
    
    byte >>= 1;
    
    if (++shifted == 8) {
      shifted = 0;
      byte    = *(++bitmap);
    }
    
    if (++cx == x+w) {
      cx = x;
      cy++;
    }
  }
}

w_void r_drawOvalQuarter(r_buffer buffer, w_int x, w_int y, w_int w, w_int h, w_int color, w_int q) {
  w_int a = w / 2;
  w_int b = h / 2;
  w_int a2 = a * a;
  w_int b2 = b * b;
  w_int fa2 = 4 * a2;
  w_int fb2 = 4 * b2;
  w_int sigma = 2 * b2 + a2 * (1 - 2 * b);
  w_int xi, yi;
  
  x += w / 2;
  y += h / 2;
  
  for(xi = 0, yi = b; b2 * xi <= a2 * yi; xi++) {
    if(q == 4) drawPixelClipped(buffer, x + xi, y + yi, color);
    if(q == 3) drawPixelClipped(buffer, x - xi, y + yi, color);
    if(q == 1) drawPixelClipped(buffer, x + xi, y - yi, color);
    if(q == 2) drawPixelClipped(buffer, x - xi, y - yi, color);
    
    if(sigma >= 0) {
      sigma += fa2 * (1 - yi);
      yi--;
    }
    sigma += b2 * (4 * xi + 6);
  }
  
  sigma = 2 * a2 + b2 * (1 - 2 * a);
  
  for(xi = a, yi = 0; a2 * yi <= b2 * xi; yi++) {
    if(q == 4) drawPixelClipped(buffer, x + xi, y + yi, color);
    if(q == 3) drawPixelClipped(buffer, x - xi, y + yi, color);
    if(q == 1) drawPixelClipped(buffer, x + xi, y - yi, color);
    if(q == 2) drawPixelClipped(buffer, x - xi, y - yi, color);
    
    if(sigma >= 0) {
      sigma += fb2 * (1 - xi);
      xi--;
    }
    sigma += a2 * (4 * yi + 6);
  }
}

w_void r_fillOvalQuarter(r_buffer buffer, w_int x, w_int y, w_int w, w_int h, w_int color, w_int q) {
  w_int a = w / 2;
  w_int b = h / 2;
  w_int a2 = a * a;
  w_int b2 = b * b;
  w_int fa2 = 4 * a2;
  w_int fb2 = 4 * b2;
  w_int sigma = 2 * b2 + a2 * (1 - 2 * b);
  w_int xi, yi;
  
  x += w / 2;
  y += h / 2;
  
  for(xi = 0, yi = b; b2 * xi <= a2 * yi; xi++) {
    if(q == 4) drawLine(buffer, x, y, x + xi, y + yi, color);
    if(q == 3) drawLine(buffer, x, y, x - xi, y + yi, color);
    if(q == 1) drawLine(buffer, x, y, x + xi, y - yi, color);
    if(q == 2) drawLine(buffer, x, y, x - xi, y - yi, color);
    
    if(sigma >= 0) {
      sigma += fa2 * (1 - yi);
      yi--;
    }
    sigma += b2 * (4 * xi + 6);
  }
  
  sigma = 2 * a2 + b2 * (1 - 2 * a);
  
  for(xi = a, yi = 0; a2 * yi <= b2 * xi; yi++) {
    if(q == 4) drawLine(buffer, x, y, x + xi, y + yi, color);
    if(q == 3) drawLine(buffer, x, y, x - xi, y + yi, color);
    if(q == 1) drawLine(buffer, x, y, x + xi, y - yi, color);
    if(q == 2) drawLine(buffer, x, y, x - xi, y - yi, color);
    
    if(sigma >= 0) {
      sigma += fb2 * (1 - xi);
      xi--;
    }
    sigma += a2 * (4 * yi + 6);
  }
}

