/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/* $Id: Image.c,v 1.18 2002/03/22 12:51:33 dumon Exp $ */

#include <string.h>
#include <stdio.h>

#include "awt-classes.h"  // F_Component_*, F_Graphics_*
#include "Image.h"
#include "realms.h"
#include "threads.h"       // currentWonkaThread
#include "ts-mem.h"       // allocMem(), releaseMem()
#include "memset.h"       // inline_memset()
#include "stack.h"
#include "exception.h"
#include "canvas.h"
#include "platform.h"
#include "Component.h"    // Component_getGraphics
#include "core-classes.h" // clazzClassCastExceptioni
#include "arrays.h"

/*
** Get the width of the r_image.
*/

int Image_getWidth(JNIEnv *env, jobject thisImage) {

  if (thisImage && thisImage[F_Image_wotsit]) {
    r_image image = (r_image)thisImage[F_Image_wotsit];
    return image->w;
  }
  else {
    return -1;
  }

}

/*
** Get the height of the r_image.
*/

int Image_getHeight(JNIEnv *env, jobject thisImage) {

  if (thisImage && thisImage[F_Image_wotsit]) {
    r_image image = (r_image)thisImage[F_Image_wotsit];
    return image->h;
  }
  else {
    return -1;
  }

}

/*
** Create the r_image from a pixel array.
*/

w_void Image_createImage(JNIEnv *env, jobject thisObject) {
  
  w_thread   thread = currentWonkaThread;
  r_image    image = (r_image)thisObject[F_Image_wotsit];

  if(image == NULL) {
    image = allocClearedMem(currentWonkaThread, sizeof(r_Image)); 
    if (image == NULL) {
      woempa(5, "WARNING: could not allocate memory.\n");
      return;
    }
  }

  // Initialize image where possible:
  image->w = thisObject[F_Image_width];
  image->h = thisObject[F_Image_height];

  // Allocate buffer:
  if(image->buffer == NULL) {
    image->buffer = (r_buffer) allocMem(thread, sizeof(r_Buffer) + pixels2bytes(image->w * image->h));
  }
  
  if (image->buffer == NULL) {
    releaseMem(image);
    woempa(5, "WARNING: could not allocate memory.\n");
    return;
  }
    
  // Initialize buffer:
  image->buffer->fw = image->w;
  image->buffer->fh = image->h;

  memcpy(
    (char *)image->buffer->buffer, 
    (char *)instance2Array_char((w_instance)thisObject[F_Image_pixels]), 
    (w_word)pixels2bytes(image->h * image->w)
  );
     
  thisObject[F_Image_wotsit] = (w_word)image;

  return;
}

/*
** Simply return the graphics object created by the Component from which this image was derived.
*/

w_instance Image_getGraphics (JNIEnv *env, jobject thisImage) {

  r_image image = (r_image)thisImage[F_Image_wotsit];
  w_instance graphicalCntxt;

  if(thisImage[F_Image_producer]) {
    throwException(JNIEnv2w_thread(env), clazzClassCastException, NULL);
    return NULL;
  }
  
  graphicalCntxt = Component_getGraphics(env, image->component->parent->instance);

  if (graphicalCntxt) {
    // Initialize graphical context:
    graphicalCntxt[F_Graphics_wotsit] = (w_word)image->component;

    // Debug output:
    woempa(5, "called Image_getGraphics(%p, %k): returns %p\n", env, instance2clazz(thisImage), graphicalCntxt);
    
    return graphicalCntxt;
  }
  else {
    return NULL;
  }
  
}

jint Image_convertRGB (JNIEnv *env, w_instance ImageClass, jint r, jint g, jint b) {
  /* Native so that the AWT classes aren't dependent on a particular Wonka
  ** build.
  */
  return rgb2pixel(r,g,b);
}

w_void Image_setPixel (JNIEnv *env, jobject this_image, jint x, jint y, jint scanlen, jint pixel) {
  w_ubyte *pixels = instance2Array_byte((w_instance)this_image[F_Image_pixels]);
  
  pixelset(pixel, pixels, scanlen, x, y);
}

w_void Image_finalize0 (JNIEnv *env, jobject thisImage) {

  r_image image = (r_image)thisImage[F_Image_wotsit];

  woempa(5, "called Image_finalize0()\n");

  if(image) {

    if (image->component) {
      releaseMem(image->component);
    }
  
    if (image->buffer) {
      releaseMem(image->buffer);
    }
  
    releaseMem(image);

  }
}

w_int Image_verifyCoordinate(r_buffer buffer, w_int x, w_int y) {

//  if (x < 0 || y < 0 || x > buffer->fw - 1 || y > buffer->fh - 1) {
  if (x < 0 || y < 0 || x > buffer->fw || y > buffer->fh) {
    return 0;
  }
  else {
    return 1;
  }

}

w_int Image_verifyBounds(r_buffer buffer, r_image image) {

  if ((image->x + image->w > buffer->fw) || (image->y + image->h > buffer->fh)) {
    return 0;
  }
  else {
    return 1;
  }

}


/*
** This function just copies a subimage of an r_image to an r_canvas.  Boundary checks and clippinng are
** performed. Scaling but no flipping is performed. The number of columns and rows copied is determined by
** the number of columns and rows in the intersection of the rectangles (sx1, sy1, sx2, sy2) and
** (dx1, dy1, dx2, dy2).
*/

w_boolean Image_drawImage(r_buffer dst_buffer, r_image image, int dx1, int dy1, int dx2, int dy2, int sx1, int sy1, int sx2, int sy2) {

//  if (dx1 == 0 && dx2 == image->w - 1 && dy1 == 0 && dy2 == image->h - 1 && dst_buffer->fw == image->buffer->fw &&
//      sx1 == 0 && sx2 == image->w - 1 && sy1 == 0 && sy2 == image->h - 1) {

    /*
    ** Image and Graphics have the same size and the image is drawn at (0, 0)
    ** -> Take a shortcut
    */

//    woempa(9, "Taking the shortcut\n");

//    memcpy(dst_buffer->buffer, image->buffer->buffer, pixels2bytes(image->w * image->h));

//    return (w_boolean)1;
//  }
//  else 
  if (!Image_verifyCoordinate(image->buffer, sx1, sy1) || !Image_verifyCoordinate(image->buffer, sx2, sy2)) {
    woempa(5, "WARNING: source coordinates (sx1,sy1)=(%d,%d), (sx2,sy2)=(%d,%d) out of source borders\n", sx1,sy1,sx2,sy2);
    return (w_boolean)0;
  }
  else if (sx1 > sx2 || sy1 > sy2) {
    woempa(5, "WARNING: sx1(%d) > sx2(%d) or sy1(%d) > sy2(%d), inversed source coordinates not allowed\n", sx1,sx2,sy1,sy2);
    return (w_boolean)0;
  }
  else if (dx1 > dx2 || dy1 > dy2) {
    woempa(5, "WARNING: dx1(%d) > dx2(%d) or dy1(%d) > dy2(%d), flipping not supported\n", dx1,dx2,dy1,dy2);
    return (w_boolean)0;
  }
  else if ((dx1 < dx2 ? dx1 : dx2) > dst_buffer->fw - 1 || (dy1 < dy2 ? dy1 : dy2) > dst_buffer->fh-1 || (dx1 < dx2 ? dx2 : dx1) < 0 || (dy1 < dy2 ? dy2 : dy1) < 0) {
    woempa(5, "WARNING: image position (dx1,dy1)=(%d,%d), (dx2,dy2)=(%d,%d) completely outside target (%d,%d)\n", dx1,dy1,dx2,dy2, dst_buffer->fw, dst_buffer->fh);
    return (w_boolean)0;
  }
  else {
    w_int i, j, k, l;
    w_int width;
    w_int height;
    r_buffer buffer = NULL;
    w_int *rows, *cols;
    
    w_int iw = image->w;           // Image width
    w_int ih = image->h;           // Image height
    w_int nw = abs(dx2 - dx1) + 1; // Destination width
    w_int nh = abs(dy2 - dy1) + 1; // Destination height
    w_int ow = abs(sx2 - sx1) + 1; // Source width
    w_int oh = abs(sy2 - sy1) + 1; // Source height

    woempa(5, "calling coordinates: dx1 = %d, dy1 = %d, dx2 = %d, dy2 = %d, sx1 = %d, sy1 = %d, sx2 = %d, sy2=%d \n", dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2);

    /*
    ** Crop & Scale (& flip ??)
    */

    if(sx1 < 0) sx1 = 0;
    if(sy1 < 0) sy1 = 0;
    if(sx2 > iw - 1) sx2 = iw - 1;
    if(sy2 > ih - 1) sy2 = ih - 1;

    width  = nw;
    height = nh;

    if(iw != ow || ih != oh) {
      woempa(9, "DRAWIMAGE: Clipping is needed : (%d, %d) -> (%d, %d)\n", iw, ih, ow, oh);
    }
    
    if(nw != ow || nh != oh) {
      woempa(9, "DRAWIMAGE: Scaling is needed : (%d, %d) -> (%d, %d)\n", ow, oh, nw, nh);
      buffer = x_mem_alloc((w_word)(sizeof(r_Buffer) + pixels2bytes(nw * nh)));

      buffer->fw = nw;
      buffer->fh = nh;
      buffer->vw = nw;
      buffer->vh = nh;
      buffer->ox = 0;
      buffer->oy = 0;
      
      rows = x_mem_alloc((oh + 1) * sizeof(w_int));
      cols = x_mem_alloc((ow + 1) * sizeof(w_int));

      for(i=0; i <= ow; i++) cols[i] = (i * nw) / ow;
      for(i=0; i <= oh; i++) rows[i] = (i * nh) / oh;

      for(i=0; i < oh; i++) 
        for(j=0; j < ow; j++) 
          for(k=0; k < (rows[i + 1] - rows[i]); k++) 
            for(l=0; l < (cols[j + 1] - cols[j]); l++)
              pixelset(pixelget(image->buffer->buffer, image->buffer->fw, j+sx1, i+sy1), 
                       buffer->buffer, buffer->fw, (cols[j] - cols[0] + l), (rows[i] - rows[0] + k));
      
      x_mem_free(cols);
      x_mem_free(rows);

      sx1 = 0;
      sy1 = 0;
      sx2 = nw - 1;
      sy2 = nh - 1;
    }

    /*
    ** Check bounds
    */

    if (dx1 < 0) {
      sx1 = sx1 - dx1;
      dx1 = 0;
    }
    if (dy1 < 0) {
      sy1 = sy1 - dy1;
      dy1 = 0;
    }
    if (dx2 > dst_buffer->fw - 1) {
      sx2 = sx2 - (dx2 - dst_buffer->fw - 1) - 2;
      dx2 = dst_buffer->fw - 1;
    }
    if (dy2 > dst_buffer->fh - 1) {
      sy2 = sy2 - (dy2 - dst_buffer->fh - 1) - 2;
      dy2 = dst_buffer->fh - 1;
    }

    woempa(5, "adapted coordinates: dx1 = %d, dy1 = %d, dx2 = %d, dy2 = %d, sx1 = %d, sy1 = %d, sx2 = %d, sy2 = %d \n", dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2);

    width  = sx2 - sx1 + 1;
    height = sy2 - sy1 + 1;

    if(buffer) {
      w_ubyte *src = buffer->buffer + pixels2bytes(sy1 * buffer->fw);
      w_ubyte *dst = dst_buffer->buffer + pixels2bytes(dy1 * dst_buffer->fw);
      
      for (j = sy1; j <= sy2;  j++) {
        pixelcopyline(dst, src, dx1, sx1, width);
        src += pixels2bytes(buffer->fw);
        dst += pixels2bytes(dst_buffer->fw);
      }
      x_mem_free(buffer);
    } 

    else {
      w_ubyte *src = image->buffer->buffer + pixels2bytes(sy1 * image->buffer->fw);
      w_ubyte *dst = dst_buffer->buffer + pixels2bytes(dy1 * dst_buffer->fw);
      
      for (j = sy1; j <= sy2; j++) {
        pixelcopyline(dst, src, dx1, sx1, width);
        src += pixels2bytes(image->buffer->fw);
        dst += pixels2bytes(dst_buffer->fw);
      }
    }
    return (w_boolean)1;
  }

}

w_void Image_conversion_index(JNIEnv *env, jobject thisImage, jobject srcArray, jobject dstArray, 
                              jobject model, jint x, jint y, jint w, jint h, jint offset, jint scansize) {
  
  w_ubyte *reds = (w_ubyte *)instance2Array_byte((w_instance)model[F_IndexColorModel_reds]);
  w_ubyte *greens = (w_ubyte *)instance2Array_byte((w_instance)model[F_IndexColorModel_greens]);
  w_ubyte *blues = (w_ubyte *)instance2Array_byte((w_instance)model[F_IndexColorModel_blues]);
  w_ubyte *src = (w_ubyte *)instance2Array_byte(srcArray);
  w_ubyte *dst = (w_ubyte *)instance2Array_char(dstArray);
  w_int   width = thisImage[F_Image_width];
  w_int   i, j;
  w_ubyte pixel;

  src += offset;
  dst += pixels2bytes(y * width + x);

  for(i=0; i<h; i++) {
    for(j=0; j<w; j++) {
      pixel = *src++;
      *(r_pixel *)dst  = rgb2pixel(reds[pixel], greens[pixel], blues[pixel]);
      dst += pixels2bytes(1);
    }
    src += scansize - w;
    dst += pixels2bytes(width - (x + w));
  }
}

w_void Image_conversion_direct(JNIEnv *env, jobject thisImage, jobject srcArray, jobject dstArray, 
                              jobject model, jint x, jint y, jint w, jint h, jint offset, jint scansize) {
  
  w_word  *src = (w_word *)instance2Array_byte(srcArray);
  w_ubyte *dst = (w_ubyte *)instance2Array_char(dstArray);
  w_int   width = thisImage[F_Image_width];
  w_int   i, j;
  w_word  pixel;

  src += offset;
  dst += pixels2bytes(y * width + x);

  for(i=0; i<h; i++) {
    for(j=0; j<w; j++) {
      pixel = *src++;
      *(r_pixel *)dst = rgb2pixel((((pixel & model[F_DirectColorModel_rmask]) >> model[F_DirectColorModel_rmaskpos]) << 8) >> model[F_DirectColorModel_rmaskbits],
                                  (((pixel & model[F_DirectColorModel_gmask]) >> model[F_DirectColorModel_gmaskpos]) << 8) >> model[F_DirectColorModel_gmaskbits],
                                  (((pixel & model[F_DirectColorModel_bmask]) >> model[F_DirectColorModel_bmaskpos]) << 8) >> model[F_DirectColorModel_bmaskbits]);
      dst += pixels2bytes(1);
    }
    src += scansize - w;
    dst += pixels2bytes(width - (x + w));
  }
}

