/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/* $Id: dispatcher.c,v 1.12 2002/03/27 17:04:21 dumon Exp $ */

#include "hashtable.h"
#include "loading.h"
#include "oswald.h"

#include "rudolph.h"
#include "registry.h"
#include "dispatch.h"
#include "platform.h"
#include "awt-classes.h"

#include "Event.h"
#include "Font.h"
#include "canvas.h"

#include "realms.h"

x_monitor tree_lock;
static x_thread disp_thread;
static w_byte *disp_memory;

JNIEnv *environment;

void dispatcher(void* argument) {

  r_event event = NULL;
  int state = 0;
  int down = 0;
  int ox = 0;
  int oy = 0;
  int x = 0;
  int y = 0;

  int     VK = 0;
  w_char  keychar = 0;
  int     mod = 0;
  int     pressed = 0;

  Wonka_AttachArgs waa;
  JavaVM *allCreatedJavaVMs[1];
  JavaVM *mrWonka;
  jint nVMs;
  jint rc;

  /*
  ** Attach rudolph's dispatcher/render thread to wonka:
  */
  
  rc = JNI_GetCreatedJavaVMs(allCreatedJavaVMs,1,&nVMs);

  if (rc == 0 && nVMs > 0) {
    waa.name = (char*)"rudolph";
    mrWonka = allCreatedJavaVMs[0];
    rc = ((*mrWonka)->AttachCurrentThread)(mrWonka, &environment, &waa);
    woempa(9, "attached rudolph's event dispatch and render thread to wonka\n");
  }
  else {
    wabort(ABORT_WONKA, "failed to attach rudolph's event and render thread to wonka\n");
  }                                          

  /*
  ** Initialize the event:
  */
  
  event = x_mem_calloc(sizeof(r_Event));
  
  /*
  ** Enter infinite event loop:
  */
  
  while (1) {
    state = 0;
    x = 0;
    y = 0;

    while(keyboard_poll(&VK, &keychar, &mod, &pressed)) {
      
      /*
      ** Received a keyboard event.
      */

      if(pressed) {
        Event_addKeyEvent(VK, (w_int)keychar, mod, R_EVENT_KEY_PRESSED);
        if(!keyboard_isMod(VK)) Event_addKeyEvent(VK, (w_int)keychar, mod, R_EVENT_KEY_TYPED);
      }
      else {
        Event_addKeyEvent(VK, (w_int)keychar, mod, R_EVENT_KEY_RELEASED);
      }
        
    }

    if (mouse_poll(&state, &x, &y)) {
      
      woempa(5, "received touchscreen event at (%i, %i): %i [P:%d, R:%d, D:%d]\n", x, y, state, R_EVENT_MOUSE_PRESSED, R_EVENT_MOUSE_RELEASED, R_EVENT_MOUSE_DRAGGED);
      
      if (state == R_EVENT_MOUSE_RELEASED) {

        if (down == 1) {

          down = 0;
          
          event->tag = R_EVENT_MOUSE_RELEASED;
          event->x = x;
          event->y = y;
    
          Event_dispatchEvent(rootCanvas, event);
        }
      }
      else if (state == R_EVENT_MOUSE_PRESSED) {

        if (down == 0) {

          down = 1;
  
          event->tag = R_EVENT_MOUSE_PRESSED;
          event->x = x;
          event->y = y;
    
          Event_dispatchEvent(rootCanvas, event);
        }
      }
      else {

        if (abs(ox - x) > 5 || abs(oy - y) > 5) {

          if (down) {
            event->tag = R_EVENT_MOUSE_DRAGGED;

            event->x = x;
            event->y = y;
    
            Event_dispatchEvent(rootCanvas, event);
          }
          else {
            event->tag = R_EVENT_MOUSE_MOVED;

            event->x = x;
            event->y = y;
    
            // Event_dispatchEvent(rootCanvas, event);
          }
          
          ox = event->x;
          oy = event->y;
        }
      }
    }
    else {
      x_thread_sleep(2);
    }
      
    mouse_flush();
    screen_update(0, 0, 0, 0);
  }

  /*
  ** Release the event:
  */
  
  x_mem_free(event);
  
}

static int signature_width  = 64;
static int signature_height = 19;

static unsigned int signature_colour[] = {
  0x0022222222, 0x00565656, 0x00727272, 0x00f8f8f8 
};

static char * signature_bitmap[] = {
  "&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&",
  "&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&",
  "&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&",
  "&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&",
  "&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&",
  "&&&&&&&&&&&&&$&&&&&&&&&&&&&&&##&&&&&&&&&&&&&&&&&&$#&&&&&&&&&&#&&",
  "&&&&&#&&&&&&&%#&&&&&&&&&&&&&#&&&&&&&&&&&&&&#&&&&&#$&$$#&&&&##&&&",
  "&&###%&&&%&&&#&&&&&&&&&&&&&$&&&&&&&&&&&&&&#&%&&&&##$&#%&&&&%&&&&",
  "&#&&&%&&%%&&&%&&&&&&&&&&&&&%&&#&&&&&&&&&&$&&%&&&&$$&&$%&&&#%&&&&",
  "&$$&#$&##$&&#&&#&%#&#$&$#&#$&#%&&&#%#&&&&%&&%&&&##%&&$%&&&&$&&&&",
  "&#&&$#&#&$&&#&#&&##&$$&%&&###&&&##&$&&&&&#&#$&&#&&$&&#$#&#&#&&&&",
  "&&&&$&#&#$&#&&%&&##&%&&%&&#%#&&&%&&%&&&&&&&#$&$&&&&&#&$&#&#&&&&&",
  "&&&&%%&&&$#&&##&&$&&%&##&&%&%&&&$&&%&&&&&&&###&&&&&&#&$%&&$&&&&&",
  "&&&&%#&&#%&&&#$&&#&##&##$&%&$#&#%#$$#&&&&&&#%&&&&&&#&&##&&$&&&&&",
  "&&&##&&&#&&&&&%#&&&#&&##&&#&#$&&%&$#&&&&&&&$#&&&&#$#&&&&&#%#&&&&",
  "&&&&&&&&&&&&&&&&&&&&&&&&&&&&&$#&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&",
  "&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&####&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&",
  "&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&",
  "&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&"
};

static void do_testpattern(w_ubyte *scr, int w, int h) {
  int x, y, xoff, yoff, col1, col2;

  /* Draw a test pattern rather than just blanking the screen; this can
  ** give us some easy clues as to why our new display type isn't working
  ** as it should.
  */

  col1 = rgb2pixel(255,255,255);
  col2 = rgb2pixel(0,0,0);
  
  woempa(9, "white pixel value = %x, black pixel value = %x\n", col1, col2);

  for (y = 0; y < h; y++) {
    for (x = 0; x < (w/2); x++) {
      pixelset(col1, scr, screen->width, x*2 +  (y&1), y);
      pixelset(col2, scr, screen->width, x*2 + !(y&1), y);
    }
  }
  
  /* Draw a white box in the middle */  
  for (y = (h/3); y < (h/3)*2; y++) {
    for (x = (w/3); x < (w/3)*2; x++) {
      pixelset(col1, scr, screen->width, x, y);
    }
  }
  
  /* Draw our signature */
  yoff = (h / 2) - (signature_height / 2);
  xoff = (w / 2) - (signature_width / 2);
  for (y = 0; y < signature_height; y++) {
    for (x = 0; x < signature_width; x++) {
      int c = signature_colour[signature_bitmap[y][x] - 35];
      pixelset(rgb2pixel((c & 0xff), ((c >> 8) & 0xff), ((c >> 16) & 0xff)), scr, w, x + xoff, y + yoff);
    }
  }
}

void dispatcher_awt(void * status) {

  static int init = 1;
  w_instance tlock = NULL;
  
  if (init == 1) {
    // Disable static initializer: 
    init = 0;

    // Reduce woempa noise a bit:
    // setAllTriggerLevel(9);
    
    // Initialize the lock:
    tlock = (w_instance)getStaticReferenceField(clazzComponent, F_Component_lock);
    tree_lock = allocMonitor(tlock);
    
    // Make sure we have a default EventQueue:
  
    classMustBeInitialized(clazzEventQueue, W_Thread_system);
    defaultEventQueue = allocInstance(W_Thread_system, clazzEventQueue);
    newGlobalReference(defaultEventQueue);

    // Make sure we have a default Toolkit:
    classMustBeInitialized(clazzToolkit, W_Thread_system);
    
    // Make sure we have system colors:
    classMustBeInitialized(clazzSystemColor, W_Thread_system);

    // Initialize screen:
    woempa(9, "initalize video driver ...\n");
    screen = screen_init();
    do_testpattern(screen->video, screen->width, screen->height);

    // Initialize mouse:
    woempa(9, "initalize mouse driver ...\n");
    mouse_init();
    
    // Initialize fonts:
    woempa(9, "initalize font engine ...\n");
    Font_init();

    // Initialize rootCanvas:
    woempa(9, "initalize component registry ...\n");
    rootCanvas = registry_constructor(screen->width, screen->height);
    
    // Create dispatcher thread:
    disp_thread = allocClearedMem_system(sizeof(x_Thread));
    disp_memory = allocMem_system(DISPATCHER_STACK_SIZE);
    x_thread_create(disp_thread, dispatcher, 0, disp_memory, DISPATCHER_STACK_SIZE, DISPATCHER_PRIORITY, TF_START);
  }
  
}

void init_awt(void) {

  /*
  ** Wonka Native Interface:
  */
  
  collectAWTFixups();
  loadAWTClasses();
}

void init_module(void) {
 
  woempa(9, "initializing AWT module\n");
  init_awt();

}

void clean_module(void) {

  woempa(9, "cleaning AWT module\n");

}

