/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: Button.c,v 1.8 2002/02/20 09:03:04 dfrederickx Exp $ 
*/

#include "ts-mem.h"       // allocMem(), releaseMem()
#include "realms.h"
#include "strings.h"      // cstring2String()
#include "threads.h"       // currentWonkaThread

#include "awt-classes.h"  // F_Button_*
#include "platform.h"     // color2pixel
#include "varia.h"
#include "canvas.h"

#include "Event.h"
#include "Button.h"
#include "Component.h"
#include "Font.h"

r_component button_pressed;

void Button_createPeer(JNIEnv *env, jobject thisButton) {

  r_component component = (r_component)thisButton[F_Component_wotsit];
  component->tag = Z_BUTTON;

}

void Button_drawButtonDefault(r_buffer buffer, r_component component) {

  w_instance buttonInstance = component->instance;
  w_instance stringInstance = (w_instance)buttonInstance[F_Button_label];
  w_string string = (w_string)stringInstance[F_String_wotsit];
    
  // Foreground color:
  w_instance fcol;

  r_pixel c1;
  r_pixel c2;
    
  // Font:
  r_font font;
          
  // Retrieve coordinates:
  w_int x = 0, y = 0, w = 0, h = 0;
  getRelativeCoordinates(component, &x, &y, &w, &h);

  // Retrieve foreground color:
  Component_getForeground(component, &fcol, Component_getColor(F_SystemColor_windowText));
    
  // Retrieve font:
  Component_getFont(component, &font);
    
  // woempa(9, "called drawButtonDefault('%w'): bbx = %i, bby = %i, bbw = %i, bbh = %i\n", string, x, y, w, h);

  c1 = color2pixel(Component_getColor(F_SystemColor_controlHighlight));
  c2 = color2pixel(Component_getColor(F_SystemColor_controlShadow));

  w -= 1;
  h -= 1;
    
  drawLine(buffer, x, y, x + w, y, c1);
  drawLine(buffer, x, y + h, x + w, y + h, c2);
  drawLine(buffer, x, y, x, y + h, c1);
  drawLine(buffer, x + w, y, x + w, y + h, c2);
                            
  x += 1;
  y += 1;
  w -= 2;
  h -= 2;
    
  drawLine(buffer, x, y, x + w, y, c1);
  drawLine(buffer, x, y + h, x + w, y + h, c2);
  drawLine(buffer, x, y, x, y + h, c1);
  drawLine(buffer, x + w, y, x + w, y + h, c2);
                                              
  x += 2;
  y += 2;
  w -= 4;
  h -= 4;
    
  Font_drawStringAligned(buffer, font, x, y, w, h, string, color2pixel(fcol), (unsigned int)ALIGNMENT_CENTER);

}


void Button_drawButtonPressed(r_buffer buffer, r_component component) { 

  r_pixel c1;
  r_pixel c2;

  w_int x, y, w, h;
  getRelativeCoordinates(component, &x, &y, &w, &h);

  c1 = color2pixel(Component_getColor(F_SystemColor_controlHighlight));
  c2 = color2pixel(Component_getColor(F_SystemColor_controlShadow));

  if (c1 > c2) {
    r_pixel swap;
    swap = c1; 
    c1 = c2; 
    c2 = swap;
  }

  w -= 1;
  h -= 1;
    
  drawLine(buffer, x, y, x + w, y, c1);
  drawLine(buffer, x, y + h, x + w, y + h, c2);
  drawLine(buffer, x, y, x, y + h, c1);
  drawLine(buffer, x + w, y, x + w, y + h, c2);
                            
  x += 1;
  y += 1;
  w -= 2;
  h -= 2;
    
  drawLine(buffer, x, y, x + w, y, c1);
  drawLine(buffer, x, y + h, x + w, y + h, c2);
  drawLine(buffer, x, y, x, y + h, c1);
  drawLine(buffer, x + w, y, x + w, y + h, c2);

}

void Button_drawButtonReleased(r_buffer buffer, r_component component) { 

  r_pixel c1;
  r_pixel c2;

  w_int x, y, w, h;
  getRelativeCoordinates(component, &x, &y, &w, &h);

  c1 = color2pixel(Component_getColor(F_SystemColor_controlShadow));
  c2 = color2pixel(Component_getColor(F_SystemColor_controlHighlight));

  if (c1 < c2) {
    r_pixel swap;
    swap = c1; 
    c1 = c2; 
    c2 = swap;
  }

  w -= 1;
  h -= 1;
  
  drawLine(buffer, x, y, x + w, y, c1);
  drawLine(buffer, x, y + h, x + w, y + h, c2);
  drawLine(buffer, x, y, x, y + h, c1);
  drawLine(buffer, x + w, y, x + w, y + h, c2);
                          
  x += 1;
  y += 1;
  w -= 2;
  h -= 2;
    
  drawLine(buffer, x, y, x + w, y, c1);
  drawLine(buffer, x, y + h, x + w, y + h, c2);
  drawLine(buffer, x, y, x, y + h, c1);
  drawLine(buffer, x + w, y, x + w, y + h, c2);

}

void Button_drawButtonFocused(r_buffer buffer, r_component component) {

  // do nothing - touch screen mode

}

void Button_eventMouseClick(r_component component, r_event event) {

  r_canvas canvas;
 
  if (component->parent && component->parent->object) {
    canvas = (r_canvas)component->parent->object;
 
    /*
    ** Validate the buffer and reallocate it if necessary:
    */
 
    if (canvas->buffer->status == 0) { 
      buffer_realloc(canvas);
    }
  
    /*
    ** If there is still a button in pressed state, release it first:
    */
  
    if (button_pressed && button_pressed != component) {
      Button_eventMouseExit(button_pressed, NULL);
    }
  
    if (event->tag == R_EVENT_MOUSE_PRESSED) {
  
      /*
      ** Keep a reference to the pressed button:
      */
  
      button_pressed = component;
    
      /*
      ** Draw a pressed button to the off-screen canvas:
      */
    
      Button_drawButtonPressed(canvas->buffer, component);

      /*
      ** Display the updated canvas:
      */
    
      canvas_canvas2screen(canvas);
      
      /*
      ** Since a button release can be triggered any
      */
      
      invalidateTreeUpwards(canvas->component, 1);

    }
    else if (event->tag == R_EVENT_MOUSE_RELEASED) {

      /*
      ** Remove the reference to the released button:
      */
    
      button_pressed = NULL;
    
      /*
      ** Display the refreshed canvas:
      */
    
      Button_drawButtonReleased(canvas->buffer, component);
     
      /*
      ** Display the updated canvas:
      */

      canvas_canvas2screen(canvas);
     
      /*
      ** Since a button release can be triggered any
      */
      
      invalidateTreeUpwards(canvas->component, 1);
    }
    else {

      /*
      ** Remark:
      **   do nothing - touch screen
      */
    
      // woempa(9, "WARNING: unhandled mouse event - event '%i' not supported\n", event->tag);
    
      /* 
      ** Example code:
      **   button->drawButtonFocused(canvas->buffer, button);
      */
    }
  }
}

void Button_eventMouseEnter(r_component component, r_event event) {

  // do nothing - touch screen mode
  
  /*
  ** Example code:
  **   r_canvas canvas = (r_canvas) button->component->parent->object;
  **   button->drawButtonFocused(canvas->buffer, button);
  **
  **   canvas_canvas2screen(canvas);
  */

}

void Button_eventMouseExit(r_component component, r_event event) {

  r_canvas canvas;
  
  if (component == button_pressed) {
    
    if (component->parent && event) {
      
      /*
      ** The above check is mandatory - GC related.
      */
      canvas = (r_canvas)component->parent->object;
      
      /*
      ** Validate the buffer and reallocate it if necessary:
      */
  
      if (canvas->buffer->status == 0) {
        buffer_realloc(canvas);
      }

      /*
      ** Remove the reference to the released button:
      */
    
      button_pressed = NULL;

      /*
      ** Display the refreshed canvas:
      */
    
      Button_drawButtonReleased(canvas->buffer, component);

      /*
      ** Display the updated canvas:
      */

      canvas_canvas2screen(canvas);
      
      /*
      ** Since a button release can be triggered any
      */
      
      invalidateTreeUpwards(canvas->component, 1);

    }
  }
  
  /*
  ** Example code:
  **   r_canvas canvas = (r_canvas) button->component->parent->object;
  **   (button->drawButtonDefault)(canvas->buffer, button);
  **   canvas_canvas2screen(canvas);
  **   canvas_drawBorder(canvas);
  */

}

void Button_eventMouseMove(r_component component, r_event event) {

  // do nothing - touch screen mode

}

