/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/* $Id: Checkbox.c,v 1.9 2002/03/21 13:04:09 gray Exp $ */

#include "heap.h"         // setReferenceField()
#include "strings.h"      // cstring2String()
#include "ts-mem.h"       // allocMem(), releaseMem()
#include "realms.h"
#include "threads.h"       // currentWonkaThread
#include "clazz.h"        // clazz2Class

#include "rudolph.h"
#include "awt-classes.h"  // F_Checkbox_*
#include "canvas.h"

#include "Checkbox.h"
#include "Component.h"
#include "Event.h"
#include "Font.h"
#include "Graphics.h"

#define R_CHECKBOX_SIZE 15

void Checkbox_createPeer(JNIEnv *env, jobject thisCheckbox) {
  
  r_component component = (r_component)thisCheckbox[F_Component_wotsit];
  component->tag = Z_CHECKBOX;

}

// This function might be depricated:
void Checkbox_update(JNIEnv *env, jobject thisCheckbox) {

  // Resolve checkbox/radiobutton data:
  r_component component = (r_component) thisCheckbox[F_Component_wotsit];

  if (component->parent) {
    r_canvas canvas = (r_canvas)component->parent->object;
  
    // Draw enabled/disabled mark:
    if (thisCheckbox[F_Checkbox_state]) {
      Checkbox_drawCheckboxEnabled(canvas->buffer, component);
    }
    else {
      Checkbox_drawCheckboxDisabled(canvas->buffer, component);
    }
  }

}

void Checkbox_drawCheckboxDefault(r_buffer buffer, r_component component) {

  w_instance checkboxInstance = component->instance;

  if (checkboxInstance[F_Component_visible]) {
    // Retrieve coordinates:
    w_int x = (w_int)checkboxInstance[F_Component_x];
    w_int y = (w_int)checkboxInstance[F_Component_y];
    w_int h = (w_int)checkboxInstance[F_Component_height];
    w_int w = (w_int)checkboxInstance[F_Component_width]; 
    
    // Retrieve label:
    w_instance stringInstance = (w_instance) checkboxInstance[F_Button_label];
    w_string string = (w_string)stringInstance[F_String_wotsit];
                                    
    // Font:
    r_font font;
    
    // Foreground color:
    w_instance fcol;
                  
    // Retrieve font:
    Component_getFont(component, &font);
    
    // Retrieve foreground color:
    Component_getForeground(component, &fcol, Component_getColor(F_SystemColor_windowText));

    // woempa(9, "called Checkbox_drawCheckboxDefault('%w'): group = %i - bbx = %i, bby = %i, bbw = %i, bbh = %i\n", string, (w_int) checkboxInstance[F_Checkbox_isGroup], x, y, h, w);
  
    // Draw checkbox label:
    Font_drawStringAligned(buffer, font, x + R_CHECKBOX_SIZE + 5, y, w - R_CHECKBOX_SIZE - 5, h, string, color2pixel(fcol), (unsigned int)ALIGNMENT_LEFT);

    if (checkboxInstance[F_Checkbox_isGroup]) {
      // Remap y-cooridate:
      y = y + h / 2;
      x = x + 6;
    
      // Draw radiobutton:
      r_drawCircle(buffer, x, y, 6, (r_pixel)0);
    }

    // Draw enabled/disabled mark:
    if (checkboxInstance[F_Checkbox_state]) {
      Checkbox_drawCheckboxEnabled(buffer, component);
    }
    else {
      Checkbox_drawCheckboxDisabled(buffer, component);
    }
  }
}

void Checkbox_drawCheckboxEnabled(r_buffer buffer, r_component component) {

  w_instance checkboxInstance = component->instance;
 
  if (checkboxInstance[F_Component_visible]) {
    // Retrieve coordinates:
    w_int x = (w_int)checkboxInstance[F_Component_x];
    w_int y = (w_int)checkboxInstance[F_Component_y];
    w_int h = (w_int)checkboxInstance[F_Component_height];

    // Boundary check:
    if (x + R_CHECKBOX_SIZE > buffer->fw) {
      woempa(9, "boundary check: checkbox exceeds bounding box of size %ix%i - prune\n", buffer->fw, buffer->fh);
      return;
    }

    // Debug output:
    // woempa(9, "called Checkbox_drawCheckboxEnabled()\n");

    if (checkboxInstance[F_Checkbox_isGroup]) {
      // Remap x- and y-cooridate:
      y = y + h / 2;
      x = x + 6;
    
      // Draw blank radiobutton background:
      r_drawFillCircle(buffer, x, y, 5, R_WHITE);  
     
      // Draw enabled mark:
      r_drawFillCircle(buffer, x, y, 3, (r_pixel)0);  
    }
    else {
      int w = R_CHECKBOX_SIZE;
      
      y = (h > w) ? y + (h - w) / 2 : y;

      drawLine(buffer, x, y, x + w-1, y, (r_pixel)rgb2pixel(40, 40, 40));
      drawLine(buffer, x, y + w-1, x + w-1, y + w-1, (r_pixel)rgb2pixel(255, 255, 255));
      drawLine(buffer, x, y, x, y + w-1, (r_pixel)rgb2pixel(40, 40, 40));
      drawLine(buffer, x + w-1, y, x + w-1, y + w-1, (r_pixel)rgb2pixel(255, 255, 255));
      
      x += 1;
      y += 1;
      w -= 2;
      
      drawLine(buffer, x, y, x + w-1, y, (r_pixel)rgb2pixel(40, 40, 40));
      drawLine(buffer, x, y + w-1, x + w-1, y + w-1, (r_pixel)rgb2pixel(255, 255, 255));
      drawLine(buffer, x, y, x, y + w-1, (r_pixel)rgb2pixel(40, 40, 40));
      drawLine(buffer, x + w-1, y, x + w-1, y + w-1, (r_pixel)rgb2pixel(255, 255, 255));
      
      x += 1;
      y += 1;
      w -= 2;
      
      r_drawFillRect(buffer, x, y, w, w, (r_pixel)rgb2pixel(120, 120, 120));  
    }
  }
}

void Checkbox_drawCheckboxDisabled(r_buffer buffer, r_component component) {

  w_instance checkboxInstance = component->instance;
 
  if (checkboxInstance[F_Component_visible]) {
    // Retrieve coordinates:
    w_int x = (w_int)checkboxInstance[F_Component_x];
    w_int y = (w_int)checkboxInstance[F_Component_y];
    w_int h = (w_int)checkboxInstance[F_Component_height];

    // Boundary check:
    if (x + R_CHECKBOX_SIZE > buffer->fw) {
      woempa(9, "boundary check: checkbox exceeds bounding box of size %ix%i - prune\n", buffer->fw, buffer->fh);
      return;
    }

    // Debug output:
    // woempa(9, "called Checkbox_drawCheckboxDisabled()\n");

    if (checkboxInstance[F_Checkbox_isGroup]) {
      // Remap x- and y-cooridate:
      y = y + h / 2;
      x = x + 6;
    
      // Draw blank radiobutton background:
      r_drawFillCircle(buffer, x, y, 5, R_WHITE);  
    }
    else {
      int w = R_CHECKBOX_SIZE;
      
      y = (h > w) ? y + (h - w) / 2 : y;

      drawLine(buffer, x, y, x + w-1, y, (r_pixel)rgb2pixel(255, 255, 255));
      drawLine(buffer, x, y + w-1, x + w-1, y + w-1, (r_pixel)rgb2pixel(40, 40, 40));
      drawLine(buffer, x, y, x, y + w-1, (r_pixel)rgb2pixel(255, 255, 255));
      drawLine(buffer, x + w-1, y, x + w-1, y + w-1, (r_pixel)rgb2pixel(40, 40, 40));
      
      x += 1;
      y += 1;
      w -= 2;
      
      drawLine(buffer, x, y, x + w-1, y, (r_pixel)rgb2pixel(255, 255, 255));
      drawLine(buffer, x, y + w-1, x + w-1, y + w-1, (r_pixel)rgb2pixel(40, 40, 40));
      drawLine(buffer, x, y, x, y + w-1, (r_pixel)rgb2pixel(255, 255, 255));
      drawLine(buffer, x + w-1, y, x + w-1, y + w-1, (r_pixel)rgb2pixel(40, 40, 40));
      
      x += 1;
      y += 1;
      w -= 2;
      
      r_drawFillRect(buffer, x, y, w, w, (r_pixel)rgb2pixel(185, 185, 185));  
    }
  }
}

void Checkbox_eventMouseClick(r_component component, r_event event) {

  w_instance checkboxInstance = (w_instance)component->instance;
  r_canvas canvas = (r_canvas)component->parent->object;
  
  
  /*
  ** Validate and reallocate the canvas buffer if necessary:
  */
  
  if (canvas->buffer->status == 0) {  
    buffer_realloc(canvas);
  }
  
  if (event->tag == R_EVENT_MOUSE_PRESSED) {

    if (checkboxInstance[F_Checkbox_isGroup]) {

      /* 
      ** Checkboxgroup - radio-button alike checkbox
      */

      /* Retrieve old active radiobutton: */
      w_instance checkboxGroupInstance = (w_instance)checkboxInstance[F_Checkbox_group];
      w_instance selectedCheckboxInstance  = (w_instance)checkboxGroupInstance[F_CheckboxGroup_selectedCheckbox];
      r_component oldCheckboxComponent = (r_component)selectedCheckboxInstance[F_Component_wotsit];
      r_canvas oldCanvas = (r_canvas)oldCheckboxComponent->parent->object;

      if (oldCheckboxComponent != component) {  // prune if possible
        // Disable active radiobutton prior to enabling the new radiobutton:
        selectedCheckboxInstance[F_Checkbox_state] = (w_int) 0;
        setReferenceField(checkboxGroupInstance, checkboxInstance, F_CheckboxGroup_selectedCheckbox);
            
        // Update radiobutton state and redraw checkbox on canvas:
        checkboxInstance[F_Checkbox_state] = (w_int) 1;

        // Redraw radio buttons in cache:
        Checkbox_drawCheckboxDisabled(oldCanvas->buffer, oldCheckboxComponent);
        Checkbox_drawCheckboxEnabled(canvas->buffer, component);      

        // Refresh canvas:
        if (canvas == oldCanvas) {  // prune if possible
          // Refresh canvas on screen:
          canvas_canvas2screen(canvas);
          
          // Tag canvases for a refresh to ensure it is re-rendered:
          invalidateTreeUpwards(canvas->component, 1);

        }
        else {

          // Refresh canvases on screen:
          canvas_canvas2screen(oldCanvas);
          canvas_canvas2screen(canvas);

          // Tag canvases for a refresh to ensure it is re-rendered:
          invalidateTreeUpwards(canvas->component, 1);
          invalidateTreeUpwards(oldCanvas->component, 1);
        }
        
        // Add event to java.awt.EventQueue:
        Checkbox_addItemEvent((w_instance)checkboxInstance); 
      }
    }
    else {
      
      /*
      ** Regular checkbox:
      */
       
      // Update checkbox state and redraw checkbox on canvas:
      if (checkboxInstance[F_Checkbox_state]) {
        checkboxInstance[F_Checkbox_state] = (w_int) 0;
	Checkbox_drawCheckboxDisabled(canvas->buffer, component); 
      } 
      else {
        checkboxInstance[F_Checkbox_state] = (w_int) 1;
	Checkbox_drawCheckboxEnabled(canvas->buffer, component); 
      }

      // Refresh canvas:
      canvas_canvas2screen(canvas);
            
      // Tag canvas for a refresh to ensure it is re-rendered:
      invalidateTreeUpwards(canvas->component, 1);
      
      // Add event to java.awt.EventQueue:
      Checkbox_addItemEvent((w_instance) checkboxInstance); 
    }  
  }

}

void Checkbox_addItemEvent(w_instance source) {

  static jmethodID method = NULL;
  w_thread thread = currentWonkaThread;
  JNIEnv *env = w_thread2JNIEnv(thread);
  w_instance AWTEvent = allocInstance(thread, clazzItemEvent); 
  
  // Debug output:
  woempa(5, "called Event_addItemEvent(%p)\n", source);

  if (AWTEvent != NULL) {
  

    if (method == NULL) {
      method = (*env)->GetMethodID(env, clazz2Class(clazzEventQueue), "postNativeEvent", "()V");
    }

    // Create appropriate AWTEvent:
    AWTEvent[F_AWTEvent_id] = ITEM_STATE_CHANGED;
    AWTEvent[F_ItemEvent_item] = source[F_Checkbox_label];
    AWTEvent[F_ItemEvent_stateChange] = (source[F_Checkbox_state]) ? SELECTED : DESELECTED;
    setReferenceField(AWTEvent, source, F_EventObject_source);

    // Attach AWTEvent to EventQueue instance:
    setReferenceField(defaultEventQueue, AWTEvent, F_EventQueue_nativeAWTEvent);

    // Call method postNativeEvent:
    (*env)->CallVoidMethod(environment, defaultEventQueue, method);
  }
}

