/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/* $Id: Component.c,v 1.31 2002/03/20 13:04:20 gray Exp $ */

#include "rudolph.h"

#include "ts-mem.h"
#include "realms.h"
#include "clazz.h"
#include "awt-classes.h"
#include "core-classes.h"
#include "checks.h"
#include "registry.h"

#include "canvas.h"
#include "dispatch.h"
#include "platform.h"

#include "Component.h"
#include "Font.h"
#include "Image.h"
#include "Event.h"

#ifdef AWT_CACHE
 #undef AWT_CACHE
#endif

w_void Component_resetFontCache(r_component current) {
 
  r_component component;
    
  /*
  ** Reset the font cache:
  */
    
  current->font = NULL;
    
  for (component = current->headChild; component != NULL; component = component->next) {

    /*
    ** Continue to invalidate the child component (if any):
    */
  
    if (component->font && !component->instance[F_Component_font]) {    
      Component_resetFontCache(component);
    }
  }

}

w_void Component_resetForegroundColorCache(r_component current) {

  r_component component;
    
  /*
  ** Reset the foreground color cache:
  */
    
  current->fgcolor = NULL;
  
  for (component = current->headChild; component != NULL; component = component->next) {

    /*
    ** Continue to invalidate the child component (if any):
    */
      
    if (component->fgcolor && !component->instance[F_Component_foreground]) {
      Component_resetForegroundColorCache(component);
    }
  }

}

w_void Component_resetBackgroundColorCache(r_component current) {

  r_component component;
  
  /*
  ** Reset the background color cache:
  */
    
  current->bgcolor = NULL;

  for (component = current->headChild; component != NULL; component = component->next) {

    /*
    ** Continue to invalidate the child component (if any):
    */
      
    if (component->bgcolor && !component->instance[F_Component_background]) {
      Component_resetBackgroundColorCache(component);
    }
  }

}

w_void Component_resetVisibilityCache(r_component current) {

  r_component component;
  
  current->visible = -1;
  
  for (component = current->headChild; component != NULL; component = component->next) {

    /*
    ** Continue to invalidate the child component (if any):
    */
    
    if (component->visible != -1) {
      if (!component->instance[F_Component_visible]) {
        Component_resetVisibilityCache(component);
      }
      else {
        component->visible = -1;
      }
    }
  }

}

w_void invalidateTreeUpwards(r_component component, w_int type) {
  
  /*
  ** Invalidate parent container:
  */
  
  if (component->parent && component->instance && component->instance[F_Component_visible]) {
    invalidateTreeUpwards(component->parent, type);
    // propagate invalidation through the component tree
  }
 
  /*
  ** Invalidate container:
  */
  
  if (component->refresh < (unsigned int)type) {
    component->refresh = (unsigned int)type;
    
    /*
    ** Note: make sure you don't overwrite component->refresh with 
    **       refresh events that are less important.  A global refresh 
    **       (type = 2) is - obviously - more important then a local 
    **       refresh (type = 1).
    */
  
  }
}            

w_void getRelativeCoordinates(r_component component, int *x, int *y, int *w, int *h) {
  
  *x = (int)component->instance[F_Component_x];
  *y = (int)component->instance[F_Component_y];
  *h = (int)component->instance[F_Component_height];
  *w = (int)component->instance[F_Component_width];
  
  if ((*x < 0) || (*y < 0) || ((*x) * (*y) > MAX_SIZE) || (*w < 0) || (*h < 0) || ((*w) * (*h) > MAX_SIZE)) {
    // Debug output:
    woempa(9, "WARNING: invalid coordinates: x = %i, y = %i, w = %i, h = %i\n", *x, *y, *w, *h);
     
    // Reset coordinates rather then passing along bugus coordinates:
    *x = 0; *y = 0; *h = 0; *w = 0;
  }
}

w_int Component_verifyBounds(r_component component) {

  /*
  ** Checks to see whether we have a valide instance:
  */
  
  int x = component->instance[F_Component_x];
  int y = component->instance[F_Component_y];
  int w = component->instance[F_Component_width];
  int h = component->instance[F_Component_height];
    
  if (x < 0 || y < 0 || w < 1 || h < 1 || (x * y ) > MAX_SIZE || (w * h) > MAX_SIZE) {
    woempa(10, "invalid bounds for %k '%p': x = %d, y = %d, w = %d, h = %d\n", (instance2clazz(component->instance)), component, x, y, w, h);

    return 0;
  }
  else {

    return 1;
  }
  
}

w_void Component_getForeground(r_component component, w_instance *colorInstance, w_instance defaultColor) {

  // Remark: don't call this function with a component whose field 'instance' is not itself a component

#ifdef AWT_CACHE
  if (component && component->fgcolor) {

    /*
    ** Return the cached color:
    */
    
    *colorInstance = component->fgcolor;

    // woempa(9, "COLOR: return cached color\n");
  }
  else
#endif 
    if (component && component->instance && (w_instance)component->instance[F_Component_foreground] != nullInstance) {

    /*
    ** Extract the color from the instance itself:
    */
    
    *colorInstance = (w_instance)component->instance[F_Component_foreground];

    // woempa(9, "COLOR: return color from instance\n");
  }
  else if (component && component->parent) {

    /*
    ** Look for the color of the parent component:
    */
    
    Component_getForeground(component->parent, colorInstance, defaultColor);

    /*
    ** Update te cache:
    */

    component->fgcolor = *colorInstance;
    
    // woempa(9, "COLOR: update the cache\n");    
  }
  else {
    
    /*
    ** No color found, return the default color:
    */
    
    *colorInstance = defaultColor;
 
    // woempa(9, "COLOR: return default color\n");
  }

}

w_void Component_getBackground(r_component component, w_instance *colorInstance, w_instance defaultColor) {

  // Remark: don't call this function with a component whose field 'instance' is not itself a component

#ifdef AWT_CACHE
  if (component && component->bgcolor) {

    /*
    ** Return the cached color:
    */

    *colorInstance = component->bgcolor;

    // woempa(9, "COLOR: return cached color\n");
  }
  else
#endif   
    if (component && component->instance && (w_instance)component->instance[F_Component_background] != nullInstance) {

    /*
    ** Extract the color from the instance itself:
    */

    *colorInstance = (w_instance)component->instance[F_Component_background];

    // woempa(9, "COLOR: return color from instance\n");
  }
  else if (component && component->parent) {

    /*
    ** Look for the color of the parent component:
    */
    
    Component_getBackground(component->parent, colorInstance, defaultColor);

    /*
    ** Update te cache:
    */

    component->bgcolor = *colorInstance;
    
    // woempa(9, "COLOR: update the cache\n");    
  }
  else {
    
    /*
    ** No color found, return the default color:
    */
    
    *colorInstance = defaultColor;
 
    // woempa(9, "COLOR: return default color\n");
  }

}

w_instance Component_getColor(int slot) {
  
  return (w_instance)getStaticReferenceField(clazzSystemColor, slot);

}

w_void Component_getFont(r_component component, r_font *font) {

  // Remark: don't call this function with a component whose field 'instance' is not itself a component

#ifdef AWT_CACHE
  if (component && component->font) {

    /*
    ** Return the cached font:
    */

    *font = (r_font)component->font[F_Font_wotsit];

  }
  else 
#endif    
    if (component && component->instance && (w_instance)component->instance[F_Component_font] != nullInstance) {
    w_instance fontInstance = (w_instance)component->instance[F_Component_font];

    /*
    ** Extract the font from the instance itself:
    */
    
    *font = (r_font)fontInstance[F_Font_wotsit];
  }
  else if (component && component->parent) {
    
    /*
    ** Look for the font of the parent component:
    */
    
    Component_getFont(component->parent, font);
  
    // NOTE: we can't update the cache

  }
  else {
    
    /*
    ** No font found, return the default font:
    */
    
    *font = (r_font)defaultFont[F_Font_wotsit];
  }
}

w_void Component_getFontInstance(r_component component, w_instance *fontInstance) {

  // Remark: don't call this function with a component whose field 'instance' is not itself a component

#ifdef AWT_CACHE
  if (component && component->font) {

    /*
    ** Return the cached font:
    */

    *fontInstance = component->font;
    
  }
  else 
#endif
    if (component && component->instance && (w_instance)component->instance[F_Component_font] != nullInstance) {

    /*
    ** Extract the font from the instance itself:
    */
   
    *fontInstance = (w_instance)component->instance[F_Component_font];

  }
  else if (component && component->parent) {
  
    /*
    ** Look for the font of the parent component:
    */

    Component_getFontInstance(component->parent, fontInstance);

    /*
    ** Update the cache:
    */

    component->font = *fontInstance;
    
  }
  else {

    /*
    ** No font found, return the default font:
    */

    *fontInstance = defaultFont;
  }
}

w_void Component_intersect(int x1, int y1, int w1, int h1, int x2, int y2, int w2, int h2, int *xr, int *yr, int *wr, int *hr) {
 
  int x3 = x1 + w1;
  int y3 = y1 + h1;
  int x4 = x2 + w2;
  int y4 = y2 + h2;
  
  if (x1 < x2) {
    *xr = x2;
  }
  else {
    *xr = x1;
  }
  
  if (y1 < y2) {
    *yr = y2;
  }
  else {
    *yr = y1;
  }
  
  if (x3 < x4) {
    *wr = x3 - *xr;
  }
  else {
    *wr = x4 - *xr;
  }

  if (y3 < y4) {
    *hr = y3 - *yr;
  }
  else {
    *hr = y4 - *yr;
  }

  if (*wr <= 0 || *hr <= 0) {
    *xr = 0;
    *yr = 0;
    *wr = 0;
    *hr = 0;
  }
}

w_void getAbsoluteCoordinates(r_component component, int *dx, int *dy, int *w, int *h, int *ox, int *oy, int depth) {

  r_canvas canvas = NULL;

  int ix;
  int iy;
  int ih;
  int iw;
  
  int rx;
  int ry;
  int rw;
  int rh;
  
  if(component == rootCanvas->component) {
    rx = 0;
    ry = 0;
    rw = screen->width;
    rh = screen->height;
  } 
  else {
    rx = (int)component->instance[F_Component_x];      // relative x coordinate
    ry = (int)component->instance[F_Component_y];      // relative y coordinate
    rw = (int)component->instance[F_Component_width];  // relative w coordinate
    rh = (int)component->instance[F_Component_height]; // relative h coordinate
  }
  
  if (depth == 0) {
    
    if (component->tag == Z_CONTAINER) {
      canvas = (r_canvas)component->object;

      *ox = canvas->buffer->ox;
      *oy = canvas->buffer->oy;
      
      *w = canvas->buffer->vw;
      *h = canvas->buffer->vh;
    }
    else {
      *ox = 0;
      *oy = 0;

      *w = rw;
      *h = rh;
    }

    *dx = rx;
    *dy = ry;
    
  }
  else {

    canvas = (r_canvas)component->object;

    Component_intersect(canvas->buffer->ox + *ox, canvas->buffer->oy + *oy, canvas->buffer->vw, canvas->buffer->vh, *ox + *dx, *oy + *dy, *w, *h, &ix, &iy, &iw, &ih);

    if (iw && ih) {
      *ox = ix - *dx;
      *oy = iy - *dy;
      
      *dx += rx - canvas->buffer->ox;
      *dy += ry - canvas->buffer->oy;

      if(*dx < 0) *dx = 0;
      if(*dy < 0) *dy = 0;
    }
    
    *w = iw;
    *h = ih;
  }
  
  // woempa(9, " %d b. dx = %d, dy = %d, ox = %d, oy = %d, w = %d, h = %d\n", depth, *dx, *dy, *ox, *oy, *w, *h);
  
  if (*h && *w) {
    if (component->parent) {
      getAbsoluteCoordinates(component->parent, dx, dy, w, h, ox, oy, depth + 1);
    }
  }
}

w_int Component_getAbsX(JNIEnv *env, jobject thisComponent) {
  w_int dx, dy, w, h, ox, oy;
  r_component component = (r_component)thisComponent[F_Component_wotsit];

  getAbsoluteCoordinates(component, &dx, &dy, &w, &h, &ox, &oy, 0);

  return dx - ox;
}

w_int Component_getAbsY(JNIEnv *env, jobject thisComponent) {
  w_int dx, dy, w, h, ox, oy;
  r_component component = (r_component)thisComponent[F_Component_wotsit];

  getAbsoluteCoordinates(component, &dx, &dy, &w, &h, &ox, &oy, 0);

  return dy - oy;
}

w_int Component_isVisible(r_component component) {
  
  if (component == rootCanvas->component) {
      
    /*
    ** The root canvas is always visible:
    */
      
    return 1;
  }
#ifdef AWT_CACHE
  else if (component->visible != -1) {

    /*
    ** Return the value from the cache:
    */
      
    return component->visible;
  }
#endif
  else if (component->instance && component->instance[F_Component_visible] != 1) {
      
    /*
    ** The component is explicitly marked as being invisible:
    */
      
    component->visible = 0;
      
    return 0;
  }
  else if (component->parent) {
      
    /*
    ** Look at the parent component, inherit its visibility and update the cache:
    */
      
    component->visible = Component_isVisible(component->parent);
      
    return component->visible;
  }
  else {
      
    /*
    ** The component is not attached to a parent and is therefore invisibile:
    */
      
    component->visible = 0;
      
    return 0;
  }
}
            
w_void Component_finalize(JNIEnv *env, jobject thisComponent) {

  r_component component;
  r_component c;
  r_canvas canvas;

  woempa(1, "Finalize (%k, %p)\n", instance2clazz(thisComponent), thisComponent);

  /*
  ** Lock the component tree:
  */
  
  rudolph_lock();
  
  woempa(1, "Locked component tree for (%k, %p)\n", instance2clazz(thisComponent), thisComponent);

  /*
  ** Resolve the component:
  */
  
  component = (r_component)thisComponent[F_Component_wotsit];

  /*
  ** We flush the events:
  */
  
  if (component == latest_entered) {
    latest_entered = NULL;
  }

  if (component == latest_pressed) {
    latest_pressed = NULL;
  }
  
  /*
  ** If the component is a java.awt.Container, release its off-screen buffers.
  */
  
  if (component->tag == Z_CONTAINER) {
    canvas = (r_canvas)component->object;
 
    releaseMem(canvas->buffer);
    releaseMem(canvas);
  }  
  
  /*
  ** Unregister component from the component tree:
  */

  registry_delComponent(component);

  /*
  ** Update the child components (if required):
  */

  for (c = component->headChild; c != NULL; c = c->next) {
    c->parent = NULL;
  }

  component->headChild = NULL;
  component->tailChild = NULL;

  /*
  ** Release the component's memory:
  */
  
  releaseMem(component);

  /*
  ** Unlock the component tree:
  */
  
  woempa(1, "Unlocking component tree for (%k, %p)\n", instance2clazz(thisComponent), thisComponent);
  
  rudolph_unlock();
}

w_void Component_createPeer(JNIEnv *env, jobject componentInstance) {
  
  r_component component = allocClearedMem(currentWonkaThread, sizeof(r_Component));

  component->tag = Z_COMPONENT;
  component->instance = componentInstance;
  component->visible = -1;
  
  // woempa(9, "called Component_constructor(%p, %k)\n", componentInstance, instance2clazz(componentInstance));
  
  componentInstance[F_Component_wotsit] = (w_word)component;
  
}

jobject Component_createImage(JNIEnv *env, jobject thisComponent, jint width, jint height) {

  r_component component = (r_component)thisComponent[F_Component_wotsit];
  w_instance imageInstance;
  r_image image;
  w_instance backgroundInstance = NULL;
  w_int i;

  // Create instance of image:
  imageInstance = allocInstance(currentWonkaThread, clazzImage);

  if (imageInstance == NULL) {
    woempa(9, "Unable to allocate imageInstance\n");

    return NULL;

  }

  // Create r_image instance:
  image = (r_image)allocMem(currentWonkaThread, sizeof(r_Image));

  if (image == NULL) {
    woempa(9, "Unable to allocate image\n");
    releaseMem(imageInstance);

    return NULL;

  }

  // Initialize image:
  //   Allocate image conponent:
  image->component = allocClearedMem(currentWonkaThread, sizeof(r_Component));
  if (image->component == NULL) {
    woempa(9, "Unable to allocate image component\n");
    releaseMem(imageInstance);
    releaseMem(image);

    return NULL;

  }


  // Initialize image component:
  image->component->tag = (r_Tag)Z_IMAGE;
  image->component->object = (unsigned long)image;
  image->component->refresh = 0;
  image->component->instance = imageInstance;
  image->component->parent = component;
  image->component->headChild = NULL;
  image->component->tailChild = NULL;
  image->component->next = NULL;
  image->component->prev = NULL;

  image->filename = NULL;
  image->x = 0;
  image->y = 0;
  image->w = width;
  image->h = height;

  // Allocate image buffer:
  
  woempa(9, "allocating %d bytes for %dx%d image\n", pixels2bytes((width + 1) * (height + 1)), width, height);
  
  image->buffer = (r_buffer)allocMem(currentWonkaThread, sizeof(r_Buffer) + pixels2bytes((width + 1) * (height + 1)));
  
  if (image->buffer == NULL) {
    woempa(9, "Unable to allocate image buffer\n");
    releaseMem(imageInstance);
    releaseMem(image);
    releaseMem(image->component);

    return NULL;
  }


  // Initialize image buffer:
  image->buffer->fw = width;
  image->buffer->fh = height;
  image->buffer->vw = width;
  image->buffer->vh = height;
  image->buffer->ox = 0;
  image->buffer->oy = 0;
  image->buffer->status = 1;

  // Add image to instance:
  if (imageInstance) {
    imageInstance[F_Image_wotsit] = (w_word)image;
  }

  // Resolve background color and draw background:
  Component_getBackground(component, &backgroundInstance, Component_getColor(F_SystemColor_window));
  for (i = 0; i < height; i++) {
     drawHLine(image->buffer, 0, i, width, color2pixel(backgroundInstance));
  }

  // Debug output:
  woempa(9, "called Component_createImage(%p, %p, %d, %d)\n", env, thisComponent, width, height);

  return (w_instance) imageInstance;
  
}

w_instance Component_getGraphics(JNIEnv *env, jobject thisComponent) {

  w_instance graphicalCntxt = NULL;
  w_instance fontInstance = NULL;
  w_instance foregroundInstance = NULL;
  w_instance backgroundInstance = NULL;
  
  r_component component = (r_component)thisComponent[F_Component_wotsit];
  
  /*
  ** Changed this to get the IAA demo running. If it causes any problems in the
  ** future, we need to find a more general fix...
  */

  // if (component->parent) {

  if (1) {
  
    /*
    ** Allocate memory for graphical context:
    */
    
    graphicalCntxt = (w_instance)allocInstance(currentWonkaThread, clazzGraphics);

    if (graphicalCntxt) {
      
      /*
      ** Initialize graphical context:
      */
      
      graphicalCntxt[F_Graphics_wotsit] = (w_word)component;

      Component_getFontInstance((r_component)thisComponent[F_Component_wotsit], &fontInstance);
      
      setReferenceField(graphicalCntxt, fontInstance, F_Graphics_font);

      Component_getForeground((r_component)thisComponent[F_Component_wotsit], &foregroundInstance, Component_getColor(F_SystemColor_windowText));
      setReferenceField(graphicalCntxt, foregroundInstance, F_Graphics_foreground);

      Component_getBackground((r_component)thisComponent[F_Component_wotsit], &backgroundInstance, Component_getColor(F_SystemColor_window));
      setReferenceField(graphicalCntxt, backgroundInstance, F_Graphics_background);

    }
    else {
      woempa(9, "Unable to allocate graphicalCntxt!\n");
    }
  
    return graphicalCntxt;
  }
  else {
    return NULL;
  }
  
}

w_void Component_awt(JNIEnv *env, jobject thisComponent, jint status) {

  /*
  ** Initialize rudolph, called by the static initializer of java.awt.Component:
  */
  
  dispatcher_awt((void *)status);
}

w_void Component_tag(JNIEnv *env, jobject thisComponent, jint type, jint cache, jboolean render) {

  r_component component = (r_component)thisComponent[F_Component_wotsit];
  
  // Debug output:
  woempa(5, "called Component_tag(type = '%k', type = %i, render = %i\n", instance2clazz(thisComponent), type, render);

  /*
  ** Invalidation process:
  **  - type = 0 : up-to-date, no update needed
  **  - type = 1 : local update, fast, use the cached version of container
  **  - type = 2 : global update, slow, re-render all parent containers on path to rootCanvas
  */
  
  /*
  ** Invalidate parent components in component-tree (bottom-up):
  */
  
  invalidateTreeUpwards(component, type);

  /*
  ** Invalidate the field caches if required (top-down):
  */

#ifdef AWT_CACHE  
  if (cache) {
    switch (cache) {
      case 1:
        Component_resetFontCache(component);
        break;
      case 2:
        Component_resetForegroundColorCache(component);
        break;
      case 3:
        Component_resetBackgroundColorCache(component);
        break;
      case 4:
          if (component->tag == Z_CONTAINER && component->parent != NULL) {
             Component_resetVisibilityCache(component->parent);
          }
          Component_resetVisibilityCache(component);
        break;
      default:
        wabort(ABORT_WONKA, "something whicked happened - unkown cache identifier parsed\n");
        break;
    }
  }
#endif
  
  if (render) {
    
    /*
    ** Render:
    */

    
    if (type == 2 && rootCanvas->component->refresh) {
      canvas_drawCanvas(rootCanvas, 0);
    }
    
    if (type == 1) {
      canvas_drawComponent(component, 1);
    }
  }
}


