/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: Container.c,v 1.19 2002/03/20 13:04:20 gray Exp $
*/

#include "ts-mem.h"       // allocMem(), releaseMem(), reallocMem()
#include "strings.h"      // String2string()
#include "loading.h"      // instance2clazz()
#include "realms.h"

#include "awt-classes.h"
#include "rudolph.h"
#include "canvas.h"
#include "registry.h"
#include "Component.h"
#include "Event.h"

void Container_createPeer(JNIEnv *env, jobject thisContainer, jboolean nested) {

  r_canvas canvas = canvas_constructor(thisContainer);
    
  if (!nested) {
    
    /*
    ** Add the canvas to the root canvas in the registry or component tree:
    */
    
    registry_addContainer(rootCanvas, canvas);
    
    /*
    ** Register container as being a global so it does not get garbage collected.
    ** Note that java.awt.Frames are never being garbage collected unless closed,
    ** even though there is no reference to them.
    */
    
    newGlobalReference(thisContainer);
  }
  
  woempa(9, "called Container_create(%k, canvas = %p, component = %p)\n", instance2clazz(thisContainer), canvas, canvas->component);
  
  thisContainer[F_Component_wotsit] = (w_word)canvas->component;
  
}

int Container_verifyBounds(r_component dst, r_component src) {
    
  r_canvas canvas = (r_canvas)dst->object;
  w_instance i = (w_instance)src->instance;

  woempa(5, "VERIFY: %d, %d %dx%d -> %dx%d\n", (int)i[F_Component_x], (int)i[F_Component_y], (int)i[F_Component_width], (int)i[F_Component_height], canvas->buffer->fw, canvas->buffer->fh);

  #ifdef RUNTIME_CHECKS
    if (canvas->buffer->ox + canvas->buffer->vw > canvas->buffer->fw || canvas->buffer->oy + canvas->buffer->vh > canvas->buffer->fh) {
      wabort(ABORT_WONKA, "invalid viewport size:  %d + %d -> %d,  %d + %d -> %d\n", canvas->buffer->ox, canvas->buffer->vw, canvas->buffer->fw, canvas->buffer->oy, canvas->buffer->vh, canvas->buffer->fh);
    }
    
    if ((dst->instance[F_Component_width] != (unsigned)canvas->buffer->fw) || (dst->instance[F_Component_height] != (unsigned)canvas->buffer->fh)) {
      wabort(ABORT_WONKA, "canvas buffer width/height mismatch: %d == %d, %d == %d\n", dst->instance[F_Component_width], canvas->buffer->fw, dst->instance[F_Component_height], canvas->buffer->fh);
    } 
  #endif

  if(src->tag == Z_CONTAINER) {
    return 1;
  } 
  else {
    return ((int)(i[F_Component_x] + i[F_Component_width]) <= canvas->buffer->fw) && ((int)(i[F_Component_y] + i[F_Component_height]) <= canvas->buffer->fh);
  }
}

void Container_addComponent(JNIEnv *env, jobject thisContainer, jobject windowInstance) {

  r_component component1 = (r_component)thisContainer[F_Component_wotsit];
  r_component component2 = (r_component)windowInstance[F_Component_wotsit];
  r_canvas canvas = (r_canvas)component1->object;

  /*
  ** Add the component to the registry or component tree:
  */
  
  registry_addComponent(canvas, component2);

}

void Container_addContainer(JNIEnv *env, jobject thisContainer, jobject windowInstance) {
  
  r_component component1 = (r_component)thisContainer[F_Component_wotsit];
  r_component component2 = (r_component)windowInstance[F_Component_wotsit];
  r_canvas canvas1 = (r_canvas)component1->object;
  r_canvas canvas2 = (r_canvas)component2->object;
      
  woempa(5, "called Container_addContainer(%k [%p], %k [%p])\n", instance2clazz(thisContainer), canvas1, instance2clazz(windowInstance), canvas2);

  /* 
  ** Add the container to the registry or component tree:
  */
  
  registry_addContainer(canvas1, canvas2);
  
}

void Container_removeComponent(JNIEnv *env, jobject thisContainer, jobject componentInstance) {
  
  r_component component = (r_component)componentInstance[F_Component_wotsit];

  woempa(9, "component instance = %p : component = %p (%k)\n", componentInstance, component,  instance2clazz(component->instance));
  /*
  ** Remove the component from the registry or component tree:
  */
   
  registry_delComponent(component);

  /*
  ** We flush the events:
  */
  
  if (component == latest_entered) {
    latest_entered = NULL;
  }

  if (component == latest_pressed) {
    latest_pressed = NULL;
  }
 
}

void Container_scale(JNIEnv *env, jobject thisContainer, jint width, jint height) {

  r_component component = (r_component)thisContainer[F_Component_wotsit];
  r_canvas canvas = (r_canvas)component->object;
  
  /*
  ** Coalesce rescale request: if the component has been rescaled 
  ** (and thus released) already, we don't have to reallocate it
  ** again.
  */
  
  buffer_release(canvas);
  
  /*
  ** Validate the requested size and update the buffer fields.
  */
  
  if (width < 1 || height < 1 || (height * width) > MAX_SIZE) {

    woempa(10, "warning: refused to scale container '%k' with canvas = %p to size w = %i, h = %i)\n", instance2clazz(thisContainer), canvas, width, height);

    /*
    ** Some layout managers and user-defined layoutmanagers might attempt
    ** to scale to java.awt.Container's to invalid sizes; in such case we
    ** release the buffer and we set the buffer size to 0x0.
    */

    canvas->buffer->fw = 0;
    canvas->buffer->fh = 0;
    canvas->buffer->ox = 0;
    canvas->buffer->oy = 0;
    canvas->buffer->vw = 0;
    canvas->buffer->vh = 0;  
  }
  else {
    
    woempa(5, "called Container_scale(instance = %k, canvas = %p, w = %i, h = %i)\n", instance2clazz(thisContainer), canvas, width, height);
    
    canvas->buffer->fw = width;
    canvas->buffer->fh = height;
    canvas->buffer->ox = 0;
    canvas->buffer->oy = 0;
    canvas->buffer->vw = width;
    canvas->buffer->vh = height;  

  }
}

