/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/* $Id: Event.c,v 1.23 2002/03/21 13:04:09 gray Exp $ */


#include "awt-classes.h"  // F_Event*
#include "heap.h"         // setReferenceField
#include "realms.h"        // W_Realm_system
#include "stack.h"        // removeLocalReference
#include "threads.h"	  // currentWonkaThread
#include "clazz.h"        // clazz2Class
#include "checks.h"
#include "canvas.h"
#include "Component.h"
#include "Event.h"
#include "Button.h"
#include "Checkbox.h"
#include "registry.h"
#include "wonkatime.h"

r_component latest_entered = NULL;
r_component latest_pressed = NULL;

inline static int contains(int rx, int ry, int rw, int rh, int px, int py) {
  return ((rx < px) && (px < rx + rw) && (ry < py) && (py < ry + rh));
}

void Event_dispatchEvent(r_canvas canvas, r_event event) {

  static r_event e = NULL;
  r_component source = NULL;
    
  /*
  ** Resolve the source component:
  */

  rudolph_lock(); 
  
  source = Event_getSourceComponent(rootCanvas, event);

  rudolph_unlock();

  if (latest_entered != source) {

    /*
    ** Initialize the event if not already done so:
    */
  
    if (e == NULL) {
      e = allocMem_system(sizeof(r_Event));
  
      if (e == NULL) {
        woempa(9, "unable to allocate memory for an event -- ignoring\n");
        return;
      }
    }
    
    /*
    ** Take care of "mouse entered" and "mouse exited" events:
    */
    
    if (latest_entered != NULL) {

      // Make sure buttons get released:
      if (latest_entered->tag == Z_BUTTON) {
        Button_eventMouseExit(latest_entered, e);
      }

      // Update the coordinates:
      e->x = event->x;
      e->y = event->y;
      
      // Dispatch a 'mouse exited' event:
      e->tag = R_EVENT_MOUSE_EXITED; 
      Event_addMouseEvent(latest_entered, e);
      
      // Dispatch a 'focus lost' event:
      // 
      // e->tag = R_EVENT_FOCUS_LOST;      
      // Event_addFocusEvent(latest_entered, e);
      // 
      
    }
    
    if (source != NULL) {
      w_instance component;
      
      // Update the coordinates:
      e->x = event->x;
      e->y = event->y;

      // Dispatch a 'mouse entered' event:
      e->tag = R_EVENT_MOUSE_ENTERED; 
      Event_addMouseEvent(source, e);
      
      // Dispatch a 'focus lost' event to the current owner.
      e->tag = R_EVENT_FOCUS_LOST;
      component = (w_instance)getStaticReferenceField(clazzComponent, F_Component_focusComponent);
      if(component) {
        Event_addFocusEvent((r_component)component[F_Component_wotsit], e);
      }
      
      // Dispatch a 'focus gained' event:
      e->tag = R_EVENT_FOCUS_GAINED;
      Event_addFocusEvent(source, e);
    }
    
  }

  /* 
  ** Should we move on or can we return directly?
  */

  if (source != NULL) {

    /*
    ** Update the static 'latest_entered' field for use above:
    */
    
    latest_entered = source;
    
    if (event->tag == R_EVENT_MOUSE_PRESSED) {
      latest_pressed = source;
    }

    if (event->tag == R_EVENT_MOUSE_RELEASED && latest_pressed == source) {
      event->tag = R_EVENT_MOUSE_CLICKED;

      /*
      ** Action events:
      **   Action events are only fired by Buttons, Lists, MenuItems and
      **   TextFields.
      */
  
      if (source->tag != Z_CONTAINER && source->tag != Z_LABEL && source->tag != Z_CHECKBOX) {
        Event_addActionEvent(source, event);
      }
      
      /*
      ** Mouse events:
      */
      
      Event_addMouseEvent(source, event);
      
      event->tag = R_EVENT_MOUSE_RELEASED;
    }

    /*
    ** Mouse and MouseMotion events:
    */

    if (event->tag == R_EVENT_MOUSE_MOVED || event->tag == R_EVENT_MOUSE_DRAGGED) {
      Event_addMouseMotionEvent(source, event);
    }
    else {
      
      /*
      ** Custom or component specific events:
      */
    
      if (source->tag == Z_BUTTON) {
        Button_eventMouseClick(source, event);
          // Will take care of pressing and releasing the buttons.
      }
      else if (source->tag == Z_CHECKBOX) {
        Checkbox_eventMouseClick(source, event);
          // Will take care of dispatching a checkbox' ItemEvents.
      }

      if (event->tag == R_EVENT_MOUSE_RELEASED && latest_pressed != NULL) {
        Event_addMouseEvent(latest_pressed, event);
        latest_pressed = NULL;
      } 
      else {
        /*
        ** A generic mouse event:
        */
        
        Event_addMouseEvent(source, event);
      }
    }

    /*
    ** Remark:
    **   Some special (less common/global) events like for instance 
    **   ItemEvents should only be triggered when a checkbox changed
    **   state and are therefore handled by the Checkbox_mouseClick()
    **   function in Checkbox.c itself.
    */
     
  }
 
}

r_component Event_getSourceComponent(r_canvas canvas, r_event event) {

  static int offset_x = 0;
  static int offset_y = 0;
  r_component component;
  r_component source = NULL;
  r_canvas c;
  int x;
  int y;
  int w;
  int h;
    
  for (component = canvas->component->headChild; component != NULL; component = component->next) {
 
    if (Component_isVisible(component)) {

      // Resolve coordinates of current component:
      getRelativeCoordinates(component, &x, &y, &w, &h);
    
      if (contains(offset_x + x, offset_y + y, w, h, event->x, event->y)) {

        if (component->tag == Z_CONTAINER) {
          
          c = (r_canvas)component->object;
          
          offset_x += (x - c->buffer->ox);
          offset_y += (y - c->buffer->oy);
          
          source = Event_getSourceComponent(c, event);
          
          if (source == NULL) {
            source = component;
          }
          
          offset_x -= (x - c->buffer->ox);
          offset_y -= (y - c->buffer->oy);
        }
        else {
          source = component;
        }
      }
    }
  }
  
  return source;

}

void Event_addMouseEvent(r_component source, r_event event) {

  if (source && source->instance[F_Component_mouseListener]) {

    static jmethodID method = NULL;
    w_thread thread = currentWonkaThread;
    JNIEnv *env = w_thread2JNIEnv(thread);
    w_instance mouseEvent = allocInstance(thread, clazzMouseEvent);
    w_int x, y, w, h, dx, dy;

    if(!mouseEvent) {
      
      return;

    }
  
    // Debug output:
    woempa(5, "called Event_addMouseEvent(%p, %p): tag = %i\n", source, event, event->tag);

    if (method == NULL) {
      method = (*env)->GetMethodID(env, clazz2Class(clazzEventQueue), "postNativeEvent", "()V");
    }

    // Resolve relative event coordinates:
    getAbsoluteCoordinates(source, &x, &y, &w, &h, &dx, &dy, 0);

    // Create appropriate java.awt.event.MouseEvent:
    mouseEvent[F_AWTEvent_id] = event->tag; 
    mouseEvent[F_MouseEvent_x] = event->x - x + dx;  // relative x coordinate
    mouseEvent[F_MouseEvent_y] = event->y - y + dy;  // relative y coordinate
    setReferenceField(mouseEvent, source->instance, F_EventObject_source);

    // Attach MouseEvent to EventQueue instance:
    setReferenceField(defaultEventQueue, mouseEvent, F_EventQueue_nativeAWTEvent);
 
    // Call method postNativeEvent:
    (*env)->CallVoidMethod(env, defaultEventQueue, method);

    // Remove reference from local reference table:
    removeLocalReference(thread->top, mouseEvent);

  }
}

void Event_addMouseMotionEvent(r_component source, r_event event) {

  if (source && source->instance[F_Component_mouseMotionListener]) {

    static jmethodID method = NULL;
    w_thread thread = currentWonkaThread;
    JNIEnv *env = w_thread2JNIEnv(thread);
    w_instance mouseEvent = allocInstance(thread, clazzMouseEvent);
    w_int x, y, w, h, dx, dy;
  
    if(!mouseEvent) {
      
      return;

    }
  
    // Debug output:
    woempa(5, "called Event_addMouseEvent(%p, %p): tag = %i\n", source, event, event->tag);

    if (method == NULL) {
      method = (*env)->GetMethodID(env, clazz2Class(clazzEventQueue), "postNativeEvent", "()V");
    }

    // Resolve relative event coordinates:
    getAbsoluteCoordinates(source, &x, &y, &w, &h, &dx, &dy, 0);

    // Create appropriate java.awt.event.MouseEvent:
    mouseEvent[F_AWTEvent_id] = event->tag; 
    mouseEvent[F_MouseEvent_x] = event->x - x + dx;  // relative x coordinate
    mouseEvent[F_MouseEvent_y] = event->y - y + dy;  // relative y coordinate
    setReferenceField(mouseEvent, source->instance, F_EventObject_source);

    // Attach MouseEvent to EventQueue instance:
    setReferenceField(defaultEventQueue, mouseEvent, F_EventQueue_nativeAWTEvent);
 
    // Call method postNativeEvent:
    (*env)->CallVoidMethod(env, defaultEventQueue, method);

    // Remove reference from local reference table:
    removeLocalReference(thread->top, mouseEvent);

  }
}

void Event_addActionEvent(r_component source, r_event event) {

  if (source && source->instance[F_Component_actionListener]) {

    if (event->tag == R_EVENT_MOUSE_CLICKED) {

      static jmethodID method = NULL;
      w_thread thread = currentWonkaThread;
      JNIEnv *env = w_thread2JNIEnv(thread);
      w_instance actionEvent = allocInstance(thread, clazzActionEvent); 

      if (!actionEvent) {
        woempa(9, "No memory for actionEvent!\n");

        return;

      }

      // Debug output:
      woempa(5, "called Event_addActionEvent(%p, %p)\n", source, event);

      if (method == NULL) {
        method = (*env)->GetMethodID(env, clazz2Class(clazzEventQueue), "postNativeEvent", "()V");
      }

      // Create appropriate java.awt.event.ActionEvent:
      actionEvent[F_AWTEvent_id] = event->tag; 
      setReferenceField(actionEvent, source->instance, F_EventObject_source);
 
      // Attach ActionEvent to EventQueue instance:
      setReferenceField(defaultEventQueue, actionEvent, F_EventQueue_nativeAWTEvent);
 
      // Call method postNativeEvent:
      (*env)->CallVoidMethod(env, defaultEventQueue, method);  

      // Remove reference from local reference table:
      removeLocalReference(thread->top, actionEvent);
    }
  }

}

void Event_addFocusEvent(r_component source, r_event event) {

  if(source && event->tag == R_EVENT_FOCUS_GAINED) {
    setStaticReferenceField(clazzComponent, F_Component_focusComponentPrev, getStaticReferenceField(clazzComponent, F_Component_focusComponent));
    setStaticReferenceField(clazzComponent, F_Component_focusComponent, source->instance);
    woempa(5, " FOCUS SOURCE : %p, %p, %p\n", source->instance, getStaticReferenceField(clazzComponent, F_Component_focusComponent), 
                                              getStaticReferenceField(clazzComponent, F_Component_focusComponentPrev));
  }

  if (source && (source->instance[F_Component_focusListener] || isAssignmentCompatible(instance2clazz(source->instance), clazzWindow))) {

    static jmethodID method = NULL;
    w_thread thread = currentWonkaThread;
    JNIEnv *env = w_thread2JNIEnv(thread);
    w_instance focusEvent = allocInstance(thread, clazzFocusEvent);
 
    // Debug output:
    woempa(5, "called Event_addFocusEvent(%p, %p)\n", source, event);   
 
    if (method == NULL) {
      method = (*env)->GetMethodID(env, clazz2Class(clazzEventQueue), "postNativeEvent", "()V");
    }

    // Create appropriate java.awt.event.FocusEvent:
    focusEvent[F_AWTEvent_id] = event->tag;
    setReferenceField(focusEvent, source->instance, F_EventObject_source);

    // Attach FocusEvent to EventQueue instance:
    setReferenceField(defaultEventQueue, focusEvent, F_EventQueue_nativeAWTEvent);

    // Call method postNativeEvent:
    (*env)->CallVoidMethod(env, defaultEventQueue, method);

    // Remove reference from local reference table:
    removeLocalReference(thread->top, focusEvent);

  }  
}

void Event_addKeyEvent(w_int VK, w_int keychar, w_int mod, w_int pressed) {

  /*
  ** Get the component with focus.
  */

  w_instance source = getStaticReferenceField(clazzComponent, F_Component_focusComponent);

  if (source && source[F_Component_keyListener]) {

    static jmethodID method = NULL;
    w_thread thread = currentWonkaThread;
    JNIEnv *env = w_thread2JNIEnv(thread);
    w_instance keyEvent = allocInstance(thread, clazzKeyEvent);
  
    if(!keyEvent) {
      
      return;

    }
  
    // Debug output:
    woempa(5, "called Event_addKeyEvent(%d, %d, %d, %d)\n", VK, keychar, mod, pressed);

    if (method == NULL) {
      method = (*env)->GetMethodID(env, clazz2Class(clazzEventQueue), "postNativeEvent", "()V");
    }

    // Create appropriate java.awt.event.MouseEvent:
    keyEvent[F_AWTEvent_id] = pressed; 
    keyEvent[F_KeyEvent_keyCode] = VK;
    keyEvent[F_KeyEvent_keyChar] = keychar;
    keyEvent[F_InputEvent_modifiers] = mod;
    keyEvent[F_InputEvent_timeStamp] = (w_long)getNativeSystemTime();
    setReferenceField(keyEvent, source, F_EventObject_source);

    // Attach KeyEvent to EventQueue instance:
    setReferenceField(defaultEventQueue, keyEvent, F_EventQueue_nativeAWTEvent);
 
    // Call method postNativeEvent:
    (*env)->CallVoidMethod(env, defaultEventQueue, method);

    // Remove reference from local reference table:
    removeLocalReference(thread->top, keyEvent);
  }
}

