/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

#include <string.h>
#include <stdio.h>

#include "hashtable.h"    // ht_register
#include "loading.h"      // classMustBeInitialized()
#include "strings.h"      // cstring2String()
#include "realms.h"
#include "threads.h"       // currentWonkaThread
#include "ts-mem.h"       // allocMem(), releaseMem()

#include "awt-classes.h"  // F_Component_*
#include "canvas.h"
#include "varia.h"

#include "Font.h"

//#define MAX_FONTS 15
//#define MAX_FONTS 20     //140302
#define MAX_FONTS 21        //140302

#include "fonts/helvR08.h"
//#include "fonts/helvR10.h"
#include "fonts/helvR12.h"
#include "fonts/helvR14.h"
#include "fonts/helvR17.h"
#include "fonts/helvR20.h"
#include "fonts/helvR25.h"
#include "fonts/helvR34.h"

#include "fonts/helvB08.h"
//#include "fonts/helvB10.h"
#include "fonts/helvB12.h"
#include "fonts/helvB14.h"
#include "fonts/helvB17.h"
#include "fonts/helvB20.h"
#include "fonts/helvB25.h"
#include "fonts/helvB34.h"

//#include "fonts/helvO08.h"
//#include "fonts/helvO10.h"
//#include "fonts/helvO12.h"
//#include "fonts/helvO14.h"
//#include "fonts/helvO17.h"
//#include "fonts/helvO20.h"
//#include "fonts/helvO25.h"
//#include "fonts/helvO34.h"

#include "fonts/courR10.h"
#include "fonts/courR12.h"
#include "fonts/courR14.h"
#include "fonts/courR17.h"
#include "fonts/courR20.h"
#include "fonts/courR25.h"
//#include "fonts/courR34.h"

//#include "fonts/courB10.h"
//#include "fonts/courB12.h"
//#include "fonts/courB14.h"
//#include "fonts/courB17.h"
//#include "fonts/courB20.h"
//#include "fonts/courB25.h"
//#include "fonts/courB34.h"

//#include "fonts/courO10.h"
//#include "fonts/courO12.h"
//#include "fonts/courO14.h"
//#include "fonts/courO17.h"
//#include "fonts/courO20.h"
//#include "fonts/courO25.h"
//#include "fonts/courO34.h"


//140302
#include "fonts/12x13ja.h"


w_instance defaultFont;

static r_font *fonts = NULL;
static int num_fonts = 0;

w_void Font_init(void) {

  w_thread thread = W_Thread_system;
  
  /*
  ** Pre-process font data: store them in a static table;
  ** allocate memory for that table in function Font_readData() to allow that
  ** Font_readData() can be called elsewhere aswell
  */

  // Debug output:
  woempa(9, "loading and preparing fonts:\n");

  // Initialize Font class:
  if (clazzFont == NULL) {
    classMustBeInitialized(clazzFont, thread);
    woempa(9, "intialised clazzFont \n");
  }

  // Load and prepare fonts:

  // Beside this function, another system function, is trying to load the font
  // courR12. It is the anakin console. It is only initialised if wonka runs
  // on anakin. If that initialisation takes place before this Font_init() is
  // called, there are no fonts loaded yet, and the console initialisation
  // loads font courR12 as the first font. For that reason, if not running
  // on anakin, that font must be loaded as the first font too.
  Font_readData((char *) "courR12", (char **)courR12_font);

  Font_readData((char *) "helvR08", (char **)helvR08_font);
//  Font_readData((char *) "helvR10", (char **)helvR10_font);
  Font_readData((char *) "helvR12", (char **)helvR12_font);
  Font_readData((char *) "helvR14", (char **)helvR14_font);
  Font_readData((char *) "helvR17", (char **)helvR17_font);
  Font_readData((char *) "helvR20", (char **)helvR20_font);
  Font_readData((char *) "helvR25", (char **)helvR25_font);
  Font_readData((char *) "helvR34", (char **)helvR34_font);

  Font_readData((char *) "helvB08", (char **)helvB08_font);
//  Font_readData((char *) "helvB10", (char **)helvB10_font);
  Font_readData((char *) "helvB12", (char **)helvB12_font);
  Font_readData((char *) "helvB14", (char **)helvB14_font);
  Font_readData((char *) "helvB17", (char **)helvB17_font);
  Font_readData((char *) "helvB20", (char **)helvB20_font);
  Font_readData((char *) "helvB25", (char **)helvB25_font);
  Font_readData((char *) "helvB34", (char **)helvB34_font);

//  Font_readData((char *) "helvO08", (char **)helvO08_font);
//  Font_readData((char *) "helvO10", (char **)helvO10_font);
//  Font_readData((char *) "helvO12", (char **)helvO12_font);
//  Font_readData((char *) "helvO14", (char **)helvO14_font);
//  Font_readData((char *) "helvO17", (char **)helvO17_font);
//  Font_readData((char *) "helvO20", (char **)helvO20_font);
//  Font_readData((char *) "helvO25", (char **)helvO25_font);
//  Font_readData((char *) "helvO34", (char **)helvO34_font);

  Font_readData((char *) "courR10", (char **)courR10_font);
  Font_readData((char *) "courR14", (char **)courR14_font);
  Font_readData((char *) "courR17", (char **)courR17_font);
  Font_readData((char *) "courR20", (char **)courR20_font);
  Font_readData((char *) "courR25", (char **)courR25_font);
//  Font_readData((char *) "courR34", (char **)courR34_font);

//  Font_readData((char *) "courB10", (char **)courB10_font);
//  Font_readData((char *) "courB12", (char **)courB12_font);
//  Font_readData((char *) "courB14", (char **)courB14_font);
//  Font_readData((char *) "courB17", (char **)courB17_font);
//  Font_readData((char *) "courB20", (char **)courB20_font);
//  Font_readData((char *) "courB25", (char **)courB25_font);
//  Font_readData((char *) "courB34", (char **)courB34_font);

//  Font_readData((char *) "courO10", (char **)courO10_font);
//  Font_readData((char *) "courO12", (char **)courO12_font);
//  Font_readData((char *) "courO14", (char **)courO14_font);
//  Font_readData((char *) "courO17", (char **)courO17_font);
//  Font_readData((char *) "courO20", (char **)courO20_font);
//  Font_readData((char *) "courO25", (char **)courO25_font);
//  Font_readData((char *) "courO34", (char **)courO34_font);

  Font_readData((char *) "f12x13ja", (char **)f12x13ja_font);


  /* 
  ** Setup a default font:
  */
  
  // Create default font:
  defaultFont = (w_instance)allocInstance(thread, clazzFont);
  defaultFont[F_Font_wotsit] = (w_word)fonts[1];
  setReferenceField(defaultFont, getStringInstance(thread, cstring2String(thread, fonts[1]->name, strlen(fonts[1]->name))), F_Font_name);
  setReferenceField(defaultFont, getStringInstance(thread, cstring2String(thread, fonts[1]->family, strlen(fonts[1]->family))), F_Font_family);
  defaultFont[F_Font_size] = fonts[1]->size;
  defaultFont[F_Font_style] = fonts[1]->style;
  newGlobalReference(defaultFont);
  
  // Debug output:
  woempa(9, "installed '%s' (style: %i, size: %i) as the default font\n", fonts[0]->name, fonts[0]->style, fonts[0]->size);
}

/*
** Native implementation of java.awt.Font.create(String fontname, int style, int size)
** the main call of java.awt.Font.<constructor>(String fontname, int style, int size)
*/

w_void Font_create(JNIEnv *env, jobject thisFont, jobject name, jint style, jint size) {
  w_string   wString;
  char*      cString;
  w_instance stringInstanceName;
  w_instance stringInstanceFamily;
  r_font     font = NULL;

  if (name!=NULL)
    wString = (w_string)name[F_String_wotsit];
  else
    wString = cstring2String(currentWonkaThread, "Default", strlen("Default"));

  cString = allocMem(currentWonkaThread, (wString->length+1) * sizeof(char));
  w_string2c_string(wString, cString);

  // get the closest match to the requested font
  font = Font_getFont(cString, style, size);

  // like in jdk, assign the 'name' parameter of this function 'Font_create', to the returned
  // java font 'thisFont'. Leave the name of the internal font struct to which it will point
  // (via the wotsit field) intact (otherwise the internal font in tabel 'fonts' is changed!).
  stringInstanceName = getStringInstance(currentWonkaThread,wString);
  setReferenceField(thisFont, stringInstanceName, F_Font_name);

  // set thisFont's family name (reuse 'wString')
  wString = cstring2String(currentWonkaThread, (char*)font->family, strlen(font->family));
  stringInstanceFamily = getStringInstance(currentWonkaThread,wString);
  setReferenceField(thisFont, stringInstanceFamily, F_Font_family);

  // set thisFont's size and style
  thisFont[F_Font_size]  = font->size;
  thisFont[F_Font_style] = font->style;

  // Attach an internal rudolph font struct to the Java font instance:
  thisFont[F_Font_wotsit] = (w_word)font;

  releaseMem(cString);

  // woempa(9, "called Font_create(): '%s, %i' --> '%s, %i'\n", cString, size, font->name, font->size);
}

/*
** Auxilliary: read an array of font data and build a font out of it
*/

w_void Font_readData(char *name, char **lines) {
  w_thread thread = W_Thread_system;
  r_char character = NULL;
  r_font font;
  char *text;
  char c;
  int loop;
  int temp = 0;
  int in_bitmap = 0;
  int hexval = 0;
  int which_char = 0;
  int which_bit = 0;
  int line_no = 0;
  int char_no = 0;
  int xpos = 0;
  int ypos = 0;
  int leadAscent = 0;
  int leadDescent = 0;
  int bitmapsize = 0;
  int fontsInd = 0;
  int encoding = 0;

  if (num_fonts >= MAX_FONTS) {
    woempa(9, "WARNING, MAX_FONTS exceeded, cannot load font '%s' \n", name);
  }
  else {
    /*
    ** Allocate memory for static array 'fonts':
    */
    
    if (fonts == NULL) {
      fonts = (r_font *)allocClearedMem(thread, MAX_FONTS * sizeof(r_font));
    }

    /*
    ** Allocate cleared memory for the new font: the memory has to be
    ** cleared because its content will be use in subsequent checks.
    */
    
    font = (r_font)allocClearedMem(thread, sizeof(r_Font));

    /*
    ** Set the new font's name:
    */
    
    font->name = (char *)allocMem(thread, strlen(name) + 1);
    strcpy(font->name, name);
    
    /*
    ** Pass 1:
    **   Calculate the amount of memory required to store the
    **   rendered character table:
    */
    
    while (lines[line_no]) {
      if (strcmp(lines[line_no++], "ENDCHAR") == 0) char_no++;
    }

    line_no = 0;

    /*
    ** Pass 2:
    **   Process the data file and pre-render each character:
    */
    
    while (lines[line_no]) {
      text = lines[line_no];

      if (strncmp(text, "STARTCHAR", 9) == 0) {
        in_bitmap = 0;
        character = (r_Char *)allocMem(thread, sizeof(r_Char));
      }
      else if (strncmp(text, "ENCODING", 8) == 0) {
        encoding = atoi(text + 9);
//140302        character->value = (char)encoding;
        character->value = (w_char)encoding;
      }
      else if (strncmp(text, "FAMILY_NAME", 11) == 0) {
        // don't copy the quotes enclosing the family name in the font file
        font->family = (char *)allocMem(thread, strlen(text + 13));
        strncpy(font->family, text + 13, strlen(text + 13) - 1);
        font->family[strlen(text + 13) - 1] = '\0';
      }
      else if (strncmp(text, "WEIGHT_NAME", 11) == 0) {
        if (strncmp(text+13, "Medium", 6) == 0) {
          font->style = font->style | (int)getStaticReferenceField(clazzFont,F_Font_PLAIN);
        }
        else if (strncmp(text+13, "Bold", 4) == 0) {
          font->style = font->style | (int)getStaticReferenceField(clazzFont,F_Font_BOLD);
        }
        else {
          woempa(9, "WARNING: un-expected value for WEIGHT_NAME in font \"%s\"; \"Medium\" is assumed\n", name);
          font->style = font->style | (int)getStaticReferenceField(clazzFont,F_Font_PLAIN);
        }
      }
      else if (strncmp(text, "SLANT", 5) == 0) {
        if (strncmp(text+7, "R", 1) == 0) {
          // leave the style unchanged
        }
        else if (strncmp(text+7, "I", 1) == 0) {
          font->style = font->style | (int)getStaticReferenceField(clazzFont,F_Font_ITALIC);
        }
        else {
          woempa(9, "WARNING: un-expected value for SLANT in font \"%s\"; \"R\" is assumed\n", name);
          // leave the style unchanged
        }
      }
      else if (strncmp(text, "PIXEL_SIZE", 10) == 0) {
        font->size = atoi(text + 11);
      }
      else if (strncmp(text, "FONT_ASCENT", 11) == 0) {
        font->ascent = atoi(text + 12);
      }
      else if (strncmp(text, "FONT_DESCENT", 12) == 0) {
        font->descent = atoi(text + 13);
      }
      else if (strncmp(text, "SPACING", 7) == 0) {
        if (strncmp(text + 8, "\"P\"", 3) == 0)
          font->proportional = 1;
        else
          font->proportional = 0;
      }
      else if (strncmp(text, "FACE_NAME", 9) == 0) {
        font->face_name = (char *)allocMem(thread, strlen(text + 10) + 1);
        strcpy(font->face_name, text + 10);
      }
      else if (strncmp(text, "SETWIDTH_NAME", 13) == 0) {
        font->width_name = (char *)allocMem(thread, strlen(text + 14) + 1);
        strcpy(font->width_name, text + 14);
      }
      else if (strncmp(text, "BBX", 3) == 0) {
        sscanf(text, "BBX %d %d %d %d", &character->width, &character->height, &character->xoffset, &character->yoffset);
        // update the max descent
        if (character->yoffset < - font->maxDescent) font->maxDescent = - character->yoffset;
        // update the max ascent
        if (character->height + character->yoffset > font->maxAscent) font->maxAscent = character->height + character->yoffset;
        // Use the width and height to allocate space for the data:
        if (character->height && character->width) character->data = (char *)allocMem(thread, character->height * character->width * sizeof(char));
      }
      else if (strncmp(text, "DWIDTH", 6) == 0) {
        sscanf(text, "DWIDTH %d %d", &character->advance, &temp);
        if (font->maxAdvance < character->advance) font->maxAdvance = character->advance;
      }
      else if (strcmp(text, "BITMAP") == 0) {
        in_bitmap = 1;
        xpos = ypos = 0;
      }
      else if (strcmp(text, "ENDCHAR") == 0) {
        if (in_bitmap) {
          in_bitmap = 0;
/*
//140302
          if  (encoding < 0 || encoding > 255) {
            releaseMem(character);
            woempa(9, "WARNING: character with encoding '%i' not stored\n", encoding);
          }
          else {
            font->chars[(unsigned char)character->value] = character;
          }
*/
          if  (encoding < 0 || (encoding > 255 && encoding < 0x3000) || encoding > 0x30ff) {
            releaseMem(character);
            woempa(9, "WARNING: character with encoding '%i' not stored\n", encoding);
          }
          else {
            font->chars[(encoding > 0x2fff && encoding < 0x3100) ? encoding-0x2f00 : encoding] = character;
          }
        }
      }
      else if (in_bitmap) {
        xpos = 0;
        which_char = -1;  //140302 ADD required for optimisation; and recommended in any case
        hexval = 0;
        for (loop = 0; loop < character->width; loop++) {
/*
          // each character in text represents 4 horizontal bitmap positions
          which_char = loop / 4;

          // Resolve character:
          c = text[which_char];

          // Calculate hexval:
          if (('0' <= c) && (c <= '9')) {
             hexval = (int)(c - '0');
          }
          else {
            hexval = (int)(c - 'A') + 10;
          }
*/

//          /*
          //140302 each character in text represents 4 horizontal bitmap positions
          // the above could be optimised as follows:
          if (which_char < loop/4) {
            which_char = loop / 4;

            // Resolve character:
            c = text[which_char];

            // Calculate hexval:
            if (('0' <= c) && (c <= '9')) {
               hexval = (int)(c - '0');
            }
            else {
              hexval = (int)(c - 'A') + 10;
            }
          }

//          */

          which_bit = 3 - (loop % 4);
          if (hexval & (1 << which_bit)) {
            character->data[ypos + xpos] = 1;
          }
          else {
            character->data[ypos + xpos] = 0;
          }
          xpos++;
        }
        ypos += character->width;
      }

      line_no++;
    }

  /*
    // Add tab character:
    if (!font->chars['\t']  && font->chars[' ']) {
      character = (r_Char *)allocMem(thread, sizeof(r_Char));
      character->value    = '\t';
      character->data     = font->chars[' ']->data;
      character->width    = font->chars[' ']->width;
      character->height   = font->chars[' ']->height;
      character->advance  = font->chars[' ']->advance;
      character->xoffset  = font->chars[' ']->xoffset;
      character->yoffset  = font->chars[' ']->yoffset;
      font->chars['\t'] = character;
    }
  */


    /*
    ** Add a small rectangle as default character, with major characteristics equal to
    ** those of the character '0',
    ** and let all missing characters in the array font->chars point to this character
    */
    
    if (font->chars['0']) {
      // allocate the character and assign its major characteristics
      character = (r_Char *)allocMem(thread, sizeof(r_Char));
      character->value    = '\0';    // this value is of no importance; never used.
      character->width    = font->chars['0']->width;
      character->height   = font->chars['0']->height;
      character->advance  = font->chars['0']->advance;
      character->xoffset  = font->chars['0']->xoffset;
      character->yoffset  = font->chars['0']->yoffset;

      // allocate space for the character bitmap
      character->data = (char *)allocMem(thread, character->width * character->height * sizeof(char));

      bitmapsize = character->width * (character->height - 1);
      // draw upper and lower rectangle sides
      for (xpos = 0; xpos < character->width; xpos++) {
        character->data[xpos] = 1;
        character->data[bitmapsize + xpos] = 1;
      }
      // draw left and right rectangle sides
      for (ypos = character->width; ypos < bitmapsize; ypos += character->width) {
        character->data[ypos] = 1;
        for (xpos = 1; xpos < character->width - 1; xpos++) {
          character->data[ypos + xpos] = 0;
        }
        character->data[ypos + character->width - 1] = 1;
      }

      // let undefined characters in the font's character table point to this default character
//140302      for (loop = 0; loop < 256; loop++) {
      for (loop = 0; loop < 512; loop++) {
        if (!font->chars[loop]) {
          font->chars[loop] = character;
        }
      }
    } // end if

    // calculate the fonts' 'leading'
    leadAscent  = (font->maxAscent > font->ascent) ? font->maxAscent - font->ascent : 0;
    leadDescent = (font->maxDescent > font->descent) ? font->maxDescent - font->descent : 0;
    font->leading = leadAscent + leadDescent;

    // Add font to font fonts table if not yet in it:
    fontsInd=0;
    while (fontsInd < num_fonts && strcmp(fonts[fontsInd]->name, name) != 0) {
      fontsInd++;
    }
    if (fontsInd < num_fonts) {
      releaseMem(font);
      woempa(9, "attempt to load font '%s' a second time refused\n", name);
    }
    else {
      fonts[num_fonts++] = font;

      // Debug output:
      woempa(9, "sucessfully pre-processed font '%s' (%i chars on %i lines) [%i font(s) loaded]\n", name, char_no, line_no, num_fonts);
      // woempa(9, "size = %i; style = %i; ascent = %i; descent = % i; maxAscent = %i; maxDescent = %i; leading = %i; maxAdvance = %i\n", font->size, font->style, font->ascent, font->descent, font->maxAscent, font->maxDescent, font->leading, font->maxAdvance);
    }
  }

}

/*
** Auxilliary: Get the available font closest to a given font name, style,size
*/

r_font Font_getFont(char *name, int style, int size) {
  // Initialize the internal font using the default font:
  r_font font = (r_font)defaultFont[F_Font_wotsit];
  int i;

  woempa(9, "looking for font '%s, %i, %i' \n", name, style, size);

  for (i = 0; i < num_fonts; i++) {
    if (fonts[i]) {
      // Check to see whether we have an exact match:
      if ((fonts[i]->name) && (strcmp(fonts[i]->name, name)) == 0) {
        font = fonts[i];
        break;
      }

      // Check to see whether we have an exact match:
      if ((fonts[i]->family) && (strcmp(fonts[i]->family, name) == 0)) {
        if (fonts[i]->size == size) {
          if (fonts[i]->style == style) {
            font = fonts[i];
            break;
          }
        }
//        else {
//          if (abs((int)(fonts[i]->size - (unsigned int) size)) < abs((int)(font->size - size))) {
//            font = fonts[i];
//          }
//        }
      }

      /*
       * In case no exact match exists, at least try to select a font
       * with a size that matches the requested font size as close as
       * possible to ensure a correct look and feel:
       */
      if (abs((int)(fonts[i]->size - (unsigned int) size)) < abs((int)(font->size - size))) {
        font = fonts[i];
      }
    }
  }

  woempa(9, "found       font '%s, %i, %i' \n", font->name, font->style, font->size);
  return font;
}

/*
** Auxilliary: return the fontheight of a certain font
*/

w_int Font_getHeight(r_font font) {
  return (font ? font->leading + font->ascent + font->descent : 0);
}

/*
** Auxilliary: return the height, ascent and descent relative to the 
** baseline of a given string of characters written in a given font
** (this is NOT the general fonts height but the effective space above
** and below the current baseline that the string needs in order to be
** displayed completely with every character of the string occupying 
** its own space)
*/

//140302
/*
w_void Font_getTextHeight(r_font font, char* text, int len, int* ascent, int* descent) {

  char* ptr;
  int textind;
  r_char character = NULL;
  *ascent  = 0;
  *descent = 0;
  ptr = text;
  for (textind = 0; textind < len; textind++) {
    character = Font_getChar(font, (int)(*ptr));
    if (*ascent < character->height + character->yoffset) {
      *ascent = character->height + character->yoffset;
    }
    if (*descent < -character->yoffset) {
      *descent = -character->yoffset;
    }
    ptr++;
  }
}
*/

w_void Font_getTextHeight(r_font font, w_string text, int len, int* ascent, int* descent) {

  w_char* ptr;
  int textind;
  r_char character = NULL;
  *ascent  = 0;
  *descent = 0;
  ptr = text->chars;
  for (textind = 0; textind < len; textind++) {
    character = Font_getChar(font, (int)(*ptr));
    if (*ascent < character->height + character->yoffset) {
      *ascent = character->height + character->yoffset;
    }
    if (*descent < -character->yoffset) {
      *descent = -character->yoffset;
    }
    ptr++;
  }
}


//140302
/*
int Font_getCStringWidth(r_font font, char* text, int len) {

  int charx;
  char* ptr;
  int textind;

  charx = 0;
  ptr = text;
  for (textind = 0; textind < len; textind++) {
    charx += (Font_getChar(font, (int)(*ptr)))->advance;
    ptr++;
  }

  return charx;
}
*/

/*
** Auxilliary: return length of a given cstring of characters written in a given font
*/

int Font_getCStringWidth(r_font font, char* text, int len) {

  int charx;
  char* ptr;
  int textind;

  charx = 0;
  ptr = text;
  for (textind = 0; textind < len; textind++) {
    charx += (Font_getChar(font, (int)(*ptr)))->advance;
    ptr++;
  }

  return charx;
}

/*
** Auxilliary: return length of a given w_string of characters written in a given font
*/

int Font_getStringWidth(r_font font, w_string string) {

  int charx;
  w_char* ptr;
  int stringInd;
  int stringLen = string->length;

  charx = 0;
  ptr = string->chars;
  for (stringInd = 0; stringInd < stringLen; stringInd++) {
    charx += (Font_getChar(font, (int)(*ptr)))->advance;
    ptr++;
  }

  return charx;
}

/*
** Auxilliary: return length of a given (unicode) characters 
** written in a given font
*/

int Font_getCharWidth(r_font font, int charno) {

  int charx;
  charx = (Font_getChar(font, charno))->advance;
  return charx;
}

/*
** Auxilliary: return dimensions of a given string of characters
** written in a given font(the dimension is given by filling in
** the pointers to length and height, whereas the height is the
** fixed font height).
*/

//140302
/*
w_void Font_getDimensions(r_font font, char* text, int len, int* width_return, int* height_return) {

  int charx;
  char* ptr;
  int textind = 0;

  charx = 0;
  ptr = text;
  for (textind = 0; textind < len; textind++) {
    charx += (Font_getChar(font, (int)(*ptr)))->advance;
    ptr++;
  }

  *width_return = charx;
  *height_return = font->maxAscent + font->maxDescent;
}
*/
w_void Font_getDimensions(r_font font, w_string text, int len, int* width_return, int* height_return) {

  int charx;
  w_char* ptr;
  int textind = 0;

  charx = 0;
  ptr = text->chars;
  for (textind = 0; textind < len; textind++) {
    charx += (Font_getChar(font, (int)(*ptr)))->advance;
    ptr++;
  }

  *width_return = charx;
  *height_return = font->maxAscent + font->maxDescent;
}


/*
** Font non-java function: Draw the desired String in the desired 
** target rectangle either left aligned, right aligned or centered.
** If necessary, the string is clipped to fit into the rectangle
** => called by Graphics.c function
*/

w_void Font_drawStringAligned(r_buffer buffer, r_font font, int x, int y, int w, int h, w_string string, int c, unsigned int alignment) {

  /*
  ** x,y,w and h specify the bounding box for drawing the string, in the given buffer.
  ** coordinates of the bounding box are (x, y, x+w-1, y+h-1).
  ** calculate a baseline and starting x position for drawing inside the bounding box
  ** and call Font_drawCharString
  */

  int charx = 0;                 // x coordinate for starting drawing text in x direction
  int chary = 0;                 // baseline coordinate for starting drawing text
//140302
//  char* text = NULL;             // cstring copy of the given w_string
  int textlen = string->length;  // number of chars in the given string

  // Prune if the passed text is empty:
  if (textlen == 0) {
    return;
  }

//140302
//  text = allocMem(currentWonkaThread, (unsigned int)(textlen + 1));
//  w_string2c_string(string, text);

  switch(alignment) {
    case 0:
      // -- left alignment --
      charx = x;
      chary = y + (h + font->maxAscent - font->maxDescent) / 2;
      break;
    case 1:
      // -- center alignment --
      // Calculate coordinate offsets:
//140302
//      charx = x + (w - Font_getTextWidth(font, text, textlen)) / 2 ;
      charx = x + (w - Font_getStringWidth(font, string)) / 2 ;
      chary = y + (h + font->maxAscent - font->maxDescent) / 2;
      break;
    case 2:
      // -- right alignment --
      // Calculate coordinate offsets:
//140302
//      charx = x + w - Font_getTextWidth(font, text, textlen);
      charx = x + w - Font_getStringWidth(font, string);
      chary = y + (h + font->maxAscent - font->maxDescent) / 2;
      break;
    default:
      woempa(10, "WARNING: something whicked happened: unknown alignment tag\n");
  }

  /*
  ** draw the characters of the text string:
  */
//140302
//  Font_drawCharString(buffer, font, charx, chary, x, y, w, h, text, textlen, c);
//
//  releaseMem(text);
  Font_drawString(buffer, font, charx, chary, x, y, w, h, string, c);

}

/*
** Font non-java function: Draw the desired String in the desired
** target rectangle using the desired font. If necessary,the string
** is clipped to fit into the rectangle
** => called by Graphics.c function Graphics_drawString()
*/

w_void Font_drawStringUnAligned(r_buffer buffer, r_font font, int charx, int chary, int x, int y, int w, int h, w_string string, int c) {

  /*
  ** x,y,w and h specify the bounding box for drawing the string, in the given buffer.
  ** coordinates of the bounding box are (x, y, x+w-1, y+h-1).
  ** baseline and starting x coordinate for drawing are given as charx and chary. They can be located
  ** outside the bounding box. If the string to be drawn falls outside this box as a whole, the function
  ** returns immediatly.
  */

//140302
//  char* text = NULL;             // cstring copy of the given w_string
  int textlen = string->length;  // number of chars in the given string


  // Prune if the passed text is empty:
  if (textlen == 0) {
    return;
  }

//140302
//  text = allocMem(currentWonkaThread, (unsigned int)(textlen + 1));
//  w_string2c_string(string, text);

  // check overlap of string area and bounding box
//140302
//  if (charx + Font_getTextWidth(font, text, textlen) < x || chary + font->maxDescent < y || charx >= x + w || chary - font->maxAscent >= y + h) {
//    releaseMem(text);
  if (charx + Font_getStringWidth(font, string) < x || chary + font->maxDescent < y || charx >= x + w || chary - font->maxAscent >= y + h) {
    return;
  }

//140302
//  Font_drawCharString(buffer, font, charx, chary, x, y, w, h, text, textlen, c);
//
//  releaseMem(text);
  Font_drawString(buffer, font, charx, chary, x, y, w, h, string, c);
}

/*
** Font non-java function: draw the desired String in the desired
** target rectangle using the desired font. If necessary, the string
** is clipped to fit into the rectangle.
** => called as an auxilliary by the <Font_drawStringUnAligned> funtion 
**    above,
** => designed to be called by Graphics.c function <>
*/

//140302
//w_void Font_drawCharString(r_buffer buffer, r_font font, int charx, int chary, int x, int y, int w, int h, char* charstring, int textlen, int c) {

w_void Font_drawString(r_buffer buffer, r_font font, int charx, int chary, int x, int y, int w, int h, w_string string, int c) {

  // x,y,w and h specify the bounding box for drawing the string, in the given buffer.
  // charx and chary specify the baseline for text drawing.

  char* bitdata = NULL;
  int bitmapcol = 0;              // counts the columns in the bitmap
  int bitmaprow = 0;              // counts the rows in the bitmap
  int bitmapoffset = 0;           // counts the number of pixels in a number of rows.
  int height;                     // height in pixels of a particular character
  int width;                      // width in pixels  of a particular character
  int advance;                    // xoffset plus width in pixels of a particular character plus some right fill pixels
  int xoffset;                    // nbr of pixels in x-direction, relative to given x, to shift the drawing of a particular character
  int yoffset;                    // nbr of pixels in y-direction, relative to given y, to shift the drawing of a particular character
  int px;                         // pixel x-coordinate in drawing buffer
  int py;                         // pixel y-coordinate in drawinng buffer
  int stringInd = 0;              // character index in the textarray.
  int stringLen = string->length; // nbr of characters in string.
  r_char character = NULL;

  // Prune if the passed text is empty:
  if (stringLen == 0) {
    return;
  }

  /*
  ** loop through the text string and render each character individually:
  */
  
  for (stringInd = 0; stringInd < stringLen; stringInd++ ) {

    // get the font data about the current character
//140302
//    character = Font_getChar(font, (int)(charstring[textind]));
    character = Font_getChar(font, (int)(string->chars[stringInd]));
    height = character->height;
    width = character->width;
    advance = character->advance;
    xoffset = character->xoffset;
    yoffset = character->yoffset;
    bitdata = character->data;

    // loop through the bitmap of the character and draw those bitmap elements having value 1
    // use of 'bitmapoffset' avoids the multiplication 'bitmaprow * width'

    bitmapoffset = 0;
    for (bitmaprow = 0; bitmaprow < height; bitmaprow++) {
      // Calculate y-coordinate:
      py = chary - (height + yoffset) + bitmaprow;
      if (py >= y && py < y + h) {
        for (bitmapcol = 0; bitmapcol < width; bitmapcol++) {
          if (bitdata[bitmapoffset + bitmapcol]) {
            // Calculate x-coordinate:
            px = charx + xoffset + bitmapcol;
            if (px >= x && px < x + w) {
              // Draw pixel:
              drawPixel(buffer, px, py, c);
            }
          }
        }
      }
      bitmapoffset += width;
    }
    charx += advance;
  }

}
