/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/* 
** $Id: FontMetrics.c,v 1.5 2002/03/18 17:40:50 vandeneede Exp $ 
*/

#include <string.h>

#include "awt-classes.h" // F_Checkbox_*
#include "strings.h"     // cstring2String()
#include "ts-mem.h"      // allocMem(), releaseMem()
#include "realms.h"
#include "threads.h"      // currentWonkaThread

#include "rudolph.h"
#include "varia.h"

#include "Font.h"

/*
** resolve the ascent of a font, in pixels
*/
jint FontMetrics_getAscent(JNIEnv *env, jobject thisFontMetrics) {
  // Resolve font:
  w_instance fontInstance = (w_instance)thisFontMetrics[F_FontMetrics_font];
  r_font font = (r_font)fontInstance[F_Font_wotsit];

  // Debug output:
  // woempa(9, "called FontMetrics_getAscent(font = %s)\n", font->name);

  // Return value (if any):
  return (font ? font->ascent : 0);
}

/*
** resolve the descent of a font, in pixels
*/
jint FontMetrics_getDescent(JNIEnv *env, jobject thisFontMetrics) {
  // Resolve font:
  w_instance fontInstance = (w_instance)thisFontMetrics[F_FontMetrics_font];
  r_font font = (r_font)fontInstance[F_Font_wotsit];

  // Debug output:
  // woempa(9, "called FontMetrics_getDescent(font = %s)\n", font->name);

  // Return value (if any):
  return (font ? font->descent : 0);
}

/*
** resolve the leading of a font, in pixels
*/
jint FontMetrics_getLeading(JNIEnv *env, jobject thisFontMetrics) {
  // Resolve font:
  w_instance fontInstance = (w_instance)thisFontMetrics[F_FontMetrics_font];
  r_font font = (r_font)fontInstance[F_Font_wotsit];

  // Debug output:
  // woempa(9, "called FontMetrics_getLeading(font = %s)\n", font->name);

  // Return value (if any):
  return (font ? font->leading : 0);
}

/*
** resolve the height of a font, in pixels
*/
jint FontMetrics_getHeight(JNIEnv *env, jobject thisFontMetrics) {
  // Resolve font:
  w_instance fontInstance = (w_instance)thisFontMetrics[F_FontMetrics_font];
  r_font font = (r_font)fontInstance[F_Font_wotsit];

  // Debug output:
  // woempa(9, "called FontMetrics_getHeight(font = %s)\n", font->name);

  // Return value (if any):
  return Font_getHeight(font);
}

/*
** resolve the maximum ascent of the given font, in pixels
*/
jint FontMetrics_getMaxAscent(JNIEnv *env, jobject thisFontMetrics) {
  // Resolve font:
  w_instance fontInstance = (w_instance)thisFontMetrics[F_FontMetrics_font];
  r_font font = (r_font)fontInstance[F_Font_wotsit];

  // Debug output:
  // woempa(9, "called FontMetrics_getMaxAscent(font = %s) \n", font->name);

  /* Return value (if any): */
  return (font ? font->maxAscent : 0);
}

/*
** resolve the maximum descent of the given font, in pixels
*/
jint FontMetrics_getMaxDescent(JNIEnv *env, jobject thisFontMetrics) {
  // Resolve font:
  w_instance fontInstance = (w_instance)thisFontMetrics[F_FontMetrics_font];
  r_font font = (r_font)fontInstance[F_Font_wotsit];

  // Debug output:
  // woempa(9, "called FontMetrics_getMaxDescent(font = %s)\n", font->name);

  // Return value (if any):
  return (font ? font->maxDescent : 0);
}

/*
** resolve the maximum advance of the given font, in pixels
*/
jint FontMetrics_getMaxAdvance(JNIEnv *env, jobject thisFontMetrics) {
  // Resolve font:
  w_instance fontInstance = (w_instance)thisFontMetrics[F_FontMetrics_font];
  r_font font = (r_font)fontInstance[F_Font_wotsit];

  // Debug output:
  // woempa(9, "called FontMetrics_getMaxAdvance( font = %s)\n", font->name);

  // Return value (if any):
  return (font ? font->maxAdvance : 0);
}

/*
** resolve width in pixels of the given string
*/
jint FontMetrics_stringWidth(JNIEnv *env, jobject thisFontMetrics, jobject textInstance) {
  if (textInstance) {
    // Resolve font
    w_instance fontInstance = (w_instance)thisFontMetrics[F_FontMetrics_font];
    r_font font = (r_font)fontInstance[F_Font_wotsit];
    // Resolve string
    w_string string = (w_string)textInstance[F_String_wotsit];
//140302
/*
    char *text = allocMem(currentWonkaThread, (textString->length + 1) * sizeof(char));

    w_int width;

    // Convert JAVA/wonka string to C string:
    w_string2c_string(textString, text);

    // Resolve text width:
    width = Font_getTextWidth(font, text, (int)strlen(text));

    // Debug output:
    // woempa(9, "called FontMetrics_stringWidth(font = %s, string = '%s') \n", font->name, text);
  
    // Release used memory:
    releaseMem(text);
*/
    w_int width;

    // Resolve text width:
    width = Font_getStringWidth(font, string);

    // Debug output:
    // woempa(9, "called FontMetrics_stringWidth(font = %s, string = '%s') \n", font->name, text);


    return width;
  }
  else {
    return 0;
  }
}

/*
** resolve width in pixels of the given character
*/
jint FontMetrics_charWidth(JNIEnv *env, jobject thisFontMetrics, jint charno) {
  // Resolve font
  w_instance fontInstance = (w_instance)thisFontMetrics[F_FontMetrics_font];
  r_font font = (r_font)fontInstance[F_Font_wotsit];
  // get width
  w_int width;
  width = (Font_getChar(font, (int)charno))->advance;

  // Debug output:
  // woempa(9, "called FontMetrics_charWidth(font = %s, char = '%c') \n", font->name, thischar);

  return width;
}

