/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: TextField.c,v 1.6 2002/02/20 09:03:04 dfrederickx Exp $ 
*/

#include "strings.h"      // cstring2String() 
#include "ts-mem.h"       // allocMem(), releaseMem()
#include "realms.h"
#include "threads.h"	  // currentWonkaThread

#include "awt-classes.h"  // F_TextField_*
#include "canvas.h"
#include "platform.h"

#include "Component.h"
#include "Font.h"
#include "Graphics.h"     // r_drawFillRect
#include "TextField.h"

void TextField_createPeer(JNIEnv *env, jobject thisTextField) {
             
  r_component component = (r_component)thisTextField[F_Component_wotsit];
  component->tag = Z_TEXTFIELD;

}

void TextField_drawTextField(r_buffer buffer, r_component component) {
#ifdef OINK
  w_instance textfieldInstance = component->instance;
 
  /*
  ** Retrieve coordinates:
  */
  
  w_int x = (w_int)textfieldInstance[F_Component_x];
  w_int y = (w_int)textfieldInstance[F_Component_y];
  w_int h = (w_int)textfieldInstance[F_Component_height];
  w_int w = (w_int)textfieldInstance[F_Component_width];
            
  /*
  ** Retrieve textfield/text:
  */
  
  w_instance stringInstance = (w_instance)textfieldInstance[F_TextComponent_text];
  w_string string = (w_string)stringInstance[F_String_wotsit];
    
  r_font font;
      
  /*
  ** Retrieve font:
  */
  
  Component_getFont(component, &font);

  /*
  ** Debug output:
  */
  
  woempa(5, "called TextField_draw('%w'): x = %d, y = %d, w = %d, h = %d\n", string, x, y, w, h);

  /*
  ** Remap coordinates:
  */
  
  y = y + h / 2 - Font_getHeight(font); 
  h = Font_getHeight(font) + 10;
  w = w - 5;
 
  /*
  ** Validate the remapping: 
  ** (Mandatory!)
  */
  
  if (y >= 0 && w >= 0 && y + h < buffer->fh && x + w < buffer->fw) {
      
    /*
    ** Draw white surface:
    */
    
    r_drawFillRect(buffer, x, y, w, h, R_WHITE);

    /*
    ** Draw border:
    */
    
    drawHLine(buffer, x, y, w, (r_pixel)rgb2pixel(40, 40, 40));
    drawHLine(buffer, x, y + h - 1, w, (r_pixel)rgb2pixel(240, 240, 240));
    drawLine(buffer, x, y, x, y + h - 1, (r_pixel)rgb2pixel(40, 40, 40));
    drawLine(buffer, x + w - 1, y, x + w - 1, y + h - 1, (r_pixel)rgb2pixel(240, 240, 240));
		    
    /*
    ** Draw truetype string:
    */
    
    Font_drawStringAligned(buffer, font, x + 3, y, w - 6, h, string, (r_pixel)0, (unsigned int)ALIGNMENT_LEFT);
  }
#endif
}
