/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: Window.c,v 1.9 2002/02/20 09:03:04 dfrederickx Exp $ 
*/

#include "ts-mem.h"       // allocMem(), releaseMem()
#include "realms.h"
#include "strings.h"      // cstring2String()
#include "threads.h"       // currentWonkaThread
#include "loading.h"      // instance2clazz()
#include "hashtable.h"    // ht_deregister()

#include "awt-classes.h"  // F_Window_*
#include "varia.h"
#include "canvas.h"
#include "Component.h"
#include "registry.h"

void Window_toBack(JNIEnv * env, w_instance windowInstance) {
  r_component component = (r_component)windowInstance[F_Component_wotsit];

  woempa(9, "called Window_toBack(windowInstance = %p): component = %p\n", windowInstance, component);
  
  if (rootCanvas->component->headChild == component) {
    return;
  }
  
  rudolph_lock();

  if (component->prev) {
    component->prev->next = component->next;
  }
 
  if (component->next) {
    component->next->prev = component->prev;
  }

  /*
  ** Update rootCanvas' tailchild (if required):
  */

  if (rootCanvas->component->tailChild == component) {
    rootCanvas->component->tailChild = rootCanvas->component->tailChild->prev;
  }
  
  if (rootCanvas->component->headChild) {
    rootCanvas->component->headChild->prev = component;
    component->next = rootCanvas->component->headChild;
    component->prev = NULL;
    rootCanvas->component->headChild = component;
  }
 
  /*
  ** Redraw the components with a lower z-order:
  */
  
  canvas_drawCanvas((r_canvas)rootCanvas->component->headChild->object, 0);
 
  rudolph_unlock(); 
}

void Window_toFront(JNIEnv * env, w_instance windowInstance) {
  r_component component = (r_component)windowInstance[F_Component_wotsit];

  woempa(9, "called Window_toFront(windowInstance = %p): component = %p\n", windowInstance, component);
  
  if (rootCanvas->component->tailChild == component) {
    return;
  }
  
  rudolph_lock();

  if (component->prev) {
    component->prev->next = component->next;
  }
 
  if (component->next) {
    component->next->prev = component->prev;
  }
  
  /*
  ** Update rootCanvas' headchild (if required):
  */

  if (rootCanvas->component->headChild == component) {
    rootCanvas->component->headChild = rootCanvas->component->headChild->next;
  }
  
  if (rootCanvas->component->tailChild) {
    rootCanvas->component->tailChild->next = component;
    component->prev = rootCanvas->component->tailChild;
    component->next = NULL;
    rootCanvas->component->tailChild = component;
  }
  
  /*
  ** Redraw the components with a higher z-order:
  */

  if (component->next) {
    canvas_canvas2screen((r_canvas)component->next->object);
  }
 
  rudolph_unlock();  
}

void Window_disposePeer(JNIEnv *env, jobject windowInstance) {
  
  r_component component = (r_component)windowInstance[F_Component_wotsit];
  
  woempa(9, "called Window_disposePeer(windowInstance = %p): component = %p\n", windowInstance, component);

  /*
  ** Remove the component from the registry or component tree:
  */
  
  registry_delComponent(component);

  
  /*
  ** Redraw the components with a lower z-order:
  */
  
  if (rootCanvas->component && rootCanvas->component->headChild) {
    canvas_drawCanvas((r_canvas)rootCanvas->component->headChild->object, 0); 
  }
  
  /*
  ** Remove the container from the global reference table such
  ** that it can be garbage collected.
  */

  if (ht_findkey(globals_hashtable, (w_word)windowInstance)) {
    ht_deregister(globals_hashtable,(w_word)windowInstance);
  }
}

void Window_relocatePeer(JNIEnv *env, jobject windowInstance) {
  
  woempa(9, "called Window_relocatePeer(windowInstance = %p): component = %p\n", windowInstance, (r_component)windowInstance[F_Component_wotsit]);
  
  /*
  ** Redraw the components with a lower z-order:
  */
  
  if (rootCanvas->component && rootCanvas->component->headChild) {
    canvas_drawCanvas((r_canvas)rootCanvas->component->headChild->object, 0); 
  }  
}

