/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/* 
** $Id: registry.c,v 1.16 2002/03/20 13:04:20 gray Exp $ 
*/

#include <string.h>

#include "ts-mem.h"
#include "realms.h"
#include "strings.h"
#include "threads.h"
#include "oswald.h"
#include "loading.h"
#include "awt-classes.h"
#include "dispatch.h"
#include "registry.h"
#include "platform.h"
#include "canvas.h"
#include "Component.h"

/*
** The registry is responsible for maintaining the internal component
** tree.  We should see how we can "merge" the component tree that is
** kept at the Java level (see java.awt.Component and Container) with
** this one in C.  [TODO]
*/

inline char *registry_spaces(w_int depth) {
  const char *spaces = "                                        ";  // 40 spaces
  return (char *)(spaces + strlen(spaces) - (2 * depth));
}
    
w_void registry_dump(r_canvas canvas, w_int depth) {

  r_component component;
  r_canvas c;

  if (canvas && canvas->component) {

    /*
    ** Mark the beginning of the component tree:
    */
    
    if (canvas == rootCanvas) { 
      woempa(9, "---------------- component tree -----------------\n");
    }

    /*
    ** Dump the component tree:
    */

    for (component = canvas->component->headChild; component != NULL; component = component->next) {
      switch(component->tag) {
        case Z_CONTAINER:
          c = (r_canvas)component->object;
          woempa(9, "%scontainer '%k' of size %dx%d (%d bytes), refresh %d (%s)\n", registry_spaces(depth), instance2clazz((w_instance)component->instance), c->buffer->fw, c->buffer->fh, (c->buffer->status ? c->buffer->fw * c->buffer->fh * sizeof(r_pixel) : 0), component->refresh, component->instance[F_Component_visible] ? "V" : "I");
          registry_dump(c, depth + 1);
          break;
        case Z_BUTTON:
          woempa(9, "%sbutton '%w' (%s)\n", registry_spaces(depth), String2string((w_instance)component->instance[F_Button_label]), component->instance[F_Component_visible] ? "V" : "I");
          break;
        case Z_CHECKBOX:
          woempa(9, "%scheckbox '%w' (%s)\n", registry_spaces(depth), String2string((w_instance)component->instance[F_Checkbox_label]), component->instance[F_Component_visible] ? "V" : "I");
          break;
        case Z_LABEL:
          woempa(9, "%slabel '%w' (%s)\n", registry_spaces(depth), String2string((w_instance)component->instance[F_Label_text]), component->instance[F_Component_visible] ? "V" : "I");
          break;
        case Z_TEXTFIELD:
          woempa(9, "%stextfield '%w' (%s)\n", registry_spaces(depth), String2string((w_instance)component->instance[F_TextComponent_text]), component->instance[F_Component_visible] ? "V" : "I");
	  break;
	default:
          woempa(9, "%sregular component '%k' with bounds(%d, %d, %d, %d) (%s)\n", registry_spaces(depth), instance2clazz(component->instance), component->instance[F_Component_x], component->instance[F_Component_y], component->instance[F_Component_width], component->instance[F_Component_height], component->instance[F_Component_visible] ? "V" : "I");
      }
    }
    
    /*
    ** Mark the end of the component tree:
    */
    
    if (canvas == rootCanvas) {
      woempa(9, "-------------------------------------------------\n");
    }
  }
}

r_canvas registry_constructor(int w, int h) {
  r_canvas canvas;
  r_buffer buffer;
  
  /*
  ** Allocate memory for off-screen buffer:
  */
  
  buffer = allocMem_system(sizeof(r_Buffer) + (pixels2bytes(w * h)));

  /*
  ** Initialize buffer:
  */
  
  buffer->fw = w;
  buffer->fh = h;
  buffer->vw = w;
  buffer->vh = h;
  buffer->ox = 0;
  buffer->oy = 0;
  
  /*
  ** Initialize canvas:
  */
  
  canvas = (r_canvas)allocMem_system(sizeof(r_Canvas)); 
  canvas->buffer = buffer;
    
  /*
  ** Link in component tree:
  */
  canvas->component = allocClearedMem_system(sizeof(r_Component));
  canvas->component->tag = Z_CONTAINER;
  canvas->component->object = (unsigned long) canvas;
  canvas->component->refresh = 2;
  canvas->component->visible = -1;

  canvas->buffer = buffer;  

  return canvas;
}

w_void registry_addContainer(r_canvas parentCanvas, r_canvas panel) {  

  woempa(5, "Add Container : %p (child: %p)\n", panel->component, panel->component->headChild);

  /*
  ** Link component in component tree:
  */
  
  panel->component->tag = Z_CONTAINER;
  panel->component->refresh = 2;
  panel->component->object = (unsigned long) panel;
  panel->component->parent = parentCanvas->component;
  panel->component->next = NULL;

  if (parentCanvas->component->headChild == NULL) {
    panel->component->prev = NULL;
    parentCanvas->component->headChild = panel->component;
    parentCanvas->component->tailChild = panel->component;
  }
  else {
    parentCanvas->component->tailChild->next = panel->component;
    panel->component->prev = parentCanvas->component->tailChild;
    parentCanvas->component->tailChild = panel->component;
  }

}

w_void registry_addComponent(r_canvas canvas, r_component component) {

  woempa(5, "Add Component : %p\n", component);

  /*
  ** Link component in component tree:
  */
  
  component->parent = canvas->component;
  component->headChild = NULL;
  component->tailChild = NULL;
  component->next = NULL;

  if (canvas->component->headChild == NULL) {
    component->prev = NULL;
    canvas->component->headChild = component;
    canvas->component->tailChild = component;
  }
  else {
    // insert to tail of list:
    component->prev = canvas->component->tailChild;
    canvas->component->tailChild->next = component;
    canvas->component->tailChild = component;
  }

}

w_void registry_delComponent(r_component component) {

  /*
  ** Update the parent's headchild (if required):
  */

  if (component->parent && component->parent->headChild == component) {
    component->parent->headChild = component->next;
  }

  /*
  ** Update parent's tailchild (if required):
  */
  
  if (component->parent && component->parent->tailChild == component) {
    component->parent->tailChild = component->prev;
  }
  
  /*
  ** Update the next and previous (if required):
  */
  
  if (component->prev) {
    component->prev->next = component->next;
  }

  if (component->next) {
    component->next->prev = component->prev;
  }

  component->parent = NULL;
  component->next = NULL;
  component->prev = NULL;

}
