#ifndef _J_ARM_IS_H
#define _J_ARM_IS_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $id$ 
*/

/* 
** Banked registers:
*/

#define R0      0               // argument 1 + return
#define R1      1               // argument 2
#define R2      2               // argument 3
#define R3      3               // argument 4
#define R4      4
#define R5      5
#define R6      6
#define R7      7

/*
** Unbanked registers:
*/

#define R8      8
#define R9      9
#define R10     10
#define R11     11              // frame pointer (FP)
#define R12     12              // instruction pointer (IR)
#define R13     13              // stack pointer (SP)
#define R14     14              // link register (LR)
#define R15     15              // program counter (PC)

/*
** Register aliases:
*/

#define FP      R11
#define IP      R12
#define SP      R13
#define LR      R14
#define PC      R15

/*
** Condition codes of 4-bit condition field (see A3-5, A3-6):
*/

#define CC_EQ   0x0      // equal
#define CC_NE   0x1      // not equal
#define CC_CS   0x2      // carry set/unsigned higher or same
#define CC_CC   0x3      // carry clear/unsigned lower
#define CC_MI   0x4      // minus/negative
#define CC_PL   0x5      // plus/positive or zero
#define CC_VS   0x6      // overflow
#define CC_VC   0x7      // no overflow
#define CC_HI   0x8      // unsigned higher
#define CC_LS   0x9      // unsigned lower or same
#define CC_GE   0xa      // signed greater than or equal
#define CC_LT   0xb      // signed less than
#define CC_GT   0xc      // signed greater than
#define CC_LE   0xd      // signed less than or equal
#define CC_AL   0xe      // always (unconditional)
#define CC_NV   0xf      // never

/*
** Bits for stack operations (see A7-78):
*/

#define  BIT_R0  (1 << R0)
#define  BIT_R1  (1 << R1)
#define  BIT_R2  (1 << R2)
#define  BIT_R3  (1 << R3)
#define  BIT_R4  (1 << R4)
#define  BIT_R5  (1 << R5)
#define  BIT_R6  (1 << R6)
#define  BIT_R7  (1 << R7)
#define  BIT_R8  (1 << R8)
#define  BIT_R9  (1 << R9)
#define  BIT_R10  (1 << R10)
#define  BIT_R11  (1 << R11)
#define  BIT_R12  (1 << R12)
#define  BIT_R13  (1 << R13)
#define  BIT_R14  (1 << R14)
#define  BIT_R15  (1 << R15)

/*
** Bit aliases (see A7-78):
*/

#define  BIT_FP  BIT_R11
#define  BIT_IP  BIT_R12
#define  BIT_SP  BIT_R13
#define  BIT_LR  BIT_R14
#define  BIT_PC  BIT_R15

/*
** Data processing instructions:
*/

#define INSN_AND   0x0   // logical AND
#define INSN_EOR   0x1   // logical exclusive OR
#define INSN_SUB   0x2   // subtract
#define INSN_RSB   0x3   // reverse subtract
#define INSN_ADD   0x4   // add
#define INSN_ADC   0x5   // add with carry
#define INSN_SBC   0x6   // subtract with carry
#define INSN_RSC   0x7   // reverses subtract with carry
#define INSN_TST   0x8   // test
#define INSN_TEQ   0x9   // test equivalence
#define INSN_CMP   0xa   // compare
#define INSN_CMN   0xb   // compared negated
#define INSN_ORR   0xc   // logical inclusive OR
#define INSN_MOV   0xd   // move
#define INSN_BIC   0xe   // bit clear
#define INSN_MVN   0xf   // move not


/*
** Binary instruction encoding:
*/

#define arm_mov_rr(rd, rs) \
  1, 32, 4, CC_AL, 2, 0x0, 1, 0x0, 4, INSN_MOV, 1, 0x0, 4, 0x0, 4, rd, 12, rs
  
#define arm_mov_ri(rd, im) \
  1, 32, 4, CC_AL, 2, 0x0, 1, 0x1, 4, INSN_MOV, 1, 0x0, 4, 0x0, 4, rd, 12, im

#define arm_add_rr(rd, rn, rm) \
  1, 32, 4, CC_AL, 2, 0x0, 1, 0x0, 4, INSN_ADD, 1, 0x0, 4, rn, 4, rd, 12, rm

#define arm_add_ri(rd, rn, im) \
  1, 32, 4, CC_AL, 2, 0x0, 1, 0x1, 4, INSN_ADD, 1, 0x0, 4, rn, 4, rd, 12, im

#define arm_sub_rr(rd, rn, rm) \
  1, 32, 4, CC_AL, 2, 0x0, 1, 0x0, 4, INSN_SUB, 1, 0x0, 4, rn, 4, rd, 12, rm

#define arm_sub_ri(rd, rn, im) \
  1, 32, 4, CC_AL, 2, 0x0, 1, 0x1, 4, INSN_SUB, 1, 0x0, 4, rn, 4, rd, 12, im

#define arm_stmdb(base, registers) \
  1, 32, 4, CC_AL, 3, 0x4, 1, 0x1, 1, 0x0, 3, 0x2, 4, base, 16, registers
  
#define arm_ldmdb(base, registers) \
  1, 32, 4, CC_AL, 3, 0x4, 1, 0x1, 1, 0x0, 3, 0x1, 4, base, 16, registers

#define arm_str_i(rd, rn, of) \
  1, 32, 4, CC_AL, 2, 0x1, 1, 0x0, 1, 0x1, 1, (of < 0 ? 0x0 : 0x1), 1, 0x0, 1, 0x0, 1, 0x0, 4, rn, 4, rd, 12, abs(of)

#define arm_str_r(rd, rn) \
  1, 32, 4, CC_AL, 2, 0x1, 1, 0x0, 1, 0x1, 1, 0x1, 1, 0x0, 1, 0x0, 1, 0x0, 4, rn, 4, rd, 12, 0x0

#define arm_bl(pc, address) \
  1, 32, 4, CC_AL, 3, 0x4, 1, 0x1, 24, (address - pc - 8) >> 2)
  
#endif