#ifndef _J_X86_IS_H
#define _J_X86_IS_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $id$ 
*/

inline static j_word endian_32(j_word w) {
  return (w >> 24) | ((w >> 8) & 0x0000ff00) | ((w << 8) & 0x00ff0000) | (w << 24);
}

/*
** Register field:
*/

#define AL     0x0
#define CL     0x1
#define DL     0x2
#define BL     0x3
#define AH     0x4
#define CH     0x5
#define DH     0x6
#define BH     0x7

#define AX     0x0
#define CX     0x1
#define DX     0x2
#define BX     0x3
//#define SP     0x4
#define BP     0x5
#define SI     0x6
#define DI     0x7

#define EAX    0x0
#define ECX    0x1
#define EDX    0x2
#define EBX    0x3
#define ESP    0x4
#define EBP    0x5
#define ESI    0x6
#define EDI    0x7

/*
** Bits
*/

#define BIT_EAX   (1 << EAX)
#define BIT_ECX   (1 << ECX)
#define BIT_EDX   (1 << EDX)
#define BIT_EBX   (1 << EBX)
#define BIT_ESP   (1 << ESP)
#define BIT_EBP   (1 << EBP)
#define BIT_ESI   (1 << ESI)
#define BIT_EDI   (1 << EDI)

/*
** Condition codes:
*/

#define COND_EQ    0x4
#define COND_NE    0x5
#define COND_LT    0xc
#define COND_GE    0xd
#define COND_LE    0xe
#define COND_GT    0xf

/*
** Binary instruction encoding:
**
**    rr = register to register
**    mr = memory to register
**    ir = immediate to register
**    im = immediate to memory
*/

#define x86_add_rr(rd, rs) \
  0, 16, 8, 0x01, 2, 0x03, 3, rd, 3, rs

#define x86_add_rm(rd, offset, rs) \
  todo
  
#define x86_add_ri(rd, ad) \
  0, 24, 8, 0x83, 5, 0x18, 3, rd, 8, (char)ad

#define x86_add_im(rd, offset, rs) \
  todo 

#define x86_sub_rr(rd, rs) \
  0, 16, 8, 0x29, 2, 0x03, 3, rd, 3, rs

#define x86_sub_rm(rd, offset, rs) \
  todo
  
#define x86_sub_ri(rd, ad) \
  0, 24, 8, 0x83, 5, 0x1d, 3, rd, 8, (char)ad

#define x86_sub_im(rd, offset, rs) \
  todo

#define x86_mul_rr(rd, rs) \
  0, 24, 8, 0x0f, 8, 0xaf, 2, 0x03, 3, rs, 3, rd

#define x86_neg_rr(rd) \
  0, 16, 8, 0xf7, 8, 0xd8 | rd

#define x86_mov_rr(rd, rs) \
  0, 16, 8, 0x89, 2, 0x3, 3, rs, 3, rd

#define x86_mov_ri(rd, ad) \
  0, 40, 8, 0xb8 | rd, 32, endian_32(ad)

#define x86_mov_rm(rd, rs, offset) \
  0, 24, 8, 0x8b, 2, 0x1, 3, rd, 3, rs, 8, offset

#define x86_mov_mr(rd, offset, rs) \
  0, 24, 8, 0x89, 2, 0x1, 3, rs, 3, rd, 8, offset
 
#define x86_lea(rd, offset, rs) \
  0, 24, 8, 0x8d, 2, 0x1, 3, rd, 3, rs, 8, offset
    
#define x86_push_r(rd) \
  0, 8, 8, 0x50 | rd

#define x86_push_i(imm) \
  0, 40, 8, 0x68, 32, endian_32(imm)
  
#define x86_pop(rd) \
  0, 8, 8, 0x58 | rd

#define x86_call(address) \
  0, 16, 8, 0xff, 8, 0xd0 | address

#define x86_gt_8(offset) \
  0, 16, 4, 0x7, 4, COND_GT, 8, offset 

#define x86_ge_8(offset) \
  0, 16, 4, 0x7, 4, COND_GE, 8, offset
  
#define x86_lt_8(offset) \
  0, 16, 4, 0x7, 4, COND_LT, 8, offset
  
#define x86_le_8(offset) \
  0, 16, 4, 0x7, 4, COND_LE, 8, offset
  
#define x86_eq_8(offset) \
  0, 16, 4, 0x7, 4, COND_EQ, 8, offset
  
#define x86_ne_8(offset) \
  0, 16, 4, 0x7, 4, COND_NE, 8, offset

#define x86_gt_32(offset) \
  0, 48, 8, 0x0f, 4, 0x8, 4, COND_GT, 32, endian_32(offset) 

#define x86_ge_32(offset) \
  0, 48, 8, 0x0f, 4, 0x8, 4, COND_GE, 32, endian_32(offset)

#define x86_lt_32(offset) \
  0, 48, 8, 0x0f, 4, 0x8, 4, COND_LT, 32, endian_32(offset)

#define x86_le_32(offset) \
  0, 48, 8, 0x0f, 4, 0x8, 4, COND_LE, 32, endian_32(offset)

#define x86_eq_32(offset) \
  0, 48, 8, 0x0f, 4, 0x8, 4, COND_EQ, 32, endian_32(offset)

#define x86_ne_32(offset) \
  0, 48, 8, 0x0f, 4, 0x8, 4, COND_NE, 32, endian_32(offset)

#define x86_jmp_8(offset) \
  0, 16, 8, 0xeb, 8, offset

#define x86_jmp_32(offset) \
  0, 40, 8, 0xe9, 32, endian_32(offset)

#define x86_jmp_r(reg) \
  0, 16, 8, 0xff, 5, 0x1c, 3, reg

#define x86_cmp_rr(rd, rs) \
  0, 16, 8, 0x39, 2, 0x3, 3, rd, 3, rs

#define x86_cmp_rm(rd, offset, rs) \
  0, 24, 8, 0x3b, 2, 0x1, 3, rd, 3, rs, 8, offset

#define x86_cmp_mr(offset, rd, rs) \
  0, 24, 8, 0x39, 2, 0x1, 3, rs, 3, rd, 8, offset

#define x86_cmp_ir(reg, imm) \
  0, 48, 8, 0x3b, 5, 0x1f, 3, reg, 32, endian_32(imm)

#define x86_cmp_im() \
  todo

#define x86_and_rr(rd, rs) \
  0, 16, 8, 0x21, 2, 0x3, 3, rd, 3, rs

#define x86_or_rr(rd, rs) \
  0, 16, 8, 0x09, 2, 0x3, 3, rd, 3, rs
  
#define x86_xor_rr(rd, rs) \
  0, 16, 8, 0x31, 2, 0x3, 3, rd, 3, rs

#define x86_shr_cl(rd) \
  0, 16, 8, 0xd3, 5, 0x1d, 3, rd  

#define x86_shl_cl(rd) \
  0, 16, 8, 0xd3, 5, 0x1c, 3, rd  

#endif
