/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: code-generator.c,v 1.20 2002/03/14 15:02:58 dbuytaert Exp $
*/

#include "oswald.h"  // memory allocation

#include "methods.h"
#include "j-spot.h"
#include "ir.h"
#include "register-allocator.h"
#include "code-generator.h"
#include "assembler.h"
#include "is.h"

void cg_prologue(j_method method) {

  woempa(9, "called prologue\n");

  /*
  ** Save the old base pointer:
  */
  
  asm_emit(method->code, x86_push_r(EBP));           // push %ebp

  /*
  ** Set base pointer as frame pointer:
  */
  
  asm_emit(method->code, 0, 16, 8, 0x89, 8, 0xe5);   // mov %esp, %ebp
  
  /*
  ** Reserve stack space for the local variables:
  */

  asm_emit(method->code, x86_sub_ri(ESP, method->stack * 4));
   
  /*
  ** Push temporary, callee-saved registers onto the stack so we can restore them later on.
  */

  if (method->usage & (1 << EBX)) {
    asm_emit(method->code, x86_push_r(EBX));
  }
  
  if (method->usage & (1 << ESI)) {
    asm_emit(method->code, x86_push_r(ESI));
  }
  
  if (method->usage & (1 << EDI)) {
    asm_emit(method->code, x86_push_r(EDI));
  }
}

void cg_epilogue(j_method method) {

  woempa(9, "called epilogue\n");

  /*
  ** Restore the temporary, callee-saved registers by popping them from the stack.
  */
  
  if (method->usage & (1 << EBX)) {
    asm_emit(method->code, x86_mov_rm(EBX, EBP, - (method->stack + 1) * 4));  
  }

  if (method->usage & (1 << ESI)) {
    asm_emit(method->code, x86_mov_rm(ESI, EBP, - (method->stack + 2) * 4));  
  }

  if (method->usage & (1 << EDI)) {
    asm_emit(method->code, x86_mov_rm(EDI, EBP, - (method->stack + 3) * 4));  
  }

  /*
  ** "leave": prepare the stack for returning:
  */
  
  asm_emit(method->code, 0, 8, 8, 0xc9);

  /*
  ** "ret": pop the return address off the stack and jump to this location:
  */
  
  asm_emit(method->code, 0, 8, 8, 0xc3);
}

void cg_node2register(j_method method, j_node node, int r) {

  // NOTE: the order of the if-clauses below is important.

  if (node) {
  
    /*
    ** A literal with a constant value.
    */

    if (node->operator == j_literal && node->data.literal.location == j_constant) {
      asm_emit(method->code, x86_mov_ri(r, (j_word)node->data.literal.value));
    } 

    /*
    ** A variable that currently resides in a register.
    */
      
    else if (method->registers[node->reg] == node) {
      if (node->reg != r) {
        asm_emit(method->code, x86_mov_rr(r, node->reg));
      }
    }

    /*
    ** An argument on the function stack.
    */
  
    else if (node->operator == j_literal && node->data.literal.location == j_parameter) {
      if (method->method->flags & ACC_STATIC) {
        asm_emit(method->code, x86_mov_rm(r, EBP, 4 * node->data.literal.value + 16));
          // + 8: skip the "meta information", see "wonka/hal/cpu/x86/scr/calls.s".
      }
      else {
        asm_emit(method->code, x86_mov_rm(r, EBP, 4 * node->data.literal.value + 12));
          // + 12: skip the "meta information", see "wonka/hal/cpu/x86/scr/calls.s".
      }
    }
  
    /*
    ** A temporary on the function stack.
    */
  
    else if (node->operator == j_literal && node->data.literal.location == j_temporary) {
      asm_emit(method->code, x86_mov_rm(r, EBP, - 4 * node->stack));
    }
  
    /*
    ** A variable that is spilled on the function stack.
    */
  
    else if (node->spill == 1) {
      asm_emit(method->code, x86_mov_rm(r, EBP, -4 * node->stack));
    }
  
    /*
    ** A variable that currently resides in a register.
    */
      
    else if (node->reg != r) {
      asm_emit(method->code, x86_mov_rr(r, node->reg));
    }
 
    /*
    ** Default.
    */
  
    else {
      woempa(9, "warning: unknown argument encountered: %q [register = %d, spill = %d, stack = %d]\n", node, node->reg, node->spill, node->stack);
      method->failed = 1;
    }
  }
}



void cg_instruction(j_method method, j_node node) {

  /*
  ** Prepare the registers.
  */

  ra_instruction(method, node, 1);
  
  /*
  ** Emit operator specific machine code.
  */
  
  switch (node->operator) {

    case j_assign:
      cg_node2register(method, node->node2, node->node1->reg);
      break;
      
    case j_neg:
      asm_emit(method->code, x86_neg_rr(node->reg));
      break;
      
    case j_sub:
      asm_emit(method->code, x86_sub_rr(node->node2->reg, node->reg));
      break;
      
    case j_add:
      asm_emit(method->code, x86_add_rr(node->node2->reg, node->reg));
      break;

    case j_mul:
      asm_emit(method->code, x86_mul_rr(node->node2->reg, node->reg));
      break;
    
    case j_and:
      asm_emit(method->code, x86_and_rr(node->node2->reg, node->reg));
      break;

    case j_or:
      asm_emit(method->code, x86_or_rr(node->node2->reg, node->reg));
      break;
      
    case j_xor:
      asm_emit(method->code, x86_xor_rr(node->node2->reg, node->reg));
      break;

    case j_call:
      /*
      ** Do nothing - the register allocator will take care of this.
      */
      
      break;

    case j_literal:
      /*
      ** Do nothing - we encountered a literal, not an instruction.  Every
      ** instruction is held responsible for "preparing" its own operands.
      ** This is required because (a) some x86 instructions can keep one or
      ** more operands in memory and (b) to get the addresses of the join 
      ** points (forward and backward references) right.
      */
      
      break;
     
    case j_jump:
      asm_emit(method->code, x86_jmp_32(0));
      asm_add_reference(method->code, method->code->step, ir_get_branch(method, node));
      break;
      
    case j_lt:
      asm_emit(method->code, x86_cmp_rr(node->node2->reg, node->node1->reg));
      asm_emit(method->code, x86_lt_32(0));
      asm_add_reference(method->code, method->code->step, ir_get_branch(method, node));
      break;

    case j_gt:
      asm_emit(method->code, x86_cmp_rr(node->node2->reg, node->node1->reg));
      asm_emit(method->code, x86_gt_32(0));
      asm_add_reference(method->code, method->code->step, ir_get_branch(method, node));
      break;
      
    case j_le:
      asm_emit(method->code, x86_cmp_rr(node->node2->reg, node->node1->reg));
      asm_emit(method->code, x86_le_32(0));
      asm_add_reference(method->code, method->code->step, ir_get_branch(method, node));
      break;
      
    case j_ge:
      asm_emit(method->code, x86_cmp_rr(node->node2->reg, node->node1->reg));
      asm_emit(method->code, x86_ge_32(0));
      asm_add_reference(method->code, method->code->step, ir_get_branch(method, node));
      break;

    case j_eq:
      asm_emit(method->code, x86_cmp_rr(node->node2->reg, node->node1->reg));
      asm_emit(method->code, x86_eq_32(0));
      asm_add_reference(method->code, method->code->step, ir_get_branch(method, node));
      break;

    case j_ne:
      asm_emit(method->code, x86_cmp_rr(node->node2->reg, node->node1->reg));
      asm_emit(method->code, x86_ne_32(0));
      asm_add_reference(method->code, method->code->step, ir_get_branch(method, node));
      break;
                
    case j_return:
      /*
      ** On x86, %eax is used to hold the result, or %edx:%eax for 64-bit results.
      */
      
      cg_node2register(method, node->node1, EAX);
      cg_epilogue(method);
      break;

    default:
      woempa(9, "warning: '%s' not supported by backend\n", node_get_operator(node));
      method->failed = 1;
  }
}

// Prototype of open source disassembler in hal/cpu/x86/src/disassemble.c:
unsigned long db_disasm(unsigned long, int, int);

j_word *cg_disassemble(j_word * address) {
  return (j_word *)db_disasm((unsigned long)address, 0, 0);
}

