/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: register-allocator.c,v 1.19 2002/03/26 09:18:31 dbuytaert Exp $
*/

#include "oswald.h"  // memory allocation
#include "methods.h"

#include "j-spot.h"
#include "ir.h"
#include "register-allocator.h"
#include "code-generator.h"
#include "assembler.h"
#include "is.h"

#define RA_REGISTER          0x7F
#define RA_MEMORY            (1 << 31)
#define RA_STACK             (1 << 30)

#define RA_INHERIT           (1 << 1)

static j_Map map[] = {
  { j_none,    0, 0, 0, 0 },
  { j_literal, RA_REGISTER | RA_MEMORY | RA_STACK, 0, 0, 0 },
  { j_add,     RA_REGISTER, RA_REGISTER, RA_REGISTER | RA_STACK, RA_INHERIT },
  { j_sub,     RA_REGISTER, RA_REGISTER, RA_REGISTER | RA_STACK, RA_INHERIT },
  { j_mul,     RA_REGISTER, RA_REGISTER, RA_REGISTER | RA_STACK, RA_INHERIT },
  { j_div,     RA_REGISTER, RA_REGISTER, RA_REGISTER | RA_STACK, RA_INHERIT },
  { j_rem,     RA_REGISTER, RA_REGISTER, RA_REGISTER | RA_STACK, RA_INHERIT },
  { j_neg,     RA_REGISTER, 0, 0, RA_INHERIT },
  { j_lsh,     RA_REGISTER, 0, 0, RA_INHERIT },
  { j_rsh,     RA_REGISTER, 0, 0, RA_INHERIT },
  { j_and,     RA_REGISTER, 0, 0, RA_INHERIT },
  { j_or,      RA_REGISTER, 0, 0, RA_INHERIT },
  { j_xor,     RA_REGISTER, 0, 0, RA_INHERIT },
  { j_le,      RA_REGISTER, RA_REGISTER, RA_REGISTER, 0, },
  { j_gt,      RA_REGISTER, RA_REGISTER, RA_REGISTER, 0, },
  { j_eq,      RA_REGISTER, RA_REGISTER, RA_REGISTER, 0, },
  { j_ne,      RA_REGISTER, RA_REGISTER, RA_REGISTER, 0, },
  { j_lt,      RA_REGISTER, RA_REGISTER, RA_REGISTER, 0, },
  { j_ge,      RA_REGISTER, RA_REGISTER, RA_REGISTER, 0, },
  { j_call,    RA_REGISTER, 0, 0, 0 },
  { j_assign,  RA_REGISTER, 0, 0, RA_INHERIT },
  { j_return,  RA_REGISTER, 0, 0, RA_INHERIT },
  { j_jump,    RA_REGISTER, 0, 0, 0 },
  { j_conv,    RA_REGISTER, 0, 0, 0 }
};

void ra_spill(j_method method, j_node node, int cg) {
  
  if (method->registers[node->reg] == node) {

    /*
    ** Emit spill code:
    */
    
    if (cg) { 
      if (node->operator == j_literal && node->data.literal.location == j_parameter) {
        if (method->method->flags & ACC_STATIC) {
          asm_emit(method->code, x86_mov_rm(node->reg, EBP, 4 * node->data.literal.value + 16));
            // + 8: skip the "meta information", see "wonka/hal/cpu/x86/scr/calls.s".
        }
        else {
          asm_emit(method->code, x86_mov_mr(EBP, 4 * node->data.literal.value +  12, node->reg));
            // + 12: skip the "meta information", see "wonka/hal/cpu/x86/scr/calls.s".
        }
      }
      else {
        asm_emit(method->code, x86_mov_mr(EBP, -4 * node->stack, node->reg));
      }
    }
    
    /*
    ** Update the method's register bank:
    */
    
    method->registers[node->reg] = NULL;

    /*
    ** Update the node's internal fields:
    */
    
    node->spill = 1;
  }
}

void ra_unspill(j_method method, j_node node, int cg) {
  
  if (node->spill == 1) {
    
    /*
    ** Emit unspill code:
    */
    
    if (cg) {
      if (node->operator == j_literal && node->data.literal.location == j_parameter) {
        if (method->method->flags & ACC_STATIC) {
          asm_emit(method->code, x86_mov_rm(node->reg, EBP, 4 * node->data.literal.value +  16));
            // + 8: skip the "meta information", see "wonka/hal/cpu/x86/scr/calls.s".
        }
        else {
          asm_emit(method->code, x86_mov_rm(node->reg, EBP, 4 * node->data.literal.value +  12));
            // + 12: skip the "meta information", see "wonka/hal/cpu/x86/scr/calls.s".
        }
      }
      else {
        asm_emit(method->code, x86_mov_rm(node->reg, EBP, -4 * node->stack));
      }
    }
    
    /*
    ** Update the method's register bank:
    */
    
    method->registers[node->reg] = node;

    /*
    ** Update the node's internal fields:
    */
    
    node->spill = 0;
  }
}

void ra_release(j_method method, j_node node, int cg) {

  if (method->registers[node->reg] == node) {
    
    node->temporary += 1;

    if (node->temporary < node->count) {
      ra_spill(method, node, cg);
    }
    else {
      method->registers[node->reg] = NULL;
    }
  }
}

void ra_allocate(j_method method, j_node node, int cg) {

  int regs;
  int c;
  int r;
  int i;
 
  c = -1; 
  r = -1;

  /*
  ** Allocate and initialize the node's register allocation information
  ** if not already done so.
  */
  
  if (node->stack == 0) {
    node->stack = ++method->stack;
    node->spill = 0;
  }

  /*
  ** Because we can only use signed 8-bit offsets, we stop compiling
  ** when a method needs more than 31 slots to store local variables.
  */
  
  if (method->stack > 32) {
    method->failed = 1;
  }
  
  /*
  ** Get the node's current register:
  */
  
  if ((node->reg > -1) && (method->registers[node->reg] == node)) {
    c = node->reg;
  }
      
  /*
  ** Register allocate.
  */
  
  if (map[node->operator].flags & RA_INHERIT && method->registers[node->node1->reg] == node->node1) {
    woempa(9, "inherit the register from %p for %p ...\n", node->node1, node);
    
    r = node->node1->reg;
  }
  else if (method->registers[node->reg] == node) {
    woempa(9, "previously allocated %p to register %d\n", node, node->reg);
    
    r = node->reg;
  }
  else {
    
    /*
    ** Get a list of registers (or stack, or memory) to choose from. 
    ** If the result is 0 then we should not be register allocating
    ** this node.
    */
  
    regs = map[node->operator].res;

    if (regs == 0) {
      wabort(ABORT_WONKA, "we should not get here, should we?\n");
    }
  
    for (i = 0; i < J_NUM_REGISTERS; i++) {

      /*
      ** If no free register has been found yet, calculate whether there
      ** is a match: we have a match when the specified node hasn't been
      ** allocated yet, when the current register(s) 'i' is availabe and
      ** when this register can be used for this particular instruction.
      */
    
      if ((method->registers[i] == NULL) && (regs & (1 << i))) {
    
        /*
        ** If there is a match, we found a free register (or register tuple)
        ** that we can use for the specified node; therefore mark it/them as
        ** being allocated.
        */
    
        r = i;

        break; 
      }
    }
  }

  /*
  ** Produce:
  */
 
  if (r != -1) { 

    if (cg) {
      woempa(9, "emit mov or unspill instruction to reg %d\n", r);
      cg_node2register(method, node, r);
    }
    
    woempa(9, "allocated %q to register %d\n", node, r);
    
    method->usage = method->usage | (1 << r);    // mark the register as being used
    method->registers[r] = node;                 // update the register bank
    node->reg = r;                               // update the node's register

  }
  else {
    woempa(9, "warning: failed to allocate %q\n", node);

    method->failed = 1;
    node->reg = -1;                              // update the node's register
  }
  
  /*
  ** Consume/free: 
  */

  if (node->node1) {
    ra_release(method, node->node1, cg);
  }
  
  if (node->node2) {
    ra_release(method, node->node2, cg);
  }
}

void ra_call(j_method method, j_node node, int cg) {
  
  j_node parameter;
  int i;
  
  /*
  ** When we are dealing with a function call, we have to do some
  ** extra housekeeping (1) to deal with those function arguments
  ** that are kept in registers and (2) to take care of the nodes
  ** that live in any of the caller-saved registers. 
  **
  ** Note that we update the method's 'reg'- field after the node
  ** has been initialized.  That is because it's only _after_ the
  ** call that some of the registers might have become available,
  ** with the exception of EAX.
  */

  /*
  ** Push EAX onto the stack: we have (a) to store its value (when in
  ** use) and (b) we want to use it to push the callee's arguments on
  ** the stack.
  */

  if (method->registers[EAX]) {
    ra_spill(method, method->registers[EAX], cg);
  }
    
  /*
  ** Push the caller saved registers on stack:
  */
  
  if (method->registers[ECX]) {
    ra_spill(method, method->registers[ECX], cg);
  }

  if (method->registers[EDX]) {
    ra_spill(method, method->registers[EDX], cg);
  } 
   
  /*
  ** Allocate a register for the return value.
  */
  
  ra_allocate(method, node, 0); 

  /*
  ** Function arguments:
  */
    
  for (i = node->data.call.argc; i > 0; i--) {
    parameter = *(j_node *)(node->data.call.args + i - 1);

    if (cg) {
      if (parameter->operator == j_literal && parameter->data.literal.location == j_constant) {
        // Push the constant on the stack:
        asm_emit(method->code, x86_push_i((j_word)(parameter->data.literal.value)));
      }
      else {
        // Load the parameter into register r:
        cg_node2register(method, parameter, EAX);

        // Push the paramater onto the stack:
        asm_emit(method->code, x86_push_r(EAX));
      }
    }

    if (method->registers[parameter->reg] == parameter) {
      method->registers[parameter->reg] = NULL;
    }
  }
   
  if (cg) {
    /*
    ** Call far, absolute indirect address given in register:
    */
  
    asm_emit(method->code, x86_mov_ri(EAX, (j_word)node->data.call.value));
    asm_emit(method->code, x86_call(EAX));

    /*
    ** Decrement the stack size to clean up the arguments:
    */
  
    if (node->data.call.argc) {
      asm_emit(method->code, x86_add_ri(ESP, 4 * node->data.call.argc));
    }

    /*
    ** Move the return value (if any) from EAX to the reserved
    ** regisiter(s):
    */

    if (node->reg != EAX) {
      asm_emit(method->code, x86_mov_rr(node->reg, EAX));
    }
  }

}

void ra_assign(j_method method, j_node node) {

  if (method->registers[node->node1->reg] != node->node1) {
    ra_allocate(method, node->node1, 0);
  } 
 
  if (method->registers[node->node2->reg] == node->node2) {
    method->registers[node->node2->reg] = NULL;
  }

  /*
  ** An assignment explicitly overwrites a given node, and therefore
  ** invalidating the spilled version on the operand cache.
  */
  
  node->node1->spill = 0;
}

void ra_instruction(j_method method, j_node node, int cg) {
 
  int i;
  
  /*
  ** Set the node's "register"-field: it's an array of nodes that 
  ** provide an overview of what registers are being allocated 
  ** (excluding its own register, if any) at a given point in the
  ** function's flow.
  */

  for (i = 0; i < J_NUM_REGISTERS; i++) {
    node->registers[i] = method->registers[i];
  }

  /*
  ** Allocate a register or a register set for the specified node.
  */
  
  switch (node->operator) {  
    
    /*
    ** Two operands.
    */
    
    case j_add:
    case j_sub:
    case j_mul:
    case j_div:
    case j_and:
    case j_xor:
    case j_or:
      ra_allocate(method, node->node1, cg);
      ra_allocate(method, node->node2, cg);
      ra_allocate(method, node, 0);
      break;
      
    case j_lt:
    case j_gt:
    case j_le:
    case j_ge:
    case j_eq:
    case j_ne:
      ra_allocate(method, node->node1, cg);
      ra_allocate(method, node->node2, cg);
        // NOTE: the comparison operators don't consume any registers.
      break;

    /*
    ** One operand.
    */
     
    case j_neg:
      ra_allocate(method, node->node1, cg);
      ra_allocate(method, node, 0);
      break;
      
    /*
    ** No operands.  Do nothing because these instructions don't consume
    ** any registers.  In case of the j_return, we KNOW that the operand
    ** is already being allocated.
    */
   
    case j_return:    
    case j_jump:
      break;

    /*
    ** Special cases:
    */
      
    case j_call:
      /*
      ** Due to the calling conventions, there are quite a few things need
      ** to be taken care of.
      */
     
      ra_call(method, node, cg);
      break;
      
    case j_assign:
      ra_assign(method, node);
      break;

    case j_literal:
      /*
      ** Do nothing - we encountered a literal, not an instruction.  Every
      ** instruction is held responsible for "preparing" its own operands.
      ** This is required because (a) some x86 instructions can keep one or
      ** more operands in memory and (b) to get the addresses of the join 
      ** points (forward and backward references) right.
      */
      
      break;
      
    default:
      woempa(9, "warning: '%s' not supported by backend\n", node_get_operator(node));
      method->failed = 1;
  }
}

void ra_basic_block(j_method method, j_node node, int cg) {

  int i;
  
  woempa(5, "--------------------------------------------------------\n");
  
  for (i = 0; i < J_NUM_REGISTERS; i++) {
    if (method->registers[i]) {
      woempa(5, " -- spill register %d to location %d: %q\n", i, method->registers[i]->stack, method->registers[i]);   
      ra_spill(method, method->registers[i], cg);
    }
  }  
  
  woempa(5, "--------------------------------------------------------\n");
}

