#ifndef _J_METHOD_H
#define _J_METHOD_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: method.h,v 1.3 2002/03/23 09:41:29 dbuytaert Exp $ 
*/

#include "j-spot.h"

typedef struct j_Method {
  // General:
  char *name;
  w_method method;           // wonka method
  int failed;                // status flag
  
  // Translation:
  unsigned char * bytecode;  // a data block with bytecodes
  int curr_pc;               // current program counter
  int inst_pc;               // current instruction pointer
  int length;                // length of the bytecode

  // Control flow:
  j_node * locals;           // an array of local variables
  j_edge * edges;            // an array of edges
  j_node * joins;              // an array of join points

  j_node start;              // the start node
  j_node exit;               // the exit node
    
  //Register allocation:
  j_node * registers;        // an array of all the allocated registers
  int usage;
  
  // Code generation:
  j_asm code;                // a data block with assembly code
  int stack;                 // size of the required C-stack

} j_Method;

j_method method_create(w_method method);
  /* Create a new method. */

void method_set_name(j_method method, char * name);
  /* Give the method a name. */

void method_set_local(j_method method, j_node node, int index);
  /* Register a local variable. */

int method_num_locals(j_method method);
  /* Get the number of local variables. */

int method_num_parameters(j_method method);
  /* Get the number of function parameters. */

void method_dump(j_method method);
  /* Dump the method to screen for debuging purpose. */

void method_free(j_method method);
  /* Free the given method. */
  
#endif /* _J_METHOD_H */
