#ifndef _J_NODE_H
#define _J_NODE_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: node.h,v 1.6 2002/03/25 09:43:59 dbuytaert Exp $ 
*/

#include "j-spot.h"
  
/*
** Operator:
** ---------
**   The operator consists of an opcode, a type suffix and a size modifier:
*/

typedef enum {
  j_none      = 0,
  j_literal   = 1,
  j_add       = 2,
  j_sub       = 3,
  j_mul       = 4,
  j_div       = 5,
  j_rem       = 6,
  j_neg       = 7,
  j_lsh       = 8,
  j_rsh       = 9,
  j_and       = 10,
  j_or        = 11,
  j_xor       = 12,
  j_le        = 13,
  j_gt        = 14,
  j_eq        = 15,
  j_ne        = 16,
  j_lt        = 17,
  j_ge        = 18,
  j_call      = 19,
  j_assign    = 20,
  j_return    = 21,
  j_jump      = 22,
  j_conv      = 23
} j_opcode;

typedef enum {
  j_constant  = 1,
  j_register  = 2,
  j_temporary = 3,
  j_parameter = 4,
} j_location;

typedef enum {
  j_address   = 1,
  j_boolean   = 2,
  j_byte      = 3, 
  j_character = 4, 
  j_short     = 5, 
  j_double    = 6,
  j_integer   = 7,
  j_float     = 8,
  j_long      = 9
} j_type;

char * operator2string(j_opcode opcode);

/*
** Node:
** -----
*/

typedef struct j_Node {
  char *name;                // the name of this node
  
  j_opcode operator;         // operator
  j_node node1;              // operand node #1, root of child DAG #1
  j_node node2;              // operand node #2, root of child DAG #2
  
  // Control flow:
  j_node incoming[5];        // a list of outgoing nodes
  j_node outgoing[2];        // a list of incoming nodes
  j_node next;               // the next sequential node
  
  int temporary;             // a temporary variable
  int count;                 // reference count
  int pc;                    // program counter
  
  // Register allocation: 
  j_node * registers;        // an array of all the allocated registers
  int stack;
  int spill;
  int reg;

  // Assembler:
  char * address;            // the address of the generated code
  
  // Detailed information:
  union {
    struct {
      int value;
    } label;
    struct {
      j_type type;
      j_location location;
      int value;
    } literal;
    struct {
      j_type type;
      j_location location;
      int (*value)(j_method method, ...);
      int size;
      int argc;
      j_node * args;
    } call;
  } data;

} j_Node;

j_node node_create_literal(j_method method, j_location location, j_type type, int value);
  /* Create a new literal. */

j_node node_create_insn(j_method method, j_opcode operator, j_node n1, j_node n2);
  /* Create a new instruction. */

j_node node_create_call(j_method method, j_type type, void * function, int argc);
  /* Create a new function call. */

void node_add_param(j_method method, j_node function, j_node parameter);
  /* Add a parameter to a call node. */

void node_add_link(j_node n1, j_node n2, int label);
  /* Add a link between node #1 and node #2 with the specified label. */

void node_set_name(j_node node, char * name);
  /* Give the node a name. */

char * node_get_operator(j_node node);
  /* Give the node's operator. */

char * node_print(char * buffer, int * remain, void * data, int width, int precision, unsigned int flags);
  /* Write a node description in the buffer. */

void node_free(j_node node);
  /* Free the given node. */

typedef int (*j_interface_function)(j_method method, ...);

#endif /* _J_NODE_H */
