#ifndef _J_SETTINGS_H
#define _J_SETTINGS_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: settings.h,v 1.1.1.1 2001/11/21 10:41:33 pollock Exp $ 
*/

/* 
** Set the maximum number of different methods that can be profiled:
**   Selecting an appropriate value is important to the success 
**   of the profiler.   Because this value is used to calculate 
**   hashtable indices it is undesirable if this value would be 
**   divisible by either 2, 3 or 5 as the distribution of these 
**   keys needs to be uniform.  Also, the larger the value, the 
**   faster the hashtable but the more memory is consumed.
**   (Some prime numbers: 509, 1021, 2039, 4093, 8191, 16381.)
*/

#define J_MAX_METHOD_NUMBER 2039 * 2

/* 
** Set the maximum number of different threads that can be profiled:
**   Selecting an appropriate value is important to the success 
**   of the profiler.   Because this value is used to calculate 
**   hashtable indices it is undesirable if this value would be 
**   divisible by either 2, 3 or 5 as the distribution of these 
**   keys needs to be uniform.  Also, the larger the value, the 
**   faster the hashtable but the more memory is consumed.
*/

#define J_MAX_THREAD_NUMBER 1024

/*
** Enable or disable extra checks:
**   With `extra checks' enabled, program execution will be
**   examined and unusual, informative or suspicious - yet 
**   harmless - events will be reported.  For instance, you
**   will be warned when too many hashtable collisions are 
**   detected.  Extra checks come with an extra cost though.
*/

#define J_EXTRA_CHECKS 1

/*
**  Set the candidate and hot-spot threshold:
**    Methods that execute frequently during hot-spot profiling
**    may be part of a potential hot-spot.  We refer to such
**    methods as candidate methods.  A method is considered a
**    candidate method if its execution counter surpass a pre-
**    defined 'candidate threshold'.
*/

#define J_CANDIDATE_THRESHOLD 64
#define J_HOTSPOT_THRESHOLD 128

/*
** Set refresh timer interval:
**   To ensure that only frequently executing methods are marked 
**   as being candidates, a refresh timer periodically clears 
**   those methods that have not surpassed the given candidate 
**   threshold. The refresh timer is implemented as a global 
**   counter that increments each time a method is invoked and 
**   triggers when it reaches a certain value.
*/

#define J_TIMER_REFRESH 10 * 1024

/*
** Set reset timer interval:
**   It is possible that the execution of a program has shifted 
**   to a new, potentially more important region of code and is 
**   no longer an attractive optimization oppurtunity.  That is
**   why we will periodically reset all our execution counters 
**   to make room for new methods. This timer is similar to the 
**   the refresh timer but clears all entries, rather then only 
**   non-candidate methods.  The reset interval should be large 
**   enough to allow saturaturation on valid hot-spots but small 
**   enough to allow quick identification of a new phase of 
**   execution.
*/

#define J_TIMER_RESET 50 * 1024

/* 
** Set report timer interval:
**   When DEBUG is enabled you can periodically dump reports
**   with information about the current candidate methods, hot
**   spots, etc.
*/ 

#define J_TIMER_REPORT 10 * 1024

/*
** Support register spills:
**    To support register spills, set this define to 1.
*/

#define J_REGISTER_SPILLS 1

#endif
