/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

package com.acunia.wonka.test.jspot;

public class ArrayTest  {
 
  /*
  ** The "int_" functions will never be compiled.
  */

  public int int_object(int x) {
    Integer array[] = new Integer[10];
    array[x] = new Integer(x);
    return array[x].intValue();
  }

  public int int_integer(int x) {
    int array[] = new int[10];
    array[x] = x;
    return array[x];
  }
  
  public short int_short(int x) {
    short array[] = new short[10];
    array[x] = (short)x;
    return array[x];
  }

  public byte int_byte(int x) {
    byte array[] = new byte[10];
    array[x] = (byte)x;
    return array[x];
  }

  public char int_char(int x) {
    char array[] = new char[10];
    array[x] = (char)x;
    return array[x];
  }

  public int int_arraylength(int array[]) {
    return array.length;
  }

  public int int_multi_1(int x, int y) {
    int array[][] = new int[10][10];
 
    for (int k = 0; k < 10; k++) {
      for (int l = 0; l < 10; l++) {
        array[k][l] = k + l;
      }
    }

    return array[x][y];
  }
  
  public int int_multi_2(int x, int y, int z) {
    int array[][][] = new int[10][10][10];
 
    for (int k = 0; k < 10; k++) {
      for (int l = 0; l < 10; l++) {
        for (int m = 0; m < 10; m++) {
          array[k][l][m] = k + l + m;
        }
      }
    }

    return array[x][y][z];
  }

  /*
  ** The "com_" functions will always be compiled.
  */

  public int com_object(int x) {
    Integer array[] = new Integer[10];
    array[x] = new Integer(x);
    return array[x].intValue();
  }

  public int com_integer(int x) {
    int array[] = new int[10];
    array[x] = x;
    return array[x];
  }

  public short com_short(int x) {
    short array[] = new short[10];
    array[x] = (short)x;
    return array[x];
  }

  public byte com_byte(int x) {
    byte array[] = new byte[10];
    array[x] = (byte)x;
    return array[x];
  }

  public char com_char(int x) {
    char array[] = new char[10];
    array[x] = (char)x;
    return array[x];
  }

  public int com_arraylength(int array[]) {
    return array.length;
  }

  public int com_multi_1(int x, int y) {
    int array[][] = new int[10][10];
 
    for (int k = 0; k < 10; k++) {
      for (int l = 0; l < 10; l++) {
        array[k][l] = k + l;
      }
    }

    return array[x][y];
  }
  
  public int com_multi_2(int x, int y, int z) {
    int array[][][] = new int[10][10][10];
 
    for (int k = 0; k < 10; k++) {
      for (int l = 0; l < 10; l++) {
        for (int m = 0; m < 10; m++) {
          array[k][l][m] = k + l + m;
        }
      }
    }

    return array[x][y][z];
  }

  public void run(JTest jt) {

    /*
    ** Test one-dimensional array operations that make use of 'newarray',
    ** 'xaload', 'xastore' and 'arraylength'.
    */

    for (int i = 0; i < 5; i++) {
      int array[] = new int[i];

      jt.assert(int_object(i), com_object(i), "com_object("+ i +")");
      jt.assert(int_integer(i), com_integer(i), "com_integer("+ i +")");
      jt.assert(int_short(i), com_short(i), "com_short("+ i +")");
      jt.assert(int_byte(i), com_byte(i), "com_byte("+ i +")");
      jt.assert(int_char(i), com_char(i), "com_char("+ i +")");
      jt.assert(int_arraylength(array), com_arraylength(array), "com_char("+ i +")");
    }

    /*
    ** The multi-dimensional array operations that make use of
    ** 'multianew', 'xaload', 'xastore'.
    */

    int d[] = { 2, 2, 0, 0, 4, 5, 1, 9, 9, 9, 3, 7 };

    // Two-dimensional arrays:
    for (int i = 0; i < 12; i = i + 2) {
      jt.assert(int_multi_1(d[i + 0], d[i + 1]), com_multi_1(d[i + 0], d[i + 1]), "com_mutli_1("+ d[i + 0] +", "+ d[i + 1] +")");
    }
    
    // Three-dimensional arrays:
    for (int i = 0; i < 12; i = i + 3) {
      jt.assert(int_multi_2(d[i + 0], d[i + 1], d[i + 2]), com_multi_2(d[i + 0], d[i + 1], d[i + 2]), "com_mutli_2("+ d[i + 0] +", "+ d[i + 1] +", "+ d[i + 2] +")");
    }
  }
}
