/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

package com.acunia.wonka.test.jspot;

import java.util.Vector;

public class MethodTest extends Dummy {
 
  private Vector v = new Vector();
 
  public static int max(int x, int y) {
    return (x > y) ? x : y;
  }

  public int cal(int a, int b, int c) {
    return a << 2 +  b << 1 + c;
  } 

  /*
  ** The "int_" functions will never be compiled.
  */

  public static int int_invokestatic_1(int x, int y) {
    return max(x, y);
  }

  public static int int_invokestatic_2(int x, int y) {
    return int_invokestatic_1(x, y);
  }

  public static int int_invokestatic_3(int x, int y) {
    return int_invokestatic_1(x, y);
  }

  public Object int_invokespecial_1(int x) {
    return new Integer(x);
  }

  public int int_invokespecial_2(int x) {
    super.invokespecial(x);
    return this.x;
  }

  public int int_invokevirtual_1(int x) {
    v.add(new Integer(x));
    return v.size();
  }

  public int int_parameter_1(int x, int y) {
    return cal(x - y, x + y, y - x);
  }

  public int int_parameter_2(int x, int y) {
    return cal((x > y ? 1 : 0), (y > x ? 1 : 0), (x == y ? 1 : 0));
  }

  public int int_parameter_3(int x, int y) {
    return cal((x > y ? x - y : y - x), (y > x ? y - x : x - y), (x == y ? x + y: x * y));
  }

  /*
  ** The "com_" functions will always be compiled.
  */

  public static int com_invokestatic_1(int x, int y) {
    return max(x, y);
  }

  public static int com_invokestatic_2(int x, int y) {

    /*
    ** Call a static Java method.
    */ 

    return int_invokestatic_1(x, y);
     // ---^^^
  }

  public static int com_invokestatic_3(int x, int y) {

    /*
    ** Call a static native method.
    */

    return com_invokestatic_1(x, y);
     // ---^^^
  }

  public Object com_invokespecial_1(int x) {
    return new Integer(x);
  }
  
  public int com_invokespecial_2(int x) {
    super.invokespecial(x);
    return this.x;
  }

  public int com_invokevirtual_1(int x) {
    v.add(new Integer(x));
    return v.size();
  }

  public int com_parameter_1(int x, int y) {
    return cal(x - y, x + y, y - x);
  }

  public int com_parameter_2(int x, int y) {
    return cal((x > y ? 1 : 0), (y > x ? 1 : 0), (x == y ? 1 : 0));
  }

  public int com_parameter_3(int x, int y) {
    return cal((x > y ? x - y : y - x), (y > x ? y - x : x - y), (x == y ? x + y: x * y));
  }

  public void run(JTest jt) {

    /*
    ** Bytecode "invokestatic".  We test both compilation of static 
    ** methods and JIT compiled methods calling static methods - both
    ** native and Java methods.
    */

    for (int i = 0; i < 5; i++) {
      jt.assert(int_invokestatic_1(2, i), com_invokestatic_1(2, i), "MethodTest.com_invokestatic_1(2, "+ i +")");
      jt.assert(int_invokestatic_2(2, i), com_invokestatic_2(2, i), "MethodTest.com_invokestatic_2(2, "+ i +")");
      jt.assert(int_invokestatic_3(2, i), com_invokestatic_3(2, i), "MethodTest.com_invokestatic_3(2, "+ i +")");
    }

    /*
    ** Bytecode "invokespecial".
    */

    for (int i = 0; i < 5; i++) {
      jt.assert(int_invokespecial_1(i), com_invokespecial_1(i), "MethodTest.com_invokespecial_1("+ i +")");
      jt.assert(int_invokespecial_2(i), com_invokespecial_2(i), "MethodTest.com_invokespecial_2("+ i +")");
    }

    /*
    ** Bytecode "invokevirtual".
    */

    for (int i = 0; i < 5; i++) {
      jt.assert(int_invokevirtual_1(i), com_invokevirtual_1(i) - 1, "MethodTest.com_invokevirtual_1("+ i +")");
    }

    /*
    ** Complex parameter passing.
    */

    for (int i = 0; i < 5; i++) {
      jt.assert(int_parameter_1(i, 10 - i), com_parameter_1(i, 10 - i), "MethodTest.com_parameter_1("+ i +", "+ (10 - i) +")");
      jt.assert(int_parameter_2(i, 10 - i), com_parameter_2(i, 10 - i), "MethodTest.com_parameter_2("+ i +", "+ (10 - i) +")");
      jt.assert(int_parameter_3(i, 10 - i), com_parameter_3(i, 10 - i), "MethodTest.com_parameter_3("+ i +", "+ (10 - i) +")");
    }
  }
}

