/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: compiler.c,v 1.22 2002/03/14 15:02:58 dbuytaert Exp $
*/

#include "j-spot.h"
#include "code-generator.h"
#include "register-allocator.h"
#include "assembler.h"
#include "ir.h"
#include "bytecode.h"
                       
void compiler_init_node(j_method method, j_node node, int depth) {

  woempa(9, "%s%q\n", j_spaces(depth), node);

  if (ir_is_join(node) || ir_is_jump(node)) {
    ra_basic_block(method, node, 0);
  }
  
  if (node->node1 && node->node1->operator != j_call) {
    compiler_init_node(method, node->node1, depth + 1);
  }
               
  if (node->node2 && node->node2->operator != j_call) {
    compiler_init_node(method, node->node2, depth + 1);
  }

  ra_instruction(method, node, 0);
}

void compiler_init(j_method method) {

  j_node node = method->start;
   
  ra_basic_block(method, node, 0);
   
  while (node) {
    compiler_init_node(method, node, 3);
    node = node->next;
  }
}

void compiler_emit_node(j_method method, j_node node, int depth) {

  woempa(9, "%s %q\n", j_spaces(depth), node);
  
  if (ir_is_join(node) || ir_is_jump(node)) {
    ra_basic_block(method, node, 1);
  }

  /*
  ** Set the node's target address.
  */
  
  node->address = method->code->step;

  if (node->node1 && node->node1->operator != j_call) {
    compiler_emit_node(method, node->node1, depth + 1);
  }
               
  if (node->node2 && node->node2->operator != j_call) {
    compiler_emit_node(method, node->node2, depth + 1);
  }

  cg_instruction(method, node);
}

void compiler_emit(j_method method) {

  j_node node = method->start;

  ra_basic_block(method, node, 0);

  cg_prologue(method);
  
  while (node) {
    compiler_emit_node(method, node, 1);
    node = node->next;
  }
}

void compile(j_method method) {

  woempa(9, "\n[pass 1]\n");

  /*
  ** Pass 1, compiler_init()
  **
  **   After calling compiler_init(), registers have been allocated,
  **   the size of the frame and the number of registers/spills have
  **   been calculated, etc.  This information is required to emit 
  **   code for the function (think also 'prologue') in the function
  **   compiler_emit().
  */
  
  if (method->failed == 0) {
    compiler_init(method);
  }
  else {
    woempa(9, "oops, register allocation failed\n");
  }
  
  woempa(9, "\n[pass 2]\n");
  
  /*
  ** Pass 2: compiler_emit()
  */
  
  if (method->failed == 0) {
    compiler_emit(method);
  }
  else {
    woempa(9, "oops, code emission failed\n");
  }

  woempa(9, "\n[done]\n");
}

