/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: control-flow.c,v 1.24 2002/03/25 08:56:52 dbuytaert Exp $
*/

#include "bytecode.h"
#include "j-spot.h"
#include "ir.h"

void cf_join_points(j_method method) {
  unsigned char code;
  int num;
  int def;
  int low;
  int hgh;
  int i;
  
  while (method->failed == 0 && method->curr_pc < method->length) {
    code = method->bytecode[method->curr_pc++];

    j_flarp(green, "pc = %d, bytecode: %d (%s)\n", method->inst_pc, code, bytecode_get_name(code));

    switch (code) {

      /*
      ** One byte argument:
      */ 
        
      case (ldc):
      case (bipush):
      case (iload):
      case (aload):
      case (istore):
      case (astore):
      case (newarray):
        method->curr_pc += 1;
        break;
      
      /*
      ** Two byte argument:
      */
     
      case (checkcast):
      case (instanceof):   
      case (sipush):
      case (ldc_w):
      case (getstatic):
      case (putstatic):
      case (getfield):
      case (putfield):
      case (invokevirtual):
      case (invokespecial):
      case (invokestatic):
      case (anewarray):
      case (iinc):
      case (new):
        method->curr_pc += 2;
        break;
 
      /*
      ** Three byte argument:
      */

      case (multianewarray):
        method->curr_pc += 3;
        break;
      
      /*
      ** TODO
      */

      case (lor):
      case (lxor):
      case (land):
      case (ishl):
      case (ishr):
      case (lshl):
      case (lshr):
      case (ladd):
      case (fadd):
      case (dadd):
      case (lsub):
      case (fsub):
      case (dsub):
      case (lmul):
      case (fmul):
      case (dmul):
      case (idiv):
      case (ldiv):
      case (fdiv):
      case (ddiv):
      case (irem):
      case (lrem):
      case (frem):
      case (drem):
      case (lneg):
      case (fneg):
      case (dneg):
      case (iushr):
      case (lushr):
      case (lcmp):
      case (fcmpl):
      case (fcmpg):
      case (dcmpl):
      case (dcmpg):
      case (lload):
      case (fload):
      case (dload):
      case (dload_0):
      case (dload_1):
      case (dload_2):
      case (dload_3):
      case (fload_0):
      case (fload_1):
      case (fload_2):
      case (fload_3):
      case (lload_0):
      case (lload_1):
      case (lload_2):
      case (lload_3):
      case (dstore_0):
      case (dstore_1):
      case (dstore_2):
      case (dstore_3):
      case (fstore_0):
      case (fstore_1):
      case (fstore_2):
      case (fstore_3):
      case (lstore_0):
      case (lstore_1):
      case (lstore_2):
      case (lstore_3):
      case (laload):
      case (baload):
      case (faload):
      case (daload):
      case (lstore):
      case (fstore):
      case (dstore):
      case (lastore):
      case (bastore):
      case (fastore):
      case (dastore):
      case (dconst_1): 
      case (fconst_1): 
      case (fconst_2): 
      case (lconst_1): 
      case (ret):
      case (jsr):
      case (jsr_w):
      case (ldc2_w):
      case (athrow):
      case (lreturn):
      case (freturn):
      case (dreturn):
      case (invokeinterface):
      case (monitorenter):
      case (monitorexit):
        method->failed = 1;
        break;
        
      /*
      ** Control flow:
      */

      case (if_icmpeq):        
      case (if_icmpne):
      case (if_icmplt):
      case (if_icmpge):
      case (if_icmpgt):
      case (if_icmple):
      case (if_acmpeq):        
      case (if_acmpne):
      case (ifnonnull):       
      case (ifnull):
      case (ifeq):
      case (ifne):
      case (iflt):
      case (ifge):
      case (ifgt):
      case (ifle):
        method->joins[method->inst_pc + bytecode_get_short(method)] = (j_node)1;
        break;

      case (goto_n):
        method->joins[method->inst_pc + bytecode_get_short(method)] = (j_node)1;
        break;
        
      case (goto_w):
        method->joins[method->inst_pc + bytecode_get_wide(method)] = (j_node)1;
        break;
      
      case (tableswitch):
        // word align the operands:
        method->curr_pc = (method->inst_pc + 4) & -4;
        
        // retrieve the default branch offset, the low endpoint and the high endpoint:
        def = bytecode_get_wide(method);
        low = bytecode_get_wide(method);
        hgh = bytecode_get_wide(method);
       
        woempa(9, "pc = %d, default = %d, low = %d, high = %d\n", method->curr_pc, def, low, hgh);

        // register the join point of each branch between the endpoints:
        for (i = 0; i < hgh - low + 1; i++) {
          method->joins[method->inst_pc + bytecode_get_wide(method)] = (j_node)1;
        }
        
        // register the join point of the default branch:
        method->joins[method->inst_pc + def] = (j_node)1;
        break;
       
      case (lookupswitch):
        // word align the operands:
        method->curr_pc = (method->inst_pc + 4) & -4;

        // retrieve the default branch offset and the number of branch/offset pairs:
        def = bytecode_get_wide(method);
        num = bytecode_get_wide(method);

        woempa(9, "default branch offset = %d, number of branch/offset pairs = %d\n", def, num);

        // register the join point of each branch:
        for (i = 0; i < num; i++) {
          bytecode_get_wide(method);
          method->joins[method->inst_pc + bytecode_get_wide(method)] = (j_node)1;
        }
        
        // register the join point of the default branch:
        method->joins[method->inst_pc + def] = (j_node)1;
        break;

      case (wide):
        code = method->bytecode[method->curr_pc++];

        switch (code) {

          /*
          ** Two byte argument:
          */
          
          case (iload):
          case (lload):
          case (fload):
          case (dload):
          case (aload):
          case (istore):
          case (lstore):
          case (fstore):
          case (dstore):
          case (astore):
            method->curr_pc++;
            method->curr_pc++;
            break;
            
          case (iinc):
            method->curr_pc++;
            method->curr_pc++;
            method->curr_pc++;
            method->curr_pc++;
            break;

          /*
          ** TODO:
          */
            
          case (ret):
            method->failed = 1;
            break;

          /*
          ** No arguments:
          */
            
          default:
            break;
        }
        
        break;
        
      /*
      ** No arguments:
      */
        
      default:
        break;
    }
    
    method->inst_pc = method->curr_pc;
  }
}

