/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: ct-hooks.c,v 1.33 2002/03/26 11:10:48 dbuytaert Exp $
*/

// wonka headers:
#include "clazz.h"
#include "arrays.h"
#include "constant.h"
#include "interpreter.h"
#include "descriptor.h"
#include "loading.h"
#include "methods.h"
#include "strings.h"
#include "checks.h"
#include "stack.h"
#include "fields.h"
#include "heap.h"

// j-spot headers:
#include "ir.h"
#include "dot.h"
#include "stabs.h"
#include "control-flow.h"
#include "compiler.h"
#include "translate.h"
#include "assembler.h"
#include "ct-hooks.h"
#include "statistics.h"

int create_name(w_method method, char * buffer) {

  w_string string;
  int i = 0;
  int p = 0;

  string = method->clazz->dotified;
  
  for (i = 0; i < (signed)string->length; i++) {
    if (string->chars[i] == '/') {
      p += sprintf(buffer + p, "."); 
    }
    else {
      p += sprintf(buffer + p, "%c", string->chars[i]);
    }
  }

  p += sprintf(buffer + p, ".");
 
  string = method->name;

  for (i = 0; i < (signed)string->length; i++) {
    if (string->chars[i] == '/') {
      p += sprintf(buffer + p, "."); 
    }
    else {
      p += sprintf(buffer + p, "%c", string->chars[i]);
    }
  }
  
  string = method->param_list;

  for (i = 0; i < (signed)string->length; i++) {
    switch (string->chars[i]) {
      case '/':
        p += sprintf(buffer + p, "_"); 
        break;
      case ';':
        break;
      default:
        p += sprintf(buffer + p, "%c", string->chars[i]);
    }
  }
  
  *(buffer + p) = '\0';

  return p;
}

void jspot_compile(w_method method) {

  static char * buffer = NULL;
  static x_mutex mutex = NULL;
  static int count = 0;
  static int calls = 0;
  j_method m = NULL;
  int i;

  /*
  ** If we get here, then this is the first time we try compiling this
  ** method.  To avoid that we would try to compile it again we update
  ** the method's dispatcher table.
  */
 
  method->dispatcher = (w_dispatcher)(method->dispatcher + 1);

  calls++;

  /*
  ** Never compile methods that start with "int_".
  */
  
  if (method->name->chars[0] == 'i' && method->name->chars[1] == 'n' && method->name->chars[2] != 't' && method->name->chars[3] != '_') { 
    return;
  }
  
  /*
  ** Only compile methods that start with "com_".
  */

  //if (method->name->chars[0] != 'c' || method->name->chars[1] != 'o' || method->name->chars[2] != 'm' || method->name->chars[3] != '_') { 
  //  return;
  //}
 
  //if (calls < 433) {
  //  return;
  //}  

  /*
  ** Never compile a method that takes a float, long or a double as a
  ** parameter; these primitive types are not being supported yet ...
  */
  
  // check the parameters:
  for (i = 0; i < method->nargs; i++) {
    if (method->arg_types[i] == clazzFloat || method->arg_types[i] == clazzLong || method->arg_types[i] == clazzDouble) {
      return;
    }
  }
        
  // check the return type:
  if (method->return_type == clazzFloat || method->return_type == clazzLong || method->return_type == clazzDouble) {
    return;
  }
 
  /*
  ** Never compile methods that catch exceptions:
  */

  if (method->numExceptions) {
    return;
  }
  
  if (mutex == NULL) {

    /*
    ** Initialize the static mutex.
    */
    
    mutex = x_mem_alloc(sizeof(x_Mutex));
    x_mutex_create(mutex);

    /*
    ** Initialize the static buffer.
    */
    
    buffer = x_mem_alloc(1024);
    
    /*
    ** Initialize the formatter.
    */
    
    x_formatter('q', node_print);
  }
  
  x_mutex_lock(mutex, x_eternal);
  
  /*
  ** Debug output:
  */
  
  j_flarp(red, "compiling method '%M' now ...\n", method);
 
  /*
  ** Create and initialize the method.
  */
   
  m = method_create(method);
  create_name(method, buffer);
  method_set_name(m, buffer); 
  
  /*
  ** Control flow: collect all control flow information.
  */
  
  if (m->failed == 0) {
    cf_join_points(m);
  }
  
  /*
  ** Translate: convert bytecode to intermediate representation.
  */

  if (m->failed == 0) {
    translate(m);
  }
  
  /*
  ** Intermediate representation: build the intermediate representation.
  */
    
  if (m->failed == 0) {
    ir(m);
  }
  
  /*
  ** Dotify: render a visual representation of the intermediate
  ** representation.
  */
    
  if (m->failed == 0) {
    dot(m);
  }

  /*
  ** Compile: convert intermediate representation to bytecode.
  */
    
  if (m->failed == 0) {
    compile(m);
  }   
  
  /*
  ** Register: register new code to the VM:
  */
  
  if (m->failed == 0) {  
    method->function.void_fun = (w_void_fun)asm_export(m->code);
    method->dispatcher = (w_dispatcher)(method->dispatcher + 2);
    method->return_address = (w_word)m->exit->address;
    method->flags |= ACC_NATIVE;
    count++;
  }

  /*
  ** Debug: dump the method to the screen and/or generate a stabs file.
  */
 
  if (m->failed == 0) {
    fprintf(stderr, "%d/%d: '%s' (%d)\n", count, calls, m->name, method->code_length);
    asm_dump(m->code);
    
    stabs(m);
  }

  /*
  ** Free the allocated resources:
  */
  
  method_free(m);

  x_mutex_unlock(mutex);
}

/*
** Compile-time interfaces:
*/

j_node ct_load_local(j_method method, int index) {

  /*
  ** We can pre-calculate the exact address of the required item in
  ** the local reference table at compile time. 
  */

  w_field field = getFieldConstant(method->method->clazz, index);

  return node_create_literal(method, j_constant, j_integer, (int)field->slot);
}

j_node ct_load_static(j_method method, int index) {
 
  /*
  ** We can pre-calculate the exact address of the required item in
  ** the constant pool at compile time. 
  */
  
  w_field field = getFieldConstant(method->method->clazz, index);

  // woempa(9, "returning pointer to static field with slot %d '%w' of %w (%p, %p)\n", field->slot, NM(field), NM(field->declaring_clazz), field->declaring_clazz->staticFields + field->slot, *(field->declaring_clazz->staticFields + field->slot));

  return node_create_literal(method, j_constant, j_address, (int)(field->declaring_clazz->staticFields + field->slot));
}

j_node ct_load_type(j_method method, int index) {
 
  /*
  ** Map the given index onto a primitive type.
  */
  
  w_clazz clazz = atype2clazz[index];
  
  woempa(9, "returning pointer to clazz '%k' (%p)\n", clazz, clazz);

  return node_create_literal(method, j_constant, j_address, (int)clazz);
}

j_node ct_load_class(j_method method, int index) {
 
  /*
  ** We can pre-calculate the exact address of the required item in
  ** the constant pool at compile time. 
  */
  
  w_clazz clazz = *((w_clazz *)method->method->clazz->constants[index].get(method->method->clazz, index));

  woempa(9, "returning pointer to clazz '%k' (%p)\n", clazz, clazz);
  
  return node_create_literal(method, j_constant, j_address, (int)clazz);
}

j_node ct_load_array(j_method method, int index) {
 
  w_clazz clazz;
  
  /*
  ** We can pre-calculate the exact address of the required item in
  ** the constant pool at compile time. 
  */
  
  clazz = *((w_clazz *)method->method->clazz->constants[index].get(method->method->clazz, index));
  clazz = seekClazzByDesc(clazz->adesc, loader2hashtable(clazz2loader(clazz)));
 
  woempa(9, "returning pointer to clazz '%k' (%p)\n", clazz, clazz);
  
  return node_create_literal(method, j_constant, j_address, (int)clazz);
}


j_node ct_load_field(j_method method, int index) {
 
  /*
  ** We can pre-calculate the exact address of the required item in
  ** the constant pool at compile time. 
  */
  
  w_field field = getFieldConstant(method->method->clazz, index);

  return node_create_literal(method, j_constant, j_address, (int)field);
}


j_node ct_load_method(j_method method, int index, int * argc, int * retc) {

  w_method m;
  int i;
  
  /*
  ** We can pre-calculate the exact address of the required item in
  ** the constant pool at compile time. 
  */
  
  m = getMethodConstant(method->method->clazz, index);
 
  /*
  ** Check whether we have a method that is being supported:
  */
  
  // check the parameter list:
  for (i = 0; i < m->nargs; i++) {
    if (m->arg_types[i] == clazz_float || m->arg_types[i] == clazz_long || m->arg_types[i] == clazz_double) {
      method->failed = 1;
    }
  }

  // check the return type:
  if (m->return_type == clazz_float || m->return_type == clazz_long || m->return_type == clazz_double) {
    method->failed = 1;
  }
        
  /*
  ** Set the number of arguments that are passed onto the method
  ** upon calling the method.
  */
  
  *argc = m->nargs;
  
  /*
  ** Set the number of arguments that are returned upon leaving
  ** the method:
  */
  
  *retc = m->return_i;
  
  /*
  ** Return the method:
  */

  return node_create_literal(method, j_constant, j_address, (int)m);
}

j_node ct_load_constant(j_method method, int index) {

  w_string string;
  w_instance instance;

  /*
  ** We can pre-calculate the exact address of the required item in
  ** the constant pool at compile time.
  */
  
  j_node node = NULL;
  w_cnt constant  = method->method->clazz->constants + index;

  if (constant->cnt_tag == CONSTANT_STRING) {

    /*
    ** Apparently we store w_strings in a class' constant pool rather
    ** than w_instances, although it are w_instances that are being 
    ** requested from that constant pool ... probably to make it so
    ** that the w_instances of the w_strings can get collected?  So
    ** in interpreted mode, "getStringInstance()" will (re)allocate 
    ** a new w_instance when needed.  However, in compiled mode, we 
    ** don't want constant string instances to get garbage collected.
    ** That is, we want all constant instances to be exactly that -
    ** compile-time constants - such that we can take benefit of it
    ** as much as possible.
    **
    ** NOTE: is the below correct?  It are constants, but can these 
    **       w_strings and w_instances get changed later on?
    */

    // get the w_string from the class' constant pool:
    string = getStringConstant(method->method->clazz, index);

    // get the w_instance (allocate one when needed):
    instance = getStringInstance(W_Thread_system, string);

    // make the string instance immortal:
    newGlobalReference(instance);
    
    node = node_create_literal(method, j_constant, j_address, (int)instance);
  }
  else if (constant->cnt_tag == CONSTANT_INTEGER) {
    node = node_create_literal(method, j_constant, j_integer, (int)getIntegerConstant(method->method->clazz, index));
  }
  else if (constant->cnt_tag == CONSTANT_FLOAT) {
    method->failed = 1;
    node = NULL;  // = node_create_literal(method, j_constant, j_float, (int)getFloatConstant(method->method->clazz, index));
  }
  else if (constant->cnt_tag == CONSTANT_LONG) {
    method->failed = 1;
    node = NULL;  // = node_create_literal(method, j_constant, j_long, (int)getLongConstant(method->method->clazz, index));
  } 
  else if (constant->cnt_tag == CONSTANT_DOUBLE) {
    method->failed = 1;
    node = NULL; // = node_create_literal(method, j_constant, j_double, (int)getDoubleConstant(method->method->clazz, index));
  }
  else {
    wabort(ABORT_WONKA, "Bad constant type %d\n", constant->cnt_tag);
  }

  woempa(9, "ct_load_constant: %q\n", node);
    
  return node;
}


