/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: dot.c,v 1.18 2002/02/26 15:28:30 dbuytaert Exp $
*/

#include "j-spot.h"
#include "strings.h"
#include "ir.h"

FILE *df;

void dot_print_wstring(FILE *fd, w_string string) {

  int i = 0;
  
  if (string != NULL) {
    for (i = 0; i < (signed)string->length; i++) {
      fprintf(fd, "%c", string->chars[i]);
    }
  }
  else {
    fprintf(fd, "<NULL>");
  }
}

void dot_emit_node(j_method method, j_node node) {

  j_node parameter;
  int i;
  
  /*
  ** Print the node's labels.  Try to do some basic formatting to
  ** high-light some of the meta data and to make the graph a bit
  ** more human-readable.
  */
  
  if (node->operator == j_literal && node->data.literal.location == j_constant && node->data.literal.type == j_address) { 
    fprintf(df, "  n%p [ label = \"constant\\n%p\" ]\n", node, (j_word *)node->data.literal.value);
  }
  else if (node->operator == j_literal && node->data.literal.location == j_constant) { 
    fprintf(df, "  n%p [ label = \"constant\\n%d\" ]\n", node, node->data.literal.value);
  }
  else if (node->operator == j_literal && node->data.literal.location == j_parameter) {
    fprintf(df, "  n%p [ label = \"parameter %d\" ]\n", node, node->data.literal.value);
  }
  else if (node->operator == j_literal && node->data.literal.location == j_temporary) {
    fprintf(df, "  n%p [ label = \"temporary %d\" ]\n", node, node->data.literal.value);
  }
  else if (node->operator == j_call) { 
    if (node->name) {
      fprintf(df, "  n%p [ label = \"call '%s'\\n %p\" ]\n", node, node->name, (j_word *)node->data.call.value);
    }
    else {
      fprintf(df, "  n%p [ label = \"call\\n%p\" ]\n", node, (j_word *)node->data.call.value);
    }
  }
  else {
    if (node->name) {
      fprintf(df, "  n%p [ label = \"%s\\n'%s'\" ]\n", node, node_get_operator(node), node->name);
    }
    else {
      fprintf(df, "  n%p [ label = %s ]\n", node, node_get_operator(node));
    }
  }

  /*
  ** If the current node is a function call, iterate through all the
  ** parameter nodes:
  */
  
  if (node->operator == j_call) {
    for (i = 0; i < node->data.call.argc; i++) {
      parameter = *(j_node *)(node->data.call.args + i);

      fprintf(df, "  n%p -> n%p\n", node, parameter);
 
      if (parameter->next == NULL) {
        dot_emit_node(method, parameter);
      }
    }
  }

  /*
  ** Link the individual DAGs:
  */
  
  if (node->outgoing[0] != NULL) {
    if (node->outgoing[1] != NULL) {
      fprintf(df, "  n%p -> n%p [ label = true, style = dotted ]\n", node, node->outgoing[0]);
    }
    else {
      fprintf(df, "  n%p -> n%p [ style = dotted ]\n", node, node->outgoing[0]);
    }
    
    // fprintf(df, "  { rank = same; n%p; n%p; }\n", node, node->outgoing[0]);
  }

  if (node->outgoing[1] != NULL) {
    if (node->outgoing[0] != NULL) {
      fprintf(df, "  n%p -> n%p [ label = false, style = dotted ]\n", node, node->outgoing[1]);
    }
    else {
      fprintf(df, "  n%p -> n%p [ style = dotted ]\n", node, node->outgoing[1]);
    }
    
    // fprintf(df, "  { rank = same; n%p; n%p; }\n", node, node->outgoing[1]);
  }
  
  if (node->outgoing[0] != NULL && node->outgoing[1] != NULL) {
    // fprintf(df, "  { rank = same; n%p; n%p; }\n", node->outgoing[0], node->outgoing[1]);
  }
  
  /*
  ** If we have one or more child nodes, connect them to their parent
  ** node:
  */

  if (node->node1) {
    fprintf(df, "  n%p -> n%p\n", node, node->node1);
   
    if (node->node1->next == NULL) { 
      dot_emit_node(method, node->node1);
    }
  }
               
  if (node->node2) {
    fprintf(df, "  n%p -> n%p\n", node, node->node2);
    
    if (node->node2->next == NULL) {
      dot_emit_node(method, node->node2);
    }
  }
 
  /*
  ** If we have exactly two child nodes, force them on the same rank:
  */
  
  if (node->node1 && node->node2) {
    // fprintf(df, "  { rank = same; n%p; n%p; }\n", node->node1, node->node2);
  }
}

void dot(j_method method) {
  
  static char * name = NULL; 
  j_node node;

  /*
  ** Initialize the temporaries.
  */
  
  if (name == NULL) {
    name = (char *)x_mem_alloc(1024);
  }
  
  node = method->start;

  sprintf(name, "user/dot/%s.dot", (method->name ? method->name : "undefined.method")); 
  
  df = fopen(name, "w");
  
  fprintf(df, "digraph G {\n");
  fprintf(df, "  page  = 11;\n");
  fprintf(df, "  start [ label = \"%s(...)\", shape = box ]\n", method->name);
  fprintf(df, "  start -> n%p [ style = dotted ]\n", node);

  while (node) {
    dot_emit_node(method, node);
    node = node->next;
  }  
  
  fprintf(df, "}\n");
 
  fclose(df);
}


