/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: hotspot.c,v 1.13 2002/03/25 09:28:12 dbuytaert Exp $ 
*/

#include "j-spot.h"
#include "hotspot.h"
#include "settings.h"
#include "method2h.h"
#include "ir.h"
#include "ct-hooks.h"

#include "debug.h"
#include "methods.h"
#include "descriptor.h"
#include "clazz.h"
#include "strings.h"
#include "ts-mem.h"

typedef struct hs_node_flat hs_node_flat;
typedef struct hs_node_tree hs_node_tree;
typedef struct hs_node_head hs_node_head;
typedef struct hs_node_stack hs_node_stack;

struct hs_node_flat {
  w_method method;
  w_int tcalls;
  w_int rcalls;
};

hs_node_flat hs_data_flat[J_MAX_METHOD_NUMBER] = {{0,},};
hs_node_flat hs_data_sort[J_MAX_METHOD_NUMBER] = {{0,},};

inline static unsigned int hs_hash_flat(w_method method) {

  unsigned int hash = (unsigned int)((unsigned int)method % J_MAX_METHOD_NUMBER);

  // woempa(9, "method = %p, hash = %d, range = %d\n", method, hash, J_MAX_METHOD_NUMBER);

  return hash;

}

void hs_refresh() {

  int i;

  for (i = 0; i < J_MAX_METHOD_NUMBER; i++) {
    if (hs_data_flat[i].method && hs_data_flat[i].rcalls < J_CANDIDATE_THRESHOLD) {
      hs_data_flat[i].rcalls = 0;
    }
  }
}

void hs_reset() {

  int i;

  for (i = 0; i < J_MAX_METHOD_NUMBER; i++) {
    if (hs_data_flat[i].method) {
      hs_data_flat[i].rcalls = 0;
    }
  }
}

void hs_report() {

  hs_node_flat swap;
  int norm_methods = 0;
  int cand_methods = 0;
  int norm_tcalls = 0;
  int cand_tcalls = 0;
  int norm_rcalls = 0;
  int cand_rcalls = 0;
  int i = 0;
  int j = 0;
  
  /*
  ** Copy profile information int a temporary array:
  */

  for (i = 0; i < J_MAX_METHOD_NUMBER; i++) {
    hs_data_sort[i] = hs_data_flat[i];
  }

  /*
  ** Sort profile data by their recent calls: 
  */
  
  for (i = 0; i < J_MAX_METHOD_NUMBER; i++) {
    for (j = i + 1; j < J_MAX_METHOD_NUMBER; j++) {
      if ((hs_data_sort[i].rcalls) && (hs_data_sort[j].rcalls) && (hs_data_sort[i].rcalls < hs_data_sort[j].rcalls)) {
        swap = hs_data_sort[i];
        hs_data_sort[i] = hs_data_sort[j];
        hs_data_sort[j] = swap;
      }
    }
  }

  j_flarp(blue, "----- PROFILER: CRITICAL PATH, CURRENT HOTSPOT -------------------------------------\n");
  j_flarp(blue, "rcalls  tcalls  size  method                                                        \n");
  j_flarp(blue, "------  ------  ----  --------------------------------------------------------------\n");
  for (i = 0; i < J_MAX_METHOD_NUMBER; i++) {
    if (hs_data_sort[i].method) {
      if ((hs_data_sort[i].rcalls > J_CANDIDATE_THRESHOLD) && (hs_data_sort[i].method->code_length)) {
        j_flarp(blue, "%6i  %6i  %4i  %M\n", hs_data_sort[i].tcalls, hs_data_sort[i].rcalls, hs_data_sort[i].method->code_length, hs_data_sort[i].method);
        cand_methods += 1;
        cand_tcalls += hs_data_sort[i].tcalls;
        cand_rcalls += hs_data_sort[i].rcalls;   
      }
      else {
        norm_methods += 1;
        norm_tcalls += hs_data_sort[i].tcalls;
        norm_rcalls += hs_data_sort[i].rcalls;
      }
    }
  }
  j_flarp(blue, "\n");
  
  if (cand_rcalls && cand_tcalls && norm_rcalls && norm_tcalls && cand_methods && norm_methods) {
    j_flarp(blue, "%8i recent calls.: %7i candidate (%2i%%), %7i non-candidate (%2i%%)\n", (cand_rcalls + norm_rcalls), cand_rcalls, (cand_rcalls * 100) / (cand_rcalls + norm_rcalls), norm_rcalls, 100 - (cand_rcalls * 100) / (cand_rcalls + norm_rcalls));
    j_flarp(blue, "%8i total calls..: %7i candidate (%2i%%), %7i non-candidate (%2i%%)\n", (cand_tcalls + norm_tcalls), cand_tcalls, (cand_tcalls * 100) / (cand_tcalls + norm_tcalls), norm_tcalls, 100 - (cand_tcalls * 100) / (cand_tcalls + norm_tcalls));
    j_flarp(blue, "%8i methods......: %7i candidate (%2i%%), %7i non-candidate (%2i%%)\n", (cand_methods + norm_methods), cand_methods, (cand_methods * 100) / (cand_methods + norm_methods), norm_methods, 100 - (cand_methods * 100) / (cand_methods + norm_methods));
  }
  
  j_flarp(blue, "------------------------------------------------------------------------------------\n");
}

void hs_method_invoke(w_method method) {

  static unsigned int refresh = 0;
  static unsigned int report = 0;
  static unsigned int reset = 0;
  unsigned int key;
  unsigned int i;

  return;
  
  // woempa(9, "invoked method %M\n", method);

  //#ifdef METHOD2H
  //  method_write(method);
  //#endif
  
  /*
  ** Generate a hash:
  */

  key = hs_hash_flat(method);

  #ifdef J_EXTRA_CHECKS
    if (J_MAX_METHOD_NUMBER < key) {
      wabort(ABORT_WONKA, "invalid hash key!\n");
    }
  #endif

  /*
  ** Lineair probing.  If the location a key is hashed into is occupied, 
  ** then we proceed down the table until the target or an empty slot is 
  ** found.  If the target is not found, it is placed in the empty address. 
  ** It does cluster but it is still very fast.
  */
    
  for (i = key; i < J_MAX_METHOD_NUMBER; i++) {
    if (hs_data_flat[i].method == NULL || hs_data_flat[i].method == method) break;
  }
  
  /*
  ** Check to see whether the hashtable is full:
  */
  
  if (i >= J_MAX_METHOD_NUMBER) { 
    wabort(ABORT_WONKA, "warning: no place to store profile information of method '%M'!\n", method); 
    return;
  }

  #ifdef J_EXTRA_CHECKS
    if (i - key > 9) { 
      //woempa(9, "warning: the hashtable is getting full and becoming slower - %i spills\n", i - key); 
    }
  #endif
  
  /* 
  ** Update the method's profile information in the hashtable: 
  */
  
  if (hs_data_flat[i].tcalls == 0) {
    hs_data_flat[i].method = method;
    hs_data_flat[i].tcalls = 0;
    hs_data_flat[i].rcalls = 0;
  }
  
  hs_data_flat[i].tcalls += 1;
  hs_data_flat[i].rcalls += 1;

  /* 
  ** Care needs to be taken of the refresh timer: 
  */
  
  if (refresh == J_TIMER_REFRESH) {
    hs_refresh();
    refresh = 0;
  }
  else {
    refresh += 1;
  }
  
  /*
  ** Care needs to be taken of the reset timer: 
  */
  
  if (reset == J_TIMER_RESET) {
    hs_reset();
    reset = 0;
  }
  else {
    reset += 1;
  }
  
  /*
  ** Care needs to be taken of the status timer: 
  */
  
  if (report == J_TIMER_REPORT) {
    hs_report();
    report = 0;
  }
  else {
    report += 1;
  }
}

void hs_method_return(w_method method) {
  jspot_compile(method);
}

