/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: ir.c,v 1.14 2002/03/23 09:41:29 dbuytaert Exp $
*/

#include "oswald.h"
#include "j-spot.h"
#include "ir.h"

int ir_is_jump(j_node node) {
  switch (node->operator) {
    case j_jump:
    case j_le:
    case j_gt:
    case j_eq:
    case j_ne:
    case j_lt:
    case j_ge:
      return 1;
    default:
      return 0;
  }
}

int ir_is_join(j_node node) {
  if (node->incoming[1]) {
    return 1;
  }
  else {
    return 0;
  }
}

void ir_add_node(j_method method, j_node node, int tedge, int fedge, int offset) {

  j_node prev;
  j_node next;

  /*
  ** Add the node to the ordered list of nodes.  The list is ordered
  ** based on the nodes' program counter.  That is, based upon their
  ** their occurence in the Java bytecode and independent of control
  ** flow.
  */
  
  next = method->start;
  prev = NULL;

  while (next && next != node && next->pc < node->pc + offset) {
    prev = next;
    next = next->next;
  }
   
  if (node != next) {     // Don't insert duplicate nodes.
    if (prev == NULL) {
      node->next = method->start;
      method->start = node;
    }
    else if (next != NULL) {
      prev->next = node;
      node->next = next;
    }
    else {
      prev->next = node;
    }

    /*
    ** Set the start and the exit nodes.
    */
    
    if (method->start == NULL) {
      method->start = node;
    }
    
    if (node->operator == j_return) {
      method->exit = node;
    }
        
    /*
    ** Take care of the outgoing edges.
    */
    
    node->outgoing[0] = (j_node)tedge;
    node->outgoing[1] = (j_node)fedge;
  }
}

j_node ir_get_node(j_method method, int pc) {

  j_node node = method->start;

  /*
  ** Get the node at the specified program counter.  If no such node 
  ** exists, we move /forward/ to the next top-level node.
  */

  while (node && node->pc < pc) {
    node = node->next;
  }

  return node;
}

j_node ir_get_branch(j_method method, j_node node) {
  return node->outgoing[0];
}

void ir_build_dags(j_method method) {

  j_node node = NULL;
  j_node next = NULL;
  
  node = method->start;
 
  woempa(9, "---------------------------------------------------");
  while (node) {
    woempa(9, "node %p - pc %d - %q: %3d %3d\n", node, node->pc, node, node->outgoing[0], node->outgoing[1]);
    node = node->next;
  }
  woempa(9, "---------------------------------------------------");
  
  node = method->start;

  while (node) {
  
    /*
    ** At the points where control /flows/ to one or more other nodes,
    ** we connect or link all the nodes involved.
    */
    
    if ((int)node->outgoing[0] == j_link_none) {
      node->outgoing[0] = NULL;
    }
    else if ((int)node->outgoing[0] == j_link_next) {
      node_add_link(node, node->next, 1);
    }
    else {
      next = ir_get_node(method, (int)node->outgoing[0]);
      node_add_link(node, next, 1);
    }
         
    if ((int)node->outgoing[1] == j_link_none) {
      node->outgoing[1] = NULL;
    } 
    else if ((int)node->outgoing[1] == j_link_next) {
      node_add_link(node, node->next, 0);
    }
    else {
      next = ir_get_node(method, (int)node->outgoing[1]);
      node_add_link(node, next, 0);
    }
    
    node = node->next;
  }
}

void ir_support_gc(j_method method) {
}

void ir(j_method method) {
  
  ir_build_dags(method);

  ir_support_gc(method);
}

