/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: method.c,v 1.20 2002/03/12 15:46:42 dbuytaert Exp $
*/

#include "oswald.h"
#include "methods.h"
#include "descriptor.h"

#include "j-spot.h"
#include "bytecode.h"
#include "register-allocator.h"
#include "ir.h"
#include "assembler.h"

j_method method_create(w_method wmethod) {

  j_method method = x_mem_calloc(sizeof(j_Method));
  
  /*
  ** Allocate memory for the bytecode:
  */
  
  method->bytecode = x_mem_alloc(wmethod->code_length * sizeof(char));
  memcpy(method->bytecode, wmethod->code, wmethod->code_length);

  /*
  ** Allocate memory for the local variables:
  */

  method->locals = x_mem_calloc((wmethod->local_i + wmethod->return_i + wmethod->arg_i + wmethod->stack_i) * sizeof(j_node));

  /*
  ** Allocate memory for the edges:
  */
  
  method->edges = x_mem_calloc(wmethod->code_length * sizeof(j_edge) * 5);
    // FIXME: currently there is a hardcoded maximum of five outgoing edges per node.
   
  /*
  ** Allocate memory for the join points:
  */

  method->joins = x_mem_calloc(wmethod->code_length * sizeof(j_node));
  
  /*
  ** Allocate memory for the registers:
  */

  method->registers = x_mem_calloc(J_NUM_REGISTERS * sizeof(j_node));
  
  /*
  ** Allocate memory for the assembly code:
  */
  
  method->code = asm_create_block();
  
  /*
  ** Initialize internal fields:
  */
 
  method->method = wmethod;
  method->curr_pc = 0;
  method->inst_pc = 0;
  method->failed = 0;
  method->length = wmethod->code_length;
  
  return method;
}

void method_set_name(j_method method, char * name) {
  /*
  ** Release the old name (if set):
  */
     
  if (method->name) {
    x_mem_free(method->name);
  }
             
  /*
  ** Copy the new name into the method's name field:
  */
 
  method->name = x_mem_alloc(strlen(name) + 1); // + 1 because of '\0'
  memcpy(method->name, name, strlen(name) + 1);
}

int method_num_locals(j_method method) {
  return method->method->local_i;
}

int method_num_parameters(j_method method) {
  if (method->method->flags & ACC_STATIC) {
    return method->method->nargs - 1;
  }
  else {
    return method->method->nargs;
  }
}

void method_free(j_method method) {

  j_node node = method->start;
  j_node next;
  
  while (node) {
    next = node->next;
    node_free(node);
    node = next;
  }
  
  if (method->name) {
    x_mem_free(method->name);
  }
  
  if (method->bytecode) {
    x_mem_free(method->bytecode);
  }
 
  if (method->locals) {
    x_mem_free(method->locals);
  }
  
  if (method->joins) {
    x_mem_free(method->joins);
  }

  if (method->edges) {
    x_mem_free(method->edges);
  }

  if (method->registers) {
    x_mem_free(method->registers);
  }
  
  if (method->code) {
    asm_free_block(method->code);
  }

  x_mem_free(method);
}

void method_dump_node(j_node node, int depth) {

  woempa(9, "%s%q\n", j_spaces(depth), node);
      
  if (node->node1) {
    method_dump_node(node->node1, depth + 1);
  }

  if (node->node2) {
    method_dump_node(node->node2, depth + 1);
  }
}
               
void method_dump(j_method method) {

  j_node node = method->start;
  
  woempa(9, "------------------------------------------------------------------\n");
  while (node) {
    method_dump_node(node, 0);
    node = node->next;
  }
  woempa(9, "------------------------------------------------------------------\n");
}
  
