/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: method2h.c,v 1.8 2002/03/24 16:48:24 dbuytaert Exp $
*/

#include "j-spot.h"
#include "settings.h"

#include "debug.h"
#include "methods.h"
#include "clazz.h"
#include "descriptor.h"
#include "strings.h"
#include "ts-mem.h"

#include <errno.h>

w_method method_ht[J_MAX_METHOD_NUMBER] = {0,};

inline static unsigned int method_hash(w_method method) {

  unsigned int hash = (unsigned int)((unsigned int)method % J_MAX_METHOD_NUMBER);
 
  // woempa(9, "method = %p, hash = %d, range = %d\n", method, hash, J_MAX_METHOD_NUMBER);
   
  return hash;
   
}

/*
** Convert a w_string to a C-style string and replace '.', '/', '$' by 
** '_'.
*/

char *method_name(w_string name) {

  static char buffer[1024];
  char *proper = buffer;
  int i;
         
  memset(buffer, '\0', 1024);
                  
  for (i = 0; i < (w_int)name->length; ++i) {
    if (name->chars[i] == '.') {
      proper[i] = '_';
    }
    else if (name->chars[i] == '/') {
      proper[i] = '_';
    }
    else if (name->chars[i] == '(') {
      proper[i] = '_';
    }
    else if (name->chars[i] == ')') {
      proper[i] = '_';
    }
    else if (name->chars[i] == '[') {
      proper[i] = '_';
    }
    else if (name->chars[i] == ']') {
      proper[i] = '_';
    }
    else if (name->chars[i] == ';') {
      proper[i] = '_';
    }
    else if (name->chars[i] == '<') {
      proper[i] = '_';
    }
    else if (name->chars[i] == '>') {
      proper[i] = '_';
    }
    else {
      proper[i] = name->chars[i];
    }
  }

  proper[i] = '\0';
                                              
  return proper;

}

void method_write(w_method method) {
   
  static char *file = "methods.h";
  static int start = 0;
  static FILE *out;
  unsigned int key;
  unsigned int i;
  unsigned int k;
  
  /*
  ** Open output file if not already done so:
  */
  
  if (!start) {
    start = 1;

    out = fopen(file, "w");

    if (out == NULL) {
      wabort(ABORT_WONKA, "Could not open file '%s': %d.\n", file, errno);
    }
  }
                              
  /*
  ** Generate a hash:
  */

  key = method_hash(method);

  #ifdef J_EXTRA_CHECKS
    if (J_MAX_METHOD_NUMBER < key) {
      wabort(ABORT_WONKA, "invalid hash key!\n");
    }
  #endif

  /*
  ** Lineair probing.  If the location a key is hashed into is occupied, 
  ** then we proceed down the table until the target or an empty slot is 
  ** found.  If the target is not found, it is placed in the empty address. 
  ** It does cluster but it is still very fast.
  */
    
  for (i = key; i < J_MAX_METHOD_NUMBER; i++) {
    if (method_ht[i] == NULL || method_ht[i] == method) break;
  }
  
  /*
  ** Check to see whether the hashtable is full:
  */
  
  if (i >= J_MAX_METHOD_NUMBER) { 
    woempa(9, "warning: no place to store profile information of method '%M'!\n", method); 
    return;
  }

  #ifdef J_EXTRA_CHECKS
    if (i - key > 9) { 
      woempa(9, "warning: the hashtable is getting full and becoming slower - %i spills\n", i - key); 
    }
  #endif
  
  if (!method_ht[i]) {
    
    /*
    ** Put method in hashtalbe:
    */
  
    method_ht[i] = method;

    /* 
    ** Write this method to a struct:
    */
  
    fprintf(out, "static w_byte %s", method_name(method->clazz->dotified));
    fprintf(out, "_%s%s[] = {", method_name(method->name), method_name(method->param_list));
    for (k = 0; k < method->code_length; k++) {
      if (k % 10 == 0) {
        fprintf(out, "\n  ");
      }

      fprintf(out, "0x%02x, ", (unsigned char)method->code[k]);
    }
    fprintf(out, "\n};\n\n");
    fflush(out);
  }


}
