/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: mimic-stack.c,v 1.8 2002/03/25 16:18:28 dbuytaert Exp $ 
*/

#include "ir.h"
#include "mimic-stack.h"
#include "settings.h"

#define COMPILER_MAX_STACK_SIZE 1000

void * stack[COMPILER_MAX_STACK_SIZE] = {0};
int offset = 0;
int tos = - 1;

void ms_init(j_method method) {
  int i;
  
  offset = 0;
  // tos = method_num_locals(method) - 1;
  tos = - 1;
  
  for (i = 0; i < COMPILER_MAX_STACK_SIZE; i++) {
    stack[i] = NULL;
  }
}

int ms_push(void * item) {
  if (tos > COMPILER_MAX_STACK_SIZE) {
    wabort(ABORT_WONKA, "mimic stack: stack overflow: tos = %i, offset = %i!\n", tos, offset);
  }

  tos = tos + 1;
  
  stack[tos] = item;

  return tos - offset;

}

void * ms_pop(void) {

  if ((tos < 0) || (tos < offset)) {
    wabort(ABORT_WONKA, "mimic stack: stack underflow: tos = %i, offset = %i!\n", tos, offset);
  }

  tos = tos - 1;
  
  return stack[tos + 1];
  
}

void * ms_top(void) {
  if ((tos < 0) || (tos < offset)) {
    wabort(ABORT_WONKA, "mimic stack: stack underflow: tos = %i, offset = %i!\n", tos, offset);
  }

  return stack[tos];
}

void * ms_get(int depth) {
  if ((tos - depth < 0) || (tos - depth < offset)) {
    return NULL;
  }
  else {
    return stack[tos - depth];
  }
}

void ms_set(int depth, void * item) {
  if ((tos - depth < 0) || (tos - depth < offset)) {
    wabort(ABORT_WONKA, "mimic stack: stack underflow: tos = %i, offset = %i!\n", tos, offset);
  }
  else {
    stack[tos - depth] = item;
  }
}

void ms_dup1(void) {
  if ((tos < 0) || (tos < offset)) {
    wabort(ABORT_WONKA, "mimic stack: stack underflow: tos = %i, offset = %i!\n", tos, offset);
  }

  tos += 1;
  stack[tos] = stack[tos - 1];
}

void ms_dup1_x1(void) {
  if ((tos < 1) || (tos < offset + 1)) {
    wabort(ABORT_WONKA, "mimic stack: stack underflow: tos = %i, offset = %i!\n", tos, offset);
  }

  tos += 1;
  stack[tos - 0] = stack[tos - 1];
  stack[tos - 1] = stack[tos - 2];
  stack[tos - 2] = stack[tos - 0];
}

void ms_dup1_x2(void) {
  if ((tos < 2) || (tos < offset + 2)) {
    wabort(ABORT_WONKA, "mimic stack: stack underflow: tos = %i, offset = %i!\n", tos, offset);
  }

  tos += 1;
  stack[tos - 0] = stack[tos - 1];
  stack[tos - 1] = stack[tos - 2];
  stack[tos - 2] = stack[tos - 3];
  stack[tos - 3] = stack[tos - 0];
}

void ms_dup2(void) {
  if ((tos < 1) || (tos < offset + 1)) {
    wabort(ABORT_WONKA, "mimic stack: stack underflow: tos = %i, offset = %i!\n", tos, offset);
  }

  tos += 2;
  stack[tos - 0] = stack[tos - 2];
  stack[tos - 1] = stack[tos - 3];   
}

void ms_dup2_x1(void) {
  if ((tos < 2) || (tos < offset + 2)) {
    wabort(ABORT_WONKA, "mimic stack: stack underflow: tos = %i, offset = %i!\n", tos, offset);
  }

  tos += 2;
  stack[tos - 0] = stack[tos - 2];
  stack[tos - 1] = stack[tos - 3];
  stack[tos - 2] = stack[tos - 4];
  stack[tos - 3] = stack[tos - 0];
  stack[tos - 4] = stack[tos - 1];   
}

void ms_dup2_x2(void) {
  if ((tos < 3) || (tos < offset + 3)) {
    wabort(ABORT_WONKA, "mimic stack: stack underflow: tos = %i, offset = %i!\n", tos, offset);
  }

  tos += 2;
  stack[tos - 0] = stack[tos - 2];
  stack[tos - 1] = stack[tos - 3];
  stack[tos - 2] = stack[tos - 4];
  stack[tos - 3] = stack[tos - 5];
  stack[tos - 4] = stack[tos - 0];
  stack[tos - 5] = stack[tos - 1];
}

void ms_swap(void) {
  if (tos < offset + 1) {
    void * swap = stack[tos];
    stack[tos] = stack[tos - 1];
    stack[tos - 1] = swap;
  }
  else {
    wabort(ABORT_WONKA, "mimic stack: stack underflow: tos = %i, offset = %i!\n", tos, offset);
  }
}

void ms_flap(void) {
  int i;
  int n = tos - offset + 1;

  stack[++tos] = (void *)offset;  // store offset
  stack[++tos] = (void *)111111;  // store delimiter

  for (i = 0; i < n; i++) {
    stack[++tos] = stack[offset + i];    
  }
  
  offset = offset + n + 2;
}

void ms_flop(void) {
  tos = offset - 3;
  offset = (int)stack[offset - 2];
}

int ms_next(void) {
  return offset;
}

void ms_set_local(j_method method, j_node node, int index) {
  woempa(9, "add local variable %p to slot %d\n", node, index);
  method->locals[index] = node;
}

j_node ms_get_local(j_method method, int index) {
  woempa(9, "get local variable %p from slot %d: %p\n", stack[offset + index], index);
  return method->locals[index];
}

void ms_dump(void) {
  int i;

  j_flarp(yellow, "---------------\n");
  j_flarp(yellow, "   tos: %i  --> stack[%i] = %p\n", tos, tos, stack[tos]);
  j_flarp(yellow, "offset: %i  --> stack[%i] = %p\n", offset, offset, stack[offset]);
  
  for (i = tos; i >= 0; i--) {
    j_flarp(yellow, "   loc: %i - value: %p\n", i, stack[i]);
  }
  j_flarp(yellow, "---------------\n");
}
