/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: node.c,v 1.34 2002/03/26 12:18:44 dbuytaert Exp $
*/

#include "oswald.h"
#include "j-spot.h"
#include "register-allocator.h"
#include "stdarg.h"
#include "ir.h"

char * node_operators[] = {
  "none",
  "literal",
  "add",
  "sub",
  "mul",
  "div",
  "rem",
  "neg",
  "lsh",
  "rsh",
  "and",
  "or",
  "xor",
  "le",
  "gt",
  "eq",
  "ne",
  "lt",
  "ge", 
  "call",
  "assign",
  "return",
  "jump",
  "convert"
};

j_node node_create_literal(j_method method, j_location location, j_type type, int value) {

  j_node node = x_mem_calloc(sizeof(j_Node));

  /*
  ** Allocate memory for the register bank:
  */
  
  node->registers = x_mem_calloc(J_NUM_REGISTERS * sizeof(j_node));
    // TODO: is this required for literals, or only for instructions?
  
  /*
  ** Create and initialize the literal's content:
  */
  
  node->operator = j_literal;
  node->data.literal.location = location;
  node->data.literal.type = type;
  node->data.literal.value = value;

  return node;
}

j_node node_create_insn(j_method method, j_opcode operator, j_node n1, j_node n2) {

  j_node node;

  /*
  ** Allocate memory for the node:
  */

  node = x_mem_calloc(sizeof(j_Node));
    
  /*
  ** Allocate memory for the register bank:
  */

  node->registers = x_mem_calloc(J_NUM_REGISTERS * sizeof(j_node));

  /*
  ** Create and initialize the node's content:
  */
  
  node->pc = method->inst_pc;
  node->operator = operator;
  node->node1 = n1;
  node->node2 = n2;
  
  if (n1) {
    /*
    ** Increase the reference count of node #1:
    */
  
    n1->count += 1;
  }

  if (n2) {    
    /*
    ** Increase the reference count of node #2:
    */
  
    n2->count += 1;
  }

  return node;
    
}

j_node node_create_call(j_method method, j_type type, void * function, int size) {

  j_node node;

  /*
  ** Allocate memory for the node:
  */

  node = x_mem_calloc(sizeof(j_Node));
    
  /*
  ** Allocate memory for the register bank:
  */

  node->registers = x_mem_calloc(J_NUM_REGISTERS * sizeof(j_node));

  /*
  ** Create and initialize the function's content:
  */
  
  node->pc = method->inst_pc;
  node->operator = j_call;

  node->data.call.location = j_register;
  node->data.call.type = type;
  node->data.call.value = (j_interface_function)function;
  node->data.call.args = x_mem_alloc((x_size)size * sizeof(j_node));
  node->data.call.size = size;
  node->data.call.argc = 0;
  
  return node; 
}

void node_set_name(j_node node, char * name) {

  /*
  ** Release the old name (if set):
  */
     
  if (node->name) {
    x_mem_free(node->name);
  }
             
  /*
  ** Copy the new name into the node's name field:
  */
 
  node->name = x_mem_alloc(strlen(name) + 1); // + 1 because of '\0'
  memcpy(node->name, name, strlen(name) + 1);
}

char *node_get_operator(j_node node) {
  return (char *)node_operators[node->operator];
}

void node_add_param(j_method method, j_node node, j_node parameter) {

  // Perform a run-time sanity check:
  if (node->operator != j_call) {
    wabort(ABORT_WONKA, "the specified node is not a valid function-node\n");
  }
  
  /*
  ** Promote function arguments that are subexpressions (non-literals,
  ** non-calls) to be root nodes.  As a result, DAGs will get split in
  ** smaller DAGs that pre-calculate subexpressions and that store the
  ** values in temporaries.  This is required for both code generation 
  ** and register allocation.  In case of the statement 'foo(bar());',
  ** we would have to call 'bar()' after we already started to setup
  ** 'foo()'s stack frame.  In essence, we split the DAG to read: 'x =
  ** bar(); foo(x);'.
  */
  
  if (parameter->operator != j_literal) {
    ir_add_node(method, parameter, j_link_next, j_link_none, j_insert_before);
  }
  
  // Increase the parameter's reference count by one:
  parameter->count += 1;

  // Increase the function's argument count by one:
  node->data.call.argc += 1;
  
  // Add the parameter to the end of the parameter list:
  *(node->data.call.args + node->data.call.size - node->data.call.argc) = parameter;
    
  // Perform a run-time sanity check: 
  if (node->data.call.argc > node->data.call.size) {
    wabort(ABORT_WONKA, "the specified node's argument count is exceeded\n");
  }
}

void node_add_link(j_node node1, j_node node2, int label) {

  int i;

  /*
  ** Take care of the outgoing edges:
  */
  
  woempa(9, "adding edge with label %d between %p at pc %d and %p at pc %d\n", label, node1, node1->pc, node2, node2->pc);
  
  if (label) {
    node1->outgoing[0] = node2;   // "true" edge
  }
  else {
    node1->outgoing[1] = node2;   // "false" edge
  }
  
  /*
  ** Take care of the incoming edges:
  */
  
  for (i = 0; i < 5; i++) {
    if (node2->incoming[i] == NULL) {
      node2->incoming[i] = node1;
      break;
    }
  }
}

char * node_print(char * buffer, int * remain, void * data, int width, int precision, unsigned int flags) {

  j_node node = data;
  int pos = 0;
 
  if (node) {

    /*
    ** Print the node's string representation/information.
    */
  
    if (node->operator == j_literal && node->data.literal.location == j_constant && node->data.literal.type == j_address) { 
      pos = sprintf(buffer, "constant '%p'", (j_word *)node->data.literal.value);
    }
    else if (node->operator == j_literal && node->data.literal.location == j_constant) { 
       pos = sprintf(buffer, "constant '%d'", node->data.literal.value);
    }
    else if (node->operator == j_literal && node->data.literal.location == j_parameter) {
      pos = sprintf(buffer, "parameter %d", node->data.literal.value);
    }
    else if (node->operator == j_literal && node->data.literal.location == j_temporary) {
      pos = sprintf(buffer, "temporary %d", node->data.literal.value);
    }
    else if (node->operator == j_call) { 
      if (node->name) {
        pos = sprintf(buffer, "call '%s' at %p", node->name, (j_word *)node->data.call.value);
      }
      else {
        pos = sprintf(buffer, "call %p", (j_word *)node->data.call.value);
      }
    }
    else {
      if (node->name) {
        pos = sprintf(buffer, "%s '%s'", node_get_operator(node), node->name);
      }
      else {
        pos = sprintf(buffer, "%s", node_get_operator(node));
      }
    }
  }
  else {
    pos = sprintf(buffer, "node 'NULL'");
  }

  *remain -= pos;
  buffer  += pos;

  return buffer;
}

void node_free(j_node node) {

  j_node parameter;
  int i;

  /*
  ** Release the node's operands (if any):
  */
  
  if (node->node1) {
    if (node->node1->count > 0) {
      node->node1->count -= 1;
    }
    else {
      node_free(node->node1);
    }
  }
  
  if (node->node2) {
    if (node->node2->count > 0) {
      node->node2->count -= 1;
    }
    else {
      node_free(node->node2);
    }
  }
  
  /*
  ** Release the node's parameters (if any):
  */

  if (node->operator == j_call) {
    for (i = 0; i < node->data.call.argc; i++) {
      parameter = *(j_node *)(node->data.call.args + i);

      if (parameter->count > 0) {
        parameter->count -= 1;
      }
      else {
        node_free(parameter);
      }
    }
  }


  /*
  ** Release the node's register bank (if any):
  */
  
  if (node->registers) {
    x_mem_free(node->registers);
  }

  /*
  ** Release the node's name (if any):
  */
  
  if (node->name) {
    x_mem_free(node->name);
  }
  
  x_mem_free(node);
}

