/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: rt-hooks.c,v 1.23 2002/03/26 16:41:27 dbuytaert Exp $
*/

// wonka headers:
#include "interpreter.h"
#include "descriptor.h"
#include "methods.h"
#include "strings.h"
#include "checks.h"
#include "stack.h"
#include "clazz.h"
#include "fields.h"
#include "arrays.h"
#include "exception.h"

// j-spot headers:
#include "ir.h"
#include "dot.h"
#include "compiler.h"
#include "translate.h"
#include "assembler.h"
#include "rt-hooks.h"
#include "call-stack.h"

int rt_cmp(j_method method, j_node n1, j_node n2) {
  wabort(ABORT_WONKA, "called rt_cmp\n");
  return 1;
}

int rt_monitorenter(j_method method, j_node node) {
  wabort(ABORT_WONKA, "called rt_monitorenter\n");
  return 1;
}

int rt_monitorexit(j_method method, j_node node) {
  wabort(ABORT_WONKA, "called rt_monitorexit\n");
  return 1;
}

int rt_athrow(j_method method, j_node node) {
  wabort(ABORT_WONKA, "called rt_athrow\n");
  return 1;
}

int rt_getstatic(j_word * data) {
  return *data;
}

int rt_putstatic(w_field field, j_word value) {

  if (field->desc->tag != D_Primitive) {
    woempa(9, "put instance %p (%k) in the static variable at slot %d of field %p\n", value, instance2clazz((w_instance)value), field->slot, field);
    setStaticReferenceField(field->declaring_clazz, field->slot, (w_instance)value);
  }
  else {
    woempa(9, "put %d in the static variable at slot %d of field %p\n", value, field->slot, field);
    field->declaring_clazz->staticFields[field->slot] = value;
  }
 
  return 1;
}

int rt_getfield(w_instance instance, w_field field) {
  if (instance == NULL) {
     throwException((w_thread)thread_current->xref, clazzNullPointerException, NULL);
     return 0;
  }
  else {
    woempa(9, "get field %p (%d) of instance %p (%k): %d\n", field, field->slot, instance, instance2clazz(instance), instance[field->slot]);
    return (int)instance[field->slot];
  }
}

int rt_putfield(w_instance instance, w_field field, j_word value) {
  
  if (field->desc->tag != D_Primitive) {
    woempa(9, "put instance %p (%k) in the local variable at slot %d of field %p of instance %p (%k)\n", value, instance2clazz((w_instance)value), field->slot, field, instance, instance2clazz(instance));
    setReferenceField(instance, (w_instance)value, field->slot);
  }
  else {
    woempa(9, "put %d in the local variable at slot %d of field %p of instance %p (%k)\n", value, field->slot, field, instance, instance2clazz(instance));
    instance[field->slot] = value;
  }

  return 1;
}

int rt_new(w_clazz clazz) {

  w_instance instance;
  w_thread thread;

  thread = (w_thread)thread_current->xref;
  instance = allocInstance(thread, clazz);
  addLocalReference(thread, instance);

  woempa(9, "created new instance %p of type '%k' (%p)\n", instance, clazz, clazz);
  
  return (int)instance;
}

int rt_instanceof(w_instance instance, w_clazz clazz) {

  if (instance == NULL) {
    
    /*
    ** If the object reference is 'null', the JVM is supposed to push 
    ** '0' on the operand stack.
    */
    
    woempa(9, "NULL instanceof clazz %k\n", clazz);
    return 0;
  }
  else {
    woempa(9, "%k instanceof clazz %k\n", instance2clazz(instance), clazz);
    return isAssignmentCompatible(instance2clazz(instance), clazz);
  }
}

int rt_checkcast(w_instance instance, w_clazz clazz) {

  if (instance && !isAssignmentCompatible(instance2clazz(instance), clazz)) {
    /*
    ** Throw a "ClassCastException" when the object reference is non-null 
    ** and if it can not be cast to the resolved type.
    */

    throwException((w_thread)thread_current->xref, clazzClassCastException, NULL);
  }

  return 1;
}

int rt_invokespecial(w_method current_method, w_method actual_method, ...) {
  
  w_clazz current_clazz = current_method->clazz;
  w_clazz actual_clazz = actual_method->clazz;
  w_thread thread = (w_thread)thread_current->xref;
  w_method caller = thread->top->native;
  w_instance exception;
  w_frame bottom;
  va_list ap;
  int result;
  int i;

  if (isSet(actual_method->flags, ACC_ABSTRACT)) {
    if (isSet(current_clazz->flags, ACC_SUPER) && isSuperClass(current_method->clazz, current_clazz->superClazz)) {
      actual_clazz = current_clazz->superClazz;
      actual_method = actual_clazz->methods[actual_method->slot];
    }
  }
  else {
    if (isSuperClass(current_method->clazz, current_clazz->superClazz)) {
      actual_clazz = current_clazz->superClazz;
    }
  
    actual_method = actual_clazz->methods[actual_method->slot];
  }

  result = 0;
  bottom = allocBottomFrame(thread, actual_method);

  if (bottom) {

    /*
    ** Native methods are dealt with differently than Java methods:
    ** native methods pass their arguments using the stack, whereas
    ** Java methods pass their arguments using local variables.  We
    ** check whether a given method is a native method on each method
    ** invocation.  After all, we don't know whether Java methods are
    ** going to compiled to native code later, and if so, we don't 
    ** want to recompile any existing methods.  
    */
    
    if (isSet(actual_method->flags, ACC_NATIVE)) { 
      woempa(9, "'%M' calls native method '%M'\n", caller, actual_method);
      
      va_start(ap, actual_method);
      for (i = 0; i < (signed)actual_method->arg_i; i++) {
        bottom->o_top[i] = (j_word)va_arg(ap, unsigned char *);
      }
      va_end(ap);

      bottom->o_top += actual_method->arg_i;
    }
    else {
      woempa(9, "'%M' calls Java method '%M'\n", caller, actual_method);
      
      va_start(ap, actual_method);
      for (i = 0; i < (signed)actual_method->arg_i; i++) {
        bottom->locals[i] = (j_word)va_arg(ap, unsigned char *);
      }
      va_end(ap);

    }

    setFlag(bottom->flags, FRAME_JNI);
    exception = runThread(thread, bottom);

    /*
    ** If an exception is thrown that can not be handled by the compiled
    ** Java method, propagate it to the calling Java method.  If this is
    ** the case, we skip the remaining code of the compiled Java method
    ** by overwriting this function's return address ...
    */
    
    if (exception) {
      cs_skip_caller((j_word)caller->return_address);
      
      setLocalException(thread, exception);
    }
    
    /*
    ** Retrieve the return value, if any.
    */
    
    if (actual_method->return_i) {
      result = *(bottom->o_top - 1);
    }
    
    releaseBottomFrame(bottom);
  }
  
  return result;
}

int rt_invokevirtual(w_method method, w_instance instance, ...) {
  
  w_method actual_method;
  w_instance exception;
  w_clazz actual_clazz;
  w_method caller;
  w_thread thread;
  w_frame bottom;
  va_list ap;
  int result;
  int i;
  
  thread = (w_thread)thread_current->xref;
  caller = thread->top->native;

  /*
  ** Instance methods are always being invoked using an object.  Due to
  ** inheritance, the type of an instance - and hence the actual method
  ** to call - is known only at run-time.  However, all private methods,
  ** final methods and methods of final classes can be bound statically
  ** as they can not be inherited, and no look-up (dynamic binding) has
  ** to be performed.
  */

  if (method->slot >= (w_int)method->clazz->numInheritableMethods) {
    actual_method = method;
  }
  else {
    actual_clazz = instance2clazz(instance);
    actual_method = actual_clazz->methods[method->slot];
  }
      
  result = 0;
  bottom = allocBottomFrame(thread, actual_method);

  if (bottom) {

    /*
    ** Native methods are dealt with differently than Java methods:
    ** native methods pass their arguments using the stack, whereas
    ** Java methods pass their arguments using local variables.  We
    ** check whether a given method is a native method on each method
    ** invocation.  After all, we don't know whether Java methods are
    ** going to compiled to native code later, and if so, we don't 
    ** want to recompile any existing methods.  
    */
    
    if (isSet(actual_method->flags, ACC_NATIVE)) { 
      woempa(9, "'%M' calls native instance method '%M' on instance %p (%k)\n", caller, actual_method, instance, instance2clazz(instance));
      
      bottom->o_top[0] = (w_word)instance;
    
      va_start(ap, instance);
      for (i = 1; i < (signed)method->arg_i; i++) {
          // NOTE: 'i' starts from '1' because we already added one local variable to the frame.
        bottom->o_top[i] = (j_word)va_arg(ap, unsigned char *);
      }
      va_end(ap);

      bottom->o_top += method->arg_i;
    }
    else {
      woempa(9, "'%M' calls Java instance method '%M' on instance %p (%k)\n", caller, actual_method, instance, instance2clazz(instance));
      
      bottom->locals[0] = (w_word)instance;
    
      va_start(ap, instance);
      for (i = 1; i < (signed)method->arg_i; i++) {
          // NOTE: 'i' starts from '1' because we already added one local variable to the frame.
        bottom->locals[i] = (j_word)va_arg(ap, unsigned char *);
      }
      va_end(ap);

    }
    
    setFlag(bottom->flags, FRAME_JNI);
    exception = runThread(thread, bottom);
    
    /*
    ** If an exception is thrown that can not be handled by the compiled
    ** Java method, propagate it to the calling Java method.  If this is
    ** the case, we skip the remaining code of the compiled Java method
    ** by overwriting this function's return address ...
    */
    
    if (exception) {
      cs_skip_caller((j_word)caller->return_address);
      
      setLocalException(thread, exception);
    }
    
    /*
    ** Retrieve the return value, if any.
    */
    
    if (method->return_i) {
      result = *(bottom->o_top - 1);
    }

    releaseBottomFrame(bottom);
  }
  
  return result;
}

int rt_invokestatic(w_method method, ...) {

  w_instance exception;
  w_thread thread;
  w_method caller;
  w_frame bottom;
  va_list ap;
  int result;
  int i;
    
  thread = (w_thread)thread_current->xref;
  caller = thread->top->native;

  result = 0;
  bottom = allocBottomFrame(thread, method);

  if (bottom) {
    
    /*
    ** Native methods are dealt with differently than Java methods:
    ** native methods pass their arguments using the stack, whereas
    ** Java methods pass their arguments using local variables.  We
    ** check whether a given method is a native method on each method
    ** invocation.  After all, we don't know whether Java methods are
    ** going to compiled to native code later, and if so, we don't 
    ** want to recompile any existing methods.  
    */
    
    if (isSet(method->flags, ACC_NATIVE)) { 
       woempa(9, "'%M' calls static native method '%M'\n", caller, method);
      
      va_start(ap, method);
      for (i = 0; i < (signed)method->arg_i; i++) {
        bottom->o_top[i] = (j_word)va_arg(ap, unsigned char *);
      }
      va_end(ap);

      bottom->o_top += method->arg_i;
    }
    else {
      woempa(9, "'%M' calls static Java method '%M'\n", caller, method);
     
      va_start(ap, method);
      for (i = 0; i < (signed)method->arg_i; i++) {
        bottom->locals[i] = (j_word)va_arg(ap, unsigned char *);
      }
      va_end(ap);

    }

    setFlag(bottom->flags, FRAME_JNI);
    exception = runThread(thread, bottom);
    
    /*
    ** If an exception is thrown that can not be handled by the compiled
    ** Java method, propagate it to the calling Java method.  If this is
    ** the case, we skip the remaining code of the compiled Java method
    ** by overwriting this function's return address ...
    */
    
    if (exception) {
      cs_skip_caller((j_word)caller->return_address);
      
      setLocalException(thread, exception);
    }
    
    /*
    ** Retrieve the return value, if any.
    */
    
    if (method->return_i) {
      result = *(bottom->o_top - 1);
    }

    releaseBottomFrame(bottom);
  }
 
  return result;
}

int rt_array_create(w_clazz clazz, int dimensions, ...) {
  
  w_instance instance;
  w_thread thread;
  int dimension[32];
  va_list ap;
  int i;
    
  woempa(9, "create a %d dimensional array of type '%k'\n", dimensions, clazz);
      
  thread = (w_thread)thread_current->xref;
 
  /*
  ** Retrieve the number of elements in each dimensions of the array
  ** and load them in memory.
  */
  
  va_start(ap, dimensions);
  for (i = 0; i < dimensions; i++) {
    dimension[i] = va_arg(ap, int);
  }
  va_end(ap);

  instance = allocArrayInstance(thread, clazz, dimensions, dimension);
  addLocalReference(thread, instance);
  
  return (int)instance;
}

int rt_array_load(w_instance array, int index, int type) {

  int value = 0;
  
  woempa(9, "load value at slot %d from array %p of type %d\n", index, array, type);
 
  switch (type) {
    case j_address:
      value = (int)instance2Array_instance(array)[index];
      break;
    case j_integer:
      value = (int)instance2Array_int(array)[index];
      break;
    case j_short:
      value = (int)instance2Array_short(array)[index];
      break;
    case j_byte:
      value = (int)instance2Array_byte(array)[index];
      break;
    case j_character:
      value = (int)instance2Array_char(array)[index];
      break;
    default:
      wabort(ABORT_WONKA, "array type %d not supported by intermediate representation\n", type);
  }
  
  return value;
}

int rt_array_store(w_instance array, int index, int value, int type) {

  woempa(9, "store value %d into slot %d of array %p of type %d\n", value, index, array, type);

  switch (type) {
    case j_address:
      setArrayReferenceField(array, (w_instance)value, index);
      break;
    case j_integer:
      instance2Array_int(array)[index] = (w_int)value;
      break;
    case j_short:
      instance2Array_short(array)[index] = (w_short)value;
      break;
    case j_byte:
      instance2Array_byte(array)[index] = (w_byte)value;
      break;
    case j_character:
      instance2Array_char(array)[index] = (w_char)value;
      break;
    default:
      wabort(ABORT_WONKA, "array type %d not supported by intermediate representation\n", type);
  }

  return 1;
}

int rt_array_length(w_instance array) {
  return array[F_Array_length];
}

