/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: stabs.c,v 1.2 2002/03/17 10:29:50 dbuytaert Exp $
*/

#include "j-spot.h"
#include "assembler.h"
#include "ir.h"
#include "strings.h"
#include "methods.h"

/*
** "stabs" is a format for information that describes a program to a 
** debugger: it is what compilers use to put debug information about 
** the program being debugged into the object files and executables.
**
** Since we don't emit object files, we generate the function symbols
** at runtime and write them to a file called "runtime-stabs.s".  You
** then need to compile this file with the stabs information using an 
** assembler:
**
**   $ as runtime-stabs.s -o runtime-stabs.o
**
** Now, you have to manually load the symbol file into gdb using:
**
**   (gdb) add-symbol-file runtime-stabs.o 0
**
** Once the debug information is loaded and gdb's symbol tables are
** being updated, gdb will be able to print informative backtraces.
*/

void stabs(j_method method) {
 
  static FILE *fd;
  
  if (fd == NULL) {
    fd = fopen("runtime-stabs.s", "w");
   
    fprintf(fd, ".stabs \"%s\", 100, 0, 0, 0\n", "compiler");
  }

  /*
  ** Announce the beginning of the specified method.
  */
  
  fprintf(fd, ".stabs \"%s:F\", 36, 0, 0, %p\n", method->name, method->code->base);

  /*
  ** Announce the end of the specified method.  The debugger considers
  ** the beginning of a new symbol the end of the current symbol so we
  ** insert a dummy (empty) symbol.
  */
  
  fprintf(fd, ".stabs \"\", 36, 0, 0, %d\n\n", method->code->step - method->code->base);
  
  fflush(fd);
}

