/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: statistics.c,v 1.3 2002/03/19 16:48:49 dbuytaert Exp $
*/

#include "stack.h"
#include "j-spot.h"
#include "methods.h"
#include "threads.h"

unsigned long max_operand_stack;
unsigned long tot_operand_stack;
unsigned long num_operand_stack;

unsigned long max_call_stack;
unsigned long tot_call_stack;
unsigned long num_call_stack;

unsigned long max_parameters;
unsigned long tot_parameters;
unsigned long num_parameters;

unsigned long max_exceptions;
unsigned long tot_exceptions;
unsigned long num_exceptions;

unsigned long max_bytecodes;
unsigned long tot_bytecodes;
unsigned long num_bytecodes;

unsigned long max_locals;
unsigned long tot_locals;
unsigned long num_locals;

unsigned long tot_returns;
unsigned long num_returns;

inline static void operand_stack(w_method method, w_thread thread, w_frame frame) {
  
  if (method) {
    if (method->stack_i > max_operand_stack) {
      max_operand_stack = method->stack_i;
    }
 
    tot_operand_stack += method->stack_i;
    num_operand_stack += 1;
  }
}

inline static void call_stack(w_method method, w_thread thread, w_frame frame) {
  
  if (thread && thread->top) {
    if (thread->top->depth > (signed)max_call_stack) {
      max_call_stack = thread->top->depth;
    }
 
    tot_call_stack += thread->top->depth;
    num_call_stack += 1;
  }
}

inline static void parameters(w_method method, w_thread thread, w_frame frame) {

  if (method) {
    if (method->nargs > max_parameters) {
      max_parameters = method->nargs;
    }

    tot_parameters += method->nargs;
    num_parameters += 1;
  }
}

inline static void exceptions(w_method method, w_thread thread, w_frame frame) {

  if (method) {
    if (method->numExceptions > (signed)max_exceptions) {
      max_exceptions = method->numExceptions;
    }

    tot_exceptions += method->numExceptions;
    num_exceptions += 1;
  }
}

inline static void returns(w_method method, w_thread thread, w_frame frame) {

  if (isNotSet(thread->flags, WT_THREAD_THROWN)) {
    tot_returns += 1;
  }

  num_returns += 1;
}

inline static void locals(w_method method, w_thread thread, w_frame frame) {

  if (method) {
    if (method->local_i > max_locals) {
      max_locals = method->local_i;
    }

    tot_locals += method->local_i;
    num_locals += 1;
  }
}

inline static void bytecodes(w_method method, w_thread thread, w_frame frame) {

  if (method) {
    if (method->code_length > max_bytecodes) {
      max_bytecodes = method->code_length;
    }

    tot_bytecodes += method->code_length;
    num_bytecodes += 1;
  }
}

void statistics_reset(void) {

  max_operand_stack = 0;
  tot_operand_stack = 0;
  num_operand_stack = 0;
  
  max_call_stack = 0;
  tot_call_stack = 0;
  num_call_stack = 0;
  
  max_parameters = 0;
  tot_parameters = 0;
  num_parameters = 0;
  
  max_exceptions = 0;
  tot_exceptions = 0;
  num_exceptions = 0;
  
  max_bytecodes = 0;
  tot_bytecodes = 0;
  num_bytecodes = 0;
  
  max_locals = 0;
  tot_locals = 0;
  num_locals = 0;

  tot_returns = 0;
  num_returns = 0;
}

void statistics_analyze(w_method method) {

  w_thread thread = currentWonkaThread;
  
  operand_stack(method, thread, thread->top);
  call_stack(method, thread, thread->top);
  parameters(method, thread, thread->top);
  exceptions(method, thread, thread->top);
  bytecodes(method, thread, thread->top);
  locals(method, thread, thread->top);
  returns(method, thread, thread->top);
}

void statistics_dump(void) {
  woempa(9, "-------------------------------------------------------------------------\n");
  woempa(9, "  operand stack depth  : %4d avg, %4d max (%d calls profiled)\n", tot_operand_stack / num_operand_stack, max_operand_stack, num_operand_stack);
  woempa(9, "  call stack depth     : %4d avg, %4d max (%d calls profiled)\n", tot_call_stack / num_call_stack, max_call_stack, num_call_stack);
  woempa(9, "  method parameters    : %4d avg, %4d max (%d calls profiled)\n", tot_parameters / num_parameters, max_parameters, num_parameters);
  woempa(9, "  local variables      : %4d avg, %4d max (%d calls profiled)\n", tot_locals / num_locals, max_locals, num_locals);
  woempa(9, "  exceptions           : %4d avg, %4d max (%d calls profield)\n", tot_exceptions / num_exceptions, max_exceptions, num_exceptions);
  woempa(9, "  bytecode size        : %4d avg, %4d max (%d calls profiled)\n", tot_bytecodes / num_bytecodes, max_bytecodes, num_bytecodes);
  // woempa(9, "  return vs exception  : %d normal returns, %d exceptions\n", tot_returns, (num_returns - tot_returns));
  woempa(9, "-------------------------------------------------------------------------\n");
}

