/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: translate.c,v 1.64 2002/03/26 11:05:14 dbuytaert Exp $
*/

#include "methods.h"

#include "j-spot.h"
#include "ir.h"
#include "control-flow.h"
#include "mimic-stack.h"
#include "ct-hooks.h"
#include "rt-hooks.h"
#include "bytecode.h"

void save_stack_context(j_method method, int target_pc) {

  ms_push((void *)target_pc);
  ms_flap();
  ms_pop();

}
      
int restore_stack_context(j_method method) {
  
  if (ms_next()) {
    int target_pc;
  
    ms_flop();
    target_pc = (int)ms_top();
    ms_pop();
  
    return target_pc;
  }
  else {
    return method->length;
  }
  
}

void spill_operands(j_method method) {
  
  int index = 0;
  int depth = 0;
 
  j_node node; 
  j_node node1;
  j_node node2;
  j_node node3;
 
  /*
  ** If (1) we encountered a join point and if (2) there are still operands
  ** on the mimic stack, we have to /spill/ them to temporary variables ...
  **
  **    |     |       |     |                 |           |
  **    +-----+       +-----+         assign "1" to tempory "t"
  **    |  1  |       |  0  |                 |           |
  **    +-----+       +-----+      ====>      |   assign "0" to tempory "t"
  **        |            |                    |           |
  **       jump          |                   jump         |
  **         \          /                      \         /   
  **          return "?"                       return "t"
  */
  
  // get the current node:
  node = ir_get_node(method, method->inst_pc);

  while (ms_get(depth)) {
      // TODO: we *often* spill too much: when there are function parameters 
      //       or identical instances/operands on the stack, they don't need
      //       to get spilled!  For a simple example, look at:
      //         com.acunia.wonka.test.jspot.ProblemTest.com_test_2().
      
    // calculate the index in the local variable table:
    index = method_num_locals(method) + depth;
        
    // get the node from the local variables:
    node1 = ms_get_local(method, index);
  
    if (node1 == NULL) {
      // create a new literal node:
      node1 = node_create_literal(method, j_temporary, j_address, index);
    }

    // add the literal to the pool of local variables:
    ms_set_local(method, node1, index);

    // pop the node from the operand stack:
    node2 = (j_node)ms_get(depth);

    // push the result back on the operand stack:
    ms_set(depth, node1);

    // create a new assignment instruction:
    node3 = node_create_insn(method, j_assign, node1, node2);
    
    // increment the stack depth:
    depth = depth + 1;
    
    /*
    ** Mark the node as a top-level node.  If the last node is a conditional
    ** or unconditional jump instruction, we want to insert the /spill code/
    ** before that instruction.  If it is any other instruction (an implicit 
    ** join point), we want to insert the spill code after that instruction.
    **
    **  |       instruction        |          |        instruction       |
    **  +--------------------------+          +--------------------------+
    **              |   <--- insert spills here            |
    **              v                                      v
    **  +--------------------------+          +--------------------------+
    **  |    branch instruction    |          |  non-branch instruction  |
    **  +--------------------------+          +--------------------------+
    **              |             insert spills here --->  |
    **              v                                      v
    **  +--------------------------+          +--------------------------+
    */

    if (ir_is_jump(node)) {
      ir_add_node(method, node3, j_link_next, j_link_none, j_insert_before);
    }
    else {
      ir_add_node(method, node3, j_link_next, j_link_none, j_insert_after);
    }
  }
}

INLINE void
load_wrapper(j_method method, j_type type, int index) {

  j_node node;
    
  /*
  ** Push or load the local variable 'number' of the current frame onto
  ** the operand stack.
  */

  // get the node from the local variables:
  node = ms_get_local(method, index);
  
  if (node == NULL) {
    // create a new literal node:
    if (index <= method_num_parameters(method)) {
      node = node_create_literal(method, j_parameter, type, index);
    }
    else {
      node = node_create_literal(method, j_temporary, type, index);
    }
    
    // add the literal to the pool of local variables:
    ms_set_local(method, node, index);
  }
  
  // push the node on the operand stack:
  ms_push(node);
  
  // debug output: 
  woempa(9, "load_wrapper: %q\n", node);
}

INLINE void
store_wrapper(j_method method, j_type type, int index) {

  j_node node1;
  j_node node2;
  j_node node3;
  
  /*
  ** Store the local variable at the top of the operand stack to local
  ** variable 'number'.
  */

  // get the node from the local variables:
  node1 = ms_get_local(method, index);
  
  if (node1 == NULL) {
    // create a new literal node:
    if (index <= method_num_parameters(method)) {
      node1 = node_create_literal(method, j_parameter, type, index);
    }
    else {
      node1 = node_create_literal(method, j_temporary, type, index);
    }

    // add the literal to the pool of local variables:
    ms_set_local(method, node1, index);
  }

  // pop the node from the operand stack:
  node2 = (j_node)ms_pop();

  // create a new node:
  node3 = node_create_insn(method, j_assign, node1, node2);

  // mark the node as a top-level node:
  ir_add_node(method, node3, j_link_next, j_link_none, j_insert_before);

  // debug output:
  woempa(9, "store_wrapper: %q\n", node3);
}

INLINE void
array_create_wrapper(j_method method, int dimensions, j_node type) {

  int i;
  j_node node;
    
  /*
  ** Allocate a new one or multi-dimensional array of the specified type.
  */

  // we create a function symbol that will call 'rt_array_create':
  node = node_create_call(method, j_address, &rt_array_create, 2 + dimensions);
  node_set_name(node, "array_create");
    
  for (i = 0; i < dimensions; i++) {
    // add the sizes of the dimensions of the the array:
    node_add_param(method, node, ms_pop());
  }
  
  // add the number of dimensions to the parameter list:
  node_add_param(method, node, node_create_literal(method, j_constant, j_integer, dimensions));

  // add the array type to the parameter list:
  node_add_param(method, node, type);

  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_next, j_link_none, j_insert_before);  
 
  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "array_create_wrapper: %q\n", node);
}

INLINE void
array_load_wrapper(j_method method, j_type type) {
  
  j_node node;
    
  /*
  ** Load a value from an array.
  */

  // we create a function symbol that will call 'rt_array_load':
  node = node_create_call(method, j_address, &rt_array_load, 3);
  node_set_name(node, "array_load");
  
  // add the type of the array:
  node_add_param(method, node, node_create_literal(method, j_constant, j_integer, (int)type));
  
  // add the index or location to store the value into:
  node_add_param(method, node, ms_pop());

  // add the reference that refers to the array:
  node_add_param(method, node, ms_pop());
 
  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_next, j_link_none, j_insert_before);  
 
  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "array_load_wrapper: %q\n", node);
}

INLINE void
array_store_wrapper(j_method method, j_type type) {
  
  j_node node;
    
  /*
  ** Store a value into an array.
  */

  // we create a function symbol that will call 'rt_array_store':
  node = node_create_call(method, j_address, &rt_array_store, 4);
  node_set_name(node, "array_store");
  
  // add the type of the array:
  node_add_param(method, node, node_create_literal(method, j_constant, j_integer, (int)type));
  
  // add the value to store:
  node_add_param(method, node, ms_pop());
  
  // add the index or location to store the value into:
  node_add_param(method, node, ms_pop());

  // add the reference that refers to the array:
  node_add_param(method, node, ms_pop());
 
  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_next, j_link_none, j_insert_before);  
 
  // debug output: 
  woempa(9, "array_store_wrapper: %q\n", node);
}

INLINE void
push_wrapper(j_method method, j_type type, int value) {

  j_node node;
    
  /*
  ** Push a constant onto the operand stack.
  */

  // create a new literal node:
  node = node_create_literal(method, j_constant, type, value);

  // push node on the mimic operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "push_wrapper: %q\n", node);
}

INLINE void
ldc_wrapper(j_method method, int value) {

  j_node node;
    
  /*
  ** Push an item from the constant pool.
  */

  // create a new literal node:
  node = ct_load_constant(method, value);
  
  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "ldc_wrapper: %q\n", node);
}

INLINE void
una_wrapper(j_method method, j_opcode opcode, j_type type) {

  j_node node1;
  j_node node2;
    
  // pop the node from the operand stack:
  node1 = (j_node)ms_pop();

  // create a new node:
  node2 = node_create_insn(method, opcode, node1, 0);
  
  // push node on the operand stack:
  ms_push(node2);

  // debug output:
  woempa(9, "una_wrapper: %q\n", node2);

}

INLINE void
bin_wrapper(j_method method, j_opcode opcode, j_type type) {

  j_node node1;
  j_node node2;
  j_node node3;
  
  // pop the nodes from the operand stack:
  node2 = (j_node)ms_pop();
  node1 = (j_node)ms_pop();

  // create a new node:
  node3 = node_create_insn(method, opcode, node1, node2);
  
  // push node on the operand stack:
  ms_push(node3);

  // debug output:
  woempa(9, "bin_wrapper: %q\n", node3);
}

INLINE void
shift_wrapper(j_method method, j_opcode opcode, j_type type) {

  j_node node1;
  j_node node2;
  j_node node3;
 
  // pop the nodes from the operand stack:
  node2 = (j_node)ms_pop();
  node1 = (j_node)ms_pop();

  // create a new node:
  node3 = node_create_insn(method, opcode, node1, node2);

  // push node on the operand stack:
  ms_push(node3);

  // debug output:
  woempa(9, "shift_wrapper: %q\n", node3);
}

INLINE void
x2y_wrapper(j_method method, j_type type1, j_type type2) {

  j_node node1;
  j_node node2;
    
  // pop the node from the operand stack:
  node1 = (j_node)ms_pop();

  // create a new node:
  node2 = node_create_insn(method, j_conv, node1, 0);
  
  // push node on the operand stack:
  ms_push(node2);

  // debug output:
  woempa(9, "x2y_wrapper: %q\n", node2);
}

INLINE void
cmp_wrapper(j_method method, j_type type, int number) {

  j_node node;
  
  // create a new symbol to save the local variable:
  node = node_create_call(method, type, &rt_cmp, 2);
  node_set_name(node, "compare");
  node_add_param(method, node, (j_node)ms_pop());
  node_add_param(method, node, (j_node)ms_pop());
  
  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_next, j_link_none, j_insert_before);

  ms_push(node);

  // debug output:
  woempa(9, "cmp_wrapper: %q\n", node);
}

INLINE void
ifx_wrapper(j_method method, j_opcode opcode, j_type type) {

  int offset;
  j_node node1;
  j_node node2;
  j_node node3;
  
  /*
  ** Pops two nodes from the operand stack for comparison.  If
  ** the comparision succeeds, an offset is calculated from the
  ** two bytes that follow the bytecode.
  */
  
  // get the branch offset:
  offset = bytecode_get_short(method);

  // pop the nodes from the operand stack:
  node1 = (j_node)ms_pop();
  node2 = (j_node)ms_pop();
  
  // create a new node:
  node3 = node_create_insn(method, opcode, node2, node1);

  // mark the node as a top-level node:
  ir_add_node(method, node3, method->inst_pc + offset, j_link_next, j_insert_before);

  // add the second branch to our internal 'todo'-list:
  save_stack_context(method, method->inst_pc + offset);

  // debug output: 
  woempa(9, "ifx_wrapper: %q\n", node3);
}

INLINE void
if0_wrapper(j_method method, j_opcode opcode, j_type type) {

  j_node node;
 
  /*
  ** The if0_wrapper compares the top of stack against zero and 
  ** branches if the comparison succeeds.
  */

  // create a new literal node:
  node = node_create_literal(method, j_constant, type, 0);

  // push node on the operand stack:
  ms_push(node);
 
  ifx_wrapper(method, opcode, type);

}

INLINE void
iinc_wrapper(j_method method, int value, int index) {

  j_node node1;
  j_node node2;
  j_node node3;
  j_node node4;

  /*
  ** Increment the local variable at the specified by "index" by the given
  ** value.
  */

  // get the local variable from the local variable pool:  
  node1 = ms_get_local(method, index);
  
  // create a new constant:
  node2 = node_create_literal(method, j_constant, j_integer, value);

  // create a small sub-tree:  
  node3 = node_create_insn(method, j_add, node1, node2);
  node4 = node_create_insn(method, j_assign, node1, node3); 
  
  // mark the node as a top-level node:
  ir_add_node(method, node4, j_link_next, j_link_none, j_insert_before);

  // debug output:
  woempa(9, "iinc_wrapper: %q\n", node4);
}

INLINE void
goto_wrapper(j_method method, int offset) {

  j_node node;
 
  // create a new node:
  node = node_create_insn(method, j_jump, 0, 0);

  // mark the node as a top-level node:
  ir_add_node(method, node, method->inst_pc + offset, j_link_none, j_insert_before);

  // jump to the target instruction:
  method->curr_pc = method->inst_pc + offset;

  // debug output:
  woempa(9, "goto: %q\n", node);
}

INLINE void
return_wrapper(j_method method, j_type type) {

  j_node node;

  // pop the node from the operand stack:
  node = (j_node)ms_pop();
  
  // create a new node:
  node = node_create_insn(method, j_return, node, 0);

  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_none, j_link_none, j_insert_before);
}

INLINE void 
translate_nop(j_method method) {
  // NOP-per-de-plopper-de-plop, do nothing, not even adding an edge
}

INLINE void 
translate_aconst_null(j_method method) {
  push_wrapper(method, j_address, 0);
}

INLINE void 
translate_iconst_m1(j_method method) {
  push_wrapper(method, j_integer, -1);
}

INLINE void 
translate_iconst_0(j_method method) {
  push_wrapper(method, j_integer, 0);
}

INLINE void 
translate_iconst_1(j_method method) {
  push_wrapper(method, j_integer, 1);
}

INLINE void 
translate_iconst_2(j_method method) {
  push_wrapper(method, j_integer, 2);
}

INLINE void 
translate_iconst_3(j_method method) {
  push_wrapper(method, j_integer, 3);
}

INLINE void 
translate_iconst_4(j_method method) {
  push_wrapper(method, j_integer, 4);
}

INLINE void 
translate_iconst_5(j_method method) {
  push_wrapper(method, j_integer, 5);
}

INLINE void
translate_lconst_0(j_method method) {
  push_wrapper(method, j_long, 0);
}

INLINE void
translate_lconst_1(j_method method) {
  push_wrapper(method, j_long, 1);
}

INLINE void
translate_fconst_0(j_method method) {
  push_wrapper(method, j_float, 0);
}

INLINE void
translate_fconst_1(j_method method) {
  push_wrapper(method, j_float, 1);
}

INLINE void
translate_fconst_2(j_method method) {
  push_wrapper(method, j_float, 2);
}

INLINE void
translate_dconst_0(j_method method) {
  push_wrapper(method, j_double, 0);
}

INLINE void
translate_dconst_1(j_method method) {
  push_wrapper(method, j_double, 1);
}

INLINE void
translate_bipush(j_method method) {
  push_wrapper(method, j_integer, (int)(char)bytecode_get_byte(method));
    // NOTE: we have to sign-extend the byte operand before casting it to a signed integer.
}

INLINE void
translate_sipush(j_method method) {
  push_wrapper(method, j_integer, (int)(short)bytecode_get_short(method));
    // NOTE: we have to sign-extend the short operand before casting it to a signed integer.
}

INLINE void
translate_ldc(j_method method) {
  ldc_wrapper(method, bytecode_get_byte(method));
}

INLINE void
translate_ldc_w(j_method method) {
  ldc_wrapper(method, bytecode_get_short(method));
}

INLINE void
translate_ldc2_w (j_method method) {
  ldc_wrapper(method, bytecode_get_short(method));
}

INLINE void
translate_iload(j_method method) {
  load_wrapper(method, j_integer, bytecode_get_byte(method));
}

INLINE void
translate_wide_iload(j_method method) {
  load_wrapper(method, j_integer, bytecode_get_short(method));
}

INLINE void
translate_lload(j_method method) {
  load_wrapper(method, j_long, bytecode_get_byte(method));
}

INLINE void
translate_wide_lload(j_method method) {
  load_wrapper(method, j_long, bytecode_get_short(method));
}

INLINE void
translate_fload(j_method method) {
  load_wrapper(method, j_float, bytecode_get_byte(method));
}

INLINE void
translate_wide_fload(j_method method) {
  load_wrapper(method, j_float, bytecode_get_short(method));
}

INLINE void
translate_dload(j_method method) {
  load_wrapper(method, j_double, bytecode_get_byte(method));  
}

INLINE void
translate_wide_dload(j_method method) {
  load_wrapper(method, j_double, bytecode_get_short(method));
}

INLINE void
translate_aload(j_method method) {
  load_wrapper(method, j_address, bytecode_get_byte(method));
}

INLINE void
translate_wide_aload(j_method method) {
  load_wrapper(method, j_address, bytecode_get_short(method));
}

INLINE void
translate_iload_0(j_method method) {
  load_wrapper(method, j_integer, 0);
}

INLINE void
translate_iload_1(j_method method) {
  load_wrapper(method, j_integer, 1);
}

INLINE void
translate_iload_2(j_method method) {
  load_wrapper(method, j_integer, 2);
}

INLINE void
translate_iload_3(j_method method) {
  load_wrapper(method, j_integer, 3);
}

INLINE void
translate_lload_0(j_method method) {
  load_wrapper(method, j_long, 0);
}

INLINE void
translate_lload_1(j_method method) {
  load_wrapper(method, j_long, 1);
}

INLINE void
translate_lload_2(j_method method) {
  load_wrapper(method, j_long, 2);
}

INLINE void
translate_lload_3(j_method method) {
  load_wrapper(method, j_long, 3);
}

INLINE void
translate_fload_0(j_method method) {
  load_wrapper(method, j_float, 0);
}

INLINE void
translate_fload_1(j_method method) {
  load_wrapper(method, j_float, 1);
}

INLINE void
translate_fload_2(j_method method) {
  load_wrapper(method, j_float, 2);
}

INLINE void
translate_fload_3(j_method method) {
  load_wrapper(method, j_float, 3);
}

INLINE void
translate_dload_0(j_method method) {
  load_wrapper(method, j_double, 0);
}

INLINE void
translate_dload_1(j_method method) {
  load_wrapper(method, j_double, 1);
}

INLINE void
translate_dload_2(j_method method) {
  load_wrapper(method, j_double, 2);
}

INLINE void
translate_dload_3(j_method method) {
  load_wrapper(method, j_double, 3);
}

INLINE void
translate_aload_0(j_method method) {
  load_wrapper(method, j_address, 0);
}

INLINE void
translate_aload_1(j_method method) {
  load_wrapper(method, j_address, 1);
}

INLINE void
translate_aload_2(j_method method) {
  load_wrapper(method, j_address, 2);
}

INLINE void
translate_aload_3(j_method method) {
  load_wrapper(method, j_address, 3);
}

INLINE void
translate_iaload(j_method method) {
  array_load_wrapper(method, j_integer);
}

INLINE void
translate_laload(j_method method) {
  array_load_wrapper(method, j_long);
}

INLINE void
translate_faload(j_method method) {
  array_load_wrapper(method, j_float);
}

INLINE void
translate_daload(j_method method) {
  array_load_wrapper(method, j_double);
}

INLINE void
translate_aaload(j_method method) {
  array_load_wrapper(method, j_address);
}

INLINE void
translate_baload(j_method method) {
  array_load_wrapper(method, j_boolean);
}

INLINE void
translate_caload(j_method method) {
  array_load_wrapper(method, j_character);
}

INLINE void
translate_saload(j_method method) {
  array_load_wrapper(method, j_short);
}

INLINE void
translate_istore(j_method method) {
  store_wrapper(method, j_integer, bytecode_get_byte(method));
}

INLINE void
translate_wide_istore(j_method method) {
  store_wrapper(method, j_integer, bytecode_get_short(method));
}

INLINE void
translate_lstore(j_method method) {
  store_wrapper(method, j_long, bytecode_get_byte(method));
}

INLINE void
translate_wide_lstore(j_method method) {
  store_wrapper(method, j_long, bytecode_get_short(method));
}

INLINE void
translate_fstore(j_method method) {
  store_wrapper(method, j_float, bytecode_get_byte(method));
}

INLINE void
translate_wide_fstore(j_method method) {
  store_wrapper(method, j_float, bytecode_get_short(method));
}

INLINE void
translate_dstore(j_method method) {
  store_wrapper(method, j_double, bytecode_get_byte(method));
}

INLINE void
translate_wide_dstore(j_method method) {
  store_wrapper(method, j_double, bytecode_get_short(method));
}

INLINE void
translate_astore(j_method method) {
  store_wrapper(method, j_address, bytecode_get_byte(method));
}

INLINE void
translate_wide_astore(j_method method) {
  store_wrapper(method, j_address, bytecode_get_short(method));
}

INLINE void
translate_istore_0(j_method method) {
  store_wrapper(method, j_integer, 0);
}

INLINE void
translate_istore_1(j_method method) {
  store_wrapper(method, j_integer, 1);
}

INLINE void
translate_istore_2(j_method method) {
  store_wrapper(method, j_integer, 2);
}

INLINE void
translate_istore_3(j_method method) {
  store_wrapper(method, j_integer, 3);
}

INLINE void
translate_lstore_0(j_method method) {
  store_wrapper(method, j_long, 0);
}

INLINE void
translate_lstore_1(j_method method) {
  store_wrapper(method, j_long, 1);
}

INLINE void
translate_lstore_2(j_method method) {
  store_wrapper(method, j_long, 2);
}

INLINE void
translate_lstore_3(j_method method) {
  store_wrapper(method, j_long, 3);
}

INLINE void
translate_fstore_0(j_method method) {
  store_wrapper(method, j_float, 0);
}

INLINE void
translate_fstore_1(j_method method) {
  store_wrapper(method, j_float, 1);
}

INLINE void
translate_fstore_2(j_method method) {
  store_wrapper(method, j_float, 2);
}

INLINE void
translate_fstore_3(j_method method) {
  store_wrapper(method, j_float, 3);
}

INLINE void
translate_dstore_0(j_method method) {
  store_wrapper(method, j_double, 0);
}

INLINE void
translate_dstore_1(j_method method) {
  store_wrapper(method, j_double, 1);
}

INLINE void
translate_dstore_2(j_method method) {
  store_wrapper(method, j_double, 2);
}

INLINE void
translate_dstore_3(j_method method) {
  store_wrapper(method, j_double, 3);
}

INLINE void
translate_astore_0(j_method method) {
  store_wrapper(method, j_address, 0);
}

INLINE void
translate_astore_1(j_method method) {
  store_wrapper(method, j_address, 1);
}

INLINE void
translate_astore_2(j_method method) {
  store_wrapper(method, j_address, 2);
}

INLINE void
translate_astore_3(j_method method) {
  store_wrapper(method, j_address, 3);
}

INLINE void
translate_iastore(j_method method) {
  array_store_wrapper(method, j_integer);
}

INLINE void
translate_lastore(j_method method) {
}

INLINE void
translate_fastore(j_method method) {
}
 
INLINE void
translate_dastore(j_method method) {
}

INLINE void
translate_aastore(j_method method) {
  array_store_wrapper(method, j_address);
}

INLINE void
translate_bastore(j_method method) {
  array_store_wrapper(method, j_byte);
}

INLINE void
translate_castore(j_method method) {
  array_store_wrapper(method, j_character);
}

INLINE void
translate_sastore(j_method method) {
  array_store_wrapper(method, j_short);
}

INLINE void
translate_pop(j_method method) {
  ms_pop();
}

INLINE void
translate_pop2(j_method method) {
  ms_pop();
  ms_pop();
}

INLINE void
translate_dup(j_method method) {
  ms_dup1();
}

INLINE void
translate_dup_x1(j_method method) {
  ms_dup1_x1();
}

INLINE void
translate_dup_x2(j_method method) {
  ms_dup1_x2();
}

INLINE void
translate_dup2(j_method method) {
  ms_dup1();
}

INLINE void
translate_dup2_x1(j_method method) {
  ms_dup1_x1();
}

INLINE void
translate_dup2_x2(j_method method) {
  ms_dup1_x2();
}

INLINE void
translate_swap(j_method method) {
  method->failed = 1;
  return;

  ms_swap();
}

INLINE void
translate_iadd(j_method method) {
  bin_wrapper(method, j_add, j_integer);
}

INLINE void
translate_ladd(j_method method) {
  bin_wrapper(method, j_add, j_long);
}

INLINE void
translate_fadd(j_method method) {
  bin_wrapper(method, j_add, j_float);
}

INLINE void
translate_dadd(j_method method) {
  bin_wrapper(method, j_add, j_double);
}

INLINE void
translate_isub(j_method method) {
  bin_wrapper(method, j_sub, j_integer);
}

INLINE void
translate_lsub(j_method method) {
  bin_wrapper(method, j_sub, j_long);
}

INLINE void
translate_fsub(j_method method) {
  bin_wrapper(method, j_sub, j_float);
}

INLINE void
translate_dsub(j_method method) {
  bin_wrapper(method, j_sub, j_double);
}

INLINE void
translate_imul(j_method method) {
  bin_wrapper(method, j_mul, j_integer);
}

INLINE void
translate_lmul(j_method method) {
  bin_wrapper(method, j_mul, j_long);
}

INLINE void
translate_fmul(j_method method) {
  bin_wrapper(method, j_mul, j_float);
}

INLINE void
translate_dmul(j_method method) {
  bin_wrapper(method, j_mul, j_double);
}

INLINE void
translate_idiv(j_method method) {
  bin_wrapper(method, j_div, j_integer);
}

INLINE void
translate_ldiv(j_method method) {
  bin_wrapper(method, j_div, j_long);
}

INLINE void
translate_fdiv(j_method method) {
  bin_wrapper(method, j_div, j_float);
}
   
INLINE void
translate_ddiv(j_method method) {
  bin_wrapper(method, j_div, j_double);
}

INLINE void
translate_irem(j_method method) {
  bin_wrapper(method, j_rem, j_integer);
}

INLINE void
translate_lrem(j_method method) {
  bin_wrapper(method, j_rem, j_long);
}

INLINE void
translate_frem(j_method method) {
  bin_wrapper(method, j_rem, j_float);
}

INLINE void
translate_drem(j_method method) {
  bin_wrapper(method, j_rem, j_double);
}

INLINE void
translate_ineg(j_method method) {
  una_wrapper(method, j_neg, j_integer);
}

INLINE void
translate_lneg(j_method method) {
  una_wrapper(method, j_neg, j_long);
}

INLINE void
translate_fneg(j_method method) {
  una_wrapper(method, j_neg, j_float);
}

INLINE void
translate_dneg(j_method method) {
  una_wrapper(method, j_neg, j_double);
}

INLINE void
translate_ishl(j_method method) {
  shift_wrapper(method, j_lsh, j_integer);
}

INLINE void
translate_lshl(j_method method) {
  shift_wrapper(method, j_lsh, j_long);
}

INLINE void
translate_ishr(j_method method) {
  shift_wrapper(method, j_rsh, j_integer);
}

INLINE void
translate_lshr(j_method method) {
  shift_wrapper(method, j_rsh, j_long);
}

INLINE void
translate_iushr(j_method method) {
  shift_wrapper(method, j_rsh, j_integer);
}

INLINE void
translate_lushr(j_method method) {
  shift_wrapper(method, j_rsh, j_long);
}

INLINE void
translate_iand(j_method method) {
  bin_wrapper(method, j_and, j_integer);
}

INLINE void
translate_land(j_method method) {
  bin_wrapper(method, j_and, j_long);
}

INLINE void
translate_ior(j_method method) {
  bin_wrapper(method, j_or, j_integer);
}

INLINE void
translate_lor(j_method method) {
  bin_wrapper(method, j_or, j_long);
}

INLINE void
translate_ixor(j_method method) {
  bin_wrapper(method, j_xor, j_integer);
}

INLINE void
translate_lxor(j_method method) {
  bin_wrapper(method, j_xor, j_long);
}

INLINE void
translate_iinc(j_method method) {
  iinc_wrapper(method, bytecode_get_byte(method), bytecode_get_byte(method));
}

INLINE void
translate_wide_iinc(j_method method) {
  iinc_wrapper(method, bytecode_get_short(method), bytecode_get_short(method));
}

INLINE void
translate_i2l(j_method method) {
  x2y_wrapper(method, j_integer, j_long);
}

INLINE void
translate_i2f(j_method method) {
  x2y_wrapper(method, j_integer, j_float);
}

INLINE void
translate_i2d(j_method method) {
  x2y_wrapper(method, j_integer, j_double);
}

INLINE void
translate_l2i(j_method method) {
  x2y_wrapper(method, j_long, j_integer);
}

INLINE void
translate_l2f(j_method method) {
  x2y_wrapper(method, j_long, j_float);
}

INLINE void
translate_l2d(j_method method) {
  x2y_wrapper(method, j_long, j_double);
}

INLINE void
translate_f2i(j_method method) {
  x2y_wrapper(method, j_float, j_integer);
}

INLINE void
translate_f2l(j_method method) {
  x2y_wrapper(method, j_float, j_long);
}

INLINE void
translate_f2d(j_method method) {
  x2y_wrapper(method, j_float, j_double);
}

INLINE void
translate_d2i(j_method method) {
  x2y_wrapper(method, j_double, j_integer);
}

INLINE void
translate_d2l(j_method method) {
  x2y_wrapper(method, j_double, j_long);
}
 
INLINE void
translate_d2f(j_method method) {
  x2y_wrapper(method, j_double, j_float);
}
 
INLINE void
translate_i2b(j_method method) {
}
 
INLINE void
translate_i2c(j_method method) {
}
 
INLINE void
translate_i2s(j_method method) {
}
 
INLINE void
translate_lcmp(j_method method) {
  cmp_wrapper(method, j_long, j_long);
}
 
INLINE void
translate_fcmpg(j_method method) {
  cmp_wrapper(method, j_float, j_float);
}

INLINE void
translate_fcmpl(j_method method) {
  cmp_wrapper(method, j_float, j_long);
}

INLINE void
translate_dcmpg(j_method method) {
  cmp_wrapper(method, j_double, j_double);
}

INLINE void
translate_dcmpl(j_method method) {
  cmp_wrapper(method, j_double, j_long);
}

INLINE void
translate_ifeq(j_method method) {
  if0_wrapper(method, j_eq, j_integer);
}

INLINE void
translate_ifne(j_method method) {
  if0_wrapper(method, j_ne, j_integer);
}

INLINE void
translate_iflt(j_method method) {
  if0_wrapper(method, j_lt, j_integer);
}

INLINE void
translate_ifge(j_method method) {
  if0_wrapper(method, j_ge, j_integer);
}

INLINE void
translate_ifgt(j_method method) {
  if0_wrapper(method, j_gt, j_integer);
}

INLINE void
translate_ifle(j_method method) {
  if0_wrapper(method, j_le, j_integer);
}

INLINE void
translate_if_icmpeq(j_method method) {
  ifx_wrapper(method, j_eq, j_integer);
}

INLINE void
translate_if_icmpne(j_method method) {
  ifx_wrapper(method, j_ne, j_integer);
}

INLINE void
translate_if_icmplt(j_method method) {
  ifx_wrapper(method, j_lt, j_integer);
}

INLINE void
translate_if_icmpge(j_method method) {
  ifx_wrapper(method, j_ge, j_integer);
}

INLINE void
translate_if_icmpgt(j_method method) {
  ifx_wrapper(method, j_gt, j_integer);
}

INLINE void
translate_if_icmple(j_method method) {
  ifx_wrapper(method, j_le, j_integer);
}

INLINE void
translate_if_acmpeq(j_method method) {
  ifx_wrapper(method, j_eq, j_address);
}

INLINE void
translate_if_acmpne(j_method method) {
  ifx_wrapper(method, j_ne, j_address);
}

INLINE void
translate_goto(j_method method) {
  goto_wrapper(method, bytecode_get_short(method));
}

INLINE void
translate_goto_w(j_method method) {
  goto_wrapper(method, bytecode_get_wide(method));
}

INLINE void
translate_jsr(j_method method) {
}

INLINE void
translate_jsr_w(j_method method) {
}

INLINE void
translate_ret(j_method method) {
}

INLINE void
translate_wide_ret(j_method method) {
}

INLINE void
translate_tableswitch(j_method method) {

  j_node node1;
  j_node node2;
  j_node node3;
  int offset;
  int def;
  int low;
  int hgh;
  int i;
  
  /*
  ** Pop the index from the stack.
  */
  
  node1 = ms_pop();
  
  /*
  ** Skip over the zero to three bytes of padding to get at the first
  ** word aligned address.
  */
          
  method->curr_pc = (method->inst_pc + 4) & -4;

  /*
  ** Retrieve the 32-bit default branch offset, the 32-bit low endpoint
  ** and the 32-bit high endpoint.
  */
  
  def = bytecode_get_wide(method);
  low = bytecode_get_wide(method);
  hgh = bytecode_get_wide(method);
 
  woempa(9, "default branch offset = %d, low endpoint = %d, high endpoint = %d\n", def, low, hgh);
  
  // create an new jump node: 
  node3 = node_create_insn(method, j_jump, 0, 0);
  
  // mark the node as a top-level node:
  ir_add_node(method, node3, method->inst_pc + def, j_link_none, j_insert_before);
  
  woempa(9, "setting pc to default branch at pc %d\n", method->inst_pc + def);

  /*
  ** Retrieve the different branch offsets: one branch offset for high,
  ** one branch offset for low, and one for each case value in between
  ** high and low.
  */
  
  for (i = 0; i < hgh - low + 1; i++) {
    // get the branch offset:
    offset = bytecode_get_wide(method);

    // create an new comparison node: 
    node2 = node_create_literal(method, j_constant, j_integer, low + i);
    node3 = node_create_insn(method, j_eq, node1, node2);

    // mark the node as a top-level node:
    ir_add_node(method, node3, method->inst_pc + offset, j_link_next, j_insert_before);
    
    // add the branch to our internal 'todo'-list:
    save_stack_context(method, method->inst_pc + offset);
    
    woempa(9, "- %d. branch with offset %d, target pc = %d\n", i, offset, method->inst_pc + offset);
  }

  // jump to the target instruction:
  method->curr_pc = method->inst_pc + def;
}

INLINE void
translate_lookupswitch(j_method method) {

  j_node node1;
  j_node node2;
  j_node node3;
  int branch;
  int offset;
  int def;
  int num;
  int i;
  
  /*
  ** Pop the index from the stack.
  */
  
  node1 = ms_pop();
  
  /*
  ** Skip over the zero to three bytes of padding to get at the first
  ** word aligned address.
  */
          
  method->curr_pc = (method->inst_pc + 4) & -4;

  /*
  ** Retrieve the 32-bit default branch offset, the 32-bit low endpoint
  ** and the 32-bit high endpoint.
  */
  
  def = bytecode_get_wide(method);
  num = bytecode_get_wide(method);
 
  woempa(9, "default branch offset = %d, number of branch/offset pairs = %d\n", def, num);
  
  // create an new jump node: 
  node3 = node_create_insn(method, j_jump, 0, 0);
  
  // mark the node as a top-level node:
  ir_add_node(method, node3, method->inst_pc + def, j_link_none, j_insert_before);
  
  woempa(9, "setting pc to default branch at pc %d\n", method->inst_pc + def);

  /*
  ** Retrieve the different branch offsets: one branch offset for high,
  ** one branch offset for low, and one for each case value in between
  ** high and low.
  */
  
  for (i = 0; i < num; i++) {
    // get the branch/offset pair:
    branch = bytecode_get_wide(method);
    offset = bytecode_get_wide(method);

    // create an new comparison node: 
    node2 = node_create_literal(method, j_constant, j_integer, branch);
    node3 = node_create_insn(method, j_eq, node1, node2);

    // mark the node as a top-level node:
    ir_add_node(method, node3, method->inst_pc + offset, j_link_next, j_insert_before);
    
    // add the branch to our internal 'todo'-list:
    save_stack_context(method, method->inst_pc + offset);
    
    woempa(9, "- %d. branch with offset %d, target pc = %d\n", i, offset, method->inst_pc + offset);
  }

  // jump to the target instruction:
  method->curr_pc = method->inst_pc + def;
}

INLINE void
translate_ireturn(j_method method) {
  return_wrapper(method, j_integer);
}

INLINE void
translate_freturn(j_method method) {
  return_wrapper(method, j_float);
}

INLINE void
translate_areturn(j_method method) {
  return_wrapper(method, j_address);
}

INLINE void
translate_lreturn(j_method method) {
  return_wrapper(method, j_long);
}

INLINE void
translate_dreturn(j_method method) {
  return_wrapper(method, j_double);
}

INLINE void
translate_vreturn(j_method method) {

  j_node node;

  // create a new node:
  node = node_create_insn(method, j_return, 0, 0);

  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_none, j_link_none, j_insert_before);

}

INLINE void
translate_ifnull(j_method method) {
  if0_wrapper(method, j_eq, j_address);
}

INLINE void
translate_ifnonnull(j_method method) {
  if0_wrapper(method, j_ne, j_address);
}

INLINE void
translate_monitorenter(j_method method) { 

  j_node node;
  
  // create a new node to save the local variable:
  node = node_create_call(method, j_address, &rt_monitorenter, 1);
  node_set_name(node, "monitorenter");
  node_add_param(method, node, (j_node)ms_pop());
  
  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_next, j_link_none, j_insert_before);

  // debug output:
  woempa(9, "monitorenter: %q\n", node);
}

INLINE void
translate_monitorexit(j_method method) {

  j_node node;
  
  // create a new node to save the local variable:
  node = node_create_call(method, j_address, &rt_monitorenter, 1);
  node_set_name(node, "monitorexit");
  node_add_param(method, node, (j_node)ms_pop());

  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_next, j_link_none, j_insert_before);
  
  // debug output:
  woempa(9, "monitorexit: %q\n", node);
}

INLINE void
translate_athrow(j_method method) {

  j_node node;

  method->failed = 1;
  return;

  // create a new symbol to save the local variable:
  node = node_create_call(method, j_address, &rt_athrow, 1);
  node_set_name(node, "athrow");  
  node_add_param(method, node, (j_node)ms_pop());

  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_next, j_link_none, j_insert_before);  
 
  // debug output:
  woempa(9, "athrow: %q\n", node);
}

INLINE void
translate_getstatic(j_method method) {

  j_node field;
  j_node node;
  
  /*
  ** Get a static field of a class.
  */

  // create a new literal node:
  field = ct_load_static(method, bytecode_get_short(method));

  // we create a function symbol that will call 'rt_getstatic':
  node = node_create_call(method, j_address, &rt_getstatic, 1);
  node_set_name(node, "getstatic");
  node_add_param(method, node, field);
 
  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_next, j_link_none, j_insert_before);

  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "getstatic: %q\n", node);
}

INLINE void
translate_putstatic(j_method method) {

  j_node field;
  j_node node;
  
  /*
  ** Set a field in an object.
  */
  
  field = ct_load_field(method, bytecode_get_short(method));

  // we create a function symbol that will call 'rt_putstatic':
  node = node_create_call(method, j_address, &rt_putstatic, 2);
  node_set_name(node, "putstatic");
  node_add_param(method, node, (j_node)ms_pop());
  node_add_param(method, node, field);

  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_next, j_link_none, j_insert_before);
 
  // debug output: 
  woempa(9, "putstatic: %q\n", node);
}

INLINE void
translate_getfield(j_method method) {

  j_node field;
  j_node node;
   
  /*
  ** Set a field in an object.
  */

  field = ct_load_field(method, bytecode_get_short(method));

  // we create a function symbol that will call 'rt_getfield':
  node = node_create_call(method, j_address, &rt_getfield, 2);
  node_set_name(node, "getfield");
  node_add_param(method, node, field);
  node_add_param(method, node, (j_node)ms_pop());
   
  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_next, j_link_none, j_insert_before);
 
  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "getfield: %q\n", node);
}

INLINE void
translate_putfield(j_method method) {

  j_node field;
  j_node node;
    
  /*
  ** Set a field in object.
  */

  field = ct_load_field(method, bytecode_get_short(method));
  
  // we create a function symbol that will call 'rt_putfield':
  node = node_create_call(method, j_address, &rt_putfield, 3);
  node_set_name(node, "putfield");
  node_add_param(method, node, (j_node)ms_pop());
  node_add_param(method, node, field);
  node_add_param(method, node, (j_node)ms_pop());

  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_next, j_link_none, j_insert_before);
 
  // debug output: 
  woempa(9, "putfield: %q\n", node);
}

INLINE void
translate_invokevirtual(j_method method) {

  j_node function;
  j_node node;
  int argc;
  int retc;
  int i;
  
  /*
  ** Invoke an instance method, dispatch based on the class.
  */
  
  // resolve the method based on a 2-byte index:
  function = ct_load_method(method, (int)bytecode_get_short(method), &argc, &retc);

  // we create a function node to call 'rt_invokevirtual':
  node = node_create_call(method, j_address, &rt_invokevirtual, argc + 2);
  node_set_name(node, "invokevirtual");

  // pop the arguments and the object reference from the stack:
  for (i = 0; i < argc + 1; i++) {
    node_add_param(method, node, (j_node)ms_pop());
  }
  
  // pass on the resolved method:
  node_add_param(method, node, function);
 
  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_next, j_link_none, j_insert_before);  
 
  // push the node on the operand stack:
  if (retc) {
    ms_push(node);
  }
  
  // debug output:
  woempa(9, "invokevirtual: %q\n", node);
}

INLINE void
translate_invokespecial(j_method method) {

  j_node function1;
  j_node function2;
  j_node node;
  int argc;
  int retc;
  int i;
  
  /*
  ** Invoke an instance method, dispatch based on the class.
  */
 
  // resolve the method based on a 2-byte index:
  function1 = ct_load_method(method, (int)bytecode_get_short(method), &argc, &retc);
 
  // we create a function node to call 'rt_invokespecial':
  node = node_create_call(method, j_address, &rt_invokespecial, argc + 3);
  node_set_name(node, "invokespecial");

  // pop the arguments and the object reference from the stack:
  for (i = 0; i < argc + 1; i++) {
    node_add_param(method, node, (j_node)ms_pop());
  }
  
  function2 = node_create_literal(method, j_constant, j_address, (int)method->method);

  // pass on the resolved methods:
  node_add_param(method, node, function1);
  node_add_param(method, node, function2);

  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_next, j_link_none, j_insert_before);  
 
  // push the node on the operand stack:
  if (retc) {
    ms_push(node);
  }
  
  // debug output:
  woempa(9, "invokespecial: %q\n", node);
}

INLINE void
translate_invokestatic(j_method method) {

  j_node function;
  j_node node;
  int argc;
  int retc;
  int i;

  /*
  ** Invoke a static class method.
  */
  
  // resolve the method based on a 2-byte index:
  function = ct_load_method(method, (int)bytecode_get_short(method), &argc, &retc);

  // we create a function node to call 'rt_invokestatic':
  node = node_create_call(method, j_address, &rt_invokestatic, argc + 1);
  node_set_name(node, "invokestatic");

  // pop the arguments from the stack:
  for (i = 0; i < argc; i++) {
    node_add_param(method, node, (j_node)ms_pop());
  }
    // NOTE: in case of "invokestatic" there is no object reference on 
    //       the stack; instead there is an object reference in the 
    //       constant pool of the current class.
  
  // pass on the resolved method:
  node_add_param(method, node, function);
 
  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_next, j_link_none, j_insert_before);  
 
  // push the node on the operand stack:
  if (retc) {
    ms_push(node);
  }
  
  // debug output: 
  woempa(9, "invokestatic: %q\n", node);
}

INLINE void
translate_invokeinterface(j_method method) {
}

INLINE void
translate_new(j_method method) {

  j_node field;
  j_node node;
    
  /*
  ** Push or load local the variable 'number' of the current frame onto
  ** the operand stack.
  */

  field = ct_load_class(method, bytecode_get_short(method));

  // we create a function symbol that will call 'rt_new':
  node = node_create_call(method, j_address, &rt_new, 1);
  node_set_name(node, "new");
  node_add_param(method, node, field);
 
  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_next, j_link_none, j_insert_before);  
 
  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "new: %q\n", node);
}

INLINE void
translate_newarray(j_method method) {
  array_create_wrapper(method, 1, ct_load_type(method, bytecode_get_byte(method))); 
}

INLINE void
translate_multianewarray(j_method method) {
  array_create_wrapper(method, bytecode_get_byte(method), ct_load_class(method, bytecode_get_short(method))); 
}

INLINE void
translate_anewarray(j_method method) {
  array_create_wrapper(method, 1, ct_load_array(method, bytecode_get_short(method))); 
}

INLINE void
translate_arraylength(j_method method) {
 
  j_node node;
    
  /*
  ** Get the length of an array.
  */

  // we create a function symbol that will call 'rt_new':
  node = node_create_call(method, j_address, &rt_array_length, 1);
  node_set_name(node, "array_length");
  node_add_param(method, node, ms_pop());
 
  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_next, j_link_none, j_insert_before);  
 
  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "arraylength: %q\n", node);
}

INLINE void
translate_checkcast(j_method method) {
  j_node node;
   
  /*
  ** Ensure that an instance is of the given type.
  */

  // we create a function symbol that will call 'rt_checkcast':
  node = node_create_call(method, j_address, &rt_checkcast, 2);
  node_set_name(node, "checkcast");
  node_add_param(method, node, ct_load_class(method, bytecode_get_short(method)));
  node_add_param(method, node, (j_node)ms_get(0));   
    // NOTE: the "checkcast" bytecode does not pop the object reference.
   
  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_next, j_link_none, j_insert_before);
 
  // debug output: 
  woempa(9, "checkcast: %q\n", node);
}

INLINE void
translate_instanceof(j_method method) {
  
  j_node node;
   
  /*
  ** Determine whether an instance is of a given type.
  */

  // we create a function symbol that will call 'rt_instanceof':
  node = node_create_call(method, j_address, &rt_instanceof, 2);
  node_set_name(node, "instanceof");
  node_add_param(method, node, ct_load_class(method, bytecode_get_short(method)));
  node_add_param(method, node, (j_node)ms_pop());
   
  // mark the node as a top-level node:
  ir_add_node(method, node, j_link_next, j_link_none, j_insert_before);
 
  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "instanceof: %q\n", node);
}

void translate(j_method method) {
 
  unsigned char code;
  
  woempa(9, "[begin of translate()]\n");
  
  /*
  ** Initialize our minic stack:
  */
 
  ms_init(method);
  method->curr_pc = 0;
  method->inst_pc = 0;

  while (method->failed == 0 && method->curr_pc < method->length) {

    if (method->bytecode[method->curr_pc] != 0xff) {
   
      code = method->bytecode[method->curr_pc++];
      
      j_flarp(white, "pc = %d, bytecode: %d (%s)\n", method->inst_pc, code, bytecode_get_name(code));

      switch (code) {
        case (nop):
          translate_nop(method);
          break;

        case (aconst_null):
          translate_aconst_null(method);
          break;

        case (iconst_m1):
          translate_iconst_m1(method);
          break;

        case (iconst_0):
          translate_iconst_0(method);
          break;

        case (iconst_1):
          translate_iconst_1(method);
          break;

        case (iconst_2):
          translate_iconst_2(method);
          break;

        case (iconst_3):
          translate_iconst_3(method);
          break;

        case (iconst_4):
          translate_iconst_4(method);
          break;

        case (iconst_5):
          translate_iconst_5(method);
          break;

        case (lconst_0):
          translate_lconst_0(method);
          break;

        case (lconst_1):
          translate_lconst_1(method);
          break;

        case (fconst_0):
          translate_fconst_0(method);
          break;

        case (fconst_1):
          translate_fconst_2(method);
          break;

        case (fconst_2):
          translate_fconst_2(method);
          break;

        case (dconst_0):
          translate_dconst_0(method);
          break;

        case (dconst_1):
          translate_dconst_1(method);
          break;

        case (bipush):
          translate_bipush(method);
          break;

        case (sipush):
          translate_sipush(method);
          break;

        case (ldc):
          translate_ldc(method);
          break;

        case (ldc_w):
          translate_ldc_w(method);
          break;

        case (ldc2_w):
          translate_ldc2_w(method);
          break;

        case (iload):
          translate_iload(method);
          break;

        case (lload):
          translate_lload(method);
          break;

        case (fload):
          translate_fload(method);
          break;

        case (dload):
          translate_dload(method);
          break;

        case (aload):
          translate_aload(method);
          break;

        case (iload_0):
          translate_iload_0(method);
          break;

        case (iload_1):
          translate_iload_1(method);
          break;

        case (iload_2):
          translate_iload_2(method);
          break;

        case (iload_3):
          translate_iload_3(method);
          break;

        case (lload_0):
          translate_lload_0(method);
          break;

        case (lload_1):
          translate_lload_1(method);
          break;

        case (lload_2):
          translate_lload_2(method);
          break;

        case (lload_3):
          translate_lload_3(method);
          break;

        case (fload_0):
          translate_fload_0(method);
          break;

        case (fload_1):
          translate_fload_1(method);
          break;

        case (fload_2):
          translate_fload_2(method);
          break;

        case (fload_3):
          translate_fload_3(method);
          break;

        case (dload_0):
          translate_dload_0(method);
          break;

        case (dload_1):
          translate_dload_1(method);
          break;

        case (dload_2):
          translate_dload_2(method);
          break;

        case (dload_3):
          translate_dload_3(method);
          break;

        case (aload_0):
          translate_aload_0(method);
          break;

        case (aload_1):
          translate_aload_1(method);
          break;

        case (aload_2):
          translate_aload_2(method);
          break;

        case (aload_3):
          translate_aload_3(method);
          break;

        case (iaload):
          translate_iaload(method);
          break;

        case (laload):
          translate_laload(method);
          break;

        case (faload):
          translate_faload(method);
          break;

        case (daload):
          translate_daload(method);
          break;

        case (aaload):
          translate_aaload(method);
          break;

        case (baload):
          translate_baload(method);
          break;

        case (caload):
          translate_caload(method);
          break;

        case (saload):
          translate_saload(method);
          break;

        case (istore):
          translate_istore(method);
          break;

        case (lstore):
          translate_lstore(method);
          break;

        case (fstore):
          translate_fstore(method);
          break;

        case (dstore):
          translate_dstore(method);
          break;

        case (astore):
          translate_astore(method);
          break;

        case (istore_0):
          translate_istore_0(method);
          break;

        case (istore_1):
          translate_istore_1(method);
          break;

        case (istore_2):
          translate_istore_2(method);
          break;

        case (istore_3):
          translate_istore_3(method);
          break;

        case (lstore_0):
          translate_lstore_0(method);
          break;

        case (lstore_1):
          translate_lstore_1(method);
          break;

        case (lstore_2):
          translate_lstore_2(method);
          break;

        case (lstore_3):
          translate_lstore_3(method);
          break;

        case (fstore_0):
          translate_fstore_0(method);
          break;

        case (fstore_1):
          translate_fstore_1(method);
          break;

        case (fstore_2):
          translate_fstore_2(method);
          break;

        case (fstore_3):
          translate_fstore_3(method);
          break;

        case (dstore_0):
          translate_dstore_0(method);
          break;

        case (dstore_1):
          translate_dstore_1(method);
          break;

        case (dstore_2):
          translate_dstore_2(method);
          break;

        case (dstore_3):
          translate_dstore_3(method);
          break;

        case (astore_0):
          translate_astore_0(method);
          break;

        case (astore_1):
          translate_astore_1(method);
          break;

        case (astore_2):
          translate_astore_2(method);
          break;

        case (astore_3):
          translate_astore_3(method);
          break;

        case (iastore):
          translate_iastore(method);
          break;

        case (lastore):
          translate_lastore(method);
          break;

        case (fastore):
          translate_fastore(method);
          break;

        case (dastore):
          translate_dastore(method);
          break;

        case (aastore):
          translate_aastore(method);
          break;

        case (bastore):
          translate_bastore(method);
          break;

        case (castore):
          translate_castore(method);
          break;

        case (sastore):
          translate_sastore(method);
          break;

        case (pop):
          translate_pop(method);
          break;

        case (pop2):
          translate_pop2(method);
          break;

        case (dup):
          translate_dup(method);
          break;

        case (dup_x1):
          translate_dup_x1(method);
          break;

        case (dup_x2):
          translate_dup_x2(method);
          break;

        case (dup2):
          translate_dup2(method);
          break;

        case (dup2_x1):        
          translate_dup2_x1(method);
          break;

        case (dup2_x2):
          translate_dup2_x2(method);
          break;

        case (swap):
          translate_swap(method);
          break;

        case (iadd):
          translate_iadd(method);
          break;

        case (ladd):
          translate_ladd(method);
          break;

        case (fadd):
          translate_fadd(method);
          break;

        case (dadd):
          translate_dadd(method);
          break;

        case (isub):
          translate_isub(method);
          break;

        case (lsub):
          translate_lsub(method);
          break;

        case (fsub):
          translate_fsub(method);
          break;

        case (dsub):
          translate_dsub(method);
          break;

        case (imul):
          translate_imul(method);
          break;

        case (lmul):
          translate_lmul(method);
          break;

        case (fmul):
          translate_fmul(method);
          break;

        case (dmul):
          translate_dmul(method);
          break;

        case (idiv):
          translate_idiv(method);
          break;

        case (ldiv):
          translate_ldiv(method);

        case (fdiv):
          translate_fdiv(method);
          break;

        case (ddiv):
          translate_ddiv(method);
          break;

        case (irem):
          translate_irem(method);
          break;

        case (lrem):
          translate_lrem(method);
          break;

        case (frem):
          translate_frem(method);
          break;

        case (drem):
          translate_drem(method);
          break;

        case (ineg):
          translate_ineg(method);
          break;

        case (lneg):
          translate_lneg(method);
          break;

        case (fneg):
          translate_fneg(method);
          break;

        case (dneg):
          translate_dneg(method);
          break;

        case (ishl):
          translate_ishl(method);
          break;

        case (lshl):
          translate_lshl(method);
          break;

        case (ishr):
          translate_ishr(method);
          break;

        case (lshr):
          translate_lshr(method);
          break;

        case (iushr):
          translate_iushr(method);
          break;

        case (lushr):
          translate_lushr(method);
          break;

        case (iand):
          translate_iand(method);
          break;

        case (land):
          translate_land(method);
          break;

        case (ior):
          translate_ior(method);
          break;

        case (lor):
          translate_lor(method);
          break;

        case (ixor):
          translate_ixor(method);
          break;

        case (lxor):
          translate_lxor(method);
          break;

        case (iinc):
          translate_iinc(method);
          break;

        case (i2l):
          translate_i2l(method);
          break;

        case (i2f):
          translate_i2f(method);
          break;

        case (i2d):
          translate_i2d(method);
          break;

        case (l2i):
          translate_l2i(method);
          break;

        case (l2f):
          translate_l2f(method);
          break;

        case (l2d):
          translate_l2d(method);
          break;

        case (f2i):
          translate_f2i(method);
          break;

        case (f2l):
          translate_f2l(method);
          break;

        case (f2d):
          translate_f2d(method);
          break;

        case (d2i):
          translate_d2i(method);
          break;

        case (d2l):
          translate_d2i(method);
          break;

        case (d2f):
          translate_d2f(method);
          break;

        case (i2b):
          translate_i2b(method);
          break;

        case (i2c):
          translate_i2c(method);
          break;

        case (i2s):
          translate_i2s(method);
          break;

        case (lcmp):
          translate_lcmp(method);
          break;

        case (fcmpl):
          translate_fcmpl(method);
          break;

        case (fcmpg):
          translate_fcmpg(method);
          break;

        case (dcmpl):
          translate_dcmpl(method);
          break;

        case (dcmpg):
          translate_dcmpg(method);
          break;

        case (ifeq):
          translate_ifeq(method);
          break;

        case (ifne):
          translate_ifne(method);
          break;

        case (iflt):
          translate_iflt(method);
          break;

        case (ifge):
          translate_ifge(method);
          break;

        case (ifgt):
          translate_ifgt(method);
          break;

        case (ifle):
          translate_ifle(method);
          break;

        case (if_icmpeq):        
          translate_if_icmpeq(method);
          break;

        case (if_icmpne):
          translate_if_icmpne(method);
          break;

        case (if_icmplt):
          translate_if_icmplt(method);
          break;

        case (if_icmpge):
          translate_if_icmpge(method);
          break;

        case (if_icmpgt):
          translate_if_icmpgt(method);
          break;

        case (if_icmple):
          translate_if_icmple(method);
          break;

        case (if_acmpeq):        
          translate_if_acmpeq(method);
          break;

        case (if_acmpne):
          translate_if_acmpne(method);
          break;

        case (goto_n):
          translate_goto(method);
          break;

        case (goto_w):
          translate_goto_w(method);
          break;

        case (jsr):
          translate_jsr(method);
          break;

        case (jsr_w):
          translate_jsr_w(method);
          break;

        case (ret):
          translate_ret(method);
          break;

        case (tableswitch): 
          translate_tableswitch(method);
          break;

        case (lookupswitch):
          translate_lookupswitch(method);
          break;

        case (ireturn):
          translate_ireturn(method);
          break;

        case (freturn):
          translate_freturn(method);
          break;

        case (areturn):
          translate_areturn(method);
          break;

        case (lreturn):
          translate_lreturn(method);
          break;

        case (dreturn):
          translate_dreturn(method);
          break;

        case (vreturn):
          translate_vreturn(method);
          break;

        case (getstatic):
          translate_getstatic(method);
          break;

        case (putstatic):
          translate_putstatic(method);
          break;

        case (getfield):
          translate_getfield(method);
          break;

        case (putfield):
          translate_putfield(method);
          break;

        case (invokevirtual):
          translate_invokevirtual(method);
          break;

        case (invokespecial):
          translate_invokespecial(method);
          break;

        case (invokestatic):
          translate_invokestatic(method);
          break;

        case (invokeinterface):
          translate_invokeinterface(method);
          break;

        case (new):
          translate_new(method);
          break;

        case (newarray):
          translate_newarray(method);
          break;

        case (anewarray):
          translate_anewarray(method);
          break;

        case (arraylength):
          translate_arraylength(method);
          break;

        case (athrow):
          translate_athrow(method);
          break;

        case (checkcast):
          translate_checkcast(method);
          break;

        case (instanceof):
          translate_instanceof(method);
          break;

        case (monitorenter):
          translate_monitorenter(method);
          break;

        case (monitorexit):
          translate_monitorexit(method);
          break;

        case (multianewarray):
          translate_multianewarray(method);
          break;

        case (ifnull):
          translate_ifnull(method);
          break;

        case (ifnonnull):
          translate_ifnonnull(method);
          break;

        case (wide):
          code = method->bytecode[method->curr_pc++];

          switch (code) {
            case (iload):
              translate_wide_iload(method);
              break;

            case (lload):
              translate_wide_lload(method);
              break;

            case (fload):
              translate_wide_fload(method);
              break;

            case (dload):
              translate_wide_dload(method);
              break;

            case (aload):
              translate_wide_aload(method);
              break;

            case (istore):
              translate_wide_istore(method);
              break;

            case (lstore):
              translate_wide_lstore(method);
              break;

            case (fstore):
              translate_wide_fstore(method);
              break;

            case (dstore):
              translate_wide_dstore(method);
              break;

            case (astore):
              translate_wide_astore(method);
              break;

            case (iinc):
              translate_wide_iinc(method);
              break;

            case (ret):
              translate_wide_ret(method);
              break;

            case (breakpoint):
              wabort(ABORT_WONKA, "ERROR: opcode `breakpoint' not implemented yet!\n");

            default:
              wabort(ABORT_WONKA, "ERROR: encounterd unknown opcode `%d'.\n", code);
          }

        case (breakpoint):
          wabort(ABORT_WONKA, "opcode `breakpoint' not implemented yet!\n");

        default:
          wabort(ABORT_WONKA, "encounterd unknown opcode `%d'.\n", code);
      }    

      if (method->curr_pc < method->length) {
        if (method->joins[method->curr_pc]) {
          woempa(9, "detected a join point at pc %d: we might have to spill the remaining stack operands ...\n", method->curr_pc);
          spill_operands(method);
        }
      }
      
      /*
      ** Indicate that we have already been here:
      */
        
      method->bytecode[method->inst_pc] = 0xff;
    }
    else {
      code = 0x00;
    }
    
    /*
    ** Check if there is anything left to be done.  The order of the if
    ** clauses is important.
    */

    if (method->curr_pc >= method->length) {
      method->curr_pc = restore_stack_context(method);
      woempa(9, "hey, we are at the end of our method - did we traversed all paths ... ? (old pc: %d, new pc: %d)\n", method->inst_pc, method->curr_pc);
    }
    else if (code >= ireturn && code <= vreturn) {
      method->curr_pc = restore_stack_context(method);
      woempa(9, "hey, we are at a return statement - did we traversed all paths ... ? (old pc: %d, new pc: %d)\n", method->inst_pc, method->curr_pc);
    }

    if (method->bytecode[method->curr_pc] == 0xff) {
      method->curr_pc = restore_stack_context(method);
      woempa(9, "hey, we have already been here - did we traversed all paths ... ? (old pc: %d, new pc: %d)\n", method->inst_pc, method->curr_pc);
    }
    
    method->inst_pc = method->curr_pc;
  }
 
  woempa(9, "[end of translate()]\n");
}

