#include <unistd.h>
#include <stdio.h>

#include "tests.h"
#include "../hal/cpu/arm/include/is.h"
#include "../hal/cpu/x86/include/is.h"

#include "assembler.h"

int tests;

#define test(code, l, r)             \
    tests++;                         \
    if (assembler_cmp(code, l, r) != 0) {    \
      fprintf(stderr, "- '%s == %s' invalid:\n     0x%x\t: %s\n     %s\t: %s\n", #l, #r, assembler_cmp(code, l, r), asm_insn(assembler_cmp(code, l, r)), #r, asm_insn(r)); \
    }    

j_word assembler_cmp(j_asm code, int size, j_word right) {

  j_word left;
  
  size = size < 4 ? size : 4;
  left = *(j_word *)(code->step - size);
 
  fprintf(stderr, "left = %p, right = %p, size = %d\n", left, right, size);
  
  return (left != right) ? left : 0;
}

void assembler_test(void) {  
  
  j_asm code = asm_create_block();

  /*
  ** Code generator test:
  */

  test(code, asm_emit(code, 1, 32, 8, 0xaa, 8, 0xbb, 8, 0xcc, 8, 0xdd), 0xaabbccdd);
  test(code, asm_emit(code, 0, 32, 8, 0xaa, 8, 0xbb, 8, 0xcc, 8, 0xdd), 0xddccbbaa);

  /*
  ** ARM instruction set:
  */ 

  test(code, asm_emit(code, arm_mov_rr(R0, R1)), 0xe1a00001);
  test(code, asm_emit(code, arm_mov_rr(R4, R3)), 0xe1a04003);
  test(code, asm_emit(code, arm_mov_rr(SP, R1)), 0xe1a0d001);
  test(code, asm_emit(code, arm_mov_rr(R1, LR)), 0xe1a0100e);
  test(code, asm_emit(code, arm_mov_rr(PC, LR)), 0xe1a0f00e);
  test(code, asm_emit(code, arm_mov_rr(LR, PC)), 0xe1a0e00f);
  test(code, asm_emit(code, arm_mov_ri(R0, 50)), 0xe3a00032);
  test(code, asm_emit(code, arm_mov_ri(R1, 20)), 0xe3a01014);
  test(code, asm_emit(code, arm_mov_ri(R2, 10)), 0xe3a0200a);
  test(code, asm_emit(code, arm_mov_ri(R3,  1)), 0xe3a03001);

  test(code, asm_emit(code, arm_add_rr(R1, R1, R2)), 0xe0811002);
  test(code, asm_emit(code, arm_add_rr(R2, R2, R1)), 0xe0822001);
  test(code, asm_emit(code, arm_add_rr(R3, R3, R2)), 0xe0833002);
  test(code, asm_emit(code, arm_add_ri(R1, R1,  4)), 0xe2811004);
  test(code, asm_emit(code, arm_add_ri(R2, R2,  4)), 0xe2822004);

  test(code, asm_emit(code, arm_sub_rr(R1, R1, R2)), 0xe0411002);
  test(code, asm_emit(code, arm_sub_ri(R1, R3,  4)), 0xe2431004);     
  test(code, asm_emit(code, arm_sub_ri(R5, R8,  8)), 0xe2485008);
  test(code, asm_emit(code, arm_sub_ri(SP, SP,  8)), 0xe24dd008);

  test(code, asm_emit(code, arm_stmdb(SP, BIT_R2 | BIT_R3)), 0xe92d000c);
  test(code, asm_emit(code, arm_stmdb(SP, BIT_R11 | BIT_R12 | BIT_LR | BIT_PC)), 0xe92dd800);
  test(code, asm_emit(code, arm_stmdb(SP, BIT_R4 | BIT_R5 | BIT_R11 | BIT_R12 | BIT_LR | BIT_PC)), 0xe92dd830);

  test(code, asm_emit(code, arm_ldmdb(R11, BIT_R4 | BIT_R5 | BIT_R11 | BIT_SP | BIT_PC)), 0xe91ba830);
  test(code, asm_emit(code, arm_ldmdb(R11, BIT_R11 | BIT_SP | BIT_PC)), 0xe91ba800);

  test(code, asm_emit(code, arm_str_i(R1, SP, 8)), 0xe58d1008);
  test(code, asm_emit(code, arm_str_i(R3, SP, 12)), 0xe58d300c);
  test(code, asm_emit(code, arm_str_i(R4, SP, 4)), 0xe58d4004);
  test(code, asm_emit(code, arm_str_i(R0, R11, -16)), 0xe50b0010);
  test(code, asm_emit(code, arm_str_i(R2, R11, -28)), 0xe50b201c);
  test(code, asm_emit(code, arm_str_i(R3, R11, -20)), 0xe50b3014);

  test(code, asm_emit(code, arm_str_r(R1, SP)), 0xe58d1000);
  test(code, asm_emit(code, arm_str_r(R1, R2)), 0xe5821000);
  test(code, asm_emit(code, arm_str_r(R2, R3)), 0xe5832000);

  /*
  ** x86 instruction set:
  */

  test(code, asm_emit(code, x86_pop(EBX)), 0x5b);
  test(code, asm_emit(code, x86_pop(EDI)), 0x5f);
  test(code, asm_emit(code, x86_pop(ESI)), 0x5e);
  
  test(code, asm_emit(code, x86_mov_rr(EAX, EDX)), 0xd089);
  test(code, asm_emit(code, x86_mov_rr(ESI, ESI)), 0xf689);
  test(code, asm_emit(code, x86_mov_rr(EBP, ESP)), 0xe589);

  test(code, asm_emit(code, x86_mov_rm(EDX, EBP, + 8)), 0x08558b);
  test(code, asm_emit(code, x86_mov_rm(EBX, EBP, - 4)), 0xfc5d8b);
  test(code, asm_emit(code, x86_mov_rm(ECX, EBP, + 12)), 0x0c4d8b);
  test(code, asm_emit(code, x86_mov_rm(EDX, ECX, + 4)), 0x04518b);
  
  test(code, asm_emit(code, x86_mov_mr(EBP, 0x8, EAX)), 0x084589);
  test(code, asm_emit(code, x86_mov_mr(EBP, 0x4, EBX)), 0x045d89);
  test(code, asm_emit(code, x86_mov_mr(EAX, 0x4, EBX)), 0x045889);
  test(code, asm_emit(code, x86_mov_mr(EAX, 0x4, EAX)), 0x044089);

  test(code, asm_emit(code, x86_add_rr(EAX, EDX)), 0xc201);
  test(code, asm_emit(code, x86_add_rr(EDX, EAX)), 0xd001);
  test(code, asm_emit(code, x86_add_rr(EBX, EDX)), 0xda01);
  test(code, asm_emit(code, x86_add_rr(EBX, EAX)), 0xd801);
  test(code, asm_emit(code, x86_add_rr(EDI, EAX)), 0xf801);
  test(code, asm_emit(code, x86_add_rr(EDI, EBX)), 0xfb01);
  
  // TODO: our test framework is not capable of testing those
  // test(code, asm_emit(code, x86_add_ri(EBX, 0x04)), 0x04c383);
  // test(code, asm_emit(code, x86_add_ri(ESP, 0x08)), 0x08c483);
  // test(code, asm_emit(code, x86_add_ri(ESP, 0x0c)), 0x0cc483);
  // test(code, asm_emit(code, x86_add_ri(ECX, 0x0ffffffd0)), 0xd0c183);
  // test(code, asm_emit(code, x86_add_ri(EBX, 0xb4f3)), 0x00b4f3c383);

  test(code, asm_emit(code, x86_sub_rr(EAX, EDX)), 0xc229);
  test(code, asm_emit(code, x86_sub_rr(EAX, ESI)), 0xc629);
  test(code, asm_emit(code, x86_sub_rr(EBX, EDX)), 0xda29);
  test(code, asm_emit(code, x86_sub_rr(ECX, EAX)), 0xc829);
  test(code, asm_emit(code, x86_sub_rr(EDX, EAX)), 0xd029);
  test(code, asm_emit(code, x86_sub_rr(EDX, ECX)), 0xd129);
 
  test(code, asm_emit(code, x86_sub_ri(ESP, 0x4)), 0x04ec83);
  test(code, asm_emit(code, x86_sub_ri(ESP, 0x8)), 0x08ec83);
  test(code, asm_emit(code, x86_sub_ri(EBX, 0x1)), 0x01eb83);
  test(code, asm_emit(code, x86_sub_ri(EAX, 0x1)), 0x01e883);
  
  test(code, asm_emit(code, x86_mul_rr(EDX, EAX)), 0xc2af0f);
  test(code, asm_emit(code, x86_mul_rr(EDI, ECX)), 0xcfaf0f);
  test(code, asm_emit(code, x86_mul_rr(EBX, ECX)), 0xcbaf0f);
  test(code, asm_emit(code, x86_mul_rr(ESI, EDX)), 0xd6af0f);
  test(code, asm_emit(code, x86_mul_rr(EDX, EAX)), 0xc2af0f);
  test(code, asm_emit(code, x86_mul_rr(ECX, EDX)), 0xd1af0f);
 
  test(code, asm_emit(code, x86_neg_rr(EDI)), 0xdff7);
  test(code, asm_emit(code, x86_neg_rr(EBX)), 0xdbf7);
  test(code, asm_emit(code, x86_neg_rr(ESI)), 0xdef7);
  test(code, asm_emit(code, x86_neg_rr(EAX)), 0xd8f7);
  test(code, asm_emit(code, x86_neg_rr(ECX)), 0xd9f7);
  test(code, asm_emit(code, x86_neg_rr(EDX)), 0xdaf7);
 
  test(code, asm_emit(code, x86_lea(ESI, 0x0, ESI)), 0x00768d);
  test(code, asm_emit(code, x86_lea(EDX, 0x4, EAX)), 0x04508d);
  test(code, asm_emit(code, x86_lea(EAX, 0x8, EBX)), 0x08438d);
  test(code, asm_emit(code, x86_lea(EBX, 0xffffffe0, ESI)), 0xe05e8d);
  test(code, asm_emit(code, x86_lea(ESP, 0xfffffff4, EBP)), 0xf4658d);
  test(code, asm_emit(code, x86_lea(ECX, 0xffffffff, EBP)), 0xff4d8d);

  test(code, asm_emit(code, x86_gt_8(0xf2)), 0xf27f);
  test(code, asm_emit(code, x86_gt_8(0xf2)), 0xf27f);
  
  test(code, asm_emit(code, x86_ge_8(0x23)), 0x237d);
  test(code, asm_emit(code, x86_ge_8(0xf2)), 0xf27d);
  
  test(code, asm_emit(code, x86_lt_8(0x23)), 0x237c);
  test(code, asm_emit(code, x86_lt_8(0xf2)), 0xf27c);
  
  test(code, asm_emit(code, x86_le_8(0xf2)), 0xf27e);
  test(code, asm_emit(code, x86_le_8(0x23)), 0x237e);
  
  test(code, asm_emit(code, x86_eq_8(0xf2)), 0xf274);
  test(code, asm_emit(code, x86_eq_8(0x23)), 0x2374);
  
  test(code, asm_emit(code, x86_ne_8(0xf2)), 0xf275);
  test(code, asm_emit(code, x86_ne_8(0x23)), 0x2375);
 
   // NOTE: the conditional jump with full displacement (32-bit memory
   //       address) can't be tested due to some limitions of the test
   //       framework.
  
  test(code, asm_emit(code, x86_jmp_8(0x10)), 0x10eb);
  test(code, asm_emit(code, x86_jmp_8(0xef)), 0xefeb);
 
   // NOTE: the unconditional jump with full displacement (32-bit memory
   //       address) can't be tested due to some limitions of the test
   //       framework.
  
  test(code, asm_emit(code, x86_cmp_rr(EDX, EAX)), 0xd039);
  test(code, asm_emit(code, x86_cmp_rr(EBX, EAX)), 0xd839);
  test(code, asm_emit(code, x86_cmp_rr(EDI, EBX)), 0xfb39);
  test(code, asm_emit(code, x86_cmp_rr(ESI, EBX)), 0xf339);

  test(code, asm_emit(code, x86_cmp_rm(ESI, 0x14, EBP)), 0x14753b);
  test(code, asm_emit(code, x86_cmp_rm(EBX, 0x18, ESI)), 0x185e3b);
  test(code, asm_emit(code, x86_cmp_rm(ESI, 0x0c, EBP)), 0x0c753b);
 
  test(code, asm_emit(code, x86_cmp_mr(0x0c, ECX, EAX)), 0x0c4139);
  test(code, asm_emit(code, x86_cmp_mr(0xe4, EBP, EDI)), 0xe47d39);
  test(code, asm_emit(code, x86_cmp_mr(0xb0, EBP, ECX)), 0xb04d39);

  // test(code, asm_emit(code, x86_cmp_ir(EAX, 0xfd42)), 0xfd42053b);
  // test(code, asm_emit(code, x86_cmp_ir(EDX, 0xfcf8)), 0xfcf8153b);
  // test(code, asm_emit(code, x86_cmp_ir(EDX, 0x1234)), 0x12341d3b);
   
    // NOTE: can not be tested yet due to some limitations of our test
    //       framework.
  
  test(code, asm_emit(code, x86_and_rr(EAX, ESI)), 0xc621);
  test(code, asm_emit(code, x86_and_rr(EAX, EBX)), 0xc321);
  test(code, asm_emit(code, x86_and_rr(ESI, EAX)), 0xf021);
  
  test(code, asm_emit(code, x86_or_rr(ECX, EDX)), 0xca09);
  test(code, asm_emit(code, x86_or_rr(EBX, EAX)), 0xd809);
  test(code, asm_emit(code, x86_or_rr(EAX, EBX)), 0xc309);
 
  test(code, asm_emit(code, x86_xor_rr(EDX, EDX)), 0xd231);
  test(code, asm_emit(code, x86_xor_rr(EBX, EBX)), 0xdb31);
  test(code, asm_emit(code, x86_xor_rr(EAX, EAX)), 0xc031);
 
  test(code, asm_emit(code, x86_shr_cl(EAX)), 0xe8d3);
  test(code, asm_emit(code, x86_shr_cl(ESI)), 0xeed3);
  test(code, asm_emit(code, x86_shr_cl(EDI)), 0xefd3);
  
  test(code, asm_emit(code, x86_shl_cl(EAX)), 0xe0d3);
  test(code, asm_emit(code, x86_shl_cl(ESI)), 0xe6d3);
  test(code, asm_emit(code, x86_shl_cl(EDX)), 0xe2d3);
  
  /*
  ** Debug output:
  */        

  fprintf(stderr, "+----------------------------------+\n");
  fprintf(stderr, "|    Verified %3d instructions.    |\n", tests);
  fprintf(stderr, "+----------------------------------+\n");

}

