/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

#include "oswald.h"  /* x_thread_... */
#include "wonka.h"   /* the w_ types */
#include "ts-mem.h"  /* allocMem_system */
#include "realms.h"   /* allocMem_system */
#include "jdwp.h"
#include "heap.h"    /* instance2clazz */
#include "fields.h"   /* w_field */
#include "clazz.h"   /* w_clazz */
#include "descriptor.h"  /* w_desc */
#include "checks.h"      /* isAssignmentCompatible */
#include "core-classes.h"

/*
** This function should move to somewhere else...
*/

static w_clazz desc2Class(w_instance classLoader, w_desc desc) {
  w_instance  ancestor = classLoader ? classLoader : systemClassLoader;
  w_hashtable class_hashtable;
  w_clazz current = NULL;

  while(1) {
    class_hashtable = loader2hashtable(ancestor);
    current = seekClazzByDesc(desc, class_hashtable);
    if (current || !ancestor) {
      break;
    }
    ancestor = (w_instance)ancestor[F_ClassLoader_parent];
  }

  if(!current) {
    woempa(9, "WARNING: Return null... This is not a good sign, be prepared to catch the core I will throw at you real soon now...\n");
  }

  return current;
}


/*
** Get the reference type of the given instance.
*/

w_void jdwp_object_type(jdwp_command_packet cmd) {
  w_int   objectID;
  w_int   reftypeID;
  w_ubyte result[5];

  /*
  ** Get the objectID from the command packet.
  */
  
  memcpy(&objectID, &cmd->data[0], 4);

  woempa(9, " Object ID: %8x\n", objectID);

  /*
  ** Get the reference type (clazz) of the instance and put it in the reply.
  */

  reftypeID = (w_int)instance2clazz((w_instance)objectID);
  memcpy(&result[1], &reftypeID, 4);

  /* TODO: Not everything is a class... Check if it's an array and fill in the 
           following field accordingly. It might even be a primitive type... */
           
  result[0] = jdwp_tt_class;
  
  woempa(9, " Reference ID: %8x\n", reftypeID);

  /*
  ** Send the reply to the debugger.
  */

  jdwp_send_reply(cmd->id, &result, 5, jdwp_err_none);
  
  return;
}


/*
** Get the values of the given fields of the given instance. The fields should be
** members of the given instance or one of it's superclasses, superinterfaces or
** implemented interfaces. There is no access control, so private fields can be
** obtained aswell.
*/

w_void jdwp_object_get_values(jdwp_command_packet cmd) {
  w_instance   instance;
  w_clazz      clazz;
  w_field      field;
  w_int        length;
  w_int        pos1 = 0;
  w_int        pos2 = 0;
  w_int        i;
  w_ubyte      *reply;
  w_ubyte      abyte;
  w_int        anint;
  w_short      ashort;
  w_long       along;

  /*
  ** Get the instance from the command packet.
  */
  
  memcpy(&instance, &cmd->data[pos1], 4);
  pos1 += 4;

  /*
  ** Get the number of fields of which the value is requested.
  */

  memcpy(&length, &cmd->data[pos1], 4);
  pos1 += 4;
  length = swap_int(length);

  /*
  ** Allocate memory for the reply.
  */

  reply = x_mem_alloc((w_word)(4 + length * 10));

  /*
  ** Store the number of values in the reply packet.
  */
  
  length = swap_int(length);
  memcpy(&reply[pos2], &length, 4);
  pos2 += 4;
  length = swap_int(length);

  woempa(9, "Get field values (%d) for instance %p\n", length, instance);

  /*
  ** Go over each of the requested fields and put their value in the reply.
  */

  for(i = 0; i < length; i++) {

    /*
    ** Get the fieldID from the command packet..
    */
    
    memcpy(&field, &cmd->data[pos1], 4);
    pos1 += 4;

    woempa(9, "  -> field %w (%p) = %d (%w)\n", field->name, field, instance[field->slot], field->desc->name);

    /*
    ** Look for the class matching the type of the field.
    */
    
    clazz = desc2Class(instance2clazz(instance)->loader, field->desc);

    /*
    ** Check the type of the field and add the value to the reply.
    ** Different types take different space in the reply.
    */
    
    if(isAssignmentCompatible(clazz, clazzThread)) {                       /* a Thread */
      woempa(9, "       Thread field\n");
      reply[pos2++] = jdwp_tag_thread;
      anint = instance[field->slot];
      memcpy(&reply[pos2], &anint, 4);
      pos2 += 4;
    }
    else if(isAssignmentCompatible(clazz, clazzThreadGroup)) {             /* a ThreadGroup */
      woempa(9, "       Threadgroup field\n");
      reply[pos2++] = jdwp_tag_thread_group;
      anint = instance[field->slot];
      memcpy(&reply[pos2], &anint, 4);
      pos2 += 4;
    }
    else if(isAssignmentCompatible(clazz, clazzClassLoader)) {             /* a ClassLoader */
      woempa(9, "       Classloader field\n");
      reply[pos2++] = jdwp_tag_class_loader;
      anint = instance[field->slot];
      memcpy(&reply[pos2], &anint, 4);
      pos2 += 4;
    }
    else if(isAssignmentCompatible(clazz, clazzString)) {                  /* a String */
      woempa(9, "       String field\n");
      reply[pos2++] = jdwp_tag_string;
      anint = instance[field->slot];
      memcpy(&reply[pos2], &anint, 4);
      pos2 += 4;
    }
    else if(field->desc->tag == D_Primitive) {                             /* a primitive type */
      woempa(9, "primitive:  %w\n", field->desc->name);
      switch(field->desc->body.primitive.type) {
        case P_boolean:   reply[pos2++] = jdwp_tag_boolean;                /* boolean */
                          abyte = (w_ubyte)instance[field->slot];
                          memcpy(&reply[pos2++], &abyte, 1);
                          break;
        case P_char:      reply[pos2++] = jdwp_tag_char;                   /* char */
                          ashort = swap_short(instance[field->slot]);
                          memcpy(&reply[pos2], &ashort, 2);
                          pos2 += 2;
                          break;
        case P_float:     reply[pos2++] = jdwp_tag_float;                  /* float */
                          anint = swap_int(instance[field->slot]);
                          memcpy(&reply[pos2], &anint, 4);
                          pos2 += 4;
                          break;
        case P_double:    reply[pos2++] = jdwp_tag_double;                 /* double */
                          along = ((w_long)instance[field->slot + 1] << 32) + (w_long)instance[field->slot];
                          along = swap_long(along);
                          memcpy(&reply[pos2], &along, 8);
                          pos2 += 8;
                          break;
        case P_byte:      reply[pos2++] = jdwp_tag_byte;                   /* byte */
                          abyte = (w_ubyte)instance[field->slot];
                          memcpy(&reply[pos2++], &abyte, 1);
                          break;
        case P_short:     reply[pos2++] = jdwp_tag_short;                  /* short */
                          ashort = swap_short(instance[field->slot]); 
                          memcpy(&reply[pos2], &ashort, 2);
                          pos2 += 2;
                          break;
        case P_int:       reply[pos2++] = jdwp_tag_int;                    /* int */
                          anint = swap_int(instance[field->slot]);
                          memcpy(&reply[pos2], &anint, 4);
                          pos2 += 4;
                          break;
        case P_long:      reply[pos2++] = jdwp_tag_long;                   /* long */
                          along = ((w_long)instance[field->slot + 1] << 32) + (w_long)instance[field->slot];
                          along = swap_long(along);
                          memcpy(&reply[pos2], &along, 8);
                          pos2 += 8;
                          break;
        case P_void:      reply[pos2++] = jdwp_tag_void;                   /* void */
                          break;
        case P_Object:    reply[pos2++] = jdwp_tag_object;                 /* Object */
                          anint = instance[field->slot];
                          memcpy(&reply[pos2], &anint, 4);
                          pos2 += 4;
                          break;
      }
    }
    else if(field->desc->tag == D_Class) {                                 /* a Class */
      woempa(9, "It's a class...\n");
      reply[pos2++] = jdwp_tag_object;
      anint = instance[field->slot];
      memcpy(&reply[pos2], &anint, 4);
      pos2 += 4;
    }
    else if(field->desc->tag == D_Array) {                                 /* an array */
      woempa(9, "Array\n");
      reply[pos2++] = jdwp_tag_array;
      anint = instance[field->slot];
      memcpy(&reply[pos2], &anint, 4);
      pos2 += 4;
    }
  }

  /*
  ** Send the reply to the debugger
  */
  
  jdwp_send_reply(cmd->id, reply, pos2, jdwp_err_none);

  /*
  ** Clean up.
  */

  x_mem_free(reply);

  return;
  
}
  

/*
** The dispatcher for the 'Object reference' command set.
*/

w_void dispatch_objref(jdwp_command_packet cmd) {

  switch((jdwp_objref_cmd)cmd->command) {
    case jdwp_objref_referenceType:       jdwp_object_type(cmd);        break;
    case jdwp_objref_getValues:           jdwp_object_get_values(cmd);  break;
    case jdwp_objref_setValues:
    case jdwp_objref_monitorInfo:
    case jdwp_objref_invokeMethod:
    case jdwp_objref_disableCollection:
    case jdwp_objref_enableCollection:
    case jdwp_objref_isCollected:
    default:
      jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_not_implemented);
  }
  
}

