/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

#include "oswald.h"      /* x_thread_... */
#include "wonka.h"       /* the w_ types */
#include "ts-mem.h"      /* allocMem_system */
#include "realms.h"      /* allocMem_system */
#include "jdwp.h"
#include "jdwp_events.h"
#include "hashtable.h"   /* hashtable stuff */
#include "descriptor.h"  /* w_desc */
#include "clazz.h"       /* w_clazz */
#include "strings.h"     /* w_string */

#include <string.h>

/*
** Return a few details about this virtual machine aswell as the JDWP version.
*/

w_void jdwp_version(jdwp_command_packet cmd) {
  w_ubyte  *desc     = "Wonka Virtual Machine";
  w_ubyte  *ver      = "0.7.1";
  w_ubyte  *name     = "Wonka";
  w_ubyte  *string;
  w_int    jdwp_ver1 = 1;
  w_int    jdwp_ver2 = 0;
  w_word   length;
  w_int    position = 0;
  w_ubyte  *result;

  /*
  ** Allocate enough memory to fit the reply in.
  */
 
  result = x_mem_alloc(strlen(desc) + strlen(ver) + strlen(name) + 8 + 24 + 1);

  /*
  ** First put in the description. The cstring has to be converted to UTF8.
  ** After the UTF8 string is copied into the result packet, memory can be freed.
  */

  string = jdwp_cstring2UTF8(desc, &length);
  memcpy(&result[position], string, length + 4);
  position += length + 4;
  x_mem_free(string);

  /*
  ** Next we add the 2 version numbers, first the major, then the minor. 
  ** For now we try to mimic JDWP version 1.0
  */

  jdwp_ver1 = swap_int(jdwp_ver1);
  jdwp_ver2 = swap_int(jdwp_ver2);
  memcpy(&result[position], &jdwp_ver1, 4);
  position += 4;
  memcpy(&result[position], &jdwp_ver2, 4);
  position += 4;

  /*
  ** Then we have the version string of Wonka
  */
  
  string = jdwp_cstring2UTF8(ver, &length);
  memcpy(&result[position], string, length + 4);
  position += length + 4;
  x_mem_free(string);

  /* 
  ** And last but not lease, the name of Wonka. 
  */

  string = jdwp_cstring2UTF8(name, &length);
  memcpy(&result[position], string, length + 4);
  position += length + 4;
  x_mem_free(string);

  /* 
  ** Send the packet back where it came from.
  */
  
  jdwp_send_reply(cmd->id, result, position, jdwp_err_none);
  
  /*
  ** Release memory used for the reply.
  */
  
  x_mem_free(result);

  return;
}


/*
** Return the reference type of all the loaded classes which match the given
** signature. 
*/

w_void jdwp_vm_classes_by_sig(jdwp_command_packet cmd) {
  w_ubyte   *reply = NULL;
  w_int     position = 0;
  w_int     length;
  w_int     ID;
  w_ubyte   *sig;
  w_clazz   clazz;
  w_Desc    desc;
  w_ubyte   reftype;

  woempa(9, " oela 1\n");

  /*
  ** Get a plain old cstring of the signature.
  */
  
  sig = jdwp_UTF82cstring(cmd->data, &length);

  /*
  ** Build a descriptor with the signature and call seekClazzByDesc to look for it.
  ** -> This results in only one clazz, while there could be many results.
  **    e.g. com.acunia.wonka.NativeThread matches java.lang.thread since it's 
  **    derived from java.lang.Thread... TODO: fix this.
  */
  
  desc.name = cstring2String(W_Thread_system, sig, (w_word)length);
  clazz = seekClazzByDesc(&desc, system_class_hashtable);

  if(clazz) {

    /*
    ** A Class has been found.
    */

    /*
    ** Allocate memory for the reply. The reply looks like this:
    **   int        classes        The number of classes that match the signature.
    ** Repeated 'classes' times:
    **   byte       reftype_tag    The type tag of this class
    **   reftypeID  typeID         The reference type ID of this class. 
    **   int        status         The status of this class.
    */

    reply = x_mem_alloc((w_word)(4 + 1 + 4 + 4));

    /*
    ** Store the number of classes.
    */

    length = swap_int(1);
    memcpy(&reply[position], &length, 4);
    position += 4;

    /*
    ** Store the reference type tag.
    */
    
    if (isSet(clazz->flags, ACC_INTERFACE)) 
      reftype = jdwp_tt_interface;
    else 
      reftype = jdwp_tt_class;
    memcpy(&reply[position], &reftype, 1);
    position += 1;

    /*
    ** Store the reference type ID.
    */
  
    ID = (w_int)(clazz);
    memcpy(&reply[position], &ID, 4);
    position += 4;

    /*
    ** Store the class status.
    */

    length = swap_int((jdwp_cs_verified | jdwp_cs_prepared | jdwp_cs_initialized));
    memcpy(&reply[position], &length, 4);
    position += 4;

    /*
    ** Send the reply to the debugger.
    */
  
    jdwp_send_reply(cmd->id, reply, position, jdwp_err_none);
    
  } else {

    /*
    ** No match -> Send a reply where 'classes' equals 0.
    */

    reply = x_mem_calloc(4);
    
    jdwp_send_reply(cmd->id, reply, 4, jdwp_err_none);
    
  }

  /*
  ** Clean up.
  */
  
  x_mem_free(sig);
  x_mem_free(reply);

  woempa(9, " oela 2\n");
  return;
}


/*
** Return all classes currently loaded by the VM.
*/

w_void jdwp_vm_classes(jdwp_command_packet cmd) {
  w_fifo    fifo;
  w_clazz   clazz= NULL;
  w_ubyte   *reply;
  w_int     position = 0;
  w_int     length;
  w_int     ID;
  w_ubyte   reftype;
  w_ubyte   *sig;

  /*
  ** Use a fifo to get all of the elements of the system_class_hashtable.
  */

  fifo = ht_list_values(system_class_hashtable);
  length = fifo->numElements;

  /*
  ** Allocate memory for the reply packet. This is the structure of the packet:
  **   int        classes       The number of classes
  ** Repeated 'classes' times:
  **   byte       reftype_tag   The reference type tag of this class.
  **   reftypeID  typeID        The reference type ID of this class.
  **   string     signature     The signature of this class.
  **   int        status        The status of this class.
  */ 

  reply = x_mem_alloc((w_word)(4 + length * (1 + 4 + 4 + 50 + 4)));

  /*
  ** Store the number of classes.
  */
  
  length = swap_int(length);
  memcpy(&reply[position], &length, 4);
  position += 4;

  /*
  ** Go over the entries one by one and add them to the packet.
  */

  while((clazz = getFifo(fifo)) != NULL) {

    /*
    ** Store the reference type tag.
    */

    if (isSet(clazz->flags, ACC_INTERFACE)) 
      reftype = jdwp_tt_interface;
    else 
      reftype = jdwp_tt_class;
    memcpy(&reply[position], &reftype, 1);
    position += 1;

    /*
    ** Store the reference type ID.
    */
  
    ID = (w_int)(clazz);
    memcpy(&reply[position], &ID, 4);
    position += 4;

    /*
    ** Store the signature of the class.
    */

    sig = jdwp_string2UTF8(clazz->desc->name, &length);
    memcpy(&reply[position], sig, (w_word)(length + 4));
    position += length + 4;
    x_mem_free(sig);

    /*
    ** Store the status of the class.
    */

    length = swap_int((jdwp_cs_verified | jdwp_cs_prepared | jdwp_cs_initialized));
    memcpy(&reply[position], &length, 4);
    position += 4;
  }

  /*
  ** Send the reply to the debugger.
  */

  jdwp_send_reply(cmd->id, reply, position, jdwp_err_none);

  /*
  ** Clean up.
  */ 
  
  releaseFifo(fifo);
  x_mem_free(reply);

  return;
}


/*
** Returns all the active threads. This includes threads created with java.lang.Thread, native threads 
** attached to Wonka with JNI and system threads created by Wonka itself. Threads which aren't started
** yet or threads that are no longer running are not included.
*/

w_void jdwp_vm_threads(jdwp_command_packet cmd) {
  w_fifo    fifo;
  w_thread  thread = NULL;
  w_ubyte   *reply;
  w_int     position = 0;
  w_int     length;
  w_int     ID;

  /*
  ** Use a fifo to get all the threads from the thread hashtable.
  ** Subtract one from the length for the JDWP thread. The debugger
  ** doesn't need to know about this.
  */

  fifo = ht_list_values(thread_hashtable);
  length = fifo->numElements - 1;

  /*
  ** Allocate memory for the reply which looks like this:
  **  int       threads  The number of threads
  **  threadID  thread   (repeated 'threads' times)
  */

  reply = x_mem_alloc((w_word)(4 + length * 4));

  /*
  ** Store the number of threads in the reply.
  */

  length = swap_int(length);
  memcpy(&reply[position], &length, 4);
  position += 4;

  /*
  ** Go over the entries one by one and put them into the reply packet.
  */

  while((thread = getFifo(fifo)) != NULL) {
    if(thread->kthread != jdwp_thread) {
      ID = (w_int)thread->Thread; 
      memcpy(&reply[position], &ID, 4);
      position += 4;
    }
  }

  /*
  ** Send the reply to the debugger.
  */

  jdwp_send_reply(cmd->id, reply, position, jdwp_err_none);

  /*
  ** Clean up.
  */

  releaseFifo(fifo);
  x_mem_free(reply);

  return;
}


/*
** Return all the threadgroups that don't have a parent. In our case this is 
** only one, namely W_ThreadGroup_system. This can be used by the debugger to
** build a tree of all the threads and threadgroups.
*/

w_void jdwp_vm_threadgroups(jdwp_command_packet cmd) {
  w_ubyte     result[8];
  w_int       count = swap_int(1);
  w_instance  threadgroup = (w_instance)W_ThreadGroup_system->ThreadGroup;

  /* 
  ** The structure of the reply data :
  **   int            count
  **   threadGroupID  ID     (repeated 'count' times)
  ** Since we only have one toplevel threadgroup, count equals 1 and ID is the address of the system
  ** threadgroup.
  */
  
  memcpy(&result[0], &count, 4);          /* Set count */
  memcpy(&result[4], &threadgroup, 4);    /* Set the threadgroup ID */
  
  /*
  ** Send the result back to the debugger.
  */
  
  jdwp_send_reply(cmd->id, &result, 8, jdwp_err_none);
  
  return;
}


/*
** Disconnect from the debugger. There are a few tasks involved after disconnecing :
**  - cancel all event requests.
**  - all threads suspended by the debugger should be started again (as many times
**    as it takes).
**  - if garbage collection was disabled, enable it again.
*/

extern w_int jdwp_connected;

w_void jdwp_vm_disconnect(jdwp_command_packet cmd) {
  
  jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_none);

  jdwp_connected = 0;
  jdwp_events_enabled = 0;

  jdwp_clear_all_events();

  /* TODO: All the stuff mentioned above... */
}


/*
** Returns the sizes of the different identifiers. These values indicate the size
** of these identifiers in all the packets sent between debugger and VM.
*/

struct idSizes {
  w_int fieldIDSize;
  w_int methodIDSize;
  w_int objectIDSize;
  w_int refTypeIDSize;
  w_int frameIDSize;
};

w_void jdwp_vm_sizes(jdwp_command_packet cmd) {
  struct idSizes sizes;

  /*
  ** All the ID's are words -> 4 bytes.
  */
  
  sizes.fieldIDSize   = swap_int(4);
  sizes.methodIDSize  = swap_int(4);
  sizes.objectIDSize  = swap_int(4);
  sizes.refTypeIDSize = swap_int(4);
  sizes.frameIDSize   = swap_int(4);

  /*
  ** Send this back to the debugger. The reply data is the eventID of the newly 
  ** created event.
  */

  jdwp_send_reply(cmd->id, &sizes, sizeof(struct idSizes), jdwp_err_none);
}


/*
** Returns the capabilities of this JPDA implementation. A zero indicates that we 
** don't have that feature, anything else means we can handle such requests.
*/

w_void jdwp_capabilities(jdwp_command_packet cmd) {
  w_ubyte reply[7];

  reply[0] = 0;  /* Watch field modifications and send modification watchpoint events */
  reply[1] = 0;  /* Watch field access and send access watchpoint events */
  reply[2] = 1;  /* Get bytecodes of methods */
  reply[3] = 0;  /* Can we detect if a field or method is 'invented' by the compiler */
  reply[4] = 0;  /* Get owned monitors information for a thread */
  reply[5] = 0;  /* Get current contended monitor of a thread */
  reply[6] = 0;  /* Get monitor information for an object */
  
  jdwp_send_reply(cmd->id, &reply, 7, jdwp_err_none);

  return;
}


/*
** Suspend all threads. Note that thread suspending/resuming works with a counter. A
** thread which has been suspended x times, needs to be resumed x times before it 
** will run again.
*/

w_void jdwp_vm_suspend_vm(jdwp_command_packet cmd) {
  w_fifo    fifo;
  w_thread  thread = NULL;

  /*
  ** Use a fifo to get all the threads from the thread hashtable.
  */

  fifo = ht_list_values(thread_hashtable);

  /*
  ** Go over them one by one and suspend them.
  */

  while((thread = getFifo(fifo)) != NULL) {
    if(thread->kthread != jdwp_thread) {
      x_thread_suspend(thread->kthread);
    }
  }

  /*
  ** Send a reply to the debugger. This is an empty reply, only the error field is 
  ** meaningful.
  */

  jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_none);

  /*
  ** Clean up.
  */

  releaseFifo(fifo);

  return;
}


/*
** Resume all threads. 
*/

w_void jdwp_vm_resume_vm(jdwp_command_packet cmd) {
  w_fifo    fifo;
  w_thread  thread = NULL;

  /*
  ** Use a fifo to get all the threads from the thread hashtable.
  */

  fifo = ht_list_values(thread_hashtable);

  /*
  ** Go over each entry and resume them.
  */

  while((thread = getFifo(fifo)) != NULL) {
    if(thread->kthread != jdwp_thread) {
      x_thread_resume(thread->kthread);
    }
  }

  /*
  ** Send an empty reply to the debugger.
  */

  jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_none);

  /*
  ** Clean up.
  */

  releaseFifo(fifo);

  return;
}


/*
** Terminate the vm with a given exit code.
*/

w_void jdwp_vm_terminate(jdwp_command_packet cmd) {
  w_int  exit_code;

  /*
  ** Get the desired exit code.
  */

  memcpy(&exit_code, &cmd->data[0], 4);

  /*
  ** Game over.
  */
  
  exit(exit_code);
}


/*
** Returns the (new) capabilities of this JPDA implementation. A zero indicates that we 
** don't have that feature, anything else means we can handle such requests.
*/

w_void jdwp_capabilities_new(jdwp_command_packet cmd) {
  w_ubyte reply[18];

  reply[0]  = 0;  /* Watch field modifications and send modification watchpoint events */
  reply[1]  = 0;  /* Watch field access and send access watchpoint events */
  reply[2]  = 1;  /* Get bytecodes of methods */
  reply[3]  = 0;  /* Can we detect if a field or method is 'invented' by the compiler */
  reply[4]  = 0;  /* Get owned monitors information for a thread */
  reply[5]  = 0;  /* Get current contended monitor of a thread */
  reply[6]  = 0;  /* Get monitor information for an object */

  /* 
  ** These are the new capabilities
  */

  reply[7]  = 0;  /* Can we redefine classes */
  reply[8]  = 0;  /* Can we add methods while redefining */
  reply[9]  = 0;  /* Can we redefine classes in an arbitrary way */
  reply[10] = 0;  /* Can we pop the top stack frame */
  reply[11] = 0;  /* Can the debugger pop multiple stack frames */
  reply[12] = 0;  /* Can the debugger pop all obsolete stack frames */
  reply[13] = 0;  /* Reserved for future additions */
  reply[14] = 0;  /* Reserved for future additions */
  reply[15] = 0;  /* Reserved for future additions */
  reply[16] = 0;  /* Reserved for future additions */
  reply[17] = 0;  /* Reserved for future additions */
  
  jdwp_send_reply(cmd->id, &reply, 18, jdwp_err_none);

  return;
}


/*
**
*/

w_void jdwp_vm_classpaths(jdwp_command_packet cmd) {
  w_ubyte *reply;

  woempa(9, "Classpaths requested -> Give 'em nothing.\n");

  reply = x_mem_calloc(12);

  jdwp_send_reply(cmd->id, reply, 12, jdwp_err_none);

  x_mem_free(reply);

  return;
}


/*
** Dispose objects
*/

w_void jdwp_vm_dispose_objects(jdwp_command_packet cmd) {

  /*
  ** Be ignorant and just say yes.
  */
  
  jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_none);

  return;
}


/*
**
*/

w_void jdwp_vm_hold_events(jdwp_command_packet cmd) {

  jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_not_implemented);

  return;
}


/*
**
*/

w_void jdwp_vm_release_events(jdwp_command_packet cmd) {

  jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_not_implemented);

  return;
}


/*
** The dispatcher for the 'Virtual machine' command set.
*/

w_void dispatch_vm(jdwp_command_packet cmd) {

  switch((jdwp_vm_cmd)cmd->command) {
    case jdwp_vm_version:               jdwp_version(cmd);               break;
    case jdwp_vm_classesBySignature:    jdwp_vm_classes_by_sig(cmd);     break;          
    case jdwp_vm_allClasses:            jdwp_vm_classes(cmd);            break;
    case jdwp_vm_allThreads:            jdwp_vm_threads(cmd);            break;
    case jdwp_vm_topLevelThreadGroups:  jdwp_vm_threadgroups(cmd);       break;
    case jdwp_vm_dispose:               jdwp_vm_disconnect(cmd);         break;
    case jdwp_vm_idSizes:               jdwp_vm_sizes(cmd);              break;
    case jdwp_vm_suspend:               jdwp_vm_suspend_vm(cmd);         break;
    case jdwp_vm_resume:                jdwp_vm_resume_vm(cmd);          break;
    case jdwp_vm_exit:                  jdwp_vm_terminate(cmd);          break;
    case jdwp_vm_capabilities:          jdwp_capabilities(cmd);          break;
    case jdwp_vm_classPaths:            jdwp_vm_classpaths(cmd);         break;
    case jdwp_vm_disposeObjects:        jdwp_vm_dispose_objects(cmd);    break;
    case jdwp_vm_holdEvents:            jdwp_vm_hold_events(cmd);        break;
    case jdwp_vm_releaseEvents:         jdwp_vm_release_events(cmd);     break;
    case jdwp_vm_capabilitiesNew:       jdwp_capabilities_new(cmd);      break;
    case jdwp_vm_createString:
    case jdwp_vm_redefineClasses:
    case jdwp_vm_popObsoleteFrames:
    default:
      jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_not_implemented);
  } 
}

