#ifndef _OSWALD_H
#define _OSWALD_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: oswald.h,v 1.14 2002/03/19 09:10:34 ssymons Exp $
*/


//ecos priorities are between 1 and 32
#define NUM_PRIORITIES     32  

//Possible values for the `flags` argument of x_thread_create
#define TF_START	  0x00000000
#define TF_SUSPENDED	  0x00000200
#define TF_JOIN_ENDED         0x00001000 /* Thread we joined with has ended normally. */
#define TF_JOIN_EXIT          0x00002000 /* Thread we joined with has called x_thread_exit. */
#define TF_JOIN_DELETED       0x00004000 /* Thread we joined with has been suspended and then deleted. */
#define x_no_wait	0
#define x_eternal 0xffffffff
#define isNotSet(x, flag)           (!isSet((x), (flag)))
#define unsetFlag(x, flag)        ((x) &= ~(flag))

#include "o4e.h"
#include "vsprintf.h"

typedef enum {
  xs_success             =  0,	  /* Guess...                                                                        */
  xs_no_instance         =  1,	  /* Could not get resource within timeout specified.                                */
  xs_not_owner           =  2,    /* Trying to unlock a mutex which this thread doesn't own currently not used.      */
  xs_deadlock            =  3,	  /* Trying to lock a mutex which is allready locked by current thread.              */
  xs_bad_context         =  4,    /* Call with timeout within IRQ context.                                           */
  xs_bad_option          =  5,    /* A bad option was passed to a signals function.                                  */
  xs_deleted             =  6,    /* The element was deleted when a call to it was tried.                            */
  xs_bad_element         =  7,    /* The call was tried on a element that was invalid.                               */
  xs_competing           =  8,    /* The call could not be completed because threads were competing for the element. */
  xs_incomplete          =  9,	  /* The call did not complete fully (only with _delete calls and waiters)           */
  xs_interrupted         = 10,	  /* An interrupt occur during the call.                                             */
  xs_tick_error          = 11,	  /* Invalid tick value supplied.                                                    */
  xs_timer_error         = 12,	  /* Timer error                                                                     */
  xs_activate_error      = 13,	  /* Timer activation error.                                                         */
  xs_bad_state           = 14,	  /* Element is in bad state for this operation.                                     */
  xs_insufficient_memory = 15,	  /* Insufficient memory to perform operation.                                       */
  xs_bad_argument        = 16,	  /* One of the passed arguments is bad or out of range.                             */
  xs_owner               = 17,	  /* Result of suspend call when the thread is still owning mutexes or monitors.     */
  xs_not_elf             = 18,	  /* (M) The passed data is not ELF compatible.                                      */
  xs_mod_error           = 19,    /* (M) The module is in error. The module->flags can give more information.        */
  xs_seq_error           = 20,	  /* (M) An operation has been attempted for which a previous step must be performed.*/
  xs_no_mem              = 21,    /* (M) No more memory to perform operation.                                        */
  xs_rel_error           = 22,    /* (M) Error happened with relocation. Unhandled type.                             */
  xs_sym_defined         = 23,    /* (M) Symbol multiply defined.                                                    */
  xs_unresolved          = 24,    /* (M) Unresolved symbols remain in module after x_module_resolve attempt.         */
  xs_unknown             = 25     /* Catch all for unknown situations.                                               */
} x_status;

/*
** Oswald API for threads that we are implementing
*/

x_status x_thread_create(x_thread thread, x_entry entry_function, void* entry_input, void *stack_start, x_size stack_size, x_size priority, x_word flags);
x_status x_thread_delete(x_thread thread);
x_size x_thread_priority_get(x_thread thread);
x_status x_thread_priority_set(x_thread thread, x_size new_priority);
x_status x_thread_resume(x_thread thread);
x_status x_thread_sleep(x_sleep timer_ticks);
x_status x_thread_suspend(x_thread thread);
x_status x_thread_join(x_thread thread, void **result, x_sleep timeout);

x_thread x_thread_current(void);
void x_thread_yield(void);

/*  These functions lock/unlock the scheduler, it's implemented in eCos so
 *  we just need to map it.
 */
void x_scheduler_disable(void);
void x_scheduler_enable(void);


/*  Semaphores
 */
x_status x_sem_create(x_Semaphore *semaphore, x_size initial_count);
x_status x_sem_delete(x_Semaphore *semaphore);
x_status x_sem_get(x_Semaphore *semaphore, x_sleep wait_option);
x_status x_sem_put(x_Semaphore *semaphore);

/*  Mutexes
 */
x_status x_mutex_create(x_Mutex *mutex);
x_status x_mutex_delete(x_Mutex *mutex);
x_status x_mutex_lock(x_Mutex *mutex, x_sleep timeout);
x_status x_mutex_unlock(x_Mutex *mutex);


/*  Queues
 *
 * We set the queue's to a size of 256, in the configtool. eCos doesn't support
 * this dynamically. This is the maximum value given in wonka (in ./wonka/src/vm/thread.c:)
 */
x_status x_queue_create(x_Queue *queue, void *queue_start, x_size queue_size);
x_status x_queue_delete(x_Queue *queue);
x_status x_queue_receive(x_Queue *queue, void **data, x_sleep wait);
x_status x_queue_send(x_Queue *queue, void *src_msg, x_sleep wait);
x_status x_queue_flush(x_Queue *queue, void(*do_this)(void *data));

/*
** For the time being we have our own implementation of x_Monitor and the
** x_monitor_... primitives.
*/

x_status x_monitor_create(x_monitor);
x_status x_monitor_delete(x_monitor);
x_status x_monitor_enter(x_monitor, x_sleep);
x_status x_monitor_exit(x_monitor);
x_status x_monitor_wait(x_monitor, x_sleep);
x_status x_monitor_notify(x_monitor);
x_status x_monitor_notify_all(x_monitor);
x_status x_thread_stop_waiting(x_thread);

/*
** The Oswald memory API.
*/
x_status x_mem_lock(x_sleep timeout);
x_status x_mem_unlock(void);

void* x_mem_alloc(x_size size);
void* x_mem_calloc(x_size size);
void x_mem_free(void*);
void *x_mem_realloc(void *old, x_size size);

x_status x_mem_walk(x_sleep timeout, void (*callback)(void * mem, void * arg), void *arg);

x_size x_mem_total(void);
x_size x_mem_gettotal(void);
#define x_mem_total	x_mem_gettotal
x_size x_mem_avail(void);

x_status x_mem_tag_set(void * mem, x_word tag);
x_word x_mem_tag_get(void * mem);
x_size x_mem_size(void * mem);
void x_mem_discard(void * block);
x_status x_mem_collect(x_size * bytes, x_size * num);
x_boolean x_mem_is_block(void * mem);

void* x_alloc_static_mem(void * memory, x_size size);

/*
** The Oswald time API (part of).
*/
x_sleep x_time_get(void);

x_size x_millis2ticks(x_size millis);
x_size x_seconds2ticks(x_size seconds);
x_size x_ticks2usecs(x_size ticks);
x_size x_usecs2ticks(x_size usecs);

x_status x_oswald_init(x_size requested_heap, x_size msec);

// status to char function 
const char* status2char(x_status status);
#endif /* _OSWALD_H */
