/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


#ifndef _O4P_H
#define _O4P_H

/*
** $Id: o4p.h,v 1.2 2002/02/12 14:52:26 gray Exp $
*/

#include "list.h"
#include "wonka.h"

 /*************************************************************************
 * Filename:                                                              *
 *   o4p.c                                                                *
 *                                                                        *
 * Description:                                                           *
 *                                                                        *
 *   "Oswald for POSIX" compatibility layer.                              *
 *                                                                        *
 *   We only implement the Oswald calls which are actually used by Wonka  *
 *   (the full Oswald API is much bigger than this).  If you don't see    *
 *   the Oswald system call you wanted here, feel free to implement it.   *
 *                                                                        *
 * Authors:                                                               *
 *   Dries Buytaert <dries.buytaert@acunia.com>                           *
 *   Chris Gray <chris.gray@acunia.com>                                   *
 *                                                                        *
 *************************************************************************/

                
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include <semaphore.h>
#include <dlfcn.h>

/// Amount of memory to reserve for `static' allocations during OS startup (Wonka doesn't need any).
#ifndef STATIC_MEMORY_SIZE
#define STATIC_MEMORY_SIZE 4096
#endif 
extern int num_x_threads;

#define S_RR    SCHED_RR
#define S_FIFO  SCHED_FIFO
#define S_OTHER SCHED_OTHER

/* 
** The default number of microseconds per timer interrupt. For linux, don't make
** it too small since the kernel would not be able to service it. Note that the
** following value was experimentally found to be a correct number for most systems. Your mileage
** may vary...
*/

typedef unsigned int           x_size;

extern x_size usecs_per_tick;

/*
 * Parameters
 */

#define O4P_OR              0x00
#define O4P_OR_CLEAR        0x01
#define O4P_AND             0x02
#define O4P_AND_CLEAR       0x03

/*  
 * Types
 */
 
typedef unsigned long ULONG;

typedef struct x_Queue {
  pthread_mutexattr_t o4p_queue_mutex_att;
  pthread_mutex_t o4p_queue_mutex;       
  pthread_cond_t o4p_queue_not_empty;
  pthread_cond_t o4p_queue_not_full; 
  int o4p_queue_deleted;

  int available;
  int capacity;

  w_word *messages;
  w_word *write;
  w_word *read;
  w_word *limit;
} x_Queue;

typedef struct o4p_Semaphore {
  sem_t posix_sem;                         /* POSIX.4 counting semaphore. */ 

  w_size count;
  int waiters;
  pthread_mutexattr_t sem_mutex_att;
  pthread_mutex_t sem_mutex;      
  pthread_cond_t sem_cond;                 /* to broadcast the freeness of the semaphore */
  int deleted;
} o4p_Semaphore;

typedef o4p_Semaphore *o4p_sem;

#define O4P_READY	   0
#define O4P_COMPLETED       1
#define O4P_TERMINATED      2
#define O4P_SUSPENDED       3
#define O4P_SLEEP           4
#define O4P_NEWBORN        12 /* used during thread_create in application_define */

#define THREAD_IS_DEAD(t)     ((t)->o4p_thread_curStatus > O4P_READY && (t)->o4p_thread_curStatus < O4P_SUSPENDED)
#define THREAD_IS_ALIVE(t)    (((t)->o4p_thread_curStatus == O4P_READY) || ((t)->o4p_thread_curStatus > O4P_TERMINATED && (t)->o4p_thread_curStatus < O4P_IO_DRIVER))

#define SR_NO_MESSAGE       1
#define SR_SUSPEND          2
#define SR_RESUME           3
#define SR_INITIALIZED      4

typedef struct x_Monitor {
  pthread_mutex_t *mutex;
  pthread_cond_t  *condvar;
  x_thread        owner;
  w_int           count;
  x_thread        waitset;
  pthread_mutex_t *waitset_mutex;
} x_Monitor;

#define MESSAGE_QUEUE_LENGTH 16

typedef struct x_Thread {
  int reqStatus;            /* requested status, on of the defines above */
  int curStatus;            /* actual status */
  pthread_mutexattr_t sleep_timer_att;   
  pthread_mutex_t sleep_timer; /* a dummy mutex to support thread_sleep */
  pthread_cond_t sleep_cond;
  pthread_mutexattr_t o4p_thread_mutex_att;
  pthread_mutex_t o4p_thread_mutex;
  pthread_attr_t attributes;           /* the attributes */
  void *o4p_thread_stack_start;        /* Starting address of thread's memory area. */        
  w_size o4p_thread_stack_size;         /* Number of bytes in the stack memory area. */
  pthread_t o4p_pthread;               /* POSIX.4 thread. */
  w_size o4p_thread_priority;
  struct sched_param o4p_thread_sched; /* Scheduling policy, incl. priority level. */
  int o4p_thread_schedPolicy;          /* SCHED_FIFO or SCHED_RR */
  void *o4p_thread_function;           /* The function the thread will call when it runs */
  void* o4p_thread_argument;           /* The argument to be passed ton that function */
  x_thread  o4p_thread_next;   /* Next thread in our linked list */

  sem_t message;
  x_monitor waiting_on;
  w_int     waiting_with;
  x_thread  next;
  x_thread  previous;

  void *xref;                          /* May be used to point to user thread control block */
  x_report report;
} x_Thread;

#define O4P_ENV_STATUS_INIT	0
#define O4P_ENV_STATUS_NORMAL	1

typedef struct O4pEnv {
  int scheduler;
  int status;
  void *staticMemory;                      /* memory used to fake Oswald's static allocation thang */
  x_thread threads;               /* pointer to first element in linked list of threads */
  pthread_mutexattr_t threadsLockAttr;
  pthread_mutex_t threadsLock;           
  
  pthread_mutexattr_t timerLockAttr;
  pthread_mutex_t timer_lock;           
  w_size timer_ticks; /* the number of ticks passed since we 'booted' */
  FILE *log;
} O4pEnv;

extern O4pEnv *o4pe;

typedef void(*x_entry)(void* arg);

void x_setup_kernel(void);
  /* Enter kernel. */
  
void setScheduler(int scheduler);

#define currentTicks                 (o4pe->timer_ticks)

/// Number of command line arguments (copy of argc)
extern int command_line_argument_count;

/// Command line arguments (copy of argv)
extern char **command_line_arguments; 

/** Memory chunk header
    +----------+
    ! reserved ! ) Just to bring the total up to 8 words.
    +----------+  
    !    id    ! User-supplied object type identifier.  Values 0..31 reserved.
    +----------+
    !   next   ! )
    +----------+ ) The linked list used to emulate Oswald's heap-walking stuff
    ! previous ! )
    +----------+
    !   file   ! )
    +----------+ ) Program location where allocated
    !   line   ! )
    +----------+
    !   size   ! Requested size in bytes
    +----------+
    !   check  ! Must point to `magic'
    +----------+
    ! contents !
    :          :
*/

typedef struct o4p_Memory_Chunk *o4p_memory_chunk;

typedef struct o4p_Memory_Chunk {
  w_word            reserved0;
  w_word            id;
  o4p_memory_chunk  next;
  o4p_memory_chunk  previous;
  char             *file;
  w_size            line;
  w_size            size;
  char             *check;
} o4p_Memory_Chunk;

/*
** We only use bits 27..18 of the `id' word: in the `real' oswald this shares
** space with the chunk size and a couple of other flags.
*/
#define TAG_MASK               0x1ff00000 // The tag can hold a 9 bit information NUMBER, numbers 0 - 31 are reserved.
#define GARBAGE_TAG            0x20000000 // Piece of memory is garbage, can be reclaimed by our OWN garbage collector

/// The maximum amount of memory we allow ourselves to allocate in a single chunk.
/// (Oswald has such a limit).
#ifndef MAX_SINGLE_ALLOC
#define MAX_SINGLE_ALLOC  8*1024*1024
#endif

x_size x_ticks2usecs(x_size ticks);

#endif /* _O4P_H */

