#ifndef _OSWALD_H
#define _OSWALD_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: oswald.h,v 1.6 2002/03/27 11:00:47 dumon Exp $
*/

#include <pthread.h>

#define NUM_PRIORITIES     64

typedef enum {
  xs_success             =  0,	  /* Guess...                                                                        */
  xs_no_instance         =  1,	  /* Could not get resource within timeout specified.                                */
  xs_not_owner           =  2,    /* Trying to unlock a mutex which this thread doesn't own currently not used.      */
  xs_deadlock            =  3,	  /* Trying to lock a mutex which is allready locked by current thread.              */
  xs_bad_context         =  4,    /* Call with timeout within IRQ context.                                           */
  xs_bad_option          =  5,    /* A bad option was passed to a signals function.                                  */
  xs_deleted             =  6,    /* The element was deleted when a call to it was tried.                            */
  xs_bad_element         =  7,    /* The call was tried on a element that was invalid.                               */
  xs_competing           =  8,    /* The call could not be completed because threads were competing for the element. */
  xs_incomplete          =  9,	  /* The call did not complete fully (only with _delete calls and waiters)           */
  xs_interrupted         = 10,	  /* An interrupt occur during the call.                                             */
  xs_tick_error          = 11,	  /* Invalid tick value supplied.                                                    */
  xs_timer_error         = 12,	  /* Timer error                                                                     */
  xs_activate_error      = 13,	  /* Timer activation error.                                                         */
  xs_bad_state           = 14,	  /* Element is in bad state for this operation.                                     */
  xs_insufficient_memory = 15,	  /* Insufficient memory to perform operation.                                       */
  xs_bad_argument        = 16,	  /* One of the passed arguments is bad or out of range.                             */
  xs_owner               = 17,	  /* Result of suspend call when the thread is still owning mutexes or monitors.     */
  xs_not_elf             = 18,	  /* (M) The passed data is not ELF compatible.                                      */
  xs_mod_error           = 19,    /* (M) The module is in error. The module->flags can give more information.        */
  xs_seq_error           = 20,	  /* (M) An operation has been attempted for which a previous step must be performed.*/
  xs_no_mem              = 21,    /* (M) No more memory to perform operation.                                        */
  xs_rel_error           = 22,    /* (M) Error happened with relocation. Unhandled type.                             */
  xs_sym_defined         = 23,    /* (M) Symbol multiply defined.                                                    */
  xs_unresolved          = 24,    /* (M) Unresolved symbols remain in module after x_module_resolve attempt.         */
  xs_unknown             = 25     /* Catch all for unknown situations.                                               */
} x_status;

const char * status2char(x_status status);

typedef unsigned int x_time;
                                                    
/*
** Sleep times are measured in `ticks'.  How long is a tick?  Your turn to guess.
*/
typedef unsigned long  x_sleep;
typedef unsigned int x_word;
typedef signed int x_int;
typedef signed long long x_long;
typedef x_word x_flags;
typedef struct x_Queue *x_queue;
typedef struct x_Monitor *x_monitor;
typedef struct x_Thread *x_thread;
typedef const char * (*x_report)(x_thread thread);  /* report generator */
typedef enum {
  false = 0x00000000,
  true  = 0x00000001,
} x_boolean;

#include "o4p.h"

#define x_Sem   o4p_Semaphore
typedef struct x_Sem   *x_sem;
#define x_Mutex o4p_Semaphore
typedef struct x_Mutex *x_mutex;

/*
** When getting a sem or locking a mutex, these are two extreme values for the
** timeout parameter: x_no_wait means always return immediately (perhaps with
** a return code of xs_no_instance), x_eternal means hang on in there for
** however long it takes.
*/
#define x_no_wait 0
#define x_eternal 0xffffffff

/*
** Possible values for the `flags' argument of x_thread_create
*/
#define TF_START              0x00000000 /* start immediately */
#define TF_SUSPENDED          0x00000200 /* initially thread is suspended */

/*
** In the real Oswald these calls disable/reenable the scheduler.  
** What we do is to lock a global mutex, which should be close enough.
*/
void x_scheduler_disable(void);
void x_scheduler_enable(void);

/*
** For the time being we have our own implementation of x_Monitor and the
** x_monitor_... primitives.
*/

x_status x_monitor_create(x_monitor);
x_status x_monitor_delete(x_monitor);
x_status x_monitor_enter(x_monitor, x_sleep);
x_status x_monitor_exit(x_monitor);
x_status x_monitor_wait(x_monitor, x_sleep);
x_status x_monitor_notify(x_monitor);
x_status x_monitor_notify_all(x_monitor);
x_status x_thread_stop_waiting(x_thread);

/*
** For the time being we have our own implementation of x_Queue and the
** x_queue_... primitives.
*/
x_status x_queue_create(x_queue queue, void *queue_start, w_size queue_size);
x_status x_queue_delete(x_queue queue);
x_status x_queue_receive(x_queue queue, void **dest_msg, x_sleep waittime);
x_status x_queue_send(x_queue queue, void *src_msg, x_sleep waittime);
x_status x_queue_flush(x_queue queue, void(*do_this)(void *data));

/*
** Ditto for semaphores and muteces.
*/
x_status x_sem_create(o4p_Semaphore *semaphore, w_size initial_count);
x_status x_sem_delete(o4p_Semaphore *semaphore);
x_status x_sem_get(o4p_Semaphore *semaphore, x_sleep wait_option);
x_status x_sem_put(o4p_Semaphore *semaphore);
#define x_mutex_create(m) x_sem_create(m,1)
#define x_mutex_delete(m) x_sem_delete(m)
#define x_mutex_lock(m,t) x_sem_get(m,t)
#define x_mutex_unlock(m) x_sem_put(m)

/*
** Crude implementation of x_mutex primitives.  Bug: x_mutex_lock treats all
** nonzero values of t as x_eternal.
#define status_p2o(s) ((s)==0?xs_success:(s)==EINVAL?xs_bad_element:(s)==EDEADLK?xs_deadlock:(s)==EBUSY?xs_no_instance:xs_unknown)
#define x_mutex_create(m)   status_p2o(pthread_mutex_init(m, NULL))
#define x_mutex_delete(m)   status_p2o(pthread_mutex_destroy(m))
#define x_mutex_lock(m,t)   status_p2o((t)?pthread_mutex_lock(m):pthread_mutex_trylock(m))
#define x_mutex_unlock(m)   status_p2o(pthread_mutex_unlock(m))
*/
  
/*
** And again for threads.
*/

x_status x_thread_create(x_thread thread, x_entry entry_function, void* entry_input, void *stack_start, w_size stack_size, w_size priority, w_word flags);
x_status x_thread_delete(x_thread thread);
w_size x_thread_priority_get(x_thread thread);
w_int x_thread_priority_set(x_thread thread, w_size new_priority);
x_status x_thread_resume(x_thread thread);
x_status x_thread_sleep(x_sleep timer_ticks);
x_status x_thread_suspend(x_thread thread);
x_status x_thread_join(x_thread thread, void **result, x_sleep timeout);

x_thread x_thread_current(void);
void x_thread_yield(void);

/*
** The Oswald time API (part of).
*/
x_sleep x_time_get(void);

x_size x_millis2ticks(x_size millis);
x_size x_seconds2ticks(x_size seconds);
x_size x_ticks2usecs(x_size ticks);
x_size x_usecs2ticks(x_size usecs);


/*
** The Oswald memory API.
*/
x_status x_mem_lock(x_sleep timeout);
x_status x_mem_unlock(void);

void *_x_mem_alloc(w_size bytes, const char *file, int line);
void *_x_mem_calloc(w_size bytes, const char *file, int line);
#define x_mem_alloc(bytes) _x_mem_alloc((bytes), __FILE__,__LINE__)
#define x_mem_calloc(bytes) _x_mem_calloc((bytes), __FILE__,__LINE__)
void x_mem_free(void*);
void *x_mem_realloc(void *old, w_size size);

x_status x_mem_walk(x_sleep timeout, void (*callback)(void * mem, void * arg), void *arg);

w_size x_mem_total(void);
#define x_mem_gettotal x_mem_total

w_size x_mem_avail(void);

x_status x_mem_tag_set(void * mem, w_word tag);
w_word x_mem_tag_get(void * mem);
w_size x_mem_size(void * mem);
void x_mem_discard(void * block);
x_status x_mem_collect(w_size * bytes, w_size * num);
w_boolean x_mem_is_block(void * mem);

void * x_alloc_static_mem(void * memory, w_size size);

/*
** This is the entry point which the OS will call when it has completed
** its own internal initialisation.
*/
w_ubyte* x_os_main(int argc, char** argv, w_ubyte *first_unused_memory);

/*
** Call this from main() to start up oswald.
** Parameters: max heap in bytes, tick interval in milliseconds.
*/
x_status x_oswald_init(x_size max_heap, x_size millis);

/*
** vsprintf.
*/

#include "vsprintf.h"

#endif /* _OSWALD_H */
