/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: o4p.c,v 1.4 2002/02/12 14:52:26 gray Exp $
*/    
 
#include "oswald.h"
#include "ficl.h"
#include "list.h"

#include <stdio.h>
#include <unistd.h>
#include <stdarg.h>
#include <sys/time.h>
#include <signal.h>

/*
** #define SIGINT_IS_WABORT to generate a wabort(ABORT_WONKA) when ^C is hit.
** (Doesn't work yet ...)
*/
// #define SIGINT_IS_WABORT

/*
** If USE_NATIVE_MALLOC is defined then x_mem_... are implemented using
** malloc() and friends: otherwise we use code borrowed from Oswald.
*/
#define USE_NATIVE_MALLOC

typedef void (*cleanpush)(void *);

#define x_boolean w_boolean
#define true      WONKA_TRUE
#define false     WONKA_FALSE
#define loempa    woempa
#define x_assert(x) if (!(x)) {wabort(ABORT_WONKA, "Assertion failed");}

#define O4P_AUTO_ACTIVATE    0x01
#define O4P_NO_ACTIVATE      0x02

int command_line_argument_count;
char** command_line_arguments;

x_size heap_remaining;
x_size usecs_per_tick;

O4pEnv *o4pe;
int num_x_threads = 0;
#ifndef D
#define D(x) printf("DEBUG %d\n", x); fflush(stdout);
#endif
pthread_key_t x_thread_key;

/*
** OS Specific stuff...
*/

#if defined (sun)

#include <thread.h>

/*
** According to our tests, the SCHED_RR is the best scheduler for this
** library under Solaris.
*/

#define DEFAULT_SCHEDULER      SCHED_OTHER
#define pthread_attr_setstackaddr(ap, start)     0
#define pthread_attr_setstacksize(ap, size)      0

#elif defined (LINUX)

/*
** Under Linux, the only scheduler that is allowed is the default SCHED_OTHER
*/

#define DEFAULT_SCHEDULER      SCHED_OTHER

/*
** Linux doesn't have the following calls.
*/

#define PTHREAD_MUTEX_NORMAL 0
#define pthread_mutexattr_settype(mutex, type)   /* nothing */
#define pthread_attr_setstackaddr(ap, start)     0
#define pthread_attr_setstacksize(ap, size)      0

#elif

#error Unknown OS

#endif

//#undef woempa
//#define woempa(level, format, a...) fprintf(stderr, format, ##a)

/*
** End of OS specific stuff.
*/

static int num_started = 0;
static int num_deleted = 0;

static x_monitor memory_monitor;

static const char * _status2char[] = {
  "success",                      /*  0 */
  "no instance",                  /*  1 */
  "not owner",          	  /*  2 */
  "deadlock",                     /*  3 */
  "bad context",                  /*  4 */
  "bad option",                   /*  5 */
  "deleted",                      /*  6 */
  "bad element",          	  /*  7 */
  "threads waiting",              /*  8 */
  "incomplete",                   /*  9 */
  "interrupted",              	  /* 10 */
  "tick error",               	  /* 11 */
  "timer error",                  /* 12 */
  "activation error",             /* 13 */
  "bad state",                    /* 14 */
  "insufficient memory",          /* 15 */
  "bad argument",                 /* 16 */
  "still owns events",            /* 17 */
  "is not ELF file",              /* 18 */
  "module is in error",           /* 19 */
  "module sequence error",        /* 20 */
  "no more memory",               /* 21 */
  "relocation failed",            /* 22 */
  "symbol multiply defined",      /* 23 */
  "unresolved symbols remain",    /* 24 */
  "(unknown)",                    /* 15 */
};
                                                    
const char * status2char(x_status status) {
  return _status2char[(status >= xs_unknown) ? xs_unknown : status];
}                                                      

void our_nanosleep(void) {

  int i = 100000;
  int j = 0;
  int k = 0;
  
  while (i--) {
    j = i;
  }
  
  k = j;
  
}

/*
** Map an Oswald priority to the correct priority for a certain
** scheduler. Returns an int value that can be used as a
** sched_param value.
*/

int mapPriority(int policy, unsigned int requested) {

  int max;
  int min;
  int result;

#ifdef DEBUG
  switch (policy) {
    case SCHED_OTHER: woempa(9, "SCHED_OTHER: TX prio = %2d -> ", requested); break;
    case SCHED_FIFO:  woempa(9, "SCHED_FIFO: TX prio = %2d -> ", requested); break;
    case SCHED_RR:    woempa(9, "SCHED_RR: TX prio = %2d -> ", requested); break;
  }
#endif

  max = sched_get_priority_max(policy);
  min = sched_get_priority_min(policy);

  if (min < 0 && max < 0) {
    result = min + ((min - max) * ((NUM_PRIORITIES - 1)-requested) / NUM_PRIORITIES);
  }
  else {
    result = min + ((max - min) * requested / NUM_PRIORITIES);
  }

#ifdef DEBUG
  woempa(9, "max prio = %d min prio = %d result prio = %d\n", max, min, result);
#endif

  return result;

}

/*
** This function is called once for setting up the Oswald emulation environment.
** It is called from main, before going to x_os_main. It prepares the
** linked list of timers and threads. If not called explicitely, the default
** policy is used (which is the best for the OS), so only call this when you
** know what you are doing.
*/

void setScheduler(int scheduler) {
  o4pe->scheduler = scheduler;
}

/*
** Initialize the emulation environment.
*/

static void oswaldEnvInit(void) {

  static O4pEnv theEnvironment;
  
  o4pe = &theEnvironment;
  
  o4pe->threads = NULL;
  o4pe->staticMemory = malloc(STATIC_MEMORY_SIZE);
  o4pe->status = O4P_ENV_STATUS_INIT;
  o4pe->timer_ticks = 0;
  o4pe->scheduler = DEFAULT_SCHEDULER;

  pthread_mutexattr_init(&o4pe->timerLockAttr);
  pthread_mutexattr_settype(&o4pe->timerLockAttr, PTHREAD_MUTEX_NORMAL);
  pthread_mutex_init(&o4pe->timer_lock, &o4pe->timerLockAttr);

  pthread_mutexattr_init(&o4pe->threadsLockAttr);
  pthread_mutexattr_settype(&o4pe->threadsLockAttr, PTHREAD_MUTEX_NORMAL);
  pthread_mutex_init(&o4pe->threadsLock, &o4pe->threadsLockAttr);

}

/*
 * Prototype:
 *   x_thread x_thread_current(void);
 * Description:     
 *   Returns a pointer to the currently executing thread running
 *   thread.  If no thread is executing, this service returns a null
 *   pointer.
 */

x_thread x_thread_current() {

  return pthread_getspecific(x_thread_key);

}

/*
** x_thread_yield() does nothing.
*/

void x_thread_yield() {}

/*
** Link a thread in our linked list.
*/

void threadRegister(x_thread xnew) {

  int res;
/*
** volatile??? Yeah, well gcc is weird sometimes.
*/
  volatile x_thread current;

  current = NULL;
  res = pthread_mutex_lock(&o4pe->threadsLock);
  if (res != 0) {
    woempa(9, "Lock failed... %d\n", res);
    exit(0);
  }

  xnew->o4p_thread_next = NULL;

  if (o4pe->threads == NULL) {
    o4pe->threads = xnew;
    current = xnew;
  }
  else {
    for (current = o4pe->threads; current != NULL; current = current->o4p_thread_next) {
      if (current->o4p_thread_next == NULL) {
        current->o4p_thread_next = xnew;
        break;
      }
    }
  }

  if (current == NULL) {
    woempa(9, "oops\n");
    exit(0);
  }

  num_x_threads += 1;

  pthread_mutex_unlock(&o4pe->threadsLock);

}

/*
** And the reverse, get rid of a thread in our list.
*/

void threadUnregister(x_thread thread) {

  int res;
  x_thread current;
  x_thread previous;

  res = pthread_mutex_lock(&o4pe->threadsLock);
  if (res != 0) {
    woempa(9, "Lock failed... %d\n", res);
    exit(0);
  }

  previous = NULL;
  for (current = o4pe->threads; current != NULL; current = current->o4p_thread_next) {
    if (current == thread) {
      break;
    }
    previous = current;
  }

  if (current == NULL) {
    /*
    ** This shouldn't happen.
    */
    woempa(9, "Thread %p not found !\n", thread);
    exit(0);
  }
  else if (previous != NULL) {
    /*
    ** Somewhere in the middle or at the end.
    */
    previous->o4p_thread_next = current->o4p_thread_next;
  }
  else {
    /*
    ** This is the first element in the list.
    */
    o4pe->threads = current->o4p_thread_next;
  }

  num_x_threads -= 1;

  pthread_mutex_unlock(&o4pe->threadsLock);

  num_deleted += 1;
//  woempa(9, "Thread unregistered, %d deleted %d started...\n", num_deleted, num_started);
}

int timedWait(pthread_cond_t *cond, pthread_mutex_t *mut, unsigned long *ticks) {

  struct timespec timeout;
  struct timeval now;
  int rval = 0;
  int status;

  while (*ticks) {
    pthread_testcancel();
    gettimeofday(&now, NULL);
    pthread_testcancel();
    if ((now.tv_usec * 1000) + (usecs_per_tick * 1000) >= 1000000000) {
      pthread_testcancel();
      timeout.tv_sec = now.tv_sec + 1;
      pthread_testcancel();
      timeout.tv_nsec = (now.tv_usec * 1000) + (usecs_per_tick * 1000) - 1000000000;
      pthread_testcancel();
    }
    else {
      pthread_testcancel();
      timeout.tv_sec = now.tv_sec;
      pthread_testcancel();
      timeout.tv_nsec = (now.tv_usec * 1000) + (usecs_per_tick * 1000);
      pthread_testcancel();
    }

    status = pthread_cond_timedwait(cond, mut, &timeout);
    pthread_testcancel();
    if (status == 0) {
      /*
      ** We have locked the mutex within the timeout period.
      */
      rval = 1;
      break;
    }
    
    pthread_testcancel();
    (*ticks)--;
    pthread_testcancel();

  }

  return rval;

}

/*
** Initialise a new monitor.
*/
x_status x_monitor_create(x_monitor monitor) {
  woempa(2, "Creating a monitor at %p\n", monitor);
  monitor->owner = NULL;
  monitor->count = 0;
  monitor->waitset = NULL;
  monitor->mutex = malloc(sizeof(pthread_mutex_t));
  monitor->condvar = malloc(sizeof(pthread_cond_t));
  monitor->waitset_mutex = malloc(sizeof(pthread_mutex_t));
  pthread_mutex_init(monitor->mutex, NULL);
  pthread_cond_init(monitor->condvar, NULL);
  pthread_mutex_init(monitor->waitset_mutex, NULL);
  woempa(2, "Monitor at %p has mutex %p, condvar %p, waitset_mutex %p\n", monitor, monitor->mutex, monitor->condvar, monitor->waitset_mutex);

  return xs_success;
}

/*
** Delete a monitor.
*/
x_status x_monitor_delete(x_monitor monitor) {
  x_thread current = x_thread_current();

  if (monitor->owner && monitor->owner != current) {
    woempa(9, "Refuse to delete monitor at %p: owned by %p, caller is %p\n", monitor, monitor->owner, current);

    return xs_not_owner;

  }
  if (!monitor->mutex) {
    woempa(9, "Refuse to delete monitor at %p: already deleted\n", monitor);

    return xs_deleted;

  }
  woempa(2, "Deleting the monitor at %p\n", monitor);

  pthread_mutex_destroy(monitor->mutex);
  free(monitor->mutex);
  monitor->mutex = NULL;
  pthread_cond_destroy(monitor->condvar);
  free(monitor->condvar);
  monitor->condvar = NULL;
  pthread_mutex_destroy(monitor->waitset_mutex);
  free(monitor->waitset_mutex);
  monitor->waitset_mutex = NULL;

  woempa(2, "Monitor at %p deleted\n", monitor);

  return xs_success;
}

/*
** Enter a monitor.
*/
x_status x_monitor_enter(x_monitor monitor, x_sleep timeout) {
  x_thread current = x_thread_current();
  int retcode; 

  woempa(2, "Thread %p is entering monitor %p\n", current, monitor);
  if (monitor->owner == current) {
    monitor->count += 1;
    woempa(2, "Thread %p already owns monitor %p, count now %d\n", current, monitor, monitor->count);
  }
  else if (monitor->owner == NULL) {
    retcode = pthread_mutex_lock(monitor->mutex);
    if (retcode) {
      woempa(9, "x_monitor_enter: pthread_mutex_lock returned %d.\n", retcode);
      return xs_unknown;
    }
    monitor->owner = current;
    monitor->count = 1;
  }
  else {
    if (timeout == x_no_wait) {
      retcode = pthread_mutex_trylock(monitor->mutex);
      if (retcode == EBUSY) {
        woempa(2, "Thread %p tried to obtain monitor %p, mutex was busy\n", current, monitor);
        return xs_no_instance;
      }
      else if (retcode) {
        woempa(9," pthread_mutex_trylock returned %d\n", retcode);
        return xs_unknown;
      }
    }
    else if (timeout == x_eternal) {
      retcode = pthread_mutex_lock(monitor->mutex);
      if (retcode) {
        woempa(9, "x_monitor_enter: pthread_mutex_lock returned %d.\n", retcode);
        return xs_unknown;
      }
    }
    else {
      if (timedWait(monitor->condvar, monitor->mutex, &timeout)) {
      }
      else {
        woempa(9, "x_monitor_enter: timedWait failed.\n");
        return xs_unknown;
      }
    }
    monitor->owner = current;
    monitor->count = 1;
    woempa(2, "Thread %p has obtained monitor %p, count = 1\n", current, monitor);
  }

  return xs_success;
}

/*
** Wait for a monitor
*/
x_status x_monitor_wait(x_monitor monitor, x_sleep timeout) {
  x_thread current = x_thread_current();
  int retcode; 
  x_status status = xs_success;

  if (monitor->owner != current) {
    woempa(9, "Thread %p is not allowed to wait on monitor %p - owner is %p\n", current, monitor, monitor->owner);

    return xs_not_owner;
  }

  woempa(2, "Thread %p is waiting on monitor %p, has count %i\n", current, monitor, monitor->count);

  current->waiting_on = monitor;
  current->waiting_with = monitor->count;

  pthread_mutex_lock(monitor->waitset_mutex);
  if (monitor->waitset == NULL) {
    monitor->waitset = current;
    current->next = current->previous = current;
    woempa(2, "Setting thread %p as waitlist of monitor %p, next is %p, previous is %p\n", current, monitor, current->next, current->previous);
  }
  else {
    list_insert(current, monitor->waitset);
    woempa(2, "Inserted thread %p into waitlist of monitor %p, next is %p, previous is %p\n", current, monitor, current->next, current->previous);
  }

  pthread_mutex_unlock(monitor->waitset_mutex);

  monitor->owner = NULL;
  monitor->count = 0;

  timedWait(monitor->condvar, monitor->mutex, &timeout);
  retcode = sem_trywait(&current->message);
  woempa(2,"Removed message\n");

  monitor->owner = current;
  monitor->count = current->waiting_with;

  current->waiting_on = NULL;
  current->waiting_with = 0;

  woempa(2, "Thread %p has re-acquired monitor %p with count %i\n", current, monitor, monitor->count);
    
  return status;
}

/*
** Notify one thread waiting on a monitor
*/
x_status x_monitor_notify(x_monitor monitor) {
  x_thread current = x_thread_current();
  x_thread other;

  if (monitor->owner != current) {
    woempa(9, "Thread %p is not allowed to notify monitor %p - owner is %p\n", current, monitor, monitor->owner);

    return xs_not_owner;
  }

  pthread_mutex_lock(monitor->waitset_mutex);

  other = monitor->waitset;
  if (other == NULL) {
    woempa(2, "Nobody was waiting on monitor %p\n", monitor);

    return xs_success;
  }
  else if (other == other->previous) {
    woempa(2, "One guy waiting on monitor %p\n", monitor);
    monitor->waitset = NULL;
  }
  else {
    woempa(2, "More guys waiting on monitor %p\n", monitor);
    list_remove(other);
  }
  woempa(2, "Thread %p was waiting on monitor %p - sending a message\n", other, monitor);

  sem_post(&other->message);
  pthread_cond_broadcast(monitor->condvar);  
  pthread_mutex_unlock(monitor->waitset_mutex);

  return xs_success;
}

/*
** Notify all threads waiting on a monitor
*/
x_status x_monitor_notify_all(x_monitor monitor) {
  x_thread current = x_thread_current();
  x_thread other;
  x_status status = xs_success;

  if (monitor->owner != current) {
    woempa(9, "Thread %p is not allowed to notify monitor %p - owner is %p\n", current, monitor, monitor->owner);

    return xs_not_owner;
  }

  pthread_mutex_lock(monitor->waitset_mutex);

  while (status == xs_success) {
    other = monitor->waitset;
    if (other == NULL) {
      break;
    }
    else if (other == other->previous) {
      woempa(2, "One guy waiting on monitor %p\n", monitor);
      monitor->waitset = NULL;
    }
    else {
      woempa(2, "More guys waiting on monitor %p\n", monitor);
      list_remove(other);
    }

    woempa(2, "Thread %p was waiting on monitor %p - sending a message\n", other, monitor);
    sem_post(&other->message);
    pthread_cond_broadcast(monitor->condvar);  
  }

  pthread_mutex_unlock(monitor->waitset_mutex);
  
  return status;
}

/*
** Leave a monitor
*/
x_status x_monitor_exit(x_monitor monitor) {
  x_thread current = x_thread_current();
  int retcode;

  if (monitor->owner != current) {
    woempa(9, "Thread %p cannot leave monitor %p - owner is %p\n", current, monitor, monitor->owner);
    return xs_not_owner;
  }

  woempa(2, "Thread %p is leaving monitor %p, has count %i\n", current, monitor, monitor->count);
  monitor->count -= 1;
  if (monitor->count == 0) {
    monitor->owner = NULL;

    retcode = pthread_mutex_unlock(monitor->mutex);
    if (retcode) {
      woempa(9," pthread_mutex_lock returned %d\n", retcode);

      return xs_unknown;
    }
    woempa(2, "Thread %p no longer owns monitor %p, count now 0\n", current, monitor);
  }
  else {
    woempa(2, "Thread %p still owns monitor %p, count now %d\n", current, monitor, monitor->count);
  }

  return xs_success;
}

x_status x_monitor_stop_waiting(x_monitor monitor, x_thread thread) {
  if (thread->waiting_on && thread->waiting_on == monitor) {
    woempa(2, "Thread %p will stop waiting on monitor %p\n", thread, monitor);
    pthread_mutex_lock(monitor->waitset_mutex);
    if (thread->next == thread) {
      monitor->waitset = NULL;
    }
    else {
      list_remove(thread);
    }
    woempa(2, "Removed thread %p from waitlist of monitor %p, next is %p, previous is %p\n", thread, monitor, thread->next, thread->previous);
    pthread_mutex_unlock(monitor->waitset_mutex);

    woempa(2, "Thread %p was waiting on monitor %p - sending a message\n", thread, monitor);
    return sem_post(&thread->message);
  }

  return xs_no_instance;
}

x_status x_thread_stop_waiting(x_thread thread) {
  x_monitor monitor = thread->waiting_on;
  if (monitor) {
    woempa(2, "Thread %p will stop waiting on monitor %p\n", thread, monitor);
    pthread_mutex_lock(monitor->waitset_mutex);
    if (thread->next == thread) {
      monitor->waitset = NULL;
    }
    else {
      list_remove(thread);
    }
    woempa(2, "Removed thread %p from waitlist of monitor %p, next is %p, previous is %p\n", thread, monitor, thread->next, thread->previous);
    pthread_mutex_unlock(monitor->waitset_mutex);

    woempa(2, "Thread %p was waiting on monitor %p - sending a message\n", thread, monitor);
    return sem_post(&thread->message);
  }

  return xs_no_instance;
}

/* 
 * Prototype:
 *    x_status x_queue_create(x_queue queue, void *queue_start,
 *                         w_size queue_size);
 * Description (of a message queue): 
 *    This service creates a message queue that is typically used for
 *    inter-thread communication.  Message queues are created either
 *    during initialization or during run-time.  Once created, 
 *    messages are copied to a queue by 'x_queue_send' and are copied
 *    from a queue by 'x_queue_receive'.  Each entry in the queue is 
 *    a `void *'.
 *    Application threads can suspend while attempting to send or
 *    receive a message from a queue.  Typically, thread suspension
 *    involves waiting for a message from an empty queue.  However,
 *    it is also possible for a thread to suspend trying to send a
 *    a message to a full queue.
 * Implementation:
 *    POSIX threads don't provide a sufficent queueing mechanism, but
 *    it can be implemented using condition variables and a mutex.
 *    The mutex is used to protect the internal data structure of the 
 *    queue, while the condition variables are used for synchronizing
 *    the producers and consumers.
 */
                                                                                                         
x_status x_queue_create(x_queue queue, void *messages, w_size capacity) {
  pthread_mutex_init(&queue->o4p_queue_mutex , NULL);
  pthread_cond_init(&queue->o4p_queue_not_empty , NULL);
  pthread_cond_init(&queue->o4p_queue_not_full , NULL);

  queue->o4p_queue_deleted = 0;

  queue->messages = (x_word *)messages;
  queue->read = queue->messages;
  queue->write = queue->messages;
  queue->available = 0;
  queue->capacity = capacity;
  queue->limit = (x_word *)messages + capacity;

  return xs_success;
}

/*
 * Prototype:
 *   x_status x_queue_delete(x_queue queue);
 * Description:
 *   Deletes the specified message queue.
 */
 
x_status x_queue_delete(x_queue queue) {
  pthread_mutex_lock(&queue->o4p_queue_mutex);
  queue->o4p_queue_deleted = 1;
  queue->available = 0;
  pthread_mutex_unlock(&queue->o4p_queue_mutex);

  return xs_success;
}

/*
 * Prototype:
 *   x_status x_queue_receive(x_queue queue, void **destination_ptr,
 *                         x_sleep wait_option);
 * Description:
 *   This service retrieves a message from the queue.  The message 
 *   retrieved is copied from the queue into the memory area specified
 *   by the destination pointer.
 * Implementation:
 *   POSIX threads don't provide a sufficent queueing mechanism, but 
 *   it can be implemented using condition variables and a mutex.
 *   The mutex is used to protect the internal data structure of the 
 *   queue, while the condition variables are used for synchronizing
 *   the producers and consumers.
 */
 
unsigned int x_queue_receive(x_queue queue, void **msg, x_sleep owait) {
  struct timeval now;
  struct timespec timeout;
  int seconds;
  long nanoseconds;
  int time_status ;
  x_status status = xs_success;

  pthread_mutex_lock(&queue->o4p_queue_mutex);

  woempa(2, "Receiving element from queue\n");

  /*
  ** We have the lock, either because nobody is reading from the queue, or another thread
  ** has called this function and went into a pthread_cond_wait state, that temporarily
  ** releases the above mutex. Anyhow, we can update the number of readers of the queue.
  */

  if (queue->o4p_queue_deleted == 1) {
    status = xs_deleted;
    goto hastalavista;
  }

  if (queue->available == 0) {
    /*
    ** Nothing here, let's see what we should do...
    */
    if (owait == x_no_wait) {
      status = xs_no_instance;
      goto hastalavista;
    }
    
    pthread_testcancel();
    if (owait == x_eternal) {
      while (queue->available == 0) {
        pthread_cond_wait(&queue->o4p_queue_not_empty, &queue->o4p_queue_mutex);
      }
    }
    else {
      /*
      ** A timed wait, set up the timeout stuff...
      */
      seconds = (int)((owait * usecs_per_tick) / 1000000);
      nanoseconds = 1000000 * ((owait * usecs_per_tick / 1000) - (seconds * 1000));
      woempa(2, "Waiting %d ticks : %ds %ld ns\n", owait, seconds, nanoseconds);
      gettimeofday(&now, NULL);
      timeout.tv_sec = now.tv_sec;
      timeout.tv_nsec = now.tv_usec * 1000;
      timeout.tv_sec += seconds;
      timeout.tv_nsec += nanoseconds;
      while (queue->available == 0) {
        time_status = pthread_cond_timedwait(&queue->o4p_queue_not_empty, &queue->o4p_queue_mutex, &timeout);
        if (time_status == ETIMEDOUT) {
          woempa(2, "Queue seems empty %d\n", queue->available);
          status = xs_no_instance;
          goto hastalavista;
        }
      }
    }
    
  }

  /*
  ** OK, when we have reached this far, everything is OK to read a message...
  */

  *(x_word *)msg = *queue->read;
  queue->read += 1;
  if (queue->read == queue->limit) {
    queue->read = queue->messages;
  }
  queue->available -= 1;

  /*
  ** OK, a message was delivered successfuly, so we can signal waiting writers that there is
  ** room again, let's signal this condition to a writing thread.
  */
  
  pthread_cond_broadcast(&queue->o4p_queue_not_full);  

hastalavista:

  pthread_mutex_unlock(&queue->o4p_queue_mutex);
 
  return status;
}

/*
 * Prototype:
 *   x_status x_queue_send(x_queue queue, void *source_ptr, 
 *                      x_sleep wait_option);
 * Description:
 *   This service sends a message to the specified message queue.  
 *   The message send is copied to the queue from the memory area
 *   specified by the source pointer.
 * Implementation:
 *   POSIX threads don't provide a queueing sufficent mechanism, but
 *   it can be implemented using condition variables and a mutex.  The
 *   mutex is used to protect the internal data structure of the 
 *   queue, while the condition variables are used for synchronizing 
 *   the producers and consumers.
 */
                                                                                                 
unsigned int x_queue_send(x_queue queue, void *msg, x_sleep wait) {
  struct timeval now;
  struct timespec timeout;
  int seconds;
  long nanoseconds;
  int time_status ;
  x_status status = xs_success;

  pthread_mutex_lock(&queue->o4p_queue_mutex);

  /*
  ** See x_queue_receive for info on the following line...
  */

  if (queue->o4p_queue_deleted == 1) {
    status = xs_deleted;
    goto hastalavista;
  }
  
  if (queue->available == queue->capacity) {
    /*
    ** Queue is full, see what we should do...
    */
    if (wait == x_no_wait) {
      status = xs_no_instance;
      goto hastalavista;
    }

    pthread_testcancel();
    if (wait == x_eternal) {
      while (queue->available == queue->capacity) {
        pthread_cond_wait(&queue->o4p_queue_not_full, &queue->o4p_queue_mutex);
      }
    }
    else {
      /*
      ** Setup timeout stuff for a timed wait...
      */
      seconds = (int)((wait * usecs_per_tick) / 1000000);
      nanoseconds = 1000000 * ((wait * usecs_per_tick / 1000) - (seconds * 1000));
      woempa(2, "Waiting %d ticks : %ds %ld ns\n", wait, seconds, nanoseconds);
      gettimeofday(&now, NULL);
      timeout.tv_sec = now.tv_sec;
      timeout.tv_nsec = now.tv_usec * 1000;
      timeout.tv_sec += seconds;
      timeout.tv_nsec += nanoseconds;
      while (queue->available == queue->capacity) {
        time_status = pthread_cond_timedwait(&queue->o4p_queue_not_full, &queue->o4p_queue_mutex, &timeout);
        if (time_status == ETIMEDOUT) {
          woempa(2, "Queue seems full %d\n", queue->available);
          status = xs_no_instance;
          goto hastalavista;
        }
      }
    }
  }

  *queue->write = (x_word)msg;
  queue->write += 1;
  if (queue->write == queue->limit) {
    queue->write = queue->messages;
  }
  queue->available += 1;
  
  /*
  ** Let's signal the fact that something is in the queue to a reading thread.
  */

  pthread_cond_broadcast(&queue->o4p_queue_not_empty);  

hastalavista:

  pthread_mutex_unlock(&queue->o4p_queue_mutex);

  return status;
}

/*
 * Prototype:
 *   x_status x_queue_flush(x_queue queue, void(*do_this)(void *data))
 * Description:
 *   This service retrieves each message from the queue in turn.  The 
 *   function `do_this' is called on each message as it is received.
 * Implementation:
 *   POSIX threads don't provide a sufficent queueing mechanism, but 
 *   it can be implemented using condition variables and a mutex.
 *   The mutex is used to protect the internal data structure of the 
 *   queue, while the condition variables are used for synchronizing
 *   the producers and consumers.
 */
 
unsigned int x_queue_flush(x_queue queue, void(*do_this)(void *data)) {
  x_status status = xs_success;
  void * data;

  pthread_mutex_lock(&queue->o4p_queue_mutex);

  /*
  ** We have the lock, either because nobody is reading from the queue, or another thread
  ** has called this function and went into a pthread_cond_wait state, that temporarily
  ** releases the above mutex. Anyhow, we can update the number of readers of the queue.
  */

  if (queue->o4p_queue_deleted == 1) {
    status = xs_deleted;
    goto hastalavista;
  }

  if (queue->available == 0) {
    status = xs_success;
    goto hastalavista;
  }

  /*
  ** OK, when we have reached this far, everything is OK to read messages ...
  */

  while (queue->available) {
    data = (void *) *queue->read;
    queue->read += 1;
    if (queue->read == queue->limit) {
      queue->read = queue->messages;
    }
    queue->available -= 1;
    do_this(data);
  }
 
  /*
  ** OK, a message was delivered successfuly, so we can signal waiting writers that there is
  ** room again, let's signal this condition to a writing thread.
  */
  
  pthread_cond_broadcast(&queue->o4p_queue_not_full);  

hastalavista:

  pthread_mutex_unlock(&queue->o4p_queue_mutex);
 
  return status;
}

/*
 * Prototype:
 *   x_status x_sem_create(o4p_Semaphore *semaphore_ptr, 
 *                            w_size initial_count);
 * Description (of counting semaphores):
 *   Creates a counting semaphore for inter-thread synchronization.
 *   The initial semaphore count is specified as an input parameter.
 *   A semaphore can be used to guard accesses to a resource, or,
 *   alternatively, just to let processes wait for something to
 *   happen.  The 'x_sem_get' and 'x_sem_put' operations
 *   basicaly decrement and increment an integer count associated with
 *   the semaphore (hence the name).  When a process waits on a
 *   semaphore whose count is positive, then the process just 
 *   continues, having decremented the semaphore count possibly to
 *   zero.  If the semaphore count is zero, then the waiting process
 *   is blocked (suspended) until another process posts the semaphore
 *   by incrementing the semaphore count.
 * Implementation:
 *   Believe it or not, but counting semaphores are provided in
 *   POSIX.4, so basicaly we just have to map the POSIX.4 interface
 *   onto the Oswald API.
 */ 
 
x_status x_sem_create(o4p_Semaphore *semaphore, w_size initial_count) {

  x_status rval = xs_success;

  semaphore->deleted = 0;
  semaphore->count = initial_count;
  semaphore->waiters = 0;

  pthread_mutexattr_init(&semaphore->sem_mutex_att);
  pthread_mutexattr_settype(&semaphore->sem_mutex_att, PTHREAD_MUTEX_NORMAL);
  pthread_mutex_init(&semaphore->sem_mutex , &semaphore->sem_mutex_att);
  pthread_cond_init(&semaphore->sem_cond, NULL);

  return rval;

}

/*
 * Prototype:
 *   x_status x_sem_delete(o4p_Semaphore *semaphore_ptr);
 * Description:
 *   The 'x_sem_delete' operation deletes the specified 
 *   counting semaphore.
 * Implementation:
 *   Believe it or not, but counting semaphores are provided in
 *   POSIX.4, so basicaly we just have to map the POSIX.4 interface
 *   onto the Oswald API.
 */
 
x_status x_sem_delete(o4p_Semaphore *semaphore) {

  x_status rval = xs_success;
  
  if (sem_destroy(&semaphore->posix_sem) == 0) {
    semaphore->deleted = 1;
  }
  else {
    rval = xs_unknown;
  }
  
  return rval;

}

/*
 * Prototype:
 *   x_status x_sem_get(o4p_Semaphore *semaphore_ptr, 
 *                         x_sleep wait_option);
 * Description: 
 *   The get operation decreases the semaphore by one.  If the 
 *   semaphore is 0, the get operation will not be succesful.
 *   Selecting x_no_wait as wait option results in an immidiate
 *   return from this call regardless of whether or not it was
 *   succesful, while selecting x_eternal causes the calling
 *   thread to suspend until a semaphore instance is available.
 * Implementation:
 *   Believe it or not, but counting semaphores are provided in
 *   POSIX.4, so basicaly we just have to map the POSIX.4 interface on
 *   the Oswald interface.  
 */

x_status x_sem_get(o4p_Semaphore *semaphore, x_sleep owait) {

  x_status rval = xs_no_instance;
  int have_it = 0;

  pthread_mutex_lock(&semaphore->sem_mutex);
  
  if (semaphore->count > 0) {
    semaphore->count -= 1;
    pthread_mutex_unlock(&semaphore->sem_mutex);
    have_it = 1;
  }
  else {

    /*
    ** If we have to wait for it, indefinetely or for a while, we unlock the mutex and wait for it,
    ** otherwise, xs_no_instance is returned by default.
    */

    if (owait == x_eternal || owait != x_no_wait) {
      semaphore->waiters += 1;
      while (! have_it) {
        if (owait == x_eternal) {
          pthread_cond_wait(&semaphore->sem_cond, &semaphore->sem_mutex);
        pthread_testcancel();
          if (semaphore->count > 0) {
            semaphore->count -= 1;
            have_it = 1;
          }
        }
        else {
          if (timedWait(&semaphore->sem_cond, &semaphore->sem_mutex, &owait)) {
            if (semaphore->count > 0) {
              semaphore->count -= 1;
              have_it = 1;
            }
          }
          else {
            rval = xs_no_instance;
            break;
          }
        }
      }
      semaphore->waiters -= 1;
    }
    pthread_mutex_unlock(&semaphore->sem_mutex);
  }
  
  return (have_it ? xs_success : rval);

}

/*
 * Prototype:
 *   x_status x_sem_put(o4p_Semaphore *semaphore_ptr);
 * Description:
 *   This service, which is the inverse of the 'x_sem_get'
 *   operation, increases the semaphore by one.  
 * Implementation:
 *   Believe it or not, but counting semaphores are provided in
 *   POSIX.4, so basicaly we just have to map the POSIX.4 interface on
 *   the Oswald interface.  
 */
 
x_status x_sem_put(o4p_Semaphore *semaphore) {

  x_status rval = xs_success;

  pthread_mutex_lock(&semaphore->sem_mutex);
  semaphore->count += 1;
  if (semaphore->waiters) {
//    woempa(9, "%p (%p) putting count %ld waiters %d\n", semaphore, x_thread_current(), semaphore->count, semaphore->waiters);
    pthread_cond_signal(&semaphore->sem_cond);
  }
  pthread_mutex_unlock(&semaphore->sem_mutex);

  return rval;
  
}

void terminate_function(void *thread_ptr) {

  x_thread thread = (x_thread)thread_ptr;

  thread->curStatus = O4P_TERMINATED;

  woempa(1,"Thread %p has terminated\n");
}

/*
** Routine that starts the thread after setting up the pthread thread
*/

void *start_routine(void *thread_ptr) {
#ifdef SIGINT_IS_WABORT
  sigset_t sigset;
#endif
  x_thread thread;

  num_started += 1;
  thread = (x_thread )thread_ptr;
  pthread_setspecific(x_thread_key, thread);
  pthread_cleanup_push(terminate_function, thread);
#ifdef SIGINT_IS_WABORT
  sigfillset(&sigset);
  sigdelset(SIGSTOP);
  pthread_sigmask(SIG_BLOCK, &sigset, NULL);
#endif
  (*(x_entry)thread->o4p_thread_function)(thread->o4p_thread_argument);
  woempa(1,"Thread %p returned normally\n", thread);
  pthread_cleanup_pop(1);

  if (thread->curStatus != O4P_TERMINATED) {
    thread->curStatus = O4P_COMPLETED;
  }
  woempa(1,"Thread %p status is now %d\n", thread, thread->curStatus);

  return thread_ptr;
  
}

/*
 * Prototype:
 *   x_status x_thread_create(x_thread thread_ptr,
 *                        void (*entry_function) (void*), void* entry_input, 
 *                        void *stack_start, w_size stack_size, w_size priority, 
 *                        w_word flags);
 * Description:
 *   This service creates an application thread that starts execution
 *   at the specified task entry function.  The stack, priority, 
 *   preemption, and time-slice are among the attributes specified by
 *   the input parameters.
 *   Threads are the most important part of Oswald.  What exactly is
 *   a thread?  A thread is typically defined as a semi-independent
 *   program segment with a dedicated purpose.  The combined 
 *   processing of all threads makes an application.
 */

x_status x_thread_create(x_thread thread, void (*entry_function)(void*), void* entry_input, void *stack_start, w_size stack_size, w_size priority, w_word flags) {

   int status = 0;
   x_status rval = xs_success;

   if ((thread == NULL) || (entry_function == NULL) || (stack_start == NULL)) {
     if (thread == NULL) woempa(9, "Thread is null\n");
     if (stack_start == NULL) woempa(9, "Stack is null\n");
     rval = xs_bad_argument;
   }
   else if ((priority < 0) || (priority > (NUM_PRIORITIES - 1))) {
     woempa(9, "Prio is %d! EXIT\n", priority);
     rval = xs_bad_argument;
     exit(0);
   }
   else {
     pthread_mutexattr_init(&thread->sleep_timer_att);
     pthread_mutexattr_settype(&thread->sleep_timer_att, PTHREAD_MUTEX_NORMAL);
     pthread_mutex_init(&thread->sleep_timer, &thread->sleep_timer_att);
     pthread_mutex_lock(&thread->sleep_timer);
     pthread_cond_init(&thread->sleep_cond, NULL);
     pthread_mutexattr_init(&thread->o4p_thread_mutex_att);
     pthread_mutexattr_settype(&thread->o4p_thread_mutex_att, PTHREAD_MUTEX_NORMAL);
     pthread_mutex_init(&thread->o4p_thread_mutex, &thread->o4p_thread_mutex_att);
     thread->o4p_thread_stack_start = stack_start;
     thread->o4p_thread_stack_size = stack_size;
     thread->o4p_thread_function = entry_function;
     pthread_attr_init(&thread->attributes);

     sem_init(&thread->message, 0, 0);
     thread->waiting_on = NULL;
     thread->waiting_with = 0;
     list_init(thread);

     /*
     ** Set scheduling and priority
     */

     pthread_attr_setinheritsched(&thread->attributes, PTHREAD_EXPLICIT_SCHED);
     switch (o4pe->scheduler) {
       case S_RR:
         status = pthread_attr_setschedpolicy(&thread->attributes, SCHED_RR);
         thread->o4p_thread_schedPolicy = SCHED_RR;
         break;

       case S_FIFO:
         status = pthread_attr_setschedpolicy(&thread->attributes, SCHED_FIFO);
         thread->o4p_thread_schedPolicy = SCHED_FIFO;
         break;

       case S_OTHER:
         status = pthread_attr_setschedpolicy(&thread->attributes, SCHED_OTHER);
         thread->o4p_thread_schedPolicy = SCHED_OTHER;
         break;
     }
     
     if (status != 0) {
       /*
       ** Fall back to other
       */
       status = pthread_attr_setschedpolicy(&thread->attributes, SCHED_OTHER);
       thread->o4p_thread_schedPolicy = SCHED_OTHER;
     }

     thread->o4p_thread_priority = priority;
     thread->o4p_thread_sched.sched_priority = mapPriority(thread->o4p_thread_schedPolicy, priority);

// Try something new for a start, otherwise we are unable to implement join
// We have to join thread explicitly now when we delete them
// This didn't seem to work anyway (zombies all over the place)
//     pthread_attr_setdetachstate(&thread->attributes, PTHREAD_CREATE_DETACHED);
     thread->o4p_thread_argument = entry_input;

     /*
     ** Set the stack
     */
     
     status = pthread_attr_setstackaddr(&thread->attributes, stack_start);
     if (status != 0) {
       woempa(9, "Set stackaddress failed with %d.\n", status);
       exit(0);
     }
     status = pthread_attr_setstacksize(&thread->attributes, (size_t)stack_size);
     if (status != 0) {
       woempa(9, "Set stacksize failed with %d.\n", status);
       exit(0);
     }
     
     threadRegister(thread);
  
     /*
     ** We don't actually start the POSIX threads emulating our Oswald 
     ** threads here when we are in x_os_main().
     */

     if (o4pe->status == O4P_ENV_STATUS_INIT) {
       if (flags & TF_SUSPENDED) {
         thread->reqStatus = O4P_NEWBORN;
       }
       else {
         thread->reqStatus = O4P_READY;
       }
     }
     else {
       if (flags & TF_SUSPENDED) {
         thread->reqStatus = O4P_NEWBORN;
       }
       else {
         thread->reqStatus = O4P_READY;
         thread->curStatus = O4P_READY;
         status = pthread_create(&thread->o4p_pthread, &thread->attributes, start_routine, (void *)thread);
         /*
         ** We can only set the scheduling parameters when the thread is created!
         */
         pthread_setschedparam(thread->o4p_pthread, thread->o4p_thread_schedPolicy, &thread->o4p_thread_sched);
         if (status != 0) {
           woempa(9, "Exit status of pthread create is %d.\n", status); fflush(stdout);
           exit(0);
         }
       }
     }
     rval = xs_success;

   }

   return rval;
}

/*
 * Prototype:
 *   x_status x_thread_delete(x_thread thread_ptr);
 * Description:
 *   Deletes the specified application thread.  Since the specified
 *   thread must be in a terminated or completed state, this service
 *   cannot be called from a thread attempting to delete itself.
 */

#if defined (linux)
#define DELETE_SLEEP_TIME 100
#elif defined (sun)
#define DELETE_SLEEP_TIME 10
#endif
 
x_status x_thread_delete(x_thread thread) {
  x_status rval = xs_success;
  void *status;

  /*
  ** Check if we are calling ourselves, if yes, 
  ** return an error since this is impossible.
  */

  if (pthread_equal(pthread_self(), thread->o4p_pthread)) {
    rval = xs_bad_state;
  }
  else {
// Is this sleep actually needed ?? It just delays deleting suspended threads extremely
// AND, suspended threads are not removed at all I think ?? FIXTHIS
//      x_thread_sleep(DELETE_SLEEP_TIME);
    threadUnregister(thread);
      
    // make shure the thread is stopped (since threads are no longer detached, we have to do this)
    pthread_join(thread->o4p_pthread, &status);
  }

  return rval;
}

 
/*
 * Prototype:
 *   w_int x_thread_priority_set(x_thread thread_ptr, 
 *                               w_size new_priority);
 * Description:
 *   Changes the priority of the specified thread.  Valid priorities
 *   range from 0 through (NUM_PRIORITIES - 1), where 0 represents the 
 *   highest priority level. 
 */
 
w_int x_thread_priority_set(x_thread thread, w_size new_priority) {
  w_int old_priority = (int) thread->o4p_thread_priority;

  /* does nothing, pthreads priorities don't work in linux anyway */

  return old_priority;

}

w_size x_thread_priority_get(x_thread thread) {
  return thread->o4p_thread_priority;
}
 
/*
 * Prototype:
 *   x_status x_thread_resume(x_thread thread_ptr);
 * Description:
 *   Resumes or prepares for executions a thread that was previously
 *   suspended by a 'x_thread_suspend'-call.  In addition, this
 *   resumes threads that were created without and automatic start.
 */
 
x_status x_thread_resume(x_thread thread) {
  x_status retValue = xs_success;
  int status;

  pthread_mutex_lock(&thread->o4p_thread_mutex);

  if (thread->reqStatus == O4P_NEWBORN) {
    thread->reqStatus = O4P_READY;
    thread->curStatus = O4P_READY;
    status = pthread_create(&thread->o4p_pthread, &thread->attributes, start_routine, (void *)thread);
    pthread_setschedparam(thread->o4p_pthread, thread->o4p_thread_schedPolicy, &thread->o4p_thread_sched);
    if (status != 0) {
      woempa(9, "Exit status of pthread create is %d.\n", status); fflush(stdout);
      exit(0);
    }
  }
  else {
#if defined (sun)
    thr_continue(thread->o4p_pthread);
#else
    pthread_kill(thread->o4p_pthread, SIGCONT);
#endif
  }
  thread->curStatus = O4P_READY;
  
  pthread_mutex_unlock(&thread->o4p_thread_mutex);

  return retValue;
} 

/* Prototype:
 *   x_status x_thread_sleep(x_sleep timer_ticks);
 * Description:
 *   This operation causes the calling trhead to suspend for the
 *   specified number of timer ticks.  Obvioulsy, this operation 
 *   should be called from a thread.  The phycical amount of time 
 *   associated with a timer tick is set too exactly one microsecond.
 */
 
x_status x_thread_sleep(x_sleep timer_ticks) {
  x_thread thread;
  unsigned long millis = x_ticks2usecs(timer_ticks) / 1000;

  /*
  ** Reset our cancellation behaviour first.
  */

  thread = x_thread_current();

  pthread_testcancel();
  
  if (thread == NULL) {
    /*
    ** We were cancelled before we got the chance of starting to sleep...
    */
    return xs_success;
  }

  thread->curStatus = O4P_SLEEP;

  timedWait(&thread->sleep_cond, &thread->sleep_timer, &millis);

  thread->curStatus = O4P_READY;

  /*
  ** Return to asynchronous cancellation behaviour.
  */
  
  return xs_success;
}

/*
 * Prototype:
 *   x_status x_thread_suspend(x_thread thread_ptr);
 * Description:
 *   Suspends the specified application thread.  A thread may call
 *   this service to suspend itself.  Once supsended, the thread must
 *   be resumed by calling 'x_thread_resume' in order to execute
 *   again.
 */

x_status x_thread_suspend(x_thread thread) {  
  x_status retValue = xs_success;
  x_sleep  timer_ticks = 2;

  woempa(9, "Suspending thread %p\n", thread);
  woempa(9, "DONT USE THIS, IS NOT EVEN WORKING A BIT, JUST SIGNALS THREAD TO STOP WHEN HE WANTS TO\n");

  pthread_mutex_lock(&thread->o4p_thread_mutex);

#if defined (sun)
  thr_suspend(thread->o4p_pthread);
#else
  pthread_kill(thread->o4p_pthread, SIGSTOP);
#endif

  woempa(9, "Waiting till suspended\n");

// HACK - give thread time to act on the signal
  timedWait(&thread->sleep_cond, &thread->sleep_timer, &timer_ticks);
  thread->curStatus = O4P_SUSPENDED;

  pthread_mutex_unlock(&thread->o4p_thread_mutex);

  woempa(9, "Suspended\n");

  return retValue;
}

x_status x_thread_join(x_thread thread, void **result, x_sleep timeout) {
  int err;
  x_status status;
  
  thread->curStatus = O4P_SLEEP;

  err = pthread_join(thread->o4p_pthread, result);
  
  woempa(2, "Join reported %s, status %i\n", strerror(err), (int)status);

  switch(err) {
  case 0:
    status = xs_success;
    break;

  case ESRCH: /* no such thread */
    status = xs_bad_element;
    break;

  case EINVAL: /* detached, or other joiner */
    status = xs_bad_state;
    break;

  case EDEADLK: /* is self */
    status = xs_deadlock;
    break;

  default:
    status = xs_unknown;
  }
  
  thread->curStatus = O4P_READY;

  return status;
}

static pthread_mutex_t Scheduler_Mutex = PTHREAD_MUTEX_INITIALIZER;

void x_scheduler_disable(void) {
  pthread_mutex_lock(&Scheduler_Mutex);
}

void x_scheduler_enable(void) {
  pthread_mutex_unlock(&Scheduler_Mutex);
}

pthread_mutex_t *heap_mutex;
static x_thread  heap_owner;
static w_int     heap_claims;

x_sleep x_time_get() {
  return o4pe->timer_ticks;
}

x_status x_mem_lock(x_sleep timeout) {
  return x_monitor_enter(memory_monitor, timeout);
}

x_status x_mem_unlock() {
  return x_monitor_exit(memory_monitor);
}

// ---------------------- USE NATIVE MALLOC ---------------------------
#ifdef USE_NATIVE_MALLOC

const char *magic = "This memory is valid.";

static o4p_Memory_Chunk Memory_Sentinel;
static o4p_memory_chunk memory_sentinel;

static void *chunk2mem(o4p_memory_chunk chunk) {
  return ((char*)chunk) + sizeof(o4p_Memory_Chunk);
}

static o4p_memory_chunk mem2chunk(void *mem) {
  return (o4p_memory_chunk)(((char*)mem) - sizeof(o4p_Memory_Chunk));
}

void x_mem_init(w_ubyte *start) {
// We were given some memory to play with, but we won't use it ...
  free(start);
  memory_sentinel = &Memory_Sentinel;
  list_init(memory_sentinel);
  memory_sentinel->id = 0x31337;
  memory_sentinel->file = __FILE__;
  memory_sentinel->line = __LINE__;
  memory_sentinel->size = 0;
  memory_sentinel->check = (char*)magic;

  memory_monitor = calloc(1, sizeof(x_Monitor));
  x_monitor_create(memory_monitor);
}

void *_x_mem_alloc(w_size size, const char *file, int line) {
  void *new = NULL;

  if (size > MAX_SINGLE_ALLOC) {
    woempa(9,"%s:d Attempt to allocate %d bytes, maximum is %d!\n",file,line,size,MAX_SINGLE_ALLOC);

    return NULL;
  }
  else if (size > heap_remaining) {
    woempa(9,"%s:d Attempt to allocate %d bytes, available space is %d!\n",file,line,size, heap_remaining);

    return NULL;
  }
  else {
    new = calloc((size + 7 + sizeof(o4p_Memory_Chunk)) / 8, 8);
  }

  if (new) {
    o4p_memory_chunk chunk = (o4p_memory_chunk)new;

    chunk->file = (char*)file;
    chunk->line = line;
    chunk->size = size;
    chunk->check = (char*)magic;
    woempa(1,"%s:%d Allocated %d bytes at %p\n", chunk->file, chunk->line, size, new);
    x_mem_lock(x_eternal);
    list_insert(memory_sentinel, chunk);
    heap_remaining -= size + sizeof(o4p_Memory_Chunk);
    woempa(1,"Heap remaining: %d bytes\n", heap_remaining);
    x_mem_unlock();

    return chunk2mem(chunk);
  }
  else {
    woempa(9,"%s:%d Out of memory!  heap_remaining was %d, setting it to 0\n", file, line, heap_remaining);
    heap_remaining = 0;
  }

  return NULL;
}

void *_x_mem_calloc(w_size size, const char *file, int line) {
  void *new = NULL;

  if (size > MAX_SINGLE_ALLOC) {
    woempa(9,"%s:d Attempt to allocate %d bytes, maximum is %d!\n",file,line,size,MAX_SINGLE_ALLOC);

    return NULL;
  }
  else if (size > heap_remaining) {
    woempa(9,"%s:d Attempt to allocate %d bytes, available space is %d!\n",file,line,size, heap_remaining);

    return NULL;
  }
  else {
    w_size calloc_size = (size + sizeof(o4p_Memory_Chunk) + 7) / 8;
    new = calloc(calloc_size, 8);
  }

  if (new) {
    o4p_memory_chunk chunk = (o4p_memory_chunk)new;

    chunk->file = (char*)file;
    chunk->line = line;
    chunk->size = size;
    chunk->check = (char*)magic;
    woempa(1,"%s:%d Allocated %d bytes at %p\n", chunk->file, chunk->line, size, new);
    x_mem_lock(x_eternal);
    list_insert(memory_sentinel, chunk);
    heap_remaining -= size + sizeof(o4p_Memory_Chunk);
    woempa(1,"Heap remaining: %d bytes\n", heap_remaining);
    x_mem_unlock();

    return chunk2mem(chunk);
  }
  else {
    woempa(9,"%s:%d Out of memory!  heap_remaining was %d, setting it to 0\n", file, line, heap_remaining);
    heap_remaining = 0;
  }

  return NULL;
}

void *x_mem_realloc(void *old, w_size size) {
  o4p_memory_chunk oldchunk = mem2chunk(old);
  o4p_memory_chunk newchunk;
  void *new;

  
  if (oldchunk->check != magic) {
    woempa(9,"Memory block %p is not valid!\n", old);

    return NULL;

  }

  if (size <= oldchunk->size) {
    x_mem_lock(x_eternal);
    heap_remaining += oldchunk->size - size;
    oldchunk->size = size;
    x_mem_unlock();
  
    return old;

  }

  new = x_mem_alloc(size);
  newchunk = mem2chunk(new);
  woempa(1, "New chunk is at %p, copying %d bytes from %p to %p\n", newchunk, oldchunk->size, old, new); 
  memcpy(new, old, oldchunk->size);
  woempa(1,"Marking chunk %p with id 0x%x\n", new, oldchunk->id);
  x_mem_tag_set(new, oldchunk->id);
  x_mem_free(old);

  return new;
}

void x_mem_free(void *block) {
  o4p_memory_chunk chunk = mem2chunk(block);
  
  if (chunk->check != magic) {
    woempa(9,"Memory block %p is not valid!\n", block);
  }

  woempa(1,"Returning %d bytes at %p allocated at %s:%d\n", chunk->size, block, chunk->file, chunk->line);
  x_mem_lock(x_eternal);
  list_remove(chunk);
  heap_remaining += chunk->size + sizeof(o4p_Memory_Chunk);
  woempa(1,"Heap remaining: %d bytes\n", heap_remaining);
  x_mem_unlock();

  free(chunk);
}

w_size x_mem_avail(void) {
  return heap_remaining;
}

x_status x_mem_walk(x_sleep timeout, void (*callback)(void * mem, void * arg), void * arg) {
  x_status status = xs_success;
  o4p_memory_chunk cursor;
  o4p_memory_chunk next;
  
  status = x_mem_lock(timeout);
  if (status != xs_success) {
    return status;
  }

  /*
  ** We loop over the chunks. We precalculate the next chunk before we call
  ** the callback for safety reasons.  (It is allowed for the callback function
  ** to release the chunk, for instance).
  */
  
  for (cursor = memory_sentinel->next; cursor != memory_sentinel; cursor = next) {
    next = cursor->next;
    callback(chunk2mem(cursor), arg);
  }

  status = x_mem_unlock();

  return status;
}

x_status x_mem_tag_set(void * mem, w_word tag) {
  o4p_memory_chunk chunk = mem2chunk(mem);
  
  if (chunk->check != magic) {
    woempa(9,"Memory block %p is not valid!\n", mem);

    return xs_unknown;
  }

  woempa(1,"Marking chunk %p (allocated at %s:%d) with id 0x%x\n", mem, chunk->file, chunk->line, tag);
  chunk->id = tag;

  return xs_success;
}

w_word x_mem_tag_get(void * mem) {
  o4p_memory_chunk chunk = mem2chunk(mem);
  
  if (chunk->check != magic) {
    woempa(9,"Memory block %p is not valid!\n", mem);

    return 0;

  }

  woempa(1,"Chunk %p (allocated at %s:%d) has id 0x%x\n", mem, chunk->file, chunk->line, chunk->id);

  return chunk->id;
}

w_size x_mem_size(void * mem) {
  o4p_memory_chunk chunk = mem2chunk(mem);
  
  if (chunk->check != magic) {
    woempa(9,"Memory block %p is not valid!\n", mem);

    return 0;

  }

  woempa(1,"Chunk %p (allocated at %s:%d) has size %d\n", mem, chunk->file, chunk->line, chunk->size);

  return chunk->size;
}

w_boolean x_mem_is_block(void * mem) {
  o4p_memory_chunk chunk = mem2chunk(mem);
  
  return chunk->check == magic;
}

struct collect_result {
  w_size collect_bytes;
  w_size collect_count;
} collect_result;

static void discard_callback(void * mem, void * arg) {
  o4p_memory_chunk chunk = mem2chunk(mem);

  // Really the struct collect_result should be passed as `arg', I'm just being lazy tonight.
  if (isSet(chunk->id, GARBAGE_TAG)) {
    collect_result.collect_bytes += chunk->size;
    collect_result.collect_count += 1;
    x_mem_free(mem);
  }
}

void x_mem_discard(void * block) {
  o4p_memory_chunk chunk = mem2chunk(block);

  setFlag(chunk->id, GARBAGE_TAG);
}

x_status x_mem_collect(w_size * bytes, w_size * num) {
  x_status status = x_mem_walk(x_eternal, discard_callback, &collect_result);
  if (status == xs_success && *bytes) {
    *bytes = collect_result.collect_bytes;
  }
  if (status == xs_success && *num) {
    *num = collect_result.collect_count;
  }

  return status;
}

static x_size heap_size; 

w_size x_mem_total() { 
  return heap_size; 
}

#else
// ---------------------- USE OSWALD MALLOC ---------------------------
#define x_list_insert list_insert
#define x_list_remove list_remove

#define PAGE_SIZE 4096

w_ubyte * heap_start;
w_ubyte * heap_end;
volatile w_ubyte * heap_current;
static volatile w_size free_bytes;

/*
** We define 2 types for use in this package. The 'x_units' type
** indicates the number of double words for a size and the 'x_bytes'
** type indicates the number of bytes for a size.
*/

typedef signed int x_units;
typedef signed int x_bytes;

typedef struct x_Chunk * x_chunk;

typedef struct x_Chunk {
  x_units prev_size;
  x_units this_size;
  x_chunk next;
  x_chunk previous;
} x_Chunk;

/*
** Conversion from a user pointer to a chunk pointer.
*/

inline static x_chunk mem2chunk(void * mem) {
  return (x_chunk)((unsigned char *)mem - 2 * sizeof(x_word));
}

/*
** The masks to extract information out of the 'this_size' field of a chunk.
*/

#define PREVIOUS_IN_USE        0x80000000 // Single bit that indicates if the previous chunk is in use
#define CURRENT_IN_USE         0x40000000 // Single bit that indicates that THIS chunk is in use
#define GARBAGE_TAG            0x20000000 // Piece of memory is garbage, can be reclaimed by our OWN garbage collector
#define TAG_MASK               0x1ff00000 // The tag can hold a 9 bit information NUMBER, numbers 0 - 31 are reserved.
#define USED_UNITS_MASK        0x000fffff // The number of double words units size of a used chunk (20 bits)
#define FREE_UNITS_MASK        0x1fffffff // The mask for the units size of a free chunk (29 bits)
#define TAG_SHIFT                    (20) // The number of bits we have to shift a tag word up to get it into position

/*
NEW BINS INFO (TODO)
*/

/*
** Note that the bin structure is internal to this file and looks exactly like a x_Chunk structure from
** a memory layout point of view. This is *ESSENTIAL* so if you change something, make it such that it looks
** and smells like an x_Chunk, only to the taste I'm indifferent...
*/

typedef struct x_Bin * x_bin;

typedef struct x_Bin {
  x_word dummy1;
  x_word binlock;
  x_chunk next;
  x_chunk previous;
} x_Bin;

/*
** The special top chunk that will always hold the chunk associated with
** the latest sbrk.
*/

static x_chunk top;

/*
** The array of all bins.
*/

static x_Bin *bins;

/*
** The bin that will hold the last remainder of a split chunk. It is the bin at index 1.
*/

static x_chunk last_remainder;

/*
** The chunk that has been allocated initially.
*/

static x_chunk initial;

/*
** Return the bin associated with the index.
*/

inline static x_chunk bin_at(w_size idx) {
  return (x_chunk)&bins[idx];
}

#define last(x)                   ((x)->previous)
#define first(x)                  ((x)->next)
#define bin_empty(bin)            ((bin)->previous == (bin))

/*
** Return the bin, next to this one.
*/

inline static x_chunk next_bin(x_chunk chunk) {
  return (x_chunk)((unsigned char *)chunk + sizeof(x_Bin));
}

inline static void tag_set(x_chunk chunk, x_word tag) {
  chunk->this_size = ((chunk->this_size & (w_size)~TAG_MASK) | ((tag << TAG_SHIFT) & TAG_MASK));
}

/*
** Function to set the 9 bits of the tag information. We need to use a lock for this
** since we touch internally used stuff, i.e. the this_size field.
*/

x_status x_mem_tag_set(void * mem, x_word tag) {

  x_chunk chunk = mem2chunk(mem);
  x_status status;
  
  status = x_mem_lock(x_eternal);
  if (status == xs_success) {
    tag_set(chunk, tag);
    status = x_mem_unlock();
  }
  
  return status;
  
}

/*
** Function to get the 9 bits of the tag information as a word. If the chunk is
** not in use, because it is being reallocated or freed, it returns tag 0.
*/

x_word x_mem_tag_get(void * mem) {

  x_word tag = 0x00000000;
  x_chunk chunk = mem2chunk(mem);

  if (chunk->this_size & CURRENT_IN_USE) {
    tag = (chunk->this_size & TAG_MASK) >> TAG_SHIFT;
  }
  
  return tag;
  
}

/*
** Return the recorded size in bytes of a memory chunk. This returns the number
** of bytes that are available in the chunk, based on the fact that chunk sizes
** are round up to units of 8 bytes. So a call of 'b = x_mem_alloc(22)' will yield a
** 'x_mem_size(b)' result of 28. An allocated chunk carries an overhead of 1 word so we substract
** 4 from the internally carried size, converted from units to bytes.
*/

w_size x_mem_size(void * mem) {
  return ((mem2chunk(mem)->this_size & USED_UNITS_MASK) << 3) - 4;
}

/*
** When RUNTIME_CHECKS is not defined, the static const x_boolean will
** make sure that the compiler will optimize away checking if clauses.
*/

#ifdef RUNTIME_CHECKS
static const x_boolean runtime_checks = true;
#else
static const x_boolean runtime_checks = false;
#endif

static const x_bytes page_size = PAGE_SIZE;
static const x_units trim_keep = PAGE_SIZE >> 3;
static const x_units trim_threshold = (1024 * 128);

/*
** Get memory from the system. Returns NULL when no more memory is
** available in the heap. When the argument is 0, it returns the current
** heap page pointer. (Should go to host, together with heap_end and stuff...)
*/

void * x_sbrk(x_bytes bytes) {

  static x_bytes allocated = 0;
  void * result = (void *)heap_current;
  heap_current += bytes;

  if (heap_current > heap_end) {
    loempa(9, "No more memory: start = 0x%08x new = 0x%08x end = 0x%08x (%d bytes allocated)!!\n", (x_word)heap_start, (x_word)heap_current, (x_word)heap_end, allocated);
    heap_current -= bytes;
    return NULL;
  }  

  if (bytes < 0) {
    loempa(9, "Shrinked %d (0x%08x) bytes, new current = 0x%08x.\n", -bytes, bytes, (x_word)heap_current);
  }

  allocated += bytes;  
  return result;

}

static w_ubyte * sbrk_base  = (w_ubyte *)(-1);
static w_int sbrked_mem = 0;

/*
** The number of available bins.
*/

static const w_size number_bins = 128;

/*
** We always return chunks aligned on double word boundaries.
*/

static const w_size alignment = 8;

/*
** The minimum size that can be allocated is the size of a x_Chunk structure. 
** For unit sizes, we need to divide by 8.
*/

static const x_units minimum_size = (sizeof(x_Chunk) >> 3);

/*
** The maximum size of a chunk in units that a user can request for. 
** The range of the USED_UNITS_MASK. Note that there is always an overhead
** of a word so the real size the user can request for is 1 word less.
*/

static const x_units maximum_size = USED_UNITS_MASK;

/*
** Round up a certain 'value' to the given 'rounding' factor.
*/ 

inline static w_size round_up(w_size value, w_size rounding) {  
  return (value + (rounding - 1)) & ~(rounding - 1);
}

/*
** Round down a certain 'value' to the given 'rounding' factor.
** (not used for the moment)
*/ 

inline static w_size round_down(w_size value, w_size rounding) {  
  return (value) & (w_size)(0 - rounding);
}

/*
** Conversion from a chunk pointer to a user pointer.
*/

inline static void * chunk2mem(x_chunk chunk) {
  return ((unsigned char *)chunk + 2 * sizeof(x_word));
}

/*
** Treat the free space in the current chunk at offset as a new chunk.
*/

inline static x_chunk treat_as_chunk(x_chunk chunk, x_units offset) {
  return (x_chunk)((unsigned char *)chunk + (offset << 3));
}

/*
** Return the in use bit of a chunk at a certain offset. Note that this information is to be 
** searched for in the chunk following this chunk.
*/

inline static x_boolean is_inuse_at_offset(x_chunk chunk, x_units offset) {
  return treat_as_chunk(chunk, offset)->this_size & PREVIOUS_IN_USE;
}

/*
** Return true if a chunk is garbage.
*/

inline static x_boolean is_garbage(x_chunk chunk) {
  return (chunk->this_size & GARBAGE_TAG);
}

/*
** See if the preceeding chunk is in use.
*/

inline static x_boolean previous_is_inuse(x_chunk chunk) {
  return (x_word)(chunk->this_size & PREVIOUS_IN_USE);
}

/*
** Set the in use bit of a chunk at a certain offset.
*/

inline static void set_inuse_at_offset(x_chunk chunk, x_units offset) {
  treat_as_chunk(chunk, offset)->this_size |= PREVIOUS_IN_USE;
}

/*
** Clear the in use bit, the garbage tag and tag contents of a chunk, so that the
** size calculations can be done on free chunks and can use the tag bits space.
*/

inline static void clear_ciu_tag(x_chunk chunk) {
  chunk->this_size &= ~ (CURRENT_IN_USE | GARBAGE_TAG | TAG_MASK);
}

/*
** For a given size, return the index in the array of bins.
*/

inline static w_int bin_index(x_units size) {

  unsigned int divisor = size >> 6;

  if (divisor == 0) {
    return size;
  }
  else {
    if (divisor <= 4) {
      return 56 + (size >> 3);
    }
    else {
      if (divisor <= 20) {
        return 91 + (size >> 6);
      }
      else {
        if (divisor <= 84) {
          return 110 + (size >> 9);
        }
        else {
          if (divisor <= 340) {
            return 119 + (size >> 12);
          }
          else {
            if (divisor <= 1364) {
              return 124 + (size >> 15);
            } 
            else {
              return 126;
            }
          }
        }
      }
    }
  }
  
}

inline static void clear_last_remainder(void) {

  last_remainder->next = last_remainder;
  last_remainder->previous = last_remainder;

}

/*
** Return the index of a bin for a small size request (request < MAX_SMALLBIN_SIZE).
*/

inline static w_size smallbin_index(x_units size) {
  return size;
}

#define MAX_SMALLBIN         63    // We have this number of small bins
#define MAX_SMALLBIN_SIZE    64    // Any request smaller than this, in UNITS, is considered small
#define SMALLBIN_WIDTH        1    // Small bins are sized 8 bytes or 1 UNIT apart

/*
** A request is considered a 'small request' if the corresponding bin and the next bin are small.
*/

inline static w_int is_small_request(x_units size) {
  return (size < MAX_SMALLBIN_SIZE - SMALLBIN_WIDTH);
}

/*
** How a user size is transformed into a normalized size of UNITS. It is
** bumped up with the overhead of a single word and then rounded up to 8 bytes or a unit.
*/

inline static x_units request2units(w_size request) {

  if (request + sizeof(x_word) < (w_size)(minimum_size << 3)) {
    return minimum_size;
  }
  else {
    return (round_up(request + sizeof(x_word), alignment)) >> 3;
  }

}

#ifdef DEBUG /* ---- DEBUG is defined. We have extensive consistency checking ---------------------------- */

/*
** The debugging routines. When DEBUG is not defined, there are inlined
** alternatives that are optimized away. Note that any comment is provided in the debugging
** versions of the routines. The non debug versions don't contain any comment.
*/

#define check_monitor(c) _check_monitor(__LINE__, c)
static void _check_monitor(int l, w_size count) {
}

#define check_status(s) _check_status(__LINE__, s)
static void _check_status(int l, x_status status) {
  if (status != xs_success) {
    loempa(9, "%d: bad status '%s'\n", l, status2char(status));
    exit(0);
  }
}

/*
** From a certain chunk, give the next one...
*/

#define next_chunk(c) _next_chunk(__LINE__, c)
inline static x_chunk _next_chunk(int l, x_chunk chunk) {

  if (chunk->this_size & CURRENT_IN_USE) {
    loempa(9, "%d: Chunk 0x%08x - 0x%08x is in use and next chunk is taken !!\n", l, chunk, chunk->this_size);
    exit(0);
  }

  return (x_chunk)((unsigned char *)chunk + ((chunk->this_size & FREE_UNITS_MASK) << 3));

}

/*
** Return the size of the previous chunk in units, when the previous chunk is free. Otherwise,
** the field chunk->prev_size is part of the data space given to the caller of x_mem_alloc.
*/

#define previous_size(c) _previous_size(__LINE__, c)
inline static x_units _previous_size(int l, x_chunk chunk) {

  if (chunk->this_size & PREVIOUS_IN_USE) {
    loempa(9, "%d: previous size field taken for a chunk that is in use.!\n", l);
    exit(0);
  }

  return chunk->prev_size;

}

/*
** From a certain chunk, give the previous one... (this only works if the previous is free !!)
*/

#define previous_chunk(c) _previous_chunk(__LINE__, c)
inline static x_chunk _previous_chunk(int l, x_chunk chunk) {

  if (chunk->this_size & PREVIOUS_IN_USE) {
    loempa(9, "%d: should not happen\n", l);
    exit(0);
  }

  return treat_as_chunk(chunk, - previous_size(chunk));

}

/*
** Return the in use bit of a chunk. Note that this information is to be 
** searched for in the chunk following this chunk.
*/

#define is_inuse(c) _is_inuse(__LINE__, c)
inline static x_word _is_inuse(int l, x_chunk chunk) {

   if (! (chunk->this_size & CURRENT_IN_USE) && chunk != top) {
     if (treat_as_chunk(chunk, chunk->this_size & FREE_UNITS_MASK)->this_size & PREVIOUS_IN_USE) {
       loempa(9, "%d: Wrong 0x%08x\n", l, chunk);
       exit(0);
     }
   }
   if (chunk->this_size & CURRENT_IN_USE && chunk != top) {
     if (! (treat_as_chunk(chunk, chunk->this_size & USED_UNITS_MASK)->this_size & PREVIOUS_IN_USE)) {
       loempa(9, "%d: Wrong 0x%08x\n", l, chunk);
       exit(0);
     }
   }

  return chunk->this_size & CURRENT_IN_USE;

}

/*
** Get the size of a chunk
*/

#define chunk_size(c) _chunk_size(__LINE__, c)
inline static x_units _chunk_size(int l, x_chunk chunk) {

  if (chunk->this_size & CURRENT_IN_USE) {
    loempa(9, "%d: getting size of in use chunk !\n", l);
    exit(0);
  }

  return chunk->this_size & FREE_UNITS_MASK;

}

/*
** Set the current in use bit of a chunk and set the tag. Note that the
** tag word should have its contents allready shifted into the right position.
*/

#define set_ciu_tag(c, t) _set_ciu_tag(__LINE__, c, t)
inline static void _set_ciu_tag(int l, x_chunk chunk, x_word tag) {

  chunk->this_size &= ~ (GARBAGE_TAG | TAG_MASK);
  chunk->this_size |= (CURRENT_IN_USE | tag);

  /*
  ** We check if the previous in use bit in the next chunk has been set properly since
  ** it pertains to this chunk that is now 'in use'.
  */
  
  if (! treat_as_chunk(chunk, chunk->this_size & USED_UNITS_MASK)->this_size & PREVIOUS_IN_USE) {
    loempa(9, "Problem at line %d.\n", l);
    exit(0);
  }

}

/*
** Set the size of a chunk and set the previous in use bit. This is only
** done on chunks that aren't in use, so we don't bother about the current in
** use bit or the tag bits. In fact, they will be cleared...
*/

#define set_size_set_piu(c, s) _set_size_set_piu(__LINE__, c, s)
inline static void _set_size_set_piu(int l, x_chunk chunk, x_units size) {

  /*
  ** If the chunk is in use and we would set the size beyond the size of an
  ** in use chunk. We complain.
  */
  
  if ((chunk->this_size & CURRENT_IN_USE) && (size & ~USED_UNITS_MASK) != 0) {
    loempa(9, "%d: size = 0x%08x (%d Mb)\n", l, size, (size << 3) / (1024 * 1024));
    exit(0);
  }

  if (size < 0) {
    loempa(9, "Size is negative!\n");
    exit(0);
  }

  if (chunk->this_size & CURRENT_IN_USE) {
    loempa(9, "%d: chunk is in use\n", l);
    exit(0);
  }
  
  chunk->this_size = (PREVIOUS_IN_USE | size);

}

/*
** Set the size of a chunk and leave the previous in use bit and the tag bits.
** Used only in x_mem_realloc.
*/

#define set_size_leave_piu(c, s) _set_size_leave_piu(__LINE__, c, s)
inline static void _set_size_leave_piu(int l, x_chunk chunk, x_units size) {

  if (runtime_checks) {

    /*
    ** If the chunk is in use and we would set the size beyond the size of an
    ** in use chunk. We complain.
    */
  
    if ((chunk->this_size & CURRENT_IN_USE) && (size & ~USED_UNITS_MASK) != 0) {
      loempa(9, "%d: size = 0x%08x (%d Mb)\n", l, size, (size << 3) / (1024 * 1024));
      exit(0);
    }

    if (size < 0) {
      loempa(9, "Size is negative!\n");
      exit(0);
    }

    if (chunk->this_size & CURRENT_IN_USE) {
      loempa(9, "%d: chunk is in use\n", l);
      exit(0);
    }
  }
  
  chunk->this_size = (chunk->this_size & ~FREE_UNITS_MASK) | size;

}

/*
** Set the size at the footer of a chunk that is not in use (free).
*/

#define set_foot(c, s) _set_foot(__LINE__, c, s)
inline static void _set_foot(int l, x_chunk chunk, x_units size) {

  if (chunk->this_size & CURRENT_IN_USE) {
    loempa(9, "Should not happen\n");
    exit(0);
  }

  if (size < 0) {
    loempa(9, "Size is negative|\n");
    exit(0);
  }

  treat_as_chunk(chunk, size)->prev_size = size;

}

/*
** Link the last remainder.
*/

#define link_last_remainder(c, s) _link_last_remainder(__LINE__, c, s)
inline static void _link_last_remainder(int l, x_chunk remainder, x_units remainder_size) {

  clear_ciu_tag(remainder);
  set_size_set_piu(remainder, remainder_size);
  set_foot(remainder, remainder_size);

  last_remainder->next = remainder;
  last_remainder->previous = remainder;
  remainder->next = last_remainder;
  remainder->previous = last_remainder;

}

static x_boolean aligned_OK(void * address) {
  return (round_up((w_size) address, alignment) == (w_size)address);
}

#define check_top(t) _check_top(__LINE__, t)
static void _check_top(int l, x_chunk chunk) { 

  if (is_inuse(chunk)) {
    loempa(9, "%d: top isn't supposed to be in use!\n", l);
    exit(0);
  }
  
  if (chunk_size(chunk) < minimum_size) {
    loempa(9, "%d: top size smaller then minimum %d < %d!\n", l, chunk_size(chunk), minimum_size);
    exit(0);
  }

  if (! aligned_OK(chunk)) {
    loempa(9, "%d: top chunk doesn not align properly.\n", l);
    exit(0);
  }

}


static void _check_chunk(int l, x_chunk chunk) { 

  x_units size;

  size = chunk->this_size & FREE_UNITS_MASK;
  if (chunk->this_size & CURRENT_IN_USE) {
    size &= USED_UNITS_MASK;
  }

  /*
  ** Check if addresses are legal.
  */
    
  x_assert((w_ubyte *)chunk >= sbrk_base);
  if (chunk != top) {
    if (! ((w_ubyte *)chunk + (size << 3) <= (w_ubyte *)top)) {
      loempa(9, "%d: wrong.\n", l);
    }
  }
  else {
    if (! ((w_ubyte *)next_chunk(top) <= sbrk_base + sbrked_mem)) {
      loempa(9, "%d: wrong.\n", l);
    }
  }

}

static void _check_free_chunk(int l, x_chunk chunk) { 

  x_units size = chunk_size(chunk);
  x_chunk next = treat_as_chunk(chunk, size);

  _check_chunk(l, chunk);

  /*
  ** Check wether it really claims to be free.
  */
  
  x_assert(! is_inuse(chunk));

  if (size >= minimum_size) {

    /*
    ** Check wether size aligns OK.
    */
  
    x_assert(((size << 3) & (alignment - 1)) == 0);

    /*
    ** Check footer field.
    */
  
    x_assert(next->prev_size == size);
  
    /*
    ** Check if consolidated correctly.
    */

    x_assert(previous_is_inuse(chunk));
    x_assert(next == top || is_inuse(next));
  
    /*
    ** Minimally check sane links.
    */
  
    x_assert(chunk->next->previous == chunk);
    x_assert(chunk->previous->next == chunk);
  }
  
}

#define check_inuse_chunk(chunk) _check_inuse_chunk(__LINE__, chunk)
static void _check_inuse_chunk(int l, x_chunk chunk) { 

  x_units size;
  x_chunk next;
  x_chunk previous;

  _check_chunk(l, chunk);

  if (! chunk->this_size & CURRENT_IN_USE) {
    loempa(9, "Chunk should be in use!\n");
    exit(0);
  }
  
  size = chunk->this_size & USED_UNITS_MASK;

  next = treat_as_chunk(chunk, size);
    
  /*
  ** Check wether it is surrounded by OK chunks. Free surrounding chunks
  ** can be checked more thoroughly.
  */

  if (! previous_is_inuse(chunk)) {
    previous = previous_chunk(chunk);
    if (_next_chunk(l, previous) != chunk) {
      loempa(9, "%d: Pointers are wrong, top = 0x%08x, lr = 0x%08x, initial = 0x%08x\n", l, top, last_remainder, initial);
      loempa(9, "previous = 0x%08x - 0x%08x, chunk = 0x%08x - 0x%08x - prev 0x%08x\n", previous, previous->this_size, chunk, chunk->this_size, chunk->prev_size);
      loempa(9, "previous->next = 0x%08x\n", previous->next);
      exit(0);
    }
    _check_free_chunk(l, previous);
  }

  if (next == top) {
    x_assert(previous_is_inuse(next));
    x_assert(chunk_size(next) >= minimum_size);
  }
  else if (! _is_inuse(l, next)) {
    _check_free_chunk(l, next);
  }

}

#define check_alloced_chunk(chunk, requested) _check_alloced_chunk(__LINE__, chunk, requested)
static void _check_alloced_chunk(int l, x_chunk chunk, x_units requested) { 

  x_units size = chunk->this_size & USED_UNITS_MASK;
  x_units room = size - requested;

  if (chunk->this_size & TAG_MASK) {
    loempa(9, "%d: tag bits are not 0.\n", l);
    exit(0);
  }

  if (chunk->this_size & GARBAGE_TAG) {
    loempa(9, "%d: garbage bit is not 0.\n", l);
    exit(0);
  }
  
  _check_inuse_chunk(l, chunk);

  /*
  ** Check if sizes are legal.
  */
  
  x_assert(size >= minimum_size);
  x_assert(((size << 3) & (alignment - 1)) == 0);
  x_assert(room >= 0);
  x_assert(room < minimum_size);

  /*
  ** Check wether it was allocated at the front of an available chunk.
  */
  
  x_assert(previous_is_inuse(chunk));
  
}

#else /* ---- DEBUG is not defined ---------------------------------------------------------------- */

inline static x_chunk next_chunk(x_chunk chunk) {
  return (x_chunk)((unsigned char *)chunk + ((chunk->this_size & FREE_UNITS_MASK) << 3));
}

inline static x_units previous_size(x_chunk chunk) {
  return chunk->prev_size;
}

inline static x_chunk previous_chunk(x_chunk chunk) {
  return treat_as_chunk(chunk, - previous_size(chunk));
}

inline static x_word is_inuse(x_chunk chunk) {
  return chunk->this_size & CURRENT_IN_USE;
}

inline static x_units chunk_size(x_chunk chunk) {
  return chunk->this_size & FREE_UNITS_MASK;
}

inline static void set_ciu_tag(x_chunk chunk, x_word tag) {
  chunk->this_size &= ~ (GARBAGE_TAG | TAG_MASK);
  chunk->this_size |= (CURRENT_IN_USE | tag);
}

inline static void set_size_set_piu(x_chunk chunk, x_units size) {
  chunk->this_size = (PREVIOUS_IN_USE | size);
}

inline static void set_size_leave_piu(x_chunk chunk, x_units size) {
  chunk->this_size = (chunk->this_size & ~FREE_UNITS_MASK) | size;
}

inline static void set_foot(x_chunk chunk, x_units size) {
  treat_as_chunk(chunk, size)->prev_size = size;
}

inline static void link_last_remainder(x_chunk remainder, x_units remainder_size) {

  clear_ciu_tag(remainder);
  set_size_set_piu(remainder, remainder_size);
  set_foot(remainder, remainder_size);

  last_remainder->next = remainder;
  last_remainder->previous = remainder;
  remainder->next = last_remainder;
  remainder->previous = last_remainder;

}

#define check_monitor(c)
#define check_status(s)
#define check_top(t)
#define check_free_chunk(chunk)
#define check_inuse_chunk(chunk)
#define check_alloced_chunk(chunk, size)

#endif /* ---- DEBUG is not defined ---------------------------------------------------------------- */

void x_mem_init(w_ubyte * start) {

  w_size i;
  x_chunk chunk;
  w_ubyte *current;
  w_ubyte *aligned;

  // TODO, see if we could get the bins into the remainder of page aligning the memory
  // with not to much code...
  bins = calloc(number_bins, sizeof(x_Bin));
  last_remainder = bin_at(1);
  clear_last_remainder();
  
  for (i = 0; i < number_bins; i++) {
    chunk = bin_at(i);
    chunk->previous = chunk;
    chunk->next = chunk;
  }

  heap_start = start;
  heap_current = start;
  heap_end = start + heap_size;
  loempa(9, "Current heap starts = 0x%08x, end = 0x%08x, bytes = %d\n", heap_start, heap_end, (w_int)(heap_end - heap_start));

  /*
  ** Page align 'heap_current' position and assign it to sbrk_base for internal use.
  */

  current = x_sbrk(0);
  aligned = (w_ubyte *)round_up((w_size)current, (w_size)page_size);
  current = x_sbrk(aligned - current);
  loempa(9, "Correction = %d bytes, current = 0x%08x\n", aligned - current, x_sbrk(0));
  sbrk_base = x_sbrk(0);

  /*
  ** Create the initial and top chunk.
  */

  top = x_sbrk(page_size);
  sbrked_mem = page_size;
  clear_ciu_tag(top);
  set_size_set_piu(top, page_size >> 3);
  initial = top;
  check_top(top);
  free_bytes = heap_end - sbrk_base;
  loempa(9, "Top chunk at 0x%08x, size %d bytes, total of %d bytes free.\n", top, chunk_size(top) << 3, free_bytes);

  memory_monitor = calloc(1, sizeof(x_Monitor));
  x_monitor_create(memory_monitor);
}

/*
** Put 'chunk' in the appropriate bin, in size order, and put it ahead of other
** free chunks of the same size.
*/

inline static void frontlink(x_chunk chunk, x_units size) {

  w_size idx;
  x_chunk bin;
  x_chunk current;

  if (size < MAX_SMALLBIN_SIZE) {
    idx = smallbin_index(size);
    bin = bin_at(idx);
    x_list_insert(bin, chunk);
  }
  else {
    idx = bin_index(size);
    bin = bin_at(idx);
    current = first(bin);
    if (! bin_empty(bin)) {
      while (current != bin && size < chunk_size(current)) {
        current = current->next;
      }
    }
    x_list_insert(current, chunk);
  }
  
}

inline static void extend_top(x_units size) {

  w_ubyte * thebrk;                       // the return value of sbrk
  x_units top_size;                       // the new size of the top chunk
  x_chunk old_top = top;                  // the previous top chunk
  x_units old_top_size = chunk_size(top);
  w_ubyte * old_end = (w_ubyte *)(treat_as_chunk(old_top, old_top_size));
  x_bytes sbrk_size;

  sbrk_size = (size + minimum_size) << 3;
  
  /*
  ** Round to page boundary and get memory from the system.
  */
  
  sbrk_size = round_up((w_size) sbrk_size, (w_size) page_size);
  thebrk = x_sbrk(sbrk_size);
  
  /*
  ** See if we had an intervening (foreign) x_sbrk call that killed the consistency of
  ** our own x_sbrk pages...
  */
  
  if (thebrk == NULL) {
    loempa(9, "sbrk killed %p ?\n", thebrk);
    return;
  }
  
  sbrked_mem += sbrk_size;

  /*
  ** See if we can just grow our current top further...
  */
    
  if (thebrk == old_end) {
    top_size = (sbrk_size >> 3) + old_top_size;
    set_size_set_piu(top, top_size);
    old_end = (w_ubyte *)(treat_as_chunk(top, top_size));
//    loempa(9, "Taking %d more Kb, starting at 0x%08x top = 0x%08x.\n", sbrk_size / 1024, thebrk, old_end);
  }
  else {
    loempa(9, "Hu.. shouldn't happen old_top_size = %d, 0x%08x <-> 0x%08x...\n", old_top_size, thebrk, old_end);
    exit(0);
  }

  check_top(top);
  
}

/*
** The plain vanilla memory allocation function, works just the same as malloc but
** works on a multithreaded system...
*/

void * _x_mem_alloc(w_size request, const char *file, int line) {

  x_status status;
  x_units size = request2units(request);
  w_size idx;
  x_chunk bin;
  x_chunk victim;
  x_units victim_size;
  x_chunk remainder;
  x_units remainder_size;      // This can become negative !

  if (size > maximum_size) {
    return NULL;
  }

  status = x_mem_lock(x_eternal);
  check_status(status);

  free_bytes -= (size << 3);

  if (is_small_request(size)) {
    idx = smallbin_index(size);
    bin = bin_at(idx);
    victim = last(bin);

    /*
    ** If the bin is empty, we can check the next one too, since the remainder would be smaller
    ** than minimum_size; remember that the small bins are spaced 8 bytes apart. So the remainder is
    ** only 8 bytes.
    */
        
    if (bin_empty(bin)) {
      bin = next_bin(bin);
      victim = last(bin);
    }

    if (! bin_empty(bin)) {
      victim_size = chunk_size(victim);
      x_list_remove(victim);
      set_inuse_at_offset(victim, victim_size);
      set_ciu_tag(victim, 0);
      check_alloced_chunk(victim, size);
      check_monitor(1);
      x_mem_unlock();
      return chunk2mem(victim);
    }

    idx += 2;
    
  }
  else {
    idx = bin_index(size);
    bin = bin_at(idx);
    
    for (victim = last(bin); victim != bin; victim = victim->previous) {
      victim_size = chunk_size(victim);
      remainder_size = victim_size - size;

      if (remainder_size >= minimum_size) {
        idx -= 1;
        break;
      }
      else if (remainder_size >= 0) {
        x_list_remove(victim);
        set_inuse_at_offset(victim, victim_size);
        set_ciu_tag(victim, 0);
        check_alloced_chunk(victim, size);
        check_monitor(1);
        x_mem_unlock();
        return chunk2mem(victim);
      }
    }
    
    idx += 1;
    
  }

  /*
  ** Try to use the last split off remainder.
  */
  
  victim = last_remainder->next;
  if (victim != last_remainder) {
    victim_size = chunk_size(victim);
    remainder_size = victim_size - size;
    
    if (remainder_size >= minimum_size) {
      remainder = treat_as_chunk(victim, size);
      set_size_set_piu(victim, size);
      set_ciu_tag(victim, 0);
      link_last_remainder(remainder, remainder_size);
      check_alloced_chunk(victim, size);
      check_monitor(1);
      x_mem_unlock();
      return chunk2mem(victim);
    }

    clear_last_remainder();
    
    if (remainder_size >= 0) {
      set_inuse_at_offset(victim, victim_size);
      set_ciu_tag(victim, 0);
      check_alloced_chunk(victim, size);
      check_monitor(1);
      x_mem_unlock();
      return chunk2mem(victim);
    }

    // place in bin    
    frontlink(victim, victim_size);
    
  }

  /*
  ** Try searching in other bins, we search back from the last chunk in a bin to the front
  ** since the largest chunks are accumulated at the bottom of a bin.
  */

  while (idx < number_bins - 1) {
    bin = bin_at(idx);
    for (victim = last(bin); victim != bin; victim = victim->previous) {
      victim_size = chunk_size(victim);
      remainder_size = victim_size - size;
      if (remainder_size >= minimum_size) {
        remainder = treat_as_chunk(victim, size);
        set_size_set_piu(victim, size);
        set_ciu_tag(victim, 0);
        x_list_remove(victim);
        link_last_remainder(remainder, remainder_size);
        check_alloced_chunk(victim, size);
        check_monitor(1);
        x_mem_unlock();
        return chunk2mem(victim);
      }
      else if (remainder_size >= 0) {
        set_inuse_at_offset(victim, victim_size);
        set_ciu_tag(victim, 0);
        x_list_remove(victim);
        check_alloced_chunk(victim, size);
        check_monitor(1);
        x_mem_unlock();
        return chunk2mem(victim);
      }
    }
    idx += 1;
  }

  /*
  ** Try to extend top...
  */

  if ((remainder_size = chunk_size(top) - size) < minimum_size) {
    extend_top(size);
    if ((remainder_size = chunk_size(top) - size) < minimum_size) {
      check_monitor(1);
      x_mem_unlock();
      loempa(9, "Returning NULL remainder_size = %d (-%d)\n", remainder_size, -remainder_size);
      free_bytes += (size << 3);
      return NULL;
    }
  }

  victim = top;
  set_size_set_piu(victim, size);
  top = treat_as_chunk(victim, size);
  clear_ciu_tag(top);
  set_size_set_piu(top, remainder_size);
  check_top(top);
  set_ciu_tag(victim, 0);
  check_alloced_chunk(victim, size);
  check_monitor(1);
  x_mem_unlock();
  return chunk2mem(victim);
  
}

/*
** ... and the memory allocation function for cleared memory ...
*/

void * _x_mem_calloc(w_size request, const char *file, int line) {

  x_word * block = x_mem_alloc(request);
  x_word * cursor;
  int duffs;

  if (block) {
    cursor = block;
    request = mem2chunk(block)->this_size & USED_UNITS_MASK; // Replace with chunk_size later...
    request *= 2;              // Convert to words
    request -= 1;              // Reduce with chunk overhead of 1 word
    duffs = (request + 7) / 8; // How many switch rounds we have to do
    switch (request % 8) {
      case 0: do { *cursor++ = 0;
      case 7:      *cursor++ = 0;
      case 6:      *cursor++ = 0;
      case 5:      *cursor++ = 0;
      case 4:      *cursor++ = 0;
      case 3:      *cursor++ = 0;
      case 2:      *cursor++ = 0;
      case 1:      *cursor++ = 0;
                 } while (--duffs > 0); 
    }
  }

  return block;
  
}

/*
** Make the chunk the new top with the given size. When the size is too large, we return the
** memory to the system.
*/

inline static void update_top(x_chunk new_top, x_units new_size) {

  x_bytes to_return;

  if (new_size > ((1024 * 1024) >> 3)) {
    loempa(9, "Trimming top with size %d bytes.\n", new_size << 3);  
    to_return = (new_size << 3) - (page_size * 2);
    to_return = round_up((w_size) to_return, (w_size)page_size);
    x_sbrk(- to_return);

    new_size -= (to_return >> 3);
    loempa(9, "Setting new size of top to %d bytes.\n", (new_size << 3));
    sbrked_mem -= to_return;
  }
  
  top = new_top;
  set_size_set_piu(top, new_size);

  check_top(top);
  
}

/*
** ... and the chunk freeing function, is called by x_mem_free ...
*/

static void chunk_free(x_chunk chunk) {

  x_chunk next;
  x_units size;
  x_units next_size;
  x_units prev_size;
  x_boolean islr;
  
  size = chunk_size(chunk);

  free_bytes += (size << 3);

  next = treat_as_chunk(chunk, size);
  
  if (next == top) {
    size += chunk_size(top);
    
    if (! previous_is_inuse(chunk)) {
      prev_size = previous_size(chunk);
      chunk = treat_as_chunk(chunk, - prev_size);
      size += prev_size;
      x_list_remove(chunk);
    }

    update_top(chunk, size);
    return;
  }

  /*
  ** We determine the size of the next chunk. We are carefull since when it is in use,
  ** we need to chop of the tag bits, if not, we keep the full size of the chunk.
  */

//replace following line later with this original when checks are removed next_size = chunk_size(next);
  next_size = next->this_size & FREE_UNITS_MASK;
  if (is_inuse(next)) {
    next_size &= USED_UNITS_MASK;
  }

  /*
  ** Clear the previous in use bit since we'll merge with the previous chunk.
  */
  
  next->this_size &= ~PREVIOUS_IN_USE;
  
  islr = false;
  
  if (! previous_is_inuse(chunk)) {
    prev_size = previous_size(chunk);
    chunk = treat_as_chunk(chunk, - prev_size);
    size += prev_size;
    if (chunk->next == last_remainder) {
      islr = true;
    }
    else {
      x_list_remove(chunk);
    }
  }
  
  if (! (is_inuse_at_offset(next, next_size))) {
    size += next_size;
    if (! islr && next->next == last_remainder) {
      islr = true;
      link_last_remainder(chunk, size);
    }
    else {
      x_list_remove(next);
    }
  }

  set_size_set_piu(chunk, size);
  set_foot(chunk, size);

  if (! islr) {
    frontlink(chunk, size);
  }

}

/*
** ... the user visible memory release function ...
*/

void x_mem_free(void * mem) {

  x_status status;
  x_chunk chunk;
  
  if (mem == NULL) {
    return;
  }
  
  chunk = mem2chunk(mem);

  status = x_mem_lock(x_eternal);
  check_status(status);
  check_inuse_chunk(chunk);
  clear_ciu_tag(chunk);
  chunk_free(chunk);
  status = x_mem_unlock();
  check_status(status);

}

/*
** A copy that is used internally to copy over contents from the source chunk to the
** destination chunk; used in x_mem_realloc.
*/

inline static void realloc_copy(x_chunk destination, x_chunk source) {

  x_word * from = chunk2mem(source);
  x_word * to = chunk2mem(destination);
  w_size words;
  int duffs;

  /*
  ** Copy the contents as words with a Duff's device.
  */

  words = (chunk_size(source) * 2) - 1;
  duffs = (words + 7) / 8;
  switch (words % 8) {
    case 0: do { *to++ = *from++;
    case 7:      *to++ = *from++;
    case 6:      *to++ = *from++;
    case 5:      *to++ = *from++;
    case 4:      *to++ = *from++;
    case 3:      *to++ = *from++;
    case 2:      *to++ = *from++;
    case 1:      *to++ = *from++;
               } while (--duffs > 0); 
  }

}

/*
** ... the memory reallocator ...
*/

void * x_mem_realloc(void * oldmem, w_size requested) {

  x_units req_size;
  x_status status;
  x_units old_size;
  x_chunk old;
  x_units new_size;
  x_chunk new;
  x_units prev_size;
  x_chunk prev;
  x_units next_size;
  x_chunk next;
  x_units rem_size;
  x_chunk rem = NULL;
  x_word tag;
  void * newmem;
  
  /*
  ** If old was NULL, behave as x_mem_alloc.
  */
  
  if (oldmem == NULL) {
    return x_mem_alloc(requested);
  }
  
  req_size = request2units(requested);

  if (req_size > maximum_size) {
    return NULL;
  }
  
  new = old = mem2chunk(oldmem);
  
  status = x_mem_lock(x_eternal);
  check_status(status);

  check_inuse_chunk(old);

  tag = old->this_size & TAG_MASK;

  clear_ciu_tag(old);
  new_size = old_size = chunk_size(old);
    
  /*
  ** Check if we are expanding or shrinking and act accordingly.
  */
  
  if (old_size < req_size) {

    /*
    ** Try expanding forward.
    */
    
    next = treat_as_chunk(old, old_size);
    
    if (next == top || ! is_inuse(next)) {
      next_size = chunk_size(next);

      /*
      ** Try forward into the top if possible (there must be a remainder left after extending 
      ** top, so available top size must be requested size + minimum_size) or otherwise try 
      ** to grow forward into the next chunk that isn't the top chunk.
      */
        
      if (next == top) {
        if (next_size + new_size >= req_size + minimum_size) {
          new_size += next_size;
          top = treat_as_chunk(old, req_size);
          clear_ciu_tag(top);
          set_size_set_piu(top, new_size - req_size);
          set_size_leave_piu(old, req_size);
          set_ciu_tag(old, tag);
          x_mem_unlock();
          return chunk2mem(old);
        }
      }
      else if (next_size + new_size >= req_size) {
        x_list_remove(next);
        new_size += next_size;
        goto split;
      }
    }
    else {
      next = NULL;
      next_size = 0;
    }

    /*
    ** Try backwards to see if we can join with preceeding free space and possibly next chunk free space.
    */
    
    if (! previous_is_inuse(old)) {
      prev = previous_chunk(old);
      prev_size = chunk_size(prev);
      
      /*
      ** Try forward and backward first, to save a later consolidation. Again, we
      ** check that the top chunk, if involved, has remainder space left after the
      ** operation.
      */
      
      if (next != NULL) {
        if (next == top) {
          if (next_size + prev_size + new_size >= req_size + minimum_size) {
            x_list_remove(prev);
            new = prev;
            realloc_copy(new, old);
            new_size += prev_size + next_size;
            top = treat_as_chunk(new, req_size);
            clear_ciu_tag(top);
            set_size_set_piu(top, new_size - req_size);
            set_size_leave_piu(new, req_size);
            set_ciu_tag(new, tag);
            x_mem_unlock();
            return chunk2mem(new);
          }
        }
        else if (next_size + prev_size + new_size >= req_size) {
          x_list_remove(next);
          x_list_remove(prev);
          new = prev;
          new_size += next_size + prev_size;
          realloc_copy(new, old);
          goto split;
        }
      }
        
      /*
      ** Backward only.
      */
        
      if (prev_size + new_size >= req_size) {
        x_list_remove(prev);
        new = prev;
        realloc_copy(new, old);
        new_size += prev_size;
        goto split;
      }
    }

    /*
    ** No avail, we ask x_mem_alloc to help out. If that even returns NULL, we make
    ** sure we bring the old chunk back in pristine order, as we got it.
    */
      
    newmem = x_mem_alloc(requested);
    check_monitor(1);
    if (newmem == NULL) {
      set_ciu_tag(old, tag);
      status = x_mem_unlock();
      check_status(status);
      return newmem;
    }
      
    new = mem2chunk(newmem);

    /*
    ** Check if the new memory comes from a chunk that follows the old memory.
    ** This could only happen when the preceeding x_mem_alloc call extended the top.
    ** If this is the case, we avoid a copy.
    */

    clear_ciu_tag(new);
    if (new == next_chunk(old)) {
      new_size += chunk_size(new);
      new = old;
      goto split;
    }
      
    /*
    ** ... else copy and free the old stuff and return.
    */
      
    realloc_copy(new, old);
    set_ciu_tag(new, tag);
    chunk_free(old);
    check_monitor(1);
    status = x_mem_unlock();
    check_status(status);
    return newmem;
      
  }    

  /*
  ** Split of the extra room in the old chunk (shrinked) or in the expanded chunk (growth).
  */
  
  split:

  if (new_size - req_size >= minimum_size) {
    rem = treat_as_chunk(new, req_size);
    rem_size = new_size - req_size;
    clear_ciu_tag(rem);
    set_size_leave_piu(new, req_size);
    set_size_set_piu(rem, rem_size);
    set_inuse_at_offset(rem, rem_size);
    set_ciu_tag(new, tag);
    chunk_free(rem);
    check_monitor(1);
  }
  else {
    set_size_leave_piu(new, new_size);
    set_inuse_at_offset(new, new_size);
    set_ciu_tag(new, tag);
    check_inuse_chunk(new);
  }

  status = x_mem_unlock();
  check_status(status);

  return chunk2mem(new);
    
}

/*
** Walk over all the memory blocks.
*/

x_status x_mem_walk(x_sleep timeout, void (*callback)(void * mem, void * arg), void * arg) {

  x_chunk cursor;
  x_chunk next;
  x_status status;
  x_units size;
  
  status = x_mem_lock(timeout);
  if (status != xs_success) {
    return status;
  }

  /*
  ** We loop over the chunks. We precalculate the next chunk before we call
  ** the callback for safety reasons.
  */
  
  for (cursor = initial; cursor < top; cursor = next) {
    size = cursor->this_size & FREE_UNITS_MASK;
    if (is_inuse(cursor)) {
      size &= USED_UNITS_MASK;
      next = treat_as_chunk(cursor, size);
      if (! is_garbage(cursor)) {
        callback(chunk2mem(cursor), arg);
      }
    }
    else {
      next = treat_as_chunk(cursor, size);
    }

    if (runtime_checks && size == 0) {
      loempa(9, "Size is 0!\n");
      exit(0);
    }
  }

  status = x_mem_unlock();

  return status;
  
}

/*
** Scan all the memory blocks with a certain tag.
*/

x_status x_mem_scan(x_sleep timeout, x_word tag, x_word mask, void (*callback)(void * mem, void * arg), void * arg) {

  x_chunk cursor;
  x_chunk next;
  x_status status;
  x_units size;
  
  status = x_mem_lock(timeout);
  if (status != xs_success) {
    return status;
  }

  /*
  ** We loop over the chunks. We precalculate the next chunk before we call
  ** the callback for safety reasons. We also precalculate the shifted tag.
  */

  tag = (tag << TAG_SHIFT) & TAG_MASK;
  
  for (cursor = initial; cursor < top; cursor = next) {
    size = cursor->this_size & FREE_UNITS_MASK;
    if (is_inuse(cursor)) {
      size &= USED_UNITS_MASK;
      next = treat_as_chunk(cursor, size);
      if (! is_garbage(cursor)) {
        if ((x_word)(cursor->this_size & TAG_MASK) == tag) {
          callback(chunk2mem(cursor), arg);
        }
      }
    }
    else {
      next = treat_as_chunk(cursor, size);
    }

    if (runtime_checks && size == 0) {
      loempa(9, "Size is 0!\n");
      exit(0);
    }
  }

  status = x_mem_unlock();

  return status;
  
}

/*
** Return true when a  block of memory is a block allocated through the memory
** allocation functions, false otherwise.
*/

x_boolean x_mem_is_block(void * mem) {

  x_chunk cursor;
  x_chunk check = mem2chunk(mem);
  x_boolean is_block = false;
  x_units size;
  x_status status;

  /*
  ** First check if the alignment is OK. It that is not the case, we should
  ** not even bother looking further ...
  */

  if (round_up((w_size) mem, alignment) != (w_size) mem) {
    return false;
  }

  /*
  ** ... now check if it has the correct bounds ...
  **
  ** Note that some of the following like 'top' and 'heap_current' are being checked while
  ** the memory allocator is not locked. They can change but their value will not be changed
  ** such that we get error results.
  */

  if ((check >= top) || (check < (x_chunk)sbrk_base) || (check >= (x_chunk)heap_current)) {
    return false;
  }
  
  /*
  ** ... now check if the chunk is in use ...
  */
  
  if (! check->this_size & CURRENT_IN_USE) {
    return false;
  }

  /*
  ** ... now check if it isn't garbage ...
  */
  
  if (is_garbage(check)) {
    return false;
  }

  /*
  ** ... OK, so the alignment, in use bit, garbage status and bounds are allright, let's dig deeper ...
  */

  status = x_mem_lock(x_eternal);
  check_status(status);
    
  for (cursor = initial; cursor < top; cursor = treat_as_chunk(cursor, size)) {

    if (cursor == check) {
      is_block = true;
      break;
    }

    size = cursor->this_size & FREE_UNITS_MASK;
    if (cursor->this_size & CURRENT_IN_USE) {
      size &= USED_UNITS_MASK;
    }
    
    if (runtime_checks && size == 0) {
      loempa(9, "Size is 0!\n");
      exit(0);
    }

  }

  status = x_mem_unlock();
  check_status(status);

  return is_block;

}

/*
** Get total and free memory.
*/

w_size x_mem_total() {

  x_status status;
  w_size total;
  
  status = x_mem_lock(x_eternal);
  if (status != xs_success) {
    return 0;
  }

  total = heap_end - heap_start;

  x_mem_unlock();

  return total;

}

w_size x_mem_avail() {
  
  x_status status;
  w_size avail;
  
  status = x_mem_lock(x_eternal);
  if (status != xs_success) {
    return 0;
  }

  avail = free_bytes;

  x_mem_unlock();

  return avail;

}

/*
** The 'fast free' routine and the corresponding garbage collector function. It's
** fast since it doesn't lock the memory functions and only sets a flag. Don't get
** too high on using it though, since the memory is not re-used untill it's garbage
** collected by x_mem_collect.
*/

void x_mem_discard(void * mem) {

  x_chunk chunk;

  if (mem) {
    chunk = mem2chunk(mem);
    chunk->this_size |= GARBAGE_TAG;
  }
  
}

x_status x_mem_collect(w_size * bytesr, w_size * numr) {

  x_chunk cursor;
  x_chunk next;
  x_status status;
  x_units size;
  w_size bytes = 0;
  w_size num = 0;
  
  status = x_mem_lock(x_eternal);
  if (status != xs_success) {
    return status;
  }

  for (cursor = initial; cursor < top; cursor = next) {
    size = cursor->this_size & FREE_UNITS_MASK;

    if (is_garbage(cursor)) {
      size &= USED_UNITS_MASK;
      num += 1;

      /*
      ** Coalesce as much garbage blocks as you can find after the current cursor...
      */      
      
      next = treat_as_chunk(cursor, size);
      while (is_garbage(next) && (next < top)) {
        size += next->this_size & USED_UNITS_MASK;
        num += 1;
        next = treat_as_chunk(next, next->this_size & USED_UNITS_MASK);
      }

      /*
      ** Clear the current in use bit, the garbage bit and tag bits, since we are giving this
      ** chunk a new size and the current in use bit needs to be reset before we free the chunk.
      */
      
      cursor->this_size = (cursor->this_size & ~(CURRENT_IN_USE | GARBAGE_TAG | FREE_UNITS_MASK)) | size;
      bytes += (size << 3);

      /*
      ** When the next chunk is free and isn't top, we skip over it for the next time
      ** around since the chunk we set free now, will be coalesced with it.
      */
      
      next = treat_as_chunk(cursor, size);
      if (next < top) {
        if (! (next->this_size & CURRENT_IN_USE)) {
          size += next->this_size & FREE_UNITS_MASK;
          next = treat_as_chunk(cursor, size);
        }
      }
      
      chunk_free(cursor);
    }
    else if (cursor->this_size & CURRENT_IN_USE) {
      size &= USED_UNITS_MASK;
      next = treat_as_chunk(cursor, size);
    }
    else {
      next = treat_as_chunk(cursor, size);
    }
    
  }

  status = x_mem_unlock();

  if (bytesr) {
    *bytesr = bytes;
  }
  
  if (numr) {
    *numr = num;
  }

  return status;
  
}

// --------------------------------------------------------------------
#endif

/*
** Fake implementaion of bogus function
*/
void * x_alloc_static_mem(void * memory, w_size size) {
  return calloc((size+7)/8, 8);
}

#ifdef SIGINT_IS_WABORT
static void handle_SIGINT(int signum) {
  wabort(ABORT_WONKA, "Interrupted!\n");
}

static struct sigaction o4p_sa;
#endif

void x_setup_kernel() {

  x_thread thread;

  pthread_mutex_t sleep_timer;
  pthread_cond_t sleep_cond;
  unsigned long ticks;
  int  countdown;
  void *temp;
    
  oswaldEnvInit();

  pthread_key_create(&x_thread_key, NULL);  

  /*
  ** The heap doesn't exist yet.
  */
  heap_remaining = 0;

  /*
  ** Let the application define some threads. Remember that the oswald kernel doesn't start
  ** scheduling until after the completion of this function. So we set the 
  ** condition.

  */

  x_os_main(command_line_argument_count, command_line_arguments, o4pe->staticMemory);

  heap_remaining = heap_size;
  temp = malloc(heap_remaining);
  while (temp == NULL) {
    heap_remaining = heap_remaining * 15 / 16;
    temp = malloc(heap_remaining);
  }
  free(temp);
  heap_remaining &= 0xfff00000;

  woempa(7,"Proposed heap size = %d (%d MB), using %d (%d MB)\n", heap_size, heap_size / (1024*1024), heap_remaining, heap_remaining / (1024 * 1024));

  heap_mutex = malloc(sizeof(pthread_mutex_t));
  pthread_mutex_init(heap_mutex, NULL);
  heap_owner = NULL;
  heap_claims = 0;

  x_mem_init(malloc(heap_remaining));

  /*
  ** We now set the status to normal and start all threads in the list that have been
  ** set to O4P_AUTO_START.
  */

  o4pe->status = O4P_ENV_STATUS_NORMAL;

  /*
  ** We don't use a mutex, we are sure that we are the only thread running
  ** at this moment.
  */

  for (thread = o4pe->threads; thread; thread = thread->o4p_thread_next) {
    if (thread->reqStatus == O4P_READY) {
      pthread_create(&thread->o4p_pthread, &thread->attributes, start_routine, (void *)thread);
      thread->curStatus = O4P_READY;
    }
  }

#ifdef SIGINT_IS_WABORT
  o4p_sa.sa_handler = handle_SIGINT;
  sigemptyset(&o4p_sa.sa_mask);
  o4p_sa.sa_flags = 0;
  if (sigaction(SIGINT, &o4p_sa, NULL) < 0) {
    wabort(ABORT_WONKA, "Unable to set up signal handler!\n");
  }
#endif

  pthread_mutex_init(&sleep_timer, NULL);
  pthread_cond_init(&sleep_cond, NULL);
               
  /*
  ** Enter the kernel loop which takes care of our timer handling...
  */
  
  countdown = 1000;
  while(1) {
    ticks = 1;
    pthread_mutex_lock(&sleep_timer);
    timedWait(&sleep_cond, &sleep_timer, &ticks);
    pthread_mutex_unlock(&sleep_timer);
    o4pe->timer_ticks++;
    if (--countdown == 0) {
      ficlExec((FICL_VM*)(wonkaPointers->ficlVM),(char*)
        "watchdog "
        "quit"
      );
      countdown = 1000;
    }
  }

  woempa(9, "Should never get here!!\n"); 

} 

x_status x_oswald_init(x_size max_heap, x_size millis) {

  heap_size = max_heap;
  usecs_per_tick = millis * 1000;
  x_setup_kernel();
  
  return xs_success;
}

/*
** Functions for time calculations, mainly copied from oswald
*/

static const x_size ticks_compensation = 2;

/*
** Return a compensated number of ticks for a number of seconds.
*/

x_size x_seconds2ticks(x_size seconds) {
  return (x_size) ((seconds * 1000) / (x_size) (usecs_per_tick / 1000) - ticks_compensation);
}

x_size x_millis2ticks(x_size millis) {

  x_size ticks = (millis / (x_size) (usecs_per_tick / 1000));
  
  return ticks ? ticks : 1;
  
}

x_size x_ticks2usecs(x_size ticks) {
  return (usecs_per_tick * ticks);
}

x_size x_usecs2ticks(x_size usecs) {
  return (usecs < usecs_per_tick) ? 1 : ((x_size)(usecs / usecs_per_tick));
}

