/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: memory.c,v 1.3 2002/03/22 09:26:50 dfrederickx Exp $
*/

#include <windows.h>
#include "oswald.h"

x_memory_chunk		reference_chunk;
x_thread			mem_walk_owner;	// pointer to the function that is currently executing x_mem_walk
Collect_Result		collect_result;	// struct used to hold the feedback of the mem_collect function

CRITICAL_SECTION	w_critical_section;
CRITICAL_SECTION	memory_critical_section;

void* chunk2mem(x_memory_chunk chunk) {
	return (((char*)chunk) + sizeof(x_Memory_Chunk));
}

x_memory_chunk mem2chunk(void* mem) {
	return (x_memory_chunk)(((char*)mem) - sizeof(x_Memory_Chunk));
}

x_size x_mem_total(void) {
	MEMORYSTATUS memoryStatus;

	memset (&memoryStatus, sizeof (MEMORYSTATUS), 0);
	memoryStatus.dwLength = sizeof (MEMORYSTATUS);

	GlobalMemoryStatus (&memoryStatus);

	return memoryStatus.dwTotalPhys;
}
 
x_size x_mem_avail(void){
	MEMORYSTATUS memoryStatus;

	memset (&memoryStatus, sizeof (MEMORYSTATUS), 0);
	memoryStatus.dwLength = sizeof (MEMORYSTATUS);

	GlobalMemoryStatus (&memoryStatus);

	return memoryStatus.dwAvailPhys;
}

x_status x_mem_lock(x_sleep timeout) {
	EnterCriticalSection(&memory_critical_section);
	return xs_success;
}

x_status x_mem_unlock(void) {
	LeaveCriticalSection(&memory_critical_section);
	return xs_success;
}

int mem_init(void) {
	
	loempa(7, "Initialising our memory environment\n");

	InitializeCriticalSection(&memory_critical_section);

	loempa(5,"  +->Initialization of our memory environment was a success...\n");
	return TRUE;
}

void* x_mem_alloc(x_size size){

	void* newMem = NULL;
	HANDLE hNewMem;
	loempa(7, "Allocating our memory heap\n");

	x_mem_lock(x_eternal);

	if(size > MAX_SINGLE_ALLOC){
		loempa(9,"  +->Attempt to allocate %d bytes, maximum memory is %d...\n", size, MAX_SINGLE_ALLOC);
		x_mem_unlock();
		return NULL;
	}
	else if(size > x_mem_avail()){
		loempa(9,"  +->Attempt to allocate %d bytes, available memory is %d...\n", size, x_mem_avail());
		x_mem_unlock();
		return NULL;
	}
	else if( ((float) (x_mem_total() - x_mem_avail())/(float) x_mem_total()) > 0.9 ){
		loempa(9,"  +->Attempt to allocate %d bytes, available memory is %d...\n", size, x_mem_avail());
		loempa(9,"  +->Het percentage is %f...\n", ((float)(x_mem_total() - x_mem_avail())/(float)x_mem_total()));
		x_mem_unlock();
		return NULL;
	}
	else{
        hNewMem = HeapCreate(HEAP_NO_SERIALIZE, 0, MAX_SINGLE_ALLOC);
        if (hNewMem == NULL) {
			loempa(9, "  +->Heap %d created with errorcode %i...\n", GetLastError());
			x_mem_unlock();
			return NULL;
        }

		newMem = HeapAlloc(hNewMem, HEAP_ZERO_MEMORY, size+sizeof(x_Memory_Chunk));
		if(newMem == NULL) {
			loempa(9, "  +->Heap allocated with errorcode %d...\n", GetLastError());
			x_mem_unlock();
			return NULL;
		}
		loempa(5,"  +->We have allocated some memory...\n");
	}

	if(newMem != NULL) {
		x_memory_chunk chunk = (x_memory_chunk) newMem;
		InitializeCriticalSection(&chunk->w_critical_section);
		EnterCriticalSection(&chunk->w_critical_section);
		chunk->size = size;
		chunk->check = TRUE;
		chunk->id = 0;
		chunk->w_hMemory_Chunk = hNewMem;

		if(reference_chunk == NULL){
			list_init(chunk);
			reference_chunk = chunk;
		}
		else{
			list_insert(reference_chunk, chunk);
		}

		LeaveCriticalSection(&chunk->w_critical_section);
		x_mem_unlock();

		return chunk2mem(chunk);
	}
	else {
		loempa(9, "  +->Not enough memory after checking that there was enough...\n");
		return NULL;
	}
}


x_status x_mem_free(void* mem) {

	x_boolean freeError = 0;
	x_boolean destroyError = 0;
	HANDLE hOldMem;

	x_memory_chunk chunk;
	x_mem_lock(x_eternal);
	chunk = mem2chunk(mem);

	loempa(7, "Freeing our memory heap\n");

	if(chunk->check != TRUE){
		loempa(9, "  +->Error, this is not a valid memory block");
		x_mem_unlock();
		return xs_unknown;
	}
	else {
		loempa(5, "  +->Returning %d bytes to our system\n",sizeof(x_memory_chunk)+chunk->size);

		EnterCriticalSection(&chunk->w_critical_section);
		if(chunk == reference_chunk) {
			if(reference_chunk->next == reference_chunk){	//only one chunk = ref_chunk
				loempa(5,"  +->There is only one mem_chunk in the linked list of chunks...\n");
				reference_chunk = NULL;
			}
			else{
				loempa(5,"  +->We are removing the reference chunk...\n");
				reference_chunk = chunk->next;	// make another one the reference_chunk
				list_remove(chunk);
			}
		}
		else{
			list_remove(chunk);
		}

		loempa(5,"  +->Memory chunk %p removed from linked list of chunks\n", chunk);

		hOldMem = chunk->w_hMemory_Chunk;
		LeaveCriticalSection(&chunk->w_critical_section);

		freeError = HeapFree(hOldMem, HEAP_NO_SERIALIZE, chunk);
		if(freeError == 0) {
			loempa(9, "  +->Heap freed with errorcode %d...\n", GetLastError());
			x_mem_unlock();
			return xs_unknown;
		}

		destroyError = HeapDestroy(hOldMem);
		if(destroyError == 0) {
			loempa(9, "  +->Heap destroyed with errorcode %d...\n", GetLastError());
			x_mem_unlock();
			return xs_unknown;
		}
	}
	x_mem_unlock();
	return xs_success;
}

void* x_mem_calloc(x_size size) {
	unsigned int i;
	char* temp;
	void* mem = x_mem_alloc(size);
	loempa(7, "Allocating and clearing memory\n");
	temp = (char*) mem;
	for(i=0;i<size;i++){
		*temp = 0;
		temp++;	
	}
	return mem;
}


void* x_mem_realloc(void* old_mem, x_size size){

	x_memory_chunk new_chunk;
	x_memory_chunk old_chunk;
	loempa(7, "Reallocating a memory heap\n");

	if (old_mem == NULL) {
		loempa(9,"  +->The old memory we received is not valid\n");
		return NULL;
	}

	if(size > MAX_SINGLE_ALLOC){
		loempa(9,"  +->Attempt to allocate %d bytes, maximum memory is %d...\n", size, MAX_SINGLE_ALLOC);
		x_mem_unlock();
		return NULL;
	}
	else if(size > x_mem_avail()){
		loempa(9,"  +->Attempt to allocate %d bytes, available memory is %d...\n", size, x_mem_avail());
		x_mem_unlock();
		return NULL;
	}
	else if( ((float) (x_mem_total() - x_mem_avail())/(float) x_mem_total()) > 0.9 ){
		loempa(9,"  +->Attempt to allocate %d bytes, available memory is %d...\n", size, x_mem_avail());
		loempa(9,"  +->Het percentage is %f...\n", ((float)(x_mem_total() - x_mem_avail())/(float)x_mem_total()));
		x_mem_unlock();
		return NULL;
	}

	x_mem_lock(x_eternal);
	old_chunk = mem2chunk(old_mem);
	if(old_chunk->check != TRUE){
		loempa(9,"  +->Block of memory %p is not a valid block\n", old_mem);
		x_mem_unlock();
		return NULL;
	}

	new_chunk = HeapReAlloc(old_chunk->w_hMemory_Chunk, HEAP_ZERO_MEMORY, old_mem, size);

	if(new_chunk == NULL) {
		loempa(9, "  +->Heap reallocated with errorcode %d...\n", GetLastError());
		x_mem_unlock();
		return NULL;
	}
	
	loempa(5,"  +->We have reallocated some memory...\n");

	x_mem_unlock();

	return new_chunk;
}

x_status x_mem_tag_set(void* mem, x_word tag) {

	x_memory_chunk chunk = mem2chunk(mem);  
	loempa(7, "x_mem_tag_set\n");

	if (chunk->check != TRUE) {
		loempa(9,"  +->Memory block %p is not valid!\n", mem);
		return xs_unknown;
	}

	loempa(5,"  +->Marking chunk %p with id 0x%x\n", mem, tag);
	chunk->id = tag;

	return xs_success;
}

x_word x_mem_tag_get(void * mem) {

	x_memory_chunk chunk = mem2chunk(mem);
	loempa(7, "x_mem_tag_get\n");

	if (chunk->check != TRUE) {
		loempa(9,"  +->Memory block %p is not valid!\n", mem);
		return 0;
	}

	loempa(5,"  +->Chunk %p has id 0x%x\n", mem, chunk->id);
	return chunk->id;
}

x_size x_mem_size(void * mem) {
	x_memory_chunk chunk = mem2chunk(mem);
	
	loempa(7, "x_mem_size\n");

	if (chunk->check != TRUE) {
		loempa(9,"  +->Memory block %p is not valid!\n", mem);
		return 0;
	}

	loempa(5,"  +->Chunk %p has size %d\n", mem, chunk->size);
	return chunk->size;
}

x_boolean x_mem_is_block(void * mem) {
	x_memory_chunk chunk = mem2chunk(mem);
	loempa(7, "x_mem_is_block\n");
	return (chunk->check == TRUE);
}

x_status x_mem_walk(x_sleep timeout, void (*callback)(void * mem, void * arg), void * arg) {

	int number;	//number of times we have to execute the for loop
	int counter;
	x_status status = xs_success;
	x_memory_chunk cursor;
	x_memory_chunk next;
	
	loempa(7, "x_mem_walk\n");

	if(reference_chunk == NULL){
		loempa(9,"  +->No memory to walk over\n");
		return xs_no_instance;
	}
	if(mem_walk_owner == x_thread_current()){		// check and return is consistent with oswald API
		loempa(9,"  +->Thread %d is calling x_mem_walk more than once\n",x_thread_current()->w_thread);
		return xs_bad_context;
	}

	status = x_mem_lock(timeout);
	if (status != xs_success) {
		return status;
	}
	
	mem_walk_owner = x_thread_current();
	
	// calculate the number of times we have to execute the for loop
	// we must use this method and the method of o4p because
	// => doesn`t work in our case when you want to collect mem and everything is garbage
	number = 0;
	for (cursor = reference_chunk->next; cursor != reference_chunk; cursor = next) {
		next = cursor->next;
		number++;
	}
	number++;	//the reference chunk also counts

	// execute the walk
	// we get the next chunk before calling the callback function
	// matter of safety => suppose the chunk is messed with and becomes invalid
	cursor = reference_chunk;
	for (counter=0; counter < number; counter++) {
		next = cursor->next;
		callback(chunk2mem(cursor), arg);
		cursor = next;
	}
	mem_walk_owner = NULL;
	status = x_mem_unlock();

	return status;
}

static void discard_callback(void * mem, void * arg) {
	x_memory_chunk chunk = mem2chunk(mem);
	collect_result = (*(Collect_Result*) arg);
	
	loempa(7, "discard_callback\n");
	if (isSet(chunk->id, GARBAGE_TAG)) {
		collect_result.collect_bytes += chunk->size;
		collect_result.collect_count += 1;
		x_mem_free(mem);
	}
}

void x_mem_discard(void * block) {

	loempa(7, "x_mem_discard\n");
	if(block != NULL){
		x_memory_chunk chunk = mem2chunk(block);
		if(chunk->check != TRUE){
			loempa(9,"  +->Not a valid memory block to discard\n");
		}
		else{
			loempa(5,"  +->Discarding memory block %p \n",block);
			setFlag(chunk->id, GARBAGE_TAG);
		}
	}
	else
		loempa(9,"  +->Memory block you are trying to discard == null\n");
}

x_status x_mem_collect(x_size * bytes, x_size * num) {
	x_status status = x_mem_walk(x_eternal, discard_callback, &collect_result);
	if (status == xs_success && bytes != NULL) {
		*bytes = collect_result.collect_bytes;
	}
	if (status == xs_success && num != NULL) {
		*num = collect_result.collect_count;
	}
	return status;
}

// allocate static memory out of our static memory pool of size STATIC_MEM
// we don't use the memory pointer, we have an implementation without it
void* x_alloc_static_mem(void* memory, x_size size){

	void* newMem;
	HANDLE hNewMem;

	loempa(7, "Allocating our memory pool\n");

	hNewMem = HeapCreate(HEAP_NO_SERIALIZE, 0, MAX_SINGLE_ALLOC);
	if (hNewMem == NULL) {
		loempa(9, "  +->Heap %d created with errorcode %d...\n", GetLastError());
		x_mem_unlock();
		return NULL;
	}

	newMem = HeapAlloc(hNewMem, HEAP_ZERO_MEMORY, size);
	if(newMem == NULL) {
		loempa(9, "  +->Heap %d allocated with errorcode %d...\n", GetLastError());
		return NULL;
	}

	else
		return newMem;
}
