/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: semaphore.c,v 1.3 2002/03/22 15:24:57 dfrederickx Exp $
*/

#include <stdlib.h>

#include <windows.h>
#include "oswald.h"

/*
 * Prototype:
 *   x_status x_sem_create(o4w_Semaphore *semaphore, x_size initial_count) {
 * Description (of counting semaphores):
 *   Creates a counting semaphore for inter-thread synchronization.
 *   The initial semaphore count is specified as an input parameter.
 *   A semaphore can be used to guard accesses to a resource, or,
 *   alternatively, just to let processes wait for something to
 *   happen.  The 'x_sem_get' and 'x_sem_put' operations
 *   basicaly decrement and increment an integer count associated with
 *   the semaphore (hence the name).  When a process waits on a
 *   semaphore whose count is positive, then the process just 
 *   continues, having decremented the semaphore count possibly to
 *   zero.  If the semaphore count is zero, then the waiting process
 *   is blocked (suspended) until another process posts the semaphore
 *   by incrementing the semaphore count.
 */ 
 
x_status x_sem_create(o4w_Semaphore *semaphore, x_size initial_count) {
	InitializeCriticalSection(&semaphore->w_critical_section);
	__try {
		EnterCriticalSection(&semaphore->w_critical_section);
			loempa(7,"o4w - semaphore - create begin\n");
		semaphore->count = 0;
		semaphore->w_hEvent=CreateSemaphore(NULL, 100, 100, NULL);
		loempa(7,"o4w - semaphore - %d\n",semaphore->w_hEvent);
		semaphore->deleted = 0;
		semaphore->waiters = 0;
	}
	__finally {
		LeaveCriticalSection(&semaphore->w_critical_section);
	}
		loempa(7,"o4w - semaphore - created\n");
	return xs_success;
}

 
/*
 * Prototype:
 *   x_status x_sem_delete(o4p_Semaphore *semaphore_ptr);
 * Description:
 *   The 'x_sem_delete' operation deletes the specified 
 *   counting semaphore.
 */
 
x_status x_sem_delete(o4w_Semaphore *semaphore) {
  	EnterCriticalSection(&semaphore->w_critical_section);
	if (!semaphore->w_hEvent) {
			loempa(7, "Refuse to delete semaphore at %p: already deleted\n", semaphore);
		return xs_deleted;
	}
		loempa(7, "Deleting the semaphore at %p\n", semaphore);
	CloseHandle(semaphore->w_hEvent);
	semaphore->w_hEvent	= NULL;
	semaphore->deleted	= 1;
	semaphore->count	= 0;
	semaphore->waiters	= 0;
	semaphore->current	= 0;
	LeaveCriticalSection(&semaphore->w_critical_section);
	DeleteCriticalSection(&semaphore->w_critical_section);
		loempa(7, "semaphore at %p deleted\n", semaphore);
	return xs_success;
}
  
/*
 * Prototype:
 *   x_status x_sem_get(o4p_Semaphore *semaphore_ptr, 
 *                         x_sleep wait_option);
 * Description: 
 *   The get operation decreases the semaphore by one.  If the 
 *   semaphore is 0, the get operation will not be succesful.
 *   Selecting x_no_wait as wait option results in an immidiate
 *   return from this call regardless of whether or not it was
 *   succesful, while selecting x_eternal causes the calling
 *   thread to suspend until a semaphore instance is available.
 */

x_status x_sem_get(o4w_Semaphore *semaphore, x_sleep owait) {
	x_status rval= xs_no_instance;
	x_boolean value;
	long lPrev;
	int have_it = 0;
	loempa(5,"o4w - semaphore - The amount of semaphores :  %d\n",semaphore->count);
	loempa(7,"o4w - semaphore - semaphore get begin\n");
	loempa(1,"o4w - semaphore - %d\n",semaphore->w_hEvent);
	
	if (semaphore->count < 100) {
		value=ReleaseSemaphore(semaphore->w_hEvent,1,&lPrev);
				loempa(7,"o4w - semaphore - OK\n");
		if (value!=0){
			semaphore->count += 1;
			have_it = 1;
			rval=xs_success;
		}
	}
	else {
		__try {
			EnterCriticalSection(&semaphore->w_critical_section);
			if (semaphore->deleted ==1) {
					loempa(7,"o4w - semaphore - semaphore already deleted\n");
				rval = xs_deleted;
				exit(0);
			}
			else{
				if( owait==x_eternal||owait!=x_no_wait){
					semaphore->waiters +=1;
					while(!have_it){
						if(owait==x_eternal){
							LeaveCriticalSection(&semaphore->w_critical_section);
							SetLastError(0);
								loempa(1,"o4w - semaphore - %d\n",semaphore->w_hEvent);
	
							PulseEvent(semaphore->w_hEvent);		// tell the ones that are trying to enter that they may enter
								loempa(1,"o4w - semaphore - %d\n",semaphore->w_hEvent);
							if(GetLastError() != 0) {
									loempa(9, "  +->Event for semaphore pulsed with errorcode %d...\n", GetLastError());
							
							}
							loempa(7,"o4w - semaphore - leave critical to wait \n");
							WaitForSingleObject(semaphore->w_hEvent,INFINITE);
							EnterCriticalSection(&semaphore->w_critical_section);
								loempa(7,"o4w - semaphore - getsemaphore after a while\n");
							if(semaphore->count<100){
								value=ReleaseSemaphore(semaphore->w_hEvent,1,&lPrev);
								if (value!=0){
									loempa(3,"o4w - semaphore - OK2 \n");
									semaphore->count += 1;
									have_it = 1;
									rval=xs_success;
								}
							}
						}
						else{
							if (WaitForSingleObject(semaphore->w_hEvent, owait) != WAIT_TIMEOUT){
								if(semaphore->count <100){
									value=ReleaseSemaphore(semaphore->w_hEvent,1,&lPrev);
									if (value!=0){
											loempa(7,"o4w - semaphore - OK3\n");
										semaphore->count += 1;
										have_it = 1;
										rval=xs_success;
									}
								}
							}
							else{
								rval=xs_no_instance;
								break;
							}
						}
					}
					semaphore->waiters -=1;
				}
			}
		}
	  	__finally {
			LeaveCriticalSection(&semaphore->w_critical_section);
		}
	}

		loempa(7,"o4w - semaphore - einde zoeken naar semaphoor\n");
	return (have_it ? xs_success : rval);
}


x_status x_sem_put(o4w_Semaphore *semaphore) {
	x_status rval = xs_success;
		loempa(7,"o4w - semaphore - The current count of semaphores PUT %d",&semaphore->count);

	__try {
		EnterCriticalSection(&semaphore->w_critical_section);
		SetLastError(0);
		semaphore->w_hEvent=CreateSemaphore(NULL,semaphore->count,100,NULL);
		if(GetLastError()==ERROR_ALREADY_EXISTS){
			semaphore->count -=1;
				loempa(1,"o4w - semaphore - found semaphore %d",&semaphore->w_hEvent);

			if(semaphore->waiters) {
				PulseEvent(semaphore->w_hEvent);

			}
		}
		else
			loempa(9,"o4w - semaphore -couldn't find semaphore %d",&semaphore->w_hEvent);
	}
	__finally {
		LeaveCriticalSection(&semaphore->w_critical_section);
	}
	return rval;
}
