#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "oswald.h"
#include "o4w.h"
#include "main_test.h"
#include "scheduler_test.h"


/*
** Test program to test the proper working of our threads
** Principle: A control thread makes 20 threads
** The control thread waits until they are all finished
** 10 threads sleep randomly
** 5 threads suspend themself or another thread
** 5 threads execute a infinite program but are deleted by the 5 previous threads (the suspend threads)
*/

extern O4wEnv * o4wenv;
// the test program
void testprogram(void* args){

	int y;
	loempa(9,"+-------------------------+   Starting up the testprogram   +-------------------------+\n");
    create_mempools();

 
    for(;;){	//forever, forever...
		
		loempa(9,"Number of threads: %d\n", o4wenv->num_threads);
		
		alloc_mempools();	
		
		do{	// control thread must be the only one running => again our hole test	    
			
			for(y=0;y<10;y++){
				x_thread_create(struct_mem[y],&sleepprog,(void*)y,NULL,1,5,TF_START);
			}
			for(y=10;y<15;y++){
				x_thread_create(struct_mem[y],&suspendprog,(void*)y,NULL,1,5,TF_START);
			}
			for(y=15;y<20;y++){
				x_thread_create(struct_mem[y],&stupidprog,(void*)y,NULL,1,5,TF_START);
			}
			
			WaitForAll();
			loempa(9,"Number of threads: %d\n", o4wenv->num_threads);
			
		} while(o4wenv->num_threads > 1);

		free_mempools();
	}

}

void create_mempools(void){
    DWORD exitSucces = 0;
    loempa(9,"Creating mempools\n");
	mempool1 = HeapCreate(HEAP_NO_SERIALIZE, 21 * sizeof(x_Thread), 0);
	exitSucces = GetLastError();
	loempa(5, "  +->mempool 1 created with errorcode %d...\n", exitSucces);
}

void alloc_mempools(void){
    int u;
    loempa(9,"Allocating mempools\n");
    for(u=0;u<20;u++){
		struct_mem[u] = HeapAlloc(mempool1, HEAP_ZERO_MEMORY, sizeof(x_Thread));
    }
}

void free_mempools(void){	
    int u;
    loempa(9,"Freeing mempools\n");
    for(u=0;u<20;u++){
		HeapFree(mempool1, HEAP_NO_SERIALIZE, struct_mem[u]);
    }
}

void WaitForAll(void){
	int i;
	loempa(9,"Waiting for other threads to finish\n");
	for(i=0;i<15;i++){
		loempa(7, "  +->Waiting on thread %d to finish\n", ((x_thread)struct_mem[i])->w_thread);
		while (x_thread_join((x_thread)struct_mem[i],NULL,0) != xs_success ){ 
			Sleep(1000);
		}
		loempa(7, "  +->Thread %d finished\n", ((x_thread)struct_mem[i])->w_thread);
		x_thread_delete((x_thread)struct_mem[i]);		
	}
	loempa(7,"  +->Done waiting for other threads to finish\n");
}

// suspend the next one in the line
// last one suspends itself (if already suspended => no problemo)
// setPrio to 4 => wait until the others are finished
// then resume threads

void sleepprog(void* args){
    int delay;
	loempa(9,"+-------------------------+   Starting up the sleep program   +-------------------------+\n");
    delay = 200 + (rand() % 50);
    loempa(9,"Thread %d is going to sleep for %d clock ticks\n", GetCurrentThreadId(), delay);
    x_thread_sleep(delay);
    loempa(9,"Thread %d is going to sleep for %d clock ticks again\n", GetCurrentThreadId(), delay);
    x_thread_sleep(delay);
}

void suspendprog(void* arg){
	int data = (int) arg;
	loempa(9,"+-------------------------+   Starting up the suspend program   +-------------------------+\n");

	if(data < 14)
		x_thread_suspend((x_thread)struct_mem[data+1]);
	else
		x_thread_suspend((x_thread)struct_mem[14]);

	x_thread_priority_set(x_thread_current(),8);
	x_thread_yield();	// yield to the next one in line

	if(data < 14)
		x_thread_resume((x_thread)struct_mem[data+1]);
	else
		x_thread_resume((x_thread)struct_mem[14]);

	//try to delete the ones that are doing the stupid program
	//will not work => not suspended or completed
	x_thread_delete((x_thread)struct_mem[data+5]);
	
	// suspends the threads => then delete them
	x_thread_suspend((x_thread)struct_mem[data+5]);
	x_thread_delete((x_thread)struct_mem[data+5]);
}

void stupidprog(void* arg){
   	int temp;
	int temp2;
	loempa(9,"+-------------------------+   Starting up the stupid program   +-------------------------+\n");
	for(;;){	// forever 
		temp=0;
		temp2=0;
		while(temp<20000){
			temp++;
			temp2 = temp / 2;
			temp2 = temp2 + temp + temp2/2;
		}
		x_thread_priority_set(x_thread_current(),8); 	//will be deleted because the other thread 
		// with prio 8 will do this
	}
}


