#ifndef _ATOMIC_H
#define _ATOMIC_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: atomic.h,v 1.1.1.1 2001/11/21 10:41:33 pollock Exp $
*/

x_word peek_swap(void *address);

inline static x_boolean compare_swap(void *address, x_word old, x_word new) {

  x_boolean result;
  
  asm volatile (
    "mov %0, #1\n\t"               /* Use result register as backoff counter, set to 1 initially. */
    "1:\n\t"                       /* Check if somebody else is attempting a swap ...             */
    "swp   %1, %1, [%2]\n\t"       /* Do the swap with our 'lock in progress' indicator.          */
    "cmn   %1, #1\n\t"             /* Compare the result with 'lock in progress' indicator (-1).  */
    "bne   2f\n\t"                 /* If no lock in progress, proceed to compare 'old' and 'new'  */
    "stmfd sp!, {r0 - r4}\n\t"     /* A lock is in progress, prepare to call backoff              */
    "mov   r0, %0\n\t"             /* Prepare backoff count argument                              */
    "bl    x_spinlock_backoff\n\t" /* Call backoff function                                       */
    "ldmfd sp!, {r0 - r4}\n\t"     /* do we have to save r4 also ?? me don't think so...          */
    "add   %0, %0, #1\n\t"         /* Increase backoff counter for possible next time.            */
    "b     1b\n\t"                 /* Try again check for 'lock in progress'.                     */
    "2:\n\t"                       /* OK, we're in the critical section.                          */
    "cmp   %1, %3\n\t"             /* See if the contents at 'address' are the same as 'old'.     */
    "streq %4, [%2]\n\t"           /* Yes, we replace with 'new' ...                              */
    "moveq %0, #1\n\t"             /* ... and return success.                                     */
    "strne %1, [%2]\n\t"           /* ... nope, replace old contents ...                          */
    "movne %0, #0\n\t"             /* ... and return failure.                                     */
    : /* outputs */
      "=r" (result)
    : /* inputs */
      "r" (-1), 
      "r" (address),
      "r" (old),
      "r" (new)
  );

  return result;
    
}

#endif /* _ATOMIC_H */