#ifndef _BITS_H
#define _BITS_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

#include <oswald.h>

/*
** $Id: bits.h,v 1.1.1.1 2001/11/21 10:41:33 pollock Exp $
**
** This header file declares functions for bit operations either as macros, inline functions
** or function declarations.
*/

/*
** See if a certain set of flags is set. We use inline functions since it is neater
** to return a value to the caller. When we pass arguments that need changing, we 
** use macros as is illustrated below with the 'setFlag' and 'unsetFlag' macros.
*/

inline static x_boolean _isSet(x_flags flags, x_flags query) {

  asm ("and %0, %1, %2\n" : "=r" (flags) : "r" (flags), "r" (query));

  return flags;

}

/*
** The following is trivial but makes code more readible. E.g.
** if (isNotSet(x, y)) is IMHO better than if (! isSet(x, y)). Anyhow,
** the programmer has a choice now and the assembly generated is OK.
*/

inline static x_boolean _isNotSet(x_flags flags, x_flags query) {

  asm ("and %0, %1, %2\n" : "=r" (flags) : "r" (flags), "r" (query));

  return ! flags;

}

/*
** The following are defines since we need to change the passed arguments
** and if we would use inline functions, it would require us to pass everything
** as a pointer (isn't it ?).
*/

#define _setFlag(flags, set) {                                      \
  asm ("orr %0, %1, %2\n" : "=r" (flags) : "r" (flags), "r" (set)); \
}

#define _unsetFlag(flags, set) {                                    \
  asm ("bic %0, %1, %2\n" : "=r" (flags) : "r" (flags), "r" (set)); \
}

#endif /* _BITS_H */
