#ifndef _CPU_H
#define _CPU_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: cpu.h,v 1.2 2002/02/06 18:12:50 buytaert Exp $
*/

#include <types.h>
#include <stdio.h>   /* temporary, printf() */

#ifndef ARM
#define ARM
#endif

#ifndef arm
#define arm
#endif

/*
** The number of registers (beside PC and SP) that need to be
** saved in a context for x_context_save and x_context_restore.
*/

#define NUM_CALLEE_SAVED 9

/*
** This structure contains the cpu state of the ARM processor.
*/

typedef struct x_cpu {
  x_word sp;
  x_word pc;  
  x_word lr;
  x_word status;
} x_cpu;

void x_init_start(void * init);
x_int x_cpu_relocate(x_int type, x_address S, x_address A, x_address P);

inline static unsigned long long x_sample(void) {
  return 0LL;        
}
        

inline static void x_atomic_swap(void * address_1, void * address_2) {

  asm volatile (
    "ldr %0, [%0]\n\t"        // Load register with *address_1
    "swp %0, %0, [%1]\n\t"    // Do the atomic swap with address_2
    "str %0, [%2]\n\t"        // Store it back as *address_1
    : /* outputs */
    : /* inputs */
      "r" (address_1),
      "r" (address_2),
      "r" (address_1)
  );

}

inline static int xi_enter_atomic(void * address, void * contents) {

  int result;
  
  asm volatile (
    "mov %0, #1\n\t"        // Use result register as scratch and load with 'looking' value
    "swp %0, %0, [%1]\n\t"  // Do the swap
    "cmp %0, #1\n\t"        // Is somebody else looking ?
    "moveq %0, #0\n\t"      // Yes, prepare 'false' result ...
    "beq 1f\n\t"            // ... and return
    "cmp %0, #0\n\t"        // Is the content NULL ?
    "streq %2, [%1]\n\t"    //   Yes, it's NULL, store new contents in
    "moveq %0, #1\n\t"      //   and make return result 'true'
    "strne %0, [%1]\n\t"    //   No, not NULL, put old contents back
    "movne %0, #0\n\t"      //   and prepare 'false' return result 
    "1:\n\t"
    : /* outputs */
      "=r" (result)
    : /* inputs */
      "r" (address),
      "r" (contents)
  );

  return result;

}

inline static int xi_exit_atomic(void * address) {

  int result;
  
  asm volatile (
    "mov %0, #1\n\t"        // Use result register as scratch and load with 'looking' value
    "swp %0, %0, [%1]\n\t"  // Do the swap
    "cmp %0, #1\n\t"        // Is somebody else looking ?
    "mov %0, #0\n\t"        // Clear scratch register, is also 'false' result when somebody else was looking
    "strne %0, [%1]\n\t"    // If nobody was looking, store 0 at address ...
    "movne %0, #1\n\t"      // ... and prepare 'true' result
    : /* outputs */
      "=r" (result)
    : /* inputs */
      "r" (address)
  );

  return result;

}

/*
** For inline assembly, use r3 as a scratch register as it's the most likely available
** scratch register for the ARM. (r0 - r3 are used as argument registers and all 
** calls with less than 4 arguments leave this register unused.)
*/

// #define x_enter_critical _x_enter_critical()
// #define x_exit_critical _x_exit_critical()

#define x_disable_interrupts _x_disable_interrupts()
inline static void _x_disable_interrupts(void) {
  asm volatile (
    "mrs r3, cpsr\n\t"
    "orr r3, r3, #0xc0\n\t"
    "msr cpsr, r3\n\t"
    : : : "r3"
  );
}

#define x_enable_interrupts _x_enable_interrupts()
inline static void _x_enable_interrupts(void) {
  asm volatile (
    "mrs r3, cpsr\n\t"
    "bic r3, r3, #0xc0\n\t"
    "msr cpsr, r3\n\t"
    : : : "r3"
  );
}

x_ubyte *x_cpu_setup(x_ubyte *memory);

x_word x_critical_status(void);

#endif /* _CPU_H */
