@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@ Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 @
@                                                                         @
@ This software is copyrighted by and is the sole property of Acunia N.V. @
@ and its licensors, if any. All rights, title, ownership, or other       @
@ interests in the software remain the property of Acunia N.V. and its    @
@ licensors, if any.                                                      @
@                                                                         @
@ This software may only be used in accordance with the corresponding     @
@ license agreement. Any unauthorized use, duplication, transmission,     @
@  distribution or disclosure of this software is expressly forbidden.    @
@                                                                         @
@ This Copyright notice may not be removed or modified without prior      @
@ written consent of Acunia N.V.                                          @
@                                                                         @
@ Acunia N.V. reserves the right to modify this software without notice.  @
@                                                                         @
@   Acunia N.V.                                                           @
@   Vanden Tymplestraat 35      info@acunia.com                           @
@   3000 Leuven                 http://www.acunia.com                     @
@   Belgium - EUROPE                                                      @
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

@ -----------------------------------------------------------------------------------------------
@
@ w_boolean compare_swap(void *address, w_word old, w_word new)@
@    r0                        r0              r1          r2
@
@ Atomically compare contents of 'address' with 'old'. When they are the same, place
@ 'new' in address and return 1. When they are not the same place 'old' value back in
@ address and return 0. We use 0xffffffff (-1) as the critical section
@ identifier. When this value is found at address, we spinlock.
@

.align 4
.text
.global xcompare_swap

xcompare_swap:
    stmfd sp!, {r4, lr}
    mov r3, r0                @ make r3 the address
    mov r0, #0                @ make r0 the counter argument
    mvn r4, #0                @ make r4 contain 0xffffffff (-1)

  cs_spin:
    swp r4, r4, [r3]          @ do the swap
    cmn r4, #1                @ do r4 + 1 and set flags@ when r4 = -1, result will be 0 (EQUAL)
    bne cs_critical           @ they were not the same so we have the critical section
    stmfd sp!, {r0 - r4}      @ we don't have it yet, save registers to call backoff
    bl x_spinlock_backoff     @ call our exponential backoff function
    ldmfd sp!, {r0 - r4}      @ restore registers again
    add r0, r0, #1            @ increment count
    b cs_spin                 @ and retry

  cs_critical:
    cmp r4, r1                @ compare with 'old' value
    bne cs_dont               @ is not the same, don't swap
    str r2, [r3]              @ they are the same, store the new value at address
    mov r0, #1                @ prepare to return 1
    b cs_end                  @ and return

  cs_dont:
    str r4, [r3]              @ place old value back
    mov r0, #0                @ prepare to return 0

  cs_end:
    ldmfd sp!, {r4, pc}       @ return to our caller

@ -----------------------------------------------------------------------------------------------
@
@ w_word peek_swap(void *address)@
@   r0                   r0
@
@ Atomically peek at the contents at 'address'. When somebody is in the critical
@ section, we spinlock. Return the contents of address.
@

.align 4
.text
.global peek_swap

peek_swap:
    stmfd sp!, {lr}
    mov r1, r0                @ make r2 the address
    mov r0, #0                @ make r0 the counter argument
    mvn r2, #0                @ make r2 contain 0xffffffff (-1)

  ps_spin:
    swp r2, r2, [r1]          @ do the swap
    cmn r2, #1                @ r2 + 1 and set flags, when r2 = -1, result will be 0 (EQUAL)
    bne ps_critical           @ they were not the same so we have the critical section
    stmfd sp!, {r0 - r2}      @ we don't have it yet, save registers to call backoff
    bl x_spinlock_backoff     @ call our exponential backoff function
    ldmfd sp!, {r0 - r2}      @ restore registers again
    add r0, r0, #1            @ increment count
    b ps_spin                 @ and retry

  ps_critical:
    str r2, [r1]              @ place old value back at address
    mov r0, r2                @ prepare return register
    ldmfd sp!, {pc}           @ and return to our caller
