/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: host.c,v 1.3 2002/02/08 14:59:54 buytaert Exp $
*/

#include <stdio.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <signal.h>
#include <unistd.h>
#include <time.h>

#include <oswald.h>

#define HOST_TIMER_SIGNAL  SIGALRM 
#define HOST_TIMER_TIMER   ITIMER_REAL 

/* 
** The default number of microseconds per timer interrupt. For linux, don't make
** it too small since the kernel would not be able to service it. Note that the
** following value was experimentally found to be a correct number for most systems. Your mileage
** may vary...
*/

static x_size usecs_per_tick = 50000;

static struct sigaction tick;
static struct sigaction oldtick;
static struct itimerval ticker;
static struct itimerval oldtimer;
static sigset_t unblock_timersig;

sigset_t unblocked_signals;

static x_ubyte * host_memory;

x_ubyte * host_memory_end;

volatile x_word critical_status = 0;
extern volatile x_word miss_preemptive;
volatile x_word preemptive_switch = 0;

void start_host_timer(void);

static void host_timer_tick(int sig) {

  if (miss_preemptive) {
    miss_preemptive = 0;
    return;
  }
  
  if (critical_status || irq_depth) {
    return;
  }

  irq_depth = 1;
  (*irq_handlers[IRQ_TICK]->top)(irq_handlers[IRQ_TICK]);
  irq_depth = 0;

  /*
  ** An interrupt handler can change thread_next, so we reschedule if necessary...
  */

  if (miss_preemptive == 0 && thread_current != thread_next) {
    preemptive_switch = 1;
    x_thread_switch(thread_current, thread_next);
  }

}

void start_host_timer () {

  x_int result;

  ticker.it_interval.tv_sec = usecs_per_tick / 1000000;
  ticker.it_interval.tv_usec = usecs_per_tick % 1000000;
  ticker.it_value.tv_sec = usecs_per_tick / 1000000;
  ticker.it_value.tv_usec = usecs_per_tick % 1000000;
  result = setitimer(HOST_TIMER_TIMER, &ticker, &oldtimer);
  if (result == -1) {
    loempa(9, "%s %d: setitimer failed.\n", __FILE__, __LINE__);
    abort();
  }

}
 
void x_host_post(void) {

  if (preemptive_switch) {

    /*
    ** Unblock the timer signal! At entry of the above handler, the kernel has disabled the fired signal
    ** and will only unblock it when we leave the handler. Since we leave through a thread
    ** switch, we would block the next delivery of the timer signal. The thread switch routine calls this
    ** post handler to unblock the signal again.
    */

    sigprocmask(SIG_UNBLOCK, &unblock_timersig, NULL); 
    //sigprocmask(SIG_SETMASK, &unblocked_signals, NULL);
    preemptive_switch = 0;
  }

#ifdef DEBUG
  if (thread_current->trigger && *thread_current->trigger != 0xaaaaaaaa) {
    loempa(9, "Trigger value is 0x%0x for thread %d.\n", *thread_current->trigger, thread_current->id);
    abort();
  }
#endif

}

void x_host_switch(void) {

  thread_current->cpu.status = critical_status;
  thread_current = thread_next;
  critical_status = thread_current->cpu.status;
  thread_current->num_switches += 1;

}                                                                               

x_ubyte * x_irqs_setup(x_ubyte *mem) {

  x_size i;
  
  irq_handlers = x_alloc_static_mem(mem, sizeof(x_irq) * IRQ_MAX);

  for (i = 0; i < IRQ_MAX; i++) {
    irq_handlers[i] = irq_default;
  }
  
  irq_handlers[IRQ_TICK] = irq_tick;

  return mem;
  
}

x_ubyte * x_host_setup(x_ubyte *memory) {

  int result;

  /*
  ** Set up our timer tick interval timer and signal handler.
  */

  tick.sa_handler = host_timer_tick;
  sigemptyset(&tick.sa_mask);
  tick.sa_flags = SA_RESTART;

  result = sigaction(HOST_TIMER_SIGNAL, &tick, &oldtick);
  if (result == -1) {
    loempa(9, "%s %d: sigaction failed.\n", __FILE__, __LINE__);
    abort();
  }

  start_host_timer();

  /*
  ** Get currently unblocked signals to unblock them again in timer handler...
  */
    
  sigemptyset(&unblock_timersig);
  sigaddset(&unblock_timersig, HOST_TIMER_SIGNAL);

  sigprocmask(SIG_UNBLOCK, NULL, &unblocked_signals);

  return memory;
  
}

#define MAX_TRY (1024 * 1024 * 80)
#define MIN_TRY (1024 * 1024 * 4)
#define TRIM_FACTOR (6)

x_status x_oswald_init(x_size max_heap, x_size millis) {

  x_size heap = max_heap;
  const x_size decrement = (1024 * 100);
  
  critical_status = 0;

  usecs_per_tick = millis * 1000;

  /*
  ** Let's try to find out what memory we can take from this box. We try with MAX_TRY and work our way
  ** down when that doesn't work. The minimum is MIN_TRY.
  */
  
  do {
    host_memory = malloc(heap);
    heap -= decrement;
  } while (host_memory == NULL && heap > MIN_TRY);
  heap += decrement;

  if (host_memory == NULL) {
    loempa(9, "Could not even get %d bytes from host.\n", heap);
    return xs_no_mem;
  }

  /*
  ** We now trim down 10% since on a PDA
  ** like the Yopi, we get killed if the system gets to thight...
  */
  
  loempa(9, "Trimming available memory down from %d to %d bytes.\n", heap, heap - (heap / TRIM_FACTOR));
  heap -= (heap / TRIM_FACTOR);
  free(host_memory);
  host_memory = malloc(heap);
  if (host_memory == NULL) {
    loempa(9, "No host memory available.\n");
    return xs_no_mem;
  }

  loempa(9, "Host memory starts at 0x%08x, %d bytes available.\n", host_memory, heap);  
  host_memory_end = host_memory + heap - 1;

  x_kernel_setup(host_memory);

  loempa(9, "Should not get here!\n");

  return xs_bad_argument;
  
}

x_word x_critical_status(void) {
  return critical_status;
}

void _x_must_be_critical(const char *f, int l, x_boolean critical) {

  if (irq_depth) {
    return;
  }

  if ( (critical && critical_status == 0) || (! critical && critical_status == 1)) {
    fprintf(stderr, "--------- %d %s:%d wrong !! current = %d required = %d ---------------------\n", thread_current->id, f, l, critical_status, critical);
    fflush(stderr);
    abort();
  }

}

/*
** Functions to convert seconds, microseconds to ticks and vice versa.
*/

static const x_size ticks_compensation = 2;

/*
** Return a compensated number of ticks for a number of seconds.
*/

x_size x_seconds2ticks(x_size seconds) {
  return (x_size) ((seconds * 1000) / (x_size) (usecs_per_tick / 1000) - ticks_compensation);
}

x_size x_millis2ticks(x_size millis) {

  x_size ticks = (millis / (x_size) (usecs_per_tick / 1000));
  
  return ticks ? ticks : 1;
  
}

x_size x_ticks2usecs(x_size ticks) {
  return (usecs_per_tick * ticks);
}

x_size x_usecs2ticks(x_size usecs) {
  return (usecs < usecs_per_tick) ? 1 : ((x_size)(usecs / usecs_per_tick));
}

int my_error;

int * __errno_location(void) {
  if(thread_current){
    return (int *) & thread_current->tsd_errno;
  }
  else {
    return (int *) & my_error;
  }
}
