/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: monitor_test.c,v 1.2 2002/02/27 10:16:28 buytaert Exp $
*/

#include <stdio.h>
#include <tests.h>
#include <string.h>

#include <oswald.h>

#define MONT_STACK_SIZE ((1024 * 3) + MARGIN)

static x_sleep second;

/*
** We check if we got out of the waiting list in correct time, but when we have
** thread dumping enabled in the init thread, this can take too much time. We then
** set the following to false to indicate that it is not a real failure.
*/

static const x_boolean wait_ticks_fail = false;

/*
** The native implementation of our old faithfull Carillon. This code uses a single
** monitor, but is very heavy on contention.
*/

#define NUM_RINGERS          8
#define RESULT_LENGTH        256

typedef struct t_Ringer *    t_ringer;

typedef struct t_Ringer {
  int id;                    // Id of message
  const char * what;         // What to say as a message
  x_ubyte *stack;            // The stack of the ringer thread
  x_size stack_size;         // The size of the stack
  x_thread thread;           // The pointer to the thread structure
} t_Ringer;

static t_Ringer Ringers[] = {
  { 1, "brown",  NULL, 0, NULL },
  { 2, "dog",    NULL, 0, NULL },
  { 3, "fox",    NULL, 0, NULL },
  { 4, "jumps",  NULL, 0, NULL },
  { 5, "lazy",   NULL, 0, NULL },
  { 6, "over",   NULL, 0, NULL },
  { 7, "quick",  NULL, 0, NULL },
  { 8, "the",    NULL, 0, NULL },
};

/*
** Which id's to play in which order and the id that is currently playing...
*/

static x_int conductor[] = { 8, 7, 1, 3, 4, 6, 8, 5, 2 };
static x_int nox_playing = 0;
static x_int num_plays = (sizeof(conductor) / sizeof(conductor[0]));
static char result[RESULT_LENGTH];
static x_size runs = 0;

/*
** The monitor that controls the playing...
*/

static x_Monitor Monitor;

/*
** How many unecessary waits did we have for 1 sequence...
*/

static x_int sequence_clashes = 0;

/*
** The maximum number of waiting threads we had in a run...
*/

static x_size max_waiting = 0;

/*
** The maximum number of competing threads we had in a run...
*/

static x_int max_competing = 0;

static void next_sequence(x_int current) {

  x_status status;

  if (Monitor.Event.n_competing > max_competing) {
    max_competing = Monitor.Event.n_competing;
  }

  if (Monitor.n_waiting > max_waiting) {
    max_waiting = Monitor.n_waiting;
  }
    
  status = x_monitor_enter(&Monitor, x_eternal);
  if (status != xs_success) {
    oempa("Status = %s\n", status2char(status));
    exit(0);
  }

  current += 1;
  if (current >= num_plays) {
    nox_playing = 0;
    if (strcmp(result, "the quick brown fox jumps over the lazy dog ") != 0) {
      oempa("Bad result = '%s'\n", result);
      exit(0);
    }
    else {
    
      // At this point, the monitor count should be 2, one for the initial entry and 1 for the enter_monitor above.
      
      if (Monitor.count != 2) {
        oempa("Monitor count isn't 2 but %d.\n", Monitor.count);
        exit(0);
      }
      
      runs += 1;
      oempa("Printing thread %d, prio = %d clashes = %d, max competing = %d, max waiting = %d\n", thread_current->id, thread_current->c_prio, sequence_clashes, max_competing, max_waiting);
      oempa("'%s' %d (cnt = %d, wait = %d comp = %d)\n", result, runs, Monitor.count, Monitor.n_waiting, Monitor.Event.n_competing);
      memset(result, 0x00, RESULT_LENGTH);
      sequence_clashes = 0;
      max_waiting = 0;
      max_competing = 0;
      x_thread_sleep(second * 10);
    }
  }  
  else {
    nox_playing = current;
  }
  
  status = x_monitor_notify_all(&Monitor);
  if (status != xs_success) {
    oempa("Status = %s\n", status2char(status));
    exit(0);
  }
    
  status = x_monitor_exit(&Monitor);
  if (status != xs_success) {
    oempa("Status = %s\n", status2char(status));
    exit(0);
  }

  if (Monitor.owner != thread_current) {
    if (thread_current->a_prio != thread_current->c_prio) {
      oempa("Our priority wasn't reset...\n");
      exit(0);
    }
  }

}

/*
** The ringer function
*/

static void Ringer(void *r) {

  t_ringer ringer = r;
  x_status status;
  x_size old_count;

  oempa("Ringer thread %d '%s' starting...\n", ringer->thread->id, ringer->what);
  
  status = x_monitor_enter(&Monitor, x_eternal);
  if (status != xs_success) {
    oempa("Status = %s\n", status2char(status));
    exit(0);
  }

  while (1) {
    if (conductor[nox_playing] == ringer->id) {
      sprintf(result + strlen(result), "%s ", ringer->what);
      next_sequence(nox_playing);
    }
    else {
      sequence_clashes += 1;
    }
    
    x_thread_sleep(1);
    
    old_count = Monitor.count;
    status = x_monitor_wait(&Monitor, x_eternal);
    if (status != xs_success) {
      oempa("Status = %s\n", status2char(status));
      exit(0);
    }
    
    // See if the count is correct w.r.t. the previous count

    if (old_count != Monitor.count) {
      oempa("Monitor count is not same as before ! %d != %d\n", old_count, Monitor.count);
      exit(0);
    }
    
    // See if our count is not getting out of bounds...
    
    if (Monitor.count > NUM_RINGERS) {
      oempa("Monitor count out of bounds %d > %d ! %d != %d\n", Monitor.count, NUM_RINGERS);
      exit(0);
    }
    
  }
  
}

static void Carillon(void *t) {

  x_int initialized = 0;
  x_int i;
  x_size prio_low = 1;
  x_size prio_high = prio_low + 1;
  x_size min_prio;
  x_size previous_min_prio = 0;
  t_ringer r;
  x_int pass = 0;
  x_status status;
  
  while (1) {
    if (! initialized) {
      x_monitor_create(&Monitor);
      for (i = 0; i < NUM_RINGERS; i++) {
        Ringers[i].thread = x_mem_get(sizeof(x_Thread));
        Ringers[i].stack_size = MONT_STACK_SIZE;
        Ringers[i].stack = x_mem_get(MONT_STACK_SIZE);
        status = x_thread_create(Ringers[i].thread, Ringer, &Ringers[i], Ringers[i].stack, Ringers[i].stack_size, prio_low, TF_START);
        if (status != xs_success) {
          oempa("%s: status = '%s'\n", __FILE__, status2char(status));
          exit(0);
        }
        else {
          oempa("Ringer thread number %d, id = %d\n", i, Ringers[i].thread->id);
        }
      }
      initialized = 1;
    }

    /*
    ** Find minimum used priority...
    */
    
    min_prio = 10000;
    for (i = 0; i < NUM_RINGERS; i++) {
      r = &Ringers[i];
      if (min_prio > r->thread->a_prio) {
        min_prio = r->thread->a_prio;
      }
    }

    if (min_prio > 5) {
      min_prio = 1;
    }

    if (previous_min_prio != 0) {
      pass += 1;
      oempa("************** Priority changing pass %d, min_prio = %d  *************************\n", pass, min_prio);
      prio_low = min_prio + 1; // introduce a shift...
      prio_high = prio_low + 1;
      x_thread_priority_set(Ringers[0].thread, prio_low);
      for (i = 1; i < NUM_RINGERS; i++) {
        r = &Ringers[i];
        x_thread_priority_set(r->thread, prio_high);
      }
    }

    previous_min_prio = min_prio;

    x_thread_sleep(second * 10);

  }
  
}

/*
** The dining philosophers program WITHOUT specific notification of Thomas Cargill. 
**
** In his paper about "Specific Notification" http://www.sni.net/~cargill, Thomas Cargill has a
** java program that implements the dining philosophers in two ways; one without his pattern for
** specific notification, which yields a correct result w.r.t synchronisation but that is very
** unfair (3 out of 5 philosophers starve to death) and another that he rewrote with his synchronization
** pattern to avoid this situation. We have implemented monitors such that this would not happen.
** Therefore, the unfair first program of Thomas should run fine on Oswald. This is checked by the
** 'fairness' check.
*/

#define NUMBER_FORKS 5

/*
** The fairness ratio defines the ratio between the most eaten plates and the least eaten
** plates. It should be 1 in steady state but getting our monitor list wrong would upset
** the test. This will be signalled by an abort...
*/

#define FAIRNESS_RATIO 3

typedef struct t_Fork * t_fork;

typedef struct t_Fork {
  x_int free;
  x_Monitor monitor;
} t_Fork;

static t_Fork forks[NUMBER_FORKS];
static const x_int number_forks = NUMBER_FORKS;

typedef struct t_Phil * t_phil;

typedef struct t_Phil {
  x_size plates;
  t_fork fork_a;
  t_fork fork_b;
  x_ubyte * stack;
  x_size stack_size;
  x_thread thread;
} t_Phil;

static t_Phil * philosophers;

static void pickup_fork(t_fork afork) {

  x_status status;
  
  status = x_monitor_enter(&afork->monitor, x_eternal);
  if (status != xs_success) {
    oempa("Status = %s\n", status2char(status));
    exit(0);
  }

  // Don't sleep here since we would destroy checking for our fairness rules...
  
  if (afork->monitor.owner != thread_current) {
    oempa("Owner is wrong!\n");
    exit(0);
  }
  
  while (afork->free == 0) {
    status = x_monitor_wait(&afork->monitor, x_eternal);
    if (status != xs_success) {
      oempa("Status = %s\n", status2char(status));
      exit(0);
    }
  }
  
  afork->free = 0;

  status = x_monitor_exit(&afork->monitor);
  if (status != xs_success) {
    oempa("Status = %s\n", status2char(status));
    exit(0);
  }
    
}

static void laydown_fork(t_fork afork) {

  x_status status;
  
  status = x_monitor_enter(&afork->monitor, x_eternal);
  if (status != xs_success) {
    oempa("Status = %s\n", status2char(status));
    exit(0);
  }
  
  // Don't sleep here since we would destroy checking for our fairness rules...

  if (afork->monitor.owner != thread_current) {
    oempa("Owner is wrong!\n");
    exit(0);
  }
  
  afork->free = 1;
  
  status = x_monitor_notify(&afork->monitor);
  if (status != xs_success) {
    oempa("Status = %s\n", status2char(status));
    exit(0);
  }

  status = x_monitor_exit(&afork->monitor);
  if (status != xs_success) {
    oempa("Status = %s\n", status2char(status));
    exit(0);
  }
    
}

static x_boolean report = true;
static x_int num_reports = 0;

static void Philosopher(void *p) {

  t_phil phil = p;
  x_int i;
  x_size max_plates;
  x_size min_plates;
  x_size ratio;
  x_size stack_used;
  x_size stack_size;
  x_size stack_left;

  oempa("Philosopher %d is getting his feet under the table...\n", thread_current->id);
    
  while (1) {
    pickup_fork(phil->fork_a);
    pickup_fork(phil->fork_b);
    
    x_thread_sleep(1);
    phil->plates++;

    /*
    ** Check the maximum and minimum number of plates consumed, and use this to check for our
    ** fairness algorithm.
    */
        
    max_plates = 0;
    min_plates = 0xffffffff;
    for (i = 0; i < number_forks; i++) {
      if (philosophers[i].plates > max_plates) {
        max_plates = philosophers[i].plates;
      }
      if (min_plates > philosophers[i].plates) {
        min_plates = philosophers[i].plates;
      }
    }

    ratio = (min_plates) ? (max_plates / min_plates) : max_plates;
    x_stack_info(thread_current, &stack_size, &stack_used, &stack_left);
    if (report) {
      oempa("Thread %d stack: used %d of %d bytes, %d bytes left.\n", thread_current->id, stack_used, stack_size, stack_left);
      oempa("Philosopher %d has eaten %d plates (max = %d, min = %d, ratio = %d)\n", thread_current->id, phil->plates, max_plates, min_plates, ratio);
      report = false;      
      num_reports += 1;
    }      

    /*
    ** Check the fairness, in steady state, fairness should be 1, but at startup, it could get to 3...
    */
    
    if (ratio > FAIRNESS_RATIO) {
      oempa("Ratio is out of bounds %d > %d\n", ratio, FAIRNESS_RATIO);
      exit(0);
    }
    
    laydown_fork(phil->fork_b);
    laydown_fork(phil->fork_a);
    
    /*
    ** Do some napping in between to digest the stuff, note that the sleeping time can not be
    ** random since we would upset our fairness check...
    */
    
    x_thread_sleep(second * 10);

  }
  
}

static void Dining_Philosophers(void *t) {

  x_int initialized = 0;
  x_int i;
  t_fork fork_a;
  t_fork fork_b;
  t_phil phil;
  x_int number = 0;
  x_status status;
  
  for (i = 0; i < number_forks; i++) {
    forks[i].free = 1;
    x_monitor_create(&forks[i].monitor);
  }
  
  while (1) {
    if (! initialized) {
      philosophers = x_mem_get(number_forks * sizeof(t_Phil));
      memset(philosophers, 0x00, number_forks * sizeof(t_Phil));
      for (i = 0; i < number_forks; i++) {
        oempa("Taking forks %d and %d for philosopher %d.\n", i, (i + 1) % number_forks, number);
        philosophers[i].thread = x_mem_get(sizeof(x_Thread));
        philosophers[i].stack = x_mem_get(MONT_STACK_SIZE);
        philosophers[i].stack_size = MONT_STACK_SIZE;
        fork_a = &forks[i];
        fork_b = &forks[(i + 1) % number_forks];
        phil = &philosophers[number++];
        if (i % 2 == 0) {
          phil->fork_a = fork_a;
          phil->fork_b = fork_b;
        }
        else {
          phil->fork_a = fork_b;
          phil->fork_b = fork_a;
        }
        status = x_thread_create(phil->thread, Philosopher, phil, phil->stack, phil->stack_size, 4, TF_START);
        if (status != xs_success) {
          oempa("%s: status = '%s'\n", __FILE__, status2char(status));
          exit(0);
        }
        else {
          oempa("Philosopher thread %d created.\n", phil->thread->id);
        }
      }
      initialized = 1;
    }
    x_thread_sleep(second * 10);
    report = true;
  }
  
}

static x_thread c_thread;
static x_ubyte * c_stack;
static x_thread p_thread;
static x_ubyte * p_stack;

static x_thread control;
static x_int state = 0;

static void monitor_tests(void * t) {

  x_status status;
  x_int counter;
  x_int i;
  x_thread x;
  x_pcb pcb;
  char * buffer = NULL;
  x_size buffer_size = 10;
  
  while (1) {
    counter += 1;
    if (state == 0) {
      buffer = x_mem_get(buffer_size);
      c_thread = x_mem_get(sizeof(x_Thread));
      c_stack = x_mem_get(MONT_STACK_SIZE);
      p_thread = x_mem_get(sizeof(x_Thread));
      p_stack = x_mem_get(MONT_STACK_SIZE);


      status = x_thread_create(c_thread, Carillon, c_thread, c_stack, MONT_STACK_SIZE, 5, TF_START);
      if (status != xs_success) {
        oempa("%s: status = '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      else {
        oempa("Carillon master thread, id %d.\n", c_thread->id);
      }
      status = x_thread_create(p_thread, Dining_Philosophers, p_thread, p_stack, MONT_STACK_SIZE, 5, TF_START);
      if (status != xs_success) {
        oempa("%s: status = '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      else {
        oempa("Dining philosopher master thread, id %d.\n", p_thread->id);
      }

      state = 1;
    }

    /*
    ** Skip state 1 since the ringers and philosophers haven't been initialized yet...
    */
        
    if (state == 2 && counter % 1 == 0) {
      for (i = 0; i < number_forks; i++) {
        x = philosophers[i].thread;
        pcb = x_prio2pcb(x->c_prio);
        oempa("%d P %d state = %10s, sticks = %3d, prios %d %d, pcb: r = %d, p = %d, %d plates\n", i, x->id, x_state2char(x), x->sticks, x->a_prio, x->c_prio, pcb->n_ready, pcb->n_pending, philosophers[i].plates);
      }
      for (i = 0; i < NUM_RINGERS; i++) {
        x = Ringers[i].thread;
        pcb = x_prio2pcb(x->c_prio);
        if (x->sticks == x_eternal) {
          x_snprintf(buffer, 10, "---");
        }
        else {
          x_snprintf(buffer, 10, "%3d", x->sticks);
        }
        oempa("%d R %d state = %10s, sticks = %s, prios %d %d, pcb: r = %d, p = %d\n", i, x->id, x_state2char(x), buffer, x->a_prio, x->c_prio, pcb->n_ready, pcb->n_pending);
      }
    }

    /*
    ** Give the ringers and Philosophers some time to initialize...
    */
    
    if (state == 1) {
      state = 2;
      x_thread_sleep(second * 10);
    }
    
    if (state == 3) {
      oempa("Thread for philosophers not found in pcbs.\n");
      exit(0);
    }

    x_thread_sleep(50 * (x_random() % 7 + 3));

  }
  
}

/*

The finite state machine to test Oswald monitors. There are 5 threads, at 3
different priorities. In the column indicated by 'M', the number of times
the tread has the monitor is noted, with a '+' sign when the thread owns the
monitor, with the '-' sign when the count is saved and the thread doesn't
have the monitor.

The state of the finite state machine is called 'state' below, but the name
of the variable in the source code lower down is 'fsm_state'. The sleep
amount for each thread at the end is indicated by FSMS.

For printing out the following state text, you'd best use landscape...

              Prio 4               |              Prio 4               |              Prio 4               |              Prio 3               |               Prio 2              |
                 1                 |                 2                 |                 3                 |                 4                 |                  5                |
---------------------------------M-+---------------------------------M-+---------------------------------M-+---------------------------------M-+---------------------------------M-+
 If state == 1, create monitor !   |                               !   |                               !   |                               !   |                               !   |
 and set state to 2. ----------!   |-> If state == 2, enter monitor!   |                               !   |                               !   |                               !   |
                               !   | and enter monitor, set state  !+2 |                               !   |                               !   |                               !   |
                               !   | to 3, sleep FSMS * 2 ticks. --!   |-> If state == 3, enter monitor!   |                               !   |                               !   |
                               !   |                               !   | should fail, set state to 4,  !   |                               !   |                               !   |
                               !   | while (state != 5), wait -----!-2 |-> try to enter with eternal,  !   |                               !   |                               !   |
                               !   | eternal.     :                !   | should succeed. Set state to  !+1 |                               !   |                               !   |
                               !   |              :                !   | 5 and notify single, while    !   |                               !   |                               !   |
                               !   |              :<---------------!   |- ! (state == 12 | state == 13)!-1 |                               !   |                               !   |
                               !   | should succeed, set state to  !+2 | wait eternal :                !   |                               !   |                               !   |
                               !   | 6 and sleep FSMS * 2 ticks. --!   |------>-------:-------->-------!   |-> If state == 6, set state to !   |                               !   |
                               !   |              :                !   |              :                !   | 7, try to enter monitor with  !   |                               !   |
                               !   | The state should be 7. <------!   |------<-------:--------<-------!   |- eternal.     :               !   |                               !   |
                               !   | Set state to 8 and just wait  !   |              :                !   |               :               !   |                               !   |
                               !   | for 5 ticks. -------->--------!-2 |------>-------:-------->-------!   |-> should succeed and state    !+1 |                               !   |
                               !   |              :                !   |              :                !   | should be 8, set state to 9   !   |                               !   |
                               !   |              :                !   |              :                !   | and notify single, wait for   !   |                               !   |
                               !   | should succeed and state <----!+2 |------<-------:--------<-------!   |- FSMS * 2 ticks.              !-1 |                               !   |
                               !   | should be 9, while state != 10!   |              :                !   |               :               !   |                               !   |
                               !   | wait eternal -------->--------!-2 |------>-------:-------->-------!   |-> should succeed, set state   !+1 |                               !   |
                               !   |              :                !   |              :                !   | to 10, notify single and      !   |                               !   |
                               !   |              :<---------------!   |------<-------:--------<-------!   |-while state != 12, wait for   !   |                               !   |
                               !   | should succeed, set state to  !+2 |              :                !   | eternal.      :               !-1 |                               !   |
                               !   | 11, while state != 14, wait   !   |              :                !   |               :               !   |                               !   |
                               !   | eternal (we don't check for   !   |              :                !   |               :               !   |                               !   |
                               !   | states 12 & 13) ----->--------!-2 |------>-------:-------->-------!   |------->-------:------->-------!   |-> If state == 11, enter the   !   |
                               !   |              :                !   |              :                !   |               :               !   | monitor, enter monitor,       !+2 |
                               !   |              :                !   |              :                !   |               :               !   | set state to 12 and notify    !   |
                               !   |              :                !   |              :                !   |               :               !   | all, while state != 16,       !   |
                               !   |              :                !   |              :                !   | should succeed and state <----!+1 |-  wait eternal                !-2 |
                               !   |              :                !   |              :                !   | should be 12, set state to 13 !   |              :                !   |
                               !   |              :                !   |              :<---------------!   |-and notify all, while the     !   |              :                !   |
                               !   |              :                !   | should succeed and state must !+1 | state != 18, wait eternal     !-1 |              :                !   |
                               !   |              :                !   | be 13, set state to 14, notify!   |               :               !   |              :                !   |
                               !   |              :<---------------!   |-all and while ! (state == 20  !   |               :               !   |              :                !   |
                               !   | should succeed and state must !+2 | || state == 18), wait eternal !-1 |               :               !   |              :                !   |
                               !   | be 14, set state to 15 and    !   |              :                !   |               :               !   |              :                !   |
                               !   | while ! (state == 18 || state !-2 |              :                !   |               :               !   |              :                !   |
 If state == 15, n_waiting <---!   |- == 20 || state == 21) wait   !   |              :                !   |               :               !   |              :                !   |
 monitor should be 4, enter    !   | with eternal.:                !   |              :                !   |               :               !   |              :                !   |
 the monitor, set state to 16, !+1 |              :                !   |              :                !   |               :               !   |              :                !   |
 notify all, while state != 17,!   |              :                !   |              :                !   |               :               !   |              :                !   |
 wait eternal -----------------!-1 |------->------:-------->-------!   |----->--------:-------->-------!   |------->-------:-------->------!   |-> should succeed, set state   !+2 |
              :                !   |              :                !   |              :                !   |               :               !   | to 17, exit monitor, check we !+1 |
              :                !   |              :                !   |              :                !   |               :               !   | are still owner and count is  !   |
 should succeed, n_waiting <---!+1 |-------<------:--------<-------!   |-----<--------:--------<-------!   |-------<-------:--------<------!   |-1, notify all and exit the    ! 0 |
 should be 1, set state to 18  !   |              :                !   |              :                !   |               :               !   | monitor.                      !   |
 and notify single, while the  !   |              :                !   |              :                !   |               :               !   |                               !   |
 state != 19, wait eternal ----!-1 |------->------:-------->-------!   |----->--------:-------->-------!   |-> should succeed, count should!+1 |                               !   |
              :                !   |              :                !   |              :                !   | be 1, set state to 19 and     !   |                               !   |
 should succeed, n_waiting is  !+1 |              :                !   |              :                !   | notify all, exit monitor.     ! 0 |                               !   |
 1, set state to 20 and notify !   |              :                !   |              :                !   |                               !   |                               !   |
 single, while state != 21,    !-1 |              :                !   |              :                !   |                               !   |                               !   |
 wait eternal -----------------!   |------->------:-------->-------!   |-> should succeed, count should!+1 |                               !   |                               !   |
                               !   |              :                !   | be 1, set state to 21 and     !   |                               !   |                               !   |
 should succeed, n_waiting <---!+1 |-------<------:--------<-------!   |-notify all, exit monitor.     ! 0 |                               !   |                               !   |
 should be 0, set state to 22  !   |              :                !   |                               !   |                               !   |                               !   |
 notify all and while state is !   |              :                !   |                               !   |                               !   |                               !   |
 not 23, wait eternal ---------!-1 |-> should succeed, set state   !+2 |                               !   |                               !   |                               !   |
              :                !   | to 23, notify all and then    !   |                               !   |                               !   |                               !   |
              :                !   | while (state != 24) {         !   |                               !   |                               !   |                               !   |
 should succeed, n_waiting <---!+1 |-  wait for 5 (10/2) ticks.    !-2 |                               !   |                               !   |                               !   |
 should be 1, now do           !   | }            :                !   |                               !   |                               !   |                               !   |
 for (i = 0; i < 10; i++) {    !   |              :                !   |                               !   |                               !   |                               !   |
   if (n_waiting == 0) {       !   |              :                !   |                               !   |                               !   |                               !   |
     break;                    !   |              :                !   |                               !   |                               !   |                               !   |
   }                           !   |              :                !   |                               !   |                               !   |                               !   |
   x_thread_sleep(1);          !   |              :                !   |                               !   |                               !   |                               !   |
 }                             !   |              :                !   |                               !   |                               !   |                               !   |
 check that i is below 10, we  !   |              :                !   |                               !   |                               !   |                               !   |
 must have seen n_waiting go   !   |              :                !   |                               !   |                               !   |                               !   |
 to 0 in the loop. Set state   !   |              :                !   |                               !   |                               !   |                               !   |
 to 24, notify single and      !   |              :                !   |                               !   |                               !   |                               !   |
 exit the monitor. ------------! 0 |-> should succeed, exit monitor!+2 |                               !   |                               !   |                               !   |
                               !   | twice and set state to 25.    ! 0 |                               !   |                               !   |                               !   |
*/

static x_monitor monitor;
static volatile x_int fsm_state;

#define set_state(x) _set_state(__FUNCTION__, __LINE__, x)

static void _set_state(const char * f, x_int l, x_int new) {
//  oempa("%d %d Setting state from %d to %d in %s:%d\n", thread_current->a_prio, thread_current->c_prio, fsm_state, new, f, l);
  fsm_state = new;
}

#define FSM_SLEEP (5)

static void mt_1(void * t) {

  x_status status;
  x_int i;

  while (1) {

    if (fsm_state == 1) {
      monitor = x_mem_get(sizeof(x_Monitor));
      memset(monitor, 0xff, sizeof(x_Monitor));
      status = x_monitor_create(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      
      if (monitor->n_waiting != 0) {
        oempa("Bad n_waiting = %d\n", monitor->n_waiting);
        exit(0);
      }

      if (monitor->l_waiting != NULL) {
        oempa("Bad l_waiting = 0x%08x\n", monitor->l_waiting);
        exit(0);
      }

      if (monitor->owner != NULL) {
        oempa("Bad owner = 0x%08x\n", monitor->owner);
        exit(0);
      }

      if (monitor->count != 0) {
        oempa("Bad count = %d\n", monitor->count);
        exit(0);
      }

      /*
      ** If we passed these simple tests, we can proceed.
      */
      
      set_state(2);
    }
    
    if (fsm_state == 15) {
      if (monitor->n_waiting != 4) {
        oempa("Waiting should be 4 in stead of %d.\n", monitor->n_waiting);
        exit(0);
      }
      status = x_monitor_enter(monitor, x_eternal);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      set_state(16);
      status = x_monitor_notify_all(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      
      while (fsm_state != 17) {
        status = x_monitor_wait(monitor, x_eternal);
        if (status != xs_success) {
          oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
          exit(0);
        }
      }
      
      if (monitor->owner != thread_current) {
        oempa("Bad owner %d != %d\n", monitor->owner->id, thread_current->id);
        exit(0);
      }
      if (monitor->count != 1) {
        oempa("Bad count %d != 1.\n", monitor->count);
        exit(0);
      }

//      if (monitor->n_waiting != 1) {
      if (monitor->n_waiting != 0) {
        oempa("Waiting should be 1 in stead of %d.\n", monitor->n_waiting);
        exit(0);
      }
      set_state(18);
      status = x_monitor_notify(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      while (fsm_state != 19) {
        status = x_monitor_wait(monitor, x_eternal);
        if (status != xs_success) {
          oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
          exit(0);
        }
      }

      if (monitor->owner != thread_current) {
        oempa("Bad owner %d != %d\n", monitor->owner->id, thread_current->id);
        exit(0);
      }
      if (monitor->count != 1) {
        oempa("Bad count %d != 1.\n", monitor->count);
        exit(0);
      }

      if (monitor->n_waiting != 1) {
        oempa("Waiting should be 1 in stead of %d.\n", monitor->n_waiting);
        exit(0);
      }
      set_state(20);
      status = x_monitor_notify(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      while (fsm_state != 21) {
        status = x_monitor_wait(monitor, x_eternal);
        if (status != xs_success) {
          oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
          exit(0);
        }
      }

      if (monitor->owner != thread_current) {
        oempa("Bad owner %d != %d\n", monitor->owner->id, thread_current->id);
        exit(0);
      }
      if (monitor->count != 1) {
        oempa("Bad count %d != 1.\n", monitor->count);
        exit(0);
      }

      if (monitor->n_waiting != 0) {
        oempa("Waiting should be 0 in stead of %d.\n", monitor->n_waiting);
        exit(0);
      }
      set_state(22);
      status = x_monitor_notify_all(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      while (fsm_state != 23) {
        status = x_monitor_wait(monitor, x_eternal);
        if (status != xs_success) {
          oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
          exit(0);
        }
      }

      if (monitor->owner != thread_current) {
        oempa("Bad owner %d != %d\n", monitor->owner->id, thread_current->id);
        exit(0);
      }
      if (monitor->count != 1) {
        oempa("Bad count %d != 1.\n", monitor->count);
        exit(0);
      }

      if (monitor->n_waiting != 1) {
        oempa("Waiting should be 1 in stead of %d.\n", monitor->n_waiting);
        exit(0);
      }
      for (i = 0; i < 10; i++) {
        if (monitor->n_waiting == 0) {
          break;
        }
        x_thread_sleep(1);
      }
      if (i != FSM_SLEEP) {
        oempa("Thread 2 didn't go out of waiting list after %d but after %d ticks.\n", FSM_SLEEP, i);
        if (wait_ticks_fail) {
          exit(0);
        }
      }
      set_state(24);
      status = x_monitor_notify(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      status = x_monitor_exit(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      
      return;
      
      
    }
    
    x_thread_sleep(FSM_SLEEP);
    
  }
  
}

static void mt_2(void * t) {

  x_status status;
  
  while (1) {
    if (fsm_state == 2) {
      status = x_monitor_enter(monitor, x_eternal);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      status = x_monitor_enter(monitor, x_eternal);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }

      set_state(3);
      x_thread_sleep(FSM_SLEEP * 2);

      while (fsm_state != 5) {
        status = x_monitor_wait(monitor, x_eternal);
        if (status != xs_success) {
          oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
          exit(0);
        }
      }

      if (monitor->owner != thread_current) {
        oempa("Bad owner %d != %d\n", monitor->owner->id, thread_current->id);
        exit(0);
      }
      if (monitor->count != 2) {
        oempa("Bad count %d != 2.\n", monitor->count);
        exit(0);
      }

      set_state(6);
      x_thread_sleep(FSM_SLEEP * 2);
      
      if (fsm_state != 7) {
        oempa("Bad state %d != 7.\n", fsm_state);
        exit(0);
      }

      set_state(8);

      status = x_monitor_wait(monitor, 5);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }

      if (fsm_state != 9) {
        oempa("Bad state %d != 9.\n", fsm_state);
        exit(0);
      }

      if (monitor->owner != thread_current) {
        oempa("Bad owner %d != %d\n", monitor->owner->id, thread_current->id);
        exit(0);
      }
      if (monitor->count != 2) {
        oempa("Bad count %d != 2.\n", monitor->count);
        exit(0);
      }

      while (fsm_state != 10) {
        status = x_monitor_wait(monitor, x_eternal);
        if (status != xs_success) {
          oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
          exit(0);
        }
      }

      if (monitor->owner != thread_current) {
        oempa("Bad owner %d != %d\n", monitor->owner->id, thread_current->id);
        exit(0);
      }
      if (monitor->count != 2) {
        oempa("Bad count %d != 2.\n", monitor->count);
        exit(0);
      }

      set_state(11);

      while (! (fsm_state == 14)) {
        status = x_monitor_wait(monitor, x_eternal);
        if (status != xs_success) {
          oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
          exit(0);
        }
      }

      if (monitor->owner != thread_current) {
        oempa("Bad owner %d != %d\n", monitor->owner->id, thread_current->id);
        exit(0);
      }
      if (monitor->count != 2) {
        oempa("Bad count %d != 2.\n", monitor->count);
        exit(0);
      }

      set_state(15);

      while (! (fsm_state == 18 || fsm_state == 20 || fsm_state == 22)) {
        status = x_monitor_wait(monitor, x_eternal);
        if (status != xs_success) {
          oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
          exit(0);
        }
      }

      if (monitor->owner != thread_current) {
        oempa("Bad owner %d != %d\n", monitor->owner->id, thread_current->id);
        exit(0);
      }
      if (monitor->count != 2) {
        oempa("Bad count %d != 2.\n", monitor->count);
        exit(0);
      }

      set_state(23);

      status = x_monitor_notify_all(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      while (fsm_state != 24) {
        status = x_monitor_wait(monitor, 5);
        if (status != xs_success) {
          oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
          exit(0);
        }
      }

      if (monitor->owner != thread_current) {
        oempa("Bad owner %d != %d\n", monitor->owner->id, thread_current->id);
        exit(0);
      }
      if (monitor->count != 2) {
        oempa("Bad count %d != 2.\n", monitor->count);
        exit(0);
      }

      status = x_monitor_exit(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      status = x_monitor_exit(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }

      set_state(25);

      return;      
      
    }
    
    x_thread_sleep(FSM_SLEEP);
    
  }
  
}

static void mt_3(void * t) {

  x_status status;
  
  while (1) {
    if (fsm_state == 3) {
      status = x_monitor_enter(monitor, 5);
      if (status != xs_no_instance) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      set_state(4);
      status = x_monitor_enter(monitor, x_eternal);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      set_state(5);
      status = x_monitor_notify(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }

      while (! (fsm_state == 12 || fsm_state == 13)) {
        status = x_monitor_wait(monitor, x_eternal);
        if (status != xs_success) {
          oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
          exit(0);
        }
      }

      if (monitor->owner != thread_current) {
        oempa("Bad owner %d != %d\n", monitor->owner->id, thread_current->id);
        exit(0);
      }
      if (monitor->count != 1) {
        oempa("Bad count %d != 1.\n", monitor->count);
        exit(0);
      }

      if (fsm_state != 13) {
        oempa("Bad state %d != 13.\n", fsm_state);
        exit(0);
      }

      set_state(14);
      status = x_monitor_notify_all(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }

      while (! (fsm_state == 20 || fsm_state == 18)) {
        status = x_monitor_wait(monitor, x_eternal);
        if (status != xs_success) {
          oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
          exit(0);
        }
      }

      if (monitor->owner != thread_current) {
        oempa("Bad owner %d != %d\n", monitor->owner->id, thread_current->id);
        exit(0);
      }
      if (monitor->count != 1) {
        oempa("Bad count %d != 1.\n", monitor->count);
        exit(0);
      }
      
      set_state(21);
      status = x_monitor_notify_all(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      status = x_monitor_exit(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      
      return;
 
    }
    
    x_thread_sleep(FSM_SLEEP);
    
  }
  
}

static void mt_4(void * t) {

  x_status status;
  
  while (1) {
    if (fsm_state == 6) {
      set_state(7);
      status = x_monitor_enter(monitor, x_eternal);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      if (fsm_state != 8) {
        oempa("Bad state %d != 8.\n", fsm_state);
        exit(0);
      }
      set_state(9);
      status = x_monitor_notify(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      status = x_monitor_wait(monitor, FSM_SLEEP * 2);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }

      if (monitor->owner != thread_current) {
        oempa("Bad owner %d != %d\n", monitor->owner->id, thread_current->id);
        exit(0);
      }
      if (monitor->count != 1) {
        oempa("Bad count %d != 1.\n", monitor->count);
        exit(0);
      }

      set_state(10);
      status = x_monitor_notify(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      while (fsm_state != 12) {
        status = x_monitor_wait(monitor, x_eternal);
        if (status != xs_success) {
          oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
          exit(0);
        }
      }

      if (monitor->owner != thread_current) {
        oempa("Bad owner %d != %d\n", monitor->owner->id, thread_current->id);
        exit(0);
      }
      if (monitor->count != 1) {
        oempa("Bad count %d != 1.\n", monitor->count);
        exit(0);
      }

      if (fsm_state != 12) {
        oempa("Bad state %d != 12.\n", fsm_state);
        exit(0);
      }
      set_state(13);
      status = x_monitor_notify(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }

      while (fsm_state != 18) {
        status = x_monitor_wait(monitor, x_eternal);
        if (status != xs_success) {
          oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
          exit(0);
        }
      }

      if (monitor->owner != thread_current) {
        oempa("Bad owner %d != %d\n", monitor->owner->id, thread_current->id);
        exit(0);
      }
      if (monitor->count != 1) {
        oempa("Bad count %d != 1.\n", monitor->count);
        exit(0);
      }

      set_state(19);
      status = x_monitor_notify_all(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      status = x_monitor_exit(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }

      return;

    }
    
    x_thread_sleep(FSM_SLEEP);
    
  }
  
}

static void mt_5(void * t) {

  x_status status;
  x_thread thread = t;
  
  while (1) {
    if (fsm_state == 11) {
      status = x_monitor_enter(monitor, x_eternal);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      status = x_monitor_enter(monitor, x_eternal);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      set_state(12);
      status = x_monitor_notify_all(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      while (fsm_state != 16) {
        status = x_monitor_wait(monitor, x_eternal);
        if (status != xs_success) {
          oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
          exit(0);
        }
      }

      if (monitor->owner != thread) {
        oempa("Bad owner %d != %d\n", monitor->owner->id, thread->id);
        exit(0);
      }
      if (monitor->count != 2) {
        oempa("Bad count %d != 2.\n", monitor->count);
        exit(0);
      }

      set_state(17);      
      status = x_monitor_exit(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      if (monitor->owner != thread) {
        oempa("Bad owner\n");
        exit(0);
      }
      status = x_monitor_notify_all(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      status = x_monitor_exit(monitor);
      if (status != xs_success) {
        oempa("%s: Bad status '%s'\n", __FILE__, status2char(status));
        exit(0);
      }

      return;
      
    }
    
    x_thread_sleep(FSM_SLEEP);
    
  }
  
}

static x_thread fsm_thread;

typedef struct mt_Thread * mt_thread;

typedef struct mt_Thread {
  x_Thread Thread;
  x_thread thread;
  x_ubyte * stack;
} mt_Thread;

static mt_Thread * mt_Threads;

#define NUM_FSM_THREADS (5)

static void monitor_fsm(void * t) {

  x_int i;
  x_int counter = 0;
  x_int check_counter = 0;
  x_int check_max = 0;
  x_status status;
  
  while (1) {
    check_counter += 1;
    if (fsm_state == 0) {
      counter += 1;
      check_counter = 0;
      mt_Threads = x_mem_get(NUM_FSM_THREADS * sizeof(mt_Thread));
      memset(mt_Threads, 0x00, NUM_FSM_THREADS * sizeof(mt_Thread));
      for (i = 0; i < NUM_FSM_THREADS; i++) {
        mt_Threads[i].thread = &mt_Threads[i].Thread;
        mt_Threads[i].stack = x_mem_get(MONT_STACK_SIZE);
      }
      status = x_thread_create(mt_Threads[0].thread, mt_1, &mt_Threads[0], mt_Threads[0].stack, MONT_STACK_SIZE, 4, TF_START);
      if (status != xs_success) {
        oempa("%s: status = '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      else {
        oempa("FSM thread 0, id %d.\n", mt_Threads[0].thread->id);
      }
      status = x_thread_create(mt_Threads[1].thread, mt_2, &mt_Threads[1], mt_Threads[1].stack, MONT_STACK_SIZE, 4, TF_START);
      if (status != xs_success) {
        oempa("%s: status = '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      else {
        oempa("FSM thread 1, id %d.\n", mt_Threads[1].thread->id);
      }
      status = x_thread_create(mt_Threads[2].thread, mt_3, &mt_Threads[2], mt_Threads[2].stack, MONT_STACK_SIZE, 4, TF_START);
      if (status != xs_success) {
        oempa("%s: status = '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      else {
        oempa("FSM thread 2, id %d.\n", mt_Threads[2].thread->id);
      }
      status = x_thread_create(mt_Threads[3].thread, mt_4, &mt_Threads[3], mt_Threads[3].stack, MONT_STACK_SIZE, 3, TF_START);
      if (status != xs_success) {
        oempa("%s: status = '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      else {
        oempa("FSM thread 3, id %d.\n", mt_Threads[3].thread->id);
      }
      status = x_thread_create(mt_Threads[4].thread, mt_5, &mt_Threads[4], mt_Threads[4].stack, MONT_STACK_SIZE, 2, TF_START);
      if (status != xs_success) {
        oempa("%s: status = '%s'\n", __FILE__, status2char(status));
        exit(0);
      }
      else {
        oempa("FSM thread 4, id %d.\n", mt_Threads[4].thread->id);
      }
      set_state(1);
    }

    if (check_counter > check_max) {
      check_max = check_counter;
    }

    if (check_counter > 300) {
      oempa("Stuck at state state = %d\n", fsm_state);
      for (i = 0; i < NUM_FSM_THREADS; i++) {
        oempa("Thread %d, id %3d, state = %s, switches = %d.\n", i + 1, mt_Threads[i].thread->id, x_state2char(mt_Threads[i].thread), mt_Threads[i].thread->num_switches);
      }
      exit(0);
    }
    
    if (fsm_state == 25) {
      oempa("At loop %d, reseting FSM, check max = %d (sizeof(x_Monitor) = %d).\n", counter, check_max, sizeof(x_Monitor));
      for (i = 0; i < NUM_FSM_THREADS; i++) {
        if (mt_Threads[i].thread->waiting_for != NULL) {
          oempa("Waiting for thread %d is not NULL.\n", i);
          exit(0);
        }
        if (mt_Threads[i].thread->l_owned != NULL) {
          oempa("Owned list for thread %d is not NULL.\n", i);
          exit(0);
        }
        if (mt_Threads[i].thread->a_prio != mt_Threads[i].thread->c_prio) {
          oempa("Priority of thread %d has not been reset properly...\n", i);
          exit(0);
        }
      }

      for (i = 0; i < NUM_FSM_THREADS; i++) {
        x_mem_free(mt_Threads[i].stack);
      }
      x_mem_free(mt_Threads);
      x_monitor_delete(monitor);
      x_mem_free(monitor);
      set_state(0);
      x_thread_sleep(second * 10);
    }

    x_thread_sleep(10);

  }
  
}

x_ubyte * monitor_test(x_ubyte * memory) {

  x_status status;

  second = x_seconds2ticks(1);
  oempa("1 second = %d ticks.\n", second);
  
  control = x_alloc_static_mem(memory, sizeof(x_Thread));
  status = x_thread_create(control, monitor_tests, control, x_alloc_static_mem(memory, MONT_STACK_SIZE), MONT_STACK_SIZE, 2, TF_START);
  if (status != xs_success) {
    oempa("%s: status = '%s'\n", __FILE__, status2char(status));
    exit(0);
  }
  else {
    oempa("Monitor control thread 1, id = %d\n", control->id);
  }

  fsm_thread = x_alloc_static_mem(memory, sizeof(x_Thread));
  status = x_thread_create(fsm_thread, monitor_fsm, fsm_thread, x_alloc_static_mem(memory, MONT_STACK_SIZE), MONT_STACK_SIZE, 2, TF_START);
  if (status != xs_success) {
    oempa("%s: status = '%s'\n", __FILE__, status2char(status));
    exit(0);
  }
  else {
    oempa("Monitor control thread 2, id = %d\n", fsm_thread->id);
  }
  
  return memory;
  
}
