/* *************************************************************
 *
 *  $Header: /home/cvs/open-wonka/tool/visualtest/java/com/acunia/wonka/test/awt/IAA/BorderedLabel.java,v 1.1 2001/11/22 10:42:27 oberfeld Exp $
 *
 *  Copyright (c) 2000 ACUNIA NV
 *
 *  This   software   is  the   confidential  and   proprietary
 *  information  of  Acunia  NV.
 *  You shall not  disclose such  Confidential  Information and
 *  shall  use it  only in  accordance  with the  terms of  the
 *  license agreement you entered into with Acunia NV.
 *
 *  ACUNIA NV MAKES NO  REPRESENTATIONS OR WARRANTIES ABOUT THE
 *  SUITABILITY OF  THE SOFTWARE,  EITHER  EXPRESS  OR IMPLIED,
 *  INCLUDING  BUT NOT  LIMITED  TO  THE IMPLIED  WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS  FOR  A  PARTICULAR  PURPOSE,  OR
 *  NON-INFRINGEMENT.  ACUNIA NV  SHALL NOT  BE LIABLE  FOR ANY
 *  DAMAGES  SUFFERED  BY  LICENSEE   AS  A  RESULT  OF  USING,
 *  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 *  Last modification by: $Author: oberfeld $
 *  Last modification on: $Date: 2001/11/22 10:42:27 $
 *
 *  *************************************************************/


package com.acunia.wonka.test.awt.IAA;

import java.awt.*;

/**
 * Displays a label with a specific border.
 *
 * @author Paul Neyens
 */
public class BorderedLabel extends Label {
  /**
   * Shadow type.
   */
  public static final int RAISED = 0;
  /**
   * Shadow type.
   */
  public static final int LOWERED = 1;
  /**
   * Border type.
   */
  public static final int ETCHED = 0;
  /**
   * Border type.
   */
  public static final int SHADOW = 1;

  private final int borderType;
  private final int shadowType;
  private final int shadowThickness;

  /**
   * Constructs a label with the specified border type and width.
   */
  public BorderedLabel(String label, int alignment, int borderType, int shadowType, int shadowThickness) {
    super(label, alignment);
    this.borderType = borderType;
    this.shadowType = shadowType;
    this.shadowThickness = shadowThickness;
  }

  /**
   * Constructs an empty label with a ETCHED, RAISED border type.
   */
  public BorderedLabel() {
    this("", LEFT, ETCHED, RAISED, 1);
  }

  /**
   * Constructs a label with a ETCHED, RAISED border type.
   */
  public BorderedLabel(String label) {
    this(label, LEFT, ETCHED, RAISED, 1);
  }

  /**
   * Constructs a label with a ETCHED, RAISED border type.
   */
  public BorderedLabel(String label, int alignment) {
    this(label, alignment, ETCHED, RAISED, 1);
  }

  /**
   * Constructs a label with the specified border type.
   * Default border width is 1 pixel.
   */
  public BorderedLabel(String label, int borderType, int shadowType) {
    this(label, LEFT, borderType, shadowType, 1);
  }

  public void paint(Graphics g) {
    super.paint(g);
    final Dimension size = getSize();
    final int width = size.width - 1;
    final int height = size.height - 1;
    final Color background = getBackground();
    final Color topLeft;
    final Color bottomRight;
    switch (shadowType) {
      case LOWERED:
        topLeft = background.darker();
        bottomRight = background.brighter();
        break;
      default:
        topLeft = background.brighter();
        bottomRight = background.darker();
    }
    switch (borderType) {
      case ETCHED:
        g.setColor(topLeft);
        for (int i = 0; i < shadowThickness; i++) {
          g.drawRect(i,i,
                     width - 2*i - shadowThickness, height - 2*i - shadowThickness);
        }
        g.setColor(bottomRight);
        for (int i = 0; i < shadowThickness; i++) {
          g.drawRect(i + shadowThickness, i + shadowThickness,
                      width - 2*i - shadowThickness, height - 2*i - shadowThickness);
        }
        break;
      default:
        g.setColor(topLeft);
        for (int i = 0; i < shadowThickness; i++) {
          g.drawRect(i,i, width - i, height - i);
        }
        g.setColor(bottomRight);
        for (int i = 0; i < shadowThickness; i++) {
          g.drawLine(i, height - i, size.width - i, height - i);
          g.drawLine(width - i, i, width - i, size.height - i);
        }
    }
  }

}
/* ***************************************************************************
 * File Revision: $Revision: 1.1 $
 * File History:
 * $Log: BorderedLabel.java,v $
 * Revision 1.1  2001/11/22 10:42:27  oberfeld
 * First of the IAA tests. Currently onlythe BorderedComponentTest is working, the rest use MediaTracker which is NOT YET available, but can make quite a challenge to test the tracker while it is being built.
 *
 * Revision 1.1.1.1  2001/04/03 09:41:25  peeters
 * awt util component
 *
 * Revision 1.1  2001/03/29 14:59:10  pollock
 * next update from old product/util tree
 *
 * Revision 1.1  2001/03/29 09:34:52  neyens
 * New awt component.
 *
 * ***************************************************************************/

