/*
 * ComboBox.java
 *
 * Created on August 13, 2001, 2:22 PM
 */

package com.acunia.wonka.test.awt.IAA;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

/**
 *
 * @author  ivan
 * @version
 */
public class ComboBox extends Container implements ImageDefs, ItemSelectable {
  
  private ImageIcon _imageIcon;
  private ItemList _list;
  private Window _listWindow;

  private class DropDownWindow extends Window {
    
    public DropDownWindow(Frame parent) {
      super(parent);
      this.setBackground(Color.white);
      this.addFocusListener(new FocusAdapter() {
        public void focusLost(FocusEvent fe) {
          DropDownWindow.this.dispose();
          _listWindow = null;
        }
      });
    }
    
    public void paint(Graphics g) {
      super.paint(g);
      Dimension s = this.getSize();
      g.drawRect(0, 0, s.width - 1, s.height - 1);
    }
    
    public Insets getInsets() {
      return new Insets(1, 1, 1, 1);
    }
  }

  public ComboBox(String labelText, Renderer renderer) {
    this(labelText, null, null, false, renderer);
  }

  
  public ComboBox(String labelText, boolean editable) {
    this(labelText, null, null, editable, null);
  }

  
  public ComboBox(String labelText, Vector items) {
    this(labelText, items, null, false, null);
  }
  
  /** Creates new ComboBox */
  public ComboBox(String labelText, Vector items, ImageButtonConfig config, boolean editable, Renderer renderer) {
    setLayout(new BorderLayout());
    ImageButtonConfig downConfig = null;
    if (config == null) {
        try {
          downConfig = new ImageButtonConfig(this, LISTIMAGE ,LISTACTIVE, LISTDISABLED);
        } catch (Exception ex) {
          ex.printStackTrace();
        }
    } else {
      downConfig = config;
    }
    _imageIcon = new ImageIcon(labelText, downConfig, ImageIcon.LEFT, editable);
    _list = renderer == null ? new ItemList(items) : new ItemList(items, renderer);
    _imageIcon.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
        if (_listWindow == null) {
          _imageIcon.requestFocus();
          Point p = ComboBox.this.getLocationOnScreen();
          Dimension d = ComboBox.this.getSize();
          Frame parentFrame = findParenFrame();
          Insets parentFrameInsets = parentFrame.getInsets();
          int width = parentFrame.getSize().width - parentFrameInsets.left - parentFrameInsets.right - 4;
          _listWindow = new DropDownWindow(parentFrame);
          _listWindow.setLayout(new BorderLayout());
          _listWindow.add(_list);
          _listWindow.pack();
          int x = parentFrame.getLocationOnScreen().x + parentFrameInsets.left + 2;
          _listWindow.setLocation(x, p.y + d.height);
          _listWindow.setSize(width, _listWindow.getSize().height);
          _listWindow.show();
        } else {
          _listWindow.dispose();
          _listWindow = null;
        }
      }
    });
    _list.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent ie) {
        if (ie.getStateChange() == ie.SELECTED) {
          _imageIcon.setLabelText(ie.getItem().toString());
          if (_listWindow != null) {
            _listWindow.dispose();
            _listWindow = null;
          }
          ComboBox.this.getParent().invalidate();
          if (ComboBox.this.getParent() != null) {
            ComboBox.this.getParent().validate();
          }
        }
      }
    });
    add(_imageIcon, BorderLayout.NORTH);
  }
  
  public ComboBox(String labelText) {
    this(labelText, (Renderer) null);
  }
  
  public void setLabelText(String text) {
    _imageIcon.setLabelText(text);
  }
  
  public String getLabelText() {
    return _imageIcon.getLabelText();
  }
  
  public void setLabelStatus(boolean status) {
    _imageIcon.setLabelStatus(status);
  }
  
  private Frame findParenFrame() {
    Container c;
    Component comp = this;
    while (!((c = comp.getParent()) instanceof Frame)) {
      comp = c;
    }
    return (Frame) c;
  }
  
  public void addItem(Object item) {
    _list.addItem(item);
  }
  
  public void removeItem(Object item) {
    _list.removeItem(item);
  }
  
  public void removeItem(int index) {
    _list.removeItem(index);
  }
  
  
  public void removeAllItems() {
    _list.removeAll();
  }
  
  public int getItemCount() {
    return _list.getItemCount();
  }
  
  public void setSelectedIndex(int index) {
    _list.setSelected(index, true);
  }
  
  public Object[] getSelectedObjects() {
    return _imageIcon.isEditable() ? new String[] { _imageIcon.getLabelText() } : _list.getSelectedObjects();
  }
  
  public void removeItemListener(ItemListener itemListener) {
    _list.removeItemListener(itemListener);
  }
  
  public void addItemListener(ItemListener itemListener) {
    _list.addItemListener(itemListener);
  }
  
  public void setRowHeight(int size) {
    _list.setUnitIncrement(size);
  }
  
  public Dimension getPreferredSize() {
    Dimension d = _imageIcon.getPreferredSize();
    Insets insets = getInsets();
    return new Dimension(d.width + insets.left + insets.right, d.height + insets.top + insets.bottom);
  }
  
  public Dimension getMinimumSize() {
    return getPreferredSize();
  }
  
  public Dimension getMaximumSize() {
    return getPreferredSize();
  }
  
  public Insets getInsets() {
    return new Insets(1, 1, 1, 1);
  }
  
  public void paint(Graphics g) {
    super.paint(g);
    Dimension s = getSize();
    g.drawRect(0, 0, s.width - 1, s.height - 1);
  }
  
  public static void main(String[] args) throws Exception {
    final Frame f = new Frame("List Test");
    f.setSize(300, 200);
    f.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent we) {
        f.dispose();
        System.exit(0);
      }
    });
    f.setLayout(new FlowLayout());
    
    ComboBox cb = new ComboBox("<CHOOSE A GREETING>", true);
    ComboBox cb2 = new ComboBox("<CHOOSE A GREETING>");
    cb2.addItem("Hello");
    cb2.addItem("Bonjour");
    cb2.addItem("Yo");
    cb2.addItem("Good morning");
    cb2.addItem("Good evening");
    cb2.setRowHeight(25);
    f.add(cb);
    f.add(cb2);
    f.add(new Button("adfija"));
    cb.addItem("Hello");
    cb.addItem("Bonjour");
    cb.addItem("Yo");
    cb.addItem("Good morning");
    cb.addItem("Good evening");
    cb.addItem("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa");
    f.show();
  }
  
}
