/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/



// Author: N.Oberfeld
// Version 1.01.01
// Created: 2001/08/29

package com.acunia.wonka.test.awt.IAA;

//import
import com.acunia.wonka.test.awt.*;
import java.io.*;        // File & Streams
import java.awt.*;       // layout & buttons
import java.util.Vector; //vector
import java.awt.event.*; // button events

public class ComboTest extends VisualTestImpl implements ImageDefs, ActionListener, ItemListener{
  /****************************************************************/
  /** variables
  */
  private ComboBox boxLeft;
  private ComboBox boxRight;
  private Button addLeft;
  private Button removeLeft;
  private Button addRight;
  private Button removeRight;

  int currentItem;

  private List display;
  /****************************************************************/
  /** constructor
  */
  public ComboTest() {
    super();

    // build screen
    setLayout(new BorderLayout());
     // event list
    display = new List(3,false);
    add(display, BorderLayout.CENTER);

    //images
    ImageButtonConfig yes = null;
    ImageButtonConfig no = null;
    try {
      yes = new ImageButtonConfig(this, YESIMAGE, YESACTIVE, YESACTIVE);
      no = new ImageButtonConfig(this, NOIMAGE, NOACTIVE, NOACTIVE);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

    // item vectors
    Vector leftvector = new Vector(8);
    Vector rightvector = new Vector(8);
    currentItem=0;
    while(currentItem<16) {
      leftvector.add("(left) Item no."+currentItem);
      currentItem++;
      rightvector.add("(right) Item no."+currentItem);
      currentItem++;
    }

    //buttons
    Panel top = new Panel(new GridLayout(3,2));
      boxLeft = new ComboBox("Left combo box", leftvector, yes, false, null);
      boxLeft.setName("Left combo box");
      boxLeft.addItemListener(this);
      top.add(boxLeft);

      boxRight = new ComboBox("right combo box", rightvector, no, true, null);
      boxRight.setName("right combo box");
      boxRight.addItemListener(this);
      top.add(boxRight);

      addLeft = new Button("Add item");
      addLeft.addActionListener(this);
      top.add(addLeft);

      addRight = new Button("Add item");
      addRight.addActionListener(this);
      top.add(addRight);

      removeLeft = new Button("Remove first");
      removeLeft.addActionListener(this);
      top.add(removeLeft);

      removeRight = new Button("Remove first");
      removeRight.addActionListener(this);
      top.add(removeRight);
    add(top, BorderLayout.NORTH);

  }

  /****************************************************************/
  /**button pressed : Display event */
  private void displayText(String text) {
    if(display.getItemCount()>40) {
      display.removeAll();
    }
    System.out.println(text);
    display.add(text,0);
  }

  /********************/
  /** Buttons */
  public void actionPerformed(ActionEvent evt) {
    Object source = evt.getSource();
    if(source == addLeft){
      boxLeft.addItem("(left) Item no."+currentItem);
      displayText("Added on left box: Item no."+currentItem);
      currentItem++;
    }
    else if(source == removeLeft && boxLeft.getItemCount()>0) {
      boxLeft.removeItem(0);
      displayText("removing first item of left box: remaining "+boxLeft.getItemCount());
    }
    else if(source == addRight){
      boxRight.addItem("(right) Item no."+currentItem);
      displayText("Added on right box: Item no."+currentItem);
      currentItem++;
    }
    else if(source == removeRight && boxRight.getItemCount()>0) {
      boxRight.removeItem(0);
      displayText("removing first item of left box: remaining "+boxRight.getItemCount());
    }
  }

  /***************************/
  /** ItemList event display */
  public void itemStateChanged(ItemEvent evt) {
    Component source = (Component)evt.getSource();
    boolean state = (evt.getStateChange() == ItemEvent.SELECTED);

    displayText("Source=<"+source.getName()+"> item= <"+evt.getItem()+"> state="+((state)?"SELECTED":"DESELECTED"));
  }


  /**************************/
  /** VisualTestEngine help */
  public String getHelpText(){
    return "Test on the construction and behavior of the giza.component.util classes used for the IAA demo:\n"+
           "The screen should show two ComboBox boxes, each consisting out of 8 elements. \n"+
           "=> Pressing the combo-box' down button forces the combo-box list to appear.\n"+
           "=> Clicking an item in the list makes the list disappear again. The name of the selected item is displayed in the list below\n"+
           "=> Scroll along the items of the ItemList using the right ScrollBar\n"+
           "=> The 'add'-buttons add one item to the list of their combo box\n"+
           "=> The 'remove first' buttons remove the first item from the list of their combo box";
  }

  /********************/
  /** test main */
  static public void main (String[] args) {
	  new ComboTest();
  }

  // (end of class CopyWriter
}
