/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

// V.1.01.00 2001/08/29 : first release
// V.1.01.01 2001/08/29 : excluded CVS directories from files list
// V.1.01.02 2001/08/29 : Option to only load checked files
// V.1.02.01 2001/08/31 : Double list and quick scan possibilities
// V.1.02.02 2001/08/31 : bugfixes and selective file-load for subdirs
// V.1.02.03 2001/08/31 : added buttons for showing scripts and c/h/java files
// V.1.02.03 2001/09/03 : logging file
// V.1.02.04 2001/09/03 : edit screen allows to place header on current text position / button for header files only
// V.1.02.05 2001/09/03 : added buttons for same directory and last directory


// Author: N.Oberfeld
// Version 1.01.01
// Created: 2001/08/29

package com.acunia.wonka.test.awt.IAA;

//import
import com.acunia.wonka.test.awt.*;
import java.io.*;        // File & Streams
import java.awt.*;       // layout & buttons
import java.awt.event.*; // button events

public class ImageComponentsTest extends VisualTestImpl implements ImageDefs, ActionListener, ItemListener{
	
  /****************************************************************/
  /** variables
  */
  private ImageButton ib1 = null;
  private ImageButton ib2 = null;
  private Checkbox cb1;
  private Checkbox cb2;

  private List display;

  /****************************************************************/
  /** constructor
  */
  public ImageComponentsTest() {
    super();

    setLayout(new BorderLayout());

    display = new List(2,false);
    display.add("Observer ImageUpdate() dispayed here");
    add(display, BorderLayout.SOUTH);

    Panel the_flow = new Panel(new FlowLayout() );
    Panel left = new Panel(new GridLayout(2,2));
      try {
        // imageLabel images
        Image newsdirect = Toolkit.getDefaultToolkit().getImage(NEWSIMAGE);
        Image newstracked = ImageUtils.loadImage(this, NEWSIMAGE);

        // imageLabels
        the_flow.add(new Label("through toolkit =>"));
        ImageLabel il1 = new ImageLabel(newsdirect);
        the_flow.add(il1);
        the_flow.add(new Label("<= ImageLabels =>"));
        ImageLabel il2 = new ImageLabel(newstracked);
        the_flow.add(il2);
        the_flow.add(new Label("<= through MediaTracker"));

        //imageButtona image
        ImageButtonConfig map = new ImageButtonConfig(this, MAPIMAGE, MAPPASSIVE, MAPACTIVE);
        //imageButtons
        ib1 = new ImageButton(map);
        ib1.setName("upper button");
        ib1.addActionListener(this);
        ib2 = new ImageButton(map);
        ib2.setName("lower button");
        ib2.addActionListener(this);

        // filtered images
        Image newsinverted = ImageUtils.createInvertedImage(newstracked);
        Image newsdisabled = ImageUtils.createDisabledImage(newstracked);
        left.add(new ImageLabel(newsinverted));
        left.add(new Label("<= inverted"));
        left.add(new ImageLabel(newsdisabled));
        left.add(new Label("<= disabled"));
      }
      catch (Exception ex) {
        ex.printStackTrace();
      }
    add(the_flow, BorderLayout.NORTH);
    add(left, BorderLayout.EAST);

      // imagebuttons panel
    Panel the_buttons = new Panel(new GridLayout(2,2) );
      cb1 = new Checkbox("Enabled",true);
      cb1.addItemListener(this);
      the_buttons.add(cb1);
      the_buttons.add(ib1);
      cb2 = new Checkbox("Enabled",true);
      cb2.addItemListener(this);
      the_buttons.add(cb2);
      the_buttons.add(ib2);
    add(the_buttons, BorderLayout.WEST);

    add(new BorderedContainer() , BorderLayout.CENTER);
  }

  /****************************************************************/
  /**button pressed : Display event */
  private void displayText(String text) {
    if(display.getItemCount()>40) {
      display.removeAll();
    }
    System.out.println(text);
    display.add(text,0);
  }

  /****************************************************************/
  /**button pressed : Display event */
  public void actionPerformed(ActionEvent evt) {
    Component source = (Component)evt.getSource();
    displayText(source.getName()+" clicked:"+evt.getID());
  }

  /****************************************************************/
  /**Checkbox state changed: */
  public void itemStateChanged(ItemEvent evt) {
    Component source = (Component)evt.getSource();
    boolean state = (evt.getStateChange() == ItemEvent.SELECTED);
    if(source == cb1){
      ib1.setEnabled(state);
    }
    else {
      ib2.setEnabled(state);
    }
  }


  public String getHelpText(){
    return "Test on the construction and behavior of the giza.component.util classes used for the IAA demo:\n"+
           "On the screen you should see:\n"+
           "=> two ImageLabel images on top of the screen (the 'news' circles). the left one is constructed with an image loaded"+
           "directly through Toolkit.getDefaultToolkit.getImage.(which means that at construction time, the image itself is not yet present.)"+
           " The right one is loaded through the ImageUtils.loadImage() function that uses a MediaTracker to ensure that the image is present\n"+
           "=> two ImageButton buttons (the 'map' roundels). This buttons can be enabled or disabled by means of the 'enabled' checkboxes"+
           " left of them. A disabled btton appears grey. Futthermore, they can be clicked. a clicked button changes color just as well"+
           " and sends an ActionPerformed message that will be displayed in the image list below\n"+
           "\nspecial AWT classes needed:\n==========================\n"+
           "->MediaTracker for the ImageLabel and ImageButton images, used in class ImageUtil\n"+
           "->(loading of png format images from file)";
  }

  /********************/
  /** test main */
  static public void main (String[] args) {
	  new ImageComponentsTest();
  }

  // (end of class CopyWriter
}
