/*
 * ImageButton.java
 *
 * Created on 7 augustus 2001, 14:43
 */

package com.acunia.wonka.test.awt.IAA;

import java.awt.*;
import java.awt.event.*;

/**
 *
 * @author  Standard
 */
public class ImageIcon extends Container {
    
    public static final int TOP = 1;
    public static final int BOTTOM = 2;
    public static final int LEFT = 3;
    public static final int RIGHT = 4;
    
    private Component _label;
    private ImageButton _imageButton;
    private int _labelAlignment;

    public ImageIcon(String labelText, ImageButtonConfig config) {
        this(labelText, config, BOTTOM, false);
    }
    
    public ImageIcon(String labelText, ImageButtonConfig config, boolean editable) {
        this(labelText, config, BOTTOM, editable);
    }
    
    public ImageIcon(String labelText, ImageButtonConfig config, int labelPos) {
        this(labelText, config, labelPos, false);
    }
    
    public ImageIcon(String labelText, ImageButtonConfig config, int labelPos, boolean editable) {
        this.setBackground(Color.white);
        _label = editable ? (Component) new TextField(labelText) : (Component) new Label(labelText);
        _label.setBackground(Color.white);
        _imageButton = new ImageButton(config);
        setLabelAlignment(labelPos);
    }
    
    public void setLabelStatus(boolean status) {
      _label.setVisible(status);
    }
    
    public String getLabelText() {
      return _label instanceof Label ? ((Label) _label).getText() : ((TextField) _label).getText();
    }
    
    public boolean isEditable() {
      return _label instanceof TextField;
    }
    
    public void setLabelText(String labelText) {
        if (_label instanceof Label) {
          ((Label) _label).setText(labelText);
        } else {
          ((TextField) _label).setText(labelText);
        }
        invalidate();
        if (getParent() != null) {
          getParent().validate();
        }
    }
    
    public void setEnabled(boolean enabled) {
        super.setEnabled(enabled);
        _label.setEnabled(enabled);
        _imageButton.setEnabled(enabled);
    }
    
    public void addActionListener(ActionListener l) {
        _imageButton.addActionListener(l);
    }
    
    public void removeActionListener(ActionListener l) {
        _imageButton.removeActionListener(l);
    }
    
    private void layoutComponents(int labelPos) {
        setLayout(labelPos == TOP || labelPos == BOTTOM 
            ? (LayoutManager) new BorderLayout()
            : (LayoutManager) new GridBagLayout()
        );
        
        if (labelPos == TOP || labelPos == BOTTOM) {
            if (_label instanceof Label) {
              ((Label) _label).setAlignment(Label.CENTER);
            }
            add(_label, labelPos == TOP ? BorderLayout.NORTH : BorderLayout.SOUTH);
            add(_imageButton, BorderLayout.CENTER);
        } else if (labelPos == LEFT || labelPos == RIGHT) {
            GridBagConstraints gbc = new GridBagConstraints();
            gbc.fill = gbc.BOTH;
            gbc.weightx = labelPos == LEFT ? 1.0 : 0.0;
            add(labelPos == LEFT ? (Component) _label : (Component) _imageButton, gbc);
            gbc.weightx = labelPos == LEFT ? 0.0 : 1.0;
            add(labelPos == LEFT ? (Component) _imageButton : (Component) _label, gbc);
        }
    }
    
    public void setLabelFont(Font font) {
      _label.setFont(font);
    }
    
    public void setLabelAlignment(int alignment) {
        _labelAlignment = alignment;
        layoutComponents(alignment);
    }
    
    public int getLabelAlignment() {
        return _labelAlignment;
    }
}