/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

// V.1.01.00 2001/08/29 : first release
// V.1.01.01 2001/08/29 : excluded CVS directories from files list
// V.1.01.02 2001/08/29 : Option to only load checked files
// V.1.02.01 2001/08/31 : Double list and quick scan possibilities
// V.1.02.02 2001/08/31 : bugfixes and selective file-load for subdirs
// V.1.02.03 2001/08/31 : added buttons for showing scripts and c/h/java files
// V.1.02.03 2001/09/03 : logging file
// V.1.02.04 2001/09/03 : edit screen allows to place header on current text position / button for header files only
// V.1.02.05 2001/09/03 : added buttons for same directory and last directory


// Author: N.Oberfeld
// Version 1.01.01
// Created: 2001/08/29

package com.acunia.wonka.test.awt.IAA;

//import
import com.acunia.wonka.test.awt.*;
import java.io.*;        // File & Streams
import java.awt.*;       // layout & buttons
import java.awt.event.*; // button events

public class ImageScrollbarTest extends VisualTestImpl implements AdjustmentListener {
	
  /****************************************************************/
  /** variables
  */
  ScrollBar iv1 = null;
  ScrollBar iv2 = null;
  ScrollBar ih1 = null;
  ScrollBar ih2 = null;
  Scrollbar jv1;
  Scrollbar jv2;
  Scrollbar jh1;
  Scrollbar jh2;
  List display;
  /****************************************************************/
  /** constructor
  */
  public ImageScrollbarTest() {
    super();

    setLayout(new BorderLayout());

    display = new List(2,false);
    display.add("Observer ImageUpdate() dispayed here");
    add(display, BorderLayout.SOUTH);


    ImageButtonConfig barup = null;
    ImageButtonConfig bardown = null;
    Image barmiddle = null;
    try {
      // imageScrollbar images
      barup = new ImageButtonConfig(this, ScrollBar.SCROLLUP_DEFAULT, ScrollBar.SCROLLUP_PASSIVE, ScrollBar.SCROLLUP_ACTIVE);
      bardown = new ImageButtonConfig(this, ScrollBar.SCROLLDOWN_DEFAULT, ScrollBar.SCROLLDOWN_PASSIVE, ScrollBar.SCROLLDOWN_ACTIVE);
      barmiddle = ImageUtils.loadImage(this, ScrollBar.SCROLLMIDDLE);
    }
    catch (Exception ex) {
      ex.printStackTrace();
      displayText(ex.toString());
    }

    if(barup != null && bardown!= null){
      // left screen scrollbars
      Panel leftbars = new Panel(new GridLayout(1,2) );
        iv1=new ScrollBar(barup, bardown, barmiddle, java.awt.Scrollbar.VERTICAL);
        iv1.addAdjustmentListener(this);
        jv1=new Scrollbar(Scrollbar.VERTICAL, iv1.getMaximum(), 1, iv1.getMinimum(), iv1.getMaximum());
        jv1.setBlockIncrement(3);
        jv1.addAdjustmentListener(this);
        leftbars.add(iv1);
        leftbars.add(jv1);
      add(leftbars, BorderLayout.WEST);
      // right screen scrollbars
      Panel rightbars = new Panel(new GridLayout(1,2) );
        iv2=new ScrollBar(barup, bardown, barmiddle, java.awt.Scrollbar.VERTICAL);
        iv2.addAdjustmentListener(this);
        jv2=new Scrollbar(Scrollbar.VERTICAL, iv2.getMaximum(), 1, iv2.getMinimum(), iv2.getMaximum());
        jv2.setBlockIncrement(3);
        jv2.addAdjustmentListener(this);
        rightbars.add(jv2);
        rightbars.add(iv2);
      add(rightbars, BorderLayout.EAST);
      // horizontal scrollbars
      Panel horbars = new Panel(new GridLayout(4,1) );
        ih1=new ScrollBar(barup, bardown, barmiddle, java.awt.Scrollbar.HORIZONTAL);
        ih1.addAdjustmentListener(this);
        jh1=new Scrollbar(Scrollbar.HORIZONTAL, ih1.getMinimum(), 1, ih1.getMinimum(), ih1.getMaximum());
        jh1.setBlockIncrement(3);
        jh1.addAdjustmentListener(this);
        ih2=new ScrollBar(barup, bardown, barmiddle, java.awt.Scrollbar.HORIZONTAL);
        ih2.addAdjustmentListener(this);
        jh2=new Scrollbar(Scrollbar.HORIZONTAL, ih2.getMinimum(), 1, ih2.getMinimum(), ih2.getMaximum());
        jh2.setBlockIncrement(3);
        jh2.addAdjustmentListener(this);
        horbars.add(ih1);
        horbars.add(jh1);
        horbars.add(jh2);
        horbars.add(ih2);
      add(horbars, BorderLayout.CENTER);
    }
  }


  /****************************************************************/
  /**scrollbar moved pressed : display event on list and mirror scroll bar events
  from util.ScrollBar to AWT Scrollbar and vice versa*/
  public void adjustmentValueChanged(AdjustmentEvent evt) {
    Component source = (Component)evt.getAdjustable();
    // action = unit-increment, unit-decrement, block_increment, block-decrement, track...
    int action = evt.getAdjustmentType();
    String actionstring = "<TRACK>";
    if(action == AdjustmentEvent.UNIT_INCREMENT) {
      actionstring = "<UNIT_INCREMENT>";
    }
    else if(action == AdjustmentEvent.UNIT_DECREMENT) {
      actionstring = "<UNIT_DECREMENT>";
    }
    else if(action == AdjustmentEvent.BLOCK_INCREMENT) {
      actionstring = "<BLOCK_INCREMENT>";
    }
    else if(action == AdjustmentEvent.BLOCK_DECREMENT) {
      actionstring = "<BLOCK_DECREMENT>";
    }
    displayText(source.getName()+" action="+actionstring+" value="+evt.getValue());
    // link
    if(source == ih1){
      jh1.setValue(ih1.getValue());
    }
    else if(source == ih2) {
      jh2.setValue(ih2.getValue());
    }
    else if(source == iv1){
      jv1.setValue(iv1.getMaximum() - iv1.getValue());
    }
    else if(source == iv2) {
      jv2.setValue(iv2.getMaximum() - iv2.getValue());
    }
    else if(source == jh1){
      ih1.setValue(jh1.getValue());
    }
    else if(source == jh2) {
      ih2.setValue(jh2.getValue());
    }
    else if(source == jv1){
      iv1.setValue(jv1.getMaximum() - jv1.getValue());
    }
    else if(source == jv2) {
      iv2.setValue(jv2.getMaximum() - jv2.getValue());
    }
  }

  /************/
  /** display */
  private void displayText(String text) {
    if(display.getItemCount()>40) {
      display.removeAll();
    }
    System.out.println(text);
    display.add(text,0);
  }


  public String getHelpText(){
    return "Test on the construction and behavior of the giza.component.util.ScrollBar class used for the IAA demo:\n"+
           "On the screen you should see:\n"+
           "=> a vertical util.ScrollBar and a java.awt.Scrollbar (note the capital B in the one and the small b in the other spelling)"+
           " on the left side of the screen\n"+
           "=> a vertical util.ScrollBar and a java.awt.Scrollbar on the right side of the screen\n"+
           "=> two sets of horizontal util.ScrollBar and a java.awt.Scrollbar in the center of the screen\n"+
           " The bars are linked so clicking the up and down buttons of the util.ScrollBar also move the scrollbox of the awt Scrollbars"+
           " and changing the position of the box in the AWT scrollbar will also change the (virtual) box value in the util.ScrollBar.\n"+
           "=> All bar movements will throw an adjustmentVauleChanged() event that will be displayed in the list below\n"+
           "\nspecial AWT classes needed:\n==========================\n"+
           "->MediaTracker for the ImageLabel and ImageButton images, used in class ImageUtil";
  }

  /********************/
  /** test main */
  static public void main (String[] args) {
	  new ImageScrollbarTest();
  }

  // (end of class CopyWriter
}
