/*
 * ImageUtils.java
 *
 * Created on August 9, 2001, 4:13 PM
 */

package com.acunia.wonka.test.awt.IAA;

import java.awt.*;
import java.awt.image.RGBImageFilter;
import java.awt.image.FilteredImageSource;
import java.io.*;

/**
 * @author  ivan
 * @version
 */
public class ImageUtils {
    static class ComplementFilter extends RGBImageFilter {
        public ComplementFilter() {
            // The filter's operation does not depend on the
            // pixel's location, so IndexColorModels can be
            // filtered directly.
            canFilterIndexColorModel = true;
        }

        public int filterRGB(int x, int y, int rgb) {
            return rgb ^ 0xffffff;
        }
    }


    static class ImageButtonDisableFilter extends RGBImageFilter {
        public ImageButtonDisableFilter() {
            canFilterIndexColorModel = true;
        }

        public int filterRGB(int x, int y, int rgb) {
            return (rgb & ~0xff000000) | 0x80000000;
        }
    }


    private ImageUtils() {
    }

    public static Image loadImage(Component c, String imageFile) throws Exception {
        System.out.println("<DEBUG> Loading image : " + (imageFile).toString() + " on component " + c.toString());
        MediaTracker tracker = new MediaTracker(c);
        Image image = null;
        if (c.getClass().getResource(imageFile) == null) {
            image = Toolkit.getDefaultToolkit().getImage(imageFile);
        } else {
            image = Toolkit.getDefaultToolkit().getImage(c.getClass().getResource(imageFile));
        }
        tracker.addImage(image, 0);
        tracker.waitForID(0);
        //System.out.println("<FINISHED> Loading image : " + imageFile);
        return image;
    }


/*
    public static Image loadImage(Component c, String imageFile) throws Exception {
System.out.println("<DEBUG>ImageUtils.loadImage()...Loading image : " + (imageFile).toString() + " on component " + c.toString());
        MediaTracker tracker = new MediaTracker(c);
        Image image = null;
        InputStream is = c.getClass().getResourceAsStream(imageFile);
        if (is != null) {
            byte[] img = readBinary(is);
System.out.println("<debug> ImageUtils.loadImage()...create from file stream : " + img.length+" bytes");
            image = Toolkit.getDefaultToolkit().createImage(img);
        }
        else {
System.out.println("<debug> ImageUtils.loadImage()...unable to open stream: create from file : " + imageFile);
            image = Toolkit.getDefaultToolkit().createImage(imageFile);
        }
        tracker.addImage(image, 0);
        tracker.waitForID(0);
System.out.println("<FINISHED> ImageUtils.loadImage()...Loaded image : " + image+" Status="+c.checkImage(image,c));
System.out.println("<FINISHED> ImageUtils.loadImage()...new image width : " + image.getWidth(c));
        return image;
    }
*/
    private static byte[] readBinary(InputStream is) {
        if (is == null) {
            return null;
        }
        byte[] data = null;
        byte[] buffer = new byte[100];
        int i = 0;
        try {
            while ((i = is.read(buffer)) != -1) {
                data = concat(data, buffer, i);
            }
            is.close();
        }
        catch (IOException ioe) {
            return null;
        }
        return data;
    }

    private static byte[] concat(byte[] b1, byte[] b2, int b2_length) {
        if (b1 == null) {
            byte[] b = new byte[b2_length];
            System.arraycopy(b2, 0, b, 0, b2_length);
            return b;
        }
        byte[] b = new byte[b1.length + b2_length];
        System.arraycopy(b1, 0, b, 0, b1.length);
        System.arraycopy(b2, 0, b, b1.length, b2_length);
        return b;
    }

    public static Image createInvertedImage(Image image) {
        return Toolkit.getDefaultToolkit().createImage(
            new FilteredImageSource(image.getSource(),
            new ComplementFilter()));
    }

    public static Image createDisabledImage(Image image) {
        return Toolkit.getDefaultToolkit().createImage(
            new FilteredImageSource(image.getSource(),
            new ImageButtonDisableFilter()));
    }
}
