/*
 * List.java
 *
 * Created on August 13, 2001, 3:53 PM
 */

package com.acunia.wonka.test.awt.IAA;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 *
 * @author  ivan
 * @version
 */
public class ItemList extends ScrollableContainer implements ItemSelectable {
    private static final Color SELECTED_COLOR = new Color(33,68,91);
    
    private class ListItem extends Component {
        
        private Object _itemObject;
        private Renderer _render;
        private boolean _selected;
        private RendererPane _rendererPane;
        
        public ListItem(Object itemObject, Renderer renderer, RendererPane rendererPane) {
            _itemObject = itemObject;
            _renderer = renderer;
            _rendererPane = rendererPane;
            this.addMouseListener(new MouseAdapter() {
                
                //private boolean fireEvent;
                
                public void mousePressed(MouseEvent me) {
                      unselectAll();
                      setSelectedAndPaint(true);
                }
                
                public void mouseReleased(MouseEvent me) {
                      fireEvent();
                }
            });
        }
        
        protected void setSelectedAndPaint(boolean selected) {
            _selected = selected;
            this.repaint();
        }
        
        private void fireEvent() {
            fire(new ItemEvent(ItemList.this, _items.indexOf(this), _itemObject, (_selected ? ItemEvent.SELECTED : ItemEvent.DESELECTED)));
        }
        
        public Object getItemObject() {
            return _itemObject;
        }
        
        public void paint(Graphics g) {
            Component c = _renderer.render(_itemObject, ItemList.this, ItemList.this._items.indexOf(this), _selected);
            Dimension d = this.getSize();
            _rendererPane.paintComponent(g, c, null, 0, 0, d.width, d.height, true);
        }
        
        public void setSelected(boolean selected) {
            setSelectedAndPaint(selected);
            fireEvent();
        }
        
        public boolean isSelected() {
            return _selected;
        }
        
        public boolean equals(Object o) {
            if (o instanceof ListItem) {
                return _itemObject.equals(((ListItem) o)._itemObject);
            }
            return false;
        }
        
        public Dimension getPreferredSize() {
            return _renderer.render(_itemObject, ItemList.this, ItemList.this._items.indexOf(this), _selected).getPreferredSize();
        }
        
        public Dimension getMinimumSize() {
            return _renderer.render(_itemObject, ItemList.this, ItemList.this._items.indexOf(this), _selected).getMinimumSize();
        }
        
        public Dimension getMaximumSize() {
            return _renderer.render(_itemObject, ItemList.this, ItemList.this._items.indexOf(this), _selected).getMaximumSize();
        }
    }
    
    public static class DefaultRenderer extends Component implements Renderer {
        
        private Object _valueObject;
        private boolean _selected;
        private Dimension _dim = new Dimension();
        
        public Component render(Object obj, ItemList list, int index, boolean selected) {
            setValueObject(obj);
            this._selected = selected;
            return this;
        }
        
        private void setValueObject(Object obj) {
            _valueObject = obj;
            Graphics g = getGraphics();
            if (g != null) {
                try {
                    FontMetrics fm = g.getFontMetrics();
                    int width = fm.stringWidth(obj.toString()) + 10;
                    int height = fm.getHeight() + fm.getDescent() + 4;
                    _dim = new Dimension(width, height);
                } finally {
                    g.dispose();
                }
            }
            
        }
        
        public void paint(Graphics g) {
            FontMetrics fm = g.getFontMetrics();
            Dimension size = this.getSize();
            g.clearRect(0, 0, size.width, size.height);
            if (this._selected) {
                g.setColor(SELECTED_COLOR);
                g.fillRect(0, 0, size.width, size.height);
             }
            g.setColor(this._selected ? Color.white : Color.black);
            g.drawString(_valueObject.toString(), 10, size.height - 5 - fm.getDescent());
        }
        
        public Dimension getPreferredSize() {
            return _dim;
        }
        
        public Dimension getMinimumSize() {
            return _dim;
        }
        
        public Dimension getMaximumSize() {
            return _dim;
        }
    }
    
    private static class ListContainer extends Container {
        
        private GridBagConstraints gbc1 = new GridBagConstraints();
        private GridBagConstraints gbc2 = new GridBagConstraints();
        
        public ListContainer() {
            setLayout(new GridBagLayout());
            gbc1.gridwidth = gbc1.REMAINDER;
            gbc1.weightx = 1.0;
            gbc1.fill = gbc1.HORIZONTAL;
            gbc1.anchor = gbc1.NORTHWEST;
            gbc2.gridwidth = gbc2.REMAINDER;
            gbc2.weightx = 1.0;
            gbc2.fill = gbc2.HORIZONTAL;
            gbc2.anchor = gbc2.NORTHWEST;
            gbc2.weighty = 1.0;
        }
        
        public Component add(Component c) {
            int n = this.getComponentCount() - 1;
            if (n != -1) {
                Component comp = this.getComponent(n);
                this.add(comp, gbc1);
            }
            this.add(c, gbc2);
            return c;
        }
    }
    
    private Vector _items;
    private Renderer _renderer;
    private RendererPane _rendererPane;
    private Container _mainContainer;
    private Vector _listeners = new Vector();
    private Dimension _cellDimension = new Dimension(-1, -1);
    
    
    /** Creates new List */
    public ItemList(Vector items, Renderer renderer) {
        super(new ListContainer());
        _mainContainer = (Container) super.getComponent();
        _items = new Vector();
        _rendererPane = new RendererPane();
        _mainContainer.add(_rendererPane);
        setRenderer(renderer);
        
        if (items != null) {
            for (int i = 0; i < items.size(); i++) {
                addItem(items.elementAt(i));
            }
        }
    }
    
    public ItemList(Renderer renderer) {
        this(new Vector(), renderer);
    }
    
    public ItemList(Vector data) {
        this(data, new DefaultRenderer());
    }
    
    public ItemList() {
        this(new DefaultRenderer());
    }
    
    public void setRenderer(Renderer renderer) {
        _renderer = renderer;
        _rendererPane.add(renderer.render("dummy", this, -1, false));
    }
    
    public void setCellDimension(Dimension d) {
        _cellDimension = d;
    }
    
    private void unselectAll() {
        for (int i = 0; i < _items.size(); i++) {
            ListItem li = (ListItem) _items.elementAt(i);
            if (li.isSelected()) {
                li.setSelected(false);
            }
        }
    }
    
    private Vector getSelectedItems() {
        Vector selectedItems = new Vector();
        for (int i = 0; i < _items.size(); i++) {
            ListItem item = (ListItem) _items.elementAt(i);
            if (item.isSelected()) {
                selectedItems.addElement(item.getItemObject());
            }
        }
        return selectedItems;
    }
    
    public int getSelectedIndex() {
        for (int i = 0; i < _items.size(); i++) {
            ListItem item = (ListItem) _items.elementAt(i);
            if (item.isSelected()) {
                return i;
            }
        }
        return -1;
    }
    
    public Object getItemAt(int index) {
      return ((ListItem) _items.elementAt(index)).getItemObject();
    }
    
    public void addItem(Object item) {
        ListItem listItem = new ListItem(item, _renderer, _rendererPane);
        _mainContainer.add(listItem);
        _items.addElement(listItem);
        _mainContainer.invalidate();
        if (_mainContainer.getParent() != null) {
          _mainContainer.getParent().validate();
        }
    }
    
    public void removeItem(Object item) {
        if (_items.contains(item)) {
            int i = _items.indexOf(item);
            _mainContainer.remove(i);
            _items.removeElementAt(i);
        }
    }
    
    public void removeItem(int index) {
        _mainContainer.remove(index + 1);
        _items.removeElementAt(index);
        _mainContainer.invalidate();
        if (_mainContainer.getParent() != null) {
          _mainContainer.getParent().validate();
        }
    }
    
    public void removeAll() {
        for (int i = _items.size() - 1; i >= 0; i--) {
            removeItem(i);
        }
    }
    
    public int getItemCount() {
        return _items.size();
    }
    
    public void setSelected(int index, boolean selected) {
        unselectAll();
        ((ListItem) _items.elementAt(index)).setSelected(selected);
        repaint();
    }
    
    public void setSize(Dimension d) {
        super.setSize(d);
        for (int i = 0; i < _items.size(); i++) {
            ((Component) _items.elementAt(i)).setSize(d);
        }
    }
    
    public Object[] getSelectedObjects() {
        Vector v = getSelectedItems();
        Object[] array = new Object[v.size()];
        for (int i = 0; i < v.size(); i++) {
            array[i] = v.elementAt(i);
        }
        return array;
    }
    
    public void removeItemListener(ItemListener itemListener) {
        _listeners.removeElement(itemListener);
    }
    
    public void addItemListener(ItemListener itemListener) {
        if (!_listeners.contains(itemListener)) {
            _listeners.addElement(itemListener);
        }
    }
    
    private void fire(ItemEvent event) {
        for (int i = 0; i < _listeners.size(); i++) {
            ((ItemListener) _listeners.elementAt(i)).itemStateChanged(event);
        }
    }
    
    /*public Dimension getPreferredSize() {
      int width = 0;
      int height = 0;
      for (int i = 0; i < _items.size(); i++) {
        Dimension d = ((Component) _items.elementAt(i)).getPreferredSize();
        height += d.height;
        if (d.width > width) {
          width = d.width;
        }
      }
      return new Dimension(width, height);
    }
    
    public Dimension getMinimumSize() {
      return getPreferredSize();
    }
    
    public Dimension getMaximumSize() {
      return getPreferredSize();
    }*/
    
    public static void main(String[] args) {
        final Frame f = new Frame("List Test");
        f.setSize(300, 200);
        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent we) {
                f.dispose();
                System.exit(0);
            }
        });
        f.setLayout(new FlowLayout());
        
        ItemList list = new ItemList();
        f.add(list, BorderLayout.CENTER);
        list.addItem("Hello");
        list.addItem("Bonjour");
        list.addItem("Yo");
        f.show();
        try {
          Thread.sleep(3000);
        } catch (Exception ex) {
          ex.printStackTrace();
        }
        list.removeItem(0);
        list.addItem("AAAAAAA");
    }
    
}