/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/



// Author: N.Oberfeld
// Version 1.01.01
// Created: 2001/08/29

package com.acunia.wonka.test.awt.IAA;

//import
import com.acunia.wonka.test.awt.*;
import java.io.*;        // File & Streams
import java.awt.*;       // layout & buttons
import java.awt.event.*; // button events

public class ItemListTest extends VisualTestImpl implements ActionListener, ItemListener{
  /****************************************************************/
  /** variables
  */
  private Button addText;
  private Button remove;

  private ItemList select;
  private int currentItem;

  private List display;

  /****************************************************************/
  /** constructor
  */
  public ItemListTest() {
    super();

    // build screen
    setLayout(new BorderLayout());

    //buttons
    Panel buttons = new Panel(new GridLayout(1,2));
      addText = new Button("Add text");
      addText.addActionListener(this);
      buttons.add(addText);
      remove = new Button("Remove");
      remove.addActionListener(this);
      buttons.add(remove);
    add(buttons, BorderLayout.NORTH);

    //lower area: buttons and list
    Panel down = new Panel(new BorderLayout());
      //ItemList
        select = new ItemList();
        for(currentItem = 0; currentItem<7; currentItem++){
          select.addItem("List item no."+currentItem);
        }
        select.addItemListener(this);
      down.add(select, BorderLayout.NORTH);
     // event list
     display = new List(3,false);
     down.add(display, BorderLayout.CENTER);
    add(down, BorderLayout.CENTER);
  }

  /****************************************************************/
  /**button pressed : Display event */
  private void displayText(String text) {
    if(display.getItemCount()>40) {
      display.removeAll();
    }
    System.out.println(text);
    display.add(text,0);
  }

  /****************************************************************/
  /**button pressed : Display event */
  public void actionPerformed(ActionEvent evt) {
    Object source = evt.getSource();
    if(source == addText){
      select.addItem("List item no."+currentItem);
      select.doLayout();
      displayText("Added item "+currentItem+" :String");
      currentItem++;
    }
    else if(source == remove) {
      int selindex = select.getSelectedIndex();
      if(selindex<0){
        displayText("Removing: no item selected");
      }
      else{
        displayText("Removing list selection "+selindex);
        select.removeItem(selindex);
      }
    }
  }

  /****************************************************************/
  /**Checkbox state changed: */
  public void itemStateChanged(ItemEvent evt) {
    Component source = (Component)evt.getSource();
    Object itemob = evt.getItem();
    String state = "unknown";
    if (evt.getStateChange() == ItemEvent.SELECTED) {
      state = "SELECTED";
    }
    else if (evt.getStateChange() == ItemEvent.DESELECTED) {
      state = "DESELECTED";
    }

    displayText("Source=<"+source.getName()+"> item= <"+itemob+"> state="+state);
  }


  public String getHelpText(){
    return "Test on the construction and behavior of the giza.component.util classes used for the IAA demo:\n"+
           "On the screen you should see an ItemList list, two buttons and a list to display ItemEvents in:\n"+
           "=> Select an item on the ItemList and see the select/deselect ItemEvent you just triggered appear on the list below\n"+
           "=> Scroll along the items of the ItemList using the right ScrollBar\n"+
           "=> Use the <add>-button to add a new item to the end of the list\n"+
           "(ATTENTION, THE ITEMLIST CODE HAS SOME MESSY VISUALISATION, SO THE LAST ITEM ISN'T ALWAYS DISPLAYED)"+
           "=> Use the <remove>-button to remove an item from the list"+
           "\nspecial AWT classes needed:\n==========================\n"+
           "->ItemList, Renderer and RendererPane\n"+
           "->ScrollableContainer (this one again using ScrollBar)";
  }

  /********************/
  /** test main */
  static public void main (String[] args) {
	  new ItemListTest();
  }

  // (end of class CopyWriter
}
