/*
 * MessageDialog.java
 *
 * Created on August 27, 2001, 1:54 PM
 */

package com.acunia.wonka.test.awt.IAA;

import java.awt.*;
import java.awt.event.*;

/**
 *
 * @author  ivan
 * @version
 */
public class MessageDialog extends Window implements ImageDefs {
  
  private ImageButton confirmButton;
  private ImageButton cancelButton;
  private MultiLineLabel messageLabel;
  private Insets insets = new Insets(0, 0, 0, 0);
  
  public static final int YES_NO_OPTION = 1;
  public static final int OK_OPTION = 2;
  
  private int optionType;
  
  /** Creates new MessageDialog */
  public MessageDialog(Frame f) {
    this(f, YES_NO_OPTION, "");
  }
  
  /** Creates new MessageDialog */
  public MessageDialog(Frame f, String message) {
    this(f, YES_NO_OPTION, message);
  }
  
  /** Creates new MessageDialog */
  public MessageDialog(Frame f, int optionType, String message) {
    super(f);
    this.optionType = optionType;
    
    setBackground(Color.white);
    setLayout(new BorderLayout());
    
    Container messagePanel = new Container(){};
    messagePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    messageLabel = new MultiLineLabel(message);
    messagePanel.add(messageLabel);
    
    Container buttonPanel = new Container(){};
    buttonPanel.setLayout(new FlowLayout());
    
    ActionListener al = new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
        MessageDialog.this.dispose();
      }
    };
    
    ImageButtonConfig confirmConfig = null;
    try {
      if (optionType == YES_NO_OPTION) {
        confirmConfig = new ImageButtonConfig(this, YESIMAGE, YESACTIVE, YESACTIVE);
      } else if (optionType == OK_OPTION) {
        confirmConfig = new ImageButtonConfig(this, OKIMAGE, OKACTIVE, OKACTIVE);
      }
      confirmButton = new ImageButton(confirmConfig);
      confirmButton.addActionListener(al);
      buttonPanel.add(confirmButton);
    } catch (Exception ex) {
      ex.printStackTrace();
    }
    
    if (optionType == YES_NO_OPTION) {
      ImageButtonConfig cancelConfig = null;
      try {
        cancelConfig = new ImageButtonConfig(this, NOIMAGE, NOACTIVE, NOACTIVE);
        cancelButton = new ImageButton(cancelConfig);
        cancelButton.addActionListener(al);
        buttonPanel.add(cancelButton);
      } catch (Exception ex) {
        ex.printStackTrace();
      }
    }
    
    add(messagePanel, BorderLayout.NORTH);
    add(buttonPanel, BorderLayout.SOUTH);
  }
  
  public void setConfirmButton(ImageButtonConfig config) {
    confirmButton.setConfig(config);
  }
  
  public void setCancelButton(ImageButtonConfig config) {
    cancelButton.setConfig(config);
  }
  
  public void setMessage(String msg) {
    messageLabel.setLabel(msg);
  }
  
  public Insets getInsets() {
    return insets;
  }
  
  public void setInsets(Insets insets) {
    this.insets = insets;
  }
  
  public Dimension getPreferredSize() {
    Dimension d = super.getPreferredSize();
    return new Dimension(d.width + insets.left + insets.right, d.height + insets.top + insets.bottom);
  }
  
  public Dimension getMaximumSize() {
    return getPreferredSize();
  }
  
  public Dimension getMinimumSize() {
    return getPreferredSize();
  }
  
  public void addConfirmActionListener(ActionListener al) {
    if (confirmButton != null) confirmButton.addActionListener(al);
  }
  
  public void removeConfirmActionListener(ActionListener al) {
    if (confirmButton != null) confirmButton.removeActionListener(al);
  }
  
  public void paint(Graphics g) {
    super.paint(g);
    Dimension d = getSize();
    g.drawRect(0, 0, d.width - 1, d.height -1);
    g.drawRect(1, 1, d.width - 2, d.height -2);
  }
  
  public static void main(String[] args) {
    MessageDialog md = new MessageDialog(new Frame());
    //md.setConfirmLabel("Delete");
    //md.setCancelLabel("Cancel");
    md.setMessage("Delete mail?");
    md.pack();
    md.show();
  }
  
}
