/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

// V.1.01.00 2001/08/29 : first release
// V.1.01.01 2001/08/29 : excluded CVS directories from files list
// V.1.01.02 2001/08/29 : Option to only load checked files
// V.1.02.01 2001/08/31 : Double list and quick scan possibilities
// V.1.02.02 2001/08/31 : bugfixes and selective file-load for subdirs
// V.1.02.03 2001/08/31 : added buttons for showing scripts and c/h/java files
// V.1.02.03 2001/09/03 : logging file
// V.1.02.04 2001/09/03 : edit screen allows to place header on current text position / button for header files only
// V.1.02.05 2001/09/03 : added buttons for same directory and last directory


// Author: N.Oberfeld
// Version 1.01.01
// Created: 2001/08/29

package com.acunia.wonka.test.awt.IAA;

//import
import com.acunia.wonka.test.awt.*;
import java.io.*;        // File & Streams
import java.awt.*;       // layout & buttons
import java.awt.event.*; // button events

public class MessageDialogTest extends VisualTestImpl implements ImageDefs, ActionListener{
	
  /****************************************************************/
  /** variables
  */
  ImageButtonConfig map = null;
  ImageButtonConfig up = null;
  ImageButtonConfig down = null;

  Button okButton;
  int okCounter;
  Button mapButton;
  int mapCounter;
  Button yesnoButton;
  int yesnoCounter;
  Button updownButton;
  int updownCounter;

  /****************************************************************/
  /** constructor
  */
  public MessageDialogTest() {
    super();


    map = null;
    up = null;
    down = null;
    try {
      //imageButton images
      up = new ImageButtonConfig(this, SCROLLUP_DEFAULT, SCROLLUP_PASSIVE, SCROLLUP_ACTIVE);
      down = new ImageButtonConfig(this,SCROLLDOWN_DEFAULT, SCROLLDOWN_PASSIVE, SCROLLDOWN_ACTIVE);
      map = new ImageButtonConfig(this, MAPIMAGE, MAPPASSIVE, MAPACTIVE);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }


    // buttons
    setLayout(new GridLayout(5,1) );
      okButton = new Button("simple ok-dialog");
      okButton.addActionListener(this);
      add(okButton);
      okCounter = 1;

      mapButton = new Button("map button dialog");
      mapButton.addActionListener(this);
      add(mapButton);
      mapCounter = 1;

      yesnoButton = new Button("simple yes-no dialog");
      yesnoButton.addActionListener(this);
      add(yesnoButton);
      yesnoCounter = 1;

      updownButton = new Button("up-down button dialog");
      updownButton.addActionListener(this);
      add(updownButton);
      updownCounter = 1;

    show();
  }

  /****************************************************************/
  /**button pressed : Display desired dialog window */
  public void actionPerformed(ActionEvent evt) {
    Component source = (Component)evt.getSource();
    Frame vtframe = vt.getFrame();
    int height = this.getSize().height;
    int quarter = height/4;

    if(source == okButton) {
      MessageDialog md = new MessageDialog(vtframe, MessageDialog.OK_OPTION,"Simple okay dialog no "+okCounter+"\n click on the icon below\nto continue");
      md.pack();
      md.setLocation(20*okCounter, 5);
      md.show();
      okCounter++;
    }
    else if(source == mapButton) {
      MessageDialog md = new MessageDialog(vtframe, MessageDialog.OK_OPTION,"Simple image dialog no "+mapCounter+"\n click on the map below\nto continue");
      md.setConfirmButton(map);
      md.pack();
      md.setLocation(20*mapCounter, quarter);
      md.show();
      mapCounter++;
    }
    else if(source == yesnoButton) {
      MessageDialog md = new MessageDialog(vtframe, "Simple yes-no dialog no "+yesnoCounter+"\n click on one of the icons below\nto continue");
      md.pack();
      md.setLocation(20*yesnoCounter, height - quarter -md.getSize().height);
      md.show();
      yesnoCounter++;
    }
    else if(source == updownButton) {
      MessageDialog md = new MessageDialog(vtframe, "Simple yes-no dialog no "+updownCounter+"\n click on the up-or down icon below\nto continue");
      md.setConfirmButton(up);
      md.setCancelButton(down);
      md.pack();
      md.setLocation(20*updownCounter, height -md.getSize().height);
      md.show();
      updownCounter++;
    }

  }



  public String getHelpText(){
    return "Test on the construction and behavior of the giza.component.util classes used for the IAA demo:\n"+
           "On the screen you should see:\n"+
           "=> two ImageLabel images on top of the screen (the 'news' circles). the left one is constructed with an image loaded"+
           "directly through Toolkit.getDefaultToolkit.getImage.(which means that at construction time, the image itself is not yet present.)"+
           " The right one is loaded through the ImageUtils.loadImage() function that uses a MediaTracker to ensure that the image is present\n"+
           "=> two ImageButton buttons (the 'map' roundels). This buttons can be enabled or disabled by means of the 'enabled' checkboxes"+
           " left of them. A disabled btton appears grey. Futthermore, they can be clicked. a clicked button changes color just as well"+
           " and sends an ActionPerformed message that will be displayed in the image list below\n"+
           "\nspecial AWT classes needed:\n==========================\n"+
           "->MediaTracker for the ImageLabel and ImageButton images, used in class ImageUtil\n"+
           "->(loading of png format images from file)";
  }

  /********************/
  /** test main */
  static public void main (String[] args) {
	  new MessageDialogTest();
  }

  // (end of class CopyWriter
}
