/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

// V.1.01.00 2001/08/29 : first release
// V.1.01.01 2001/08/29 : excluded CVS directories from files list
// V.1.01.02 2001/08/29 : Option to only load checked files
// V.1.02.01 2001/08/31 : Double list and quick scan possibilities
// V.1.02.02 2001/08/31 : bugfixes and selective file-load for subdirs
// V.1.02.03 2001/08/31 : added buttons for showing scripts and c/h/java files
// V.1.02.03 2001/09/03 : logging file
// V.1.02.04 2001/09/03 : edit screen allows to place header on current text position / button for header files only
// V.1.02.05 2001/09/03 : added buttons for same directory and last directory


// Author: N.Oberfeld
// Version 1.01.01
// Created: 2001/08/29

package com.acunia.wonka.test.awt.IAA;

//import
import com.acunia.wonka.test.awt.*;
import java.io.*;        // File & Streams
import java.awt.*;       // layout & buttons
import java.awt.event.*; // button events

public class MultiLineTest extends VisualTestImpl {
	
  /****************************************************************/
  /** variables
  */

  /****************************************************************/
  /** constructor
  */
  public MultiLineTest() {
    super();
      // larger font
      setFont(new Font("courR10",Font.PLAIN,10));
      // multiline labels
      MultiLineLabel l1 = new MultiLineLabel("left\nborders(10,20)",10, 20,MultiLineLabel.LEFT );
      l1.setBackground(Color.red);
      MultiLineLabel l2 = new MultiLineLabel("left vert. grid\nborders(3,3)",3,3,MultiLineLabel.LEFT );
      l2.setBackground(Color.blue);

      MultiLineLabel r1 = new MultiLineLabel("right\nbrds=3,3",3,3,MultiLineLabel.RIGHT);
      r1.setBackground(Color.red);
      MultiLineLabel r2 = new MultiLineLabel("right\nhor. grid\nbrds=10,20",10,20,MultiLineLabel.RIGHT);
      r2.setBackground(Color.blue);

      MultiLineLabel c1 = new MultiLineLabel("1\n2\n3\n4\n5\nLarge label in the remaining Center\n5\n4\n3\n2\n1",3,3,MultiLineLabel.CENTER);
      c1.setBackground(Color.white);

      MultiLineLabel t1 = new MultiLineLabel("Left",5,5,MultiLineLabel.LEFT);
      t1.setBackground(Color.red);
      MultiLineLabel t2 = new MultiLineLabel("Centered\nFlowLayout()",5,5,MultiLineLabel.CENTER );
      t2.setBackground(Color.blue);
      MultiLineLabel t3 = new MultiLineLabel("Right\nFlowLayout()\n(all borders 5,5)",5,5,MultiLineLabel.RIGHT);
      t3.setBackground(Color.green);

      MultiLineLabel b1 = new MultiLineLabel("Right",20,10,MultiLineLabel.RIGHT);
      b1.setBackground(Color.red);
      MultiLineLabel b2 = new MultiLineLabel("Centered\nFlowLayout()",20,10,MultiLineLabel.CENTER );
      b2.setBackground(Color.blue);
      MultiLineLabel b3 = new MultiLineLabel("Left aligned\nFlowLayout()\n(all borders 20,10)", 20,10,MultiLineLabel.LEFT);
      b3.setBackground(Color.green);

  // build screen
    setLayout(new BorderLayout());

    // screen layout

    Panel top = new Panel(new FlowLayout());
      top.add(t1);
      top.add(t2);
      top.add(t3);
    add(top, BorderLayout.NORTH);

    Panel left = new Panel(new GridLayout(2,1));
      left.add(l1);
      left.add(l2);
    add(left, BorderLayout.WEST);

    add(c1, BorderLayout.CENTER);

    Panel right = new Panel(new GridLayout(1,2));
      right.add(r1);
      right.add(r2);
    add(right, BorderLayout.EAST);


    Panel bottom = new Panel(new FlowLayout());
      bottom.add(b1);
      bottom.add(b2);
      bottom.add(b3);
    add(bottom, BorderLayout.SOUTH);
  }

  /*********************/
  /** VirtualTest help */
  public String getHelpText(){
    return "Test on the construction and behavior of the giza.component.util classes used for the IAA demo:\n"+
           "Tested here is the MultilineLabel class:\n"+
           "If this test is performed correctly, you should see on the screen:\n"+
           "-> on top of the screen, aligned in a flowlayout, (so each label is shown in its mimimum size)..\n"+
           "  -> a red label one line deep and left aligned, with a border of 5 pixels horizontaly and 5 pixels verticaly\n"+
           "  -> a blue label two lines deep and center aligned, with a border of 5 pixels horizontaly and 5 pixels verticaly\n"+
           "  -> a green label three lines deep and right aligned, with a border of 5 pixels horizontaly and 5 pixels verticaly\n"+
           "-> on the bottom of the screen, aligned in a flowlayout, (so each label is shown in its mimimum size)..\n"+
           "  -> a red label one line deep and right aligned, with a border of 20 pixels horizontaly and 10 pixels verticaly\n"+
           "  -> a blue label two lines deep and center aligned, with a border of 20 pixels horizontaly and 10 pixels verticaly\n"+
           "  -> a green label three lines deep and left aligned, with a border of 20 pixels horizontaly and 10 pixels verticaly\n"+
           "(Note the different sizes of the corresponding labels, caused just by the different border widths)\n"+
           "-> on center left of the screen a vertical gridlayout forces two labels into a specified height"+
           " (and also forces the smaller one into the width of the larger one),"+
           " The red upper label has the biggest borders, but the smallest text,"+
           " the blue lower one the biggest text, but the smallest borders\n"+
           " Note that both labels have the same height and their text is perfectly centered,regardless their vertical borders\n"+
           " Note that both borders have the same width, but due to its larger horizontal border, the lower text is displayed more inwards"+
           "-> on center right of the screen a horizontal gridlayout forces two labels into a specified height"+
           " (and also forces the smaller one into the width of the larger one),"+
           " The red left label has the smallest borders and smallest text, the blue right one both the biggest and borders\n"+
           " Note that both labels have the same height and their text is perfectly centered,regardless their vertical borders\n"+
           " Note that both borders have the same width, but due to its larger horizontal border, the right text is displayed more inwards"+
           "-> on the complete center the screen a large white label is forced into the remaining space. Note how it is centered vertically"+
           " to display at least the inner 5 of its 7 lines. Also note how the central text is cropped\n "+
           "(the text is center-aligned)";
  }

  /********************/
  /** test main */
  static public void main (String[] args) {
	  new MultiLineTest();
  }

  // (end of class CopyWriter
}
