package com.acunia.wonka.test.awt.IAA;

import java.awt.*;
import java.awt.event.*;

public class RendererPane extends Container {
    /**
     * Construct a CellRendererPane object.
     */
    public RendererPane() {
        super();
        setLayout(null);
        setVisible(false);
    }
    
    /**
     * Overridden to avoid propogating a invalidate up the tree when the
     * cell renderer child is configured.
     */
    public void invalidate() { }
    
    
    /**
     * Shouldn't be called.
     */
    public void paint(Graphics g) { }
    
    
    /**
     * Shouldn't be called.
     */
    public void update(Graphics g) { }
    
    
    /**
     * If the specified component is already a child of this then we don't
     * bother doing anything - stacking order doesn't matter for cell
     * renderer components (CellRendererPane doesn't paint anyway).<
     */
    protected void addImpl(Component x, Object constraints, int index) {
        if (x.getParent() == this) {
            return;
        }
        else {
            super.addImpl(x, constraints, index);
        }
    }
    
    
    public void paintComponent(Graphics g, Component c, Container p, int x, int y, int w, int h, boolean shouldValidate) {
        if (c == null) {
            if (p != null) {
                Color oldColor = g.getColor();
                g.setColor(p.getBackground());
                g.fillRect(x, y, w, h);
                g.setColor(oldColor);
            }
            return;
        }
        
        if (c.getParent() != this) {
            this.add(c);
        }
        
        c.setBounds(x, y, w, h);
        
        if(shouldValidate) {
            c.validate();
        }
        
        
        Graphics cg = g.create(x, y, w, h);
        try {
            c.paint(cg);
        }
        finally {
            cg.dispose();
        }
        c.setBounds(-w, -h, 0, 0);
    }
    
    
    /**
     * Calls this.paintComponent(g, c, p, x, y, w, h, false).
     */
    public void paintComponent(Graphics g, Component c, Container p, int x, int y, int w, int h) {
        paintComponent(g, c, p, x, y, w, h, false);
    }
    
    
    /**
     * Calls this.paintComponent() with the rectangles x,y,width,height fields.
     */
    public void paintComponent(Graphics g, Component c, Container p, Rectangle r) {
        paintComponent(g, c, p, r.x, r.y, r.width, r.height);
    }
        
}
