/*
 * ScrollBar.java
 *
 * Created on August 9, 2001, 4:02 PM
 */

package com.acunia.wonka.test.awt.IAA;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
/**
 *
 * @author  ivan
 * @version
 */
public class ScrollBar extends Container implements ImageDefs, Adjustable {
    

    private Vector _listeners = new Vector();
    private ImageButton _upButton;
    private ImageButton _downButton;
    private ImageLabel _middleImage;
    private int _orientation;
    private int _minimum;
    private int _maximum = 10;
    private int _value = Integer.MIN_VALUE;
    private int _unitIncrement = 1;
    

    /** Creates new ScrollBar */
    public ScrollBar(ImageButtonConfig upConfig,
                     ImageButtonConfig downConfig,
                     Image middle, int orientation) {
        _upButton = new ImageButton(upConfig);
        _downButton = new ImageButton(downConfig);
        if (middle != null)  _middleImage = new ImageLabel(middle);
        _orientation = orientation;
        setBackground(Color.white);
        setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();
        if (orientation == VERTICAL) {
            gbc.gridwidth = gbc.REMAINDER;
            add(_upButton, gbc);
            if (middle != null){
               add(_middleImage, gbc);
            }
            add(_downButton, gbc);
        }
        else {
            add(_downButton, gbc);
            add(_upButton, gbc);
        }
        
        ActionListener actionListener = new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                Object obj = ae.getSource();
                if (obj == _upButton) {
                    int v = _value + _unitIncrement;
                    setCurrentValue(v <= _maximum ? v : _maximum);
                } else if (obj == _downButton) {
                    int v = _value - _unitIncrement;
                    setCurrentValue(v >= _minimum ? v : _minimum);
                }
                
            }
        };
        _upButton.addActionListener(actionListener);
        _downButton.addActionListener(actionListener);
        setCurrentValue(0);
    }
    
    private void setCurrentValue(int value) {
        if (_value != value) {
            int type = value > _value ? AdjustmentEvent.UNIT_INCREMENT : AdjustmentEvent.UNIT_DECREMENT;
            AdjustmentEvent event = new AdjustmentEvent(this, 0, type, value);
            _value = value;
            _upButton.setEnabled(_value != _maximum);
            _downButton.setEnabled(_value != _minimum);
            for (int i = 0; i < _listeners.size(); i++) {
                ((AdjustmentListener) _listeners.elementAt(i)).adjustmentValueChanged(event);
            }
        }
    }
    
    public ImageButtonConfig getUpButtonConfig() {
        return _upButton.getConfig();
    }
    
    public ImageButtonConfig getDownButtonConfig() {
        return _downButton.getConfig();
    }

    public Dimension getPreferredSize() {
        Dimension upDim = _upButton.getPreferredSize();
        Dimension midDim = _middleImage.getPreferredSize();
        Dimension downDim = _downButton.getPreferredSize();
        return new Dimension(upDim.width, upDim.height + midDim.height + downDim.height);
    }
    
    public Dimension getMinimumSize() {
        return getPreferredSize();
    }
    
    public Dimension getMaximumSize() {
        return getPreferredSize();
    }
    
    public void setEnabled(boolean enabled) {
        _upButton.setEnabled(enabled ? (_value != _maximum) : false);
        _downButton.setEnabled(enabled ? (_value != _minimum) : false);
    }
    
    public void setValue(int param) {
        setCurrentValue(param < _minimum ? _minimum : (param > _maximum ? _maximum : param));
    }
    
    public int getUnitIncrement() {
        return _unitIncrement;
    }
    
    public int getVisibleAmount() {
        throw new RuntimeException("Not supported");
    }
    
    public int getBlockIncrement() {
        throw new RuntimeException("Not supported");
    }
    
    public int getValue() {
        return _value;
    }
    
    public void setUnitIncrement(int param) {
        _unitIncrement = param;
    }
    
    public void addAdjustmentListener(AdjustmentListener adjustmentListener) {
        if (!_listeners.contains(adjustmentListener)) {
            _listeners.addElement(adjustmentListener);
        }
    }
    
    public void setBlockIncrement(int param) {
        throw new RuntimeException("Not supported");
    }
    
    public void setVisibleAmount(int param) {
        throw new RuntimeException("Not supported");
    }
    
    public void setMinimum(int param) {
        _minimum = param;
        if (_value < _minimum) {
            setCurrentValue(_minimum);
        }
    }
    
    public void removeAdjustmentListener(AdjustmentListener adjustmentListener) {
        _listeners.removeElement(adjustmentListener);
    }
    
    public int getOrientation() {
        return _orientation;
    }
    
    public void setMaximum(int param) {
        _maximum = param;
        if (_value > _maximum) {
            setCurrentValue(_maximum);
        }
    }
    
    public int getMaximum() {
        return _maximum;
    }
    
    public int getMinimum() {
        return _minimum;
    }
    
}
