/*
 * ScrollPane.java
 *
 * Created on August 13, 2001, 4:00 PM
 */

package com.acunia.wonka.test.awt.IAA;

import java.awt.*;
import java.awt.event.*;

/**
 *
 * @author  ivan
 * @version
 */
public class ScrollableContainer extends BorderedContainer {
    
    protected ScrollPane _scrollPane;
    protected ScrollBar _scrollBar;
    
    /** Creates new ScrollPane */
    public ScrollableContainer(Component component) {
        setBackground(Color.white);
        setBorderEnabled(false);
        setLayout(new BorderLayout(0, 0));
        ImageButtonConfig upConfig = null;
        ImageButtonConfig downConfig = null;
        Image middleImage = null;
        try {
          upConfig = new ImageButtonConfig(this, ScrollBar.SCROLLUP_DEFAULT, ScrollBar.SCROLLUP_ACTIVE, ScrollBar.SCROLLUP_PASSIVE);
          downConfig = new ImageButtonConfig(this, ScrollBar.SCROLLDOWN_DEFAULT, ScrollBar.SCROLLDOWN_ACTIVE, ScrollBar.SCROLLDOWN_PASSIVE);
          middleImage = ImageUtils.loadImage(this, ScrollBar.SCROLLMIDDLE);
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
        
        _scrollPane = new ScrollPane(ScrollPane.SCROLLBARS_NEVER);
        _scrollBar = new ScrollBar(upConfig, downConfig, middleImage, Adjustable.VERTICAL);
        _scrollBar.setUnitIncrement(20);
        _scrollBar.addAdjustmentListener(new AdjustmentListener() {
            public void adjustmentValueChanged(AdjustmentEvent e) {
                _scrollPane.setScrollPosition(0, -e.getValue());
            }
        });
        _scrollPane.add(component);
        add(_scrollPane, BorderLayout.CENTER);
        add(_scrollBar, BorderLayout.EAST);
    }
    
    private void updateScrollBarData() {
        Dimension vpSize = _scrollPane.getViewportSize();
        Dimension compSize = _scrollPane.getComponent(0).getPreferredSize();
        if (compSize.height <= vpSize.height) {
            _scrollBar.setEnabled(false);
            //_scrollBar.setVisible(false);
        } else {
            int height = compSize.height - _scrollPane.getViewportSize().height;
            _scrollBar.setMinimum(-height);
            _scrollBar.setMaximum(0);
            _scrollBar.setEnabled(true);
            //_scrollBar.setVisible(true);
        }
    }
    
    public void doLayout() {
        super.doLayout();
        updateScrollBarData();
    }
    
    public void setUnitIncrement(int inc) {
        _scrollBar.setUnitIncrement(inc);
    }
    
    public int getUnitIncrement() {
        return _scrollBar.getUnitIncrement();
    }
    
    public Component getComponent() {
        return _scrollPane.getComponent(0);
    }
    
    public Dimension getViewPortSize() {
        return _scrollPane.getViewportSize();
    }
    
    
    public static void main(String[] args) {
        final Frame f = new Frame();
        f.setSize(200, 288);
        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent we) {
                f.dispose();
                System.exit(0);
            }
        });
        f.setLayout(new BorderLayout());
        
        Panel p = new Panel(new GridLayout(31, 1));
        p.add(new Button("1"));
        p.add(new Button("2"));
        p.add(new Button("3"));
        p.add(new Button("4"));
        p.add(new Button("5"));
        p.add(new Button("6"));
        p.add(new Button("7"));
        p.add(new Button("8"));
        p.add(new Button("9"));
        p.add(new Button("10"));
        p.add(new Button("11"));
        p.add(new Button("12"));
        p.add(new Button("13"));
        p.add(new Button("14"));
        p.add(new Button("15"));
        p.add(new Button("16"));
        p.add(new Button("17"));
        p.add(new Button("18"));
        p.add(new Button("19"));
        p.add(new Button("20"));
        p.add(new Button("21"));
        p.add(new Button("22"));
        p.add(new Button("23"));
        p.add(new Button("24"));
        p.add(new Button("25"));
        p.add(new Button("26"));
        p.add(new Button("27"));
        p.add(new Button("28"));
        p.add(new Button("29"));
        p.add(new Button("30"));
        p.add(new Button("31"));
        ScrollableContainer sc = new ScrollableContainer(p);
        f.add(sc);
        f.show();
    }
}
