/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

package com.acunia.wonka.test.awt.IAA;

import com.acunia.wonka.test.awt.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;

public class ScrollableContainerTest extends VisualTestImpl implements ImageDefs {
	
  private ImageIcon[] icons;
  private List display;

  public ScrollableContainerTest() {
    super();
    // map images
    ImageButtonConfig mapimages = null;
    try {
      mapimages = new ImageButtonConfig(this, MAPIMAGE, MAPACTIVE, MAPPASSIVE);
    }
    catch (Exception ex) {
      ex.printStackTrace();
      System.out.println("Unable to load Images "+ MAPIMAGE);
    }
    if(mapimages!= null){
      icons = new ImageIcon[16];
      for(int i=0; i < icons.length; i++) {
        icons[i] = new ActionImageIcon(" ScrollableContainer ImageIcon #", i+1, mapimages, ImageIcon.BOTTOM);
      }
    }
    else {
      icons = new ImageIcon[0];
    }

    //layout
    setLayout(new BorderLayout());
    // scrollable container
      Panel buttons = new Panel(new GridLayout(4,4));
      for(int i=0; i < icons.length; i++) {
       buttons.add(icons[i]);
      }
      ScrollableContainer sc = new ScrollableContainer(buttons);
    add(sc, BorderLayout.CENTER);
    //list
      display = new List(3,false);
    add(display, BorderLayout.SOUTH);
  }

  /****************************************************************/
  /** event list display */
  private void displayText(String text) {
    if(display.getItemCount()>40) {
      display.removeAll();
    }
    System.out.println(text);
    display.add(text,0);
  }

  /****************************************************************/
  /** Sspecial ImageIcon class with propper actionListener */
  class ActionImageIcon extends ImageIcon implements ActionListener {
    private int iconNo;

    public ActionImageIcon(String labelText, int number, ImageButtonConfig config, int labelPos){
      super(labelText+number, config, labelPos, false);
      iconNo = number;
      // special class handles its own action listener
      addActionListener(this);
    }

    public void actionPerformed(ActionEvent evt) {
      displayText("Clicked ImageIcon no."+iconNo);
    }
  }

  /****************************************************************/
  /** VisualtestEngine help */
  public String getHelpText(){
    return "Test on the construction and behavior of the ScrollableContainer class used for the IAA demo:\n\n"+
           "On the screen you should see a large <ScrollableContainer> type Scrollpane. The pane contains 4 rows of 4 ImageIcon elements."+
           " but not all of them are visible at the same time.\n"+
           "=> At startup, the ScrollPane can only display the first two of the four rows, yet you can use the pane's vertical scrollbar"+
           " to scroll down to the lower rows and up again to the higher.\n"+
           "=> Of the four columns, the ScrollPane only has space to display the first two columns and half of the third."+
           " Since there is no horizontal scrollbar,the fourth column is out of reach permanently.\n\n"+
           "Clicking one of the ImageIcons will display a message in the list below.";
  }

}
