/*
 * TabbedPane.java
 *
 * Created on August 22, 2001, 1:07 PM
 */

package com.acunia.wonka.test.awt.IAA;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.StringTokenizer;

/**
 *
 * @author  ivan
 * @version
 */
public class TabbedPane extends Container {
  
  public static final int NORTH = 1;
  public static final int EAST = 2;
  public static final int SOUTH = 3;
  public static final int WEST = 4;
  
  private static final Color GRAY1 = new Color(221,222,225);
  private static final Color GRAY2 = new Color(142,145,156);
  private static final Color GRAY3 = new Color(147,147,147);
  
  private class SouthContainer extends Container {
    
    private GridBagConstraints gbc1 = new GridBagConstraints();
    private GridBagConstraints gbc2 = new GridBagConstraints();
    
    public SouthContainer() {
      this.setLayout(new GridBagLayout());
      gbc1.fill = gbc1.VERTICAL;
      gbc1.insets = new Insets(0, 0, 0, 0);
      gbc2.fill = gbc2.VERTICAL;
      gbc2.insets = gbc1.insets;
      gbc2.anchor = gbc2.WEST;
      gbc2.weightx = 1.0;
    }
    
    public void paint(Graphics g) {
      Dimension s = this.getSize();
      g.setColor(GRAY2);
      g.drawLine(0, 0, s.width, 0);
      g.setColor(GRAY3);
      g.drawLine(0, 1, s.width, 1);
     // g.setColor(GRAY3);
     // g.drawLine(0, 2, s.width, 2);
      g.setColor(Color.lightGray);
      g.fillRect(0, 2, s.width, s.height - 2);
      super.paint(g);
    }
    
    public Component add(Component comp) {
      int i = this.getComponentCount() - 1;
      if (i != -1) {
        Component c = this.getComponent(i);
        this.remove(i);
        this.add(c, gbc1);
      }
      this.add(comp, gbc2);
      return comp;
    }
  }
  
  private static interface Selectable {
    public void setSelected(boolean selected);
    public boolean isSelected();
  }
  
  public class DefaultTabComponent extends Component implements Selectable {
    
    private boolean selected;
    private ToggleButton button;
    
    public DefaultTabComponent(ToggleButton button) {
      this.button = button;
    }
    
    public void setSelected(boolean selected) {
      this.selected = selected;
      button.setSelected(selected);
      this.repaint();
    }
    
    public boolean isSelected() {
      return selected;
    }
    
    public void paint(Graphics g) {
      button.paint(g);
    }
    
    public Dimension getPreferredSize() {
      return button.getPreferredSize();
    }
    
    public Dimension getMinimumSize() {
      return getPreferredSize();
    }
    
    public Dimension getMaximumSize() {
      return getPreferredSize();
    }
  }
  
  private CardLayout card;
  private Container mainPanel;
  private Container tabPanel;
  private int selectedTab = -1;
  private Vector listeners = new Vector();
  private MouseListener tabClickListener;
  
  /** Creates new TabbedPane */
  public TabbedPane() {
    this(NORTH);
  }
  
  public TabbedPane(int orientation) {
    setLayout(new BorderLayout(0, 0));
    
    tabClickListener = new MouseAdapter() {
      public void mouseReleased(MouseEvent me) {
        Component[] array = TabbedPane.this.tabPanel.getComponents();
        for (int i = 0; i < array.length; i++) {
          if (array[i] == me.getSource()) {
            TabbedPane.this.setSelectedTab(i);
            break;
          }
        }
      }
    };
    
    card = new CardLayout();
    mainPanel = new Container(){};
    mainPanel.setLayout(card);
    
    tabPanel = createTabPanel(orientation);
    add(mainPanel);
    add(tabPanel, BorderLayout.SOUTH);
  }
  
  private Container createTabPanel(int orientation) {
    Container panel = null;
    switch (orientation) {
      case SOUTH:
        panel = new SouthContainer();
        break;
      default:
        throw new RuntimeException("Orientation not supported");
    }
    return panel;
  }
  
  public void addTab(Container pane, ToggleButton button) {
    mainPanel.add("" + getTabCount(), pane);
    Component c = new DefaultTabComponent(button);
    c.addMouseListener(tabClickListener);
    tabPanel.add(c);
  }
  
  public void removeTab(int index) {
    mainPanel.remove(index);
    tabPanel.remove(index);
  }
  
  public int getTabCount() {
    return mainPanel.getComponentCount();
  }
  
  public int getSelectedTab() {
    return selectedTab;
  }
  
  public void updateTab(int index, ToggleButton button) {
    tabPanel.remove(index);
    Component c = new DefaultTabComponent(button);
    c.addMouseListener(tabClickListener);
    tabPanel.add(c, index);
  }
  
  public void setSelectedTab(int index) {
    if (index != selectedTab) {
      int tabCount = getTabCount();
      for (int i = 0; i < tabCount; i++) {
        ((Selectable) tabPanel.getComponent(i)).setSelected(index == i);
      }
      fire(new TabEvent(selectedTab, index));
      card.show(mainPanel, "" + index);
      selectedTab = index;
    }
  }
  
  public void addTabListener(TabListener l) {
    if (!listeners.contains(l)) {
      listeners.addElement(l);
    }
  }
  
  public void removeTabListener(TabListener l) {
    listeners.removeElement(l);
  }
  
  private void fire(TabEvent evt) {
    for (int i = 0; i < listeners.size(); i++) {
      ((TabListener) listeners.elementAt(i)).tabClicked(evt);
    }
  }
  
  public static void main(String[] args) {
    final Frame f = new Frame("Test");
    f.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent we) {
        f.dispose();
        System.exit(0);
      }
    });
    f.setSize(300, 200);
    f.setLayout(new BorderLayout());
    TabbedPane tp = new TabbedPane(TabbedPane.SOUTH);
    Panel p1 = new Panel();
    p1.add(new Button("Porn here!!!"));
    Panel p2 = new Panel();
    p2.add(new Label("Hello sucker!!!"));
    
    //tp.addTab(p1, "Tabke 1\nYeah");
    //tp.addTab(p2, "Tabke 2");
    //tp.setSelectedTab(0);
    //f.add(tp);
    f.show();
  }
}
