/*
 * TextPane.java
 *
 * Created on September 3, 2001, 11:34 AM
 */

package com.acunia.wonka.test.awt.IAA;

import java.awt.*;
import java.awt.event.*;
import java.util.StringTokenizer;

/**
 *
 * @author  ivan
 * @version
 */
public class TextPane extends Container implements ImageDefs {
  
  private static class AcuniaTextArea extends TextArea {
    
    public AcuniaTextArea() {
      super("", 1, 1, TextArea.SCROLLBARS_NONE);
      this.setBackground(Color.white);
    }
    
    public Dimension getPreferredSize() {
      return ((ScrollPane) this.getParent()).getViewportSize();
    }
  }

  protected TextArea _textArea;
  protected ScrollBar _scrollBar;
  
  /** Creates new ScrollPane */
  public TextPane() {
    setBackground(Color.white);
    setLayout(new BorderLayout(0, 0));
    ImageButtonConfig upConfig = null;
    ImageButtonConfig downConfig = null;
    Image middleImage = null;
    try {
      upConfig = new ImageButtonConfig(this, SCROLLUP_DEFAULT, SCROLLUP_PASSIVE, SCROLLUP_ACTIVE);
      downConfig = new ImageButtonConfig(this, SCROLLDOWN_DEFAULT, SCROLLDOWN_PASSIVE, SCROLLDOWN_ACTIVE);
      middleImage = ImageUtils.loadImage(this, SCROLLMIDDLE);
    } catch (Exception ex) {
      ex.printStackTrace();
    }
    
    _scrollBar = new ScrollBar(upConfig, downConfig, middleImage, ScrollBar.VERTICAL);
    _scrollBar.setUnitIncrement(1);
    _scrollBar.setMinimum(Integer.MIN_VALUE);
    _scrollBar.setMaximum(Integer.MAX_VALUE);
    _scrollBar.setValue(1);
    _textArea = new AcuniaTextArea();
    add(_textArea, BorderLayout.CENTER);
    add(_scrollBar, BorderLayout.EAST);
    _scrollBar.addAdjustmentListener(new AdjustmentListener() {
      public void adjustmentValueChanged(AdjustmentEvent e) {
        int newPos = _textArea.getCaretPosition();
        if (e.getAdjustmentType() == e.UNIT_INCREMENT) {
          newPos -= 20;
        } else if (e.getAdjustmentType() == e.UNIT_DECREMENT) {
          newPos += 20;
        }
        setCursorPos(newPos);
      }
    });

  }
  
  private void setCursorPos(int pos) {
    if (pos < 0) {
      pos = 0;
    } else if (pos > _textArea.getText().length()) {
      pos = _textArea.getText().length();
    }
    _textArea.setCaretPosition(pos);
  }
  
  public TextArea getTextArea() {
    return _textArea;
  }
  
  public void setEnabled(boolean enabled) {
    _textArea.setEnabled(enabled);
    _scrollBar.setEnabled(enabled);
  }
  
  public Insets getInsets() {
    return new Insets(1, 1, 1, 1);
  }
  
  public void paint(Graphics g) {
    Dimension d = getSize();
    super.paint(g);
    g.drawRect(0, 0, d.width - 1, d.height - 1);
  }
  
  public static void main(String[] args) {
    final Frame f = new Frame();
    f.setSize(200, 288);
    f.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent we) {
        f.dispose();
        System.exit(0);
      }
    });
    f.setLayout(new BorderLayout());
    TextPane tp = new TextPane();
    f.add(tp);
    f.show();
  }}
