/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


// Author: D. Buytaert
// Created: 2001/05/10

package com.acunia.wonka.test.awt.misc;

import java.awt.*;
import com.acunia.wonka.test.awt.*;
import java.util.*;

public class MemoryUsage extends VisualTestImpl {

  static private int history = 100;
  static private int f1 = 1;
  static private int f2 = 50;

  static private LinkedList ll1;
  static private LinkedList ll2;
  static private LinkedList ll3;
  static private LinkedList ll4;

  static private long minMem = 0;
  static private long maxMem = 0;
  static private int count = 1;

  static private Color c1;
  static private Color c2;
  static private Color c3;
  static private Color c4;

  static {
    minMem = Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory();
    maxMem = Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory();

    ll1 = new LinkedList();
    ll2 = new LinkedList();
    ll3 = new LinkedList();
    ll4 = new LinkedList();

    c1 = (Color.red).brighter();
    c2 = (Color.blue).brighter();
    c3 = (Color.red).darker();
    c4 = (Color.blue).darker();
  }

  public MemoryUsage() {
    long mem = Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory();

    minMem = (mem < minMem) ? mem : minMem;
    maxMem = (mem > maxMem) ? mem : maxMem;

    if (count % f1 == 0) {
      ll1.addLast(new Long(mem));
      if (ll1.size() > history) ll1.removeFirst();
    }

    if (count % f2 == 0) {
      ll2.addLast(new Long(mem));
      if (ll2.size() > history) ll2.removeFirst();
    }

    if (count % (f1 * history) == 0) {
      long min = ((Long)Collections.min(ll1)).longValue();
      long max = ((Long)Collections.max(ll1)).longValue();
      long avg = (long)(max + min) / 2;
      ll3.addLast(new Long(avg));
      if (ll3.size() > history) ll3.removeFirst();
    }

    if (count % (f2 * history) == 0) {
      long min = ((Long)Collections.min(ll2)).longValue();
      long max = ((Long)Collections.max(ll2)).longValue();
      long avg = (long)(max + min) / 2;
      ll4.addLast(new Long(avg));
      if (ll4.size() > history) ll4.removeFirst();
    }
  
    count++;
  }

  public void plot(LinkedList ll, Graphics g, Color c) {
    if (ll.size() > 0) {
      // long min = ((Long)Collections.min(ll)).longValue();
      // long max = ((Long)Collections.max(ll)).longValue();
      long min = minMem;
      long max = maxMem;

      if (max != min) {
        int offset = (int)(max - min) / (getHeight() - 45);
        int middle = (int)(max + min) / 2;
        int baseln = (getHeight() / 2 + 20);
        int step = (getWidth() / history);

        g.setColor(Color.black);
        g.drawLine(0, baseln, getWidth() - 1, baseln);
        g.setColor(c);

        int a = baseln;
        int b = baseln;
        int x = 0;
        
        Iterator iterator = ll.iterator();

        while (iterator.hasNext()) {
          long n = ((Long)iterator.next()).longValue();
          b = baseln + ((int)(n - middle) / offset);
 
          g.drawLine(x, a, x + step, b);
  
          a = b;
          x += step;
        }
      }
    }
  }  
  
  public void paint(Graphics g) {
    plot(ll1, g, c1);
    plot(ll2, g, c2);
    plot(ll3, g, c3);
    plot(ll4, g, c4);

    g.setColor(Color.black);
    g.drawString("Ran "+ count +" times." , 5, 10);
    g.drawString("Minimal memory usage: "+ minMem +" bytes.", 5, 22);
    g.drawString("Maximal memory usage: "+ maxMem +" bytes.", 5, 34);
    g.drawString("x-axis: time / y-axis: available memory", 5, getHeight() - 8);
  }

  public String getHelpText() {
    return "Plots the memory usage into a chart.";
  }
}
