/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: argument.c,v 1.1 2002/03/21 09:12:55 dfrederickx Exp $
*/

#include <stdlib.h>
#include <sys/stat.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "oswald.h"
#include "jni.h"

#ifndef CLASSPATH
#define CLASSPATH NULL
#endif

Wonka_InitArgs  system_InitArgs;
int             woempa_bytecodecount   = 0;

#ifdef JDWP
char            *jdwp_args = NULL;
int             jdwp_enabled = 0;
#endif

#if defined(AWT_XSIM) || defined(AWT_NONE)
char            *awt_args = NULL;
#endif

#ifdef JDWP

void print_jdwp_help(void) {
  
  printf("  Usage of -Xjdwp:   -Xjdwp:<option>=<value>, ...\n");
  printf("\n");
  printf("Options :\n");
  printf("  suspend=y|n         Suspend the vm after startup (default: n) (not yet implemeted)\n");
  printf("  transport=<name>    Transport type. dt_socket is the only supported transport.\n");
  printf("                      (default: dt_socket)\n");
  printf("  address=<address>   Address to connect/listen to. (default: 127.0.0.1:5555)\n");
  printf("  server=y|n          Listen for the debugger or connect to the debugger. \n");
  printf("                      (default: n -> connect to the debugger)\n");
  printf("\n");
  exit(1);

}

#endif  /* JDWP */

extern  const char *banner;

void args_read() {
  int             i;
  Wonka_InitArgs  *args = &system_InitArgs;
  char            **properties;
  int             prop_count = 0;
  char            *classpath = CLASSPATH;
  char            *compile_classpath;

  memset(args, 0, sizeof(Wonka_InitArgs));
  properties = x_mem_alloc(20 * sizeof(char *));
  memset(properties, 0, sizeof(char *) * 20);

  args->properties = properties;
  args->classpath = classpath;

  for(i = 0; i < command_line_argument_count; i++) {

    if((strcmp(command_line_arguments[i], "-cp") == 0) || 
       (strcmp(command_line_arguments[i], "-classpath") == 0)) {
      i++;
      if(classpath) {
        compile_classpath = classpath;
        woempa(9, "Adding compiled in classpath '%s'\n", compile_classpath);
        classpath = x_mem_calloc(strlen(command_line_arguments[i]) + strlen(compile_classpath) + 2);
        strcat(classpath, compile_classpath);
        strcat(classpath, ":");
        strcat(classpath, command_line_arguments[i]);
      } 
      else {
        classpath = command_line_arguments[i];
      }
      args->classpath = classpath;
      woempa(9, "Classpath: '%s'\n", args->classpath);
    }
    else if(strncmp(command_line_arguments[i], "-D", 2) == 0) {
      // printf("  property:  '%s'\n", &command_line_arguments[i][2]);
      properties[prop_count++] = (char *)&command_line_arguments[i][2];
    }
#ifdef JDWP 

    else if(strncmp(command_line_arguments[i], "-Xjdwp:", 7) == 0) {
      jdwp_args = (char *)&command_line_arguments[i][7];
      // printf("  jdwp:      %s\n", jdwp_args);
      if(strcmp(jdwp_args, "help") == 0) {
        print_jdwp_help();
      }
    }
    else if(strncmp(command_line_arguments[i], "-Xdebug", 7) == 0) {
      jdwp_enabled = 1;
    }

#endif  /* JDWP */

#if defined(AWT_XSIM) || defined(AWT_NONE)

    else if(strncmp(command_line_arguments[i], "-Wrudolph:", 10) == 0) { 
      awt_args = (char *)&command_line_arguments[i][10];
    }
    
#endif  /* AWT_XSIM */

    else if(strncmp(command_line_arguments[i], "-", 1) == 0) {
      printf(" Unrecognized option: %s\n", command_line_arguments[i]);
      exit(1);
    } else {

      /*
      ** It's not an option, so it has to be the main class and 
      ** it's arguments. Since command_line_argument_count and 
      ** command_line_arguments were updated each time we went
      ** through this loop, they're now pointing right at the
      ** requested startclass.
      */

      command_line_argument_count = command_line_argument_count - i;

      return;

    }
    
    command_line_argument_count = command_line_argument_count - i;
    ++command_line_arguments;

  }

  command_line_argument_count = 0;

}
