/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: exec.c,v 1.1 2002/03/21 09:12:55 dfrederickx Exp $
*/

#include <signal.h>
#include <errno.h>
#include <stdlib.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/socket.h>
#include <sys/signal.h>
#include <sys/types.h>
#include <sys/resource.h>
#include <sys/wait.h>
#include <unistd.h>
#include "wonka.h"
#include "oswald.h"
#include "exec.h"

w_int host_exec(w_ubyte **cmd, w_ubyte **env, w_int *fd_in, w_int *fd_out, w_int *fd_err) {
  w_int pid;
  w_int socks_inout[2];
  w_int socks_err[2];

  socketpair(PF_UNIX, SOCK_STREAM, 0, socks_inout);
  socketpair(PF_UNIX, SOCK_STREAM, 0, socks_err);

  *fd_in = socks_inout[0];
  *fd_out = socks_inout[0];
  *fd_err = socks_err[0];
  
  fcntl(*fd_in, F_SETFL, O_NONBLOCK);
  fcntl(*fd_err, F_SETFL, O_NONBLOCK);
  
  if(!(pid = fork())) {

    /*
    ** Close stdin, stdout and stderr.
    */
    
    close(0);
    close(1);
    close(2);

    /*
    ** Reassign stdin, stdout and stderr to the socketpairs.
    */

    dup2(socks_inout[1], 0);
    dup2(socks_inout[1], 1);
    dup2(socks_err[1], 2);

    /*
    ** Execute program.
    */ 

    execvp(cmd[0], (char **)cmd);
    
    woempa(9, "Execution of child process failed...\n");

    exit(0);
  }

  woempa(9, "child process has pid %d\n", pid);

  return pid;

}

w_void host_destroy(w_int pid) {
  kill(pid, 9);
}

#define ERESTARTSYS 512

w_int host_wait(w_int pid) { 
  w_int result;
  w_int status;
  w_int keepgoing = ERESTARTSYS;
 
  while(keepgoing == ERESTARTSYS) {
    result = wait3(&status, WNOHANG, NULL);
    keepgoing = errno;
    x_thread_sleep(5);
  } 

  if(WIFEXITED(status)) {
    return WEXITSTATUS(status);
  }
  else {
    return -1;
  }
}

w_int  host_read(w_int fd) {
  w_ubyte b;
  w_int   result;

  result = read(fd, &b, 1);
  if(result == -1 || result == 0) {
    return -1;
  }
  else {
    return b;
  }
}

w_void  host_write(w_int fd, w_int b) {
  w_ubyte c = b;
  write(fd, &c, 1);
}

w_void  host_close(w_int fd) {
  close(fd);
}

