#ifndef _CLAZZ_H
#define _CLAZZ_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: clazz.h,v 1.21 2002/03/27 11:47:05 gray Exp $
*/

#include "wonka.h"
#include "fields.h"
#include "hashtable.h"
#include "heap.h"
#include "realms.h"
#include "strings.h"
#include "oswald.h"
#include "bar.h"
#include "core-classes.h"
#include "threads.h"
#include "wordset.h"

typedef unsigned char               u1;
typedef unsigned short              u2;
typedef unsigned int                u4;

/*
** Note: structure w_Clazz is shadowed by the FICL class w-clazz.
** If you change the layout of this structure then you may need to
** adapt wonka/ficl/claudia.fr as well.
*/

typedef struct w_Clazz {

  w_string dotified; 
  char   *label;
  
  w_flags flags;
  w_instance loader;     /* the ClassLoader that defined this class     */

  w_short cmajor;
  w_short cminor;
  /*
  ** The superclass and all the interfaces which this class implements.
  */
  w_clazz superClazz;
  w_size numInterfaces;
  w_clazz* interfaces;
  
  /*
  ** The constant pool: see constant.h for details.
  */
  w_size numConstants;
  w_cnt constants;

  /*
  ** The class members.  In addition to the arrays of fields and methods
  ** we create a hashtable of "members", which maps two w_string's to a w_field
  ** or w_method.  For a field, the first w_string is the name of the field
  ** and the second w_string is NULL; for a method the first w_string is
  ** the name of the method and the second holds a list of the 
  ** descriptors of its parameters, enclosed in parentheses e.g.
  ** append(Ljava/lang/String;) . (Note that the result is not included).
  */

  w_ushort numFields;
  w_ushort numStaticFields;
  w_ushort numReferenceFields;
  w_ushort numPrimitiveFields;
  w_field fields;

  w_size numMethods;
  w_method *methods;
 
  /*
  ** The number of methods that a subclass can override and/or inherit.
  ** All the non static classes, non private methods and the non initializers
  */  

  w_size numInheritableMethods;

  /*
  ** The number of superclasses of this
  ** class taking into account the java/lang/Object root class.
  */
  
  w_size   numSuperClasses;
  w_word *staticFields;  /* the static data fields for this class */
  w_size numStaticWords; /* the number of static field words */

  w_size instanceSize;   /* the instance size in words */
  w_aclazz aclazz;       /* if this an array clazz, a pointer to the array information */
  x_monitor  resolution_monitor; /* used to protect resolution of constants */

  w_instance Class;      /* the Class instance for this clazz */
  w_desc desc;           /* the descriptor for this clazz */
  w_desc adesc;          /* the descriptor for a 1 dimensional array of this clazz */
                         /* some commonly-occurring methods, if present: */
  w_method defaultInit;  /*   <init>()V           default initializer    */
  w_method clinit;       /*   <clinit>            class initializer      */
  w_method run;          /*   run()V              runnable entry point   */
  w_method finalizer;    /*   finalize()V         instance finalizer     */


  w_long suid;          /* the cached SUID of this clazz or 0 */

  /* Pointers to functions which either verify, prepare, etc. this      */
  /* or do nothing (because it is already verified, prepared, ...).     */
  w_instance (*verify)(w_clazz);
  w_instance (*prepare)(w_clazz);
  w_instance (*resolve)(w_clazz);
  w_instance (*initialize)(w_clazz);
  w_string   failure_message;
  char *file;
  int line;
  // New stuff for cache
  w_size bytes_needed;     /* The number of bytes we have to allocate for an instance of this clazz. */
  w_size cache_hits;       /* The number of cache hits we had.                                       */
  w_size cache_miss;       /* The number of times our cache was empty.                               */
  x_mutex cache_lock;      /* The mutex lock for the cache.                                          */
  w_wordset cache;         /* The cached instances.                            */

  w_string source;         /* source filename */
  w_thread loading_thread; /* Thread currently busy loading (preparing, ... this thread */
} w_Clazz;

#define getClazzState(c) (((c)->flags & CLAZZ_STATE_MASK) >> CLAZZ_STATE_SHIFT)
#define setClazzState(c,s) (c)->flags = (((c)->flags & ~CLAZZ_STATE_MASK) | ((s) << CLAZZ_STATE_SHIFT))

/*
** An unloaded class consists of a name-classloader pair.  The state subfield
** of the flags will always be zero (CLAZZ_STATE_UNLOADED).
*/
typedef struct w_UnloadedClazz {

  w_string dotified; 
  char   *label;
  
  w_flags flags;
  w_instance loader;     /* the ClassLoader that will define this class     */
} w_UnloadedClazz;

/*
** Compare two w_Clazz/w_UnloadedClazz pointers for equivalence: they are
** equivalent if they have the same name and classloader.
*/
static inline w_boolean same_clazz(w_clazz clazz1, w_clazz clazz2) {
  return (clazz1 == clazz2) || (clazz1->dotified == clazz2->dotified && clazz1->loader == clazz2->loader);
}

/*
** The low-order 16 bits of the `flags' field are the ACC_... flags as they
** appear in the class file format.  The remaining fields are defined below.
*/
#define CLAZZ_STATE_MASK        0x000f0000
#define CLAZZ_STATE_SHIFT               16
#define CLAZZ_STATE_UNLOADED             0
#define CLAZZ_STATE_LOADING              1
#define CLAZZ_STATE_SUPERS_LOADED        2
#define CLAZZ_STATE_LOADED               4
#define CLAZZ_STATE_VERIFYING            5
#define CLAZZ_STATE_VERIFIED             6
#define CLAZZ_STATE_PREPARING            7
#define CLAZZ_STATE_PREPARED             8
#define CLAZZ_STATE_INITIALIZING        11
#define CLAZZ_STATE_INITIALIZED         12
#define CLAZZ_STATE_BROKEN              15

#define CLAZZ_IS_PRIMITIVE      0x00800000
#define CLAZZ_IS_THROWABLE      0x02000000
#define CLAZZ_IS_THREAD         0x04000000
#define CLAZZ_IS_THREADGROUP    0x08000000
#define CLAZZ_IS_REFERENCE      0x10000000
#define CLAZZ_IS_CLASSLOADER    0x20000000
#define CLAZZ_HAS_FINALIZER     0x40000000
#define CLAZZ_HAS_WOTSITS       0x80000000

#define CLAZZ_HERITABLE_FLAGS  (CLAZZ_IS_REFERENCE | CLAZZ_IS_THREADGROUP | CLAZZ_IS_THROWABLE | CLAZZ_IS_THREAD | CLAZZ_IS_CLASSLOADER)

/** Attach an instance of java.lang.Class to this clazz.
 ** Note that this is not currently used for array clazzes
 ** (createSingleArrayClazzFromDesc has its own code to do
 ** the equivalent thing).
 */
w_instance attachClassInstance(w_clazz);

/*
** clazz2Class returns the w_instance of Class associated with this w_clazz.
*/

static inline w_instance clazz2Class(w_clazz clazz) {

  w_instance Class = clazz ? clazz->Class : NULL;

 if (Class == NULL) {
    Class = attachClassInstance(clazz);
 }
 
 return Class;

}

static inline w_instance clazz2loader(w_clazz clazz) {
  return clazz->loader;
}

static inline w_clazz Class2clazz(w_instance Class) {

  if (Class == NULL) {
    return NULL;
  }

  if (isPersistentRef(Class)) {
    Class = pinstance2local_copy(Class);
    if (Class == NULL) {
      return NULL;
    }
  }

  return (w_clazz)Class[F_Class_wotsit];

}

extern w_hashtable system_class_hashtable;
w_clazz namedClassMustBeLoaded(w_instance classLoader, w_string name, w_instance *exception);

#define classMustBeInitialized(c,t) _classMustBeInitialized(&(c),(t))

static inline w_instance 
_classMustBeInitialized(w_clazz *clazzptr, w_thread thread) {
  w_instance exception = NULL;

  if (getClazzState(*clazzptr) == CLAZZ_STATE_UNLOADED) {
    *clazzptr = namedClassMustBeLoaded((*clazzptr)->loader, (*clazzptr)->dotified, &exception);
    deregisterString((*clazzptr)->dotified);
    releaseMem(*clazzptr);
  }

  if (exception) {

    return exception;

  }
  return (*clazzptr)->initialize(*clazzptr);
}

/// Get a pointer to a static field of a class
inline static void *getStaticPrimitiveField(w_clazz clazz, w_int slot) {
  classMustBeInitialized(clazz, currentWonkaThread);
  return &clazz->staticFields[slot];
}

/// Get a copy of a reference field of a class
inline static w_instance getStaticReferenceField(w_clazz clazz, w_int slot) {
  classMustBeInitialized(clazz, currentWonkaThread);
  return (w_instance)clazz->staticFields[slot];
}

/// Set a reference field of a class.
// TODO: change this to use clazz, slot???
inline static void setStaticReferenceField(w_clazz clazz, w_int slot, w_instance child) {
  w_instance parent = clazz2Class(clazz);
  w_word *child_flags;
  w_word *parent_flags;

  classMustBeInitialized(clazz, currentWonkaThread);

  if (isNullInstance(child)) {
    clazz->staticFields[slot] = 0;

    return;

  }

  parent_flags = instance2flagsptr(parent);
  child_flags = instance2flagsptr(child);

  woempa(1,"Parent flags: 0x%08x\n",*parent_flags);
  woempa(1,"Child flags: 0x%08x\n",*child_flags);
  if (isSet(*parent_flags,O_PERSISTENT)) {
    wabort(ABORT_WONKA,"clazz2Class(clazz%w) returned a persistent object\n",clazz->dotified);
  }

#ifdef ENABLE_PS
  if (isSet(*child_flags,O_PERSISTENT)) {
    woempa(1,"  --> markPersistentRef(child)\n");
    markPersistentRef(child);
  }

  if (isSet(*parent_flags,O_LOCAL_COPY)) {
    woempa(9,"Parent is %p (%w)\n",parent,Class2clazz(parent)->name);
    besmirchLocalCopy(parent);
  }
#endif

  clazz->staticFields[slot] = (w_word)child;

  if (isSet(*parent_flags, O_BLACK | O_NEAR_BLACK)) {
    if (isNotSet(*child_flags, O_BLACK | O_NEAR_BLACK)) {
      setFlag(*child_flags, O_GRAY);
    }
  }
}

w_clazz _allocClazz(const char *file, int line, w_thread thread);
#define allocClazz(t) _allocClazz(__FILE__, __LINE__, t)
w_clazz createClazz(w_thread, w_string name, w_bar source, w_instance loader, w_instance *loading_problem);
void destroyClazz(w_clazz);
void registerClazz(w_clazz, w_hashtable);
void createClazzDescriptor(w_clazz);
void startClasses(void);

/*
** getMethod_name_sig retrieves the method of `clazz' with a given name
** (e.g. `add') and signature (e.g. `(Ljava/lang/Object;)' - note that the
** return type is not present).  Both `name' and `sig' are w_string's.
*/
w_method getMethod_name_sig(w_clazz, w_string name, w_string sig);

/*
** getMethod_name_desc  retrieves the method of `clazz' with a given name
** (e.g. `add') and descriptor (e.g. `(Ljava/lang/Object;)V' - note that the
** return type -is- present).  Both `name' and `desc' are w_string's.
*/
static inline w_method getMethod_name_desc(w_clazz clazz, w_string name, w_string descname) {
  w_char  *sigbuf = allocMem_system(descname->length * sizeof(w_char));
  w_size   siglen;
  w_string sigstring;
  w_method method;

  woempa(1,"searching for method %w%w in %w\n",name, descname, clazz->dotified);
  siglen = w_string2chars(descname,sigbuf);
  while (sigbuf[siglen-1] != ')') {
    --siglen;
  }
  sigstring = unicode2String(W_Thread_system, sigbuf, siglen);
  releaseMem(sigbuf);
  method = getMethod_name_sig(clazz, name, sigstring);
  getMethod_name_sig(clazz, name, sigstring);
  deregisterString(sigstring);

  return method;
}

/*
** getField retrieves the field of `clazz' with a given name.
*/
w_field getField(w_clazz, w_string);

/*
** If a class with the given name has already been loaded by `loader',
** return a pointer to its w_Clazz structure; otherwise return a pointer
** to a w_UnloadedClazz which encapsulates the name and loader.
*/
w_clazz identifyClazz(w_string name, w_instance loader);

extern w_clazz clazz_boolean;
extern w_clazz clazz_char;
extern w_clazz clazz_float;
extern w_clazz clazz_double;
extern w_clazz clazz_byte;
extern w_clazz clazz_short;
extern w_clazz clazz_int;
extern w_clazz clazz_long;
extern w_clazz clazz_void;

extern w_clazz clazzArrayOf_Object;
extern w_clazz clazzArrayOf_Class;
extern w_clazz clazzArrayOf_String;
extern w_clazz clazzArrayOf_Constructor;
extern w_clazz clazzArrayOf_Field;
extern w_clazz clazzArrayOf_Method;

extern w_string string_angle_brackets_init;
extern w_string string_angle_brackets_clinit;
extern w_string string_no_params;
extern w_string string_no_params_V;
extern w_string string_boolean;
extern w_string string_byte;
extern w_string string_char;
extern w_string string_class;
extern w_string string_double;
extern w_string string_finalize;
extern w_string string_run;
extern w_string string_float;
extern w_string string_int;
extern w_string string_interface;
extern w_string string_long;
extern w_string string_short;
extern w_string string_void;

// Names of classfile attributes
extern w_string string_ConstantValue;
extern w_string string_Code;
extern w_string string_Deprecated;
extern w_string string_Exceptions;
extern w_string string_LineNumberTable;
extern w_string string_LocalVariableTable;
extern w_string string_Reference;
extern w_string string_SourceFile;
extern w_string string_Synthetic;

// Strings used when throwing ClassFormatError
extern w_string string_error_wrong_name;
extern w_string string_error_too_short;
extern w_string string_error_too_long;
extern w_string string_error_bad_magic;
extern w_string string_error_bad_constant;

extern char * print_clazz(char*, int*, void*, int w, int p, unsigned int f);


#endif /* _CLAZZ_H */
