#ifndef _CONSTANT_H
#define _CONSTANT_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: constant.h,v 1.7 2002/03/27 13:46:34 gray Exp $
*/

#include "wonka.h"

/*
** For more info on the structure of the constant pool, refer
** to any book on the classfile structure.
*/

/*
** UTF8-encoded Unicode string.  (Actually it's not quite UTF8, owing
** to special encoding of u0000 and presence of a trailing null byte).
*/
typedef struct w_Utf8 {
  u2 refcount;
  u2 length;
  w_string string;
} w_Utf8;

/*
** Format used for Field, Method and IMethod constants.
*/
typedef struct w_Member {
  u2 class_index;
  u2 name_and_type_index;
} w_Member;

/*
** A name-and-type constant contains the indices of two ITF8 constants:
** the name of the member and its descriptor.
*/
typedef struct w_Name_and_Type {
  u2 name_index;
  u2 descriptor_index;
} w_Name_and_Type;

/*
** Our mighty Union for holding any constant, either in its unresolved
** form (see w_Utf8, w_Member, w_Name_and_Type above) or the result of
** resolution.  Note that Integer, Float, Long and Double are resolved
** while reading in the class file (it's just a matter of putting the
** bytes in the right order).  Unresolved Class and String constants
** are just the index of a UTF8, it's not worth defining a struct for
** them.  Note also that Name-and-Type constants don't get resolved.
**
** Field `get' points to function which, when passed the clazz and
** index of this constant, will resolve it if necessary and return
** a pointer to the resolved data: see the various function definitions
** below. 
*/
typedef struct w_Cnt {
  void* (*get)(w_clazz, w_int);
  w_tag cnt_tag;
  union {
    w_Name_and_Type nat; /* Name and type (resolution n/a)      */
    w_Utf8   utf8;       /* UTF8 string (unresolved)            */
    w_int    strindex;   /* Unresolved string (index of a UTF8) */
    w_int    clindex;    /* Unresolved class (index of a UTF8)  */
    w_Member member;     /* Unresolved field/method/imethod     */
                         /* (index of a name-and-type)          */
    w_clazz  clazz;      /* Resolved class                      */
    w_method method;     /* Resolved method/imethod             */
    w_field  field;      /* Resolved field                      */
    w_string string;     /* Resolved string                     */
    w_int    intval;     /* Integer (resolved immediately)      */
    w_u64    longval;    /* Long (resolved immediately)         */
    w_float  floatval;   /* Float (resolved immediately)        */
    w_u64    doubleval;  /* Double (resolved immediately)       */
  } info;
} w_Cnt;

/*
** Function get_resolved simply returns a pointer to the `info' field;
** the `get' field is set to point to this function when a constant has
** been resolved, or immediately in the case of Integer/Float/Long/Double.
*/
void * get_resolved(w_clazz, w_int);

/*
** Function get_wrong_word is used for the second word of a two-word
** constant (Long/Double).  Should never be called.
*/
void * get_wrong_word(w_clazz, w_int);

/*
**  Function get_broken is used for constants we already tried unsuccessfully 
** to resolve.  Throws the same exception as last time (and returns NULL).
*/
void *get_broken(w_clazz, w_int);

/*
** Functions get_unresolved_XXXX (XXXX = String, Class, Field, Method,
** IMethod) are used for unresolve String, Class, ... constants.  When
** the function is called it will try to resolve the constant and if
** successful will overwrite itself with get_resolved.  If it fails
** it will set an exception in the thread control block, overwrite 
** itself with get_broken, and return NULL.
**
** Function get_unresolved_Other is used for constants for which the
** get() function should never be used.
*/
void * get_unresolved_String(w_clazz, w_int);
void * get_unresolved_Class(w_clazz, w_int);
void * get_unresolved_Field(w_clazz, w_int);
void * get_unresolved_Method(w_clazz, w_int);
void * get_unresolved_IMethod(w_clazz, w_int);
void * get_unresolved_Other(w_clazz, w_int);

/*
** The functions getXXXXConstant return the contents of a constant after
** resolution, dereferenced and cast to an appropriate type.  If RUNTIME_CHECKS
** is set then these functions also check that the constant really is of
** the correct type.
*/

static inline w_int getIntegerConstant(w_clazz clazz, w_int idx) {
  w_cnt c = &clazz->constants[idx];
  void *info;

#ifdef RUNTIME_CHECKS
  if (c->cnt_tag != CONSTANT_INTEGER) {
    wabort(ABORT_THREAD, "Constant[%d] of %k is type %d, must be %d\n", idx, clazz, c->cnt_tag, CONSTANT_INTEGER);
  }
#endif

  info = (*c->get)(clazz, idx);

  return info ? *((w_int*)info) : 0;
}

static inline w_float getFloatConstant(w_clazz clazz, w_int idx) {
  w_cnt c = &clazz->constants[idx];
  void *info;

#ifdef RUNTIME_CHECKS
  if (c->cnt_tag != CONSTANT_FLOAT) {
    wabort(ABORT_THREAD, "Constant[%d] of %k is type %d, must be %d\n", idx, clazz, c->cnt_tag, CONSTANT_FLOAT);
  }
#endif

  info = (*c->get)(clazz, idx);

  return info ? *((w_float*)info) : 0;
}

static inline w_u64 getLongConstant(w_clazz clazz, w_int idx) {
  w_cnt c = &clazz->constants[idx];
  void *info;

#ifdef RUNTIME_CHECKS
  if (c->cnt_tag != CONSTANT_LONG) {
    wabort(ABORT_THREAD, "Constant[%d] of %k is type %d, must be %d\n", idx, clazz, c->cnt_tag, CONSTANT_LONG);
  }
#endif

  info = (*c->get)(clazz, idx);

  return info ? *((w_u64*)info) : (w_u64)0LL;
}

static inline w_u64 getDoubleConstant(w_clazz clazz, w_int idx) {
  w_cnt c = &clazz->constants[idx];
  void *info;

#ifdef RUNTIME_CHECKS
  if (c->cnt_tag != CONSTANT_DOUBLE) {
    wabort(ABORT_THREAD, "Constant[%d] of %k is type %d, must be %d\n", idx, clazz, c->cnt_tag, CONSTANT_DOUBLE);
  }
#endif

  info = (*c->get)(clazz, idx);

  return info ? *((w_u64*)info) : (w_u64)0LL;
}

static inline w_string getStringConstant(w_clazz clazz, w_int idx) {
  w_cnt c = &clazz->constants[idx];
  void *info;

#ifdef RUNTIME_CHECKS
  if (c->cnt_tag != CONSTANT_STRING) {
    wabort(ABORT_THREAD, "Constant[%d] of %k is type %d, must be %d\n", idx, clazz, c->cnt_tag, CONSTANT_STRING);
  }
#endif

  x_monitor_enter(clazz->resolution_monitor, x_eternal);
  info = (*c->get)(clazz, idx);
  x_monitor_exit(clazz->resolution_monitor);

  return info ? *((w_string*)info) : NULL;
}

static inline w_clazz getClassConstant(w_clazz clazz, w_int idx) {
  w_cnt c = &clazz->constants[idx];
  void *info;

#ifdef RUNTIME_CHECKS
  if (c->cnt_tag != CONSTANT_CLASS) {
    wabort(ABORT_THREAD, "Constant[%d] of %k is type %d, must be %d\n", idx, clazz, c->cnt_tag, CONSTANT_CLASS);
  }
#endif

  x_monitor_enter(clazz->resolution_monitor, x_eternal);
  info = (*c->get)(clazz, idx);
  x_monitor_exit(clazz->resolution_monitor);

  return info ? *((w_clazz*)info) : NULL;
}

static inline w_field getFieldConstant(w_clazz clazz, w_int idx) {
  w_cnt c = &clazz->constants[idx];
  void *info;

#ifdef RUNTIME_CHECKS
  if (c->cnt_tag != CONSTANT_FIELD) {
    wabort(ABORT_THREAD, "Constant[%d] of %k is type %d, must be %d\n", idx, clazz, c->cnt_tag, CONSTANT_FIELD);
  }
#endif

  x_monitor_enter(clazz->resolution_monitor, x_eternal);
  info = (*c->get)(clazz, idx);
  x_monitor_exit(clazz->resolution_monitor);

  return info ? *((w_field*)info) : NULL;
}

static inline w_method getMethodConstant(w_clazz clazz, w_int idx) {
  w_cnt c = &clazz->constants[idx];
  void *info;

#ifdef RUNTIME_CHECKS
  if (c->cnt_tag != CONSTANT_METHOD) {
    wabort(ABORT_THREAD, "Constant[%d] of %k is type %d, must be %d\n", idx, clazz, c->cnt_tag, CONSTANT_METHOD);
  }
#endif

  x_monitor_enter(clazz->resolution_monitor, x_eternal);
  info = (*c->get)(clazz, idx);
  x_monitor_exit(clazz->resolution_monitor);

  return info ? *((w_method*)info) : NULL;
}

static inline w_method getIMethodConstant(w_clazz clazz, w_int idx) {
  w_cnt c = &clazz->constants[idx];
  void *info;

#ifdef RUNTIME_CHECKS
  if (c->cnt_tag != CONSTANT_IMETHOD) {
    wabort(ABORT_THREAD, "Constant[%d] of %k is type %d, must be %d\n", idx, clazz, c->cnt_tag, CONSTANT_IMETHOD);
  }
#endif

  x_monitor_enter(clazz->resolution_monitor, x_eternal);
  info = (*c->get)(clazz, idx);
  x_monitor_exit(clazz->resolution_monitor);

  return info ? *((w_method*)info) : NULL;
}

/*
** Function resolveUtf8 returns the result of resolving the given UTF8
** constant.
*/
w_string resolveUtf8(w_clazz, w_int idx, w_thread);

void dissolveConstant(w_clazz, int idx);

void dumpPools(int fd, w_clazz clazz);

#endif /* _CONSTANT_H */
