#ifndef _DESCRIPTOR_H
#define _DESCRIPTOR_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: descriptor.h,v 1.4 2002/03/22 11:53:54 gray Exp $
*/

#include "hashtable.h"
#include "wonka.h"

extern const char primitive2char[];
extern const char *primitive2name[];
extern w_desc primitive2desc[];
extern w_instance primitive2classInstance[];
extern w_clazz primitive2clazz[];
extern const unsigned char desc2bits[];
extern const unsigned char desc2words[];
extern w_clazz primitive2wrapper[];
extern w_size primitive2wrapperSlot[];

#define sizeInBits(d)  ((d)->tag == D_Primitive ? desc2bits[(d)->body.primitive.type]  : desc2bits[(d)->tag])
#define sizeInWords(d) ((d)->tag == D_Primitive ? desc2words[(d)->body.primitive.type] : desc2words[(d)->tag])

/*
** A descriptor for a primitive type.
*/

typedef struct P_Desc {
  P_Type type;
} P_Desc;

/*
** A descriptor for methods.
*/

typedef struct M_Desc {
  w_desc retDesc;
  int numArgs;
  w_desc *args;
} M_Desc;

/*
** A descriptor for arrays. The 'type' element contains a descriptor of what
** is the type that the array holds. That can well be a descriptor for another
** array, in case of multi dimensional arrays.
*/

typedef struct A_Desc {
  w_desc component;
} A_Desc;

/*
** And now up to the descriptor itself...
**
** Notes:
**
** 1) the D_Type enum is declared in wonka.h
** 2) the name string is registered only once, even when 
**    the descriptor is registered X times. The name string
**    gets the flag STR_DESCRIPTOR set.
*/

typedef struct w_Desc {
  w_string name;
  w_int    registered;
  D_Type   tag;
  union {
    P_Desc primitive;
    M_Desc method;
    A_Desc array;
  } body;
} w_Desc;

/*
** Some descriptors that are very frequently used, are available through
** a descriptor constant.  (The primitive descriptors *must* be defined
** here: parseAndCreate assumes this.)
*/

extern w_desc desc_byte;              /* primitive BYTE                  */
extern w_desc desc_char;              /* primitive CHAR                  */
extern w_desc desc_double;            /* primitive DOUBLE IEEE 754       */
extern w_desc desc_float;             /* primitive FLOAT IEEE 754        */
extern w_desc desc_int;               /* primitive INTEGER               */
extern w_desc desc_long;              /* primitive LONG                  */
extern w_desc desc_short;             /* primitive SHORT                 */
extern w_desc desc_boolean;           /* primitive BOOLEAN               */
extern w_desc desc_void;              /* primitive VOID                  */
extern w_desc desc_Object;            /* Ljava/lang/Object;              */
extern w_desc desc_String;            /* Ljava/lang/String;              */
extern w_desc desc_ArrayOf_Object;    /* [Ljava/lang/Object;             */
extern w_desc desc_ArrayOf_Class;     /* [Ljava/lang/Class;              */
extern w_desc desc_ArrayOf_String;    /* [Ljava/lang/String;             */

/*
** An array that maps primitive array types to their respective descriptor.
** The indexes correspond with the 'newarray' opcode section.
*/

extern w_desc atype2desc[];

/*
** Create a new Wonka descriptor from a Wonka string which holds a
** FieldDescriptor as defined in section 4.3.2 of the JVM Spec.  
** Returns a pointer to the w_Desc structure, or null if an error 
** occurred while parsing the string
*/

w_desc _newFieldDescriptor(const char *f, int l, w_thread thread, w_string string);
#define newFieldDescriptor(t, s) _newFieldDescriptor(__FILE__, __LINE__, t, s)

/*
** Create a new Wonka descriptor from a Wonka string which holds a
** MethodDescriptor as defined in section 4.3.3 of the JVM Spec.  
** Returns a pointer to the w_Desc structure, or null if an error 
** occurred while parsing the string
*/

w_desc _newMethodDescriptor(const char *f, int l, w_thread thread, w_string string);
#define newMethodDescriptor(r, s) _newMethodDescriptor(__FILE__, __LINE__, r, s)

/// Create an array descriptor given an element class and a number of dimensions
w_desc createArrayDescriptor(w_thread thread, w_clazz clazz, w_int dim);

/*
** Find the existing descriptor corresponding to a registered Wonka string.  
** Use this when you don't want to create a new descriptor or to increment the 
** refCount of an existing one - for that you must use newDescriptor (v.s.).

w_desc _findDescriptorByName(const char *file, int line, w_string string);
#define findDescriptorByName(s) _findDescriptorByName(__FILE__, __LINE__, s)
*/

w_ubyte *initDescriptors(w_ubyte *memory);
void startDescriptors(void);
void dumpDescriptors(void);
w_desc _registerDescriptor(const char *f, int l, w_thread thread, w_desc desc);
#define registerDescriptor(r, d) _registerDescriptor(__FILE__, __LINE__, r, d)

void _deregisterDescriptor(const char *f, int l, w_desc desc);
#define deregisterDescriptor(d) _deregisterDescriptor(__FILE__, __LINE__, d)

w_boolean ht_descCompare(w_word desc1_word, w_word desc2_word);
w_word ht_descHash(w_word desc_word);

w_desc parameterArray2Desc(w_thread thread, w_instance parameterArray);

/*
** See if a clazz is a primitive clazz, e.g. 'int.class' and return
** the appropriate index if that is the case, i.e. one that can be used
** to index the primitive2... arrays. Returns 0 if the clazz is not a primitive 
** clazz.
*/
w_int isPrimitiveClazz(w_clazz clazz);

#define w_desc_equals(a,b) ((a)==(b))

/*
** Parse a descriptor (sub)string for the first class descriptor it contains,
** returning a pointer to a w_Clazz or w_UnloadedClazz.  The substring parsed
** starts at string->chars[*start] and ends just before string->char[end];
** after parsing, *start will index the first character of the next descriptor
** in the string or be equal to end if there is none.
** 
** If parsing fails, NULL is returned and *start is undefined.
*/
w_clazz parseDescriptor(w_string descriptor, w_size *start, w_size end, w_instance loader);

#endif /* _DESCRIPTOR_H */
