#ifndef _FIELD_H
#define _FIELD_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: fields.h,v 1.3 2002/03/26 10:44:14 gray Exp $
*/

#include "heap.h"
#include "wonka.h"

/*
** Define F_Field_wotsit. This way we don't need to include the huge 
** core-classes.h file and as a result speed up compilation a lot since 
** almost every .c file includes this file.
*/

extern w_int F_Field_wotsit;

/*
** Fields of w_Field:
**  declaring_clazz  the declaring clazz
**  label           the C string "field"
**  name            the field's name
**  desc            its descriptor
**  flags           low 16 bits: ACC_xxx flags from class file
**                  high 16 bits: Wonka FIELD_xxx flags, see below
**  slot            the slot in a w_instance that this field lives in: 64 bit
**                  values live at 'slot' (LSW) and 'slot' + 1 (MSW).
**  initval         index in the constant pool of the constant initialiser (0 if none)
*/
typedef struct w_Field {
  w_clazz  declaring_clazz;
  char    *label;
  w_string name;
  w_desc   desc;
  w_flags  flags;
  w_ushort slot;
  w_ushort initval;
} w_Field;

/*
** Wonka flags.  The following combinations are possible:
**  WOTSIT  ARRAY  REFERENCE  LONG  meaning
**     0       0       0       1    primitive field, long or double
**     0       0       0       0    primitive field, other
**     0       0       1       0    reference field, not an array
**     0       1       1       0    array reference field
**     1       0       0       0    wotsit
*/
#define FIELD_IS_LONG       0x10000
#define FIELD_IS_REFERENCE  0x20000
#define FIELD_IS_ARRAY      0x40000
#define FIELD_IS_WOTSIT     0x80000

/*
** fieldSize(f) returns 2 if f->flags has FIELD_IS_LONG set, else 1.
*/
#define fieldSize(f) (1 + !!isSet((f)->flags,  FIELD_IS_LONG))

/*
** Matching criteria
*/

#define MATCH_STATIC_FIELD                         1
#define MATCH_INSTANCE_FIELD                      -1
// #define MATCH_STATIC_OR_INSTANCE                   0

#define MATCH_PRIVATE_FIELD                        1
#define MATCH_PUBLIC_FIELD                        -1
// #define MATCH_PUBLIC_OR_PRIVATE                    0

// #define MATCH_ANY_DESCRIPTOR                       NULL

#define MATCH_ANY                                  0

w_int fieldMatch(w_field field, w_string name, w_desc desc, w_int isStatic, w_int isPrivate);
w_field searchClazzHierarchyForField(w_clazz clazz, w_string string, w_desc desc, w_int isStatic, w_int isPrivate);
w_field searchClazzAndInterfacesForField(w_clazz clazz, w_string string, w_desc desc, w_int isStatic, w_int isPrivate);
w_size findFieldOffset(w_clazz clazz, const char * utf8name);
w_boolean isFieldFromClazz(w_clazz clazz, w_field field);
void fieldTableDump(w_clazz clazz);

static inline w_field Field2field(w_instance Field) {

  if (Field == NULL) {
    return NULL;
  }

  if (isPersistentRef(Field)) {
    Field = pinstance2local_copy(Field);
    if (Field == NULL) {
      return NULL;
    }
  }

  return (w_field)Field[F_Field_wotsit];

}

#endif /* _FIELD_H */
