#ifndef _LOADING_H
#define _LOADING_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: loading.h,v 1.12 2002/03/26 10:44:14 gray Exp $ 
** 
** The Wonka kernel is software copyright by SmartMove NV (1999).
** Please see the file Copyright for information on its legal use.
** 
*/

#include "clazz.h"
#include "oswald.h"
#include "wonka.h"

w_clazz seekClazzByName(w_string name, w_hashtable class_hashtable);
w_clazz seekClazzByDesc(w_desc, w_hashtable class_hashtable);
w_clazz namedClassMustBeLoaded(w_instance classLoader, w_string name, w_instance *exception);
w_clazz descClassMustBeLoaded(w_instance classLoader, w_desc desc, w_instance *exception);
w_clazz namedArrayClassMustBeLoaded(w_instance classLoader, w_string name, w_instance *exception);
w_clazz descArrayClassMustBeLoaded(w_instance classLoader, w_desc desc, w_instance *exception);

/*
** If the given string begins with L, strip the leading L and trailing ';'.
** Returns a registered w_string.
*/
w_string undescriptifyClassName(w_string string);

/*
** Create an array clazz (a clazz with a ->aclazz) from a w_desc
*/
w_clazz createArrayClazzFromDesc(w_instance classLoader, w_desc desc, w_instance *exception);
/*
** Result of createClazzTables is NULL if everything OK, clazz of an exception
** otherwise (e.g. clazzAbstractMethodError).
*/
w_clazz createClazzTables(w_instance classLoader, w_clazz clazz);

static inline w_clazz mustBeLoaded(w_clazz clazz, w_instance *exception) {
  w_clazz result = clazz;

  if (getClazzState(clazz) == CLAZZ_STATE_UNLOADED) {
    result = namedClassMustBeLoaded(clazz->loader, clazz->dotified, exception);
    deregisterString(clazz->dotified);
    releaseMem(clazz);
  }

  return result;
}

/*
** The following functions go in groups of four: 
** classMustBeVerified/classIsVerified/verifyClass/alreadyVerified,
** classMustBePrepared/classIsPrepared/prepareClass/alreadyPrepared, etc..
** Initially the clazz structure is set up so that clazz->verify
** points to verifyClazz, clazz->prepare points to prepareClass, etc..
** When one the static inlines classMustBeVerified etc. (defined in
** loading.h) is executed for the first time on this class, it calls
** verifyClass (etc.) which in turn updates class->verify to point at
** alreadyVerified (etc.).
*/

#define classMustBeVerified(c,v) _classMustBeVerified(&(c),(v))

static inline w_instance 
_classMustBeVerified(w_clazz *clazzptr, w_thread thread) {
  w_instance exception = NULL;

  *clazzptr = mustBeLoaded(*clazzptr, &exception);
  if (exception) {

    return exception;

  }

  return (*clazzptr)->verify(*clazzptr);
}

w_instance verifyClass(w_clazz);
w_instance alreadyVerified(w_clazz);
w_instance verifyFailed(w_clazz);

static inline w_boolean 
classIsVerified(w_clazz clazz) {
  return clazz->verify == alreadyVerified;
}

#define classMustBePrepared(c,v) _classMustBePrepared(&(c),(v))

static inline w_instance 
_classMustBePrepared(w_clazz *clazzptr, w_thread thread) {
  w_instance exception = NULL;

  *clazzptr = mustBeLoaded(*clazzptr, &exception);
  if (exception) {

    return exception;

  }

  return (*clazzptr)->prepare(*clazzptr);
}

w_instance prepareClass(w_clazz);
w_instance alreadyPrepared(w_clazz);

static inline w_boolean 
classIsPrepared(w_clazz clazz) {
  return clazz->prepare == alreadyPrepared;
}

#define classMustBeResolved(c,v) _classMustBeResolved(&(c),(v))

static inline w_instance 
_classMustBeResolved(w_clazz *clazzptr, w_thread thread) {
  w_instance exception = NULL;

  *clazzptr = mustBeLoaded(*clazzptr, &exception);
  if (exception) {

    return exception;

  }

  return (*clazzptr)->resolve(*clazzptr);
}

w_instance resolveClass(w_clazz);
w_instance alreadyResolved(w_clazz);

static inline w_boolean 
classIsResolved(w_clazz clazz) {
  return clazz->resolve == alreadyResolved;
}

// TODO: get rid of this silly synonym

w_instance initializeClass(w_clazz);
w_instance alreadyInitialized(w_clazz);

static inline w_boolean 
classIsInitialized(w_clazz clazz) {
  return clazz->initialize == alreadyInitialized;
}

/// Class is already FUBAR -- throw a NoClassDefFoundError
w_instance alreadyBroken(w_clazz);

extern w_hashtable fixup_hashtable;

extern w_string string_clinit_fullname;

static inline w_hashtable loader2hashtable(w_instance loader) {
  if (loader == NULL) {

    return system_class_hashtable;

  }
  else {

    return (w_hashtable)(loader[F_ClassLoader_hashtable]);
  }
}

#define loader2monitor(l) (&loader2hashtable((l))->monitor)

/** resolveConstantPool resolves any remaining unresolved items
** in the constant pool, and (TODO) frees up the memory which was used to
** store them.
** It returns an instance of Throwable if an exception occurs, else NULL.
*/
w_instance resolveConstantPool(w_clazz, w_thread);

void setClassSymbol(w_clazz clazz);

void createFieldTable(w_thread, w_clazz clazz);
/*
** Result of registerInterfaceMethods is NULL if everything OK, clazz of an exception
** otherwise (e.g. clazzAbstractMethodError).
*/
w_clazz registerInterfaceMethods(w_thread, w_clazz);
void initializeStaticFields(w_clazz);
w_classData getSystemClassData(w_string classname);

w_clazz loadBootstrapClass(w_string name, w_instance *exception);
w_clazz loadNonBootstrapClass(w_instance classLoader, w_string name, w_instance *exception);

/*
** Function to be called during VM startup, to initialise some data structures.
*/
void startLoading(void);

/*
** Name of the zipfile which contains the bootstrap classes, as a C string.
*/
char     *zipname;

/*
** Set the system class loader to 'scl'.  From this point on classes on
** the bootstrap class path will be loaded by scl, not by loadBootstrapClass.
*/
void setSystemClassLoader(w_instance scl);

/*
** Hashtable mapping clazz x imethod -> method
*/
extern w_hashtable2k interface_hashtable;

/*
** The unique instance of SystemClassLoader (or NULL if not yet installed).
** Must only be set once, using setSystemClassLoader()!
*/
extern w_instance systemClassLoader;

extern w_clazz *(make_loaded)(w_thread, w_clazz, w_instance*);
extern w_clazz *(make_verified)(w_thread, w_clazz, w_instance*);
extern w_clazz *(make_prepared)(w_thread, w_clazz, w_instance*);
extern w_clazz *(make_resolved)(w_thread, w_clazz, w_instance*);
extern w_clazz *(make_initialized)(w_thread, w_clazz, w_instance*);

#endif /* _LOADING_H */
