#ifndef _METHOD_H
#define _METHOD_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: methods.h,v 1.9 2002/03/27 11:47:05 gray Exp $
*/

#include "wonka.h"
#include "dispatcher.h"
#include "heap.h"

/*
** Define F_Method_wotsit. This way we don't need to include the huge core-classes.h file 
** and as a result speed up compilation a lot since almost every .c file includes this file.
*/

extern w_int F_Method_wotsit;

/*
** The slot number for a method that still isn't processed fully.
*/

#define SLOT_NOT_ALLOCATED                     -1

/*
** Debugging attributes.
*/

typedef struct w_LineNum {
  w_short start_pc;
  w_short line_number;
} w_LineNum;

typedef struct w_LocalVar {
  /*
  ** The ranges of the pc where this local variable is active.
  */
  w_pc start_pc;
  w_pc end_pc;
  /* 
  ** The slot into the frame->locals[] array 
  ** where this local variable lives.
  */
  w_short slot; 
  /*
  ** A Name and Type union; after resolution, the name and desc indexes
  ** are converted into a real w_string and a w_desc.
  */
  w_ushort name_index;
  w_ushort desc_index;
} w_LocalVar;

typedef struct w_Exception {
  w_ushort start_pc;
  w_ushort end_pc;
  w_ushort handler_pc;
  w_ushort type_index;
} w_Exception;

typedef struct w_Method {
  w_int slot;              /* if non native, the slot in the table XXX make union */ 
  w_size arg_i;            /* The number of argument words. !! DONT move !! Assembly depends on it.    */
  w_function function;     /* if native, the function pointer. !! DONT move !! Assembly depends on it. */
  w_dispatcher dispatcher; /* The correct set of call/return handlers. */
  char  *label;

  w_ushort local_i;
  w_ushort return_i;
  w_ushort growth_i;         /* stack_i + local_i - arg_i, how many NEW items slots we need */
  w_ushort stack_i;

  w_code code;             /* limit of array size = 65535                  */
  w_size code_length;       /* why is this an int ?                        */

  w_string name;           /* method name (by itself)                      */
  w_string param_list;     /* list of args, in parentheses                 */
  w_flags flags;           /* the access flags for this method             */
  w_clazz clazz;           /* the clazz this method belongs to             */

  /*
  ** The number of exceptions that are in the code attribute
  ** of this method. The exception information is stored in the
  ** exceptions array and the number of exception classes that this method can 
  ** throw, plus the array where we store the indices of the class constants 
  ** which define them.
  */

  w_short numExceptions;
  w_short numThrows;

  w_exception exceptions;
  w_ushort  *throws;

  /*
  ** The debugging information for this method.
  */

  w_short numLocalVars;
  w_short numLineNums;

  w_LocalVar *localVars;
  w_LineNum *lineNums;

/* index of the constant pool entry which holds the signature */
  w_ushort desc_index;
  w_ushort nargs;        
  w_clazz *arg_types;
  w_clazz  return_type;

  #ifdef JSPOT
    /*
    ** The return address of a J-Spot compiled method.  
    **   FIXME: as soon I figured out what new fields are required to 
    **          support J-Spot compiled methods, I'll probably bundle
    **          them in a seperate struct.
    */
  
    w_word return_address;
  #endif
    
} w_Method;

#define METHOD_NEEDS_LINKING   0x00010000
#define METHOD_IS_CLINIT       0x00100000
#define METHOD_IS_CONSTRUCTOR  0x00200000

void _registerNativeMethod(w_clazz clazz, w_fun_dec fp, const char *name, const char *sig);
#define registerNativeMethod(clazz, function, name, sig) _registerNativeMethod(clazz, (w_fun_dec)function, name, sig) 
void executeNativeMethod(w_frame frame, w_method method, w_code current);
w_int code2line(w_method method, w_code current, w_size *pc);

/*
** Formatter functions.
*/

char * print_method_m(char * buffer, int * remain, void * data, int w, int p, unsigned int f);
char * print_method_M(char * buffer, int * remain, void * data, int w, int p, unsigned int f);

#if defined(DEBUG)
void methodTableDump(w_clazz clazz);
#endif

static inline w_method Method2method(w_instance Method) {

  if (Method == NULL) {
    return NULL;
  }

  if (isPersistentRef(Method)) {
    Method = pinstance2local_copy(Method);
    if (Method == NULL) {
      return NULL;
    }
  }

  return (w_method)Method[F_Method_wotsit];

}

void registerNatives(w_clazz clazz, const JNINativeMethod *methods, w_int mcount);

#endif /* _METHOD_H */
