#ifndef _REALM_H
#define _REALM_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: realms.h,v 1.1 2002/02/20 08:49:05 dfrederickx Exp $
*/

#include "threads.h"
#include "wonka.h"

/* Define F_ThreadGroup_wotsit. This way we don't need to include the huge core-classes.h file 
** and as a result speed up compilation a lot since almost every .c file includes this file.
*/

extern w_int F_ThreadGroup_wotsit; 

/*
** A "realm" is a nexus for accounting functions.
** For every realm we maintain quotas for thread, memory and cpu usage.
** We also record the ThreadGroup for which the Realm
** it was created; if this is NULL, and there are no threads
** left running in the realm, then the realm is "obsolete" and can be 
** deleted from the system.
*/

/* The w_Realm structure
*/
typedef struct w_Realm {
/*
** The parent of this Realm, or NULL if this is System Realm.
*/
  w_realm    parent;
/*
** An ASCII string which begins with "realm"
*/
  char      *label;
/*
** How many child Realms do we have?
*/
  w_int      num_children;
/*
** The corresponding ThreadGroup
*/
  w_instance ThreadGroup;
/*
** Maximum memory this Realm may allocate (bytes)
*/
  w_size     memory_quota;
/*
** Maximum number of threads this Realm may spawn (including manager thread)
*/
  w_size     thread_quota;
/*
** Maximum CPU this Realm may use (0x00010000 represents 100%)
*/
  w_size     cpu_quota;
/*
** Bytes of memory currently allocated
*/
  w_size     memory_usage;
/*
** Threads currently running
*/
  w_size     thread_usage;
/*
** Current CPU usage
*/
  w_size     cpu_usage;
/*
** Locking
*/
  x_mutex    mutex;
/*
** Index in the realm_table
*/
  w_int      index;
} w_Realm;

/*
** The unique System Realm
*/
extern w_realm W_Realm_system;

/*
** Find the Realm to which a Thread belongs
*/
static inline w_realm thread2realm(w_thread thread) {
  if (thread && thread->group) {
    checkGroupPointer(thread->group);

    return thread->group->realm;
  }
  
  return W_Realm_system;
}

/*
** Initialise the System Realm.
*/
void initRealms(void);

/*
** Create a new Realm.
*/
w_realm newRealm(w_group group, w_realm parent, w_long memory, w_int threads);

/*
** Destroy a Realm.
*/
void destroyRealm(w_realm);

/*
** Check whether a memory request would be within the quota of a Realm.
*/
w_boolean allowMemoryRequest(w_realm, w_size);

#define JNIEnv2realm(e) thread2realm(JNIEnv2w_thread(e))
#define currentRealm    thread2realm(currentWonkaThread)

char *realmDescription(w_realm);

/*
** Maximum number of realms which may exist at one time.  Must be no greater
** than REALM_INDEX_MASK.
*/
#define MAX_REALMS 100

#endif
