#ifndef _STRINGS_H
#define _STRINGS_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: strings.h,v 1.4 2002/02/19 16:07:01 gray Exp $
*/

#include "heap.h"
#include "wonka.h"

/*
** If NEWSTRINGREG is defined, registerString/deregisterString use the
** `refcount' field of the w_String header in conjunction with ht_read/
** ht_write/ht_erase instead of the special ht_register/ht_deregister.
** The string_hashtable then contains keys only (key==value), saving an
** average of two words per string (one word per hashtable slot).
** TODO: measure the performance impact (if any).
*/
#define NEWSTRINGREG

/*
** Define F_String_wotsit. This way we don't need to include the huge core-classes.h file 
** and as a result speed up compilation a lot since almost every .c file includes this file.
*/

extern w_int F_String_wotsit; 

/*
** The Wonka string structure.  Incorporates a reference count and a cached
** copy of the hash function, as well as a pointer to the canonical instance
** (if any) of this string.  Use function hashString() to calculate and 
** cache the hashcode.
** The canonical instance is created when getStringInstance() is called,
** and removed by GC when the instance becomes inaccessible.
*/

typedef struct w_String {
  w_word      refcount;
  w_word      wonka_hash;
  w_instance  instance;
  w_size      length;
  w_char      chars[4]; // variable length, ashley
} w_String;

/*
** The compareStrings function does a compare on two unicode Wonka strings
** as documented in the compareTo method for java/lang/String.
*/

w_int compareStrings(w_string string1, w_string string2);

/*
** w_string2chars converts a w_string to an array of unicode characters.
** The caller is responsible for providing a sufficient amount of memory
** for the unpacked characters.  The result returned is the number of
** w_chars written (equal to source->numChars).
** The source w_string need not be registered, but *must* be well-formed.
*/

w_size w_string2chars(w_string source, w_char *dest);

#define WONKA_DUMMY_CHAR 0xFFFF

/*
** putw_char outputs one unicode character as UTF8.  The character is
** stored at *dest, and dest is then incremented.
**
** N.B. first parameter is logically a w_char, but is declared as int
** to keep gcc happy.  (On some platforms the w_char is widened to int
** before being put on the stack, and gcc -Wconversion warns about this).
*/

void putw_char(int ch, w_ubyte **dest);

/*
** w_charLength calculates the length in characters of a well-formed UTF8 string.
*/

w_size w_charLength(w_ubyte *utf8);

/*
** compareUtf8 compares two UTF8 strings
*/

w_int compareUtf8(w_ubyte* utf8a, w_ubyte* utf8b);

/*
** isPrefixUtf8 determines whether string a is a prefix of string b
*/

w_boolean isPrefixUtf8(w_ubyte* utf8a, w_ubyte* utf8b);

//void registerStaticString(w_string string);

/*
** Dump the string hashtable to stdout
*/

void stringDump(void);


/*
** ht_stringCompare() compares two strings for equality and returns WONKA_TRUE
** iff they are identical.  Canonical equality function for string-based
** hashtables.
*/
w_boolean ht_stringCompare(w_word string1_word, w_word string2_word);

/*
** ht_stringHash() returns the hashcode cached within a w_string.  Use this
** for string-based hashtables.
*/
w_word ht_stringHash(w_word string_word);

w_string cstring2String(w_thread thread, const char *cstring, w_size length);

w_string unicode2String(w_thread thread, w_char *chars, w_size length);

w_string utf2String(w_thread thread, const char *utf8string, w_size length);

w_string registerString(w_string string);

void deregisterString(w_string string);

w_instance getStringInstance(w_thread thread, w_string s);
w_int compareString(w_string string1, w_string string2);
w_ubyte *string2UTF8(w_thread thread, w_string string, w_int * buflen);

/*
** unicodeAddNumber will add a certain 'number' of unicode characters from
** the 'source' buffer and catenate them to the 'destination' buffer. The caller
** must supply a buffer of a suitable size, no checking is done! The function
** returns the number of unicode characters added so that it is handly to daisy chain calls
** as in this example:
**
** w_char *buffer = allocMem(...);
** w_int length = 0;
** ...
** length += unicodeAddNumber(buffer + length, whatever_source, 15);
** length += unicodeAddNumber(buffer + length, another_source, 20);
** ...
*/

w_int unicodeAddNumber(w_char * destination, w_char * source, w_int number);

/*
** stringAdd will copy over the 'source' string to the 'destination' buffer.
** The caller is again expected to give a suitably sized buffer. It returns the
** number of characters added to the buffer so that the calls can again be daisy chained,
** as in this example:
**
** w_char *buffer = allocMem(...);
** w_int length = 0;
** ...
** length += stringAdd(buffer + length, string1);
** length += stringAdd(buffer + length, string2);
** ...
*/

w_int stringAdd(w_char *destination, w_string source);

/*
** The cstringAdd is equivalent with stringAdd but takes a '\0' terminated C string
** as the source argument. 
*/

w_int cstringAdd(w_char *destinationr, const char *source);

/*
** dots2slashes takes a Wonka string and returns a registered Wonka
** string in which the dots are replaced with slashes If the string already
** contains slashes then these are replaced by DEL characters(!). Why? 
** Because the only use for this function is to convert fully-qualified
** class names from Java to "internal" format, and in Java format a slash
** should never occur.
** slashes2dots does the exact opposite.
*/

w_string dots2slashes(w_thread thread, w_string string);
w_string slashes2dots(w_thread thread, w_string string);

/*
** startStrings is called from startWonka: it creates the hashtable
** used by the string registration process, and populates it with all
** those commonly-used strings we were just talking about.
*/

void startStrings(void);

/// Lookup table to get the string which describes an array type
extern w_string atype2string[];

/// Special string `<NULL>'
extern w_string string_NULL;

/// Special string `<BOGUS>'
extern w_string string_BOGUS;

/// Special string `<EMPTY>'
extern w_string string_EMPTY;

#define w_string_equals(a, b)                  ((a) == (b))

void dumpStrings(void);

extern w_hashtable string_hashtable;

static inline w_string String2string(w_instance String) {

  if (String == NULL) {
    return NULL;
  }

  if (isPersistentRef(String)) {
    String = pinstance2local_copy(String);
    if (String == NULL) {
      return NULL;
    }
  }

  return (w_string)String[F_String_wotsit];

}

static inline void attachString(w_instance strinstance, w_string string) {
  if (strinstance[F_String_wotsit]) {
    deregisterString((w_string)strinstance[F_String_wotsit]);
  }
  strinstance[F_String_wotsit] = (w_word)registerString(string);
}

#endif /* _STRINGS_H */
