#ifndef _THREADS_H
#define _THREADS_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: threads.h,v 1.3 2002/02/18 11:59:14 gray Exp $
*/

#include "jni.h"
#include "oswald.h"
#include "wonka.h"

extern w_size numThreads;
extern w_thread W_Thread_system;
extern w_thread W_Thread_sysInit;
extern w_instance I_Thread_sysInit;
extern w_group W_ThreadGroup_system;
extern w_instance I_ThreadGroup_system;
extern w_method registerThread_method;
extern w_method deregisterThread_method;

void systemGroupManagerEntry(void);

#define WONKA_DEFAULT_STACK_SIZE            (1024 * 64) // 32
#define WONKA_GC_STACK_SIZE                 (1024 * 64) //  8
#define WONKA_INIT_STACK_SIZE               (1024 * 64) // 19
#define GROUP_MANAGER_STACK_SIZE            (1024 * 64) //  4
#define SYSTEM_GROUP_MANAGER_STACK_SIZE     (1024 * 64) // 14
#define DRIVER_STACK_SIZE                   (1024 * 64) //  8

#define USER_PRIORITY                       5  // This is a java priority

#define GROUP_MANAGER_BOX_SIZE              256
#define GROUP_MANAGER_PRIORITY              10 // Java priority
/* N.B. The kernel priority is set a notch higher, quasi 10.5 */

#define SYSTEM_GROUP_MANAGER_BOX_SIZE       256
#define SYSTEM_GROUP_MANAGER_PRIORITY       15 // Java priority
/* N.B. The kernel priority is set a notch higher, quasi 15.5 */

#define DEFAULT_BOUNCE_LIMIT                500
#define THREAD_NAME_BUFFER_SIZE             64

/*
** Map a Java priority to a kernel thread priority.
** Parameter jp is the Java priority, and the sign of parameter trim
** is used to set priorities ``just above'' or ``just below'' jp
** (in the Java sense of ``above/below'').
*/
#define REVERSE_PRIORITY
#ifdef  REVERSE_PRIORITY
#define _j2k(jp) (NUM_PRIORITIES*(16-(jp))/16)
#else
#define _j2k(jp) (NUM_PRIORITIES*((jp))/16)
#endif
#define priority_j2k(jp,trim) (((jp)<1 ? _j2k(1) : (jp)>15 ? _j2k(15) : _j2k(jp)) + ((trim)>0 ? -1 : (trim)<0 ? 1 : 0))

#define WG_SYSTEM                           0x00000001   /* only 1 group has this flag set...          */
#define w_groupFromThreadInstance(i)        ((w_thread)(i)[F_Thread_wotsit])->group)
#define w_threadFromThreadInstance(i)       ((w_thread)(i)[F_Thread_wotsit]))

typedef struct w_Group {
  w_group parent;
  char   *label;
  w_flags flags;
  w_string name;
  w_int maxPriority;
  w_boolean isDaemon;
  x_mutex mutex;
  w_instance ThreadGroup;
  w_realm realm;

  /*
  ** A queue of messages and the space, for the group manager thread
  ** and the declaration of the queue manager thread and
  ** it's stack space.
  */

  x_queue box;
  w_ubyte *boxSpace;
  w_thread manager;

} w_Group;

/// Check that a w_group pointer really does point to a w_Group
static inline w_group checkGroupPointer(w_group g) {
#ifdef RUNTIME_CHECKS
  if (*((w_word*)g->label) != *((w_word*)"group")) {
    wabort(ABORT_WONKA,"Alleged w_group pointer %p has incorrect label\n",g);
  }
#endif
  return g;
}

/*
** Thread states
*/

#define W_THREAD_UNSTARTED                  0
#define W_THREAD_READY                      1
#define W_THREAD_LOCKING                    2
#define W_THREAD_WAITING                    3
#define W_THREAD_SUSPENDED                  5
#define W_THREAD_DYING                      7
#define W_THREAD_DEAD                       8

extern const char* state2char[];
extern const unsigned char state2liveness[];

#define STATE(t)                            (state2char[(t)->state])
#define threadIsAlive(t)                    (state2liveness[(t)->state])

#define WT_GROUP_MANAGER                 0x00000001 /* the thread is group manager, mutually exclusive with next */
#define WT_NORMAL_THREAD                 0x00000002 /* the thread is a normal thread                             */

#define WT_THREAD_INTERRUPTED            0x00000040 /* the thread has been interrupted */
#define WT_THREAD_KILLED                 0x00000080 /* an asynchronous exception was thrown and has not yet been picked up by the interpreter */
#define WT_THREAD_THROWN                 0x00000100 /* a synchronous exception was thrown and has not yet been picked up by the interpreter */

/*
** The JNIEnv* parameter of a JNI call points at field natenv of the current
** Wonka thread.  Putting natenv as the first field of w_Thread simplifies
** the inline functions w_thread2JNIEnv and JNIEnv2w_thread.
*/

typedef struct w_Thread {
  
  /*
  ** The JNI environment pointer. Make sure it is always the first element of the
  ** w_Thread structure since the JNIEnv2w_thread relies on this! An environment
  ** pointer can then just be cast to a w_thread pointer...
  */
  
  const JNINativeInterface*   natenv;

  char      *label;
  w_flags    flags;  
  w_frame    top;
  w_string   name;
  w_int      state;
  x_queue    messages;
  w_group    group;
  w_instance Thread;
  w_boolean  isDaemon;
  w_frame    bottom;     /* the starting frame for this thread, when active; otherwise NULL */
  w_size     jpriority;

  /*
  ** The native part of a Wonka thread, it's kernel thread, the stack and it's size.
  */
  
  x_thread  kthread;
  w_ubyte *   kstack;
  w_size     ksize;
  w_size     kpriority;
} w_Thread;

/// Check that a w_thread pointer really does point to a w_Thread
#ifdef RUNTIME_CHECKS
static INLINE w_thread checkThreadPointer(w_thread t) {
#ifdef RUNTIME_CHECKS
  if (*((w_word*)t->label) != *((w_word*)"thread")
        && *((w_word*)t->label) != *((w_word*)"group")) {
    wabort(ABORT_WONKA,"Alleged w_thread pointer %p has incorrect label\n",t);
  }
#endif
  return t;
}
#else
#define checkThreadPointer(t) (t)
#endif

inline static w_thread  JNIEnv2w_thread(JNIEnv *env) {
  return checkThreadPointer((w_thread)env);
}

inline static JNIEnv *w_thread2JNIEnv(w_Thread *t) {
  return &checkThreadPointer(t)->natenv;
}

inline static w_instance  JNIEnv2Thread(JNIEnv *env) {
  return JNIEnv2w_thread(env)->Thread;
}

/*
** From an env pointer, give back the current top frame
*/

#define JNIEnv2frame(e)       (JNIEnv2w_thread(e)->top)

extern w_hashtable thread_hashtable;

w_ubyte *initKernel(w_ubyte *);
void startKernel(void);
w_thread w_threadCreate(w_thread parent, w_object root);

w_int nextGroupId(void);
w_int nextThreadId(void);

w_thread createThread(w_thread new, w_thread current, w_instance Thread, w_group group, w_string name, w_size stacksize);
w_thread allocThread(w_thread thread);

void createThreadGroup(w_instance ThreadGroup, w_group parent, w_string name, w_size stacksize);

void requestThreadTermination(w_thread); 
void requestThreadSuspension(w_thread); 
void requestThreadResumption(w_thread); 

w_method  findRunMethod(w_clazz);
w_thread  _currentWonkaThread(const char *f, int l);
w_group getRootGroup(w_group group);

char *threadDescription(w_thread);
char *groupDescription(w_group);

void addThreadToGroup(w_thread thread, w_group group);
void removeThreadFromGroup(w_thread thread, w_group group);

/*
** Function sendMessage queues a message for the manager of the thread group
** of the addressee.
*/
void sendMessage(w_int request, w_thread sender, w_thread adressee, void *content);

#define STACK_PRESET                        0xaa

#define currentWonkaThread     _currentWonkaThread(__FUNCTION__, __LINE__)
#define currentWonkaGroup      ((currentWonkaThread)->group)

/*
** Thread priorities:
**
** ---------------+-----------------------------------------------------------
** kernel prio    | Description of thread running at this kernel priority
** ---------------+-----------------------------------------------------------
**    0           |
**    1           |
**    2           | Garbage collector run priority
**    3           |
**    4           | Thread specific waiter thread
**    5           | Locker service thread
**    6           | ThreadGroup system manager thread
**    7           | ThreadSystem thread (Init thread)
**    8           | Stream threads
**    9           |
**   10           | Finalizer thread
**   11           | Normal Java thread group manager
**   12           | Java priority 10
**   13           |
**   14           | Java priority 9
**   15           |
**   16           | Java priority 8
**   17           |
**   18           | Java priority 7
**   19           |
**   20           | Java priority 6
**   21           | TCP/IP protocol stack
**   22           | Java priority 5
**   23           |
**   24           | Java priority 4
**   25           |
**   26           | Java priority 3
**   27           |
**   28           | Java priority 2
**   29           |
**   30           | Java priority 1
**   31           | Garbage collector iddle thread
** ---------------+-----------------------------------------------------------
*/

#define EXCEPTION_NO_MESSAGE (char *)NULL

extern const char *message2char[];

#define MESSAGE(m)                          (message2char[m->request])

#define MESSAGE_INTERRUPT                   0  // Not currently used
#define MESSAGE_THROWABLE_PENDING           1  // Not currently used
#define MESSAGE_SEND                        2  // Not currently used
#define MESSAGE_CONSUMED                    3
#define MESSAGE_ADDRESSEE_UNKNOWN           4  // Not currently used
#define MESSAGE_MANAGER_BOX_FULL            5  // Not currently used
#define MESSAGE_ADDRESSEE_BOX_FULL          6  // Not currently used
#define MESSAGE_HOLD_OFF                    7  // Not currently used
#define MESSAGE_RELEASE                     8
#define MESSAGE_START_THREAD                9  // Not currently used
#define MESSAGE_STOP_THREAD                10

typedef struct w_Message {

  /*
  ** One of the above defined MESSAGE_ types.
  */

  w_int request;

  /*
  ** The thread sent the message.
  */
  
  w_thread sender;
  
  /*
  ** The receiving thread.
  */
  
  w_thread addressee;
  
  /*
  ** The content of the message.
  */

  void *content;

  /*
  ** The number of times the message bounced.
  */

  w_int bounced;

} w_Message;

void groupManagerEntry(void* manager);

extern const char *thread_report(x_thread);

#endif /* _THREADS_H */
