#ifndef _WONKA_H
#define _WONKA_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** This <stdlib.h> is the only C library include file that we
** take into wonka.h, since it defines NULL.
*/

#include <stdlib.h>

/*
** $Id: wonka.h,v 1.11 2002/03/20 13:04:20 gray Exp $
** 
** The Wonka kernel is software copyright by SmartMove NV (1999).
** Please see the file Copyright for information on it's legal use.
** 
*/

/*
** Define CACHE_INSTANCES to enable the per-class instance cache (didn't 
** work on 20010905: crashes in deregisterString [CG]).
** #define CACHE_INSTANCES
*/

#define NM(e) ((e)->name)

#define isSet(x, flag)              ((x) & (flag))
#define isNotSet(x, flag)           (!isSet((x), (flag)))
#define setFlag(x, flag)            ((x) |= (flag))
#define unsetFlag(x, flag)          ((x) &= ~(flag))
#define maskFlags(m, f)             ((m) & (f))

typedef unsigned int                w_word;
typedef long long                   w_dword;
typedef unsigned int                w_size;

/*
** Include processor dependent stuff from 
** the ./processor/<arch> directory.
*/

#include "processor.h"

#if (defined(DEBUG) || defined(LOWMEM))
#define INLINE
#else
#define INLINE inline
#endif

/*
** An object instance is just an array of words that holds field variables.
** It is the trailing part of a w_object.
*/

typedef struct w_Object            *w_object;
typedef w_word                     *w_instance;


#define WONKA_TRUE   1
#define WONKA_FALSE  0

typedef struct w_Array *              w_array;
typedef struct w_Aclazz *             w_aclazz;
typedef struct w_Method *             w_method;
typedef struct w_Field *              w_field;
typedef struct w_Frame *              w_frame;
typedef struct w_Desc *               w_desc;
typedef struct w_Local *              w_local;
typedef int                           w_color;
typedef const struct JNINativeInterface *   w_env;
typedef unsigned int                  w_pc; /* the program counter */
typedef unsigned char *               w_code; /* an opcode */
typedef unsigned char                 w_opcode;
typedef struct w_String *             w_string;
typedef struct w_Clazz *              w_clazz;
typedef struct w_Exception *          w_exception;
typedef struct w_Triple *             w_triple;
typedef struct w_Dispatcher *         w_dispatcher;
typedef struct w_Leaf *               w_leaf;
typedef struct w_Root *               w_root;
typedef struct w_Hashtable *          w_hashtable;
typedef struct w_Hashtable2k *        w_hashtable2k;
typedef struct w_Space *              w_space;
typedef struct w_StreamDriver *       w_streamDriver;
typedef struct w_ByteArrayStream *    w_byteArrayStream;
typedef struct w_BufferedStream *     w_bufferedStream;
typedef struct w_Jnode *              w_jnode;
typedef struct w_Jnode_free *         w_jnode_free;
typedef struct w_Jbuffer    *         w_jbuffer;
typedef struct w_Pobject *            w_pobject;
typedef struct w_Pss *                w_pss;
typedef struct w_Stream *             w_stream;
typedef struct w_Fifo *               w_fifo;
typedef struct w_Message *            w_message;
typedef struct w_Pipe *               w_pipe;
typedef struct w_ClassData *          w_classData;
typedef struct w_Repository *         w_repository;
typedef struct w_Rentry *             w_rentry;
typedef struct w_SHA *                w_sha;

/*
** This one is also defined in core-classes.h, but to eliminate the
** need to include core-classes.h in every file that uses
** it, we define it here also.
*/

#ifndef WS_SUCCESS
#define WS_SUCCESS 0
#endif

/*
** The access flags for methods, fields and classes and that type that
** will hold them. Note that w_flags is of size int, while in the
** JVM standard book it is only 'short' (u2) in size. We do this so that
** all structures are on word boundaries and we use the extra flag space
** for our own flags.
*/

typedef unsigned int                w_flags;

/*
** The official flags...
*/

#define ACC_PUBLIC                  0x00000001
#define ACC_PRIVATE                 0x00000002
#define ACC_PROTECTED               0x00000004
#define ACC_STATIC                  0x00000008
#define ACC_FINAL                   0x00000010
#define ACC_SYNCHRONIZED            0x00000020
#define ACC_SUPER                   0x00000020
#define ACC_VOLATILE                0x00000040
#define ACC_TRANSIENT               0x00000080
#define ACC_NATIVE                  0x00000100
#define ACC_INTERFACE               0x00000200
#define ACC_ABSTRACT                0x00000400
#define ACC_STRICT                  0x00000800
#define ACC_FLAGS                   0x00000fff

/*
** Our own Wonka flags...

#define ACC_PRIMITIVE               0x00001000
#define ACC_DEPRECATED              0x00002000
#define ACC_SYNTHETIC               0x00004000
*/

/*
** The constant type `tag' and the values it can take.
*/
typedef unsigned int                w_tag;

#define CONSTANT_UTF8               0x00000001
#define CONSTANT_UNICODE            0x00000002
#define CONSTANT_INTEGER            0x00000003
#define CONSTANT_FLOAT              0x00000004
#define CONSTANT_LONG               0x00000005
#define CONSTANT_DOUBLE             0x00000006
#define CONSTANT_CLASS              0x00000007
#define CONSTANT_STRING             0x00000008
#define CONSTANT_FIELD              0x00000009
#define CONSTANT_METHOD             0x0000000A
#define CONSTANT_IMETHOD            0x0000000B
#define CONSTANT_NAME_AND_TYPE      0x0000000C
#define CONSTANT_NOT_RESOLVED       0x00000010
#define NO_VALID_ENTRY              0x00000020

/*
** An entry in our constant pool array.
*/
typedef struct w_Cnt               *w_cnt;

/*
** A table with mapping of class (static) names and the fixup function
** for this class; each mapping ends with a { NULL, NULL }
*/

typedef void (*w_fixup)(w_clazz);

/*
typedef struct w_wtable {
  const char *chars;
  w_fixup fixup;
} w_wtable;
    
*/
/*
** The function pointer declarations for native methods.
**
**   w_word = w_word_fun(void) a pointer to a function that returns a w_word;
**                             we declare that it doesn't take any arguments 
**                             since we will build the stack ourselves.
**   w_long = w_long_fun(void) a pointer to a function that returns a w_long;
**            w_void_fun(void) a pointer to a function that returns void; 
**                             again, with this function, we will build the 
**                             stack of arguments ourselves.
** w_fun_dec(w_word this, ...) the function pointer declaration that is 
**                             used in the registerMethod macro defined in 
**                             method.h. We defy the checking mechanism of 
**                             the C compiler, with this hack, but we know 
**                             what we're doing, right? right...
**
** These types are combined in the union w_Function that will be the type
** of a 'generic' native function pointer.
**
** Functions that return a value will have this result automatically copied
** to the callers stack top, as with normal JVM semantics.
*/

typedef w_word  (*w_word_fun)(void);
typedef w_long  (*w_long_fun)(void);
typedef void    (*w_void_fun)(void);
typedef void    (*w_fun_dec)(w_word this, ...);

union w_Function {
  w_word_fun word_fun;
  w_long_fun long_fun;
  w_void_fun void_fun;
};

typedef union w_Function            w_function;


/*
** Wonka RTOS related stuff.
*/

typedef struct w_Thread            *w_thread;
typedef struct w_Group             *w_group;
typedef struct w_Realm             *w_realm;
typedef unsigned long               w_time;

/*
** The Primitive type tags. Note that some descriptors are set to a number
** that corresponds with the 'newarray' bytecode.
*/

typedef enum {
  P_boolean     =  4,
  P_char        =  5,
  P_float       =  6,
  P_double      =  7,
  P_byte        =  8,
  P_short       =  9,
  P_int         = 10,
  P_long        = 11,
  P_void        = 12,
  P_Object      = 13
} P_Type;

/*
** The different types of descriptors. Note that the numbers follow the numbers of the 
** P_Type table so that we can easily make a LUTS out of it for sizeInBits and sizeInWords.
** See the file descriptor.h for the macros...
*/

typedef enum {
  D_Primitive   =  0,
  D_Class       = 14,
  D_Array       = 15,
  D_Method      = 16
} D_Type;

/*
** General utility functions
*/

void startNatives(void);
void startPlatform(void);

extern w_int bytesInUse;

void startWonka(void*);

extern w_clazz firstClazz;
extern w_clazz lastClazz;

#define NULL_INSTANCE_IS_NULL

/*
** Java 'null' is represented by (w_instance)0, which is equivalent
** to NULL when used as a pointer.
*/
#define nullInstance ((w_instance)0)
#define isNullInstance(i) (!(i))

/*
** The systemClassLoader is an instance to a kernel class loader.
*/

extern w_instance                    SystemClassLoader;

/*
** A special clazz that we use to clone array clazzes from. It inherits all clazzObject
** stuff and replaces the 'clone' method slot and adds 2 interfaces: Cloneable and java.io.Serializable
*/

extern w_clazz                       clazz_Array;

void reportMem(void);

/*
** PutString outputs a null-terminated string to the debug device.
** PutHex and PutDec output a number (in hex or decimal respectively),
** to the given number of places.
*/

void PutString(char *s);
void PutHex (unsigned int Value, unsigned int Width);
void PutDec (unsigned int Value, unsigned int Width);

void setAllTriggerLevel(int trigger);
void _setTriggerLevel(const char *file, int trigger);

/*
** Logging & debugging stuff.
*/

#include "debug.h"
#include <stdarg.h>

#ifdef FICL

#define FICL_PORTNUM 0;

w_int smartBoot(void);
void smartMon(void);
void smartAbort(void);

#define SMARTBOOT_COLD_BOOT 0
#define SMARTBOOT_WARM_BOOT 1

#endif

/*
** The area of memory at *wonkaPointers holds pointers to some key data 
** structures for debugging or warm-booting:
**  - first_hashtable is an entry into the doubly-linked circular list of
**    all hashtables;
**    list of all repositories.
** The field bootmode holds either SMARTBOOT_COLD_BOOT or SMARTBOOT_WARM_BOOT.
**
** wonkaPointers is initalized during boot-up to point to an area of "static"
** memory.  This area is *not* automatically zeroed, so the initialization
** code can check the existing contents for plausibility to see if a Wonka
** system is already loaded. 
**
** N.B. This structure is shadowed by the FICL object class w-pointers.
** If you change this definition then you need to update smartAss.fr also.
*/

typedef struct w_Pointers {
  w_int        bootmode;
  w_hashtable  first_hashtable;
  char        *PS_start;
  char        *PS_end;
  void        *ficlVM;
  void        *dp;
  void        *envp;
  void        *localp;
  void        *vmList;
} w_Pointers;

extern w_Pointers *wonkaPointers;
w_ushort CCITT_16(w_ubyte *data, w_size length);
w_word CCITT_32(w_ubyte *data, w_size length);
w_word update_ISO3309_CRC(w_word crc, w_ubyte *buf, w_size len);
w_void make_ISO3309_CRC_table(w_void);
w_int wonka_vsnprintf(char * buf, w_int bufsize, const char *fmt, va_list args);
w_int wonka_snprintf(char * buf, w_int bufsize, const char *fmt, ...);

#ifndef DISABLE_GC
#ifndef ENABLE_GC
#define ENABLE_GC
#endif
#endif

#ifdef ENABLE_PS
#define PS_PHY_SIZE (4 * 1024 * 1024)
#define PS_LOG_SIZE PS_PHY_SIZE
#else
#define PS_PHY_SIZE 0
#define PS_LOG_SIZE 0
#ifndef DISABLE_PS
#define DISABLE_PS
#endif
#endif

char *threadDescription(w_thread t);

extern const char *banner;

/*
** Some utility structures to handle 64 bit values in a somewhat CPU independent way.
**
** We represent the bytes, shorts and words in a 64 or 32 bit variable as follows
**
**        +-----------------------------------+-----------------------------------+
** bit #  | 63 ............................32 | 31............................. 0 |
** bits   | MSB ............................. | ............................. LSB |
**        +-----------------------------------+-----------------------------------+
** words  |              WORD_MSW             |             WORD_LSW              |
**        +-----------------+-----------------+-----------------+-----------------+
** shorts |     SHORT_3     |     SHORT_2     |     SHORT_1     |    SHORT_0      |
**        +--------+--------+--------+--------+--------+--------+--------+--------+
** bytes  | BYTE_7 | BYTE_6 | BYTE_5 | BYTE_4 | BYTE_3 | BYTE_2 | BYTE_1 | BYTE_0 |
**        +--------+--------+--------+--------+--------+--------+--------+--------+
*/

typedef union w_u64 {
  w_ulong u64;
  w_long  s64;
  w_ubyte ubytes[8];
  w_ushort ushorts[4];
  w_word words[2];
} w_u64;

typedef union w_u32 {
  w_word u32;
  w_int  s32;
  w_ubyte ubytes[4];
  w_ubyte u08[4];
  w_ushort ushorts[2];
} w_u32;

typedef union w_u16 {
  w_ushort u16;
  w_short  s16;
  w_ubyte u8[2];
} w_u16;

typedef union {
  w_ubyte    Byte;
  w_char     Char;
  w_float    Float;
  w_int      Int;
  w_short    Short;
  w_boolean  Boolean;
  w_instance Reference;
  w_double   Double;
  w_long     Long;
  w_u64      u64;         /* To make it easier to manipulate MSW and LSW and different CPU's */
} w_value;

/*
** Check if either LITTLE_ENDIAN or BIG_ENDIAN is defined, or both.
*/

#ifdef LITTLE_ENDIAN 
#ifdef BIG_ENDIAN
#error "Little endian and big endian is defined !! Please select only 1 in processor.h Preferably, the correct one..."
#endif
#endif

#ifndef LITTLE_ENDIAN
#ifndef BIG_ENDIAN
#error "Either big endian or little endian should be defined !! Please supply the correct one in processor.h..."
#endif
#endif

/*
** Indexes for LITTLE endian CPU's
*/

#ifdef LITTLE_ENDIAN

#define WORD_MSW                       1
#define WORD_LSW                       0

#define BYTE_0                         0
#define BYTE_1                         1
#define BYTE_2                         2
#define BYTE_3                         3
#define BYTE_4                         4
#define BYTE_5                         5
#define BYTE_6                         6
#define BYTE_7                         7

#define SHORT_0                        0
#define SHORT_1                        1
#define SHORT_2                        2
#define SHORT_3                        3

#endif

/*
** Indexes for BIG endian CPU's
*/

#ifdef BIG_ENDIAN

#define WORD_MSW                       0
#define WORD_LSW                       1

#define BYTE_0                         7
#define BYTE_1                         6
#define BYTE_2                         5
#define BYTE_3                         4
#define BYTE_4                         3
#define BYTE_5                         2
#define BYTE_6                         1
#define BYTE_7                         0

#define SHORT_0                        3
#define SHORT_1                        2
#define SHORT_2                        1
#define SHORT_3                        0

#endif /* LITTLE_ENDIAN or BIG_ENDIAN */

extern void (*puts_vector)(char *s);
extern w_boolean memoryAddressIsValid(w_size addr);

int strcmp(const char *p1, const char *p2);

#endif /* _WONKA_H */
