/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

package com.acunia.resource;

import java.util.ResourceBundle;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Calendar;
import java.util.TimeZone;
import java.util.SimpleTimeZone;
import java.util.MissingResourceException;

public class TimeZoneResourceBundle extends ResourceBundle {


  private static Hashtable timeZones;	

  private synchronized static void createTimeZones(){
    if(timeZones == null){
      timeZones = new Hashtable(THETIMEZONES.length*2);
      for(int k = 0 ; k < THETIMEZONES.length ; k++){
        timeZones.put(THETIMEZONES[k].getID(),THETIMEZONES[k]);
      }
    }
  }

  private static final TimeZone[] THETIMEZONES = {
    new SimpleTimeZone(0,"GMT"),
    new SimpleTimeZone(0,"WET",Calendar.MARCH, -1, Calendar.SUNDAY, 1000*3600,
                    Calendar.OCTOBER, -1, Calendar.SUNDAY, 1000*3600, 1000*3600),
    new SimpleTimeZone(1000*3600,"ECT",Calendar.MARCH, -1, Calendar.SUNDAY,  2000*3600,
          					Calendar.OCTOBER, -1, Calendar.SUNDAY,  2000*3600, 1000*3600),
    new SimpleTimeZone(-8000*3600,"PST",Calendar.APRIL, 1, Calendar.SUNDAY,  2000*3600,
          					Calendar.OCTOBER, -1, Calendar.SUNDAY,  2000*3600, 1000*3600),
    new SimpleTimeZone(-11000*3600,"MIT",Calendar.APRIL, 1, Calendar.SUNDAY,  2000*3600,
          					Calendar.OCTOBER, -1, Calendar.SUNDAY,  2000*3600, 1000*3600),
    new SimpleTimeZone(-10000*3600,"HST",Calendar.APRIL, 1, Calendar.SUNDAY,  2000*3600,
          					Calendar.OCTOBER, -1, Calendar.SUNDAY,  2000*3600, 1000*3600),
    new SimpleTimeZone(-9000*3600,"AST",Calendar.APRIL, 1, Calendar.SUNDAY,  2000*3600,
          					Calendar.OCTOBER, -1, Calendar.SUNDAY,  2000*3600, 1000*3600),
    new SimpleTimeZone(-7000*3600,"MST",Calendar.APRIL, 1, Calendar.SUNDAY,  2000*3600,
          					Calendar.OCTOBER, -1, Calendar.SUNDAY,  2000*3600, 1000*3600),
    new SimpleTimeZone(-6000*3600,"CST",Calendar.APRIL, 1, Calendar.SUNDAY,  2000*3600,
          					Calendar.OCTOBER, -1, Calendar.SUNDAY,  2000*3600, 1000*3600),
    new SimpleTimeZone(-5000*3600,"EST",Calendar.APRIL, 1, Calendar.SUNDAY,  2000*3600,
          					Calendar.OCTOBER, -1, Calendar.SUNDAY,  2000*3600, 1000*3600),
    new SimpleTimeZone(-3500*3600,"NST",Calendar.APRIL, 1, Calendar.SUNDAY,  2000*3600,
          					Calendar.OCTOBER, -1, Calendar.SUNDAY,  2000*3600, 1000*3600),
    new SimpleTimeZone(8000*3600,"CTT",Calendar.APRIL, 1, Calendar.SUNDAY,  2000*3600,
          					Calendar.OCTOBER, -1, Calendar.SUNDAY,  2000*3600, 1000*3600),
    new SimpleTimeZone(9000*3600,"JST",Calendar.APRIL, 1, Calendar.SUNDAY,  2000*3600,
          					Calendar.OCTOBER, -1, Calendar.SUNDAY,  2000*3600, 1000*3600),
    new SimpleTimeZone(-7000*3600,"PNT"),
    new SimpleTimeZone(-5000*3600,"IET"),
    new SimpleTimeZone(-4000*3600,"PRT"),
    new SimpleTimeZone(-3500*3600,"CNT"),
    new SimpleTimeZone(-3000*3600,"AGT"),
    new SimpleTimeZone(-3000*3600,"BET"),
    new SimpleTimeZone(-1000*3600,"CAT"),
    new SimpleTimeZone( 1000*3600,"EET"),
    new SimpleTimeZone( 2000*3600,"ART"),
    new SimpleTimeZone( 3000*3600,"EAT"),
    new SimpleTimeZone( 3500*3600,"MET"),
    new SimpleTimeZone( 4000*3600,"NET"),
    new SimpleTimeZone( 5000*3600,"PLT"),
    new SimpleTimeZone( 5500*3600,"IST"),
    new SimpleTimeZone( 6000*3600,"BST"),
    new SimpleTimeZone( 7000*3600,"VST"),
    new SimpleTimeZone( 8000*3600,"CTT"),
    new SimpleTimeZone( 9500*3600,"ACT"),
    new SimpleTimeZone(10000*3600,"AET"),
    new SimpleTimeZone(11000*3600,"SST"),
    new SimpleTimeZone(12000*3600,"NST"),         					
  };


  /**
  ** the constructor build a hashtable with String keys and TimeZoneResource Objects as value <br>
  ** the keys are the TimeZoneIDs and the TimeZoneResource can be used to create (Simple)TimeZone Objects
  **
  */
  public TimeZoneResourceBundle() {
    if(timeZones == null){
      createTimeZones();
    }
  }

  //required implementation of abstract methods of ResourceBundle
  protected Object handleGetObject(String key) throws MissingResourceException {
   	Object o = timeZones.get(key);
   	if (o != null) {
   		return o;
   	}
   	throw new MissingResourceException("Oops, resource not found","TimeZoneResourceBundle","key");
  }

  public Enumeration getKeys() {
  	return timeZones.keys();
  }


// methods for easy use ( TimeZone OBJECT )
  /**
  ** this method is called by the static method getAvailableIDs() of TimeZone <br>
  ** MAKE SURE THIS METHOD STAYS AVAILABLE !!!
  */
  public String[] getKeysArray() {
  	int length = timeZones.size();
  	String [] keys = new String[length];
  	Enumeration e = timeZones.keys();
  	int i=0;
  	while (e.hasMoreElements()) {
  		keys[i++] = (String) e.nextElement();
  	}
  	return keys;
  }	

  /**
  ** this method is called by the static method getAvailableIDs(int rawOffset) of TimeZone <br>
  ** MAKE SURE THIS METHOD STAYS AVAILABLE !!!
  */
  public String[] getKeysArray(int rOffset) {
  	int length = timeZones.size();
  	String [] keys = new String[length];
  	Enumeration e = timeZones.keys();
  	int i=0;
  	while (e.hasMoreElements()) {
  	  String s = (String) e.nextElement();
  		if (rOffset == ((SimpleTimeZone)timeZones.get(s)).getRawOffset()){
  			keys[i++] = s;
  		}
  	}
  	String[] okeys = new String[i];
  	System.arraycopy(keys , 0 , okeys , 0 , i);
  	return okeys;
  }	

  /**
  ** this method is called by the static method getTimeZone(String ID) of TimeZone <br>
  ** MAKE SURE THIS METHOD STAYS AVAILABLE !!!
  */
  public TimeZone getTimeZone(String keyID) {
  	SimpleTimeZone tzRes =(SimpleTimeZone) timeZones.get(keyID);
  	if (tzRes == null) {
    	return (TimeZone)timeZones.get("GMT");
  	}
  	return tzRes;
  }
}