/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: CRLFOutputStream.java,v 1.1.1.1 2001/11/21 10:41:34 pollock Exp $
*/

package com.acunia.wonka;

import java.io.OutputStream;
import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;

  /**
   * Class <code>CRLFOutputStream</code> converts linefeed characters
   * (Unicode char 000A) to some sequence such as CRLF.
   */
public class CRLFOutputStream extends FilterOutputStream {

  String  prefix;   // The string to be prefixed to each line (or null)
  boolean started;  // True iff prefix was already printed for this line

  /**
   * The constructor with just an output stream implements the default 
   * mapping LF-->CRLF.
   */
  public CRLFOutputStream(OutputStream out) {
    super(out);
    prefix = null;
    started = false;
  }

  /**
   * The constructor with an output stream and a String can be used 
   * to replace LF by any sequence, for example "\r\nDEBUG:\t".
   */
  public CRLFOutputStream(OutputStream out, String s) {
    super(out);
    prefix = s;
    started = false;
  }

  public void write(int b) throws IOException, NullPointerException {
    if (!started && prefix!=null) {
      write_prefix();
      started = true;
    }
    if ((b&0x7f)==10) {
      out.write(13);
      started = false;
    }
    out.write(b);
  }

   public void write(byte[] b)
     throws IOException, NullPointerException
  {
    write(b,0,b.length);
  }

  public void write(byte[] b, int off, int len)
     throws IOException, NullPointerException
  {
    int i,j,l;

    if (b==null) throw new NullPointerException();

    if (off<0 || len<0 || off+len>b.length) throw new ArrayIndexOutOfBoundsException();

    i = off;
    l = off + len;
    while (i < l) {
      if (!started && prefix!=null) {
        write_prefix();
        started = true;
      }

      for (j=i;j<l;++j) {
        if ((b[j]&0x7f)==10) break;
      }
      if (j<l) {
        if (j-i > 0) {
          out.write(b,i,j-i);
        }
        out.write(13);
        out.write(10);
        started = false;
        ++j;
      }
      else {
        out.write(b,i,j-i);
      }
      i = j;
    }
  }

  private void write_prefix() throws IOException {
    try {
      byte[] p = prefix.getBytes();
      out.write(p);
    } catch (UnsupportedEncodingException uee) {
    }
  }
}

