/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: Init.java,v 1.7 2002/03/27 08:09:43 gray Exp $
*/

package com.acunia.wonka;

import java.io.IOException;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Enumeration;
import java.util.Locale;
import java.util.Properties;
import java.util.TimeZone;
import java.util.Vector;

/**
 ** Class Init is a little odd.  It is a singleton, the only instance of
 ** which is created by the virtual machine itself.  It claims to implement
 ** Runnable, but the run() method is also called by the virtual machine
 ** itself during startup: the thread which does this (SystemInitThread)
 ** is actually the same one that called the constructor.
 */

final class Init implements Runnable {

  private static Init theInit;
  private static boolean SLEEP = false;
  private String arch = System.getProperty("os.arch").toLowerCase(Locale.getDefault());
  private String start_command = getStartCommProp(arch);

  private Method invoke_method;
  private Object[] invoke_args = new Object[1];

  private native String[] getCmdLineProperties();

  private Init(String[] args) {
    if (theInit != null) {
      System.err.println("Init: Attempt to create a second instance foiled.");

      return;

    }
    theInit = this;

  // Kick the Persistent Storage mechanism into existence
    PersistentRoot thePersistentRoot = PersistentRoot.getInstance();

    String start_class_name;
    String [] start_class_args;
    Class start_class = null;
  
    parseCmdLineProperties();
    
    if (args != null && args.length > 0 && args[0] != null) {
      start_class_name = args[0];
      start_class_args = getCommandlineArgs(args);
    }
    else {
      int first_space = start_command.indexOf(' ');
      if (first_space >= 0) {
        start_class_name = start_command.substring(0,first_space);
        start_class_args = getSysPropArgs(start_command.substring(first_space+1));
      }
      else {
        start_class_name = start_command;
        start_class_args = new String[0];
      }
    }
    invoke_args[0] = start_class_args;

    if (start_class_name != null) {
      try {
        start_class = Class.forName(start_class_name, false, ClassLoader.getSystemClassLoader());
        System.out.println("Init: start_class = "+start_class);
      } catch (ClassNotFoundException e) {
        System.err.println("Init: no such class as "+start_class_name+"!");
      }
    }

    if (start_class == null) {
      System.err.println("Init: no start class found.  Game over.");

      return;

    }

    try {
      invoke_method = start_class.getMethod("main",new Class[]{Class.forName("[Ljava.lang.String;")});
      System.out.println("Init: will invoke "+invoke_method+" of "+start_class);
      invoke_method.setAccessible(true);
    } catch (SecurityException e) {
      System.err.println("Init: Couldn't set "+invoke_method+" accessible! got "+e);
    } catch (ClassNotFoundException e) {
      System.err.println("Init: no class [Ljava.lang.String; found! got "+e);
    } catch (NoSuchMethodException e) {
      System.err.println("Init: no main() method found! got "+e);
    }
  }

  public void run() {

  // Install the default SecurityManager if required
    if (SecurityConfiguration.SET_SECURITY_MANAGER) {
      System.out.println("Init: installing SecurityManager");
      if (SecurityConfiguration.USE_ACCESS_CONTROLLER) {
        System.out.println("Init: note that Wonka libraries will not use this SecurityManager, but will call AccessController directly."); 
      }
      else if (!SecurityConfiguration.USE_SECURITY_MANAGER) {
        System.out.println("Init: note that Wonka libraries will not use this SecurityManager (security checks are disabled)."); 
      }
      System.setSecurityManager(new SecurityManager());
    }
  // Start up the Garbage Collector
    System.out.println("Init: starting Garbage Collector");
    GarbageCollector gc = GarbageCollector.getInstance();
  // Start up the Finalizer
    System.out.println("Init: starting Finalizer");
    Finalizer f = Finalizer.getInstance();
  // Set the default timezone to the default default
    TimeZone.setDefault(null);
    System.out.println("Init: default TImeZone is "+TimeZone.getDefault());

    try {
      if (invoke_method == null) {
        System.err.println("Init: no start method found.");
      }
      else {
        System.out.println("Init: starting "+invoke_method+" ...");
        invoke_method.invoke(null,invoke_args);
      }

/*
      Class claudiaClass;
      Object claudiaInstance;
      try {
        claudiaClass = Class.forName("com.acunia.wonka.Claudia");
        claudiaInstance = claudiaClass.newInstance();
      }
      catch (ClassNotFoundException e) {
        System.err.println("Could not find Claudia, sorry.");
      }
      catch (Exception e) {
        e.printStackTrace();
      }
*/

      if (SLEEP) {
        System.out.println("Init: entering infinite sleep() ...");
        try {
          while (true) {
            Thread.sleep(1000000);
          }
        } catch (InterruptedException e) {
          e.printStackTrace();
        }
      }
    } catch (IllegalAccessException e) {
      System.err.println("Init: error invoking "+invoke_method+"! got "+e);
    } catch (InvocationTargetException e) {
      Thread this_thread = Thread.currentThread();
      System.err.println("Init: "+invoke_method+" caught "+e);
      this_thread.getThreadGroup().uncaughtException(this_thread,e.getTargetException());
    }
  }

  private String[] getCommandlineArgs(String[]args) {
    Vector temp_args = new Vector();

    for (int i = 1; i<args.length; ++i) {
      String this_arg = args[i];
      argumentOrSysProp(this_arg,temp_args);
    }
    return marshallArgs(temp_args);
  }

  private String[] getSysPropArgs(String command_line) {
    Vector temp_args = new Vector();
    String[] start_class_args;

    while (command_line != "") {
      int first_space = command_line.indexOf(' ');
      String this_arg;
      if (first_space >= 0) {
        this_arg = command_line.substring(0,first_space);
        command_line = command_line.substring(first_space).trim();
      }
      else {
        this_arg = command_line;
        command_line = "";
      }
      argumentOrSysProp(this_arg,temp_args);
    }
    return marshallArgs(temp_args);
  }

  void argumentOrSysProp(String this_arg, Vector temp_args) {
    if (this_arg.startsWith("-D")) {
      Properties sysprops = System.getProperties();
      String key, value;
      int equals_at = this_arg.indexOf('=');
      if (equals_at < 0) {
        key = this_arg.substring(2);
        System.out.println("Init: Deleting system property '"+key+"'");
        sysprops.remove(key);
      }
      else if (equals_at == 0) {
        System.out.println("Init: Ignoring nameless property in "+this_arg);
      }
      else if (equals_at == this_arg.length()-1) {
        key = this_arg.substring(2,equals_at);
        System.out.println("Init: Deleting system property '"+key+"'");
        sysprops.remove(key);
      }
      else {
        key = this_arg.substring(2, equals_at);
        value = this_arg.substring(equals_at + 1);
        System.out.println("Init: Setting system property '"+key+"' to '"+value+"'");
        sysprops.put(key,value);
      }
    }
    else {
      System.out.println("Init: passing argument"+this_arg);
      temp_args.addElement(this_arg);
    }
  }

  void parseCmdLineProperties() {
    String[] props = getCmdLineProperties();
    Properties sysprops = System.getProperties();
    if(props != null) {
      for(int i=0; i < props.length; i++) {
        String key, value;
        int equals_at = props[i].indexOf('=');
        if (equals_at < 0) {
          key = props[i];
          System.out.println("Init: Deleting system property '"+key+"'");
          sysprops.remove(key);
        }
        else if (equals_at == 0) {
          System.out.println("Init: Ignoring nameless property in "+props[i]);
        }
        else if (equals_at == props[i].length()-1) {
          key = props[i].substring(0,equals_at);
          System.out.println("Init: Deleting system property '"+key+"'");
          sysprops.remove(key);
        }
        else {
          key = props[i].substring(0, equals_at);
          value = props[i].substring(equals_at + 1);
          System.out.println("Init: Setting system property '"+key+"' to '"+value+"'");
          sysprops.put(key,value);
        }
      }
    } 
  }

  String[] marshallArgs(Vector temp_args) {
    String[] start_class_args = new String[temp_args.size()];

    for (int i = 0; i < temp_args.size(); ++i) {
      start_class_args[i] = (String)temp_args.elementAt(i);
    }

    return start_class_args;
  }

  String getStartCommProp(String arch) {
    String arch1, arch2;

    arch1 = "linux";
    arch2 = null;

    System.out.println("Init: looking up System property com.acunia.wonka.Init."+arch1+".start");
    String command = System.getProperty("com.acunia.wonka.Init."+arch1+".start");
    if (command == null && arch2 != null) {
      System.out.println("Init: looking up System property com.acunia.wonka.Init."+arch2+".start");
      command = System.getProperty("com.acunia.wonka.Init."+arch2+".start");
    }
    if (command == null) {
      System.err.println("Init: no start command found.  Game over.");

      return null;

    }

    return command.trim();

  }

}

