/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: PersistentRoot.java,v 1.2 2001/12/19 13:23:22 gray Exp $
*/

package com.acunia.wonka;

import com.acunia.wonka.Etc;
import java.util.*;

public class PersistentRoot implements PersistentStore, Runnable {

  /*
  ** PersistentRoot is a singleton: its sole instance is S.
  */

  private static PersistentRoot S;

  public static native boolean havePOS();

  public static synchronized PersistentRoot getInstance() {
    if (havePOS()) {
      if (S == null) {
        S = new PersistentRoot();
      }

      return S;
    }
    else {
      return null;
    }
  }

  private Hashtable contents;

  /*
  ** create() creates the persistent counterpart of S.
  */

  private native void create();

  private native void sync();

  private native boolean retrieve();

  private PersistentRoot() {
    Thread daemon = new Thread(this,"PersistentStorage");
    create();
    if (contents == null) {
      contents = new Hashtable();
      Etc.woempa(7,"Allocated new Hashtable "+contents);
    }
    daemon.setDaemon(true);
    daemon.start();
  }

  public synchronized void createPersistentObject(String name, Object o) throws SecurityException {
// TODO: security check
    Etc.woempa(7,"Putting {"+name+"="+o+"} into "+contents);
    contents.put(name,o);
    Etc.woempa(7,"Contents now: "+contents);
  }

  public synchronized Object restorePersistentObject(String name) throws SecurityException {
// TODO: security check
    Etc.woempa(7,"Getting {"+name+"} from "+contents);
    return contents.get(name);
  }

  public Object deletePersistentObject(String name) throws SecurityException {
// TODO: security check
    Etc.woempa(7,"Removing {"+name+"} from "+contents);
    return contents.remove(name);
  }

  public void run() {
    synchronized(this) {
      while (true) {
        try {
          this.wait(10000);
          this.sync();
        }
        catch (InterruptedException e) {
        }
      }
    }
  }

}
