/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: BufferedOutputStream.java,v 1.1.1.1 2001/11/21 10:41:34 pollock Exp $
*/

package java.io;

public class BufferedOutputStream extends FilterOutputStream {

  protected byte[] buf;
  protected int count;

  public BufferedOutputStream(OutputStream out) {
    this(out, 512);
  }

  public BufferedOutputStream(OutputStream out, int size) {
    super(out);
    if (size < 1 ) throw new IllegalArgumentException("size should be bigger than 0");
    buf = new byte[size];
    count = 0;
  }

  public void write(int b) throws IOException, NullPointerException {
    if (count == buf.length) {
     	out.write(buf,0,count);
    	count=0;
    }
    buf[count++] = (byte)b;

  }

// this method is inherited from FilterOutputStream !!!
/*
   public void write(byte[] b)
     throws IOException, NullPointerException
  {
    if (b==null) throw new NullPointerException();

    int soFar = 0;
    int left = b.length;
    int thisTime;

    while(count+left>=buf.length) {
      thisTime = buf.length-count;
      System.arraycopy(b, soFar, buf, count, thisTime);
      soFar = soFar + thisTime;
      left = left - thisTime;
      count = buf.length;
      flush();
    }

    System.arraycopy(b, soFar, buf, count, left);
    count += left;
    if(count==buf.length) flush();
  }
*/
  public void write(byte[] b, int off, int len)
     throws IOException, NullPointerException
  {
    if (off+len > b.length || off < 0 || len < 0) throw new ArrayIndexOutOfBoundsException();

    if (len+count > buf.length) {
    	out.write(buf,0,count);
    	count=0;
    	out.write(b,off,len);
    }
    else {
    	System.arraycopy(b, off, buf, count, len);
    	count += len;
    }
  }

  public void flush() throws IOException {   
    out.write(buf, 0, count);
    count = 0;
    out.flush();
  }

// this method is inherited from FilterOutputStream !!!
/*
  public void close() throws IOException {
    flush();
    out.close();
  }
*/
}

