/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: DataInputStream.java,v 1.2 2002/02/18 14:12:07 ruelens Exp $
*/

package java.io;

public class DataInputStream extends FilterInputStream implements DataInput {

  public DataInputStream(InputStream input) {
    super(input);
  }

  public final void readFully(byte[] b) throws IOException {
    readFully(b, 0, b.length);
  }

  public final void readFully(byte[] b, int off, int len) throws IOException {
    if (off < 0 || len < 0 || (off+len) > b.length){
      throw new IndexOutOfBoundsException();
    }
    int done = 0;
    while(done < len){
      int read = in.read(b, off+done, len-done);
      if(read < 0){
  	    throw new EOFException();
      }
      done += read;
    }
  }

  public int skipBytes(int n) throws IOException {
    int done = 0;
    while(done < n){
      int skipped = (int)in.skip(n-done);
      if(skipped == 0){
  	    throw new EOFException();
      }
      done += skipped;
    }
    return done;
  }

  public boolean readBoolean() throws IOException {
    int next = in.read();
    if (next == -1){
        throw new EOFException();
    }
    return (next > 0);
  }

  public byte readByte() throws IOException {
    int next = in.read();
    if (next == -1){
      throw new EOFException();
    }
    return (byte)next;
  }

  public int readUnsignedByte() throws IOException {
    int next = in.read();
    if (next == -1){
      throw new EOFException();
    }
    return next;
  }

  public short readShort() throws IOException {
    return (short)readUnsignedShort();
  }

  public int readUnsignedShort() throws IOException {
    int result = 0;
    for(int i=0; i<2; i++) {
      int next = in.read();
      if(next == -1){
  	    throw new EOFException();
      }
      result = (result << 8) | (next & 0xFF);
    }
    return result;
  }

  public char readChar() throws IOException {
    return (char) readUnsignedShort();
  }

  public int readInt() throws IOException {
    int result = 0;
    for( int i=0; i<4; i++) {
      int next = in.read();
      if (next == -1){
  	    throw new EOFException();
      }
      result = (result << 8) | (((char)next) & 0xFF);
    }
    return result;
  }

  public long readLong() throws IOException {
    long result = 0;
    for( int i=0; i<8; i++) {
      int next = in.read();
      if (next == -1){
  	    throw new EOFException();
      }
      result = (result << 8) | (next & 0xFF);
    }
    return result;
  }

  public float readFloat() throws IOException {
    return Float.intBitsToFloat(readInt());
  }

  public double readDouble() throws IOException {
    return Double.longBitsToDouble(readLong());
  }

  public String readLine() throws IOException {
    StringBuffer result = new StringBuffer();
    while(true) {
      int next = in.read();
      if (next == -1){
  	    break;
      }
      if (next == '\r' || next == '\n'){
  	    break;
      }	
      result.append((char)(next & 0xff));
    }
    return result.toString();
  }

  public String readUTF() throws IOException {
    int UTFlength = readUnsignedShort();
    if (UTFlength < 0){
      throw new UTFDataFormatException();
    }
    byte[] bytes = new byte[UTFlength];
    readFully(bytes);
    return new String(bytes,"UTF8");
  }
}
