/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


package java.io;

import com.acunia.wonka.SecurityConfiguration;
import java.security.AccessController;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.StringTokenizer;

public class File implements Comparable{

  public static final String separator;
  public static final char   separatorChar;
  public static final String pathSeparator;
  public static final char   pathSeparatorChar;

  private static String current_working_dir = null;
  private static String current_root_dir = null;

  private String     prefix;     
  private String     dirpath;   
  private boolean    absolute;  
  private String     filename;  
  private String     fullname;
  private String     absname;  // Absolute filesystem path = ./blah/blah, only for internal use
                               // The . is needed to get a relative path inside fsroot

  private ByteArrayOutputStream baos;
  private byte[] b;

  private static native String get_CWD();
  private static native String get_root();

  static {
    int i = 0;
    
    current_working_dir = get_CWD();
    current_root_dir = get_root();

    /*
    ** Substract the root path from the current path.
    */

    while(i < current_root_dir.length() && 
          i < current_working_dir.length() && 
          current_working_dir.charAt(i) == current_root_dir.charAt(i)) i++;
   
    if(i == current_root_dir.length()) {
      
      /*
      **  The current working directory is in the current root path.
      */

      current_working_dir = "/" + current_working_dir.substring(i);
    
    }
    else {

      /*
      ** The current working directory is outside the current root path.
      ** Change it to /
      */

      current_working_dir = "/";
    }
    com.acunia.wonka.Etc.woempa(10, "current_working_dir = " + current_working_dir);
    
    separator = GetSystemProperty.FILE_SEPARATOR;
    pathSeparator = GetSystemProperty.PATH_SEPARATOR;
    separatorChar = separator.charAt(0);
    pathSeparatorChar = pathSeparator.charAt(0);
  }

  public static File[] listRoots() {
    return new File[] { new File("/") };
  }

  private String pack(String path) {
    String result = "";
    LinkedList list = new LinkedList();
    StringTokenizer st = new StringTokenizer(path, "/");

    while (st.hasMoreTokens()) {
      String token = st.nextToken();
      if(!token.equals(".")) { 
        if(!token.equals("..")) {
          list.add(token);
        } else {
          if(list.size() > 0)
            list.removeLast();
        }
      }
    }

    Iterator iter = list.iterator();
    while(iter.hasNext()) 
      result += "/" + (String)iter.next();
      
    if(!path.startsWith("/") && (result.length() > 0)) result = result.substring(1);
    if(path.endsWith("/")) result += "/";

    return result; 
  }

  private String stripSlashes(String path) {
    String result = "";
    StringTokenizer st = new StringTokenizer(path, "/");

    while (st.hasMoreTokens()) {
      result += "/" + st.nextToken();
    }

    if(!path.startsWith("/")) result = result.substring(1);
    if(path.endsWith("/")) result += "/";

    return result; 
  }

  public File(String path) throws NullPointerException {
    if (path==null) throw new NullPointerException();

    String relpath;
    filename = "";

    if(path.length() == 0) path = current_working_dir;

    if (path.charAt(0)==separatorChar) {  // Absolute path
      relpath = path.substring(1);
      absolute = true;
      prefix = "" + separatorChar;
    } else {
      relpath = path;
      absolute = false;
      prefix = "";
    }

    path = stripSlashes(path);

    try {

      int dirpathlen = relpath.lastIndexOf(separatorChar);
      if (dirpathlen<0) {
        dirpath = "";
        filename = relpath;
      } else {
        dirpath = relpath.substring(0,dirpathlen);
        filename = relpath.substring(dirpathlen+1);
      }
    } catch (IndexOutOfBoundsException e) {
      e.printStackTrace();
    }

//    dirpath += prefix;

    fullname = path;
    absname = "." + stripSlashes("/" + (absolute ? "" : current_working_dir + "/") + fullname);

  }

  public File(String dirname, String name) throws NullPointerException {
    this(name == null ? null : (dirname == null ? "" : (dirname.equals("") ? "" : dirname + separatorChar)) + name);
  }

  public File(File dir, String name) throws NullPointerException {
    this((dir == null ? "" : dir.getPath()), name);
  }

  public String toString() {
    return fullname;
  }

  public boolean equals(Object obj) {
    if ((obj != null) && obj instanceof File) {
      return (((File)obj).getPath().equals(getPath()));
    }
    return false;
  }

  public int compareTo(File pathname) {
    return (getPath().compareTo(pathname.getPath()));
  }

  public int compareTo(Object obj) {
    if ((obj != null) && obj instanceof File) {
      return (getAbsolutePath().compareTo(((File)obj).getAbsolutePath()));
    } else { throw new ClassCastException(); }
  }

  public int hashCode() {
    return getPath().hashCode() ^ 1234321;
  }

  public String getName() {
    String result = new String(filename);
    int snip = filename.lastIndexOf((int)separatorChar);
    if (snip >= 0) result = result.substring(snip+1);

    return result;
  }

  public String getPath() {
    return fullname;
  }

  public String getAbsolutePath() {
    return stripSlashes("/" + (absolute ? "" : current_working_dir + "/") + fullname);
  }

  public File getAbsoluteFile() {
    return new File(getAbsolutePath());
  }

  public String getCanonicalPath() throws IOException {
    return pack(getAbsolutePath());
  }

  public File getCanonicalFile() throws IOException {
    return new File(getCanonicalPath());
  }

  public String getParent() {
    return (absolute ? "/" : "") + dirpath;
  }

  public File getParentFile() {
    return new File(getParent());
  }

  public boolean isAbsolute() {
    return absolute;
  }

  public boolean isHidden() {
    return (filename.charAt(0) == '.');
  }
  
  public boolean createNewFile() throws IOException {
    if(!this.exists()) {
      try {
        FileOutputStream fo = new FileOutputStream(this);
        fo.close();
        return true;
      } catch(FileNotFoundException e) { 
      }
    }
    return false;
  }

  public native boolean exists() throws SecurityException;

  public native String[] list() throws SecurityException;

  public String[] list(FilenameFilter filter) throws SecurityException {
    String[] files = this.list();
    int count = 0;
    for(int i=0; i < files.length; i++) 
      if(filter.accept(this.getParentFile(), files[i])) count++;

    String[] result = new String[count];
    count = 0;
    for(int i=0; i < files.length; i++) 
      if(filter.accept(this.getParentFile(), files[i])) {
        result[count] = files[i];
        count++;
      }
    return result;
  }

  public File[] listFiles() throws SecurityException {
    String[] filenames = this.list();
    File[] files = new File[filenames.length];
    for(int i=0; i<filenames.length; i++) 
      files[i] = new File(dirpath, filenames[i]);
    return files;
  }

  public File[] listFiles(FilenameFilter filter) throws SecurityException {
    String[] filenames = this.list(filter);
    File[] files = new File[filenames.length];
    for(int i=0; i<filenames.length; i++) 
      files[i] = new File(dirpath, filenames[i]);
    return files;
  }

  public native boolean canRead() throws SecurityException;

  public native boolean canWrite() throws SecurityException;

  public native boolean isFile() throws SecurityException;

  public native boolean isDirectory() throws SecurityException;

  public native long lastModified() throws SecurityException;

  public boolean setLastModified(long time) throws SecurityException {
    return false;
  }

  public void deleteOnExit() throws SecurityException {
    return;
  }

  public boolean setReadOnly() throws SecurityException {
    return false;
  }

  public native long length() throws SecurityException;

  public native boolean mkdir() throws SecurityException;

  public boolean mkdirs() throws SecurityException {
    boolean result = false;
    if(!this.getAbsolutePath().equals(separator)) {
       getParentFile().mkdirs();
       result = mkdir();
    }
    return result;
  }

  private native boolean rename(String src, String dest);
  
  public boolean renameTo(File dest) throws SecurityException {
    return rename(this.getAbsolutePath(), dest.getAbsolutePath());
  }

  public native boolean delete() throws SecurityException;

  public URL toURL() throws MalformedURLException {
    return new URL("file","",pack(getAbsolutePath()));
  }
  
}
