/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: InputStreamReader.java,v 1.3 2002/02/01 17:20:16 dumon Exp $
*/

package java.io;

public class InputStreamReader extends Reader {

  private InputStream in;
  private String encoding;

  public InputStreamReader(InputStream in) {
    super(in);
    this.in = in;
    encoding = "8859_1";
  }

  public InputStreamReader(InputStream in, String enc) throws UnsupportedEncodingException {
    super(in);
    if(enc != "ISO-8859-1" && enc != "8859_1" && enc != "UTF8"){
      throw new UnsupportedEncodingException("Only 8859_1 & UTF8 is supported, " + enc + " not...");
    }
    encoding = enc;
    this.in = in;
  }

  public void close() throws IOException {
    synchronized(lock){
       if(in != null){
         in.close();
         encoding = null;
         in = null;
       }
    }
  }

  public String getEncoding(){
    return encoding;
  }

  public int read() throws IOException {
    synchronized(lock){
      if(in == null){
        throw new IOException();
      }
      return 0xff & in.read();
    }
  }

  public int read(char[] buf, int off, int len) throws IOException{
    synchronized(lock){
      if(in == null){
        throw new IOException();
      }
      if(buf.length < (off + len) || off < 0 || len < 0){
        throw new ArrayIndexOutOfBoundsException("in InputStreamReader");
      }
      byte[] bytes = new byte[len];
      len = in.read(bytes,0,len);
      for (int i=0 ; i < len ; i++){
        buf[off+i] = (char)(0xff & bytes[i]);
      }
      return len;
    }
  }

  public boolean ready() throws IOException{
    synchronized(lock){
      if(in == null){
        throw new IOException();
      }
      return in.available() > 0;
    }
  }
}
