/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: ObjectOutputStream.java,v 1.2 2002/02/14 09:55:26 ruelens Exp $
*/

//TODO: Check synchronization.

package java.io;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.lang.reflect.Method;
import java.lang.reflect.Field;

public class ObjectOutputStream extends OutputStream implements ObjectOutput, ObjectStreamConstants {

  private static final boolean DEBUG = false;

  private ObjectStreamClass currentObjectStreamClass;

  /** Stream itself. */
  private DataOutputStream dos = null;

  /** Current object being written to the stream. */
  private Object targetObject = null;

  /** Helper, completely dependent on 'targetObject'. */
  private ObjectStreamClass targetOSClass = null;

  // ----------------------------------------------------------------
  //   Buffering blockdata

  /** Maximum buffer size for blockdata is 1024. */
  private static final int BUFFERSIZE = 1024;

  /** Buffer for blockdata. */
  private byte[] blockDataBuffer = new byte[BUFFERSIZE];

  /** Current position to add data into the blockdata buffer. */
  private int bufferPointer = 0;

  // ----------------------------------------------------------------


  // ----------------------------------------------------------------
  //   Handles

  /** Current free handle id. */
  private int handleCounter = baseWireHandle;

  /**
   * Container for all objects that have been written to the stream.
   * They are identified by their handle, an int.
   */
  private Hashtable handles = new Hashtable();


  // ----------------------------------------------------------------

  //TODO: Subclassing support needs to be implemented.
  /**
   * Flag indicating a subclass constructed a stream using the default
   * constructor. Flag is checked in writeObject and if it is true
   * writeObjectOverride is called.
   */
  //TODO: what if a subclass uses ObjectOutputStream(OutputStream)
  //      to construct a new stream?
  private boolean subclassoverrides = false;

  /** Object replacement? */
  private boolean replaceObjectEnabled = false;

  /** Bytes written to stream. */
  private Vector bytesWritten = new Vector();


  //OK
  protected ObjectOutputStream() throws IOException, SecurityException {
    initialize();
    subclassoverrides = true;

    //TODO: IOException - Thrown if not called by a subclass.
    //      SecurityException - if a security manager exists
    //      and its checkPermission method denies enabling subclassing.

    //TODO: ObjectOutputStream()
    throw new RuntimeException("Implementation does not support subclassing.");
  }

  //OK
  public ObjectOutputStream(OutputStream out) throws IOException {
    initialize();

    dos = new DataOutputStream(out);
    writeStreamHeader();
  }

  //OK
  protected void annotateClass(Class cl) throws IOException {
    // Default implementation does nothing. Subclasses can override.
  }

  //OK
  public void close() throws IOException {
    flush();
    dos.close();
  }

  //OK
  public void defaultWriteObject() throws IOException, NotActiveException {
    if (DEBUG) print ("SER: In method 'defaultWriteObject'.");

    Object currentTarget = targetObject;

    //TODO: Write a better check.
    if (currentTarget==null) {
      throw new NotActiveException();
    }

    Field targetField;
    ObjectStreamField[] osfields;
    if (DEBUG) print ("SER: Serializing: " + currentObjectStreamClass.getName());
    osfields = currentObjectStreamClass.getMyFields();
    if (osfields == null) {
      return;
    }
    try {
      for (int j = 0; j < osfields.length; j++) {
        if (DEBUG) print ("SER: Doing field " + j);
        targetField = osfields[j].getRefField();
        if (DEBUG) print ("SER: Target is " + targetField);
        if (osfields[j].isPrimitive()) {
          if (DEBUG) print ("SER: Type is primitive.");
          switch (osfields[j].getTypeCode()) {
            case 'B':
              writeByteNow(targetField.getByte(currentTarget));
              break;
            case 'C':
              writeCharNow(targetField.getChar(currentTarget));
              break;
            case 'D':
              writeDoubleNow(targetField.getDouble(currentTarget));
              break;
            case 'F':
              writeFloatNow(targetField.getFloat(currentTarget));
              break;
            case 'I':
              writeIntNow(targetField.getInt(currentTarget));
              break;
            case 'J':
              writeLongNow(targetField.getLong(currentTarget));
              break;
            case 'S':
              writeShortNow(targetField.getShort(currentTarget));
              break;
            case 'Z':
              writeBooleanNow(targetField.getBoolean(currentTarget));
              break;
          }
        }
        else {
          // array or primitive type, should always call writeObject
          if (DEBUG) print ( "SER: Object with currentTarget is " +currentTarget
                             + " of class " + currentTarget.getClass() );
          writeObject(targetField.get(currentTarget));
        }
      }
    }
    catch (IllegalAccessException exc) {
      //TODO: What can we do here?
      error("Problem in defaultWriteObject", exc);
    }

  }

  //OK
  public void drain() throws IOException {
    if (bufferPointer == 0) {
      return;
    }

    if (bufferPointer < 256) {
      writeByteNow (ObjectStreamConstants.TC_BLOCKDATA);
      writeByteNow ((byte)bufferPointer);
    }
    else {
      writeByteNow (ObjectStreamConstants.TC_BLOCKDATALONG);
      writeIntNow  (bufferPointer);
    }

    writeNow (blockDataBuffer,0,bufferPointer);
    bufferPointer=0;
  }

  //OK
  protected boolean enableReplaceObject(boolean enable) throws SecurityException {
    boolean oldValue = this.replaceObjectEnabled;

    //TODO: SecurityException - if a security manager exists
    //      and its checkPermission method denies enabling the stream to
    //      do replacement of objects in the stream.

    replaceObjectEnabled = enable;
    return oldValue;
  }

  //OK
  public void flush() throws IOException {
    drain();
    dos.flush();
  }

  //OK
  protected Object replaceObject(Object obj) throws IOException {
    //TODO: replaceObject(Object obj)
    throw new RuntimeException("Implementation does not support object replacement.");
    }

  // Can this be called by writeObject if there is a problem.
  public void reset() throws IOException {
    flush();
    writeByteNow (TC_RESET);
    flush();

    initialize();
  }

  //OK
  public void write(int data) throws IOException {
    writeBuffer((byte)(data >>> 0));
  }

  //OK
  public void write(byte[] b) throws IOException, NullPointerException {
    write (b, 0, b.length);
  }

  //OK
  public void write(byte[] b, int off, int len)
      throws IOException, NullPointerException, ArrayIndexOutOfBoundsException {
    writeBuffer (b, off, len);
  }

  //OK
  public void writeBoolean(boolean data) throws IOException {
    writeBuffer( data ? (byte)0x01 : (byte)0x00 );
  }

  //OK
  public void writeByte(int data) throws IOException {
    writeBuffer((byte)(data >>> 0));
  }

  //OK
  public void writeBytes(String data) throws IOException {
    write(data.getBytes());
  }

  //OK
  public void writeChars(String data) throws IOException {
    for (int i = 0; i < data.length(); i++) {
      writeChar (data.charAt(i));
    }
  }

  //OK
  public void writeChar(int data) throws IOException {
    writeBuffer((byte)(data >>> 8));
    writeBuffer((byte)(data >>> 0));
  }

  //OK
  public void writeDouble(double data) throws IOException {
    writeLong (Double.doubleToLongBits(data));
  }

  //OK
  public void writeFloat(float data) throws IOException {
    writeInt (Float.floatToIntBits(data));
  }

  //OK
  public void writeInt(int data) throws IOException {
    writeBuffer((byte)(data >>> 24));
    writeBuffer((byte)(data >>> 16));
    writeBuffer((byte)(data >>> 8));
    writeBuffer((byte)(data >>> 0));
  }

  //OK
  public void writeLong(long data) throws IOException {
    writeBuffer((byte)(data >>> 56));
    writeBuffer((byte)(data >>> 48));
    writeBuffer((byte)(data >>> 40));
    writeBuffer((byte)(data >>> 32));
    writeBuffer((byte)(data >>> 24));
    writeBuffer((byte)(data >>> 16));
    writeBuffer((byte)(data >>> 8));
    writeBuffer((byte)(data >>> 0));
  }

  //OK
  public void writeShort(int data) throws IOException {
    writeBuffer((byte)(data >>> 8));
    writeBuffer((byte)(data >>> 0));
  }

  //OK
  protected void writeObjectOverride() throws NotActiveException, IOException {
    // Default implementation does nothing. Subclasses can override.
  }

  //OK
  protected void writeStreamHeader() throws IOException {
    writeShortNow(ObjectStreamConstants.STREAM_MAGIC);
    writeShortNow(ObjectStreamConstants.STREAM_VERSION);
  }

  //OK
  public void writeUTF(String data) throws IOException {
    writeShort(data.length());
    write (data.getBytes());
  }


  // ----------------------------------------------------------------
  //   Helper methods for class.
  // ----------------------------------------------------------------

  private void addByte(byte b) {
    int size = bytesWritten.size();
    if (DEBUG) print ("SER: Byte number " + size + ") " + b);

    bytesWritten.add(new Byte(b));
  }

  private synchronized void assignNewHandle(Object object) {
    handles.put (object, new Integer(handleCounter));
    handleCounter++;
  }

  private void print (String s) {
    if (DEBUG) {
      System.out.println(s);
    }
  }

  private void error (String message) {
    System.out.println(message);
    try {
      this.close();
    }
    catch(IOException ioe){}
    throw new InternalError(message);
  }

  private void error (String message, Throwable error) {
    System.out.println(message);
    error.printStackTrace();
    try {
      this.close();
    }
    catch(IOException ioe){}
    throw new InternalError(message);
  }

  //OK
  private int getHandle(Object obj) {
    Integer answer = (Integer)handles.get(obj);
    return answer == null ? -1 : answer.intValue();
  }

  private void initialize() {
    // Leaves stream and unchanged subclassoverrides.
    // Everything else is reset.

    setTargetObject (null);
    bufferPointer = 0;
    handleCounter = baseWireHandle;

    handles = new Hashtable();

    replaceObjectEnabled = false;

    bytesWritten = new Vector();
  }

  private synchronized void setTargetObject(Object object) {
    if (DEBUG) print ("SER: Setting targetobject to " + object);
    this.targetObject  = object;

    if (object != null) {
      this.targetOSClass = ObjectStreamClass.lookup(targetObject.getClass());
    } else {
      this.targetOSClass = null;
    }
  }

  //OK
  private void writeHandle (int handleID) throws IOException {
    writeByteNow (ObjectStreamConstants.TC_REFERENCE);
    writeIntNow  (handleID);
  }

  private void writeBuffer (byte[] b, int off, int len) throws IOException {
    final int totalBytesToDo = len-off;

    int bytesDone = 0;
    while (bytesDone < totalBytesToDo) {
      // Minimum of 'buffer space' and 'bytes left to do'.
      int totalBytesToCopy = Math.min ( BUFFERSIZE - bufferPointer,
                                       totalBytesToDo - bytesDone );
      System.arraycopy( b, off+bytesDone,
                        this.blockDataBuffer, this.bufferPointer,
                        totalBytesToCopy );

      this.bufferPointer += totalBytesToCopy;
      if (this.bufferPointer == BUFFERSIZE) {
        drain();
      }

      bytesDone += totalBytesToCopy;
    }
  }

  //TODO: Synchronize this?
  //OK
  private void writeBuffer (byte b) throws IOException {
    if (bufferPointer == BUFFERSIZE) {
      drain();
    }

    blockDataBuffer[bufferPointer]=b;
    bufferPointer++;
  }

  //OK
  private void writeBooleanNow (boolean data) throws IOException {
    if (DEBUG) {
      addByte(data ? (byte)0x01 : (byte)0x00);
    }

    dos.writeBoolean(data);
  }

  //OK
  private void writeByteNow (int data) throws IOException {
    if (DEBUG) {
      addByte((byte)data);
    }

    dos.writeByte(data);
  }

  //OK
  private void writeCharNow (int data) throws IOException {
    if (DEBUG) {
      addByte((byte)(data >>> 8));
      addByte((byte)(data >>> 0));
    }

    dos.writeChar(data);
  }

  //OK
  private void writeDoubleNow (double data) throws IOException {
    if (DEBUG) {
      writeLongNow (Double.doubleToLongBits(data));
      return;
    }

    dos.writeDouble(data);
  }

  //OK
  private void writeFloatNow (float data) throws IOException {
    if (DEBUG) {
    int i = Float.floatToIntBits(data);
    System.out.println("Writing float: " + data);
    System.out.println("Writing float: " + i);
    System.out.println((byte)(i >>> 24));
    System.out.println((byte)(i >>> 16));
    System.out.println((byte)(i >>> 8));
    System.out.println((byte)(i >>> 0));
      writeIntNow (i);
      return;
    }

    dos.writeFloat(data);
  }

  //OK
  private void writeIntNow (int data) throws IOException {
    if (DEBUG) {
      addByte((byte)(data >>> 24));
      addByte((byte)(data >>> 16));
      addByte((byte)(data >>> 8));
      addByte((byte)(data >>> 0));
    }

    dos.writeInt(data);
  }

  //OK
  private void writeNow (byte[] bytes, int offset, int length)
    throws IOException {
    if (DEBUG) {
      for (int i = offset; i < offset+length; i++) {
        writeByteNow(bytes[i]);
      }
      return;
    }

    dos.write (bytes, offset, length);
  }

  //OK
  private void writeLongNow (long data) throws IOException {
    if (DEBUG) {
      addByte((byte)(data >>> 56));
      addByte((byte)(data >>> 48));
      addByte((byte)(data >>> 40));
      addByte((byte)(data >>> 32));
      addByte((byte)(data >>> 24));
      addByte((byte)(data >>> 16));
      addByte((byte)(data >>> 8));
      addByte((byte)(data >>> 0));
    }

    dos.writeLong(data);
  }

  //OK
  private void writeShortNow (int data) throws IOException {
    if (DEBUG) {
      addByte((byte)(data >>> 8));
      addByte((byte)(data >>> 0));
    }

    dos.writeShort(data);
  }

  private void writeLongUTFNow (String data) throws IOException {
    writeLong(data.length());
    writeNow(data.getBytes(), 0, data.length());
  }

  //OK
  private void writeUTFNow (String data) throws IOException {
    byte [] bytes = data.getBytes("UTF8");
    writeShortNow(bytes.length);
    writeNow(bytes, 0, bytes.length);
  }

  // ----------------------------------------------------------------
  //   Methods implementing the grammar.
  // ----------------------------------------------------------------

  //OK
  /**
   * classAnnotation: endBlockData
   *                  contents endBlockData
   */
  private void writeClassAnnotation(ObjectStreamClass osc) throws IOException {
    //TODO: Call annotateClass.
    if (DEBUG) print ("SER: TC_ENDBLOCKDATA: method 'writeClassAnnotation'.");
    writeEndBlockData();
  }

  private void writeEndBlockData() throws IOException {
    if (DEBUG) print ("SER: TC_ENDBLOCKDATA: method 'writeEndBlockData'.");
    drain();
    writeByteNow(ObjectStreamConstants.TC_ENDBLOCKDATA);
  }

  //OK
  //TODO: Support for serialization for 1.1 and 1.2.?
  private void writeClassData() throws IOException {
    byte classDescFlag;

    Vector oscs = new Vector();
    ObjectStreamClass current = targetOSClass;
    while (current != null) {
      oscs.add(current);
      current = current.getParentObjectStreamClass();
    }

    for (int i = oscs.size()-1; i >= 0; i--) {
      currentObjectStreamClass = (ObjectStreamClass)oscs.elementAt(i);
      if (DEBUG) print ("SER: Current ObjectStreamClass " + currentObjectStreamClass);
      classDescFlag = currentObjectStreamClass.getClassDescFlag();
      if ( (SC_SERIALIZABLE & classDescFlag) != 0 ) {
        if ( (SC_WRITE_METHOD & classDescFlag) == 0 ) {
          // nowrclass
          defaultWriteObject();
        }
        else {
          // wrclass objectAnnotation
          //TODO: make sure if defaultWriteObject should be called before
          //      invoking writeMethod
  
          //TODO: maybe migrate this method, does not belong in ObjectStreamClass
          currentObjectStreamClass.invokeWriteMethod("writeObject",targetObject,this);
  
          if (DEBUG) print ("SER: End of method 'writeClassData'.");
          this.writeEndBlockData();
        }
      }
      else if ( (SC_EXTERNALIZABLE & classDescFlag) != 0 ) {
        if (DEBUG) print ("SER: Object is externalizable.");
        if ( (SC_BLOCK_DATA & classDescFlag) == 0 ) {
          // externalContents
          //TODO: Finish implementation.
        }
        else {
          // objectAnnotation
          //TODO: Finish implementation. Call writeExternal?
  
          this.writeEndBlockData();
          if (DEBUG) print ("SER: End of method 'writeClassData'.");
        }
      }
      //currentObjectStreamClass = currentObjectStreamClass.getParentObjectStreamClass();
    }
  }

  //OK
  /**
   * classDesc: newClassDesc
   *            nullReference
   *            (ClassDesc)prevObject
   */
  private void writeClassDesc (ObjectStreamClass osc) throws IOException {
    // nullReference
    if (osc == null) {
      writeByteNow(ObjectStreamConstants.TC_NULL);
      return;
    }

    // (ClassDesc)prevObject
    int handle = getHandle(osc);
    if (handle >= baseWireHandle) {
      writeHandle(handle);
      return;
    }

    // newClassDesc
    writeNewClassDesc(osc);
  }

  //OK
  private void writeClassDescFlags(ObjectStreamClass osc)
    throws IOException {
    writeByteNow(osc.getClassDescFlag());
  }

  //OK
  /** classDescInfo: classDescFlags fields classAnnotation superClassDesc */
  private void writeClassDescInfo(ObjectStreamClass osc)
    throws IOException {
    // classDescFlags
    writeClassDescFlags(osc);

    // fields
    writeFields(osc);

    // classAnnotation
    writeClassAnnotation(osc);

    // superClassDesc
    writeSuperClassDesc(osc);
  }

  //OK
  /** fields: (short)<count> fieldDesc[count] */
  private void writeFields(ObjectStreamClass oscData) throws IOException {
    //TODO: getMyFields??? Check for inheritance?
    ObjectStreamField[] osfield = oscData.getMyFields();

    // short<count>
    writeShortNow(osfield.length);

    // fieldDesc[count]
    for (int i = 0; i < osfield.length; i++) {
      writeFieldDesc(osfield[i]);
    }
  }

  //OK
  /**
   * fieldDesc: primitiveDesc
   *            objectDesc
   */
  private void writeFieldDesc (ObjectStreamField osfield) throws IOException{
    if (osfield.isPrimitive()) {
      writePrimitiveDesc(osfield);
    }
    else {
      writeObjectDesc(osfield);
    }
  }

  //OK
  private boolean isPrimitiveArray (Class clazz) {
    // dummy instance for getTypeString()
    ObjectStreamField dummy = new ObjectStreamField("dummy", clazz);
    String typeString = dummy.getTypeString();

    if (typeString.length() >= 2) {
      if (typeString.charAt(0) != '[') {
        return false;
      }

      char type = typeString.charAt(1);
      if ( ( type != 'L') && (type != '[') ) {
        return true;
      }
    }

    return false;
  }

  //OK
  private char getPrimitiveArrayTypeCode(Class clazz) {
    if (!isPrimitiveArray(clazz)) {
      error ("This is not a primitive array, please do check first.");
    }

    // dummy instance for getTypeString()
    ObjectStreamField dummy = new ObjectStreamField("dummy", clazz);
    return (char) dummy.getTypeString().charAt(1);
  }

  //OK
  /**
   * newArray : TC_ARRAY classDesc newHandle (int)<size> values[size]
   */
  private void writeNewArray(Class clazz, Object myArray)
    throws IOException {

    // TC_ARRAY
    writeByteNow(ObjectStreamConstants.TC_ARRAY);

    // classDesc
    Class myType = myArray.getClass();
    writeClassDesc(ObjectStreamClass.lookup(myType));

    // newHandle
    assignNewHandle(myArray);

    // (int)<size> values[size]
    if (isPrimitiveArray(clazz)) {
      switch (getPrimitiveArrayTypeCode(clazz)) {
        // byte array
        case 'B': {
          // (int)<size>
          byte[] byteArray = (byte[]) myArray;
          int myArrayLength = byteArray.length;
          writeIntNow(myArrayLength);

          // values[size]
          for (int i = 0; i < myArrayLength; i++) {
            writeByteNow(byteArray[i]);
          }
          break;
        }
        // char array
        case 'C': {
          // (int)<size>
          char[] charArray = (char[]) myArray;
          int myArrayLength = charArray.length;
          writeIntNow(myArrayLength);

          // values[size]
          for (int i = 0; i < myArrayLength; i++) {
            writeCharNow(charArray[i]);
          }
          break;
        }
        // double array
        case 'D': {
          // (int)<size>
          double[] doubleArray = (double[]) myArray;
          int myArrayLength = doubleArray.length;
          writeIntNow(myArrayLength);

          // values[size]
          for (int i = 0; i < myArrayLength; i++) {
            writeDoubleNow(doubleArray[i]);
          }
          break;
        }
        // float array
        case 'F': {
          // (int)<size>
          float[] floatArray = (float[]) myArray;
          int myArrayLength = floatArray.length;
          writeIntNow(myArrayLength);

          // values[size]
          for (int i = 0; i < myArrayLength; i++) {
            writeFloatNow(floatArray[i]);
          }
          break;
        }
        case 'I': {
          // (int)<size>
          int[] intArray = (int[]) myArray;
          int myArrayLength = intArray.length;
          writeIntNow(myArrayLength);

          // values[size]
          for (int i = 0; i < myArrayLength; i++) {
            writeIntNow(intArray[i]);
          }
          break;
        }
        case 'J': {
          // (int)<size>
          long[] longArray = (long[]) myArray;
          int myArrayLength = longArray.length;
          writeIntNow(myArrayLength);

          // values[size]
          for (int i = 0; i < myArrayLength; i++) {
            writeLongNow(longArray[i]);
          }
          break;
        }
        case 'S': {
          // (int)<size>
          short[] shortArray = (short[]) myArray;
          int myArrayLength = shortArray.length;
          writeIntNow(myArrayLength);

          // values[size]
          for (int i = 0; i < myArrayLength; i++) {
            writeShortNow(shortArray[i]);
          }
          break;
        }
        case 'Z': {
          // (int)<size>
          boolean[] booleanArray = (boolean[]) myArray;
          int myArrayLength = booleanArray.length;
          writeIntNow(myArrayLength);

          // values[size]
          for (int i = 0; i < myArrayLength; i++) {
            writeBooleanNow(booleanArray[i]);
          }
          break;
        }
      }
    }
    else {
      // (int)<size>
      Object[] objectArray = (Object[])myArray;
      int myArrayLength = objectArray.length;
      writeIntNow(myArrayLength);

      // values[size]
      for (int i = 0; i < myArrayLength; i++) {
        writeObject(objectArray[i]);
      }
    }
  }

  //OK
  /** newClass: TC_CLASS classDesc newHandle */
  private void writeNewClass (Class clazz) throws IOException {
    writeByteNow(ObjectStreamConstants.TC_CLASS);

    //TODO: Is this possible?
    //      'ocs == null' means it is not Serializable or a bug.
    //      At the moment 'writeClassDesc' will write a TC_NULL
    ObjectStreamClass osc = ObjectStreamClass.lookup(clazz);
    writeClassDesc(osc);

    assignNewHandle(clazz);
  }

  //OK
  /**
   * newClassDesc:
   *   TC_CLASSDESC className serialVersionUID newHandle classDescInfo
   */
  private void writeNewClassDesc(ObjectStreamClass osc)
    throws IOException {
    // TC_CLASSDESC
    writeByteNow(ObjectStreamConstants.TC_CLASSDESC);

    // className
    writeUTFNow (osc.getName());

    // serialVersionUID
    writeLongNow(osc.getSerialVersionUID());

    // newHandle
    assignNewHandle(osc);

    // classDescInfo
    writeClassDescInfo(osc);
  }

  //OK
  /** newObject: TC_OBJECT classDesc newHandle classData[] */
  private void writeNewObject() throws IOException {
    writeByteNow(ObjectStreamConstants.TC_OBJECT);
    if (DEBUG) print ("SER: In method 'writeNewObject', writing class description.");
    writeClassDesc(this.targetOSClass);
    assignNewHandle(this.targetObject);
    if (DEBUG) print ("SER: In method 'writeNewObject', writing class data.");
    writeClassData();
  }

  //OK
  public final void writeObject(Object obj) throws IOException {
    Object remember = this.targetObject;

    setTargetObject(obj);

    // Subclasses.
    if (subclassoverrides) {
      writeObjectOverride();
      setTargetObject(remember);
      return;
    }

    // Flush buffer.
    drain();

    // 'null' object.
    if (obj == null) {
      writeByteNow(ObjectStreamConstants.TC_NULL);
      setTargetObject(remember);
      return;
    }

    //TODO: page 16 item 4 -> if object has been previously replaced
    //      Not yet supported: see replaceObject(Object obj).

    // Write a handle if the object has already been written to the stream.
    int objectHandle = getHandle(obj);
    if (objectHandle >= baseWireHandle) {
      writeHandle (objectHandle);
      setTargetObject(remember);
      return;
    }

    //TODO: New class?
    // Write a java.lang.Class object to the stream.
    if (obj instanceof Class) {
      writeNewClass((Class)obj);
      setTargetObject(remember);
      return;
    }

    //TODO: Check writeClassDesc usage.
    // Write a java.io.ObjectStreamClass object to the stream.
    if (obj instanceof ObjectStreamClass) {
      writeClassDesc((ObjectStreamClass)obj);
      setTargetObject(remember);
      return;
    }

    //TODO: page 16 item 8 -> if object must be replaced
    //      Not yet supported: see replaceObject(Object obj).
    /* Implement this later (Johan Vos):
      if (obj.class.hasMethod(writeReplace) {
        obj=obj.writeReplace();
      }
      if (enableReplaceObject(uh)) {
        replaceObject(obj);
      }
    */

    // Write a java.lang.String to the stream.
    if (obj instanceof java.lang.String) {
      writeString((String)obj);
      setTargetObject(remember);
      return;
    }

    Class clazz = obj.getClass();
    if (clazz.isArray()) {
      writeNewArray (clazz, obj);
      setTargetObject(remember);
      return;
    }

    if ( !ObjectStreamClass.isSerializable(clazz)
         && !ObjectStreamClass.isExternalizable(clazz) ) {
      throw new NotSerializableException (obj.getClass().getName());
    }

    writeNewObject();
    setTargetObject (remember);
  }

  //OK
  /** objectDesc: obj_typecode fieldName className1 */
  private void writeObjectDesc(ObjectStreamField osfield) throws IOException{
    writeByteNow(osfield.getTypeCode());
    writeUTFNow(osfield.getName());
    //TODO: Should a handle be created here?
    //      Answer: Handle is created in writeString!
    writeString(osfield.getTypeString());
  }

  //OK
  /** primitiveDesc: prim_typecode fieldName */
  private void writePrimitiveDesc(ObjectStreamField osfield) throws IOException {
    writeByteNow(osfield.getTypeCode());
    writeUTFNow(osfield.getName());
  }

  //OK
  private void writeString(String data) throws IOException {
    int handle = getHandle(data);

    if (handle != -1) {
      writeHandle(handle);
    }
    else {
      if (data.length() < 65536) {
        writeByteNow(TC_STRING);
        assignNewHandle(data);
        writeUTFNow(data);
      }
      else {
        writeByteNow(TC_LONGSTRING);
        assignNewHandle(data);
        writeLongUTFNow(data);
      }
    }
  }

  //OK
  private void writeSuperClassDesc(ObjectStreamClass oscData)
    throws IOException {
    //TODO: Check if information on non-serializable superclasses are streamed.
    writeClassDesc(ObjectStreamClass.lookup(oscData.forClass().getSuperclass()));
  }
}
