/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: ObjectStreamClass.java,v 1.3 2002/02/14 09:55:26 ruelens Exp $
*/

package java.io;


import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Comparator;
import java.util.Arrays;
import java.util.Hashtable;
import java.util.Vector;


public class ObjectStreamClass implements Serializable {
  private static final boolean DEBUG = false;

  /** Cache for all ObjectStreamClass instances. */
  private static Hashtable cache = new Hashtable();

  /** Class which is described by this object. */
  private Class myClazz;

  /** Classes suid. */
  private long suid;

  /**
   * ObjectStreamFields representing the fields
   * of the current class only, not its superclasses.
   */
  private ObjectStreamField[] myFields = NO_FIELDS;

  private ObjectStreamClass parentOSClass;

  //OK
  static ObjectStreamField[] NO_FIELDS = new ObjectStreamField[0];

  //OK
  private ObjectStreamClass(Class clazz) {
    this.myClazz = clazz;
    this.suid    = getSuidFor(clazz);

    this.fillFields();

    Class superclazz = clazz.getSuperclass();
    if (superclazz != null) {
      this.parentOSClass = new ObjectStreamClass(superclazz);
    }

    //TODO: Can class be unloaded, reloaded,...?
    //TODO: What if there already was a class in the cache?
    //      Do we need to check serialVersionUIDs?
    cache.put(clazz,this);
  }

  //OK
  private static long getSuidFor (Class clazz) {
    try {
      Field field = clazz.getDeclaredField("serialVersionUID");
      field.setAccessible(true);
      //TODO: Check if 'null' is ok.
      return field.getLong(null);
    }
    catch (NoSuchFieldException e) {
      return calculateSerialVersionUID(clazz);
    }
    catch (IllegalAccessException e) {
      //TODO: Security problem here!
      return 0;
    }
  }

  //OK
  protected static boolean isSerializable (Class clazz) {
    return java.io.Serializable.class.isAssignableFrom (clazz);
  }

  //OK
  protected static boolean isExternalizable (Class clazz) {
    return java.io.Externalizable.class.isAssignableFrom (clazz);
  }

  /** Filters out the fields which are transient or static. */
  private Field[] filterFields (Vector list) {
    Vector passed = new Vector();

    int modifier;
    Field field;
    for (int i = 0; i < list.size(); i++) {
      field = (Field)list.elementAt(i);
      modifier = field.getModifiers();
      if ( !Modifier.isTransient(modifier)
           && !Modifier.isStatic(modifier) ) {
        passed.add(field);
      }
    }

    Field[] result = new Field[passed.size()];
    for (int i = 0; i < passed.size(); i++) {
      result[i] = (Field)passed.elementAt(i);
    }

    return result;
  }

  /**
   * Creates a ObjectStreamField array using a Field array.
   * Result will be sorted according to JDK implementation of
   * serialization of fields.
   */
  private ObjectStreamField[] createObjectStreamFields (Field[] fields) {
    ObjectStreamField[] result = new ObjectStreamField[fields.length];

    // Check for bugs.
    int modifier;
    for (int i = 0; i < fields.length; i++) {
      modifier = fields[i].getModifiers();

      if ( Modifier.isTransient(modifier)
           || Modifier.isStatic(modifier) ) {
        error ("Modifier is static or transient. Is impossible.");
      }
    }

    //TODO: Check if sort is ok, otherwise uncomment these lines.

    // First add primitives.
    int elementsDone = 0;
    for (int i = 0; i < fields.length; i++) {
      if (fields[i].getType().isPrimitive()) {
        ObjectStreamField osField = new ObjectStreamField (fields[i]);

        result[elementsDone]= osField;
        elementsDone++;
      }
    }

    // Then add others.
    for (int i = 0; i < fields.length; i++) {
      if ( ! fields[i].getType().isPrimitive() ) {
        ObjectStreamField osField = new ObjectStreamField (fields[i]);
        result[elementsDone]= osField;
        elementsDone++;
      }
    }

/*
    System.out.println ("Array is null: " + (result==null) );
    for (int i = 0; i < result.length; i++) {
      System.out.println("" + i + ": " + result[i]);
    }
*/

    Arrays.sort(result);
    return result;
  }

  private void fillFields() {
    if ( isExternalizable(myClazz) ) {
      return;
    }

    // make array of serializable fields
    // if the class has serialPersistentsFields defined, use them
    Field persistent = null;
    ObjectStreamField[] osfields = null;
    try {
      persistent = myClazz.getClass().getDeclaredField("serialPersistentFields");
      osfields   = (ObjectStreamField[])persistent.get(myClazz);

      //TODO: Check: Why copy?
      //fields = new ObjectStreamField[osfields.length];
      //System.arraycopy(osfields,0,fields,0,osfields.length);

      //TODO: Continue? Return? Set refFields and myFields?
      return;
    }
    catch (NoSuchFieldException e) {
      // System.out.println("JVDBG: NoSuchFieldException catched, ok.\n");
      // This happens when the class did not define its persistent fields.
      // This is ok, continue the function.
    }
    catch (IllegalAccessException e) {
      //TODO: Security problem here!
      // possible... when serialPersistentFields is not public?
    }

    // No serialPersistentField defined, construct fields array.

    // this.myFields
    Vector localCandidates = new Vector();
    Field[] localCandidatesAsArray = myClazz.getDeclaredFields();
    for (int i = 0; i < localCandidatesAsArray.length; i++) {
      localCandidates.addElement (localCandidatesAsArray[i]);
    }

    Field[] filteredLocal = filterFields (localCandidates);
    this.myFields = createObjectStreamFields(filteredLocal);

    // Make all fields accessible for reflection.
    for (int i = 0; i < this.myFields.length; i++) {
      this.myFields[i].getRefField().setAccessible(true);
    }
  }

  public static ObjectStreamClass lookup (Class clazz) {
    //TODO: What if 'cl' is null?
    // This is a temporary solution.
    // Changing it may result in unexpected behaviour in other methods.
    if (clazz == null) {
      return null;
    }

    // System.out.println("JVDBG: lookup for class "+clazz.getName());

    //TODO: Cache everything. Even those that do not work.
    // Return 'null' if it isn't Serializable or Externalizable.
    if ( !isSerializable(clazz) && !isExternalizable(clazz) ) {
      return null;
    }

    // Return cached object if there is one.
    if (cache.get(clazz) != null) {
      return (ObjectStreamClass)cache.get(clazz);
    }

    return new ObjectStreamClass(clazz);
  }

  public String getName() {
    return this.myClazz.getName();
  }

  public long getSerialVersionUID() {
    return this.suid;
  }

  public Class forClass() {
    return this.myClazz;

    /*
    // Try to load class from current ClassLoader.
    Class tmpClazz;
    ClassLoader myClazzLoader = this.getClass().getClassLoader();
    if (myClazzLoader==null) {
      if (this.myClazz!=null) {
        System.out.println("JVDBG: this should not occur.");
        System.out.println("JVDBG: return forClass: "+this.myClazz);
        return this.myClazz;
      }
      else {
        // let's try the current classloader...
        try {
          this.myClazz=Class.forName(getName());
          this.resolved=true;
          return this.myClazz;
        }
        catch (Exception e) {
          System.out.println("JVDBG: cannot resolve class "+getName());
        }
      }
    }
    try {
      tmpClazz=myClazzLoader.loadClass(getName());
      this.myClazz=tmpClazz;
      this.resolved=true;
      return tmpClazz;
    }
    catch (ClassNotFoundException e) {
      return null;
    }
    */
  }

  public ObjectStreamField[] getMyFields() {
    return myFields;
  }

  public String toString() {
    return super.toString()
           + "-" + getName()
           + ": static final long serialVersionUID = " + this.suid;
  }

  // ----------------------------------------------------------------
  //   Methods implementing the grammar.
  // ----------------------------------------------------------------

  //OK
  private void readClassAnnotation(ObjectInputStream is) throws IOException {
    //TODO: do something extra with classannotation
    while (is.readByteNow() != ObjectStreamConstants.TC_ENDBLOCKDATA) {
      ;
    }
  }

  //OK
  private void readClassDescFlags(ObjectInputStream is) throws IOException {
    //TODO: extra checks?
    is.readByteNow();
  }

  //OK
  /** classDescInfo: classDescFlags fields classAnnotation superClassDesc */
  protected void readClassDescInfo(ObjectInputStream is)
    throws IOException,ClassNotFoundException {
    // classDescFlags
    readClassDescFlags(is);

    // fields
    readFields(is);

    // classAnnotation
    readClassAnnotation(is);

    // superClassDesc
    this.parentOSClass = is.readSuperClassDesc();

    // System.out.println("done reading classdescinfo");
  }

  // ----------------------------------------------------------------

  /** fields: (short)<count> fieldDesc[count] */
  private void readFields(ObjectInputStream is)
    throws ClassNotFoundException, IOException {
    // (short)<count>
    short numberOfFields = is.readShortNow();

    if (numberOfFields != this.myFields.length) {
      throw new IOException ("Classes do not match. (Different number of fields : " + numberOfFields + " != " + this.myFields.length + ") -> " + this.myClazz.getName());
    }

    // fieldDesc[count]
    for (int i = 0; i < numberOfFields; i++) {
      readFieldDesc(i, is);
    }

    // System.out.println("Done reading fields (de headers toch)");
  }

  private void readFieldDesc(int position, ObjectInputStream is)
    throws ClassNotFoundException, IOException {

    // typecode and fieldName
    byte   typecode  = is.readByteNow();
    String fieldname = is.readUTFNow();

    // System.out.println (position + ") " + typecode + " # " + fieldname);

    if ( (typecode != this.myFields[position].getTypeCode())
       || !(fieldname.equals(this.myFields[position].getName())) ) {
      throw new IOException ("Classes do not match. (Field type differs) -> " + this.myClazz.getName());
    }

    // primitive desc
    if ( (typecode != '[') && (typecode != 'L') ) {
      return;
    }

    // no primitive typecode, reading object desc
    // we have to read the className1 of the field type now

    String classname = (String)is.readObject();

    classname = classname.replace('/','.');
    if (typecode == 'L') {
      classname = classname.substring(1, classname.length()-1);
    }

    if (!this.myFields[position].getType().getName().equals(classname)) {
      throw new IOException ( "Classes do not match. "
                            + "Expected: " + this.myFields[position].getType().getName() + ". "
                            + "Read: " + classname + ".");
    }
  }

  protected void readFieldData(ObjectInputStream is, Object obj)
    throws IOException, IllegalArgumentException,
           ClassNotFoundException, OptionalDataException {

    for (int i = 0; i < this.myFields.length; i++) {
      try {
        if (DEBUG) {
          debug ("OSC: Assigning " + obj
                       + " with type " + this.myFields[i].getTypeCode()
                       + " to "  + this.myFields[i].getName());
        }

        if (this.myFields[i].isPrimitive()) {
          switch (this.myFields[i].getTypeCode()) {
            case 'B': assignByte(is,obj,i);break;
            case 'C': assignChar(is,obj,i);break;
            case 'D': assignDouble(is,obj,i);break;
            case 'F': assignFloat(is,obj,i);break;
            case 'I': assignInt(is, obj,i);break;
            case 'J': assignLong(is,obj,i);break;
            case 'S': assignShort(is,obj,i);break;
            case 'Z': assignBoolean(is,obj,i);break;
          }
        }
        else {
          assignObject(is,obj,i);
        }
      }
      catch (IllegalAccessException exc) {
        //TODO: is this the correct exception?
        exc.printStackTrace();
        throw new IOException ("Illegal access: " + exc.getMessage());
      }
    }
  }

  private void assignByte(ObjectInputStream is, Object obj, int i)
    throws IOException, IllegalAccessException {
    byte value = is.readByteNow();
    this.myFields[i].getRefField().setByte(obj,value);
  }

  private void assignChar(ObjectInputStream is, Object obj, int i)
    throws IOException, IllegalAccessException {
    char value = is.readCharNow();
    this.myFields[i].getRefField().setChar(obj,value);
  }


  private void assignDouble(ObjectInputStream is, Object obj, int i)
    throws IOException, IllegalAccessException {
    double value = is.readDoubleNow();
    this.myFields[i].getRefField().setDouble(obj,value);
  }

  private void assignFloat(ObjectInputStream is, Object obj, int i)
    throws IOException, IllegalAccessException {
    float value = is.readFloatNow();
    this.myFields[i].getRefField().setFloat(obj,value);
  }

  private void assignInt(ObjectInputStream is, Object obj, int i)
    throws IOException, IllegalAccessException {
    int value = is.readIntNow();
    this.myFields[i].getRefField().setInt(obj,value);
  }

  private void assignLong(ObjectInputStream is, Object obj, int i)
    throws IOException, IllegalAccessException {
    long value = is.readLongNow();
    this.myFields[i].getRefField().setLong(obj,value);
  }

  private void assignShort(ObjectInputStream is, Object obj, int i)
    throws IOException, IllegalAccessException {
    short value = is.readShortNow();
    this.myFields[i].getRefField().setShort(obj,value);
  }

  private void assignBoolean(ObjectInputStream is, Object obj, int i)
    throws IOException, IllegalAccessException {
    boolean value = is.readBooleanNow();
    this.myFields[i].getRefField().setBoolean(obj,value);
  }

  private void assignObject(ObjectInputStream is, Object obj, int i)
    throws IOException,ClassNotFoundException, IllegalAccessException {
    Object fieldObj=is.readObject();

    if (DEBUG) {
      if (this.myFields[i].getName().equals("this$0")) {
        System.out.println("Setting this$0");
        // Trying really hard to make it accessible!
        this.myFields[i].getRefField().setAccessible(true);
      }
    }

    this.myFields[i].getRefField().set(obj,fieldObj);
  }

  private static long calculateSerialVersionUID(Class cl) {
    return com.acunia.wonka.Wonka.suid(cl);
  }

  private void debug (String s) {
    if (DEBUG) {
      System.out.println(s);
    }
  }

  private boolean hasWriteObjectMethod() {
    try {
      getWriteObjectMethod();
      return true;
    }
    catch (NoSuchMethodException exc) {
      return false;
    }
  }

  boolean hasReadObjectMethod() {
    try {
      getReadObjectMethod();
      return true;
    }
    catch (NoSuchMethodException exc) {
      return false;
    }
  }

  private Method getWriteObjectMethod() throws NoSuchMethodException {
    Class[] args = {java.io.ObjectOutputStream.class};
    Method m = myClazz.getDeclaredMethod("writeObject", args);
    m.setAccessible(true);
    return m;

  }

  private Method getReadObjectMethod() throws NoSuchMethodException {
    Class[] args = {java.io.ObjectInputStream.class};
// System.out.println("GETreadobecjt on "+myClazz);
    return myClazz.getDeclaredMethod("readObject", args);
  }

  protected void invokeWriteMethod (String name, Object target, ObjectOutputStream oos)
      throws IOException {
    try {
      Method writeObject = getWriteObjectMethod();
      Object[] args = {oos};
      writeObject.invoke (target, args);
    }
    catch (NoSuchMethodException exc) {
      //TODO: what kind of exception should we throw here?
      throw new IOException("NoSuchMethodException while doing writeObject(ObjectOutputStream) with target " +
                              target.getClass().getName() );
    }
    catch (IllegalAccessException exc) {
      //TODO: what kind of exception should we throw here?
      throw new IOException("IllegalAccessException while doing writeObject(ObjectOutputStream) with target " +
                              target.getClass().getName() );
    }
    catch (java.lang.reflect.InvocationTargetException exc) {
      //TODO: maybe the subexception is interesting...
      throw new IOException("InvocationTargetException while doing writeObject(ObjectOutputStream) with target " +
                              target.getClass().getName() );
    }
  }

  protected void invokeReadMethod (String name, Object target, ObjectInputStream ois)
      throws IOException {
    try {
      Method readObject = getReadObjectMethod();
      readObject.setAccessible(true);
      Object[] args = {ois};
      readObject.invoke (target, args);
    }
    catch (NoSuchMethodException exc) {
      //TODO: what kind of exception should we throw here?
      throw new IOException("NoSuchMethodException while doing readObject(ObjectInputStream) with target " +
                              target.getClass().getName() );
    }
    catch (IllegalAccessException exc) {
      //TODO: what kind of exception should we throw here?
      throw new IOException("IllegalAccessException while doing readObject(ObjectInputStream) with target " +
                              target.getClass().getName() );
    }
    catch (java.lang.reflect.InvocationTargetException exc) {
      //TODO: maybe the subexception is interesting...
      throw new IOException("InvocationTargetException while doing readObject(ObjectInputStream) with target " +
                              target.getClass().getName() );
    }
  }

  //TODO: Is this used?
  public static native boolean hasClinit(Class cl);

  private static void error (String message) {
    System.out.println(message);
    System.exit(1);
  }

  private static void error (String message, Throwable error) {
    System.out.println(message);
    error.printStackTrace();
    System.exit(1);
  }

  protected ObjectStreamClass getParentObjectStreamClass() {
    return this.parentOSClass;
  }

  protected byte getClassDescFlag() {
    //TODO: Not supported:
    //       - writeObject
    //       - Externalizable, and its blockdata flag.
    // Probably correct here, but maybe needs some other code also.

    byte flag = 0x00;
    if ( ObjectStreamClass.isExternalizable(this.forClass()) )
    {
      // Externalizable.
      flag = (byte)(flag | ObjectStreamConstants.SC_EXTERNALIZABLE);

      // Set when using JDK1.2 and STREAM_PROTOCOL_2 and Externalizable.
      flag = (byte)(flag | ObjectStreamConstants.SC_BLOCK_DATA);
    }
    else if ( ObjectStreamClass.isSerializable (this.forClass()) ) {
      // Serializable.
      flag = (byte)(flag | ObjectStreamConstants.SC_SERIALIZABLE);

      // Set when there is a writeObject method and Serializable.
      //TODO: Check if parameters match.
      //TODO: Move met from ObjectStreamClass to this one?

      if (this.hasWriteObjectMethod()) {
        flag = (byte)(flag | ObjectStreamConstants.SC_WRITE_METHOD);
      }
    }

    return flag;
  }
}
