/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: ObjectStreamField.java,v 1.1.1.1 2001/11/21 10:41:34 pollock Exp $
*/

package java.io;

import java.lang.reflect.Field;

public class ObjectStreamField implements Comparable {

  private char code;
  private String name;
  private Class type;
  private transient boolean isLocal;
  private transient Field field;

  protected native void create(String name, Class type);

  public ObjectStreamField (String name, Class type) {
    this.name = name;
    this.type = type;
    create(name, type);
  }

  ObjectStreamField (Field field) {
    this.field = field;
    this.name  = field.getName();
    this.type  = field.getType();

    create(name, type);
  }

  // protected native Field getField();

  public void setLocal(boolean value) {
    this.isLocal = value;
  }
  public String getName() {
    return this.name;
  }
  
  public Class getType() {
    return this.type;
  }

  public char getTypeCode() {
    return this.code;
  }

  public native String getTypeString();

  public String getComponentTypeString() {
    String arrayString = getTypeString();
    if (arrayString.charAt(0)!='[') return null; // no array
    return arrayString.substring(1);
  }

  public boolean isPrimitive() {

    char code = getTypeCode();

    if ( (code=='B') || (code=='C') || (code=='D') || (code=='F') || (code=='I') || (code=='J') || (code=='S') || (code=='Z')) {
      return true;
    }
    else {
      return false;
    }
  }

  public boolean isArray() {
    return (getTypeCode()=='[');
  }
  
  public String toString() {
    return name;
  }

  public int compareTo (Object o) {
    int answer;
    if (o instanceof ObjectStreamField) {
      ObjectStreamField target = (ObjectStreamField)o;
      if (this.isPrimitive() != target.isPrimitive()) {
        answer =  (this.isPrimitive()? -1 : 1 );
      }
      else {
        answer = this.getName().compareTo(target.getName());
      }
    }
    else {
      throw new RuntimeException ("Try to compare a ObjectStreamField with something else");
    }
    return answer;
  }

  protected Field getRefField() {
    return this.field;
  }
}
