/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: OutputStreamWriter.java,v 1.1.1.1 2001/11/21 10:41:34 pollock Exp $
*/

package java.io;

public class OutputStreamWriter extends Writer {

  private OutputStream out;
  private String encoding;
  private StringBuffer buf = new StringBuffer(512);

  public OutputStreamWriter(OutputStream out) {
    super(out);
    this.out = out;
    encoding = "8859_1";
  }

  public OutputStreamWriter(OutputStream out, String enc) throws UnsupportedEncodingException {
    super(out);
    if(enc == null){
      throw new UnsupportedEncodingException();
    }
    this.out = out;
    encoding = enc;
  }

  public void close() throws IOException {
    synchronized(lock){
      if(out != null){
        out.write(buf.toString().getBytes(encoding));
        out.close();
        out = null;
        buf = null;
        encoding = null;
      }
    }
  }

  public void flush() throws IOException {
    synchronized(lock){
       if(out == null){
         throw new IOException("Reader is closed");
       }
       out.write(buf.toString().getBytes(encoding));
       out.flush();
       buf.setLength(0);
    }
  }

  private String getEncoding(){
    return encoding;
  }

  public void write(int ch) throws IOException {
    synchronized(lock){
       if(out == null){
         throw new IOException("Reader is closed");
       }
       buf.append((char)ch);
       if(buf.length() > 1024){
         out.write(buf.toString().getBytes(encoding));
         buf.setLength(0);
       }
    }
  }

  public void write(char[] chars, int off, int len) throws IOException {
    synchronized(lock){
       if(out == null){
         throw new IOException("Reader is closed");
       }
       buf.append(chars, off, len);
       if(buf.length() > 1024){
         out.write(buf.toString().getBytes(encoding));
         buf.setLength(0);
       }
    }
  }

  public void write(String str, int off, int len) throws IOException {
    synchronized(lock){
       if(out == null){
         throw new IOException("Reader is closed");
       }
       buf.append(str.substring(off, len));
       if(buf.length() > 1024){
         out.write(buf.toString().getBytes(encoding));
         buf.setLength(0);
       }
    }
  }
}
