/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/



package java.io;

public class RandomAccessFile implements DataOutput, DataInput {

  private FileDescriptor fd;
  private String mode;

  private native int createFromString(String name, String mode);

  public RandomAccessFile(String name, String mode) throws FileNotFoundException {
    if(!(mode.equals("r") || mode.equals("rw"))) throw new IllegalArgumentException();
    this.mode = mode;
    if(createFromString("." + (new File(name)).getAbsolutePath(), (this.mode == "r" ? this.mode : "r+")) != 0) {
      throw new FileNotFoundException(name + " (" + mode + ")");
    }
  }

  public RandomAccessFile(File file, String mode) throws IOException {
    if(!(mode.equals("r") || mode.equals("rw"))) throw new IllegalArgumentException();
    this.mode = mode;
    if(createFromString("." + file.getAbsolutePath(), (this.mode == "r" ? this.mode : "r+")) != 0) {
      throw new FileNotFoundException("" + file + " (" + mode + ")");
    }
  }

  public final FileDescriptor getFD()
    throws IOException {
    return fd;
  }

  public native int read() 
    throws IOException;

  private native int readIntoBuffer(byte[]b, int off, int len);

  public int read(byte[] b, int off, int len) 
    throws IOException, NullPointerException, ArrayIndexOutOfBoundsException {
    
    if(off<0 || len<0 || off+len > b.length)
      throw new ArrayIndexOutOfBoundsException();
    if(len == 0) {
      return 0;
    }
    return readIntoBuffer(b, off, len);
  }
 
  public int read(byte[] b)
    throws IOException, NullPointerException {
    if(b.length == 0) return 0; 
    return readIntoBuffer(b, 0, b.length);
  }

  public final void readFully(byte[] b)
    throws IOException, NullPointerException, ArrayIndexOutOfBoundsException, EOFException {
    if(read(b) == -1)
      throw new EOFException();
  }

  public final void readFully(byte[] b, int off, int len)
    throws IOException, NullPointerException, EOFException {
    if(read(b, off, len) == -1) throw new EOFException();
  }

  public native int skipBytes(int n)
    throws IOException;

  public native void write(int b)
    throws IOException;

  private native void writeFromBuffer(byte[] b, int off, int len)
    throws IOException;

  public void write(byte[] b)
    throws IOException, NullPointerException {

    if(b==null) throw new NullPointerException();
    writeFromBuffer(b, 0, b.length);
  }

  public void write(byte[] b, int off, int len)
    throws IOException, NullPointerException, ArrayIndexOutOfBoundsException {

    if(b==null) throw new NullPointerException();

    if(off<0 || len<0 || off+len>b.length)
      throw new ArrayIndexOutOfBoundsException();

    writeFromBuffer(b, off,len);
  }

  public native long getFilePointer()
    throws IOException;

  public native void seek(long pos)
    throws IOException;

  public native long length()
    throws IOException;

  public native void setLength(long NewLength)
    throws IOException;

  public native void close()
    throws IOException;

  public final boolean readBoolean() throws IOException {
    int a = this.read();

    if(a == 0) 
      return false;
    else
      return true;
  }

  public final byte readByte() throws IOException {
    return (byte)this.read();
  }

  public final int readUnsignedByte() throws IOException {
    return this.read();
  }

  public final short readShort() throws IOException {
    return (short)((this.read() << 8) | this.read());
  }

  public final int readUnsignedShort() throws IOException {
    return (int)((this.read() << 8) | this.read());
  }

  public final char readChar() throws IOException {
    return (char)((this.read() << 8) | this.read());
  }

  public final int readInt() throws IOException {
    return (int)((this.read() << 24) | (this.read() << 16) | (this.read() << 8) | this.read());
  }

  public final long readLong() throws IOException {
    return (long)( ((long)this.read() << 56) | ((long)this.read() << 48) | 
                   ((long)this.read() << 40) | ((long)this.read() << 32) |
                   ((long)this.read() << 24) | ((long)this.read() << 16) |
                   ((long)this.read() << 8) | ((long)this.read()) );
  }

  public final float readFloat() throws IOException {
    return Float.intBitsToFloat(this.readInt());
  }

  public final double readDouble() throws IOException {
    return Double.longBitsToDouble(this.readLong());
  }

  public final void writeBoolean(boolean v) throws IOException {
    if(v) this.write(1); else this.write(0);
  }

  public final void writeByte(int v) throws IOException {
    this.write(v);
  }
  
  public final void writeShort(int v) throws IOException {
    this.write(v >> 8);
    this.write(v & 0x00FF);
  }

  public final void writeChar(int v) throws IOException {
    this.write((v >> 8) & 0x000000FF);
    this.write(v & 0x000000FF);
  }

  public final void writeInt(int v)throws IOException {
    this.write((v >> 24) & 0x000000FF);
    this.write((v >> 16) & 0x000000FF);
    this.write((v >> 8) & 0x000000FF);
    this.write(v & 0x000000FF);
  }

  public final void writeLong(long v) throws IOException {
    this.write((int)((v >> 56) & 0x000000FF));
    this.write((int)((v >> 48) & 0x000000FF));
    this.write((int)((v >> 40) & 0x000000FF));
    this.write((int)((v >> 32) & 0x000000FF));
    this.write((int)((v >> 24) & 0x000000FF));
    this.write((int)((v >> 16) & 0x000000FF));
    this.write((int)((v >> 8) & 0x000000FF));
    this.write((int)(v & 0x000000FF));
  }

  public final void writeFloat(float v) throws IOException {
    this.writeInt(Float.floatToIntBits(v));
  }

  public final void writeDouble(double v) throws IOException {
    this.writeLong(Double.doubleToLongBits(v));
  }

  public final String readLine()
    throws IOException {
    return null;
  }

  public final String readUTF() 
    throws IOException {
    return null;
  }

  public final void writeBytes(String s)
    throws IOException {
  }

  public final void writeChars(String s)
    throws IOException {
  }

  public final void writeUTF(String str) 
    throws IOException {
  }

  protected void finalize() {
    try {
      if(fd != null) close();
    } catch(Exception e) {
    }
  }

}
