/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: StreamTokenizer.java,v 1.3 2002/03/13 16:21:03 dumon Exp $
*/

package java.io;

public class StreamTokenizer {

  public int ttype;
  public double nval;
  public String sval;

  public static final int TT_EOF = -1;  	//values specified according the java language specification
  public static final int TT_EOL= '\n';	//values specified according the java language specification
  public static final int TT_NUMBER = -2; //values specified according the java language specification
  public static final int TT_WORD = -3;   //values specified according the java language specification


  private final Range commentChars = new Range(-2,-2);
  private final Range quoteChars = new Range(-2,-2);
  private final Range digitChars = new Range(-2,-2);
  private final Range wordChars = new Range(-2,-2);
  private final Range whtspChars = new Range(-2,-2);

  private boolean cppstyle=false; //slashslash comments allowed
  private boolean cpstyle=false;	//slashstar comments allowed	
  private boolean eolSign = false;//end_of_line taken into account
  private boolean tolower=false;  //translate chars to lowerCase ...
  private int lineNr=0;
  private String line="";
  private boolean notEOF=true;

  private int currentpos=0;
  private boolean pushback=false;


  private BufferedReader buffer;

  public StreamTokenizer(Reader reader) {
    buffer = new BufferedReader(reader);
    commentChars.next = new Range(((int)'/'),((int)'/'),null);
    digitChars.next = new Range((int)'0', (int)'9', null);
    add(digitChars , (int)'-',(int)'-');
    add(digitChars , (int)'.',(int)'.');
    quoteChars.next = new Range(((int)'"'),((int)'"'),null);
    add(quoteChars,((int)'\''),((int)'\''));
    whtspChars.next = new Range(0,32, null);
    wordChars.next = new Range(160,255,null);
    add(wordChars, (int)'a', (int)'z');
    add(wordChars, (int)'A', (int)'Z');
  }
  
  public StreamTokenizer(InputStream is) {
    this(new InputStreamReader(is));
  }
  
  public void eolIsSignificant(boolean eolSignificant) {
    eolSign = eolSignificant;
  }

  public int lineno() {
    return lineNr;
  }

  public void lowerCaseMode(boolean lower) {
    tolower = lower;
  }

  private boolean getChar()throws IOException {

    //System.out.println("debugging - getchar is called");
    while (currentpos >= line.length() && notEOF){			
      String newline = buffer.readLine();
      if (newline != null) {
        line = newline;
        lineNr++;
        currentpos=0;
        if (eolSign && lineNr > 0) {
          ttype = TT_EOL;
          return false;
        }
      }
      else {
        line="";
        notEOF=false;
        ttype = TT_EOF;
        return false;
      }
    }
    return (notEOF);
  }

  private boolean handleNextChar()throws IOException {
    int ch = (int) line.charAt(currentpos);
    // as long we have a white spaces skip them ...
    while (isInRange(whtspChars, ch)) {
      //System.out.println("debugging - whitespace encountered -->"+line);
      currentpos++;
      if(!getChar()) {
        return false;
      }
      ch = (int) line.charAt(currentpos);
    }
    // if we have a comment char, we skip the rest of the line		
    if (isInRange(commentChars, ch)) {
      //System.out.println("debugging - commentchar encountered");
      currentpos = line.length();
      return false;
    }
    // if we have a quote char, we look for the same char again (or EOL)
    if (isInRange(quoteChars, ch)) {
      //System.out.println("debugging - quote char encountered");
      ttype = ch;
      int newpos = line.indexOf(ch, ++currentpos);
      newpos = (newpos == -1 ? line.length() : newpos);
      sval = line.substring(currentpos , newpos);
      if (tolower) {
        sval = sval.toLowerCase();
      }
      currentpos = newpos+1;
      return true;
    }
    if (isInRange(wordChars, ch)){
      //System.out.println("debugging - word char encountered");
      ttype= TT_WORD;
      int newpos = currentpos;
      while (++currentpos < line.length()) {
        int nch = (int)line.charAt(currentpos);
        if (!isInRange(wordChars, nch)&& !isInRange(digitChars, nch)) {
          break;
        }
      }
      sval = line.substring(newpos, currentpos);
      if (tolower) {
        sval = sval.toLowerCase();
      }
      return true;
    }
    if (isInRange(digitChars, ch)){
      //System.out.println("debugging - digit encountered");
      ttype= TT_NUMBER;
      int newpos = currentpos;
      int dot = ch == (int)'.' ? 1 : 0;
      while (++currentpos < line.length()) {
        int nch = (int)line.charAt(currentpos);
        if (!isInRange(digitChars, nch)) {
          break;
        }
        if (nch == (int)'-' || (nch == (int)'.' && dot == 1)) {
          break;
        }
        if (nch == (int)'.') { dot++; }

      }
      if (ch != (int)'-' || currentpos != (newpos+1)) {
        nval = new Double(line.substring(newpos,currentpos)).doubleValue();
        return true;		 		
      }
      currentpos = newpos;
    }
    if (ch == (int)'/' && cppstyle && currentpos < line.length()) { //check if // is allowed
      if (line.charAt(currentpos+1) == (int)'/') {
        //System.out.println("debugging - // encountered -->"+line);		
        currentpos = line.length();
        return false;
      }              	
    }
    if (ch == (int)'/' && cpstyle && currentpos < line.length()) { //check if /* */ is allowed
      if (line.charAt(currentpos+1) == (int)'*') {
        //System.out.println("debugging - /* */ encountered -->"+line);		
        currentpos = line.indexOf("*/", currentpos+2);
        while ( currentpos == -1 ) {
          //System.out.println("debugging in while - /* */ encountered -->"+line);		
          line = buffer.readLine();
          lineNr++;
          if (line == null) {
            notEOF = false;	
            line ="";
            lineNr--;
            ttype = TT_EOF;
            return true;
          }
          currentpos = line.indexOf("*/", 0);
        }
        currentpos +=2;
        return false;
      }

    }
    //System.out.println("debugging - ordinary char encountered");		
    ttype = ch;
    currentpos++;
    return true;		       			
  }

  public int nextToken()throws IOException {		
    if (!pushback) {
      while (getChar()) {
        //System.out.println("debugging - handling a char");				 	
        if (handleNextChar()){
          break;
        }	
      }
    }
    else {
      pushback = false;
    }
    return ttype;	
  }

  public void pushBack(){
    pushback=true;
  }

  public void slashSlashComments(boolean cpp) {
    cppstyle = cpp;
  }

  public void slashStarComments(boolean cpp) {
    cpstyle = cpp;
  }


  // functions for toying around with the range objects

  private boolean isInRange(Range range, int ch) {
    range = range.next;	
    while (range != null) {
      if ( range.low <= ch && ch <= range.high) {
        return true;
      }
      range = range.next;	 	 	
    }
    return false;
  }

  private boolean add(Range startrange, int low, int high){
    Range oldR = startrange;
    Range range = oldR.next;	
    while (range != null) {
      if (range.high < low || range.low > high) { //Ranges don't overlap
        oldR = range;
        range = range.next;
      }	
      else {
        if (range.low <= low && range.high >= high) { //new range is already defined
          return false; // no action required				 	
        }
        //the range overlap we take the largest possible range to continue
        low = ( low < range.low ? low : range.low );
        high = ( high > range.high ? high : range.high );
        //we cut out the already defined Range
        range = range.next;
        oldR.next = range;
      }
    }
    //we add the new Range to the list
    oldR.next = new Range(low, high);	 	
    return true; //action required --> remove in other list
  } 	

  private void removeRange(Range startRange, int low, int high) {
    Range oldR = startRange;
    Range range = oldR.next;	
    while (range != null) {
      if (range.high < low || range.low > high) { //Ranges don't overlap
        oldR = range;
        range = range.next;
      }	
      else {
        if (range.low >= low && range.high <= high) { //the range is defined
          if (range.low == low && range.high == high) {
            // the ranges are equal --> remove the Range element
            oldR.next = range.next;
            break;					 	
          }
          else {
            //this range is completly in the range to remove
            range = range.next;
            oldR.next = range;				
          }
        }
        else {
          if (range.low < low && range.high > high){
            oldR.next = new Range(range.low, low-1, range);
            range.low = high+1;

          }
          else {
            //the ranges overlap, we cut off the overlapping range
            if( range.high <= high ) {
              range.high = low-1;								
            }
            else {
              range.low = high+1;
            }
          }
          //we check the next Range ...
          oldR = range;
          range = range.next;
        }
      }
    }
  }

  public void commentChar(int ch) {
    if (ch >= 0 && ch <= 255){
      if (add(commentChars, ch, ch)){
        removeRange(quoteChars, ch, ch);
        removeRange(digitChars, ch, ch);
        removeRange(wordChars, ch, ch);
        removeRange(whtspChars, ch, ch);
      }
    }
  }

  public void ordinaryChar(int ch) {
    if (ch >= 0 && ch <= 255){
      removeRange(commentChars, ch, ch);
      removeRange(quoteChars, ch, ch);
      removeRange(digitChars, ch, ch);
      removeRange(wordChars, ch, ch);
      removeRange(whtspChars, ch, ch);
    }	
  }

  public void ordinaryChars(int low, int hi) {
    low = ( low > 0 ? low : 0 );
    hi = (hi > 255 ? 255 : hi );
    if (low < hi){
      removeRange(commentChars, low, hi);
      removeRange(quoteChars, low, hi);
      removeRange(digitChars, low, hi);
      removeRange(wordChars, low, hi);
      removeRange(whtspChars, low, hi);
    }	
  }

  public void parseNumbers() {
    int n0 = ((int)'0');
    int n9 = ((int)'9');
    int np = ((int)'.');
    int nm = ((int)'-');
    add(digitChars, n0, n9);
    removeRange(commentChars, n0, n9);
    removeRange(quoteChars, n0, n9);
    removeRange(wordChars, n0, n9);
    removeRange(whtspChars, n0, n9);
    add(digitChars, np, np);
    removeRange(commentChars, np, np);
    removeRange(quoteChars, np, np);
    removeRange(wordChars, np, np);
    removeRange(whtspChars, np, np);
    add(digitChars, nm, nm);
    removeRange(commentChars, nm, nm);
    removeRange(quoteChars, nm, nm);
    removeRange(wordChars, nm, nm);
    removeRange(whtspChars, nm, nm);
  }

  public void quoteChar(int ch) {
    if ( ch >= 0 && ch <= 255){
      if (add(quoteChars, ch, ch)) {
        removeRange(commentChars, ch, ch);
        removeRange(digitChars, ch, ch);
        removeRange(wordChars, ch, ch);
        removeRange(whtspChars, ch, ch);
      }
    }	
  }

  public void resetSyntax() {

    commentChars.next = null;
    digitChars.next = null;
    quoteChars.next = null;
    whtspChars.next = null;
    wordChars.next = null;

  }

  public String toString(){
    StringBuffer buf = 	new StringBuffer(64);
    buf.append("StreamTokenizer: line = ");
    buf.append(line);
    buf.append(" last token :'");
    switch(ttype){
      case TT_WORD:
        buf.append(sval);
        break;
      case TT_NUMBER:
        buf.append(nval);
        break;
      case TT_EOF:
        buf.append("EOF");
        break;
      case TT_EOL:
        buf.append("EOL");
        break;
      default:
        buf.append((char)ttype);
    }
    buf.append('\'');
    return buf.toString();	
  }

  public void whitespaceChars(int low, int hi) {
    low = (low > 0 ? low : 0);
    hi = (hi > 255 ? 255 : hi);
    if ( low < hi){
      if (add(whtspChars, low, hi)){	 	
        removeRange(commentChars, low, hi);
        removeRange(quoteChars, low, hi);
        removeRange(digitChars, low, hi);
        removeRange(wordChars, low, hi);
      }
    }	
  }	

  public void wordChars(int low, int hi) {
    low = (low > 0 ? low : 0);
    hi = (hi > 255 ? 255 : hi);
    if (low < hi){
      if (add(wordChars, low, hi)){	 	
        removeRange(commentChars, low, hi);
        removeRange(quoteChars, low, hi);
        removeRange(digitChars, low, hi);
        removeRange(whtspChars, low, hi);
      }
    }	
  }

  /**
   ** there are six kinds of special characters:
   ** comments-, quote-, word-, digit-, whitespace- and ordinary characters.
   ** a character can be only one of those so we have the Range innerclass to help us keep track of them
   ** we only need 5 tracks of ranges (by default a character is an ordinary character)
   */

  private class Range {

    public int low;
    public int high;
    public Range next;

    public Range (int lo, int hi) {
      low = lo;
      high = hi;
    }

    public Range (int lo, int hi, Range nxt) {
      low = lo;
      high = hi;
      next = nxt;
    }
    /*
    // REMOVE THIS METHOD
    public void debug(){
    System.out.println(this+" low ="+low+", high ="+high+", next -->"+next);
    if (next != null) { next.debug(); }
    }
     */
  }
}
