/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: StringBufferInputStream.java,v 1.1.1.1 2001/11/21 10:41:35 pollock Exp $
*/

package java.io;


public class StringBufferInputStream extends InputStream {
 
  protected String buffer;

  protected int pos;

  protected int count;

  private char[] chars;

  public StringBufferInputStream(String s)
    throws NullPointerException
  {
    if (s==null) throw new NullPointerException();

    buffer = s;
    pos = 0;
    count = s.length();
    chars = s.toCharArray();
  }

  public int read() throws IOException {

    if (pos==count) return -1;

    return chars[pos++] & 0xff;
  }
  
  public int read(byte[] b, int off, int len)
    throws IOException, NullPointerException, ArrayIndexOutOfBoundsException
  {
    if (b==null) throw new NullPointerException();

    if(off<0 || len<0 || off+len>b.length) throw new ArrayIndexOutOfBoundsException()
;

    if (pos==count) return -1;

    int k = (len<count-pos ? len : count-pos);
    int i;

    for(i=0;i<k;++i) {
      b[i+off] = (byte)(chars[pos++] & 0xff);
    }

    return k;

  }

  
  public long skip(long n)
    throws IOException
  {
    long k = (n<count-pos ? n : count-pos);

    pos += (int)k;

    return k;
  }


  public int available()
    throws IOException
  {
    return count-pos;
  }

  public void reset()
  {
    pos = 0;
  }


}
