/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: StringBuffer.java,v 1.2 2002/03/27 08:57:16 ruelens Exp $
*/

package java.lang;

public class StringBuffer implements java.io.Serializable {

  private static final long serialVersionUID = 3388685877147921107L;

  private char[] value;
  private int count;
  private boolean shared;

  public StringBuffer() {
    value = new char[16];
    count = 0;
  }

  public StringBuffer(int length) throws NegativeArraySizeException {
    value = new char[length];
    count = 0;
  }

  native private void createFromString(String string);
  
  public StringBuffer(String string) {
    createFromString(string);
  }

  public int length() {
    return count;
  }

  public int capacity() {
    return value.length;
  }

  native public void ensureCapacity(int minimumCapacity);

  native public void setLength(int newLength) throws StringIndexOutOfBoundsException;


  public char charAt(int index) throws IndexOutOfBoundsException {
    if (index >= count) throw new IndexOutOfBoundsException();
    return value[index];
  }

  native public void getChars(int srcOffset, int srcEnd, char[] dst, int dstOffset) throws IndexOutOfBoundsException;

  public void setCharAt(int index, char ch) throws IndexOutOfBoundsException {
    if (index < 0 || index >= count) {
      throw new IndexOutOfBoundsException();
    }
    value[index] = ch;
  }

  public StringBuffer append(Object obj) {
    if (obj == null) {
      return this.append("null");
    }
    return append(obj.toString());
  }

  native public StringBuffer append(String s);

  public StringBuffer append(char[] str) throws NullPointerException  {
    return this.append(new String(str));
  }

  public StringBuffer append(char[] str, int offset, int len) throws NullPointerException, StringIndexOutOfBoundsException {
    return this.append(new String(str,offset,len));
  }

  public StringBuffer append(boolean b) {
    return this.append(String.valueOf(b));
  }

  public StringBuffer append(char c) {
    this.ensureCapacity(count + 1);
    value[count++] = c;
    return this;
  }

  public StringBuffer append(int i) {
    return this.append(String.valueOf(i));
  }

  public StringBuffer append(long l) {
    return this.append(String.valueOf(l));
  }

  public StringBuffer append(float f) {
    return this.append(String.valueOf(f));
  }

  public StringBuffer append(double d) {
    return this.append(String.valueOf(d));
  }

  native public StringBuffer deleteCharAt(int index) throws StringIndexOutOfBoundsException;

  native public StringBuffer replace(int start, int end, String str) throws StringIndexOutOfBoundsException;

  public String substring(int start) {
    return this.substring(start, this.count);
  }

  native public String substring(int start, int end) throws StringIndexOutOfBoundsException;
  
  native public StringBuffer insert(int offset, String str) throws StringIndexOutOfBoundsException;

  public StringBuffer insert(int offset, Object obj) throws StringIndexOutOfBoundsException {
    return this.insert(offset, String.valueOf(obj));
  }

// WAS:  native public StringBuffer insert(int index, char[] str, int offset, int len);

  public StringBuffer insert(int offset, char[] str, int stroffset, int strlen) throws NullPointerException, StringIndexOutOfBoundsException {
    return this.insert(offset, new String(str,stroffset,strlen));
  }

  public StringBuffer insert(int offset, char[] str) throws NullPointerException, StringIndexOutOfBoundsException {
// WAS:    return this.insert(offset, str, 0, str.length);
    return this.insert(offset, new String(str));
  }

  public StringBuffer insert(int offset, boolean b) throws StringIndexOutOfBoundsException {
    return this.insert(offset, String.valueOf(b));
  }

  public StringBuffer insert(int offset, char c) throws StringIndexOutOfBoundsException {
    return this.insert(offset, String.valueOf(c));
  }

  public StringBuffer insert(int offset, int i) throws StringIndexOutOfBoundsException {
    return this.insert(offset, String.valueOf(i));
  }

  public StringBuffer insert(int offset, long l) throws StringIndexOutOfBoundsException {
    return this.insert(offset, String.valueOf(l));
  }

  public StringBuffer insert(int offset, float f) throws StringIndexOutOfBoundsException {
    return this.insert(offset, String.valueOf(f));
  }

  public StringBuffer insert(int offset, double d) throws StringIndexOutOfBoundsException {
    return this.insert(offset, String.valueOf(d));
  }

  public native StringBuffer reverse();

  public native String toString();

  public native synchronized StringBuffer delete(int start, int end);

}
