/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: System.java,v 1.5 2002/03/27 08:09:43 gray Exp $
*/

package java.lang;

import com.acunia.wonka.Etc;

import java.util.Properties;
import java.util.PropertyPermission;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.io.FileDescriptor; 
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.BufferedOutputStream;
import javax.comm.CommPort;
import javax.comm.CommPortIdentifier;
import com.acunia.wonka.NullInputStream;
import com.acunia.wonka.NullOutputStream;
import com.acunia.wonka.CRLFOutputStream;

public final class System {

  public static InputStream in;
  public static PrintStream out;
  public static PrintStream err;

  private static SecurityManager theSecurityManager;
  private static Runtime theRuntime = Runtime.getRuntime();
  private static Properties systemProperties = new Properties(createDefaultProperties());
  private static boolean bootstrapping = true;

  static {
    try {
      InputStream syspropstream = ClassLoader.getSystemResourceAsStream("system.properties");
      systemProperties.load(syspropstream);
    } catch (IOException e) {
      e.printStackTrace();
    }

    bootstrapping = false;

    try {
      CommPortIdentifier system_in_port_id = CommPortIdentifier.getPortIdentifier("stdin");
      CommPort system_in_port = system_in_port_id.open("System.in",1000);
      in  = system_in_port.getInputStream();

      boolean  system_out_autoflush = Boolean.getBoolean("com.acunia.wonka.System.out.autoflush");
      CommPortIdentifier system_out_port_id = CommPortIdentifier.getPortIdentifier("stdout");
      CommPort system_out_port = null;

      if (system_out_port_id==system_in_port_id) {
        out = new PrintStream(new BufferedOutputStream(system_in_port.getOutputStream(), 128),system_out_autoflush);
      }
      else {
        system_out_port = system_out_port_id.open("System.out",1000);
        out = new PrintStream(new BufferedOutputStream(system_out_port.getOutputStream(), 128),system_out_autoflush);
      }

      boolean  system_err_autoflush = Boolean.getBoolean("com.acunia.wonka.System.err.autoflush");
      CommPortIdentifier system_err_port_id = CommPortIdentifier.getPortIdentifier("stderr");
      if (system_err_port_id==system_in_port_id) {
        err = new PrintStream(new BufferedOutputStream(system_in_port.getOutputStream(), 128),system_err_autoflush);
      }
      else if (system_err_port_id==system_out_port_id) {
        err = new PrintStream(new BufferedOutputStream(system_out_port.getOutputStream(), 128),system_err_autoflush);
      }
      else {
        CommPort system_err_port = system_err_port_id.open("System.err",1000);
        err = new PrintStream(new BufferedOutputStream(system_err_port.getOutputStream(), 128),system_err_autoflush);
      }
    } catch (Exception e) {
    }

  }

  private static Properties createDefaultProperties() {
    Properties systemDefaultProperties = new Properties();

    systemDefaultProperties.put("java.version","1.2");
    systemDefaultProperties.put("java.vendor","ACUNIA");
    systemDefaultProperties.put("java.vendor.url","http://www.acunia.com");
    systemDefaultProperties.put("java.home","com/acunia/wonka");
    systemDefaultProperties.put("java.class.version","1.0");
    systemDefaultProperties.put("java.class.path","");
    systemDefaultProperties.put("os.name","Wonka");
    systemDefaultProperties.put("os.arch",Etc.getPlatform());
    systemDefaultProperties.put("os.version","1.0");
    systemDefaultProperties.put("file.separator","/");
    systemDefaultProperties.put("file.encoding","8859_1");
    systemDefaultProperties.put("path.separator",":");
    systemDefaultProperties.put("line.separator","\n");
    systemDefaultProperties.put("user.name","");
    systemDefaultProperties.put("user.home","");
    systemDefaultProperties.put("user.dir","");

    return systemDefaultProperties;
  }

  public static void setIn(InputStream newIn) {
    if (in!=null) {
      try {
        Etc.woempa(9,"setIn: closing "+in);
        in.close();
      } catch (java.io.IOException e) {}
    }

    in = newIn;
  }

  public static void setOut(PrintStream newOut) {
    if (out!=null) {
        Etc.woempa(9,"setOut: closing "+in);
        out.close();
    }
    out = newOut;
  }

  public static void setErr(PrintStream newErr) {
    if (err!=null) {
        Etc.woempa(9,"setErr: closing "+in);
        err.close();
    }

    err = newErr;
  }

  public static SecurityManager getSecurityManager() { 
    return theSecurityManager;
  }

  public static void setSecurityManager(SecurityManager sm)
    throws SecurityException
  { 
    if (theSecurityManager!=null) {
      if (com.acunia.wonka.SecurityConfiguration.USE_ACCESS_CONTROLLER
       || com.acunia.wonka.SecurityConfiguration.USE_SECURITY_MANAGER) {
        java.security.AccessController.checkPermission(new RuntimePermission("setSecurityManager"));
      }
    }
    theSecurityManager = sm;
  }

  public static native long currentTimeMillis();

  private static void propertyCheck(String propname) {
    if (bootstrapping) {

      return;

    }

    if (com.acunia.wonka.SecurityConfiguration.USE_ACCESS_CONTROLLER) {
      java.security.AccessController.checkPermission(new PropertyPermission(propname, "read"));
    }
    else if (com.acunia.wonka.SecurityConfiguration.USE_SECURITY_MANAGER) {
      if (theSecurityManager != null) {
        theSecurityManager.checkPropertyAccess(propname);
      }
    }
  }

  private static void propertiesCheck() {
    if (bootstrapping) {

      return;

    }

    if (com.acunia.wonka.SecurityConfiguration.USE_ACCESS_CONTROLLER) {
      java.security.AccessController.checkPermission(new PropertyPermission("*", "read,write"));
    }
    else if (com.acunia.wonka.SecurityConfiguration.USE_SECURITY_MANAGER) {
      if (theSecurityManager != null) {
        theSecurityManager.checkPropertiesAccess();
      }
    }
  }

  public static Properties getProperties() 
    throws SecurityException
  {
    propertiesCheck();

    return systemProperties;
  }

  public static void setProperties(Properties props) 
    throws SecurityException
  {
    propertiesCheck();

    systemProperties = props;
  }


  public static String getProperty(String key) 
    throws SecurityException
  {
    propertyCheck(key);

    try {

      return systemProperties.getProperty(key);

    }
    catch (NullPointerException npe) {

      return null;

    }
  }

    
  public static String getProperty(String key, String defaults)
    throws SecurityException
  {
    propertyCheck(key);

    try {

      return systemProperties.getProperty(key,defaults);

    }
    catch (NullPointerException npe) {

      return defaults;

    }
  }

  public static String setProperty(String key, String defaults)
    throws SecurityException
  {
    if (!bootstrapping) {
      if (com.acunia.wonka.SecurityConfiguration.USE_ACCESS_CONTROLLER) {
        java.security.AccessController.checkPermission(new PropertyPermission(key, "write"));
      }
      else if (com.acunia.wonka.SecurityConfiguration.USE_SECURITY_MANAGER) {
        if (theSecurityManager != null) {
          theSecurityManager.checkPermission(new PropertyPermission(key, "write"));
        }
      }
    }

    return (String)systemProperties.setProperty(key,defaults);
  }


  public static void exit(int status) 
    throws SecurityException
  {
    theRuntime.exit(status);
  }

  public static void gc() {
    theRuntime.gc();
  }

  public static void runFinalization() {
    theRuntime.runFinalization();
  }

  public static native char arrayelementtype(Object a);

  private static native void arraycopycheck(Object src, int srcOffset, Object dst, int dstOffset, int length);

  private static native String getdescriptor(Object o);

  private static native void bitarraycopy(Object src, int srcOffset, Object dst, int dstOffset, int length);

  private static native void bytearraycopy(Object src, int srcOffset, Object dst, int dstOffset, int length);

  private static native void shortarraycopy(Object src, int srcOffset, Object dst, int dstOffset, int length);

  private static native void longarraycopy(Object src, int srcOffset, Object dst, int dstOffset, int length);

  private static native void wordarraycopy(Object src, int srcOffset, Object dst, int dstOffset, int length);

  private static native void primitivearraycopy(Object src, int srcOffset, Object dst, int dstOffset, int length);

  private static native void referencearraycopy(Object src, int srcOffset, Object dst, int dstOffset, int length);

  public static void load(String libname) 
    throws SecurityException, UnsatisfiedLinkError
  {
    throw new UnsatisfiedLinkError("not implemented");
  }

  public static void loadLibrary(String libname) 
    throws SecurityException, UnsatisfiedLinkError
  {
    // throw new UnsatisfiedLinkError("not implemented");
    Runtime.getRuntime().loadLibrary(libname);
  }
  
  public String mapLibraryName(String libname) {
    return "lib" + libname + ".so";
  }
  
  public static void arraycopy(Object src, int srcOffset, Object dst, int dstOffset, int length)
    throws NullPointerException, ArrayStoreException, ArrayIndexOutOfBoundsException
  {
    if (src==null || dst==null) throw new NullPointerException();

    Class srcClass = src.getClass();
    Class dstClass = dst.getClass();

    if (!srcClass.isArray())
      throw new ArrayStoreException("source is not an array!");

    if (!dstClass.isArray())
      throw new ArrayStoreException("destination is not an array!");

    String srcclass = getdescriptor(src);
    String dstclass = getdescriptor(dst);

// The first char of each descriptor must be '[', but what comes next?

    char srcelementtype = srcclass.charAt(1);
    char dstelementtype = dstclass.charAt(1);

    switch (srcelementtype) {
    case 'Z':
      if (dstelementtype=='Z') {
        bitarraycopy(src, srcOffset, dst, dstOffset, length);
      }
      else {
        throw new ArrayStoreException("attempt to copy from Z to "+dstclass);
      }
      break;

    case 'B':
      if (dstelementtype=='B') {
        bytearraycopy(src, srcOffset, dst, dstOffset, length);
      }
      else {
        throw new ArrayStoreException("attempt to copy from B to "+dstclass);
      }
      break;

    case 'C':
      if (dstelementtype=='C') {
        shortarraycopy(src, srcOffset, dst, dstOffset, length);
      }
      else {
        throw new ArrayStoreException("attempt to copy from C to "+dstclass);
      }
      break;

    case 'S':
      if (dstelementtype=='S') {
        shortarraycopy(src, srcOffset, dst, dstOffset, length);
      }
      else {
        throw new ArrayStoreException("attempt to copy from "+srcclass+" to "+dstclass);
      }
      break;

    case 'I':
      if (dstelementtype=='I') {
        wordarraycopy(src, srcOffset, dst, dstOffset, length);
      }
      else {
        throw new ArrayStoreException("attempt to copy from "+srcclass+" to "+dstclass);
      }
      break;

    case 'F':
      if (dstelementtype=='F') {
        wordarraycopy(src, srcOffset, dst, dstOffset, length);
      }
      else {
        throw new ArrayStoreException("attempt to copy from "+srcclass+" to "+dstclass);
      }
      break;

    case 'J':
      if (dstelementtype=='J') {
        longarraycopy(src, srcOffset, dst, dstOffset, length);
      }
      else {
        throw new ArrayStoreException("attempt to copy from "+srcclass+" to "+dstclass);
      }
      break;

    case 'D':
      if (dstelementtype=='D') {
        longarraycopy(src, srcOffset, dst, dstOffset, length);
      }
      else {
        throw new ArrayStoreException("attempt to copy from "+srcclass+" to "+dstclass);
      }
      break;

    case 'L':
    case '[':
      referencearraycopy(src, srcOffset, dst, dstOffset, length);
      break;

    default:
      throw new ArrayStoreException(srcclass+"???????\n");}

  }

  public static native int identityHashCode(Object o);

}
