/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: Throwable.java,v 1.2 2002/02/26 09:10:52 gray Exp $
*/

package java.lang;

import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.Serializable;

public class Throwable implements Serializable {

  /*
  ** Note: this class is initialized "by hand" before the VM is fully
  ** initialized.  Consequently it must not have a static initializer.
  ** (It can have static variables, and even constant initial values
  ** for those variables, but nothing fancier and certainly no static{}
  ** clause.)
  */
  
  private final String detailMessage;

  public Throwable() {
    detailMessage = null;
  }

  public Throwable(String message) {
    detailMessage = message;
  }

  public String getMessage() {
    return detailMessage;
  }

  public String getLocalizedMessage() {
    return detailMessage;
  }

  public String toString() {
    return this.getClass().getName()+": "+this.detailMessage;
  }

  public native Throwable fillInStackTrace();

  public void printStackTrace() {
    printStackTrace(System.err);
  }

//  public native void printStackTrace(PrintStream s);
  public native String startStackTrace();
  public native String nextStackTrace();

  public void printStackTrace(PrintStream stream) {
    String tracerecord;

    stream.println("*** Start of stack trace");
    tracerecord = startStackTrace();
    stream.println(tracerecord);

    tracerecord = nextStackTrace();
    while (tracerecord!=null) {
      stream.println(tracerecord);
      tracerecord = nextStackTrace();
    }
    stream.println("*** End of stack trace");
  }

  public void printStackTrace(PrintWriter stream) {
    String tracerecord;

    stream.println("*** Start of stack trace");
    tracerecord = startStackTrace();
    stream.println(tracerecord);

    tracerecord = nextStackTrace();
    while (tracerecord!=null) {
      stream.println(tracerecord);
      tracerecord = nextStackTrace();
    }
    stream.println("*** End of stack trace");
  }

  protected native void finalize();


}
