/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: Array.java,v 1.1.1.1 2001/11/21 10:41:35 pollock Exp $
*/

package java.lang.reflect;

public final class Array {

  /*
  ** This class is a utility: it cannot be instantiated.
  */
  private Array(){}

  public static native Object newInstance(Class componentType, int length)
    throws NullPointerException, NegativeArraySizeException;
    
  public static native Object newInstance(Class componentType, int dimensions[])
    throws NullPointerException, IllegalArgumentException, NegativeArraySizeException;
  
  public static native int getLength(Object array)
    throws NullPointerException, IllegalArgumentException;
  
  public static native Object get(Object array, int index)
    throws NullPointerException, IllegalArgumentException, ArrayIndexOutOfBoundsException;

  public static boolean getBoolean(Object array, int index)
    throws NullPointerException, IllegalArgumentException, ArrayIndexOutOfBoundsException
  {
    boolean [] za;
    try {
      za = (boolean[])array;
    }
    catch (ClassCastException e) {
      throw new IllegalArgumentException();
    }
    return za[index];
  }

  public static byte getByte(Object array, int index)
    throws NullPointerException, IllegalArgumentException, ArrayIndexOutOfBoundsException
  {
    byte [] ba;
    try {
      ba = (byte[])array;
    }
    catch (ClassCastException e) {
      throw new IllegalArgumentException();
    }
    return ba[index];
  }

  public static char getChar(Object array, int index)
    throws NullPointerException, IllegalArgumentException, ArrayIndexOutOfBoundsException
  {
    char [] ca;
    try {
      ca = (char[])array;
    }
    catch (ClassCastException e) {
      throw new IllegalArgumentException();
    }
    return ca[index];
  }

  public static short getShort(Object array, int index)
    throws NullPointerException, IllegalArgumentException, ArrayIndexOutOfBoundsException
  {
      Class arrayclass = array.getClass().getComponentType();
      if ( arrayclass == null) throw new IllegalArgumentException("Object is not an Array");
      if (!arrayclass.isPrimitive()) throw new IllegalArgumentException("Object is not an Array of primitive types");
      if (!arrayclass.toString().equals("byte") && !arrayclass.toString().equals("short"))
       	throw new IllegalArgumentException("Object is not an Array of short or byte");

      Number nm;
      try {
          nm = (Number)Array.get(array , index);
          }
      catch (ClassCastException cce) { throw new IllegalArgumentException("Object is not an Array of numeric primitive types");}
      return nm.shortValue();
  }

  public static int getInt(Object array, int index)
    throws NullPointerException, IllegalArgumentException, ArrayIndexOutOfBoundsException
  {
      Class arrayclass = array.getClass().getComponentType();
      if ( arrayclass == null) throw new IllegalArgumentException("Object is not an Array");
      if (!arrayclass.isPrimitive()) throw new IllegalArgumentException("Object is not an Array of primitive types");
      if (arrayclass.toString().equals("char")) { return (int)getChar(array,index);}
      if (!arrayclass.toString().equals("byte") && !arrayclass.toString().equals("short") && !arrayclass.toString().equals("int"))
      	throw new IllegalArgumentException("wrong primitive array type");
      Number nm;
      try {
          nm = (Number)Array.get(array , index);
          }
      catch (ClassCastException cce) { throw new IllegalArgumentException("Object is not an Array of numeric primitive types");}
      return nm.intValue();
  }

  public static long getLong(Object array, int index)
    throws NullPointerException, IllegalArgumentException, ArrayIndexOutOfBoundsException
  {
      Class arrayclass = array.getClass().getComponentType();
      if ( arrayclass == null) throw new IllegalArgumentException("Object is not an Array");
      if (!arrayclass.isPrimitive()) throw new IllegalArgumentException("Object is not an Array of primitive types");
      if (arrayclass.toString().equals("char")) { return (long)getChar(array,index);}
      if (arrayclass.toString().equals("float") || arrayclass.toString().equals("double"))
      	throw new IllegalArgumentException("wrong primitive array type");
      Number nm;
      try {
          nm = (Number)Array.get(array , index);
          }
      catch (ClassCastException cce) { throw new IllegalArgumentException("Object is not an Array of numeric primitive types");}
      return nm.longValue();
  }

  public static float getFloat(Object array, int index)
    throws NullPointerException, IllegalArgumentException, ArrayIndexOutOfBoundsException
  {
      Class arrayclass = array.getClass().getComponentType();
      if ( arrayclass == null) throw new IllegalArgumentException("Object is not an Array");
      if (!arrayclass.isPrimitive()) throw new IllegalArgumentException("Object is not an Array of primitive types");
      if (arrayclass.toString().equals("char")) { return (float)getChar(array,index);}
      if (arrayclass.toString().equals("double"))
      	throw new IllegalArgumentException("wrong primitive array type");
      Number nm;
      try {
          nm = (Number)Array.get(array , index);
          }
      catch (ClassCastException cce) { throw new IllegalArgumentException("Object is not an Array of numeric primitive types");}
      return nm.floatValue();
  }

  public static double getDouble(Object array, int index)
    throws NullPointerException, IllegalArgumentException, ArrayIndexOutOfBoundsException
  {
      Class arrayclass = array.getClass().getComponentType();
      if ( arrayclass == null) throw new IllegalArgumentException("Object is not an Array");
      if (!arrayclass.isPrimitive()) throw new IllegalArgumentException("Object is not an Array of primitive types");
      if (arrayclass.toString().equals("char")) { return (double)getChar(array,index);}
      Number nm;
      try {
          nm = (Number)Array.get(array , index);
          }
      catch (ClassCastException cce) { throw new IllegalArgumentException("Object is not an Array of numeric primitive types");}
      return nm.doubleValue();

/*    double [] da;
    try {
      da = (double[]) array;
    }
    catch (ClassCastException e) {
      throw new IllegalArgumentException();
    }
    return da[index]; */
  }

  public static void set(Object array, int index, Object value)
    throws NullPointerException, IllegalArgumentException, ArrayIndexOutOfBoundsException {

  }

  public static void setBoolean(Object array, int index, boolean z)
    throws NullPointerException, IllegalArgumentException, ArrayIndexOutOfBoundsException {

  }

  public static void setByte(Object array, int index, byte b)
    throws NullPointerException, IllegalArgumentException, ArrayIndexOutOfBoundsException {

  }

  public static void setChar(Object array, int index, char c)
    throws NullPointerException, IllegalArgumentException, ArrayIndexOutOfBoundsException {

  }

  public static void setShort(Object array, int index, short s)
    throws NullPointerException, IllegalArgumentException, ArrayIndexOutOfBoundsException {

  }

  public static void setInt(Object array, int index, int i)
    throws NullPointerException, IllegalArgumentException, ArrayIndexOutOfBoundsException {

  }

  public static void setLong(Object array, int index, long l)
    throws NullPointerException, IllegalArgumentException, ArrayIndexOutOfBoundsException {

  }

  public static void setFloat(Object array, int index, float f)
    throws NullPointerException, IllegalArgumentException, ArrayIndexOutOfBoundsException {

  }

  public static void setDouble(Object array, int index, double d)
    throws NullPointerException, IllegalArgumentException, ArrayIndexOutOfBoundsException {

  }

}
