/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: URL.java,v 1.4 2002/03/08 12:59:13 gray Exp $
*/

package java.net;

import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;

public final class URL implements java.io.Serializable {

  private static final char OR='|';

  /**
   ** The factory for URL stream handlers, if one has been set
   ** (using setURLStreamHandlerFactory).
   */
  private static URLStreamHandlerFactory factory;

  /**
   ** Our cache of URL stream handlers, indexed by protocol.
   */
  private static Hashtable handlers;

  /**
   ** The protocol, i.e. the part before the first colon (:).
   */
  private String protocol;

  /**
   ** The protocol, i.e. the part after the last hash sign (#).
   */
  private String fragment;

  /**
   ** The host machine, if any (set by the URL stream handler).
   */
  private String host = "";

  /**
   ** The port to be used, or -1 for the default (set by the URL stream handler).
   */
  private int port = -1;

  /**
   ** The query string, if any (set by the URL stream handler).
   */
  private String query = "";

  /**
   ** The remainder, which may or not be the path to a file (set by the URL stream handler).
   */
  private String path;

  private int hashCode;

  private transient URLStreamHandler streamHandler;

  /**
   ** Constructor to build a URL from scratch, given its external form.
   ** We split the URL into its three most basic components: the protocol,
   ** the fragment indicator (if present), and "the rest".  We then use the
   ** protocol to identify a URLStreamHandler, and invoke that handler's  
   ** parseURL method to process the "rest" part of the URL.
   **
   ** @param	location	A string reresenting a URL.
   */
  public URL (String location) throws MalformedURLException {
      int colon = location.indexOf(":");
      if (colon < 0) {
        throw new MalformedURLException();
      }
      int hash = location.lastIndexOf("#");
      if (hash < 0) {
        hash = location.length();
      }
      else {
        fragment = location.substring(hash + 1);
      }

      if (location.startsWith("file://") && location.lastIndexOf('/') == 6) {
        this.protocol = "file";
        setHandler(this.protocol);
        streamHandler.parseURL(this, location, 7, hash);
com.acunia.wonka.Etc.woempa(7, "URL("+location+"): OTF hack engaged for "+location);
com.acunia.wonka.Etc.woempa(7, "URL("+location+"): "+streamHandler);
com.acunia.wonka.Etc.woempa(7, "URL("+location+"): "+this);
      }
      else if (colon>=0) {
        this.protocol = location.substring(0, colon);
        setHandler(this.protocol);
        streamHandler.parseURL(this, location, colon+1, hash);
com.acunia.wonka.Etc.woempa(7, "URL("+location+"): "+streamHandler);
com.acunia.wonka.Etc.woempa(7, "URL("+location+"): "+this);
      }
      else {
        throw new MalformedURLException();
      }
  }

  public URL(String protocol, String host, String path) throws MalformedURLException {
    this(protocol,host,-1,path,null);
  }

  public URL(String protocol, String host, int port, String File) throws MalformedURLException {
    this(protocol,host,port,File,null);
  }

  public URL(String protocol, String host, int port, String File, URLStreamHandler handler) throws MalformedURLException {
    if (protocol == null || host == null || File ==null) {
       throw new NullPointerException();
    }
    if (port < -1 || port > 65535) {
       throw new MalformedURLException("illegal port specified");
    }
    this.protocol = protocol;
    if (handler == null) {
      setHandler(this.protocol);
    }
    else {
       streamHandler = handler;
    }
    this.host = host;
    this.path = File;
  }

  public URL(URL context, String spec) throws MalformedURLException {
     this(context,spec, null);
  }

/**
** if spec is a relative url it should only contain a path (or path + fragment).
** The given context should be non-null in case of a relative url, otherwise null is allowed
** The specified handler can be null
*/
  public URL(URL context, String spec, URLStreamHandler handler) throws MalformedURLException {
    // we should find out if spec is a relative URL or not
    int i = spec.indexOf(':');
    if(i > -1) { // we have a winner it is a non relative URL ...
        protocol = spec.substring(0,i);
      if (handler == null) {
        setHandler(this.protocol);
      }
      else {
         streamHandler = handler;
      }
      streamHandler.parseURL(this,spec, i+1, spec.length());
      if (path == null || host == null) {
         throw new MalformedURLException();
      }
    }
    else { // we have a relative URL we copy protocol, host and port from ohter
        this.protocol = context.protocol;
        this.host = context.host;
        this.port = context.port;
        if (handler == null) {
          setHandler(this.protocol);
      }
      else {
         streamHandler = handler;
      }
      i = spec.indexOf('#');
      if ( i != -1) {
         path = context.path + spec.substring(0,i);
         fragment = spec.substring(i+1);
      }
      else {
         path = context.path + spec;
      }
    }      
  }

  public boolean equals(Object o) {
     if (!(o instanceof URL)){
           return false;
     }
     URL url = (URL)o;
     return (this.protocol.equals(url.protocol) && this.port == url.port
       && this.host.equals(url.host) && this.path.equals(url.path)
       && (fragment == null ?  url.fragment == null : this.fragment.equals(url.fragment)));    
  }  

  public final Object getContent()throws IOException {
    return openConnection().getContent();
  }

  public String getProtocol() {
    return this.protocol;
  }

  public String getHost() {
    return this.host;
  }

  public int getPort() {
    return this.port;
  }

  public String getFile() {
    return this.path;
  }

  public String getRef() {
    return this.fragment;
  }

  public String getQuery() {
    return this.query;
  }

  public int hashCode() {
        if (hashCode == 0) {
           hashCode = path.hashCode() ^ host.hashCode() ^ protocol.hashCode();
        }
        return hashCode;
  }

  public synchronized static void setURLStreamHandlerFactory(URLStreamHandlerFactory newfact) {
     if (newfact == null) {
       throw new NullPointerException();
     }
     factory = newfact;
  }

  public URLConnection openConnection() throws IOException {
    setHandler(this.protocol);
    com.acunia.wonka.Etc.woempa(7, "URL/openConnection: streamHandler = "+streamHandler);
    return streamHandler.openConnection(this);
  }

  public final InputStream openStream() throws IOException {
    return openConnection().getInputStream();

  }

  public boolean sameFile(URL url) {
     return (this.protocol.equals(url.protocol) && this.port == url.port
       && this.host.equals(url.host) && this.path.equals(url.path));
  }  

  public String toExternalForm() {
    try {
      setHandler(this.protocol);
      com.acunia.wonka.Etc.woempa(7, "URL/toExternalForm: streamHandler = "+streamHandler);
      return streamHandler.toExternalForm(this);
    }
    catch(IOException ioe) {
       com.acunia.wonka.Etc.woempa(7, "URL/toExternalForm: "+ioe);
       return protocol+"://"+host+(port == -1 ? "" : ":"+port)+"/"+path+(fragment == null ? "" :"#"+fragment);
    }
  }

  public String toString () {
     return toExternalForm();
  }

  protected void set(String protocol, String host, int port, String path, String ref){
  if ( host == null || path == null) {
     throw new NullPointerException();
  }
  if (port <-1 || port > 65535) {
     throw new IllegalArgumentException();
  }
  if (!protocol.equals(this.protocol)) {
        streamHandler = null;
  }
  this.protocol = protocol;
  this.host = host;
  this.port = port;
  this.path = path;
  this.fragment = ref;
  }


  private synchronized void setHandler(String protocol) throws MalformedURLException {
    if (streamHandler == null) {
      streamHandler = getHandler(protocol);
    }
  }

  private synchronized void setHandler() throws MalformedURLException {
    com.acunia.wonka.Etc.woempa(7, "URL/setHandler: protocol = "+protocol);
    if (factory == null) {
      String pkgs = GetSystemProperty.PROTOCOL_HANDLER_PKGS;
      String name;
      while (pkgs.length() > 0 ) {
        int i = pkgs.indexOf(OR);                  
          if(i == -1) {
            name = pkgs;
            pkgs = "";
          }
          else {
            name = pkgs.substring(0,i);
            pkgs = pkgs.substring(i+1);                  
          }
        name = name+"."+protocol+".Handler";
        try {
          streamHandler = (URLStreamHandler)Class.forName(name).newInstance();
          com.acunia.wonka.Etc.woempa(7, "URL/setHandler: streamHandler = "+streamHandler);

          return;                          

        }
        catch(Exception e){
          /*if we catch an Exception here we should continue looking ...*/
        }
      }

      throw new MalformedURLException("no Handler found for the "+protocol+" protocol");
           
    }
    else {
      streamHandler = factory.createURLStreamHandler(protocol);
      com.acunia.wonka.Etc.woempa(7, "URL/setHandler: factory created streamHandler = "+streamHandler);
    }
  }

// package protected methods to update an URL.

  void setHost(String host) {
    if (host == null) {
       throw new NullPointerException();
    }
    hashCode=0;
    this.host = host;
  }  
  void setProtocol(String protocol) {
    if (!protocol.equals(this.protocol)) {
          streamHandler = null;
      hashCode=0;
      this.protocol = protocol;
    }
  }  
  void setFile(String path) {
    if (path == null) {
       throw new NullPointerException();
    }
    hashCode=0;
    this.path = path;
  }  
  void setRef(String ref) {
    this.fragment = ref;
  }  
  void setQuery(String query) {
    this.query = query;
  }  
  void setPort(int port) {
    if (port <-1 || port > 65535) {
       throw new IllegalArgumentException();
    }
    this.port = port;
  }  

  /**
   ** Find the URLStreamHandler which is associated with a given protocol.
   ** We cache the ones we already found in a Hashtable, so we never try
   ** to create the same URLStreamHandler twice.
   */

  private static synchronized URLStreamHandler getHandler(String protocol) throws MalformedURLException {
    URLStreamHandler handler = null;

    com.acunia.wonka.Etc.woempa(7, "URL/getHandler: protocol = "+protocol);
    if (handlers == null) {
      handlers = new Hashtable(11);
    }
    else {
      handler = (URLStreamHandler)handlers.get(protocol);
    }

    if (handler != null) {
      com.acunia.wonka.Etc.woempa(7, "URL/getHandler: found cached handler "+handler);

      return handler;

    }

    if (factory != null) {
      handler = factory.createURLStreamHandler(protocol);
      com.acunia.wonka.Etc.woempa(7, "URL/getHandler: factory created streamHandler = "+handler);
      handlers.put(protocol, handler);

      return handler; 

    }

    String pkgs = GetSystemProperty.PROTOCOL_HANDLER_PKGS;
    String name;
    while (pkgs.length() > 0 ) {
      int i = pkgs.indexOf(OR);                  
        if(i == -1) {
          name = pkgs;
          pkgs = "";
        }
        else {
          name = pkgs.substring(0,i);
          pkgs = pkgs.substring(i+1);                  
        }
      name = name+"."+protocol+".Handler";
      try {
        handler = (URLStreamHandler)Class.forName(name).newInstance();
        com.acunia.wonka.Etc.woempa(7, "URL/getHandler: handler = "+handler);
        handlers.put(protocol, handler);

        return handler;

      }
      catch(Exception e){
        /*if we catch an Exception here we should continue looking ...*/
      }
    }

    throw new MalformedURLException("no Handler found for the "+protocol+" protocol");
  }
}
