/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/



package java.net;

import java.security.SecureClassLoader;
import java.security.PermissionCollection;
import java.security.CodeSource;
import java.util.jar.Manifest;
import java.util.jar.Attributes;
import java.io.InputStream;
import java.io.IOException;
import java.util.Vector;
import java.util.Enumeration;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;

/**
 *
 * $Id: URLClassLoader.java,v 1.5 2002/03/13 08:54:07 gray Exp $
 *
 */
public class URLClassLoader extends SecureClassLoader{

  URL [] urls;
  URLConnection connection[];
  URLStreamHandlerFactory factory;

  private static final Manifest emptyMan = new Manifest();

  private static void permissionCheck() {
    if (com.acunia.wonka.SecurityConfiguration.USE_ACCESS_CONTROLLER) {
    // Note: we assume that java.net is a restricted package.
      java.security.AccessController.checkPermission(new RuntimePermission("accessClassInPackage.java.net"));
    }
    else if (com.acunia.wonka.SecurityConfiguration.USE_SECURITY_MANAGER) {
      SecurityManager sm = System.getSecurityManager();
      if (sm != null) {
        sm.checkPackageAccess("java.net");
      }
    }
  }

  public URLClassLoader(URL[] urls) throws SecurityException {
    prepare(urls);
  }

  public URLClassLoader(URL[] urls, ClassLoader parent) throws SecurityException {
    super(parent);
    prepare(urls);
  }

  public URLClassLoader(URL[] urls, ClassLoader parent, URLStreamHandlerFactory factory) throws SecurityException {
    super(parent);
    this.factory = factory;
    prepare(urls);
  }

  public static URLClassLoader newInstance(final URL[] urls) throws SecurityException{
    permissionCheck();
    //TODO: do as a privileged action
    return new URLClassLoader(urls);
  }

  public static URLClassLoader newInstance(final URL[] urls, ClassLoader parent) throws SecurityException{
    permissionCheck();
    //TODO: do as a privileged action
    return new URLClassLoader(urls, parent);
  }

  protected void addURL(URL url){
    if (url == null){
      throw new NullPointerException();
    }
    try {
      URLConnection[] newconn = new URLConnection[urls.length+1];
      System.arraycopy(connection, 0, newconn, 0, urls.length);
      newconn[urls.length] = url.openConnection();
      connection = newconn;
      URL[] newurls = new URL[urls.length+1];
      System.arraycopy(urls, 0, newurls, 0, urls.length);
      newurls[urls.length] = url;
      urls = newurls;
    }
    catch (IOException ioe) {
      ioe.printStackTrace();
    }
  }

  protected Package definePackage(String pname, Manifest man, URL url) throws IllegalArgumentException {
    Attributes attr = man.getMainAttributes();
    String specTitle = attr.getValue("Specification-Title");
    String specVersion  = attr.getValue("Specification-Version");
    String specVendor = attr.getValue("Specification-Vendor");
    String implTitle  = attr.getValue("Implementation-Title");
    String implVersion  = attr.getValue("Implementation-Version");
    String implVendor   = attr.getValue("Implementation-Vendor");
    return definePackage(pname, specTitle, specVersion, specVendor, implTitle, implVersion, implVendor, url);
  }

  protected Class findClass(String name) throws ClassNotFoundException {
    String filename = name.replace('.','/') + ".class";
    URL    url = null;
    byte[] bytes = null;
    int    length = 0;
    InputStream is;

    com.acunia.wonka.Etc.woempa(7, "URLClassLoader/findClass: Using "+this+" to find "+name);
    for (int i=0; bytes == null && i < urls.length ; i++){
      URLConnection conn = connection[i];
      url = urls[i];

      try {
        if (conn instanceof JarURLConnection) {
          JarURLConnection jurlconn = (JarURLConnection)conn;
          JarEntry jentry = jurlconn.getJarEntry();
          JarFile jfile = jurlconn.getJarFile();
          if (jentry == null) {
            ZipEntry ze = jfile.getEntry(filename);
            com.acunia.wonka.Etc.woempa(7, "URLClassLoader/findClass: "+jurlconn+"/"+filename+" -> "+ze);
            if (ze != null) {
              length = (int) ze.getSize();
              bytes = new byte[length];
              is = jfile.getInputStream(ze);
              for (int j = 0; j < length;) {
                j += is.read(bytes, j, length - j);
              }
            }
            // else not found, continue search
          }
          else { // jarfile, but only one entry applies
                 // Not supported for now (is it allowed???)
            continue;
          }
        }
        else { // Must be a directory ...
          com.acunia.wonka.Etc.woempa(7, "URLClassLoader/findClass: Dir URL: "+url);
          URL cat = new URL(url, filename);
          com.acunia.wonka.Etc.woempa(7, "URLClassLoader/findClass: look for "+cat);
          URLConnection c = cat.openConnection();
          com.acunia.wonka.Etc.woempa(7, "URLClassLoader/findClass: open "+c);
          if (c instanceof HttpURLConnection) {
            if (((HttpURLConnection)c).getResponseCode() != HttpURLConnection.HTTP_OK) {
            // Ignore this one
              continue;
            }
          }
          is = c.getInputStream();
          bytes = new byte[65536];
          length = 0;
          for (int j = 0; length < 65536 ;) {
            j = is.read(bytes, length, 65536 - length);
            if(j == -1){
              break;
            }
            length += j;
          }
        }
      }
      catch (IOException ioe) {
        // If it happened when trying to open a file, just try the next file.
        // If it happened during a file, we need to throw away what we read.
        bytes = null;
      }

      if (bytes == null) {
        com.acunia.wonka.Etc.woempa(7, "URLClassLoader/findClass: still looking for "+name);
      }
    }

    if (bytes != null) {
      int idx = name.lastIndexOf('.');
      String pname = name.substring(0,idx == -1 ? 0 : idx);
      com.acunia.wonka.Etc.woempa(7, "URLClassLoader/findClass: found "+name+", package is "+pname);
      // TODO: get manifest right
      if (getPackage(pname) == null){
        definePackage(pname, emptyMan , url);
      }

      com.acunia.wonka.Etc.woempa(7, "URLClassLoader/findClass: defining "+name);
      return defineClass(name, bytes, 0, length);

    }

    throw new ClassNotFoundException();
  }

  public URL findResource(String name){
  // TODO; wasn't there something about leading slash?
    String filename = name;//.replace('.','/');
    URL    url;
    URL    result = null;
    byte[] bytes = null;
    int    length = 0;
    InputStream is;

    for (int i=0; result == null && i < urls.length ; i++){
      URLConnection conn = connection[i];
      url = urls[i];

      try {
        if (conn instanceof JarURLConnection) {
          JarURLConnection jurlconn = (JarURLConnection)conn;
          JarEntry entry = jurlconn.getJarEntry();
          JarFile jfile = jurlconn.getJarFile();
          if (entry == null) {
            ZipEntry ze = jfile.getEntry(filename);
            if (ze != null) {
              result = new URL(url, filename);
            }
            // else not found, continue search
          }
          else { // jarfile, but only one entry applies
            if (entry.getName() == filename) {
              result = url;
            }
            // else not found, continue search
          }
        }
        else { // Must be a directory ...
          URL cat = new URL(url, filename);
          URLConnection c = cat.openConnection();
          if (c instanceof HttpURLConnection) {
            if (((HttpURLConnection)c).getResponseCode() != HttpURLConnection.HTTP_OK) {
            // Ignore this one
              continue;
            }
          }
          else {
            // Force an exception to be thrown if the file does not exist
            c.connect();
          }
          result = cat;
        }
      }
      catch (IOException ioe) {
        // If it happened when trying to open a file, just try the next file.
        // If it happened during a file, we need to throw away what we read.
        result = null;
      }
    }
    return result;
  }

  public Enumeration findResources(String name) throws IOException{
  // TODO; wasn't there something about leading slash?
    String filename = name.replace('.','/');
    URL    url;
    URL    result = null;
    byte[] bytes = null;
    int    length = 0;
    InputStream is;
    Vector v = new Vector();

    for (int i=0; i < urls.length ; i++){
      URLConnection conn = connection[i];
      url = urls[i];

      try {
        if (conn instanceof JarURLConnection) {
          JarURLConnection jurlconn = (JarURLConnection)conn;
          JarEntry entry = jurlconn.getJarEntry();
          JarFile jfile = jurlconn.getJarFile();
          if (entry == null) {
            ZipEntry ze = jfile.getEntry(filename);
            if (ze != null) {
              v.addElement(new URL(url, filename));
            }
            // else not found, continue search
          }
          else { // jarfile, but only one entry applies
            if (entry.getName() == filename) {
              v.addElement(url);
            }
            // else not found, continue search
          }
        }
        else { // Must be a directory ...
          URL cat = new URL(url, filename);
          URLConnection c = cat.openConnection();
          if (c instanceof HttpURLConnection) {
            if (((HttpURLConnection)c).getResponseCode() != HttpURLConnection.HTTP_OK) {
            // Ignore this one
              continue;
            }
          }
          v.addElement(cat);
        }
      }
      catch (IOException ioe) {
      }
    }

    return v.elements();
  }

  protected PermissionCollection getPermissions(CodeSource codes){
        PermissionCollection pc = super.getPermissions(codes);
        //ToDo add extra permissions to allow usage of the URL's
        return pc;
  }

/**
** returns a clone of the original URL array (for securiry reasons).
*/
  public URL[] getURLs(){
    return (URL[])urls.clone();
  }

  private void prepare(URL[] urls) {
    int length = urls.length;
    this.urls = new URL[length];
    connection = new URLConnection[length];
    for (int i = 0; i < length; ++i) {
      String name = urls[i].toString();
      com.acunia.wonka.Etc.woempa(7, "URLClassLoader/prepare: urls["+i+"] = "+name);
      if (name.endsWith("/")) {
        this.urls[i] = urls[i];
      }
      else {
        com.acunia.wonka.Etc.woempa(7, "URLClassLoader/prepare:   name = "+name);
        try {
// This should work, no?
          this.urls[i] = new URL("jar:" + name + "!/");
          com.acunia.wonka.Etc.woempa(7, "URLClassLoader/prepare: url = "+this.urls[i]);
          com.acunia.wonka.Etc.woempa(7, "URLClassLoader/prepare: protocol = "+this.urls[i].getProtocol());
        }
        catch (java.net.MalformedURLException mue) {
          mue.printStackTrace();
        }
      }

      try {
        connection[i] = this.urls[i].openConnection();
        com.acunia.wonka.Etc.woempa(7, "URLClassLoader/prepare: opened connection["+i+"] = "+connection[i]);
      }
      catch (IOException ioe) {
        ioe.printStackTrace();
      }
    }
  }

}

