/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


package java.net;

import java.io.IOException;

public abstract class URLStreamHandler {

  protected abstract URLConnection openConnection(URL url) throws IOException ;
  
/**
**  The default implementation of parseURL parses an "http-like" URL:
**  <ul><li>If the first two characters are `//', then everything up to
**          the next `/' is treated as `host & port'.
**      <li>If present, `host and port' consist of either host:port or
**          just `host', where `port' is a number and `host' is a string.
**      <li>Everything after the last `?' is a query string.
**      <li>The rest is `the path'.
**  </ul>
**
**  @param start points to the character after the colon (':') indicating the 
**  end of the protocol
**  @param end   points to the character after the end of the URL spec.
**  @status implemented
**  @remark uses package protected method to update the url ...
*/  
  protected void parseURL(URL url, String spec, int start, int end) {
    int here = start;
    int there = end;

    try {
      if (end > start + 2 && spec.substring(start, start + 2).equals("//")) {
        int third_slash = spec.indexOf("/", start + 2);
        if (third_slash < 0) {
          third_slash = end;
        }
        String host_and_port = spec.substring(start+2, third_slash);
        com.acunia.wonka.Etc.woempa(7, "URLStreamHandler/parseURL: host & port ="+host_and_port);
        int colon = host_and_port.indexOf(':');
        if (colon < 0) {
          url.setHost(host_and_port);
        }
        else {
          com.acunia.wonka.Etc.woempa(7, "URLStreamHandler/parseURL: setting host to "+host_and_port.substring(0, colon));
          url.setHost(host_and_port.substring(0, colon));
          com.acunia.wonka.Etc.woempa(7, "URLStreamHandler/parseURL: setting port to "+host_and_port.substring(colon+1, third_slash));
          url.setPort(Integer.parseInt(host_and_port.substring(colon + 1, third_slash)));
        }

        here = third_slash;
      }

      int query = spec.lastIndexOf("?");
      if (query >= here) {
        com.acunia.wonka.Etc.woempa(7, "URLStreamHandler/parseURL: setting query to "+spec.substring(query+1));
        url.setQuery(spec.substring(query + 1));
        there = query;
      }

      int hash = spec.lastIndexOf("#", there);
      if (hash >= here) {
        com.acunia.wonka.Etc.woempa(7, "URLStreamHandler/parseURL: setting query to "+spec.substring(hash+1, there));
        url.setRef(spec.substring(hash + 1, there));
        there = hash;
      }

      com.acunia.wonka.Etc.woempa(7, "URLStreamHandler/parseURL: setting file to "+spec.substring(here, there));
      url.setFile(spec.substring(here, there));

    }
    catch(Exception e){ /* all exceptions cause a silent return */ }
  }

 /**
  ** Reverse the parsing of a URL, i.e. recreate the string from which it was
  ** created.  Again, the default implementation assumes a "http-like" syntax..
  */
  protected String toExternalForm(URL url) {
    StringBuffer buf = new StringBuffer(url.getProtocol());
    buf.append("://");
    buf.append(url.getHost());
    if (url.getPort() != -1) {
      buf.append(':');
      buf.append(url.getPort());       
    }
    buf.append('/');
    buf.append(url.getFile());

    String query = url.getQuery();
    if (query != null) {
      buf.append('?');
      buf.append(query);               
    }

    String ref = url.getRef();
    if (ref != null) {
      buf.append('#');
      buf.append(ref);               
    }

    com.acunia.wonka.Etc.woempa(7, "URLStreamHandler/toExternalForm: "+buf);
    return buf.toString();
  }

/**
**  @status implemented
**  @remark uses package protected methods to update the url ...
*/  
  protected void setURL(URL url, String protocol, String host, int port, String file, String ref) {
     url.set(protocol, host, port, file, ref);
  }
  
}
