/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/**
 * $Id: AccessControlContext.java,v 1.3 2002/02/25 10:09:49 ruelens Exp $
 */

package java.security;

import java.util.Enumeration;
import java.util.Vector;

/** An AccessControlContext is a set of ProtectionDomains.
 **@author ACUNIA NV
 **@version $Version:$
*/
public final class AccessControlContext {

  /**
   ** We store the ProtectionDomain's in a Vector, ignoring duplicates.
   */
  Vector domains;

  /** The first time the hashcode is requested we cache it for later use.
   */
  int hashcode;

  /** We also cache the last permission for which checkPermission() succeeded.
   */
  Permission o_k_perm;

  /** The main public constructor.
   */
  public AccessControlContext (ProtectionDomain[] context) {
    domains = new Vector(context.length);
    for (int i = 0; i < context.length; ++i) {
      domains.addElement(context[i]);
    }
    com.acunia.wonka.Etc.woempa(7,"A:"+domains.toString());
//    domains.trimToSize();
  }

  /** Our special constructor.
   ** The second parameter is used to pre-populate the Vector
   ** with the ProtectionDomains of an existing context.
   */
  AccessControlContext (ProtectionDomain[] context, AccessControlContext acc) {
    if (acc == null) {
      domains = new Vector();
    }
    else {
      domains = (Vector)acc.domains.clone();
    }

    ProtectionDomain pd;
    for (int i = 0; i < context.length; ++i) {
      pd = context[i];
      if (!domains.contains(pd)) {
        domains.addElement(pd);
      }
    }
    com.acunia.wonka.Etc.woempa(7,"B:"+domains.toString());
//    domains.trimToSize();
  }

  /** Another special constructor.
   ** The second and third parameters are used to pre-populate the Vector
   ** with the ProtectionDomains of existing contexts.
   */
  AccessControlContext (ProtectionDomain[] context, AccessControlContext acc1, AccessControlContext acc2) {
    if (acc1 == null) {
      domains = new Vector();
    }
    else {
      domains = (Vector)acc1.domains.clone();
    }

    ProtectionDomain pd;

    if (acc2 != null) {
      Enumeration acc2enum = acc2.domains.elements();
      while (acc2enum.hasMoreElements()) {
        pd = (ProtectionDomain)acc2enum.nextElement();
        if (!domains.contains(pd)) {
          domains.addElement(pd);
        }
      }
    }

    for (int i = 0; i < context.length; ++i) {
      pd = context[i];
      if (!domains.contains(pd)) {
        domains.addElement(pd);
      }
    }
    com.acunia.wonka.Etc.woempa(7,"C:"+domains.toString());
//    domains.trimToSize();
  }

  /** Check that the given Permission is implied by every domain in this context.
   */
  public void checkPermission (Permission perm) throws AccessControlException {
    if (perm.equals(o_k_perm)) {
      return;
    }

    Enumeration pdenum = domains.elements();
    while(pdenum.hasMoreElements()) {
      ProtectionDomain pd = (ProtectionDomain)pdenum.nextElement();
      if (!pd.implies(perm)) {
        throw new AccessControlException("Not implied by "+pd, perm);
      }
    }
    o_k_perm = perm;
  }

  /** Two AccessControlContexts are considered equal if they contain the same elements (not necessarily in the same order).
   */
  public boolean equals(Object o) {
     if(!(o instanceof AccessControlContext)){
      return false;
    }
    AccessControlContext that = (AccessControlContext)o;
    if (this.domains.size() != that.domains.size()) {

      return false;

    }

    Enumeration pdenum = this.domains.elements();
    while(pdenum.hasMoreElements()) {
      ProtectionDomain pd = (ProtectionDomain)pdenum.nextElement();
      if (!that.domains.contains(pd)) {

        return false;

      }
    }

    return true;
  }

  public int hashCode() {
    if (hashcode == 0) {
      Enumeration pdenum = this.domains.elements();
      while(pdenum.hasMoreElements()) {
        ProtectionDomain pd = (ProtectionDomain)pdenum.nextElement();
        hashcode ^= pd.hashCode();
      }
    }
    return hashcode;
  }

}
