/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: KeyStore.java,v 1.3 2002/03/15 13:57:03 ruelens Exp $
*/

package java.security;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.util.Date;
import java.util.Enumeration;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;

public class KeyStore {

  public static String getDefaultType(){
    return "none";
  }

  public static KeyStore getInstance(String type) throws NoSuchAlgorithmException {
    SecurityAction action = new SecurityAction(type, "KeyStore.");
    return new KeyStore((KeyStoreSpi)action.spi, action.provider, type);
  }

  public static KeyStore getInstance(String type, String provider)
    throws NoSuchAlgorithmException, NoSuchProviderException {

    SecurityAction action = new SecurityAction(type, provider, "KeyStore.");
    return new KeyStore((KeyStoreSpi)action.spi, action.provider, type);
  }

  private KeyStoreSpi keySpi;
  private Provider provider;
  private String type;

  protected KeyStore(KeyStoreSpi keyStoreSpi, Provider provider, String type){
    this.keySpi = keyStoreSpi;
    this.provider = provider;
    this.type = type;
  }

  public final Enumeration aliases() throws KeyStoreException {
    return keySpi.engineAliases();
  }

  public final boolean containsAlias(String alias) throws KeyStoreException {
    return keySpi.engineContainsAlias(alias);
  }

  public final void deleteEntry(String alias) throws KeyStoreException {
    keySpi.engineDeleteEntry(alias);
  }

  public final Certificate getCertificate(String alias) throws KeyStoreException {
    return keySpi.engineGetCertificate(alias);
  }

  public final String getCertificateAlias(Certificate cert) throws KeyStoreException {
    return keySpi.engineGetCertificateAlias(cert);
  }

  public final Certificate[] getCertificateChain(String alias) throws KeyStoreException {
    return keySpi.engineGetCertificateChain(alias);
  }

  public final Date getCreationDate(String alias) throws KeyStoreException {
    return keySpi.engineGetCreationDate(alias);
  }

  public final Key getKey(String alias, char[] password)
    throws KeyStoreException, NoSuchAlgorithmException, UnrecoverableKeyException {
    return keySpi.engineGetKey(alias, password);
  }

  public final Provider getProvider(){
    return provider;
  }

  public final String getType(){
    return type;
  }

  public final boolean isCertificateEntry(String alias) throws KeyStoreException {
    return keySpi.engineIsCertificateEntry(alias);
  }

  public final boolean isKeyEntry(String alias)  throws KeyStoreException {
    return keySpi.engineIsKeyEntry(alias);
  }

  public final void load(InputStream stream, char[] password)
    throws IOException, NoSuchAlgorithmException, CertificateException{
    keySpi.engineLoad(stream, password);
  }

  public final void setCertificateEntry(String alias, Certificate cert) throws KeyStoreException {
    keySpi.engineSetCertificateEntry(alias, cert);
  }

  public final void setKeyEntry(String alias, byte[] key, Certificate[] chain) throws KeyStoreException {
    keySpi.engineSetKeyEntry(alias, key, chain);
  }

  public final void setKeyEntry(String alias, Key key, char[] password, Certificate[] chain) throws KeyStoreException {
    keySpi.engineSetKeyEntry(alias, key, password, chain);
  }

  public final int size() throws KeyStoreException {
    return keySpi.engineSize();
  }

  public final void store(OutputStream stream, char[] password)
    throws IOException, NoSuchAlgorithmException, CertificateException  {

    keySpi.engineStore(stream, password);
  }
}