/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/**
 * $Id: AbstractMap.java,v 1.1.1.1 2001/11/21 10:41:35 pollock Exp $
 */

package java.util;

public abstract class AbstractMap implements Map {

  private transient AbstractSet keySet = null;
  private transient AbstractCollection valuecol = null;

  protected AbstractMap() {
  }

  public int size() {
    return entrySet().size();
  }

  public boolean isEmpty() {
    return (size()==0);
  }

  public boolean containsValue(Object value) {
    Set maps = entrySet();
    boolean found = false;
    Iterator it = maps.iterator();
    while ( (!(found)) && (it.hasNext()) ) {
      Map.Entry myEntry = (Map.Entry)it.next();
      if ((value == null ? myEntry.getValue()== null : value.equals(myEntry.getValue()))) {
        found = true;
      }
    }
    return found;
  }

  public boolean containsKey(Object key) {
    Set maps = entrySet();
    boolean found = false;
    Iterator it = maps.iterator();
    while ( (!(found)) && (it.hasNext()) ) {
      Map.Entry myEntry = (Map.Entry)it.next();
      if ((key == null ? myEntry.getKey()== null : key.equals(myEntry.getKey()))) {
        found = true;
      }
    }
    return found;
  }

  public Object get(Object key) {
    Object answer = null;
    Set maps = entrySet();
    boolean found = false;
    Iterator it = maps.iterator();
    while ( (!(found)) && (it.hasNext()) ) {
      Map.Entry myEntry = (Map.Entry)it.next();
      if ((key == null ? myEntry.getKey()== null : key.equals(myEntry.getKey()))) {
        found = true;
        answer = myEntry.getValue();
      }
    }
    return answer;
  }

  public Object put(Object key, Object value) throws
UnsupportedOperationException {
    throw new UnsupportedOperationException();
  }

  public Object remove(Object key) {
    Object answer = null;
    Set maps = entrySet();
    boolean found = false;
    Iterator it = maps.iterator();
    while ( (!(found)) && (it.hasNext()) ) {
      Map.Entry myEntry = (Map.Entry)it.next();
      if  ((key == null ? myEntry.getKey()== null : key.equals(myEntry.getKey()))){
        found = true;
        answer = myEntry.getValue();
        it.remove();
      }
    }
    return answer;
  }

  public void putAll(Map t) {
    Iterator addIt = t.entrySet().iterator();
    while (addIt.hasNext()) {
      Map.Entry myEntry = (Map.Entry)addIt.next();
      put (myEntry.getKey(),myEntry.getValue());
    }

  }

  public void clear() {
    entrySet().clear();
  }

  public Set keySet() {
    if (keySet == null) {
      keySet = new KSet();
    }
    return keySet;

  }

  public Collection values() {
  	if (valuecol == null) {
  		valuecol = new VCollection();
  	}
  	return valuecol;
  	
  }

  public abstract Set entrySet();

  public boolean equals (Object o) {
    if (!(o instanceof Map)) return false;
    Map target = (Map)o;
    Object value;
    if (!this.keySet().equals(target.keySet()))
           return false;
    Iterator it = target.entrySet().iterator();
    Map.Entry me;
    while (it.hasNext()) {
    	me = (Map.Entry) it.next();
    	value = me.getValue();
    	if (!(value==null ? get(me.getKey())==null : value.equals(get(me.getKey()))))
            return false;
    }
    return true;
  }

  public int hashCode() {
    int answer = 0;
    Set maps = entrySet();
    Iterator it = maps.iterator();
    boolean go = it.hasNext();
    while (go) {
      Map.Entry myEntry = (Map.Entry)it.next();
      answer += myEntry.hashCode();
      go = it.hasNext();
    }
    return answer;
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append("{");
    Set maps = entrySet();
    Iterator it = maps.iterator();
    boolean go = it.hasNext();
    while (go) {
      Map.Entry myEntry = (Map.Entry)it.next();
      sb.append(String.valueOf(myEntry.getKey()));
      sb.append("=");
      sb.append(String.valueOf(myEntry.getValue()));
      go = it.hasNext();
      if (go) {
        sb.append(", ");
      }
    }
    sb.append("}");
    return sb.toString();
  }

  private class KSet extends AbstractSet {
   	
  	public KSet() {}

  	public boolean contains(Object e) {
  		return containsKey(e);
  	}
  	
  	public Iterator iterator() {
  	 	return new KIterator();
  	}
  	
  	public int size() {
    		return entrySet().size();
  	}
  }

  private class KIterator implements Iterator {

  	private Iterator it=entrySet().iterator();
   	
   	public KIterator() {}
   	
   	public boolean hasNext() {
   	  	return it.hasNext();
   	}
	
   	public Object next() {
   		return ((Map.Entry)it.next()).getKey();
   	}
   	public void remove() {
   	 	it.remove();
   	}
  }

  private class VIterator implements Iterator {

  	private Iterator it=entrySet().iterator();
   	
   	public VIterator() {}
   	
   	public boolean hasNext() {
   	  	return it.hasNext();
   	}
	
   	public Object next() {
   		return ((Map.Entry)it.next()).getValue();
   	}
   	public void remove() {
   	 	it.remove();
   	}
  }

  private class VCollection extends AbstractCollection {
   	
  	public VCollection() {}

  	public boolean contains(Object e) {
  		return containsValue(e);
  	}
  	
  	public Iterator iterator() {
  	 	return new VIterator();
  	}
  	
  	public int size() {
    		return entrySet().size();
  	}
  }
}
