/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: BitSet.java,v 1.1.1.1 2001/11/21 10:41:35 pollock Exp $
*/

package java.util;
/*
* the current implementation of BitSet doesn't allow the BitSet to shrink
* the size will only grow !!!
* a clear will at a position larger then the size will grow the BitSet
*
*/
public final class BitSet implements Cloneable {

  private boolean theBits[];
  private int     currentLength;

/*
*  the size of the bitset is always rounded up to the next 64 increment
*/
  private int roundup (int n) {
    return ((n+63)/64)*64;
  }

  public BitSet() {
    theBits = new boolean[64];
    currentLength = 0;
  }
  
  public BitSet(int nbits) {
    if (nbits < 0 ) throw new NegativeArraySizeException();
    theBits = new boolean[roundup(nbits)];
    currentLength = 0;
  }

  public String toString() {
    if (currentLength<=0) return "{}";

    int i;
    StringBuffer buf = new StringBuffer();

    for (i=0;i<currentLength;++i) {
      if (theBits[i]) {
        buf.append(i);
        buf.append(", ");
      }
    }

    buf.setLength(buf.length()-2);

    return "{"+buf+"}";
  }

  public synchronized boolean equals(Object obj) {
    if (obj == null || !(obj instanceof BitSet)) {
      
      return false;

    }

    BitSet that = (BitSet)obj;
    int i;
    int lthis = currentLength;
    int lthat = that.size();
    int lmin  = lthis<lthat? lthis : lthat;

    for (i=0;i<lmin;++i) {
      if (theBits[i]!=that.get(i)) {

        return false;

      }
    }
    if (lthis>lthat) {
      for (i=lthat;i<lthis;++i) {
        if (theBits[i]) {

          return false;

        }
      }
    }
    if (lthis<lthat) {
      for (i=lthis;i<lthat;++i) {
        if (that.get(i)) {

          return false;

        }
      }
    }

      return true;

  }
  
  public synchronized int hashCode() {
    int i;
    long bits[] = new long[(currentLength+63)/64];

    for (i=0;i<currentLength;++i) {
      if (theBits[i]) bits[i/64] |= 1L << (i%64);
    }

    long h = 1234;

    for (i=bits.length-1;i>=0;i--) {
      h ^= bits[i] * (i+1);
    }

    return (int)((h>>32)^h);

  }
  
  public synchronized Object clone() {
    int i;
    BitSet theClone = new BitSet(roundup(currentLength));

    for (i=0;i<currentLength;++i) {
      if (theBits[i]) theClone.set(i);
    }

    return theClone;

  }

  
  public boolean get(int bitIndex) {

    return bitIndex>=currentLength ? false : theBits[bitIndex];

  }
  
  private void grow(int newLength) {
    int i,n;

    n = roundup(newLength);
    if (theBits==null) {
      theBits = new boolean[n];
    }
    else if (theBits.length<n) {
      boolean newBits[] = new boolean[n];
      for (i=0;i<theBits.length;++i) {
        if (theBits[i]) newBits[i] = true;
      }
      theBits = newBits;
    }
  }
/*
  private void shrink() {
    int i,n;

    n = roundup(currentLength);
    if (theBits.length>n*2) {
      boolean newBits[] = new boolean[n];
      for (i=0;i<currentLength;++i) {
        if (theBits[i]) newBits[i] = true;
      }
      theBits = newBits;
    }
  }
*/
  public synchronized void set(int bitIndex) {
    if (bitIndex>=currentLength) {
       currentLength = bitIndex+1;
       grow(bitIndex+1);
    }
    theBits[bitIndex] = true;
  }
/*
* clear can make the BitSet grow (this is not implemented in JDK)
* if you clear a position greater than the current size
*/
  public synchronized void clear(int bitIndex) {
    if (bitIndex>=theBits.length) {
      grow(bitIndex+1);
    }
    theBits[bitIndex] = false;
    if (bitIndex==currentLength-1) {
      for (currentLength=currentLength-1;currentLength>0;currentLength--) {
        if (theBits[currentLength-1]) break;
      }
    }
//    shrink();
  }
  
/*
*  if the parameter bitset is larger than this BitSet, then all those extra bits are
*  ignored this is not so in the JDK 1.2, but we follow the spefication mentioned in
*  The Java class libraries Second edition volume 1 by Chan, Lee and Kramer
*/
  public void and(BitSet bitset) {
    int i,l,s;
    s = size();
    l = bitset.size();
    for (i=0;i<l&&i<currentLength;i++) {
      if (!bitset.get(i)) clear(i);
    }
    for (i=l ; i < s ; i++) {
    clear(i);
    }
  }

/*
*  if the parameter bitset is larger than this BitSet, then all those extra bits are
*  ignored this is not so in the JDK 1.2, but we follow the spefication mentioned in
*  The Java class libraries Second edition volume 1 by Chan, Lee and Kramer
*/
  public void or(BitSet bitset) {
    int i;
    int l = Math.min(bitset.size() , size());
    for (i=0 ; i < l ; ++i) {
      if (bitset.get(i)) set(i);
    }
  }

/*
*  if the parameter bitset is larger than this BitSet, then all those extra bits are
*  ignored this is not so in the JDK 1.2, but we follow the spefication mentioned in
*  The Java class libraries Second edition volume 1 by Chan, Lee and Kramer
*/
  public synchronized void xor(BitSet bitset) {
    int i,l;
    l = Math.min(bitset.size(),size());
    boolean b;
    for (i=0;i<l;i++) {
       b= bitset.get(i)^theBits[i];
       if (b != theBits[i])
       { if (b) set(i);
         else  clear(i);
       }
      }
  }

  public int size() {
    return theBits.length;
  }

/*
* this method is added in JDK 1.2
*/
  public int length() {
    return currentLength;
  }	
/*
*  this method is added in JDK 1.2
*
*  if the parameter bitset is larger than this BitSet, then all those extra bits are
*  ignored this is not so in the JDK 1.2, but we follow the spefication mentioned in
*  The Java class libraries Second edition volume 1 by Chan, Lee and Kramer
*/
  public void andNot(BitSet bs) {
    int i,l,s;
    l = bs.size();
    for (i=0;i<l&&i<currentLength;++i) {
      if (bs.get(i)) clear(i);
    }
  }
}
