/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: Random.java,v 1.1.1.1 2001/11/21 10:41:35 pollock Exp $
*/

package java.util;

import com.acunia.wonka.PersistentRoot;

public class Random {

  protected long seed;
  protected double nextNextGaussian;
  protected boolean haveNextNextGaussian = false;

  private Long saved_seed;

  public Random() {
  // According to Sun's API this should use the current time of day as seed.
  // However it's quite possible that we don't yet know the time of day (no
  // GPS signal yet), so instead we try to find a saved value in persistent
  // storage.  If that fails we use currentTimeMillis (which may not be so
  // random, if this is being called as part of a repeatable startup sequence).
    if (saved_seed == null) {
      PersistentRoot S = PersistentRoot.getInstance();
      if (S != null) {
        saved_seed = getSavedSeed(S);
      }
      else {
        System.out.println("Unable to find PersistentRoot S!!!");
      }

      if (saved_seed != null) {
        System.out.println("Found saved seed "+saved_seed+".");
      }
      else {
        saved_seed = new Long(System.currentTimeMillis());
        System.out.println("Did not find saved seed, setting seed to "+saved_seed+".");
      }

      this.seed = (saved_seed.longValue() ^ 0x5DEECE66DL) & ((1L<<48) - 1);

      if (S != null) {
        setSavedSeed(S);
      }
    }

    setSeed(saved_seed.longValue());
  }

  public Random(long seed) {
    setSeed(seed);
  }

  public void setSeed(long seed) {
    this.seed = (seed ^ 0x5DEECE66DL) & ((1L<<48) - 1);
    haveNextNextGaussian = false;
  }

  private Long getSavedSeed(PersistentRoot S) {
    Long result;

    return (Long)S.restorePersistentObject("java.util.Random.seed");
  }

  private void setSavedSeed(PersistentRoot S) {
    long next_seed = (seed * 0x5DEECE66DL + 0xBL) & ((1L << 48) - 1);
    S.createPersistentObject("java.util.Random.seed",new Long(next_seed));
    System.out.println("Saving next_seed "+next_seed+".");
  }

  protected int next(int bits) {
    seed = (seed * 0x5DEECE66DL + 0xBL) & ((1L << 48) - 1);
    return (int)(seed >>> (48-bits));
  }

  public int nextInt() {
    return next(32);
  }

  public int nextInt(int n) {
  	if (n<=0) {
   		throw new IllegalArgumentException("n must be positive");
     	}
        if ((n & -n) == n)  {// i.e., n is a power of 2
         	return (int)((n * (long)next(31)) >> 31);
        }
        int bits, val;
        do {
        	bits = next(31);
                val = bits % n;
        } while(bits - val + (n-1) < 0);
        return val;
  }

  public long nextLong() {
    return ((long)next(32)<<32) + next(32);
  }

  public float nextFloat() {
    return next(24) / ((float)(1<<24));
  }

  public double nextDouble() {
    return (((long)next(26)<<27) + next(27)) / (double)(1L<<53);
  }

  public double nextGaussian(){
    if (haveNextNextGaussian) {
      haveNextNextGaussian = false;
      return nextNextGaussian;
    }
    else {
      double v1, v2, s;
      do {
        v1 = 2 * nextDouble() - 1;
        v2 = 2 * nextDouble() - 1;
        s = v1 * v1 + v2 * v2;
      } while (s >= 1);
      double norm = Math.sqrt(-2 * Math.log(s)/s);
      nextNextGaussian = v2 * norm;
      haveNextNextGaussian = true;
      return v1 * norm;
    }
  }

  public void nextBytes(byte [] buf){
   	for (int i=0; i < buf.length ; i++){
   	 	buf[i] = (byte)next(8);
   	}
  }

  public boolean nextBoolean() {
  	return next(1) != 0;
  }
}

