/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/**
 * $Id: ZipEntry.java,v 1.1.1.1 2001/11/21 10:41:35 pollock Exp $
 */
package java.util.zip;

public class ZipEntry implements ZipConstants {

  public static final int DEFLATED=8;
  public static final int STORED=0;	

  private ZipFile zipFile;

  private String comment=null;
  private String  name;
  private int method=-1;
  private long time=-1;
  private long crc=-1;
  private long size=-1; //uncompressed size
  private long compressedSize=-1;
  private byte[] extra=null;//extra data...

  /**
  ** this field is only used by ZipFile !
  */
  long pointer;// position of Header in ZipFile

  public ZipEntry(ZipEntry ze) {
    this.name = ze.name;
    this.comment = ze.comment;
    this.method = ze.method;
    this.time = ze.time;
    this.crc = ze.crc;
    this.size = ze.size;
    this.compressedSize = ze.compressedSize;
    this.extra = ze.extra;
    this.pointer = ze.pointer;
  }
  public ZipEntry(String zname) {
  	if (zname.length() > 65535) {
         	throw new IllegalArgumentException();
  	}
  	name = zname;
  }

  public String getComment() {
   	return comment;
  }

  public long getCompressedSize(){
   	return compressedSize;
  }

  public long getCrc(){
   	return crc;
  }

  public byte[] getExtra() {
   	return extra;
  }

  public int getMethod(){
   	return method;
  }

   public String getName() {
    return this.name;
  }

  public long getSize() {
  	return size;
  }

  public long getTime() {
   	return time;
  }

  public boolean isDirectory() {
  	return name.endsWith("/");
  }

  public void setComment(String cmnt) {
 	if(cmnt != null) {
 		if (cmnt.length() > 65535){
  			throw new IllegalArgumentException();   	
  	 	}
  	}
  	comment = cmnt;
  }

  public void setCrc(long CRC) {
   	crc = CRC;
  }

  public void setExtra(byte[] xtra) {
   	if (xtra != null) {
   	 	if (xtra.length > 65535){
   	 	 	throw new IllegalArgumentException();
   	 	}
   	}
   	extra = xtra;
  }

  public void setMethod(int m) {
   	if (m != STORED && m != DEFLATED) {
   	 	throw new IllegalArgumentException();
   	}
   	method = m;   	
  }

  public void setSize(long sz) {
   	if (sz < 0 ) {
   	 	throw new IllegalArgumentException();
   	}
   	size = sz;
  }

  public void setTime(long t) {
   	time = t;
  }
/**
** remove this Method no part of Standard API
**/
  public String toString() {
    String answer;
    answer = "ZipEntry: "+getName();
    return answer;
  }

  public void setCompressedSize(long cpsize){
   	compressedSize = cpsize;
  }


}
