/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/**
 * $Id: ZipFile.java,v 1.3 2002/03/12 14:46:59 ruelens Exp $
 */

package java.util.zip;

import java.io.File;
import java.io.RandomAccessFile;
import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;

/**
 ** Class for manipulating zip files.
 ** IMPORTANT: if you add any set...() methods to this class, be sure to
 ** override them in com.acunia.wonka.net.jar.ImmutableJarFile!
 */
public class ZipFile implements ZipConstants {

  private final static byte last = endCenDirS[3];

  private HashMap entries;
  private File file;
  private int cache;

  /**
  ** This is an optimization for using JarFile
  */
  private ZipEntry newZipEntry(String name){
   	try {
   		if (Class.forName("java.util.jar.JarFile").isInstance(this)){
   	 		return new java.util.jar.JarEntry(name);
   		} 	
   		return new ZipEntry(name);
   	} catch(ClassNotFoundException e){
   		return new ZipEntry(name);   	
   	}
  }



  public ZipFile (String name) throws IOException {
  	this(new File(name));
  }

  public ZipFile (File file) throws ZipException, IOException {
  	this.file = file;
  	getEntries();
  	if (entries == null){
 	   	throw new ZipException("Bad or No ZipFile "); 	
  	}
  }

  public void close() throws IOException {
   	entries=new HashMap(0); 	
  }

  public Enumeration entries(){
  	return new Enum();
  }

  public ZipEntry getEntry(String zname) {
  	return (ZipEntry)entries.get(zname);
  }

  public String getName(){
   	return file.getName();
  }

  public InputStream getInputStream(ZipEntry ze) throws IOException{
    ZipEntry zipE = (ZipEntry)entries.get(ze.getName());
    RandomAccessFile raf = new RandomAccessFile(file ,"r");
    raf.seek(zipE.pointer);
    return new ByteArrayInputStream(checkHeader(raf,ze));
  }

  /**
  ** since jdk 1.2 ...
  */
  public int size() {
   	return entries.size();
  }	

  private void getEntries() throws IOException {
       	//our file system uses buffers of at least 1024 bytes
       	RandomAccessFile raf = new RandomAccessFile(file ,"r");
       	long pos = raf.length();
       	int size = (1024 > pos ? (int)pos : 1024);
       	byte [] bytes = new byte[size];
       	while (pos > 0) {
       		pos -= size;
       		raf.seek(pos);
       		raf.readFully(bytes,0,size);
       		size = locateEndCD(bytes, size);
       		if (size == -1) {
       		 	break;
       		}
       		if (size > 0) {
       		 	System.arraycopy(bytes,0,bytes,bytes.length-size,size);
       		}
	       	size = (1024 > pos ? (int)pos : 1024)-size;
       	}
  }

  private int locateEndCD(byte[] bytes, int size) throws IOException {
	//we start looking for  endCenDirS
        for (int p=size  ; --p > -1 ; ) {
         	if (bytes[p] == last) {
         		//this might be the last byte of the signature!
         		if (p >=3) {
         		 	if (bytes[p-1] == endCenDirS[2] && bytes[p-2] == endCenDirS[1]
         		 		&& bytes[p-3] == endCenDirS[0]) {
         		 		//we have found a signature ...
         		 		if(readEntries(bytes, size, p+1)){
         		 		   	return -1;
         		 		}
         		        }
         		}
         		else {
         			return p;         		
         		} 	
         	}
        }
        return 0;
  }

  private boolean readEntries(byte [] bytes, int size, int pos) throws IOException {
   	int nrE = size - pos;
   	nrE = (16 > nrE ? nrE : 16);
	byte [] b = new byte [16];
   	System.arraycopy(bytes, pos, b,0, nrE);
 	RandomAccessFile raf = new RandomAccessFile(file, "r");
   	if (nrE < 16){
   		raf.readFully(b,nrE,16-nrE);
   	}
   	//b should contain 16 bytes of the possible header
   	nrE = (0x0ff&(char)b[6]) + (0x0ff &(char)b[7])*256;
   	long stP = ZipInputStream.bytesToLong(b, 12);
  	size = (int)ZipInputStream.bytesToLong(b, 8);
   	if (nrE < 0 || stP+(long)size > raf.length() || size < 0) {
   		// we encountered a fony header
		return false;
   	}
   	raf.seek(stP);
   	b = new byte[size];
   	raf.readFully(b);
   	pos=0;
   	entries = new HashMap(((int)(nrE/0.74f))+1);
   	for (int i=0; i < nrE ; i++) {
   	        if (pos+46 > size) {
   		 	throw new ZipException();//bad Header ...   			
   	        }
   		if (cenFileHeaderS[0] != b[pos] || cenFileHeaderS[1] != b[pos+1]
   			|| cenFileHeaderS[2] != b[pos+2] || cenFileHeaderS[3] != b[pos+3]){
   		 	throw new ZipException();//bad Header ...   			
   		}
   		// we have found a correct header ...
   		// lets skip useless bytes
   		pos+=28;
   		int len = (0x0ff&(char)b[pos]) + (0x0ff &(char)b[pos+1])*256;
   		pos+=18;
   		if (pos+len > size) {
   		 	throw new ZipException();
   		}	
   		ZipEntry ze = newZipEntry(new String(b,pos,len));
   		pos-=36;
   		int hlp = (0x0ff&(char)b[pos]) + (0x0ff &(char)b[pos+1])*256;
   		if (hlp != 0 && hlp != 8) {
   		 	throw new ZipException();
   		}
   		ze.setMethod(hlp);
   		pos+=2;
   		ze.setTime(ZipInputStream.getDate(b,pos));
   		pos+=4;
   		ze.setCrc(ZipInputStream.bytesToLong(b,pos));
   		pos+=4;
   		ze.setCompressedSize(ZipInputStream.bytesToLong(b,pos));
   		pos+=4;
   		ze.setSize(ZipInputStream.bytesToLong(b,pos));   		
   		pos+=6;
   		hlp = (0x0ff&(char)b[pos]) + (0x0ff &(char)b[pos+1])*256;
   		pos+=2;
   		int com = (0x0ff&(char)b[pos]) + (0x0ff &(char)b[pos+1])*256;
   		if(hlp < 0 || com < 0 || pos+14+len+hlp+com > size) {
   		 	throw new ZipException();
   		}
   		pos+=10;
   		if(entries.put(ze.getName(),ze) != null){
   		 	throw new ZipException("duplicate entry Found");
   		}
   		ze.pointer = ZipInputStream.bytesToLong(b,pos);
   		pos+= 4+len;
   		if (hlp > 0) {
   		 	byte [] extra = new byte[hlp];
   		 	System.arraycopy(b,pos,extra,0,hlp);
   		 	ze.setExtra(extra);
   		 	pos += hlp;
   		}
   		if (com > 0) {
   		 	ze.setComment(new String(b,pos,com));
   		 	pos += com;
   		}
   	}
   	return true;
  }


  private byte[] checkHeader(RandomAccessFile raf, ZipEntry ze) throws IOException {
 	byte [] header = new byte [30];
        raf.readFully(header,0,30);
	if ( header[0] != locFileHeaderS[0] || header[1] != locFileHeaderS[1]
	      || header[2] != locFileHeaderS[2] || header[3] != locFileHeaderS[3]) {
	 	throw new ZipException("corrupt header found");
	}	

	// we have 30 bytes --> 4 are already checked  Signature
	// version needed --> ... (2 bytes)
	// general purpose bytes (2)
	int mode = (0x0ff&(char)header[4]) + (0x0ff &(char)header[5])*256;
	if (mode > 20) {
	 	throw new ZipException("higher zip version needed");
	}
	mode = (0x08&(0x0ff&(char)header[6]));
	int fnlen = (0x0ff&(char)header[26]) + (0x0ff &(char)header[27])*256;
	byte [] fname = new byte[fnlen];
	raf.readFully(fname,0, fnlen);
	String name = new String(fname, 0, fnlen);
	if (!ze.getName().equals(name)) {
	 	throw new ZipException("entry pointed to wrong data");
	}
	fnlen = (0x0ff&(char)header[28]) + (0x0ff &(char)header[29])*256;
	fname = new byte[fnlen];
	if (fnlen != 0) {
		raf.readFully(fname,0, fnlen);
	}
	mode = (0x0ff&(char)header[8]) + (0x0ff &(char)header[9])*256;	
        if (mode != 8 && mode != 0) {
	 	      throw new ZipException("stream is corrupted");
        }
        if (mode == 8) {
        	Inflater inf = new Inflater(true);
        	header = new byte[(int) ze.getCompressedSize()];
        	raf.readFully(header);
        	inf.setInput(header);
        	header = new byte[(int) ze.getSize()];
	        try {
			      int ret = inf.inflate(header);
            CRC32 crc = new CRC32();
            crc.update(header, 0, ret);
            if (crc.getValue() != ze.getCrc()) throw new ZipException("Bad CRC.");
		      }
		      catch(DataFormatException dfe){
		 	      throw new ZipException();
		      }
		      finally{
		        inf.end();
		      }     	
        }
        else {
           	header = new byte[(int) ze.getSize()];
           	raf.readFully(header);
        }
        return header;
	// the rest of the bytes are not verified	
  }

  private class Enum implements Enumeration  {
  	
  	private Iterator it = entries.values().iterator(); 	

        public boolean hasMoreElements() {
         	return it.hasNext();
        }
        public Object nextElement(){
        	return it.next();
        }

  }
}
