/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/**
 * $Id: ZipInputStream.java,v 1.2 2001/11/23 13:36:55 dumon Exp $
 */
package java.util.zip;

import java.io.InputStream;
import java.io.IOException;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.Date;
import java.util.GregorianCalendar;

/**
**	Basic InputStream for reading zip formatted streams.
** GENERAL NOTE:
**	we use in.read(byte[] , int, int) to read bytes.  In some occasions we
**	might not get all the bytes we want (not yet available in the inputstream in)
**      there might be some bytes left in buf.  Whenever we need bytes we ask them from
**	getBytes([BII) --> this method will take the remaining bytes from buf or else
**	it will get them directly from in ...
*/
public class ZipInputStream extends InflaterInputStream implements ZipConstants {
				
	private boolean closed=false;
        private CRC32 crc = new CRC32();
        private LinkedList entries = new LinkedList();
        	
	private ZipEntry currentEntry=null;
	private boolean entryOpen=false;
  private boolean preset=false;
  private boolean inflating;
  private boolean inCentDir=false;
	private boolean allowDataDescriptor;
	private long dataCount;
	private int used;
        	
	public ZipInputStream(InputStream in) {
	 	super(in, new Inflater(true));
	}
	
	protected ZipEntry createZipEntry(String zname) {
		return new ZipEntry(zname);
	}
	
	public int available() throws IOException{
	 	return (closed ? 0 : 1);
	}
	public void close() throws IOException {
		if (!closed) {
			if (entryOpen) {
				closeEntry();
			}
			super.close();
			closed=true;
			currentEntry=null;
		}
	}

//TODO ...	

/**
** skip will only skip bytes within the current Entry ...
** call getNextEntry() to skip more data
**
*/	
	public long skip(long n) throws IOException {
	 	return super.skip(n);
	}

	public ZipEntry getNextEntry() throws IOException {
	 	closeEntry();
    readLFHeader();
    if(inflating && entryOpen){
      //if there is no entryOpen there is no need to preset the buf
     	presetBuf();
    }	
	 	return currentEntry;
	}
		
	public void closeEntry() throws IOException {
    if (entryOpen){	
  		// we discard all remaining data by calling read ...
  		byte [] data = new byte [128];
  		while(read(data,0,128) != -1) {}
  	  used = len - inf.getRemaining();  	
  		if (preset) {
  		  if (inflating) {
  		   	if (currentEntry.getCrc() != crc.getValue() ||
  		   		currentEntry.getSize() != inf.getTotalOut() ||
  		   		currentEntry.getCompressedSize() == (inf.getTotalIn()+skipTillNextHeader())) {
  		   		throw new ZipException("data was corrupted");	
  		   	}
  		  }
  		  else {
  		   	if (currentEntry.getCrc() != crc.getValue()	|| dataCount != 0) {
  		      throw new ZipException("data was corrupted");			        		
  		    }					
  	    }
  		}
  		else {
  		 	skipTillNextHeader();
  		 	if (getBytes(data,0,16) < 16) {
  		    throw new ZipException("data was corrupted");			        				 	
  		 	}
  		 	if (data[0] != dataDescS[0] || data[1] != dataDescS[1] ||
  		 	    data[2] != dataDescS[2] || data[3] != dataDescS[3] ||
  		 	    crc.getValue() != bytesToLong(data, 4)) {
  		   	throw new ZipException("corrupt dataDescriptor");			        				 	
  		 	}		 	
  		 	currentEntry.setCrc(crc.getValue());
  		 	currentEntry.setCompressedSize(bytesToLong(data, 8));
  		 	currentEntry.setSize(bytesToLong(data, 12));		
  		}
  		entryOpen=false;
  	}	
	}
	
/**
** read will only read bytes in the current Entry ...
** call getNextEntry to read more bytes ...
**
*/	
	public int read(byte[] buff, int offset, int len) throws IOException {
    int read=-1;
	  if(entryOpen) {
	  	if (inflating) {
		   	read = super.read(buff, offset, len);
	     	if (inf.needsDictionary()) {
	        throw new ZipException();
	     	}
	      dataCount -= read;
		  }	
	    else {
	     	if (dataCount > 0) {
	        len = (len > dataCount ? (int)dataCount : len);
	        read = getBytes(buff, offset, len);
	        dataCount -= read;
	      }	          	
	    }
	    if (read != -1) {
	      crc.update(buff, offset, read);
	    }
	  }
	  return read;
	}
	
	private void readLFHeader() throws IOException {
    byte [] header = new byte [22];
    int l = 0;
    while (true) {
      l = getBytes(header,l,22-l);
	    if (l == -1) {
	 	    // we reached the end of the stream ...
	 	    currentEntry=null;
	      break;
	    }
	    else {
	    	// we read some bytes and we expect this to be a header ...
	 	    if (header[0] != 80 || header[1] != 75 ){
	 	 	    throw new ZipException("corrupt header found");
	 	    }
	 	    if (header[2] == 3 && header[3] == 4) {
 	    		parseLocalFileHeader(header);
	 	      break;
	 	    }
	 	    else if (header[2] == 1 && header[3] == 2) {
	 	    	parseFileHeader(header);
	 	    	l = 0;
	 		  }
	 		  else if (header[2] == 5 && header[3] == 6) {
	 	      parseEndCD(header);
	 	      l = 0;
	      }
	 		  else if (allowDataDescriptor && header[2] == 7 && header[3] == 8) {
	        //lets hope we don't come here to often ...
	        allowDataDescriptor = false;
	 		    for(int i=4 ; i <16 ; i++){
	 		      if(header[i] != 0){
    	 	 	    throw new ZipException("header data is wrong"); 		
	 		      }	 		
	 		    }
	 		    l = 6;
	 		    System.arraycopy(header,16,header,0,6);
	      }
	 			else {
			 	  throw new ZipException("corrupt header found");			 		
	 	    }
	    }
	  }	
	}
		
/**
** when we locate a FileHeader, it means we found the central directory ...
** the FileHeaders should have been preceeded by a LocalFileHeader.  	
** So we have a reference in our LinkedList to this Header. If we don't
** find one or data is inconsistant then we throw a ZipException.
*/
  private void parseFileHeader(byte[] header) throws IOException {
  	// we have 22 bytes --> 4 are already checked  Signature
  	// version made by and version needed --> skip ... (4 bytes)
  	// general purpose bytes (2) skip
  	byte [] buff = new byte [46];
  	System.arraycopy(header,0,buff,0,22);
  	// we read some more bytes since the header has 46 bytes ...
  	if ( getBytes(buff, 22, 24) < 24 ) {
  	 	throw new ZipException("bytes missing --> corrupt header");
  	}
  	int mode = (0x0ff&(char)buff[10]) + (0x0ff &(char)buff[11])*256;
    long time = getDate(buff , 12);
    long crc32 = bytesToLong(buff, 16);
    long compSize = bytesToLong(buff, 20);
    long size = bytesToLong(buff, 24);
  	int fnlen = (0x0ff&(char)buff[28]) + (0x0ff &(char)buff[29])*256;
  	byte [] fname = new byte[fnlen];
  	getBytes(fname,0, fnlen);
  	String name = new String(fname, 0, fnlen);
  	ListIterator li = entries.listIterator(0);
  	ZipEntry ze=null;
  	while (li.hasNext()) {
  		ze = (ZipEntry) li.next();
  		if (name.equals(ze.getName())) {
  		 	break;
  		}			
  	}
  	if (ze == null) {
  	 	throw new ZipException("corrupt header");	
  	}	
  	if ( time != ze.getTime() || crc32 != ze.getCrc() || size != ze.getSize() || compSize != ze.getCompressedSize()) {
  	 	throw new ZipException("corrupt header");
  	}
  	fnlen = (0x0ff&(char)buff[30]) + (0x0ff &(char)buff[31])*256;
  	fname = new byte[fnlen];
  	if (fnlen == 0) {
  		ze.setExtra(null);	 	
  	}
  	else {
  		getBytes(fname,0, fnlen);
  		ze.setExtra(fname);	 	
  	}
  	fnlen = (0x0ff&(char)buff[32]) + (0x0ff &(char)buff[33])*256;
  	fname = new byte[fnlen];
  	if (fnlen == 0) {
  		ze.setComment(null);	 	
  	}
  	else {
  		getBytes(fname,0, fnlen);
  		ze.setComment(new String(fname, 0, fnlen));	 	
  	}
  	inCentDir=true;
  	// the rest of the bytes are not verified	
  }

  private void parseLocalFileHeader(byte[] header) throws IOException {
	// we have 22 bytes --> 4 are already checked  Signature
	// version needed --> ... (2 bytes)
	// general purpose bytes (2)
	if(inCentDir) {
	 	throw new ZipException("stream is corrupted");
	}
	int mode = (0x0ff&(char)header[4]) + (0x0ff &(char)header[5])*256;
	if (mode > 20) {
	 	throw new ZipException("higher zip version needed");
	}
	mode = (0x08&(0x0ff&(char)header[6]));
	// we read some more bytes since the header has 30 bytes ...
	if ( getBytes(header, 0, 8) < 8 ) {
	 	throw new ZipException("bytes missing --> corrupt header");
	}
	int fnlen = (0x0ff&(char)header[4]) + (0x0ff &(char)header[5])*256;
	byte [] fname = new byte[fnlen];
	getBytes(fname,0, fnlen);
	String name = new String(fname, 0, fnlen);
	ListIterator li = entries.listIterator(0);
	ZipEntry ze;
	while (li.hasNext()) {
		ze = (ZipEntry) li.next();
		if (name.equals(ze.getName())) {
		 	throw new ZipException("file name already exist");
		}		
	}	
	ze = createZipEntry(name);
	if (mode != 8) {
		preset = true;
    ze.setCrc(bytesToLong(header, 14));
    ze.setCompressedSize(bytesToLong(header, 18));
	  ze.setSize(bytesToLong(header, 0));
	}
	else {
	 	preset = false;
	}
	mode = (0x0ff&(char)header[8]) + (0x0ff &(char)header[9])*256;	
  if (mode != 8 && mode != 0) {
	 	throw new ZipException("stream is corrupted");
  }
  if (mode == 8) {
    inflating=true;
  }
  else {
   	if (!preset) {
	   	throw new ZipException("stream is corrupted");         	
    }
    inflating=false;
    dataCount = ze.getSize();
  }
	inf.reset();
  ze.setMethod(mode);
  ze.setTime(getDate(header , 10));
	fnlen = (0x0ff&(char)header[6]) + (0x0ff &(char)header[7])*256;
	fname = new byte[fnlen];
	if (fnlen == 0) {
		ze.setExtra(null);	 	
	}
	else {
		getBytes(fname,0, fnlen);
		ze.setExtra(fname);	 	
	}
	entries.addLast(ze);
	currentEntry=ze;
  crc.reset();
  if(preset && ze.getCompressedSize() == 0){
    entryOpen=false;	
    allowDataDescriptor = true;
  }
	else {
    entryOpen=true;	
	}
	// the rest of the bytes are not verified	
	
  }
  private void parseEndCD(byte[] header) throws IOException {
	  if (((0x0ff&(char)header[10]) + (0x0ff &(char)header[11])*256) != entries.size()) {
	 	  throw new ZipException("corrupt header");
	  }
    int commentSize = ((0x0ff&(char)header[20]) + (0x0ff &(char)header[21])*256);
	  getBytes(new byte[commentSize],0,commentSize);
	  entries.clear();
	  inCentDir = false;	                                                                         	
  }
   	
/**
**	this method is static (default acces) so ZipFile is able call it
**
**	Note on the Time format:
**		time(first 2 bytes):
**			bit 0-4    : seconds / 2 	(0-29)
**			bit 5-10   : minute        	(0-59)
**			bit 11-15  : hour of 24h clock	(0-23)
**		date(last 2 bytes) :
**			bit 0-4    : day of month 	(1-31)
**			bit 5-8    : month 		(1-12)
**			bit 9-15   : year - 1980
*/	
  static long getDate(byte [] header, int offset) {
   	int hour = (0x0ff&(char)header[offset]) + (0x0ff &(char)header[offset+1])*256;
   	int seconds = 2*(hour & 0x1f);
   	hour = (hour>>>5);
   	int minutes = (hour & 0x3f);
   	hour = (hour>>>6);
   	int year = (0x0ff&(char)header[offset+2]) + (0x0ff &(char)header[offset+3])*256;
   	int day = (year & 0x1f);
   	year = (year>>>5);
   	int month = (year & 0x0f)-1;
   	year = (year>>>4)+1980;	
   	GregorianCalendar gc = new GregorianCalendar(year, month, day, hour, minutes, seconds);
   	return gc.getTime().getTime();
  }
	
/**
**	this method is static (default acces) so ZipFile is able call it
*/	
  static long bytesToLong(byte [] header, int offset) {
  	long result = (0x0ff&(char)header[offset++]);
  	result += (0x0ff &(char)header[offset++])<<8;
  	result += (0x0ff&(char)header[offset++])<<16;
  	result += ((long)(0x0ff &(char)header[offset]))<<24;			
   	return result;
  }	

  /**
  ** while reading through the stream some bytes are read and put into buf.
  ** if we need bytes we should check if there are unread bytes in buf
  ** getBytes will read bytes from buf or in (if necessary)
  */
  private int getBytes(byte [] target, int offset, int length) throws IOException {
	int rd = 0;
	while (length > 0){		
		int get = len - used;
		//System.out.println("DEBUG: getBytes get ="+get+", len = "+len+", used = "+used);
		get = (get > length ? length : get);
		if (get > 0) {
		 	System.arraycopy(buf, used, target, offset, get);
		 	used +=get;
		}
		else {
		 	get = in.read(target, offset, length);
		 	if (get == -1) {
		 		if (rd == 0){
		 		 	rd--;
		 		}
		 	 	break;	
		 	}		 	
		}
	 	length -= get;
	 	rd +=get;
	 	offset+=get;
	}
	return rd;
  }

  private void presetBuf(){
    len -= used;
    System.arraycopy(buf, used, buf, 0, len);
	  inf.setInput(buf,0,len);
  }

  private int skipTillNextHeader() throws IOException{
   	int skipped=0;
   	byte [] head = new byte[1];
   	getBytes(head,0,1);
   	while (head[0] != 80) {
   	 	skipped++;
	   	getBytes(head,0,1);   	 	
   	}
   	used--;
   	buf[used] = head[0];   	
   	return skipped;
  }
}
