/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: main.c,v 1.9 2002/04/08 11:10:21 vandeneede Exp $
*/

#include <unistd.h>
#include "argument.h" 
#include "misc.h"
#include "oswald.h"

Wonka_InitArgs  system_InitArgs;

char *woempa_dump_file = NULL;
int   woempa_stderr = 1;
int   woempa_bytecodecount   = 0;
int   woempa_bytecodetrigger = 0;

/*
** PutString will write to standard error.
*/
void debug_puts(char *s) {
  write(2, s, strlen(s));
}

/*
** Help-style command line options and the texts which go with them.
*/
char *help_option_names[] = {"-help", "-?", "-X", "-W", "-version", NULL};

const char *help_texts[] = {
  "Usage: wonka [-options] [class [args...]]\n"
  "                                If wonka is started without a class, a default class is used.\n"
  "                                The default class can be changed in wo nka's system.properties.\n"
  " options:\n"
  "   -cp -classpath <classpath>   set search path for classes and resour ces\n"
  "   -D<name>=<value>             set a system property\n"
  "   -version                     print version\n"
  "   -? -help                     print this explanation\n"
  "   -X                           print help on non-standard options\n"
  "   -W                           print help on Wonka specific options\n"
  "\n"
,
  "Usage: wonka [-options] [class [args...]]\n"
  "                                If wonka is started without a class, a default class is used.\n"
  "                                The default class can be changed in wo nka's system.properties.\n"
  " options:\n"
  "   -cp -classpath <classpath>   set search path for classes and resour ces\n"
  "   -D<name>=<value>             set a system property\n"
  "   -version                     print version\n"
  "   -? -help                     print this explanation\n"
  "   -X                           print help on non-standard options\n"
  "   -W                           print help on Wonka specific options\n"
  "\n"
,
  "  -Xmx=<size>         set maximum Java heap size (doesn't do anything) \n"
  "  -Xbootclasspath:<classpath>\n"
  "                      set the bootclasspath. The first element must be a jar\n"
  "                      or zip file which contains the system classes, e.g.\n"
  "                      /system/wre.jar .\n"
#ifdef JDWP
  "  -Xdebug             enable remote debugging\n"
  "  -Xjdwp:...          set jdwp options. (use -Xjdwp:help for more info)\n"
#endif
  "\n"
,
#ifdef DEBUG
  "  -Wlogfile=<file>    Enable dumping the debug output to a file.\n"
  "  -Woempa=y|n         Show debug output on stderr. (default: y)\n"
  "  -Wdebugfrom=<count> Enable level 1 debugging from bytecode count\n"
#endif
#if defined(AWT_XSIM) || defined(AWT_NONE)
  "  -Wrudolph:<w>x<h>   Change the geometry of the AWT window on X.\n"
#endif
  "\n"
,
"Wonka version " VERSION_STRING "\n"
};

/*
** If there is only one argument and it is one of -help, -?, -X, -W, or -version,
** justVisiting() prints an appropriate text and returns WONKA_TRUE.  Otherwise
** WONKA_FALSE is returned.
*/
w_boolean justVisiting(int argument_count, char *arguments[]) {
  char ** cptr = help_option_names;
  char  *argument = arguments[1];

  if (argument_count == 2) {
    while (*cptr) {
      if (strcmp(argument, *cptr) == 0) {
        PutString((char *)help_texts[cptr - help_option_names]);

        return WONKA_TRUE;
      }
      ++cptr;
    }
  }
  return WONKA_FALSE;
}

/*
** Search the command line for a max heap size, and if found process and remove it.
*/
w_int getMaxHeapSize(int *argument_count_ptr, char *arguments[]) {
  w_int i;
  w_size j;
  char *mx = NULL;
  char  ch;
  w_int maxsize = 0;

  for (i = 1; i < *argument_count_ptr; ++i) {
    if (strncmp(arguments[i], "-Xmx", 4) == 0) {
      mx = arguments[i];
      PutString("Maximum heap size = ");
      PutString(mx+5);
      PutString("\n");

      for (j = 5; j < strlen(arguments[i]); ++j) {
        ch = mx[j];
        if (ch >= '0' && ch <= '9') {
          maxsize = maxsize * 10 + ch - '0';
        }
        else if (ch == 'M' || ch == 'm') {
          maxsize = maxsize * 1000000;
          break;
        }
        else if (ch == 'K' || ch == 'k') {
          maxsize = maxsize * 1000;
          break;
        }
        else {
          break;
        }
      }
    }
    else if (mx) {
      arguments[i - 1] = arguments[i];
    }
  }

  *argument_count_ptr -= !!mx;

  return maxsize;
}

/*
** Search the command line for a dump file, and if found process and remove it.
** Do the same also for the -Woempa=y|n and -Wdebugfrom=<count> commands.
*/
void getLogFile(int *argument_count_ptr, char ** arguments) {
  w_int i;
  w_int n;
  char  yn = 0;

  n = 0;
  for (i = 1; i < *argument_count_ptr; ++i) {
    if (n) {
      arguments[i - n] = arguments[i];
    }
    if (strncmp(arguments[i], "-Wlogfile=", 10) == 0) {
      woempa_dump_file = (char *)&arguments[i][10];
      ++n;
    }
    else if(strncmp(arguments[i], "-Woempa=", 8) == 0) {
      yn = arguments[i][8];
      ++n;
    }
    else if(strncmp(arguments[i], "-Wdebugfrom=", 12) == 0) {
      woempa_bytecodetrigger = atoi(&arguments[i][12]);
      ++n;
    }
  }


  if (yn) {
    if(yn == 'y') {
      woempa_stderr = 1;
    }
    else {
#ifdef DEBUG
      if(yn != 'n') {
        PutString("Illegal argument for -Woempa (should be 'y' or 'n'), treating as 'n'\n");
      }
#endif
      woempa_stderr = 0;
    }
  }

#ifndef DEBUG
  if (n) {
    PutString("Warning: one or more -W parameters ignored because DEBUG is set to false\n");
  }
#endif

  *argument_count_ptr -= n;
}

/*
** x_os_main is the function called by Oswald/o4p during system
** startup.  At this point we have no threads, no heap, no nothing: just
** a pointer to the first byte of memory after the Oswald/o4p data segment.
** We initialize all our static memory structures and the heap, and then
** start our initial threads.
*/

w_ubyte* x_os_main(int argument_count, char ** arguments, w_ubyte* FirstUnusedMemory) {
  return initWonka(FirstUnusedMemory);
}

int main(int argc, char * argv[]) {
  int max_heap_size;

  puts_vector = debug_puts;
  if (justVisiting(argc,  argv)) {
     return 1;
  }

  getLogFile(&argc, argv);

  max_heap_size = getMaxHeapSize(&argc, argv);
  if (max_heap_size == 0) {
    max_heap_size = 64 * 1024 * 1024;
  }

  command_line_argument_count = argc - 1;
  command_line_arguments = argv + 1;
  x_oswald_init(max_heap_size, 50);

  return 0;
}
