/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: fifo.c,v 1.3 2002/02/20 08:53:05 dfrederickx Exp $
*/

#include "fifo.h"
#include "ts-mem.h"
#include "realms.h"

/* returns number of bytes saved during contraction */

int contractFifo(w_fifo f) {

  return 0;

}

void releaseFifo(w_fifo f) {

  FifoLeaf *current = f->putFifoLeaf;
  FifoLeaf *last = f->putFifoLeaf;
  FifoLeaf *victim;

  do {
    victim = current;
    current = current->next;
    releaseMem(victim);
  } while (current != last);

  releaseMem(f);

}

#ifdef BAR
// NO PRINTF !!
void dumpFifo(w_fifo f) {

  FifoLeaf *fl = f->putFifoLeaf;
  FifoLeaf *last = f->putFifoLeaf ;
  int i;

  printf("***** Dump of fifo *****\n");
  do {
    printf("\nFifoleaf %p\n", fl);
    for (i = 0; i < (int)f->leafElements; i++) {
      printf("   %10s", ((int)f->putFifoLeaf == (int)fl && (int)f->putFifoIndex == i) ? "EQIX --> " : "         ");
      printf(" %2d [0x%08lx] ", i, (unsigned long)fl->data[i]);
      printf("%10s\n", ((int)f->getFifoLeaf == (int)fl && (int)f->getFifoIndex == (int)i) ? " <-- DQIX" : "         ");
    }
    fl = fl->next;
  } while (fl != last);
  
}
#endif

w_fifo allocFifo(w_realm realm, w_size leafElements) {

  /* XXX */
  w_fifo f = allocMem_realm(realm, sizeof(w_Fifo));
  
  if (f == NULL) {
    return NULL;
  }

  f->putFifoIndex = 0;
  f->getFifoIndex = 0;
  f->numElements = 0;
  f->realm = realm;

  if (leafElements == 0) {
    f->leafElements = FIFO_DEFAULT_LEAF_SIZE;
  }
  else {
    f->leafElements = leafElements;
  }
  f->leafSize = sizeof(FifoLeaf) + (sizeof(void *) * (f->leafElements - 1));

  /* XXX */
  f->putFifoLeaf = allocMem_realm(f->realm, f->leafSize);

  if (f->putFifoLeaf == NULL) {
    releaseMem(f);
    return NULL;
  }

  f->getFifoLeaf = f->putFifoLeaf;
  f->putFifoLeaf->next = f->putFifoLeaf;

  return f;
   
}

void *getFifo(w_fifo f) {

  void *e;

  if (f->numElements == 0) {
    return NULL;
  }

//  woempa(1,"Defifoing element %p in leaf %p at position %d\n", f->getFifoLeaf->data[f->getFifoIndex], f->getFifoLeaf, f->getFifoIndex);

  e = f->getFifoLeaf->data[f->getFifoIndex++];
  f->numElements--;

  if (f->getFifoIndex == f->leafElements) {
    f->getFifoLeaf = f->getFifoLeaf->next;
    f->getFifoIndex = 0;
  }

  return e;
  
}

w_int putFifo(void *e, w_fifo f) {

//  woempa(1,"Enfifoing element %p in leaf %p at index %d\n", e, f->putFifoLeaf, f->putFifoIndex);

  f->putFifoLeaf->data[f->putFifoIndex++] = e;
  f->numElements++;

  if (f->putFifoIndex == f->leafElements) {
    if (f->getFifoLeaf != f->putFifoLeaf->next) {
      /*
      ** We can recycle a leaf; we just restart with the next leaf
      */
      f->putFifoLeaf = f->putFifoLeaf->next;
    }
    else {
      /*
      ** We link in a new leaf
      */
      FifoLeaf *newLeaf = allocMem_realm(f->realm, f->leafSize);
      if (!newLeaf) {
        woempa(9, "No memory to add leaf to fifo %p\n", f);

        return -1;

      }
      
      newLeaf->next = f->putFifoLeaf->next;
      f->putFifoLeaf->next = newLeaf;
      f->putFifoLeaf = newLeaf;
    }
    f->putFifoIndex = 0;
  }

  return 0;

}

