/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: constant.c,v 1.18 2002/03/27 13:46:34 gray Exp $
*/

#include <string.h>
#include <stdio.h>

#include "clazz.h"
#include "constant.h"
#include "core-classes.h"
#include "descriptor.h"
#include "exception.h"
#include "fields.h"
#include "hashtable.h"
#include "loading.h"
#include "methods.h"
#include "realms.h"
#include "strings.h"
#include "ts-mem.h"
#include "heap.h"

/*
*/
w_instance createLinkageError(w_clazz failing_clazz, char *remark, w_clazz member_clazz, w_string member_name, w_string member_sig) {
  w_size length = failing_clazz->dotified->length + strlen(remark) + member_clazz->dotified->length + member_name->length + member_sig->length + 10;
  w_char *buffer = allocMem_system(length * sizeof(w_char));
  w_size  offset = 0;
  char   *cptr = remark;
  w_instance result;

  woempa(1, "Creating a LinkageError: failing class is %w, remark is `%s', member %w/%w %w\n", failing_clazz, remark, member_clazz, member_name, member_sig);
  offset = w_string2chars(failing_clazz->dotified, buffer);
  buffer[offset++] = ':';
  buffer[offset++] = ' ';
  while (*cptr) {
     buffer[offset++] = *cptr++;
  }
  buffer[offset++] = ':';
  buffer[offset++] = ' ';
  offset += w_string2chars(member_clazz->dotified, buffer + offset);
  buffer[offset++] = '/';
  offset += w_string2chars(member_name, buffer + offset);
  buffer[offset++] = ' ';
  offset += w_string2chars(member_sig, buffer + offset);
  length = offset;

  result = createExceptionInstance(currentWonkaThread, clazzLinkageError, unicode2String(currentWonkaThread, buffer, length));
  releaseMem(buffer);

  return result;
}

/*
** Get the Utf8 string from the constant pool array.
*/

w_string resolveUtf8(w_clazz clazz, w_int idx, w_thread thread) {
  w_cnt c = &clazz->constants[idx];

  return registerString(c->info.utf8.string);
}

/*
** Decrement the refcount of a UTF8 constant, and if it falls to zero
** release the memory occupied by the string.
*/
void decrementUtf8Refcount(w_clazz clazz, int idx) {
  w_cnt c = &clazz->constants[idx];

  --c->info.utf8.refcount;
  woempa(1, "Class %k: decrement refcount of utf8 constant[%d] (`%w'), is now %d.\n", clazz, idx, c->info.utf8.string, c->info.utf8.refcount);
#ifdef RUNTIME_CHECKS
  if (c->info.utf8.refcount == 0xffffffff) {
    wabort(ABORT_WONKA, "Nom de dieu! Class %k UTF8 constant[%d] has refcount %d!\n", clazz, idx, c->info.utf8.refcount);
  }
  else
#endif
  if (!c->info.utf8.refcount && c->info.utf8.string) {
    deregisterString(c->info.utf8.string);
    c->info.utf8.string = NULL;
  }
}

void dissolveConstant(w_clazz clazz, int idx) {
  w_cnt c = &clazz->constants[idx];

  if (c->cnt_tag == CONSTANT_UTF8) {
    if (c->info.utf8.string) {
      deregisterString(c->info.utf8.string);
      c->info.utf8.string = NULL;
    }
  }
}

/*
** The simplest form of resolution function, used for primitives and for
** constants which were already resolved.
*/
void * get_resolved(w_clazz clazz, w_int idx) {
  return &clazz->constants[idx].info;
}

/*
** Re-throw an exception of the same class as one we threw before.
*/
void * get_broken(w_clazz clazz, w_int idx) {
  throwException(currentWonkaThread, clazz->constants[idx].info.clazz, NULL);
  return NULL;
}

/*
** This one should never be called: it is used as the resolution function
** of the second word of a two-word pair (Long, Double).
*/
void * get_wrong_word(w_clazz clazz, w_int idx) {
  throwException(currentWonkaThread, clazzClassFormatError, NULL);
  return NULL;
}

/*
** Resolve a string constant.
*/
void * get_unresolved_String(w_clazz clazz, w_int idx) {
  w_cnt c = &clazz->constants[idx];
  w_int    utf8index =  c->info.strindex;

  if (c->get != get_resolved) {
    woempa(1, "Resolving String at index %d class index = %d\n", idx, c->info.member.class_index);
    c->info.string = resolveUtf8(clazz, utf8index, currentWonkaThread);
    decrementUtf8Refcount(clazz, utf8index);
    c->get = get_resolved;
  }

  return &clazz->constants[idx].info;
}

/*
** Resolve a class constant.
*/
void * get_unresolved_Class(w_clazz clazz, w_int idx) {
  w_cnt    c = &clazz->constants[idx];
  w_thread thread = currentWonkaThread;
  w_int    utf8index =  c->info.clindex;
  w_instance exception = NULL;
  w_clazz target_clazz = NULL;
  w_desc desc;
  w_instance loader = clazz2loader(clazz);
  w_string slashed;
  w_string dotified;

  woempa(1, "Resolving Class at index %d class index = %d\n", idx, c->info.clindex);
  slashed = resolveUtf8(clazz, utf8index, thread);
  dotified = slashes2dots(thread, slashed);
  decrementUtf8Refcount(clazz, utf8index);
  woempa(1,"Resolving class %w\n", dotified);

  if (slashed->chars[0] == (w_char)'[') {
    desc = newFieldDescriptor(thread, slashed);
    if (desc) {
      target_clazz = descClassMustBeLoaded(loader, desc, &exception);
      deregisterDescriptor(desc);
    }
    else {
      exception = createExceptionInstance(thread,clazzClassFormatError, dotified);
    }
  }
  else {
//    target_clazz = identifyClazz(dotified, loader);
    target_clazz = namedClassMustBeLoaded(loader, dotified, &exception);
  }

  if (exception == NULL && target_clazz) {
    woempa(1, "Resolved class constant %d (%w) of '%k' into class '%k'.\n", idx,slashed, clazz, target_clazz);
    deregisterString(slashed);
    c->info.clazz = target_clazz;
    c->get = get_resolved;
    
    return &clazz->constants[idx].info;

  }
  else {
    woempa(9,"Could not resolve  class constant %d (%w) of '%k'\n", idx, slashed, clazz);
    deregisterString(slashed);
    if (exception == NULL) {
      woempa(9,"Throwing ClassNotFoundException\n");
      exception = createExceptionInstance(thread,clazzClassNotFoundException,dotified);
    }
    setLocalException(currentWonkaThread, exception);
    clazz->constants[idx].info.clazz = instance2clazz(exception);
    clazz->constants[idx].get = get_broken;

    return NULL;

  }
}

/*
** Resolve a field constant.
*/
void * get_unresolved_Field(w_clazz clazz, w_int idx) {
  w_cnt c = &clazz->constants[idx];
  w_thread thread = currentWonkaThread;
  w_instance exception = NULL;
  w_field    field = NULL;
  w_clazz search_clazz;
  w_cnt nat_cnt;
  w_desc desc = NULL;
  w_string desc_string;
  w_string name;

  woempa(1, "Resolving Field at index %d class index = %d\n", idx, c->info.member.class_index);
  search_clazz = getClassConstant(clazz, c->info.member.class_index);

  if (!search_clazz) {
    woempa(7, "Couldn't find class where field should be declared -> LinkageError\n");
    clazz->constants[idx].info.clazz = clazzLinkageError;
    clazz->constants[idx].get = get_broken;

    return NULL;

  }
  
  exception = classMustBeInitialized(search_clazz, thread);
  if (exception) {
    woempa(9,"Couldn't prepare class %k: %k\n", search_clazz, instance2clazz(exception));
    clazz->constants[idx].info.clazz = clazzLinkageError;
    clazz->constants[idx].get = get_broken;

    return NULL;

  }

  while (search_clazz) {
    woempa(1, "  searching class %k.\n", search_clazz);
  /*
  ** Look in the class for a field with the correct name
  ** and descriptor.
  */
    nat_cnt = &clazz->constants[c->info.member.name_and_type_index];
    name = resolveUtf8(clazz, nat_cnt->info.nat.name_index, thread);
    decrementUtf8Refcount(clazz, nat_cnt->info.nat.name_index);
    desc_string = resolveUtf8(clazz, nat_cnt->info.nat.descriptor_index, thread);
    decrementUtf8Refcount(clazz, nat_cnt->info.nat.descriptor_index);
    desc = newFieldDescriptor(thread, desc_string);

    if (desc == NULL) {
      wabort(ABORT_WONKA,"whoops - failed to parse descriptor %p\n", NULL);
    }

    exception = classMustBePrepared(search_clazz, thread);
    if (exception) {

      break;

    }

    field = getField(search_clazz, name);
    if (field && w_desc_equals(desc, field->desc)) {
      woempa(1, "  found it in class %k, slot %d.\n", search_clazz,field->slot);
      break;
    }
    else if (field) {
      exception =  createLinkageError(clazz, (char *)"field has wrong type", search_clazz, name, desc_string);
      break;
    }
    else if (search_clazz->superClazz==NULL) {
      exception =  createLinkageError(clazz, (char *)"field not found", search_clazz, name, desc_string);
      break;
    }
    search_clazz = search_clazz->superClazz;
  } 

  if (!exception) {
    field->declaring_clazz = mustBeLoaded(field->declaring_clazz, &exception);
  }

  if (exception) {
    woempa(9,"Couldn't prepare class %k: %k\n", search_clazz, instance2clazz(exception));
    setLocalException(thread, exception);
    clazz->constants[idx].info.clazz = instance2clazz(exception);
    clazz->constants[idx].get = get_broken;

    return NULL;

  }
  else {
    c->info.field = field;
    deregisterDescriptor(desc);
    c->get = get_resolved;

    return &clazz->constants[idx].info;

  }
}

/*
** Resolve a method constant.
** TODO: refactor this to share code with get_unresolved_Field.
*/
void * get_unresolved_Method(w_clazz clazz, w_int idx) {
  w_cnt c = &clazz->constants[idx];
  w_thread thread = currentWonkaThread;
  w_instance exception = NULL;
  w_method   method = NULL;
  w_clazz search_clazz;
  w_cnt nat_cnt;
  w_string name;
  w_string desc_string;
  w_size   i;

  woempa(1, "Resolving method at index %d of %k class index = %d\n", idx, clazz, c->info.member.class_index);
  search_clazz = getClassConstant(clazz, c->info.member.class_index);

  if (!search_clazz) {
    woempa(7, "Couldn't find class where method should be declared -> LinkageError\n");
    clazz->constants[idx].info.clazz = clazzLinkageError;
    clazz->constants[idx].get = get_broken;

    return NULL;

  }

  woempa(1, "Method was declared in class %k.\n", search_clazz);
  /*
  ** Look in the class for a method with the correct name
  ** and descriptor.
  */
  nat_cnt = &clazz->constants[c->info.member.name_and_type_index];
  woempa(1,"Name & type at index %d holds {%d,%d}\n",c->info.member.name_and_type_index,nat_cnt->info.nat.name_index,nat_cnt->info.nat.descriptor_index);
  name = resolveUtf8(clazz, nat_cnt->info.nat.name_index, thread);
  decrementUtf8Refcount(clazz, nat_cnt->info.nat.name_index);
  desc_string = resolveUtf8(clazz, nat_cnt->info.nat.descriptor_index, thread);
  decrementUtf8Refcount(clazz, nat_cnt->info.nat.descriptor_index);
  woempa(1,"  -> name is `%w', type is `%w'\n",name,desc_string);

  exception = classMustBeInitialized(search_clazz, thread);
  if (exception) {
    woempa(9,"Couldn't prepare class %k: %k\n", search_clazz, instance2clazz(exception));
    clazz->constants[idx].info.clazz = clazzLinkageError;
    clazz->constants[idx].get = get_broken;

    return NULL;

  }

  method = getMethod_name_desc(search_clazz, name, desc_string);

  if (method) {
    woempa(1, "found method %w%w in class %k.\n", name, desc_string, search_clazz);
    c->get = get_resolved;
    clazz->constants[idx].info.method = method;

    for (i = 0; i < method->nargs && !exception; ++i) {
      method->arg_types[i] = mustBeLoaded(method->arg_types[i], &exception);
    }

    if (!exception) {
      method->return_type = mustBeLoaded(method->return_type, &exception);
    }

    if (exception) {
      woempa(9,"Couldn't prepare class %k: %k\n", search_clazz, instance2clazz(exception));
      clazz->constants[idx].info.clazz = instance2clazz(exception);
      clazz->constants[idx].get = get_broken;

      return NULL;

    }

    return &clazz->constants[idx].info;

  }
  else {
    clazz->constants[idx].info.clazz = clazzLinkageError;
    clazz->constants[idx].get = get_broken;
    setLocalException(currentWonkaThread, createLinkageError(clazz, (char *)"method not found", search_clazz, name, desc_string));

    return NULL;

  }
}

/*
** Resolve an interface method constant.
** TODO: refactor this to share code with preceding two functions.
*/
void * get_unresolved_IMethod(w_clazz clazz, w_int idx) {
  w_cnt c = &clazz->constants[idx];
  w_thread thread = currentWonkaThread;
  w_instance exception = NULL;
  w_method   imethod = NULL;
  w_clazz search_clazz;
  w_cnt nat_cnt;
  w_string desc_string;
  w_string name;
  w_size   i;

  woempa(1, "Resolving imethod at index %d of %k class index = %d\n", idx, clazz, c->info.member.class_index);
  search_clazz = getClassConstant(clazz, c->info.member.class_index);

  if (!search_clazz) {
    woempa(7, "Couldn't find class where imethod should be declared -> LinkageError\n");
    clazz->constants[idx].info.clazz = clazzLinkageError;
    clazz->constants[idx].get = get_broken;

    return NULL;

  }


  woempa(1, "IMethod was declared in class %k.\n", search_clazz);
  /*
  ** Look in the class for a method with the correct name
  ** and descriptor.
  */
  nat_cnt = &clazz->constants[c->info.member.name_and_type_index];
  woempa(1,"Name & type at index %d holds {%d,%d}\n",c->info.member.name_and_type_index,nat_cnt->info.nat.name_index,nat_cnt->info.nat.descriptor_index);
  name = resolveUtf8(clazz, nat_cnt->info.nat.name_index, thread);
  decrementUtf8Refcount(clazz, nat_cnt->info.nat.name_index);
  desc_string = resolveUtf8(clazz, nat_cnt->info.nat.descriptor_index, thread);
  decrementUtf8Refcount(clazz, nat_cnt->info.nat.descriptor_index);
  woempa(1,"  -> name is `%w', type is `%w'\n",name,desc_string);

  exception = classMustBeInitialized(search_clazz, thread);
  if (exception) {
    woempa(9,"Couldn't prepare class %k: %k\n",search_clazz, instance2clazz(exception));
    clazz->constants[idx].info.clazz = clazzLinkageError;
    clazz->constants[idx].get = get_broken;

    return NULL;

  }

  imethod = getMethod_name_desc(search_clazz, name, desc_string);
  if (imethod) {
    woempa(1, "found interface method %w%w in class %k.\n", name, desc_string, search_clazz);
    c->get = get_resolved;
    clazz->constants[idx].info.method = imethod;

    for (i = 0; i < imethod->nargs && !exception; ++i) {
      imethod->arg_types[i] = mustBeLoaded(imethod->arg_types[i], &exception);
    }

    if (!exception) {
      imethod->return_type = mustBeLoaded(imethod->return_type, &exception);
    }

    if (exception) {
      woempa(9,"Couldn't prepare class %k: %k\n", search_clazz, instance2clazz(exception));
      clazz->constants[idx].info.clazz = instance2clazz(exception);
      clazz->constants[idx].get = get_broken;

      return NULL;

    }

    return &clazz->constants[idx].info;

  }
  else {
    clazz->constants[idx].info.clazz = clazzLinkageError;
    clazz->constants[idx].get = get_broken;
    setLocalException(currentWonkaThread, createLinkageError(clazz, (char *)"method not found", search_clazz, name, desc_string));

    return NULL;

  }
}

/*
** Another one that shouldn't be called: used for e.g. name&type, which
** it doesn't make sense to resolve.
*/
void * get_unresolved_Other(w_clazz clazz, w_int idx) {
  woempa(9, "Cannot resolve a constant of type %d\n", clazz->constants[idx].cnt_tag);
  throwException(currentWonkaThread, clazzClassFormatError, NULL);

  return &clazz->constants[idx].info;
}


/*
** Print out all contant pool entries of a class.
*/
void dumpPools(int fd, w_clazz clazz) {

  w_size i;
  w_cnt  c;
  w_string string;

  woempa(1,"=== Constant Pool Dump for %k (fd = %d) ===\n", clazz, fd);
  for (i = 1; i < clazz->numConstants; i++) {
    if (clazz->constants) {
      c = &clazz->constants[i];
      switch (c->cnt_tag) {
      case CONSTANT_CLASS:
        if (c->get == get_resolved) {
          woempa(1, "%4d Class \"%k\"\n", i, c->info.clazz);
        }
        else {
          woempa(1, "%4d Class:     name index = %d\n", i, c->info.clindex);
        }
        break;

      case CONSTANT_UTF8:
        woempa(1, "%4d UTF8:      '%w'\n", i, c->info.utf8.string);
        break;

      case CONSTANT_INTEGER:
        woempa(1, "%4d Integer:   bytes 0x%x\n", i, c->info.intval);
        break;

      case CONSTANT_FLOAT:
        woempa(1, "%4d Float:     bytes 0x%x\n", i, c->info.floatval);
        break;

      case CONSTANT_LONG:
        woempa(1, "%4d Long:      MSW 0x%x LSW 0x%x\n", i, c->info.longval.words[WORD_MSW], c->info.longval.words[WORD_LSW]);
        break;

      case CONSTANT_DOUBLE:
        woempa(1, "%4d Double:    MSW 0x%x LSW 0x%x\n", i, c->info.doubleval.words[WORD_MSW], c->info.doubleval.words[WORD_LSW]);
        break;

      case CONSTANT_STRING:
        if (c->get == get_resolved) {
          string = c->info.string;
          woempa(1, "%d String \"%w\"\n", i, string);
        }
        else {
          woempa(1, "%4d String:    index %d\n", i, c->info.strindex);
        }
        break;

      case CONSTANT_FIELD:
        if (c->get == get_resolved) {
          woempa(1, "%d Field class  \"%k\"\n", i, c->info.field->declaring_clazz);
          woempa(1, "%d Field field  \"%w\" (slot %d)\n", i, c->info.field->name, c->info.field->slot);
        }
        else {
          woempa(1, "%4d Field:     class index %d name&type %d\n", i, c->info.member.class_index, c->info.member.name_and_type_index);
        }
        break;

      case CONSTANT_METHOD:
        if (c->get == get_resolved) {
          woempa(1, "%d Method method %w in slot %d from class %k\n", i, c->info.method->name, c->info.method->slot, c->info.method->clazz);
        }
        else {
          woempa(1, "%4d Method:    class index %d name&type %d\n", i, c->info.member.class_index, c->info.member.name_and_type_index);
        }
        break;

      case CONSTANT_IMETHOD:
        if (c->get == get_resolved) {
          woempa(1, "%d iMethod method %w in slot %d from class %k\n", i, c->info.method->name, c->info.method->slot, c->info.method->clazz);
        }
        else {
          woempa(1, "%4d IMethod:   class index %d name&type %d\n", i, c->info.member.class_index, c->info.member.name_and_type_index);
        }
        break;

      case CONSTANT_NAME_AND_TYPE:
        woempa(1, "%4d Name&type: name index %d descriptor index %d\n", i, c->info.nat.name_index, c->info.nat.descriptor_index); 
        break;

      case NO_VALID_ENTRY:
        woempa(1,"Skipping second word of double-size constant\n");
        break;

      default:
        woempa(9,"clazz %k index %d c->cnt_tag = %d\n", clazz, i, c->cnt_tag);
        wabort(ABORT_WONKA, "Should not get here...\n");
      }
    }
  }
  woempa(1,"=== End Constant Pool Dump for %k ===\n", clazz);

}


