/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


#include "file_driver.h"
#include "ts-mem.h"

#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
//#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include "vfs.h"

#if defined(WINNT) || defined(ECOS)
  #define DEFFILEMODE (VFS_S_IRUSR|VFS_S_IWUSR|VFS_S_IRGRP|VFS_S_IWGRP|VFS_S_IROTH|VFS_S_IWOTH)
#endif

typedef struct w_Control_File {
  w_int fd;
  w_driver_perm mode;
} w_Control_File;

typedef w_Control_File *w_control_file;

static char *pathname[] = { "", "", "", "", };

w_void file_initDevice(w_device device) {
  w_control_file c;
  device->control = allocMem_system(sizeof(w_Control_File));
  
  c = device->control;
  c->mode = wdp_none;
  c->fd = -1;
}

w_void file_termDevice(w_device device) {
  releaseMem(device->control);
}

/*
** Somewhat ad-hoc function to set the path of a file device
*/
void file_set_path(int n, char *s) {
  if (n >= 0 && n < 4) {
    char *buf = allocMem_system(strlen(s+1));
    strcpy(s, buf);
    pathname[n] = buf;
  }
  else {
    woempa(9, "file devnum %d out of range\n", n);
  }
}

w_device file_open(w_device device, w_driver_perm mode) {
  w_control_file c = device->control;
  w_int p;

  if(device->usage == 0) {
  
    switch(mode) {
      case wdp_read:
        p = VFS_O_RDONLY | VFS_O_NONBLOCK;
        break;
      case wdp_write:
        p = VFS_O_WRONLY | VFS_O_CREAT | VFS_O_TRUNC | VFS_O_NONBLOCK;
        break;
      case wdp_read_write:
        p = VFS_O_RDWR | VFS_O_NONBLOCK;
        break;
      default:
        return 0;
    }

    c->fd = vfs_open(pathname[device->familyMember], p, DEFFILEMODE);
    
    if(c->fd == -1) return 0;
    c->mode = mode;
    
  }
  
  device->usage += 1;
  device->driver->usage += 1;

  return device;
}

w_void file_close(w_device device) {
  w_control_file c = device->control;

  if(device->usage == 1) {
    vfs_close(c->fd);
    c->fd = -1;
  }

  device->usage -= 1;
  device->driver->usage -= 1;
}

w_driver_status file_read(w_device device, w_ubyte *bytes, w_int length, w_int *lread, x_sleep timeout) {
  w_control_file c = device->control;
  fd_set rfds;
  struct timeval tv;
  w_int retval;

  if (c->mode == wdp_read || c->mode == wdp_read_write) {
  
    FD_ZERO(&rfds);
    FD_SET(c->fd, &rfds);
    if (timeout == x_eternal) {
      // this should not be used, unless you know exactly what you are doing
      // I change eternal to 1 sec, which in cpu terms is quite eternal already
      tv.tv_sec = 1;
      tv.tv_usec = 0;
    }
    else {
      tv.tv_sec = 0;
      tv.tv_usec = x_ticks2usecs(timeout);
    }
    retval = select(c->fd + 1, &rfds, NULL, NULL, &tv);
    
    if (retval == 1) {
      *lread = vfs_read(c->fd, bytes, (unsigned)length); 

      if (*lread == 0 && length != 0) {
        return wds_data_exhausted;
      }
      else if (*lread == -1) {
        return wds_internal_error;
      }
      else {
        return wds_success;
      }
    }
    else if (retval == 0) {
      return wds_no_instance;
    }
    else {
      return wds_internal_error;
    }
  }
  else {
    return wds_write_only;
  }
}

w_driver_status file_write(w_device device, w_ubyte *bytes, w_int length, w_int *lwritn, x_sleep timeout) {
  w_control_file c = device->control;
  fd_set wfds;
  struct timeval tv;
  w_int retval;

  if (c->mode == wdp_write || c->mode == wdp_read_write) {
    FD_ZERO(&wfds);
    FD_SET(c->fd, &wfds);
    if (timeout == x_eternal) {
      // this should not be used, unless you know exactly what you are doing
      // I change eternal to 1 sec, which in cpu terms is quite eternal already
      tv.tv_sec = 1;
      tv.tv_usec = 0;
    }
    else {
      tv.tv_sec = 0;
      tv.tv_usec = x_ticks2usecs(timeout);
    }
    retval = select(c->fd + 1, NULL, &wfds, NULL, &tv);

    if (retval == 1) {
      *lwritn = vfs_write(c->fd, bytes, (unsigned)length); 

      if (*lwritn == 0 && length != 0) {
        return wds_space_exhausted;
      }
      else if(*lwritn == -1){
        return wds_internal_error;
      }
      else {
        return wds_success;
      }
    }
    else if (retval == 0) {
      return wds_no_instance;
    }
    else {
      return wds_internal_error;
    }
  }
  else {
    return wds_read_only;
  }
}

w_driver_status file_seek(w_device device, w_int offset, x_sleep timeout) {
  w_control_file c = device->control;

  if (vfs_lseek(c->fd, offset, SEEK_SET) != -1) {
    return wds_success;
  }
  else {
    return wds_internal_error;
  }
}

w_driver_status file_set(w_device device, w_driver_ioctl command, w_word param, x_sleep timeout) {
  return wds_not_implemented;
}

w_driver_status file_query(w_device device, w_driver_ioctl query, w_word *reply, x_sleep timeout) {
  switch (query) {
    case(wdi_get_available):
      *reply = 0;
      return wds_success;
    case(wdi_get_bitrate):
    case(wdi_get_databits):
    case(wdi_get_stopbits):
    case(wdi_get_flowcontrol):
    case(wdi_get_parity):
    case(wdi_get_signals):
      return wds_success;

    default:
      return wds_no_such_command;
  }

  return wds_no_such_command;
}

w_Driver_ByteSerial file_driver = {
  "file-driver-linux_v0.1",
  "file",
  0,
  NULL,
  file_initDevice,
  file_termDevice,  
  file_open,
  file_close,  
  file_read,
  file_write,
  file_seek,
  file_set,
  file_query
};
