/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: debug-term.c,v 1.2 2002/02/18 11:59:14 gray Exp $
*/

#include <string.h>

#include "driver_byteserial.h"
#include "oswald.h"
#include "ts-mem.h"
#include "wonka.h"

#include <unistd.h>

#define DEBUG_WRITE_BUFFER_SIZE 256

typedef struct w_Control_Debug {
  w_device underlying_device;
  x_mutex    mutex;
  w_byte     write_buffer[DEBUG_WRITE_BUFFER_SIZE];
  w_byte     *write_pointer;
} w_Control_Debug;

typedef w_Control_Debug *w_control_debug;

w_void debug_initDevice(w_device device) {
  w_control_debug c;
  device->control = allocMem_system(sizeof(w_Control_Debug));
  c = device->control;
  c->mutex = allocMem_system(sizeof(x_Mutex));
  
  c->underlying_device = NULL;
  c->write_pointer = c->write_buffer;
  x_mutex_create(c->mutex);
}

w_void debug_termDevice(w_device device) {
  w_control_debug c = device->control;
  x_mutex_delete(c->mutex);
  releaseMem(c->mutex);
  releaseMem(device->control);
}

w_device debug_open(w_device device, w_driver_perm mode) {
  w_control_debug c = device->control;
  
  c->underlying_device = deviceBSOpen("stderr", wdp_read_write);
  
  device->usage += 1;
  device->driver->usage += 1;
  
  return device;
}

w_void debug_close(w_device device) {
  w_control_debug c = device->control;
  
  deviceBSClose(c->underlying_device);

  device->usage -= 1;
  device->driver->usage -= 1;
}

w_driver_status debug_read(w_device device, w_ubyte *bytes, w_int length, w_int *lread, x_sleep timeout) {
  w_control_debug c = device->control;

  w_byte *inptr = bytes;
  w_byte gotbyte;
  w_byte linefeed = 10;
  w_int  retcode = wds_success;
  w_int  lread1;

  memset(bytes, 0, (unsigned long)length);
  while (inptr - bytes < (w_int)length) {
    retcode = deviceBSRead(c->underlying_device, &gotbyte, 1, &lread1, timeout);
    if (retcode != wds_success) {
      woempa(9,"Darn it: couldn't read from %s\n",c->underlying_device->name);
      break;
    }

    if (gotbyte==linefeed) {
      if (*inptr=='\\') {
        inptr--;
      } else {
        break;
      }
    } else {
      *inptr++ = gotbyte;
    }
  }
  *lread = inptr - bytes;
  
  return retcode;
  
}

w_driver_status debug_write(w_device device, w_ubyte *bytes, w_int length, w_int *lwritn, x_sleep timeout) {
  w_control_debug c = device->control;
  x_status mutex_error = x_mutex_lock(c->mutex, timeout);
  w_int    i;
  w_int    result = 0;
  w_size dummy;

  for (i = 0; i < length && result == 0; ++i) {
    if (c->write_pointer == c->write_buffer+DEBUG_WRITE_BUFFER_SIZE) {
      result = deviceBSWrite(c->underlying_device, c->write_buffer, c->write_pointer - c->write_buffer, &dummy, timeout);
      c->write_pointer = c->write_buffer;
    }
    *c->write_pointer++ = bytes[i];
    if (bytes[i] == '\n') {
      result = deviceBSWrite(c->underlying_device, c->write_buffer, c->write_pointer - c->write_buffer, &dummy, timeout);
      c->write_pointer = c->write_buffer;
    }
  }

  if (!mutex_error) {
    x_mutex_unlock(c->mutex);
  }
  
  *lwritn = i - 1;

  return result;
}

w_driver_status debug_seek(w_device device, w_int offset, x_sleep timeout) {
  w_control_debug c = device->control;
  return deviceBSSeek(c->underlying_device, offset, timeout);
}

w_driver_status debug_set(w_device device, w_driver_ioctl command, w_word param, x_sleep timeout) {
  w_control_debug c = device->control;
  return deviceBSSet(c->underlying_device, command, param, timeout);
}

w_driver_status debug_query(w_device device, w_driver_ioctl query, w_word *reply, x_sleep timeout) {
  w_control_debug c = device->control;
  return deviceBSQuery(c->underlying_device, query, reply, timeout);
}

w_Driver_ByteSerial debug_driver = {
  "debug-driver-linux_v0.1",
  "debug",
  0,
  NULL,
  debug_initDevice,
  debug_termDevice,  
  debug_open,
  debug_close,  
  debug_read,
  debug_write,
  debug_seek,
  debug_set,
  debug_query
};

