/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

#ifdef FSENABLE

#include <wonka.h>
#include <ts-mem.h>

#include <stdlib.h>
#include <stdio.h>

#include "driver_blockrandom.h"

typedef struct disk_Partition_Entry {
  w_ubyte     state;
  w_ubyte     head_begin;
  w_ushort    cyl_sect_begin;
  w_ubyte     type;
  w_ushort    cyl_sect_end;
  w_word      first_sect;
  w_word      sectors;
} disk_Partition_Entry;

static w_ubyte empty_entry[16];

typedef struct w_Control_Disk {
  disk_Partition_Entry  partitions[4];
  FILE *fd;
} w_Control_Disk;

typedef w_Control_Disk *w_control_disk;

/*
** Prototypes 
*/

w_void disk_initDevice(w_device device);
w_void disk_termDevice(w_device device);
w_device disk_open(w_device device);
w_void disk_close(w_device device);
w_driver_status disk_read(w_device device, w_ubyte *bytes, w_int sector, w_int size, x_sleep timeout);
w_driver_status disk_write(w_device device, w_ubyte *bytes, w_int sector, w_int size, x_sleep timeout);
w_driver_status disk_set(w_device device, w_driver_ioctl command, w_word param, x_sleep timeout);
w_driver_status disk_query(w_device device, w_driver_ioctl query, w_word *reply, x_sleep timeout);


w_void disk_initDevice(w_device device) {
  w_control_disk   disk;
  w_int            i;
  w_ubyte          *buffer;
  w_ubyte          name[5] = "hdax";
  w_device       main_device;

  device->control = allocMem_system(sizeof(w_Control_Disk));
  disk = device->control;

  switch(device->familyMember) {
    case 0:
      /* This is the actual device itself, the other familyMembers represent the partitions of the disk */

      /* Open the image file */
      
      disk->fd = fopen("image", "r+");
      
      /* Read the partition table */

      memset(&empty_entry, 0, 16);
      
      buffer = allocMem_system(1024);
      fread(buffer, 1024, 1, disk->fd);
      memcpy((void *)disk->partitions, (buffer + 0x1be), 16 * 4);
      releaseMem(buffer);

      /* Show the partition table to the crowd */

      woempa(7, "--- Partition table ---------------------------------------\n");
      woempa(7, "----- type -- begin (H/C/S) ------ end (C/S) offset  blocks\n");
      for(i=0; i<4; i++) {
        woempa(7, "%d:  %c   %02x   %2d   %6d  %2d   %6d  %2d   %6d   %6d\n", i,
            (disk->partitions[i].state == 0x80 ? '*' : ' '),
            disk->partitions[i].type,
            disk->partitions[i].head_begin,
            (disk->partitions[i].cyl_sect_begin >> 8) | ((disk->partitions[i].cyl_sect_begin & 0x00C0) << 8),
            (disk->partitions[i].cyl_sect_begin & 0x003F),
            (disk->partitions[i].cyl_sect_end >> 8) | ((disk->partitions[i].cyl_sect_end & 0x00C0) << 8),
            (disk->partitions[i].cyl_sect_end & 0x003F),
            disk->partitions[i].first_sect,
            disk->partitions[i].sectors >> 1);
      }

      /* Make a device for every valid partition entry */

      for(i=0; i<4; i++) {
        if(memcmp(&disk->partitions[i], &empty_entry, 16) != 0) {
          sprintf(name, "hda%d", (i + 1));
          registerDevice(name, "hda", (i + 1), wdt_block_random);
        }
      }
      
      break;
    
    case 1:
    case 2:
    case 3:
    case 4:
      /* If one of the partitions gets initialized, make a copy of the main device controlblock */
      main_device = getDeviceByName("hda");
      memcpy(disk, main_device->control, sizeof(w_Control_Disk));
    default:
  }
  
}

w_void disk_termDevice(w_device device) {
  w_control_disk disk = device->control;
  
  if(device->familyMember == 0) {
    fclose(disk->fd);
  }
 
  releaseMem(device->control); 
}


w_device disk_open(w_device device) {

  device->usage += 1;
  device->driver->usage += 1;
  
  return device;
}


w_void disk_close(w_device device) {

  device->usage -= 1;
  device->driver->usage -= 1;
}


w_driver_status disk_read(w_device device, w_ubyte *bytes, w_int sector, w_int size, x_sleep timeout) {
  w_control_disk disk = device->control;
  
  woempa(2, "sector %d   size %d\n", sector, size);
  fseek(disk->fd, (long)((sector + disk->partitions[device->familyMember - 1].first_sect) * 512), SEEK_SET);
  fread(bytes, (w_word)size, 1, disk->fd); 
  
  return wds_success;
}


w_driver_status disk_write(w_device device, w_ubyte *bytes, w_int sector, w_int size, x_sleep timeout) {
  w_control_disk disk = device->control;
  
  woempa(2, "sector %d   size %d\n", sector, size);
  fseek(disk->fd, (long)((sector + disk->partitions[device->familyMember - 1].first_sect) * 512), SEEK_SET);
  if(fwrite(bytes, (w_word)size, 1, disk->fd) == 0) {
    woempa(9, "Write error\n");
    
  }
  return wds_success;
}


w_driver_status disk_set(w_device device, w_driver_ioctl command, w_word param, x_sleep timeout) {
  return wds_not_implemented;
}


w_driver_status disk_query(w_device device, w_driver_ioctl query, w_word *reply, x_sleep timeout) {
  return wds_not_implemented;
}


w_Driver_BlockRandom disk_driver = {
  "disk-image-driver-linux_v0.1",
  "hda",
  0,
  NULL,
  disk_initDevice,
  disk_termDevice,
  disk_open,
  disk_close,
  disk_read,
  disk_write,
  disk_set,
  disk_query
};

#endif

