/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: collector.c,v 1.32 2002/03/26 10:44:14 gray Exp $
*/



/*
** add mutex spinlock around sweep/checkwindow in interpreter.c
**
** check exception.c line 49
** check clinit.c where the exeception is allocated
** check jni.c with instance allocation
*/

#define PRINTRATE 10

#include "arrays.h"
#include "checks.h"
#include "clazz.h"
#include "core-classes.h"
#include "descriptor.h"
#include "fields.h"
#include "fifo.h"
#include "hashtable.h"
#include "interpreter.h"
#ifdef OS_NONE
#include "comms.h"
#endif
#include "jni.h"
#include "loading.h"
#include "locks.h"
#include "ts-mem.h"
#include "methods.h"
#include "oswald.h"
#include "stack.h"
#include "strings.h"
#include "threads.h"
#include "list.h"
#include "constant.h"

#include "misc.h" /* for hexdump */

extern w_hashtable instance_hashtable;
w_boolean isValidInstance(w_instance instance);
void persistentGarbageCollector(void);

/*
** After freeing BYTES_PER_MEAL bytes, the garbage collector will have a
** `nap attack', i.e. will sleep for a short time to give other threads
** a chance.
** If BYTES_PER_MEAL is not defined, GC will not take naps.
*/
#define BYTES_PER_MEAL (128 * 1024)

/*
** MILLIS_PER_NAP defines the duration of a nap when memory_load_factor is
** zero: higher memory load leads to shorter naps.
*/
#define MILLIS_PER_NAP 250

/*
** If DISTRIBUTED_GC is defined, the reclaim callback mechanism is activated.
*/
#define DISTRIBUTED_GC

/*
** If USE_DISCARD_COLLECT is defined, discardMem and collectMem
** will be used instead of releaseMem.
*/
#define USE_DISCARD_COLLECT

/*
** If SECOND_CHANCE is defined, an instance will only be collected if it is
** found to be unreachable in two consecutive GC passes.
*/
#define SECOND_CHANCE

/*
** FIFO which holds all instances in the current GC window: the GC window
** is the set of instances which existed at the moment the current GC cycle
** began.  We give this one quite a large leaf size, as it is normal to have
** 1000's of objects.
*/
static w_fifo window_fifo;
#define WINDOW_FIFO_LEAF_SIZE   4095

/*
** FIFO used to mark all strongly reachable instances.  We give this one
** quite a large leaf size too.
*/
static w_fifo strongly_reachable_fifo;
#define STRONG_FIFO_LEAF_SIZE 4095

/*
** FIFOs used to mark instances which are reachable only via reference types. 
** We give these a smaller leaf size.
*/
static w_fifo softly_reachable_fifo;
static w_fifo weakly_reachable_fifo;
static w_fifo phantom_reachable_fifo;
static w_fifo finalize_reachable_fifo;
#define OTHER_FIFO_LEAF_SIZE 63

/*
** FIFO used to hold instances of Reference on which enqueue() should be called.. 
*/
w_fifo enqueue_fifo;
#define ENQUEUE_FIFO_LEAF_SIZE         OTHER_FIFO_LEAF_SIZE

/*
** Mutex used to protect the fifo from conflicting accesses.
*/
static x_Mutex enqueue_fifo_Mutex;
x_mutex enqueue_fifo_mutex;

/*
** List of instances whose state has changed since the current GC cycle began,
** or which have been flagged as immortal.  These are added to the normal
** roots (the normal roots are the static fields of all classes and the
** stack frames of all threads).
*/
w_fifo gray_green_fifo;
#define GRAY_FIFO_LEAF_SIZE       OTHER_FIFO_LEAF_SIZE

/*
** Mutex used to protect the fifo from conflicting accesses.
*/
static x_Mutex gray_green_fifo_Mutex;
static x_mutex gray_green_fifo_mutex;

/*
** List of instances waiting to be finalized.
*/
w_fifo finalizer_fifo;
#define FINALIZER_FIFO_LEAF_SIZE   OTHER_FIFO_LEAF_SIZE

/*
** Mutex used to protect the fifo from conflicting accesses.
*/
static x_Mutex finalizer_fifo_Mutex;
x_mutex finalizer_fifo_mutex;

/*
** List of references waiting to be obliterated.
static w_fifo deadref_fifo;
#define DEADREF_FIFO_LEAF_SIZE     OTHER_FIFO_LEAF_SIZE
*/

/*
** The Wonka thread in which GC is performed.
*/
w_thread gc_thread;

/*
** The unique instance of GarbageCollector corresponding to gc_thread.
*/
w_instance gc_instance;

/*
** A lock (monitor) associated with the GC thread.
*/
x_monitor gc_monitor;

/*
** The number of times the monitor has been notify'd.
*/
static w_int *gc_kicks_pointer;

/*
** The `total' memory is the amount returned by x_mem_total, minus 
** min_heap_free.  Calculated during GC startup.
*/
static w_int memory_total;

/*
** The memory load factor is the integer part of (rho / (1-rho)), where
** rho = (memory_total - x_mem_avail()) / memory_total .
** It is calculated at the start of each GC cycle.
*/
static w_int memory_load_factor;

/*
** The name of the `enqueue' method.
*/
static w_string string_enqueue;

/*
** For testing ...
*/
w_int sting_count = 0;

/*
** A macro to sting the collector where it hurts the most. For normal
** behaviour, undef sting_collector.
*/

#define sting_collector()  {                    \
  unsigned int rs = (unsigned int)(rand() % 2); \
  if (rs) {                                     \
    sting_count++;                              \
    x_thread_sleep(1);                          \
  }                                             \
}

#undef sting_collector
#define sting_collector()

/*
** Stuff to do with debugging output ...
*/
#define BUF_SIZE 33

char *printFlags(w_flags code) {

  static char buffer[BUF_SIZE];
  w_word mask = (unsigned)(1 << 31);
  w_int i = 0;
  
  memset(buffer, 0x00, BUF_SIZE);
  while (mask) {
    if ((w_word)code & mask) {
      buffer[i++] = '1';
    }
    else {
      buffer[i++] = '0';
    }
    mask >>= 1;
  }

  return buffer;
  
}

/*
** If instance cacheing is enabled, add an instance to the cache for its class.
**
** Returns true if the instance was added to a cache, false otherwise.
*/
static w_boolean cacheInstance(w_object object) {
#ifdef CACHE_INSTANCES
  x_status status;

  if (isSet(object->flags, O_CACHEABLE)) {
    status = x_mutex_lock(object->clazz->cache_lock, x_eternal);
    if (status == xs_success) {
      addToWordset(gc_thread, &object->clazz->cache, (w_word)object);
      resetMemoryOwnerR(object, W_Realm_system);
      object->pobject = NULL;
      object->hashcode = 0;
      object->flags = O_CACHEABLE;
      x_mutex_unlock(object->clazz->cache_lock);
      woempa(9, "CACHE %w, %d avail, %d hits, %d missed.\n", object->clazz->name, sizeOfWordset(&object->clazz->cache), object->clazz->cache_hits, object->clazz->cache_miss);
    }
    else {
      wabort(ABORT_WONKA, "CACHE sem error.\n");
    }

    return WONKA_TRUE;
  }
#endif

  return WONKA_FALSE;
}

/*
** Reclaim an instance by either adding it to an instance cache (see above)
** or returning its memory to the global object heap.
** The value returned is the size of the instance in bytes (the amount of 
** memory freed is generally more than this).
*/
static w_int releaseInstance(w_object object) {
  w_clazz clazz;
  w_size  bytes = 0;

  if (object == NULL) {
    wabort(ABORT_WONKA, "Releasing NULL object !!\n");
  }
  
  clazz = object->clazz;
  woempa(1, "(GC) Releasing %p (object %p) = %k flags %s\n", (char *)object->fields, object, clazz, printFlags(object->flags));

  ht_erase(instance_hashtable,(w_word)object->fields);
  if ( ! cacheInstance(object)) {
    if (clazz->aclazz) {
      bytes = (clazz->aclazz->bits * object->fields[F_Array_length] + 7) / 8 + 4;
    }
    else {
      bytes = clazz->bytes_needed;
    }
    woempa(1,"(GC) -> freeing %d bytes\n", bytes);

#ifdef USE_DISCARD_COLLECT
    discardMem(object);
#else
    releaseMem(object);
#endif
  }

  return bytes;
}

#ifdef CACHE_INSTANCES
/*
** Clean out the instance cache of a single clazz. The static 'cache_keep' indicates
** how many instances we want to keep in the cache, when it is larger or equal to 0.
** When it is set to a negative value, the caches are not cleared. The special value
** of 0 for this variable, means that the 'keep' value is calculated as follows:
**
** keep = ((cache_misses for this class / 128) * cache_factor) + cache_factor;
*/

static w_int cache_keep = -1;
static w_int cache_factor = 25;

void emptySingleCache(w_word name_word, w_word clazz_word) {

  w_clazz clazz = (w_clazz)clazz_word;
  x_status status;
  w_object object;
  w_int avail = sizeOfWordset(&clazz->cache);
  w_size keep;
  
  if (cache_keep == 0) {
    keep = ((clazz->cache_miss / 128) * cache_factor) + cache_factor;
  }
  else {
    keep = cache_keep;
  }

  woempa(9, "Emptying cache of %w, keep = %d\n", clazz->name, keep);

  if (sizeOfWordset(&clazz->cache) > keep) {
    status = x_mutex_lock(clazz->cache_lock, x_eternal);
    if (status == xs_success) {
      while (sizeOfWordset(&clazz->cache) > keep) {
        object = (w_object)takeFirstFromWordset(&clazz->cache);
        releaseMem(object);
      }
      x_mutex_unlock(clazz->cache_lock);
    }
    else {
      wabort(ABORT_WONKA, "CACHE sem error.\n");
    }
  }

  if (avail != sizeOfWordset(&clazz->cache) || sizeOfWordset(&clazz->cache)) {
    woempa(9, "Keep %5d, removed %5d, remain %5d, hits %5d, miss %5d of %w.\n", keep, avail - sizeOfWordset(&clazz->cache), sizeOfWordset(&clazz->cache), clazz->cache_hits, clazz->cache_miss, clazz->name);
  }    

}
#endif

/*
** Persistent Storage issues.
** If this is a strong reference to a persistent object, tell the persistent
** garbage collector that this object is reachable from transient memory.
** This is the only processing we do for persistent objects.
** If this reference is to a local copy, tell the persistent garbage collector  ** it is reachable (not necessarily stronly) and then carry on marking the
** object.
** Function checkPS returns WONKA_TRUE if the object should be processed
** further, WONKA_FALSE otherwise.
*/
#ifdef ENABLE_PS
static w_boolean checkPS(w_instance ref, w_word flag) {
  if (isPersistentRef(ref)) {
    if (flag == O_BLACK) {
      markPersistentRef(ref);
    }

    return WONKA_FALSE;

  }
  else if (typeOfInstance(ref) == INSTANCE_IS_PERSISTENT) {
    if (flag == O_BLACK) {
      markPersistentInstance(ref);
    }

    return WONKA_FALSE;

  }
  else if (isLocalCopy(ref)) {
      markLocalCopy(ref);
  }

  return WONKA_TRUE;
}
#else
static w_boolean checkPS(w_instance ref, w_word flag) {
  return WONKA_TRUE;
}
#endif

/*
** Function isMarked returns true iff the object in question is already
** marked either with the given flag or by one which `dominates' it, i.e.
** precedes it in the series 
**   O_BLACK > O_SOFT_BLACK > O_WEAK_BLACK > O_PHANTOM_BLACK > O_FINALIZE_BLACK.
** Note that we rely on the fact that the flags in question appear in
** the object flags word contiguously and in the order shown above.
*/
static w_boolean isMarked(w_object o, w_word flag) {
  return (o->flags & (O_BLACK | O_NEAR_BLACK)) > (flag - 1);
}

#define isNotMarked(o,flag) !isMarked(o,flag)

/*
** If a Class is reachable (because instances exist, or because it is
** known to a ClassLoader), and it has been initialised, we queue all
** of its (unmarked) static fields for marking.
*/
w_int markClazzReachable(w_clazz clazz, w_fifo fifo, w_word flag) {
  w_instance child_instance;
  w_object   child_object;
  w_int      i;
  w_int      queued = 0;

  if (classIsInitialized(clazz)) {
    woempa(1,"(GC) Marking static fields of class %k\n",clazz);
    for (i = 0; i < (w_int)clazz->numStaticFields; ++i) {
      if (isSet(clazz->fields[i].flags, FIELD_IS_REFERENCE)) {
        child_instance = (w_instance)clazz->staticFields[clazz->fields[i].slot];
        woempa(1, "(GC)    --> field %w of %k contains instance %p.\n", clazz->fields[i].name, clazz, child_instance);
        if (child_instance) {
          child_object = instance2object(child_instance);
          if (checkPS(child_instance, flag) && isNotMarked(child_object, flag)) {
            ++queued;
            if (putFifo(child_instance, fifo) < 0) {
              woempa(9, "Shiver my timbers! Couldn't push an instance onto fifo %p\n", fifo);

              return -1;

            };
          }
        }
      }
      else {
        woempa(1, "(GC)    --> field %w of %k is primitive, getting out.\n",  clazz->fields[i].name, clazz);
        break;
      }
    }
  }

  return queued;
}

/*
** Mark one frame of a Java stack: we push onto the fifo every local variable
** and every stack item which looks like a reference and hasn't already been
** marked.
*/
w_int markFrameReachable(w_frame frame, w_fifo fifo, w_word flag) {
  w_int queued = 0;
  w_word * item;
  w_word * locals;
  w_instance child_instance;
  w_object   child_object;
  w_instance thisThread = frame->thread->Thread;

  woempa(1,"(GC) Tracing Java frame %p: %M\n",frame,frame->method);

  item = frame->o_top;
  if (item == NULL) {
    woempa(1, "Top is NULL, skipping frame...\n");
    return 0;
  }

  locals = frame->locals;
  woempa(1,"top is %p, locals at %p\n",item,locals);
  while (--item >= locals) {
    child_instance = (w_instance)*item;
    // Be conservative: if it looks like a reference then it probably is one.
    // We do this because a stack item can change type from one moment to the next.
    if (child_instance && child_instance != thisThread && isValidInstance(child_instance)) {
      child_object = instance2object(child_instance);
      if (checkPS(child_instance, flag) && isNotMarked(child_object, flag)) {
        ++queued;
        if (putFifo(child_instance, fifo) < 0) {
          woempa(9, "Shiver my timbers! Couldn't push an instance onto fifo %p\n", fifo);

          return -1;

        }
      }
    }
  }

  return queued;
}

/*
** If a Thread is reachable (e.g. is reachable from the system thread group)
** and it has a valid Java stack then we mark every frame.
*/
w_int markThreadReachable(w_object object, w_fifo fifo, w_word flag) {
  w_thread thread = (w_thread)object->fields[F_Thread_wotsit];
  w_int numFrames = 0;
  w_int queued = 0;
  w_int retcode;
  volatile w_frame frame;

  if (!thread->bottom || !thread->bottom->o_top) {
    woempa(9,"(GC) Thread %w has no stack frame, refusing to trace it\n",NM(thread));

    return 0;

  }

  woempa(1, "(GC) Marking stack of thread %w.\n", NM(thread));

  x_scheduler_disable();
  for (frame = thread->top; frame; frame = (volatile w_frame)frame->previous) {
    numFrames += 1;
    retcode = markFrameReachable(frame, fifo, flag);
    if (retcode < 0) {
      x_scheduler_enable();

      return retcode;

    }
    queued += retcode;
  }
  x_scheduler_enable();
  woempa(1, "(GC) Enfifoed %d objects in %d frames of thread %w.\n", queued, numFrames, NM(thread));

  return queued;
}

/*
** If a ClassLoader is reachable (because it has loaded classes, or otherwise)
** then we queue all of the (unmarked) classes in its cache for marking.
*/
w_int markClassLoaderReachable(w_hashtable ht, w_fifo fifo, w_word flag) {
  w_int  queued = 0;
  w_int  retcode;
  w_fifo class_fifo;
  w_clazz clazz;

  class_fifo = ht_list_values(ht);
  if (!class_fifo) {

    return -1;

  }
  while ((clazz = getFifo(class_fifo))) {
    retcode = markClazzReachable(clazz, fifo, flag);
    if (retcode < 0) {

      return retcode;

    }
    queued += retcode;
  }
  releaseFifo(class_fifo);


  /*
  ** If cache_keep is non negative, clean out the caches... TODO, consider having a Java field
  ** in the ClassLoader class, that contains this value on a case by case basis...
  */
  
#ifdef CACHE_INSTANCES
  if (cache_keep >= 0) {
    ht_every(ht, emptySingleCache);
  }
#endif

  return queued;
}

/*
** If a Reference is reachable, and it actually refers to something (the
** `referent', and the referent is `condemned', we zap the reference.
*/
w_int markReferenceReachable(w_instance referent, w_fifo fifo, w_word flag) {
  w_int  queued = 0;
  w_int  retcode = 0;
  w_object ref_obj;
 
  if (referent == NULL) {

    return 0;

  }

  ref_obj = instance2object(referent);

  if ((flag == O_SOFT_BLACK) && isNotMarked(ref_obj, O_SOFT_BLACK)) {
    ++queued;
     retcode = putFifo(referent, softly_reachable_fifo);
  }
  else if ((flag == O_WEAK_BLACK) && isNotMarked(ref_obj, O_WEAK_BLACK)) {
    ++queued;
     retcode = putFifo(referent, weakly_reachable_fifo);
  }
  else if ((flag == O_PHANTOM_BLACK) && isNotMarked(ref_obj, O_PHANTOM_BLACK)) {
    ++queued;
     retcode = putFifo(referent, phantom_reachable_fifo);
  }

  return retcode < 0 ? retcode : queued;
}

w_int markChildren(w_object o, w_fifo fifo, w_word flag);
void  enqueueReference(w_instance instance);

/*
** Mark all instances in the fifo with the given flag, if they're not
** already marked with the same flag or one which `dominates' it the
** sense explained in the definition of isMarked() above.
** If we mark an instance then we also mark its Class (if not already
** marked) and append all its children to the fifo.  (`Children' means
** elements of a reference array or reference fields of a non-array
** instance). If it is an instance of Thread, Class, ClassLoader, or
** Reference (or a subclass) then there is also some type-specific
** scanning to be done.
** Before doing all this we apply a couple of sanity checks and also check
** for persistent instances, which are handled elsewhere.
*/
w_int markFifo(w_fifo fifo, w_word flag) {
  w_instance parent_instance;
  w_object   parent_object;
  w_instance child_instance;
  w_object   child_object;
  w_int      i;
  w_int      retcode;
  w_int      queued;

  queued = 0;

  woempa(1,"Marking FIFO %p, flag is 0x%02x\n", fifo, flag);

  while ((parent_instance = getFifo(fifo))) {
    parent_object = instance2object(parent_instance);
    woempa(1,"(GC) Tracing instance %k@%p\n", parent_object->clazz,parent_instance);

    if (isSet(parent_object->flags, O_TARGET)) {
      woempa(9, "!!!!! object %k@%p was a TARGET!\n", parent_object->clazz, parent_instance);
      if (parent_object->clazz == clazzString) {
        woempa(9, "  Contents: '%w'\n",String2string(parent_instance));
      }
      unsetFlag(parent_object->flags, O_TARGET | O_GARBAGE);
    }

    if (!isValidInstance(parent_instance)) {
      woempa(9, "Not an object %p\n", parent_object);
    }

    if (!checkPS(parent_instance, flag)) {

      continue;

    }

    /*
    ** If this object is already marked at this or a stronger level, skip it.
    ** Otherwise, mark it now and carry on.
    */
    if (isMarked(parent_object, flag)) {

      continue;

    }

    setFlag(parent_object->flags, flag);

    /*
    ** Mark the Class of this instance.
    */
    child_object = instance2object(clazz2Class(parent_object->clazz));
    if (isNotMarked(child_object, flag)) {
      ++queued;
      if (putFifo(child_object->fields, fifo) < 0) {
        woempa(9, "Hell's bells, fifo %p is full!\n", fifo);

        return -1;

      }
    }

    /*
    ** Now call all the children...
    */
    
    if (parent_object->clazz->aclazz) {
      /*
      ** It's an array clazz; if the array clazz stores non primitives 
      ** (other arrays or references), put them all in the FIFO.
      */
      if (parent_object->clazz->aclazz->depth > 1 || isNotSet(parent_object->clazz->aclazz->componentClazz->flags, CLAZZ_IS_PRIMITIVE)) {
        woempa(1,"(GC) Tracing references from %k@%p\n",parent_object->clazz,parent_instance);
        for (i = 0; i < (w_int)parent_instance[F_Array_length]; i++) {
          if ((child_instance = (w_instance)parent_instance[F_Array_data + i])) {
            if (!isValidInstance(child_instance)) {
              wabort(ABORT_WONKA,"(GC) Instance %p of %k has an element [%d] holding %p\n",parent_instance,parent_object->clazz,i,child_instance);
            }
            else {
              child_object = instance2object(child_instance);
              if (isNotMarked(child_object, flag)) {
                ++queued;
                if (putFifo(child_instance, fifo) < 0) {
                  woempa(9, "Hell's bells, fifo %p is full!\n", fifo);

                  return -1;

                }
              }
            }
          } 
        } 
      }
    }
    else {
      /*
      ** It's a normal clazz, just go over all the fields that could contain 
      ** references to other instances.
      */
      retcode = markChildren(parent_object, fifo, flag);
      if (retcode < 0) {

        return retcode;

      }
      queued += retcode;

      /*
      ** Some types of object also need further processing.
      ** For example, for an instance of Class we scan all the static fields.
      */
      if (parent_object->clazz == clazzClass && parent_instance[F_Class_wotsit]) {
        woempa(1,"(GC) Object %p is instance of %k\n",parent_object,parent_object->clazz);
        retcode = markClazzReachable((w_clazz)parent_instance[F_Class_wotsit], fifo, flag);
        if (retcode < 0) {

          return retcode;

        }
        queued += retcode;
      }

      /*
      ** For a thread, we scan the stack.
      */
      if (isSet(parent_object->clazz->flags,CLAZZ_IS_THREAD) && parent_instance[F_Thread_wotsit]) {
        woempa(1,"(GC) %p is instance of %k, a subclass of Thread\n",parent_instance,parent_object->clazz);
        retcode = markThreadReachable(parent_object, fifo, flag);
        if (retcode < 0) {

          return retcode;

        }
        queued += retcode;
      }

      if (isSet(parent_object->clazz->flags,CLAZZ_IS_THREADGROUP) && parent_instance[F_ThreadGroup_wotsit]) {
        woempa(1,"(GC) Object %p is instance of %k, a subclass of ThreadGroup\n",parent_object,parent_object->clazz);
      }

      /*
      ** For a classloader, we scan all the classes it has loaded.
      */
      if (isSet(parent_object->clazz->flags,CLAZZ_IS_CLASSLOADER) && parent_instance[F_ClassLoader_hashtable]) {
        woempa(1,"(GC) %p is instance of %k, a subclass of ClassLoader\n",parent_instance,parent_object->clazz);
        retcode = markClassLoaderReachable((w_hashtable)parent_instance[F_ClassLoader_hashtable], fifo, flag);
	if (retcode < 0) {

	  return retcode;

	}
	queued += retcode;
      }

      /*
      ** For a reference type, we scan the referent: but only if the reference
      ** we followed to get here was at least as strong as the reference type,
      ** and then the fifo we use depends on the type of rederence.
      */
      if (isSet(parent_object->clazz->flags,CLAZZ_IS_REFERENCE) && parent_instance[F_Reference_referent]) {
        w_object ref_obj = instance2object((w_instance)parent_instance[F_Reference_referent]);
        woempa(1,"(GC) Object %p is instance of %k, a subclass of Reference\n",parent_object,parent_object->clazz);
        woempa(1,"(GC) It refers to instance %p of %k\n", ref_obj, ref_obj->clazz);
        if (isSuperClass(clazzSoftReference,parent_object->clazz)) {
          woempa(1, "(GC) Marking soft reference to %k at %p\n",instance2clazz((w_instance)parent_instance[F_Reference_referent]), (w_instance)parent_instance[F_Reference_referent]);
          if (isSet(ref_obj->flags, O_CONDEMNED)) {
            parent_instance[F_Reference_referent] = 0;
            if (isSet(parent_object->flags, O_ENQUEUEABLE)) {
              enqueueReference(parent_instance);      
            }
          }
          else {
            retcode = markReferenceReachable(ref_obj->fields, fifo, O_SOFT_BLACK);
            if (retcode < 0) {

              return retcode;

            }
            queued += retcode;
          }
        }
        else if (isSuperClass(clazzWeakReference,parent_object->clazz)) {
          woempa(1, "(GC) Marking weak reference to %k at %p\n",instance2clazz((w_instance)parent_instance[F_Reference_referent]), (w_instance)parent_instance[F_Reference_referent]);
          if (isSet(ref_obj->flags, O_CONDEMNED)) {
            parent_instance[F_Reference_referent] = 0;
            if (isSet(parent_object->flags, O_ENQUEUEABLE)) {
              enqueueReference(parent_instance);      
            }
          }
          else {
            retcode = markReferenceReachable(ref_obj->fields, fifo, O_WEAK_BLACK);
            if (retcode < 0) {

              return retcode;

            }
            queued += retcode;
          }
        }
        else if (isSuperClass(clazzPhantomReference,parent_object->clazz)) {
          woempa(1, "(GC) Marking phantom reference to %k at %p\n",instance2clazz((w_instance)parent_instance[F_Reference_referent]), (w_instance)parent_instance[F_Reference_referent]);
          if (isSet(ref_obj->flags, O_CONDEMNED)) {
            if (isSet(parent_object->flags, O_ENQUEUEABLE)) {
              enqueueReference(parent_instance);      
            }
          }
          retcode = markReferenceReachable(ref_obj->fields, fifo, O_PHANTOM_BLACK);
          if (retcode < 0) {

            return retcode;

          }
          queued += retcode;
        }
        else {
          woempa(9, "Eh??? %k is not a subclass of SoftReference, WeakReference, or PhantomReference!\n", parent_object->clazz);
        }
      }

    }
  }
  woempa(1,"Finished marking FIFO %p\n", fifo);

  return queued;
}

/*
** Push an instance onto a fifo and mark the fifo.
*/
w_int markReachable(w_instance ref, w_fifo fifo, w_word flag) {
  w_object o = instance2object(ref);
  w_int    retcode;

  if (isMarked(o, flag)) {

    return 0;

  }

  retcode = putFifo(ref, fifo);
  if (retcode < 0) {

    return retcode;

  }

  retcode = markFifo(fifo, flag);
  
  return retcode < 0 ? retcode : retcode + 1;
}

#define GC_MAX_PRIORITY 10

/*
** Function to set the priority of the GC thread to the maximum allowed.
*/
static inline void
maximizePriority(void) {
  woempa(1,"(GC) New priority = %d\n",GC_MAX_PRIORITY);
  gc_thread->jpriority = GC_MAX_PRIORITY;
  gc_thread->kpriority = priority_j2k(GC_MAX_PRIORITY,0);
  x_thread_priority_set(gc_thread->kthread, gc_thread->kpriority);
}

/*
** Function kickGC() sets the priority of the GC thread to maximum and
** interrupts the thread.  Use this to `forcibly persuade' GC to wake
** up and do something.
*/
void kickGC() {
  if (gc_thread) {
    woempa(9,"(GC) Ouch! OK, I'll try to speed things up a bit ...\n");
    *gc_kicks_pointer += 3;
    woempa(9,"(GC) kicks = %d\n",*gc_kicks_pointer);
    maximizePriority();
    setFlag(gc_thread->flags, WT_THREAD_INTERRUPTED);
    x_thread_stop_waiting(gc_thread->kthread);
  }
  else {
    woempa(9,"(GC) kickGC called before GC thread was started?\n");
  }
}

/*
** The number of objects added to the gray_green_fifo since the last time
** we started to scan the fifo.  Marked volatile because it is always
** incremented by a mutator thread, and we want the GC thread to notice
** that it has changed.
*/
volatile w_int grayed;

/*
** Function markTransientRef should be called whenever a new instance is 
** created or a new reference is created to an existing instance.
** It adds the instance to the gray_green_fifo and increments `grayed'.
*/
void markTransientRef(w_instance instance) {
  w_flags *flagsptr = instance2flagsptr(instance);

  if (gray_green_fifo && isSet(*flagsptr, O_WINDOW)) {
    ++grayed;
    setFlag(*flagsptr, O_GRAY);
    x_mutex_lock(gray_green_fifo_mutex, x_eternal);
    // Retcode not checked => living dangerously!
    putFifo(instance, gray_green_fifo);
    x_mutex_unlock(gray_green_fifo_mutex);
  }
}

w_int markChildren(w_object o, w_fifo fifo, w_word flag) {
  w_instance child_instance;
  w_object   child_object;
  w_int      queued;
  w_clazz    clazz;
  w_int      i;

  queued = 0;
  clazz = o->clazz;
  woempa(1, "Tracing instance fields of %k@%p\n", clazz, o->fields);
  while (clazz) {
    for (i = clazz->numStaticFields; i < clazz->numFields; ++i) {
      if (isSet(clazz->fields[i].flags, FIELD_IS_REFERENCE)) {
        woempa(1, "Class %w field[%d] (`%k') is a reference, tracing it\n", clazz, i, clazz->fields[i].name);
        if ((child_instance = (w_instance)o->fields[clazz->fields[i].slot])) {
          if (!isValidInstance(child_instance)) {
            wabort(ABORT_WONKA, "Instance %p of %k has a reference field[%d] holding %p. valid instance = %s\n",
              o->fields, o->clazz, clazz->fields[i].slot, child_instance, isValidInstance(child_instance) ? "yes" : "no");
          }
          child_object = instance2object(child_instance);
          if (checkPS(child_instance, flag) && isNotMarked(child_object, flag)) {
            ++queued;
            if (putFifo(child_instance, fifo) < 0) {
              woempa(9, "Hell's bells! fifo %p is full\n", fifo);

              return -1;

            }
          }
        }
      }
      else {
        woempa(1, "Class %w field[%d] (`%k') is a primitive, getting out\n", clazz, i, clazz->fields[i].name);
        break;
      }
    }
    clazz = clazz->superClazz;
  }

/*
  cifs = o->clazz->icifs;
  if (cifs) {
    woempa(1,"(GC) Tracing references from %k@%p\n",o->clazz,o->fields);
    for (;*cifs;++cifs) {
      woempa(1, "Scanning slot[%d]\n", (*cifs)->slot);
      if ((child_instance = (w_instance)o->fields[(*cifs)->slot])) {
        if (!isValidInstance(child_instance)) {
          wabort(ABORT_WONKA, "Instance %p of %k has a reference field[%d] holding %p. valid instance = %s\n",
            o->fields, o->clazz, (*cifs)->slot, child_instance, isValidInstance(child_instance) ? "yes" : "no");
        }
        child_object = instance2object(child_instance);
        if (checkPS(child_instance, flag) && isNotMarked(child_object, flag)) {
          ++queued;
          if (putFifo(child_instance, fifo) < 0) {
            woempa(9, "Hell's bells! fifo %p is full\n", fifo);

            return -1;

          }
        }
      }
    }
  }
*/


  return queued;
}

/**
 * Our garbage collection algorithm has three main phases:
 * - the Preparation phase takes a snapshot of the instance heap,
 *   and makes a list of collectable (non-immortal objects which
 *   existed at a certain moment.
 * - the Mark phase marks all instances which are strongly, softly,
 *   weakly, or phantom-reachable.
 * - the Sweep phase reclaims the memory of all the items which were
 *   identified as collectable in the Preparsatione phase, and not marked
 *   reachable in the Mark phase.  It also performs finalization and
 *   identifies which instances are only reachable via soft or weak counters, 
 *   and clears and enqueues references where appropriate.
 */

#define GC_PHASE_UNREADY   0
#define GC_PHASE_PREPARE   1
#define GC_PHASE_MARK      2
#define GC_PHASE_SWEEP     3
#define GC_PHASE_COMPLETE  4
w_int gc_phase = GC_PHASE_UNREADY;

/*
** The thread (if any) which is currently running a PREPARE/MARK sequence
** on the heap.
*/
static w_thread marking_thread;

/*
 * In the Preparation phase, we take a snapshot of the instance heap
 *  and build two fifo's: the `window' fifo, which contains all mortal
 *  items in the snapshot, and the `gray-green' fifo, which initially
 *  contains all the immortal (SYSTEM or GREEN) items.
 */
void preparation_iteration_1(w_word key, w_word value, void* foo, void *bar) {
  w_object object = instance2object((w_instance)key);

  unsetFlag(object->flags, O_BLACK | O_NEAR_BLACK);
}

void preparation_iteration_2(w_word key, w_word value, void* foo, void *bar) {
  w_object object = instance2object((w_instance)key);
  w_int    retcode;
  w_int    marked;

  if (isSet(object->flags, O_GREEN | O_SYSTEM)) {
    ++*((w_int*)foo);
    x_mutex_lock(gray_green_fifo_mutex, x_eternal);
    retcode = putFifo(object->fields, gray_green_fifo);
    if (retcode < 0) {
      wabort(ABORT_WONKA, "Couldn't add to gray_green_fifo, PANIC\n");
    }
    woempa(1,"Instance %p -> gray_green\n", object->fields);
    x_mutex_unlock(gray_green_fifo_mutex);
  }
  else {
    ++*((w_int*)bar);
    setFlag(object->flags, O_WINDOW);
    retcode = putFifo(object->fields, window_fifo);
    if (retcode < 0) {
      wabort(ABORT_WONKA, "Couldn't add to window_fifo, PANIC\n");
    }
    woempa(1,"Instance %p -> window\n", object->fields);

    if (isSet(object->flags, (O_FINALIZING | O_FINALIZABLE | O_ENQUEUEABLE | O_ENQUEUEING))) {
      marked = markChildren(object, finalize_reachable_fifo, O_FINALIZE_BLACK);
      if (marked < 0) {
        wabort(ABORT_WONKA, "Couldn't add to finalize_reachable_fifo, PANIC\n");
      }
      woempa(1, "Marked %d children of yet-to-be-finalized %k@%p\n", marked, object->clazz, object->fields);
    }
  }
}

w_int preparationPhase(void) {
  w_int marked1;
  w_int marked2;
  w_int marked3;
  w_int gray_greens = 0;
  w_int candidates = 0;
  w_int memory_busy = memory_total - x_mem_avail();

  memory_load_factor = memory_busy / (memory_total - memory_busy);
  if (memory_load_factor < 0) {
    memory_load_factor = 0;
  }
/* [CG 20020116]
   Don't do this - keep the fifo

  window_fifo = allocFifo(W_Realm_system, WINDOW_FIFO_LEAF_SIZE);
*/

  ht_lock(instance_hashtable);
  woempa(1, "(GC) Begin preparation phase, %d instances\n", instance_hashtable->occupancy);
  marked1 = ht_iterate_no_lock(instance_hashtable, preparation_iteration_1, NULL, NULL);
  marked2 = ht_iterate_no_lock(instance_hashtable, preparation_iteration_2, &gray_greens, &candidates);
  ht_unlock(instance_hashtable);

  if (marked1 != marked2) {
    wabort(ABORT_WONKA, "Allah is great!  %d != %d\n", marked1, marked2);
  }
  woempa(1 , "(GC) Found %d objects: %d GRAY/GREENs, %d candidates for collection.\n", marked1, gray_greens, candidates);
  
  marked3 = markFifo(finalize_reachable_fifo, O_FINALIZE_BLACK);
  woempa(1, "(GC) Marked %d objects FINALIZE_BLACK\n", marked3);

  return marked3;
}

w_int markGrayGreens(void);
/*
 * In the Mark phase, we first mark all the (transient) `roots':
 *  - system thread group (and hence all threads)
 *  - system class hashtable (static fields of system classes)
 *  - global references hashtable (includes current local references)
 * For every item we mark, we append all references it contains to
 * the appropriate fifo (strong, soft, weak, or phantom).  Therefore
 * by the end of this phase we have marked all items in the window
 * which were reachable at the time the snapshot was taken.
 *
 * During this time new objects and new references to existing objects
 * can be created: all of these are appended to the gray-green fifo.
 * We now scan the gray-green fifo until it is exhausted, treating the
 * items found in it as strongly reachable (which they may or not be -
 * we are playing safe).  Even during this scan new references can be 
 * added to the gray-green fifo, but when it becomes empty we can be sure 
 * that we have achieved closure.  We now make one final pass over the
 * strong, soft, weak and phantom fifo's (in that order) before proceeding
 * to the Sweep phase.
 *
 * Returns the number of objects marked if successful (useless, and
 * probably inaccurate, but at least nonnegative), or a negative error
 * code.  In the latter case the mark phase did not complete, and it
 * would be folly to sweep.
 */
w_int markPhase(void) {
  w_fifo     temp_fifo;
  w_clazz    clazz;
  w_object   object;
  w_instance instance;
  w_int      retcode;
  w_int      marked = 0;

  woempa(1,"(GC) Marking system ThreadGroup.\n");
  markReachable(W_ThreadGroup_system->ThreadGroup, strongly_reachable_fifo, O_BLACK);

  woempa(1,"(GC) Marking system class hashtable.\n");
  temp_fifo = ht_list_values(system_class_hashtable);
  if (!temp_fifo) {

    return -1;

  }
  while ((clazz = getFifo(temp_fifo))) {
    retcode = markClazzReachable(clazz, strongly_reachable_fifo, O_BLACK);
    if (retcode < 0) {

      return retcode;

    }
  }
  marked += retcode;
  releaseFifo(temp_fifo);

  woempa(1,"(GC) Marking globals/locals hashtable.\n");
  temp_fifo = ht_list_keys(globals_hashtable);
  if (!temp_fifo) {

    return -1;

  }
  while ((instance = getFifo(temp_fifo))) {
    object = instance2object(instance);
    woempa(1, "(GC) Global reference to %k@%p.\n", object->clazz, instance);
    if (isNotMarked(object, O_BLACK)) {
      retcode = putFifo(instance, strongly_reachable_fifo);
      if (retcode < 0) {
        woempa(9, "Oof! strongly_reachable_fifo is full\n");

        return retcode;

      }
    }
  }
  markFifo(strongly_reachable_fifo, O_BLACK);
  releaseFifo(temp_fifo);

  do {
    grayed = 0;
    woempa(1,"(GC) Marking Grays and Greens.\n");
    marked = markGrayGreens();
    woempa(1,"Marked %d GRAYs and GREENs, %d GRAYs left\n",marked, grayed);
  } while (grayed);

  markFifo(strongly_reachable_fifo, O_BLACK);
  markFifo(softly_reachable_fifo, O_SOFT_BLACK);
  markFifo(weakly_reachable_fifo, O_WEAK_BLACK);
  markFifo(phantom_reachable_fifo, O_PHANTOM_BLACK);

  return marked;
}

w_int collectThreadGarbage(w_object object) {
  w_thread thread = (w_thread)object->fields[F_Thread_wotsit];

  if (!thread || ht_read(thread_hashtable,(w_word)thread->kthread) == 0) {

    return 1;

  }
  else {
    woempa(9, "Hold on a moment - thread '%w' is still running...\n", NM(thread));
    return 0;

  }
}

w_int collectOtherGarbage(w_object object) {

  return 1;
}

static w_boolean checkExternalRefs(w_object object) {
#ifdef ENABLE_PS
  if (isSet(object->flags, O_LOCAL_COPY) && !releaseLocalCopy(object->fields)) {
    woempa(1,"Not releasing %k@%p, as is externally reachable (from Persistent Object Store)\n",object->clazz,object->fields);

    return WONKA_FALSE;

  }
#endif
  return WONKA_TRUE;
}

static w_boolean checkStrongRefs(w_object object) {
  if (isSet(object->flags, O_BLACK | O_GRAY)) {
    woempa(1, "Not collecting %k@%p because it is strongly reachable (%s %s)\n", object->clazz, object->fields, isSet(object->flags, O_BLACK)?"BLACK":"", isSet(object->flags, O_GRAY)?"GRAY":"");

    return WONKA_FALSE;

  }

  return WONKA_TRUE;
}

static w_boolean checkSoftRefs(w_object object) {
  if (isSet(object->flags, O_SOFT_BLACK)) {
    woempa(1, "Not collecting %k@%p because it is softly reachable\n", object->clazz, object->fields);
    if ((memory_load_factor > 0) || (*gc_kicks_pointer > 0)) {
      woempa(1, "Condemning %k@%p because it is only softly reachable, memory load factor is %d, gc_kicks is %d\n", object->clazz, object->fields, memory_load_factor, *gc_kicks_pointer);
      setFlag(object->flags, O_CONDEMNED);
    }

    return WONKA_FALSE;
  }

  return WONKA_TRUE;
}

static w_boolean checkWeakRefs(w_object object) {
  if (isSet(object->flags, O_WEAK_BLACK)) {
    woempa(1, "Not collecting %k@%p because it is weakly reachable\n", object->clazz, object->fields);
    setFlag(object->flags, O_CONDEMNED);

    return WONKA_FALSE;
  }

  return WONKA_TRUE;
}

static w_boolean checkFinalization(w_object object) {

  if (isSet(object->flags, (O_FINALIZING | O_ENQUEUEING))) {
    woempa(1, "Not collecting %k@%p)because it is currently being enqueued or finalized\n", object->clazz, object->fields);

    return WONKA_FALSE;

  }

  if (isNotSet(object->flags, O_FINALIZABLE)) {

    return WONKA_TRUE;

  }

  woempa(1, "Not collecting %k@%p because it is finalizable.\n", object->clazz, object->fields);

  if (object->clazz == clazzString) {
    if (object->fields[F_String_wotsit]) {
      woempa(1,"(GC) Releasing String %p '%w'\n",object->fields,object->fields[F_String_wotsit]);
    }
    else {
      woempa(9, "(GC) java.lang.String @ %p has wotsit==null !\n", object->fields);
    }
    String_finalize(w_thread2JNIEnv(currentWonkaThread), object->fields);
    unsetFlag(object->flags, O_FINALIZABLE);
  
    return WONKA_FALSE;

  }

  woempa(1, "Putting %k@%p onto finalization queue.\n", object->clazz, object->fields);
  x_mutex_lock(finalizer_fifo_mutex, x_eternal);
  if (putFifo(object->fields, finalizer_fifo) < 0) {
    woempa(9, "Couldn't put  %k@%p onto finalizer_fifo, punting\n", object->clazz, object->fields);
  }
  else {
    setFlag(object->flags, O_FINALIZING);
    unsetFlag(object->flags, O_FINALIZABLE);
  }
  x_mutex_unlock(finalizer_fifo_mutex);
  
  return WONKA_FALSE;
}

static w_boolean checkPhantomRefs(w_object object) {
  if (isSet(object->flags, O_PHANTOM_BLACK)) {
    woempa(1, "Not collecting %k@%p)because it is reachable via phantom reference(s)\n", object->clazz, object->fields);
    setFlag(object->flags, O_CONDEMNED);

    return WONKA_FALSE;

  }
  if (isSet(object->flags, O_FINALIZE_BLACK)) {
    woempa(1, "Not collecting %k@%p)because it is reachable via yet-to-be-finalized object(s)\n", object->clazz, object->fields);

    return WONKA_FALSE;

  }

  return WONKA_TRUE;
}

/*
** Sweep the window_fifo until either at least `target' bytes are freed
** or the fifo is exhausted.  Caller must own gc_monitor.
*/
w_size sweep(w_thread thread, w_int target) {
  w_instance instance;
  w_object object;
  w_int    objects_examined = 0;
  w_int    objects_freed = 0;
  w_int    bytes_freed = 0;
  w_int    collected;

  if (gc_phase < GC_PHASE_SWEEP) {
    woempa(9, "Not in SWEEP phase.\n");

    return 0;

  }

  while(window_fifo) {
    instance = getFifo(window_fifo);
    if (!instance) {
    woempa(1, "Exhausted window_fifo after collecting %d bytes in %d objects.\n", bytes_freed, objects_freed);

      break;

    }
    ++objects_examined;
    object = instance2object(instance);
    woempa(1, "Examining %k@%p\n", object->clazz, instance);
    unsetFlag(object->flags, O_GRAY);
    if ( checkExternalRefs(object)
      && checkStrongRefs(object)
      && checkSoftRefs(object)
      && checkWeakRefs(object)
      && checkFinalization(object)
      && checkPhantomRefs(object)
       ) {
#ifdef SECOND_CHANCE
      if (isSet(object->flags, O_GARBAGE))
#else
      if (isSet(object->flags, O_TARGET | O_GARBAGE)) // <<<=== !!!
#endif
      {
        if (isSet(object->clazz->flags,CLAZZ_IS_THREAD)) {
          collected = collectThreadGarbage(object);
        }
        else {
          collected = collectOtherGarbage(object);
        }

        if (collected) {
          releaseMonitor(object->fields);
          bytes_freed += releaseInstance(object);
          instance_returned += 1;
          instance_use -= 1;
        }
        objects_freed += collected;

        if (bytes_freed >= target) {
          woempa(1, "Target was %d, freed %d, job done.\n", target, bytes_freed);

          break;

        }
      }
      else {
        if (isSet(object->flags, O_TARGET)) {
          setFlag(object->flags, O_GARBAGE);
        }
        else {
          setFlag(object->flags, O_TARGET);
        }
      }
    }
  }

  woempa(1, "(GC) Collected %i bytes from %d objects out of %d.\n", bytes_freed, objects_freed, objects_examined);

#ifdef USE_DISCARD_COLLECT
  x_mem_collect(&bytes_freed, &objects_freed);
  woempa(1, "(GC) Oswald sez: Collected %i bytes from %d objects.\n", bytes_freed, objects_freed);
#endif

  return bytes_freed;
}

void sweepPhase(void) {
#ifdef BYTES_PER_MEAL
  w_size   collected = 0;

  while(1) {
    x_sleep  winks = x_millis2ticks((x_size)(MILLIS_PER_NAP / (memory_load_factor + 1)));

    if (collected && *gc_kicks_pointer == 0) {
      woempa(1, "(GC) Collected %d bytes so far, taking %d winks.\n", collected, winks);
      x_monitor_wait(gc_monitor, winks);
    }
    collected = sweep(gc_thread, BYTES_PER_MEAL);
    if (collected < BYTES_PER_MEAL) {

      break;

    }
  } 
#else
  sweep(gc_thread, 2000000000);
#endif

/* [CG 20020116]
   Don't do this - keep the fifo

  if (window_fifo) {
    releaseFifo(window_fifo);
    window_fifo = NULL;
  }
  else {
    wabort(ABORT_WONKA,"??????\n");
  }
*/
}

w_int markGrayGreens() {
  w_instance instance;
  w_object   object;
  w_int      graysLeft;
  w_int      retcode;

  graysLeft = 0;
  x_mutex_lock(gray_green_fifo_mutex, x_eternal);
  while ((instance = getFifo(gray_green_fifo))) {
    x_mutex_unlock(gray_green_fifo_mutex);
    object = instance2object(instance);

    if (isNotMarked(object, O_BLACK)) {
      retcode = putFifo(instance, strongly_reachable_fifo);
      if (retcode < 0) {
        woempa(9, "Amaai amaai, strongly_reachable_fifo is full\n");

        return retcode;

      }
      ++graysLeft;
    }
    x_mutex_lock(gray_green_fifo_mutex, x_eternal);
  }
  x_mutex_unlock(gray_green_fifo_mutex);
  markFifo(strongly_reachable_fifo, O_BLACK);

  return graysLeft;
}

/*
char *reportCollector() {

  static char buffer[256];

  x_snprintf(buffer, 256, "Collector: %d instances %d freed (remaining %d bytes out of %d ) ticks = %ld, stings = %d.", instance_use, instance_returned, x_mem_avail(), x_mem_total(), x_time_get(), sting_count);

  return buffer;
  
}
*/

void enqueueReference(w_instance instance) {
  x_mutex_lock(enqueue_fifo_mutex, x_eternal);
  if (putFifo(instance, enqueue_fifo) < 0) {
    woempa(9, "Couldn't put  %k@%p onto enqueuefifo, punting\n", instance2clazz(instance), instance);
  }
  else {
    setFlag(instance2flags(instance), O_ENQUEUEING);
    unsetFlag(instance2flags(instance), O_ENQUEUEABLE);
  }
  x_mutex_unlock(enqueue_fifo_mutex);
}

/*
** List of w_reclaim_listener's, and a monitor to protect access thereto.
*/
w_wordset reclaim_listener_list = NULL;
x_Monitor reclaim_listener_Monitor;
x_monitor reclaim_listener_monitor = NULL;

/*
** We only start making callbacks after accumulating RECLAIM_THRESHOLD
** bytes' worth of requests.  The requests are accumulated in reclaim_accumulator.
*/
#define MIN_RECLAIM_THRESHOLD (256 * 1024)
#define MAX_RECLAIM_THRESHOLD (16 * 1024 * 1024)
static w_size reclaim_threshold = MIN_RECLAIM_THRESHOLD;
w_size  reclaim_accumulator = 0;

void registerReclaimCallback(w_reclaim_callback callback) {
  x_status status;

  if (!reclaim_listener_monitor) {
    woempa(1, "Creating reclaim_listener_monitor\n");
    reclaim_listener_monitor = &reclaim_listener_Monitor;
    x_monitor_create(reclaim_listener_monitor);
  }

  status = x_monitor_enter(reclaim_listener_monitor, x_eternal);
  woempa(1, "Entered reclaim_listener_monitor, status = %d\n", status);
  addToWordset(gc_thread, &reclaim_listener_list, (w_word)callback);
  status = x_monitor_exit(reclaim_listener_monitor);
  woempa(1, "Left reclaim_listener_monitor, status = %d\n", status);
}

void deregisterReclaimCallback(w_reclaim_callback callback) {
  x_status status;

  if (!reclaim_listener_monitor) {
    wabort(ABORT_WONKA, "No reclaim_listener_monitor!\n");
  }

  status = x_monitor_enter(reclaim_listener_monitor, x_eternal);
  woempa(1, "Entered reclaim_listener_monitor, status = %d\n", status);
  removeFromWordset(&reclaim_listener_list, (w_word)callback);
  status = x_monitor_exit(reclaim_listener_monitor);
  woempa(1, "Left reclaim_listener_monitor, status = %d\n", status);
}

/*
** Function gc_reclaim() is called when we would like to reclaim `requested'
** bytes of memory.  Note: after calling the callbacks we reset the accumulator
** to zero even if the amount reclaimed fell short of that requested.  We
** assume that the inability (or unwillingness) to release memory is likely
** to last a little while, and setting reclaim_accumulator = remaining would
** tend to trigger another reclaim cycle rather soon.  For further study.
*/

w_size gc_reclaim(w_int requested, w_instance caller) {
#ifdef DISTRIBUTED_GC
  x_status status;
  w_size   i;
  w_size   n;
  w_int   remaining = 0;

  if (!reclaim_listener_monitor) {
    woempa(1, "Call to gc_reclaim() before any listener added\n");

    return 0;

  }

  status = x_monitor_enter(reclaim_listener_monitor, x_eternal);
  woempa(1, "Entered reclaim_listener_monitor, status = %d\n", status);
  reclaim_accumulator += requested * (memory_load_factor + 1);
  if (reclaim_accumulator > reclaim_threshold) {
    remaining = reclaim_accumulator;
    n = (reclaim_listener_list->occupancy) / sizeof(w_word);
    for (i = 0; remaining > 0 && i < n; ++i) {
      x_monitor_enter(gc_monitor, x_eternal);
      remaining -= ((w_reclaim_callback)((w_word*)reclaim_listener_list->contents)[i])(remaining, caller);
      x_monitor_exit(gc_monitor);
    }
    reclaim_accumulator = 0;
    if (remaining > 0) {
      woempa(1, "Still need  %d bytes\n", remaining);
    }
    if (MAX_RECLAIM_THRESHOLD / (memory_load_factor + 1) > MIN_RECLAIM_THRESHOLD) {
      reclaim_threshold = MAX_RECLAIM_THRESHOLD / (memory_load_factor + 1);
    }
  }
  status = x_monitor_exit(reclaim_listener_monitor);
  woempa(1, "Left reclaim_listener_monitor, status = %d\n", status);

  return requested - remaining;
#else
  return 0;
#endif
}

/*
** Function internal_reclaim_callback calls `sweep' to try to reclaim memory.
*/
w_size internal_reclaim_callback(w_int requested, w_instance instance) {
  w_size reclaimed = 0;

  x_monitor_enter(gc_monitor, x_eternal);
  reclaimed += gc_request(requested);
  woempa(1,"Called with requested = %d, instance = %p : reclaimed %d\n", requested, instance, reclaimed);
  x_monitor_exit(gc_monitor);

  return reclaimed;
}

/*
** Function gc_create() is called by the constructor of GarbageCollector
** (which is a singleton, so this code is only executed once).
*/
void gc_create(JNIEnv *env, w_instance theGarbageCollector) {
  string_enqueue = cstring2String(W_Thread_system, "enqueue", 7);
  gc_instance = theGarbageCollector;
  gc_thread = JNIEnv2w_thread(env);
  gc_monitor = allocMonitor(theGarbageCollector);
  gc_kicks_pointer = (w_int*)&theGarbageCollector[F_GarbageCollector_kicks];
  instance_returned = 0;
  marking_thread = NULL;
  memory_total = x_mem_total() - min_heap_free;
  memory_load_factor = 1;
  gray_green_fifo_mutex = &gray_green_fifo_Mutex;
  finalizer_fifo_mutex = &finalizer_fifo_Mutex;
  enqueue_fifo_mutex = &enqueue_fifo_Mutex;
  x_mutex_create(gray_green_fifo_mutex);
  x_mutex_create(finalizer_fifo_mutex);
  x_mutex_create(enqueue_fifo_mutex);
  window_fifo = allocFifo(W_Realm_system, WINDOW_FIFO_LEAF_SIZE);
  strongly_reachable_fifo = allocFifo(W_Realm_system, STRONG_FIFO_LEAF_SIZE);
  softly_reachable_fifo = allocFifo(W_Realm_system, OTHER_FIFO_LEAF_SIZE);
  weakly_reachable_fifo = allocFifo(W_Realm_system, OTHER_FIFO_LEAF_SIZE);
  phantom_reachable_fifo = allocFifo(W_Realm_system, OTHER_FIFO_LEAF_SIZE);
  finalize_reachable_fifo = allocFifo(W_Realm_system, OTHER_FIFO_LEAF_SIZE);
  enqueue_fifo = allocFifo(W_Realm_system, ENQUEUE_FIFO_LEAF_SIZE);
  finalizer_fifo = allocFifo(W_Realm_system, FINALIZER_FIFO_LEAF_SIZE);
  // deadref_fifo = allocFifo(W_Realm_system, DEADREF_FIFO_LEAF_SIZE);
  gray_green_fifo = allocFifo(W_Realm_system, GRAY_FIFO_LEAF_SIZE);
  classMustBeInitialized(clazzReference, W_ThreadGroup_system->manager);
  classMustBeInitialized(clazzFinalizer, W_ThreadGroup_system->manager);
  woempa(7,"         window_fifo at %p\n", window_fifo);
  woempa(7,"      finalizer_fifo at %p\n", finalizer_fifo);
  woempa(7,"        enqueue_fifo at %p\n", enqueue_fifo);
  woempa(7,"     gray_green_fifo at %p\n", gray_green_fifo);
  woempa(7, "W_ThreadGroup_system = %p.\n", W_ThreadGroup_system);
  woempa(7, "     W_Thread_system = %p.\n", W_Thread_system);
  woempa(7, "      W_Realm_system = %p.\n", W_Realm_system);
  woempa(7, "           gc_thread = %p.\n", gc_thread);
  woempa(7, "   Current instances = %d.\n", instance_use);

  woempa(7, "Registering internal_reclaim_callback\n");
  registerReclaimCallback(internal_reclaim_callback);
}


w_int gc_collect(w_instance theGarbageCollector) {
  w_int gc_pass_count = theGarbageCollector[F_GarbageCollector_passes];
  w_int done = 0;
  w_int retcode;

  woempa(7,"(GC) Begin pass %d\n",gc_pass_count);
  if (gc_pass_count % PRINTRATE == 0) woempa(7, "(GC) %d Collector: %d instances %d freed (remaining %d bytes out of %d) ticks = %ld, sting = %d, priority = %d, wait time = %d.\n", gc_pass_count, instance_use, instance_returned, x_mem_avail(), x_mem_total(), x_time_get(), sting_count, gc_thread->jpriority,gc_instance[F_GarbageCollector_sleep_millis]);
  sting_count = 0;

#ifdef ENABLE_PS
  persistentGarbageCollector();
#endif

  while (done < 2) {
    switch (gc_phase) {
      case GC_PHASE_UNREADY:
      case GC_PHASE_COMPLETE:
        woempa(1, "Thread %w: Phase = COMPLETE, doing a PREPARE and MARK\n", currentWonkaThread->name);
        marking_thread = currentWonkaThread;
        gc_phase = GC_PHASE_PREPARE;
        retcode = preparationPhase();
        if (retcode < 0) {
          woempa(9, "Aborting PREPARE\n");

          break;

        }
        gc_phase = GC_PHASE_MARK;
        retcode = markPhase();
        if (retcode < 0) {
          woempa(9, "Aborting PREPARE\n");

          break;

        }
        woempa(1,"(GC) Thread %w: Entering SWEEP phase.\n", currentWonkaThread->name);
        gc_phase = GC_PHASE_SWEEP;
        marking_thread = NULL;
        x_monitor_notify_all(gc_monitor);
        done += 1;
        /* fall through */

      case GC_PHASE_SWEEP:
        woempa(1, "Thread %w: Phase = SWEEP, joining in the fun\n", currentWonkaThread->name);
        sweepPhase();
        gc_phase = GC_PHASE_COMPLETE;
        x_monitor_notify_all(gc_monitor);
        done *= 2;
        woempa(1, "Thread %w: Have %sperformed a complete cycle\n", currentWonkaThread->name, done?"":"not yet ");
        break;

      default: // GC_PHASE_PREPARE/MARK
        woempa(1, "Thread %w: Phase = PREPARE/MARK, waiting\n", currentWonkaThread->name);
        x_monitor_wait(gc_monitor, 10); // micronap
    }
  }
  woempa(7,"(GC) End pass %d : %d instances returned so far\n",gc_pass_count, instance_returned);

  return instance_returned;
}

w_int gc_request(w_int requested) {
  w_int released = 0;
  w_int remaining = requested;
  w_int swept;
  w_int tries;
  w_int retcode;

  if (marking_thread == currentWonkaThread) {
    woempa(1, "Thread %w is already running a PREPARE/MARK, ignoring request\n", marking_thread->name);
    tries = 0;
  }
  else {
    tries = 3;
  }

  while (tries > 0 && remaining > 0) {
    switch (gc_phase) {
      case GC_PHASE_PREPARE:
      case GC_PHASE_MARK:
        woempa(1, "Thread %w: Phase = PREPARE/MARK, waiting\n", currentWonkaThread->name);
        x_monitor_wait(gc_monitor, 10); // micronap
        break;

      case GC_PHASE_SWEEP:
        woempa(1, "Thread %w: Phase = SWEEP, joining in the fun\n", currentWonkaThread->name);
        swept = sweep(gc_thread, remaining);
        released += swept;
        remaining -= swept;
        woempa(1, "Thread %w: Reclaimed %d bytes, %d to go\n", currentWonkaThread->name, swept, remaining);
        if (swept == 0) {
          gc_phase = GC_PHASE_COMPLETE;
          x_monitor_notify_all(gc_monitor);
        }
        tries -= 1;

        break;

      case GC_PHASE_COMPLETE:
        woempa(1, "Thread %w: Phase = COMPLETE, doing a PREPARE and MARK\n", currentWonkaThread->name);
        marking_thread = currentWonkaThread;
        gc_phase = GC_PHASE_PREPARE;
        retcode = preparationPhase();
        if (retcode < 0) {
          woempa(9, "Aborting PREPARE\n");

          break;

        }
        gc_phase = GC_PHASE_MARK;
        retcode = markPhase();
        if (retcode < 0) {
          woempa(9, "Aborting PREPARE\n");

          break;

        }
        gc_phase = GC_PHASE_SWEEP;
        marking_thread = NULL;
        x_monitor_notify_all(gc_monitor);
        break;

      default: 
        /*  GC_PHASE_UNREADY - don't try to reclaim anything yet */
        tries = 0;
    }
  }
  woempa(7, "Released: %d bytes\n", released);

  return released;
}

void gc_housekeeping(void) {
#ifdef CACHE_INSTANCES
  /*
  ** If cache_keep is non negative, clean out the system caches.
  */

  if (cache_keep >= 0) {
    ht_every(system_class_hashtable, emptySingleCache);
  }

  /*
  ** After each 20'th pass, we clean out the caches...
  */
    
  if (cache_keep >= 0) {
    cache_keep = -1;
  }
    
  if (gc_pass_count % 30 == 0) {
    cache_keep = 0;
  }
#endif 
}

void initHeap(void) {
}

