/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: persistency.c,v 1.12 2002/03/22 11:53:54 gray Exp $
*/

#include <string.h>

#include "arrays.h"
#include "core-classes.h"
#include "clazz.h"
#include "descriptor.h"
#include "exception.h"
#include "fifo.h"
#include "heap.h"
#include "loading.h"
#include "locks.h"
#include "pm.h"
#include "ps-mem.h"
#include "strings.h"
#include "hashtable.h"
#include "threads.h"
#include "ts-mem.h"
#include "stack.h"
#include "ieee754.h"

#define bytes2words(b) (((b)+sizeof(w_word)-1)/sizeof(w_word))
#define bits2words(b)  (((b)+8*sizeof(w_word)-1)/(8*sizeof(w_word)))

char *PS_start;
char *PS_end;

w_hashtable l2p_hashtable;
w_hashtable proots_hashtable;

w_fifo proots_fifo;
w_fifo dirty_fifo;

w_pobject alloc_P_Object (w_size objsize);
w_instance alloc_P_Instance (w_instance);
w_instance alloc_P_String  (w_string);
w_instance alloc_P_Clazz  (w_clazz clazz);
w_instance createPersistentReference(w_instance ref);
w_instance findPersistentClass(w_clazz clazz);
void readInstance (w_instance pinstance, w_instance tinstance);
void writeClazz (w_clazz clazz, w_instance pinstance);
void writeString (w_string string, w_instance pinstance);
w_string readString (w_instance pinstance);
w_clazz  readClazz (w_instance pinstance, w_instance loader);
void sweep_P_Object(w_pobject pobject);
void scanLocalCopy(w_word wtinstance, w_word wpinstance);
w_size traceRoot (w_instance pinstance);

#ifdef ENABLE_PS
/*
** Give the the type of a reference: address of a transient instance,
** address of the "local copy" of a persistent instance, 
** address of a persistent instance, or persistent reference.
*/
int typeOfInstance(w_instance instance) {
w_word flags = 0;

  if (instance) {
    flags = instance2flags(instance);
  }

  switch (flags & (O_PERSISTENT | O_LOCAL_COPY)) {
  case 0:
    return INSTANCE_IS_TRANSIENT;

  case O_LOCAL_COPY:
    return INSTANCE_IS_LOCAL_COPY;

  case O_PERSISTENT:
    return INSTANCE_IS_PERSISTENT;

  default:
    woempa(9,"Impossible flags 0x%08x for instance %p\n",flags,instance);
    return 42;
  }
}

inline static w_instance inline_local_copy2pinstance(w_instance pinstance) {
  return (w_instance)ht_read(l2p_hashtable, (w_word)pinstance);
}

w_instance local_copy2pinstance(w_instance pinstance) {
  return inline_local_copy2pinstance(pinstance);;
}

w_instance findPersistentString(w_string string) {
  w_instance tString = getStringInstance(W_Thread_system,string);
  w_instance pString;

  ht_lock(l2p_hashtable);
  pString = local_copy2pinstance(tString);
  if (pString == NULL) {
    pString = alloc_P_String(string);
    woempa(7,"  --> created new persistent string '%w' @ %p\n",string,pString);
    ht_write_no_lock(l2p_hashtable, (w_word)tString, (w_word)pString);
    instance2pobject(pString)->local_copy = instance2object(tString);
    instance2pobject(pString)->hashcode = (w_int)tString;
    woempa(7,"      persistent hashcode is 0x%08x\n", tString);
    setFlag(instance2flags(tString),O_LOCAL_COPY);
    writeString(string,pString);
  }
  ht_unlock(l2p_hashtable);

  return pinstance2ref(pString);
}

w_instance findPersistentClass(w_clazz clazz) {
  w_instance tClass = clazz2Class(clazz);
  w_instance pClass;

  ht_lock(l2p_hashtable);
  pClass = local_copy2pinstance(tClass);
  if (pClass == NULL) {
    pClass = alloc_P_Clazz(clazz);
    woempa(7,"  --> created new persistent class '%k' @ %p\n", clazz,pClass);
    ht_write_no_lock(l2p_hashtable, (w_word)tClass, (w_word)pClass);
    instance2pobject(pClass)->local_copy = instance2object(tClass);
    instance2pobject(pClass)->class = findPersistentClass(clazzClass);
    instance2pobject(pClass)->hashcode = (w_int)tClass;
    woempa(7,"      persistent hashcode is 0x%08x\n", tClass);
    setFlag(instance2flags(tClass),O_LOCAL_COPY);
    writeClazz(clazz,pClass);
  }
  ht_unlock(l2p_hashtable);

  return pinstance2ref(pClass);
}


/*
** pinstance2local_copy finds or creates a local copy corresponding to 
** the given persistent instance.
*/

w_instance 
pinstance2local_copy(w_instance pinstance) {
  w_pobject pobject = instance2pobject(pinstance);
  w_object local_copy;

  ht_lock(l2p_hashtable);
  local_copy = pobject->local_copy;
  woempa(7,"local_copy of %p is %p\n", pinstance, local_copy);
  if (local_copy == NULL) {
    w_instance new;
    // TODO: can we be sure that there is always a local copy of java.lang.Class?
    w_instance ourClass = ref2local_copy(pobject->class);
    w_clazz clazz = Class2clazz(ourClass);

    woempa(7,"No local copy of %p exists yet, allocating an instance of %k\n", pinstance,  clazz);
    if (clazz->aclazz) {
      w_word length;
      length = pinstance[0];
      new = allocArrayInstanceWithFlags(W_Thread_system, clazz, 1, &length, O_LOCAL_COPY);
      if (new == NULL) {
        wabort(ABORT_WONKA,"Blimey! No memory available for local copy of array\n");
      }
      local_copy = instance2object(new);
    }
    else {
      new = allocInstanceWithFlags(W_Thread_system,clazz,O_LOCAL_COPY);
      if (new == NULL) {
        wabort(ABORT_WONKA,"Blimey! No memory available for local copy of instance\n");
      }
      local_copy = instance2object(new);
    }

    local_copy->clazz = clazz;
    setFlag(local_copy->flags, O_LOCAL_COPY);
    pobject->local_copy = local_copy;
    woempa(7,"Local copy of %k@%p is %p\n", clazz,instance2pobject(pinstance)->fields,local_copy->fields);
    readInstance(pinstance,new);
    ht_write_no_lock(l2p_hashtable, (w_word)new, (w_word)pinstance);
  }
  ht_unlock(l2p_hashtable);

  woempa(7,"Returning %p (flags = 0x%08x) as local copy of %p\n", local_copy->fields, local_copy->flags, pinstance);

  return local_copy->fields;
}

w_instance 
ref2local_copy(w_instance ref) {
  return pinstance2local_copy(ref2pinstance(ref));
}

w_instance
deswizzle(w_instance ref) {
  w_instance result = ref;
  w_word     flags;

  if (ref) {
    flags = instance2flags(ref);
    woempa(7,"Flags: 0x%08x\n",flags);
    if (isPersistentRef(ref)) {
      woempa(9, "Persistent ref %p ???\n", ref);
    }
    else if (isSet(flags, O_PERSISTENT)) {
      result = pinstance2ref(ref);
    }
    else {
      if (isSet(flags, O_LOCAL_COPY)) {
        result = pinstance2ref(local_copy2pinstance(ref));
        woempa(7,"  --> existing persistent instance @ %p --> ref %p\n",local_copy2pinstance(ref), pinstance2ref(local_copy2pinstance(ref)));
      }
      else {
    // Hm, it's just a normal instance.  Create a persistent instance
    // and make this a local copy.
        woempa(7,"  --> no persistent instance for 0x%08x\n", ref);
        result = createPersistentReference(ref);
      }
    }
  }

  woempa(7, "(PS) De-swizzled %p to %p\n", ref, result);
  return result;
}

w_instance
swizzle(w_instance ref) {
  w_instance result = ref;

  if (ref) {
    result = pinstance2local_copy(ref2pinstance(ref));
  }
  woempa(7, "(PS) Swizzled %p to %p\n", ref, result);

  return result;
}

/*
** pinstance2clazz gets a pointer to the w_Clazz structure associated with
** the given persistent instance.
*/

w_clazz
pinstance2clazz(w_instance pinstance) {
  w_instance pClass;
  w_instance tClass;
  w_clazz    clazz;
  w_instance loader;

  woempa(7,"Looking up clazz of pinstance %p\n",pinstance);
  clazz = getCurrentClazz(currentWonkaThread);
  woempa(7,"Calling class is %p\n",clazz);
  loader = clazz?clazz->loader:NULL;
  if (loader) {
    woempa(7,"Loader is %w@%p\n",loader);
  }
  else {
    woempa(7,"Using bootstrap class loader\n");
  }

  pClass = instance2pobject(pinstance)->class;
  woempa(7,"Persistent class reference is %p\n", pClass);
  tClass = swizzle(pClass);
  woempa(7,"Instance %p has (persistent) class %p, tClass = %p\n",pinstance,pClass,tClass);
  if (tClass) {
    woempa(7," --> local copy of Class is %p (%k)\n",tClass, Class2clazz(tClass));

    return Class2clazz(tClass);

  }

  clazz = readClazz (pClass,loader);

  return clazz;
}

/*
** alloc_P_Object just allocates space for a persistent object.  Well
** actually it does a little more: its zero's out the memory and sets
** the O_PERSISTENT and O_BLACK flags in the pobject header.  (We set
** O_BLACK because objects are sometimes allocated by the sweep phase
** of persistent GC; if they were allocated WHITE then they might be
** immediately collected.  Guess we could use GRAY for this ...)
*/

w_pobject
alloc_P_Object (w_size objsize) {
  w_pobject pobject;
  w_size bytes = sizeof(w_Pobject) + (objsize - 1)*sizeof(w_word);

  woempa(7,"Total space to reserve in PSmem = %d bytes\n",bytes);
  pobject = (w_pobject)allocPSmem(W_Thread_system,1,bytes);
  memset(pobject,0,bytes);
  pobject->flags = O_PERSISTENT | O_BLACK;
  woempa(7,"Object allocated at %p\n",pobject);
  updatePSblock(pobject);

  return pobject;
}


/*
** alloc_P_Array allocates space for a one-dimensional persistent array
** arrayInstance is used as a template to get the array class and length.
*/

w_instance
alloc_P_Array (w_instance arrayInstance) {
  w_clazz clazz = instance2clazz(arrayInstance);
  w_size  words = bits2words(arrayInstance[F_Array_length] * clazz->aclazz->bits);
  w_pobject pobject = alloc_P_Object(words+1);

  woempa(7,"Created object of size %d at %p\n",words+1,pobject);
  pobject->class = findPersistentClass(clazz);

  return pobject->fields;
}


/*
** alloc_P_String allocates space for a persistent string and sets the 
** pobject->class accordingly.  
*/

w_instance
alloc_P_String  (w_string string) {
  w_size words = bytes2words(string->length * sizeof(w_char));
  w_pobject pobject = alloc_P_Object(words + 1);

  woempa(7,"Created object of size %d at %p\n",words+1,pobject);
  pobject->class = findPersistentClass(clazzString);
  woempa(7,"Persistent string %p has class %p\n",pobject,pobject->class);

  return pobject->fields;
}


w_instance
alloc_P_PersistentRoot (w_instance thePersistentRoot) {
  w_size words = clazzPersistentRoot->instanceSize;
  w_pobject pobject = alloc_P_Object(words);

  woempa(7,"Created root object of size %d at %p\n",words,pobject);
  pobject->class = findPersistentClass(clazzPersistentRoot);
  woempa(7,"Persistent root %p has class %p\n",pobject,pobject->class);

  return pobject->fields;
}

w_instance
alloc_P_Clazz  (w_clazz clazz) {
  w_pobject pClass = alloc_P_Object(clazzClass->instanceSize + 1 + clazz->numStaticWords);

  woempa(7,"Created object of size %d at %p\n",clazzClass->instanceSize,pClass);
  if (clazz == clazzClass) {
    pClass->class = pinstance2ref(pClass->fields);
    woempa(7,"  --> created new persistent class '%k' @ %p\n", clazz,pClass->fields);
  }
  else {
    pClass->class = findPersistentClass(clazzClass);
  }
  woempa(7,"Persistent class %p (%k) has class %p, local copy %p\n",pClass, clazz,pClass->class,pClass->local_copy);

  return pClass->fields;
}

w_instance
alloc_P_Instance (w_instance tinstance) {
  w_pobject pobject;
  w_object  tobject = instance2object(tinstance);
  w_clazz clazz = tobject->clazz;

  woempa(7,"allocating for %k@%p\n", clazz,tinstance);

  ht_lock(l2p_hashtable);
  if (clazz==clazzClass) {
    pobject = instance2pobject(alloc_P_Clazz(Class2clazz(tinstance)));
  }
  else if (clazz==clazzString) {
    pobject = instance2pobject(alloc_P_String(String2string(tinstance)));
  }
  else if (clazz==clazzPersistentRoot) {
    pobject = instance2pobject(alloc_P_PersistentRoot(tinstance));
  }
  else if (isSet(clazz->flags, CLAZZ_HAS_WOTSITS)) {

    wabort(ABORT_WONKA,"Sorry: class '%k' has wotsits and is not Class or String\n", clazz);

    pobject = NULL;
  }
  else if (clazz->aclazz) {
    pobject = instance2pobject(alloc_P_Array(tinstance));
  }
  else {
    pobject = alloc_P_Object(clazz->instanceSize);
    pobject->class = findPersistentClass(clazz);
  }

  pobject->local_copy = tobject;
  pobject->hashcode = (w_int)tinstance;
  woempa(7,"      persistent hashcode is 0x%08x\n", tinstance);
  ht_write_no_lock(l2p_hashtable, (w_word)tinstance, (w_word)pobject->fields);
  woempa(7,"Local copy of %k@%p is %p\n", clazz,pobject->fields,pobject->local_copy->fields);
  ht_unlock(l2p_hashtable);

  return pobject->fields;
}

/*
** release_P_Object releases the space used for a persistent object.
*/

void
release_P_Object (w_pobject pobject) {
  woempa(7,"Object released at %p\n",pobject);
  releasePSmem(pobject);
  updatePSblock(pobject);
}

w_instance
createPersistentInstance(w_instance tinstance) {
  w_clazz    clazz = instance2clazz(tinstance);
  w_instance new_pinstance;
  w_int      fieldnum;
  w_int      child_slot;
  w_instance child;

  woempa(7,"Creating persistent instance using %k@%p\n", clazz, tinstance);

  if (clazz == clazzString) {
    new_pinstance = findPersistentString(String2string(tinstance));
  }
  else {
    ht_lock(l2p_hashtable);
    new_pinstance = alloc_P_Instance(tinstance);
    woempa(7,"  --> created new persistent instance of %k@0x%08x\n", clazz,new_pinstance);
    ht_write_no_lock(l2p_hashtable, (w_word)tinstance, (w_word)new_pinstance);
    setFlag(instance2flags(tinstance),O_LOCAL_COPY);
    instance2pobject(new_pinstance)->hashcode = (w_int)tinstance;
    woempa(7,"      persistent hashcode is 0x%08x\n", tinstance);
    woempa(7,"Flags: 0x%08x\n",instance2flags(tinstance));
    besmirchLocalCopy(tinstance);
    if (clazz->aclazz) {
      for (child_slot = F_Array_data; child_slot < F_Array_data + (w_int)tinstance[F_Array_length]; ++child_slot) {
        child = (w_instance)tinstance[child_slot];
        if (child) {
          woempa(7, "Slot [%d] child instance: %k@%p is%s a local copy\n", child_slot, instance2clazz(child), child, isSet(instance2flags(child), O_LOCAL_COPY) ? "" : " not");
          if (isNotSet(instance2flags(child), O_LOCAL_COPY)) {
            createPersistentInstance((w_instance)tinstance[child_slot]);
          }
        }
      }
    }
    else {
      while (clazz) {
        for (i = clazz->numStaticFields; i < clazz->numFields; ++i) {
          if (isSet(clazz->fields[i].flags, FIELD_IS_REFERENCE)) {
            child_slot = clazz->fields[i].slot;
            child = (w_instance)tinstance[child_slot];
            if (child) {
              woempa(7, "Slot [%d] child instance: %k@%p is%s a local copy\n", child_slot, instance2clazz(child), child, isSet(instance2flags(child), O_LOCAL_COPY) ? "" : " not");
            }
            if (isNotSet(instance2flags(child), O_LOCAL_COPY)) {
              createPersistentInstance((w_instance)tinstance[child_slot]);
            }
          }
          else {
            break;
          }
        }
        clazz = clazz->superClazz;
      }
    }
    ht_unlock(l2p_hashtable);
  }

  return new_pinstance;
}

w_instance
createPersistentReference(w_instance instance) {

  return pinstance2ref(createPersistentInstance(instance));

}

void
writeArray  (w_instance tarray,  w_instance parray) {
  w_size     length = tarray[F_Array_length];
  w_size     words;
  w_word    *srcptr = tarray+F_Array_data;
  w_word    *dstptr = parray+1;
  w_clazz    array_clazz = instance2clazz(tarray);
  w_size     i;

  parray[0] = length;
  words = bits2words(length*array_clazz->aclazz->bits);

  woempa(7, "Copying %d array elements (%d words) from %p\n", length, words, srcptr);
  /*
  ** If the array clazz stores non primitives (other arrays or references), 
  ** then if any of these references are to local copies then they have to
  ** be de-swizzled.
  */
  if (array_clazz->aclazz->depth > 1 || isNotSet(array_clazz->aclazz->componentClazz->flags, CLAZZ_IS_PRIMITIVE)){
    woempa(7, "De-swizzling %d array elements from %p\n", words, srcptr);
    for (i = 0; i < words; ++i) {
      *dstptr++ = (w_word)deswizzle((w_instance)*srcptr++);
    }
  }
  else {
    for (i = 0; i < words; ++i) {
      *dstptr++ = *srcptr++;
    }
  }

  return;
}


void 
writeString (w_string string, w_instance pinstance) {
  w_size  length = string->length;
  w_char  *srcptr = (w_char*)string->chars;
  w_char  *dstptr = (w_char*)pinstance;
  w_size  i;

  i = 0;

  woempa(7,"Writing string '%w'\n",string);
  woempa(7, "Copying %d characters (+ length) from %p\n", length, srcptr);
  *dstptr++ = length;
  while (i<length) {
    *dstptr++ = *srcptr++;
    ++i;
  }

  woempa(7,"Copied %d shorts in total\n",dstptr-(w_char*)pinstance);

  return;
}

w_word*
writeBackLocalCopy (w_instance tinstance, w_instance pinstance) {
  w_clazz  clazz;
  w_size   size;
  w_size   i;

  clazz = instance2clazz(tinstance);
  size = clazz->instanceSize;
  woempa(7,"Writing an instance of %w, has %d instance words\n",clazz->name,size);

  while (clazz) {
    for (i = clazz->numStaticFields; i < clazz->numFields; ++i) {
      if (isSet(clazz->fields[i].flags, FIELD_IS_REFERENCE)) {
        pinstance[clazz->fields[i].slot] = (w_word)deswizzle((w_instance)tinstance[clazz->fields[i].slot]);
      }
      else {
        pinstance[clazz->fields[i].slot] = tinstance[clazz->fields[i].slot];
      }
    }
    clazz = clazz->superClazz;
  }

  return pinstance + size;
}


void
writeClazz (w_clazz clazz, w_instance pinstance) {
  w_size   size;
  w_word  *srcptr;
  w_word  *dstptr;
  w_size   nextreferencefield;
  w_size   i;

  woempa(7,"Writing class %w into %p\n",clazz->name,pinstance);
  dstptr = writeBackLocalCopy(clazz2Class(clazz),pinstance);
  woempa(7,"Substituting string '%w' for F_Class_wotsit at %p\n",clazz->name,dstptr-1);
  *(dstptr-1) = (w_word)findPersistentString(clazz->name);

  woempa(7,"Copied %d words\n",dstptr-pinstance);

  size = clazz->numStaticFields;
  woempa(7,"Writing statics of %w, has %d static fields\n",clazz->name,size);

  for (i = 0; i < clazz->numStaticFields; ++i) {
    if (isSet(clazz->fields[i].flags, FIELD_IS_REFERENCE)) {
      dstptr[clazz->fields[i].slot] = (w_word)deswizzle((w_instance)clazz->staticFields[clazz->fields[i].slot]);
    }
    else {
      break;
    }
  }

  return;
}

void
writePersistentRoot (w_instance tinstance, w_instance pinstance) {
  w_word  *dstptr;

  woempa(7,"Writing PersistentRoot\n");
  dstptr = writeBackLocalCopy(tinstance,pinstance);

  return;
}

void
writeInstance (w_instance tinstance, w_instance pinstance) {
  w_word  *dstptr;
  w_clazz  clazz;

  if (!tinstance) {
    wabort(ABORT_WONKA,"Trying to make the null instance permanent!\n");

    return;

  }

  clazz = instance2clazz(tinstance);

  ht_lock(l2p_hashtable);
  if (clazz->aclazz) {
    woempa(7,"%p is an array %k\n",tinstance, clazz);

    writeArray(tinstance,pinstance);

  }
  else if (clazz==clazzClass) {
    woempa(7,"%p is the instance of java/lang/Class for class %k\n",tinstance, (w_clazz)tinstance[F_Class_wotsit]);

    writeClazz(Class2clazz(tinstance), pinstance);

  }
  else if (clazz==clazzString) {
    woempa(7,"%p is a string '%w'\n",tinstance,String2string(tinstance));

    writeString(String2string(tinstance), pinstance);

  }
  else if (clazz==clazzPersistentRoot) {
    woempa(7,"%p is a PersistentRoot\n",tinstance);

    writePersistentRoot(tinstance,pinstance);

  }
  else if (isSet(clazz->flags, CLAZZ_HAS_WOTSITS)) {
    woempa(7,"%p has wotsits, no can do\n",tinstance);
  }
  else {
    woempa(7,"%p is an instance of %k\n",tinstance, clazz);
    dstptr = writeBackLocalCopy(tinstance,pinstance);
  }

  woempa(7,"tinstance %p, pinstance %p\n",tinstance,pinstance);
//  ht_write_no_lock(l2p_hashtable, (w_word)tinstance, (w_word)pinstance);
  setFlag(instance2flags(pinstance), O_DIRTY);
  unsetFlag(instance2flags(tinstance), O_DIRTY);
  ht_unlock(l2p_hashtable);

  return;
}


void
readArray  (w_instance tarray,  w_instance parray) {
  w_size     length = parray[0];
  w_size     words;
  w_word    *srcptr = parray+1;
  w_word    *dstptr = tarray+F_Array_data;
  w_clazz    array_clazz = instance2clazz(tarray);
  w_size     i;

  tarray[F_Array_length] = length;
  words = bits2words(length*array_clazz->aclazz->bits);

  woempa(7, "Copying %d array elements (%d words) from %p\n", length, words, srcptr);
  /*
  ** If the array clazz stores non primitives (other arrays or references), 
  ** then all of these references have to be de-swizzled.
  */
  if (array_clazz->aclazz->depth > 1 || isNotSet(array_clazz->aclazz->componentClazz->flags, CLAZZ_IS_PRIMITIVE)){
    woempa(7, "swizzling %d array elements from %p\n", words, srcptr);
    for (i = 0; i < words; ++i) {
      *dstptr++ = (w_word)swizzle((w_instance)*srcptr++);
    }
  }
  else {
    for (i = 0; i < words; ++i) {
      *dstptr++ = *srcptr++;
    }
  }


  unsetFlag(instance2flags(tarray), O_DIRTY);

  return;
}


w_string 
readString (w_instance pinstance) {
  w_char *chars = (w_char*)pinstance;
  w_size  length = *chars++;
  w_string string;
  w_object strobject;
  w_instance String;

  ht_lock(l2p_hashtable);
  strobject = instance2pobject(pinstance)->local_copy;
  if (strobject) {
    woempa(7,"Local copy of pobject %p is object %p, string is `%w'\n", instance2pobject(pinstance), strobject, String2string(strobject->fields)); 
    String = strobject->fields;
  }
  else {
    woempa(7,"Didn't find a local_copy pointer in %p: creating a string from %d chars at %p\n", pinstance, length, chars);
    string = unicode2String(W_Thread_system, chars, length);
    woempa(7,"Result is `%w'\n",string);
    String = getStringInstance(W_Thread_system,string);
    if (isSet(instance2object(String)->flags,O_LOCAL_COPY)) {
      woempa(9,"HUH? persistent String %p (string `%w') doesn't have a local_copy pointer, yet calling getStringInstance on string %p (`%w') yields transient String %p with O_LOCAL_COPY set (pinstance is %p)\n",pinstance,string,string,string,String,local_copy2pinstance(String));
    }
    else {
      setFlag(instance2flags(String),O_LOCAL_COPY);
      unsetFlag(instance2flags(String), O_DIRTY);
    }
    instance2pobject(pinstance)->local_copy = strobject;
    woempa(7,"Local copy of String %p is %p\n", pinstance, String);
    instance2pobject(pinstance)->hashcode = (w_word)String;
    woempa(7,"      persistent hashcode is 0x%08x\n", String);
    ht_write_no_lock(l2p_hashtable, (w_word)String, (w_word)pinstance);
  }
  ht_unlock(l2p_hashtable);

  return String2string(String);
}

w_clazz
readClazz (w_instance pinstance, w_instance loader) {
  w_instance exception = NULL;
  w_instance pName;
  w_string   classname;
  w_clazz    clazz;
  w_instance tClass;

  ht_lock(l2p_hashtable);
  pName = ref2pinstance((w_instance)pinstance[F_Class_wotsit]);
  woempa(7,"Name of %p is persistent String %p\n",pinstance,pName);
/*
  classnameString = ref2local_copy(pName);
  woempa(7,"Local copy of %p is %p\n", pName, classnameString);
  if (classnameString) {
    classname = (w_string)classnameString[F_String_wotsit];
    classname = registerString(classname);
    woempa(7,"  --> name is %w\n",classname);
  }
  else {
    classname = readString(pName);
  }
  woempa(7,"  --> class %w\n",classname);
*/
  classname = readString(pName);

  clazz = namedClassMustBeLoaded(loader, classname, &exception);
  deregisterString(classname);

  if (!clazz || exception) {
    wabort(ABORT_WONKA,"Erk!  Could not find class %w using loader %k@%p: got %k\n",classname, loader?instance2clazz(loader):clazzClassLoader,loader, instance2clazz(exception));
  }

  tClass = clazz2Class(clazz);
  woempa(7,"clazz %k, (transient) Class: %p\n", clazz,tClass);
  setFlag(instance2flags(tClass),O_LOCAL_COPY);
  unsetFlag(instance2flags(tClass), O_DIRTY);
  instance2pobject(pinstance)->local_copy = instance2object(tClass);
  woempa(7,"Local copy of %k@%p is %p\n", clazzClass,pinstance,tClass);
  ht_write_no_lock(l2p_hashtable, (w_word)tClass, (w_word)pinstance);
  woempa(7,"Substituting %p for class name string '%w'\n", clazz, clazz->name);
  tClass[F_Class_wotsit] = (w_word)clazz;
  ht_unlock(l2p_hashtable);

  return clazz;
}

void
readStatics (w_clazz clazz) {
  w_size     size;
  w_size     i;
  w_word    *srcptr;
  w_word    *dstptr;
  w_instance pinstance = local_copy2pinstance(clazz2Class(clazz));

  classMustBeResolved(clazz,currentWonkaThread);

  size = clazz->numStaticWords;
  woempa(7,"Reading statics of %w, has %d static fields\n",clazz->name,size);
  srcptr = pinstance;
  dstptr = clazz->staticFields;

  for (i = 0; i < size; ++i) {
    if (isSet(clazz->fields[i].flags, FIELD_IS_REFERENCE)) {
      clazz->staticFields[clazz->fields[i].slot] = (w_word)swizzle((w_instance)srcptr[clazz->fields[i].slot]);
    }
    else {
      break;
    }
  }

  return;
}

void
readPersistentRoot (w_instance pinstance, w_instance tinstance) {
  w_int    nextreferencefield;
  w_size   size;
  w_size   i;
  w_word  *srcptr;
  w_word  *dstptr;

  woempa(7,"Reading PersistentRoot\n");
  size = clazzPersistentRoot->instanceSize;

  ht_lock(l2p_hashtable);
  while (clazz) {
    for (i = clazz->numStaticFields; i < clazz->numFields; ++i) {
      if (isSet(clazz->fields[i].flags, FIELD_IS_REFERENCE)) {
        tinstance[clazz->fields[i].slot] = (w_word)swizzle((w_instance)pinstance[clazz->fields[i].slot]);
      }
      else {
        tinstance[clazz->fields[i].slot] = pinstance[clazz->fields[i].slot];
      }
    }
     clazz = clazz->superClazz;
  }

  woempa(7,"Copied %d words in total\n",dstptr-pinstance);
  ht_write_no_lock(l2p_hashtable, (w_word)tinstance, (w_word)pinstance);
  setFlag(instance2flags(tinstance), O_LOCAL_COPY);
  unsetFlag(instance2flags(tinstance), O_DIRTY);
  ht_unlock(l2p_hashtable);
}

void
readInstance (w_instance pinstance, w_instance tinstance) {
  w_int    nextreferencefield;
  w_size   size;
  w_size   i;
  w_word  *srcptr;
  w_word  *dstptr;
  w_clazz  clazz;

  woempa(7,"pinstance=%p tinstance=%p\n",pinstance,tinstance);
  clazz = instance2clazz(tinstance);
  classMustBeInitialized(clazz,currentWonkaThread);

  if (clazz->aclazz) {
    woempa(7,"%p is an array %k\n",tinstance, clazz);
    readArray(pinstance,tinstance);
  }
  else if (clazz==clazzClass) {
    woempa(9,"%p is a class, no can do\n",tinstance);
  }
  else if (clazz==clazzString) {
    tinstance[F_String_wotsit] = (w_word)readString(pinstance);
  }
  else if (clazz==clazzPersistentRoot) {
    readPersistentRoot(pinstance,tinstance);
  }
  else if (isSet(clazz->flags, CLAZZ_HAS_WOTSITS)) {
    woempa(9,"%p has wotsits, no can do\n",tinstance);
  }
  else {
    woempa(9,"%p is an ordinary instance of %k\n",tinstance, clazz);
  /*
  ** OK, so this is an instance of a non-array class, not String or Class,
  ** and it doesn't have wotsits.  So we copy all the fields across,  
  ** swizzling each reference field.
  */

    size = clazz->instanceSize;
    woempa(7,"instanceSize of %k is %d\n", clazz,size);

    while (clazz) {
      for (i = clazz->numStaticFields; i < clazz->numFields; ++i) {
        if (isSet(clazz->fields[i].flags, FIELD_IS_REFERENCE)) {
          pinstance[clazz->fields[i].slot] = (w_word)swizzle((w_instance)tinstance[clazz->fields[i].slot]);
        }
        else {
          pinstance[clazz->fields[i].slot] = tinstance[clazz->fields[i].slot];
        }
      }
      clazz = clazz->superClazz;
    }
  }

  unsetFlag(instance2flags(tinstance), O_DIRTY);
}

static w_pobject pobject_S;

w_int
suckRoot(w_instance thePersistentRoot) {
  w_pobject  pobject;
  w_pobject  pobj_Class = NULL;
  w_instance pinstance;
  w_instance pClass;
  w_instance pString;
  w_clazz    clazz;

  pobject_S = (w_pobject)firstPSblock();
  if (pobject_S == NULL) {
    woempa(9,"No root object found\n");

    return -1;

  }

  woempa(7,"<<<<<<<<<<<<< Starting the big suck >>>>>>>>>>>>>>\n");
  woempa(7,"  Big suck phase 1: clean up\n");
  pobject = (w_pobject)firstPSblock();
  while (pobject) {
    pobject->local_copy = NULL;
    pobject->flags = O_PERSISTENT;

    woempa(7,"After %p comes %p\n",pobject,nextPSblock(pobject));
    pobject = (w_pobject)nextPSblock(pobject);
  }

  woempa(7,"  Big suck phase 2: identify persistent java.lang.Class\n");
  pobject = (w_pobject)firstPSblock();
  while (pobject) {
    if (pobject->class == instance2pobject(ref2pinstance(pobject->class))->class) {
      woempa(7,"  Found Class java.lang.Class at %p\n", pobject);
      pobj_Class = pobject;
      clazz = readClazz (pobject->fields, NULL);
      woempa(7,"  %k\n", clazz);

      break;

    }
    woempa(7,"After %p comes %p\n",pobject,nextPSblock(pobject));
    pobject = (w_pobject)nextPSblock(pobject);
  }

  woempa(7,"  Big suck phase 3\n");
  pobject = (w_pobject)firstPSblock();
  while (pobject) {
    pString = ref2pinstance((w_instance)((w_instance)ref2pinstance(pobject->class))[F_Class_wotsit]);
    woempa(7, "  %p  %w\n", pobject, readString(pString));
    if (pobject->class == instance2pobject(ref2pinstance(pobject->class))->class) {
      pString = ref2pinstance((w_instance)pobject->fields[F_Class_wotsit]);
      woempa(7, "    Class %w \n", readString(pString));
    }
    pobject = (w_pobject)nextPSblock(pobject);
  }

  pobject = (w_pobject)firstPSblock();
  while (pobject) {
    pinstance = pobject->fields;
    pClass = pobject->class;
    woempa(7,"pobject %p (pinstance %p) has class %p\n",pobject,pinstance,pClass);

    clazz = readClazz (ref2pinstance(pClass), NULL);
    if (clazz->aclazz) {
    }
    else if (clazz == clazzString && instance2pobject(pinstance)->local_copy == NULL) {
      w_string   string;
      string = readString(pinstance);
      woempa(7,"Persistent %p is the string '%w'\n",pinstance,string);
    }
    else if (clazz == clazzClass && instance2pobject(pinstance)->local_copy == NULL) {
      w_clazz   c;
      c = readClazz(pinstance,NULL);
      woempa(7,"Persistent %p is the class %k\n",pinstance, c);
    }
    else {
      woempa(7,"Persistent %p is an instance of %k\n",pinstance, clazz);
    }

    woempa(7,"After %p comes %p\n",pobject,nextPSblock(pobject));
    pobject = (w_pobject)nextPSblock(pobject);
  }

  pobject_S->local_copy = instance2object(thePersistentRoot);
  woempa(7,"Local copy of %k@%p is %p\n", clazzPersistentRoot, pobject_S->fields, pobject_S->local_copy->fields);

  woempa(7,"  Found root object at %p\n", pobject_S);
  readPersistentRoot(pobject_S->fields, thePersistentRoot);

  woempa(7,"<<<<<<<<<<<<< Here ends the big suck >>>>>>>>>>>>>\n");

  return 0;
}

void
blowRoot(w_instance thePersistentRoot) {
  woempa(7,"<<<<<<<<<<<<< Blowing a new root     >>>>>>>>>>>>>\n");
  pobject_S = instance2pobject(alloc_P_Instance(thePersistentRoot));
  writeInstance(thePersistentRoot, pobject_S->fields);
}

#define PS_MAN_STACK_SIZE 8192
#define PS_MAN_PRIORITY   (priority_j2k(11,0))
#define PS_MAN_TIME_SLICE 30

void
createRoot(w_instance thePersistentRoot) {
  startPSmem();

  PSmemory_monitor = allocMonitor(thePersistentRoot);
  l2p_hashtable = ht_create(W_Realm_system, "hashtable:local_copy->pinstance", 101, NULL, NULL, 0, 0);
  proots_hashtable = ht_create(W_Realm_system, "hashtable:proots", 101, NULL, NULL, 0, 0);

  dirty_fifo = allocFifo(W_Realm_system, 511);

  if (suckRoot(thePersistentRoot) != 0) {
    blowRoot(thePersistentRoot);
  }

  setFlag(instance2flags(thePersistentRoot),O_LOCAL_COPY);
  ht_write(l2p_hashtable, (w_word)thePersistentRoot, (w_word)pobject_S->fields);
  writeBackLocalCopy(thePersistentRoot, pobject_S->fields);

  woempa(7,"Persistent root S is at %p, is %s %s\n", pobject_S->fields, isSet(pobject_S->flags, O_DIRTY) ? "dirty" : "", isSet(pobject_S->flags, O_PERSISTENT) ? "persistent" : "");
  woempa(7,"    Local copy is at %p, is %s %s\n", thePersistentRoot, isSet(instance2object(thePersistentRoot)->flags, O_DIRTY) ? "dirty" : "", isSet(instance2object(thePersistentRoot)->flags, O_LOCAL_COPY) ? "local copy" : "");
}

w_size
traceArray  (w_instance parray) {
  w_size     length;
  w_word    *elementptr;
  w_clazz    array_clazz;
  w_size     i;
  w_size     marked = 0;

  array_clazz = pinstance2clazz(parray);
  woempa(7,"Tracing %k at %p\n", array_clazz,parray);
  if (array_clazz->aclazz->depth <= 1 && isSet(array_clazz->aclazz->componentClazz->flags, CLAZZ_IS_PRIMITIVE)){

    return 0;

  }

  length = parray[0];
  elementptr = parray + 1;
  for (i = 0; i < length; ++i) {
    w_instance pinstance = ref2pinstance((w_instance)*elementptr++);
    woempa(7, "  element[%d] is pinstance %p\n", i, pinstance);
    if (pinstance && isNotSet(instance2flags(pinstance), O_BLACK)) {
      woempa(7,"(PSGC) Marking %k@%p BLACK and pushing it onto the proots_fifo\n", pinstance2clazz(pinstance),pinstance);
      setFlag(instance2flags(pinstance), O_BLACK);
      putFifo(pinstance,proots_fifo);
      marked++;
    }
  }
  woempa(7,"Traced %d words, marked %d items\n",elementptr-parray,marked);

  return marked;
}


w_size
traceClass (w_instance pinstance) {
  w_instance tinstance;
  w_clazz  clazz;
  w_size   size;
  w_word  *fieldptr;
  w_size   nextreferencefield;
  w_size   i;
  w_size   marked = 0;

  tinstance = pinstance2local_copy(pinstance);
  clazz = Class2clazz(tinstance);

  fieldptr = clazz->staticFields;
  size = clazz->numStaticFields;
  woempa(7,"  --> trace the %d static variables of %k\n",size, clazz);

  for (i = 0; i < size; ++i) {
    if (isSet(clazz->fields[i].flags, FIELD_IS_REFERENCE)) {
      w_instance child_pinstance = ref2pinstance((w_instance)clazz->staticFields[clazz->fields[i].slot]);
      woempa(7, "  element[%d] is pinstance %p\n", i, child_pinstance);
      if (child_pinstance && isNotSet(instance2flags(child_pinstance), O_BLACK)) {
        woempa(7,"(PSGC) Marking %k@%p BLACK and pushing it onto the proots_fifo\n", pinstance2clazz(child_pinstance),child_pinstance);
        setFlag(instance2flags(child_pinstance), O_BLACK);
        putFifo(child_pinstance,proots_fifo);
        marked++;
      }
    }
  }

  return marked;
}

w_size
traceRoot (w_instance pinstance) {
  w_clazz  clazz;
  w_word  *flagsptr;
  w_word  *fieldptr;
  w_size   nextreferencefield;
  w_size   size;
  w_size   i;
  w_size   marked = 0;
  w_instance pClass;

  if (pinstance == NULL) {
    woempa(7,"NULL is an external root?  You crazy guy.\n");

    return 0;

  }

  flagsptr = instance2flagsptr((w_instance)pinstance);

#ifdef RUNTIME_CHECKS
  if (isNotSet(*flagsptr, O_PERSISTENT)) {
    woempa(7,"%p is an external root?  You crazy guy.\n",pinstance);

    return 0;

  }
#endif

  if (isSet(*flagsptr, O_BLACK)) {
    woempa(7,"No need to trace %p, it's already BLACK\n",pinstance);

    return 0;

  }

  clazz = pinstance2clazz(pinstance);
  woempa(7,"Tracing %k@%p\n", clazz,pinstance);

  marked = traceClass(ref2pinstance(instance2pobject(pinstance)->class));

  woempa(7,"Tracing %k@%p\n", clazz,pinstance);
  setFlag(*flagsptr,O_BLACK);

  if (clazz->aclazz) {
    woempa(7,"  --> is an array, trace its elements\n");

    marked += traceArray(pinstance);

  }
/*
  else if (clazz==clazzClass) {

    marked += traceClass(pinstance);

  }
*/
  else if (clazz==clazzString) {
    woempa(7,"  --> nothing to trace\n");
  }
  else {
    fieldptr = pinstance;
    size = clazz->numInstanceFields;
    woempa(7,"  --> class %w has %d instance fields\n",clazz->name,size);

    while (clazz) {
      for (i = clazz->numStaticFields; i < clazz->numFields; ++i) {
        if (isSet(clazz->fields[i].flags, FIELD_IS_REFERENCE)) {
          w_instance child_pinstance = ref2pinstance((w_instance)pinstance[clazz->fields[i].slot]);
          woempa(7, "  element[%d] is pinstance %p\n", i, child_pinstance);
          if (child_pinstance && isNotSet(instance2flags(child_pinstance), O_BLACK)) {
            woempa(7,"(PSGC) Marking %k@%p BLACK and pushing it onto the proots_fifo\n", pinstance2clazz(child_pinstance),child_pinstance);
            setFlag(instance2flags(child_pinstance), O_BLACK);
            putFifo(child_pinstance,proots_fifo);
            marked++;
          }
        }
        else {
          break;
        }
      }
      clazz = clazz->superClazz;
    }

    pClass = ref2pinstance(instance2pobject(pinstance)->class);
    woempa(7,"(PSGC) Appending persistent Class %p to the proots_hashtable\n",ref2pinstance(instance2pobject(pinstance)->class));
    putFifo(pClass, proots_fifo);
    woempa(7,"Traced %d words, marked %d items\n",fieldptr-pinstance,marked);
  }

  return marked;
}

void persistentGarbageCollector(void) {
  w_instance dirty_boy;
  w_pobject  pobject ;
  w_instance pinstance;
  w_size dirty_count;
  w_size marked;
 
  if (! pobject_S) {
    woempa(7,"Could not run persistent garbage collector, as no persistent infrastructure present\n");

     return;

  }

  woempa(7,"Begin persistent garbage collection\n");

  ht_lock(l2p_hashtable);
  proots_fifo = ht_list_keys(proots_hashtable);
  if (!proots_fifo) {
    woempa(9, "Desolated, could not create proots_fifo\n");

    return;

  }
  ht_destroy(proots_hashtable);
  proots_hashtable = ht_create(W_Realm_system, "hashtable:proots", 101, NULL, NULL, 0, 0);
  
  do {
    dirty_count = 0;
    while ((dirty_boy = getFifo(dirty_fifo))) {
      // Check that O_DIRTY is still set, we may have been enFifoed twice
      if (isSet(instance2flags(dirty_boy),O_DIRTY)) {
        pinstance = local_copy2pinstance(dirty_boy);
        woempa(7,"Local copy %p of %p is dirty, writing it back\n",dirty_boy,pinstance);
        writeInstance(dirty_boy,pinstance);
        ++dirty_count;
      }
    }
    woempa(7,"Handled %d dirty local copies\n",dirty_count);

    woempa(7,"(PSGC) Pushing %p onto the proots_fifo\n", pobject_S->fields);
    putFifo(pobject_S->fields, proots_fifo);
    pinstance = local_copy2pinstance(clazz2Class(clazzClass));
    if (pinstance) {
      woempa(7,"(PSGC) Pushing %p onto the proots_fifo\n", pinstance);
      putFifo(pinstance,proots_fifo);
    }
    else {
      wabort(ABORT_WONKA,"Class java.lang.Class is not persistent!\n");
    }

    marked = 0;
    while ((pinstance = (w_instance)getFifo(proots_fifo))) {
      marked += traceRoot(pinstance);
    }
    woempa(7,"Marked %d persistent items\n",marked);
  } while (dirty_count || marked);

  pobject = (w_pobject)firstPSblock();
  do {
    pinstance = pobject->fields;
    sweep_P_Object(pobject);
    pobject = (w_pobject)nextPSblock(pobject);
  } while (pobject);
  ht_unlock(l2p_hashtable);

  woempa(7,"End persistent garbage collection\n");
}

void
markPersistentInstance(w_instance pinstance) {
  if (isSet(instance2flags(pinstance), O_BLACK)) {

    return;

  }
  woempa(7,"(PSGC) Marking %k@%p BLACK and pushing it onto the proots_fifo\n", pinstance2clazz(pinstance),pinstance);
  setFlag(instance2flags(pinstance), O_BLACK);
  ht_write(proots_hashtable, (w_word)pinstance, (w_word)pinstance);
}


void
markPersistentRef(w_instance ref) {
  markPersistentInstance(ref2pinstance(ref));
}

void
sweep_P_Object(w_pobject pobject) {
  w_word *flagsptr = &pobject->flags;
#ifdef DEBUG
  w_instance pinstance = pobject->fields;
#endif

  if (isSet(*flagsptr,O_BLACK)) {
    woempa(7,"Sweeping %k@%p: was BLACK, clearing flag\n", pinstance2clazz(pinstance),pinstance);
    unsetFlag(*flagsptr,O_BLACK);
    if (isSet(*flagsptr,O_DIRTY)) {
      woempa(7,"Sweeping %k@%p: was DIRTY, updating backing store\n", pinstance2clazz(pinstance),pinstance);
      updatePSblock(pobject);
      unsetFlag(*flagsptr,O_DIRTY);
    }
  }
  else {
    woempa(7,"Sweeping %k@%p: was WHITE, releasing memory and updating backing store\n", pinstance2clazz(pinstance),pinstance);
// Except we don't, because there seems to be a bug in our garbage collector ...
//    release_P_Object(pobject);
  }
    
}

w_boolean 
markLocalCopy(w_instance local_instance) {
  w_instance pinstance = local_copy2pinstance(local_instance);
  woempa(7,"(PSGC) Pushing %p onto the proots_fifo\n", pinstance);
  ht_write(proots_hashtable, (w_word)pinstance, (w_word)pinstance);
  return WONKA_TRUE;
}

w_boolean 
releaseLocalCopy(w_instance local_instance) {
  ht_lock(l2p_hashtable);
  if (isNotSet(instance2flags(local_instance), O_DIRTY) && instance2object(local_instance)->clazz != clazzString && instance2object(local_instance)->clazz != clazzClass) {
    w_word erased = ht_erase_no_lock(l2p_hashtable, (w_word)local_instance);

    if (erased) {
      instance2pobject(erased)->local_copy = NULL;
      woempa(7, "Erased l2p_hashtable mapping %p->%p\n", local_instance, erased);
    }
    else { 
      wabort(ABORT_WONKA, "How come? %p should be a local copy, but when I want to erase it from l2p_hashtable it's not there!\n", local_instance);
    }

    unsetFlag(instance2flags(local_instance), O_LOCAL_COPY);
    ht_unlock(l2p_hashtable);

    return WONKA_TRUE;

  };
  ht_unlock(l2p_hashtable);

  return WONKA_FALSE;
}

void
besmirchLocalCopy(w_instance local_instance) {
  w_word *flagptr = instance2flagsptr(local_instance);

  woempa(7,"Local copy is %p, flags @ %p are 0x%08x\n",local_instance,flagptr,*flagptr);
  if ((*flagptr & (O_LOCAL_COPY | O_DIRTY)) == O_LOCAL_COPY) {
    setFlag(*flagptr,O_DIRTY);
    woempa(7,"Pushing %p onto the dirty_fifo\n",local_instance);
    putFifo(local_instance,dirty_fifo);
  }
}

#endif
