/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

#include "checks.h"
#include "clazz.h"
#include "constant.h"
#include "descriptor.h"
#include "exception.h"
#include "fields.h"
#include "interpreter.h"
#include "loading.h"
#include "methods.h"
#include "stack.h"
#include "wonka.h"

w_clazz initializeMethod(w_method method) {
  return NULL;
}

void initializeStaticFields(w_clazz clazz) {
  w_size  i;
  w_field field;
  w_cnt   c;

  woempa(1, "Setting the static fields of '%k' to default values.\n", clazz);

  for (i = 0; i < clazz->numStaticFields; i++) {
    field = &clazz->fields[i];
    if (field->initval) {
      c = &clazz->constants[field->initval];
      switch (c->cnt_tag) {
        case CONSTANT_STRING:
          {
            w_string string = getStringConstant(clazz, field->initval);
            w_instance instance = getStringInstance(currentWonkaThread, string);

            clazz->staticFields[field->slot] = (w_word)instance;
            woempa(1,"UnGREENING String at %p (static field initialisation)\n", instance);
            unsetFlag(instance2flags(instance),O_GREEN);
          }
          woempa(1, "Initialized static field '%w' (type %k) of '%w' to 0x%08x.\n", NM(field), NM(field->desc), clazz,clazz->staticFields[field->slot]);
          break;
      
        case CONSTANT_INTEGER:
          clazz->staticFields[field->slot] = getIntegerConstant(clazz, field->initval);
          woempa(1, "Initialized static field '%w' (type %k) of '%w' to 0x%08x.\n", NM(field), NM(field->desc), clazz,clazz->staticFields[field->slot]);
          break;

        case CONSTANT_FLOAT:
          clazz->staticFields[field->slot] = getFloatConstant(clazz, field->initval);
          woempa(1, "Initialized static field '%w' (type %k) of '%w' to 0x%08x.\n", NM(field), NM(field->desc), clazz,clazz->staticFields[field->slot]);
          break;

        case CONSTANT_LONG:
          {
            w_u64 u64 = getLongConstant(clazz, field->initval);

            clazz->staticFields[field->slot] = u64.words[0];
            clazz->staticFields [ field->slot + 1] = u64.words[1];
            woempa(1, "Initialized static field '%w' (type %k) of '%w' to 0x%08x%08x.\n", NM(field), NM(field->desc), clazz,clazz->staticFields[field->slot+1],clazz->staticFields[field->slot]);
          }
          break;

        case CONSTANT_DOUBLE:
          {
            w_u64 u64 = getDoubleConstant(clazz, field->initval);

            clazz->staticFields[field->slot] = u64.words[0];
            clazz->staticFields [ field->slot + 1] = u64.words[1];
            woempa(1, "Initialized static field '%w' (type %k) of '%w' to 0x%08x%08x.\n", NM(field), NM(field->desc), clazz,clazz->staticFields[field->slot+1],clazz->staticFields[field->slot]);
          }
          break;

        default:
          wabort(ABORT_WONKA, "Static field '%w' of %k has initializer with cnt_tag %d.\n", field->name, clazz, c->cnt_tag);
      }
    }
    else {
      clazz->staticFields[field->slot] = 0;
      if (isSet(field->flags, FIELD_IS_LONG)) {
        clazz->staticFields[field->slot + 1] = 0;
      }
    }
  }
}

/*
**   - if the class is not abstract, it is not allowed to contain any
**     abstract methods.
**   - identify the default initializer <init>()V and record its address.
*/

static w_instance scanMethods(w_clazz clazz) {
  w_method m;
  w_size   i;

  for (i = 0; i < clazz->numMethods; i++) {
    m = clazz->methods[i];

    if (!(clazz->flags & ACC_ABSTRACT) && (m->flags & ACC_ABSTRACT)) {
      woempa(7,"AbstractMethodError detected: non-abstract class %k fails to implement %M\n",clazz, m);

      return createExceptionInstance(currentWonkaThread, clazzAbstractMethodError, clazz->dotified);

    }

    initializeMethod(m);
  }

  m = getMethod_name_sig(clazz, string_angle_brackets_init, string_no_params);
  if (m && m->clazz == clazz) {
    clazz->defaultInit = m;
  }
  else {
    clazz->defaultInit = NULL;
  }

  m = getMethod_name_sig(clazz, string_finalize, string_no_params);
  if (m && m->clazz != clazzObject) {
    setFlag(clazz->flags, CLAZZ_HAS_FINALIZER);
    clazz->finalizer = m;
  }
  else {
    clazz->finalizer = NULL;
  }

  return NULL;
}

/*
** Initialize a class.
** The result returned is either NULL_or an exception.
**     clazz     the class to be initialised
**     thread    the calling thread
**     frame     the current stack frame
*/

w_instance initializeClass(w_clazz clazz) {
  w_thread   thread = currentWonkaThread;
  w_instance exception = NULL;
  w_frame    frame;

  if (classIsInitialized(clazz)) {

    return NULL;

  }

  x_monitor_enter(clazz->resolution_monitor, x_eternal);
  if (classIsInitialized(clazz)) {
    x_monitor_exit(clazz->resolution_monitor);

    return NULL;

  }

  exception = classMustBeResolved(clazz, thread);
  if (exception) {
    woempa(1, "Failed to resolve class %k prior to initializing it: got %k.\n", clazz, instance2clazz(exception));
    x_monitor_exit(clazz->resolution_monitor);

    return exception;

  }

  if (getClazzState(clazz) == CLAZZ_STATE_BROKEN) {
    x_monitor_exit(clazz->resolution_monitor);

    return alreadyBroken(clazz);

  }

  woempa(1, "Thread %w: initializing class %k.\n", NM(thread), clazz);
  setClazzState(clazz, CLAZZ_STATE_INITIALIZING);

  /*
  ** If this is a non-interface class and the super needs initializing,
  ** do it (recursively).  Stop one short of the ultimate superclass,
  ** namely Object.  If superclass initialisation fails then mark
  ** current class as broken,  wake up any other threads that wanted it
  ** initialized, and propagate the exception.
  */

  if ((isNotSet(clazz->flags, ACC_INTERFACE)) && (clazz->numSuperClasses > 1))
  {
    exception = classMustBeInitialized(clazz->superClazz, thread);
  }

  if (!exception) {
    exception = scanMethods(clazz);
  }

  if (!exception) {
  /*
  ** [8] If this class has a finalizer, set the `finalizer' field.
  **     If this is a local copy of a persistent class, read the static
  **     variables from persistent storage: otherwise, Set the static fields
  **     to default values and execute the <clinit> (if any).
  */

    if (clazz->numStaticWords > 0) {
      clazz->staticFields = allocClearedMem(thread, clazz->numStaticWords * sizeof(w_word));
    }
    if (isLocalCopy(clazz2Class(clazz))) {
      woempa(7, "java.lang.Class@%p (%k) is persistent, not running <clinit>\n", clazz2Class(clazz), clazz);
      readStatics(clazz);
    }
    else {
    // We optimistically set the class as initialized, as we want the
    // Garbage Collector to trace static variables as soon as they are
    // filled in ...
    // TODO: we don't need both the next line and to initialize all
    // static fields even if they don't have a ConstantValue ...
    //  memset(clazz->staticFields,0,clazz->numStaticWords*sizeof(w_word));
      clazz->initialize = alreadyInitialized;
      if (!clazz->aclazz) {
        initializeStaticFields(clazz);

        if (clazz->clinit) {
          woempa(1, "Class %k has <clinit> method %m, so let's run it.\n", clazz, clazz->clinit);
          frame = allocBottomFrame(thread, clazz->clinit);
          if (frame) {
            setFlag(frame->flags, FRAME_CLINIT);
            woempa(1,"running %M in thread %w\n",clazz->clinit,NM(thread));
            exception = runThread(thread, frame);
            if (exception) {
              woempa(8,"<clinit> of %k threw %k\n", clazz, instance2clazz(exception));
              setReferenceField(thread->Thread, NULL, F_Thread_thrown);
              unsetFlag(thread->flags, (WT_THREAD_KILLED | WT_THREAD_THROWN));
            }
            else {
              releaseBottomFrame(frame);
              woempa(1,"Successfully ran <clinit> of %k\n", clazz);
            }
          }
          else {
          /*
          ** VMError was already thrown.  This is just to make sure
          ** we treat <clinit> as having failed.
          */
            exception = (w_instance)thread->Thread[F_Thread_thrown];
          }
        }
      }
    }
  }

  /*
  ** If <clinit> terminated normally, mark class initialized
  */

  if (exception == NULL) {
    setClazzState(clazz, CLAZZ_STATE_INITIALIZED);
    woempa(1, "Initialisation of %k terminated normally. Shouting it out...\n",  clazz);
  }
  else {
  /*
  ** [10] If <clinit> failed with something other than Error,
  **      treat it as ExceptionInInitializerError.
  */

    if(!isSuperClass(clazzError,instance2clazz(exception))) {
      exception = createExceptionInstance(thread, clazzExceptionInInitializerError, instance2clazz(exception)->dotified);
    }

  /*
  ** [11] Mark the class as erroneous and propagate the exception
  */

    woempa(9, "<clinit> for %k did NOT terminate normally. Spreading the rumour...\n",  clazz);
    setClazzState(clazz, CLAZZ_STATE_BROKEN);
    clazz->initialize = alreadyBroken;
  }

  x_monitor_exit(clazz->resolution_monitor);

  return exception;

}

w_instance alreadyInitialized(w_clazz clazz) {
  woempa(1, "Class %k is already initialized.\n",  clazz);

  return NULL;

}


