/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: debug.c,v 1.11 2002/03/22 15:51:14 ssymons Exp $
*/

#include <string.h>
#include <stdio.h>

#include "oswald.h"
#include "ts-mem.h"
#include "realms.h"
#include "threads.h"
#include "stack.h"
#include "methods.h"
#include "clazz.h"
#include "hashtable.h"

#ifdef FICL
#include "ficl.h"
#include "sysdep.h"
#endif

#include "vfs.h"
#include "vfs_fcntl.h"

const char *abortMessages[] = {
  "Information",
  "ABORTING thread",
  "ABORTING group",
  "ABORTING root group",
  "ABORT Wonka",
};

x_Sem woempaSemaphore;

#ifndef VERSION_STRING
#define VERSION_STRING "<unknown>"
#endif

const char *banner = VERSION_STRING " (Developers' version)"
                     " was compiled on " __DATE__ 
                     " at " __TIME__ ".\r\n";

void (*puts_vector)(char *s);

void PutString(char *s) {
  if (puts_vector) {
    (*puts_vector)(s);
  }
}

/*
w_word processkey(w_word local_word, w_word value) {

  w_int count = (w_int)value;

  woempa(9, "Processkey running...\n");  
  if (count) {
    count -= 1;
    woempa(9, "Original %p count was %d, now %d.\n", (w_local)local_word, ((w_local)local_word)->count, count);
  }
  else {
    woempa(9, "This is strange...\n");
  }
  
  return (w_word)count;
  
}
*/

char hexDigit(int n) {
  return n<10 ? n+'0' : n+'a'-10;
}

void PutHex (unsigned int Value, unsigned int Width) {
  char buf[9];
  int i, x;

  x = Value;
  for (i=7;i>=0;--i) {
    buf[i] = hexDigit(x&0xf);
    x >>= 4;
  }
  buf[8] = 0;
  PutString(Width>8?buf:buf+8-Width);
}

void PutDec (unsigned int Value, unsigned int Width) {
  char buf[12];
  int i, x;
  char sign;

  x = Value;
  if (x<0) {
    sign = '-';
    x = -x;
  }
  else sign = 0;

  for (i=10;i>=0;--i) {
    buf[i] = '0'+(x%10);
    x /= 10;
  }
  buf[11] = 0;
  PutString(Width>11?buf:buf+11-Width);
}

/*
** We use a hashtable to store the woempa trigger level associated with
** each sourcefile: the key is a C string, and if the string contains a
** '/' character then the hashcode and equality functions ignore everything
** to the left of the last '/' in the string.  (In other words, the whole
** path doesn't have to match, just the 'basename').  If a file is not
** found in the hashtable then its trigger level is assumed to be
** woempa_default_trigger_level.
**
** There is one special case" file "hashtable.c" has its own trigger level
** outside of the hashtable, to avoid a deadlock when woempa is called
** from inside a hashtable function.
*/

static w_hashtable woempa_hashtable = NULL;
static w_int woempa_default_trigger_level = 1;

static inline char *
get_basename(const char *filename) {
  char *basename = (char *)filename + strlen(filename);
  while (basename-filename > 0 && *--basename != '/');

  return basename;
}

w_word
filename_hash(w_word nameword) {
  char *filename = (char*)nameword;
  char *basename = filename + strlen(filename);
  int   hashcode = 0x654321;
  char  one_byte;

  basename = get_basename(filename);

  while ((one_byte = *basename++)) {
    hashcode = (hashcode * 255) ^ one_byte;
  }

  return hashcode;
}

w_boolean
filenames_equal(w_word nameword1, w_word nameword2) {
  char *filename1 = (char*)nameword1;
  char *filename2 = (char*)nameword2;
  char *basename1;
  char *basename2;

  basename1 = get_basename(filename1);
  basename2 = get_basename(filename2);

  return strcmp(basename1,basename2) == 0;
}

static int hashtable_trigger_level = 1;

void _setTriggerLevel(const char *file, int trigger) {
  if (strcmp(get_basename(file),"hashtable.c") == 0) {
    hashtable_trigger_level = trigger;
  }
  else {

    if (woempa_hashtable == NULL) {
      woempa_hashtable = ht_create(W_Realm_system, (char*)"hashtable:woempa", 31, filename_hash, filenames_equal,0,0);
    }

  ht_write(woempa_hashtable,(w_word)file,(w_word)trigger);
  }
}

void setAllTriggerLevel(int trigger) {
  if (woempa_hashtable) {
    ht_destroy(woempa_hashtable);
    woempa_hashtable = NULL;
  }
  hashtable_trigger_level = trigger;
  woempa_default_trigger_level = trigger;
}

/*
 
void watchdogEntry(void* data) {

  x_thread_sleep(1000);

  while (1) {
    if (x_sem_get(&woempaSemaphore,1000)==0) {
#ifdef DEBUG    
#ifdef FICL
      ficlExec((FICL_VM*)(wonkaPointers->ficlVM),(char*)
        "watchdog "
        "quit"
      );
#endif
#endif
      (void)x_sem_put(&woempaSemaphore);
    }
    (void)x_thread_sleep(10000);
  }
}

x_thread watchdog;
#define wd_stacksize (1024 * 16)
unsigned char wd_stack[wd_stacksize];

void initDoggy () {
	memset ( wd_stack, 0x00, wd_stacksize);

	x_thread_create ( &watchdog,
		watchdogEntry,
		NULL,
		wd_stack,
		wd_stacksize,
		1, TF_START);
}

*/

#define BUFSIZE        4096
static char woempa_buffer[BUFSIZE];

/*
** A small woempa aware function that can be called from the gdb debugger...
*/

void wprintf(const char *fmt, ...) {

  va_list ap;

  va_start(ap, fmt);
  x_vsnprintf(woempa_buffer, BUFSIZE, fmt , ap);
  va_end(ap);
#ifdef LINUX
  write(1, woempa_buffer, strlen(woempa_buffer));
#else
  PutString(woempa_buffer);
  PutString((char *)"\15");
#endif

}

extern char *woempa_dump_file;
extern int  woempa_stderr;
extern int  woempa_bytecodecount;
extern int  woempa_bytecodetrigger;

void _woempa(const char *file, const char *function, int line, int level, const char *fmt, ... ) {

  va_list ap;
  int trigger = 0;
  w_size bufsize = BUFSIZE;
  static int od = 0;
    
  if (woempa_bytecodetrigger > 0 && 
      woempa_bytecodecount > woempa_bytecodetrigger)
    woempa_default_trigger_level = 2;

  if(woempa_dump_file) {
    if (! od) {
      /*         name     <--- flags --------------------------->  <------ modes -------------->  */
      od = open(woempa_dump_file, (VFS_O_WRONLY | VFS_O_CREAT | VFS_O_TRUNC), (VFS_S_IRUSR | VFS_S_IWUSR | VFS_S_IXUSR | VFS_S_IRGRP | VFS_S_IROTH));
    }
  }
  
  if (strcmp(get_basename(file),"hashtable.c") == 0) {
    trigger = hashtable_trigger_level;
  }
  else if (woempa_hashtable) {
    trigger = (int)ht_read(woempa_hashtable,(w_word)file);
  }
  else {
    trigger = woempa_default_trigger_level;
  }

  if (trigger < 1 || trigger > 9) {
    trigger = woempa_default_trigger_level;
  }
  
  if (trigger <= level && x_sem_get(&woempaSemaphore,25)==0) {
      
    w_thread jthread = currentWonkaThread;
    w_frame  jframe  = jthread ? jthread->top : NULL;
    w_method jmeth   = jframe  ? (jframe->native ? jframe->native : jframe->method) : NULL;
    
    x_snprintf(woempa_buffer, bufsize, "%p (%d) %d [%k/%w:%d] [%s:%d] : ",
      jthread,
      level,
      woempa_bytecodecount,
      jmeth ? jmeth->clazz : NULL,
      jmeth ? jmeth->name : NULL,
      jframe ? jframe->pc : 0,
      function,
      line
    );

    va_start (ap, fmt);
    (void)x_vsnprintf(woempa_buffer + strlen(woempa_buffer), bufsize - strlen(woempa_buffer), fmt , ap);
    va_end (ap);
   
#ifdef OS_NONE
    if(woempa_stderr) {
      PutString(woempa_buffer);
      PutString((char *)"\15");
    }
#else
#ifndef ECOS
    if(od) { 
      (void)write(od, woempa_buffer, strlen(woempa_buffer));
    }
#endif //ECOS
 
   
    if(woempa_stderr) {
      if (level >= 7 || (woempa_bytecodecount > woempa_bytecodetrigger && woempa_bytecodetrigger > 0)) {
        (void)write(2, woempa_buffer, strlen(woempa_buffer));
      }
    }
#endif

    (void)x_sem_put(&woempaSemaphore);

  }

}

void _wabort(const char *function, int line, int scope, const char *fmt, ... ) {
  x_thread kthread;
  w_thread thread;
  va_list ap;
  w_size bufsize = BUFSIZE;

  (void)x_sem_get(&woempaSemaphore, x_eternal);
#ifdef OS_NONE
  (void)write(1, (char *)"\15",1);
#endif
  x_snprintf(woempa_buffer, bufsize, "\n%s: %s %4d: ", abortMessages[scope], function, line);

  va_start (ap, fmt);
  (void)x_vsnprintf(woempa_buffer + strlen(woempa_buffer), bufsize - strlen(woempa_buffer), fmt , ap);
  va_end (ap);

  (void)write(1, woempa_buffer, strlen(woempa_buffer));
#ifdef OS_NONE
  (void)write(1, (char *)"\15",1);
#endif
  kthread = x_thread_current();
/*
** Replace with thread_hashtable lookup for now
  thread = (w_thread)kthread->wx_user_data;
*/
  thread = (w_thread)ht_read(thread_hashtable, (w_word)kthread);
  if ((*(w_word*)thread->label)!=(*(w_word*)"thread")){
    woempa(9,"currentWonkaThread called at %s:%d from non-wonka thread (label = %s)\n", function, line, thread->label);
    thread = NULL;
  }

  {
    w_fifo thread_list = ht_list_keys(thread_hashtable);
    if (thread_list) {
      x_thread other_thread;
      while ((other_thread = getFifo(thread_list))) {
        if (other_thread != kthread) {
          x_thread_suspend(other_thread);
        }
      }
    }
  }

#ifdef DEBUG
  if (thread) {
    PutString((char*)"Failing thread: ");
    PutString(threadDescription(thread)); 
    PutString((char *)"\12");
  }
  else {
    PutString((char*)"Unable to identify failing thread, sorry mate...\n");
  }
#ifdef OS_NONE
  (void)write(1, (char *)"\15",1);
#endif
#endif

#ifdef FICL
  smartAbort();
#endif
  (void)x_sem_put(&woempaSemaphore);

  if (scope > ABORT_INFO) {
#ifdef OS_NONE
    (void)x_thread_sleep(100);
    x_thread_deletei(x_thread_current());
#else
    abort();
#endif
  }
  
}

