/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: inits.c,v 1.9 2002/03/24 16:26:28 dbuytaert Exp $
*/

#include <string.h>

#include "argument.h"
#include "clazz.h"
#include "device.h"
#include "methods.h"
#include "driver_virtual.h"
#include "debug_driver.h"
#include "file_driver.h"
#include "deflate_driver.h"
#include "locks.h"
#include "loopback.h"
#include "ps-mem.h"
#include "sio.h"
#include "strings.h"
#include "wonka.h"
#include "vfs.h"

#ifdef FICL
#include "ficl.h"
#include "sysdep.h"
#endif

#ifdef MODULES
extern void  x_symtab_kernel(void);
#endif
//TODO: get rid of these extern's
extern w_Driver_ByteSerial sio_driver;
extern w_Driver_ByteSerial wfd_driver;
extern void mouse_set_path(char *s);

w_Pointers *wonkaPointers;
char *bootclasspath = "/" BOOTCLASSDIR "/:/" BOOTCLASSDIR "/" BOOTCLASSFILE;

#ifndef DEVCONFDIR
#define DEVCONFDIR "system"
#endif

#ifndef DEVCONFFILE
#define DEVCONFFILE "device.config"
#endif

extern x_Sem woempaSemaphore;

w_ubyte *initWonka(w_ubyte *start_heap) {
#ifdef FICL
  initFicl();
#endif


#ifdef ENABLE_PS
  initPSmem();
#endif

  initRealms();
  initHeap();
  initLocks();

  (void)x_sem_create(&woempaSemaphore, 1);

#ifdef MODULES
  x_symtab_kernel();
#endif

  start_heap = initKernel(start_heap);

  return start_heap;

}

#ifdef FSENABLE
#ifdef ECOS
  extern w_driver image_driver;
#else
  extern w_driver disk_driver;
#endif //ECOS
#endif
  
#ifdef JDWP  
  w_void jdwp_init(w_void);
#endif

w_int testWonka(void);

void startVFS(void) {
  init_vfs();
#ifdef FSENABLE
#ifdef ECOS
    vfs_mount((char *)"hdb", (char *)"/", (char *)"e2fs", VFS_MOUNT_RW);
#else
    vfs_mount((char *)"hda4", (char *)"/", (char *)"e2fs", VFS_MOUNT_RW);
#endif // ECOS
#else
  vfs_mount(0, (char *)"/", (char *)"wromfs", VFS_MOUNT_RO); 
#endif
}

#ifdef FICL
static char             *devconf_data;
static int              fd;
static struct vfs_STAT  statbuf;
static w_word           devconf_size;
#define FICLBUF_SIZE    2048
static char             ficlbuf[FICLBUF_SIZE];
static const char      *devconf_filename = DEVCONFDIR "/" DEVCONFFILE;

/* 
** print2ficl formats a string and passes it ti FICL for execution.
** Note that the buffer is fixed-length (FICLBUF_SIZE bytes), and
** if the FICL command is too long it will be truncated.
*/
static void print2ficl(const char *fmt, ...) {
  w_int l;
  va_list ap;

  va_start(ap, fmt);
  l = x_vsnprintf(ficlbuf, FICLBUF_SIZE, fmt , ap);
  va_end(ap);
  ficlbuf[l] = 0;
  ficlExec ((FICL_VM*)(wonkaPointers->ficlVM),ficlbuf);
}

/*
** ficl_attach_device implements the FICL word `(attach-device)':
** ( caddr1 u1   u2   caddr2 u3  -- )
**   family name nbr  pathname
** associates a device (used in e.g. (register-device))
** with a path such as /dev/ttyS0 or ./dump.
** Only works with families 'sio' and 'file'.
*/
static void ficl_attach_device (FICL_VM *pVM) {
  w_int path_length = stackPopINT(pVM->pStack);
  char* path_chars  = stackPopPtr(pVM->pStack);
  w_int number      = stackPopINT(pVM->pStack);
  w_int family_name_length = stackPopINT(pVM->pStack);
  char* family_name_chars  = stackPopPtr(pVM->pStack);
  path_chars[path_length] = 0;
  family_name_chars[family_name_length] = 0;
  // TODO: something less blatantly hackful
  if (strcmp(family_name_chars, "sio") == 0) {
    woempa(7,"attach-device %s %d to %s\n", family_name_chars, number,path_chars);
    sio_set_path(number, path_chars);
  }
  else if (strcmp(family_name_chars, "file") == 0) {
    woempa(7,"attach-device %s %d to %s\n", family_name_chars, number,path_chars);
    file_set_path(number, path_chars);
  }
  else {
    woempa(9,"attach-device %s %d to %s\n", family_name_chars, number,path_chars);
  }
}

#if defined(AWT_FDEV) || defined (AWT_XSIM)
/*
** ficl_attach_mouse implements the FICL word `(attach-mouse)':
** ( caddr1 u1   u2   caddr2 u3  -- )
**   family name nbr  pathname
** associates the mouse device with a path such as /dev/touchscreen/anakin.
** Currently family name and nbr are ignored.
*/
static void ficl_attach_mouse (FICL_VM *pVM) {
  w_int path_length = stackPopINT(pVM->pStack);
  char* path_chars  = stackPopPtr(pVM->pStack);
  w_int number      = stackPopINT(pVM->pStack);
  w_int family_name_length = stackPopINT(pVM->pStack);
  char* family_name_chars  = stackPopPtr(pVM->pStack);
  number = 0; /* tace, compiler */
  path_chars[path_length] = 0;
  family_name_chars[family_name_length] = 0;
  woempa(7,"attach-mouse %s %d to %s\n", family_name_chars, number,path_chars);
  mouse_set_path(path_chars);
}
#endif

/*
** ficl_register_device implements the FICL word `(register-device)':
** ( caddr1 u1     caddr2 u2    u3      u4      -- )
**   device name   family name  devnum  devtype
** registers a device with the name caddr1 u1 using the driver caddr2 u2,
** with device number u3 and device type u4.
*/
static void ficl_register_device (FICL_VM *pVM) {
  w_int device_type        = stackPopINT(pVM->pStack);
  w_int number             = stackPopINT(pVM->pStack);
  w_int family_name_length = stackPopINT(pVM->pStack);
  char* family_name_chars  = stackPopPtr(pVM->pStack);
  w_int device_name_length = stackPopINT(pVM->pStack);
  char* device_name_chars  = stackPopPtr(pVM->pStack);
  device_name_chars[device_name_length] = 0;
  family_name_chars[family_name_length] = 0;
  woempa(7,"register-device type %d name %s family %s number %d\n", device_type, device_name_chars, family_name_chars, number);
  registerDevice(device_name_chars, family_name_chars, number, device_type);
}

/*
** If a device configuration file is present, read it in and and pass it 
** to FICL for execution.  We first define the necessary vocabulary:
** `(register-device)' and `(set-file-name)' (see above), the constant 
** `byte-serial', and some wrapper words  `register-serial-device'
** and `set-file-name' for syntactic sugar.
*/
void readDeviceConfig(void) {
  ficlBuild((char*)"(attach-device)", ficl_attach_device, FW_DEFAULT);
#if defined(AWT_FDEV) || defined (AWT_XSIM)
  ficlBuild((char*)"(attach-mouse)", ficl_attach_mouse, FW_DEFAULT);
#endif
  ficlBuild((char*)"(register-device)", ficl_register_device, FW_DEFAULT);

  print2ficl("%d constant max-devnam-length ", 32);
  print2ficl("%d constant byte-serial ", wdt_byte_serial);
  print2ficl("variable device-name max-devnam-length allot "
             "variable family-name max-devnam-length allot ");
  print2ficl(": attach-serial-device " 
             "  device-name max-devnam-length 1+ 0 fill "
             "  family-name max-devnam-length 1+ 0 fill "
             "  bl word family-name max-devnam-length 1+ move "
             "  family-name count "
             "  bl word count evaluate "
             "  bl word count "
             "  (attach-device) "
             "; ");
  print2ficl(": attach-mouse-device " 
             "  device-name max-devnam-length 1+ 0 fill "
             "  family-name max-devnam-length 1+ 0 fill "
             "  bl word family-name max-devnam-length 1+ move "
             "  family-name count "
             "  bl word count evaluate "
             "  bl word count "
             "  (attach-mouse) "
             "; ");
  print2ficl(": register-serial-device " 
             "  device-name max-devnam-length 1+ 0 fill "
             "  family-name max-devnam-length 1+ 0 fill "
             "  bl word device-name max-devnam-length 1+ move "
             "  bl word family-name max-devnam-length 1+ move "
             "  device-name count family-name count "
             "  bl word count evaluate "
             "  byte-serial (register-device) "
             "; ");

  if (vfs_stat(devconf_filename, &statbuf) == 0) {
    devconf_size = statbuf.st_size;
    woempa(7, "Found %s, %d bytes\n", devconf_filename, devconf_size);
    devconf_data = allocMem(W_Thread_system, devconf_size+1);
    fd = vfs_open(devconf_filename, VFS_O_RDONLY, 0);
    vfs_read(fd, devconf_data, devconf_size);
    vfs_close(fd);
    devconf_data[devconf_size] = 0;
    woempa(7, "Found device configuration in file %s\n", devconf_filename);
    ficlExec((FICL_VM*)wonkaPointers->ficlVM, devconf_data);
    print2ficl(" quit");
    releaseMem(devconf_data);
  }
  else {
    devconf_data = 
      "register-serial-device stdin   wfd  0 \n"
      "register-serial-device stdout  wfd  1 \n"
      "register-serial-device stderr  wfd  2 \n"
      "register-serial-device unzip_  zip  0 \n"
      "register-serial-device zip_   zip 20 "
    ;
    woempa(9, "Could not find `%s', assuming default configuration:\n%s\n", devconf_filename, devconf_data);
    print2ficl("%s quit", devconf_data);
  }

}
#endif

extern char *get_default_classpath(void);

#if defined(AWT_XSIM) || defined(AWT_NONE)
char            *awt_args = NULL;
#endif

#ifdef JDWP
int      jdwp_enabled = 0;
char     *jdwp_args = NULL;
void     print_jdwp_help(void);
#endif

void args_read(void) {
  Wonka_InitArgs  *args = &system_InitArgs;
  char            **properties;
  int             prop_count = 0;
  char            *classpath;
  memset(args, 0, sizeof(Wonka_InitArgs));
  properties = allocClearedMem_system(100 * sizeof(char *));

  classpath = get_default_classpath();

  woempa(7, "Found %d command line arguments\n", command_line_argument_count);
  while (command_line_argument_count) {
    w_int extra_args = 0;
    woempa(7, "Analysing command line argument `%s'\n", command_line_arguments[0]);

    if((strcmp(command_line_arguments[0], "-cp") == 0) || 
       (strcmp(command_line_arguments[0], "-classpath") == 0)) {
      extra_args = 1;
      woempa(7, "Found a classpath argument `%s'\n", command_line_arguments[1]);
      if(classpath) {
        woempa(7, "Overrides the default classpath `%s'\n", classpath);
      } 
      classpath = command_line_arguments[1];
    }
    else if(strncmp(command_line_arguments[0], "-D", 2) == 0) {
      woempa(7, "Found a property argument `%s'\n", command_line_arguments[0] + 2);
      properties[prop_count++] = (char *)&command_line_arguments[0][2];
    }
    else if(strncmp(command_line_arguments[0], "-Xbootclasspath:", 16) == 0) {
      woempa(7, "Found an Xbootclasspath argument `%s'\n", command_line_arguments[0] + 16);
      bootclasspath = (char *)&command_line_arguments[0][16];
    }
#ifdef JDWP 

    else if(strncmp(command_line_arguments[0], "-Xjdwp:", 7) == 0) {
      woempa(7, "Found an Xjdwp argument `%s'\n", command_line_arguments[0] + 7);
      jdwp_args = (char *)&command_line_arguments[0][7];
      if(strcmp(jdwp_args, "help") == 0) {
        print_jdwp_help();
      }
    }
    else if(strncmp(command_line_arguments[0], "-Xdebug", 7) == 0) {
      woempa(7, "Found an Xdebug argument\n");
      jdwp_enabled = 1;
    }

#endif  /* JDWP */

#if defined(AWT_XSIM) || defined(AWT_NONE)

    else if(strncmp(command_line_arguments[0], "-Wrudolph:", 10) == 0) { 
      woempa(7, "Found a Wrudolph argument `%s'\n", command_line_arguments[0] + 10);
      awt_args = (char *)&command_line_arguments[0][10];
    }
    
#endif  /* AWT_XSIM */

    else if(strncmp(command_line_arguments[0], "-", 1) == 0) {
      woempa(7, " Unrecognized option: %s\n", command_line_arguments[0]);

      exit(1);

    } else {
      woempa(7, " Not an option: %s, passing remaining %d arguments on to Init/main()\n", command_line_arguments[0], command_line_argument_count);

      /*
      ** It's not an option, so it has to be the main class and 
      ** it's arguments. Since command_line_argument_count and 
      ** command_line_arguments were updated each time we went
      ** through this loop, they're now pointing right at the
      ** requested startclass.
      */

      break;

    }
    
    command_line_argument_count = command_line_argument_count - 1 - extra_args;
    command_line_arguments += extra_args + 1;

  }

  args->properties = reallocMem(properties, (prop_count + 1) * sizeof(void *));;
  args->classpath  = classpath;

#ifdef DEBUG
  {
    w_int i;
    for (i = 0; i < prop_count; ++i) {
      woempa(7, "Property: %s\n", properties[i]);
    }
    woempa(7, "Classpath: %s\n", args->classpath);
    woempa(7, "Xbootclasspath: %s\n", bootclasspath);
    for (i = 0; i < command_line_argument_count; ++i) {
      woempa(7, "Java argument: %s\n", command_line_arguments[i]);
    }
  }
#endif
}

void startWonka(void* data) {
  JavaVM *vm;
  JNIEnv *env;

  x_formatter('k', print_clazz);
  x_formatter('m', print_method_m);
  x_formatter('M', print_method_M);

  make_ISO3309_CRC_table();

  /*
  ** Here we start routines that require a valid heap (for malloc) to
  ** be set up...
  */

  args_read();

  startStrings();

  startDeviceRegistry();
  startDriverRegistry();
  
  registerDevice("debug0", "debug", 0, wdt_byte_serial);
  registerDriver((w_driver)&debug_driver);

#ifdef FSENABLE
#ifdef ECOS
  registerDevice("hdb", "hdb", 0, wdt_block_random);
  registerDriver((w_driver)&image_driver);
#else
  registerDevice("hda", "hda", 0, wdt_block_random);
  registerDriver((w_driver)&disk_driver);
#endif // ECOS
#endif

  /* Initialize known filesystems */
  
#if (defined(FSENABLE)) 
  init_e2fs(); 
#endif
  startVFS();

  startPlatform();
#ifdef FICL
  readDeviceConfig();
#else
// intialise some devices
//  registerDevice("sio0", "sio", 0, wdt_byte_serial);
//  registerDevice("sio1", "sio", 1, wdt_byte_serial);
//  registerDevice("sio2", "sio", 2, wdt_byte_serial);
//  registerDevice("sio3", "sio", 3, wdt_byte_serial);

//  registerDevice("file0", "file", 0, wdt_byte_serial);
//  registerDevice("file1", "file", 1, wdt_byte_serial);
//  registerDevice("file2", "file", 2, wdt_byte_serial);
//  registerDevice("file3", "file", 3, wdt_byte_serial);

  registerDevice("stdin",  "wfd", 0, wdt_byte_serial);
  registerDevice("stdout", "wfd", 1, wdt_byte_serial);
  registerDevice("stderr", "wfd", 2, wdt_byte_serial);
  
  registerDevice("unzip_", "zip", 0, wdt_byte_serial);
  registerDevice("zip_", "zip", 20, wdt_byte_serial);

  registerDevice("loop_", "loop", 0, wdt_byte_serial);
 
#ifndef ECOS
  mouse_set_path("/dev/touchscreen/anakin");
#endif // ECOS
#endif
#ifndef ECOS
  registerDriver((w_driver)&sio_driver);
#endif // ECOS
  registerDriver((w_driver)&wfd_driver);
  registerDriver((w_driver)&deflate_driver);
  registerDriver((w_driver)&loopback_driver);
  registerDriver((w_driver)&file_driver);

#ifdef FICL
  ficlExec((FICL_VM*)(wonkaPointers->ficlVM),(char*)
    " .( Starting up Claudia ... ) cr quit "
  );
  startClaudia();
#endif

  PutString((char*)"Calling JNI_CreateJavaVM() ...\n");
  JNI_CreateJavaVM(&vm, &env, &system_InitArgs);

#ifdef JDWP
  jdwp_init();
#endif
  
  groupManagerEntry(W_ThreadGroup_system);

}

