/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


#include "oswald.h"
#include "wonka.h"
#include "ts-mem.h"
#include "realms.h"

#include <stdio.h>  /* for sprintf */
#include <string.h> /* for strlen */

#ifndef SHARED_OBJECTS

#ifdef OSWALD

#include "modules.h"
#include "vfs.h"

void *loadModule(char * name) {
  
  x_status         status;
  x_ubyte          *elfdata;
  int              fd;
  struct vfs_STAT  statbuf;
  w_word           elfsize;
  x_Module         Module;
  x_module         module = &Module;
  char             *filename;
  
  filename = allocMem_system(strlen(name) + 12);
  sprintf(filename, "./system/%s.o", name);
  
  if(vfs_stat(filename, &statbuf) != -1) { 
    elfsize = statbuf.st_size;
    woempa(9, "%s is %d bytes\n", filename, elfsize);
    elfdata = allocMem_system(elfsize);
    fd = vfs_open(filename, VFS_O_RDONLY, 0);
    vfs_read(fd, elfdata, elfsize);
    vfs_close(fd);
    releaseMem(filename);
  
    status = x_module_load(module, elfdata, x_mem_alloc, x_mem_free);
    if(status != xs_success) {
      woempa(9, "Loading of lib %s failed... (%d)\n", name, status);
      releaseMem(elfdata);
      return NULL;
    }

    status = x_module_resolve(module);
    if(status != xs_success) {
      woempa(9, "Resolving of lib %s failed... (%d)\n", name, status);
      releaseMem(elfdata);
      return NULL;
    }
  
    status = x_module_relocate(module);
    if(status != xs_success) {
      woempa(9, "Relocating of lib %s failed... (%d)\n", name, status);
      releaseMem(elfdata);
      return NULL;
    }
  
    status = x_module_init(module);
    if(status != xs_success) {
      woempa(9, "Initialization of lib %s failed... (%d)\n", name, status);
      releaseMem(elfdata);
      return NULL;
    }
  
  } else {
    releaseMem(filename);
    woempa(9, "Could not find lib %s.\n", name);
    return NULL;
  }

  return (void *)module;
}

x_boolean searchForSymbol(x_module module, void * argument) {
  return 1;
}

void *lookupModuleSymbol(w_ubyte *name) {
 /* Iterate through all the loaded modules to find the symbol */
  x_module_iterate(searchForSymbol, name);
  return NULL; 
}

#else /* !OSWALD */

void *loadModule(w_ubyte *name) { return NULL; }
void *lookupModuleSymbol(w_ubyte *name) { return NULL; }

#endif /* OSWALD */

#else /* SHARED_OBJECTS */

#include <dlfcn.h>

static void **handles = NULL;
static void **current = NULL;

void *loadModule(w_ubyte *name) {
  char *filename;
  char *error;
  void *handle;
  int  offset = (int)current - (int)handles;

  if((offset % (10 * sizeof(void *))) == 0) {
    handles = x_mem_realloc(handles, offset + 10 * sizeof(void *));
    current = (void **)((int)handles + offset);
  }

  filename = allocMem_system(strlen(name) + 10);
  sprintf(filename, "lib%s.so", name);
  
  woempa(9, "%s\n", filename);
  
  handle = dlopen(filename, RTLD_LAZY | RTLD_GLOBAL);
  error = dlerror();
 
  if(!error) *current++ = handle;

  woempa(9, "dlerror() : %s\n", error);
  releaseMem(filename);

  return handle;
}

void *lookupModuleSymbol(w_ubyte *name) {
  void *symbol = NULL;
  void **check = handles;

  woempa(9, "%s\n", name);

  while(symbol == NULL && check != current) {
    symbol = dlsym(*check++, name);
    woempa(9, "dlerror() : %s\n", dlerror());
  }
  
  return symbol;
}

#endif /* SHARED_OBJECTS */ 

