/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: DeviceOutputStream.c,v 1.3 2002/02/20 08:55:25 dfrederickx Exp $
*/

#include "arrays.h"
#include "core-classes.h"
#include "exception.h"
#include "comms.h"
#include "hashtable.h"
#include "heap.h"
#include "ts-mem.h"
#include "strings.h"
#include "threads.h"
#include "driver_byteserial.h"

/**@name File DeviceOutputStream.c
*/
//@{

///
/**@name Helper function for object constructor
*/
//@{
/// Create an OutputStream by name
/**@type native method
** @name createFromString
** Initialise an instance of DeviceOutputStream using the named Device.
** Called from the constructor \textsf{DeviceOutputStream(String)}.
** Returns the unique id given by registerDevice().
*/
w_int
DeviceOutputStream_createFromString
(  JNIEnv *env, w_instance thisOutputStream,
  w_instance nameString
) {
  w_thread   thread = JNIEnv2w_thread(env);
  w_string   nameStr;
  w_device commport;
  
  w_ubyte    *name;
  w_ubyte    *dummy = NULL;
  w_int      i;

  if (isNullInstance(nameString)) {
    throwException(thread,clazzNullPointerException,NULL);
    return 0;
  }

  nameStr = String2string(nameString);
  name = allocMem(thread, nameStr->length + 1);
 
  dummy = (w_ubyte *)name;
  for (i = 0; i < (w_int)nameStr->length; ++i) {
     (*dummy) = nameStr->chars[i];
     dummy++;
  }
  (*dummy) = '\0';

  commport = getDeviceByName(name);
  if (commport->type != wdt_byte_serial) {
    woempa(9,"Device %s is of wrong type! (%d s/b %d)\n", name, commport->type, wdt_byte_serial);
    commport = NULL;
  }
  releaseMem(name);
  if (commport) {
    if (deviceBSOpen(name, wdp_read) == NULL) {
      throwException(thread,clazzIOException,NULL);
    }
    return (w_int)commport;
  }
  else {
    throwException(thread,clazzIllegalArgumentException,NULL);
  }
  return 0;
}
//@}


///
/**@name Helper functions for write methods
*/
//@{
/// Write bytes                 
/**@type native method
** @name WriteFromBuffer(I[BII)I                    
** Write bytes to the output stream from an array.
*/
void
DeviceOutputStream_writeFromBuffer
(  JNIEnv *env, w_instance thisOutputStream,
  w_int devid,
  w_instance ByteArray, w_int offset, w_int length
) {
  w_thread   thread = JNIEnv2w_thread(env);
  w_device device = (w_device)devid;
  w_sbyte    *byte_array = instance2Array_byte(ByteArray);
  w_sbyte    *dest = byte_array + offset;
  w_size     dummy;

  if (length<0 || offset<0 || length+offset > (w_int)ByteArray[F_Array_length]) {
    throwException(thread,clazzArrayIndexOutOfBoundsException,NULL);
    return;
  }

  if (deviceBSWrite(device, dest, length, &dummy, x_eternal) != wds_success) {
    throwException(thread,clazzIOException,NULL);
  }
}
//@}


//@}
//@}
