/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: UARTDriver.c,v 1.3 2002/02/20 08:56:05 dfrederickx Exp $
*/

#include <string.h>

#include "clazz.h"
#include "comms.h"
#include "core-classes.h"
#include "exception.h"
#include "fifo.h"
#include "hashtable.h"
#include "heap.h"
#include "methods.h"
#include "strings.h"
#include "threads.h"
#include "driver_byteserial.h"

/**@name File UARTDriver.c
   @author Chris Gray
   @version $Revision: 1.3 $
*/
//@{

/**@name A few static variables.

    Two of these are used in a thread-unsafe way.  This is OK so long as
    device driver is carried out in series by a single thread (but will
    crash horribly if anyone tries to initialize multiple drivers in
    parallel.  TODO -- FIXME by designing an API for this sort of thing ...
*/
//@{
/// The class javax.comm.CommPortIdentifier - can be static, because never changes
static jclass class_CommPortIdentifier;
/// The method CommPortIdentifier.addPortName - can be static, because never changes
static jmethodID method_addPortName;
/// The JNI environment in which initialize() is called.  THIS IS NOT THREAD SAFE!
static JNIEnv *theEnv;
/// The CommDriver from which initialize() is called.  THIS IS NOT THREAD SAFE!
static jobject theUARTDriver;
/// The list of names.  THIS IS NOT THREAD SAFE!
static w_fifo namelist;
//@}

/// If the given device is a byte serial device, add its name to namelist.
//
static void scanport(w_device device) {
  w_string nameString = cstring2String(currentWonkaThread, device->name, strlen(device->name));
  woempa(7,"Adding %w to namelist\n", nameString);
  putFifo(nameString,namelist);
}

/// Return the name of the first byte serial device.
/// Must be called before any call to UARTDriver_static_nextUARTname.
w_instance
UARTDriver_static_firstUARTname (JNIEnv *env, w_instance classUARTDriver
) {
  w_string name;

  namelist = allocFifo(W_Realm_system,15);
  everyDeviceByDriverFamily(scanport, "sio");
  putFifo(cstring2String(currentWonkaThread, "stdin", 5), namelist);
  putFifo(cstring2String(currentWonkaThread, "stdout", 6), namelist);
  putFifo(cstring2String(currentWonkaThread, "stderr", 6), namelist);
  
  name = getFifo(namelist);
  if (name) {
    woempa(7,"Returning %w\n",name);

    return getStringInstance(JNIEnv2w_thread(env),name);

  }
  else {
    woempa(7,"Did not find any byte serial devices\n");
    releaseFifo(namelist);

    return NULL;
  }
}

/// Return the name of the next byte serial device.
/// Must be called after UARTDriver_static_nextUARTname, and must not be
/// again after it has returned NULL.
w_instance
UARTDriver_static_nextUARTname (JNIEnv *env, w_instance classUARTDriver
) {
  w_string name;
  name = (w_string)getFifo(namelist);
  if (name) {
    woempa(7,"Returning %w\n",name);

    return getStringInstance(JNIEnv2w_thread(env),name);

  }
  else {
    woempa(7,"Exhausted all byte serial devices\n");
    releaseFifo(namelist);

    return NULL;
  }
}

/// If the given device is a byte serial device, call CommPortIdentifier.addPortName
static void addport(w_device device) {
  w_string nameString = cstring2String(currentWonkaThread, device->name, strlen(device->name));
  w_instance name = getStringInstance(JNIEnv2w_thread(theEnv),nameString);

  woempa(9,"Invoking CommPortIdentifier.addPortName(%w,1,UARTDriver@%p)\n",nameString,theUARTDriver);
  (*theEnv)->CallStaticVoidMethod(theEnv, class_CommPortIdentifier, method_addPortName, name, 1, theUARTDriver);
}

///
/**@type native method
** @name initialize()V
**
*/
void
UARTDriver_initialize (JNIEnv *env, w_instance thisUARTDriver
) {
  theEnv = env;
  theUARTDriver = thisUARTDriver;
  class_CommPortIdentifier = (*env)->FindClass(env,"javax/comm/CommPortIdentifier");
  woempa(9,"class_CommPortIdentifier = %p\n",class_CommPortIdentifier);
  method_addPortName = (*env)->GetStaticMethodID(env,class_CommPortIdentifier,"addPortName","(Ljava/lang/String;ILjavax/comm/CommDriver;)V");
  woempa(9,"method_addPortName = %p\n",method_addPortName);
  everyDeviceByDriverFamily(addport, "sio");
  putFifo(cstring2String(currentWonkaThread, "stdin", 5), namelist);
  putFifo(cstring2String(currentWonkaThread, "stdout", 6), namelist);
  putFifo(cstring2String(currentWonkaThread, "stderr", 6), namelist);
}

//@}
