/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: UARTInputStream.c,v 1.5 2002/03/19 10:39:56 gray Exp $
*/

#include "arrays.h"
#include "core-classes.h"
#include "comms.h"
#include "exception.h"
#include "hashtable.h"
#include "heap.h"
#include "ts-mem.h"
#include "strings.h"
#include "threads.h"
#include "driver_byteserial.h"

/*
** Create an InputStream by name
** native method
** @name createFromString
** Initialise an instance of UARTInputStream using the named UART.
** Called from the constructor \textsf{UARTInputStream(String)}.
*/
void
UARTInputStream_createFromString
(  JNIEnv *env, w_instance thisInputStream,
  w_instance nameString
) {
  w_thread   thread = JNIEnv2w_thread(env);
  w_string   nameStr;
  w_device commport;
  
  w_ubyte    *name;
  w_ubyte    *dummy = NULL;
  w_int      i;

  if (isNullInstance(nameString)) {
    throwException(thread,clazzNullPointerException,NULL);
    return;
  }

  nameStr = String2string(nameString);
  name = allocMem(thread, nameStr->length + 1);
 
  dummy = (w_ubyte *)name;
  for (i = 0; i < (w_int)nameStr->length; ++i) {
     (*dummy) = nameStr->chars[i];
     dummy++;
  }
  (*dummy) = '\0';

  commport = getDeviceByName(name);
  if (commport->type != wdt_byte_serial) {
    woempa(9,"Device %s is of wrong type! (%d s/b %d)\n", name, commport->type, wdt_byte_serial);
    commport = NULL;
  }
  if (commport) {
    thisInputStream[F_UARTInputStream_wotsit] = (w_word)commport;
    if (deviceBSOpen(name, wdp_read) == NULL) {
      throwException(thread,clazzIOException,NULL);
    }
  }
  else {
    throwException(thread,clazzIllegalArgumentException,NULL);
  }
  releaseMem(name);
}


/* Read one byte                 
** native method
** read()I                                 
** Read one byte from the input stream.
** 
*/
w_int
UARTInputStream_read
(  JNIEnv *env, w_instance thisInputStream
) {
  w_device s = (w_device)thisInputStream[F_UARTInputStream_wotsit];
  w_word     temp;
  w_sbyte    b;

  if (deviceBSRead(s, &b, 1, &temp, x_eternal) != wds_success) {
    return -1;
  }

  return b & 0xff;


}

/* Read many bytes                 
** native method
** readIntoBuffer([BII)I                    
** Read bytes from the input stream into an array.
*/
w_int
UARTInputStream_readIntoBuffer
(  JNIEnv *env, w_instance thisInputStream,
  w_instance ByteArray, w_int offset, w_int length
) {
  w_thread     thread = JNIEnv2w_thread(env);
  w_device   s = (w_device)thisInputStream[F_UARTInputStream_wotsit];
  w_sbyte      *byte_array  = instance2Array_byte(ByteArray);
  w_sbyte      *dest  = byte_array + offset;
  w_size       lread;

  if (length<0 || offset<0 || length+offset > (w_int)ByteArray[F_Array_length]) {
    throwException(thread,clazzArrayIndexOutOfBoundsException,NULL);
    return -1;
  }

  
  if (deviceBSRead(s, dest, length, &lread, x_eternal) != wds_success) {
    throwException(thread,clazzIOException,NULL);
  }

  return lread;
}


/* native method
** skip0(L)L                    
** Skip bytes from the input stream.
** We do this by asking the underlying driver to ``read into nowhere''.
*/
w_long
UARTInputStream_skip0
(  JNIEnv *env, w_instance thisInputStream,
  w_long n
) {
  w_thread     thread = JNIEnv2w_thread(env);
  w_device   s = (w_device)thisInputStream[F_UARTInputStream_wotsit];
  w_long       l = 0;
  w_size       lread;
  w_int        rc = wds_success;
  w_ubyte      *buffer = allocMem(thread, (w_size)(l > 32768 ? 32768 : l));

  for (l=0;l<n-32767 && rc==wds_success;l+=32767) {
    rc = deviceBSRead(s, buffer, 32767, &lread, x_eternal);
  }
  rc = deviceBSRead(s, buffer, (w_int)l, &lread, x_eternal);

  if (rc != wds_success) {
    throwException(thread,clazzIOException,NULL);
  }

  releaseMem(buffer);
  
  return l;
}

/* native method
** @name available()I                    
** Ask the underlying driver how many bytes of input are waiting to be read.
*/
w_int
UARTInputStream_available
(  JNIEnv *env, w_instance thisInputStream
) {
  w_thread   thread = JNIEnv2w_thread(env);
  w_device s = (w_device)thisInputStream[F_UARTInputStream_wotsit];
  w_word     reply;

  if(deviceBSQuery(s, wdi_get_available, &reply, x_eternal) != wds_success) {
    throwException(thread,clazzIOException,NULL);
  }

  return reply;

}

/* native method
** close0()V
** Ask the underlying driver to close the device for input.
*/
void
UARTInputStream_close0
(  JNIEnv *env, w_instance thisInputStream
) {
  w_device s = (w_device)thisInputStream[F_UARTInputStream_wotsit];

  if (s) deviceBSClose(s);
}

