/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: ObjectStreamField.c,v 1.6 2002/03/20 13:04:20 gray Exp $
*/

#include <string.h>

#include "core-classes.h"
#include "clazz.h"
#include "fields.h"
#include "heap.h"
#include "loading.h"
#include "realms.h"
#include "strings.h"
#include "threads.h"
#include "ts-mem.h"

void ObjectStreamField_create(JNIEnv *env, w_instance this, w_instance String, w_instance Type) {

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz type;
  w_instance Field;

  if (isNullInstance(Type)) {
    wabort(ABORT_WONKA, "Type is NULL\n");
  }  

  type = Class2clazz(Type);
  
  if (type->aclazz) {
    this[F_ObjectStreamField_code] = (w_word)'[';
  }
  else if (type == clazz_byte) {
    this[F_ObjectStreamField_code] = (w_word)'B';
  }
  else if (type == clazz_char) {
    this[F_ObjectStreamField_code] = (w_word)'C';
  }
  else if (type == clazz_double) {
    this[F_ObjectStreamField_code] = (w_word)'D';
  }
  else if (type == clazz_float) {
    this[F_ObjectStreamField_code] = (w_word)'F';
  }
  else if (type == clazz_int) {
    this[F_ObjectStreamField_code] = (w_word)'I';
  }
  else if (type == clazz_long) {
    this[F_ObjectStreamField_code] = (w_word)'J';
  }
  else if (type == clazz_short) {
    this[F_ObjectStreamField_code] = (w_word)'S';
  }
  else if (type == clazz_boolean) {
    this[F_ObjectStreamField_code] = (w_word)'Z';
  }
  else {
    /*
    ** Must be another class or Interface type.
    */
    this[F_ObjectStreamField_code] = (w_word)'L';
  }

  woempa(9, "fieldname is '%w', type is Class '%k', type code set at '%c'.\n", String2string(String), type, (char)this[F_ObjectStreamField_code]);

// TODO: exception = ...
  classMustBeInitialized(clazzField,thread);
  Field = allocInstance(thread, clazzField);
  if (Field) {
    Field[F_Field_wotsit] = (w_word)nullInstance;
    setReferenceField(this, Field, F_ObjectStreamField_wotsit);
    this[F_ObjectStreamField_type] = (w_word)Type;
  }

}

w_instance ObjectStreamField_getField(JNIEnv *env, w_instance this) {

  return (w_instance)this[F_ObjectStreamField_wotsit];
  
}

#define TMP_BUFFER_SIZE 256

// Steven, chris:
// it appears that when the instance is an array, the F_ObjectStreamField_type is 
// already in the right format (e.g. [I and [java/lang/String; )
// when the .._type is an Object however, the L...; conversion has to be done.

w_instance ObjectStreamField_getTypeString(JNIEnv *env, w_instance this) {

  w_thread thread = JNIEnv2w_thread(env);
  w_string typeString;
  w_char *buffer;
  w_clazz clazz;
  w_instance Class;
  w_char code = (w_char)this[F_ObjectStreamField_code];
  w_size length;
  w_instance result;
  w_size i;
  w_char ch;

  buffer = allocMem(thread, TMP_BUFFER_SIZE * sizeof(w_byte));

  buffer[0] = (w_byte)code;
  length = 1;
  
  if (code == (w_char)'L' || code == (w_char)'[') {
    Class = (w_instance)this[F_ObjectStreamField_type];
    clazz = Class2clazz(Class);
    woempa(9, "Clazz to return as type is '%k'.\n", clazz);
    if (code == (w_char)'L') {
      for (i = 0; i < clazz->dotified->length; ++i) {
        ch = clazz->dotified->chars[i];
        buffer[i + 1] = ch == '.' ? '/' : ch;
      }
      buffer[clazz->dotified->length +1] = ';';
      length = clazz->dotified->length + 2;
    }
    else {
      for (i = 0; i < clazz->dotified->length; ++i) {
        ch = clazz->dotified->chars[i];
        buffer[i] = ch == '.' ? '/' : ch;
      }
      length = clazz->dotified->length;
    }
  }

  typeString = unicode2String(thread, buffer, length);
  releaseMem(buffer);
  
  woempa(1, "Returning typestring %w.\n", typeString);
  
  result = getStringInstance(thread, typeString);
  // typeString is now registered twice, but once is enough ...
  deregisterString(typeString);
  
  return result;
}

w_instance ObjectStreamField_toString(JNIEnv *env, w_instance this) {

  wabort(ABORT_WONKA, "Unimplemented\n"); return NULL;

}
