/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: Character.c,v 1.6 2002/02/20 08:59:08 dfrederickx Exp $
*/

#include <string.h>
#include "chars.h"
#include "checks.h"
#include "clazz.h"
#include "core-classes.h"
#include "descriptor.h"
#include "hashtable.h"
#include "heap.h"
#include "methods.h"
#include "strings.h"
#include "threads.h"
#include "unicode.h"

/*
** Call this from your static initialiser.
*/
void
Character_createTables
( JNIEnv *env, w_instance ClassCharacter
) {
  createCharacterTables();
}

/*
** Get the general category code of a character, as a 2-character String.
*/
w_instance
Character_getCategory
( JNIEnv *env, w_instance ClassCharacter, w_char ch
) {
  return getStringInstance(currentWonkaThread, charToCategory(ch));
}

/*
** Returns true iff the character code is assigned in Unicode.
*/
w_boolean
Character_isDefined
( JNIEnv *env, w_instance ClassCharacter, w_char ch
) {
  return charIsDefined(ch);
}

/*
** Returns true iff ch is a digit according to Java.
*/
w_boolean
Character_isDigit
( JNIEnv *env, w_instance ClassCharacter, w_char ch
) {
  return charIsDigit(ch);
}

/*
** Returns true iff ch is lower-case according to Java.
** Note that for some reason characters 2000 to 2FFF are never considered
** to be lower-case.
*/
w_boolean
Character_isLowerCase
( JNIEnv *env, w_instance ClassCharacter, w_char ch
) {
  return (ch < 0x2000 || ch > 0x2fff) && charIsLower(ch);
}

/*
** Returns true iff ch is upper-case according to Java.
** Note that for some reason characters 2000 to 2FFF are never considered
** to be upper-case.
*/
w_boolean
Character_isUpperCase
( JNIEnv *env, w_instance ClassCharacter, w_char ch
) {
  return (ch < 0x2000 || ch > 0x2fff) && charIsUpper(ch);
}

/*
** Returns true iff ch is title-case according to Java.
** Note that for some reason characters 2000 to 2FFF are never considered
** to be title-case.
*/
w_boolean
Character_isTitleCase
( JNIEnv *env, w_instance ClassCharacter, w_char ch
) {
  return (ch < 0x2000 || ch > 0x2fff) && charIsTitle(ch);
}

/*
** Returns true iff ch is a letter according to Java.
*/
w_boolean
Character_isLetter
( JNIEnv *env, w_instance ClassCharacter, w_char ch
) {
  return charIsLetter(ch);
}

/*
** Returns true iff ch is a formatting character according to Java.
*/
w_boolean
Character_isFormat
( JNIEnv *env, w_instance ClassCharacter, w_char ch
) {
  return charIsFormat(ch);
}

/*
** Returns true iff ch is a whitespace character according to Java.
*/
w_boolean
Character_isWhitespace
( JNIEnv *env, w_instance ClassCharacter, w_char ch
) {
  return charIsWhitespace(ch);
}

/*
** Convert a character to its lower-case equivalent.
** If ch has no lower-case equivalent then ch is returned unchanged.
** Note that a character can have a lower-case equivalent even though
** both isUpperCase() and isTitleCase() return false. <shrug>
*/
w_char
Character_toLowerCase
( JNIEnv *env, w_instance ClassCharacter, w_char ch
) {
  return char2lower(ch);
}

/*
** Convert a character to its upper-case equivalent.
** If ch has no upper-case equivalent then ch is returned unchanged.
** Note that a character can have an upper-case equivalent even though
** isLowerCase() returns false. <shrug>
*/
w_char
Character_toUpperCase
( JNIEnv *env, w_instance ClassCharacter, w_char ch
) {
  return char2upper(ch);
}

/*
** Convert a character to its title-case equivalent.
** If ch has no title-case equivalent then ch is returned unchanged.
** Note that a character can have a lower-case equivalent even though
** isLowerCase() returns false. <shrug>
*/
w_char
Character_toTitleCase
( JNIEnv *env, w_instance ClassCharacter, w_char ch
) {
  return char2title(ch);
}

/*
** Return the decimal digit value associated with ch, or -1 if none.
** Note that function char2digit() implements the JDK 1.1 definition,
** while charToDigitValue() implements JDK 1.4: there seems to be no
** difference in practice (based on the results of the Mauve test).
** We leave the old version in for now, because it uses less static data.
*/
w_int
Character_digitValue
( JNIEnv *env, w_instance ClassCharacter, w_char ch
) {
  return char2digit(ch);
// Or maybe:
// return charToDigitValue(ch);
}

/*
** Return the numeric value associated with a character, or -1 if it has none,
** or -2 if the numeric value is not a nonnegative integer (e.g. it is
** negative or fractional).
*/
w_int
Character_numericValue
( JNIEnv *env, w_instance ClassCharacter, w_char ch
) {
  return charToNumericValue(ch);
}

w_instance
Character_getWrappedClass(JNIEnv *env, w_instance thisClass) {

  return primitive2classInstance[P_char];
  
}
