/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: Class.c,v 1.21 2002/03/26 10:44:14 gray Exp $
*/

#include <string.h>

#include "arrays.h"
#include "checks.h"
#include "clazz.h"
#include "core-classes.h"
#include "descriptor.h"
#include "exception.h"
#include "fields.h"
#include "fifo.h"
#include "heap.h"
#include "interpreter.h"
#include "loading.h"
#include "methods.h"
#include "strings.h"
#include "stack.h"
#include "threads.h"
#include "ts-mem.h"

#define PUBLIC   0 //this is the value of Member.PUBLIC
#define DECLARED 1 //this is the value of Member.DECLARED

w_string
parameterArray2sig(w_thread thread, w_instance parameterArray) {

  w_instance Class;
  w_clazz clazz;
  w_size  nparms;
  w_size  i;
  w_size  j;
  w_int   primitive_index;
  w_int   buffersize;
  w_char *buffer;
  w_char *bufptr;
  w_char  ch;
  w_string string;
  
  /*
  ** Calculate the length of the full name.  We start with 2 for the
  ** parentheses around the parameter list.  Then for each element of
  ** parameterArray we add:
  **   - for a primitive type, 1 (e.g. Z)
  **   - for a non-array reference type, the length of clazz->name plus
  **     2 for the enclosing L...; (e.g. Ljava/lang/String;).
  **   - for an array reference type, the length of clazz->name (if it
  **     is an array of references then the L...; is already included,
  **     e.g. [[I or [Ljava/lang/Object;.
  */
  buffersize = 2;
  nparms = parameterArray[F_Array_length];

  for (i = 0; i < nparms; i++) {
    Class = (w_instance)parameterArray[F_Array_data + i];
    clazz = Class2clazz(Class);
    primitive_index = isPrimitiveClazz(clazz);
    if (primitive_index) {
      buffersize += 1;
    }
    else {
      buffersize += clazz->dotified->length;
      if (! clazz->aclazz) {
        buffersize += 2;
      }
    }
  }

  buffer = allocMem_system(buffersize * sizeof(w_char));

  buffer[0] = '(';
  bufptr = buffer + 1;
  /*
  ** Copy the parameters.
  */
  for (i = 0; i < nparms; i++) {
    Class = (w_instance)parameterArray[F_Array_data + i];
    clazz = Class2clazz(Class);
    primitive_index = isPrimitiveClazz(clazz);
    if (primitive_index) {
      *bufptr++ = primitive2char[primitive_index];
    }
    else {
      if (! clazz->aclazz) {
        *bufptr++ = 'L';
      }
      for (j = 0; j < clazz->dotified->length; ++j) {
        ch = clazz->dotified->chars[j];
        *bufptr++ = ch == '.' ? '/' : ch;
      }
      if (! clazz->aclazz) {
        *bufptr++ = ';';
      }
    }
  }
  *bufptr++ = ')';


  /*
  ** Now parse it into a wonka string.
  */

  string = unicode2String(thread, buffer, (w_size)(bufptr-buffer));

  releaseMem(buffer);
  
  woempa(9, "signature is '%w'.\n", string);

  return string;  
}
 
void Class_finalize(JNIEnv *env, w_instance this) {
  w_clazz clazz = Class2clazz(this);
  if (clazz) {
    woempa(8, "Destroying Class instance %k\n",Class2clazz(this));
    destroyClazz(clazz);
  }
  else {
    woempa(8, "Destroying anonymous Class instance (? ? ?)\n");
  }
}

w_instance Class_getName(JNIEnv *env, w_instance Class) {

  return getStringInstance(JNIEnv2w_thread(env), Class2clazz(Class)->dotified);

}

w_boolean Class_isPrimitive(JNIEnv *env, w_instance Class) {

  return isSet(Class2clazz(Class)->flags, CLAZZ_IS_PRIMITIVE);

}

w_boolean Class_isArray(JNIEnv *env, w_instance Class) {

  return (Class2clazz(Class)->aclazz != NULL);

}

static inline int is_interface(w_instance Class) {

  return isSet(Class2clazz(Class)->flags, ACC_INTERFACE);

}

static inline w_clazz superclass_of(w_instance Class) {

  return Class2clazz(Class)->superClazz;

}

w_boolean Class_isInterface(JNIEnv *env, w_instance Class) {

  return is_interface(Class);

}

w_boolean Class_isInstance(JNIEnv *env, w_instance this, w_instance theObject) {

  if (theObject) {
    w_clazz this_clazz = Class2clazz(this);
    w_clazz that_clazz = instance2clazz(theObject);

    return isAssignmentCompatible(that_clazz, this_clazz); 

  }

  return WONKA_FALSE;
}

w_boolean Class_isAssignableFrom(JNIEnv *env, w_instance this, w_instance theClass) {

  w_thread thread = JNIEnv2w_thread(env);

  if (theClass) {
    w_clazz this_clazz = Class2clazz(this);
    w_clazz that_clazz = Class2clazz(theClass);

    if (isSuperClass(this_clazz, that_clazz)) {
      return WONKA_TRUE;
    }
    else if (isSet(this_clazz->flags, ACC_INTERFACE)) {
      return implementsInterface(that_clazz, this_clazz);
    }

    return WONKA_FALSE;
  }
  
  throwException(thread,clazzNullPointerException,NULL);

  return WONKA_FALSE;
}

w_instance Class_getSuperclass(JNIEnv *env, w_instance Class) {

  w_instance Super = NULL;
  w_clazz clazz = Class2clazz(Class);

  if (! clazz) {

    return NULL;

  }

  woempa(1, "instance of '%k'.\n", clazz);

  if (isSet(clazz->flags, ACC_INTERFACE)) {
    woempa(1, "--> is interface; returning null instance.\n");
  }
  else if (clazz == clazzObject) {
    woempa(1, "--> is Object; returning null instance.\n");
  }
  else if (isSet(clazz->flags, CLAZZ_IS_PRIMITIVE)) {
    woempa(1, "--> is primitive; returning null instance.\n");
  }
  else if (clazz->aclazz) {
    woempa(1, "--> is array; returning Object Class.\n");
    Super = clazzObject->Class;
  }
  else {      
    woempa(1, "--> super class is '%k'.\n", clazz->superClazz);
    Super = clazz2Class(clazz->superClazz);
  }

  return Super;
  
}

w_instance Class_getInterfaces(JNIEnv *env, w_instance this) {
  w_thread thread = JNIEnv2w_thread(env);
  w_int   i;
  w_instance Array;
  w_instance exception;
  w_clazz clazz = Class2clazz(this);
  w_clazz interfaze;
  w_int   length;

  woempa(1, "getInterfaces of instance of %k, %d interfaces.\n", clazz, clazz->numInterfaces);

  exception = NULL;

  length = clazz->numInterfaces;
  Array = allocArrayInstance(thread, clazzArrayOf_Class, 1, &length);

  if (Array) {
    for (i = 0; i < (w_int)clazz->numInterfaces; i++) {
      interfaze = clazz->interfaces[i];
      if (interfaze->Class == NULL) {
        attachClassInstance(interfaze);
      }
      woempa(1, "--> interface %d = %k.\n", i, interfaze);
      setArrayReferenceField(Array, clazz2Class(interfaze), i);
    }
  }

  return Array;

}

w_instance Class_newInstance0(JNIEnv *env, w_instance this) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz = Class2clazz(this);
  w_clazz calling_clazz;
  w_instance newInstance = NULL;
  w_instance exception = NULL;
  w_frame  bottom;

  exception = classMustBeInitialized(clazz, thread);
  if (exception) {
    setLocalException(thread,exception);

    return NULL;

  }

  if (clazz->defaultInit==NULL) {

    woempa(8, "%k has no default initializer, throwing InstantiationException\n", clazz);
    exception = createExceptionInstance(thread,clazzInstantiationException,clazz->dotified);
    setLocalException(thread,exception);

    return NULL;

  }


  calling_clazz = getCallingClazz(thread);
  if (!isAllowedToCall(calling_clazz,clazz->defaultInit, WONKA_FALSE)) {
    exception = createExceptionInstance(thread,clazzIllegalAccessException, clazz->dotified);
  }


  if (exception == NULL) {
    newInstance = allocInstance(JNIEnv2w_thread(env), clazz);
  }

  if (exception == NULL) {
    bottom = allocBottomFrame(thread, clazz->defaultInit);
    if (bottom) {
      setFlag(bottom->flags, FRAME_REFLECTION);
      bottom->locals[0] = (w_word)newInstance;
      handleSynchronization((w_frame)bottom, clazz->defaultInit);
      exception = runThread(thread, bottom);
      releaseBottomFrame(bottom);
    }
    else {
      woempa(8, "Couldn't allocBottomFrame.\n");

      return NULL;
    }
  }

  if (exception) {
    woempa(8, "%M threw %k.\n", clazz->defaultInit, instance2clazz(exception));
    setLocalException(thread,exception);

    return NULL;
  }
  else {
    woempa(1,"created new instance of %k\n", clazz);

    return newInstance;
  }

}

w_instance Class_forName_S(JNIEnv *env, w_instance thisClass, w_instance theString) {

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz;
  w_instance exception = NULL;
  w_string classname;
  w_clazz  calling_clazz;
  w_instance loader;

  if(theString == NULL){
    throwException(thread,clazzNullPointerException,NULL);
    return NULL;
  }

  classname = String2string(theString);
  woempa(1, "called with string %w.\n", classname);

  calling_clazz = getCallingClazz(thread);
  loader = clazz2loader(calling_clazz);
//  clazz = namedClassMustBeLoaded(loader, classname, &exception);
  clazz = mustBeLoaded(identifyClazz(classname, loader), &exception);
  if (!clazz) {
    throwException(thread,clazzClassNotFoundException,classname);

    return NULL;

  } 

  if (!exception) {
    exception = classMustBeInitialized(clazz, thread);
  }

  if (exception) {
    setLocalException(thread,exception);

    return NULL;

  }

  return clazz2Class(clazz);
}

w_instance Class_forName_SZCL(JNIEnv *env, w_instance thisClass, w_instance theString, w_boolean initialize, w_instance theClassLoader) {

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz;
  w_instance exception = NULL;
  w_string classname;

  if(theString == NULL){
    throwException(thread,clazzNullPointerException,NULL);
    return NULL;
  }

  classname = String2string(theString);
  woempa(1, "called with string %w initialize=%s classloader %k@%p.\n", classname,initialize?"true":"false", instance2clazz(theClassLoader),theClassLoader);

//  clazz = namedClassMustBeLoaded(theClassLoader, classname, &exception);
  clazz = mustBeLoaded(identifyClazz(classname, theClassLoader), &exception);
  if (!clazz) {
    throwException(thread,clazzClassNotFoundException,classname);

    return NULL;

  } 

  if (!exception && initialize) {
    exception = classMustBeInitialized(clazz, thread);
  }

  if (exception) {
    setLocalException(thread,exception);

    return NULL;

  }

/* not needed?
  if (clazz->Class == NULL) {
    attachClassInstance(clazz,thread);
  }
*/

  return clazz2Class(clazz);
}

w_instance
Class_get_constructors
( JNIEnv *env, w_instance thisClass, w_int mtype
) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz = Class2clazz(thisClass);
  w_int i;
  w_int numMethods = 0;
  w_int numRelevantConstructors;
  w_method method;
  w_instance Constructor;
  w_instance Array;
  w_instance exception;

  if (clazz) numMethods = clazz->numMethods;

  numRelevantConstructors = 0;
  for (i = 0; i < numMethods; i++) {
    method = clazz->methods[i];
    if (w_string_equals(method->name,string_angle_brackets_init)) {
      numRelevantConstructors += 
        (mtype==PUBLIC && isSet(method->flags, ACC_PUBLIC))
        || (mtype==DECLARED && clazz->methods[i]->clazz == clazz);
    }
  }

  exception = NULL;

  Array = allocArrayInstance(thread, clazzArrayOf_Constructor, 1, &numRelevantConstructors);

  if (Array) {
    numRelevantConstructors = 0;
    for (i = 0; i < numMethods; i++) {
      method = clazz->methods[i];
      if (w_string_equals(method->name,string_angle_brackets_init)) {
        if ((mtype==PUBLIC && isSet(method->flags, ACC_PUBLIC))
         || (mtype==DECLARED && clazz->methods[i]->clazz == clazz)
          ) {
          Constructor = allocInstance(JNIEnv2w_thread(env), clazzConstructor);
          if (!Constructor) {
            break;
          }
          Constructor[F_Constructor_wotsit] = (w_word)method;
          setArrayReferenceField(Array, Constructor, numRelevantConstructors);
          numRelevantConstructors += 1;
        }
      }
    }
  }

  return Array;

}

w_instance
Class_get_fields ( JNIEnv *env, w_instance thisClass, w_int mtype) {

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz = Class2clazz(thisClass);
  w_int relevant;
  w_size i;
  w_int numRelevantFields;
  w_field field;
  w_clazz current_clazz;
  w_instance Field;
  w_instance Array;
  w_instance exception;

  /*
  ** Find the number of appropriate fields first.  
  */

  woempa(1,"looking for %s fields of %k\n",mtype==PUBLIC?"public":"declared", clazz);
  numRelevantFields = 0;
  current_clazz=clazz;
  while (current_clazz) {
    for (i = 0; i < current_clazz->numFields; i++) {
      field = &current_clazz->fields[i];
      woempa(1,"%02d: field %w is %s, declared in %k\n",i,NM(field),isSet(field->flags, ACC_PUBLIC)?"public":"non-public", field->declaring_clazz);
      relevant = (mtype==PUBLIC && isSet(field->flags, ACC_PUBLIC))
          || (mtype==DECLARED && current_clazz->fields[i].declaring_clazz == current_clazz);
      woempa(1,"field[%d] %w is %srelevant\n",i,NM(field),relevant?"":"ir");
      numRelevantFields += relevant;
    }
    woempa(1,"found %d suitable fields in %k\n", numRelevantFields,  current_clazz);
    if (mtype==DECLARED) break;
    if (isSet(current_clazz->flags,ACC_INTERFACE)) {
      current_clazz = current_clazz->interfaces[0];
    }
    else {
      current_clazz = current_clazz->superClazz;
    }
  }
  
  exception = NULL;

  Array = allocArrayInstance(thread, clazzArrayOf_Field, 1, &numRelevantFields);

  if (Array) {
    numRelevantFields = 0;
    current_clazz=clazz;
    while (current_clazz) {
      for (i = 0; i < current_clazz->numFields; i++) {
        field = &current_clazz->fields[i];
        if ((mtype==PUBLIC && isSet(field->flags, ACC_PUBLIC))
         || (mtype==DECLARED && current_clazz->fields[i].declaring_clazz == current_clazz)
          ) {
          Field = allocInstance(JNIEnv2w_thread(env), clazzField);
          if (Field==NULL) {
            break;
          }
          Field[F_Field_wotsit] = (w_word)field;
          setArrayReferenceField(Array, Field, numRelevantFields);
          numRelevantFields += 1;
        }
      }
      if (mtype==DECLARED) break;
      if (isSet(current_clazz->flags,ACC_INTERFACE)) {
        current_clazz = current_clazz->interfaces[0];
      }
      else {
        current_clazz = current_clazz->superClazz;
      }
    }
  }

  return Array;

}

w_instance
Class_get_methods
( JNIEnv *env, w_instance thisClass, w_int mtype
) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz = Class2clazz(thisClass);
  w_int i;
  w_int numMethods = 0;
  w_word numRelevantMethods;
  w_method method;
  w_instance Method;
  w_instance Array;
  w_instance exception = NULL;
  w_fifo relevantMethods;

  if (clazz) {
    numMethods = clazz->numMethods;
    exception = classMustBePrepared(clazz, thread);
  }

  if (!exception) {
    exception = classMustBeInitialized(clazzMethod, thread);
  }

  if (exception) {
    setLocalException(thread,exception);

    return NULL;
  }


  /*
  ** Find the number of appropriate methods first. Leave out the 
  ** <clinit> and <init> methods for the class.
  */

  woempa(1,"Class %k type %s\n", clazz,mtype==PUBLIC?"public":"declared");
  relevantMethods = allocFifo(W_Realm_system, (w_size)numMethods);
  numRelevantMethods = 0;
  for (i = 0; i < numMethods; i++) {
    method = clazz->methods[i];
    if (method->name->chars[0]!='<' &&
         (  (mtype==PUBLIC && isSet(method->flags, ACC_PUBLIC))
         || (mtype==DECLARED && clazz->methods[i]->clazz == clazz)
         )
       ) {
      ++numRelevantMethods;
      woempa(1,"Method %d: %m\n",numRelevantMethods,method);
      if (putFifo(method,relevantMethods) < 0) {
        throwOutOfMemoryError(thread);

        return NULL;

      }
    }
    else {
      woempa(1,"Method %m is not relevant\n",method);
    }
  }
  woempa(1,"Class %k has %d relevant methods\n", clazz,numRelevantMethods);
  Array = allocArrayInstance(thread, clazzArrayOf_Method, 1, &numRelevantMethods);

  if (Array) {
    for (i = 0; i < (w_int)numRelevantMethods; i++) {
      method = getFifo(relevantMethods);
      Method = allocInstance(thread, clazzMethod);
      if (Method==NULL) {
        break;
      }
      woempa(1,"Method %d: %m\n",i+1,method);
      Method[F_Method_wotsit] = (w_word)method;
      setArrayReferenceField(Array, Method, i);
    }
    releaseFifo(relevantMethods);
  }

  return Array;

}

w_instance Class_get_one_constructor(JNIEnv *env, w_instance thisClass, w_instance AParameters, w_int mtype) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz = Class2clazz(thisClass);
  w_string  sig;
  w_instance exception;
  w_instance Constructor;
  w_method constructor;

  if(AParameters == NULL){
    w_int length = 0;
    w_clazz clazzClassArray = descArrayClassMustBeLoaded(NULL, clazzClass->adesc,&exception);

    // TODO: check exception ...

    AParameters = allocArrayInstance(thread, clazzClassArray, 1, &length);
    if(AParameters == NULL){
      throwOutOfMemoryError(thread);
      return NULL;
    }
  }

  sig = parameterArray2sig(thread, AParameters);

  exception = classMustBePrepared(clazz, thread);
  // TODO: check exception ...
  woempa(1,"Looking for %w%w in %k\n",string_angle_brackets_init, sig, clazz);
  constructor = getMethod_name_sig(clazz, string_angle_brackets_init, sig);
  if (constructor) {
    woempa(1,"Found %m\n",constructor);
    if (mtype==PUBLIC && isNotSet(constructor->flags, ACC_PUBLIC)) {
      woempa(9,"Aw shucks: constructor is not public\n");
      constructor = NULL;
    }
    else if (mtype==DECLARED && constructor->clazz!=clazz) {
      woempa(1,"Aw shucks: constructor is declared in %k not %k\n", constructor->clazz, clazz);
      constructor = NULL;
    }
  }

  if (constructor) {
    Constructor = allocInstance(JNIEnv2w_thread(env), clazzConstructor);
    if (Constructor==NULL) {
 
     return NULL;

    }
    woempa(1,"setting field[%d] of Constructor @ %p to %p\n",F_Constructor_wotsit,Constructor,constructor);
    Constructor[F_Constructor_wotsit] = (w_word)constructor;
    return Constructor;
  }
  else {
    woempa(9, "Did not find correct constructor!\n");
    throwException(thread,clazzNoSuchMethodException,NULL);

    return NULL;

  }
}

w_field seekFieldInClazz (w_clazz clazz, w_string name, int mtype) {
  w_field  candidate;
  w_field  result = NULL;
  w_size    i;

/*
** 1. Seek field declared in class itself
*/
  woempa(1,"looking for %s field %w in class %k\n",mtype==DECLARED?"declared":"public",name, clazz);
  classMustBeResolved(clazz, currentWonkaThread);  

  for (i = 0; i < clazz->numFields; i++) {
    candidate = &clazz->fields[i];
    woempa(1,"looking at field %w: %s declared in class %k\n",NM(candidate),isSet(candidate->flags, ACC_PUBLIC)?"public":"", clazz);
    if ((mtype==DECLARED || isSet(candidate->flags, ACC_PUBLIC))
        && (mtype==PUBLIC || candidate->declaring_clazz == clazz)
        && candidate->name==name
      ) {
      result = candidate;
      break;
    }
  }

/*
** 2. Seek field declared in the superinterfaces
*/

  for (i=0; !result && i<clazz->numInterfaces; i++) {
    result = seekFieldInClazz(clazz->interfaces[i],name,mtype);
  }

/*
** 3. Seek field declared in the superclass
*/

  if (!result && clazz->superClazz) {
    result = seekFieldInClazz(clazz->superClazz,name,mtype);
  }

  return result;
}

w_instance
Class_get_one_field
( JNIEnv *env, w_instance thisClass, w_instance fieldNameString, w_int mtype
) {
  w_thread   thread = JNIEnv2w_thread(env);
  w_clazz    clazz = Class2clazz(thisClass);
  w_string   fieldName;
  w_instance exception;
  w_field    field;
  w_instance Field;

  if(fieldNameString == NULL){
    throwException(thread,clazzNoSuchFieldException,NULL);
    return NULL;
  }


  fieldName = String2string(fieldNameString);
//  field = seekFieldInClazz(clazz,fieldName,mtype);
  exception = classMustBePrepared(clazz, thread);
  // TODO: check exception ...
  woempa(1,"Looking for %w in %k\n",fieldName, clazz);
  field = getField(clazz, fieldName);

  if (field) {
    woempa(1,"Found %w\n",field->name);
    if (mtype==PUBLIC && isNotSet(field->flags, ACC_PUBLIC)) {
      woempa(9,"Aw shucks: field is not public\n");
      field = NULL;
    }
    else if (mtype==DECLARED && field->declaring_clazz != clazz) {
      woempa(9,"Aw shucks: field is declared in %k not %k\n", field->declaring_clazz, clazz);
      field = NULL;
    }
  }

  if (field) {
    Field = allocInstance(JNIEnv2w_thread(env), clazzField);
    if (Field==NULL) {

      return NULL;

    }
    else {
      Field[F_Field_wotsit] = (w_word)field;

      return Field;

    }
  }
  
  woempa(9,"Couldn't find field %w in class %k\n",fieldName, clazz);
  throwException(thread,clazzNoSuchFieldException,fieldName);

  return NULL;
}

w_instance
Class_get_one_method
( JNIEnv *env, w_instance thisClass, w_instance methodNameString, w_instance AParameterTypes, w_int mtype
) {
  w_thread   thread = JNIEnv2w_thread(env);
  w_clazz    clazz = Class2clazz(thisClass);
  w_string   methodName = String2string(methodNameString);
  w_string   sig;
  w_instance exception;
  w_method   method;
  w_instance Method;

  if(AParameterTypes == NULL){
    w_int length = 0;
    w_clazz clazzClassArray = descArrayClassMustBeLoaded(NULL, clazzClass->adesc,&exception);

    // TODO: check exception ...

    AParameterTypes = allocArrayInstance(thread, clazzClassArray, 1, &length);
    if(AParameterTypes == NULL){
      throwOutOfMemoryError(thread);
      return NULL;
    }
  }

  sig = parameterArray2sig(thread, AParameterTypes);

  exception = classMustBePrepared(clazz, thread);
  // TODO: check exception ...
  woempa(1,"Looking for %w%w in %k\n",methodName, sig,  clazz);
  method = getMethod_name_sig(clazz, methodName, sig);
  if (method) {
    woempa(1,"Found %m\n",method);
    if (mtype==PUBLIC && isNotSet(method->flags, ACC_PUBLIC)) {
      woempa(9,"Aw shucks: method is not public\n");
      method = NULL;
    }
    else if (mtype==DECLARED && method->clazz!=clazz) {
      woempa(9,"Aw shucks: method is declared in %k not %k\n", method->clazz, clazz);
      method = NULL;
    }
  }

  if (method) {
    Method = allocInstance(JNIEnv2w_thread(env), clazzMethod);
    if (Method==NULL) {
 
     return NULL;

    }
    woempa(1,"setting field[%d] of Method @ %p to %p\n",F_Method_wotsit,Method,method);
    Method[F_Method_wotsit] = (w_word)method;
    return Method;
  }
  else {
    woempa(9, "Did not find correct method!\n");
    throwException(thread,clazzNoSuchMethodException,methodName);

    return NULL;

  }
}

w_instance Class_getComponentType(JNIEnv *env, w_instance thisClass) {
  w_clazz clazz = Class2clazz(thisClass);
  w_instance result = NULL;

  if (clazz && clazz->aclazz) {
    result = clazz2Class(clazz->aclazz->componentClazz);
  }

  if (result) {
    woempa(1, "Result is %k.\n", ((w_clazz)result[F_Class_wotsit]));
  }
  return result;
    
}

w_int Class_getModifiers(JNIEnv *env, w_instance Class) {
  w_word flags = Class2clazz(Class)->flags;

  woempa(1, "Class %k has modifiers %s %s %s %s %s %s (0x%08x).\n", Class2clazz(Class), isSet(flags,ACC_PUBLIC)?"public":"",isSet(flags,ACC_PRIVATE)?"private":"",isSet(flags,ACC_PROTECTED)?"protected":"",isSet(flags,ACC_ABSTRACT)?"abstract":"",isSet(flags,ACC_FINAL)?"final":"",isSet(flags,ACC_INTERFACE)?"interface":"",flags & (ACC_PUBLIC | ACC_PRIVATE | ACC_PROTECTED | ACC_ABSTRACT | ACC_FINAL));

  return (w_int)(flags & (ACC_PUBLIC | ACC_PRIVATE | ACC_PROTECTED | ACC_ABSTRACT | ACC_FINAL | ACC_INTERFACE));
  
}
/*
** Returns the class which called Class.getCallingClass(). 
*/
w_instance
Class_getCallingClass 
( JNIEnv *env, w_instance thisClassLoader
) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz = getCallingClazz(thread);
  
  return clazz2Class(clazz);
}

/*
** Returns the class loader of the class which called Class.getCallingClassLoader(). 
w_instance
Class_getCallingClassLoader 
( JNIEnv *env, w_instance thisClassLoader
) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz = getCallingClazz(thread);
  
  return clazz->loader;
}
*/

