/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: ClassLoader.c,v 1.11 2002/03/26 10:44:14 gray Exp $
*/

#include <string.h>
#include "arrays.h"
#include "bar.h"
#include "clazz.h"
#include "core-classes.h"
#include "fields.h"
#include "exception.h"
#include "hashtable.h"
#include "heap.h"
#include "list.h"
#include "locks.h"
#include "loading.h"
#include "methods.h"
#include "oswald.h"
#include "stack.h"
#include "strings.h"
#include "ts-mem.h"
#include "vfs.h"

extern Wonka_InitArgs *system_vm_args;

#define CLASSLOADER_HASHTABLE_SIZE       89

void ClassLoader_create(JNIEnv *env, w_instance ClassLoader) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz = instance2clazz(ClassLoader);
  w_hashtable class_hashtable;
  char   * label_buf;

  label_buf = allocMem(JNIEnv2w_thread(env), (w_size)(clazz->dotified->length + 36));
  x_snprintf(label_buf, clazz->dotified->length + 36, "hashtable:%w-classes", clazz->dotified);
  class_hashtable = ht_create(thread2realm(thread), label_buf, CLASSLOADER_HASHTABLE_SIZE, ht_stringHash, NULL, 0, 0);
  ClassLoader[F_ClassLoader_hashtable] = (w_word)class_hashtable;
  woempa(1, "Created new ClassLoader %k@%p. (%p)\n", clazz,ClassLoader);
}


w_instance ClassLoader_defineClass(JNIEnv *env, w_instance This, w_instance nameString, w_instance Data, w_int offset, w_int length, w_instance pd) {

  w_thread thread = JNIEnv2w_thread(env);
  w_byte    *data = instance2Array_byte(Data);
  x_monitor  class_loading_lock = loader2monitor(This);
  w_clazz    clazz;
  w_instance theClass;
  w_BAR bar;
  w_instance exception;
  w_string name;
  
  woempa(1,"installing %w from %p (thread %w)\n",String2string(nameString),data,NM(thread));

  bar.buffer = data + offset;
  bar.length = length;
  bar.current = 0;

  woempa(1,"Aquiring class loading lock @ %p for %w\n",class_loading_lock,NM(thread));
  x_monitor_enter(class_loading_lock, x_eternal);

  exception = NULL;

  if (nameString) {
    name = String2string(nameString);
  }
  else {
    name = NULL;
  }

  clazz = createClazz(thread, name, &bar, This, &exception);

  x_monitor_exit(class_loading_lock);
  
  if (exception) {
    setLocalException(thread,exception);

    return nullInstance;

  }

  theClass = clazz2Class(clazz);
  setReferenceField(theClass,pd,F_Class_domain);
  setReferenceField(theClass,This,F_Class_loader);

  return theClass;

}

void
ClassLoader_resolveClass
( JNIEnv *env, w_instance This, w_instance theClass
) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz = Class2clazz(theClass);

  if (theClass) {
    w_instance exception = classMustBeResolved(clazz, thread);
    if (exception) {
      setLocalException(thread,exception);
    }
  }
  else {
    throwException(thread, clazzNullPointerException, NULL);
  }
}

w_instance ClassLoader_findLoadedClass(JNIEnv *env, w_instance This, w_instance nameString) {

  w_clazz clazz;
  w_string name;

  if (!isNullInstance(nameString)) {
    name = String2string(nameString);
    woempa(1, "called with string %w.\n", name);

    clazz = seekClazzByName(name, loader2hashtable(This));

    if (clazz) {
      return clazz2Class(clazz);
    } 
  }

  return nullInstance;

}

w_instance ClassLoader_findBootstrapClass(JNIEnv *env, w_instance This, w_instance nameString) {

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz;
  w_string name;
  w_instance exception = NULL;

  if (isNullInstance(nameString)) {
    throwException_C(thread, clazzNullPointerException, (char*)"name is null");

    return nullInstance;

  }

  name = String2string(nameString);
  woempa(1, "called with string %w.\n", name);

  clazz = seekClazzByName(name, system_class_hashtable);

  if (!clazz) {
    x_monitor_enter(&system_class_hashtable->monitor, x_eternal);
    clazz = loadBootstrapClass(name,&exception);
    x_monitor_exit(&system_class_hashtable->monitor);
  }

  if (instance2clazz(exception)==clazzNoClassDefFoundError) {
    throwException(thread, clazzClassNotFoundException, name);
  }
  else if (exception) {
    setLocalException(thread,exception);
  }

  return clazz ? clazz2Class(clazz) : NULL;
}


void
ClassLoader_setSigners
( JNIEnv *env, w_instance This,
  w_instance theClass, w_instance theObjectArray
) {
  // TODO 
}

w_instance ClassLoader_loadClass(JNIEnv *env, w_instance Class, w_instance nameString, w_boolean resolve) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz;
  w_string name = String2string(nameString);
  w_instance exception;

  woempa(1, "Loading class %w.\n", name);
  exception = NULL;
  
  clazz = namedClassMustBeLoaded(nullInstance, name, &exception);
//  clazz = mustBeLoaded(identifyClazz(name, NULL), &exception);

  if (exception) {
    setLocalException(thread,exception);

    return nullInstance;

  }
  else if (clazz) {
    if (clazz->Class == NULL) {
      attachClassInstance(clazz);  
    }

    if (resolve) {
      classMustBeResolved(clazz, thread);
    }

    woempa(1, "--> clazz at %p.\n", clazz);
    return clazz->Class;
  }
  else {
    woempa(1, "--> NOT found! Throwing ClassNotFoundException.\n");
    throwException(thread, clazzClassNotFoundException, NULL);
    return nullInstance;
  }
}

  
w_instance
ClassLoader_get_system_resource_as_bytes
( JNIEnv *env, w_instance classClassLoader,
  w_instance nameString
) {
  w_string name = String2string(nameString);
  w_instance theByteArray = nullInstance;
  char             *filename_ASCII;
  int              fd;
  struct vfs_STAT  statbuf;
  w_size           file_size;

  filename_ASCII = allocMem_system(name->length + 10);
  x_snprintf(filename_ASCII, name->length + 10, "system/%w" , name);
  woempa(1,"Filename string %w, opening file %s\n",name,filename_ASCII);
  if(vfs_stat(filename_ASCII, &statbuf) != -1) { 
// TODO: check is not a directory ...
    file_size = statbuf.st_size;
    woempa(1, "%s is %d bytes\n", filename_ASCII, file_size);
    theByteArray = allocArrayInstance(W_ThreadGroup_system->manager, atype2clazz[P_byte], 1, &file_size);

    fd = vfs_open(filename_ASCII, VFS_O_RDONLY, 0);
    vfs_read(fd, instance2Array_byte(theByteArray), file_size);
    vfs_close(fd);
  } 
  else {
    woempa(9,"Couldn't open file %s\n",filename_ASCII);
  }

  releaseMem(filename_ASCII);

  return theByteArray;
}

/*
** Returns the class loader of the class which called ClassLoader.getCallingClassLoader().
**
** This logic is also needed by the java.util.ResourceBundle Class's getBundle(...)
** it will call this method straigth from java so w_instance pointer will not always be the Class of ClassLoader
*/
w_instance
ClassLoader_getCallingClassLoader 
( JNIEnv *env, w_instance thisClassLoader
) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz = getCallingClazz(thread);
  
  return clazz->loader;
}

w_instance ClassLoader_getCommandLineClasspath(JNIEnv *env, w_instance class) {
  w_thread thread = JNIEnv2w_thread(env);
  w_instance Result;
  w_string result;
  char    *envcp;
 
  envcp = getenv("WONKA_CLASSPATH");
  Result = allocInstance(thread, clazzString);
    
  if (Result) {
    if(system_vm_args->classpath) {
      result = cstring2String(thread, system_vm_args->classpath, strlen(system_vm_args->classpath));
    }
    else {
      /* HACK! HACK! HACK! */
      result = cstring2String(thread, "./:/system/:/user/", 18);
    }
    attachString(Result, result);
  }


  return Result;

}
