/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: Runtime.c,v 1.5 2002/03/26 10:44:15 gray Exp $
*/

#include "core-classes.h"
#include "exception.h"
#include "hashtable.h"
#include "realms.h"
#include "ts-mem.h"
#include "strings.h"
#include "stack.h"
#include "clazz.h"
#include "misc.h"
#include "methods.h"
#include "wonka.h"
#include "arrays.h"
#include "exec.h"

void Runtime_static_exit0 (JNIEnv* env, w_instance thisClass, w_int exitcode) {
#ifdef LINUX
  exit(exitcode);
#else
  wabort(ABORT_WONKA, "exit(%d)\n",exitcode);
#endif
}

typedef void (*w_loadlib)(w_thread,w_instance);

void Runtime_static_initLibraries(JNIEnv* env, w_instance thisClass) {
}

void Runtime_loadLibrary0 (JNIEnv* env, w_instance thisRuntime, w_instance libnameString) {
  w_thread thread = JNIEnv2w_thread(env);
  w_trace    trace = createStackTrace(thread->top);
  w_trace    current;
  w_instance currentClass;
  w_instance loader = NULL;
  w_string   libname = String2string(libnameString);
  w_ubyte    *name;
  w_ubyte    *dummy = NULL;
  w_int      i;

  current = trace;
  while (current) {
    currentClass = clazz2Class(current->method->clazz);
    if (currentClass[F_Class_loader]) {
      loader = (w_instance)currentClass[F_Class_loader];
      break;
    }
    current = current->previous;
  }
  destroyStackTrace(trace);

  name = allocMem_system(libname->length + 1);
 
  dummy = (w_ubyte *)name;
  for (i = 0; i < (w_int)libname->length; ++i) {
     (*dummy) = libname->chars[i];
     dummy++;
  }
  (*dummy) = '\0';

  if(loadModule(name)) {
    woempa(7, "Successfully loaded library %w\n", libname);
  }
  else {
    woempa(9, "Could not find library %w\n",libname);
    throwException(thread, clazzLinkageError, libname);
  }

  releaseMem(name);
}

w_long
Runtime_totalMemory (JNIEnv* env, w_instance thisRuntime) {
  return (w_long)(x_mem_total());
}

w_long
Runtime_freeMemory (JNIEnv* env, w_instance thisRuntime) {
  return (w_long)(x_mem_avail());
}


static w_void string2c(w_string string, char *c_string) {
  if (string->length > 0) {
    char *dummy = (char *)c_string;
    int i = 0;
        
    for (i = 0; i < (w_int)string->length; ++i) {
       (*dummy) = string->chars[i];
       dummy++;
    }
    (*dummy) = '\0';
  }
  else {
    *c_string = '\0';
  }
}

w_instance Runtime_exec0 (JNIEnv* jnienv, w_instance thisRuntime, w_instance cmdArray, w_instance envArray) {
  w_thread    thread = JNIEnv2w_thread(jnienv);
  w_int       cmdlength = 0; 
  w_int       envlength = 0; 
  w_int       i;
  w_ubyte     **cmd = NULL;
  w_ubyte     **env = NULL;
  w_string    string;
  w_instance  process = nullInstance;
  w_int       pid, fd_in, fd_out, fd_err;

  /*
  ** Check if there's a command to execute.
  */
  
  if(cmdArray == NULL || cmdArray[F_Array_length] == 0) {
    return process;
  }

  /*
  ** Turn the cmd String array into a plain old char array (Last element should be NULL).
  */

  cmdlength = cmdArray[F_Array_length];
  cmd = x_mem_calloc((cmdlength + 1) * sizeof(w_word));
  for(i = 0; i < cmdlength; i++) {
    string = String2string((w_instance)cmdArray[F_Array_data + i]);
    cmd[i] = x_mem_alloc(string->length + 2);
    string2c(string, cmd[i]);
  }

  /*
  ** Turn the env String array into a plain old char array (Last element should be NULL).
  ** If envArray is NULL, there's no environment.
  */
  
  if(envArray != NULL) {
    envlength = envArray[F_Array_length];
    env = x_mem_calloc((envlength + 1) * sizeof(w_word));
    for(i = 0; i < envlength; i++) {
      string = String2string((w_instance)envArray[F_Array_data + i]);
      env[i] = x_mem_alloc(string->length + 2);
      string2c(string, env[i]);
    }
  }

  /*
  ** Call host_exec.
  */

  pid = host_exec(cmd, env, &fd_in, &fd_out, &fd_err);

  /*
  ** Instantiate a Process object.
  */

  process = allocInstance(thread, clazzNativeProcess);

  process[F_NativeProcess_process_id] = pid;
  process[F_NativeProcess_fd_in] = fd_in;
  process[F_NativeProcess_fd_out] = fd_out;
  process[F_NativeProcess_fd_err] = fd_err;

  /*
  ** Clean up.
  */
  
  for(i = 0; i < cmdlength; i++) x_mem_free(cmd[i]);
  for(i = 0; i < envlength; i++) x_mem_free(env[i]);
  if(cmd != NULL) x_mem_free(cmd);
  if(env != NULL) x_mem_free(env);

  /*
  ** Return the Process instance.
  */
  
  return process;
}
