/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: SecurityManager.c,v 1.3 2002/02/20 08:59:08 dfrederickx Exp $
*/

#include "arrays.h"
#include "clazz.h"
#include "methods.h"
#include "stack.h"
#include "threads.h"

w_instance 
SecurityManager_getClassContext
( JNIEnv *env, w_instance thisSecurityManger) {
  w_thread   thread = JNIEnv2w_thread(env);
  w_trace    trace = createStackTrace(thread->top);
  w_trace    current;
  w_int      count;
  w_instance result;
  w_instance *result_array;
  w_word item;

  current = trace;
  for (count=0;current;++count) {
    current = current->previous;
  }
  item = count;
  result = allocArrayInstance(thread, clazzArrayOf_Class, 1, &item);

  if (result) {
    result_array = instance2Array_instance(result);

    current = trace;
    for (count=0;current;++count) {
      result_array[count] = current->method->clazz->Class;
      current = current->previous;
    }
  }
  destroyStackTrace(trace);

  return result;
}


w_instance 
SecurityManager_getNonPrivilegedClassContext
( JNIEnv *env, w_instance thisSecurityManger) {
  w_thread   thread = JNIEnv2w_thread(env);
  w_trace    trace = createStackTrace(thread->top);
  w_trace    current;
  w_int      count;
  w_instance result;
  w_instance *result_array;
  w_word item;

  current = trace;
  for (count=0;current;++count) {
    current = isSet(current->flags, FRAME_PRIVILEGED) ? NULL : current->previous;
  }
  item = count;
  result = allocArrayInstance(thread, clazzArrayOf_Class, 1, &item);

  if (result) {
    result_array = instance2Array_instance(result);

    current = trace;
    for (count=0;current;++count) {
      result_array[count] = current->method->clazz->Class;
      current = current->previous;
    }
  }
  destroyStackTrace(trace);

  return result;
}


w_instance 
SecurityManager_currentClassLoader
( JNIEnv *env, w_instance thisSecurityManger) {
  w_thread   thread = JNIEnv2w_thread(env);
  w_trace    trace = createStackTrace(thread->top);
  w_trace    current;
  w_instance currentClass;
  w_instance result = NULL;

  current = trace;
  while (current) {
    currentClass = clazz2Class(current->method->clazz);
    if (currentClass[F_Class_loader]) {
      result = (w_instance)currentClass[F_Class_loader];
      break;
    }
    current = current->previous;
  }
  destroyStackTrace(trace);

  return result;
}

w_instance 
SecurityManager_currentLoadedClass
( JNIEnv *env, w_instance thisSecurityManger) {
  w_thread   thread = JNIEnv2w_thread(env);
  w_trace    trace = createStackTrace(thread->top);
  w_trace    current;
  w_instance currentClass;
  w_instance result = NULL;

  current = trace;
  while (current) {
    currentClass = clazz2Class(current->method->clazz);
    if (currentClass[F_Class_loader]) {
      result = currentClass;
      break;
    }
    current = current->previous;
  }
  destroyStackTrace(trace);

  return result;
}

  
