/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: String.c,v 1.11 2002/03/19 10:37:21 gray Exp $
*/

#include <string.h>
#include "arrays.h"
#include "chars.h"
#include "checks.h"
#include "core-classes.h"
#include "clazz.h"
#include "exception.h"
#include "loading.h"
#include "Math.h"
#include "descriptor.h"
#include "fields.h"
#include "heap.h"
#include "ts-mem.h"
#include "methods.h"
#include "realms.h"
#include "strings.h"
#include "threads.h"
#include "strings.h"
#include "chars.h"
#include "ieee754.h"
#include "stack.h"

// #include "hashtable.h" // remove this ...

/*
** Create an empty string
*/

void String_create_empty(JNIEnv *env, w_instance String) {
  
  String[F_String_wotsit] = (w_word)cstring2String(W_Thread_system,"",0);

  woempa(1, "%p new string at %p, empty\n", String, (char *)(String2string(String)));
    
}

/*
** Create a string instance just like an existing string
*/

void String_create_String(JNIEnv *env, w_instance String, w_instance value) {
  w_thread thread = JNIEnv2w_thread(env);

  if (isNullInstance(value)) {
    throwException(thread, clazzNullPointerException, NULL);
  } 
  else {
    String[F_String_wotsit] = (w_word)registerString(String2string(value));
  }
}

/*
** Create a string instance from a StringBuffer
*/

void String_create_StringBuffer(JNIEnv *env, w_instance String, w_instance value) {
  w_thread thread = JNIEnv2w_thread(env);

  if (isNullInstance(value)) {
    throwException(thread, clazzNullPointerException, NULL);
  } 
  else {
    String[F_String_wotsit] = (w_word)unicode2String(thread, instance2Array_char((w_instance)value[F_StringBuffer_value]), value[F_StringBuffer_count]);
  }

}

/*
** Create a string instance from an array of Characters
*/

void String_create_char(JNIEnv *env, w_instance String, w_instance charArray, w_int offset, w_int count) {

  w_thread thread = JNIEnv2w_thread(env);
  w_int length;
  
  if (isNullInstance(charArray)) {
    throwException(thread, clazzNullPointerException, NULL);
  } 
  else {
    length = instance2Array_length(charArray);
    if (offset < 0 || count < 0 || count + offset > length) {
      throwException(thread, clazzArrayIndexOutOfBoundsException, NULL);
    } 
    else {
      String[F_String_wotsit] = (w_word)unicode2String(thread, instance2Array_char(charArray) + offset, (w_size)count);
    }
  }

}

/*
** Create a string instance from an array of ASCII bytes
*/

void String_create_byte(JNIEnv *env, w_instance String, w_instance byteArray, w_int hibyte, w_int offset, w_int count) {
  w_thread thread = JNIEnv2w_thread(env);

  if(isNullInstance(byteArray)) {
    throwException(thread, clazzNullPointerException, NULL);
  } 
  else {
    w_sbyte *buffer = instance2Array_byte(byteArray);
    w_int   length = (w_int)byteArray[F_Array_length];

    if(offset < 0 || count < 0 || count + offset > length) {
      throwException(thread, clazzArrayIndexOutOfBoundsException, NULL);
    } 
    else {

      if (hibyte < 0) {
        String[F_String_wotsit] = (w_word)utf2String(thread, buffer+offset, (w_size)count);
      }
      else {
        w_char *charbuff = allocMem(thread, length * sizeof(w_char));
        w_byte *from = buffer+offset;
        w_char *to = charbuff;
        w_int   i;

        for(i = 0; i < count; ++i) {
          *to++ = (hibyte<<8) | (*from++);
        }
        String[F_String_wotsit] = (w_word)unicode2String(thread, charbuff, (w_size)count);
        releaseMem(charbuff);
      }
    }
  }

}


void String_finalize(JNIEnv* env, w_instance this) {

  w_string string = String2string(this);

  if (!string) {
    woempa(9,"Hm, F_String_wotsit of java.lang.String@%p is zero.\n",this);

    return;

  }

  /*
  ** If this string->instance points to the about to be destructed String object,
  ** set the pointer to NULL.
  */
  
  if (string && string->instance == this) {
    string->instance = NULL;
  }
  woempa(1,"java.lang.String@%p has F_String_wotsit %p ('%w'): deregistering.\n",this, string, string);
  deregisterString(string);
}

/*
** Vacuous.
*/

w_instance String_toString(JNIEnv *env, w_instance theString) { 

  return theString;

}

w_instance String_toCharArray(JNIEnv *env, w_instance This) {

  w_word length;
  w_string this = String2string(This);
  w_instance result;
  
  length = (w_word)this->length;
  result = allocArrayInstance(JNIEnv2w_thread(env), atype2clazz[P_char], 1, &length);
  if (result) {
    memcpy(instance2Array_char(result), this->chars, this->length * sizeof(w_char));
  }

  return result;
  
}

w_int String_compareTo(JNIEnv *env, w_instance This, w_instance String) {
  w_thread thread = JNIEnv2w_thread(env);
  w_string this = String2string(This);
  w_string string;
  w_int result = 0;
  w_int i;

  if (isNullInstance(String)) {
    throwException(thread, clazzNullPointerException, NULL);
  }
  else {
    string = String2string(String);
    if (this->length == 0) {
      result -= string->length;
    }
    else if (string->length == 0) {
      result = this->length;
    }
    else if (string != this) {
      for (i = 0; i < (w_int)this->length; i++) {

        /*
        ** Calculate possible result based on character compare.
        */
        
        if (this->chars[i] != string->chars[i]) {
          result = (w_int)(this->chars[i] - string->chars[i]);
          break;
        }

        /*
        ** Check that String and This don't differ in length only, otherwise calculate length difference.
        */
        
        if ((i + 1) >= (w_int)string->length || (i + 1) >= (w_int)this->length) {
          result = this->length - string->length;
          break;
        }

      }
    }
  }

  return result;
  
}

w_instance String_getBytes(JNIEnv *env, w_instance This, w_boolean Enc) {

  w_thread thread = JNIEnv2w_thread(env);
  w_instance result;
  w_string this = String2string(This);
  w_byte *dst;
  w_int i;

  if (Enc == WONKA_TRUE){
    w_byte * utfbytes = string2UTF8(thread, this, &i);

    i -= 2;
    result = allocArrayInstance(thread, atype2clazz[P_byte], 1, (w_word *)&i);
    if (result) {
      w_byte * utfsrc = utfbytes + 2;// skip the two first bytes (the length)
      w_int k=0;
      dst = instance2Array_byte(result);
      for( ; k < i ; k++){
        *dst++ = *utfsrc++;
      }
      releaseMem(utfbytes);
    }
  }
  else {
    w_char *src;
    w_word   length = (w_word)this->length;

    result = allocArrayInstance(thread, atype2clazz[P_byte], 1, &length);
    if (result) {
      dst = instance2Array_byte(result);
      src = this->chars;
      for (i = 0; i < (w_int)this->length; i++) {
        *dst++ = (w_byte)(*src++);
      }
    }
  }
  return result;

}

/*
** Compare two strings for equality
*/

w_boolean String_equals(JNIEnv *env, w_instance thisString, w_instance otherString) {

  if (isNullInstance(otherString) || ! isSuperClass(clazzString, instance2clazz(otherString))) {
    return WONKA_FALSE;
  }
  else {
    woempa(1, "Comparing %w to %w.\n", String2string(thisString), String2string(otherString));
    return String2string(thisString) == String2string(otherString);
  }

}

/*
** Compare two strings for equality modulo case
*/

w_boolean String_equalsIgnoreCase(JNIEnv *env, w_instance This, w_instance Other) {

  w_boolean result = WONKA_FALSE;
  w_string this;
  w_string other;
  w_int i;

  /*
  ** First check wether our arguments are not null and are of class String or a subclass, 
  ** if that checks out fine, we go further...
  */

  if (! isNullInstance(Other) && isSuperClass(clazzString, instance2clazz(Other))) {
    this = String2string(This);
    other = String2string(Other);
    /*
    ** Do a quick pointer check first.
    */
    if (this == other) {
      result = WONKA_TRUE;
    }
    else if (this->length == other->length) {
      /*
      ** Preset result to TRUE and compare character by character; when we hit an unequal character,
      ** we set result to FALSE and break...
      */
      result = WONKA_TRUE;
      for (i = 0; i < (w_int)this->length; i++) {
        if ( this->chars[i] != other->chars[i] && (char2lower(this->chars[i]) != char2lower(other->chars[i])) && (char2upper(this->chars[i]) != char2upper(other->chars[i]))) {
          result = WONKA_FALSE;
          break;
        }
      }
    }
  }
  
  return result;

}

/*
** Get the hashcode of a string. 
*/

w_int String_hashCode(JNIEnv *env, w_instance thisString) {
  w_string string = String2string(thisString);
  w_int hash = 0;
  w_char *s;
  w_size i;
  s = string->chars;
   
  for (i=0; i<(w_size)(string->length); ++i) hash = hash*31 + *s++;

  return hash;
}

/*
** Length in unicode characters
*/

w_int String_length(JNIEnv *env, w_instance String) {

  return String2string(String)->length;

}

w_boolean String_regionMatches(JNIEnv *env, w_instance This, w_boolean ic, w_int to, w_instance Other, w_int oo, w_int len) {
  w_thread thread = JNIEnv2w_thread(env);
  w_boolean result = WONKA_TRUE;
  w_string this = String2string(This);
  w_string other;
  w_int i;
  
  if (! isNullInstance(Other)) {
    other = String2string(Other);
    if (to < 0 || oo < 0 || to + len > (w_int)this->length || oo + len > (w_int)other->length) {
      result = WONKA_FALSE;
    }
    else {
      if (ic == WONKA_FALSE) {
        for (i = 0; i < len; i++) {
          if (this->chars[i + to] != other->chars[oo + i]) {
            result = WONKA_FALSE;
            break;
          }
        }
      }
      else {
        for (i = 0; i < len; i++) {
          if ((char2upper(this->chars[i + to]) != char2upper(other->chars[oo + i])) && (char2lower(this->chars[i + to]) != char2lower(other->chars[oo + i]))) {
            result = WONKA_FALSE;
            break;
          }
        }
      }
    }
  }
  else {
    throwException(thread, clazzNullPointerException, NULL);
  }
  
  return result;
  
}

w_boolean String_startsWith(JNIEnv *env, w_instance This, w_instance Prefix, w_int offset) {
  w_thread thread = JNIEnv2w_thread(env);
  w_string this = String2string(This);
  w_string prefix;
  w_byte *t;
  w_byte *p;
  w_boolean result = WONKA_FALSE;

  if (isNullInstance(Prefix)) {
    throwException(thread, clazzNullPointerException, NULL);
    return result;
  }

  prefix = String2string(Prefix);
  if (prefix->length == 0) result = WONKA_TRUE;
  else {
      if (prefix == this && offset == 0) {
      result = WONKA_TRUE;
    }
    else if (prefix->length && offset < (w_int)this->length && prefix->length <= (this->length - offset)) {
      t = (w_byte *)(this->chars + offset);
      p = (w_byte *)prefix->chars;
      result  = (memcmp(t, p, prefix->length * sizeof(w_char)) == 0);
    }
  }

  return result;

}

w_boolean String_endsWith(JNIEnv *env, w_instance This, w_instance Suffix) {
  w_thread thread = JNIEnv2w_thread(env);
  w_string this = String2string(This);
  w_string suffix;
  w_byte *t;
  w_byte *s;
  w_boolean result = WONKA_FALSE;
  
  if (! isNullInstance(Suffix)) {
    suffix = String2string(Suffix);
  if (suffix->length == 0 ) result = WONKA_TRUE;
  else {
    if (suffix == this) {
      result = WONKA_TRUE;
    }
    else if (suffix->length && suffix->length <= this->length) {
      t = (w_byte *)(this->chars + this->length - suffix->length);
      s = (w_byte *)suffix->chars;
      result = (memcmp(t, s, suffix->length * sizeof(w_char)) == 0);
    }
  }
  }
  else {
    throwException(thread, clazzNullPointerException, NULL);
  }
  
  return result;

}

/*
** Get one character of a string.  The characters are numbered from 0.
*/

w_char String_charAt(JNIEnv *env, w_instance thisString, w_int Index) {
  w_thread thread = JNIEnv2w_thread(env);
  w_string string = String2string(thisString);
  w_char result;

  if (Index < 0 || Index >= (w_int)string->length) {
    throwException(thread, clazzStringIndexOutOfBoundsException, NULL);
    result = 0x0000;
  }
  else {
    result = string->chars[Index];
  }
  
  return result; 
 
}

void String_getChars(JNIEnv *env, w_instance This, w_int srcBegin, w_int srcEnd, w_instance Dst, w_int dstBegin) {
  w_thread thread = JNIEnv2w_thread(env);
  w_string this;
  w_char *dst;
  w_int length;

  this = String2string(This);
  if (isNullInstance(Dst)) {
    throwException(thread, clazzNullPointerException, NULL);
  }
  else if (srcBegin < 0 || srcBegin > srcEnd || srcEnd > (w_int)this->length || dstBegin < 0 || dstBegin + srcEnd - srcBegin > instance2Array_length(Dst)) {
    throwException(thread, clazzIndexOutOfBoundsException, NULL);
  }
  else {
    if ((srcEnd - srcBegin) > 0) {
      dst = instance2Array_char(Dst);
      dst += dstBegin;
      length = srcEnd - srcBegin;
      memcpy(dst, this->chars+srcBegin, length * sizeof(w_char));
    }
  }
  
} 


void String_copyBytes(JNIEnv *env, w_instance Source, w_int srcBegin, w_int srcEnd, w_instance dstByteArray, w_int dstBegin) {
  w_string source = String2string(Source);
  w_sbyte *destination;
  w_int length;
  w_int i;
  
  destination = instance2Array_byte(dstByteArray);
  destination += dstBegin;
  length = srcEnd - srcBegin;
  
  for (i = 0; i < length; i++) {
    *destination++ = (w_byte)source->chars[i];
  }

}


w_int String_indexOf_char(JNIEnv *env, w_instance thisString, w_int ch, w_int offset) {
  w_string string;
  w_int result = -1;

  string = String2string(thisString);
  
  offset = offset < 0 ? 0 : offset;

  if (offset < (w_int)string->length) {
    for (  ; offset < (w_int)string->length; offset++) {
      if ((w_char)ch == string->chars[offset]) {
        result = offset;
        break;
      }
    }
  }

  return result;
  
}

w_int String_indexOf_String(JNIEnv *env, w_instance thisString, w_instance otherString, w_int fromIndex) {
  w_thread thread = JNIEnv2w_thread(env);
  w_string string;
  w_string other;
  w_int where;
  w_int max;
  w_int s;
  w_int o;
  w_int result = -1;

  if (isNullInstance(otherString)) {
    throwException(thread, clazzNullPointerException, NULL);
  }
  else {
    string = String2string(thisString);
    other = String2string(otherString);
    woempa(1,"findstring '%w' in '%w' starting at %d\n",other,string,fromIndex);
    
    if (fromIndex < (w_int)string->length) {
      where = fromIndex<0?0:fromIndex;
      max = string->length - other->length;
      while (where <= max) {
        woempa(1,"trying offset %d\n",where);
        for (s = where, o = 0; o < (w_int)other->length; o++, s++) {
          if (string->chars[s] != other->chars[o]) {
            woempa(1,"mismatch after %d chars\n",o);
            break;
          }
        }
        if (o == (w_int)other->length) {
          woempa(1,"match at offset %d\n",where);

          return where;

        }
        where += 1;
      }
    }
  }
  
  return result;

}

w_int String_lastIndexOf_char(JNIEnv *env, w_instance thisString, w_int ch, w_int offset) {
  w_string string = String2string(thisString);
  w_int i;
  w_int result = -1;

  string = String2string(thisString);

  if (0 <= offset) {
    offset = (offset < (w_int)string->length) ? offset : (w_int)string->length - 1;
    for (i = offset; i >= 0; i--) {
      if (ch == string->chars[i]) {
        result = i;
        break;
      }
    }
  }

  woempa(1, "Looked for last index of '%c', offset %d in %w, result %d.\n", (char)ch, offset, string, result);

  return result;
  
}

w_int String_lastIndexOf_String(JNIEnv *env, w_instance thisString, w_instance otherString, w_int offset) {
  w_thread thread = JNIEnv2w_thread(env);
  w_string string;
  w_string other;
  w_int where;
  w_int max;
  w_int s;
  w_int o;
  w_int result = -1;

  if (isNullInstance(otherString)) {
    throwException(thread, clazzNullPointerException, NULL);
  }
  else if (0 <= offset) {
    string = String2string(thisString);
    other = String2string(otherString);
    woempa(1, "%w <-> %w starting at %d.\n", other, string, offset);
    offset = (offset < (w_int)string->length) ? offset : (w_int)string->length;
    max = string->length - other->length;
    if (other->length == 0) {
    	return  offset;
    }
    if (max < 0) return -1;
    where = ( max < offset ?  max : offset);
    while (where+1) {
      woempa(1, "trying offset %d.\n", where);
      for (s = where, o = 0; o < (w_int)other->length; o++, s++) {
        if (string->chars[s] != other->chars[o]) {
          woempa(1, "mismatch after %d chars\n", o);
          break;
        }
      }
      if (o == (w_int)other->length) {
        result = where;
        break;
      }
      where -= 1;
    }

    woempa(1, "Result: %w <-> %w starting at %d, result %d.\n", other, string, offset, result);

  }
  
  return result;

}

w_instance String_toUpperCase(JNIEnv *env, w_instance This, w_instance Locale) {
  w_thread thread = JNIEnv2w_thread(env);
  w_string this = String2string(This);
  w_instance result = This;
  w_int i;
  w_char *buffer;
  w_char *src;
  w_char *dst;
  w_string string;
  
  /*
  ** Check that we have lowercase characters in the This string.
  */
  
  src = this->chars;
  for (i = 0; i < (w_int)this->length; i++) {
    if (*src != char2upper(*src++)) {
      break;
    }
  }

  if (i < (w_int)this->length) {
    /*
    ** We do, allocate buffer and convert, we copy over these characters that were OK
    ** and convert character by character from there...
    */
    buffer = allocMem(thread, this->length * sizeof(w_char));
    dst = buffer;
    src = this->chars;
    memcpy(dst, src, i * sizeof(w_char));
    dst += i;
    src += i;
    for (/* i is set allready */; i < (w_int)this->length; i++) {
      *dst++ = char2upper(*src++);
    }
    string = unicode2String(thread, buffer, this->length);
    result = getStringInstance(thread, string);
  // string is now registered twice, which is once too many
    deregisterString(string);
    releaseMem(buffer);
  }
    
  return result;

}

w_instance String_toLowerCase(JNIEnv *env, w_instance This, w_instance Locale) {
  w_thread thread = JNIEnv2w_thread(env);
  w_realm  realm;
  w_string this = String2string(This);
  w_instance result = This;
  w_int i;
  w_char *buffer;
  w_char *src;
  w_char *dst;
  w_string string;
  
  /*
  ** Check that we have uppercase characters in the This string.
  */
  
  src = this->chars;
  for (i = 0; i < (w_int)this->length; i++) {
    if (*src != char2lower(*src++)) {
      break;
    }
  }

  if (i < (w_int)this->length) {
    /*
    ** We do, allocate buffer and convert, we copy over these characters that were OK
    ** and convert character by character from there...
    */
    realm = thread2realm(thread);
    buffer = allocMem(thread, this->length * sizeof(w_char));
    dst = buffer;
    src = this->chars;
    memcpy(dst, src, i * sizeof(w_char));
    dst += i;
    src += i;
    for (/* i is set allready */; i < (w_int)this->length; i++) {
      *dst++ = char2lower(*src++);
    }
    string = unicode2String(thread, buffer, this->length);
    result = getStringInstance(thread, string);
    releaseMem(buffer);
    deregisterString(string);
  }
    
  return result;

}

w_instance String_replace(JNIEnv *env, w_instance This, w_char oldChar, w_char newChar) {
  w_thread thread = JNIEnv2w_thread(env);
  w_string this;
  w_instance Result = This;
  w_string result;
  w_char *buffer;
  w_int i;

  if (isNullInstance(This)) {
    throwException(thread, clazzNullPointerException, NULL);
  }
  else {
    this = String2string(This);
    for (i = 0; i < (w_int)this->length; i++) {
      if (this->chars[i] == oldChar) {
        break;
      }
    }
    if (i < (w_int)this->length) {
      buffer = allocMem(thread, this->length * sizeof(w_char));
      memcpy(buffer, this->chars, this->length * sizeof(w_char));
      for (i = 0; i < (w_int)this->length; i++) {
        if (buffer[i] == oldChar) buffer[i] = newChar;
      }
      Result = allocInstance(thread, clazzString);
      if (Result) {
        result = unicode2String(thread, buffer, this->length);
        attachString(Result, result);
      }

      releaseMem(buffer);
    }
  }

  return Result;

}

w_instance String_concat(JNIEnv *env, w_instance This, w_instance String) {
  w_thread thread = JNIEnv2w_thread(env);
  w_string this;
  w_string string;
  w_instance Result = nullInstance;
  w_string result;
  w_char *buffer;

  if (isNullInstance(String)) {
    throwException(thread, clazzNullPointerException, NULL);
  }
  else {
    this = String2string(This);
    string = String2string(String);
    if (string->length) {
      buffer = allocMem(thread, (string->length + this->length) * sizeof(w_char));
      memcpy(buffer, this->chars, this->length * sizeof(w_char));
      memcpy(buffer + this->length, string->chars, string->length * sizeof(w_char));
      Result = allocInstance(thread, clazzString);
      if (Result) {
        result = unicode2String(thread, buffer, this->length + string->length);
        attachString(Result, result);
      }

      releaseMem(buffer);
    }
    else {
      Result = This;
    }
  }
  
  return Result;

}

w_instance String_substring(JNIEnv *env, w_instance This, w_int offset, int endIndex) {

  w_thread thread = JNIEnv2w_thread(env);
  w_string this;
  w_instance SubString = nullInstance;
  w_string subString;

    this = String2string(This);
    if (offset < 0 || offset > (w_int)this->length || offset > endIndex || endIndex > (w_int)this->length) {
      throwException(thread, clazzIndexOutOfBoundsException, NULL);
    }
    else {
      SubString = allocInstance(thread, clazzString);
      if (SubString) {
        subString = unicode2String(thread, this->chars + offset, (w_size)(endIndex - offset));
        attachString(SubString, subString);
        woempa(1, "Result is %w\n", subString);
      }
    }

  return SubString;

}

/*
**
*/

w_instance String_trim(JNIEnv *env, w_instance This) {
  w_thread thread = JNIEnv2w_thread(env);
  w_string this = String2string(This);
  w_instance Result = This;
  w_string result;
  w_int i;
  w_int leading;
  w_int trailing;
  w_char *buffer;

  /*
  ** First check if we have leading and trailing whitespace and a string which is
  ** not empty.
  */

  if (this->length > 0 && (this->chars[0] <= 0x0020 || this->chars[this->length - 1] <= 0x0020)) {
    buffer = allocMem(thread, this->length * sizeof(w_char));
    memcpy(buffer, this->chars, this->length * sizeof(w_char));
    leading = 0;
    trailing = 0;
    for (i = 0; i < (w_int)this->length; i++) {
      if (buffer[i] > 0x0020) {
        break;
      }
      leading += 1;
    }
    for (i = this->length - 1; i >= 0; i--) {
      if (buffer[i] > 0x0020) {
        break;
      }
      trailing += 1;
    }
    if (leading != (w_int)this->length) {
      Result = allocInstance(thread, clazzString);
      if (Result) {
        result = unicode2String(thread, buffer + leading, this->length - trailing - leading);
        attachString(Result, result);
      }
      releaseMem(buffer);
    }
    else {
      Result = getStringInstance(thread, cstring2String(W_Thread_system,"",0));
    }
  }

  return Result;

}

w_instance String_intern(JNIEnv *env, w_instance thisString) { 

  return getStringInstance(JNIEnv2w_thread(env), String2string(thisString));

}

w_instance String_static_valueOf_char(JNIEnv *env, w_instance stringClass, w_char c) {
  w_string string = unicode2String(JNIEnv2w_thread(env), &c, 1);
  w_instance result= getStringInstance(JNIEnv2w_thread(env), string);
  // string is now registered twice, which is once too many
  deregisterString(string);
  return result;
}


