/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: StringBuffer.c,v 1.6 2002/03/20 13:04:20 gray Exp $
*/

#include <string.h>
#include "arrays.h"
#include "chars.h"
#include "clazz.h"
#include "core-classes.h"
#include "descriptor.h"
#include "exception.h"
#include "fields.h"
#include "heap.h"
#include "loading.h"
#include "ts-mem.h"
#include "methods.h"
#include "strings.h"
#include "threads.h"
#include "Math.h"
#include "stack.h"

/*
** Reallocate a buffer of 'size' and copy over the old contents.
** Can only be used to make a buffer larger, not smaller.
*/

void buffersize(w_thread thread, w_instance StringBuffer, w_int size) {

  w_instance newbuffer;
  w_char *old;
  w_char *new;
  w_word length;
  w_int i;
  
  length = (w_word)size;
  woempa(1,"currently used %d, requested size is %d\n",StringBuffer[F_StringBuffer_count],size);
  wassert((int)StringBuffer[F_StringBuffer_count]<=size);

  newbuffer = allocArrayInstance(thread, atype2clazz[P_char], 1, &length);
  if (newbuffer) {
    old = instance2Array_char((w_instance)StringBuffer[F_StringBuffer_value]);
    new = instance2Array_char(newbuffer);
    for (i = 0; i < (w_int)StringBuffer[F_StringBuffer_count]; i++) {
      *new++ = *old++;
    }

    setReferenceField(StringBuffer, newbuffer, F_StringBuffer_value);
  }
}

void StringBuffer_ensureCapacity(JNIEnv *env, w_instance StringBuffer, w_int minimum) {

  w_instance buffer = (w_instance)StringBuffer[F_StringBuffer_value];
  w_int oldsize = instance2Array_length(buffer);
  woempa(1,"%k at %p: minimum capacity is %d\n", instance2clazz(StringBuffer), StringBuffer, minimum);

  if (minimum > 0 && minimum > oldsize) {
    w_int newsize = minimum > 2 * oldsize ? minimum : 2 * oldsize + 2;
    woempa(1,"Current buffer has length %d, new buffer will have length %d\n", oldsize, newsize);
    buffersize(JNIEnv2w_thread(env), StringBuffer, newsize);
  }

}

void StringBuffer_setLength(JNIEnv *env, w_instance StringBuffer, w_int newLength) {
  w_thread thread = JNIEnv2w_thread(env);
  w_char *dst;

  if (newLength < 0) {
    throwException(thread, clazzIndexOutOfBoundsException, NULL);
  }
  else {
    if (newLength > (w_int)StringBuffer[F_StringBuffer_count]) {
      StringBuffer_ensureCapacity(env, StringBuffer, newLength);
      if (isNotSet(thread->flags, (WT_THREAD_THROWN | WT_THREAD_KILLED))) {
        dst = instance2Array_char((w_instance)StringBuffer[F_StringBuffer_value]);
        dst += (w_int)StringBuffer[F_StringBuffer_count];
        memset(dst, 0x00, (newLength - (w_int)StringBuffer[F_StringBuffer_count]) * sizeof(w_char));
      }
    }
    StringBuffer[F_StringBuffer_count] = (w_word)newLength;
  }
}

void StringBuffer_createFromString(JNIEnv *env, w_instance StringBuffer, w_instance String) {
  w_string string;
  //w_Item length;
  w_word length;
  w_instance buffer;

  if (! isNullInstance(String)) {  
    string = String2string(String);
    length = (w_word)(string->length + 16);
  }
  else {
    string = NULL;
    length = 16;
  }

  if ((w_int)length < 0) {
    wabort(ABORT_WONKA, "Length is negative (%d)!\n", (w_int)length);
  }

  buffer = allocArrayInstance(JNIEnv2w_thread(env), atype2clazz[P_char], 1, &length);

  if (buffer) {
    if (string) {
      memcpy(instance2Array_char(buffer), string->chars, string->length * sizeof(w_char));
      StringBuffer[F_StringBuffer_count] = (w_word)string->length;
    }
    setReferenceField(StringBuffer, buffer, F_StringBuffer_value);
  }
  
}

/*
** Internal service routine to insert an array of unicode characters into
** another such array. The parameters are:
**  - the point at which the characters are to be inserted; 
**  - the -remaining- length of the destination array (i.e. the length 
**    after the insertion point); 
**  - the start of the characters to be inserted; and 
**  - the number of characters to be inserted.
**
** This routine performs NO BOUNDARY CHECKS, so make sure you don't pass
** it garbage arguments.  You can safely call it with dest_length==0 though,
** when it acts as an 'append'.
*/

void insertChars(w_char *dest, w_int dest_length, w_char *srce, w_int srce_length) {
  w_int i;

  /*
  ** First make a 'hole' where the new characters fit into by copying 
  ** the existing characters from the 'hole' to a higher position, 
  ** make sure we do this copying backwards...
  */
  for (i = dest_length - 1; i >= 0; i--) {
    dest[srce_length + i] = dest[i];
  }

  /*
  ** Now copy over the characters from the src array into the 'hole'.
  */
  memcpy(dest,srce,srce_length*sizeof(w_char));
}

w_instance StringBuffer_append_String(JNIEnv *env, w_instance thisStringBuffer, w_instance theString) {
  w_thread thread = JNIEnv2w_thread(env);
  w_string string;
  w_char *src;
  w_char *dst;
  w_char  null_chars[4];
  w_int   src_length;
  
  if (isNullInstance(theString)) {
    string = NULL;
    null_chars[0] = 'n';
    null_chars[1] = 'u';
    null_chars[2] = 'l';
    null_chars[3] = 'l';
    src = null_chars;
    src_length = 4;
  }
  else {
    string = String2string(theString);
    src = string->chars;
    src_length = string->length;
  }

  woempa(1, "Appending string '%w' to StringBuffer.\n", string);

  StringBuffer_ensureCapacity(env, thisStringBuffer, src_length + (w_int)thisStringBuffer[F_StringBuffer_count]);

  if (isNotSet(thread->flags, (WT_THREAD_THROWN | WT_THREAD_KILLED))) {
    dst = instance2Array_char((w_instance)thisStringBuffer[F_StringBuffer_value])
          + (w_int)thisStringBuffer[F_StringBuffer_count];
    insertChars(dst, 0, src, src_length);
  
    thisStringBuffer[F_StringBuffer_count] += src_length;
  }

  return thisStringBuffer;
   
}

w_instance StringBuffer_insert_String(JNIEnv *env, w_instance thisStringBuffer, int Index, w_instance theString) {
  w_thread thread = JNIEnv2w_thread(env);
  w_string string;
  w_char *src;
  w_char *dst;
  w_char  null_chars[4];
  w_int  src_length;
  w_int  dst_length;

  if (  Index > (int)(thisStringBuffer[F_StringBuffer_count]) || Index < 0) {
    throwException(thread, clazzStringIndexOutOfBoundsException, NULL);
    return thisStringBuffer;
  }
  if (isNullInstance(theString)) {
    string = NULL;
    null_chars[0] = 'n';
    null_chars[1] = 'u';
    null_chars[2] = 'l';
    null_chars[3] = 'l';
    src = null_chars;
    src_length = 4;
  }
  else {
    string = String2string(theString);
    src = string->chars;
    src_length = string->length;
  }

  woempa(1, "Inserting string '%w' into StringBuffer at offset %d.\n", string, Index);

  StringBuffer_ensureCapacity(env, thisStringBuffer, src_length + (w_int)thisStringBuffer[F_StringBuffer_count]);

  if (isNotSet(thread->flags, (WT_THREAD_THROWN | WT_THREAD_KILLED))) {
    dst = instance2Array_char((w_instance)thisStringBuffer[F_StringBuffer_value])
          + Index;
    dst_length = (w_int)thisStringBuffer[F_StringBuffer_count] - Index;
    insertChars(dst, dst_length, src, src_length);
  
    thisStringBuffer[F_StringBuffer_count] += src_length;
  }

  return thisStringBuffer;
  
}

void StringBuffer_getChars(JNIEnv *env, w_instance StringBuffer, w_int srcBegin, w_int srcEnd, w_instance dst, w_int dstBegin) {

  w_thread thread = JNIEnv2w_thread(env);
  w_instance buffer = (w_instance)StringBuffer[F_StringBuffer_value];
  w_char *srcbuffer;
  w_char *dstbuffer;
  w_int i;

  if (! isNullInstance(dst)) {
    if (dstBegin < 0 || srcBegin < 0 || srcBegin > srcEnd || srcEnd > instance2Array_length(buffer) || dstBegin + srcEnd - srcBegin > instance2Array_length(dst)) {
      throwException(thread, clazzIndexOutOfBoundsException, NULL);
    }
    else {
      dstbuffer = instance2Array_char(dst);
      srcbuffer = instance2Array_char(buffer);
      dstbuffer += dstBegin;
      srcbuffer += srcBegin;
// [CG 20000413] don't grok this. The number of chars to be copied is (srcEnd - srcBegin), non?
//      for (i = 0; i < dstBegin + (srcEnd - srcBegin); i++) {
      for (i = 0; i < (srcEnd - srcBegin); i++) {
        *dstbuffer++ = *srcbuffer++;
      }
    }
  }
  else {
    throwException(thread, clazzNullPointerException, NULL);
  }

}

w_instance StringBuffer_deleteCharAt(JNIEnv *env, w_instance StringBuffer, w_int Index) {

  w_thread thread = JNIEnv2w_thread(env);
  w_int length = (w_int)StringBuffer[F_StringBuffer_count];
  w_char *dst;
  w_int i;

  if (Index < 0 || Index >= length) {
    throwException(thread, clazzStringIndexOutOfBoundsException, NULL);
  }
  else {
    dst = instance2Array_char((w_instance)StringBuffer[F_StringBuffer_value]);
    dst += Index;

    for (i = 0; i < length - Index - 1; i++) {
      dst[i] = dst[i + 1];
    }
    StringBuffer[F_StringBuffer_count] -= 1;
  }

  return StringBuffer;

}

w_instance StringBuffer_delete(JNIEnv *env, w_instance StringBuffer, w_int start, w_int end) {

  w_thread thread = JNIEnv2w_thread(env);
  w_int length = (w_int)StringBuffer[F_StringBuffer_count];
  w_char *dst;
  w_int num;
  w_int i;

  end = end > length ? length : end;

  if (start < 0 || start > length || start > end) {
    throwException(thread, clazzStringIndexOutOfBoundsException, NULL);
  }
  else {
    dst = instance2Array_char((w_instance)StringBuffer[F_StringBuffer_value]);
    dst += start;
    num = end - start;
    for (i = 0; i < length - end; i++) {
      dst[i] = dst[num + i];
    }
    StringBuffer[F_StringBuffer_count] -= num;
  }
  return StringBuffer;
}

w_instance StringBuffer_replace(JNIEnv *env, w_instance thisStringBuffer, w_int start, w_int end, w_instance theString) {

  w_thread thread = JNIEnv2w_thread(env);
  w_int length = (w_int)thisStringBuffer[F_StringBuffer_count];
  w_char *src;
  w_char *dst;
  w_int  src_length = 0;  // mp 20000921
  w_int  dst_length = 0;  // mp 20000921

  dst_length = (w_int)thisStringBuffer[F_StringBuffer_count];
  end = (end >= length) ? length : end;
  if (start < 0 || start > length || start > end) {
    throwException(thread, clazzStringIndexOutOfBoundsException, NULL);
  }
  else if (isNullInstance(theString)) {
    throwException(thread, clazzNullPointerException, NULL);
  }
  else if ((src_length = String2string(theString)->length) > end - start) {
    StringBuffer_ensureCapacity(env, thisStringBuffer, dst_length + src_length-(end-start));
  }

  if (isSet(thread->flags, (WT_THREAD_THROWN | WT_THREAD_KILLED))) {

    return nullInstance;

  }

  src = String2string(theString)->chars;
  dst = instance2Array_char((w_instance)thisStringBuffer[F_StringBuffer_value]);

  /*
  ** The next part is tricky, so we do it in three stages:
  ** 1. If the new string is longer than the space it needs to fit in
  **    (src_length > end-start), first we insert the excess (the last
  **    src_length-(end-start) characters) at position 'end'.
  ** 2. We copy the new string into the buffer at position 'start'.
  ** 3. If the new string is shorter than the space it needed to fit
  **    in (src_length < end-start) then we move the characters in 
  **    the buffer starting at position 'end' to 'start+src_length'.
  */
  if (end - start < src_length) {
    insertChars(dst+end, dst_length-end, src+end-start, src_length+start-end);
  }

  memcpy(dst+start,src,src_length*sizeof(w_char));
  if (end - start > src_length) {
    memcpy(dst+start+src_length,dst+end,(dst_length-end)*sizeof(w_char));
  }
  
  thisStringBuffer[F_StringBuffer_count] += src_length - (end-start);

  return thisStringBuffer;
  
}

w_instance StringBuffer_reverse(JNIEnv *env, w_instance StringBuffer) {

  w_char *a;
  w_char *z;
  w_char sc;
  
  if (1 < (w_int)StringBuffer[F_StringBuffer_count]) {
    a = instance2Array_char((w_instance)StringBuffer[F_StringBuffer_value]);
    z = a;
    z += StringBuffer[F_StringBuffer_count] - 1;
    while (a < z) {
      sc = *a;
      *a++ = *z;
      *z-- = sc;
    }
  }
  
  return StringBuffer;

}

w_instance StringBuffer_substring(JNIEnv *env, w_instance StringBuffer, w_int start, w_int end) {

  w_thread thread = JNIEnv2w_thread(env);
  w_int length = (w_int)StringBuffer[F_StringBuffer_count];
  w_string string;
  w_instance result = nullInstance;
  w_char *src;
  
  if (start < 0 || end < 0 || start > end || start > length || end > length) {
    throwException(thread, clazzStringIndexOutOfBoundsException, NULL);
  }
  else {
    src = instance2Array_char((w_instance)StringBuffer[F_StringBuffer_value]);
    src += start;
    string = unicode2String(JNIEnv2w_thread(env), src, (w_size)(end - start));
    result = getStringInstance(JNIEnv2w_thread(env), string);
    // string is now registered twice, when once would be enough
    deregisterString(string);
  }

  return result;

}

w_instance StringBuffer_toString(JNIEnv *env, w_instance StringBuffer) {

  w_string string = unicode2String(JNIEnv2w_thread(env), instance2Array_char((w_instance)StringBuffer[F_StringBuffer_value]), (w_size)StringBuffer[F_StringBuffer_count]);
  w_instance String = getStringInstance(JNIEnv2w_thread(env), string);
  
  // string is now registered twice, when once would be enough
  deregisterString(string);

  return String;
}

