/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: System.c,v 1.6 2002/03/26 09:51:48 dumon Exp $
*/

#include <string.h>

#include "arrays.h"
#include "clazz.h"
#include "core-classes.h"
#include "descriptor.h"
#include "exception.h"
#include "heap.h"
#include "loading.h"
#include "checks.h"
#include "ts-mem.h"
#include "oswald.h"
#include "strings.h"
#include "threads.h"
#include "wonkatime.h"

w_long
System_static_currentTimeMillis
( JNIEnv *env, w_instance classSystem
) {
  return (w_long)getNativeSystemTime();
}

w_instance System_static_getdescriptor(JNIEnv *env, w_instance classSystem, w_instance anObject) {
  w_clazz clazz = instance2clazz(anObject);
  
  return getStringInstance(JNIEnv2w_thread(env), clazz->desc->name);

}

void
System_static_referencearraycopy
( JNIEnv *env, w_instance classSystem,
  w_instance srcObject, w_int srcOffset,
  w_instance dstObject, w_int dstOffset,
  w_int copyLength
) {
  w_thread thread = JNIEnv2w_thread(env);
  w_int i;
  w_instance srcElement;

  woempa(1,"srcOffset is %d, dstOffset is %d, copyLength is %d\n",srcOffset,dstOffset,copyLength);
  woempa(1,"src length is %d, dst length is %d\n",(w_int)srcObject[F_Array_length],(w_int)dstObject[F_Array_length]);
  if (srcOffset<0 || dstOffset<0 || copyLength<0
      || srcOffset+copyLength > (w_int)srcObject[F_Array_length]
      || dstOffset+copyLength > (w_int)dstObject[F_Array_length]
     ) {
    throwException(thread, clazzArrayIndexOutOfBoundsException, NULL);

    return;
  }

  // eliminate trivial cases
  if (copyLength==0 || (srcObject==dstObject && srcOffset==dstOffset)) {
    woempa(1,"nothing to do!\n");

    return;

  }

  if(srcObject==dstObject) {
  // no need to check types when shuffling within same array
    if (srcOffset>dstOffset) {
      woempa(1,"srcOffset>dstOffset, copying from left to right\n");
      for (i=0;i<copyLength;++i) {
        setArrayReferenceField(dstObject, instance2Array_instance(srcObject)[srcOffset+i], dstOffset + i);
      }
    }
    else {
      woempa(1,"srcOffset>=dstOffset, copying from right to left\n");
      for (i=copyLength-1;i>=0;--i) {
        setArrayReferenceField(dstObject, instance2Array_instance(srcObject)[srcOffset+i], dstOffset + i);
      }
    }
  } else {
    woempa(1,"first word of source is %p\n",srcObject+F_Array_data+srcOffset);
    for(i=0;i<copyLength;++i) {
      srcElement = ((w_instance*)(srcObject+F_Array_data))[srcOffset+i];
      if (srcElement == nullInstance || isAssignmentCompatible(instance2clazz(srcElement),instance2clazz(dstObject)->aclazz->componentClazz)) {
//NGC        ((w_instance*)(dstObject+F_Array_data))[i] = srcElement;
        setArrayReferenceField(dstObject, srcElement, dstOffset + i);
      } else {
        throwException(thread, clazzArrayStoreException, NULL);
        break;
      }
    }

  }
}

void 
System_static_bitarraycopy
(  JNIEnv *env, w_instance classSystem,
  w_instance srcObject, w_int srcOffset,
  w_instance dstObject, w_int dstOffset,
  w_int copyLength
)
{
  w_thread thread = JNIEnv2w_thread(env);
  w_byte *fromByte;
  w_int   fromMask;
  w_byte *toByte;
  w_int   toMask;
  w_int   i;

  woempa(1,"srcOffset is %d, dstOffset is %d, copyLength is %d\n",srcOffset,dstOffset,copyLength);
  woempa(1,"src length is %d, dst length is %d\n",(w_int)srcObject[F_Array_length],(w_int)dstObject[F_Array_length]);
  if (srcOffset<0 || dstOffset<0 || copyLength<0
      || srcOffset+copyLength > (w_int)srcObject[F_Array_length]
      || dstOffset+copyLength > (w_int)dstObject[F_Array_length]
     ) {
    throwException(thread, clazzArrayIndexOutOfBoundsException, NULL);

    return;
  }

  // eliminate trivial cases
  if (copyLength==0 || (srcObject==dstObject && srcOffset==dstOffset)) {
    woempa(1,"nothing to do!\n");

    return;

  }

  if (srcOffset>dstOffset) {
    woempa(1,"srcOffset>dstOffset, copying from left to right\n");
    // could be same or different object, but in any case a L-R copy is safe
    fromByte = ((w_byte*)(srcObject+F_Array_data))+srcOffset/8;
    fromMask = 1 << (srcOffset%8);
    woempa(1,"first byte of source is %p, mask is %02x\n",fromByte,fromMask);
    toByte = ((w_byte*)(dstObject+F_Array_data))+dstOffset/8;
    toMask = 1 << (dstOffset%8);
    woempa(1,"first byte of dest is %p, mask is %02x\n",toByte,toMask);

    for (i=0;i<copyLength;++i) {
      if ((*fromByte)&fromMask) {
        *toByte |= toMask;
      }
      else *toByte &= 0xffffffff^toMask;

      fromMask <<= 1;
      if (fromMask==0x100) {
        ++fromByte;
        fromMask = 1;
      }

      toMask <<= 1;
      if (toMask==0x100) {
        ++toByte;
        toMask = 1;
      }
    }
  }
  else {
    woempa(1,"srcOffset>=dstOffset, copying from right to left\n");
    // could be same or different object, but in any case we copy R-L
    fromByte = ((w_byte*)(srcObject+F_Array_data))+(srcOffset+copyLength-1)/8;
    fromMask = 1 << ((srcOffset+copyLength-1)%8);
    woempa(1,"first byte of source is %p, mask is %02x\n",fromByte,fromMask);
    toByte = ((w_byte*)(dstObject+F_Array_data))+(dstOffset+copyLength-1)/8;
    toMask = 1 << ((dstOffset+copyLength-1)%8);
    woempa(1,"first byte of dest is %p, mask is %02x\n",toByte,toMask);

    for (i=0;i<copyLength;++i) {
      if ((*fromByte)&fromMask) {
        *toByte |= toMask;
      }
      else *toByte &= 0xffffffff^toMask;

      fromMask >>= 1;
      if (fromMask==0) {
        --fromByte;
        fromMask = 0x80;
      }

      toMask >>= 1;
      if (toMask==0) {
        --toByte;
        toMask = 0x80;
      }
    }
  }
  woempa(1,"last byte of source was %p, mask was %02x\n",fromByte,fromMask);
  woempa(1,"last byte of dest was %p, mask was %02x\n",toByte,toMask);
}

void 
System_static_bytearraycopy
(  JNIEnv *env, w_instance classSystem,
  w_instance srcObject, w_int srcOffset,
  w_instance dstObject, w_int dstOffset,
  w_int copyLength
)
{

  w_thread thread = JNIEnv2w_thread(env);
  w_byte *fromByte;
  w_byte *toByte;
  w_int   i;

  woempa(1,"srcOffset is %d, dstOffset is %d, copyLength is %d\n",srcOffset,dstOffset,copyLength);
  woempa(1,"src length is %d, dst length is %d\n",(w_int)srcObject[F_Array_length],(w_int)dstObject[F_Array_length]);
  if (srcOffset<0 || dstOffset<0 || copyLength<0
      || srcOffset+copyLength > (w_int)srcObject[F_Array_length]
      || dstOffset+copyLength > (w_int)dstObject[F_Array_length]
     ) {
    throwException(thread, clazzArrayIndexOutOfBoundsException, NULL);

    return;
  }

  // eliminate trivial cases
  if (copyLength==0 || (srcObject==dstObject && srcOffset==dstOffset)) {
    woempa(1,"nothing to do!\n");

    return;

  }

  if (srcOffset>dstOffset) {
    woempa(1,"srcOffset>dstOffset, copying from left to right\n");
    // could be same or different object, but in any case a L-R copy is safe
    fromByte = ((w_byte*)(srcObject+F_Array_data))+srcOffset;
    toByte = ((w_byte*)(dstObject+F_Array_data))+dstOffset;
    woempa(1,"first byte of source is %p, dest is %p\n",fromByte,toByte);

    for (i=0;i<copyLength;++i) {
      *toByte++ = *fromByte++;
    }
  }
  else {
    woempa(1,"srcOffset>=dstOffset, copying from right to left\n");
    // could be same or different object, but in any case we copy R-L
    fromByte = ((w_byte*)(srcObject+F_Array_data))+(srcOffset+copyLength-1);
    toByte = ((w_byte*)(dstObject+F_Array_data))+(dstOffset+copyLength-1);
    woempa(1,"first byte of source is %p, dest is %p\n",fromByte,toByte);
    for (i=0;i<copyLength;++i) {
      *toByte-- = *fromByte--;
    }
  }
  woempa(1,"last byte of source was %p, dest was %p\n",fromByte,toByte);
}

void 
System_static_shortarraycopy
(  JNIEnv *env, w_instance classSystem,
  w_instance srcObject, w_int srcOffset,
  w_instance dstObject, w_int dstOffset,
  w_int copyLength
)
{

  w_thread thread = JNIEnv2w_thread(env);
  w_short *fromShort;
  w_short *toShort;
  w_int   i;

  woempa(1,"srcOffset is %d, dstOffset is %d, copyLength is %d\n",srcOffset,dstOffset,copyLength);
  woempa(1,"src length is %d, dst length is %d\n",(w_int)srcObject[F_Array_length],(w_int)dstObject[F_Array_length]);
  if (srcOffset<0 || dstOffset<0 || copyLength<0
      || srcOffset+copyLength > (w_int)srcObject[F_Array_length]
      || dstOffset+copyLength > (w_int)dstObject[F_Array_length]
     ) {
    throwException(thread, clazzArrayIndexOutOfBoundsException, NULL);

    return;
  }

  // eliminate trivial cases
  if (copyLength==0 || (srcObject==dstObject && srcOffset==dstOffset)) {
    woempa(1,"nothing to do!\n");

    return;

  }

  if (srcOffset>dstOffset) {
    woempa(1,"srcOffset>dstOffset, copying from left to right\n");
    // could be same or different object, but in any case a L-R copy is safe
    fromShort = ((w_short*)(srcObject+F_Array_data))+srcOffset;
    toShort = ((w_short*)(dstObject+F_Array_data))+dstOffset;
    woempa(1,"first short of source is %p, dest is %p\n",fromShort,toShort);

    for (i=0;i<copyLength;++i) {
      *toShort++ = *fromShort++;
    }
  }
  else {
    woempa(1,"srcOffset>=dstOffset, copying from right to left\n");
    // could be same or different object, but in any case we copy R-L
    fromShort = ((w_short*)(srcObject+F_Array_data))+(srcOffset+copyLength-1);
    toShort = ((w_short*)(dstObject+F_Array_data))+(dstOffset+copyLength-1);
    woempa(1,"first short of source is %p, dest is %p\n",fromShort,toShort);
    for (i=0;i<copyLength;++i) {
      *toShort-- = *fromShort--;
    }
  }
  woempa(1,"last short of source was %p, dest was %p\n",fromShort,toShort);
}

void 
System_static_wordarraycopy
(  JNIEnv *env, w_instance classSystem,
  w_instance srcObject, w_int srcOffset,
  w_instance dstObject, w_int dstOffset,
  w_int copyLength
)
{

  w_thread thread = JNIEnv2w_thread(env);
  w_word *fromWord;
  w_word *toWord;
  w_int   i;

  woempa(1,"srcOffset is %d, dstOffset is %d, copyLength is %d\n",srcOffset,dstOffset,copyLength);
  woempa(1,"src length is %d, dst length is %d\n",(w_int)srcObject[F_Array_length],(w_int)dstObject[F_Array_length]);
  if (srcOffset<0 || dstOffset<0 || copyLength<0
      || srcOffset+copyLength > (w_int)srcObject[F_Array_length]
      || dstOffset+copyLength > (w_int)dstObject[F_Array_length]
     ) {
    throwException(thread, clazzArrayIndexOutOfBoundsException, NULL);

    return;
  }

  // eliminate trivial cases
  if (copyLength==0 || (srcObject==dstObject && srcOffset==dstOffset)) {
    woempa(1,"nothing to do!\n");

    return;

  }

  if (srcOffset>dstOffset) {
    woempa(1,"srcOffset>dstOffset, copying from left to right\n");
    // could be same or different object, but in any case a L-R copy is safe
    fromWord = ((w_word*)(srcObject+F_Array_data))+srcOffset;
    toWord = ((w_word*)(dstObject+F_Array_data))+dstOffset;
    woempa(1,"first word of source is %p, dest is %p\n",fromWord,toWord);

    for (i=0;i<copyLength;++i) {
      *toWord++ = *fromWord++;
    }
  }
  else {
    woempa(1,"srcOffset>=dstOffset, copying from right to left\n");
    // could be same or different object, but in any case we copy R-L
    fromWord = ((w_word*)(srcObject+F_Array_data))+(srcOffset+copyLength-1);
    toWord = ((w_word*)(dstObject+F_Array_data))+(dstOffset+copyLength-1);
    woempa(1,"first word of source is %p, dest is %p\n",fromWord,toWord);
    for (i=0;i<copyLength;++i) {
      *toWord-- = *fromWord--;
    }
  }
  woempa(1,"last word of source was %p, dest was %p\n",fromWord,toWord);
}

void 
System_static_longarraycopy
(  JNIEnv *env, w_instance classSystem,
  w_instance srcObject, w_int srcOffset,
  w_instance dstObject, w_int dstOffset,
  w_int copyLength
)
{

  w_thread thread = JNIEnv2w_thread(env);
  w_long *fromLong;
  w_long *toLong;
  w_int   i;

  woempa(1,"srcOffset is %d, dstOffset is %d, copyLength is %d\n",srcOffset,dstOffset,copyLength);
  woempa(1,"src length is %d, dst length is %d\n",(w_int)srcObject[F_Array_length],(w_int)dstObject[F_Array_length]);
  if (srcOffset<0 || dstOffset<0 || copyLength<0
      || srcOffset+copyLength > (w_int)srcObject[F_Array_length]
      || dstOffset+copyLength > (w_int)dstObject[F_Array_length]
     ) {
    throwException(thread, clazzArrayIndexOutOfBoundsException, NULL);

    return;
  }

  // eliminate trivial cases
  if (copyLength==0 || (srcObject==dstObject && srcOffset==dstOffset)) {
    woempa(1,"nothing to do!\n");

    return;

  }

  if (srcOffset>dstOffset) {
    woempa(1,"srcOffset>dstOffset, copying from right to left\n");
    // could be same or different object, but in any case a L-R copy is safe
    fromLong = ((w_long*)(srcObject+F_Array_data))+srcOffset;
    toLong = ((w_long*)(dstObject+F_Array_data))+dstOffset;
    woempa(1,"first long of source is %p, dest is %p\n",fromLong,toLong);

    for (i=0;i<copyLength;++i) {
      *toLong++ = *fromLong++;
    }
  }
  else {
    woempa(1,"srcOffset>=dstOffset, copying from left to right\n");
    // could be same or different object, but in any case we copy R-L
    fromLong = ((w_long*)(srcObject+F_Array_data))+(srcOffset+copyLength-1);
    toLong = ((w_long*)(dstObject+F_Array_data))+(dstOffset+copyLength-1);
    woempa(1,"first long of source is %p, dest is %p\n",fromLong,toLong);
    for (i=0;i<copyLength;++i) {
      *toLong-- = *fromLong--;
    }
  }
  woempa(1,"last long of source was %p, dest was %p\n",fromLong,toLong);
}

void
System_static_primitivearraycopy
( JNIEnv *env, w_instance classSystem,
  w_instance srcObject, w_int srcOffset,
  w_instance dstObject, w_int dstOffset,
  w_int copyLength
)
{

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz srcClazz;
  w_size srcObytes;
  w_size dstObytes;
  w_size cpyLbytes;
  char *temp;
  
  srcClazz = instance2clazz(srcObject); // dstClazz must be same!!!

  woempa(1,"srcObject is %p\n",srcObject);
  woempa(1,"srcClazz is %p, dstClazz is %p\n",srcClazz,instance2clazz(dstObject));
  woempa(1,"srcClazz->desc is %p '%w'\n", srcClazz->desc, srcClazz->desc->name);
//  woempa(1,"srcClazz->desc->body.array is %p\n", srcClazz->desc->body.array);
  woempa(1,"srcClazz->desc->body.array.component is %p\n", srcClazz->desc->body.array.component);
  switch(srcClazz->desc->body.array.component->body.primitive.type) {

    case P_byte:
      srcObytes = srcOffset;
      dstObytes = dstOffset;
      cpyLbytes = copyLength;
      break;

    case P_char:
    case P_short:
      srcObytes = srcOffset*2;
      dstObytes = dstOffset*2;
      cpyLbytes = copyLength*2;
      break;

    case P_double:
    case P_long:
      srcObytes = srcOffset*8;
      dstObytes = dstOffset*8;
      cpyLbytes = copyLength*8;
      break;

    default:
      srcObytes = srcOffset*4;
      dstObytes = dstOffset*4;
      cpyLbytes = copyLength*4;

  }

  woempa(1,"offsets in bytes: src %d, dst %d.  Copying %d bytes\n",srcObytes,dstObytes,cpyLbytes);
  
  if(srcObject==dstObject) {
    temp = (char*)allocMem(thread, cpyLbytes);
    memcpy(temp,((char*)(srcObject+F_Array_data))+srcObytes,cpyLbytes);
    memcpy(((char*)(dstObject+F_Array_data))+dstObytes,temp,cpyLbytes);
    releaseMem(temp);
  } else {
    memcpy(((char*)(dstObject+F_Array_data))+dstObytes,((char*)(srcObject+F_Array_data))+srcObytes,cpyLbytes);
  }

}

w_int System_identityHashCode(JNIEnv *env, jclass clazz, w_instance obj) {
  if(obj == NULL) {
    return 0;
  }

  return Object_hashCode(env, obj);
}


