/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: Thread.c,v 1.11 2002/03/20 13:04:20 gray Exp $
*/

#include <stdio.h>
#include <string.h>

#include "clazz.h"
#include "descriptor.h"
#include "exception.h"
#include "hashtable.h"
#include "locks.h"
#include "strings.h"
#include "stack.h"
#include "methods.h"
#include "fields.h"
#include "loading.h"
#include "realms.h"
#include "threads.h"
#include "arrays.h"
#include "ts-mem.h"
#include "interpreter.h"
#include "core-classes.h"
#include "heap.h"
#include "jdwp.h"
#include "jdwp_events.h"

extern const char *dumpThread(x_thread);

w_instance Thread_currentThread(JNIEnv *env, w_instance ThreadClass) {

  woempa(1, "Current Thread instance is %p.\n", JNIEnv2Thread(env));
  
  return JNIEnv2Thread(env);

}


static w_int seqnum = 0;

void Thread_create(JNIEnv *env, w_instance thisThread, w_instance parentThreadGroup, w_instance nameString, w_instance theRunnable) {
  char       *buffer;
  w_string   name;
  w_thread   currentthread = JNIEnv2w_thread(env);
  w_thread   newthread;
  w_thread   oldthread;
  w_group    parentgroup;
  w_clazz    runnable_clazz;
  w_size     stacksize;

  runnable_clazz = instance2clazz(theRunnable);
  if (runnable_clazz == clazzGarbageCollector) {
    stacksize = WONKA_GC_STACK_SIZE;
  }
  else if (runnable_clazz == clazzInit) {
    stacksize = WONKA_INIT_STACK_SIZE;
  }
  else {
    stacksize = WONKA_DEFAULT_STACK_SIZE;
  }

  woempa(1, "parentThreadGroup is %p.\n", parentThreadGroup);
  woempa(9,"Runnable is a %k, stack size will be %d\n", runnable_clazz, stacksize);

  parentgroup = (w_group)parentThreadGroup[F_ThreadGroup_wotsit];

  woempa(1, "w_Group of new thread is %p %w.\n", parentgroup, NM(parentgroup));

  if (isNullInstance(nameString)) {
    buffer = allocMem(parentgroup->manager, THREAD_NAME_BUFFER_SIZE * sizeof(w_byte));
    x_snprintf(buffer, THREAD_NAME_BUFFER_SIZE, "Thread-%d", ++seqnum);
    name = cstring2String(parentgroup->manager, buffer, strlen(buffer));
    releaseMem(buffer);
  } 
  else {
    name = String2string(nameString);
    registerString(name);
  }

  woempa(1, "Creating thread '%w' for group %p, is%s the system group.\n", name, parentgroup, parentgroup == W_ThreadGroup_system ? "" : " not");
  newthread = allocThread(parentgroup->manager);
  thisThread[F_Thread_wotsit] = (w_word)newthread;

  createThread(newthread, currentthread, thisThread, parentgroup, name, stacksize);

/* [CG 20011015] Used to happen here, now in Thread_start
  addThreadToGroup(newthread,parentgroup);
*/

  woempa(1, "registering os thread %p as thread instance %p\n", newthread->kthread, newthread);
  oldthread = (w_thread)ht_write(thread_hashtable, (w_word)newthread->kthread, (w_word)newthread);

  if(oldthread) {
    wabort(ABORT_WONKA, "Sapristi! that os thread %p was already registered!\n", newthread->kthread);
  }

  woempa(1, "Created thread '%w' @ %p\n", name, thisThread);

  x_thread_create(newthread->kthread,
    interpreterEntry, (void*)newthread, newthread->kstack,
    newthread->ksize,
    newthread->kpriority,
    TF_SUSPENDED);
  newthread->kthread->report = thread_report;
}

void Thread_finalize(JNIEnv *env, w_instance thisThread) {
  w_thread thread = (w_thread)thisThread[F_Thread_wotsit];

  thisThread[F_Thread_wotsit] = 0;

  woempa(9, "cleaning up remains of thread '%w'.\n", NM(thread));
  if (thread->bottom) {
    releaseStack(thread->bottom);
  }
// TODO: more stuff to release?
  releaseMem(thread);
}

w_instance Thread_toString(JNIEnv *env, w_instance thisThread) {

  w_thread thread = (w_thread)thisThread[F_Thread_wotsit];
  w_group  group  = thread->group;
  w_string threadname = thread->name;
  w_string groupname = group->name;
  w_string string;
  w_size   bufflen;
  w_char  *buffer;
  w_char  *slider;
  w_instance result;

  woempa(1, "Group is %p%s.\n", group, group == W_ThreadGroup_system ? "(SYSTEM)" : "");

  bufflen = 24 + threadname->length + groupname->length;
  buffer = allocMem(thread, bufflen * sizeof(w_char));
  slider = buffer;

  woempa(1, "Thread instance %p, thread %p, name %p.\n", thisThread, thread, threadname);

  *slider++ = 'T';
  *slider++ = 'h';
  *slider++ = 'r';
  *slider++ = 'e';
  *slider++ = 'a';
  *slider++ = 'd';
  *slider++ = '[';
//  w_string2chars(threadname,slider);
  memcpy(slider, threadname->chars, sizeof(w_char) * threadname->length);
  slider += threadname->length;
  *slider++ = ',';
  if(thread->jpriority>=10) {
    *slider++ = '1';
  }
  *slider++ = '0'+(thread->jpriority % 10);
  *slider++ = ',';
//  w_string2chars(groupname,slider);
  memcpy(slider, groupname->chars, sizeof(w_char) * groupname->length);
  slider += groupname->length; 
  *slider++ = ']';

//  string = chars2w_string(thread, buffer,(w_size)(slider-buffer));
  string = unicode2String(thread, buffer, (w_size)(slider - buffer));

  releaseMem(buffer);

  result = getStringInstance(thread, string);
  // string is now registered twice ...
  deregisterString(string);

  return result;
}

w_instance Thread_getName(JNIEnv *env, w_instance thisThread) {
  w_thread thread = (w_thread)thisThread[F_Thread_wotsit];

  return getStringInstance(thread, thread->name);;

}

void Thread_setName0(JNIEnv *env, w_instance thisThread, w_instance nameString) {

  w_thread thread = (w_thread)thisThread[F_Thread_wotsit];
  w_string oldname = thread->name;
  w_string newname = String2string(nameString);

  registerString(newname);
  deregisterString(oldname);

  thread->name = newname;

}


void Thread_start(JNIEnv *env, w_instance thisThread) {
  w_thread this_thread = (w_thread)thisThread[F_Thread_wotsit];
  w_thread that_thread = JNIEnv2w_thread(env);

  /*
  ** create the bottom frame for this thread.
  */

  woempa(1,"instance %p is of class %k\n",thisThread,instance2clazz(thisThread));

  switch(this_thread->state) {

    case(W_THREAD_UNSTARTED):
      woempa(1, "Starting JAVA Thread '%w' (%p).\n", NM(this_thread), thisThread);

      addThreadToGroup(this_thread,this_thread->group);
      x_thread_resume(this_thread->kthread);
      this_thread->state = W_THREAD_READY;

      if(jpda_hooks) {
        jdwp_event_thread_start(this_thread);
      }
      
      break;

    case(W_THREAD_READY):
    case(W_THREAD_WAITING):
    case(W_THREAD_SUSPENDED):
      woempa(7,"Thread %w is already started!\n",NM(this_thread));
      throwException(that_thread, clazzIllegalThreadStateException, NULL);

    default:
      woempa(7,"Thread %w is already dead!\n",NM(this_thread));
  }
}

void Thread_stop0(JNIEnv *env, w_instance thisThread, w_instance throwable) {

  w_thread thread;

  thread = (w_thread)thisThread[F_Thread_wotsit];

  if (thread->state == W_THREAD_DYING) {
    woempa(1,"Thread %w is already dying, forget the %k\n",NM(thread), instance2clazz(throwable));

    return;

  }

  woempa(1,"Killing thread %w with a %k\n",NM(thread), instance2clazz(throwable));
  thread->flags |= WT_THREAD_KILLED;
  setReferenceField(thisThread, throwable, F_Thread_thrown);

  if(jpda_hooks) {
    jdwp_event_thread_end(thread);
  }
  
  /*
  ** [CG 20010925] I guess we need to do this?
  */
  if (thread->state == W_THREAD_SUSPENDED) {
    x_thread_resume(thread->kthread);
  }
}

void Thread_suspend0(JNIEnv *env, w_instance thisThread) {

  w_thread thread;

  thread = (w_thread)thisThread[F_Thread_wotsit];

  x_thread_suspend(thread->kthread);
}

void Thread_resume0(JNIEnv *env, w_instance thisThread) {

  w_thread thread;

  thread = (w_thread)thisThread[F_Thread_wotsit];

  x_thread_resume(thread->kthread);
}


w_int Thread_getPriority(JNIEnv *env, w_instance thisThread) {

  w_thread thread;

  thread = (w_thread)thisThread[F_Thread_wotsit];

  woempa(1,"thread %p group %p instance %p\n",thread,thread->group,thread->group->ThreadGroup);

  return thread->jpriority;

}

void Thread_setPriority0(JNIEnv *env, w_instance thisThread, w_int newPriority) {

  w_thread thread = (w_thread)thisThread[F_Thread_wotsit];

  woempa(1,"thread %p group %p instance %p\n",thread,thread->group,thread->group->ThreadGroup);
  woempa(1,"requested priority is %d, group max priority is %d\n",newPriority, thread->group->maxPriority);

  thread->jpriority = newPriority;

  thread->kpriority = priority_j2k(thread->jpriority,0);
  x_thread_priority_set(thread->kthread, thread->kpriority);

}

w_boolean Thread_isDaemon(JNIEnv *env, w_instance thisThread) {

  w_thread thread = (w_thread)thisThread[F_Thread_wotsit];

  woempa(1,"thread %p group %p instance %p\n",thread,thread->group,thread->group->ThreadGroup);

  return thread->isDaemon;

}

void Thread_setDaemon0(JNIEnv *env, w_instance thisThread, w_boolean on) {

  w_thread thread = (w_thread)thisThread[F_Thread_wotsit];

  woempa(1, "thread %p group %p instance %p\n", thread, thread->group, thread->group->ThreadGroup);
  thread->isDaemon = on;

}

w_boolean Thread_isAlive(JNIEnv *env, w_instance thisThread) {

  w_thread thread = (w_thread)thisThread[F_Thread_wotsit];

  woempa(1,"thread %p group %p\n",thread,thread->group);

  return threadIsAlive(thread);

}

w_boolean Thread_isInterrupted(JNIEnv *env, w_instance thisThread) {

  w_thread thread = (w_thread)thisThread[F_Thread_wotsit];

  woempa(1,"thread %p group %p\n",thread,thread->group);

  return (thread->flags & WT_THREAD_INTERRUPTED) ? WONKA_TRUE : WONKA_FALSE;

}

void Thread_interrupt(JNIEnv *env, w_instance thisThread) {
  w_thread this_thread = (w_thread)thisThread[F_Thread_wotsit];

  woempa(7, "thread %w is interrupting %w\n",NM(JNIEnv2w_thread(env)),NM(this_thread));

  setFlag(this_thread->flags, WT_THREAD_INTERRUPTED);
  if (this_thread->state == W_THREAD_WAITING) {
    x_thread_stop_waiting(this_thread->kthread);
  }
}

w_boolean Thread_static_interrupted(JNIEnv *env, w_instance classThread) {

  w_thread thread = JNIEnv2w_thread(env);
  w_boolean interrupted = (thread->flags & WT_THREAD_INTERRUPTED) ? WONKA_TRUE : WONKA_FALSE;

  woempa(1,"thread %p group %p instance %p: interrupted flag was %d\n",thread,thread->group,thread->group->ThreadGroup,interrupted);

  thread->flags &= ~WT_THREAD_INTERRUPTED;

  return interrupted;

}

void Thread_static_yield(JNIEnv *env, w_instance ThreadClass) {

#ifdef DEBUG
  w_thread thread = JNIEnv2w_thread(env);

  woempa(1, "Env %p; yielding on instance of %k (thread %w).\n", env,  instance2clazz(thread->Thread), thread->name);
#endif

  x_thread_yield();
}

