/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: ThreadGroup.c,v 1.6 2002/03/19 10:37:21 gray Exp $
*/

#include <string.h>

#include "arrays.h"
#include "clazz.h"
#include "core-classes.h"
#include "fields.h"
#include "heap.h"
#include "loading.h"
#include "methods.h"
#include "realms.h"
#include "stack.h"
#include "strings.h"
#include "threads.h"
#include "ts-mem.h"

void ThreadGroup_create(JNIEnv *env, w_instance ThreadGroup, w_instance parentThreadGroup, w_instance nameString) {

  w_group parent;
  w_string name;

  if (!isNullInstance(parentThreadGroup)) {
    parent  = (w_group)parentThreadGroup[F_ThreadGroup_wotsit];
  }
  else {
    parent = NULL;
  }

  name = String2string(nameString);
  registerString(name);

  createThreadGroup(ThreadGroup, parent, name, GROUP_MANAGER_STACK_SIZE);
}

void
ThreadGroup_finalize (JNIEnv *env, w_instance thisThreadGroup) {
  w_group group = (w_group)thisThreadGroup[F_ThreadGroup_wotsit];
  w_realm realm = group->realm;

  woempa(7, "Cleaning up remains of thread group %w.\n", NM(group));

  x_queue_delete(group->box);
  releaseMem(group->box);
  group->box = NULL;
  releaseMem(group->boxSpace);
// TODO clean up complete manager thread
//  releaseMem(group->manager);
//  releaseMem(group->managerStack);

  deregisterString(group->name);

  if (realm->ThreadGroup == thisThreadGroup) {
    realm->ThreadGroup = NULL;
    woempa(7, "Realm %p has lost its ThreadGroup\n");
  }

  releaseMem(group);
}

w_int
ThreadGroup_getMaxPriority
( JNIEnv *env, w_instance thisThreadGroup
) {

  return ((w_group)(thisThreadGroup[F_ThreadGroup_wotsit]))->maxPriority;

}

void
ThreadGroup_setMaxPriority0
( JNIEnv *env, w_instance thisThreadGroup, 
  w_int newMaxPriority
) {

  ((w_group)(thisThreadGroup[F_ThreadGroup_wotsit]))->maxPriority = newMaxPriority;

}

w_instance 
ThreadGroup_getName
( JNIEnv *env, w_instance ThreadGroup
) {
  w_group group = (w_group)ThreadGroup[F_ThreadGroup_wotsit];

  if(group->name==NULL) {

    return nullInstance;

  }

  return getStringInstance(JNIEnv2w_thread(env), group->name);;
  
}

void
ThreadGroup_checkAccess
( JNIEnv *env, w_instance ThreadGroup
) {

  return;

}

