/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: Throwable.c,v 1.10 2002/03/26 10:44:15 gray Exp $
*/

#include <string.h>

#include "Throwable.h"
#include "clazz.h"
#include "core-classes.h"
#include "exception.h"
#include "ts-mem.h"
#include "strings.h"
#include "heap.h"
#include "jni.h"
#include "loading.h"
#include "realms.h"
#include "stack.h"
#include "descriptor.h"
#include "threads.h"
#include "methods.h"

void Throwable_finalize(JNIEnv *env, w_instance this) {
  destroyStackTrace((w_trace)this[F_Throwable_trace]);
}

w_instance Throwable_fillInStackTrace(JNIEnv *env, w_instance thisThrowable) {

  fillInStackTrace(thisThrowable);

  return thisThrowable;
  
}


w_instance
Throwable_startStackTrace
( JNIEnv *env, w_instance thisThrowable
) {
  w_thread thread = JNIEnv2w_thread(env);
  w_trace trace = (w_trace)thisThrowable[F_Throwable_trace];
  w_frame top = (w_frame)thisThrowable[F_Throwable_frame];
  w_string name = instance2clazz(thisThrowable)->dotified;
  w_instance message_instance = (w_instance)thisThrowable[F_Throwable_detailMessage];
  w_size   buflen;
  char *buffer;
  w_string string;
  w_instance result;

  if (trace) {
    woempa(1,"Stack trace %p was already filled in\n",trace);
  }
  else {
    if (top) {
      woempa(1,"Stack top %p was already filled in\n",top);
    }
    else {
      top = (JNIEnv2w_thread(env))->top;
      woempa(1,"Stack top was not filled in, inserting %p\n",top);
      thisThrowable[F_Throwable_frame] = (w_word)top;
    }
    fillInStackTrace(thisThrowable);
  }

  if (!isNullInstance(message_instance)) {
    w_string message = String2string(message_instance);

    buflen = name->length + message->length + 4;
    buffer = (char*)allocMem(thread, (w_size)buflen);
    x_snprintf(buffer,buflen,"%w : %w", name, message);
  }
  else {
    buflen = name->length + 1;
    buffer = (char*)allocMem(thread, (w_size)buflen);
    x_snprintf(buffer,buflen,"%w", name);
  }
  string = cstring2String(thread, buffer, strlen(buffer));
  releaseMem(buffer);

  result = getStringInstance(thread, string);
  // now string is registered twice ...
  deregisterString(string);

  return result;
}

w_instance
Throwable_nextStackTrace
( JNIEnv *env, w_instance thisThrowable
) {
  w_thread thread = JNIEnv2w_thread(env);
  w_trace trace = (w_trace)thisThrowable[F_Throwable_trace];
  w_code current = (w_code)thisThrowable[F_Throwable_current];
  w_string string;
  w_instance result = nullInstance;

  if (trace) {
    w_size line;
    w_size pc;
    w_size buflen;
    char  *buffer;
    char  *finger;
    w_instance loader;
    w_clazz loader_clazz;

    loader = clazz2loader(trace->method->clazz);
    buflen = trace->method->clazz->dotified->length + trace->method->name->length + trace->method->param_list->length + trace->method->return_type->dotified->length + 128;
    buffer = allocMem_system(buflen);
    line = code2line(trace->method, current, &pc);
    finger = buffer + x_snprintf(buffer, buflen, "%M [pc = ", trace->method);

    if (pc != 0xffffffff) {
      finger += x_snprintf(finger, 24, "%d, line = %d]", pc, line);
    }
    else {
      finger += x_snprintf(finger, 24, "???, line = ???]");
    }

    if (loader) {
      loader_clazz = instance2clazz(loader);
      finger += x_snprintf(finger, 80, "\n  loaded by %k@%p", loader_clazz, loader);
    }
    string = cstring2String(thread, buffer, strlen(buffer));
    releaseMem(buffer);

    thisThrowable[F_Throwable_trace] = (w_word)trace->previous;
    thisThrowable[F_Throwable_current] = (w_word)trace->caller_current;

    result = getStringInstance(thread, string);
    // now string is registered twice ...
    deregisterString(string);

  }

  return result;
}

