/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: Array.c,v 1.5 2002/03/20 13:04:20 gray Exp $
** 
** Implementation of the native methods for java/lang/reflect/Array
*/

#include <string.h>

#include "arrays.h"
#include "clazz.h"
#include "core-classes.h"
#include "descriptor.h"
#include "exception.h"
#include "heap.h"
#include "loading.h"
#include "ts-mem.h"
#include "reflection.h"
#include "strings.h"
#include "stack.h"
#include "threads.h"

w_instance Array_static_get(JNIEnv *env, w_instance Class, w_instance Array, w_int idx) {

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz = instance2clazz(Array);
  w_aclazz aclazz = clazz->aclazz;
  w_clazz cclazz;
  w_desc  cdesc;

  if (Array == nullInstance) {
    throwException(thread, clazzNullPointerException, NULL);
    return NULL;
  }

  if (aclazz == NULL) {
    throwException_C(thread, clazzIllegalArgumentException, (char*)"not an array class");
    return NULL;
  }

  if (idx < 0 || idx >= (w_int)Array[F_Array_length]) {
    throwException(thread, clazzArrayIndexOutOfBoundsException, NULL);
    return NULL;
  }

  cclazz = aclazz->componentClazz;
  cdesc  = aclazz->component;

  if (isNotSet(cclazz->flags, CLAZZ_IS_PRIMITIVE)) {
    return instance2Array_instance(Array)[idx];
  }
  else {
    w_instance wrapped = createWrapperInstance(thread, cdesc->body.primitive.type);
    w_word wrappee;
    w_double vdouble;
    w_long vlong;

    if (wrapped) {
      if (cclazz->desc == desc_byte) {
        wrapped[F_Byte_value] = instance2Array_byte(Array)[idx];
      }
      else if (cclazz->desc == desc_char) {
        wrapped[F_Character_value] = instance2Array_char(Array)[idx];
      }
      else if (cclazz->desc == desc_short) { 
        wrapped[F_Short_value] = instance2Array_short(Array)[idx];
      }
      else if (cclazz->desc == desc_int) { 
        wrapped[F_Integer_value] = instance2Array_int(Array)[idx];
      }
      else if (cclazz->desc == desc_long) { 
        vlong = instance2Array_long(Array)[idx];
        wrapped[F_Long_value + 1 ] = (w_word)MSW_PART(vlong);
        wrapped[F_Long_value     ] = (w_word)LSW_PART(vlong);
      }
      else if (cclazz->desc == desc_float) { 
        wrapped[F_Float_value] = instance2Array_float(Array)[idx];
      }
      else if (cclazz->desc == desc_double) { 
        vdouble = instance2Array_double(Array)[idx];
        wrapped[F_Double_value + 1 ] = (w_word)MSW_PART(vdouble);
        wrapped[F_Double_value     ] = (w_word)LSW_PART(vdouble);
      }
      else if (cclazz->desc == desc_boolean) { 
        w_int byteidx = idx / 8;
        w_int bitidx = idx % 8;

        w_sbyte bytevalue = instance2Array_byte(Array)[byteidx];
        bytevalue = (bytevalue >> bitidx) & 1;
        wrappee = (w_word)bytevalue;
        wrapped[F_Boolean_value] = wrappee;
      }
      else {
        wrappee = 0;
        wabort(ABORT_WONKA, "Don't get it: class %k has component type %w, component symbol is (DONT EXIST ANYMORE)\n", clazz,NM(aclazz->component));
      }

      return wrapped;
    }
    else {
      return nullInstance;
    }
  }

}

w_int Array_static_getLength(JNIEnv *env, w_instance Class, w_instance Array) {
  w_thread thread = JNIEnv2w_thread(env);

  if ( Array == nullInstance ) {
  	throwException(thread, clazzNullPointerException, NULL);
  	return -1;
  }	
  if (instance2clazz(Array)->aclazz == NULL){
    throwException_C(thread, clazzIllegalArgumentException, (char*)"not an array class");
    return -1;
  }	
  return instance2Array_length(Array);
  
}

w_instance Array_static_newInstance_single(JNIEnv *env, w_instance Class, w_instance Component, w_int length) {
  w_thread thread = JNIEnv2w_thread(env);
  w_word dimensions;
  w_instance Array = NULL;
  w_clazz component_clazz = Class2clazz(Component);
  w_clazz array_clazz;
  w_instance exception;
  w_desc arrayDesc = component_clazz->adesc;

  if (length < 0) {
    throwException(thread, clazzNegativeArraySizeException, NULL);

    return NULL;

  }
  else if (isNullInstance(Component)) {
    throwException(thread, clazzNullPointerException, NULL);

    return NULL;

  }

  if (!arrayDesc) {
    w_char * desc_buffer;
    w_string desc_string;
    w_size   i;
    w_char   ch;

    desc_buffer = allocMem(thread, (component_clazz->dotified->length + 3) * sizeof(w_char));
    desc_buffer[0] = (w_char)'[';
    desc_buffer[1] = (w_char)'L';
    for (i = 0; i < component_clazz->dotified->length; ++i) {
      ch = component_clazz->dotified->chars[i];
      desc_buffer[i + 2] = (ch == '.') ? '/' : ch;
    }
    desc_buffer[component_clazz->dotified->length + 2] = ';';
    desc_string = unicode2String(thread, desc_buffer, component_clazz->dotified->length + 3);
    arrayDesc = newFieldDescriptor(thread, desc_string);
    if (arrayDesc == NULL) {
      wabort(ABORT_WONKA,"whoops - failed to parse descriptor `%w'\n", desc_string);
    }

    deregisterString(desc_string);

    woempa(1, "Registered descriptor %w for array of %k.\n", NM(arrayDesc),  component_clazz);

    component_clazz->adesc = arrayDesc;
  }
  woempa(9, "Creating instance of 1 dimension and component %p '%k' (array desc = '%w').\n", component_clazz, component_clazz, NM(component_clazz->adesc));

  dimensions = length;
  exception = NULL;
  array_clazz = descArrayClassMustBeLoaded(clazz2loader(Class2clazz(Component)), Class2clazz(Component)->adesc, &exception);
  if (!exception) {
    exception = classMustBeResolved(array_clazz, thread);
  }
  if (exception) {
    setLocalException(thread,exception);
  }
  else {
    Array = allocArrayInstance(thread, array_clazz, 1, &dimensions);
  }

  return Array;
  
}

w_instance Array_static_newInstance_multi(JNIEnv *env, w_instance Class, w_instance Component, w_instance Dimensions) {

  w_word * dimensions;
  w_int i;
  w_int j;
  w_int *lengths;
  w_instance Array = NULL;
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz;
  w_desc desc;
  w_instance exception;

  woempa(9, "Creating instance of %d dimension(s) and component '%k'.\n", instance2Array_length(Dimensions), Class2clazz(Component));
  if (isNullInstance(Component) || isNullInstance(Dimensions)) {
    throwException(thread, clazzNullPointerException, NULL);
  }  
  else {

    /*
    ** Check and copy our Dimensions elements...
    */

    j = instance2Array_length(Dimensions);
    if (j < 1 || j > 255) {
      throwException_C(thread, clazzIllegalArgumentException, (char*)"illegal number of dimensions");
    }
    else {

      /*
      ** Create an array of w_int for the length of each dimension...
      */

      dimensions = allocMem(thread, j * sizeof(w_word));
      lengths = instance2Array_int(Dimensions);
      for (i = 0; i < j; i++) {
        if (lengths[i] < 0) {
          woempa(9, "lengths[%d] = %d !!\n", i, lengths[i]);
          throwException(thread, clazzNegativeArraySizeException, NULL);
          break;
        }
        dimensions[i] = lengths[i];
      }

      /*
      ** Create now an array clazz from the dimensions and the component clazz
      */

      if (isNotSet(thread->flags, (WT_THREAD_THROWN | WT_THREAD_KILLED))) {
        desc = createArrayDescriptor(thread, Class2clazz(Component), j);
        exception = NULL;
        clazz = descArrayClassMustBeLoaded(clazz2loader(Class2clazz(Component)), desc,&exception);
        deregisterDescriptor(desc);

        if (!exception) {
          exception = classMustBeResolved(clazz, thread);
        }
        if (exception) {
          setLocalException(thread,exception);
        }
        else {
          Array = allocArrayInstance(thread, clazz, j, dimensions);
        }
      }
      releaseMem(dimensions);
    }
  }
  
  return Array;
  
}

