/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: Constructor.c,v 1.12 2002/03/27 11:47:05 gray Exp $
*
** Implementation of the native methods for java/lang/reflect/Constructor
*/

#include "checks.h"
#include "clazz.h"
#include "constant.h"
#include "core-classes.h"
#include "descriptor.h"
#include "exception.h"
#include "reflection.h"
#include "strings.h"
#include "realms.h"
#include "methods.h"
#include "threads.h"
#include "heap.h"
#include "loading.h"
#include "stack.h"
#include "constant.h"

w_instance Constructor_getDeclaringClass(JNIEnv *env, w_instance thisConstructor) {
  w_method method;

  method = (w_method)thisConstructor[F_Constructor_wotsit];
  
  return clazz2Class(method->clazz);
  
}

w_instance Constructor_getName(JNIEnv *env, w_instance thisConstructor) {
  w_method method = (w_method)thisConstructor[F_Constructor_wotsit];
  
  return getStringInstance(JNIEnv2w_thread(env), method->clazz->dotified);
  
}

w_int Constructor_getModifiers(JNIEnv *env, w_instance thisConstructor) {

  w_method method = (w_method)thisConstructor[F_Constructor_wotsit];
  
  return method->flags & 0xffff;
  
}

w_instance Constructor_getParameterTypes(JNIEnv *env, w_instance thisConstructor) {
  w_thread thread = JNIEnv2w_thread(env);
  w_method method = (w_method)thisConstructor[F_Constructor_wotsit];
  w_instance exception = NULL;
  w_instance Parameters;
  w_clazz parameter;
  w_int numParameters = method->nargs;
  w_int i;
  w_word length;
  
  length = numParameters;
  Parameters = allocArrayInstance(thread, clazzArrayOf_Class, 1, &length);
  
  if (Parameters) {
    for (i = 0; i < numParameters; i++) {
      method->arg_types[i] = mustBeLoaded(method->arg_types[i], &exception);
      if (exception) {
        woempa(7, "Failed: %k\n", instance2clazz(exception));
        setLocalException(thread,exception);

        return NULL;
      }

      parameter = method->arg_types[i];

      setArrayReferenceField(Parameters, clazz2Class(parameter), i);
    }
  }

  return Parameters;
   
}

w_instance Constructor_getExceptionTypes(JNIEnv *env, w_instance thisConstructor) {

  w_thread thread = JNIEnv2w_thread(env);
  w_method method;
  w_instance Exceptions;
  w_clazz exception;
  w_int numthrows;
  w_int i;
  w_word length;

  method = (w_method)thisConstructor[F_Constructor_wotsit];
  // TODO exception =
  classMustBeResolved(method->clazz, thread);
  numthrows = method->numThrows;
  length = numthrows;

  Exceptions = allocArrayInstance(thread, clazzArrayOf_Class, 1, &length);

  if (Exceptions) {
    for (i = 0; i < numthrows; i++) {
      exception = getClassConstant(method->clazz, method->throws[i]);
      // TODO exception =
      classMustBePrepared(exception, thread);
      setArrayReferenceField(Exceptions, clazz2Class(exception), i);
    }
  }

  return Exceptions;

}

w_instance Constructor_newInstance0(JNIEnv *env, w_instance thisConstructor, w_instance Arguments) {
  w_thread thread = JNIEnv2w_thread(env);
  w_method init;
  w_frame frame;
  w_instance new = NULL;
  w_clazz calling_clazz = getCallingClazz(thread);
  w_instance calling_instance = getCallingInstance(thread);

  woempa(1,"Calling class is %k\n", calling_clazz);
  if (calling_instance) {
    woempa(1,"Calling instance is %k at %p\n", instance2clazz(calling_instance),calling_instance);
  }
  else {
    woempa(1,"No calling instance, calling method is static\n");
  }

  init = (w_method)thisConstructor[F_Constructor_wotsit];

  if (thisConstructor[F_AccessibleObject_accessible] == 0
           && !isAllowedToCall(calling_clazz, init, WONKA_FALSE)) {
    throwException(thread, clazzIllegalAccessException, NULL);
  }
  else {
    classMustBeInitialized(init->clazz, thread);
    new = allocInstance(thread, init->clazz);
  
    if (new) {
      w_instance exception = NULL;


      frame = invoke(env, init, new, Arguments, &exception);
      if (frame) {
        releaseBottomFrame(frame);
      }

      if (exception) {
        woempa(7, "(REFLECTION) invoke failed: %k\n", instance2clazz(exception));
        setLocalException(thread,exception);
      }
    }
  }

  return new;
    
}
