/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: Field.c,v 1.9 2002/03/26 10:44:15 gray Exp $
** 
**
** Implementation of the native methods for java/lang/reflect/Field
*/

#include <string.h>

#include "checks.h"
#include "core-classes.h"
#include "exception.h"
#include "fields.h"
#include "loading.h"
#include "strings.h"
#include "descriptor.h"
#include "clazz.h"
#include "arrays.h"
#include "threads.h"
#include "heap.h"
#include "ts-mem.h"
#include "ieee754.h"
#include "reflection.h"
#include "stack.h"

w_instance Field_getDeclaringClass(JNIEnv *env, w_instance Field) {

  return clazz2Class(Field2field(Field)->declaring_clazz);
  
}

w_instance Field_getName(JNIEnv *env, w_instance Field) {

  return getStringInstance(JNIEnv2w_thread(env), Field2field(Field)->name);

}

w_int Field_getModifiers(JNIEnv *env, w_instance Field) {

  return Field2field(Field)->flags & ACC_FLAGS;

}

w_instance Field_getType(JNIEnv *env, w_instance Field) {
  w_thread thread = JNIEnv2w_thread(env);
  w_field  field = Field2field(Field);
  w_clazz  clazz;
  w_instance Class;
  w_instance exception;

  Class = NULL;
  if (field->desc->tag == D_Primitive) {
    woempa(7, "Primitive type %d\n", field->desc->body.primitive.type);
    Class = primitive2classInstance[field->desc->body.primitive.type];
  }
  else if (field->desc->tag == D_Class || field->desc->tag == D_Array) {
    woempa(7, "Type is %d (Class or Array Class)\n", field->desc->tag);
    exception = NULL;
    clazz = descClassMustBeLoaded(clazz2loader(field->declaring_clazz), field->desc, &exception);
    if (exception) {
      setLocalException(thread,exception);
    }
    else {
      Class = clazz2Class(clazz);
    }
  }

  woempa(7, "Returning class instance of '%k'.\n", Class2clazz(Class));
  
  return Class;

}

w_boolean Field_equals(JNIEnv *env, w_instance Field, w_instance Object) {

  w_field field = Field2field(Field);
  w_clazz objectClazz = instance2clazz(Object);
  w_clazz declaring_1 = field->declaring_clazz;
  w_clazz declaring_2;

  /*
  ** 1) Check if Object is indeed a Field instance.
  ** 2) Check that they have the same declaring class.
  ** 3) Check that they have the same name.
  */
  
  if (objectClazz == clazzField) {
    declaring_2 = ((w_field)Object[F_Field_wotsit])->declaring_clazz;
    if (declaring_1 == declaring_2) {
      if (field->name == ((w_field)Object[F_Field_wotsit])->name) {
        return WONKA_TRUE;
      }
    }
  }

  return WONKA_FALSE;
 
}

/*
** Utility function to get data from an 'Object' instance or from the static Class data, trough
** a field ('Field' instance) and return it's possibly widened value in the T_data array as type
** T_desc. If the widening doesn't work, we throw an IllegalArgumentException.
*/

void get_convert_and_assign(JNIEnv *env, w_instance thisField, w_instance theObject, w_desc T_desc, w_word T_data[]) {

  w_thread thread = JNIEnv2w_thread(env);
  w_field field = Field2field(thisField);
  w_word *from;
  w_clazz calling_clazz;
  w_instance calling_instance;

  if (isSet(field->flags, ACC_STATIC)) {
    w_instance exception = classMustBeInitialized(field->declaring_clazz, thread);
    if (exception) {
      woempa(7,"Static initializer of %k threw %k\n", field->declaring_clazz, instance2clazz(exception));
      setLocalException(thread, exception);

      return;

    }
    from = field->declaring_clazz->staticFields + field->slot;
  }
  else {
    if (theObject == nullInstance) {
      woempa(7,"Object == null for non-static field %w of %k\n",NM(field), field->declaring_clazz);
      throwException(thread, clazzNullPointerException, NULL);

      return;

    }

    if (!isSuperClass(field->declaring_clazz,instance2clazz(theObject))) {
      woempa(7,"%k at %p is not a subclass of %k, does not have field %w\n", instance2clazz(theObject),theObject, field->declaring_clazz,NM(field));
      throwException_C(thread, clazzIllegalArgumentException, (char*)"not field of this class");

      return;

    }

    from = theObject + field->slot;
  }

  calling_clazz = getCallingClazz(thread);
  calling_instance = getCallingInstance(thread);
  if (thisField[F_AccessibleObject_accessible] == 0
      && !isAllowedToAccess(calling_clazz, field, (w_boolean)(calling_instance && (calling_instance == theObject)))) {
    throwException(thread, clazzIllegalAccessException, NULL);
  }
  else if (convert_and_assign(field->desc, from, T_desc, T_data)) {
    throwException_C(thread, clazzIllegalArgumentException, (char*)"widening not possible");
  }

}  

/*
** Here come the specific get methods.
*/

w_boolean Field_getBoolean(JNIEnv *env, w_instance Field, w_instance Object) {

  w_word to[1];

  get_convert_and_assign(env, Field, Object, primitive2desc[P_boolean], to);

  return to[0];

}

w_sbyte Field_getByte(JNIEnv *env, w_instance Field, w_instance Object) {

  w_word to[1];

  get_convert_and_assign(env, Field, Object, primitive2desc[P_byte], to);

  return to[0];

}

w_char Field_getChar(JNIEnv *env, w_instance Field, w_instance Object) {

  w_word to[1];

  get_convert_and_assign(env, Field, Object, primitive2desc[P_char], to);

  return to[0];
  
}

w_short Field_getShort(JNIEnv *env, w_instance Field, w_instance Object) {

  w_word to[1];

  get_convert_and_assign(env, Field, Object, primitive2desc[P_short], to);

  return to[0];

}

w_int Field_getInt(JNIEnv *env, w_instance Field, w_instance Object) {

  w_word to[1];

  get_convert_and_assign(env, Field, Object, primitive2desc[P_int], to);

  return to[0];

}

w_long Field_getLong(JNIEnv *env, w_instance Field, w_instance Object) {

  w_word to[2];

  get_convert_and_assign(env, Field, Object, primitive2desc[P_long], to);

  return WORDS2LONG(to[1], to[0]);

}

w_float Field_getFloat(JNIEnv *env, w_instance Field, w_instance Object) {

  w_word to[1];

  get_convert_and_assign(env, Field, Object, primitive2desc[P_float], to);

  return to[0];

}

w_double Field_getDouble(JNIEnv *env, w_instance Field, w_instance Object) {

  w_word to[2];

  get_convert_and_assign(env, Field, Object, primitive2desc[P_double], to);

  return WORDS2LONG(to[1], to[0]);

}

w_instance Field_get(JNIEnv *env, w_instance Field, w_instance Object) {

  w_thread thread = JNIEnv2w_thread(env);
  w_field field = Field2field(Field);
  w_instance result = nullInstance;
  w_word *to;

  /*
  ** Get the underlying value of the field.
  */

  woempa(1, "Getting value of field '%w', descriptor '%w' of instance '%p'\n", NM(field), NM(field->desc),Object);

  if (isNullInstance(Object) && isNotSet(field->flags, ACC_STATIC)) {
    throwException(thread, clazzNullPointerException, NULL);
  }
  else if (isNotSet(field->flags, ACC_STATIC) &&  ! isFieldFromClazz(instance2clazz(Object), field)) {
    throwException_C(thread, clazzIllegalArgumentException, (char*)"not field of this class");
  }
/*
** TODO: Access checks! (private/protected/public ...)
*/
  else if (field->desc->tag == D_Class || field->desc->tag == D_Array) {
    if (isSet(field->flags, ACC_STATIC)) {
/*
** TODO: exception =
*/
      classMustBeInitialized(field->declaring_clazz, thread);
      result = (w_instance)field->declaring_clazz->staticFields[field->slot];
    }
    else {
      result = (w_instance)Object[field->slot];
    }
  }
  else if (field->desc->tag == D_Primitive) {
    result = createWrapperInstance(thread, field->desc->body.primitive.type);
    if (result) {
      to = result + primitive2wrapperSlot[field->desc->body.primitive.type];
      get_convert_and_assign(env, Field, Object, field->desc, to);
    }
    else {
      result = nullInstance;
    }
  }

  return result;
      
}

static void set_convert_and_assign(JNIEnv *env, w_instance thisField, w_instance theObject, w_desc F_desc, w_word F_data[]) {

  w_thread thread = JNIEnv2w_thread(env);
  w_field field = Field2field(thisField);
  w_clazz calling_clazz;
  w_instance calling_instance;
  w_word *to;
  
  if (isSet(field->flags, ACC_STATIC)) {
/*
** TODO: exception =
*/
    classMustBeInitialized(field->declaring_clazz, thread);
    to = field->declaring_clazz->staticFields + field->slot;
  }
  else {
    to = theObject + field->slot;
  }

  calling_clazz = getCallingClazz(thread);
  calling_instance = getCallingInstance(thread);
  if (thisField[F_AccessibleObject_accessible] == 0
      && !isAllowedToAccess(calling_clazz, field, (w_boolean)(calling_instance && (calling_instance == theObject)))) {
    throwException(thread, clazzIllegalAccessException, NULL);
  }
  else if (convert_and_assign(F_desc, F_data, field->desc, to)) {
    throwException_C(thread, clazzIllegalArgumentException, (char*)"widening not possible");
  }

}  

void Field_setBoolean(JNIEnv *env, w_instance Field, w_instance Object, w_boolean z) {

  w_word from[1];

  from[0] = z;

  set_convert_and_assign(env, Field, Object, primitive2desc[P_boolean], from);

}

void Field_setByte(JNIEnv *env, w_instance Field, w_instance Object, w_sbyte b) {

  w_word from[1];

  from[0] = b;

  set_convert_and_assign(env, Field, Object, primitive2desc[P_byte], from);

}

void Field_setChar(JNIEnv *env, w_instance Field, w_instance Object, w_char c) {

  w_word from[1];

  from[0] = c;

  set_convert_and_assign(env, Field, Object, primitive2desc[P_char], from);

}

void Field_setShort(JNIEnv *env, w_instance Field, w_instance Object, w_short s) {

  w_word from[1];

  from[0] = s;

  set_convert_and_assign(env, Field, Object, primitive2desc[P_short], from);

}

void Field_setInt(JNIEnv *env, w_instance Field, w_instance Object, w_int i) {

  w_word from[1];

  from[0] = i;

  set_convert_and_assign(env, Field, Object, primitive2desc[P_int], from);

}

void Field_setLong(JNIEnv *env, w_instance Field, w_instance Object, w_long l) {

  w_word from[2];

  from[1] = MSW_PART(l);
  from[0] = LSW_PART(l);

  set_convert_and_assign(env, Field, Object, primitive2desc[P_long], from);

}

void Field_setFloat(JNIEnv *env, w_instance Field, w_instance Object, w_float f) {

  w_word from[1];

  from[0] = f;

  set_convert_and_assign(env, Field, Object, primitive2desc[P_float], from);

}

void Field_setDouble(JNIEnv *env, w_instance Field, w_instance Object, w_double d) {

  w_word from[2];

  from[1] = MSW_PART(d);
  from[0] = LSW_PART(d);

  set_convert_and_assign(env, Field, Object, primitive2desc[P_double], from);

}


void Field_set(JNIEnv *env, w_instance Field, w_instance Object, w_instance Value) {

  w_thread thread = JNIEnv2w_thread(env);
  w_field field = Field2field(Field);
  w_word *data = NULL;
  w_desc desc;

  /*
  ** Do some first hand checks... TODO security 
  */

  if (field->declaring_clazz==NULL) {

    wabort(ABORT_WONKA,"Instance of Field has clazz==NULL\n");

  }

  /*
  ** checks on arguments --> need to be done !
  */


  if (isNotSet(field->flags,ACC_STATIC) && isNullInstance(Object)) {
    throwException(thread, clazzNullPointerException, NULL);
  }
  else if (field->desc->tag == D_Primitive && isNullInstance(Value)) {
    throwException(thread, clazzNullPointerException, NULL);
  }
  else if (isNotSet(field->flags,ACC_STATIC) && !isSuperClass(field->declaring_clazz,instance2clazz(Object))) {
    throwException_C(thread, clazzIllegalArgumentException, (char*)"not field of this class");
  }
  else if (!Field[F_AccessibleObject_accessible] && isSet(field->flags, ACC_FINAL)) {
      throwException_C(thread, clazzIllegalAccessException, (char*)"field is FINAL");
    /*
    ** Missing checks:
    **   not ACC_FINAL and caller is another package -> IllegalAccessException
    **   ACC_PRIVATE and caller is not exact same class -> idem ditto
    **   ACC_PROTECTED and caller is not same/subclass  -> idem ditto
    */
  }
  else if (field->desc->tag == D_Class || field->desc->tag == D_Array) {

    /*
    ** It's a reference, see if the field is declared in the clazz of Object
    ** or in one of the super classes and this of course only when the field is not static...
    */

    if (!isSet(field->flags, ACC_STATIC) && ! isSuperClass(field->declaring_clazz, instance2clazz(Object))) {
      throwException_C(thread, clazzIllegalArgumentException, (char*)"not field of this class");
    }
    else if (isSet(field->flags, ACC_STATIC)) {
/*
** TODO: exception =
*/
      classMustBeInitialized(field->declaring_clazz, thread);
      setStaticReferenceField(field->declaring_clazz, (w_int)field->slot, Value);
    }
    else {
      if(NULL==Value) {
        setReferenceField(Object,nullInstance,field->slot);
      }
      else {
        setReferenceField(Object, Value, field->slot);
      }
    }
  }
  else if (field->desc->tag == D_Primitive) {

    /*
    ** It's a primitive field, get the appropriate descriptor and
    ** the unwrapped data and see if a widening assignment can happen.
    ** If the Value instance is not of a wrapper clazz, we get a NULL
    ** descriptor back...
    */


    desc = unwrapValue(Value, &data);
    if (desc) {
      set_convert_and_assign(env, Field, Object, desc, data);
    }
    else {
      throwException(thread, clazzIllegalArgumentException, NULL);
    }
  }
  else {
    throwException(thread, clazzIllegalArgumentException, NULL);
  }

}

w_instance Field_toString(JNIEnv *env, w_instance Field) {

  w_thread thread = JNIEnv2w_thread(env);
  w_field field = Field2field(Field);
  w_size length;
  w_desc desc;
  w_char *buffer;
  w_string string;
  w_instance result;

  /*
  ** Calculate the length first.
  */
  
  length = field->name->length;
  length += field->declaring_clazz->dotified->length;

  desc = field->desc;
  if (desc->tag == D_Class) {
    length += desc->name->length;
  }
  else {
    length += 8;
  }

  /*
  ** Make some extra headroom for other spaces and dots, modifiers, ...
  */

  length += 80;

  /*
  ** Now allocate the memory. We allocate too much memory but it's just a working buffer...
  ** [CG 20000412] Put qualifiers in canonical order 
  */
  
  buffer = allocMem(thread, length * sizeof(w_char));

  /*
  ** Fill 'em up
  */

  length = 0;
  if (isSet(field->flags, ACC_PUBLIC)) {
    length += cstringAdd(buffer + length, "public ");
  }

  if (isSet(field->flags, ACC_PROTECTED)) {
    length += cstringAdd(buffer + length, "protected ");
  }

  if (isSet(field->flags, ACC_PRIVATE)) {
    length += cstringAdd(buffer + length, "private ");
  }

  if (isSet(field->flags, ACC_STATIC)) {
    length += cstringAdd(buffer + length, "static ");
  }

  if (isSet(field->flags, ACC_FINAL)) {
    length += cstringAdd(buffer + length, "final ");
  }

  if (isSet(field->flags, ACC_VOLATILE)) {
    length += cstringAdd(buffer + length, "volatile ");
  }
  
  if (isSet(field->flags, ACC_TRANSIENT)) {
    length += cstringAdd(buffer + length, "transient ");
  }

  if (desc->tag == D_Class) {
    length += unicodeAddNumber(buffer + length, desc->name->chars + 1, (w_int)desc->name->length - 2);
  }
  else {
    if (desc->tag == D_Array) {
      length += stringAdd(buffer + length, desc->name);
    }
    else {
      switch (desc->body.primitive.type) {
        case P_boolean: length += stringAdd(buffer + length, string_boolean); break;
        case P_char:    length += stringAdd(buffer + length, string_char);    break;
        case P_byte:    length += stringAdd(buffer + length, string_byte);    break;
        case P_short:   length += stringAdd(buffer + length, string_short);   break;
        case P_int:     length += stringAdd(buffer + length, string_int);     break;
        case P_long:    length += stringAdd(buffer + length, string_long);    break;
        case P_float:   length += stringAdd(buffer + length, string_float);   break;
        case P_double:  length += stringAdd(buffer + length, string_double);  break;
        default:        length += cstringAdd(buffer + length, "??????");
      }
    }
  }

  length += cstringAdd(buffer + length, " ");
  length += stringAdd(buffer + length, field->declaring_clazz->dotified);
  length += cstringAdd(buffer + length, ".");
  length += stringAdd(buffer + length, field->name);

  /*
  ** Create the Wonka string and release our working buffer...
  */
  
  string = unicode2String(thread, buffer, length);
  releaseMem(buffer);

  woempa(1, "Created: %w\n", string);
  
  result = getStringInstance(thread, string);
  deregisterString(string);

  return result;
  
}
