/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: Method.c,v 1.12 2002/03/27 11:47:05 gray Exp $
**
** Implementation of the native methods for java/lang/reflect/Method
*/

#include <string.h>

#include "core-classes.h"
#include "clazz.h"
#include "constant.h"
#include "loading.h"
#include "methods.h"
#include "strings.h"
#include "descriptor.h"
#include "arrays.h"
#include "interpreter.h"
#include "checks.h"
#include "stack.h"
#include "threads.h"
#include "heap.h"
#include "ts-mem.h"
#include "reflection.h"
#include "exception.h"

w_instance Method_getDeclaringClass(JNIEnv *env, w_instance thisMethod) {
  w_method method;

  method = (w_method)thisMethod[F_Method_wotsit];
  
  return clazz2Class(method->clazz);
  
}

w_instance Method_getName(JNIEnv *env, w_instance thisMethod) {
  w_method method = (w_method)thisMethod[F_Method_wotsit];
  
  return getStringInstance(JNIEnv2w_thread(env), method->name);
  
}

w_int Method_getModifiers(JNIEnv *env, w_instance thisMethod) {

  w_method method = (w_method)thisMethod[F_Method_wotsit];
  
  return method->flags & 0xffff;
  
}

w_instance Method_getReturnType(JNIEnv *env, w_instance thisMethod) {
  w_thread thread = JNIEnv2w_thread(env);
  w_method method = (w_method)thisMethod[F_Method_wotsit];
  w_instance exception = NULL;

  method->return_type = mustBeLoaded(method->return_type, &exception);

  if (exception) {
    woempa(7, "Failed: %k\n", instance2clazz(exception));
    setLocalException(thread,exception);

    return NULL;
  }

  return clazz2Class(method->return_type);
  
}

w_instance Method_getParameterTypes(JNIEnv *env, w_instance thisMethod) {
  w_thread thread = JNIEnv2w_thread(env);
  w_method method = (w_method)thisMethod[F_Method_wotsit];
  w_instance exception = NULL;
  w_instance Parameters;
  w_clazz parameter;
  w_int numParameters = method->nargs;
  w_int i;
  w_word length;
  
  length = numParameters;
  Parameters = allocArrayInstance(thread, clazzArrayOf_Class, 1, &length);
  
  if (Parameters) {
    for (i = 0; i < numParameters; i++) {
      method->arg_types[i] = mustBeLoaded(method->arg_types[i], &exception);
      if (exception) {
        woempa(7, "Failed: %k\n", instance2clazz(exception));
        setLocalException(thread,exception);

        return NULL;
      }

      parameter = method->arg_types[i];

      setArrayReferenceField(Parameters, clazz2Class(parameter), i);
    }
  }

  return Parameters;
   
}

/*
**
*/

w_instance Method_getExceptionTypes(JNIEnv *env, w_instance thisMethod) {

  w_thread thread = JNIEnv2w_thread(env);
  w_method method;
  w_instance Exceptions;
  w_clazz  exception;
  w_int    numthrows;
  w_int    i;
  w_word   length;

  method = (w_method)thisMethod[F_Method_wotsit];
  // TODO exception =
  classMustBeResolved(method->clazz, thread);
  numthrows = method->numThrows;
  length = numthrows;

  Exceptions = allocArrayInstance(thread, clazzArrayOf_Class, 1, &length);

  if (Exceptions) {
    for (i = 0; i < numthrows; i++) {
      exception = getClassConstant(method->clazz, method->throws[i]);
      // TODO exception =
      classMustBePrepared(exception, thread);
      wassert(exception);	
      setArrayReferenceField(Exceptions, clazz2Class(exception), i);
    }
  }

  return Exceptions;

}

w_boolean Method_equals(JNIEnv *env, w_instance thisMethod, w_instance theObject) {

  w_method this_method = (w_method)thisMethod[F_Method_wotsit];
  w_boolean result = WONKA_FALSE;
  w_clazz objectClazz = instance2clazz(theObject);
  w_method that_method;

  if (objectClazz == clazzMethod) {
    that_method = (w_method)theObject[F_Method_wotsit];
    result = (this_method==that_method);
  }

  return result;
   
}

w_instance Method_invoke0(JNIEnv *env, w_instance thisMethod, w_instance theObject, w_instance Arguments) {
  w_thread thread = JNIEnv2w_thread(env);
  w_method method = (w_method)thisMethod[F_Method_wotsit];
  w_frame frame;
  w_instance This = NULL;
  w_desc returns;
  w_instance result = NULL;
  w_clazz clazz;
  w_clazz calling_clazz = getCallingClazz(thread);
  w_instance calling_instance = getCallingInstance(thread);
  w_instance exception = NULL;

  woempa(1,"Calling class is %k\n", calling_clazz);
  if (calling_instance) {
    woempa(1,"Calling instance is %k at %p\n", instance2clazz(calling_instance),calling_instance);
  }
  else {
    woempa(1,"No calling instance, calling method is static\n");
  }

  /*
  ** Stop the call if AccessibleObject is not set and the caller would not
  ** normally be able to access this method.
  */
  if (thisMethod[F_AccessibleObject_accessible] == 0
     && !isAllowedToCall(calling_clazz, method, (w_boolean)(calling_instance && (calling_instance == theObject)))) {
    throwException(thread, clazzIllegalAccessException, NULL);

    return NULL;

  }

  method->return_type = mustBeLoaded(method->return_type, &exception);
  if (exception) {
    woempa(7, "(REFLECTION) invoke failed: %k\n", instance2clazz(exception));
    setLocalException(thread,exception);
  }

  woempa(1, "Asked to invoke Method %M on instance %p of %k.\n", method, This, instance2clazz(This));

  if (isSet(method->flags, ACC_STATIC)) {
    This = clazz2Class(method->clazz);
    woempa(1, "Invoking static method %M on class instance of %k.\n", method, instance2clazz(This));
    /*
    ** Static method: ignore theObject, pass class where method was defined
    */
    This = clazz2Class(method->clazz);
  }
  else {
    /*
    ** Nonstatic method: use theObject to determine the actual class, and
    ** use that to find a possibly overridden method.
    */
    This = theObject;

    if (isNotSet(method->flags, ACC_PRIVATE)) {
      clazz = instance2clazz(This);
      method = clazz->methods[method->slot];
    }
    woempa(1, "After overriding analysis, invoking %M on class instance %p of %k.\n", method, This, instance2clazz(This));

  }        

  frame = invoke(env, method, This, Arguments, &exception);
  if (frame) {
    returns = method->return_type->desc;
    if (isSet(method->return_type->flags, CLAZZ_IS_PRIMITIVE)) {
      if (method->return_type == clazz_void) {
        result = NULL;
      }
      else {
        result = createWrapperInstance(thread, returns->body.primitive.type);
        if (sizeInWords(returns) == 2) {
          woempa(1, "Primitive 64 bit stack result is 0x%08x%08x\n", *(frame->o_top-1), *(frame->o_top-2));
          result[primitive2wrapperSlot[returns->body.primitive.type] + 1] = *(frame->o_top-1);
          result[primitive2wrapperSlot[returns->body.primitive.type] + 0] = *(frame->o_top-2);
        }
        else {
          woempa(1, "Primitive 32 bit stack result is 0x%08x\n", *(frame->o_top-1));
          result[primitive2wrapperSlot[returns->body.primitive.type]] = *(frame->o_top-1);
        }
      }
    }
    else {
      result = (w_instance)*(frame->o_top-1);
    }
    releaseBottomFrame(frame);
  }

  if (exception) {
    woempa(7, "(REFLECTION) invoke failed: %k\n", instance2clazz(exception));
    setLocalException(thread,exception);
  }

  return result;

}

