/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/**
 * $Id: InetAddress.c,v 1.4 2002/02/27 10:16:28 buytaert Exp $
 */

#include "network.h"

#include "arrays.h"
#include "core-classes.h"
#include "exception.h"
#include "realms.h"
#include "strings.h"
#include "threads.h"         
#include "ts-mem.h"

w_boolean InetAddress_lookupName(JNIEnv *env, w_instance thisAddress){
  w_boolean result = WONKA_FALSE;
  w_thread thread = JNIEnv2w_thread(env);
  w_string w_host = String2string((w_instance)thisAddress[F_InetAddress_addressCache]);
  struct hostent* host=NULL;
  char* hostname = (char *)allocMem (thread, (w_host->length+1) * sizeof(char));

  if (!hostname){
    throwException(thread, clazzOutOfMemoryError, NULL);
  }
  x_snprintf(hostname, w_host->length + 1, "%w", w_host);
  host = w_gethostbyname(hostname);
  if(host != NULL){
    thisAddress[F_InetAddress_hostName] = (w_word)(*env)->NewStringUTF(env, host->h_name);
    result = WONKA_TRUE;
  }
  releaseMem(hostname);
  return result;
}

/**
** hostName will be set to name ...
*/

void InetAddress_createInetAddress (JNIEnv *env, w_instance InetAddress, w_instance name) {

  w_thread thread = JNIEnv2w_thread(env);
  struct hostent* host=NULL;
  long ipnumber;
  w_String* w_host;
  char* hostname;

  /*
  ** Retrieving the C string from the name instance
  */

  w_host = String2string(name);
  hostname = (char *)allocMem (thread, (w_host->length+1) * sizeof(char));
  x_snprintf(hostname, w_host->length + 1, "%w", w_host);

  /*
  ** Getting the hostent struct...
  ** struct hostent {
  **   char* h_name;		--> name of host
  **   char** h_aliases; 	--> list of host aliases
  **   int h_addrtype;		--> type of host address (AF_INET for IPv4)
  **   int h_length;		--> address length
  **   char** h_addr_list;	--> list of addresses
  ** }
  */

  woempa(6,"Using DNS to get information...%s (%p)\n",hostname,hostname);
  host = w_gethostbyname(hostname);

  if (host==NULL) {
    throwException(thread, clazzUnknownHostException, NULL);
    releaseMem(hostname);
    return;
  }
  woempa(7,"displaying DNS information... name = %s (%p)\n",host->h_name,hostname);
  woempa(7,"displaying DNS information... length = %d (%p)\n",host->h_length,hostname);

  /*
  ** Creating and copying the hostname to private 'hostName' field of InetAddress Object
  */

  InetAddress[F_InetAddress_hostName] = (w_word)name;

  /*
  ** Copying the 'type' into the 'type' private field of InetAddress Object
  ** Loopback	-> type = 1
  ** IPv4 	-> type = 2
  */
  if (host->h_addrtype == AF_LOCAL) {
    InetAddress[F_InetAddress_family] = 1;
  }
  if (host->h_addrtype == AF_INET) {
    InetAddress[F_InetAddress_family] = 2;
  }

  /*
  ** 1.Retrieving the dotted decimal representation of the ip number.
  ** 2.Copying the octet stuff into the 'address' private field of InetAddress Object
  */

  ipnumber = *((unsigned long*)host->h_addr_list[0]);
  releaseMem(hostname);
  InetAddress[F_InetAddress_address] = htonl(ipnumber);
}

w_instance InetAddress_getLocalName (JNIEnv *env, w_instance clazz) {
  w_thread thread = JNIEnv2w_thread(env);
  int length = 255;
  w_instance Name;
  char* name = (char*) allocMem(thread, 255 * sizeof(char));
  
  if (w_gethostname(name, (w_size)length) != 0) {
    woempa(9,"Uhoh, something is wrong ... first time in the desert..., which way to Vegas ?\n");
  }
  woempa(7,"w_gethostname => '%s'\n",name);
  Name = (*env)->NewStringUTF(env,name);
  if (Name==NULL) {
    woempa(9,"Uhoh, something is wrong ... second time in the desert..., which way to Vegas ?\n");
  }
  releaseMem(name);
  return Name;
}
