/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/**
 * $Id: PlainSocketImpl.c,v 1.10 2002/03/19 10:33:49 gray Exp $
 */


#include <string.h>

#include "arrays.h"
#include "core-classes.h"
#include "ts-mem.h"
#include "strings.h"
#include "threads.h"         
#include "exception.h"
#include "network.h"

/**
** SocketImpl use calls to functions defined by network.h. All these calls work with a descriptor
** the descriptor is stored in the wotsit of SocketImpl ! We don't use the FileDescritor Object,
** it will be left null ... . This means other implementions should use this wotsit value to make calls to
** the network.  Note: this is not the way to do it according to java API.  Although using a FileDescriptor
** Almost implies using native code. By leaving that descriptor null we prevent it from being misused ...
*/


w_int PlainSocketImpl_clear(w_instance ThisImpl) {
  w_int sock = ThisImpl[F_PlainSocketImpl_wotsit];
  w_int res;	
  if(sock != -1) {
   	res = w_socketclose(sock);
   	if (res == -1) {
   	//woempa(9,"ERROR while closing socket %p (desc=%i) %s\n",ThisImpl,sock, w_strerror((int)w_errno(sock)));
   	}
   	ThisImpl[F_PlainSocketImpl_closed] = WONKA_TRUE;
    ThisImpl[F_PlainSocketImpl_wotsit] = (w_word)-1;
  }
  else {
    woempa(6, "socket %p was already closed !\n",ThisImpl);
    res = 0;
  }
  return res;
}

void PlainSocketImpl_close(JNIEnv* env , w_instance ThisImpl){
	if (PlainSocketImpl_clear(ThisImpl)==-1) {
	  throwException(JNIEnv2w_thread(env), clazzIOException, NULL);
	}
}

void PlainSocketImpl_finalize(JNIEnv* env , w_instance ThisImpl) {
	PlainSocketImpl_clear(ThisImpl);
}

w_int PlainSocketImpl_getSocket(JNIEnv* env , w_instance ThisImpl){
  w_int sock = ThisImpl[F_PlainSocketImpl_wotsit];
  if(sock == -1) {
  	throwException(JNIEnv2w_thread(env), clazzSocketException, NULL);
  }
  return sock;
}

void PlainSocketImpl_nativeCreate(JNIEnv* env , w_instance ThisImpl) {
  w_int sock = w_socket (PF_INET, SOCK_STREAM, 0);
  ThisImpl[F_PlainSocketImpl_wotsit] = sock;	
  if (sock == -1) {
    throwException(JNIEnv2w_thread(env), clazzIOException, NULL);
    woempa(9, "Not able to create a socket for %p\n",ThisImpl);
  }
}

void PlainSocketImpl_connect(JNIEnv* env , w_instance ThisImpl, w_instance address, w_int port) {
  if (!address) {
    	throwException(JNIEnv2w_thread(env), clazzNullPointerException, NULL);
  }
  else {
  	struct sockaddr_in sa;
  	w_int res = address[F_InetAddress_family];
  	w_int sock = ThisImpl[F_PlainSocketImpl_wotsit];

    if (sock ==-1) {
	    woempa(9, "socket %i was closed\n",sock);
	    throwException(JNIEnv2w_thread(env), clazzIOException, NULL);
  	  return;
  	}  	  	
  	memset(&sa,0,sizeof(sa));
  	sa.sin_addr.s_addr = htonl((w_int)address[F_InetAddress_address]);
  	//w_printIntAsIpString(sa.sin_addr.s_addr);
  	sa.sin_family = AF_INET;
  	sa.sin_port = w_switchPortBytes(port);
    //w_comparePorts(sa.sin_port,port);    	
  	
  	woempa (7,"INET: connect, port %d addr %lx\n", sa.sin_port, sa.sin_addr.s_addr);
  	res = w_connect (sock, (struct sockaddr*)&sa, sizeof (struct sockaddr_in));
  	woempa (6,"socketfd = %d, connect result = %d\n",sock, res);
  	if (res == -1) {
  		//woempa(9,"ERROR in connect = %s\n", w_strerror((int)w_errno(sock)));
		  throwException(JNIEnv2w_thread(env), clazzConnectException, NULL);
  		return;
  	}

  	ThisImpl[F_SocketImpl_port] = port;
  	ThisImpl[F_SocketImpl_address] = (w_int)address;
   	{
   	  socklen_t namelen = sizeof(sa);
 	    res = w_getsockname(sock , (struct sockaddr*)&sa , &namelen);
    	if(res == -1){
        //woempa(9,"ERROR in connect = %s\n", w_strerror((int)w_errno(sock))); 	
	      throwException(JNIEnv2w_thread(env), clazzConnectException, NULL);
 	    }
 	    else {	
      	ThisImpl[F_SocketImpl_localport] = w_switchPortBytes(sa.sin_port);
        //w_comparePorts(sa.sin_port,ThisImpl[F_SocketImpl_localport]);    	
      }
    }
  }
}

w_int PlainSocketImpl_read(JNIEnv* env , w_instance ThisImpl, w_instance byteArray, w_int off, w_int length) {
	// first checking our Arguments ...
  w_int sock = (int)ThisImpl[F_PlainSocketImpl_wotsit];
  w_int res;	

  woempa(6, "reading %i bytes from SocketImpl %p (desp %i)\n",length,ThisImpl,sock);

  if (!byteArray) {
    throwException(JNIEnv2w_thread(env), clazzNullPointerException, NULL);
    return -1;
  }
  else {
    w_int arrayLength = instance2Array_length(byteArray);
    if ((off < 0) || (length < 0) || (off + length > arrayLength)) {
    	throwException(JNIEnv2w_thread(env), clazzArrayIndexOutOfBoundsException, NULL);
    	return -1;
    }
  }
  if (sock == -1) {
	  throwException(JNIEnv2w_thread(env), clazzIOException, NULL);
    return -1;  	
  }

  if (length == 0) {
    return 0;
  }
  /*
  ** All seems OK, let's read...
  */
  {
    w_int timeout = ThisImpl[F_PlainSocketImpl_timeout];
    res = w_recv(sock, instance2Array_byte(byteArray) + off, (w_word)length, 0, &timeout);

    if (res == -1) {
      if(timeout == -1){
        woempa(9,"Error in read: timeout expired\n");
        throwException(JNIEnv2w_thread(env), clazzInterruptedIOException, NULL);
      }
      else {
        //woempa(9,"got error while reading from SocketImpl %p --> error = %s\n", ThisImpl, w_strerror(w_errno(sock)));
        throwException(JNIEnv2w_thread(env), clazzIOException, NULL);
      }
    }
    woempa(6, "reading %i bytes from SocketImpl %p --> got %i\n",length,ThisImpl,res);
    if (res == 0) { res--; }
    return res;
  }
}

  void PlainSocketImpl_write(JNIEnv * env, w_instance ThisImpl, w_instance byteArray, w_int off, w_int length){
  w_int sock = (w_int)ThisImpl[F_PlainSocketImpl_wotsit];
  w_int res;	

  woempa(6, "writing %i bytes to SocketImpl %p\n",length,ThisImpl);

  if (!byteArray) {
    throwException(JNIEnv2w_thread(env), clazzNullPointerException, NULL);
    return;
  }
  else {
    w_int arrayLength = instance2Array_length(byteArray);
    if ((off < 0) || (length < 0) || (off + length > arrayLength)) {
	    throwException(JNIEnv2w_thread(env), clazzArrayIndexOutOfBoundsException, NULL);
    	return;
    }
  }

  if (sock == -1) {
	  throwException(JNIEnv2w_thread(env), clazzIOException, NULL);
    return;  	
  }

  if (length == 0) {
    return;
  }

  res = w_send(sock, instance2Array_byte(byteArray) + off, (w_word)length, MSG_NOSIGNAL);

  if (res == -1) {
    woempa(9,"got error while sending from SocketImpl %p --> error = %i\n", ThisImpl, w_errno(sock));
    throwException(JNIEnv2w_thread(env), clazzIOException, NULL);
  }
}

void PlainSocketImpl_bind(JNIEnv* env , w_instance ThisImpl, w_instance address, int port){
  if (!address) {
 	throwException(JNIEnv2w_thread(env), clazzNullPointerException, NULL);
  }
  else {
  	struct sockaddr_in sa;
  	w_int res = address[F_InetAddress_family];
  	w_int sock = ThisImpl[F_PlainSocketImpl_wotsit];
  	
    if (sock ==-1) {
	    woempa(9, "socket %i was closed\n",sock);
		  throwException(JNIEnv2w_thread(env), clazzIOException, NULL);
  	 	return;
  	}  	  	  	
  	memset(&sa,0,sizeof(sa));
  	sa.sin_family = AF_INET;
   	sa.sin_addr.s_addr = htonl((w_int)address[F_InetAddress_address]);
  	//w_printIntAsIpString(sa.sin_addr.s_addr);
  	sa.sin_port = w_switchPortBytes(port);
    //w_comparePorts(sa.sin_port,port);    	
  	woempa(7,"INET: connect, port %d addr %x (%x)\n", sa.sin_port, sa.sin_addr.s_addr,address[F_InetAddress_address]);
  	res = w_bind (sock, (struct sockaddr*)&sa, sizeof (struct sockaddr_in));
  	woempa(6,"socketfd = %d, bindresult = %d\n",sock, res);
  	if (res == -1) {
  		//woempa(9,"ERROR = %s\n", w_strerror((int)w_errno(sock)));
		  throwException(JNIEnv2w_thread(env), clazzBindException, NULL);
  		return;
  	}
  	if(port == 0){
   	  socklen_t namelen = sizeof(sa);
 	    res = w_getsockname(sock , (struct sockaddr*)&sa , &namelen);
    	if(res == -1){
        //woempa(9,"ERROR in bind (getsockname) = %s\n", w_strerror((int)w_errno(sock))); 	
	      throwException(JNIEnv2w_thread(env), clazzBindException, NULL);
 	    }
 	    else {
      	port = w_switchPortBytes(sa.sin_port);
        //w_comparePorts(sa.sin_port,port);    	
    	}
    }  	
  	ThisImpl[F_SocketImpl_localport] = port;
  }

}


void PlainSocketImpl_listen(JNIEnv* env , w_instance ThisImpl, w_int backlog) {
  w_int sock = ThisImpl[F_PlainSocketImpl_wotsit];
  if(sock == -1) {
   	woempa(9, "socket %i is already closed\n",sock);
	  throwException(JNIEnv2w_thread(env), clazzIOException, NULL);
  }
  else {
  	w_int res = w_listen(sock,(int)backlog);
  	woempa(6, "listen called on socket %p (desc %i)\n",ThisImpl,sock);
		if (res == -1) {
			//woempa(9,"ERROR occured while listen was called --> %s\n", w_strerror((int)w_errno(sock)));
			throwException(JNIEnv2w_thread(env), clazzIOException, NULL);
		}
  }	
}

int PlainSocketImpl_accept(JNIEnv* env , w_instance ThisImpl, w_instance newImpl) {
  //newImpl will not be null (this function is a private method)
  w_int sock = ThisImpl[F_PlainSocketImpl_wotsit];
  if(sock == -1 || !newImpl) {
    woempa(9, "socket %i is already closed\n",sock);
	  throwException(JNIEnv2w_thread(env), clazzIOException, NULL);
  }
  else {// accept is done in two steps step
  	// step 1. get a socket from the system
  	struct sockaddr_in sa;
  	w_int newsocket;
  	socklen_t bytelen = sizeof (struct sockaddr_in);
  		
  	memset(&sa,0,sizeof(sa)); //lets play safe
  	newsocket = w_accept(sock, (struct sockaddr *)&sa , &bytelen, (w_int)ThisImpl[F_PlainSocketImpl_timeout]);
  	
  	//set the wotsit value to -1 or the valid socket descriptor
  	newImpl[F_PlainSocketImpl_wotsit] = newsocket;
  	
  	if (newsocket == -1) {
  	  if(bytelen == 0){
  	    woempa(9,"ERROR accept timed out\n");
  		  throwException(JNIEnv2w_thread(env), clazzInterruptedIOException, NULL);  	
  	  }
  	  else {
  	    //woempa(9,"ERROR occured while accept was called --> %s\n", w_strerror((int)w_errno(sock)));
  		  throwException(JNIEnv2w_thread(env), clazzIOException, NULL);
  	  }
  		return sock;
  	}  	
  	// step 2. set the fields of newImpl.
  	newImpl[F_SocketImpl_port] = w_switchPortBytes(sa.sin_port);
    //w_comparePorts(sa.sin_port,newImpl[F_SocketImpl_port]);    	
 	  bytelen = sizeof(sa);
    newsocket = w_getsockname(newsocket , (struct sockaddr*)&sa , &bytelen);
  	if(newsocket == -1){
      //woempa(9,"ERROR in accept (getsockname) = %s\n", w_strerror((int)w_errno(sock))); 	
      throwException(JNIEnv2w_thread(env), clazzIOException, NULL);
     }
     else {
      newImpl[F_SocketImpl_localport] = w_switchPortBytes(sa.sin_port);
      //w_comparePorts(sa.sin_port,newImpl[F_SocketImpl_port]);    	
  	}
  	//w_printIntAsIpString(sa.sin_addr.s_addr);
	  return ntohl(sa.sin_addr.s_addr);  	
  }
  return sock;
}

w_int PlainSocketImpl_available(JNIEnv* env , w_instance ThisImpl){
  w_int sock = ThisImpl[F_PlainSocketImpl_wotsit];
  if(sock == -1) {
    woempa(9, "socket %i is already closed\n",sock);
	  throwException(JNIEnv2w_thread(env), clazzIOException, NULL);
  }
  else { 	
    w_int arg=0;
    w_int res = ioctl(sock, FIONREAD, &arg);
    if(res == -1) {
      //woempa(9, "Error in Available 'ioctl' failed: %s\n", w_strerror((int)w_errno(sock)));
	    throwException(JNIEnv2w_thread(env), clazzIOException, NULL);
    }
    woempa(6, "Available bytes %x, %x\n",res,arg);
    return arg;  		
  }
 	return sock;
}


