/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: AccessController.c,v 1.15 2002/03/27 11:47:05 gray Exp $
*/

#include "arrays.h"
#include "core-classes.h"
#include "descriptor.h"
#include "interpreter.h"
#include "loading.h"
#include "stack.h"
#include "strings.h"
#include "threads.h"
#include "wordset.h"
#include "clazz.h"
#include "methods.h"
#include "exception.h"
#include "wonka.h"
#include "wordset.h"

static  w_method run_method = NULL;

w_instance
do_privileged_action(w_thread thread, w_instance action, w_instance inherited_context) {
  w_frame  new_frame;
  w_instance result = NULL;

  if (!run_method) {
     run_method = getMethod_name_sig(instance2clazz(action), string_run, string_no_params);
  }

// TODO: if called by reflection, set PRIVILEGED flag on the frame which
// invoked reflection, not that of Method.reflect() ...

  new_frame = allocBottomFrame(thread, instance2clazz(action)->methods[run_method->slot]);
  if (new_frame) {
    w_instance exception;

    new_frame->locals[0] = (w_word)action;
    setFlag(new_frame->flags, FRAME_PRIVILEGED);

    woempa(1, "(PRIVILEGED) Starting frame with %M, parameter is %k@%p.\n", run_method, instance2clazz(action), action);
    handleSynchronization(new_frame, run_method);
    exception = runThread(thread, new_frame);
    if (exception) {
      woempa(1, "(PRIVILEGED) Done calling %M, exception %k was thrown.\n", run_method, instance2clazz(exception));
      setLocalException(thread, exception);
    }
    else{
      result = (w_instance)new_frame->o_top[-1];
    }
    releaseBottomFrame(new_frame);
    woempa(1, "(PRIVILEGED) Done calling %M, returned %p.\n", run_method, result);
  }

  return result;
}

static w_clazz    clazzArrayOf_ProtectionDomain;

void AccessController_static_initialize(JNIEnv *env, w_instance classAccessController) {
  w_thread thread = JNIEnv2w_thread(env);
  w_instance exception = NULL;
  w_desc     adesc;

  classMustBeInitialized(clazzProtectionDomain, thread);
  adesc = clazzProtectionDomain->adesc;
  if (!adesc) {
    w_string desc_string = cstring2String(thread, "[Ljava/security/ProtectionDomain;",33);
    adesc = newFieldDescriptor(thread, desc_string);
    if (adesc == NULL) {
      wabort(ABORT_WONKA,"whoops - failed to parse descriptor `%w'\n", desc_string);
    }

    deregisterString(desc_string);

    woempa(1, "Registered descriptor %w for array of %k.\n", NM(adesc), clazzProtectionDomain);

    clazzProtectionDomain->adesc = adesc;
  }
  clazzArrayOf_ProtectionDomain = descArrayClassMustBeLoaded(NULL, adesc, &exception);
  // TODO: handle exception

}

w_instance 
AccessController_static_doPrivileged0
( JNIEnv *env, w_instance classAccessController, 
  w_instance action, w_instance context
) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz = instance2clazz(action);
  w_instance result;

  
 // TODO: is this really needed? 
  classMustBeInitialized(clazz, thread);

  result = do_privileged_action(thread, action, context);

  return result;
}

w_instance 
AccessController_static_get_calling_domains
( JNIEnv *env, w_instance AccessController) {
  w_thread   thread = JNIEnv2w_thread(env);
  w_frame    current;
  w_int      count;
  w_wordset  domains = NULL;
  w_instance result;
  w_instance *result_array;
  w_word     item;
  w_int      i;

  current = thread->top ? thread->top->previous : NULL;
  while (current && isSet(current->flags, FRAME_REFLECTION)) {
    woempa(1,"Frame %p (%M) is a reflection frame, skipping\n",current, current->method);
    current = current->previous;
  }

  while (current) {
    w_instance domain = (w_instance)clazz2Class(current->method->clazz)[F_Class_domain];
    if (domain && !isInWordset(&domains, (w_word)domain)) {
      addToWordset(thread, &domains, (w_word)domain);
    }
    else {
      woempa(1,"Frame %p (%M) has no associated protection domain, skipping\n",current, current->method);
    }
    woempa(1,"Frame %p (%M) is %sprivileged.\n",current, current->method, isSet(current->flags, FRAME_PRIVILEGED)?"":"not ");
    current = isSet(current->flags, FRAME_PRIVILEGED) ? NULL : current->previous;
  }
  count = sizeOfWordset(&domains);
  woempa(1,"Found %d protection domains on stack\n", count);

  item = count;
  result = allocArrayInstance(thread, clazzArrayOf_ProtectionDomain, 1, &item);

  if (count == 0) {

    return result;

  }

  if (result) {
    result_array = instance2Array_instance(result);
    /*
    ** Fill in the array.
    */
    for (i = 0; i < count; ++i) {
      result_array[i] = (w_instance)elementOfWordset(&domains, i);
      woempa(1, "result[%d] = %p\n", i, result_array[i]);
    }
  }
  else {
    woempa(9, "Could not allocate result array %k with size %d\n", clazzArrayOf_ProtectionDomain, count); 
  }

  releaseWordset(&domains);

  return result;
}


w_instance 
AccessController_static_get_inherited_context
( JNIEnv *env, w_instance AccessController) {
  w_thread   thread = JNIEnv2w_thread(env);
  w_frame    current;

  current = thread->top ? thread->top->previous : NULL;
  while (current && !current->inherited_context) {
    current = current->previous;
  }

  if (current) {
    woempa(1, "Inherited context is %p, found in frame %p\n", current->inherited_context, current);
  }
  else {
    woempa(1, "No inherited context found, returning NULL\n");
  }

  return current ? current->inherited_context : NULL;
}

