/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: Vector.c,v 1.4 2002/03/19 10:31:04 gray Exp $
*/

#include <string.h>

#include "arrays.h"
#include "clazz.h"
#include "core-classes.h"
#include "exception.h"
#include "heap.h"
#include "realms.h"
#include "threads.h"

/*
** bumpElementData makes space for a new element to be inserted, or fills
** the space left when an element was removed.  The object will be created
** or has been removed at elementData[idx]: direction is +1 for an insertion
** and -1 for a removal.  (No other value is allowed).
*/

void
Vector_bumpElementData
( JNIEnv *env, w_instance thisVector, int idx, int distance
) {
  w_thread  thread = JNIEnv2w_thread(env); 
  w_instance array = (w_instance)thisVector[F_Vector_elementData];
  w_instance *data = instance2Array_instance(array);
  w_size     count;
  w_instance *start;
  w_instance *temp;

  woempa(1,"Vector @ %p array @ %p data @ %p: bump index %d distance %d \n", thisVector, array, data, idx, distance);

/*
** For an insertion we have to move (elementCount-idx) words, starting at
** elementData[idx]:
**
**  elementData[idx]            ==> elementData[idx+1]
**  ...
**  elementData[elementCount-1] ==> elementData[elementCount]
**
**  For a deletion we have to move (elementCount-idx-1) words, starting at
** elementData[idx+1];
**
**  elementData[idx+1]          ==> elementData[idx]
**  ...
**  elementData[elementCount-1] ==> elementData[elementCount-2]
**
** We do this via an intermediate buffer because we are totally paranoid
** wrt other people's C libraries.
*/

  count = (w_size)thisVector[F_Vector_elementCount]-idx;
  start = data+idx;

  if (distance<0) {
    ++start;
    --count;
  }

  temp = allocMem(thread, count * sizeof(w_instance));
  memcpy(temp, start, count*sizeof(w_word));
  memcpy(start+distance, temp, count*sizeof(w_word));

  releaseMem(temp);
}


void Vector_copyInto(JNIEnv *env, w_instance thisVector, w_instance anArrayOfObjects) {

// copy this.Vector.elementData into anArrayOfObjects

  w_thread  thread = JNIEnv2w_thread(env); 
  w_instance thisarray; 
  w_instance *thisdata;
  w_instance *targetdata;
  w_int targetlength;
  w_int count;

  targetdata = instance2Array_instance(anArrayOfObjects);
  targetlength = (w_int)anArrayOfObjects[F_Array_length];
  count = thisVector[F_Vector_elementCount];

  if (count==0) return;

  if(count > targetlength) {
    throwException(thread, clazzArrayIndexOutOfBoundsException, NULL);
  }
  else {
    thisarray = (w_instance)thisVector[F_Vector_elementData];
    thisdata = instance2Array_instance(thisarray);

    memcpy(targetdata, thisdata, count * sizeof(w_word));
  }
}


