/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: Deflater.c,v 1.4 2002/03/19 10:31:04 gray Exp $
*/

#include <string.h>

#include "clazz.h"
#include "arrays.h"
#include "exception.h"
#include "heap.h"
#include "core-classes.h"
#include "deflate_driver.h"

void Deflater_updateLvl(JNIEnv *env, w_instance thisDeflater) {
  w_int lvl = thisDeflater[F_Deflater_level];
  w_device zstream = (w_device) thisDeflater[F_Deflater_wotsit];
  if (lvl < 0 ) {
     lvl = 8;
  }
  if(zstream){
    deviceBSSet(zstream, wdi_set_compression_level , (w_word)lvl , 1);
  }
}

void Deflater_create(JNIEnv *env, w_instance thisDeflater) {
  w_device zstream = deviceBSOpen("zip_", wdp_none);
  thisDeflater[F_Deflater_wotsit] = (w_int)zstream;
  Deflater_updateLvl(env, thisDeflater);
}

void Deflater_reset(JNIEnv *env, w_instance thisDeflater) {
  w_device zstream = (w_device) thisDeflater[F_Deflater_wotsit];
  if (zstream) {
    deviceBSSet(zstream, wdi_reset, 0, 1);
  }
}

void Deflater_finalize(JNIEnv *env, w_instance thisDeflater) {
  w_device zstream = (w_device) thisDeflater[F_Deflater_wotsit];
  woempa(6,"Finalizing Deflater (%p), zstream = %p\n",thisDeflater, zstream);
  if (zstream) {
    thisDeflater[F_Deflater_wotsit] = 0;
    deviceBSClose(zstream);
  }
}

void Deflater_end(JNIEnv *env, w_instance thisDeflater) {
  w_device zstream = (w_device) thisDeflater[F_Deflater_wotsit];
  if (zstream) {
    thisDeflater[F_Deflater_wotsit] = 0;
    deviceBSClose(zstream);
  }
}

void Deflater_setDictionary(JNIEnv *env, w_instance thisDeflater, w_instance byteArray, w_int off, w_int len) {
  w_thread thread = JNIEnv2w_thread(env);

  if (byteArray == nullInstance) {
    throwException(thread, clazzNullPointerException, NULL);
  }
  else {
    if (off < 0 || len < 0 || off+len > (w_int)byteArray[F_Array_length]) {
      throwException(thread, clazzArrayIndexOutOfBoundsException, NULL);
    }
    else {
      w_device zstream = (w_device)thisDeflater[F_Deflater_wotsit];
      if (zstream){
        w_sbyte * data = instance2Array_byte(byteArray);
        w_driver_status status;
        w_size l;

        data += off;
        while ((status = deviceBSSet(zstream, wdi_send_dictionary, 1, 1)) == wds_no_instance);
        if (status != wds_success) {
          woempa(9, "internal error occured while setting dictionary for %p\n",thisDeflater);
        }
        //the write will push the whole block in one go in the queue ...
        while ((status = deviceBSWrite(zstream, data, len, &l, 1)) == wds_no_instance);
        if (status != wds_success) {
          woempa(9, "internal error occured while setting dictionary for %p\n",thisDeflater);
        }
      }
    }
  }
}

void Deflater_setInput(JNIEnv *env, w_instance thisDeflater, w_instance byteArray, w_int off, w_int len) {
  w_device zstream = (w_device)thisDeflater[F_Deflater_wotsit];
  w_sbyte * data = instance2Array_byte(byteArray) + off;
  w_driver_status status;
  w_size l;

  woempa(6, "DEBUG setting input for %p \n",thisDeflater);
  while ((status = deviceBSWrite(zstream, data, len, &l, 10)) == wds_no_instance);
  if (status != wds_success) {
    woempa(9, "internal error occured while setting input for %p (data is lost)\n",thisDeflater);
  }
}


w_int Deflater_deflate(JNIEnv *env, w_instance thisDeflater, w_instance byteArray, w_int off, w_int len) {
  w_thread thread = JNIEnv2w_thread(env);
  w_int ret=0;

  if (!(thisDeflater[F_Deflater_wotsit])) {
    return 0;
  }
  if (byteArray == nullInstance) {
    throwException(thread, clazzNullPointerException, NULL);
  }
  else {
    if (off < 0 || len < 0 || off+len > (w_int)byteArray[F_Array_length]) {
      throwException(thread, clazzArrayIndexOutOfBoundsException, NULL);
    }
    else {
      w_device zstream = (w_device)thisDeflater[F_Deflater_wotsit];
      w_sbyte * data = instance2Array_byte(byteArray);
      w_driver_status status;
      w_size l;

      if(len == 0) {
        return ret;
      }
      data += off;

      while((status = deviceBSRead(zstream, data, len, &l, 10)) == wds_no_instance){
        w_word reply;

        while((status = deviceBSQuery(zstream, wdi_get_need_more_input, &reply, 1)) == wds_no_instance);
        if(status != wds_success){
          woempa(9, "ERROR OCCURED in deflate - query status...\n");
        }
        ret += l;
        if(reply){
          woempa(6, "DEBUG - deflater needs more input\n");
          thisDeflater[F_Deflater_needsInput] = WONKA_TRUE;
          return ret;
        }
        data += l;
        len -= l;
      }
      if (status == wds_data_exhausted) {
        woempa(6, "DEBUG - deflater inflated all bytes (EOF)\n");
        thisDeflater[F_Deflater_finished] = 2;
      }
      else if (status == wds_internal_error) {
        woempa(6, "DEBUG - deflater had an internal error\n");
      }
      ret += l;
    }
  }                    
  return ret;  
}

void Deflater_finish(JNIEnv *env, w_instance thisDeflater) {
  w_device zstream = (w_device) thisDeflater[F_Deflater_wotsit];
  if (zstream) {
    while (deviceBSSet(zstream, wdi_end_of_input, 0, 1) == wds_no_instance);
    thisDeflater[F_Deflater_finished] = 1;
  }
}
