/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: Inflater.c,v 1.7 2002/03/19 10:31:04 gray Exp $
*/

#include <string.h>

#include "clazz.h"
#include "arrays.h"
#include "exception.h"
#include "heap.h"
#include "core-classes.h"
#include "deflate_driver.h"

void Inflater_create(JNIEnv *env, w_instance thisInflater) {
  woempa(6,"DEBUG - creating new inflater\n");
  {
    w_device unzipper = deviceBSOpen("unzip_", wdp_none);
    woempa(6,"DEBUG - created new inflater (%p)\n",unzipper);
    while(deviceBSSet(unzipper, wdi_set_no_auto, 1, 1) == wds_no_instance);
    thisInflater[F_Inflater_wotsit] = (w_int)unzipper;
    woempa(6,"DEBUG - create finished - new inflater (%p)\n",unzipper);
  }
}

void Inflater_reset(JNIEnv *env, w_instance thisInflater) {
  w_device zstream = (w_device) thisInflater[F_Inflater_wotsit];
  if (zstream) {
    woempa(6, "reseting inflater\n");
    while(deviceBSSet(zstream, wdi_reset, 0, 1) == wds_no_instance);
  }
}

void Inflater_finalize(JNIEnv *env, w_instance thisInflater) {
  Inflater_end(env, thisInflater);
/* Inflater_destructor was:
  w_device zstream = (w_device) thisInflater[F_Inflater_wotsit];
  woempa(6,"Finalizing Inflater (%p), zstream = %p\n",thisInflater, zstream);
  if (zstream) {
    thisInflater[F_Inflater_wotsit] = 0;
    deviceBSClose(zstream);
  }
*/
}

void Inflater_end(JNIEnv *env, w_instance thisInflater) {
  woempa(6,"ending Inflater (%p)\n",thisInflater);
  {
    w_device zstream = (w_device) thisInflater[F_Inflater_wotsit];
    if (zstream) {
      w_driver_status status;
      w_word reply;

      while((status = deviceBSQuery(zstream, wdi_get_processed_size, &reply, 1)) == wds_no_instance);
      if(status != wds_success){
        woempa(9, "ERROR OCCURED in end - query status...\n");
      }
      thisInflater[F_Inflater_remain] = thisInflater[F_Inflater_totalIn] - (w_int)reply - thisInflater[F_Inflater_remain];
      thisInflater[F_Inflater_wotsit] = 0;
      deviceBSClose(zstream);
    }
  }
}

void Inflater_setDictionary(JNIEnv *env, w_instance thisInflater, w_instance byteArray, w_int off, w_int len) {
  w_thread thread = JNIEnv2w_thread(env);

  if (byteArray == nullInstance) {
    throwException(thread, clazzNullPointerException, NULL);
  }
  else {
    if (off < 0 || len < 0 || off+len > (w_int)byteArray[F_Array_length]) {
      throwException(thread, clazzArrayIndexOutOfBoundsException, NULL);
    }
    else {
      w_device zstream = (w_device)thisInflater[F_Inflater_wotsit];
      w_sbyte * data = instance2Array_byte(byteArray) + off;

      if (zstream) {
        w_size l;
        w_driver_status status;
        while ((status = deviceBSSet(zstream, wdi_send_dictionary, 1, 1)) == wds_no_instance);
        if (status != wds_success) {
          woempa(9, "internal error occured while setting dictionary for %p\n",thisInflater);
        }
        //the write will push the whole block in one go in the queue ...
        while ((status = deviceBSWrite(zstream, data, len, &l, 1)) == wds_no_instance);
        if (status == wds_success) {
          thisInflater[F_Inflater_needsDict] = WONKA_FALSE;
          thisInflater[F_Inflater_dictAdler] = 0;
        }
        else {
          woempa(9, "internal error occured while setting dictionary for %p\n",thisInflater);
        }
      }
    }
  }
}

void Inflater_setInput(JNIEnv *env, w_instance thisInflater, w_instance byteArray, w_int off, w_int len) {
  w_device zstream = (w_device)thisInflater[F_Inflater_wotsit];
  w_sbyte * data = instance2Array_byte(byteArray) + off;
  w_driver_status status;
  w_size l;

  woempa(6, "DEBUG setting input for %p \n",thisInflater);
  while ((status = deviceBSWrite(zstream, data, len, &l, 10)) == wds_no_instance);
  if (status != wds_success) {
    woempa(9, "internal error occured while setting input for %p (data is lost)\n",thisInflater);
  }
}

w_int Inflater_inflate(JNIEnv *env, w_instance thisInflater, w_instance byteArray, w_int off, w_int len) {
  w_thread thread = JNIEnv2w_thread(env);
  w_int ret=0;

  if (!(thisInflater[F_Inflater_wotsit])){
    return -1;
  }
  if (byteArray == nullInstance) {
    throwException(thread, clazzNullPointerException, NULL);
  }
  else {
    if (off < 0 || len < 0 || off+len > (w_int)byteArray[F_Array_length]) {
      throwException(thread, clazzArrayIndexOutOfBoundsException, NULL);
    }
    else {
      if (len == 0){
        return 0;
      }
      else{
        w_device zstream = (w_device)thisInflater[F_Inflater_wotsit];
        w_sbyte * data = instance2Array_byte(byteArray);
        w_driver_status status;
        w_size l;

        woempa(6, "DEBUG - inflating bytes\n");
        data += off;
        while((status = deviceBSRead(zstream, data, len, &l, x_millis2ticks(500))) == wds_no_instance){
          w_word reply;

          while((status = deviceBSQuery(zstream, wdi_get_need_more_input, &reply, 1)) == wds_no_instance);
          if(status != wds_success){
            woempa(9, "ERROR OCCURED in inflate - query status...\n");
          }
          ret += l;
          if(reply){
            woempa(6, "DEBUG - inflater needs more input\n");
            thisInflater[F_Inflater_needsInput] = WONKA_TRUE;
            return ret;
          }
          data += l;
          len -= l;
        }
        if (status == wds_data_exhausted) {
          woempa(6, "DEBUG - inflater inflated all bytes (EOF)\n");
          thisInflater[F_Inflater_finished] = WONKA_TRUE;
        }
        else if (status == wds_internal_error) {
          woempa(9, "DEBUG - inflater had an internal error\n");
          throwException(thread, clazzDataFormatException, NULL);
        }
        ret += l;
      }
    }
  }
  return ret;  
}

w_int Inflater_getRemaining(JNIEnv *env, w_instance thisInflater) {
  w_device zstream = (w_device) thisInflater[F_Inflater_wotsit];
  if (zstream) {
    w_driver_status status;
    w_word reply;
    w_int ret;
    while((status = deviceBSQuery(zstream, wdi_get_processed_size, &reply, 1)) == wds_no_instance);
    if(status != wds_success){
      woempa(9, "ERROR OCCURED in getRemaining - query status...\n");
    }
    ret = thisInflater[F_Inflater_totalIn] - (w_int)reply - thisInflater[F_Inflater_remain];
    woempa(6, "getRemaining returns %i (bytes processed %i, in = %i, rem = %i)\n",ret, reply,
      thisInflater[F_Inflater_totalIn],thisInflater[F_Inflater_remain]);
   return ret;
  }
  else {
    woempa(6, "getRemaining returns %i\n",thisInflater[F_Inflater_remain]);
    return thisInflater[F_Inflater_remain];
  }
}
