/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: arrays.c,v 1.2 2002/02/26 10:58:30 gray Exp $
** 
*/

#include <string.h>

#include "wonka.h"
#include "arrays.h"
#include "exception.h"
#include "clazz.h"
#include "descriptor.h"
#include "loading.h"
#include "hashtable.h"
#include "ts-mem.h"
#include "methods.h"
#include "strings.h"

/*
** These global variables hold the offset in the array instance of the length of the
** array and the offset to the first word of data in the array instance.
*/

w_int                         F_Array_length;
w_int                         F_Array_data;

/*
** The array handling functions...
** Create the clazz_Array handle that is the mother of all other array clazzes.
*/

static w_Method Array_clone_method;
static w_clazz  Array_interfaces[2];

w_clazz createClazzArray(void) {
  w_string string_clone = cstring2String(W_Thread_system,"clone",5);
  w_clazz clazz = allocClazz(W_Thread_system);
  w_method arrayclone = &Array_clone_method;
  w_size   i;

  memcpy(clazz, clazzObject, sizeof(w_Clazz));
  clazz->methods = allocMem(W_Thread_system, clazzObject->numMethods * sizeof(w_method));
  memcpy(clazz->methods, clazzObject->methods, (w_size)clazzObject->numMethods * sizeof(w_method));
  for (i = 0; i < clazz->numMethods; i++) {
    if (clazz->methods[i]->name == string_clone) {
      woempa(1, "Overriding 'clone' method of Object for Array 'clone', slot is %d.\n", i);
      memcpy(arrayclone, clazzObject->methods[i], sizeof(w_Method));
      /*
      ** The special array clone throws no check exceptions. So it behaves 
      ** like Object.clone but throws an InternalError.
      ** Also note that an array clone is shallow; so when a multidimensional
      ** array is cloned, only the highest dimension is cloned, not the lower
      ** ones.
      */
      arrayclone->function = (w_function)(w_void_fun)Array_clone;
      clazz->methods[i] = arrayclone;
      break;
    }
  }

  setFlag(clazz->flags, ACC_ABSTRACT | ACC_FINAL);
  clazz->numInterfaces = 2;
  // TODO: the following should become static memory declared and defined at the top of this file...
  clazz->interfaces = Array_interfaces;
  clazz->interfaces[0] = clazzCloneable;
  clazz->interfaces[1] = clazzSerializable;
  clazz->superClazz = clazzObject;

  return clazz;

}

w_instance Array_clone(JNIEnv *env, w_instance thisObject) {

  w_thread thread = JNIEnv2w_thread(env);
  w_instance theClone;

  theClone =  cloneArray(thread, thisObject);

  return theClone;

}

/*
** To clone an array, we first create an array instance of one dimension,
** with the same clazz and length as the original.  We then copy the length
** and data (i.e., (size in words + 1) words) from the original instance to
** the cloned instance.
*/

w_instance cloneArray(w_thread thread, w_instance originalArrayInstance) {

  w_clazz clazz = instance2clazz(originalArrayInstance);
//  w_Item  length;
  w_word length;
  w_int   bitsPerWord = sizeof(w_word)*8;
  w_int   sizeInBeets;
  w_int   sizeInWoids;
  w_int   cloneSize;
  w_instance clonedArrayInstance;

  length = (w_int)originalArrayInstance[F_Array_length];
  sizeInBeets = (clazz->aclazz->bits)*length;
  sizeInWoids = (sizeInBeets+bitsPerWord-1)/bitsPerWord;
  cloneSize   = (sizeInWoids+1)*sizeof(w_word);
  clonedArrayInstance = allocArrayInstance(thread, clazz, 1, &length);

  if (clonedArrayInstance) {
    woempa(1, "length %d.\n", length);
    memcpy(clonedArrayInstance, originalArrayInstance, (w_size)cloneSize);
  }
  else {
    throwOutOfMemoryError(thread);
  }

  return clonedArrayInstance;
}

w_desc getComponent(w_instance Array) {

  w_clazz clazz = instance2clazz(Array);
  w_desc component = NULL;
  
  if (clazz->aclazz) {
    component = clazz->aclazz->component;
  }
  
  return component;

}
