/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: chars.c,v 1.2 2002/01/24 12:56:09 gray Exp $
*/

#include "chars.h"
#include "ts-mem.h"
#include "unicode.h"

/*
** Routines for dealing with Unicode
*/

w_int char2class(int ch) {

  return CHAR_CLASS(ch);

}

w_int char2digit(int ch) {

  return CHAR_SUBCLASS(ch);

}

w_boolean charIsDefined(int ch) {

  return CHAR_CLASS(ch) != NONESUCH;

}

w_boolean charIsLower(int ch) {

  return CHAR_CLASS(ch) == LOWERCASE;

}

w_boolean charIsUpper(int ch) {

  return CHAR_CLASS(ch) == UPPERCASE;

}

w_boolean charIsTitle(int ch) {

  return CHAR_CLASS(ch) == TITLECASE;

}

w_boolean charIsLetter(int ch) {

  return CHAR_CLASS(ch) > NONESUCH && CHAR_CLASS(ch) < DIGIT;

}

w_boolean charIsDigit(int ch) {

  return CHAR_CLASS(ch) == DIGIT;

}

w_boolean charIsFormat(int ch) {

  return CHAR_CLASS(ch) == FORMAT;

}

w_boolean charIsWhitespace(int ch) {

  return CHAR_CLASS(ch) == WHITESPACE;

}

/*
** Look up a conversion in one of the tables of special cases
*/

static w_char lookupconversion(int ch, const w_char_conversion tab[]) {

  const w_char_conversion *chcon = tab;

  while (chcon->from != ch) {

    if (chcon->from == 0) {
      return ch;
    }
    chcon++;
  }

  return chcon->to;

}
 
/*
** char2lower returns the lower-case equivalent of ch if there is one, 
** or ch (unchanged) if there is not.
*/

w_char char2lower(int ch) {

  w_int cc = CHAR_TABLE_ENTRY(ch);
  w_int hi = cc / 16;
  w_int lo = cc % 16;

  switch (hi) {
    case UPPERCASE:
    case TITLECASE:
      switch(lo) {
        case  0: return ch;
        case 15: return lookupconversion(ch, to_lower_case);
        default: return ch + upper_to_lower_delta[lo];
      }
    default: return ch;
  }

}

/*
** char2upper returns the upper-case equivalent of ch if there is one, 
** or ch (unchanged) if there is not.
*/

w_char char2upper(int ch) {

  w_int cc = CHAR_TABLE_ENTRY(ch);
  w_int hi = cc / 16;
  w_int lo = cc % 16;

  switch(hi) {
    case LOWERCASE:
    case TITLECASE:
      switch (lo) {
        case  0: return ch;
        case 15: return lookupconversion(ch, to_upper_case);
        default: return ch - upper_to_lower_delta[lo];
      }
    default: return ch;
  }

}

/*
** char2title returns the title-case equivalent of ch if there is one, 
** or ch (unchanged) if there is not.
*/

w_char char2title(int ch) {

  w_int cc = CHAR_TABLE_ENTRY(ch);
  w_int hi = cc / 16;

  switch (hi) {
    case LOWERCASE:
    case UPPERCASE: {
        w_char t = lookupconversion(ch, to_title_case);

        if (t != ch) {
          return t;
        }

        return char2upper(ch);

   }

    default: return ch;

  }

}
