/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: dispatcher.c,v 1.14 2002/03/27 11:47:05 gray Exp $
*/

// TODO, we should be able to get rid of clazz2Class in the calls, try it...

#include <string.h>

#include "clazz.h"
#include "dispatcher.h"
#include "strings.h"
#include "threads.h"
#include "methods.h"
#include "descriptor.h"
#include "stack.h"
#include "exception.h"
#include "locks.h"
#include "hashtable.h"
#include "misc.h"

/*
** Include the CPU specific stack contruction declarations.
*/

#include "calls.h"

/*
** --------------------------------------------------------------------------------------------------------------------------
**
**                                     The JAVA method call/return functions.
**
** The functions are declared here only. They are defined in stack.c where the Java stack operations are also defined.
**
** --------------------------------------------------------------------------------------------------------------------------
*/

/*
** (JAVA - CALL) The synchronised instance method, both normal and profiled functions.
*/

extern void cj_ins_syn   (w_frame * frameref, w_code * coderef, w_method method);

extern void cj_ins_syn_P (w_frame * frameref, w_code * coderef, w_method method);

/*
** (JAVA - CALL) The NON synchronised instance method, both normal and profiled functions.
*/

extern void cj_ins_uns   (w_frame * frameref, w_code * coderef, w_method method);

extern void cj_ins_uns_P (w_frame * frameref, w_code * coderef, w_method method);

/*
** (JAVA - CALL) The synchronised static method, both normal and profiled functions.
*/

extern void cj_stc_syn   (w_frame * frameref, w_code * coderef, w_method method);

extern void cj_stc_syn_P (w_frame * frameref, w_code * coderef, w_method method);

/*
** (JAVA - CALL) The NON synchronised static method, both normal and profiled functions.
*/

extern void cj_stc_uns   (w_frame * frameref, w_code * coderef, w_method method);

extern void cj_stc_uns_P (w_frame * frameref, w_code * coderef, w_method method);

/*
** (JAVA - RETURN) The synchronised method, both normal and profiled functions.
*/

extern void rj_xxx_syn_V   (w_frame * frameref, w_code * coderef);

extern void rj_xxx_syn_V_P (w_frame * frameref, w_code * coderef);

extern void rj_xxx_syn_W   (w_frame * frameref, w_code * coderef);

extern void rj_xxx_syn_W_P (w_frame * frameref, w_code * coderef);

extern void rj_xxx_syn_D   (w_frame * frameref, w_code * coderef);

extern void rj_xxx_syn_D_P (w_frame * frameref, w_code * coderef);

/*
** (JAVA - RETURN) The NON synchronised method, both normal and profiled functions.
*/

extern void rj_xxx_uns_V   (w_frame * frameref, w_code * coderef);

extern void rj_xxx_uns_V_P (w_frame * frameref, w_code * coderef);

extern void rj_xxx_uns_W   (w_frame * frameref, w_code * coderef);

extern void rj_xxx_uns_W_P (w_frame * frameref, w_code * coderef);

extern void rj_xxx_uns_D   (w_frame * frameref, w_code * coderef);

extern void rj_xxx_uns_D_P (w_frame * frameref, w_code * coderef);

/*
** --------------------------------------------------------------------------------------------------------------------------
**
**                                     The native method call only functions.
**
** --------------------------------------------------------------------------------------------------------------------------
*/

/*
** Helper functions to prepare a frame for calling a native function.
*/

void throwLinkageError(w_frame *frameptr, w_code *codeptr, w_method method) {
  w_frame  old_frame = *frameptr;
  w_code   old_code  = *codeptr;
  w_thread thread = old_frame->thread;
  w_instance error = allocInstance(thread, clazzLinkageError);
  w_instance Thread = thread->Thread;


  if (error==NULL) {
    wabort(ABORT_WONKA,"Aaargh! Couldn't allocate a LinkageError\n");
  }
  else if (Thread==NULL) {
    wabort(ABORT_WONKA,"Aw shucks, wanted to throw a LinkageError but no java.lang.Thread around to throw it in\n");
  }
  woempa(7, "Throwing LinkageError wrt %M\n", method);
  setReferenceField(error, getStringInstance(thread,method->name), F_Throwable_detailMessage);
  setReferenceField(Thread, error, F_Thread_thrown);
  (*frameptr)->pc = *codeptr - 1 - (*frameptr)->method->code;
  *frameptr = handleThrowable(error,old_frame,old_code,codeptr);
  woempa(9,"New frame is %p, code pointer is %p\n",*frameptr,*codeptr);
}

w_ubyte *mangleName(w_word start, w_word stop, w_string source, w_ubyte *destination) {
  w_ubyte *iter = destination;
  w_word i;
  w_char c;
  w_ubyte hex[5];

  for(i = start; i < stop; i++) {
    c = source->chars[i];
    if(c & 0xFF80) { /* unicode */
      sprintf((w_ubyte *)&hex, "%x", c);
      *(iter++) = '_';
      *(iter++) = '0';
      strcpy(iter, (w_ubyte *)&hex);
      iter += 4;
    } else {
      switch(c & 0x00FF) {
        case '.' : *(iter++) = '_'; break ;
        case '/' : *(iter++) = '_'; break ;
        case '[' : *(iter++) = '_'; *(iter++) = '3'; break;
        case ';' : *(iter++) = '_'; *(iter++) = '2'; break;
        case '_' : *(iter++) = '_'; *(iter++) = '1'; break ;
        default  : *(iter++) = c & 0x00FF;
      }
    }
  }
  
  return iter;
}

void searchNativeMethod(w_method method) {
  
  /* 
  ** Allocate memory for the JNI names :
  ** - short : e.g Java_java_lang_String_hashcode is fairly easy, we need 5 (Java_) + 1 
  **           (the _ between class & method) + 1 (\0 at the end) = 7 extra bytes + a few
  **           more for mangling
  ** - long  : Different story because [ is changed in _3 and ; is changed in _2. About 25
  **           extra bytes should cover it, but this is dangerous and should be changed
  */
  
  w_ubyte *jni_short = allocMem_system(7 + method->clazz->dotified->length + method->name->length + 25);
  w_ubyte *jni_long = allocMem_system(7 + method->clazz->dotified->length + method->name->length + method->param_list->length + method->return_type->dotified->length + 25);
  w_ubyte *iter = jni_short;
  
  strcpy(jni_short, "Java_");  /* First put in the Java_ at the beginning */
  iter += 5;                   /* Skip 5 characters */

  /*
  ** Go over the class (+package) name and add those to the jni name. Also change /'s into _'s 
  */

  iter = mangleName(0, method->clazz->dotified->length, method->clazz->dotified, iter);
  
  *(iter++) = '_';             /* The _ between class & method */
  
  /*
  ** Go over the method name and add those to the jni name. 
  */
  
  iter = mangleName(0, method->name->length, method->name, iter);
  
  *(iter++) = '\0';            /* End of the string */

  woempa(9, "JNI short : '%s'\n", jni_short);

  /*
  ** For the full blown name, we start from the short name and add the parameters
  */
  
  strcpy(jni_long, jni_short); /* Make a copy */
  
  iter = (w_ubyte *)(jni_long + strlen(jni_short));  /* Update our iteration pointer */
  *(iter++) = '_';             /* Add the 2 _'s between method and parameter */
  *(iter++) = '_';

  /*
  ** Go over the parameters and add those to the jni name. Also change /'s in _'s, 
  ** ;'s in _2's and ['s in _3's
  */
  
  iter = mangleName(1, method->param_list->length-1, method->param_list, iter);

  *(iter++) = '\0';            /* End of the string */
  
  woempa(9, "JNI long :  '%s'\n", jni_long);

  method->function.void_fun = lookupModuleSymbol(jni_long);
  if (method->function.void_fun == NULL) method->function.void_fun = lookupModuleSymbol(jni_short);
    
  releaseMem(jni_short);       /* Be a good boy and return the toys after we */
  releaseMem(jni_long);        /* finnished playing */
}

inline static w_boolean // TRUE if everything OK, FALSE if exception thrown
prepareCall(w_frame *frameptr, w_code *codeptr, w_method method) {
  if (method->function.void_fun == NULL) {
    woempa(9,"Oh deary me: method %M has no code -> Trying to look it up\n",method);
    searchNativeMethod(method);
    if (method->function.void_fun == NULL) {
      woempa(9,"Oh deary me: method %M has no code\n",method);
      throwLinkageError(frameptr,codeptr,method);

      return WONKA_FALSE;
    }
  }
  (*frameptr)->native = method;  
  unsetFlag(method->flags, METHOD_NEEDS_LINKING);
  setFlag((*frameptr)->flags, FRAME_NATIVE);
  
  return WONKA_TRUE;
}

/*
** Helper function for clearing the local references, setting the status and preparing the 
** frame fields for a return. Since 90% of most native calls don't produce any local references,
** we check for the presence of the local table here and not in removeAllLocalReferences itself.
** We only remove the local reference table if there was at list a first local reference.
*/

inline static void prepareReturn(w_frame frame) {

  if (frame->firstRef) {
    ht_deregister(globals_hashtable, (w_word)frame->firstRef);
    woempa(1,"UnGREENing %k at %p (is frame->firstRef)\n", instance2clazz(frame->firstRef),frame->firstRef);
    unsetFlag(instance2flags(frame->firstRef),O_GREEN);
    frame->firstRef = NULL;
    if (frame->localRefs) {
      removeAllLocalReferences(frame);
    }
  }

  frame->native = NULL;  
  unsetFlag(frame->flags, FRAME_NATIVE);

}

/*
** (NATIVE) The NON synchronised instance methods, both normal and profiled functions.
*/

static void na_ins_uns_V (w_frame * frameref, w_code * coderef, w_method method) {

  w_frame frame = * frameref;
  w_word dummy;

//  woempa(9, "VOID CALLING top = %d   %w %w\n", frame->o_top, method->name, method->desc->name);

  if (prepareCall(frameref, coderef, method)) {

    call_instance(w_thread2JNIEnv(frame->thread), method, frame->o_top, &dummy);
    frame->o_top -= method->arg_i;
  
    prepareReturn(frame);
  }
}

static void na_ins_uns_V_P (w_frame * frameref, w_code * coderef, w_method method) {
  wabort(ABORT_WONKA, "Not implemented yet.\n");
}

static void na_ins_uns_W (w_frame * frameref, w_code * coderef, w_method method) {

  w_frame frame = * frameref;
  w_word dummy;
  w_word result;

//  woempa(9, "WORD CALLING top = %d   %w %w\n", frame->o_top, method->name, method->desc->name);

  if (prepareCall(frameref, coderef, method)) {

    result = call_instance(w_thread2JNIEnv(frame->thread), method, frame->o_top, &dummy);
    frame->o_top -= method->arg_i;
    frame->o_top[0] = result;
    frame->o_top += 1;
  
    prepareReturn(frame);
  }
}

static void na_ins_uns_W_P (w_frame * frameref, w_code * coderef, w_method method) {
  wabort(ABORT_WONKA, "Not implemented yet.\n");
}

static void na_ins_uns_D (w_frame * frameref, w_code * coderef, w_method method) {

  w_frame frame = * frameref;
  w_word dummy;
  w_u64 result;

//  woempa(9, "DOUBLE WORD CALLING top = %d   %w %w\n", frame->o_top, method->name, method->desc->name);

  if (prepareCall(frameref, coderef, method)) {

    result.u64 = call_instance(w_thread2JNIEnv(frame->thread), method, frame->o_top, &dummy);
    frame->o_top -= method->arg_i;
    frame->o_top[0] = result.words[WORD_LSW];
    frame->o_top[1] = result.words[WORD_MSW];
    frame->o_top += 2;
  
    prepareReturn(frame);
  }
}

static void na_ins_uns_D_P (w_frame * frameref, w_code * coderef, w_method method) {
  wabort(ABORT_WONKA, "Not implemented yet.\n");
}

/*
** (NATIVE) The NON synchronised static methods, both normal and profiled functions.
*/

static void na_stc_uns_V (w_frame * frameref, w_code * coderef, w_method method) {

  w_frame frame = * frameref;
  w_word dummy;

//  woempa(9, "STATIC VOID CALLING arg_i = %d   %w %w\n", method->arg_i, method->name, method->desc->name);

  if (prepareCall(frameref, coderef, method)) {

    call_static(w_thread2JNIEnv(frame->thread), method, frame->o_top, &dummy, clazz2Class(method->clazz));
    frame->o_top -= method->arg_i;
  
    prepareReturn(frame);
  }
}

static void na_stc_uns_V_P (w_frame * frameref, w_code * coderef, w_method method) {
  wabort(ABORT_WONKA, "Not implemented yet.\n");
}

static void na_stc_uns_W (w_frame * frameref, w_code * coderef, w_method method) {

  w_frame frame = * frameref;
  w_word dummy;
  w_word result;

//  woempa(9, "STATIC WORD CALLING arg_i = %d   %w %w\n", method->arg_i, method->name, method->desc->name);

  if (prepareCall(frameref, coderef, method)) {

    result = call_static(w_thread2JNIEnv(frame->thread), method, frame->o_top, &dummy, clazz2Class(method->clazz));
    frame->o_top -= method->arg_i;
    frame->o_top[0] = result;
    frame->o_top += 1;
  
    prepareReturn(frame);
  }
}

static void na_stc_uns_W_P (w_frame * frameref, w_code * coderef, w_method method) {
  wabort(ABORT_WONKA, "Not implemented yet.\n");
}

static void na_stc_uns_D (w_frame * frameref, w_code * coderef, w_method method) {

  w_frame frame = * frameref;
  w_word dummy;
  w_u64 result;

//  woempa(9, "STATIC DOUBLE WORD CALLING arg_i = %d   %w %w\n", method->arg_i, method->name, method->desc->name);

  if (prepareCall(frameref, coderef, method)) {

    result.u64 = call_static(w_thread2JNIEnv(frame->thread), method, frame->o_top, &dummy, clazz2Class(method->clazz));
    frame->o_top -= method->arg_i;
    frame->o_top[0] = result.words[WORD_LSW];
    frame->o_top[1] = result.words[WORD_MSW];
    frame->o_top += 2;
  
    prepareReturn(frame);
  }
}

static void na_stc_uns_D_P (w_frame * frameref, w_code * coderef, w_method method) {
  wabort(ABORT_WONKA, "Not implemented yet.\n");
}

/*
** (NATIVE) The synchronised instance methods, both normal and profiled functions.
*/

static void na_ins_syn_V (w_frame * frameref, w_code * coderef, w_method method) {

  w_frame frame = * frameref;
  w_instance instance = (w_instance) (frame->o_top[ - method->arg_i ]);
  w_word dummy;

//  woempa(9, "SYNCH %d %w %w\n", method->arg_i, method->name, method->desc->name);

  frame->thread->state = W_THREAD_LOCKING;
  enterMonitor(instance);
  frame->thread->state = W_THREAD_READY;

  if (prepareCall(frameref, coderef, method)) {

    call_instance(w_thread2JNIEnv(frame->thread), method, frame->o_top, &dummy);
    frame->o_top -= method->arg_i;

    prepareReturn(frame);
  }
  
  exitMonitor(instance);
}

static void na_ins_syn_V_P (w_frame * frameref, w_code * coderef, w_method method) {
  wabort(ABORT_WONKA, "Not implemented yet.\n");
}

static void na_ins_syn_W (w_frame * frameref, w_code * coderef, w_method method) {

  w_frame frame = * frameref;
  w_instance instance = (w_instance) (frame->o_top[ - method->arg_i ]);
  w_word dummy;
  w_word result;

//  woempa(9, "SYNCH %d %w %w\n", method->arg_i, method->name, method->desc->name);
  frame->thread->state = W_THREAD_LOCKING;
  enterMonitor(instance);
  frame->thread->state = W_THREAD_READY;

  if (prepareCall(frameref, coderef, method)) {

    result = call_instance(w_thread2JNIEnv(frame->thread), method, frame->o_top, &dummy);
    frame->o_top -= method->arg_i;
    frame->o_top[0] = result;
    frame->o_top += 1;
  
    prepareReturn(frame);
   } 

  exitMonitor(instance);
}

static void na_ins_syn_W_P (w_frame * frameref, w_code * coderef, w_method method) {
  wabort(ABORT_WONKA, "Not implemented yet.\n");
}

static void na_ins_syn_D (w_frame * frameref, w_code * coderef, w_method method) {

  w_frame frame = * frameref;
  w_instance instance = (w_instance) (frame->o_top[ - method->arg_i ]);
  w_word dummy;
  w_u64 result;

//  woempa(9, "SYNCH %d %w %w\n", method->arg_i, method->name, method->desc->name);
  frame->thread->state = W_THREAD_LOCKING;
  enterMonitor(instance);
  frame->thread->state = W_THREAD_READY;

  if (prepareCall(frameref, coderef, method)) {

    result.u64 = call_instance(w_thread2JNIEnv(frame->thread), method, frame->o_top, &dummy);
    frame->o_top -= method->arg_i;
    frame->o_top[0] = result.words[WORD_LSW];
    frame->o_top[1] = result.words[WORD_MSW];
    frame->o_top += 2;
    
    prepareReturn(frame);
  }

  exitMonitor(instance);
}

static void na_ins_syn_D_P (w_frame * frameref, w_code * coderef, w_method method) {
  wabort(ABORT_WONKA, "Not implemented yet.\n");
}


/*
** (NATIVE) The synchronised static methods, both normal and profiled functions.
*/

static void na_stc_syn_V (w_frame * frameref, w_code * coderef, w_method method) {

  w_frame frame = * frameref;
  w_word dummy;
  w_instance instance = clazz2Class(method->clazz);

//  woempa(9, "SYNCH %d %w %w\n", method->arg_i, method->name, method->desc->name);
  frame->thread->state = W_THREAD_LOCKING;
  enterMonitor(instance);
  frame->thread->state = W_THREAD_READY;

  if (prepareCall(frameref, coderef, method)) {

    call_static(w_thread2JNIEnv(frame->thread), method, frame->o_top, &dummy, clazz2Class(method->clazz));
    frame->o_top -= method->arg_i;
  
    prepareReturn(frame);
  }

  exitMonitor(instance);
}

static void na_stc_syn_V_P (w_frame * frameref, w_code * coderef, w_method method) {
  wabort(ABORT_WONKA, "Not implemented yet.\n");
}

static void na_stc_syn_W (w_frame * frameref, w_code * coderef, w_method method) {

  w_frame frame = * frameref;
  w_word dummy;
  w_word result;
  w_instance instance = clazz2Class(method->clazz);

//  woempa(9, "SYNCH %d %w %w\n", method->arg_i, method->name, method->desc->name);
  frame->thread->state = W_THREAD_LOCKING;
  enterMonitor(instance);
  frame->thread->state = W_THREAD_READY;

  if (prepareCall(frameref, coderef, method)) {

    result = call_static(w_thread2JNIEnv(frame->thread), method, frame->o_top, &dummy, clazz2Class(method->clazz));
    frame->o_top -= method->arg_i;
    frame->o_top[0] = result;
    frame->o_top += 1;
  
    prepareReturn(frame);
  }

  exitMonitor(instance);
}

static void na_stc_syn_W_P (w_frame * frameref, w_code * coderef, w_method method) {
  wabort(ABORT_WONKA, "Not implemented yet.\n");
}

static void na_stc_syn_D (w_frame * frameref, w_code * coderef, w_method method) {

  w_frame frame = * frameref;
  w_word dummy;
  w_u64 result;
  w_instance instance = clazz2Class(method->clazz);

//  woempa(9, "SYNCH %d %w %w\n", method->arg_i, method->name, method->desc->name);
  frame->thread->state = W_THREAD_LOCKING;
  enterMonitor(instance);
  frame->thread->state = W_THREAD_READY;

  if (prepareCall(frameref, coderef, method)) {

    result.u64 = call_static(w_thread2JNIEnv(frame->thread), method, frame->o_top, &dummy, clazz2Class(method->clazz));
    frame->o_top -= method->arg_i;
    frame->o_top[0] = result.words[WORD_LSW];
    frame->o_top[1] = result.words[WORD_MSW];
    frame->o_top += 2;
  
    prepareReturn(frame);
  }

  exitMonitor(instance);
}

static void na_stc_syn_D_P (w_frame * frameref, w_code * coderef, w_method method) {
  wabort(ABORT_WONKA, "Not implemented yet.\n");
}


/*
** The dispatcher matrix built as indicated by the drawing.
*/

w_Dispatcher dispatchers[12][6] = {

  /* 
  ** +----------------------- NATIVE -----------------------------+  +--------------------------- BYTE CODE ------------------------------------+  +------------------------- J-SPOT ---------------------------+
  ** |                                                            |  |                                                                          |  |                                                            |
  */
  
  {  { na_ins_syn_V_P, NULL,  0, 0 }, { na_ins_syn_V, NULL,  0, 1 }, { cj_ins_syn_P, rj_xxx_syn_V_P,  0, 2 }, { cj_ins_syn, rj_xxx_syn_V,  0, 3 }, { NULL, NULL,  0, 4 }, { na_ins_syn_V, rj_xxx_syn_V,  0, 5 }, },
  {  { na_ins_syn_W_P, NULL,  1, 0 }, { na_ins_syn_W, NULL,  1, 1 }, { cj_ins_syn_P, rj_xxx_syn_W_P,  1, 2 }, { cj_ins_syn, rj_xxx_syn_W,  1, 3 }, { NULL, NULL,  1, 4 }, { na_ins_syn_W, rj_xxx_syn_W,  1, 5 }, },
  {  { na_ins_syn_D_P, NULL,  2, 0 }, { na_ins_syn_D, NULL,  2, 1 }, { cj_ins_syn_P, rj_xxx_syn_D_P,  2, 2 }, { cj_ins_syn, rj_xxx_syn_D,  2, 3 }, { NULL, NULL,  2, 4 }, { na_ins_syn_D, rj_xxx_syn_D,  2, 5 }, },

  {  { na_ins_uns_V_P, NULL,  3, 0 }, { na_ins_uns_V, NULL,  3, 1 }, { cj_ins_uns_P, rj_xxx_uns_V_P,  3, 2 }, { cj_ins_uns, rj_xxx_uns_V,  3, 3 }, { NULL, NULL,  3, 4 }, { na_ins_uns_V, rj_xxx_uns_V,  3, 5 }, },
  {  { na_ins_uns_W_P, NULL,  4, 0 }, { na_ins_uns_W, NULL,  4, 1 }, { cj_ins_uns_P, rj_xxx_uns_W_P,  4, 2 }, { cj_ins_uns, rj_xxx_uns_W,  4, 3 }, { NULL, NULL,  4, 4 }, { na_ins_uns_W, rj_xxx_uns_W,  4, 5 }, },
  {  { na_ins_uns_D_P, NULL,  5, 0 }, { na_ins_uns_D, NULL,  5, 1 }, { cj_ins_uns_P, rj_xxx_uns_D_P,  5, 2 }, { cj_ins_uns, rj_xxx_uns_D,  5, 3 }, { NULL, NULL,  5, 4 }, { na_ins_uns_D, rj_xxx_uns_D,  5, 5 }, },

  {  { na_stc_syn_V_P, NULL,  6, 0 }, { na_stc_syn_V, NULL,  6, 1 }, { cj_stc_syn_P, rj_xxx_syn_V_P,  6, 2 }, { cj_stc_syn, rj_xxx_syn_V,  6, 3 }, { NULL, NULL,  6, 4 }, { na_stc_syn_V, rj_xxx_syn_V,  6, 5 }, },
  {  { na_stc_syn_W_P, NULL,  7, 0 }, { na_stc_syn_W, NULL,  7, 1 }, { cj_stc_syn_P, rj_xxx_syn_W_P,  7, 2 }, { cj_stc_syn, rj_xxx_syn_W,  7, 3 }, { NULL, NULL,  7, 4 }, { na_stc_syn_W, rj_xxx_syn_W,  7, 5 }, },
  {  { na_stc_syn_D_P, NULL,  8, 0 }, { na_stc_syn_D, NULL,  8, 1 }, { cj_stc_syn_P, rj_xxx_syn_D_P,  8, 2 }, { cj_stc_syn, rj_xxx_syn_D,  8, 3 }, { NULL, NULL,  8, 4 }, { na_stc_syn_D, rj_xxx_syn_D,  8, 5 }, },

  {  { na_stc_uns_V_P, NULL,  9, 0 }, { na_stc_uns_V, NULL,  9, 1 }, { cj_stc_uns_P, rj_xxx_uns_V_P,  9, 2 }, { cj_stc_uns, rj_xxx_uns_V,  9, 3 }, { NULL, NULL,  9, 4 }, { na_stc_uns_V, rj_xxx_uns_V,  9, 5 }, },
  {  { na_stc_uns_W_P, NULL, 10, 0 }, { na_stc_uns_W, NULL, 10, 1 }, { cj_stc_uns_P, rj_xxx_uns_W_P, 10, 2 }, { cj_stc_uns, rj_xxx_uns_W, 10, 3 }, { NULL, NULL, 10, 4 }, { na_stc_uns_W, rj_xxx_uns_W, 10, 5 }, },
  {  { na_stc_uns_D_P, NULL, 11, 0 }, { na_stc_uns_D, NULL, 11, 1 }, { cj_stc_uns_P, rj_xxx_uns_D_P, 11, 2 }, { cj_stc_uns, rj_xxx_uns_D, 11, 3 }, { NULL, NULL, 11, 4 }, { na_stc_uns_D, rj_xxx_uns_D, 11, 5 }, }
};

/*
** I can't get this inline stuff to work when the -fomit-frame-pointer flag is given. Otherwise,
** it works. For the moment, we rely on a call_instance function being available in calls.s of the
** machine directory.
*/

inline static void _call_instance_V (void * env, w_method method, w_word * top, w_word * dummy) {

  w_word stack_words = method->arg_i + 2; /* Account for the hidden JNIEnv argument and dummy pointer. */
  w_word * call;
  
  call = (w_word *)method->function.void_fun;
  
  asm volatile(
    "pushl %0\n\t"           /* put dummy pointer on stack */
    "cs_loop:\n\t"
    "subl $4, %%eax\n\t"     /* Decrement stack item pointer. Is OK at entry since stack pointer is 1 off. */
    "pushl (%%eax)\n\t"      /* Push data on the X86 stack. */
    "decl %%edx\n\t"         /* Decrement argument counter. */
    "jnz cs_loop\n\t"        /* If more to push, push... */
    "pushl %5\n\t"           /* Push JNIEnv pointer on stack. */
    "call *%%ecx\n\t"        /* Call the function. */
    "addl %1, %%esp\n\t"     /* Restore stack pointer as it was before we started pushing... */
    : /*  outputs  */
    : /*  inputs   */
      "m" (dummy),
      "m" (stack_words),
      "d" (method->arg_i),   /* Move method argument count in %edx. */
      "a" (top),             /* Move stack top in %eax. */
      "c" (call),            /* Move function pointer into %ecx. */
      "m" (env)
  );

}
