/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: jni.c,v 1.23 2002/03/26 10:44:15 gray Exp $
*/

#include <string.h>
#include <stdio.h>

#include "jni.h"

#include "arrays.h"
#include "bar.h"
#include "clazz.h"
#include "core-classes.h"
#include "descriptor.h"
#include "fields.h"
#include "hashtable.h"
#include "heap.h"
#include "ieee754.h"
#include "interpreter.h"
#include "loading.h"
#include "strings.h"
#include "threads.h"
#include "stack.h"
#include "checks.h"
#include "locks.h"
#include "exception.h"
#include "ts-mem.h"
#include "methods.h"
#include "misc.h"

w_hashtable globals_hashtable;

#define GLOBALS_HASHTABLE_SIZE   1439

#ifdef RUDOLPH
#ifndef MODULES
extern void init_awt(void);
#endif
#endif

struct JavaVMInitArgs *system_vm_args;

typedef union JNITypes {
  jboolean  z;
  jbyte     b;
  jchar     c;
  jshort    s;
  jint      i;
  jfloat    f;
  jlong     j;
  jdouble   d;
  jobject   o;
} JNITypes;

/*
** Write a C array of booleans (stored one per byte) into a Java array instance.
*/

void writeBooleansToArray(w_instance Array, w_int start, w_int length, jboolean *elements) {
  w_int    i;
  w_sbyte *  data_8;
  w_byte   bitmask;

  /*
  ** Find the correct offset (start / 8) and the correct bitposition
  ** (1 << (start % 8)) to start with...
  */

  data_8 = instance2Array_byte(Array) + (int)(start / 8);
  bitmask = 1;
  if (start % 8) bitmask <<= (start % 8);
  for (i = 0; i < length; i++) {
    if (elements[i]) {
      (*data_8) |=  bitmask;   /* set bit TRUE */
    }
    else {
      (*data_8) &= ~bitmask;   /* set bit FALSE */
    }
    
    bitmask <<= 1;
    if ((bitmask&0xff) == 0) {
      data_8 += 1;
      bitmask = 1;
    }
  }
  woempa(1, "(JNI) Wrote %d boolean elements from %d of Array %p.\n", length, start, Array);

}

/*
** Write a C array of bytes back into a Java array instance.
*/

void writeBytesToArray(w_instance Array, w_int start, w_int length, jbyte *elements) {
  w_sbyte *  dest = instance2Array_byte(Array) + start;

  memcpy(dest,elements,length*sizeof(w_byte));
  woempa(1, "(JNI) Wrote %d 8 bit elements from %d of Array %p.\n", length, start, Array);
}

/*
** Write a C array of shorts back into a Java array instance.
*/

void writeShortsToArray(w_instance Array, w_int start, w_int length, jshort *elements) {
  w_short * dest = instance2Array_short(Array) + start;

  memcpy(dest,elements,length*sizeof(w_short));
  woempa(1, "(JNI) Wrote %d 16 bit elements from %d of Array %p.\n", length, start, Array);
}

/*
** Write a C array of chars back into a Java array instance.
*/

void writeCharsToArray(w_instance Array, w_int start, w_int length, jchar *elements) {
  w_char* dest = instance2Array_char(Array) + start;

  memcpy(dest,elements,length*sizeof(w_char));
  woempa(1, "(JNI) Wrote %d 16 bit elements from %d of Array %p.\n", length, start, Array);
}

/*
** Write a C array of words back into a Java array instance.
*/

void writeIntsToArray(w_instance Array, w_int start, w_int length, w_int*elements) {
  w_int* dest = instance2Array_int(Array) + start;

  memcpy(dest,elements,length*sizeof(w_int));
  woempa(1, "(JNI) Written %d ints from %d of Array %p.\n", length, start, Array);
}

/*
** Write a C array of longs back into a Java array instance.
*/

void writeLongsToArray(w_instance Array, w_int start, w_int length, w_long *elements) {
  w_long* dest = instance2Array_long(Array) + start;

  memcpy(dest,elements,length*sizeof(w_long));
  woempa(1, "(JNI) Written %d longs from %d of Array %p.\n", length, start, Array);
}

/*
** Write a C array of floats back into a Java array instance.
*/

void writeFloatsToArray(w_instance Array, w_int start, w_int length, w_float *elements) {
  w_float* dest = instance2Array_float(Array) + start;

  memcpy(dest,elements,length*sizeof(w_float));
  woempa(1, "(JNI) Written %d 32 bit elements from %d of Array %p.\n", length, start, Array);
}

/*
** Write a C array of doubles back into a Java array instance.
*/

void writeDoublesToArray(w_instance Array, w_int start, w_int length, w_double *elements) {
  w_double* dest = instance2Array_double(Array) + start;

  memcpy(dest,elements,length*sizeof(w_double));
  woempa(1, "(JNI) Written %d doubles from %d of Array %p.\n", length, start, Array);
}

void readBooleansFromArray(w_instance Array, w_int start, w_int length, jboolean *buffer) {

  w_sbyte*  data_8;
  w_int    i;
  w_byte   bitmask;

  /*
  ** Find the correct offset (start / 8) and the correct bitposition
  ** (1 << (start % 8)) to start with...
  */

  data_8 = instance2Array_byte(Array) + (int)(start / 8);
  bitmask = 1 << (start % 8);
  for (i = 0; i < length; i++) {
    buffer[i] = (w_word)(*data_8 & bitmask);
    bitmask <<= 1;
    if (bitmask == 0) {
      data_8 += 1;
      bitmask = 1;
    }
  }
  woempa(1, "(JNI) Read %d boolean elements from Array %p offset %d into buffer %p.\n", length, Array, start, buffer);
}

void readBytesFromArray(w_instance Array, w_int start, w_int length, w_byte *buffer) {
  w_sbyte*  source = instance2Array_byte(Array) + start;

  memcpy(buffer,source,length*sizeof(w_byte));
  woempa(1, "(JNI) Read %d bytes from Array %p offset %d into buffer %p.\n", length, Array, start, buffer);
}

void readCharsFromArray(w_instance Array, w_int start, w_int length, w_char *buffer) {
  w_char*  source = instance2Array_char(Array) + start;

  memcpy(buffer,source,length*sizeof(w_char));
  woempa(1, "(JNI) Read %d chars from Array %p offset %d into buffer %p.\n", length, Array, start, buffer);
}

void readShortsFromArray(w_instance Array, w_int start, w_int length, w_short *buffer) {
  w_short*  source = instance2Array_short(Array) + start;

  memcpy(buffer,source,length*sizeof(w_short));
  woempa(1, "(JNI) Read %d shorts from Array %p offset %d into buffer %p.\n", length, Array, start, buffer);
}

void readIntsFromArray(w_instance Array, w_int start, w_int length, w_int*buffer) {
  w_int*  source = instance2Array_int(Array) + start;

  memcpy(buffer,source,length*sizeof(w_int));
  woempa(1, "(JNI) Read %d ints from Array %p offset %d into buffer %p.\n", length, Array, start, buffer);
}

void readLongsFromArray(w_instance Array, w_int start, w_int length, w_long *buffer) {
  w_long*  source = instance2Array_long(Array) + start;

  memcpy(buffer,source,length*sizeof(w_long));
  woempa(1, "(JNI) Read %d longs from Array %p offset %d into buffer %p.\n", length, Array, start, buffer);
}

void readFloatsFromArray(w_instance Array, w_int start, w_int length, w_float *buffer) {
  w_float*  source = instance2Array_float(Array) + start;

  memcpy(buffer,source,length*sizeof(w_float));
  woempa(1, "(JNI) Read %d floats from Array %p offset %d into buffer %p.\n", length, Array, start, buffer);
}

void readDoublesFromArray(w_instance Array, w_int start, w_int length, w_double *buffer) {
  w_double*  source = instance2Array_double(Array) + start;

  memcpy(buffer,source,length*sizeof(w_double));
  woempa(1, "(JNI) Read %d doubles from Array %p offset %d into buffer %p.\n", length, Array, start, buffer);
}

/*
** Convert an array of jvalues into items, starting from a method descriptor.
*/

// TODO, see if we can clean this up more after removing w_item ...

w_int jvalues2items(w_method method, jvalue arguments[], w_word items[]) {

  w_clazz arg_clazz;
  w_int i;
  w_int j = 0;
  w_word * item;
  w_int  words = method->nargs;
  w_instance exception;

  for (i = 0; i < method->nargs; i++) {
    method->arg_types[i] = mustBeLoaded(method->arg_types[i], &exception);
    arg_clazz = method->arg_types[i];
    item = &items[j++];
    if (arg_clazz->desc->tag == D_Primitive) {
      switch (arg_clazz->desc->body.primitive.type) {
        case (P_boolean): 
          *item = arguments[i].z; 
          break;

        case (P_char):    
          *item = arguments[i].c; 
          break;

        case (P_float):   
          *item = arguments[i].f; 
          break;

        case (P_byte):    
          *item = arguments[i].b; 
          break;

        case (P_short):   
          *item = arguments[i].s; 
          break;

        case (P_int): 
          *item = arguments[i].i; 
          break;

        case (P_long): {
          // IMPLEMENT FOR OTHER ENDIANESS MACHINE !!
          item[0] = LSW_PART(arguments[i].j);
          item[1] = MSW_PART(arguments[i].j);
          ++j;
          words++;
          break;
        }

        case (P_double): {
          // IMPLEMENT FOR OTHER ENDIANESS MACHINE !!
          item[0] = LSW_PART(arguments[i].d);
          item[1] = MSW_PART(arguments[i].d);
          ++j;
          words++;
          break;
        }

        default: 
          wabort(ABORT_WONKA, "Incorrect primitive type %d.\n", arg_clazz->desc->body.primitive.type);        
      }
    }
    else if(arg_clazz->desc->tag == D_Class || arg_clazz->desc->tag == D_Array) {
      *item = (w_word)arguments[i].l;
    }
    else {
      wabort(ABORT_WONKA, "Dude...\n");
    }
  }

  return words;
}

/*
** Convert a va_list into items, starting from a method descriptor.
*/

// TODO idem
w_int va_list2items(w_method method, va_list list, w_word items[]) {

  w_clazz arg_clazz;
  w_int i;
  w_int j = 0;
  w_word * item;
  w_int words = method->nargs;
  w_instance exception;

  for (i = 0; i < method->nargs; i++) {
    method->arg_types[i] = mustBeLoaded(method->arg_types[i], &exception);
    arg_clazz = method->arg_types[i];
    item = &items[j++];
    if (arg_clazz->desc->tag == D_Primitive) {
      switch (arg_clazz->desc->body.primitive.type) {
        case (P_boolean): 
          *item = va_arg(list, w_word);
          break;

        case (P_char):    
          *item = va_arg(list, w_word);
          break;

        case (P_float):   
          *item = va_arg(list, w_word);
          break;

        case (P_byte):    
          *item = va_arg(list, w_word);
          break;

        case (P_short):   
          *item = va_arg(list, w_word);
          break;

        case (P_int):     
          *item = va_arg(list, w_word);
          break;

        case (P_long): {
          // IMPLEMENT FOR OTHER ENDIANESS MACHINE !!
          item[0] = va_arg(list, w_word);
          item[1] = va_arg(list, w_word);
          ++j;
          words++;
          break;
        }

        case (P_double): {
          // IMPLEMENT FOR OTHER ENDIANESS MACHINE !!
          item[0] = va_arg(list, w_word);
          item[1] = va_arg(list, w_word);
          ++j;
          words++;
          break;
        }

        default: wabort(ABORT_WONKA, "Incorrect primitive type %d.\n", arg_clazz->desc->body.primitive.type);        
      }
    }
    else if (arg_clazz->desc->tag == D_Class || arg_clazz->desc->tag == D_Array) {
      *item = (w_word)va_arg(list, jobject);
    }
    else {
      wabort(ABORT_WONKA, "Dude...\n");
    }
  }

  return words;
}

/*
** Generic calls for the virtual method invocations.
*/

w_word run_32_method(w_thread thread, w_frame bottom) {
  w_word result;
  w_instance exception;

  woempa(9,"running 32_method '%w' for thread '%w', TOS @ %p\n",NM(bottom->method),NM(thread),bottom->o_top);
  setFlag(bottom->flags, FRAME_JNI);
  exception = runThread(thread, (w_frame)bottom);
  if(exception) {
//    setReferenceField(thread->Thread, exception, F_Thread_thrown);
    setLocalException(thread, exception);
  }
  bottom->o_top -= 1;
  result = *bottom->o_top;
  woempa(9,"finished running 32_method '%w' for thread '%w', TOS @ %p\n",NM(bottom->method),NM(thread),bottom->o_top);
  releaseBottomFrame(bottom);

  return result;
}

w_long run_64_method(w_thread thread, w_frame bottom) {
  w_long result;
  w_instance exception;

  woempa(1,"running 64_method '%w' for thread '%w', TOS @ %p\n",NM(bottom->method),NM(thread),bottom->o_top);
  setFlag(bottom->flags, FRAME_JNI);
  exception = runThread(thread, (w_frame)bottom);
  if(exception) {
//    setReferenceField(thread->Thread, exception, F_Thread_thrown);
    setLocalException(thread, exception);
  }
  bottom->o_top -= 1;
  result = *bottom->o_top;
  result <<= 32;
  bottom->o_top -= 1;
  result |= *bottom->o_top;
  woempa(1,"finished running 64_method '%w' for thread '%w', TOS @ %p\n",NM(bottom->method),NM(thread),bottom->o_top);
  releaseBottomFrame(bottom);

  return result;
}

void run_void_method(w_thread thread, w_frame bottom) {
  w_instance exception;
  
  woempa(1,"running void_method '%w' for thread '%w', TOS @ %p\n",NM(bottom->method),NM(thread),bottom->o_top);
  setFlag(bottom->flags, FRAME_JNI);
  exception = runThread(thread, (w_frame)bottom);
  if(exception) {
//    setReferenceField(thread->Thread, exception, F_Thread_thrown);
    setLocalException(thread, exception);
  }
  woempa(1,"finished running void_method '%w' for thread '%w', TOS @ %p\n",NM(bottom->method),NM(thread),bottom->o_top);
  releaseBottomFrame(bottom);
}

/*
** Get the current group for memory allocation. When we are not running
** from within a JAVA thread, the thread instance will be NULL and
** we assume that we are running from within some kind of system thread.
*/

w_group JNI_Group(JNIEnv *env) {
  w_instance Thread = JNIEnv2w_thread(env)->Thread;
  
  if (Thread) {
    return ((w_thread)Thread[F_Thread_wotsit])->group;
  }
  else {
    return W_ThreadGroup_system;
  }
}

jint GetVersion(JNIEnv *env) {
  return 0x00010002;
}

jclass DefineClass(JNIEnv *env, const char *name, jobject loader, const jbyte *buf, jsize buflen) {
  w_thread  thread = JNIEnv2w_thread(env);
  x_monitor monitor = loader2monitor(loader);
  w_clazz   clazz;
  w_instance theClass;
  w_BAR bar;
  w_instance exception = NULL;

  woempa(1,"installing %s from %p using %k@%p (thread %w)\n",name,buf,instance2clazz(loader),loader,NM(thread));

  bar.buffer = (w_byte*)buf;
  bar.length = buflen;
  bar.current = 0;

  x_monitor_enter(monitor, x_eternal);

// [CG 20010322] not passing the name given, as I can't find a consistent
// description of what format it used (slashes or dots, etc.)
  clazz = createClazz(thread, NULL, &bar, loader, &exception);

  x_monitor_exit(monitor);

  if (exception) {
    setLocalException(thread,exception);

    return nullInstance;

  }

  theClass = clazz2Class(clazz);
  setReferenceField(theClass,loader,F_Class_loader);

  return theClass;
}

void ExceptionClear(JNIEnv* env) {
  w_thread thread = JNIEnv2w_thread(env);
  if (thread->Thread) {
    setReferenceField(thread->Thread, NULL, F_Thread_thrown);
  }
  unsetFlag(thread->flags, (WT_THREAD_THROWN | WT_THREAD_KILLED));
}

void ExceptionDescribe(JNIEnv* env) {
  w_thread thread = JNIEnv2w_thread(env);
  w_instance throwable;

  if (thread->Thread && (throwable = (w_instance)thread->Thread[F_Thread_thrown])) {
    woempa(1,"Thread '%w' has thrown an instance of %k\n",NM(thread),instance2clazz(throwable));
    if (throwable[F_Throwable_detailMessage]) {
       woempa(1,"  Message is: %w\n",(w_string)throwable[F_Throwable_detailMessage]);
    }
  }
}

jthrowable ExceptionOccurred(JNIEnv *env) {
  w_thread thread = JNIEnv2w_thread(env);
  w_instance throwable;

  if (thread->Thread && (throwable = (w_instance)thread->Thread[F_Thread_thrown])) {

    return throwable;

  }

  return NULL;
  
}

void FatalError(JNIEnv *env, const char *message) {
  wabort(ABORT_WONKA, "FatalError: %s\n",message);
}

    
jclass FindClass(JNIEnv *env, const char *Name) {
  w_instance exception = NULL;
  w_thread thread = JNIEnv2w_thread(env);
  w_string name = cstring2String(thread, Name, strlen(Name));
  w_string dotified;
  w_method caller = getCallingMethod(thread);
  w_instance loader;
  w_clazz clazz;
  jclass result;

  woempa(1, "(JNI) Searching for class '%s'.\n", Name);
  if (caller) {
    loader = clazz2loader(caller->clazz);
    woempa(1, "(JNI) Using class loader %p of %k.\n", loader, instance2clazz(loader));
  }
  else {
    loader = NULL;
    woempa(1, "(JNI) No calling method - maybe a native thread? Using null classloader <shrug>\n");
  }

  dotified = undescriptifyClassName(name);
  clazz = namedClassMustBeLoaded(loader, dotified, &exception);
  deregisterString(dotified);

  if (exception == NULL && clazz == NULL) {
    exception = createExceptionInstance(thread,clazzClassNotFoundException,name);
  }

  if (exception) {
    setLocalException(thread,exception);
    return NULL;
  }

  classMustBeResolved(clazz, thread);

  result = clazz2Class(clazz);
  
  deregisterString(name);
  
  return result;
}

jclass GetSuperclass(JNIEnv *env, jclass class) {
  return clazz2Class(Class2clazz(class)->superClazz);
}

jboolean IsAssignableFrom(JNIEnv *env, jclass class1, jclass class2) {
  return isAssignmentCompatible(Class2clazz(class1), Class2clazz(class2));
}

jint Throw(JNIEnv *env, jthrowable theThrowable) {
  w_thread thread = JNIEnv2w_thread(env);

  setLocalException(thread, theThrowable);
  woempa(7, "threw %k@%p\n", instance2clazz(theThrowable), theThrowable);
  
  return 0;
}

jint ThrowNew(JNIEnv *env, jclass class, const char *message) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz  = Class2clazz(class);
  w_string string = cstring2String(thread,message,strlen(message));
  w_instance newThrowable = allocInstance(thread,clazz);
  w_instance theMessage = getStringInstance(thread,string);

  setReferenceField(newThrowable,theMessage,F_Throwable_detailMessage);
  setLocalException(thread, newThrowable);
  
  return 0;
}


jobject NewGlobalRef(JNIEnv *env, jobject obj) {
  newGlobalReference(obj);
  return obj;
}

jobject NewLocalRef(JNIEnv *env, jobject obj) {
  w_thread thread = JNIEnv2w_thread(env);

  woempa(1,"creating new local reference to instance %p of %k\n",obj,instance2clazz((w_instance)obj));
  addLocalReference(thread, obj);

  return obj;
}

void DeleteGlobalRef(JNIEnv *env, jobject obj) {

  /*
  ** Decrement the reference count, if the new reference count returned is 0, we erase
  ** the entry from the hashtable.
  */

  woempa(1,"deleting global reference to instance %p\n",obj);
  ht_deregister(globals_hashtable,(w_word)obj);
}

void DeleteLocalRef(JNIEnv *env, jobject instance) {
  w_thread thread = JNIEnv2w_thread(env);
  
  woempa(1,"deleting local reference to instance %p of %k\n",instance,instance2clazz((w_instance)instance));
  removeLocalReference(thread->top, instance);
}

jboolean IsSameObject(JNIEnv *env, jobject ref1, jobject ref2) {
  return ref1 == ref2;
}

jobject AllocObject(JNIEnv *env, jclass class) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz  = Class2clazz(class);
  w_instance new;
  w_instance exception;
  
  exception = classMustBeInitialized(clazz, thread);
  if (exception == NULL) {
    new = allocInstance(thread, clazz);

    if (new) {
      woempa(1, "(JNI) new object %p of clazz %k.\n", new, Class2clazz(class));
 
      addLocalReference(thread, new);
    }
    else {
      throwOutOfMemoryError(thread);
    }

    return new;

  }
  else {
    setLocalException(thread, exception);

    return NULL;
  }
}

jobject NewObject(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  w_thread thread = JNIEnv2w_thread(env);
//  w_realm  realm  = thread2realm(thread);
  w_clazz  clazz  = Class2clazz(class);
  w_instance new;
  w_frame bottom;
  va_list list;

// TODO: exception = 
  classMustBeInitialized(clazz, thread);
  new  = allocInstance(thread, clazz);

  if (new) {
    addLocalReference(thread, new);

    va_start(list, methodID);
    bottom = allocBottomFrame(thread, methodID);
    if (bottom) {
      bottom->locals[0] = (w_word)new;
      va_list2items(methodID, list, &bottom->locals[1]);
      va_end(list);

      run_void_method(thread, bottom);
    }
  }
  else {
    throwOutOfMemoryError(thread);
  }

  return new;
}


jobject NewObjectV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  w_thread thread = JNIEnv2w_thread(env);
//  w_realm  realm  = thread2realm(thread);
  w_clazz  clazz  = Class2clazz(class);
  w_instance new;
  w_frame bottom;

// TODO: exception = 
  classMustBeInitialized(clazz, thread);
  new = allocInstance(thread, clazz);

  if (new) {
    addLocalReference(thread, new);

    bottom = allocBottomFrame(thread, methodID);
    if (bottom) {
      bottom->locals[0] = (w_word)new;
      va_list2items(methodID, args, &bottom->locals[1]);

      run_void_method(thread, bottom);
    }
  }
  else {
    throwOutOfMemoryError(thread);
  }

  return new;
}

jobject NewObjectA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz  = Class2clazz(class);
  w_instance new;
  w_frame bottom;

// TODO: exception = 
  classMustBeInitialized(clazz, thread);
  new = allocInstance(thread, clazz);

  if (new) {
    addLocalReference(thread, new);

    bottom = allocBottomFrame(thread, methodID);
    if (bottom) {
      bottom->locals[0] = (w_word)new;
      jvalues2items(methodID, args, &bottom->locals[1]);

      run_void_method(thread, bottom);
    }
  }
  else {
    throwOutOfMemoryError(thread);
  }

  return new;
}

jboolean IsInstanceOf(JNIEnv *env, jobject obj, jclass class) {

///By definition, a null object is instance of every class, and therefore always should return true
  if(!obj)
    return JNI_TRUE;
  else
    return isAssignmentCompatible(instance2clazz(obj), Class2clazz(class));
}

jclass GetObjectClass(JNIEnv *env, jobject obj) {
  w_clazz clazz = instance2clazz(obj);
  return clazz2Class(clazz);
}

/*
** Search for a method with a certain template in a certain clazz, based on a clazz.
** The template is in fact a method but we have to resolve the correct method ourselves.
** It may well be the method that was passed as a template.
*/

w_method resolveMethodForCall(w_clazz clazz, w_method template) {
  return clazz->methods[template->slot];
}

/*
** => Calling java class functions froom within JNI c-code
*/

w_frame bottomFrameArgs(w_thread thread, w_instance instance, w_method method, va_list args) {
  w_frame  bottom = allocBottomFrame(thread, method);
  if (bottom) {
    if (isSet(method->flags, ACC_NATIVE)) {
      if(instance) {
        bottom->o_top[0] = (w_word)instance;
        bottom->o_top++;
      }
      bottom->o_top += va_list2items(method, args, bottom->o_top);
    }
    else {
      if(instance) {
        bottom->locals[0] = (w_word)instance;
        va_list2items(method, args, &bottom->locals[1]);
      }
      else {
        va_list2items(method, args, bottom->locals);
      }
    }
  }
  return bottom;
}

w_frame bottomFrameArgsA(w_thread thread, w_instance instance, w_method method, jvalue *args) {
  w_frame bottom = allocBottomFrame(thread, method);
  if (bottom) {
    if (isSet(method->flags, ACC_NATIVE)) {
      if(instance) {
        bottom->o_top[0] = (w_word)instance;
        bottom->o_top++;
      }
      bottom->o_top += jvalues2items(method, args, bottom->o_top);
    }
    else {
      if(instance) {
        bottom->locals[0] = (w_word)instance;
        jvalues2items(method, args, &bottom->locals[1]);
      }
      else {
        jvalues2items(method, args, bottom->locals);
      }
    }
  }
  return bottom;
}

JNITypes CallMethod32(JNIEnv *env, jobject obj, w_method method, va_list args) {
  w_thread thread = JNIEnv2w_thread(env);
  JNITypes result;
  w_frame  bottom = bottomFrameArgs(thread, (w_instance)obj, method, args);
  
  if (bottom) {
    result = (JNITypes)run_32_method(thread, bottom);
  }

  return result;
}

JNITypes CallMethod32A(JNIEnv *env, jobject obj, w_method method, jvalue *args) {
  w_thread thread = JNIEnv2w_thread(env);
  JNITypes result;
  w_frame  bottom = bottomFrameArgsA(thread, (w_instance)obj, method, args);
  
  if (bottom) {
    result = (JNITypes)run_32_method(thread, bottom);
  }

  return result;
}

JNITypes CallMethod64(JNIEnv *env, jobject obj, w_method method, va_list args) {
  w_thread thread = JNIEnv2w_thread(env);
  JNITypes result;
  w_frame  bottom = bottomFrameArgs(thread, (w_instance)obj, method, args);
  
  if (bottom) {
    result = (JNITypes)run_64_method(thread, bottom);
  }

  return result;
}

JNITypes CallMethod64A(JNIEnv *env, jobject obj, w_method method, jvalue *args) {
  w_thread thread = JNIEnv2w_thread(env);
  JNITypes result;
  w_frame  bottom = bottomFrameArgsA(thread, (w_instance)obj, method, args);
  
  if (bottom) {
    result = (JNITypes)run_64_method(thread, bottom);
  }

  return result;
}

w_void CallMethodVoid(JNIEnv *env, jobject obj, w_method method, va_list args) {
  w_thread thread = JNIEnv2w_thread(env);
  w_frame  bottom = bottomFrameArgs(thread, (w_instance)obj, method, args);
  
  if (bottom) {
    run_void_method(thread, bottom);
  }
}

w_void CallMethodVoidA(JNIEnv *env, jobject obj, w_method method, jvalue *args) {
  w_thread thread = JNIEnv2w_thread(env);
  w_frame  bottom = bottomFrameArgsA(thread, (w_instance)obj, method, args);
  
  if (bottom) {
    run_void_method(thread, bottom);
  }
}

/*
** => object function calls
*/

jobject CallObjectMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {
  va_list  list;
  jobject  result = NULL;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).o;
  va_end(list);
  
  return result;
}

jobject CallObjectMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  return (CallMethod32(env, obj, method, args)).o; 
}

jobject CallObjectMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  return (CallMethod32A(env, obj, method, args)).o; 
}

/*
** => boolean function calls
*/

jboolean CallBooleanMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {
  va_list  list;
  jboolean result = 0;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).z;
  va_end(list);
  
  return result;
}

jboolean CallBooleanMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  return (CallMethod32(env, obj, method, args)).z; 
}

jboolean CallBooleanMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  return (CallMethod32A(env, obj, method, args)).z; 
}

/*
** => byte function calls
*/

jbyte CallByteMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {
  va_list  list;
  jbyte    result = 0;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).b;
  va_end(list);
  
  return result;
}

jbyte CallByteMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  return (CallMethod32(env, obj, method, args)).b; 
}

jbyte CallByteMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  return (CallMethod32A(env, obj, method, args)).b; 
}

/*
** => char function calls
*/

jchar CallCharMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {
  va_list  list;
  jchar    result = 0;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).c;
  va_end(list);
  
  return result;
}

jchar CallCharMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  return (CallMethod32(env, obj, method, args)).c; 
}

jchar CallCharMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  return (CallMethod32A(env, obj, method, args)).c; 
}

/*
** => short function calls
*/

jshort CallShortMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {
  va_list  list;
  jshort   result = 0;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).s;
  va_end(list);
  
  return result;
}

jshort CallShortMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  return (CallMethod32(env, obj, method, args)).s; 
}

jshort CallShortMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  return (CallMethod32A(env, obj, method, args)).s; 
}

/*
** => int function calls
*/

jint CallIntMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {
  va_list  list;
  jint     result = 0;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).i;
  va_end(list);
  
  return result;
}

jint CallIntMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  return (CallMethod32(env, obj, method, args)).i; 
}

jint CallIntMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  return (CallMethod32A(env, obj, method, args)).i; 
}

/*
** => long function calls
*/

jlong CallLongMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {
  va_list  list;
  jlong    result = 0;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  result = (CallMethod64(env, obj, method, list)).j;
  va_end(list);
  
  return result;
}

jlong CallLongMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  return (CallMethod64(env, obj, method, args)).j; 
}

jlong CallLongMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  return (CallMethod64A(env, obj, method, args)).j; 
}

/*
** => float function calls
*/

jfloat CallFloatMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {
  va_list  list;
  jfloat   result = 0;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).f;
  va_end(list);
  
  return result;
}

jfloat CallFloatMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  return (CallMethod32(env, obj, method, args)).f; 
}

jfloat CallFloatMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  return (CallMethod32A(env, obj, method, args)).f; 
}

/*
** => double function calls
*/

jdouble CallDoubleMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {
  va_list  list;
  jdouble    result = 0;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  result = (CallMethod64(env, obj, method, list)).d;
  va_end(list);
  
  return result;
}

jdouble CallDoubleMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  return (CallMethod64(env, obj, method, args)).d; 
}

jdouble CallDoubleMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  return (CallMethod64A(env, obj, method, args)).d; 
}

/*
** => void function calls
*/

w_void CallVoidMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {
  va_list  list;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  CallMethodVoid(env, obj, method, list);
  va_end(list);
}

w_void CallVoidMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  CallMethodVoid(env, obj, method, args); 
}

w_void CallVoidMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  CallMethodVoidA(env, obj, method, args); 
}

/*
** =>  nonvirtual object function calls
*/

jobject CallNonvirtualObjectMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  jobject  result = NULL;
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).o;
  va_end(list);
  
  return result;
}

jobject CallNonvirtualObjectMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  return (CallMethod32(env, obj, method, args)).o; 
}

jobject CallNonvirtualObjectMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  return (CallMethod32A(env, obj, method, args)).o; 
}

/*
** =>  nonvirtual boolean function calls
*/

jboolean CallNonvirtualBooleanMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  jboolean result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).z;
  va_end(list);
  
  return result;
}

jboolean CallNonvirtualBooleanMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  return (CallMethod32(env, obj, method, args)).z; 
}

jboolean CallNonvirtualBooleanMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  return (CallMethod32A(env, obj, method, args)).z; 
}

/*
** =>  nonvirtual byte function calls
*/

jbyte CallNonvirtualByteMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  jbyte    result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).b;
  va_end(list);
  
  return result;
}

jbyte CallNonvirtualByteMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  return (CallMethod32(env, obj, method, args)).b; 
}

jbyte CallNonvirtualByteMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  return (CallMethod32A(env, obj, method, args)).b; 
}

/*
** =>  nonvirtual char function calls
*/

jchar CallNonvirtualCharMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  jchar    result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).c;
  va_end(list);
  
  return result;
}

jchar CallNonvirtualCharMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  return (CallMethod32(env, obj, method, args)).c; 
}

jchar CallNonvirtualCharMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  return (CallMethod32A(env, obj, method, args)).c; 
}

/*
** =>  nonvirtual short function calls
*/

jshort CallNonvirtualShortMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  jshort   result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).s;
  va_end(list);
  
  return result;
}

jshort CallNonvirtualShortMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  return (CallMethod32(env, obj, method, args)).s; 
}

jshort CallNonvirtualShortMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  return (CallMethod32A(env, obj, method, args)).s; 
}

/*
** =>  nonvirtual int function calls
*/

jint CallNonvirtualIntMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  jint    result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).i;
  va_end(list);
  
  return result;
}

jint CallNonvirtualIntMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  return (CallMethod32(env, obj, method, args)).i; 
}

jint CallNonvirtualIntMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  return (CallMethod32A(env, obj, method, args)).i; 
}

/*
** =>  nonvirtual long function calls
*/

jlong CallNonvirtualLongMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  jlong    result = 0;
  
  va_start(list, methodID);
  result = (CallMethod64(env, obj, method, list)).j;
  va_end(list);
  
  return result;
}

jlong CallNonvirtualLongMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  return (CallMethod64(env, obj, method, args)).j; 
}

jlong CallNonvirtualLongMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  return (CallMethod64A(env, obj, method, args)).j; 
}

/*
** =>  nonvirtual float function calls
*/

jfloat CallNonvirtualFloatMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  jfloat   result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).f;
  va_end(list);
  
  return result;
}

jfloat CallNonvirtualFloatMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  return (CallMethod32(env, obj, method, args)).f; 
}

jfloat CallNonvirtualFloatMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  return (CallMethod32A(env, obj, method, args)).f; 
}

/*
** =>  nonvirtual double function calls
*/

jdouble CallNonvirtualDoubleMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  jlong    result = 0;
  
  va_start(list, methodID);
  result = (CallMethod64(env, obj, method, list)).d;
  va_end(list);
  
  return result;
}

jdouble CallNonvirtualDoubleMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  return (CallMethod64(env, obj, method, args)).d; 
}

jdouble CallNonvirtualDoubleMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  return (CallMethod64A(env, obj, method, args)).d; 
}

/*
** =>  nonvirtual void function calls
*/

w_void CallNonvirtualVoidMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  
  va_start(list, methodID);
  CallMethodVoid(env, obj, method, list);
  va_end(list);
}

w_void CallNonvirtualVoidMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  CallMethodVoid(env, obj, method, args); 
}

w_void CallNonvirtualVoidMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {
  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  CallMethodVoidA(env, obj, method, args); 
}

/*
** =>  member variable access
*/

/*
** =>  retrieve the member variable field ID
*/

jfieldID GetFieldID(JNIEnv *env, jclass class, const char *name, const char *sig) {

  w_string name_string = cstring2String(JNIEnv2w_thread(env), (char *)name, strlen(name));
  w_string desc_string = cstring2String(JNIEnv2w_thread(env), (char *)sig, strlen(sig));
  w_desc desc = newFieldDescriptor(JNIEnv2w_thread(env), desc_string);
// TODO: error if desc==NULL ...
  w_field result;

  woempa(1, "(JNI) name='%s', signature='%s'\n",name,sig);
    
  /*
  ** Search for it, match only instance fields, but match any of the 
  ** "public" or "private" field modifiers.
  */
  
  result = searchClazzHierarchyForField(Class2clazz(class), name_string, desc, MATCH_INSTANCE_FIELD, MATCH_ANY);

  deregisterString(desc_string);
  deregisterDescriptor(desc);
  deregisterString(name_string);
  
  return result;
}

/*
** =>  Get the value of a variable, given the variables field ID
*/

JNITypes GetGeneric32Field(JNIEnv *env, jobject obj, jfieldID fieldID) {
  w_clazz clazz = instance2clazz((w_instance)obj);
  // TODO: exception = ...
  classMustBeInitialized(clazz, JNIEnv2w_thread(env));
  return (JNITypes)((w_instance)obj)[fieldID->slot];
}

JNITypes GetGeneric64Field(JNIEnv *env, jobject obj, jfieldID fieldID) {
  w_clazz clazz = instance2clazz((w_instance)obj);
  w_long result;
  // TODO: exception = ...
  classMustBeInitialized(clazz, JNIEnv2w_thread(env));
  result =  ((w_instance)obj)[fieldID->slot + 1];  /* MSW */
  result <<= 32;
  result |= ((w_instance)obj)[fieldID->slot    ];  /* LSW */
  return (JNITypes)result;
}

jobject GetObjectField(JNIEnv *env, jobject obj, jfieldID fieldID) {
  return (GetGeneric32Field(env, obj, fieldID)).o;
}

jboolean GetBooleanField(JNIEnv *env, jobject obj, jfieldID fieldID) {
  return (GetGeneric32Field(env, obj, fieldID)).z;
}

jbyte GetByteField(JNIEnv *env, jobject obj, jfieldID fieldID) {
  return (GetGeneric32Field(env, obj, fieldID)).b;
}

jchar GetCharField(JNIEnv *env, jobject obj, jfieldID fieldID) {
  return (GetGeneric32Field(env, obj, fieldID)).c;
}

jshort GetShortField(JNIEnv *env, jobject obj, jfieldID fieldID) {
  return (GetGeneric32Field(env, obj, fieldID)).s;
}

jint GetIntField(JNIEnv *env, jobject obj, jfieldID fieldID) {
  return (GetGeneric32Field(env, obj, fieldID)).i;
}

jlong GetLongField(JNIEnv *env, jobject obj, jfieldID fieldID) {
  return (GetGeneric64Field(env, obj, fieldID)).j;
}

jfloat GetFloatField(JNIEnv *env, jobject obj, jfieldID fieldID) {
  return (GetGeneric32Field(env, obj, fieldID)).f;
}

jdouble GetDoubleField(JNIEnv *env, jobject obj, jfieldID fieldID) {
  return (GetGeneric64Field(env, obj, fieldID)).d;
}

/*
** =>  set the a variable to a certain value, given the variables field ID
*/

w_void SetGeneric32Field(JNIEnv *env, jobject obj, jfieldID fieldID, w_word value) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz = instance2clazz((w_instance)obj);
  // TODO: exception = ...
  classMustBeInitialized(clazz, thread);
  woempa(1,"set field[%d] of instance %p of %k to 0x%08x\n",fieldID->slot,(w_instance)obj,instance2clazz((w_instance)obj),value);
  ((w_instance)obj)[fieldID->slot] = value;
}

w_void SetGeneric64Field(JNIEnv *env, jobject obj, jfieldID fieldID, jlong value) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz = instance2clazz((w_instance)obj);
  // TODO: exception = ...
  classMustBeInitialized(clazz, thread);
  
  woempa(1,"set field[%d-%d] of instance %p of %k to 0x%08x%08x\n",fieldID->slot,fieldID->slot+1,
          (w_instance)obj,instance2clazz((w_instance)obj),LSW_PART(value),MSW_PART(value));

  ((w_instance)obj)[fieldID->slot    ] = LSW_PART(value);
  ((w_instance)obj)[fieldID->slot + 1] = MSW_PART(value);
}

w_void SetObjectField(JNIEnv *env, jobject obj, jfieldID fieldID, jobject value) {
  w_instance instance = obj;
  w_instance wval = value;
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz = instance2clazz(instance);
  // TODO: exception = ...
  classMustBeInitialized(clazz, thread);
  woempa(1,"set field[%d] of instance %p of %k to %p\n",fieldID->slot,instance,instance2clazz(instance),wval);

//NGC  instance[fieldID->slot] = (w_word)wval;
  setReferenceField(instance, wval, fieldID->slot);
}

w_void SetBooleanField(JNIEnv *env, jobject obj, jfieldID fieldID, jboolean value) {
  SetGeneric32Field(env, obj, fieldID, (w_word)value);
}

w_void SetByteField(JNIEnv *env, jobject obj, jfieldID fieldID, jbyte value) {
  SetGeneric32Field(env, obj, fieldID, (w_word)value);
}

w_void SetCharField(JNIEnv *env, jobject obj, jfieldID fieldID, jchar value) {
  SetGeneric32Field(env, obj, fieldID, (w_word)value);
}

w_void SetShortField(JNIEnv *env, jobject obj, jfieldID fieldID, jshort value) {
  SetGeneric32Field(env, obj, fieldID, (w_word)value);
}

w_void SetIntField(JNIEnv *env, jobject obj, jfieldID fieldID, jint value) {
  SetGeneric32Field(env, obj, fieldID, (w_word)value);
}

w_void SetLongField(JNIEnv *env, jobject obj, jfieldID fieldID, jlong value) {
  SetGeneric64Field(env, obj, fieldID, (jlong)value);
}

w_void SetFloatField(JNIEnv *env, jobject obj, jfieldID fieldID, jfloat value) {
  SetGeneric32Field(env, obj, fieldID, (w_word)value);
}

w_void SetDoubleField(JNIEnv *env, jobject obj, jfieldID fieldID, jdouble value) {
  SetGeneric64Field(env, obj, fieldID, (jlong)value);
}

/*
** =>  auxilliary...
*/

w_method getAnyMethodID(w_thread thread, jclass class, const char *Name, const char *Desc, w_boolean isStatic) {
  w_clazz clazz = Class2clazz(class);
  w_method method = NULL;
  w_string name;
  w_string sig;
  w_size   siglen;

// TODO: exception =
  classMustBeInitialized(clazz, thread);

  siglen = strlen(Desc);
  while (Desc[siglen - 1] != ')') {
    --siglen;
  }

  name = cstring2String(thread, Name, strlen(Name));
  sig = cstring2String(thread, Desc, siglen);

  while (clazz && !method) {
    woempa(7,"Seeking  method %w%w in class %k\n", name, sig, clazz);
    method = getMethod_name_sig(clazz, name, sig);
 
    if (method == NULL) {
      woempa(7,"Did not find method %w%w in class %k\n", name, sig, clazz);
      clazz = isStatic ? clazz->superClazz : NULL;
    } 
    else if (!!isSet(method->flags,ACC_STATIC) != !!isStatic) {
      woempa(9,"%w.ACC_STATIC = %d, wanted %d : Returning NULL!!!\n",NM(method),isSet(method->flags,ACC_STATIC),isStatic);
      method = NULL;
      clazz = NULL;
    }
  }

  deregisterString(name);
  deregisterString(sig);

  return method;
}

/*
** =>  static  function calls
*/

/*
** =>  static function method ID retrieval
*/

jmethodID GetStaticMethodID(JNIEnv *env, jclass class, const char *Name, const char *Sig) {
  return getAnyMethodID(JNIEnv2w_thread(env), class, Name, Sig, WONKA_TRUE);
}

jmethodID GetMethodID(JNIEnv *env, jclass class, const char *Name, const char *Sig) {
  return getAnyMethodID(JNIEnv2w_thread(env), class, Name, Sig, WONKA_FALSE);
}

/*
** =>  static object function calls
*/

jobject CallStaticObjectMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  jobject  result = NULL;
  
  va_start(list, methodID);
  result = (CallMethod32(env, NULL, methodID, list)).o;
  va_end(list);
  
  return result;
}

jobject CallStaticObjectMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  return (CallMethod32(env, NULL, methodID, args)).o; 
}

jobject CallStaticObjectMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {
  return (CallMethod32A(env, NULL, methodID, args)).o; 
}

/*
** =>  static boolean function calls
*/

jboolean CallStaticBooleanMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  jboolean result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, NULL, methodID, list)).z;
  va_end(list);
  
  return result;
}

jboolean CallStaticBooleanMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  return (CallMethod32(env, NULL, methodID, args)).z; 
}

jboolean CallStaticBooleanMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {
  return (CallMethod32A(env, NULL, methodID, args)).z; 
}

/*
** =>  static byte function calls
*/

jbyte CallStaticByteMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  jbyte    result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, NULL, methodID, list)).b;
  va_end(list);
  
  return result;
}

jbyte CallStaticByteMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  return (CallMethod32(env, NULL, methodID, args)).b; 
}

jbyte CallStaticByteMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {
  return (CallMethod32A(env, NULL, methodID, args)).b; 
}

/*
** =>  static char function calls
*/

jchar CallStaticCharMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  jchar    result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, NULL, methodID, list)).c;
  va_end(list);
  
  return result;
}

jchar CallStaticCharMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  return (CallMethod32(env, NULL, methodID, args)).c; 
}

jchar CallStaticCharMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {
  return (CallMethod32A(env, NULL, methodID, args)).c; 
}

/*
** =>  static short function calls
*/

jshort CallStaticShortMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  jshort   result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, NULL, methodID, list)).s;
  va_end(list);
  
  return result;
}

jshort CallStaticShortMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  return (CallMethod32(env, NULL, methodID, args)).s; 
}

jshort CallStaticShortMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {
  return (CallMethod32A(env, NULL, methodID, args)).s; 
}

/*
** =>  static int function calls
*/

jint CallStaticIntMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  jint     result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, NULL, methodID, list)).i;
  va_end(list);
  
  return result;
}

jint CallStaticIntMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  return (CallMethod32(env, NULL, methodID, args)).i; 
}

jint CallStaticIntMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {
  return (CallMethod32A(env, NULL, methodID, args)).i; 
}

/*
** =>  static long function calls
*/

jlong CallStaticLongMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  jlong    result = 0;
  
  va_start(list, methodID);
  result = (CallMethod64(env, NULL, methodID, list)).j;
  va_end(list);
  
  return result;
}

jlong CallStaticLongMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  return (CallMethod64(env, NULL, methodID, args)).j; 
}

jlong CallStaticLongMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {
  return (CallMethod64A(env, NULL, methodID, args)).j; 
}

/*
** =>  static float function calls
*/

jfloat CallStaticFloatMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  jfloat   result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, NULL, methodID, list)).f;
  va_end(list);
  
  return result;
}

jfloat CallStaticFloatMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  return (CallMethod32(env, NULL, methodID, args)).f; 
}

jfloat CallStaticFloatMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {
  return (CallMethod32A(env, NULL, methodID, args)).f; 
}

/*
** =>  static double function calls
*/

jdouble CallStaticDoubleMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  jdouble  result = 0;
  
  va_start(list, methodID);
  result = (CallMethod64(env, NULL, methodID, list)).d;
  va_end(list);
  
  return result;
}

jdouble CallStaticDoubleMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  return (CallMethod64(env, NULL, methodID, args)).d; 
}

jdouble CallStaticDoubleMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {
  return (CallMethod64A(env, NULL, methodID, args)).d; 
}

/*
** =>  static void function calls
*/
   
w_void CallStaticVoidMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  
  va_start(list, methodID);
  CallMethodVoid(env, NULL, methodID, list);
  va_end(list);
}

w_void CallStaticVoidMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  CallMethodVoid(env, NULL, methodID, args); 
}

w_void CallStaticVoidMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {     
  CallMethodVoidA(env, NULL, methodID, args); 
}

/*
** static member variable field access:
** get the member variable field ID
*/

jfieldID GetStaticFieldID(JNIEnv *env, jclass class, const char *name, const char *sig) {

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz  = Class2clazz(class);
  w_string name_string = cstring2String(JNIEnv2w_thread(env), (char *)name, strlen(name));
  w_string desc_string = cstring2String(JNIEnv2w_thread(env), (char *)sig, strlen(sig));
  w_desc desc = newFieldDescriptor(JNIEnv2w_thread(env), desc_string);
// TODO: error if desc==NULL ...
  w_field result;
  
  classMustBeInitialized(clazz, thread);
  
  woempa(1, "(JNI) Returning field ID for field '%s', descriptor '%s'.\n", name, sig);
  
  result = searchClazzHierarchyForField(Class2clazz(class), name_string, desc, MATCH_STATIC_FIELD, 0);

  deregisterString(desc_string);
  deregisterDescriptor(desc);
  deregisterString(name_string);
  
  return result;
}

/*
** get the static member variable value from field ID
*/

JNITypes GetStatic32Field(JNIEnv *env, jclass class, jfieldID fieldID) {
  w_clazz  clazz  = Class2clazz(class);
  w_thread thread = JNIEnv2w_thread(env);
  classMustBeInitialized(clazz, thread);

  woempa(1, "(JNI) Getting field '%w' at slot %d from class %k.\n", NM(fieldID), fieldID->slot, clazz);

  return (JNITypes)clazz->staticFields[fieldID->slot];
}

JNITypes GetStatic64Field(JNIEnv *env, jclass class, jfieldID fieldID) {
  w_clazz clazz = Class2clazz(class);
  w_thread thread = JNIEnv2w_thread(env);
  w_long result;

  classMustBeInitialized(clazz, thread);
  woempa(1, "(JNI) Getting field '%w' at slot %d from class %k.\n", NM(fieldID), fieldID->slot, clazz);
  
  result = clazz->staticFields[fieldID->slot + 1];
  result <<= 32;
  result |= clazz->staticFields[fieldID->slot];
  
  return (JNITypes)result;
}

jobject GetStaticObjectField(JNIEnv *env, jclass class, jfieldID fieldID) {
  return (GetStatic32Field(env, class, fieldID)).o;
}

jboolean GetStaticBooleanField(JNIEnv *env, jclass class, jfieldID fieldID) {
  return (GetStatic32Field(env, class, fieldID)).z;
}

jbyte GetStaticByteField(JNIEnv *env, jclass class, jfieldID fieldID) {
  return (GetStatic32Field(env, class, fieldID)).b;
}

jchar GetStaticCharField(JNIEnv *env, jclass class, jfieldID fieldID) {
  return (GetStatic32Field(env, class, fieldID)).c;
}

jshort GetStaticShortField(JNIEnv *env, jclass class, jfieldID fieldID) {
  return (GetStatic32Field(env, class, fieldID)).s;
}

jint GetStaticIntField(JNIEnv *env, jclass class, jfieldID fieldID) {
  return (GetStatic32Field(env, class, fieldID)).i;
}

jlong GetStaticLongField(JNIEnv *env, jclass class, jfieldID fieldID) {
  return (GetStatic64Field(env, class, fieldID)).j;
}

jfloat GetStaticFloatField(JNIEnv *env, jclass class, jfieldID fieldID) {
  return (GetStatic32Field(env, class, fieldID)).f;
}

jdouble GetStaticDoubleField(JNIEnv *env, jclass class, jfieldID fieldID) {
  return (GetStatic64Field(env, class, fieldID)).d;
}

/*
** set the static member variable in a field ID to given value
*/

w_void SetStatic32Field(JNIEnv *env, jclass class, jfieldID fieldID, w_word value) {
  w_clazz clazz = Class2clazz(class);
  w_thread thread = JNIEnv2w_thread(env);
  classMustBeInitialized(clazz, thread);

  clazz->staticFields[fieldID->slot] = value;
}

w_void SetStatic64Field(JNIEnv *env, jclass class, jfieldID fieldID, jlong value) {
  w_clazz clazz = Class2clazz(class);
  w_thread thread = JNIEnv2w_thread(env);
  classMustBeInitialized(clazz, thread);

  clazz->staticFields[fieldID->slot   ] = (w_word)LSW_PART(value);
  clazz->staticFields[fieldID->slot +1] = (w_word)MSW_PART(value);
}

w_void SetStaticObjectField(JNIEnv *env, jclass class, jfieldID fieldID, jobject value) {
  setStaticReferenceField(fieldID->declaring_clazz, fieldID->slot, value);
}

w_void SetStaticBooleanField(JNIEnv *env, jclass class, jfieldID fieldID, jboolean value) {
  SetStatic32Field(env, class, fieldID, (w_word)value);
}

w_void SetStaticByteField(JNIEnv *env, jclass class, jfieldID fieldID, jbyte value) {
  SetStatic32Field(env, class, fieldID, (w_word)value);
}

w_void SetStaticCharField(JNIEnv *env, jclass class, jfieldID fieldID, jchar value) {
  SetStatic32Field(env, class, fieldID, (w_word)value);
}

w_void SetStaticShortField(JNIEnv *env, jclass class, jfieldID fieldID, jshort value) {
  SetStatic32Field(env, class, fieldID, (w_word)value);
}

w_void SetStaticIntField(JNIEnv *env, jclass class, jfieldID fieldID, jint value) {
  SetStatic32Field(env, class, fieldID, (w_word)value);
}

w_void SetStaticLongField(JNIEnv *env, jclass class, jfieldID fieldID, jlong value) {
  SetStatic64Field(env, class, fieldID, (jlong)value);
}

w_void SetStaticFloatField(JNIEnv *env, jclass class, jfieldID fieldID, jfloat value) {
  SetStatic32Field(env, class, fieldID, (w_word)value);
}

w_void SetStaticDoubleField(JNIEnv *env, jclass class, jfieldID fieldID, jdouble value) {
  SetStatic64Field(env, class, fieldID, (jlong)value);
}

jstring NewString(JNIEnv *env, const jchar *uchars, jsize len) {

  w_thread thread = JNIEnv2w_thread(env);
  w_string string = unicode2String(JNIEnv2w_thread(env), (w_char*)uchars, (w_size)len);
  w_instance String = getStringInstance(thread, string);
  addLocalReference(thread, String);
  
  return String;
}

jsize GetStringLength(JNIEnv *env, jstring String) {

  w_string string = String2string(String);

  woempa(1, "(JNI) Length of '%s' (%p) is %d JAVA chars.\n", string->chars, String, string->length);
  
  return string->length;
  
}

const jchar *GetStringChars(JNIEnv *env, jstring String, jboolean *isCopy) {

  w_string string = String2string(String);
  w_char  *chars = NULL;

  if (string->length) {
    chars  = allocMem(JNIEnv2w_thread(env), string->length * sizeof(w_char));
    w_string2chars(string,chars);
  }
  woempa(1,"(JNI) created array of %d chars at %p\n",string->length,chars);

  if (isCopy) {
    *isCopy = WONKA_TRUE;
  }

  return chars;
  
}

void ReleaseStringChars(JNIEnv *env, jstring string, const jchar *chars) {
  if (chars) {
    woempa(1,"(JNI) release array of chars at %p\n",chars);
    releaseMem((jchar*)chars);
  }
}
          
jstring NewStringUTF(JNIEnv *env, const char *bytes) {

  w_thread thread = JNIEnv2w_thread(env);
  w_string string = utf2String(JNIEnv2w_thread(env), (char *)bytes, strlen(bytes));
  w_instance String = getStringInstance(thread, string);
  addLocalReference(thread, String);
    
  return String;
  
}

jsize GetStringUTFLength(JNIEnv *env, jstring String) {
  w_string string = String2string(String);
  w_char  *utf16ptr = string->chars;
  w_size   i;
  jsize    result = 0;
 
  for (i=0; i<string->length; ++i) {
    if (*utf16ptr == 0) {
      result += 2;
    }
    else if (*utf16ptr < 0x80 ) {
      result += 1;
    }
    else if (*utf16ptr < 0x800 ) {
      result += 2;
    }
    else {
      result += 3;
    }
    utf16ptr++;
  }

  return result;
}

const jbyte *GetStringUTFChars(JNIEnv *env, jstring String, jboolean *isCopy) {
  w_string string = String2string(String);
  w_size   max_length = string->length*3;
  jbyte   *result = allocMem(JNIEnv2w_thread(env), (max_length + 1) * sizeof(jbyte));
  jbyte   *utf8ptr = result;
  w_char  *utf16ptr = string->chars;
  w_size   i;
  
  for (i=0; i<string->length; ++i) {
    if (*utf16ptr == 0) {
      *utf8ptr++ = 0xc0;
      *utf8ptr++ = 0x80;
    }
    else if (*utf16ptr < 0x80 ) {
      *utf8ptr++ = *utf16ptr;
    }
    else if (*utf16ptr < 0x800 ) {
      *utf8ptr++ = 0xc0 + (*utf16ptr >> 6);
      *utf8ptr++ = 0x80 + (*utf16ptr & 0x3f);
    }
    else {
      *utf8ptr++ = 0xe0 + (*utf16ptr >> 12);
      *utf8ptr++ = 0x80 + ((*utf16ptr >> 6) & 0x3f);
      *utf8ptr++ = 0x80 + (*utf16ptr & 0x3f);
    }
    utf16ptr++;
  }
  *utf8ptr = '\0'; /* Terminating string */
  if (isCopy) {
    *isCopy = JNI_TRUE;
  }

  return result;
}

void ReleaseStringUTFChars(JNIEnv *env, jstring string, const char *utf) {
  releaseMem((void*)utf);
}                  

jsize GetArrayLength(JNIEnv *env, jarray array) {
  return (jsize)array[F_Array_length];
}
  
jarray NewObjectArray(JNIEnv *env, jsize length, jclass elementType, jobject initialElement) {
  w_instance initval = initialElement;
  w_thread thread = JNIEnv2w_thread(env);
  w_instance Array;
  w_int i;
  w_instance exception = NULL;
  w_clazz elementClazz = Class2clazz(elementType);
  w_desc arrayDesc = elementClazz->adesc;
  w_instance loader = clazz2loader(elementClazz);
  w_clazz arrayClazz;
  w_word alength;

  woempa(9,"Asked to construct an array of %d %k's, all set to 0x%08x\n",length,elementClazz,initval);
  if (!arrayDesc) {
    w_char * desc_buffer;
    w_string desc_string;
    w_char   ch;

    desc_buffer = allocMem(thread, (elementClazz->dotified->length + 3) * sizeof(w_char));
    desc_buffer[0] = (w_char)'[';
    desc_buffer[1] = (w_char)'L';
    for (i = 0; i < (w_int)elementClazz->dotified->length; i++) {
      ch = elementClazz->dotified->chars[i];
      if (ch == '.') {
        desc_buffer[i + 2] = '/';
      }
      else {
        desc_buffer[i + 2] = ch;
      }
    }
    desc_buffer[elementClazz->dotified->length + 2] = ';';
    desc_string = unicode2String(thread, desc_buffer, elementClazz->dotified->length + 3);
    arrayDesc = newFieldDescriptor(thread, desc_string);
    if (arrayDesc == NULL) {
      wabort(ABORT_WONKA,"whoops - failed to parse descriptor `%w'\n", desc_string);
    }

    deregisterString(desc_string);

    woempa(1, "Registered descriptor %w for array of %w.\n", NM(arrayDesc), elementClazz->dotified);

    elementClazz->adesc = arrayDesc;
  }
  arrayClazz = descArrayClassMustBeLoaded(loader,arrayDesc,&exception);
  if (exception) {
    setLocalException(thread,exception);

    return NULL;
  }

  alength = length;
  Array = allocArrayInstance(JNIEnv2w_thread(env), arrayClazz, 1, &alength);

  if (Array) {
    addLocalReference(thread, Array);

    for (i = 0; i < length; i++) {
      Array[F_Array_data + i] = (w_word)initval;
    }

    woempa(1, "(JNI) Length %d, type %k, Array = %p.\n", length, clazz_Array, Array);
  }
  else {
    throwOutOfMemoryError(thread);
  }

  return Array;
  
}

jobject GetObjectArrayElement(JNIEnv *env, jobjectArray Array, jsize aindex) {
  w_instance *instanceArray = instance2Array_instance(Array);

//  woempa(1, "(JNI) (%k)(%p)[%d] = %p.\n", instance2clazz(Array), Array, aindex, (char *)Array[F_Array_data + aindex]);
//  return Array[F_Array_data + aindex];
  woempa(1, "(JNI) (%k)(%p)[%d] = %p.\n", instance2clazz(Array), Array, aindex, instanceArray[aindex]);
  return instanceArray[aindex];
  
}

void SetObjectArrayElement(JNIEnv *env, jobjectArray Array, jsize aindex, jobject value) {
  w_instance wval = value;

  woempa(1, "(JNI) (%k)(%p)[%d] = %p (%k).\n", instance2clazz(Array), Array, aindex, wval, instance2clazz(wval));
  Array[F_Array_data + aindex] = (w_word)wval;
  
}

w_instance newTypeArray(w_thread thread, w_clazz clazz, w_int length) {

  w_instance Array;
  w_word alength;
  
  alength = length;
  Array = allocArrayInstance(thread, clazz, 1, &alength);

  if (Array == NULL) {
    /*
    ** The memory allocation routine will have launched an OutOfMemoryException...
    */
    return nullInstance;
  }

  addLocalReference(thread, Array);
  
  woempa(1, "(JNI) Length %d, type %k, Array = %p.\n", length, clazz, Array);
  
  return Array;
  
}

/*
** Create primitive array types. Look at the 'newarray' opcode explanation
** for the meaning of the array indexes.
*/

jbooleanArray NewBooleanArray(JNIEnv *env, jsize length) {
  return newTypeArray(JNIEnv2w_thread(env), atype2clazz[P_boolean], length);
}

jbyteArray NewByteArray(JNIEnv *env, jsize length) {
  return newTypeArray(JNIEnv2w_thread(env), atype2clazz[P_byte], length);
}

jcharArray NewCharArray(JNIEnv *env, jsize length) {
  return newTypeArray(JNIEnv2w_thread(env), atype2clazz[P_char], length);
}

jshortArray NewShortArray(JNIEnv *env, jsize length) {
  return newTypeArray(JNIEnv2w_thread(env), atype2clazz[P_short], length);
}

jintArray NewIntArray(JNIEnv *env, jsize length) {
  return newTypeArray(JNIEnv2w_thread(env), atype2clazz[P_int], length);
}

jlongArray NewLongArray(JNIEnv *env, jsize length) {
  return newTypeArray(JNIEnv2w_thread(env), atype2clazz[P_long], length);
}

jfloatArray NewFloatArray(JNIEnv *env, jsize length) {
  return newTypeArray(JNIEnv2w_thread(env), atype2clazz[P_float], length);
}

jdoubleArray NewDoubleArray(JNIEnv *env, jsize length) {
  return newTypeArray(JNIEnv2w_thread(env), atype2clazz[P_double], length);
}

jboolean *GetBooleanArrayElements(JNIEnv *env, jbooleanArray array, jboolean *isCopy) {

  w_instance array_instance = array;
  w_size length = array_instance[F_Array_length];
  jboolean *buffer = allocMem(JNIEnv2w_thread(env), length * sizeof(jboolean));
  jboolean* dest = buffer;
  w_sbyte *arrayAsBytes = instance2Array_byte(array_instance);
  w_size i;
  
  woempa(1,"(JNI) array of %d bits at %p expands to %d jbooleans (%d bytes) at %p\n",length,array_instance,length,(w_size)(length*sizeof(jboolean)),buffer);
  for (i=0;i<length;++i) {
    *dest++ = !!(arrayAsBytes[i/8] & (1<<(i%8)));
  }
  
  if (isCopy) {
    *isCopy = WONKA_TRUE;
  }

  return buffer;
}

jbyte *GetByteArrayElements(JNIEnv *env, jbyteArray array, jboolean *isCopy) {

  w_instance array_instance = array;
  w_size length = array_instance[F_Array_length];
  jbyte *buffer = allocMem(JNIEnv2w_thread(env), length * sizeof(jbyte));
  jbyte *dest = buffer;
  w_sbyte *srce = instance2Array_byte(array_instance);
  w_size i;
  
  woempa(1,"(JNI) copy %d bytes from %p to %p\n",length,array_instance+F_Array_data,buffer);
  for (i=0;i<length;++i) {
    *dest++ = *srce++;
  }
  
  if (isCopy) {
    *isCopy = WONKA_TRUE;
  }

  return buffer;
}

jchar *GetCharArrayElements(JNIEnv *env, jcharArray array, jboolean *isCopy) {
  w_instance array_instance = array;
  w_size length = array_instance[F_Array_length];
  jchar *buffer = allocMem(JNIEnv2w_thread(env), length * sizeof(jchar));
  jchar *dest = buffer;
  w_char *srce = instance2Array_char(array_instance);
  w_size i;
  
  woempa(1,"(JNI) copy %d char's (%d bytes) from %p to %p\n",length,(w_size)(length*sizeof(w_char)),array_instance+F_Array_data,buffer);
  for (i=0;i<length;++i) {
    *dest++ = *srce++;
  }
  
  if (isCopy) {
    *isCopy = WONKA_TRUE;
  }

  return buffer;
}

jshort *GetShortArrayElements(JNIEnv *env, jshortArray array, jboolean *isCopy) {

  w_instance array_instance = array;
  w_size length = array_instance[F_Array_length];
  jshort *buffer = allocMem(JNIEnv2w_thread(env), length * sizeof(jshort));
  jshort *dest = buffer;
  w_short *srce = instance2Array_short(array_instance);
  w_size i;
  
  woempa(1,"(JNI) copy %d short's (%d bytes) from %p to %p\n",length,(w_size)(length*sizeof(w_short)),array_instance+F_Array_data,buffer);
  for (i=0;i<length;++i) {
    *dest++ = *srce++;
  }
  
  if (isCopy) {
    *isCopy = WONKA_TRUE;
  }

  return buffer;
}

jint *GetIntArrayElements(JNIEnv *env, jintArray array, jboolean *isCopy) {
  w_instance array_instance = array;
  w_size length = array_instance[F_Array_length];
  jint  *buffer = allocMem(JNIEnv2w_thread(env), length * sizeof(jint));
  jint  *dest = buffer;
  w_int *srce = instance2Array_int(array_instance);
  w_size i;
  
  woempa(1,"(JNI) copy %d int's from %p to %p\n",length,array_instance+F_Array_data,buffer);
  for (i=0;i<length;++i) {
    *dest++ = *srce++;
  }
  
  if (isCopy) {
    *isCopy = WONKA_TRUE;
  }
  
  return buffer;
}

jlong *GetLongArrayElements(JNIEnv *env, jlongArray array, jboolean *isCopy) {
  w_instance array_instance = array;
  w_size length = array_instance[F_Array_length];
  jlong  *buffer = (jlong*)allocMem(JNIEnv2w_thread(env),length*sizeof(jlong));
  jlong  *dest = buffer;
  w_long *srce = instance2Array_long(array_instance);
  w_size i;
  
  woempa(1,"(JNI) copy %d long's from %p to %p\n",length,array_instance+F_Array_data,buffer);
  for (i=0;i<length;++i) {
    *dest++ = *srce++;
  }
  
  if (isCopy) {
    *isCopy = WONKA_TRUE;
  }
  
  return buffer;
}

jfloat *GetFloatArrayElements(JNIEnv *env, jfloatArray array, jboolean *isCopy) {
  w_instance array_instance = array;
  w_size length = array_instance[F_Array_length];
  jfloat  *buffer = (jfloat*)allocMem(JNIEnv2w_thread(env),length*sizeof(jfloat));
  jfloat  *dest = buffer;
  w_float *srce = instance2Array_float(array_instance);
  w_size i;
  
  woempa(1,"(JNI) copy %d float's (%d bytes) from %p to %p\n",length,(w_size)(length*sizeof(w_float)),array_instance+F_Array_data,buffer);
  for (i=0;i<length;++i) {
    *dest++ = *srce++;
  }
  
  if (isCopy) {
    *isCopy = WONKA_TRUE;
  }
  
  return buffer;
}

jdouble *GetDoubleArrayElements(JNIEnv *env, jdoubleArray array, jboolean *isCopy) {
  w_instance array_instance = array;
  w_size length = array_instance[F_Array_length];
  jdouble *buffer = (jdouble*)allocMem(JNIEnv2w_thread(env),length*sizeof(jdouble));
  jdouble  *dest = buffer;
  w_double *srce = instance2Array_double(array_instance);
  w_size i;

  woempa(1,"(JNI) copy %d double's (%d bytes) from %p to %p\n",length,(w_size)(length*sizeof(w_double)),array_instance+F_Array_data,buffer);
  for (i=0;i<length;++i) {
    *dest++ = *srce++;
  }
  
  if (isCopy) {
    *isCopy = WONKA_TRUE;
  }
  
  return buffer;
}

void ReleaseBooleanArrayElements(JNIEnv *env, jbooleanArray array, jboolean *elements, jint mode) {
  w_instance array_instance = array;
  w_int length = array_instance[F_Array_length];

  if (mode!=JNI_ABORT) {
    writeBooleansToArray(array_instance, 0, length, elements);
  }
  
  if (mode!=JNI_COMMIT) {
    woempa(1,"(JNI) release copy at %p\n",elements);
    releaseMem(elements);
  }
}

void ReleaseByteArrayElements(JNIEnv *env, jbyteArray array, jbyte *elements, jint mode) {
  w_instance array_instance = array;
  w_int length = array_instance[F_Array_length];

  if (mode!=JNI_ABORT) {
    writeBytesToArray(array_instance, 0, length, elements);
  }
  
  if (mode!=JNI_COMMIT) {
    woempa(1,"(JNI) release copy at %p\n",elements);
    releaseMem(elements);
  }
}

void ReleaseCharArrayElements(JNIEnv *env, jcharArray array, jchar *elements, jint mode) {
  w_instance array_instance = array;
  w_int length = array_instance[F_Array_length];

  if (mode!=JNI_ABORT) {
    writeCharsToArray(array_instance, 0, length, elements);
  }
  
  if (mode!=JNI_COMMIT) {
    woempa(1,"(JNI) release copy at %p\n",elements);
    releaseMem(elements);
  }
}

void ReleaseShortArrayElements(JNIEnv *env, jshortArray array, jshort *elements, jint mode) {
  w_instance array_instance = array;
  w_int length = array_instance[F_Array_length];

  if (mode!=JNI_ABORT) {
    writeShortsToArray(array_instance, 0, length, elements);
  }
  
  if (mode!=JNI_COMMIT) {
    woempa(1,"(JNI) release copy at %p\n",elements);
    releaseMem(elements);
  }
}

void ReleaseIntArrayElements(JNIEnv *env, jintArray array, jint *elements, jint mode) {
  w_instance array_instance = array;
  w_int length = array_instance[F_Array_length];

  if (mode!=JNI_ABORT) {
    writeIntsToArray(array_instance, 0, length, elements);
  }
  
  if (mode!=JNI_COMMIT) {
    woempa(1,"(JNI) release copy at %p\n",elements);
    releaseMem(elements);
  }
}

void ReleaseLongArrayElements(JNIEnv *env, jlongArray array, jlong *elements, jint mode) {
  w_instance array_instance = array;
  w_int length = array_instance[F_Array_length];

  if (mode!=JNI_ABORT) {
    writeLongsToArray(array_instance, 0, length, elements);
  }
  
  if (mode!=JNI_COMMIT) {
    woempa(1,"(JNI) release copy at %p\n",elements);
    releaseMem(elements);
  }
}

void ReleaseFloatArrayElements(JNIEnv *env, jfloatArray array, jfloat *elements, jint mode) {
  w_instance array_instance = array;
  w_int length = array_instance[F_Array_length];

  if (mode!=JNI_ABORT) {
    writeFloatsToArray(array_instance, 0, length, elements);
  }
  
  if (mode!=JNI_COMMIT) {
    woempa(1,"(JNI) release copy at %p\n",elements);
    releaseMem(elements);
  }
}

void ReleaseDoubleArrayElements(JNIEnv *env, jdoubleArray array, jdouble *elements, jint mode) {
  w_instance array_instance = array;
  w_int length = array_instance[F_Array_length];

  if (mode!=JNI_ABORT) {
    writeDoublesToArray(array_instance, 0, length, elements);
  }
  
  if (mode!=JNI_COMMIT) {
    woempa(1,"(JNI) release copy at %p\n",elements);
    releaseMem(elements);
  }
}                                                        

w_void GetBooleanArrayRegion(JNIEnv *env, jbooleanArray array, jsize start, jsize len, jboolean *buf) {
  readBooleansFromArray(array, start, len, buf);
}

w_void GetByteArrayRegion(JNIEnv *env, jbyteArray array, jsize start, jsize len, jbyte *buf) {
  readBytesFromArray(array, start, len, buf);
}

w_void GetCharArrayRegion(JNIEnv *env, jcharArray array, jsize start, jsize len, jchar *buf) {
  readCharsFromArray(array, start, len, buf);
}

w_void GetShortArrayRegion(JNIEnv *env, jshortArray array, jsize start, jsize len, jshort *buf) {
  readShortsFromArray(array, start, len, buf);
}

w_void GetIntArrayRegion(JNIEnv *env, jintArray array, jsize start, jsize len, jint *buf) {
  readIntsFromArray(array, start, len, buf);
}

w_void GetLongArrayRegion(JNIEnv *env, jlongArray array, jsize start, jsize len, jlong *buf) {
  readLongsFromArray(array, start, len, buf);
}

w_void GetFloatArrayRegion(JNIEnv *env, jfloatArray array, jsize start, jsize len, jfloat *buf) {
  readFloatsFromArray(array, start, len, buf);
}

w_void GetDoubleArrayRegion(JNIEnv *env, jdoubleArray array, jsize start, jsize len, jdouble *buf) {
  readDoublesFromArray(array, start, len, buf);
}

w_void SetBooleanArrayRegion(JNIEnv *env, jbooleanArray array, jsize start, jsize len, jboolean *buf) {
  writeBooleansToArray(array, start, len, buf);
}

w_void SetByteArrayRegion(JNIEnv *env, jbyteArray array, jsize start, jsize len, jbyte *buf) {
  writeBytesToArray(array, start, len, buf);
}

w_void SetCharArrayRegion(JNIEnv *env, jcharArray array, jsize start, jsize len, jchar *buf) {
  writeShortsToArray(array, start, len, buf);
}

w_void SetShortArrayRegion(JNIEnv *env, jshortArray array, jsize start, jsize len, jshort *buf) {
  writeShortsToArray(array, start, len, buf);
}

w_void SetIntArrayRegion(JNIEnv *env, jintArray array, jsize start, jsize len, jint *buf) {
  writeIntsToArray(array, start, len, buf);
}

w_void SetLongArrayRegion(JNIEnv *env, jlongArray array, jsize start, jsize len, jlong *buf) {
  writeLongsToArray(array, start, len, buf);
}

w_void SetFloatArrayRegion(JNIEnv *env, jfloatArray array, jsize start, jsize len, jfloat *buf) {
  writeFloatsToArray(array, start, len, buf);
}

w_void SetDoubleArrayRegion(JNIEnv *env, jdoubleArray array, jsize start, jsize len, jdouble *buf) {
  writeDoublesToArray(array, start, len, buf);
}
                
jint RegisterNatives(JNIEnv *env, jclass class, const JNINativeMethod *methods, jint nMethods) {
  registerNatives(Class2clazz(class), methods, nMethods);
  return 0;
}

jint UnregisterNatives(JNIEnv *env, jclass class) {
  wabort(ABORT_WONKA, "JNI function not yet implemented.\n");
  return -1;
}
  
jint MonitorEnter(JNIEnv *env, jobject obj) {
  w_thread thread = JNIEnv2w_thread(env);

  thread->state = W_THREAD_LOCKING;
  enterMonitor((w_instance)obj);
  thread->state = W_THREAD_READY;
  
  return 0;
  
}

jint MonitorExit(JNIEnv *env, jobject obj) {
  exitMonitor((w_instance)obj);
  return 0;
}
 

/*
** GetPrimitiveArrayCritical / ReleasePrimitiveArrayCritical:
** since all array buffer locks in Wonka are critical, we can just forward these calls
** to the standard Get<type>Arrayelements/Release<type>Arrayelements functions
** according to the right types of array
**
** Version II: merge the codes for the IsInstanceOf(type) and Get/Release<type>Arrayelements into one
** with as few double code and redirections as possible
*/

void* GetPrimitiveArrayCritical(JNIEnv *env, jarray array, jboolean *isCopy)  {
  
  w_instance array_instance = array;
  void     *buffer = NULL;
  w_desc   desc = instance2clazz(array)->desc;
  jboolean tmp;

  /*
  ** according to the array type, fill in buffer space and grab array data pointer
  */

  if(desc == clazz_boolean->adesc ) {
    buffer = GetBooleanArrayElements(env, array_instance, &tmp);
  }
  else if(desc == clazz_byte->adesc) {
    buffer = GetByteArrayElements(env, array_instance, &tmp);
  }
  else if(desc == clazz_char->adesc ) {
    buffer = GetCharArrayElements(env, array_instance, &tmp);
  }
  else if(desc == clazz_short->adesc ) {
    buffer = GetShortArrayElements(env, array_instance, &tmp);
  }
  else if(desc == clazz_int->adesc ) {
    buffer = GetIntArrayElements(env, array_instance, &tmp);
  }
  else if(desc == clazz_long->adesc ) {
    buffer = GetLongArrayElements(env, array_instance, &tmp);
  }
  else if(desc == clazz_float->adesc ) {
    buffer = GetFloatArrayElements(env, array_instance, &tmp);
  }
  else if(desc == clazz_double->adesc ) {
    buffer = GetDoubleArrayElements(env, array_instance, &tmp);
  }
  else {
    buffer = NULL;
  }

  /*
  ** set copy flag to true (this routine ALWAYS copies)
  */

  if (isCopy)
    *isCopy = tmp;

  return buffer;
}


void  ReleasePrimitiveArrayCritical(JNIEnv *env, jarray array, void *carray, jint mode) {
  w_instance array_instance = array;
  w_int length = array_instance[F_Array_length];
  w_desc desc = instance2clazz(array)->desc;

  if (mode!=JNI_ABORT) {
    if(desc == clazz_boolean->adesc ) {
      writeBooleansToArray(array_instance, 0, length, (jboolean*)carray);
    }
    else if(desc == clazz_byte->adesc ) {
      writeBytesToArray(array_instance, 0, length, (jbyte*)carray);
    }
    else if(desc == clazz_char->adesc ) {
      writeCharsToArray(array_instance, 0, length, (jchar*)carray);
    }
    else if(desc == clazz_short->adesc ) {
      writeShortsToArray(array_instance, 0, length, (jshort*)carray);
    }
    else if(desc == clazz_int->adesc ) {
      writeIntsToArray(array_instance, 0, length, (jint*)carray);
    }
    else if(desc == clazz_long->adesc ) {
      writeLongsToArray(array_instance, 0, length, (jlong*)carray);
    }
    else if(desc == clazz_float->adesc ) {
      writeFloatsToArray(array_instance, 0, length, (jfloat*)carray);
    }
    else if(desc == clazz_double->adesc ) {
      writeDoublesToArray(array_instance, 0, length, (jdouble*)carray);
    }
  }

  if (mode!=JNI_COMMIT) {
    woempa(1,"(JNI) release copy at %p\n",carray);
    releaseMem(carray);
  }
}

/*
** GetStringCritical / ReleaseStringCritical:
** since all string buffer locks in Wonka are critical, we can just forward these calls
** to the standard GetStringchars/ReleaseStringChars functions
*/

const jchar *GetStringCritical(JNIEnv *env, jstring String, jboolean *isCopy) {
  woempa(1,"redirecting GetStringCritical to GetStringChars(Always critical in Wonka)\n");
  return GetStringChars(env,String,isCopy);
}


void ReleaseStringCritical(JNIEnv *env, jstring string, const jchar *chars) {
  woempa(1,"redirecting ReleaseStringCritical to ReleaseStringChars(Always critical in Wonka)\n");
  ReleaseStringChars(env, string,chars);
}

/*
** There is only one JVM; he wears green trousers and a plum jacket,
** and sports a top hat and a walking-cane.
*/

static JavaVM mrWonka;

/*
** You can ask for any version you want, so long as it's positive.
** Mr. Wonka hates negative attitudes.
*/

jint JNI_GetDefaultJavaVMInitArgs(void *vm_args) {
  return ((Wonka_InitArgs*)vm_args)->version & 0x80000000 ? -1 : 0;
}

/*
** He is unique.
*/

jint JNI_GetCreatedJavaVMs(JavaVM **vmBuf, jsize bufLen, jsize *nVMs) {
  if (bufLen) {
    mrWonka = &w_JNIInvokeInterface;
    *vmBuf = &mrWonka;
    *nVMs = 1;
  }
  return 0;
}

jint JNI_CreateJavaVM(JavaVM **p_VM, JNIEnv **p_env, void *vm_args) {

  globals_hashtable = ht_create(W_Realm_system, (char*)"hashtable:global-refs", GLOBALS_HASHTABLE_SIZE, NULL, NULL, 0, 0);
  woempa(1, "(JNI) created globals_hashtable at %p\n", globals_hashtable);

  system_vm_args = (struct JavaVMInitArgs *)vm_args;

  startDescriptors();

  startHeap();
  startLoading();
  startKernel();
  
#ifdef RUDOLPH  
#ifdef MODULES 
  loadModule("mod_awt");
#else 
  init_awt();
#endif
#endif

  return 0;
}

jint DestroyJavaVM(JavaVM *vm) {
  woempa(9,"Sorry, you can't do that.\n");
  return -1;
}

/*
** Bogus native method used for verisimilitude.
** TODO: make this more realistic?
*/

static w_Method Bogus;

/*
** Pointer to bogus method.
*/

static w_method bogus = &Bogus;

/*
** In order to visit the Wonka Factory, you must be in possesion of valid
** "pieces of identity".  This function will furnish you with them.
**
** Note: not every field of the w_Thread structure is filled in here.
** Maybe we need to do more?
*/

jint AttachCurrentThread(JavaVM *vm, JNIEnv **p_env, void *thr_args) {
  w_instance Thread;
  w_thread   thread;
  w_frame    frame;
  w_string   method_name;
  w_method   method;
  Wonka_AttachArgs *wonka_args = thr_args;
  char *cname = (char*)"C-Thread";

  if (thread_hashtable == NULL) {

    return -1;

  }

  if (wonka_args && wonka_args->name) {
    cname = wonka_args->name;
  }

  thread  = allocClearedMem_system(sizeof(w_Thread));
  thread->natenv = &w_JNINativeInterface;
  thread->label = (char*)"thread:native";
  thread->name = cstring2String(W_ThreadGroup_system->manager,cname,strlen(cname));
  thread->state = W_THREAD_READY;
  thread->messages = (x_queue)allocMem(W_ThreadGroup_system->manager, sizeof(x_Queue));
  x_queue_create(thread->messages, allocMem_system(4*sizeof(void*)),4);
  thread->group = W_ThreadGroup_system;
  
  /* 
  ** The following is bogus, but means that child threads default to a reasonable priority
  */

  thread->jpriority = W_ThreadGroup_system->maxPriority / 2; 
  thread->isDaemon = WONKA_FALSE;
  thread->flags = 0;
  thread->kthread = (x_thread)x_thread_current();
  thread->kthread->xref = thread;
  thread->kpriority = x_thread_priority_get(thread->kthread);
  woempa(1, "Registering os thread %p as Java thread %p\n", thread->kthread, thread);
  ht_write(thread_hashtable, (w_word)thread->kthread, (w_word)thread);
  classMustBeInitialized(clazzNativeThread, thread);

  Thread = allocInstance(W_ThreadGroup_system->manager, clazzNativeThread);
  Thread[F_Thread_parent] = (w_word)I_ThreadGroup_system;

  Thread[F_Thread_wotsit] = (w_word)thread;
  thread->Thread = Thread;
  Thread[F_Thread_wotsit] = (w_word)thread;
  setReferenceField(Thread, I_ThreadGroup_system, F_Thread_parent);
  thread->Thread = Thread;
  woempa(1, "Native thread %p ('%w') now has instance of %k at %p.\n", thread, NM(thread), clazzNativeThread, Thread);

  method_name = cstring2String(W_Thread_system,"_",1);
  method = getMethod_name_sig(clazzNativeThread, method_name, string_no_params);
  deregisterString(method_name);
  if (method==NULL) {
    wabort(ABORT_WONKA,"Uh oh: class %k doesn't have a run() method.  Game over.\n",clazzNativeThread);
  }

  frame = allocBottomFrame(thread,method);
  frame->locals[0] = (w_word)I_Thread_sysInit;
  if (frame) {
    bogus->clazz = clazzNativeThread;
    frame->native = bogus;
 
    addThreadToGroup(thread,W_ThreadGroup_system);
    numThreads += 1;

    *p_env = w_thread2JNIEnv(thread);

    return 0;

  }
  else {

    return -1;

  }
}

/*
** Thankyou for visiting the Wonka factory.  Don't forget to give your
** badge to the security squirrel on the way out.
**
** TODO: figure out how to release all monitors held by a thread which
** is dumb enough to call this function from inside a monitor. 8-0
*/

jint DetachCurrentThread(JavaVM *vm) {
  w_thread thread = currentWonkaThread;

  releaseBottomFrame(thread->bottom);
  removeThreadFromGroup(thread,W_ThreadGroup_system);
  ht_erase(thread_hashtable,(w_word)thread->kthread);
  numThreads -= 1;

  return 0;
}


const struct JNINativeInterface w_JNINativeInterface = {

  NULL,
  NULL,
  NULL,
  NULL,
  GetVersion,
  
  DefineClass,
  FindClass,

  NULL,
  NULL,
  NULL,
  
  GetSuperclass,                        /* 10 */
  IsAssignableFrom,
  
  NULL,

  Throw,
  ThrowNew,
  ExceptionOccurred,
  ExceptionDescribe,
  ExceptionClear,
  FatalError,
  
  NULL,
  NULL,                                 /* 20 */
  
  NewGlobalRef,
  DeleteGlobalRef,
  DeleteLocalRef,
  IsSameObject,
  
  NewLocalRef,
  NULL,
  
  AllocObject,
  NewObject,
  NewObjectV,
  NewObjectA,                           /* 30 */

  GetObjectClass,
  IsInstanceOf,
  
  GetMethodID,

  CallObjectMethod,
  CallObjectMethodV,
  CallObjectMethodA,
  CallBooleanMethod,
  CallBooleanMethodV,
  CallBooleanMethodA,
  CallByteMethod,                       /* 40 */
  CallByteMethodV,
  CallByteMethodA,
  CallCharMethod,
  CallCharMethodV,
  CallCharMethodA,
  CallShortMethod,
  CallShortMethodV,
  CallShortMethodA,
  CallIntMethod,
  CallIntMethodV,                       /* 50 */
  CallIntMethodA,
  CallLongMethod,
  CallLongMethodV,
  CallLongMethodA,
  CallFloatMethod,
  CallFloatMethodV,
  CallFloatMethodA,
  CallDoubleMethod,
  CallDoubleMethodV,
  CallDoubleMethodA,                    /* 60 */
  CallVoidMethod,
  CallVoidMethodV,
  CallVoidMethodA,

  CallNonvirtualObjectMethod,
  CallNonvirtualObjectMethodV,
  CallNonvirtualObjectMethodA,
  CallNonvirtualBooleanMethod,
  CallNonvirtualBooleanMethodV,
  CallNonvirtualBooleanMethodA,
  CallNonvirtualByteMethod,             /* 70 */
  CallNonvirtualByteMethodV,
  CallNonvirtualByteMethodA,
  CallNonvirtualCharMethod,
  CallNonvirtualCharMethodV,
  CallNonvirtualCharMethodA,
  CallNonvirtualShortMethod,
  CallNonvirtualShortMethodV,
  CallNonvirtualShortMethodA,
  CallNonvirtualIntMethod,
  CallNonvirtualIntMethodV,             /* 80 */
  CallNonvirtualIntMethodA,
  CallNonvirtualLongMethod,
  CallNonvirtualLongMethodV,
  CallNonvirtualLongMethodA,
  CallNonvirtualFloatMethod,
  CallNonvirtualFloatMethodV,
  CallNonvirtualFloatMethodA,
  CallNonvirtualDoubleMethod,
  CallNonvirtualDoubleMethodV,
  CallNonvirtualDoubleMethodA,          /* 90 */
  CallNonvirtualVoidMethod,
  CallNonvirtualVoidMethodV,
  CallNonvirtualVoidMethodA,

  GetFieldID,

  GetObjectField,
  GetBooleanField,
  GetByteField,
  GetCharField,
  GetShortField,
  GetIntField,                          /* 100 */
  GetLongField,
  GetFloatField,
  GetDoubleField,
  SetObjectField,
  SetBooleanField,
  SetByteField,
  SetCharField,
  SetShortField,
  SetIntField,
  SetLongField,                         /* 110 */
  SetFloatField,
  SetDoubleField,

  GetStaticMethodID,

  CallStaticObjectMethod,
  CallStaticObjectMethodV,
  CallStaticObjectMethodA,
  CallStaticBooleanMethod,
  CallStaticBooleanMethodV,
  CallStaticBooleanMethodA,
  CallStaticByteMethod,                 /* 120 */
  CallStaticByteMethodV,
  CallStaticByteMethodA,
  CallStaticCharMethod,
  CallStaticCharMethodV,
  CallStaticCharMethodA,
  CallStaticShortMethod,
  CallStaticShortMethodV,
  CallStaticShortMethodA,
  CallStaticIntMethod,
  CallStaticIntMethodV,                 /* 130 */
  CallStaticIntMethodA,
  CallStaticLongMethod,
  CallStaticLongMethodV,
  CallStaticLongMethodA,
  CallStaticFloatMethod,
  CallStaticFloatMethodV,
  CallStaticFloatMethodA,
  CallStaticDoubleMethod,
  CallStaticDoubleMethodV,
  CallStaticDoubleMethodA,              /* 140 */
  CallStaticVoidMethod,
  CallStaticVoidMethodV,
  CallStaticVoidMethodA,

  GetStaticFieldID,

  GetStaticObjectField,
  GetStaticBooleanField,
  GetStaticByteField,
  GetStaticCharField,
  GetStaticShortField,
  GetStaticIntField,                    /* 150 */
  GetStaticLongField,
  GetStaticFloatField,
  GetStaticDoubleField,
  SetStaticObjectField,
  SetStaticBooleanField,
  SetStaticByteField,
  SetStaticCharField,
  SetStaticShortField,
  SetStaticIntField,
  SetStaticLongField,                   /* 160 */
  SetStaticFloatField,
  SetStaticDoubleField,

  NewString,
  GetStringLength,
  GetStringChars,
  ReleaseStringChars,

  NewStringUTF,
  GetStringUTFLength,
  GetStringUTFChars,
  ReleaseStringUTFChars,                /* 170 */

  GetArrayLength,
  
  NewObjectArray,
  GetObjectArrayElement,
  SetObjectArrayElement,

  NewBooleanArray,
  NewByteArray,
  NewCharArray,
  NewShortArray,
  NewIntArray,
  NewLongArray,                         /* 180 */
  NewFloatArray,
  NewDoubleArray,

  GetBooleanArrayElements,
  GetByteArrayElements,
  GetCharArrayElements,
  GetShortArrayElements,
  GetIntArrayElements,
  GetLongArrayElements,
  GetFloatArrayElements,
  GetDoubleArrayElements,               /* 190 */

  ReleaseBooleanArrayElements,
  ReleaseByteArrayElements,
  ReleaseCharArrayElements,
  ReleaseShortArrayElements,
  ReleaseIntArrayElements,
  ReleaseLongArrayElements,
  ReleaseFloatArrayElements,
  ReleaseDoubleArrayElements,

  GetBooleanArrayRegion,
  GetByteArrayRegion,                   /* 200 */
  GetCharArrayRegion,
  GetShortArrayRegion,
  GetIntArrayRegion,
  GetLongArrayRegion,
  GetFloatArrayRegion,
  GetDoubleArrayRegion,

  SetBooleanArrayRegion,
  SetByteArrayRegion,
  SetCharArrayRegion,
  SetShortArrayRegion,                  /* 210 */
  SetIntArrayRegion,
  SetLongArrayRegion,
  SetFloatArrayRegion,
  SetDoubleArrayRegion,

  RegisterNatives,
  UnregisterNatives,
  
  MonitorEnter,
  MonitorExit,

  NULL, //reserved219,
  NULL, //GetStringRegion,              /* 220 */
  NULL, //reserved221,
  GetPrimitiveArrayCritical,
  ReleasePrimitiveArrayCritical,
  GetStringCritical,
  ReleaseStringCritical,
  NULL, //NewWeakGlobalRef,
  NULL, //reserved227,
  NULL, //ExceptionCheck,

};

const struct JNIInvokeInterface w_JNIInvokeInterface = {
  NULL,
  NULL,
  NULL,
  DestroyJavaVM,
  AttachCurrentThread,
  DetachCurrentThread,
};


