/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/



#include <string.h>

#include "arrays.h"
#include "bar.h"
#include "clazz.h"
#include "checks.h"
#include "constant.h"
#include "core-classes.h"
#include "descriptor.h"
#include "exception.h"
#include "fields.h"
#include "fifo.h"
#include "interpreter.h"
#include "heap.h"
#include "list.h"
#include "loading.h"
#include "locks.h"
#include "ts-mem.h"
#include "methods.h"
#include "stack.h"
#include "strings.h"
#include "wonka.h"
#include "threads.h"
#include "hashtable.h"
#include "repository.h"
#include "vfs.h"
#include "argument.h"
#include "jdwp.h"
#include "jdwp_events.h"

/*
** Table of initializers, indexed by CLAZZ_STATE.
*/
w_clazz *(make_initialized)(w_thread, w_clazz, w_instance*);

w_clazz (*initialize_table[])(w_thread, w_clazz, w_instance*) = {
};

w_instance systemClassLoader = NULL;

/*
** A hashtable which maps class names onto ``fixup'' functions.
*/
w_hashtable fixup_hashtable;
/*
** The initial size of \texttt{fixup_hashtable}
*/
#define NATIVE_FUN_HT_SIZE 37

/*
** A hashtable which maps class x imethod -> method
*/
w_hashtable2k interface_hashtable;
/*
** The initial size of \texttt{interface_hashtable}
*/
#define INTERFACE_HT_SIZE 101

char     *zipname;

#ifndef BOOTCLASSDIR
#define BOOTCLASSDIR "."
#endif

#ifndef BOOTCLASSFILE
#define BOOTCLASSFILE "classes.zip"
#endif

/*
** Some well-known strings that are required during loading.
** They are registered in startLoading().
*/
// Names of primitive classes
w_string string_boolean;
w_string string_byte;
w_string string_char;
w_string string_double;
w_string string_float;
w_string string_int;
w_string string_long;
w_string string_short;
w_string string_void;

// Names of classfile attributes
w_string string_ConstantValue;
w_string string_Code;
w_string string_Deprecated;
w_string string_Exceptions;
w_string string_LineNumberTable;
w_string string_LocalVariableTable;
w_string string_Reference;
w_string string_SourceFile;
w_string string_Synthetic;

// Names and signatures of special methods
w_string string_run;
w_string string_finalize;
w_string string_angle_brackets_init;
w_string string_angle_brackets_clinit;
w_string string_no_params_V;
w_string string_no_params;

// Strings used when throwing ClassFormatError
w_string string_error_wrong_name;
w_string string_error_too_short;
w_string string_error_too_long;
w_string string_error_bad_magic;
w_string string_error_bad_constant;

/*
** The Class objects representing the primitive classess, and the w_clazz
** pointers to their companion w_Clazz structures.
**
** N.B. Not to be confused with the "wrapper" classes Boolean, Character, etc.!
** Think of boolean.class, char.class, ...
*/
w_instance Class_boolean;
w_instance Class_char;
w_instance Class_float;
w_instance Class_double;
w_instance Class_byte;
w_instance Class_short;
w_instance Class_int;
w_instance Class_long;
w_instance Class_void;

w_clazz atype2clazz[12];

w_method loadClass_method;

void setSystemClassLoader(w_instance scl) {
  x_monitor_enter(&system_class_hashtable->monitor, x_eternal);
  if (systemClassLoader) {
    woempa(9, "Ahoy there! Someone tried to instal SystemClassLoader twice ...\n");
  }
  else {
    woempa(7, "*** SystemClassLoader created, instance is %p ***\n", scl);
    systemClassLoader = scl;
    if (loader2hashtable(scl)->occupancy) {
      wabort(ABORT_WONKA, "SystemClassLoader hashtable already contains %d entries!\n", loader2hashtable(scl)->occupancy);
    }
/*
    ht_destroy(loader2hashtable(scl));
    scl[F_ClassLoader_hashtable] = (w_word)system_class_hashtable;
*/
    destroyRepository(W_Repository_system);
    W_Repository_system = NULL;
  }
  x_monitor_exit(&system_class_hashtable->monitor);
}

static w_clazz createPrimitive(w_string name) {
  w_clazz clazz = allocClazz(W_Thread_system);

  clazz->dotified = registerString(name);
  clazz->flags = CLAZZ_IS_PRIMITIVE | (CLAZZ_STATE_LOADED << CLAZZ_STATE_SHIFT);
  clazz->loader = NULL;
  woempa(7, "Created clazz_%w @ %p\n", name, clazz);

  return clazz;
}

static void preparePrimitive(w_clazz clazz, w_string string, w_instance * classref, w_desc desc, w_int pi) {

  w_instance Class;

  clazz->verify = alreadyVerified;
  clazz->prepare = alreadyPrepared;
  Class = allocInstanceWithFlags(W_Thread_system, clazzClass, O_SYSTEM);
    
  Class[F_Class_loader] = (w_word)NULL;
  Class[F_Class_wotsit] = (w_word)clazz;
  clazz->Class = Class;
  clazz->desc = desc;
  primitive2classInstance[pi] = Class;
  primitive2clazz[pi] = clazz;

  /*
  ** The void class doesn't have an array descriptor.
  */
    
  if (pi == P_void) {
    clazz->adesc = NULL;
  }
  else {
    clazz->adesc = atype2desc[pi];
  }

  registerClazz(clazz, system_class_hashtable);

  *classref = Class;
}

void startLoading(void) {

  char            *files_data;
  int              fd;
  struct vfs_STAT  statbuf;
  w_word           files_size;
  w_instance       dummy = NULL;
  w_int            i;
  w_int            j;

  w_string string_java_lang_Object = cstring2String(W_Thread_system, "java.lang.Object", 16);
  w_string string_java_lang_Cloneable = cstring2String(W_Thread_system, "java.lang.Cloneable", 19);
  w_string string_java_io_Serializable = cstring2String(W_Thread_system, "java.io.Serializable", 20);
  w_string string_java_lang_Throwable = cstring2String(W_Thread_system, "java.lang.Throwable", 19);
  w_string string_java_lang_Class = cstring2String(W_Thread_system, "java.lang.Class", 15);
  w_string string_java_lang_String = cstring2String(W_Thread_system, "java.lang.String", 16);
  w_string string_java_lang_ClassLoader = cstring2String(W_Thread_system, "java.lang.ClassLoader", 21);
  w_string string_java_lang_Thread = cstring2String(W_Thread_system, "java.lang.Thread", 16);
  w_string string_java_lang_ThreadGroup = cstring2String(W_Thread_system, "java.lang.ThreadGroup", 21);
  w_string string_java_lang_ref_Reference = cstring2String(W_ThreadGroup_system->manager, "java.lang.ref.Reference", 23);

  w_string string_ArrayOf_Object  = cstring2String(W_Thread_system, "[Ljava/lang/Object;", 19);
  w_string string_ArrayOf_Class   = cstring2String(W_Thread_system, "[Ljava/lang/Class;", 18);
  w_string string_ArrayOf_String  = cstring2String(W_Thread_system, "[Ljava/lang/String;", 19);
  w_string string_ArrayOf_Constructor  = cstring2String(W_Thread_system, "[Ljava/lang/reflect/Constructor;", 32);
  w_string string_ArrayOf_Field  = cstring2String(W_Thread_system, "[Ljava/lang/reflect/Field;", 26);
  w_string string_ArrayOf_Method  = cstring2String(W_Thread_system, "[Ljava/lang/reflect/Method;", 27);

  w_string string_loadClass_name = cstring2String(W_Thread_system, "loadClass", 9);
  w_string string_loadClass_sig = cstring2String(W_Thread_system, "(Ljava/lang/String;Z)", 21);

  string_boolean = cstring2String(W_Thread_system, "boolean", 7);
  string_byte = cstring2String(W_Thread_system, "byte", 4);
  string_char = cstring2String(W_Thread_system, "char", 4);
  string_double = cstring2String(W_Thread_system, "double", 6);
  string_float = cstring2String(W_Thread_system, "float", 5);
  string_int = cstring2String(W_Thread_system, "int", 3);
  string_long = cstring2String(W_Thread_system, "long", 4);
  string_short = cstring2String(W_Thread_system, "short", 5);
  string_void = cstring2String(W_Thread_system, "void", 4);

  string_ConstantValue = cstring2String(W_Thread_system, "ConstantValue", 13);
  string_Code = cstring2String(W_Thread_system, "Code", 4);
  string_Deprecated = cstring2String(W_Thread_system, "Deprecated", 10);
  string_Exceptions = cstring2String(W_Thread_system, "Exceptions", 10);
  string_LineNumberTable = cstring2String(W_Thread_system, "LineNumberTable", 15);
  string_LocalVariableTable = cstring2String(W_Thread_system, "LocalVariableTable", 18);
  string_Reference = cstring2String(W_Thread_system, "java/lang/ref/Reference", 23);
  string_SourceFile = cstring2String(W_Thread_system, "SourceFile", 10);
  string_Synthetic = cstring2String(W_Thread_system, "Synthetic", 9);

  string_error_wrong_name = cstring2String(W_Thread_system, "class has wrong name", 21);
  string_error_too_short = cstring2String(W_Thread_system, "class file truncated", 20);
  string_error_too_long = cstring2String(W_Thread_system, "junk at end of class file", 25);
  string_error_bad_magic = cstring2String(W_Thread_system, "bad magic number", 16);
  string_error_bad_constant = cstring2String(W_Thread_system, "bad constant pool entry", 23);

  string_run = cstring2String(W_Thread_system, "run", 3);
  string_finalize = cstring2String(W_Thread_system, "finalize", 8);
  string_angle_brackets_init = cstring2String(W_Thread_system, "<init>", 6);
  string_angle_brackets_clinit = cstring2String(W_Thread_system, "<clinit>", 8);
  string_no_params = cstring2String(W_Thread_system, "()", 2);
  string_no_params_V = cstring2String(W_Thread_system, "()V", 3);

  clazz_boolean = createPrimitive(string_boolean);
  clazz_char    = createPrimitive(string_char);
  clazz_float   = createPrimitive(string_float);
  clazz_double  = createPrimitive(string_double);
  clazz_byte    = createPrimitive(string_byte);
  clazz_short   = createPrimitive(string_short);
  clazz_int     = createPrimitive(string_int);
  clazz_long    = createPrimitive(string_long);
  clazz_void    = createPrimitive(string_void);

  /*
  ** Hack the bootclasspath to extract the first element which does not end
  ** in '/': this must be a jar (or zip) file.
  ** Skip over the leading '/' if present.
  */
  zipname = NULL;

  if (bootclasspath) {
    i = 0;
    j = 0;
    woempa(7, "bootclasspath is %s\n", bootclasspath);
    while (*bootclasspath) {
      for (j = i; j < (w_int)strlen(bootclasspath) && bootclasspath[j] != ':'; ++j);
      woempa(1, "Element ends at bootclasspath[%d]\n", j);
      if (j && bootclasspath[j - 1] != '/') {
        woempa(1, "Element does not end in `/', so we use bootclasspath[%d..%d]\n", i, j);

        break;

      }
      woempa(1, "Element ends with `/', start again from bootclasspath[%d]\n", j + 1);
      i = j + 1;
    }

    if (i < (w_int)strlen(bootclasspath)) {
      zipname = allocMem_system(j - i + 1);
      strncpy(zipname, &bootclasspath[i], j - i);
      zipname[j - i] = 0;
      if (zipname[0] == '/') {
        woempa(7, "Path `%s' starts with `/', so skip first char\n", zipname);
        ++zipname;
      }
      woempa(7, "Using path `%s'\n", zipname);
    }
  }

  /*
  ** If that didn't work (or we have no bootclasspath), use the default.
  */
  if (zipname == NULL) {
    zipname = (char*)BOOTCLASSDIR "/" BOOTCLASSFILE;
  }

  /*
  ** Read in the bootstrap classes
  */
  if(vfs_stat(zipname, &statbuf) == -1) { 
    wabort(ABORT_WONKA, "Could not find boot class library`%s'.\n", zipname);
  } 

  files_size = statbuf.st_size;
  woempa(9, "%s is %d bytes\n", zipname, files_size);
  files_data = allocMem(W_Thread_system, files_size);
  fd = vfs_open(zipname, VFS_O_RDONLY, 0);
  vfs_read(fd, files_data, files_size);
  vfs_close(fd);
  W_Repository_system = createRepository(W_Thread_system, (w_byte *)files_data, files_size);
  releaseMem(files_data);

  fixup_hashtable = ht_create(W_Realm_system,(char*)"hashtable:fixups",NATIVE_FUN_HT_SIZE,ht_stringHash,NULL,0,0);
  interface_hashtable = ht2k_create(W_Realm_system,(char*)"hashtable:interfaces",INTERFACE_HT_SIZE);

  system_class_hashtable = ht_create(W_Realm_system,(char*)"hashtable:system-classes", 2345, ht_stringHash, NULL, 0, 0);
  woempa(1,"created system_class_hashtable at %p\n",system_class_hashtable);

  collectCoreFixups();

  /*
  ** Initialize base classes, we have to resolve things manually so keep
  ** this procedure intact.
  **  1) load and prepare class java/lang/Object "by hand".
  **     Create the null instance (of class java/lang/Object).
  **  2) create clazz_Array. clazz_Array acts like clazzObject but has
  **     a modified method table and a modified interfaces table. 
  **     It overrides the clone method of java/lang/Object and adds two 
  **     new interfaces: Cloneable and Serializable.
  **  3) load and prepare java/lang/Cloneable, java/io/Serializable and java/lang/Throwable. 
  **     (We need to do this before any subclasses of Throwable are loaded,
  **     in order to ensure that they are marked CLAZZ_IS_THROWABLE).
  **  4) load and prepare java/lang/Class.
  **  5) attach an instance of java/lang/Class to each of java/lang/Object,
  **     java/lang/Cloneable, java/io/Serializable, java/lang/Throwable, java/lang/Class.
  **  6) load java/lang/String.
  **  7) load java/lang/ClassLoader.
  **  8) load java/lang/Thread and java/lang/ThreadGroup.
  **  9) load java/lang/ref/Reference.
  **  10) load all the classes mentioned in core-classes.in.
  **  11) initialize classes java/lang/Object,
  **     java/lang/Class, java/lang/String, java/lang/Thread,
  **     java/lang/ThreadGroup, and java/lang/Throwable.
  **  12) For each primitive class xxx, create:
  **       -An instance Class_xxx of class java/lang/Class, linked to
  **        a w_Clazz structure (clazz_xxx) and thereby to the primitive
  **        descriptor (desc_xxx) and array descriptor (atype2desc[P_xxx]))
  **        which were created in startDescriptors (descriptor.c).
  **        The name associated with the Class instance is "xxx.class".
  **       -Entries in the arrays primitive2classInstance[], atype2clazz[],
  **        and atype2desc[], which are indexed by P_xxx.
  **
  ** From that moment on, it should be possible to load any class without
  ** resolution problems and  to create instances of Object, Class, String
  ** Thread, or ThreadGroup without first checking that the class is prepared.
  */
  
  x_monitor_enter(&system_class_hashtable->monitor, x_eternal);

  /****
  * 1 *
  ****/

  woempa(7,"Step 1: load & prepare java.lang.Object\n");
// load class java/lang/Object
  clazzObject = loadBootstrapClass(string_java_lang_Object, &dummy);
//  prepare class java/lang/Object
  createClazzTables(NULL, clazzObject);
  clazzObject->verify  = alreadyVerified;
  clazzObject->prepare = alreadyPrepared;

  /****
  * 2 *
  ****/

  woempa(7,"Step 2: create array pseudo-class\n");
  F_Array_length = clazzObject->instanceSize; 
  F_Array_data = clazzObject->instanceSize + 1; 
  clazz_Array = createClazzArray();

 /****
 * 3 *
 ****/

  woempa(7,"Step 3: load & prepare java.lang.Cloneable java.io.Serializable, java.lang.Throwable\n");
// load class java/lang/Cloneable
  clazzCloneable = seekClazzByName(string_java_lang_Cloneable, system_class_hashtable);
  if (!clazzCloneable) {
    clazzCloneable = loadBootstrapClass(string_java_lang_Cloneable, &dummy);
  }
// prepare interface java/lang/Cloneable
  createClazzTables(NULL,clazzCloneable);
  clazzCloneable->flags |= CLAZZ_IS_THROWABLE;
  clazzCloneable->verify  = alreadyVerified;
  clazzCloneable->prepare = alreadyPrepared;
// load class java/lang/Serializable
  clazzSerializable = seekClazzByName(string_java_io_Serializable, system_class_hashtable);
  if (!clazzSerializable) {
    clazzSerializable = loadBootstrapClass(string_java_io_Serializable, &dummy);
  }
// prepare interface java/io/Serializable
  createClazzTables(NULL,clazzSerializable);
  clazzSerializable->flags |= CLAZZ_IS_THROWABLE;
  clazzSerializable->verify  = alreadyVerified;
  clazzSerializable->prepare = alreadyPrepared;
// load class java/lang/Throwable
  clazzThrowable = seekClazzByName(string_java_lang_Throwable, system_class_hashtable);
  if (!clazzThrowable) {
    clazzThrowable = loadBootstrapClass(string_java_lang_Throwable, &dummy);
  }
// prepare class java/lang/Throwable
  createClazzTables(NULL,clazzThrowable);
  clazzThrowable->flags |= CLAZZ_IS_THROWABLE;
  clazzThrowable->verify  = alreadyVerified;
  clazzThrowable->prepare = alreadyPrepared;
 
  clazz_Array->interfaces[0] = clazzCloneable;
  clazz_Array->interfaces[1] = clazzSerializable;
    
 /****
 * 4 *
 ****/

  woempa(7,"Step 4: load & prepare java.lang.Class\n");
// load class java/lang/Class
  clazzClass = seekClazzByName(string_java_lang_Class, system_class_hashtable);
  if (!clazzClass) {
    clazzClass = loadBootstrapClass(string_java_lang_Class, &dummy);
  }
//  prepare class java/lang/Class
  createClazzTables(NULL,clazzClass);
  clazzClass->verify  = alreadyVerified;
  clazzClass->prepare = alreadyPrepared;

 /****
 * 5 *
 ****/

  woempa(7,"Step 5: attach instance of Class to Object, Cloneable, Serializable, Throwable, Class\n");
  attachClassInstance(clazzObject);
  clazz2Class(clazzObject)[F_Class_loader] = 0;
  attachClassInstance(clazzCloneable);
  clazz2Class(clazzCloneable)[F_Class_loader] = 0;
  attachClassInstance(clazzSerializable);
  clazz2Class(clazzSerializable)[F_Class_loader] = 0;
  attachClassInstance(clazzThrowable);
  clazz2Class(clazzThrowable)[F_Class_loader] = 0;
  attachClassInstance(clazzClass);
  clazz2Class(clazzClass)[F_Class_loader] = 0;

/****
 * 6 *
 ****/

  woempa(7,"Step 6: load java.lang.String\n");
// load class java/lang/String
  clazzString = seekClazzByName(string_java_lang_String, system_class_hashtable);
  if (!clazzString) {
    clazzString = loadBootstrapClass(string_java_lang_String, &dummy);
  }

 /****
 * 7 *
 ****/

  woempa(7,"Step 7: load java.lang.ClassLoader\n");
// load class java/lang/ClassLoader
  clazzClassLoader = seekClazzByName(string_java_lang_ClassLoader, system_class_hashtable);
  if (!clazzClassLoader) {
    clazzClassLoader = loadBootstrapClass(string_java_lang_ClassLoader, &dummy);
  }
  clazzClassLoader->flags |= CLAZZ_IS_CLASSLOADER;

 /****
 * 8 *
 ****/

  woempa(7,"Step 8: load java.lang.Thread, java.lang.ThreadGroup\n");
// load class java/lang/Thread
  clazzThread = seekClazzByName(string_java_lang_Thread, system_class_hashtable);
  if (!clazzThread) {
    clazzThread = loadBootstrapClass(string_java_lang_Thread, &dummy);
  }
  clazzThread->flags |= CLAZZ_IS_THREAD;
// load class java/lang/ThreadGroup
  clazzThreadGroup = seekClazzByName(string_java_lang_ThreadGroup, system_class_hashtable);
  if (!clazzThreadGroup) {
    clazzThreadGroup = loadBootstrapClass(string_java_lang_ThreadGroup, &dummy);
  }
  clazzThreadGroup->flags |= CLAZZ_IS_THREADGROUP;

 /****
 * 9 *
 ****/

  woempa(7,"Step 9: load java.lang.ref.Reference\n");
// load class java/lang/ref/Reference
  clazzReference = seekClazzByName(string_java_lang_ref_Reference, system_class_hashtable);
  if (!clazzReference) {
    clazzReference = loadBootstrapClass(string_java_lang_ref_Reference, &dummy);
  }
  clazzReference->flags |= CLAZZ_IS_REFERENCE;

 /*****
 * 10 *
 *****/
  woempa(7,"Step 10: loading core classes \n");
  loadCoreClasses();
  x_monitor_exit(&system_class_hashtable->monitor);

 /*****
 * 11 *
 *****/
  woempa(7, "Step 11: initialize Object, String, Class, ClassLoader, Throwable, Thread, ThreadGroup\n");
  classMustBeInitialized(clazzObject, W_ThreadGroup_system->manager);
  classMustBeInitialized(clazzString, W_ThreadGroup_system->manager);
  classMustBeInitialized(clazzClass, W_ThreadGroup_system->manager);
  classMustBeInitialized(clazzClassLoader, W_ThreadGroup_system->manager);
  classMustBeInitialized(clazzThrowable, W_ThreadGroup_system->manager);
  classMustBeInitialized(clazzThread, W_ThreadGroup_system->manager);
  classMustBeInitialized(clazzThreadGroup, W_ThreadGroup_system->manager);

 /*****
 * 12 *
 *****/
  woempa(7,"Step 12: create primitive classes and their array classes\n");

  /*
  ** Create the primitive array clazzes. Note that we have to initialize
  ** our table of array descriptors first. We can not do that in 
  ** initDescriptors since it would invoke createArrayClazz, when parsing 
  ** the strings of the descriptors and createArrayClazz requires clazzObject 
  ** to be properly initialized and initDescriptors is run before we initialize
  ** clazzObject; comprendo? si...
  ** See the explanation of the 'newarray' opcode in any good JVM book, if the numbers in
  ** the for() loops look like MADGIC to you...
  */

  for (i = 4; i < 12; i++) {
    atype2desc[i] = newFieldDescriptor(W_ThreadGroup_system->manager, atype2string[i]);
    atype2clazz[i] = seekClazzByDesc(atype2desc[i], system_class_hashtable);
    if (!atype2clazz[i]) {
      atype2clazz[i] = createArrayClazzFromDesc(NULL, atype2desc[i],&dummy);
    }
  }

  preparePrimitive(clazz_boolean, string_boolean, &Class_boolean, desc_boolean, P_boolean);
  preparePrimitive(clazz_char, string_char, &Class_char, desc_char, P_char);
  preparePrimitive(clazz_float, string_float, &Class_float, desc_float, P_float);
  preparePrimitive(clazz_double, string_double, &Class_double, desc_double, P_double);
  preparePrimitive(clazz_byte, string_byte, &Class_byte, desc_byte, P_byte);
  preparePrimitive(clazz_short, string_short, &Class_short, desc_short, P_short);
  preparePrimitive(clazz_int, string_int, &Class_int, desc_int, P_int);
  preparePrimitive(clazz_long, string_long, &Class_long, desc_long, P_long);
  preparePrimitive(clazz_void, string_void, &Class_void, desc_void, P_void);

  /*
  ** Now we can attach the componentClazz of every array type.
  */
  for (i = 4; i < 12; i++) {
    atype2clazz[i]->aclazz->componentClazz = primitive2clazz[i];
  }

  /*
  ** For some strange reason, SUN seems to fix the SUIDs for arrays of primitives. 
  ** So we set the cached SUID for the primitive array clazzes to the one given by 'serialver'.
  */
  
  atype2clazz[P_boolean]->suid = 6309297032502205922LL;
  atype2clazz[P_char]->suid = -5753798564021173076LL;
  atype2clazz[P_float]->suid = 836686056779680834LL;
  atype2clazz[P_double]->suid = 4514449696888150558LL;
  atype2clazz[P_byte]->suid = -5984413125824719648LL;
  atype2clazz[P_short]->suid = -1188055269542874886LL;
  atype2clazz[P_int]->suid = 5600894804908749477LL;
  atype2clazz[P_long]->suid = 8655923659555304851LL;

  /*
  ** Create some other frequently used descriptors and classes.
  ** Again, fix some SUID's of these classes...
  */
  
  desc_ArrayOf_Object = newFieldDescriptor(W_Thread_system, string_ArrayOf_Object);
  clazzArrayOf_Object = descArrayClassMustBeLoaded(NULL, desc_ArrayOf_Object, &dummy);
  clazzArrayOf_Object->suid = -8012369246846506644LL;

  desc_ArrayOf_Class = newFieldDescriptor(W_Thread_system, string_ArrayOf_Class);
  clazzArrayOf_Class = descArrayClassMustBeLoaded(NULL, desc_ArrayOf_Class, &dummy);
  clazzArrayOf_Class->suid = -6118465897992725863LL;

  desc_ArrayOf_String = newFieldDescriptor(W_Thread_system, string_ArrayOf_String);
  clazzArrayOf_String = descArrayClassMustBeLoaded(NULL, desc_ArrayOf_String, &dummy);
  clazzArrayOf_String->suid = -5921575005990323385LL;

  clazzArrayOf_Constructor = namedArrayClassMustBeLoaded(NULL, string_ArrayOf_Constructor, &dummy);
  clazzArrayOf_Field = namedArrayClassMustBeLoaded(NULL, string_ArrayOf_Field, &dummy);
  clazzArrayOf_Method = namedArrayClassMustBeLoaded(NULL, string_ArrayOf_Method, &dummy);
  /*
  ** Set up the primitive2wrapper array...
  */
  
  primitive2wrapper[P_boolean] = clazzBoolean;
  classMustBePrepared(clazzBoolean, W_ThreadGroup_system->manager);
  primitive2wrapperSlot[P_boolean] = F_Boolean_value;

  primitive2wrapper[P_char] = clazzCharacter;
  classMustBePrepared(clazzCharacter, W_ThreadGroup_system->manager);
  primitive2wrapperSlot[P_char] = F_Character_value;

  primitive2wrapper[P_float] = clazzFloat;
  classMustBePrepared(clazzFloat, W_ThreadGroup_system->manager);
  primitive2wrapperSlot[P_float] = F_Float_value;

  primitive2wrapper[P_double] = clazzDouble;
  classMustBePrepared(clazzDouble, W_ThreadGroup_system->manager);
  primitive2wrapperSlot[P_double] = F_Double_value;

  primitive2wrapper[P_byte] = clazzByte;
  classMustBePrepared(clazzByte, W_ThreadGroup_system->manager);
  primitive2wrapperSlot[P_byte] = F_Byte_value;

  primitive2wrapper[P_short] = clazzShort;
  classMustBePrepared(clazzShort, W_ThreadGroup_system->manager);
  primitive2wrapperSlot[P_short] = F_Short_value;

  primitive2wrapper[P_int] = clazzInteger;
  classMustBePrepared(clazzInteger, W_ThreadGroup_system->manager);
  primitive2wrapperSlot[P_int] = F_Integer_value;

  primitive2wrapper[P_long] = clazzLong;
  classMustBePrepared(clazzLong, W_ThreadGroup_system->manager);
  primitive2wrapperSlot[P_long] = F_Long_value;

  loadClass_method = getMethod_name_sig(clazzClassLoader, string_loadClass_name, string_loadClass_sig);
  if (loadClass_method == NULL) {
    wabort(ABORT_WONKA, "Uh oh: class %k doesn't have an instance method %w%w. Game over.\n", clazzClassLoader, string_loadClass_name, string_loadClass_sig);
  }
  woempa(7, "Forced class loading complete, loaded %d classes.\n",system_class_hashtable->occupancy);

}

/*
** If the given string begins with L, strip the leading L and trailing ';'.
** Returns a registered w_string.
*/

w_string undescriptifyClassName(w_string string) {

  w_string FQName = string;
  w_size   i;
  w_char *charbuf;


  woempa(1, "Starting with %w\n", string);
  if (string->chars[0] == 'V') {
    FQName = registerString(string_void);
  }
  else if (string->chars[0] == 'I') {
    FQName = registerString(string_int);
  }
  else if (string->chars[0] == 'J') {
    FQName = registerString(string_long);
  }
  else if (string->chars[0] == 'Z') {
    FQName = registerString(string_boolean);
  }
  else if (string->chars[0] == 'F') {
    FQName = registerString(string_float);
  }
  else if (string->chars[0] == 'D') {
    FQName = registerString(string_double);
  }
  else if (string->chars[0] == 'C') {
    FQName = registerString(string_char);
  }
  else if (string->chars[0] == 'B') {
    FQName = registerString(string_byte);
  }
  else if (string->chars[0] == 'L') {
    charbuf = allocMem_system(string->length * sizeof(w_char));
    for (i = 1; i < string->length - 1; ++i) {
      charbuf[i - 1] = (string->chars[i] == '/') ? '.' : string->chars[i];
    }
    FQName = unicode2String(currentWonkaThread, charbuf, string->length - 2);
    releaseMem(charbuf);
  }
  else {
    FQName = slashes2dots(currentWonkaThread, FQName);
  }

  return FQName;
  
}

/*
** Search for the clazz called 'classname', and return its address or NULL.
** 'classname' is in the format returnd by Class.getName().
*/

w_clazz seekClazzByName(w_string classname, w_hashtable class_hashtable) {
  return (w_clazz)ht_read_no_lock(class_hashtable, (w_word)classname);
}

/*
** Ditto, but in the case of a non-array class the 'L' and ';' are present.
** Caller must have exclusive access to class_hashtable.
*/

w_clazz seekClazzByDesc(w_desc desc, w_hashtable class_hashtable) {
  w_clazz  result = NULL;
  w_string FQNstring;

  FQNstring = undescriptifyClassName(desc->name);

  woempa(1, "Searching %s for class with descriptor `%w' -> FQNstring `%w'.\n", class_hashtable->label, NM(desc), FQNstring);
  result = (w_clazz)ht_read_no_lock(class_hashtable, (w_word)FQNstring);

  deregisterString(FQNstring);

  return result;

}

/*
** Create an array clazz starting from a descriptor for the array Class in
** the form [<type>
**
** Where <type> can be either:
**
** - a primitive type like 'J'
** - a class descriptor like 'Ljava/lang/Object;'
** - or '[<type>', where <type> can be ...
**
** The clazz corresponding to <type> must already exist.  To create an array
** clazz and all intermediate clazzes use createArrayClazzFromDesc(), which
** calls createSingleArrayClazzFromDesc() in a loop (to reduce stack consumption).
*/

w_clazz createSingleArrayClazzFromDesc(w_instance classLoader, w_desc desc, w_instance *exception) {

  w_clazz clazz;
  w_aclazz aclazz;
  w_instance Class = NULL;
  w_desc element;
  w_desc component;
  w_int dimensions = 0;
  w_hashtable class_hashtable = loader2hashtable(classLoader);

  wassert(desc->tag == D_Array);

  component = desc->body.array.component;

  /*
  ** Find the number of dimensions the element type and the component type.
  */

  element = desc;
  while (element->tag == D_Array) {
    dimensions += 1;
    element = element->body.array.component;
  }

  /*
  ** We make a clone of clazz_Array. We don't need to increment
  ** all refcounts that are being used in this clazz.
  */

  clazz = allocClazz(currentWonkaThread);

  if (clazz == NULL) {
    woempa(7, "Could not allocate w_Clazz for %w\n", desc->name);

    return NULL;

  }

  memcpy(clazz, clazz_Array, sizeof(w_Clazz));

  if (element->tag == D_Primitive) {
    clazz->loader = NULL;
  }
  else {
    clazz->loader = classLoader;
  }

  if (clazzClass && clazzClass->prepare == alreadyPrepared) {
    Class = attachClassInstance(clazz);
  }
  else {
    woempa(1, "Can't attach an instance of Class to class %k yet, 'coz class Class don't exist.\n", clazz);
  }

  aclazz = allocMem(currentWonkaThread, sizeof(w_Aclazz));

  aclazz->clazz = clazz;
  aclazz->depth = dimensions;
  aclazz->element = element;
  aclazz->component = component;

  /*
  ** We change some stuff in the clazz_Array clone, so that it becomes an array clazz.
  ** - the name changes and we increment the refCount of the string.
  ** - we assign the newly created aclazz to the clazz->aclazz.
  */

  clazz->desc = registerDescriptor(currentWonkaThread, desc);
  clazz->dotified = undescriptifyClassName(desc->name);
  clazz->aclazz = aclazz;

  if (dimensions == 1 && element->tag == D_Primitive) {
    aclazz->componentClazz = primitive2clazz[element->body.primitive.type];
  }
  else {
    w_int i;
    w_clazz componentClazz;
    w_instance componentClass;

    componentClazz = descClassMustBeLoaded(classLoader, aclazz->component, exception);
    if (!componentClazz) {
      if (!*exception) {
        *exception = createExceptionInstance(currentWonkaThread, clazzClassNotFoundException, NM(aclazz->component));
      }

      return NULL;

    }
    aclazz->componentClazz = componentClazz;
    if (clazzClass && clazzClass->prepare == alreadyPrepared) {
      componentClass = clazz2Class(componentClazz);
      for (i = 0; i < F_Class_wotsit; ++i) {
        woempa(1,"Copying field[%d] (0x%08x) from Class %k to %k\n",i,componentClass[i],componentClazz,clazz);
        Class[i] = componentClass[i];
      }
    }
  }

  aclazz->bits = sizeInBits(component);

  clazz->verify = alreadyVerified;
  clazz->prepare = alreadyPrepared;
  clazz->resolve = alreadyResolved;
  registerClazz(clazz,class_hashtable);

  clazz->adesc = createArrayDescriptor(currentWonkaThread, clazz, 1);

  woempa(1, "Array clazz %k at %p created by %p; bits per cell is %d, single dimensional array desc of this class is '%w'.\n", clazz,clazz,clazz->loader, aclazz->bits, clazz->adesc->name);

  return clazz;

}

/*
** Create an array clazz starting from a descriptor for the array Class as
** shown for createSingleArrayClazzFromDesc(), together with all intermediate
** array clazzes in the case of a multidimensional array.
*/
w_clazz createArrayClazzFromDesc(w_instance classLoader, w_desc desc, w_instance *exception) {

  w_clazz clazz;
  w_clazz previous = NULL;
  w_clazz result = NULL;
  w_desc peel;

  wassert(desc->tag == D_Array);

  woempa(1, "Creating array class(es) for descriptor %w.\n", desc->name);

  /*
  ** to avoid recursion we start up creating the array class of the highest dimension.
  ** To make sure this will work correctly we must use  the correct classloader.
  ** All array classes must be loaded by the classloader who loaded the base class ...
  */
  peel = desc;
  while (peel->tag == D_Array) {
    peel = peel->body.array.component;
  }
  if (peel->tag != D_Primitive) {
    woempa(1,"Loading non primitive array class for '%w'\n",NM(peel));
    clazz = descClassMustBeLoaded(classLoader, peel, exception);
    if(clazz){
      woempa(1,"Loading non primitive class '%w' succeeded (by %p)\n",NM(peel),clazz->loader);
      classLoader = clazz->loader;
    }
    else{
      woempa(1,"Loading non primitive class '%w' failed (%p)\n",NM(peel),exception);
      return NULL;
    }
  }
  else {
    /*
    ** primitive arrays are loaded by the bootstrap classloader
    */
    woempa(1,"Loading primitive array class '%w'\n",NM(peel));
    classLoader = NULL;
  }

  /*
  ** Create each single array dimension clazz of a lower dimension.
  ** Note that we are guaranteed of 1 loop through, so 'previous' will be
  ** set to a correct value.
  */

  peel = desc;
  while (peel->tag == D_Array) {
    woempa(1, "looking for class %w\n",NM(peel));
    clazz = seekClazzByDesc(peel, loader2hashtable(classLoader));

    if (clazz == NULL) {
      clazz = createSingleArrayClazzFromDesc(classLoader, peel,exception);
      if (clazz == NULL) {
        woempa(1,"Unable to create Class instance for %w\n",NM(peel));
        return NULL;
      }
      woempa(1, "Created unexisting array clazz %k, dimensions %d.\n", clazz, clazz->aclazz->depth);
    }
    else {
      woempa(1, "Array clazz %k, dimensions %d, existed allready.\n", clazz, clazz->aclazz->depth);
    }
    if (previous) {
      /*
      ** If we already created a lower dimension (previous != NULL)
      ** and if we did not yet have a resulting class to return, then
      ** the lower dimension is the clazz we need to return.
      */
      if (result == NULL) {
        result = previous;
      }
      /*
      ** Assign the newly created (or found) array clazz to the component
      ** clazz of the previous dimension.
      */
      previous->aclazz->componentClazz = clazz;
      woempa(1, "Element of %k is %k.\n", previous, clazz);
    }
    previous = clazz;
    peel = peel->body.array.component;
  }

  /*
  ** Note that we still have to set the component clazz of the lowest array
  ** in case the element is not a primitive. It is usefull that we have the
  ** string descriptor representation of this element's clazz from the previous
  ** loop...
  **
  ** Note also that the lowest array is 'previous'.
  */

  if (previous->aclazz->component->tag != D_Primitive) {
    previous->aclazz->componentClazz = seekClazzByDesc(peel, loader2hashtable(classLoader));
  }

  /*
  ** If we had just 1 single go through the above loop, result will still be
  ** NULL but previous will hold the correct clazz pointer to return. We are
  ** guaranteed of one single go through the loop.
  */

  if (result == NULL) {
    result = previous;
  }
  return result;

}

/*
** If the class with the given name (in "dotted" form) is not loaded,
** then if it is an array class then try to create it, else abort.
** Use descClassMustBeLoaded if the starting point is a w_desc.
*/

w_clazz namedArrayClassMustBeLoaded(w_instance classLoader, w_string name, w_instance *exception) {
  w_thread    thread = currentWonkaThread;
  w_instance  ancestor;
  w_hashtable class_hashtable;
  x_monitor   class_loading_lock;
  w_desc      desc;
  w_clazz  current;

  ancestor = classLoader ? classLoader : systemClassLoader ? systemClassLoader : NULL;
  class_hashtable = loader2hashtable(ancestor);
  class_loading_lock = &class_hashtable->monitor;

  while(1) {
    x_monitor_enter(class_loading_lock, x_eternal);
    current = seekClazzByName(name, class_hashtable);
    x_monitor_exit(class_loading_lock);
    if (current || ! ancestor) {

      break;

    }

    ancestor = NULL; // (w_instance)ancestor[F_ClassLoader_parent];
    class_hashtable = loader2hashtable(ancestor);
    class_loading_lock = &class_hashtable->monitor;
  }

  if (current == NULL) {
    desc = newFieldDescriptor(thread, name);
    if (desc == NULL) {
      woempa(9, "Failed to create descriptor for %w\n", name);
    }
    else {
      current = createArrayClazzFromDesc(classLoader, desc, exception);
    }
  }
  else {
    woempa(1, "Found clazz %w in existing list at %p.\n", name, current);
  }

  if (*exception == NULL && current == NULL) {
    woempa(1, "Didn't find clazz %w in existing list.\n", name);
    *exception = createExceptionInstance(thread, clazzClassNotFoundException, name);
  }

  return current;

}

w_clazz namedClassMustBeLoaded(w_instance classLoader, w_string name, w_instance *exception) {
  w_thread    thread = currentWonkaThread;
  w_instance  ancestor;
  w_hashtable class_hashtable;
  x_monitor   class_loading_lock;
  w_clazz current;

  woempa(1, "Class %w must be loaded.\n", name);

  if (name->chars[0] == (w_char)'[') {
    return namedArrayClassMustBeLoaded(classLoader, name, exception);
  }

  ancestor = classLoader ? classLoader : systemClassLoader ? systemClassLoader : NULL;
  class_hashtable = loader2hashtable(ancestor);
  class_loading_lock = &class_hashtable->monitor;

  while(1) {
    x_monitor_enter(class_loading_lock, x_eternal);
    current = seekClazzByName(name, class_hashtable);
    x_monitor_exit(class_loading_lock);
    if (current || ! ancestor) {

      break;

    }

    ancestor = NULL; // (w_instance)ancestor[F_ClassLoader_parent];
    class_hashtable = loader2hashtable(ancestor);
    class_loading_lock = &class_hashtable->monitor;
  }

  if (current == NULL) {
    if (classLoader == NULL) {
      current = loadBootstrapClass(name, exception);
    }
    else {
      current = loadNonBootstrapClass(classLoader, name, exception);
    }
  }
  else {
    woempa(2, "Found clazz %k in existing list at %p.\n", current, current);
  }

  if (*exception == NULL && current == NULL) {
    *exception = createExceptionInstance(thread, clazzClassNotFoundException, name);
  }

  return current;

}

w_clazz descArrayClassMustBeLoaded(w_instance classLoader, w_desc desc, w_instance *exception) {
  w_thread    thread = currentWonkaThread;
  w_instance  ancestor;
  w_hashtable class_hashtable;
  x_monitor   class_loading_lock;
  w_clazz current;

  ancestor = classLoader ? classLoader : systemClassLoader ? systemClassLoader : NULL;
  class_hashtable = loader2hashtable(ancestor);
  class_loading_lock = &class_hashtable->monitor;
    
  while(1) {
    x_monitor_enter(class_loading_lock, x_eternal);
    current = seekClazzByDesc(desc, class_hashtable);
    x_monitor_exit(class_loading_lock);
    woempa(1, "looking for clazz: %p (loader %p)\n",current,ancestor);
    if (current || ! ancestor) {

      break;

    }

    ancestor = NULL; // (w_instance)ancestor[F_ClassLoader_parent];
    class_hashtable = loader2hashtable(ancestor);
    class_loading_lock = &class_hashtable->monitor;
  }

  if (current == NULL) {
    current = createArrayClazzFromDesc(classLoader, desc, exception);
    woempa(1, "created clazz %k in existing list at %p by %p.\n", current, current, ancestor);

  }
  else {
    woempa(1, "Found clazz %k in existing list at %p.\n", current, current);
  }

  if (*exception == NULL && current == NULL) {
    *exception = createExceptionInstance(thread, clazzClassNotFoundException, NM(desc));
  }

  return current;

}

w_clazz descClassMustBeLoaded(w_instance classLoader, w_desc desc, w_instance *exception) {
  w_thread    thread = currentWonkaThread;
  w_instance  ancestor;
  w_hashtable class_hashtable;
  x_monitor   class_loading_lock;
  w_string name = NULL;
  w_clazz current;

  if (desc->tag == D_Array) {
    return descArrayClassMustBeLoaded(classLoader, desc, exception);
  }

  ancestor = classLoader ? classLoader : systemClassLoader ? systemClassLoader : NULL;
  class_hashtable = loader2hashtable(ancestor);
  class_loading_lock = &class_hashtable->monitor;

  while(1) {
    x_monitor_enter(class_loading_lock, x_eternal);
    current = seekClazzByDesc(desc, class_hashtable);
    x_monitor_exit(class_loading_lock);
    if (current || ! ancestor) {

      break;

    }
    
    ancestor = NULL; // (w_instance)ancestor[F_ClassLoader_parent];
    class_hashtable = loader2hashtable(ancestor);
    class_loading_lock =  &class_hashtable->monitor;
  }

  if (current == NULL) {
    name = undescriptifyClassName(desc->name);

    if (classLoader == NULL) {
      current = loadBootstrapClass(name, exception);
    }
    else {
      current = loadNonBootstrapClass(classLoader, name, exception);
    }

    deregisterString(name);
  }
  else {
    woempa(1, "Found clazz %k in existing list at %p.\n", current,current);
  }

  if (*exception == NULL && current == NULL) {
    *exception = createExceptionInstance(thread, clazzClassNotFoundException, name);
  }

  return current;

}

w_instance resolveClass(w_clazz clazz) {
  w_thread   thread = currentWonkaThread;
  w_size     i;
  w_method   method;
  w_instance exception = NULL;
  w_clazz   exception_clazz = NULL;

  if (classIsResolved(clazz)) {

    return NULL;

  }

  x_monitor_enter(clazz->resolution_monitor, x_eternal);

  if (classIsResolved(clazz)) {
    x_monitor_exit(clazz->resolution_monitor);

    return NULL;

  }

  exception = classMustBePrepared(clazz, thread);
  if (exception) {
    x_monitor_exit(clazz->resolution_monitor);

    return exception;

  }

  if (getClazzState(clazz) == CLAZZ_STATE_BROKEN) {
    x_monitor_exit(clazz->resolution_monitor);
    woempa(1, "Class %k is broken.\n", clazz);

    return alreadyBroken(clazz);

  }

  woempa(1,"Resolving class %k ...\n",clazz);

  for (i = 0; i < clazz->numMethods; i++) {
    method = clazz->methods[i];
    /*
    ** Precalculate a value we need in pushJavaFrame:
    ** the number of local and stack items minus the number we inherit from
    ** the parent frame.
    */
    method->growth_i = STACK_SLACK + method->stack_i + method->local_i - method->arg_i;

  }

  if (isNotSet(clazz->flags, ACC_INTERFACE)) {
    exception_clazz = registerInterfaceMethods(thread,clazz);
  }

  if (exception_clazz == NULL) {
    DEBUG_STATEMENT(dumpPools(1, clazz));
    woempa(1,"Finished resolving class %k, spreading the news\n",clazz);
  }
  else {
    woempa(9,"Error resolving class %k - %k!\n", clazz,exception_clazz);
    setClazzState(clazz, CLAZZ_STATE_BROKEN);
    clazz->resolve = alreadyBroken;
    clazz->initialize = alreadyBroken;
    exception = createExceptionInstance(thread,clazzLinkageError,exception_clazz->dotified);
  }

  clazz->resolve = alreadyResolved;
  x_monitor_exit(clazz->resolution_monitor);

  return exception;

}

w_instance alreadyResolved(w_clazz clazz) {
  woempa(1, "Class %k is already resolved.\n", clazz);

  return NULL;

}


w_instance resolveConstantPool(w_clazz clazz, w_thread thread) {
  w_instance exception = NULL;

  woempa(1, "Start resolving constant pool of %k.\n", clazz);
  woempa(1, "%s pool of %k.\n", !exception?"Resolved":"Failed to resolve", clazz);

  return exception;
}

/*
** Function to dump out one entry of an interface hashtable.
** The key is a pointer to an interface method: the value is a pointer
** to its implementation.
*/
void dumpOneInterfaceMapping(w_word key, w_word value) {
#ifdef DEBUG
  w_method itfMethod = (w_method) key;
  w_method impMethod = (w_method) value;
  woempa(1,"  Interface method %M is implemented by %M\n", itfMethod, impMethod);
#endif
}

/*
** For each class which implements an interface (or extends a class which
** implements an interface), we build a hashtable mapping interface methods
** onto methods of this class; so when an interface method is invoked, we
** just need to look up the interface method in the hashtable relating to
** the class of the objectref on the stack.
**
** Note that a single method can implement more than one interface method;
** either because two interfaces both declare methods with the same name and
** signature, or because one interface extends another.  In these cases there
** will be an entry in the hashtable for each interface method, all mapping
** to the same class method.
*/

/*
** Register all interface methods that we implement in a hashtable.
** Note that we have to loop over all interfaces that we implement,
** meaning not only those in our own interfaces table but also those
** in the interfaces table of all our superclasses.
**
** XXX We could add a quick lookup table for 'instanceof' and 'checkcast'
*/

w_int countSuperInterfaces(w_clazz clazz) {
  w_int totalInterfaces;
  w_int extraInterfaces;
  w_size i;

  totalInterfaces = clazz->numInterfaces;
  woempa(1,"counting %d superinterfaces for class %k\n",clazz->numInterfaces,clazz);
  for (i=0; i<clazz->numInterfaces; ++i) {
    extraInterfaces = countSuperInterfaces(clazz->interfaces[i]);
    woempa(1,"superinterface %k contributes %d interfaces\n",clazz->interfaces[i],extraInterfaces);
    totalInterfaces += extraInterfaces;
  }
  woempa(1,"class %k has a total of %d superinterfaces\n",clazz,totalInterfaces);

  return totalInterfaces;
}

w_int
countInterfaces(w_clazz clazz) {
  w_int totalInterfaces;
  w_int extraInterfaces;
  w_clazz super_clazz;

  woempa(1,"counting interfaces for class %k\n",clazz);
  if ((super_clazz = clazz->superClazz)) {
    woempa(1,"superclass is %k\n",super_clazz);
    totalInterfaces = countInterfaces(super_clazz);
  }
  else {
    totalInterfaces = 0;
  }
  woempa(1,"class %k inherits %d superinterfaces from its superclasses\n",clazz,totalInterfaces);
  extraInterfaces = countSuperInterfaces(clazz);
  woempa(1,"class %k superclasses contributed %d interfaces\n",clazz,extraInterfaces);
  totalInterfaces += extraInterfaces;
  woempa(1,"class %k has %d interfaces in total\n",clazz,totalInterfaces);

  return totalInterfaces;
}

w_int
collectSuperInterfaces(w_clazz clazz, w_clazz* interfaceList) {
  w_clazz interfaze;
  w_clazz *interfaceListPtr;
  w_clazz *interfaceListOrig;
  w_size i;

  interfaceListOrig= interfaceList;
  interfaceListPtr = interfaceList;
  woempa(1,"collecting %d superinterfaces for class %k\n",clazz->numInterfaces,clazz);
  for (i=0;i<clazz->numInterfaces;++i) {
    interfaze = clazz->interfaces[i];
    woempa(1,"     adding interface %k to list\n",interfaze);
    *interfaceListPtr++ = interfaze;
    interfaceListPtr += collectSuperInterfaces(interfaze,interfaceListPtr);
  }
  *interfaceListPtr = NULL;

  return interfaceListPtr - interfaceListOrig;
}

w_int
collectInterfaces(w_clazz clazz, w_clazz* interfaceList) {
  w_clazz super_clazz;
  w_clazz *interfaceListPtr;
  w_clazz *interfaceListOrig;

  interfaceListPtr = interfaceList;
  interfaceListOrig = interfaceList;
  woempa(1,"collecting interfaces for class %k\n",clazz);
  if ((super_clazz = clazz->superClazz)) {
    woempa(1,"superclass is %k\n", super_clazz);
    interfaceListPtr += collectInterfaces(super_clazz, interfaceListPtr);
  }
  interfaceListPtr += collectSuperInterfaces(clazz, interfaceListPtr);
  *interfaceListPtr = NULL;

  return interfaceListPtr - interfaceList;
}

w_clazz // Exception class or NULL 
registerInterfaceMethods(w_thread thread, w_clazz clazz) {
  w_clazz interfaze;
  w_method method;
  w_method imethod;
  w_size i;
  w_int totalInterfaces;
  w_clazz *interfaceList;
  w_clazz *interfaceListPtr;
  w_clazz  exception_clazz = NULL;

  /*
  ** Just how many interfaces do we implement?  Count our own interfaces,
  ** those of our superclass(es), and for all of the above, all their
  ** superinterfaces.
  */

  totalInterfaces = countInterfaces(clazz);

  /*
  ** If none found, nowt to do
  */

  if (totalInterfaces==0) {

    return NULL;

  }

  /*
  ** Put all of the above into a temporary array.
  */

  interfaceList = allocMem(thread, (totalInterfaces + 1) * sizeof(w_clazz));

  collectInterfaces(clazz,interfaceList);

  /*
  ** Ensure that all superinterfaces all already resolved
  */
  interfaceListPtr = interfaceList;
  while ((interfaze = *interfaceListPtr++)) {
    woempa(1,"Class %k must be resolved\n", interfaze);
    classMustBeResolved(interfaze, thread);
  }

  /*
  ** For every interface, check our class to see if we implement all of
  ** its interface methods (except the <clinit>, if any).
  ** Note : one of our methods can implement several interface methods,
  ** maybe even from unrelated interfaces.
  */
  interfaceListPtr = interfaceList;
  while ((interfaze = *interfaceListPtr++) && exception_clazz == NULL) {
    woempa(1,"scanning interface %k to check all methods are implemented\n", interfaze);
    for (i = 0; i < interfaze->numMethods; i++) {
      imethod = interfaze->methods[i];
      if (isNotSet(imethod->flags,ACC_ABSTRACT)) {
        woempa(1,"Imethod %w is not abstract, so we don't have to implement it\n",NM(imethod));
        continue;

      }
      woempa(1,"  searching class %k for a method matching %m\n", clazz,imethod);
      method = getMethod_name_sig(clazz, imethod->name, imethod->param_list);
      if (method) {
        woempa(1,"  Interface method %M is implemented by %M\n", imethod, method);
        ht2k_write(interface_hashtable, (w_word)clazz, (w_word)imethod, (w_word)method);
      }
      else {
      
        if (!(clazz->flags & ACC_ABSTRACT)) {
          woempa(9,"AbstractMethodError detected: non-interface/non-abstract class %k fails to implement %M\n",clazz, imethod);
          exception_clazz = clazzAbstractMethodError;
        }
      }
    }
  }

#ifdef BAR
  if (ifht) {
    woempa(9,">>>>>>>>>>>>> %s <<<<<<<<<<<<<\n",ifht->label);
    ht_every(ifht,dumpOneInterfaceMapping);
    woempa(9,">>>>>>>>>>> end %s <<<<<<<<<<<\n",ifht->label);
  }
#endif

  releaseMem(interfaceList);

  return exception_clazz;
}

w_clazz loadNonBootstrapClass(w_instance classLoader, w_string name, w_instance *exception) {
  w_thread thread = currentWonkaThread;
  w_instance theClass;
  w_frame  bottom;
  w_clazz  classLoader_clazz = instance2clazz(classLoader);
  w_clazz  clazz;

  classMustBeInitialized(classLoader_clazz, thread);
  woempa(1, "Trying to load class %w using %M\n", name, loadClass_method);

  bottom = allocBottomFrame(thread, loadClass_method);
  if (bottom) {
    bottom->locals[0] = (w_word)classLoader;
    bottom->locals[1] = (w_word)getStringInstance(thread, name);
    bottom->locals[2] = WONKA_FALSE;
    handleSynchronization(bottom, loadClass_method);
    thread->state = W_THREAD_READY;

    woempa(1, "Running %M\n", loadClass_method);
    *exception = runThread(thread, bottom);
    theClass = (w_instance) * (bottom->o_top-1);
    releaseBottomFrame(bottom);
  }
  else {

    return NULL;

  }

  if (*exception) {
    woempa(9,"Oh. I asked %k@%p to load %w, and it threw a %k!\n", instance2clazz(classLoader),classLoader,name, instance2clazz(*exception));

    return NULL;
  }
  else if (theClass == NULL) {
    woempa(9,"Ah. I asked %k@%p to load %w, and it returned NULL!\n", instance2clazz(classLoader),classLoader,name);
    *exception = createExceptionInstance(thread,clazzClassNotFoundException,name);

    return NULL;
  }

  clazz = Class2clazz(theClass);

  if (clazz->dotified != name) {
    woempa(9,"Scandal! I asked %k@%p to load %w, and the bounder loaded %k!\n", instance2clazz(classLoader), classLoader, name, clazz);
    *exception = createExceptionInstance(thread,clazzLinkageError,name);

    return NULL;
  }

  registerClazz(clazz, loader2hashtable(classLoader));

  return clazz;
}

w_clazz loadBootstrapClass(w_string name, w_instance *exception) {
  w_thread thread = currentWonkaThread;
  w_char  *buffer;
  w_size   length;
  w_string filename;
  w_clazz  clazz;
  w_rentry entry;
  w_BAR    bar;
  w_size   i;
  w_char   ch;

  if (systemClassLoader) {

     return loadNonBootstrapClass(systemClassLoader, name, exception);

  }

  clazz = seekClazzByName(name, system_class_hashtable);
  if (clazz) {

    return clazz;

  }

  length = name->length;
  buffer = allocMem_system((length + 6) * sizeof(w_char));
  for (i = 0; i < name->length; ++i) {
    ch = name->chars[i];
    buffer[i] = ch == '.' ? '/' : ch;
  }
  buffer[length++] = '.';
  buffer[length++] = 'c';
  buffer[length++] = 'l';
  buffer[length++] = 'a';
  buffer[length++] = 's';
  buffer[length++] = 's';
  filename = unicode2String(W_Thread_system, buffer, length);
  releaseMem(buffer);
  woempa(1, "Need a file called '%w'\n", filename);
  entry = (w_rentry)ht_read(W_Repository_system->hashtable, (w_word)filename);
  deregisterString(filename);

  if (entry) {
    bar.buffer = entry->u_data;
    bar.length = entry->u_size;
    bar.current = 0;

    // We don't pass the name for checking, because we trust the repository ...
    clazz = createClazz(currentWonkaThread, NULL, &bar, NULL, exception);

    woempa(7,"Deleting class %w (%d bytes) from the repository\n",name,entry->u_size);
    deleteRepositoryEntry(W_Repository_system, filename);
    return clazz;

  }

  if (!clazzClassNotFoundException) {
    wabort(ABORT_WONKA,"Swoggle my horn! Attempt to load system class %w failed, and clazzNoClassDefFoundError is not yet defined ...\n",name);
  }
  else if (!*exception) {
    *exception = createExceptionInstance(thread, clazzNoClassDefFoundError, name);
  }

  return NULL;
}

w_instance alreadyBroken(w_clazz clazz) {
  w_thread thread = currentWonkaThread;

  woempa(9, "Class %k is FUBAR\n", clazz);

  return createExceptionInstance(thread, clazzNoClassDefFoundError, clazz->dotified);

}

