/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: methods.c,v 1.10 2002/03/27 11:47:05 gray Exp $
*/

#include <string.h>

#include "clazz.h"
#include "interpreter.h"
#include "loading.h"
#include "methods.h"
#include "descriptor.h"
#include "realms.h"
#include "stack.h"
#include "strings.h"
#include "threads.h"

/*
** From a byte code pointer of a method, give back the source line; as a bonus,
** set the pc that has been passed as a pointer to the actual PC.
*/

w_int code2line(w_method method, w_code opcodeptr, w_size *pc) {

  w_size this_line = 0;
  w_size this_pc;
  w_int i;

  if (isSet(method->flags, ACC_NATIVE)) {
    *pc = 0;
  }
  else {
    *pc = (w_size)(opcodeptr - method->code);
    woempa(1,"Method %p has code %p: codeptr %p => pc %d\n",method,method->code,opcodeptr,*pc);
    for (i = 0; i < method->numLineNums; i++) {
      this_line = method->lineNums[i].line_number;
      this_pc = method->lineNums[i].start_pc;
      woempa(1,"  line %d is pc %d\n",this_line,this_pc);
      if (this_pc <= (*pc)) {
        if ((i + 1) < method->numLineNums) {
          w_size next_pc = method->lineNums[i + 1].start_pc;
          if (next_pc > (*pc)) {
            woempa(1,"  line %d is pc %d, so use previous\n",method->lineNums[i + 1].line_number,next_pc);
            break;
          }
        }
        else {
          woempa(1,"  line %d is last line, so use it\n",this_line);
          break;
        }
      }
    }
  }

  return this_line;
  
}

void _registerNativeMethod(w_clazz clazz, w_fun_dec fp, const char * utf8name, const char * utf8sig) {

  w_method method = NULL;
  w_size i;
  w_string name_string;
  w_string sig_string;

  name_string = utf2String(W_Thread_system, utf8name, strlen(utf8name));
  i = strlen(utf8sig);
  while (i > 0 && utf8sig[i] != ')') {
    --i;
  }
  sig_string = utf2String(W_Thread_system, utf8sig, i + 1);
  method = getMethod_name_sig(clazz, name_string, sig_string);

  if (!method) {
    wabort(ABORT_WONKA, "No method %w%w found in %k.\n", name_string, sig_string, clazz);
  }
  else {
    /*
    ** This is not really true but we know better than the compiler in this case...
    */
    method->function.word_fun = (w_word_fun)fp;
  }
}

void registerNatives(w_clazz clazz, const JNINativeMethod *methods, w_int mcount) {
  w_int i;
  for (i = 0; i < mcount; i++) {
    registerNativeMethod(clazz, methods[i].fnPtr, methods[i].name, methods[i].signature);
  }
}

char * print_method_m(char * buffer, int * remain, void * data, int w, int p, unsigned int f) {
  w_method method = data;
  w_char *chars;
  w_char *end;
  
  if (method == NULL) {
    strncpy(buffer, (char *)"<NULL>", *remain);
    if (*remain < 6) {
      *buffer += *remain;
      *remain = 0;
    }
    else {
      *buffer += 6;
      *remain -= 6;
    }
  }
  else {
    chars = method->name->chars;
    end = chars + method->name->length;
    while (*remain > 0 && chars < end) {
      *buffer++ = *chars++;
      *remain -= 1;
    }

    chars = method->param_list->chars;
    end = chars + method->param_list->length;
    while (*remain > 0 && chars < end) {
      *buffer++ = *chars++;
      *remain -= 1;
    }
  }
  
  return buffer;
}

char * print_method_M(char * buffer, int * remain, void * data, int w, int p, unsigned int f) {
  w_method method = data;
  w_char *chars;
  w_char *end;
  
  if (method == NULL) {
    strncpy(buffer, (char *)"<NULL>", *remain);
    if (*remain < 6) {
      *buffer += *remain;
      *remain = 0;
    }
    else {
      *buffer += 6;
      *remain -= 6;
    }
  }
  else {
    chars = method->clazz->dotified->chars;
    end = chars + method->clazz->dotified->length;
    while (*remain > 0 && chars < end) {
      *buffer++ = *chars++;
      *remain -= 1;
    }

    if (*remain > 0) {
      *buffer++ = '.';
      *remain -= 1;
    }
    
    chars = method->name->chars;
    end = chars + method->name->length;
    while (*remain > 0 && chars < end) {
      *buffer++ = *chars++;
      *remain -= 1;
    }

    chars = method->param_list->chars;
    end = chars + method->param_list->length;
    while (*remain > 0 && chars < end) {
      *buffer++ = *chars++;
      *remain -= 1;
    }
  }
  
  return buffer;
}

#if defined (DEBUG)

void methodTableDump(w_clazz clazz) {

  w_size i;
  w_method method;

  woempa(9,"=== Method Table Dump for %k ===\n",clazz);
  woempa(9,"  %d methods, of which %d inheritable.\n", clazz->numMethods,clazz->numInheritableMethods);
  i=0;
  woempa(9,"Inheritable methods:\n");
  while (i < clazz->numInheritableMethods) {
    method = clazz->methods[i]; 
    woempa(9,"slot %3d %w : from class %k\n",method->slot, NM(method),method->clazz);
    ++i;
  }
  woempa(9,"Non-inheritable methods:\n");
  while (i < clazz->numMethods) {
    method = clazz->methods[i]; 
    woempa(9,"slot %3d %w : from class %k\n",method->slot, NM(method),method->clazz);
//    woempa(9,"slot %3d %w\n",method->slot, NM(method));
//    woempa(9,"     from class \"%k\"\n", method->clazz);
    ++i;
  }
  woempa(9,"=== End Method Table Dump for %k ===\n",clazz);

}

#endif /* DEBUG */

