/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: realms.c,v 1.2 2002/02/27 10:16:28 buytaert Exp $
** 
** The Wonka kernel is software copyright by SmartMove NV (1999).
** Please see the file Copyright for information on it's legal use.
** 
*/

#include "core-classes.h"
#include "heap.h"
#include "locks.h"
#include "ts-mem.h"
#include "realms.h"
#include "threads.h"
#include "wonkatime.h"
#include "wonka.h"

/*
** A table which allows realms to be referred to by a 7-bit index rather
** than a 32-bit pointer.  It is protected by realm_table_mutex.  Slot[0]
** is not used (always holds NULL), because chunks allocated by OSwald 
** itself have tag 0.
*/
w_realm realm_table[MAX_REALMS];
static x_Mutex realm_table_mutex;

/*
** The system realm, from which all other realms are descended.
** Occupies slot[1] of realm_table.
*/
w_Realm systemRealm;
static x_Mutex systemRealm_mutex;
w_realm W_Realm_system;

/*
** The lowest value of x_mem_avail() at which we will still allow instances
** to be created.  (We try to ensure that there is always space to allocate
** some native data structures, e.g. to handle an OutOfMemoryError ...)
*/
w_size min_heap_free;

void initRealms() {
  systemRealm.parent = NULL;
  systemRealm.label = (char*)"realm:system";
  systemRealm.num_children = 0;
  systemRealm.memory_quota = 64000000;
  systemRealm.memory_usage = 0;
  systemRealm.thread_quota = 0x7fffffff;
  systemRealm.thread_usage = 0;
  systemRealm.cpu_quota = 0x10000/10; // 10%
  systemRealm.cpu_usage = 0;
  systemRealm.index = 1;
  realm_table[1] = &systemRealm;
  systemRealm.mutex = &systemRealm_mutex;
  x_mutex_create(systemRealm.mutex);
  W_Realm_system = &systemRealm;
  min_heap_free =  0x7fffffff;
  x_mutex_create(&realm_table_mutex);
}

/*
** Allocate a new realm, initialize it, and find a free slot for it in realm_tanle.
*/
w_realm newRealm(w_group group, w_realm parent, w_long memory, w_int threads) {
  w_int   idx;
  w_realm realm = NULL;
  w_thread debiteur = ((w_group)parent->ThreadGroup[F_ThreadGroup_wotsit])->manager;
  
  x_mutex_lock(&realm_table_mutex, x_eternal);
  for (idx = 2; idx < MAX_REALMS; ++idx) {
    if (realm_table[idx] == NULL) {
      realm = allocMem(debiteur, sizeof(w_Realm));
      woempa(7, "New realm %p has index %d\n", realm, idx);
      realm_table[idx] = realm;
      break;
    }
  }
  x_mutex_unlock(&realm_table_mutex);

  if (idx < MAX_REALMS) {
    realm->parent = parent;
    realm->label = (char*)"realm";
    realm->index = idx;
    realm->ThreadGroup = group->ThreadGroup;
    parent->num_children++;
    realm->memory_quota = memory;
    realm->memory_usage = 0;
    realm->thread_quota = threads;
    realm->thread_usage = 0;

    realm->mutex = allocMem(debiteur, sizeof(x_Mutex));
    x_mutex_create(realm->mutex);
  
  }
  else {
    woempa(9, "Realm table is full!!!\n");
  }

  return realm;
}

/*
** Release all resources used by a w_Realm.
*/
void destroyRealm(w_realm realm) {
  woempa(7, "Destroying realm %p\n", realm);
  realm->parent->num_children--;
  x_mutex_delete(realm->mutex);
  releaseMem(realm->mutex);
  x_mutex_lock(&realm_table_mutex, x_eternal);
  realm_table[realm->index] = NULL;
  releaseMem(realm);
  x_mutex_unlock(&realm_table_mutex);
}

#define MAX_MEM_WAIT_MS 10000
  
w_boolean checkRealmMemoryQuota(w_realm realm, w_int bytes) {

  if (realm->memory_usage + bytes <= realm->memory_quota) {

    return WONKA_TRUE;

  }

  return WONKA_FALSE;

}

static char realm_report_buffer[92];

const char *realm_report(w_thread t) {
  w_realm r = t->group->realm;
  (void)x_snprintf(realm_report_buffer, 92, "Realm %w maxp %d memory %d/%d threads %d/%d",
        NM(t->group), t->group->maxPriority,
        r->memory_usage,r->memory_quota,r->thread_usage,r->thread_quota);

  return realm_report_buffer;
}
 
char *realmDescription(w_realm r) {

  (void)x_snprintf(realm_report_buffer, 92, "Realm %-24w max priority %d memory quota %-10d used %-10d thread quota %-2d used %-2d",
        NM((w_group)r->ThreadGroup[F_ThreadGroup_wotsit]), ((w_group)r->ThreadGroup[F_ThreadGroup_wotsit])->maxPriority,
        r->memory_quota,r->memory_usage,r->thread_quota,r->thread_usage);

  return realm_report_buffer;

}
