/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/



/*
** $Id: stack.c,v 1.10 2002/03/26 10:44:15 gray Exp $
*/

#undef debug

#include <string.h>

#include "clazz.h"
#include "core-classes.h"
#include "descriptor.h"
#include "dispatcher.h"
#include "exception.h"
#include "hashtable.h"
#include "realms.h"
#include "stack.h"
#include "wordset.h"
#include "ts-mem.h"
#include "methods.h"
#include "strings.h"
#include "locks.h"
#include "heap.h"
#include "threads.h"

#ifdef JSPOT
  #include "hotspot.h"
#endif


w_method getCurrentMethod(w_thread thread) {
  
  if (thread->top && thread->top->method) {
    woempa(1, "Current method is %k/%w.%w.\n", thread->top->method->clazz, thread->top->method->name, thread->top->method->param_list);
    return thread->top->method;
  }

  return NULL;
    
}

w_method getCallingMethod(w_thread thread) {
  
  if (thread->top && thread->top->previous && thread->top->previous->method) {
    woempa(1, "Method %M is being called by %M.\n", thread->top->method, thread->top->previous->method);
    return thread->top->previous->method;
  }

  return NULL;
    
}

w_clazz getCurrentClazz(w_thread thread) {

  w_method current = getCurrentMethod(thread);

  if (current) {
    woempa(1, "Current clazz is %k.\n", current->clazz);
    return current->clazz;
  }
  else {
    return NULL;
  }

}

w_instance getCurrentInstance(w_thread thread) {

  if (thread->top && thread->top->method && isNotSet(thread->top->method->flags, ACC_STATIC)) {
    return (w_instance)thread->top->locals[0];
  }

  return NULL;
}

w_clazz getCallingClazz(w_thread thread) {

  w_method calling = getCallingMethod(thread);

  if (calling) {
    woempa(1, "Calling clazz is %k.\n", calling->clazz);
    return calling->clazz;
  }
  else {
    return NULL;
  }

}

w_instance getCallingInstance(w_thread thread) {
  
  if (thread->top && thread->top->previous && thread->top->previous->method && isNotSet(thread->top->previous->method->flags, ACC_STATIC)) {
    return (w_instance)thread->top->previous->locals[0];
  }

  return NULL;
    
}

void removeLocalReference(w_frame frame, w_instance instance) {

  woempa(1, "Removing reference to instance %p from frame %p.\n", instance, frame);

  if (frame->firstRef == instance) {
    ht_deregister(globals_hashtable, (w_word)instance);
    unsetFlag(instance2flags(frame->firstRef),O_GREEN);
    frame->firstRef = (w_instance)takeFirstFromWordset(&frame->localRefs);
  }
  else if (removeFromWordset(&frame->localRefs, (w_word)instance)) {
    ht_deregister(globals_hashtable, (w_word)instance);
    unsetFlag(instance2flags(instance),O_GREEN);
  }
  else {
    woempa(9, "Did not find instance %p in locals table.\n", instance);
  }

}

/*
** Remove all local references that a frame has. Note that this function should only be called when 
** we are sure that there are local references! The following is a typical run of tests we have for
** Wonka (6600 tests). It shows the number of local references cleared.
**
**  64475 calls with 0 local references
**  20633 calls with 1 local reference
**      7 calls with 2 local references
**      4 calls with 3 local references
**      4 calls with 4 local references
**      1 calls with 5 local references
**      1 calls with 6 local references
**
** Thus, we conclude:
**
** 1) This function should only be called, when there are local references. I.e., the check should happen
**    with the caller (there are only 2 for the moment, prepareReturn in dispatcher.c and releaseStack in
**    this file itself.
** 2) A single reference cache is best made in the frame structure itself to handle the remaining bulk
**    if single local references.
**
** This single reference will be set directly only in the _xxx_allocInstance functions.
** It MUST be removed manually by the normal callers of removeAllLocalReferences as indicated in 1) above.
*/

void removeAllLocalReferences(w_frame frame) {

  w_instance *slot;
  w_size i;

  woempa(1, "frame %p ->native %p ->method %p\n",frame,frame->native,frame->method);
  woempa(1, "Clearing %d local refs for %M.\n", frame->localRefs->occupancy/sizeof(w_instance), frame->native ? frame->native : frame->method);

  slot = (w_instance *) frame->localRefs->contents;
  for (i = 0; i < frame->localRefs->occupancy / sizeof(w_instance); i++) {
    woempa(1, "Slot %d contains %p.\n", i, *slot);
    if (*slot) {
      woempa(1, "--> Removing reference to %p (%k).\n", *slot, instance2clazz(*slot));
      woempa(1, "UnGREENing %k at %p (is in frame->localRefs).\n", instance2clazz(*slot), *slot);
      ht_deregister(globals_hashtable, (w_word)*slot);
      unsetFlag(instance2flags(*slot),O_GREEN);
    }
    slot += 1;
  }
  releaseWordset(&frame->localRefs);
}

/*
** Get the current calling height of frames. Frames start at 0 at the bottom of the thread.
** TODO, if this function is called a lot, we better have a height indicator in the frame itself
*/

w_int callHeight(w_frame frame) {
  return frame->depth;

/*
  w_int fdepth = 0;
  w_frame current;
  w_thread thread = frame->thread;
  
  for (current = thread->bottom; current != frame; current = current->next) {
    fdepth += 1;
  }
  
  return fdepth;
*/
}

void dumpFrame(const char *m, w_frame frame) {

  w_word * item;
  w_size i;

#define DUMP_LEVEL 9

  woempa(DUMP_LEVEL, "Executing %M\n", frame->method);
  woempa(DUMP_LEVEL, "**** %s (frame = %p)\n", m, frame);
  woempa(DUMP_LEVEL, "   Stack  base %p top %p.\n", frame->o_base, frame->o_top);

  for (item = frame->o_top - 1; item >= frame->o_base; item--) {
    woempa(DUMP_LEVEL, "--> %d 0x%x\n", *item, *item);
  }

  woempa(DUMP_LEVEL, "   Locals\n");
  item = frame->locals;
  for (i = 0; i < frame->method->local_i; i++) {
    woempa(DUMP_LEVEL, "--> [%2d] %d 0x%x.\n", i, item[i], item[i]);
  }

  if (frame->previous) {
    woempa(DUMP_LEVEL, "  calling method: %M pc = %d.\n", frame->previous->method, (int)(frame->saved_code - frame->previous->method->code));
  }
  
}

/*
** Reset the operand stack of a frame to the initial position.
*/

void clearJavaOperands(w_frame frame) {

  woempa(1, "Cleared stack for method '%w'.\n", NM(frame->method));
  frame->o_top = frame->o_base;
  
}

/*
** Discard a frame without copying any return values. Used inside exception handling code.
**
** This works like popJavaFrame except that we don't copy the return values. Note that the
** parent stack pointer is allready properly aligned by pushJavaFrame.
*/

w_frame clearJavaFrame(w_frame current, w_code *c_code) {

  w_frame parent = current->previous;
  w_thread thread = current->thread;

  if (parent == NULL) {
    return NULL;
  }

  woempa(1, "Clearing java frame %p\n",current);

  /*
  ** First, make sure the garbage collector doesn't see invalidated
  ** leaf data and clean up local references.
  */
  
  thread->top = parent;
  current->o_top = NULL;
  current->o_base = NULL;

  /*
  ** Adjust the 'remain' information in the leaf
  */

  if (current->leaf != parent->leaf) {
    /*
    ** We had a leaf break. Return the leaf and adjust nothing...
    */
    releaseMem(current->leaf);
    current->leaf = NULL;
  }
  else {
    parent->leaf->remain = parent->saved_remain;
  }

  /*
  ** Restore the code pointer
  */

  *c_code = current->saved_code;

  /*
  ** Unlock the lock that this frame possibly held
  */
  
  if (current->monitor) {
    woempa(1,"Releasing lock %p\n", current->monitor);
    x_monitor_exit(current->monitor);
    current->monitor = NULL;
  }
  
  return parent;

}

/*
** Synchronize on an instance.
*/

inline static void synchronizeOn(w_instance instance, w_frame frame) {

  frame->monitor = allocMonitor(instance);
  
  woempa(1, "Thread %w synchronizing on %k@%p to run %M, monitor = %p.\n", frame->thread->name, instance2clazz(instance), instance, frame->method, frame->monitor);

  x_monitor_enter(frame->monitor, x_eternal);
  
}

inline static void i_handleSynchronization(w_frame frame, w_method method) {

  if (isSet(method->flags, ACC_SYNCHRONIZED)) {
    if (isSet(method->flags, ACC_STATIC)) {
      woempa(1,"synchronizing on Class %k\n",method->clazz);
      synchronizeOn(clazz2Class(method->clazz), frame);
    }
    else {
      woempa(1,"synchronizing on object @ %p\n",(w_word*)frame->locals[0]);
      synchronizeOn((w_instance)frame->locals[0], frame);
    }
  }

}

// the non inlined version
void handleSynchronization(w_frame frame, w_method method) {

  if (isSet(method->flags, ACC_SYNCHRONIZED)) {
    if (isSet(method->flags, ACC_STATIC)) {
      synchronizeOn(clazz2Class(method->clazz), frame);
    }
    else {
      synchronizeOn((w_instance)frame->locals[0], frame);
    }
  }

}

w_frame allocFrame(w_frame parent) {

  w_frame frame;
  
  // Use allocClearedMem to save a dozen assignment statements ...
  frame = allocClearedMem(parent->thread, sizeof(w_Frame));
  if (!frame) {
    wabort(ABORT_WONKA, "No memory for frame!\n");
  }
  frame->previous = parent;
  frame->depth = parent->depth + 1;
  parent->next = frame;
  
  return frame;
  
}

w_leaf allocLeaf(w_thread thread, w_size size) {

  w_leaf leaf;

  leaf = allocMem(thread, sizeof(w_Leaf) + (size * sizeof(w_word)));
  if (!leaf) {
    wabort(ABORT_WONKA, "No memory for leaf!\n");
  }
  leaf->size = size;
  leaf->remain = size;

  return leaf;
  
}

w_frame allocBottomFrame(w_thread thread, w_method method) {

  w_frame frame;
  w_frame parent = thread->top;
  w_size remain_i;

  /*
  ** Note that in the if/else clause of pushing on an existing frame or creating a new frame, the
  ** following statement is in both clauses, so it seems that they can be taken out of the clauses
  ** and be put at the end. This is not the case since this statement should not be seen in isolation.
  ** In one case, we have the statements
  ** if (...) {
  **   ...
  **   thread->top = frame;
  **   ...
  ** }
  ** else {
  **   ...
  **   thread->top = frame;
  **   thread->bottom = frame;
  **   ...
  ** }
  **
  ** For the garbage collector, we really want the assignment of the top frame to be happening before
  ** the assignment to the bottom frame.
  */
  
  if (parent) {
    woempa(1, "Starting from parent frame %p.\n",parent);
    /*
    ** We have a top frame for this thread allready. Push a new frame on it as the
    ** new bottom frame, but making sure that we don't mess up the parent frame
    ** in any way. Since this method is unrelated to the parent frame, we can
    ** simplify things, with respect to the stuff that is handled in pushJavaFrame.
    */
    if (parent->next) {
      frame = parent->next;
      woempa(1,"starting with recycled frame %p\n",frame);
    }
    else if (parent->depth >= MAX_STACK_DEPTH) {
      woempa(9, "Eela, already have %d frames on stack, not allocating any more\n", parent->depth);
      throwException(thread, clazzStackOverflowError, NULL);

      return NULL;
    }
    else {
      frame = allocFrame(parent);
      woempa(1,"starting with new frame %p\n",frame);
    }
    frame->thread = thread;

    if (parent->leaf) {
      parent->saved_remain = parent->leaf->remain;

    /*
    ** Calculate the remaining space in the leaf: this is done in 3 steps;
    **
    ** 1: take the remaining space of the leaf as recorded in the leaf itself. This is
    **    the amount that was left in the leaf when the parent frame started being active.
    ** 2: substract from that the number of locals that are not passed arguments by the parent method. 
    **    The parent is not consuming any space for it; this space is hosted by the frame that called the
    **    parent. The number of 'unhosted' locals is (parent->method->local_i - parent->method->arg_i).
    ** 3: substract from that, the number of stack items that is in use by the parent. This is the
    **    parent->o_top - parent->o_base;
    */

      remain_i = parent->leaf->remain;                                /* 1 */
      remain_i -= (parent->method->local_i - parent->method->arg_i);  /* 2 */
      remain_i -= (w_size)(parent->o_top - parent->o_base);           /* 3 */
    }
    else {
      remain_i = 0;
    }

    /*
    ** Now depending on the space left/required allocate a new leaf or go further with
    ** the parent leaf. In each case, since this is a bottom frame, we don't inherit any arguments
    ** from the parent frame.
    */

    if (remain_i < method->growth_i) {
      frame->leaf = allocLeaf(thread, (w_size)((method->stack_i + method->local_i) * 128));
      frame->locals = frame->leaf->items;
    }
    else {
      parent->leaf->remain = remain_i;
      frame->leaf = parent->leaf;
      frame->locals = parent->o_top;
    }
    
    frame->o_base = frame->locals + method->local_i;
    frame->o_top = frame->o_base;
    frame->method = method;
    thread->top = frame;
    woempa(1, "Pushing on top of existing frame, o_base is %p, o_top is %p\n", frame->o_base, frame->o_top);
  }
  else {
    // Use allocClearedMem to save a dozen assignment statements ...
    frame = allocClearedMem(thread, sizeof(w_Frame));
    if (!frame) {
      wabort(ABORT_WONKA, "No memory for frame!\n");
    }
    woempa(9, "Starting stack with new frame %p.\n", frame);
    setFlag(frame->flags, FRAME_ROCK_BOTTOM);
    frame->leaf = allocLeaf(thread, (w_size)((method->stack_i + method->local_i) * 128));
    frame->method = method;
    frame->thread = thread;
    frame->locals = frame->leaf->items;
    frame->o_base = frame->locals + method->local_i;
    frame->o_top = frame->o_base;
    thread->top = frame;
    thread->bottom = frame;
  }
  setFlag(frame->flags, FRAME_BOTTOM);

  woempa(1, "method = '%w' frame = %p, %d locals @ %p\n", NM(method), frame, method->local_i, frame->locals);
  woempa(1, "  %d stack items @ %p.\n", method->stack_i, frame->o_base);

  return frame;
    
}

void releaseBottomFrame(w_frame frame) {

  w_frame parent = frame->previous;
  w_frame cursor;

  frame->thread->top = parent;
  frame->o_top = NULL;
  frame->o_base = NULL;

  /*
  ** See comments above in popJavaFrame
  */
  
  if (parent) {
    woempa(1, "Reinstating parent frame %p\n", parent);
    if (frame->leaf != parent->leaf) {
      releaseMem(frame->leaf);
      frame->leaf = NULL;
    }
    else {
      parent->leaf->remain = parent->saved_remain;
    }
  }

  /*
  ** Unlock the lock that this frame possibly held
  */
  
  if (frame->monitor) {
    woempa(1,"Releasing lock %p\n", frame->monitor);
    x_monitor_exit(frame->monitor);
    frame->monitor = NULL;
  }

  /*
  ** Go over all the frames to make
  ** them inactive (reset all flags). We don't release the frame since they
  ** can be reused by later calls.
  */

  cursor = frame;
  while (cursor) {
    cursor->flags = 0;
    cursor = cursor->next;
  }
  
  woempa(1, "Released frames higher than and including %p.\n", frame);
  
}

/*
** Release the memory used for all stack frames and leaves starting
** with frame 'bottom'.  Take care to release each leaf just once
** (a leaf can be shared by several frames).
*/

void releaseStack(w_frame bottom) {

  w_frame this_frame = bottom;
  w_frame next_frame;

  while (this_frame) {
    next_frame = this_frame->next;
    if (next_frame == NULL || next_frame->leaf != this_frame->leaf) {
      releaseMem(this_frame->leaf);
    }

    /*
    ** See if this frame has a single local reference, and if so, remove it, then check for more...
    */
    
    if (this_frame->firstRef) {
      ht_deregister(globals_hashtable, (w_word)this_frame->firstRef);
      this_frame->firstRef = NULL;
      if (this_frame->localRefs) {
        removeAllLocalReferences(this_frame);
      }
    }

    releaseMem(this_frame);
    this_frame = next_frame;
  }
}

w_trace createStackTrace(w_frame top) {

  w_thread thread = top->thread;
  w_frame cursor;
  w_trace start = NULL;
  w_trace older = NULL;
  w_trace trace;
  w_trace native;

  /*
  ** Now create trace structures, starting from the top, going down...
  */

  cursor = top;
  while (cursor) {
    /*
    ** See if this java frame is a host for an active native method call. The actual
    ** native method data will be in the rootframe.
    */
    
    if (isSet(cursor->flags, FRAME_NATIVE)) {

      /*
      ** Yep, it is, the hostframe->native_code then contains the code pointer + 1 
      ** (the interpreter increments this after reading the opcode) at the moment of 
      ** dispatch to the native method. We create an extra trace structure and we tack
      ** it *in front* of the older trace structure; we also record in the native
      ** trace structure that there is a possible skip between itself and the older
      ** trace structure. In any case, we set up everything such that when we exit this
      ** if clause, we can just do as if our nose bleeds...
      */

      woempa(1, "Tracing native method %M\n", cursor->native);
      native = allocMem(thread, sizeof(w_Trace));
      if (!native) {
        wabort(ABORT_WONKA, "No memory for native!\n");
      }
      native->previous = NULL;
      native->method = cursor->native;
      native->caller_current = cursor->native_code - 1;
      native->flags = cursor->flags;

      if (start == NULL) {
        start = native;
      }
      else {
        older->previous = native;
      }
      older = native;
    }

    if (cursor->method) {
      woempa(1, "Tracing Java method %M\n", cursor->method);
      trace = allocMem(thread, sizeof(w_Trace));
      if (!trace) {
        wabort(ABORT_WONKA, "No memory for trace!\n");
      }
      trace->previous = NULL;
      trace->method = cursor->method;
      /*
      ** Note that in the interpreter loop, the code is immediately incremented with 1,
      ** so our actual call happened 1 position back...
      */
      trace->caller_current = cursor->saved_code - 1;
      trace->flags = cursor->flags;

      if (start == NULL) {
        start = trace;
      }
      else {
        older->previous = trace;
      }
      
      older = trace;
    }

    /*
    ** Move one frame lower...
    */
    native = NULL;
    cursor = cursor->previous;
  }

  return start;
  
}

void destroyStackTrace(w_trace t) {

  w_trace current = t;
  w_trace todelete;

  while (current) {
    todelete = current;
    current = todelete->previous;
    releaseMem(todelete);
  }

}

/*
** --------------------------------------------------------------------------------------------------------------------------
**
**                                     The JAVA method call/return functions.
**
** The definitions of the Java call/return functions. They are included in the dispatcher matrix in dispatcher.c
**
** --------------------------------------------------------------------------------------------------------------------------
*/

/*
** J-Spot's profiler hooks:
*/

void profiler_invoke(w_method method) {

  #ifdef JSPOT
    hs_method_invoke(method);
  #endif

}

void profiler_return(w_method method) {

  #ifdef JSPOT
    hs_method_return(method);
  #endif

}

/*
** The common operations for each java method call.
*/

inline static w_frame pushCommon(w_frame parent, w_code * coderef, w_method method) {

  w_size remain_i;
  w_size frame_i = method->stack_i + method->local_i;
  w_frame frame;
  w_thread thread = parent->thread;

  if (parent->next) {
    frame = parent->next;
  }
  else {
    frame = allocFrame(parent);
  }

  frame->cached_context = NULL;
  frame->inherited_context = NULL;

  /*
  ** Note that we allready adjust the parents stack pointer such that we
  ** virtually pop of the passed arguments. We do it here and not in popJavaFrame since we 
  ** have the information allready available. See the comments that contain VIRTUAL POP. 
  ** Another advantage of doing it here allready at push time, is that the garbage collector
  ** is not going to visit the item twice; once as a stack argument and the other time as
  ** a local variable. 
  */

  /*
  ** Calculate if we have enough space left in the parents leaf.
  ** Note that the current status of 'remain' is old, i.e. we have to
  ** substract from it the number of items we have currently in use
  ** and add the number of argument since they are allready in the leaf.
  */

  parent->saved_remain = parent->leaf->remain;
  remain_i = parent->leaf->remain;
  remain_i -= (parent->method->local_i - parent->method->arg_i) + (w_size)(parent->o_top - parent->o_base);
  if (remain_i < method->growth_i) {

    /*
    ** We need a new leaf; we will have to introduce a leaf break between the current
    ** frame and the newly to be created frame. Allocate a new leaf that can accomodate at
    ** least 16 of these methods. Please use a binary value like 8 so that it can be
    ** optimized into a shift operation.
    */

    frame->leaf = allocLeaf(thread, (w_size)(frame_i * 128));
    frame->locals = frame->leaf->items;

    /*
    ** Now we have to copy over the arguments from the parent frame instead
    ** of inheriting them...
    */

    memcpy(frame->locals, parent->o_top - method->arg_i, method->arg_i * sizeof(w_word));

  }
  else {

    /*
    ** Enough space is available in the current leaf, adjust what is left in the leaf.
    */

    parent->leaf->remain = remain_i;
    frame->leaf = parent->leaf;
    frame->locals = parent->o_top - method->arg_i;

  }

  /*
  ** These settings remain the same for breaked leafs or inherited leafs.
  */
  
  frame->o_base = frame->locals + method->local_i;
  frame->o_top = frame->o_base;
  frame->saved_code = * coderef;
  *coderef = method->code;
  frame->method = method;
  woempa(1,"Pushed frame for %M, o_base is %p, o_top is %p\n", method, frame->o_base, frame->o_top);

  /*
  ** Now it is safe to assign the new top frame for the thread; note that
  ** the last piece of 'thread->top = frame' is an atomic move.
  */
  
  frame->thread = thread;
  thread->top = frame;

  /*
  ** VIRTUAL POP
  ** Only now we can set the new stackpointer in the parent because otherwise
  ** the garbage collector could discard the passed arguments, since they are 
  ** on the top of the parents stack and we only assigned the new topframe
  ** in previous statement.
  */
  
  if (remain_i < method->growth_i) {
    parent->o_top -= method->arg_i;
  }
  else {
    parent->o_top = frame->locals;
  }

  //woempa(1, "Called by '%w' of '%w' pc = %d.\n",  NM(frame->previous->method), NM(frame->previous->method->clazz), (int)(frame->saved_code - frame->previous->method->code));

  return frame;

}

/*
** This dispatcher is used for byte code methods that only contain a VOID RETURN as their first
** opcode. Mostly, an empty constructor. It just pops of the arguments from the stack and doesn't
** change to a new frame.
*/

static void cj_vreturn (w_frame * frameref, w_code * coderef, w_method method) {

  
  (*frameref)->o_top -= method->arg_i;

}

w_Dispatcher dispatch_vreturn = { cj_vreturn, NULL };

/*
** (JAVA - CALL) The synchronised instance method, both normal and profiled functions.
*/

void cj_ins_syn   (w_frame * frameref, w_code * coderef, w_method method) {

  woempa(1, "XXX cj_%s_%s ->  %m\n", isSet(method->flags, ACC_STATIC) ? "stc" : "ins", isSet(method->flags, ACC_SYNCHRONIZED) ? "syn" : "uns", method);
  
  *frameref = pushCommon(*frameref, coderef, method);
  
  synchronizeOn( (w_instance) (*frameref)->locals[0], *frameref);

}

void cj_ins_syn_P (w_frame * frameref, w_code * coderef, w_method method) {
  
  profiler_invoke(method);
  cj_ins_syn(frameref, coderef, method);
}

/*
** (JAVA - CALL) The NON synchronised instance method, both normal and profiled functions.
*/

void cj_ins_uns   (w_frame * frameref, w_code * coderef, w_method method) {

  woempa(1, "XXX cj_%s_%s ->  %m\n", isSet(method->flags, ACC_STATIC) ? "stc" : "ins", isSet(method->flags, ACC_SYNCHRONIZED) ? "syn" : "uns", method);

  *frameref = pushCommon(*frameref, coderef, method);

}

void cj_ins_uns_P (w_frame * frameref, w_code * coderef, w_method method) {
  
  profiler_invoke(method);
  cj_ins_uns(frameref, coderef, method);

}

/*
** (JAVA - CALL) The synchronised static method, both normal and profiled functions.
*/

void cj_stc_syn   (w_frame * frameref, w_code * coderef, w_method method) {

  *frameref = pushCommon(*frameref, coderef, method);

  synchronizeOn(clazz2Class(method->clazz), *frameref);
  
}

void cj_stc_syn_P (w_frame * frameref, w_code * coderef, w_method method) {
  
  profiler_invoke(method);
  cj_stc_syn(frameref, coderef, method);

}

/*
** (JAVA - CALL) The NON synchronised static method, both normal and profiled functions.
*/

void cj_stc_uns   (w_frame * frameref, w_code * coderef, w_method method) {

//  woempa(9, "XXX cj_%s_%s ->  %m\n", isSet(method->flags, ACC_STATIC) ? "stc" : "ins", isSet(method->flags, ACC_SYNCHRONIZED) ? "syn" : "uns", method);

  *frameref = pushCommon(*frameref, coderef, method);

}

void cj_stc_uns_P (w_frame * frameref, w_code * coderef, w_method method) {
  
  profiler_invoke(method);
  cj_stc_uns(frameref, coderef, method);

}

/*
** The common operations for each Java return function.
*/

inline static w_frame popCommon(w_frame current, w_code * c_code) {

  w_frame parent = current->previous;
  w_thread thread = current->thread;

  /*
  ** The return values have been copied back allready by the appropriate
  ** Java return function. We have to make sure the garbage collector doesn't 
  ** see invalidated leaf data.
  */

  thread->top = parent;
  current->o_top = NULL;
  current->o_base = NULL;

  /*
  ** Adjust the 'remain' information in the leaf
  */

  if (current->leaf != parent->leaf) {
    /*
    ** We had a leaf break. Return the leaf and adjust nothing...
    */
    releaseMem(current->leaf);
    current->leaf = NULL;
  }
  else {
    parent->leaf->remain = parent->saved_remain;
  }

  /*
  ** Restore the code pointer
  */

  *c_code = current->saved_code;

  return parent;
      
}

/*
** (JAVA - RETURN) The synchronised method, both normal and profiled functions.
*/

void rj_xxx_syn_V   (w_frame * frameref, w_code * coderef) {

  w_frame current = *frameref;

//  woempa(9, "XXX rj_xxx_%s_V ->  %m\n", isSet((*frameref)->method->flags, ACC_SYNCHRONIZED) ? "syn" : "uns", (*frameref)->method);
  
  *frameref = popCommon(current, coderef);
  
  woempa(1,"Releasing lock %p\n", current->monitor);
  x_monitor_exit(current->monitor);
  current->monitor = NULL;
  
}

void rj_xxx_syn_V_P (w_frame * frameref, w_code * coderef) {
 
  w_method method = (*frameref)->method;
  rj_xxx_syn_V(frameref, coderef);
  profiler_return(method);
  
}

void rj_xxx_syn_W   (w_frame * frameref, w_code * coderef) {

  w_frame current = *frameref;

//  woempa(9, "XXX rj_xxx_%s_W ->  %m\n", isSet((*frameref)->method->flags, ACC_SYNCHRONIZED) ? "syn" : "uns", (*frameref)->method);

  /*
  ** Copy one word back to the parents stack top.
  */
  
  current->previous->o_top[0] = current->o_top[-1];
  current->previous->o_top += 1;

  *frameref = popCommon(current, coderef);
  
  woempa(1,"Releasing lock %p\n", current->monitor);
  x_monitor_exit(current->monitor);
  current->monitor = NULL;

}

void rj_xxx_syn_W_P (w_frame * frameref, w_code * coderef) {

  w_method method = (*frameref)->method;
  rj_xxx_syn_W(frameref, coderef);
  profiler_return(method);

}

void rj_xxx_syn_D   (w_frame * frameref, w_code * coderef) {

  w_frame current = *frameref;

//  woempa(9, "XXX rj_xxx_%s_D ->  %m\n", isSet((*frameref)->method->flags, ACC_SYNCHRONIZED) ? "syn" : "uns", (*frameref)->method);

  /*
  ** Copy two words back to the parents stack top.
  */

  (*frameref)->previous->o_top[0] = (*frameref)->o_top[-2];
  (*frameref)->previous->o_top[1] = (*frameref)->o_top[-1];
  (*frameref)->previous->o_top += 2;


  *frameref = popCommon(current, coderef);
  
  woempa(1,"Releasing lock %p\n", current->monitor);
  x_monitor_exit(current->monitor);
  current->monitor = NULL;

}

void rj_xxx_syn_D_P (w_frame * frameref, w_code * coderef) {
  
  w_method method = (*frameref)->method;
  rj_xxx_syn_D(frameref, coderef);
  profiler_return(method);

}

/*
** (JAVA - RETURN) The NON synchronised method, both normal and profiled functions.
*/

void rj_xxx_uns_V   (w_frame * frameref, w_code * coderef) {

//  woempa(9, "XXX rj_xxx_%s_V ->  %m\n", isSet((*frameref)->method->flags, ACC_SYNCHRONIZED) ? "syn" : "uns", (*frameref)->method);

  *frameref = popCommon(*frameref, coderef);

}

void rj_xxx_uns_V_P (w_frame * frameref, w_code * coderef) {

  w_method method = (*frameref)->method;
  rj_xxx_uns_V(frameref, coderef);
  profiler_return(method);

}

void rj_xxx_uns_W   (w_frame * frameref, w_code * coderef) {

//  woempa(9, "XXX rj_xxx_%s_W ->  %m\n", isSet((*frameref)->method->flags, ACC_SYNCHRONIZED) ? "syn" : "uns", (*frameref)->method);

  /*
  ** Copy one word back to the parents stack top.
  */
  
  (*frameref)->previous->o_top[0] = (*frameref)->o_top[-1];
  (*frameref)->previous->o_top += 1;

  *frameref = popCommon(*frameref, coderef);

}

void rj_xxx_uns_W_P (w_frame * frameref, w_code * coderef) {
  
  w_method method = (*frameref)->method;
  rj_xxx_uns_W(frameref, coderef);
  profiler_return(method);

}

void rj_xxx_uns_D   (w_frame * frameref, w_code * coderef) {

//  woempa(9, "XXX rj_xxx_%s_D ->  %m\n", isSet((*frameref)->method->flags, ACC_SYNCHRONIZED) ? "syn" : "uns", (*frameref)->method);

  /*
  ** Copy two words back to the parents stack top.
  */

  (*frameref)->previous->o_top[0] = (*frameref)->o_top[-2];
  (*frameref)->previous->o_top[1] = (*frameref)->o_top[-1];
  (*frameref)->previous->o_top += 2;

  *frameref = popCommon(*frameref, coderef);

}

void rj_xxx_uns_D_P (w_frame * frameref, w_code * coderef) {

  w_method method = (*frameref)->method;
  rj_xxx_uns_D(frameref, coderef);
  profiler_return(method);
  
}

