/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: threads.c,v 1.12 2002/04/08 10:01:28 dumon Exp $
*/

#include <string.h>

#include "arrays.h"
#include "clazz.h"
#include "core-classes.h"
#include "descriptor.h"
#include "exception.h"
#include "fields.h"
#include "hashtable.h"
#include "heap.h"
#include "interpreter.h"
#include "jni.h"
#include "ts-mem.h"
#include "methods.h"
#include "locks.h"
#include "loading.h"
#include "realms.h"
#include "stack.h"
#include "strings.h"
#include "threads.h"
#include "list.h"

w_hashtable thread_hashtable;

w_method run_method;
w_method registerThread_method;
w_method deregisterThread_method;
w_method runShutdownHooks_method;

static w_int nondaemon_thread_count;

/*
** Statics and globals
*/

const char *message2char[] = {
  "MES = 'Interrupt'",
  "MES = 'Throwable pending'",
  "MES = 'Send message'",
  "MES = 'Message was consumed'",
  "MES = 'Addressee unknown'",
  "MES = 'Mail box of manager full'",
  "MES = 'Addressee mail box full'",
  "MES = 'Message hold off'",
  "MES = 'Release message to memory'",
};

w_size numThreads = 0;
w_size numGroups = 0;

const char* state2char[] = {
  "Unstarted", 
  "Ready",
  "Locking",
  "Waiting",
  "???4",
  "Suspended",
  "???6",
  "Dying",
  "Dead",
  "UNKNOWN!!"
};

const unsigned char state2liveness[] = {
  0, /* 0: Unstarted      */
  1, /* 1: Ready          */
  1, /* 2: Locking        */
  1, /* 3: Waiting        */
  0, /* 4: Not used       */
  1, /* 5: Suspended      */
  0, /* 6: Not used       */
  0, /* 7: Dying          */
  0, /* 8: Dead           */
  0, /* 9: It's dead, Jim */
};

/*
** The internal Wonka representation of the system group and thread
** and the static slots for java/lang/Thread and java/lang/ThreadGroup.
*/

w_instance I_Thread_sysInit;
w_instance I_ThreadGroup_system;

w_thread W_Thread_sysInit;
w_thread W_Thread_system;
w_group  W_ThreadGroup_system;

static x_Mutex idLock;

#define MESSAGE_QUEUE_SLOTS 4
static x_Queue system_thread_messages;
static w_word system_thread_message_space[MESSAGE_QUEUE_SLOTS];

/*
** Hashtable relating host OS thread control block to our w_Thread structure.
**    key:   host OS thread id, 
**    value: address of our w_Thread structure
*/

w_hashtable thread_hashtable;

#define THREAD_HASHTABLE_SIZE 51

/*
** Hashtable relating host OS thread control block to our w_Group structure.
**    key:   host OS thread id, 
**    value: address of our w_Thread structure
*/

w_hashtable threadgroup_hashtable;

#define THREADGROUP_HASHTABLE_SIZE 11

/*
** Allocate and clear out the necessary fields for a new thread structure.
*/

w_thread allocThread(w_thread thread) {

  w_thread new;
  
  new = allocClearedMem(thread, sizeof(w_Thread));
  
  return new;
  
}

w_thread createThread(w_thread newthread, w_thread parentthread, w_instance Thread, w_group parentgroup, w_string name, w_size stacksize) {

  w_frame frame;
  x_queue queue;

  newthread->natenv = &w_JNINativeInterface;
  newthread->label = (char *)"thread";
  newthread->name = name;
  newthread->group = parentgroup;
//SB  newthread->messages = (x_queue)allocClearedMem(W_ThreadGroup_system->manager, sizeof(x_Queue));
//SB  x_queue_create(newthread->messages, allocClearedMem_system(MESSAGE_QUEUE_SLOTS * sizeof(void *)), MESSAGE_QUEUE_SLOTS);
  queue = allocClearedMem(W_ThreadGroup_system->manager, sizeof(x_Queue) + (MESSAGE_QUEUE_SLOTS * sizeof(void *)));
  x_queue_create(queue, (w_ubyte *)queue + sizeof(x_Queue), MESSAGE_QUEUE_SLOTS);
  newthread->messages = queue;
  newthread->Thread = Thread;
  newthread->kthread = (x_thread)allocClearedMem(newthread, sizeof(x_Thread));
  newthread->kthread->xref = newthread;
  newthread->ksize = stacksize;
  newthread->kstack = allocMem(newthread, newthread->ksize);
//SB  memset(newthread->kstack,0,newthread->ksize);
  if (parentthread->jpriority > 0) {
    newthread->jpriority = parentthread->jpriority;
  }
  else {
// HACK for GM-FT only ...
    woempa(9,"WARNING! Thread `%w' has parent `%w', which is a native thread\n",NM(newthread),NM(parentthread));
    woempa(9," ^ ^ ^   Setting thread priority to 5 ...\n");
    woempa(9," | | | Fix this!\n");
    newthread->jpriority = 5;
//
  }
  newthread->kpriority = priority_j2k(newthread->jpriority,0);
  newthread->isDaemon = parentthread->isDaemon;
  newthread->state = W_THREAD_UNSTARTED;

  setReferenceField(Thread,getStringInstance(W_Thread_system,name),F_Thread_name);
  woempa(1,"looking for method %k/_run()\n",clazzThread);
  frame = allocBottomFrame(newthread, run_method);
  if (frame) {
    frame->locals[0] = (w_word)Thread;
    newthread->top = frame;
 
    numThreads += 1;
  }

  return newthread;
}

void activateThread (w_thread thread) {

  x_thread_resume(thread->kthread);
  thread->state = W_THREAD_READY;

}

void terminateThread(w_thread thread) {
  w_int dummy;
  void *dunny = &dummy;

  x_thread_join(thread->kthread, &dunny, x_eternal);
  thread->state = W_THREAD_DEAD;
  thread->kthread->xref = NULL;
  thread->kthread->report = NULL;
  ht_erase(thread_hashtable,(w_word)thread->kthread);
  x_thread_delete(thread->kthread);
  x_queue_delete(thread->messages);
  if (thread == W_Thread_sysInit) {
    woempa(9,"This is %w, so I won't releaseMem memory that wasn't allocMem'd ...\n", thread->name);
  }
  else {
    releaseMem(thread->kthread);  
    releaseMem(thread->kstack);  
    releaseMem(thread->messages);
  }
/*
** We don't release the Java stack or w_Thread structure yet, 'cos
** GC might still try to mark this thread ...
*/
}


void addThreadToGroup(w_thread thread, w_group group) {
  w_frame   frame;
  w_realm   realm;

  realm = group->realm;

  /*
  ** Do the accounting of the thread use
  */
  if (!thread->isDaemon) {
    nondaemon_thread_count += 1;
    woempa(1, "Adding non-daemon thread %w, total now %d\n", thread->name, nondaemon_thread_count);
  }
  realm->thread_usage += 1;

  frame = allocBottomFrame(currentWonkaThread, registerThread_method);
  if (frame) {
    frame->locals[0] = (w_word)group->ThreadGroup;
    frame->locals[1] = (w_word)thread->Thread;
    handleSynchronization(frame, registerThread_method);

    woempa(9, "Running %M\n", registerThread_method);
    runThread(currentWonkaThread, frame);
    releaseBottomFrame(frame);
  }
}

void removeThreadFromGroup(w_thread thread, w_group group) {
  w_frame   frame;
  w_realm   realm;
  w_field   field;
  w_instance instance;

  realm = group->realm;

  if (!thread->isDaemon) {
    woempa(1, "Removing non-daemon thread %w\n", thread->name);
    if (--nondaemon_thread_count == 0) {
      woempa(9,"All non-daemon threads have terminated ...\n");
      field = getField(clazzRuntime, cstring2String(W_Thread_system, "theRuntime", 10));
      woempa(1, "theRuntime is field[%d] of %k\n", field->slot, field->declaring_clazz);
      instance = getStaticReferenceField(clazzRuntime, field->slot);
      woempa(1, "instance is %k at %p\n", instance2clazz(instance), instance);
      frame = allocBottomFrame(currentWonkaThread, runShutdownHooks_method);
      if (frame) {
        frame->locals[0] = (w_word)instance;
        handleSynchronization(frame, runShutdownHooks_method);

        woempa(7, "Running %M\n", runShutdownHooks_method);
        runThread(currentWonkaThread, frame);
        releaseBottomFrame(frame);
      }
    }
    else {
      woempa(1, "Removed non-daemon thread %w, total now %d\n", thread->name, nondaemon_thread_count);
    }
  }
  realm->thread_usage -= 1;

  if (thread->group != group) {
    woempa(9,"Fascinating... we should remove thread %p from group %p, but %p->group is %p\n",thread,group,thread,thread->group);
  }

  frame = allocBottomFrame(currentWonkaThread, deregisterThread_method);
  if (frame) {
    frame->locals[0] = (w_word)group->ThreadGroup;
    frame->locals[1] = (w_word)thread->Thread;
    handleSynchronization(frame, deregisterThread_method);

    woempa(1, "Running %M\n", deregisterThread_method);
    runThread(currentWonkaThread, frame);
    releaseBottomFrame(frame);
  }

  thread->group = NULL;

  if (nondaemon_thread_count == 0) {
    woempa(1, "Farewell cruel world ...\n");
    exit(0);
  }
}

void requestThreadTermination(w_thread thread) {
  thread->state = W_THREAD_DYING;
  woempa(1, "thread %p '%w' te salutat.\n",thread,thread->name);
  sendMessage(MESSAGE_STOP_THREAD, thread, thread, NULL);
}
 
w_thread _currentWonkaThread(const char *f, int l) {
  x_thread kthread;
  w_thread  thread;

  kthread = x_thread_current();

  if(!kthread)
    return NULL; /* Fix for crash on O4P. */
  
  thread = (w_thread)kthread->xref;
  
  if (!thread)
    return NULL; /* MB: Thread can't have been set up yet */
  
  if (*((w_word*)thread->label)!=*((w_word*)"thread")  
     && *((w_word*)thread->label)!=*((w_word*)"group")){
    woempa(9,"currentWonkaThread called at %s:%d from non-wonka thread %p\n", f, l, kthread);
    wabort(ABORT_WONKA, "%s (%d) No Wonka thread found for kernel thread %p!\n", f, l, kthread);
  }

  return thread;
}

void deregisterThreadGroup(x_thread osthread) {
}

void createThreadGroup(w_instance ThreadGroup, w_group parent, w_string name, w_size stacksize) {
  w_group oldgroup;
  w_group new;
  w_thread debiteur;

  /*
  ** Parent is null if we are creating the system thread group. Don't use the 
  ** parent's mutex then...
  ** The group structure itself is allocated on behalf of the parent.
  */

  if (parent) {
    x_mutex_lock(parent->mutex, x_eternal);
    debiteur = parent->manager;
  }
  else {
    debiteur = W_Thread_system;
  }

  new = allocClearedMem(debiteur, sizeof(w_Group));
  new->label = (char*)"group";

  if (parent == NULL) {
    new->realm = W_Realm_system;
  } 
  else {
    new->realm = parent->realm;
    if (new->realm->ThreadGroup==NULL) {
      new->realm->ThreadGroup = ThreadGroup;
    }
  }

  
  new->manager = allocThread(debiteur);

  /*
  ** From this point, allocation is on behalf of the new group...
  */
  new->manager->label = (char*)"thread:group-manager";
  new->manager->group = new;
  debiteur = new->manager;
  new->mutex = (x_mutex)allocMem(debiteur, sizeof(x_Mutex));
  new->box = (x_queue)allocMem(debiteur, sizeof(x_Queue));
  new->boxSpace = (w_byte *)allocClearedMem(debiteur, GROUP_MANAGER_BOX_SIZE);
  new->manager->natenv = &w_JNINativeInterface;
  new->manager->kthread = (x_thread)allocClearedMem(debiteur, sizeof(x_Thread));
  new->manager->kthread->xref = new->manager;
  new->manager->ksize = stacksize;
  new->manager->kstack = allocMem(debiteur, new->manager->ksize);
  memset(new->manager->kstack,0,new->manager->ksize);
  new->manager->jpriority = GROUP_MANAGER_PRIORITY;
  new->manager->kpriority = priority_j2k(GROUP_MANAGER_PRIORITY,1);
  new->parent = parent;
  new->name = name;
  new->maxPriority = parent ? parent->maxPriority : 10;
  new->ThreadGroup = ThreadGroup;

  x_queue_create(new->box, new->boxSpace, GROUP_MANAGER_BOX_SIZE / sizeof(w_word)); // Oswald queue's use number of WORDS as size !!

  x_mutex_create(new->mutex);

  ThreadGroup[F_ThreadGroup_wotsit] = (w_word)new;
  setReferenceField(ThreadGroup,getStringInstance(W_Thread_system,name),F_ThreadGroup_name);

  if (parent) {
    x_mutex_unlock(parent->mutex);
  }

  numGroups += 1;

  /*
  ** Now start the manager thread.
  */
  
// TODO - find a substitute for this!
//  addRootReference(new->manager, ThreadGroup);

  /*
  ** We also do the accounting of thread use in the realm. Each thread group manager consumes a thread...
  */

  new->realm->thread_usage += 1;

  /*
  ** TODO, check quota limit...
  */

  woempa(1, "registering os thread %p as Java threadGroup instance %p (%k)\n", new->manager->kthread, new->ThreadGroup, instance2clazz(new->ThreadGroup));
  oldgroup = (w_group)ht_write(threadgroup_hashtable, (w_word)new->manager->kthread, (w_word)new);

  if(oldgroup) {
    wabort(ABORT_WONKA, "Caramba! that os thread was already registered as %p!\n", oldgroup);
  }
 
  x_thread_create(new->manager->kthread, 
    groupManagerEntry, new, 
    new->manager->kstack, new->manager->ksize, 
    new->manager->kpriority, TF_START); 
  new->manager->kthread->report = thread_report;
}

void sendMessage(w_int request, w_thread sender, w_thread adressee, void *content) {
  w_boolean succeeded = WONKA_FALSE;
  w_message message;
  x_status  status;
  
  while (!succeeded) {
    message = (w_message)allocMem(sender, sizeof(w_Message));
    if (message) {
      message->request = request;
      message->sender = sender;
      message->addressee = adressee;
      message->bounced = 0;
      message->content = content;
  
      status = x_queue_send(message->addressee->group->box, (void*)message, 1);
      if (status == xs_success) {
        succeeded = WONKA_TRUE;
      }
      else {
        x_thread_sleep(1);
      }
    }
  }
}

static void releaseMessage(w_message message) {

  releaseMem(message);
  
}

void groupManagerEntry(void* WonkaThreadGroup) {

  w_group group = WonkaThreadGroup;
  w_message message;
  x_status status;
  
  while (1) {
    status = x_queue_receive(group->box, (void**)&message, x_eternal);
    if (status != xs_success) {
      woempa(9, "Status was '%s'\n", status2char(status));
      continue;
    }
    woempa(1, "Received message request %d.\n", message->request);
    switch (message->request) {

      case (MESSAGE_STOP_THREAD):
        removeThreadFromGroup(message->addressee, message->addressee->group);
        terminateThread(message->addressee);
        releaseMessage(message);         
        break;

      default: 
        woempa(1, "Unknown message request %d.\n", message->request); break;
    }
  }

}

w_ubyte *initKernel(w_ubyte *start_heap) {
  x_mutex_create(&idLock);

  /*
  ** Create our internal system thread and group representation
  */

  W_ThreadGroup_system = x_alloc_static_mem(start_heap, sizeof(w_Group));
  W_ThreadGroup_system->label = (char*)"group:system";
  W_ThreadGroup_system->parent = W_ThreadGroup_system;
  W_ThreadGroup_system->maxPriority = 10;
  W_ThreadGroup_system->realm = W_Realm_system;
  setFlag(W_ThreadGroup_system->flags, WG_SYSTEM);
  // TODO make the following dependent upon the system...
  W_ThreadGroup_system->mutex = x_alloc_static_mem(start_heap, sizeof(x_Mutex));
  x_mutex_create(W_ThreadGroup_system->mutex);
  
  W_ThreadGroup_system->box = x_alloc_static_mem(start_heap, sizeof(x_Queue));
  W_ThreadGroup_system->boxSpace = x_alloc_static_mem(start_heap, SYSTEM_GROUP_MANAGER_BOX_SIZE);
  x_queue_create(W_ThreadGroup_system->box, W_ThreadGroup_system->boxSpace, SYSTEM_GROUP_MANAGER_BOX_SIZE / sizeof(w_word)); // Size is WORDS !!
  
  W_ThreadGroup_system->manager = x_alloc_static_mem(start_heap, sizeof(w_Thread));
  memset(W_ThreadGroup_system->manager, 0, sizeof(w_Thread));
  
  W_ThreadGroup_system->manager->natenv = &w_JNINativeInterface;
  W_ThreadGroup_system->manager->label = (char*)"thread:group-manager:system";
  W_ThreadGroup_system->manager->kthread = x_alloc_static_mem(start_heap, sizeof(x_Thread));
  W_ThreadGroup_system->manager->kthread->xref = W_ThreadGroup_system->manager;
  W_ThreadGroup_system->manager->ksize = SYSTEM_GROUP_MANAGER_STACK_SIZE;
  W_ThreadGroup_system->manager->kstack = x_alloc_static_mem(start_heap, SYSTEM_GROUP_MANAGER_STACK_SIZE);
  memset(W_ThreadGroup_system->manager->kstack, 0, SYSTEM_GROUP_MANAGER_STACK_SIZE);
  W_ThreadGroup_system->manager->kpriority = SYSTEM_GROUP_MANAGER_PRIORITY;
  W_ThreadGroup_system->manager->jpriority = priority_j2k(SYSTEM_GROUP_MANAGER_PRIORITY,1);
  W_ThreadGroup_system->manager->group = W_ThreadGroup_system;
  
  /*
  ** W_ThreadGroup_system->manager is also known as W_Thread_system,
  ** the debiteur of last resort ...
  */
  W_Thread_system = W_ThreadGroup_system->manager;

  x_thread_create(W_ThreadGroup_system->manager->kthread, 
    startWonka, NULL, W_ThreadGroup_system->manager->kstack, 
    W_ThreadGroup_system->manager->ksize,
    W_ThreadGroup_system->manager->kpriority, TF_START);
  W_ThreadGroup_system->manager->kthread->report = thread_report;
  PutString((char*)"Launched system thread, returning control to kernel\r\n");

  return start_heap;
}

void startInitialThreads(void* data) {

  w_instance exception = NULL;
  w_method method = NULL;
  w_instance arglist;
  w_word dims;
  w_int    i;
  w_clazz  clazz_Init;
  w_instance instance_of_Init;
  w_frame  bottom;
  w_string string_Init_class = cstring2String(W_Thread_sysInit, "com.acunia.wonka.Init", 21);

  woempa(9, "********* %d instances in use after initial loading. ************\n", instance_use);

/*
** First we gather the command line arguments (if any: in the embedded case
** there cannot be any), and use the first as the name of an initial class
** to run and the rest as arguments to be passed to that class's <init>(String[])
** method.  The name goes in arglist[0] and the arguments go in arglists[1...].
*/

  if (command_line_argument_count > 0) {
    dims = command_line_argument_count;
    woempa(1,"Allocating array of %d String[s]\n",dims);
    arglist = allocArrayInstanceWithFlags(W_Thread_sysInit, clazzArrayOf_String, 1, &dims, O_SYSTEM);

    for (i = 0; i < command_line_argument_count; i++) {
      setArrayReferenceField(arglist, getStringInstance(W_Thread_sysInit, cstring2String(W_Thread_sysInit, command_line_arguments[i], strlen(command_line_arguments[i]))), i);
      woempa(9,"args[%d] = \"%w\"\n",i,String2string((w_instance)arglist[F_Array_data + i]));
    }
  }
  else {
    dims = 0;
    woempa(1,"Not allocating an array of String[s]\n");
    arglist = NULL;
  }

/*
** Now we call the private constructor of class Init with arglist as its 
** parameter.
*/
  woempa(1, "Invoking <init>([Ljava.lang.String;) of %w ...\n", string_Init_class);

  clazz_Init = seekClazzByName(string_Init_class, system_class_hashtable);
  if (!clazz_Init) {
    wabort(ABORT_WONKA,"Uh oh: didn't find class %w.  Game over\n", string_Init_class);
  }
  deregisterString(string_Init_class);

  instance_of_Init = allocInstance(W_Thread_sysInit,clazz_Init);

  /*
  ** Find the initial class in the data, create the class and find the 
  ** method main in it. Create a bottom frame for it and assign it to
  ** the top frame of the system thread; then start the ball...
  */
  method = getMethod_name_sig(clazz_Init, cstring2String(W_Thread_system,"<init>", 6), cstring2String(W_Thread_system, "([Ljava/lang/String;)", 21));
  if (method==NULL) {
    wabort(ABORT_WONKA,"Uh oh: class %k doesn't have an <init>(String[]).  Game over.\n",clazz_Init);
  }
  woempa(1,"Class %k has <init>(String[]) @ %p\n",clazz_Init, method);
  bottom = allocBottomFrame(W_Thread_sysInit, method);
  if (bottom) {
    bottom->locals[0] = (w_word)instance_of_Init;
    bottom->locals[1] = (w_word)arglist;
    W_Thread_sysInit->state = W_THREAD_READY;
    exception = runThread(W_Thread_sysInit, bottom) ;
    releaseBottomFrame(bottom);
  }
  if (exception) {
    woempa(9,"thread %w has terminated abnormally with a %k\n",NM(W_Thread_sysInit),instance2clazz(exception));
  }
  else {
    woempa(1,"thread %w has terminated normally\n",NM(W_Thread_sysInit));
  }

  /*
  ** Having thus created the Init singleton, we invoke its run() method.
  */
  method = getMethod_name_sig(clazz_Init, cstring2String(W_Thread_system,"run", 3), cstring2String(W_Thread_system, "()", 2));
  if (method==NULL) {
    wabort(ABORT_WONKA,"Uh oh: class %k doesn't have a method 'run()V'.  Game over.\n",clazz_Init);
  }

  bottom = allocBottomFrame(W_Thread_sysInit, method);
  bottom->flags |= FRAME_PRIVILEGED;
  bottom->locals[0] = (w_word)instance_of_Init;
  handleSynchronization(bottom, method);

  exception = runThread(W_Thread_sysInit, bottom) ;
  if (exception) {
    woempa(9,"thread %w has terminated abnormally with a %k\n",NM(W_Thread_sysInit),instance2clazz(exception));
  }
  else {
    woempa(9,"thread %w has terminated normally\n",NM(W_Thread_sysInit));
  }
  requestThreadTermination(W_Thread_sysInit);
}

/*
** In the function startKernel, we switch from the booting allocation functions to the normal
** operation functions. Since we don't want to expose these functions to other files, we declare
** them only here. We do it here since there we start spawning threads at the end of this function.
*/

w_instance _xxxx_norm_allocInstance(w_thread thread, w_clazz clazz);
w_instance _xxxx_norm_allocInstanceWithFlags(w_thread thread, w_clazz clazz, w_flags flags);
w_instance _xxxx_norm_allocArrayInstance(w_thread thread, w_clazz clazz, w_size size, w_flags flags);

void startKernel() {

  w_string string_sysThread;
  w_string string_sysThreadGroup;

  nondaemon_thread_count = 0;

  thread_hashtable = ht_create(W_Realm_system, (char*)"hashtable:threads", THREAD_HASHTABLE_SIZE, NULL, NULL, 0, 0);
  woempa(1, "Created thread_hashtable at %p\n",thread_hashtable);
  threadgroup_hashtable = ht_create(W_Realm_system, (char*)"hashtable:groups", THREADGROUP_HASHTABLE_SIZE, NULL, NULL, 0, 0);
  woempa(1, "Created threadgroup_hashtable at %p\n",threadgroup_hashtable);

  classMustBeInitialized(clazzRealm,W_ThreadGroup_system->manager);
  classMustBeInitialized(clazzThread,W_ThreadGroup_system->manager);
  classMustBeInitialized(clazzRuntime,W_ThreadGroup_system->manager);
  run_method = getMethod_name_sig(clazzThread, cstring2String(W_Thread_system,"_run",4), string_no_params); 
  woempa(1,"run_method = %p\n", run_method);
  registerThread_method = getMethod_name_sig(clazzThreadGroup, cstring2String(W_Thread_system,"registerThread",14), cstring2String(W_Thread_system,"(Ljava/lang/Thread;)",20));
  deregisterThread_method = getMethod_name_sig(clazzThreadGroup, cstring2String(W_Thread_system,"deregisterThread",16), cstring2String(W_Thread_system,"(Ljava/lang/Thread;)",20));
  runShutdownHooks_method = getMethod_name_sig(clazzRuntime, cstring2String(W_Thread_system,"runShutdownHooks",16), string_no_params);
  woempa(1,"run_method is %M\n",run_method);
  woempa(1,"runShutdownHooks_method is %M\n",runShutdownHooks_method);

  I_ThreadGroup_system = allocInstanceWithFlags(W_ThreadGroup_system->manager, clazzRealm, O_SYSTEM);
  woempa(1,"created I_ThreadGroup_system at %p\n",I_ThreadGroup_system);
  W_ThreadGroup_system->ThreadGroup = I_ThreadGroup_system;
  I_ThreadGroup_system[F_ThreadGroup_wotsit] = (w_word)W_ThreadGroup_system;
  string_sysThreadGroup = cstring2String(W_ThreadGroup_system->manager, "SystemThreadGroup", 17);
  setReferenceField(I_ThreadGroup_system,getStringInstance(W_Thread_system,string_sysThreadGroup),F_ThreadGroup_name);
  W_ThreadGroup_system->name = string_sysThreadGroup;

  I_Thread_sysInit = allocInstanceWithFlags(W_ThreadGroup_system->manager, clazzThread, O_SYSTEM);
  string_sysThread = cstring2String(W_ThreadGroup_system->manager, "SystemInitThread", 16);
  setReferenceField(I_Thread_sysInit,getStringInstance(W_Thread_system,string_sysThread),F_Thread_name);

  W_Thread_sysInit = allocThread(W_ThreadGroup_system->manager);
  W_Thread_sysInit->natenv = &w_JNINativeInterface;
  W_Thread_sysInit->label = (char*)"thread:sysInit";
  W_Thread_sysInit->flags = FRAME_BOTTOM;
  W_Thread_sysInit->top = NULL;
  W_Thread_sysInit->name = string_sysThread;
  W_Thread_sysInit->state = W_THREAD_UNSTARTED;
  W_Thread_sysInit->messages = &system_thread_messages;
  memset(&system_thread_messages,0,sizeof(x_Queue));
  x_queue_create(W_Thread_sysInit->messages, system_thread_message_space,4);
  W_Thread_sysInit->group = W_ThreadGroup_system;
  W_Thread_sysInit->isDaemon = WONKA_FALSE;
  W_Thread_sysInit->bottom = NULL;
  W_Thread_sysInit->jpriority = USER_PRIORITY;
  W_Thread_sysInit->Thread = I_Thread_sysInit;

  addThreadToGroup(W_Thread_sysInit,W_ThreadGroup_system);

  numThreads += 1;

  /*
  ** TODO, check quota limit...
  */

  woempa(1, "registering os thread %p as Java threadGroup instance %p (%k)\n", W_ThreadGroup_system->manager->kthread, W_ThreadGroup_system->ThreadGroup, instance2clazz(W_ThreadGroup_system->ThreadGroup));
  ht_write(threadgroup_hashtable, (w_word)W_ThreadGroup_system->manager->kthread, (w_word)W_ThreadGroup_system);

  W_ThreadGroup_system->name = string_sysThreadGroup;
  W_Thread_sysInit->kstack = allocMem(W_ThreadGroup_system->manager, WONKA_INIT_STACK_SIZE);
  memset(W_Thread_sysInit->kstack,0,WONKA_INIT_STACK_SIZE);
  W_Thread_sysInit->ksize = WONKA_INIT_STACK_SIZE;
  W_Thread_sysInit->kthread = (x_thread)allocClearedMem(W_ThreadGroup_system->manager, sizeof(x_Thread));
  W_Thread_sysInit->kthread->xref = W_Thread_sysInit;
  W_Thread_sysInit->kpriority = priority_j2k(USER_PRIORITY,0);
  W_Realm_system->ThreadGroup = I_ThreadGroup_system;
  woempa(7,"W_Realm_system @ %p super @ %p (group %p) parent @ %p quotas memory %d, threads %d\n",W_Realm_system,W_Realm_system->ThreadGroup,(w_instance)I_ThreadGroup_system[F_ThreadGroup_wotsit],NULL,W_Realm_system->memory_quota,W_Realm_system->thread_quota);

  I_Thread_sysInit[F_Thread_wotsit] = (w_word)W_Thread_sysInit;
  setReferenceField(I_Thread_sysInit, I_ThreadGroup_system, F_Thread_parent);
  woempa(1, "'%w' instance at %p.\n", NM(W_Thread_sysInit), I_Thread_sysInit);
  woempa(1, "W_Thread_sysInit at %p.\n", W_Thread_sysInit);
  woempa(1, "registering os thread %p as thread instance %p\n", W_Thread_sysInit->kthread, W_Thread_sysInit);
  ht_write(thread_hashtable, (w_word)W_Thread_sysInit->kthread, (w_word)W_Thread_sysInit);

  /*
  ** Switch from the boot allocation functions to the normal operation functions.
  */
  
  _xxxx_allocInstance = _xxxx_norm_allocInstance;
  _xxxx_allocInstanceWithFlags = _xxxx_norm_allocInstanceWithFlags;
  _xxxx_allocArrayInstance = _xxxx_norm_allocArrayInstance;

  x_thread_create(W_Thread_sysInit->kthread, 
    startInitialThreads, 0, W_Thread_sysInit->kstack, W_Thread_sysInit->ksize, 
    W_Thread_sysInit->kpriority, TF_START);
  W_Thread_sysInit->kthread->xref = W_Thread_sysInit;
  W_Thread_sysInit->kthread->report = thread_report;
}

static   char thread_report_buffer[92];

const char *group_report(w_thread t);
const char *realm_report(w_thread t);

const char *thread_report(x_thread x) {
  w_thread t = x->xref;
  w_group  g = t->group;
  w_method method = NULL;
  w_size   dummy;
  w_int    length;
  w_string thread_name;
  char    *state;

  if (g->manager == t) {

    return group_report(t);

  }

  thread_name = t->name ? t->name : string_NULL;
  state = (char*)(t->state < 10 ? state2char[t->state] : state2char[10]);

  if (!t->top || !((method = t->top->native) || (method = t->top->method))) {
    x_snprintf(thread_report_buffer, 92, "Thread%w p%-2d %s (no stack frame available)", 
      thread_name, t->jpriority, state);
  }
  else {
    if (t->top->native) {
      length = x_snprintf(thread_report_buffer, 92, "Thread %w p%-2d %s in native %M",
        thread_name, t->jpriority, state, method);
    }
    else {
      if (g) { // CHRIS WORKAROUND...
      (void)x_snprintf(thread_report_buffer, 92, "Thread %w p%-2d %s line %d [%05d] in %M",
        thread_name, t->jpriority, state, code2line(method, method->code + t->top->pc, &dummy), t->top->pc, method);
      }
      else {
      (void)x_snprintf(thread_report_buffer, 92, "Thread %w p%-2d %s line %d [%05d] in %M",
        thread_name, t->jpriority, state, code2line(method, method->code + t->top->pc, &dummy), t->top->pc, method);
      } // END WORKAROUND
    }
  }

  return thread_report_buffer;

}

char *threadDescription(w_thread t) {
  w_int    state = t->state;
  w_method method = NULL;
  w_int    length;
  w_string thread_name;

  if (state > 9) {
    state = 10; // 10 is unknown state
  }

  thread_name = t->name ? t->name : string_NULL;

  if (!t->top || !((method = t->top->native) || (method = t->top->method)) || !method->name) {
    x_snprintf(thread_report_buffer, 92, "%s %24w %-13s (no stack frame available)", 
      "Thread", thread_name, state2char[state]);
  }
  else {
      length = x_snprintf(thread_report_buffer, 92, "%s %24w p%-2d %-13s in native %M",
        "Thread", thread_name, t->jpriority, state2char[state], method);
  }

  return thread_report_buffer;

}

static char group_report_buffer[64];

const char *group_report(w_thread t) {
  w_group g = t->group;

  if (g->ThreadGroup && g->realm->ThreadGroup==g->ThreadGroup) {
  // Actually this is a Realm
    return realmDescription (g->realm);

  }
  if (g->name) {
    (void)x_snprintf(group_report_buffer, 64, "Group %-24w max priority %d", NM(g), g->maxPriority);
  }
  else {
    (void)x_snprintf(group_report_buffer, 64, "Group (unnamed)           max priority %d", NM(g), g->maxPriority);
  }
  return group_report_buffer;
}

char *groupDescription(w_group g) {

  if (g->ThreadGroup && g->realm->ThreadGroup==g->ThreadGroup) {
  // Actually this is a Realm
    return realmDescription (g->realm);

  }
  (void)x_snprintf(group_report_buffer, 64, "Group %-24w max priority %d", NM(g), g->maxPriority);

  return group_report_buffer;

}

