/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: wonka.c,v 1.13 2002/03/26 10:44:15 gray Exp $
*/

#include <string.h>
#include <stdio.h>

/*
** For the woempa stuff on Linux or Sparc
*/

#include "clazz.h"
#include "loading.h"
#include "methods.h"
#include "descriptor.h"
#include "ts-mem.h"
#include "realms.h"
#include "stack.h"
#include "sha.h"
#include "strings.h"

#ifdef FICL
#include "ficl.h"
#include "sysdep.h"
#endif

#include "vfs.h"
#include "vfs_fcntl.h"

w_instance                    SystemClassLoader;

#ifdef LINUX
#define UART0 0
#define BAUD_115200 0
void InitCom0(int p, int s) {
  PutString ((char*)"BANG BANG-A-BANG\r\n");
}

#endif

w_word *crc_table;

// --------------------------------------------


w_int sort_interfaces(const void * v_i1, const void * v_i2) {

  w_clazz i1 = *(w_clazz *)v_i1;
  w_clazz i2 = *(w_clazz *)v_i2;
 
  return compareStrings(i1->dotified, i2->dotified);
  
}

w_int sort_methods(const void * v_m1, const void * v_m2) {

  w_method m1 = *(w_method *)v_m1;
  w_method m2 = *(w_method *)v_m2;
  w_string s1;
  w_string s2;
  w_char *b1;
  w_char *b2;
  w_int result;
  w_instance exception;
  
  m1->return_type = mustBeLoaded(m1->return_type, &exception);
  m2->return_type = mustBeLoaded(m2->return_type, &exception);

  b1 = allocMem_system((m1->name->length + m1->param_list->length + m1->return_type->desc->name->length) * sizeof(w_char));
  stringAdd(b1, m1->name);
  stringAdd(b1 + m1->name->length, m1->param_list);
  stringAdd(b1 + m1->name->length + m1->param_list->length, m1->return_type->desc->name);
  s1 = unicode2String(W_Thread_system, b1, m1->name->length + m1->param_list->length + m1->return_type->desc->name->length);
  releaseMem(b1);

  b2 = allocMem_system((m2->name->length + m2->param_list->length + m2->return_type->desc->name->length) * sizeof(w_char));
  stringAdd(b2, m2->name);
  stringAdd(b2 + m2->name->length, m2->param_list);
  stringAdd(b2 + m2->name->length + m2->param_list->length, m2->return_type->desc->name);
  s2 = unicode2String(W_Thread_system, b2, m2->name->length + m2->param_list->length + m2->return_type->desc->name->length);
  releaseMem(b2);

  woempa(1, "SERIALISE: %w <=> %w\n", s1, s2);
  result = compareStrings(s1, s2);
  
  deregisterString(s1);
  deregisterString(s2);

  return result;
    
}

w_int sort_fields(const void * v_f1, const void * v_f2) {

  w_field f1 = *(w_field *)v_f1;
  w_field f2 = *(w_field *)v_f2;

  return compareStrings(f1->name, f2->name);

}

typedef union  {
  w_word W;
  w_byte B[4];
} word2bytes;

static w_int addModifiers(w_byte *buffer, w_word modifiers) {

  word2bytes u;
  
  u.W = (modifiers & ACC_FLAGS);
#ifdef LITTLE_ENDIAN
  buffer[0] = u.B[3];
  buffer[1] = u.B[2];
  buffer[2] = u.B[1];
  buffer[3] = u.B[0];
#else
  buffer[0] = u.B[0];
  buffer[1] = u.B[1];
  buffer[2] = u.B[2];
  buffer[3] = u.B[3];
#endif  

  return 4;

}    

w_word bytes2word(w_byte *bytes) {

  word2bytes u;
  
#ifdef LITTLE_ENDIAN
  u.B[3] = bytes[0];
  u.B[2] = bytes[1];
  u.B[1] = bytes[2];
  u.B[0] = bytes[3];
#else
  u.B[0] = bytes[0];
  u.B[1] = bytes[1];
  u.B[2] = bytes[2];
  u.B[3] = bytes[3];
#endif  

  return u.W;

}    

#include <ctype.h>
                                        
w_long Wonka_suid(JNIEnv *env, w_instance Wonka, w_instance Class) {
  w_instance exception = NULL;
  w_clazz clazz = Class2clazz(Class);
  w_long suid = 0;
  w_clazz *interfaces = NULL;
  w_size numInterfaces = 0;
  w_field *fields = NULL;
  w_size numFields = 0;
  w_method *constructors = NULL;
  w_size numConstructors = 0;
  w_method *methods = NULL;
  w_size numMethods = 0;
  w_size i;
  w_size chars;
  w_byte *buffer;
  w_byte *utf8;
  w_size length;
  w_sha sha;
  w_string tmp;
  w_char *sig;
  w_flags flags;

  woempa(7, "SERIALISE: Calculating suid of clazz %k\n", clazz);

  /*
  ** See if we have a chached version of the SUID in our sleeves...
  */

  if (clazz->suid != 0) {
    woempa(1, "SERIALISE: Returning cached suid 0x%x%x\n", MSW_PART(clazz->suid), LSW_PART(clazz->suid));
    return clazz->suid;
  }


  if (clazz->aclazz == NULL) {
    /*
    ** First create the arrays so that they can be sorted, some arrays are overallocated
    ** but they are released soon enough so that is not a problem.
    */

  numInterfaces = clazz->numInterfaces;
  if (numInterfaces) {
    interfaces = allocMem_system(clazz->numInterfaces * sizeof(w_clazz));
    memcpy(interfaces, clazz->interfaces, numInterfaces * sizeof(w_clazz));
    qsort(interfaces, numInterfaces, sizeof(w_clazz), sort_interfaces);
  }

  if (clazz->numFields) {
    fields = allocMem_system(clazz->numFields * sizeof(w_field));
    for (i = 0; i < clazz->numFields; i++) {
      if (clazz->fields[i].declaring_clazz == clazz) {
        flags = clazz->fields[i].flags;
        if (! (isSet(flags, ACC_PRIVATE) && isSet(flags, ACC_STATIC)) && ! (isSet(flags, ACC_PRIVATE) && isSet(flags, ACC_TRANSIENT)) ) {
          fields[numFields++] = &clazz->fields[i];
        }
      }
    }
    qsort(fields, numFields, sizeof(w_field), sort_fields);
  }

  /*
  ** We can not directly rule out that constructors or methods do not apply to this algorithm. So
  ** we have to allocate memory, but still can end up with numConstructors and numMethods being 0...
  */

  constructors = allocMem_system(clazz->numMethods * sizeof(w_method));
  for (i = 0; i < clazz->numMethods; i++) {
    if (clazz->methods[i]->clazz == clazz && (clazz->methods[i]->name == string_angle_brackets_init) && isNotSet(clazz->methods[i]->flags, ACC_PRIVATE)) {
      constructors[numConstructors++] = clazz->methods[i];
    }
  }
  if (numConstructors) {
    qsort(constructors, numConstructors, sizeof(w_method), sort_methods);
  }

  methods = allocMem_system(clazz->numMethods * sizeof(w_method));
  for (i = 0; i < clazz->numMethods; i++) {
    if (clazz->methods[i]->clazz == clazz && (clazz->methods[i]->name != string_angle_brackets_init) && (clazz->methods[i]->name != string_angle_brackets_clinit) && isNotSet(clazz->methods[i]->flags, ACC_PRIVATE)) {
      methods[numMethods++] = clazz->methods[i];
    }
  }
  if (numMethods) {
    qsort(methods, numMethods, sizeof(w_method), sort_methods);
  }

  /*
  ** Count the number of chars we need to convert back to UTF8, we need this count so we can allocate a suitably size
  ** buffer to convert the strings to UTF8, add the 32 bit modifier integers and calculate the SHA signature from.
  ** Note that we add 2 to each length to accomodate the 2 bytes for the length in the UTF8 byte sequence, and
  ** we add sizeof(w_word) for accomodating the modifiers.
  */
  
  chars = clazz->dotified->length + 2;
  chars += sizeof(w_word);
  
  for (i = 0; i < numInterfaces; i++) {
    chars += interfaces[i]->dotified->length + 2;
    woempa(1, "  INTERFACE %2d: %w\n", i, interfaces[i]->dotified);
  }

  if (clazz->clinit) {
    clazz->clinit->return_type = mustBeLoaded(clazz->clinit->return_type, &exception);
    chars += clazz->clinit->name->length;
    chars += clazz->clinit->param_list->length;
    chars += clazz->clinit->return_type->desc->name->length;
    chars += sizeof(w_word) + 2 + 2;
  }
  
  for (i = 0; i < numFields; i++) {
    chars += fields[i]->name->length;
    chars += fields[i]->desc->name->length;
    chars += sizeof(w_word) + 2 + 2;
    woempa(1, "      FIELD %2d: %w\n", i, fields[i]->name);
  }

  for (i = 0; i < numConstructors; i++) {
    constructors[i]->return_type = mustBeLoaded(constructors[i]->return_type, &exception);
    chars += constructors[i]->name->length;
    chars += constructors[i]->param_list->length;
    chars += constructors[i]->return_type->desc->name->length;
    chars += sizeof(w_word) + 2 + 2;
    woempa(1, "CONSTRUCTOR %2d: %m\n", i, constructors[i]);
  }

  for (i = 0; i < numMethods; i++) {
    methods[i]->return_type = mustBeLoaded(methods[i]->return_type, &exception);
    chars += methods[i]->name->length;
    chars += methods[i]->param_list->length;
    chars += methods[i]->return_type->desc->name->length;
    chars += sizeof(w_word) + 2 + 2;
    woempa(1, "     METHOD %2d: %m\n", i, methods[i]);
  }

  /*
  ** Now allocate a working buffer and convert strings back to UTF8, add the modifiers at the same time, for
  ** each w_char we allocate 3 bytes for the worst case UTF8 encoding.
  */
  
  buffer = allocMem_system(chars * (3 * sizeof(w_byte)));
  chars = 0;

  utf8 = string2UTF8(JNIEnv2w_thread(env), clazz->dotified, &length);
  woempa(1, "SERIALISE: utf8 = %s\n", utf8+2);
  memcpy(buffer, utf8, length);
  releaseMem(utf8);
  chars += length;

  /*
  ** It seems that this ACC_SUPER flag, which is always set by the compiler, needs to be unset
  ** before calculating the SUID. Try to find this when it's not in the SUN specs for serialization...
  */
  
  flags = clazz->flags;
  unsetFlag(flags, ACC_SUPER);
  chars += addModifiers(buffer + chars, flags);

  for (i = 0; i < numInterfaces; i++) {
    utf8 = string2UTF8(JNIEnv2w_thread(env), interfaces[i]->dotified, &length);
    woempa(1, "SERIALISE: utf8 = %s\n", utf8+2);
    memcpy(buffer + chars, utf8, length);
    releaseMem(utf8);
    chars += length;
  }

  for (i = 0; i < numFields; i++) {
    utf8 = string2UTF8(JNIEnv2w_thread(env), fields[i]->name, &length);
    woempa(1, "SERIALISE: utf8 = %s\n", utf8+2);
    memcpy(buffer + chars, utf8, length);
    releaseMem(utf8);
    chars += length;
    chars += addModifiers(buffer + chars, fields[i]->flags);
    woempa(1, "SERIALISE: buffer length = %d\n", chars);
    utf8 = string2UTF8(JNIEnv2w_thread(env), fields[i]->desc->name, &length);
    woempa(1, "SERIALISE: utf8 = %s\n", utf8+2);
    memcpy(buffer + chars, utf8, length);
    releaseMem(utf8);
    chars += length;
  }
  
  if (clazz->clinit) {
    utf8 = string2UTF8(JNIEnv2w_thread(env), clazz->clinit->name, &length);
    woempa(1, "SERIALISE: utf8 = %s\n", utf8+2);
    memcpy(buffer + chars, utf8, length);
    releaseMem(utf8);
    chars += length;
    chars += addModifiers(buffer + chars, ACC_STATIC);
    woempa(1, "SERIALISE: buffer length = %d\n", chars);
    memcpy(buffer + chars, "()V", 3);
    chars += 3;
  }
  
  for (i = 0; i < numConstructors; i++) {
    utf8 = string2UTF8(JNIEnv2w_thread(env), constructors[i]->name, &length);
    woempa(1, "SERIALISE: utf8 = %s\n", utf8+2);
    memcpy(buffer + chars, utf8, length);
    releaseMem(utf8);
    chars += length;
    chars += addModifiers(buffer + chars, constructors[i]->flags);
    sig = allocMem(JNIEnv2w_thread(env), (constructors[i]->param_list->length + constructors[i]->return_type->desc->name->length) * sizeof(w_char));
    memcpy(sig, constructors[i]->param_list->chars, constructors[i]->param_list->length * sizeof(w_char));
    memcpy(sig + constructors[i]->param_list->length, constructors[i]->return_type->desc->name->chars, constructors[i]->return_type->desc->name->length * sizeof(w_char));
    tmp = unicode2String(JNIEnv2w_thread(env), sig, constructors[i]->param_list->length + constructors[i]->return_type->desc->name->length);
    releaseMem(sig);
    tmp = slashes2dots(JNIEnv2w_thread(env), tmp);
    utf8 = string2UTF8(JNIEnv2w_thread(env), tmp, &length);
    woempa(1, "SERIALISE: utf8 = %s\n", utf8+2);
    deregisterString(tmp);
    memcpy(buffer + chars, utf8, length);
    releaseMem(utf8);
    chars += length;
  }
  
  for (i = 0; i < numMethods; i++) {
    utf8 = string2UTF8(JNIEnv2w_thread(env), methods[i]->name, &length);
    woempa(1, "SERIALISE: utf8 = %s\n", utf8+2);
    memcpy(buffer + chars, utf8, length);
    releaseMem(utf8);
    chars += length;
    chars += addModifiers(buffer + chars, methods[i]->flags);
    sig = allocMem(JNIEnv2w_thread(env), (methods[i]->param_list->length + methods[i]->return_type->desc->name->length) * sizeof(w_char));
    memcpy(sig, methods[i]->param_list->chars, methods[i]->param_list->length * sizeof(w_char));
    memcpy(sig + methods[i]->param_list->length, methods[i]->return_type->desc->name->chars, methods[i]->return_type->desc->name->length * sizeof(w_char));
    tmp = unicode2String(JNIEnv2w_thread(env), sig, methods[i]->param_list->length + methods[i]->return_type->desc->name->length);
    releaseMem(sig);
    tmp = slashes2dots(JNIEnv2w_thread(env), tmp);
    utf8 = string2UTF8(JNIEnv2w_thread(env), tmp, &length);
    woempa(1, "SERIALISE: utf8 = %s\n", utf8+2);
    deregisterString(tmp);
    memcpy(buffer + chars, utf8, length);
    releaseMem(utf8);
    chars += length;
  }
  }
  else {

    /*
    ** It is an array clazz, just take name and modifiers and process along...
    ** Allocate working buffer and proceed...
    */

    buffer = allocMem_system(2 + clazz->desc->name->length * (3 * sizeof(w_byte)));
    chars = 0;
    tmp = slashes2dots(JNIEnv2w_thread(env), clazz->desc->name);
    utf8 = string2UTF8(JNIEnv2w_thread(env), tmp, &length);
    woempa(1, "SERIALISE: utf8 = %s\n", utf8+2);
    deregisterString(tmp);
    memcpy(buffer + chars, utf8, length);
    releaseMem(utf8);
    chars += length;
    flags = clazz->flags;
    unsetFlag(flags, ACC_SUPER);
    chars += addModifiers(buffer + chars, flags);

  }

    woempa(1, "SERIALISE: buffer length = %d\n", chars);
  sha = allocSha(JNIEnv2w_thread(env));
  processSha(sha, buffer, chars);
  finishSha(sha);
  
//  suid = WORDS2LONG(sha->signature.W[1], sha->signature.W[0]);
// TODO this is a hack, solve this properly...
  suid = sha->signature.B[4];
  suid <<= 8;  suid |= sha->signature.B[5];
  suid <<= 8;  suid |= sha->signature.B[6];
  suid <<= 8;  suid |= sha->signature.B[7];
  suid <<= 8;  suid |= sha->signature.B[0];
  suid <<= 8;  suid |= sha->signature.B[1];
  suid <<= 8;  suid |= sha->signature.B[2];
  suid <<= 8;  suid |= sha->signature.B[3];

  woempa(7,"SERIALISE: suid = %02x %02x %02x %02x %02x %02x %02x %02x\n",
    sha->signature.B[7], sha->signature.B[6], sha->signature.B[5], sha->signature.B[4],
    sha->signature.B[3], sha->signature.B[2], sha->signature.B[1], sha->signature.B[0]);

#ifdef DEBUGGING_SUID
#ifdef LINUX
  {
  static int counter = 0;
  char name[100];
  FILE* f;
  
  memset(name, 0x00, 100);
  sprintf(name, "suid-%d", counter++);
  f = fopen(name, "w");
  fwrite(buffer, 1, chars, f);
  fclose(f);
/*
  memset(name, 0, 100);
  sprintf(name, "(%02x %02x %02x %02x) (%02x %02x %02x %02x)",
    sha->signature.B[7], sha->signature.B[6], sha->signature.B[5], sha->signature.B[4],
    sha->signature.B[3], sha->signature.B[2], sha->signature.B[1], sha->signature.B[0]);
 woempa(1, "LADIES, the suid bytes are '%s'\n", name);
  sprintf(name, "%lld (0x%016llx)", suid, suid);
  woempa(1, "and the suid is '%s'\n", name);
*/  
}
#endif // LINUX
#endif // DEBUGGING_SUID

  if (interfaces) {
    releaseMem(interfaces);
  }

  if (fields) {
    releaseMem(fields);
  }

  if (constructors) {
    releaseMem(constructors);
  }

  if (methods) {
    releaseMem(methods);
  }

  releaseMem(buffer);

  releaseSha(sha);

  /*
  ** Cache the SUID in the clazz structure for the next time around...
  */
  
  clazz->suid = suid;
  
  return suid;

}

/*
** Calculate the CCITT CRC16 checksum. The code is taken from Appendix C, of the document,
** prENV 278/4/7/011-2, B/TPEG "TPEG Specification - Part 2, Syntax, Semantics and Framing Structure
** Version 1.2 (November 1999)."
**
** This document can be found on the EBU web site, http://www.ebu.ch
**
** The polynomial used is: x^16 + x^12 + x^5 + 1
*/

#define CCITT_swap(a)           (((a) << 8) | ((a) >> 8))

w_ushort CCITT_16(w_ubyte *data, w_size length) {

  w_ushort crc = 0xffff;
  w_size i;
  
  for (i = 0; i < length; i++) {
    crc = (w_ushort)(CCITT_swap(crc) ^ (w_ushort)(*data++));
    crc ^= ((w_ubyte)(crc) >> 4);
    crc = (w_ushort)(crc ^ (CCITT_swap((w_ubyte)(crc)) << 4) ^ ((w_ubyte)(crc) << 5));
  }
  
  return crc ^ 0xffff;

}

/*
** The CRC32 code (taken from RFC2083: PNG Specification)
*/

/* 
** Make the table for a fast CRC. 
*/
w_void make_ISO3309_CRC_table(w_void) {
  w_word c;
  w_int n, k;
  crc_table = allocMem_system(256 * sizeof(w_word));
  for (n = 0; n < 256; n++) {
    c = n;
    for (k = 0; k < 8; k++) {
      if (c & 1) {
        c = 0xedb88320L ^ (c >> 1);
      }
      else {
        c = c >> 1;
      }
      crc_table[n] = c;
    }
  }
}

/*
** Release the table
*/
w_void release_ISO3309_CRC_table(w_void) {
  woempa(7, "Releasing ISO3309 CRC table at %p\n", crc_table);
  releaseMem(crc_table);
}

/* 
** Update a running CRC with the bytes buf[0..len-1]--the CRC
** should be initialized to all 1's, and the transmitted value
** is the 1's complement of the final running CRC (see the
** crc() routine below)).
*/
w_word update_ISO3309_CRC(w_word crc, w_ubyte *buf, w_size len) {
  w_word c = crc;
  w_size n;

  for (n = 0; n < len; n++) {
    c = crc_table[(c ^ buf[n]) & 0xff] ^ (c >> 8);
  }
  return c;
}

/* 
** Return the CRC of the bytes buf[0..len-1]. 
*/
w_word CCITT_32(w_ubyte *buf, w_size len) {
  return update_ISO3309_CRC(0xffffffffL, buf, len) ^ 0xffffffffL;
}

int strcmp(const char *p1, const char *p2) {

  unsigned char c1;
  unsigned char c2;
  unsigned char *s1 = (unsigned char *) p1;
  unsigned char *s2 = (unsigned char *) p2;
         
  do {
    c1 = (unsigned char) *s1++;
    c2 = (unsigned char) *s2++;
    if (c1 == '\0') {
      return c1 - c2;
    }
  } while (c1 == c2);
  
  return c1 - c2;

}  
                                       
