/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

package com.acunia.wonka.rudolph.taskbar;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.lang.reflect.*;

import com.acunia.wonka.rudolph.taskbar.applets.*;

public class TaskBar extends Frame implements FocusListener {

  public static final int BOTTOM = 0;
  public static final int TOP = 1;
  public static final int LEFT = 2;
  public static final int RIGHT = 3;

  public static final int HORIZONTAL = 0;
  public static final int VERTICAL = 1;
  
  private static boolean enabled = false;
  private static int location = TOP;
  private static int size = 0;
  private static Properties properties;

  private static TaskBar taskbar = null;

  private int tb_width;
  private int tb_height;

  private Vector applets;

  public static TaskBar getTaskBar() {
    if(properties == null) loadProperties();
    if(taskbar == null && enabled) taskbar = new TaskBar();
    return taskbar;
  }

  public TaskBar() {
    setLayout(new TaskBarLayout());

    Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();

    tb_width = (location == LEFT || location == RIGHT ? size : screen.width);
    tb_height = (location == TOP || location == BOTTOM ? size : screen.height);

    int tx = (location == RIGHT ? screen.width - size : 0);
    int ty = (location == BOTTOM ? screen.height - size : 0);

    setBounds(tx, ty, tb_width, tb_height);
    setBackground(Color.black);

    addFocusListener(this);

    applets = new Vector();

    loadApplets();

    setVisible(true);
  }

  /*
  ** Load the taskbar properties.
  */

  private static void loadProperties() {
    properties = new Properties();
  
    try {
      properties.load(ClassLoader.getSystemResourceAsStream("taskbar.properties"));
    } 
    catch(Exception e) {
    }

    enabled = (Boolean.valueOf(properties.getProperty("taskbar", "false"))).booleanValue();

    if(!enabled) {
      
      return;
      
    }

    String loc = properties.getProperty("taskbar.location", "top");
    if(loc.equals("top")) {
      location = TOP;
    }
    else if(loc.equals("bottom")) {
      location = BOTTOM;
    }
    else if(loc.equals("left")) {
      location = LEFT;
    }
    else if(loc.equals("right")) {
      location = RIGHT;
    }

    size = Integer.parseInt(properties.getProperty("taskbar.size", "32"));

  }

  /*
  ** Load the applets.
  */

  private void loadApplets() {
    String classname = null;
    int i = 1;
    TaskBarConstraints constr;
    TaskBarApplet applet;
    Class appletClass;
    Constructor appletConstructor;

    while((classname = properties.getProperty("taskbar.applet." + i + ".class")) != null) {
      int position = Integer.parseInt(properties.getProperty("taskbar.applet." + i + ".position", "-1"));
     
      try {
        appletClass = Class.forName(classname);
        appletConstructor = appletClass.getConstructor(new Class[]{TaskBar.class});
        applet = (TaskBarApplet)appletConstructor.newInstance(new Object[]{this});
      
        if(position != -1) {
          constr = new TaskBarConstraints(position);
        }
        else {
          constr = new TaskBarConstraints();
        }

        addApplet(applet, constr);
      }
      catch(Exception e) {
        System.out.println("Oh no : " + e);
      }
      
      i++;
    }
    
  }

  /*
  ** Get the taskbar size, location & orientation.
  */
  
  public static int getBarSize() {
    return size;
  }

  public static int getBarLocation() {
    return location;
  }

  public static int getBarOrientation() {
    return (location == LEFT || location == RIGHT ? VERTICAL : HORIZONTAL);
  }

  /*
  ** Size can not be changed, that's why these 3 return the same size.
  */

  public Dimension getPreferredSize() {
    return new Dimension(tb_width, tb_height);
  }
  
  public Dimension getMinimumSize() {
    return new Dimension(tb_width, tb_height);
  }
  
  public Dimension getMaximumSize() {
    return new Dimension(tb_width, tb_height);
  }

  /*
  ** Applet stuff.
  */

  public void addApplet(TaskBarApplet applet, TaskBarConstraints consts) {
    add(applet, consts);
    applets.add(applet);
  }

  public int getAppletCount() {
    return applets.size();
  }

  public TaskBarApplet getApplet(int i) {
    return (TaskBarApplet)applets.elementAt(i);
  }
  
  /*
  ** paint -> draws the background image.
  */
  
  public void paint(Graphics g) {
    Image img;
    if(Toolkit.getDefaultToolkit().getScreenSize().width > 320) {
      img = Toolkit.getDefaultToolkit().createImage(imageData_large);
    }
    else {
      img = Toolkit.getDefaultToolkit().createImage(imageData);
    }
    g.drawImage(img, 0, 0, null);
  }

  /*
  ** Get rid of the focus if we receive it.
  */
  
  public void focusGained(FocusEvent evt) {
    Component.revertFocus();
  }

  public void focusLost(FocusEvent evt) {
  }

  /*
  ** Properties.
  */

  public Properties getProperties() {
    return properties;
  }

  private static final byte[] imageData = {
 -119,   80,   78,   71,   13,   10,   26,   10,    0,    0,    0,   13,   73,   72,   68,   82,    0,    0,    0,  -16,    0,    0,    0,   32,    8,
    6,    0,    0,    0,   44,  107,  -11,  -67,    0,    0,    0,    6,   98,   75,   71,   68,    0,    0,    0,   51,    0, -105,  -79,    5,  -26,
   81,    0,    0,    0,    9,  112,   72,   89,  115,    0,    0,   11,   18,    0,    0,   11,   18,    1,  -46,  -35,  126,   -4,    0,    0,    0,
    7,  116,   73,   77,   69,    7,  -46,    1,   14,   17,   17,   57,   95,  -91,   43,   88,    0,    0,   32,    0,   73,   68,   65,   84,  120,
 -100,  -43,  -99,  121, -100,   30,   69,  -71,  -17,  -65,  -75,  116,  -65,  -53,   76,  102,  -55,  100,  -33,   73,   32, -124,   64,   32,    4,
    8, -103,    3,  -56,   22,   22,   89,   35, -102,  -80,   41, -118,  -24,   61,  110,   71,   36,  -88,  120,   80,  -15,  -36,   43,   30,   68,
  -82,    1,  -47,   11,  -24,  -15,  120,   20,   21,  -71, -119,  -56,   38,    2,  -78,  -56, -102,    9,   16,   32,    4,   66,   18,   32, -112,
   48,  -39,  -41,   89,   50,  -13,   46,  -35,   93,  117,   -1,  -88,  -22,  126,  -33,    9,  -24, -127, -125,   30,  -68, -107,   79,  -89,  -33,
  -73,  -33,  -18,  -22,  -22,  -22,   -6,   61,  -53,  -17,  121,  -86,   70,    0, -106,   -1,  114,  -47,    8,    2,    4,   33,   16,    0,  -62,
   31,  -73,  123,  -20,   -9,   -4,   -4,  -25, -118,  120, -101,  -49,  -75,   58,  -13,   -7, -120,   48,   39, -120,   35,   69,  -87, -110, -112,
   36,   17,   80,    6, -110,   -1,   98,   -5,  -33,   73,  -55,   65,  -10,  124,  -38,  111,  -46,  -73,   75,  -32,  -98,  -53,    2,    6, -120,
   -3,   22,    1,   85,  -96,  -14,   55,  108,  -41,  -33,  -80,    8,  -31,  -98,   78,    8,   -9,   89,   72,   68,  -74,    7,   33,  -36,  -13,
 -117,  -70,  -33,  -10,   -4,   62,  -16,   28,  -79,  -57,   57,  -62,  -41,  -29, -114,   73,   33,   65,  -32,  -10,    0,  -18,  -74,   72,   33,
  -79,  -44,   62,   35,   64,   32,   65,  -90,  -19,  -48,    0,   72,  -87,  -35,  -71,   66,  -94,  -92,    4,  -31,  -34, -111, -108,   26, -124,
   66,   74, -119,   16,   26,   33, -108,   63,  -90, -111,   82,   -5,   99,  -18,  -77,  -44,   45,   72,   61,   26,   37,   13,  -58,  -20,  -62,
  -38,    8,   41,   52,   72, -123,   86,   33,    8, -123, -110,    1,   66,   10, -108,  -46,    8,  -87, -112,   82,   32,   16,    8,  -95,  124,
  -93,    1,  -21,  -58, -126,  -75, -106,   36,   49,   88, -109, -112, -104,    4,  107,   12,   16, -111,  -60,   49, -119, -119,  -80,   54,  -63,
   90, -115, -112, -125,  -79,   70,   98,  -30,   13,   36,   73,   55,  -40,    8,   99,   98,  -80,    9, -119, -119,  -79,  -42, -128, -119,   49,
   54,  -63, -102,   24,   99,   99,  -84,   73,  -64,   26, -116,   77,  -48,  110,   64,  -66,   -5,  -94,  -92,   64,  -55,   16,   33,   66,   32,
  116,   13,   23,  123, -128,  -76,   -2,  -85, -128,  -74,  -63,   67,   -8,  -26,  101,  -33,  103,  -24,  -32,   97,    0,   84,  -94,   50,  -41,
   92,   -1,  117,   86,  -82,  122,   -7,  109,  -49,   31,   88, -105,   68, -118,   60,   82,   36, -124,   57, -127,  -79,   16,   37,   16,   39,
   17,   54,  -39,  -13,  -28,   -9,   94,  116,   32,   17,  -28, -127,   28, -106,   92,   93,  -77,   82,  -64,  -18,   41, -112,    4,   22,  -33,
   23, -128,  -96,    2,   84,  -80, -108, -119,   35,  -13,   87,  111,  -33,  -33,  -84,    8, -127,  -12,  -32,  -78,   30,  112,  -46,  -63,    6,
 -103,    2,   16, -127, -111,   41,  -64,  -55,  -50,    7,   16,   50,   -3,  -18,  -85, -109,   78,   24,  -92,  -96,  -52, -114,  103,   32,   38,
   -5,   93,   10,   50,  -64, -118,  -70,  -17,   41,  120,  -63,   29,  115,  -97,  -91,   -5,   44,  101,  118,   76,    9,  -23,    5, -120,  107,
   68,   42,   36,  -92,   80,   72,  -91,    0,    7,   86,   43,   52,   82,   42,   15,  110, -115,  -44, -115,  -28,  114,   35,   16,  -46,   16,
   69,  -69,   16,   68,   72,   21,   32,  101, -128, -112,   10,  -91,   66,  -92,   82,   40,   21,   32,  -91,   66,  -85,  -64,    9,    5,   41,
   93,   61,    8,   44,   18,   41,  -84,    7,  -80,  -51,   64, -101,   36,    9, -119, -119,   48,   73,   66, -100,    8,   20,   18,   97,    5,
   38, -119,  -64,  -58,   88,  118,  -94,  -62,  -63,   36,  122,   56,   42, -118, -119,  -29,  -35,   72,    9,  -58,  -30,   -2,  -77,   22,   99,
   36,  -46,   26, -116,   17,   40,  -85,   48,   54,   61,   46,  -48,  -88,  -32,   93,  -65,   99,   21,   64,   96,    2,  -76,   10,   17, -124,
   88,   97,  124,  -49,  -17,  113,  -94,  -11,  -61,  -36,  -70,   31,   79,   62,  -31,  116,   -2,  -15,   75,  -89,   14,   56,  101,  123,  -41,
   69,   92,   -2,  -51,   47,   15,   24,   64,  -62,  -17,   -3,    1,   95, -105,   64,   74, -127, -108,   33,   74,   10,  100,   81,   82,  -86,
 -108,  -80,   85,   69, -126,  122,   47,  102,  -60,  -64,  103,    3, -126,   80,   34,   68,    1,  -56,  -41,   30,  -28,  109,   -9,  -11,   37,
  -43,  -58,  105,  -69,  115,  -92,  -38,   91,  -21,   18,   81,  -43,   -4,   77,  -19, -124,  119,   91,   22,  -34,   -6,  -85,  -73,   61,  -66,
  -32,  -70,  -21,  121,  106,  -55,   82,   -9,   24,   25,   18,    5,  -74,   14,  101,    2, -127, -110,   32,  -21,  -76,  -85,   20,   32,  -46,
  127,   66,   96, -108,   59,  -65,  -90,  105,  -87,    1,   74,  122, -112,  -19,  -95,  101,  -63,  -23, -128,   12,  -40,  -94,  -42, -116,    1,
  -96,   21,   89,   83,   72,  -79,  -99,   90,    7,    7,   78,   63, -128,   11,   47,   56, -121, -101,  127,   -7,   91,   94,  124,  -23,   85,
 -124,  127,    6,   33,    5,  -45,  -90,   77,  -26,   99,  -25,  -99,  -63,  -81,  111,  -67, -113,   21,   47,  -81,  -11,    0,  -44,  -28,   11,
   67, -111,   42,   71,   28,  109,   69,   16,  -95,  116, -128, -112,   33,   82,  105,   55,  -34, -108,   66,  -23, -128,   80, -121,    8,  -27,
    0,  -83, -107,  112,  -32, -106,  106,  -64,   51,   90,  107,  -79,   22, -110,   36,   38,   49,   22, -109,  -60,   36, -119,   34, -118,   34,
  -92, -108,   36,   82,   96, -116,   36,   17,    2,  107,  -64, -102,   24,   65,   55,   97,  -40, -122,   81,  -61,  -96,   92,  -58,   24,   11,
   38,   65,   10, -123,  -75,  -42, -113,   44,  -81,   32,  -83,    5,   35,  -78,  -95,  -90,   17,  -31,  -69,  122,  -15,   34,    0,   25,   66,
   16,  -27,   80,   42,  -60, -104,    4,   15,   57, -121,   83,   -1,  -98,  -87,  -57,  -76, -128,   65,   13,  109,  124,  -14,  -94,   11,  -34,
   82,  -33, -121,   62,  114,   50,  -33,  -69,  106,    1,  -35,  -91,   77,  -18, -128,  -11,   47,  -37,   10,  103,  -87,  -90,  -33,   -3,   64,
   50, -111,   69,   13,    9,    8,   75,   26,   72,   80,   42,  -94,  -65,    4,  -15,   95,   65,  -55,   41,    5, -123,   16, -108,   42,  -32,
   44, -109,  122,   77,  -69,  -25,  -25,   61,   75,  -67,  -64,  -87,    7,  115,   30,  -48,  104,   93,  -94,   84, -123,  -28,  -17,    4,  -59,
  115,   63,  114,   22, -117,   23,  119,  112,  -19,  117,   63,  -56, -114,  -51, -102,  117,   56,   29,   79,   60,  -62,  -68,  115,   46,  -32,
  -74,  -37,  127,  -97, -103,  -48,   41,   16,  -83,   20,   53,  -51,   44,  -63,   -6,  -33,   37,  120,  117,  -23,  -66,   91,    1,  -54, -125,
   89, -111,  -66,   63, -115, -112,  -46,  107,   76,    9,   66,  -30,   21,   40, -112, -102,  -57,  -18, -107,   99,  -35,  -53, -108, -104,  -20,
   60,   33,   36, -111,  -86, -127,   53,  -67,  -91, -108,  -50,   54,   64,   58,   97,   48,  106,  -28,   48,   78,   58,  -15,   24,  110,  -71,
  -27,   87, -108,   74,  -35,   52,   12,   26, -126, -108,   10,   41,   21,  -93,   70,   14,  -29, -124,  -29,  103,  113,  -53,   45,   55,  -45,
  -33,  -65, -109,   65,  -51,  -93,  -56,  -27,  -38,    8,  115,   67,  -87,   86,  -73,   34,  -88,  -94, -108,   70,  -23,  -64, -103,  -40,   42,
   36,  -44,    1,   42,    8,    8,  116, -120,  -42,   14,  -44, -127,  -42,   72,  -91,  -48,  -54,  105,  -32,   84,  120, -128,    3,  112,   98,
   44,  -58,   36,   78,  -21,  -58,    1,  113,  -94,   80,   90,   17,   71,   17,  113,   44, -120,  -30,    8,   33,  -64,   36, -106,   68,   88,
  -84,  -83,   96,  109,   15,   65,  110,   40,  -42,  -18,  -90,   90,  -39, -122,   18,   10,  107,   44,  -58,   58,  -31,  104,  -83,   23, -116,
   70,   32, -123,    0,  -93,  -64,  -14,  -18,   52,  -80,   80,  -96,   67,   40, -108,   21,   90,  -25,   48,   38,   65,  -42, -125,   55,   29,
  -61,    6,  -86,   77,  110,  -92,   90,   -1,  -33,  -88,   -3,   70,  114,  -24,  -31,   83,  -34,   82,  -25,  -80,   17,  109,   28,   59,  -25,
   40,   22,  -35,  -67,  -48,  -33,  -60,  -90,   53,   18,  -10,   40,   47,  126,   29,  -96, -123,   16,   84,    7,   25, -120,   37, -127,  -43,
  -28, -126,    2,   82,   38,   84,  -94, -124,   -8,   61,    2,   67,    9,   24, -108,    3,   29, -122,  120,  -55, -111,  109,  -42,  -42,  -17,
  -77,  -89,  -38,  -77,  119,  -36,   -1,  117,  -66,   94,  -83,  104, -126,  -96,    1,   45,  -85,  -12,   86,   32,   -7,  107, -103,   11,  -17,
  -79,  108,  -40,  -80,  -98,  -33,   -2,  -10,   54,   80,    5,    0,  126,  123,   -5,   31,   24,   59,  118,   28,   11,  111,  -67,   25, -107,
  107,    6,   85,  -56,  -98,    7,  -81,   97,  -83,  -12, -128,  -11,   90,   45,  -45,  -50,   66,  100,  -26,  -84,    0, -124,   -9,   83,   73,
  -51,   90,  -87,  -67,  -17, -102,  106,   92,    7,   90,   39,  -89,  -35,   40,  -78,   30,  -52,  -18, -104,    1,   99,   60,   48,   12,   72,
   73,  -50,   56,  -64,   10,  -96,  -86,  -35,  117,   53,    1,   32,  107,  -66,   50,   80,  -22,  -17,   33,   78,   42,   72,    9,   82,  -43,
   76,  118, -128,  -66,  -34,  -19,  -60,  -43,   18,   90,  -25,    9,  114, -125,   49,   73,   15,  -42,  116,   35,  -92,    3,  -90, -108,   10,
  -91,   67,    2,   29,  -96,  117,   64,   16,  -26,   80,   42,   71,   24, -124,  -24,   64,   17,  -24,    0,  -91,   21,   90,  105,   47,   28,
  -16,  -18,    6,   24,   99,  -79,  -58, -110, -104, -104,   56,   78, -120,  -29, -120,   56, -110,   84,  -91,   68,   75,   73,  -43,  -55,   48,
  -94,  -40,   34,   68,    0,  -58,   98,   19,   11,  -90,    7,   99,  114, -124,  -71,   54, -116,  -23,  -59,   38,   22,   35,   44,  -46,   90,
 -116,  -79,   24,   11,   90, -121,   32,   18,    4,   49,   73, -110,   96, -110,    8, -119,   12,  121,  -89, -101,  -52, -123,  -56,  124, -120,
   86,   57,   42,  -83,   49,  -43,   86,   75,  -44,    2, -107,   22,   65,  -75,   85,   16,  -75,   72,  -73,  -75,   42,  -84,   12,  -79,  -46,
   -5, -124,   34,  -28,  -30,   79,   95,   74, -104,  127,  -85,  -80,    8,  115, -102,  -49, -100,  127,    9,    8,  119,   30, -124,  -39,  -75,
   81,  -85,  -54,  -22,  -84,  -74,   10,   42,   45,    2,   43,   20,    9, -110,  -88,   73,   18,  -24,   34,   65,   80,   36,  -52, -121,    8,
  -11,  -50,  -97,   99,  -49,   77,  -87, -112, -122,  124,   72,   88,    8,   81,   90,  -94, -108,   69,   42, -117, -112, -114,  -90,  -45,   66,
  -94, -124,   66,    9,  -19,  -73,    0,   89,  -73,   41,   17,  -44,   -3,  -90,  -48,   66,  -94,  -67, -106, -110,  -54,  -94, -108,   69,  105,
   73,   88,  112,   -9,   81,  -17,  -95,  -83,  127,  -75,   45,   45,   82,   67,   48,   40,  -37, -124,  114,   -2,  -66,  -87,  -10, -128,   12,
   57,   -4,  -16,  -61,  121,  -20,  -95,  -69,   41,   -9,  110,  -28,  -31, -121,  -18,  102,  -26,  -31,   51,  -79,   42,  -28,   67,  103,  -51,
  -95,  123,  -25, -101,  -52,   -7,  -48,   25,   32,  115,   32,   67,  118,  108,  123,  -99,  127,   -5,  -23,   13,   88,  -27,  -34,  -57,  105,
  103, -100,  -50, -122, -115,  -81,  112,  -38, -103,  103,   96,  101,  -98,   83,   79,   61, -123,   55,  -41,   46,  -29,  -51,  -75,  -53,   -8,
  -58,  -41,   47,  101,  -28,  -56,  -47,    8,   17,   98,  -91,  -26,   -6,   31,  125, -113,   53,  -81,   46,  101,   -8,  -56, -111,   44,   -4,
  -65,   63,  -29,  -28,   83,   62, -120,  -43,   69, -122, -113,   26,  -53,  101, -105,  127, -107, -107,  -85,  -97,   99,  -59,   43,  -49,  113,
  -62,  -55,   39,   98,  101,   72,  -34, -124,   20,  108,  -56,   -8,   33,  -93,   -7,  -43,  -51,   63,  -27,  -91,  101, -117,  121,  -15,   -7,
   39,   56,  -24,  -96,  -87,  -75,   71,   19,    2, -107,    2,   82,   57,  -97,   21,   96,  -42,  -84,   89,  -84,   94,  -15,    0,   79,   60,
  120,   29,  -41,   95,  115,   54, -125,    7,   91,  -92,   84,   28,  123,  -12,   52,   30,   -2,   -3,   21,   28,   -9, -127,    3,    8,    2,
 -123,   14,   28,  120,   23,   -3,   -4,  -29,  -36,   -6,  -17,  -25, -112,  -53,    7,   20,   10,  121,   10,  -59,    2,  -59,   98, -111, -122,
 -122,   34, -115, -115,  121,   26,   26,   11,   52,  102,   91,  -98, -122, -122,    2,   13,  -59,    6, -118,  -59,   60, -123,   66,  -98,  124,
   62,  -92, -112,   15,    9,  114,   33,   97, -104,   35,    8,   66,  -62,   32,   64,    7,   26,  -87,   52,  -62, -109,   98,  -42,  -10,  -95,
  116,  -98,   32,  108,   65,   40, -127,   84,  -50,   79,   40,   22,   11,  -76,   52,  105,   90, -102,  -95,  -79, -112,  -48,   80,   72,   24,
   84,   52,  -76,   52, -125,  116,  -20,  -22,  127,  -66,  -55,  -64, -103,   18,   42,    8,  -88,  -76,    9,   44,    1,   35, -101,  -89,   49,
  125,  -62, -121,   56,  108,  -81,   15,  -47, -104,   27, -123,  -75,    1,  -42,    6,   24,  -21,  -81,  -77,    1, -106, -128,  -55,   19,   14,
  -30,  -28,  -29,  103,  103,   29,  -69,  118,  101,   -1,    0,   16,   79,  -98,   54, -104, -119,   99,  -89,   97,  113,  -25,  -29,  -81,   55,
  -66,   62,  -21,  -21,   65,  104, -116,  -48,   88,  -85,  -79,   73,   72,  -75,   57,   70,  -53, -112,   32,   23,  -96, -125,  119,  -10,   28,
  111,  -73,  -27, -125, -128,   98,   24,   32,   85, -128,  -74,   10,  101,   53,  -46,    4,   72,  -29,  -18,   91,  -33, -114,    4,  -65,  -39,
  -70,  -51,   31,  -85,  127,  126,   75, -112,  -43,  -95,  -84,   70,   91, -123,   84,  -18,   62,   -7,   -9,  -48,  -42,  -65,  -34, -106,   13,
  -13,  -20,  -40,  -84, -103,   51, -103,   53,  115,    6,  -33,  -65,  -10,  122,   55,  -48,   15, -101,  -55,  111,  126,  113,    3,  -77,   14,
   63, -124,  121,  103,  -97,  -57,  -84, -103,   51,  -72,   -7,  -89,   63,  -32, -112, -125,   15,  -28,  -87,  103, -106,  -77,  -28,  -87,  -91,
 -108,  122,  -73,   80,  -83,  -10, -109,  -70,   98,  -43,   74,   47,   61,   93, -101,  -79,   66,   35,   68,  -64,  -45,  -49,   60,  -57,   61,
  119,   -1, -122,   79,   92,  112,   22,   55,  -35,  -16,   93,  -98,  126,  -26,   57,   22,   44,   88,  -64,  -89,   63,  117,   62,  -41,   45,
   -8,   22,  113, -110,  -32,  -36,   21,  -89,   30,  -81,  -71,   -6,  107,  -52,   56,   -8,    0,  -74,  111,   91,   71,  111,  -17,   46,  -82,
   -5,  -63,  -65,  114,  -31, -123,  -25,  113,  -61,   13,   63,  -90,  -93,  -93, -125,    5,  -41,   -3,   43,  -13,  -26,  -99,    1,  104, -124,
  -44,   92,  -11,   -3,   43, -103,  126,  -48,    1,   92,   -1,  -61,   31,  -47,  -47,  -47,  -63,  -57,  -50,  -97, -105,   61, -103,  -16,   86,
 -126,   76,   -3,   96,  -81, -127,  -25,  -49,  -97,  -49,  -46,  103,   95,   96,  -63, -126,    5,   76,  -35,  119,    4, -105,  126,   -2,   40,
  -86, -107,   62,   86,  -66,  -78,   25, -128,  -68,  124, -123,   45,   27, -105,  -93, -125, -112,   92,  -82,   70,   96,   -2,  -12,    7,  103,
   80,   40,  -28,   41,   22,   10,   20,   10,   57,  -14, -123,   28, -123,   98, -127, -122,   98, -127, -122, -122,    2,  -59,   98,  -98, -122,
   98, -111,  124,   33,   71,  -66,   16,   80,   44,  -28,  -55,  -25,  -35, -106,  -53,  -27,  -55, -123,    1,   65,  -24,  113,  -92,   53,   74,
  105,  -89,  -51, -107,    2,   41,  -79,  -90,  -97,  -60,  -20,   70,    5,   77,  104, -107,   35,    8,   11,   52,   53,    6,  -28,  -13,   22,
   37,   21, -119,  113,  100, -103,   69, -110,    0, -119,    1, -115,  122,    7,   62,  -80,  116,   66,  -74,  -87,  -87, -115,  -68,   46,   50,
  -78,  117,   34,  -43,  -88,   66, -100,  -60,   36,  -58,   48,  -94,  109,   31,  102,   31,   50, -113,  -67,   -9,  -98,  -56,  -13,  111,  -36,
  -57,  -46,   23,  -97,  103,  -15, -118,   63, -110,   -6,  -35,   39,  124,  -32,   68, -122, -116,  112,   29,  -47,  -65,   59,  -30, -102,   27,
  127,  -60,  119,  -82,  -72, -104, -106,   33,  -18,   88,  107,  107,   51,  103,  -97,   62, -113,  -85,  110,   -6,  -82,  -17,  121,   50,   55,
  -46,  -42,   49, -105,  -42,   19,   23,    9, -110,   88,   72,  -80,  121,  100,  107, -103,  -96,   63,   36,   72,   32,   74,  -93,   57,  -17,
  -94,  -28,   20,   20,  -14,   16,   40,   79,   -6,   73,  -17, -126,    9,   87, -107,   39,   -4, -100,   49,  109,  -22, -116,  -25,   58,   11,
   57,   51,  -84,  -21,   72, -107,  -76,  -35,   42,   -3,   93, -126,  -78,   78,  -31,   21, -124,   11,   54,   85,  -34,  103,  127,  120,  -18,
  -36,  -71,  -40,  -71,  115,  -33,  114,   -4,  -70,  107,   23,   64,  -40,  -54, -121,  -49,   58, -115,  -79,   99,   70,   49,  118,  -36,   94,
  -84,  -17,   92,  -53,  -60,  101,  -81,  -15,  -21,  -97,   -1, -128,  -85,  -66,  125,   25,   39, -100,   -6,   49,   54,  109,  -34,  -50,  -62,
 -123,   11,  105,  108, -103,  -64, -103,  103,  126,   48,  -85,  -13,  -10,   59,   -2,  -64,  -97,   30,   95,  -55,   79,  110,   -6,   30,  119,
  -36,  121,   47,   91,  -74,  117,  -13,  -39,  -49,  124, -110,  -89,  -97,  126, -114,  -61,   15,   63, -124,   48,  -33,  -60,   61,   -9,   62,
  -62,   67,   15,  -36,  -59,  -39,  115,  103,  115,  -37,   93,   79,  -69, -114,  -61, -127,  107,  -39,   11,   47,   83,   46,  -11,  112,  -22,
   25,   31,  -27,  -32,  -23,   -5,  -13,  -43,  -53,   46,  -25, -102,  -17,   93,  -59,  -32,  -74,  -79,  124,  -21, -118,  -81,  -14, -119,   79,
  -98,  -49,  -45,  -49,   60,   71,   91,  -53,    8,   14, -100,  -66,   63,   95,  -69,  -20,  114,  -82,   -2,  -34,   85,   12,   29,   57,  -98,
  111,   -4,  -13, -105,   -7,  -30,   63,  125,    1,  -16, -116,  120,  106,   62,   -5,  -80,   15,  -64, -126,    5,   11,  -72,  -12,  -46,   75,
  105,  105, -101,  -60,  -18, -110,  -26, -118,  -81,  127, -111,  -93,  103,  -35,  -61,  115,   43,   -5,   89,  116,   -5,   98,  -26,  -49,  -97,
  -49,  -38,  -11,   87,  -78,   99,  119,  -56,   63,   28,  -66,   23,    0,  -13,  -26,   57,  -63, -112,  -49,  -25,    8,  -62, -112,  124,   78,
   19,    6,   33,   65,   40,  -47,   82,   33,  -92,  -64,   90, -117,   73,   12,  -43,   40,  -95,   26,   69, -108,  -53,   49,   82,    9,   31,
  -38,    2, -124,  113,  110,    3,    6,   72,   16,   36,   88,   18,  -92,   85, -128,   34,   33,    1,   91,   34,  -52, -115,   64,  -38,   86,
  114,  122,   27,  -42,   40,  -84,   53, -114,  -12,  -78,  -18, -103,   60,   69, -128,  -27,   29, -110,   88, -115,  -59,   65,    4,    5,   77,
   76,   31,   93,   81,   47,  -69,  -74,  108,    1,   11,  -58,  -72,   90,   94,  -40,  -16,   24,   -9,   45,   -1,   57,  -29,    7,   31,  -56,
  -87,   51,   62,  -54,  -59,  -25,  -99,  -54,   39,   74,   31,  -25,   91,   63,   -2,  103,   74,  -91, -104,  115,  -25,  -43, -104,  -25,  -43,
  -81,  -83,  -31,  103,  -65,  -69, -107,   99, -113,   59, -116, -113,  -52,   57,   22,    0,  -91,   36,  -25,  -98,  119,   18,   -1,   -6,  -29,
    5,  -75, -101,  -42,   92,  -81,   12,  -72,  -92,  -28, -101, -108,   88,  -85,  -63,   74,  -84,   48,  -82,   19,    3,   80,    9,   36,  -43,
  119,   48,  114,  -21,  110, -111,  -53,   57,  -30,   42,  117,  111, -115,   23,   26, -106,  -70,   99,   -2,  124,   83,   39,   72,  -46,   31,
   66,    5,   90, -125,   86,  -98, -128,   17,  -82, -125,   35,    3,  113,  -30,  -82,   73,   93,   68,  -27,  -21,   45,   40,  -88,   90,  -88,
 -106,  -34,   83,   16,   -2,   61, -105,   69, -117,   22,  101,    3,  115,  -49,   34, -102,  -89,   48,  118,  -52,   40,    0,  -42,  119,  -82,
   37,  108,  -39, -113,   45,   59,   99,   54,  110,  -19,  -31,  -61,  103,  -98,   68,  -71,  111,   51,  119,  -36,  -13,    8,   31,   58,  -13,
  100,   -6,  -70,  -41,  113,  -40,   33,  -45,   -7,  -35,   29,  127,   32, -119,   -6,   56,  -12, -112,  105,   -4,  -31,  -63,   37,    0,  -36,
  -14,  -21,   -1,   64,    5,  121,   70, -113,   30,  -63,  -70,   55,  -33,  -32, -100,  115,   47,  -96,  117,  -24,  -92,   76,    0,  -50,   60,
  108,    6,   63,   -5,  -59,  -17,   50,   41,  -67,  100,  -55,   18,  -58,   76, -100,   69,  -95,  -40,  -62,  -60,  125, -100,   57,   60,   98,
   -8,   16,   46,   -6,  -12, -105,  104,  106,   26,   74,  115,  -53,   72,  -58, -114,   29,   75,   67,  -93,  -90,  101,  -16,   32,    0,   94,
  127,  -29,   53,   38, -116,  -34, -113,  113,   35,   38,  -47,  -75,  -87,  102,  -35,  -43,   88,   98,   23,   86,   74,    1,  -68,  100,  -55,
   18,  -58,   79,   58, -126,   -3,  103, -100,   11,  -70,   13, -128, -125,  -89,   79,  -27,  -10,  123,  110,  -26,  -23,  101,  123,  113,  -28,
  -84,  -83,   92,  -65,  -32,   27,   28,  115,  -14,   37,  124,  -18,   39,  -25,  -79,  117,   91,   23,  -99,  -99,  -99, -100,  117,  -34, -107,
   20,   10,   57,   10,   -7,   28, -123,   66,   64,   62,   23,   16, -124, -118,   64,   75, -108,   20,   36,  -58,   18,  -57, -122,   40,   50,
 -108,  -53,    1, -127,  -82,   82,   42,  121,   18,   74,   24,   44,    9,  -58,   38,  -28,  108,    8,   54,  -95,  106,   19,    2, -101,   96,
  109, -128,  -53,  101, -112,   96,  -53,   88,   91,  -90,  -79,  -79,   64,   82,   85,   24,   98,   76,   54, -120,   82, -115, -122,   23,   24,
   26,   61,  -64,   31,  122, -101, -110,   11,   10,  -96,    5,   21,   -6,   60,  -75,   19,  -42,    1,  -55,  -57,  -85, -128,  -40,  -62, -102,
  -99,   43,  -72,  -15,  -95,  127,  -31,  -75,   77,  -25,   49,  -25,   -8,  -45,   -7,  -59,  -43,   63,   99,   -7,  -85,  -53,   56,  112,   -6,
 -124,  -84,  -66,   59,  -17,   94, -126,  -79, -126,   39,   -2,  -12,   90,    6,   96, -128,  113,  -93, -121,  113,  -58,  -79,   31,  -26,  -82,
   63,  -35,   77,   26,   51,  116,  111,  -63,  109,  -90,  -18, -104,   17,   96,  -84,  -58,  -94,  -79,   26, -124,  -87,  -94,  115,   32,   29,
  -17, -111,   73,  -88,   -1,  -84, -124,   26,  -14, -127,    7,  -84,  -16,  -41,    2,   86,  -43,  -76,  112,   98,  124,  -65,  122,   78,  -53,
  120,  109,   28,    8,  -56,   23,  -96,   16,  -44,   66,   30,   64,   70,  -30,    0, -108,   19,  -88,   86,  125, -102, -119,  117, -126,  -64,
  -69,   97,  -28,    3,  -89, -127,   43,  -15,   59,  107,  -21,  -33,  -84,    4,   45,   80,  116,   90,   70,  -56,  116, -116,  120,  114,   40,
   77,  -84,    0,   68,  -48, -120,   77,  -55,   40,  -64,   68,  125,  108,  -40,  -76, -125,  -50,  -11, -101, -104,   59,  119,   46,  -97,   -5,
  -57,  -13,   -7,  -22,  -41,   -2, -123,   81,   35, -121,   49,  127,   -2,  124,  -98,   91,   -2,   26,   27,   54,  109,  -91,  -77,  -77, -109,
   98,  -93,   19,    4,   99,  -57, -116,  -31,  -61,   31,  -98, -125,   14,   26,   16,    2,  -18,  -68,   -5, -113,  116,  116,  116,   80,   41,
   -9,  -30,  -59,   31,    0, -123, -122,   97,  -66,   67,   21,    0,  -19,  -19,  -19,   76, -100,   52,  -59, -123,  119, -128,   59,  -17,  -68,
 -121,  117,  -21,  -34,   96,   72,  -37,  -72,  -20, -102,  124,  -82,   17,  124,  -62,   69,  -42,  102,  -87,   17,   42,  -56,   98,  -54,  -11,
  -60,  -30,  -32,  -95,  123,    3,  101, -124,  -88,   13, -106,  -18,  -99,  -99,  -84,  121,  125,   59,   79,   60,  -11,   10,  115,  -25,   12,
   99,   88,  -45,    6,    0,  -66,  123,  -43,   -1,   98,  -59,  -54,  -41,   57,  -24,  -16,  -47,   20,   11,   57, -118,  -59,   28,   13,   13,
   33,   13, -123, -128,   66,   94,   18,    4,   26,   41,  -36,  -40, -120,  -94, -104,  114,  -59,  -48,   23,   74, -108,   74,  -51,  118, -117,
  -59,   49,  -45,   88,  -73,  -59, -119,  115,  -85, -116,   81,   40,   27,    3,   18,  107,   37,  -40,   42,   -7,   48,   33, -105,   43,  -48,
   31,   75,  -84,  -83,  -75,   57,  -43,  104,  -58,  -54,   44,  -58,  -90,   81,  127,   62, -111,  -93,   41,  -33,  -58,  -18,  104,   39,   72,
 -117,  -42,  -38,  105,  -98,   52,   86,  -19,   27, -100,  125,  119,  -19,   36,  -78,    9,   -9,  -65,   -4,   27,   74,  113,  -60,   23,   71,
  -98,  -53,  -87,  -77,  107,   32,  -19,  -39,   85,  -27,  -79,  -89,   94,  -64,  -96,   -7,  125,  -57,   31,  -72,  -78,   -5,  -29,   52,   54,
   59,    1, -110,   43,  104,   78,   56,  126,   22,  119,   60,  118,  111,  -58,    1,  -41, -115,  -97, -116,  -79,   20,   41,  -40, -124,    4,
   66,  119,  111,    9,   50,   48,  -88,    8,   18,  -19,   -6,  -24,  -19, -118,   80,   18,    2,  -49, -124,   90,   67,   94,   26,   66,  109,
  106,  124,  -77,  -41,  -98,   88,   48,   30,  -76,   86,  -42,  -76,  113,   26,  126,  -53,  105,   40,    6,  -50,  -20,   78,  -81,   77,  -75,
  -84,   17,  100,  121,   90, -123,   16,    2,   13,  -91,  -86,   75,   58,  -79,  -78,  -90,  -67,   67,    9,  -71,    4,   42,  -17,   55,   35,
   45,    4,  -23,   24,   72,  -39,  -31,  -44,  -38,  -23,  -36,  -80,   13, -128,   49,   99,  -58,   80,   17,   33,  -61, -121,   13,  101,  -12,
  -56,  -31,   44,  -18,  120,   26, -128,  103,  -98,   95,  -51,   51,  -49,  -66,  -60, -105,  -66,  116,    9,    0,  -41,   92,   -3,   63,   -7,
  -24, -123,   -1,    4,  -64,  -25,   -1,  -15,   99,   60,   -3,  -52,   50,  -98,   94,  -70, -116,  -74,   81, -121,  -80,  113,  -13,  118,  -42,
  -81,   95,  -49,   57,  -25,  -98,   79,  -82,   97,    8,   88,  -53,   89,  103,  -98,  -62,   -6,  -11,   27,  -55,   53,   12,  -82, -109,  -40,
 -112,   70,   70,   54,  111,  -37,    5,   56,   75,  -31,  -90,  -97,   -4, -126,   32,   44,   50,  106,  -60,    8,  102,  -50, -100,   65,  119,
   79,  -60,  -10,   29,  -67,   89,   -5,   94,  124,  -66,   19,   33,   53,   -5,   29,  -80,  111,   86, -115, -108,   10,  -87,  -75,  127,   55,
   42, -109,  -84,  -77,  102,  -51,   98,  -51, -122, -107,   24,   27,   49,  108,  104,   51,    0,   29,   29,   29,  -66,   43,   20,   47,  -83,
  -22,  -26,  -56,  -51,  -37,   88,  -72,  -48,   69,   70,  -82,  -67,  -10,   90,   78,   58,   -3,   98,   38,  -19,   59, -125,   66,   49,  -96,
  -79,   49,  100,   80,   99,  -64,  -96,   98,   72,   62,  -81,    8,   67, -105,    1, -106,   24,   67,  -75,  -86,  -56, -107,   19, -108,   15,
  -96,   24,   44,  -79, -119, -119,   77,   76,   46,   12,   49,   38,   34,   78, -100,  -42,   54,   70,  -95,  -75,  116, -128,  -76,   10,  -84,
  -60,   24, -119,  -42,   85, -124,  106,   70, -120,   16,   65,  -43,   89,  105,  -23,  -40,   71,  101,   10,  -91,  -79,  -62,  -97,  -41,  -64,
  -59,   96,   16,   -3,  113,   25,   17,    4,   72,  -19,   56,   10,  -21,  -75,   69,   10,  -28,   52,  -15,   42,   -3,  -98,   10,  -47,  -56,
   64,  103,   -9,   74,   38,   31,  -40,   72,  -91,  -49,  -78,  107,  107,  -60,  -48, -111, -118,  -27,  -85,  -41,  -16,   -8,  -78,  -91,   32,
   66,  -74,  110,  -19,  -25,  -55,   37,  -85,   56,  -23,  -92,    3,  -77,  123,  -98,  117,   -6,   63,  112,  -27,   -9,   39,  -79,  -71,  -89,
   51,  -45,  -14,  114,   15,   16,  -89,  -81,  -39, -118,    0,   35, -100,   -6,   51,   26, -108,  117,    0,   22,    9,  111,  -75,   75,    3,
 -119,  110,  -52,  -95,  -13,   97,   86, -127,   22,   80,   12,   33,   72,  -86,   68,  -69,   43, -104,  -86,  -55,  124,  -43,   84,   19,   27,
   -1,   92, -103,   89,  -19,  125,  -38,   66,  -50, -103,  -61,  -87,  -20,   22,  -87,   69,   98,   93,   23,  -92,   22, -113,  -62, -127,   92,
  -28,  -95,  -81,  -20,  -62,   71,   89,  -46, -126,  -64, -103,   -3,  -58,  107,   -7,   -9,  -83,  -56, -116, -107,  118,   24, -110,   62,   44,
    4,  -53, -105,  -81,    6,   96,  -31,  -62, -123,  124,  -22, -126,   -7,   -4,  -22,  -89,   87,  114,  -48,  -12,   -3,   57,   -9,  -68, -113,
   35,  -61,   38, -112,   33,   55,   -2,  -20,   46,   30,  -72,  -13, -121,  116,   44,  121,   10, -128,  -27,  -81,  116,  -13,  -44,  -46,  -27,
  -76,  -73,  -73,  115,  -28,  -47,   39, -110,   43,  -74, -111,  107,  -99,  -64,  115,   47,  -82,  -31,  -76,   19,  -38,  -71,  -28, -110,   75,
   89,  -12,  -32,   90,   46,   58,   -1, -125,  124,  115,   -2,  121,  -36,  121,  -49,   67,  124,  -10,  -30,  -17,   80,  -81, -127,  117,  -48,
    8,   66,  -80,   98, -107,   35, -107,   46,  -71,  -28,   18,   86,  -81,  -39,  -55,  -70, -115,   49,  -65,   -6,   -9,   43, -104,  126,  -32,
  100,   62,   -1, -123,  111,  -16,   -8, -109,  -53,  -77,  -33,  -41,  -67,  -79, -109,  -29, -114,  -99,  -51,  -36,  115,   79,  -53,  -22,  105,
  -84,   58,  -30,   80,    0,   82,   40, -124,  116,  -38,  121,   -2,   -4,   -7, -108,  -86,   55,  -79,  -67,   55,  -32,  -84,  -45, -113,    0,
   28,   72,  -89,   29,   58,    7,  -87,    4,  111,  116,  -18,   96,  -51,   27,  -37,   24,   57,   98,   40,   29,   29,   29,    0,   28,   61,
   -5,    2,  114,  -71, -112,   66,   62,  -96,  -95,   16,  -48,  -44,   24,  -46,  -48,   16,  -48,   88,  -48,  -28,  114,  -50,   98,   49,    6,
   42,   21,  -61,  -18,   32,   70,   72,   -9,   94,  -93,  -40,   16,   85,    3,  -30,   40,  -58, -104, -124,   32,  -42,  -60,   74,   17,  107,
 -123,   78,   20, -119,   82,  104,   35,   93,   58,  -91, -110, -124,   58,   71,   16,   68,   40,  -87, -112,   58,   32,   49,  110,  -96,   57,
 -106,   58,  -43,  -27, -126,  -90,  -78,  -77,  -20,   52,  -30,  -83,  -95,  -99,   66,  -48,  -56,  -96,   66,   43,   91,  122,   58,  -47,   58,
    0,  -81,  -43,  -22,  113,  -47,  -42,   48, -126, -111,   45,   99,   88,  -34,  -71,  -44,  105,   97,  -81,   54,   83,  -19,  -36,   50,   34,
  -49,  -56,    9,   33,  113,   21,   54,  -82,   45,  -45,  -40,   34,   -7,  -35,  109,   79,   57,   54,   25,  -24, -113,   34,   22,   63,  -71,
  102,    0, -128, -101, -101, -117, -100,  124,  -46,   97,   -4,  -57,  109, -101,  -77,    1,  101,  -88,   13,  120, -127,    7, -121,  -60,   49,
  -47,   86,   97, -116,   11,   -5,    8,   18,  124, -102,  -85,   75,   69,   78, -121,  103,   94,   19,  -76,   52,   32, -123,  112,   38,  -96,
  -73,  -81,    3,  101,    9,   52,  -24,   48, -128,  -80, -120,  -23,  -18,  -57, -106,   99,   71,   84,  -91,   62,  -80,  -87,   35,  -79,  -68,
  -59,   81,   12,   65,  -43,  105,  -27,   12,  -68,  -87,  -87,  111,  107,  -66,  112,  106,  -10,    7,   64,   46, -128,  -66,  -40,   95,  -25,
   45,   32,  -83,   65,   71,  -17,  115,   92,   88,   72,  -46,   49,  -32, -102,  -31,  -94,  -78, -125,   66,  120,  -10,  -79,   39, -104,  115,
  -58,   -7,   92,  -10,  -49,   95,   96,  -59,  -85,   29,  -68,  -80,  -20,  101, -114,   61,  102,   54, -113,   60,   -6,   16,  -83,  -61,   15,
  -62,  -56, -128,  103,   94,  120,   13, -128,  -11,  -99,  111,  -94,  115,   45,  108,  -36,  -46,  -51,  -26,  -83,  -35,    0,   60,   -7,  -40,
    3,   12,  -39,  -25,   20, -112, -102,  -49,  125,  -19,   70,  110,   -3,  -11,  -51,   -4,  -22,  -25,   -1, -121,  107,   -4,  -83,   63,   -7,
  -39,  -53,   -7, -113, -101,  -82,  -94,  -95,  101,   28,  -11,    0,   70,  -24,   44,  -69,  105,  -10,   25,   95,  -27, -108,   99,   70,  112,
  -49,  -99,   63,    7,  -32, -123,  -27,  -81,  113,  -52,  -79,   39,  -13,  -24,   35,   -9,   51,  113,  -17,   35,   57,  -29,  -12,   47,  -14,
  -49, -105,  -97,  -53,  109,  -73,  -69,  -33,  127,  -10,  -45,   95,  -13,  -55,   79,  -99,  -17,  -86,   17, -118,   33,  125, -102,   93,   77,
    0,   42,   75,    0,   89,  -80,   96,    1,   95,   -1,  -38,  124,    0,   94,   92,  -15,   42,  -19,  -19,   -1,    0,  -64,  -28,    3, -114,
   69,   74, -127, -110, -118,   95,   -2,  118,   21,   71,  -74,   79,  101,   -3,   -6,  -11, -116,   25,  127,    0,   74,   43,  -62,   64, -110,
   11,   21,   -7,  -68,   36,  -97,   83,   52,   22,   52, -115, -115, -118,  124,  -88,  -48,   26,  -30,   24, -126,  -64, -103,  127,   73,  108,
   40,  -28,   37,  -27, -118,   36,    8,   21,  -86,  -94, -112, -111,   64,   40, -103, -123,  -76,  -92, -110,   46,  -97,   90,  -70, -112, -111,
   52,    2,  -91,    4,  -42,   24,  -84,   74,   80,   50,   36,  -62,   89,   18,  -79,   49,   32,   83,   97,   47, -100,  -69,   40,   64,   48,
   -4,  -92,  -73,   12,  -97,   41,  -61,   15,   98,  -11,  -42,   23,   65,   26,  116,   14,  -62,  -68,  -17,   95,   83,   51, -105,  -83, -123,
  -77,   15,   61,  -97,   15,  -51,   62, -111,  -43, -101,   86,  -80,  -91,  -68, -114,   32,   16,  -24,   80,   48,  -74,  109,   60,  -89,   28,
   55, -109,   81,  -29,  -13,   24,   99,  -39,  -76,  -82, -126,   16,    9,   -1,   -5,   -1,  -36,  -50, -113,   22,  -34,  -30,   52, -105,  117,
  126,  -17,  -53,   15,   -1, -108,   66,  -79,   38,   68,   94,   88,  -70, -103, -103,  115,   47,  -60, -113,   37,   -9,   34,   -4,  127,   66,
  -32,   51,  110,   64,  -94,    8,   85,   72, -112,  -73,   40,   21,   65,   53,  -95,   20,   67,   95,    9,  -30, -118,   55,  -93,    3,   69,
  -82,  109, -112, -117,  -77,   89,   71,  -63,   11,   12,   86,   88,   26,  117,   76,   83, -104,   32,  -92,  -59,   26, -105,   33,   85,  -34,
  -47, -121,  -87,   90,   18,  106,  -52,  115, -110, -110,  117,   56,  -83,   61,  126,  -80,   35,  -84,  -30,    4,  118,  -11, -111,  -75,   83,
  120, -122,   80,  122,   19,  -38,  -13,   37,  -88,  -70,  118,   -9, -106,   93,  -58, -104,  -94,  -58,   38,  118,   85,  -96,  -81,   78,  -32,
   -4,  -73, -106,   45,   -9,   67,  110,   56,  -76,   76,  -49,   14,  -91,  -66,  -30, -108,   86, -105,   20,   33, -128,   82,  -87, -117,   13,
 -101,   94,  -90,  -65,  -44,   69,   67,  -79, -123,   49,   35,   -9,  -89,  105,   80,   27,  -85,  -69,  -56,  -46,   40,  119,  -67,  126,   39,
  -71,  -58,  -47,   12,   26,  125,    4,   82,   10,  -70,   55,   44,  -90,  -46,  -45,  -55,  -24,   67,   46,    2,   68, -106,  -72,   81,  -38,
   -7,   58,   59,  -42,   60, -104,  -35,  111,  -40,   62,   39,   48,  -88,  117,   47, -120,   43,  108,   93,   -5,   56,  -67,   59,   95,  103,
  -54,  -84,  -49,  100,   73,   25,  -62,   10,  -94,  -54,  110,  -74,  109,  124, -106,  -99, -101,   95,    2,  -96,  -40,   56, -108,   49,  -29,
   14,  -93,  -87,  121,   24,  -38,   74,  -30,  -88,  -97,  -43,  -81,   60,   76,  111,  -17,   86,    0,   38, -116,   62, -112,  -75,   27, -106,
  115,  -24,  -44,   19, -104,   48,  124,   18,  -69, -102,  -45, -119,   21,  -80,  117,  -61,  -21,   44,  127,   -6,   94,  -10,  -34,  -17,  120,
   94,   91,   -7,   16,    0,  -83,  109,   99, -103,  121,  -12,  -57,  104,  106,   25,   65,   24,   22,  -55, -123,    5,   62,  119,  -47,    7,
   56,  -28,  -64,   49,    8,   33,  -40,   -1,  -96,  -29,  -72,  -16,  -77,  -41,  -46,  -46,  -44,   64,   75,  115,  -98,  -63,   45,    5, -102,
 -102,    2,    6,   55, -121,   52,   22,   53,  -59, -126,    3,   94, -110,   88,   -6,   75, -106,  -35,   -3,   49,   59,  -69,  -85,  -12,  -10,
   70,  -20,  -36,   85,  -94,  -85,  -89,   66,  119,   79,   31,   -3,  -91,   18,  -91,  114, -103,   82,  -87,   68,  -91,   92,  -94,   92,   41,
   83,  -87, -108, -119,  -94,   18,  113,   84,   33, -114,  -53,  104,    5,   -7,  -48,  -96, -125,  -31,   84,   42,  -37,   40, -105,  -74,   96,
 -116,   37,   54,   46,  -65,   58,   49,    9, -125,  -69,   33,   22, -106,   24,   16, -116,   56,  117,    0, -128,  -91, -112,   28,   60,  -10,
   48,  -98,  125,  -13,   41,   84,    8,   42,   15,   97,   46,   77,  -82,  -10,  -26,  114,   70,  -53,  -62,   65,   19,  -90,  -13,  -19,  127,
   -6,   31,   76,  -35,  127,   24, -123,   70,   65, -104, -105,    4,  -95,   51,   41, -108,  114,    8,  -36,  -35,   93,  101,   -5, -122,   10,
   35,   39, -122,  -84,  125,  -93, -121,   27,  111,  -66, -121,   95,  -34,  -10,   48,   85,   91,  -31,  -10, -101,   -2, -123,  -39,  -57,  -17,
 -105,  -35,  -65,  -81,  -81,  -60,    7,   78,  -65, -126,   23,  -41,  -82,   34,  -51,  119,  -83,  -97,   -4,   35,   60,   64, -112, -118, -100,
   12,   81,  121,   73,   40,   43, -120,   56,  -90,  100,   28, -128, -109,  -78,   99,  -93,  117,  107,   35,  -70,  -95,    0,   54, -121, -112,
  -38,  -89,  -93,   89,  -84, -119,   25, -108,  -85,  -48,   28,  -58,  110, -122, -121,  127,  -92,  -88,   47,  -94,  -38,   85,  118,   38,  -77,
  -15,  -32,   77,  125,   95,   11,   19, -102,   33,   31,  -70, -109,  -45,   46,  -40,  -42,  -25, -101,   39,   61,  112,   33,  107,  -73, -108,
  -75,   80, -110,   16,   78,    3, -105,   35,  119, -114,   18,  -96,    2,  -59, -114,  -66, -124,  -35,  -27,   -1,   50,    4,  -33,   89,   81,
  -22,   29,  -27,  112,   58,   18,   75,   80,   12,   36,   19, -102,  -36, -125,  -44,   79,   76, -112,   62,   57,   37,  -99,  113,  -76,  -66,
   23,   74,  -90,    6,  122,   37,   21,   86,  105,  -41,   31,   66,   32,   11, -115,  -34,  124,   77,   83,   13,   69, -106,  -33, -100,  114,
    6,  -92,  -13,   20,   42,   21, -126,   48, -105,  -99,   35,  113,  102, -114,   11,    1,   65, -110,   84,  -68,   96,  -76,   72,   99, -111,
   34,   65,  -93, -112,    2,  -76,  -16,  -71,  -39,    8, -108, -128,   64,   42,  -76,    4,  109,    5,  -38,  127,  -33,  -42, -100,   10,   27,
   73,  -88, -101,    9,  -62,  -95,   84,  -54,   27,   92,  -69,   85,  -34,  101,   94,    5,  121, -122,   15,   27,  -62,  -87,   39,   28,  -52,
 -119,  -57,   76,  -31,  -30,   -7,   87,  112,   -3,  -75,  -33,  -26,  -54,  -21,  -98,  -94,  -95,  -95,   64,  -13,   32,    7,  -32,  -42, -106,
   60,   45,   77,   57,   90, -102,   53,  -51, -115, -102,   66,   94, -110,   11,    5, -107,  -86,  -91,   84,   54,  116,  -17, -114,  -23,  -22,
 -114,  -23,  -22,  -87,  -48,  -43,   93,  -90,  -85,  -69,   66,   79,  111,   63,  -67,  125,   -3,   84,   42,  101,   -6,   -5,   -5,   41, -105,
   75,   84,  -86,   21,  -94,   74, -119,   40,   42,   17,   69,   21,  -94,   20,  -64,   65, -126,   10,   70,   16,   87,  -73,   83,  -86,  108,
   39,   73,   82,  -16,   90,    6,  119,   91,  -86, -119,   33,  -15,    0, -106,  -88, -128,   -6,  109,  -10,  -76, -109,  120,  -77,  107,  -93,
   -5,   46,    3,  -57,  -30,  -87,    0,   33,  -35,  -26, -110,  -95,  -35,  -34,  -56, -128,  -25,  -41,  -83,  -32,  -20,  -53,  -66,  -63,  -43,
   55,  -36,  -50,  -90, -115,  125,  -98,  -67,   28,  -24,  -44,   73,   13,   81,   21,   42,  101,   24,   49,  -68, -119,  -17,   92,  126,   30,
   75,   -1,  -72, -128, -109,  102,   29,  -59,  -30,  -89,   94,   31,  112,  110,   67,   67, -127,   11,   63,  114,   10,  -42,  -33,  -45,  -86,
    0,   43,    3,  -36,  -76,  -59,  -38,  125, -123,    8,   92,  -78,    0,    1,   66,    7,    8, -107,  115,   -2, -114,  111,   55,   65,   30,
  -43,  -40, -120, -112,   13,  -96,   27,   65,  -73,   64,  -48,  -26,  -10,  -70,   17,   41,   27,   16,   50,    7,   34,   71,   58,   77,   80,
   23,   11, -120,   32, -121,   21,  -82,   47,  -46,  103,   22,   34,   96,  112,   67, -128,  -44, -127,  -13,  -67,  -91,  107, -109,   17,    1,
  -83,   13,  -18,  -77,  -59,   31, -109,   65,  -42,  119,    8,  119,   45,  -62,   -3,  -90,  -45,  -29,   42,   96,  -48,  -24,  -95, -116,   60,
  120,   34,  -83,   99, -122,  -78,  -25,   59,   -8,  107,  110,   34,  -56,   51,  -21,   27,  -89,   32, -126,   -4,  127,  126,  -66,   12,   64,
 -123,  -98,  -80,  116, -119,   51,   89,   31, -117,  -96,   46,  -37,   45,    0,  -87,    9, -108,    6,  -87,  125,   31, -123,   88,  -31,  -65,
   11,   -9,   29,  -95,   92,   32,   86,   10,  -84, -108,   88,   41,  -80,   50,   61,  -90,  -79,  -46,   37,   99,    8,   25,   32, -108,  -54,
 -110,  117,  -36,   12,   55,  -19,  -37,  -93,   57,  -31,  -40,   25,  -68,  -68,   -8,   70,   94,   90,  124,  -93,  -21,   83,  -87,   -3,   56,
  -44,  124,  -25,  -54,  127,  -28, -119,   39,  110,  -30,  -24,  -29,   14,  -13,  -84,  -77,  107,  -57,   17,   71,   31,  -54,  -19,   15,   -1,
 -112,   -7,   87,   92,    4,  104,   70,  -10,  106,   63,   57,   65,   35,   48, -103,  -39,   42,  124,  106,  -93,  -12,   74,  -94,  -85,  -69,
  -97,  -97,  -33,  -14,    0,    7,   31,  126,    6,  -41,   95,   -5,  109,  -50,   58,  -17,   91,  110,   34,   65,  125,  106,  -83,    1,  -84,
  -95, -104, -105,  -76,   54,   43,  -14,   57,    7,  -34,   36,  -79,   78,   78,   14,   72,   26,  -16,  -41,  -72,  -84,    2,    4,  -42,    9,
  117,    9,   74,  -38,   76,   57,   73,  -23,   93,   51,   11, -127,  118,   26,   29,   44,   74, -123,   40,  -83, -111,   90,   51,  -84,  -49,
  -11, -121,  -44,  -18,   29,   72,   21,  -96, -111,  -71,    1,    0,   42, -122,  -51,  -12,   70,   37, -112,   57, -124, -101,   78, -103,   -7,
  109,  105,  -79,  -11,  -49,   99,  -96,   47,  -87,  -14,  -29,  -69,  -17,  -28,  -90,  -69,  -18,  100,  -22,  -16,   41,   -4,  -28, -102, -117,
   56,  120,  -58,  -88,  -20,   -4,   55,   86, -108,  -40,  -78,  -74,   74,  -21,    8,  -51,  -89,  -65,  121,    3,  119,   62,  -72, -108,  -42,
   66,   51,  -97,   62,   -5,   68,   90,  -14,   45,   84,  -54,   17,  -71,  -70,   52,  -53,  -77,  -26,  -52,   96,   -2,  117,  117,   83,   -8,
   82,   45, -116,   23,  -35,   -2,  -77,   37,  -12, -125,    3,  -92,  114,   -2,  -85,   80,  -50,   15, -106,   65,   14,   41,  -13,   88,   81,
    0,  -43, -128,   21,   77,    8, -111,  -61,  -38,   10,  -56,   30,   -9,    2,    5,   24,   21,  101,   14,  -66,  -75,   32,   66,   11,  -58,
  -43, -107,  102,   97,   88,  -21,  -52,  -26, -116,    9,  -12,   86, -120,   21,   62,  -74,   43,  107, -102,   55,  109,   95,   74, -120,  101,
  -55,   40,  -34,  -52,   70, -126,   84, -126,  -90,  -47,  -61, -104,   -7,  -87,   15,  -16,  -48, -115, -113,   33,  -41, -105,   49,  127,   35,
   71,  -40,    2,   75,  -66,   -5,   39,   28,   32,   -2,  -14,  -44,  -47,  -76,    5,   17,  100,  -90, -114,   99,  -20,  113,   -3,  -20,  118,
  126,   82,    3,   68,   72,   23,   41,  -56,  126,   83,   88,  -97,   61,  -31,  -98,   63,  -16,  -13,   81,  -92,  -33,   59,  -64,  -92, -100,
 -119, -101,   24,  -31,  -17,   33,   98,  -84,   84,  -18, -104,    0,   65, -102,   75,   45,    6, -124,   35,   94,  124,  -14,   71,   76,   63,
  -22,   98, -124,  -79,  -40,  -44,   23,    1,   86,  -67,  124,   63,   91,  -74, -105,   24,   53,  124,   95,  -49,   83,  -70,  -29,  -21,  -73,
  -66,  -54,   19,   47,   -2,  -98,   99,  -90,  -49,   97,  116,  -81,   98,   75,   11,    8,  105,  -67, -123,  -24,  102,   -5,   32, -115,    7,
 -111,    5, -111,  -48,   56,  104,   48,   71,   30,   -1,    9, -114,   63,  -11,   51,  -28,  -13,   69,   16,   46,   15,   62,   54,    9,  113,
   98, -120,   77,   76,   37,   82,   68,   81,  -19,  -99,   21,  -14, -110,  109,   59,   98,   74, -107, -124,  114,  -59,   80, -119,   12,  113,
   18,   -5,  -13, -109,   90,   62,   61,  -42,   77,   59,  -12,   86, -115,   16,    6,   33,  109,   54,  118, -116,   41,   99,  108, -114,   80,
    9, -108,   50,   72,   52,   67,  -70,   36,   85,  -93,   73,   84,  -30,    4, -128,    1, -119,  -49, -113,   70,   12,  124,  -87,   71,   78,
  -98,  -55,  -67,   43,   -2,  -28,  -91,  -81,   63,   88,  -57,   50,  -89,   51,    7,  -45,  119,   76, -102,   25,  -30,  125,  -58,   94,  -77,
 -101,  -26, -106,  124,  102,   62,   71,  113,  -52, -101,  -81,   86,   80,   18, -110,  -60,   50,  -72,  -40,   10,  104,  118,  -11,   -9,  -15,
  -67,  -97,  -33,   78,   67,   62,  -49,    1,  -45, -121,  114,  -36,  113, -109,  -77,   54,   12,  105,  107,  -32,  -36,   19,   79,  -28,  -42,
    7,   31,  -50,  -40,  -24,  -52,   15, -106,  -11,   95,    2, -105,   68,  -18,   27,   41,   60,   45,  110,    5,   62,   20,   17,   56, -115,
   33,  -14,   32, -117,  -96,    7,   67,  -68,   19,   76,   21,   35,   34, -116, -116,   17, -119,  -81,   61,  101,  -42,  125,   72,   69,  -44,
  -79,  -47,  -23,  -96,   78, -103,  104,    0, -108,  -21, -117,   56,  -27,    4,  -10,    0,  113,  -38,   68,   91,  -49, -100,   -5,   62,  -77,
    6,  122,   54,  -11,  -15,  -12,  -49,   22,  -45,  -77,  -79,   15,   99,  106,   33, -114,  -65,   73,  -55,   58,  -16,   47,   23,   33,    0,
   43,   40,   25,  -36,  100,    4, -100,  -50,  -88,  -11,  -71,   59,  -55,   61,  -94,  -92,  -33,  -30,  -56,  -63,  -12,   90,  -87,  -64,  106,
    7,  -44,   84,  -56,  -23,  -48,  -13,    1,  -94,   38,   -4,  -68,   27, -108, -102,  -43,  -58,   88,  -92,  -48, -104,  -60,   34,  -76,  118,
 -103,   73,   -2,    4,   67,  -30,  -26,  -65,  -42,   21,   35,  -76,    7, -101,   96,  -32, -125,   73,  -84,  -48,  -50,  -67,  -87,   31,  -41,
   34,   61,   94,  -13,  107, -108,   10, -120, -124,  -12,   46,    1,  100,  -38,   17, -117,  -15,   25,   82,  -42,   38,   88, -109,  -72,  -55,
    4,   38,  -15,   73,   26,   14,  -96,   97, -112,   80,  -87,   14,  116,   75,  118,   -9,  -57, -108,  -85, -122,  -66,   82,   76,  -71, -110,
   80,  -87,   26,  -86, -111,   33, -114,   19, -116,    7,  -79,  -79,  -58,  107,   99, -125,  -75, -122,   -6,   -7,  -27,   82,   88, -112,   14,
  -84,   57,  -87,   64,   88,   70,  -12,   42,  -86,   66,   58, -112,  107,  -25,   10, -103,   44,  -50,  106,  -47,  -50, -124,  -84, -107,   65,
  -51,   57,    6,  -27,  -38,  -88, -108,  118,  -44,   98,  -23,  -87,  -10,  -96, -114, -119,  -10,  126,   34,  -44,  105,  103,   11,   31,   60,
   98,   22,   19,  -10,  106,  -55,  -22,  -21,  -35, -103,  -80,  121,   99,  -52, -104,  -79,   78,  -61,   78,  106, -101,    4,   44,  -50,   -6,
  -67,  -65,   98,   89,  -14,  -36,  -38,    1,    0,   14,  115, -102,   57,  -77,   15,  -29,   55,   15,   61, -103, -127,  118, -128,   48,   17,
   78, -102,   91,  -31,   38,   92,   35,   37,   66,   38,  -50,  -44,   81,   30,   36,   86,   -5,  -64,  -66,    7,  -92,   72,  -64,  116,   35,
   68,  -30,  -29,  106,   46,   54,  102,  -91, -105,   76,  126,   60,   24, -101,   32, -108,  114,   62,  -65,  112,   32,   21,   22,  -70,   74,
  -48, -100,   -9,   99,  -44,   -5,  -58,   66,  -63, -114,   62,  111,   41,  -54,  116,    8,  -43,   41, -116,   90,  -68,   29, -119,  119,   67,
   -3,   32,  -18,  -39,   82,  -94,  127,   87, -103,  -82,   94,  -53,  -98,  -17,  -32,   -3,   44,  -62,  103,  -94,  108,  -24, -121,  -47, -115,
  -44, -110,   31,  -92,  -12,  -15,  119,  103,  -71,  108,  -24,   75,  -83, -111,   20,  -32,  -46,   79,   47,  -44,   88,    5,    2, -127,   48,
  110,   37,   12,  -22,   65, -114,   -5,  -32,  -94,   22, -114,   43,  -79,  113,    5,   68, -120,   16,  -62,  -89,  -81,   38,  110,    2, -110,
 -105, -124,   -9,   63,  -70, -118,  -69,  -17,  123, -110,  -45,   79,   62, -126, -123,   11,   23,  -14,  -99,   31,   61, -117, -107,  112,  -43,
  -65, -100,  -57,  -15,  -57,  -72,   40,  -58,  -62, -123,   11,  -39,  127,  -38, -121,    1,   13,    2,  -66,  -10,  -51,  -13,    0, -105,    2,
   57,  118,  -60,   84, -112,   78,  -40, -113,  -18,   18,   52,  -49,   28,  -51,  -88,  -31,   99,   48,   73,   27,   79,   46,   93,  -23,   -3,
  119, -100,  -90,  -59,  114,  -24,   65,  -29,  -48,   58,   96,  -61,  -26,  -35,  -20,  -77,   87,   27,   65, -112,   35,    8,    2,  -86,   17,
  -12,  -11,   71,    4,   21,   69,  -96,  -96,   92,   29,   24, -126,  -19,  -22,  -83,   98,  -79,   20,  -14,  105,   92,   88,  -80,  109,   71,
   76,   20,   71,   68,   73,  -52,  -56,  -31,   69,   26,  -14,   69,  -86,  113, -107,  -88,   26,  -79,  109,   71,   23,  -85,   94,  -37, -115,
   16,    6,   41, -115,   19,   48,   38,  -64,   24, -119,   49,   17,   66,   43,   15,  -38,   52,  -68, -109,    0,   10,  -31,  -25,   28,   11,
   44,   26,   57,   48, -116,  -44,   48,   72,   49,  110,  -56,   56,  -74,  111,  -24,  113,  -99,   44,  -22,   52,   79,   26,   46,  -14,   89,
   74, -103,   89,  -21,   53,  -80, -110, -118,  -71,  103,   78,   39,  -48,   53,  -23,   -9,   -6,   43,  -67,  -20,  -18,  -18,   67,  -17,  -35,
 -118,   14,    4,  -37,  -53,  -69,   93,    0,   52,  -51,  112,    2,  110,  -70,   -3,   17,  -66,  -14,  -91,  -29,    6,   92,  119,  -62,  -15,
  123,   51,  113,  -56,   94,  -68,  -34,  -75,  -66,   78,   -6,  -41,  -35,   83,  -32,   95, -106,   15,  108,   11,  -31, -110,  112,  124,  -62,
 -124,   73,    4,   38,  -45,   22,    6,  107,  -53, -128,   51, -105,  -99,   92, -121,   68,    8,  -49,   80,   43,  -92, -127,   68, -128,  -80,
   17,   66,  -72,   73,  -38,  -46,  122,  -78,  -54,   63,   -5,  -58,  -35,   48,  -66,  -43,   -5,   42,    6,  118, -106,   28,  -96,  -21,   19,
   78,  -46,  100, -109,  116,  -80,  -42, -121, -105,   34,   92,   -5,  -45,  -13,  -93,    4, -110, -116,   -7,  122,   -1,   75,   77, -109,   66,
   79,   34,  -96,   12,   99,   27,  101,  102,   -6,   11,  -21,  -64,  -70,  -79,   31,  122,   99,  -25,   10,  -72,  103,  -11,   76, -108,  -49,
   57,   78,  -25,  -16,   90,   31,   28,  -75,   65,  -82,  -10,  123,  122,   51,   47,  -39,  108,   18,    1,   42,  -93,  -17,  -83,   77,  -65,
  -42, -103,  -41,   64, -112,  -81,   41, -123,  -75,   43,  -17,   98,  -30,    1,  103,  101,   29,  -71,   96,  -63,    2,  -26,  -49,  -97,  -49,
  -53,   47,   -3, -114, -111,   35,  -65,  -11, -106,   -2,  108,   63,  -28,   60,  -84, -128, -119,   83,  -58,  112,  -54, -100, -103,   76,  -38,
  119,   52,  -83,   67, -102,    0,   56,  -16, -111,   17,  -84,  -33,  -46,  -49,  125,   15,   47,  -13,  -17,   46,  -31,   51,   23,   28,    3,
  -64,  -54,  -43,  107,  -39,  111,  -33,    9,   89,   61,   59,  118,  -10,  112,  -53,  109,   75, -112,  106,   18,    0,  -27,  -14,  -64,   52,
  -70,  -82,  -98, -120,  -23,   -5,   13,  -94,  -91,  -55,   97,  -22,   -5,   63,   -4,   45,   91,  118,  -60, -116,   28,  -67,   47, -121,   28,
   48, -108,  -61,  -90, -113,  -96,  121,   80,   62,   59,  127,  -21,  -74,  -99,   92,  125,  -35,   26,  -74,  116,   25,  -33,   31,   10,  -95,
   44,   82,   40,   74, -107, -120,   81,  125, -118,  -78,  116, -122,  -73,   11, -123,  -92,   51,   97,   15,   92,    0,    0,   16,  -77,   73,
   68,   65,   84,  110, -123,  117,   46, -124,  -80,  111,   37,  -79,  -54,   81, -107, -103,   19,  -89,   34,   85, -104,   17,   49,   25,  -63,
 -111,  110,  -11,  -41,  -44,   -3,  -34,  126,  -48,   20,   14, -103,   54,   54,  107,   96,   28,  -59,   44,   95,  -38,   77,   75,   75, -111,
 -122,   65, -118,   32,   39,   88,  -37,  -19,    9,   50,   93,  -85,  -93,  107,  119,  -52,   51,   79,  111,   26,  -48,   25, -123,   98,  -56,
  -55,   39,  -17,  -25,  -22,   23,  117,   -9,  -51,    8,  -90,   28,   82,  -27,   28,  -79, -106, -110,   93,  -11,  -25,   90,   77,   82,  -79,
   94,    0,  -59,   32,  -86,   64,   25,   68,   21,   43,   98,   18,   32,  -74,  -54,  -83,  123,  -28,  -81,   49,   85,  -31,  -14,   82,   -3,
  125,  -84,  111,  103,   74,  102,   85,    9,  -40,  -44,   31,  -80,  -93,   28,  -48,   85,  117,  -28, -103,   84,   41,  -63,   51,  112,  -61,
   31,  -77,  -66,  111,  -54,    4,   68,  -44, -111,   91,   50,   32,  -10,  -77, -104,   -2, -106,   36,  -42,  -69,  -33,  114,   32,  115,   88,
   21,  -46, -109,  -28,  120,  -71,   39,   96,  109,  127,  -64, -122,   -2, -128,   55,   -5,    2,   86,  -11,    4,  -12,   36,   57, -124,  116,
   27,  -46, -111,  126,  -42, -109, -118,  -50,  109,  -47,   88, -107,  115,  -97,   81,  -40,   40,    1,  -85,   28,  -95, -105,   18, -125,   86,
   65,   98,  124,  102, -111,  -69,   38,  -19,   23,   68, -120,  -15,  -92,   96,   26,  -36,   95,  -10,  -14,  -58,  -38,  -32,   16, -126,  -39,
  -57,   28,  -60,  -20,  -93,   15,    0,   92,  110,   51,   56,   45,  108,  101,   58,   45, -107,   44,    9,  -61,  122, -110,  -14, -118,  107,
   62,  -50,  -95,   71,  -20,  -57,  -21,  107,   95,  -89,  -40,  -40,  -54,   -4,   -7,  -13, -103,  125,  -52,   44,   62,  113,  -10,  113, -116,
   29, -102,   -8,  101,  118,  106,   38,  121,  -41,  -50,   77, -116,   26,   55,    5,   33,    4, -117,   22,   45,  -94,  109,  112,   19,  -94,
  -70, -122,   21,   47,   62,   77,   95,  127, -103,   82,  -71, -106,  120,   63,  102,  -52,   24,   14,  -36,  -73, -127, -106,  -90, -128, -115,
   27,   55,  -47,  -34,  -34,  -50, -105,  -65,   56, -105,  106,   84,  101,  -81,  -15, -115,  -52,   62,  106,    2,  -51, -125,  -14,   52,  -75,
   12,   69,    8,   65,   71,   71,    7,  -61, -122,   14,  -26,   -5,  -33,   -7,    2,  -37,  -73,  -82,  -61,   -6,  -60,   13,  -91,    2,  -92,
  -50,   99,   76,   63,  111,   20,   32,   65,  -69,  -23, -103,   58,    4,   29,   98,    2,   71,  100,   89,  -19,  -41,  -10,  -38,  115,  -98,
  -24,  -29,  -99,  -53,  104,   63,   96,   50,  -61,    7, -115,  116,  -13,  114,   69,  -24,   88,   47,  -31,  126,   23,   34,   68, -118,   16,
   81,  127,  -99,   -1,  -19,   11,  103,  -51,  -90,  -40,   80,   51,   43,  118,  110, -115,  -39,  -67,   21,   10,   45, -102,  -58,   38, -119,
   53,  -48,  -73,   91,   32,  -92, -101,   47,  -22, -106,   45,    9,  -23, -113,   13, -113,   62,   55, -112, -115,   86,   74,  -14,  -59,  121,
   39,   32,  101,  -35,  -67,   68,  -35,   61, -107,  127,   16,  -23,   23, -102,  -77,   33,    4,  -82,  -67,  -42,  -97,   19,   -9,   27,  -97,
 -110,  105, -127,   24,  -31,   96, -117, -112, -106,  -60,    8,   98,  -17,   51,  -71,  -59,  -47,   52,   73,    9,  -41,   38,  -23,   58,   76,
  -90,  -49,  -85,  -36,  103,  -87,   66,   42,   38,   36,   17,  -82,   -3,  -46,   63, -121, -110,   33,   18,  127, -114,  -17,   31,  -87,  106,
  125, -107,   16,   82,   73,  106,  117,    9,  -65,  -59, -124,  -60,   98,   96,   -1,  -65,  -97, -101, -112,  -95,   99,  -47,   83,   33,  -24,
   23,  126,   43, -103, -128,   30,   19,  -48,  -17,  -25,  106,   59,   51,   57,  -56,   -6,   94,    8,   93,  -57,  -68,  107,    4,   62,  -93,
  -58,  -81,   63,   37,  -92,   70,   36,    6,   27,  -59,  -32,   55,  107,   12,   86,  -44,   64, -115,  -81,  -49,   42,  127,  -66,  114,  108,
  -74,  -75,   14,  -40,   55,  -33,  -10,   44,  -32,  -46,   32,   15,  -99,  113, -112,    3,   -2,  -37, -108,   23, -105,  -35,  -54, -100,  -77,
   14,    7,   92,  118,   85,  107,  -53,  120,  -84,  -46, -100,   60,  -25,  -16,   12,  -44,  115,   62,  116,   58,  -59,  -58,  102,  118,   38,
    7,  -16,  -43,  111,  -70,  -76, -110,  -77,   78,   61, -110,  117,  107,   94,  -16,   41,  -70,  100,  -41,   99,   45, -105,  125,  -25,   46,
   94,  124,  -45,   89,    0,   95,   -8,   -4,  103,  121,  122,  -15,   61,  -12,  -11, -105,   40,   85,  106,    1,   -4, -123,   11,   23,   50,
  -72,   37,  -57,  115,  -49,  -65,  -64, -121,   63,  124,   22,  -85,   95,  121, -125, -117,   47,   -1,   57,  -93,  -58,  -18,  -53, -101,  -21,
  -69,   -8,   -2, -113,   -2,   47,  -13,  -26,  -51,  -93,  -73,  123,   59,   95,   -4,  -58,  111,  120,  113,   77,  109,  -63,  -61,   -5,  127,
  123,   45,   82,  -71,    5,   15,  -92,   84,    4,   65,   30,  -93,   36,   35,  -74,    7, -104,   62, -127, -116,  -64,   24, -123,    8, -108,
  103,   -6,   93,  -76,   64,  -28,   66,  -28,    0,  -51,   42,    3,  -70,  119,   89,   70,   79,    8, -103,   55,  -29,  104,    2,  -14,   78,
   51,  -39,   90,   40,   41,  125,   81,   54,   13, -111,  120, -115,  115,  -32,   -8,   73,   28,   57,  -85, -106,   92,   14,  -80,   98,   89,
   47,   -7,  -68,  100,  -24,   16,   73,  -21,   80,  -59, -101, -101,  -69,   88,   -6,  -46,  -26,   90,  120,  -90,  -18,  -66,  -73,  -36,  -77,
  -68,  110,  -59,   11,   87,   70, -113,  -49,   49,  107,  -33,   -3,  -78,  123, -120,   52,   52,   34,    2,  108,  -86,    1,   82, -115,   33,
    3,  -80,   94,  -62,   -5,   61,   70,   81,  -19,   77,   48,  -11,   76, -110,   -9,  -37,   34,   41, -119,  108,   74,  114,    5,   68,  125,
  -72,   68,  -22,   52,  124,  -28,  -97,   47,  -45,  -80,  -66,  -67, -119,   13,   40,  -59,   46, -124,   36,  -91,  -37,  -46,  -10,   73,  -65,
  -91,  125,   36, -107,  -45,  -78,  125,   73,   64,  -20,  -49,  -49,   -6, -102, -128,  -40,  120,   13,   44,   -1,   62,  -74,   90,  -24,   40,
  125,  126,   23,  102,  -77,  -54,  -11,  -75,  -83,  -21, -105,  122,  -85,  -56,  -88,   28,  110,  109,  -76, -100,    7,  109,  -51,  -54,  -80,
   34,  -75,  -28,  -62,  -20,   26,   39,  100,  -45,  -80,  -96,   11,   41,  -91,   44,  -71,   32,  -64,  -94,   93,   88,  -50, -125,  -33,   93,
  -17,    0,   59,  118,  -20,   88, -122,  -76,    6,   84,   -6,   93,  -98,  -12, -126,    5,   11,  -24,  -24,  -24,  -32,   -7,   23,   94,  -11,
  -17,   87,   50,  117,   74,  -51,   10, -100,  -76,   -9,   49,   88,  -85,  -39,  123,  -54,  104,    0,  -42,  -81,   95,   79,  -75,   63,  -32,
 -116,   67,   63,   69,  108,  -85,  108,  -22,  113,   51, -102,  -38,  -37,  -37,   89,  -75,  -20,   97, -112,    3,    5,  -61,  -63,  -77,   62,
   72, -100,   68,  -60,   81,   13,  -84, -113,   62,  -16,  107,   74, -107,   50, -107,   74,   13, -124,  -19,  -19,  -19,    0,  124,   -2,  115,
  -97,   97,  -13,  -74,  126,   62,   -6,   63,   -2, -107, -111,   99,   -9,  -91,   26,   85,  -39,  -39,  -35,  -53,  -70, -115,   85,   42,  106,
   60,  -13,   47,  -65, -114, -113,  -49,  107,  103,  -10,   81,   83,    7,  -36,    7,  -31,  -42,  -38,   10, -125,   16,   99,  -53, -116,  -39,
  -31,   66, -110, -119,   80, -108,   42, -110,   74,   63,  -48,    3,  -70,   10,  -86,  -20,   54,  -47,   11,  122,  -32,  -28,  110,  -72,  -29,
  -39,  101,   92,   -6,  -47,  -93,   56,  -14,  -48, -119,  -84,  -39,  118,   56,   15,  108,   88,  -22,  -78,   47,   69,   45,  125,  -80,   54,
  -37, -128,   44, -100,  116,  -58,   49,   83,   25,   58,  -94, -104,  -43,  -77,  -69,  -73,  -54,  -26,  -75,   17,  -61,  -57,   22,  105,   29,
   30,   50,  104,  104,  -64,   79,   -2,  109,   37,   61,  113, -107,   -6,  101,  124,   82,  -74,  118,  -29,  -82,   94,   86,  -66,  -80, -117,
  -87,  -45,    7,  103,  -65,  -27,  -62, -112,   79,  -98,   57, -109,  -89,   22,  -84,   27,  -24,    7,   11,   16,   40,   47,   84, -116,   39,
   31,    4, -119,  -88,  101,   80,  -91,  -59,   86,  -96,  -78,   43,   65,   55,  104,   84,   62,   93,   95,  -47, -107,  -56,   66,  127,  127,
 -116,  -82,   36, -104,   72,   56,   32,  122,   -1,   94,  122,   63,   55,  -95,   70,  -32,   73,   -1,   -4,   85,  -64,   70, -112,  -41, -112,
   -9,    4, -106,   28,  -40,   60,   16,  -48,  111,  -96,   26, -125,   73,  -88,   17, -126,   -2,   -9, -118,   -3,   59, -104,  -56,  -80,  103,
  -55,  -62,   93,   53,  -97,  -42,   31,  118,   41,  -94,   41,  117, -100,   -6,   -9,  -23,  -70,  101, -120,   44,   61,  -44,  -91,  -85,  -70,
   -9,   99,  -84,  112,  -92,  -97,  -65,   70,   -6,   -6,  -84,  -92,  110,   49,   59,   81,   35, -109,  -83,   -9,  123,  -91,   24,  -72,  -64,
  -87,    5, -125,  -30,  -66,   71,   95,  -31,  -28,  -93,   39,   99,  -30,   18,  -41,  127,   -9,   34,  -64, -103,  -49,  -79, -102,  -60,  -10,
   93,   85,  -26,  -50,  -99,   75, -104,  107,    2,    4,   27,   54,  108,  -90,  -77,  -77,   19, -104,  -22,  -29,  -12,   53,  -45,  -72,   88,
  108,   37,  -52,   15,  -95,   34,  -35,  106,   43,  105,  -39,  -66,   -7,   13,  -52,   30,  -85,  -76,   38,  -58,   18,   39,  -47,  -64, -100,
  124,  -96,   90,  -87,   16,  -59,   53,   31,  -72,  -93,  -93, -125,  -10,  -10,  118,   22,   44,   88,  -64,   19,  -49,  110,   37,   22,   67,
  -87,   84,  -54,   36,   73,   68,   67,   94,  113,  -31,  -25,   78,  101,   -4, -104, -113,    1,  -80,   -6,  -43,   78,  118,  -19,  -40,   12,
  -29, -121,  -43,   42,   84,   10, -127,   69,    9,   69,  -59,  -60,   84, -115,  -62,    8,   72, -124,  -14,  -63,   39,   75,  -65, -123,   56,
  118,  -53,  -43,   70,  -62,   69,  117,  -34,  118,   50,  -61,  -65,  -35,   -9,   52,   95, -104,  -45,  -50,  -71,  -27,   67,   80, -113,  107,
   30,  -35,  -78, -100,   88,   68,  -39,   11,   30,   16,  -93,   54,   16,  106,  -51,  -66,   35,  -58,  -15,   -4,   19,  -35,  -40,  -60,  -27,
  107, -106,  122,   34,    2,   37,   25,   49,   46,   96,  -28, -124, -128,   77,  -37,  123,   -7,  -27,   31,   87,   56,   83,  109,  -49,   27,
   74,  -24, -115,   12,   51,   62,  117, -109,  123,  -55,  117,    4, -112,  -79,    6,  -85,  -21, -109,  -19,  -35,   -7,    2,  -27,   77,   45,
  -25,  -36,   91,   97,   48,   62,   67,  -54,  -42, -109,   93,    0,    9,  -60,   61,   16,   -9,   27,  -92,  -10,  104,   51,   80, -119,   12,
   38, -128,  -26,   80,  103,  -15,  -55,   52,  -18,  -99,  -78,  -46,  -54,  -42,  -27,   42,  -37,  116, -112,   58,   96,   -9,   37,  110,  -30,
 -122,   86,   16,  -56,   26,  112,  -83,  117,  -57, -109,   84,   24,  -24,   26,  -80,   83, -126,  -85, -100,  -72,   57,  -63,  127,   47,    4,
   86,   86,   36,  -40,  -70,   28,   86,   33, -124, -101,   97,  -27,  -83,   24,  -21,    1,   43,  -22,   37,   22, -112,   45,  -29,  106,   29,
    0,   29,   91,  -19,  102,  -40,  -92,   97,   40,  -21, -119,   65,   43,  -45,  -55,  -23,  -11,   35,   65, -112,  -59,  -39,  -21,   98,  -64,
   86,  -72,  112, -121,  -80, -106,   75,  -81,  -68, -113, -109, -113,  -98,  -52,  -98,  -91,  -79,  117,   18,   74,   57,  114,  -88,   90,  -23,
  -95,  107,  -41,   27,   44,  125,   22, -106,   45,   91,  -51, -116,  -61,   62,   64,   34,  124,   40,  -91,  -18,   33,  -83,   10,   24,  -74,
   89,  -80,  107,  -12,  -64,   23,   48,  121,  -37,  -64,  -17,    6,  -31,   22, -112,  -40,   35,   78,   95, -119,   -6, -119,  -29, -102,    6,
  -98,   55,  111,   30,  119,  -36,  113,   55,  -19,  -19,  -19,  -20,  -40,  121,   31,  119,   60,  -80, -108,  -47,   99,   39,   19, -101, -120,
  -77,  -49,   56, -126,  -15,   99,    6,  103,  -25,  117,  116,  116,  112,  -46,   41,  115,   -8,  -23, -113,  127, -104,   93,   63,  105,  107,
   64,  -25,   72,   55,  126,   71,  109,   22,  -12,   43, -101,  -14,   86,  -40,  -60,   69,   78,  -84,  112,   73,   36,   86,  -38, -116,   80,
  126,   11,   11,   13,  112,  -13,   67,   47,  -15, -111,   99,   15,   96,  -30,  -44,   65, -100,   45,  -90,   51,  122,   -7,   96,   30,  -34,
  -72, -126,   77,  -91,  -99,  -60,   54,  113,  108,   93, -102,   -8,   32,   28,  -69,  122,  -47,  -11,  -73,  -47,   22,   52,  -79,   -1,  -32,
  -79,  -52, -100,  -80,   23,   83,   39,  -73,   48,  122,   98, -114,   49,   19,    2,   26, -102,   21,   87,  127,   -1,   37,  -34,  -36,  -34,
 -121,  -48,  111,  -67,   95,   -6,   -2, -116,  117,  -45,   -8,  -96,    6,  -42,  116,   17,  -77,   61, -109,   57,    4,   10,   41,    3, -108,
   74,   99,   51,   22,  -21,   57, -123,   44, -119,   98,  -49,   98,  -64,   12, -104,  -16,  -81,  -88,   24,   69,  -55,  -72,   73,  -10,  105,
   68,   78,  120,   82,   48,  -99,  -57,  -85,  -20,  -64, -119,   -2,  -62,  -41,    5,  126,   41,  119,  -21,    0,  -23,    9,  -62,  108,   58,
  -98,   80,   78,  -23,   10,  -33,  -98,   52, -100,   90,   74,  -96, -110,  -90,  -91,  -66,   69, -102,  -67,  -65,  -59,  -54,   58,  -51,  -22,
  -63,   38,   72,  -29,  -36,  -23,   75,    1, -124,  116, -127, -124, -116,  110,   39,  -45,  -50,   78, -127,  -89,  -55,   29,  126,   18, -119,
   15,   51,  -43,   86,   88,  -15,   43,   91,   90,  -78, -124, -114,  -84,   30,  -46,  -70,  -84,   79,   63,  112,   66,   32,   93,   80,   33,
  -99,  -26,   55,  111,  -34,   60,   22,   45,   90,  -60,  -92, -125,   63,  -59,  -41,  -81,  126, -112, -114,  -93,   38,   49,  -41,  -81,   48,
  114,  -25,   93,   15,  -48,   60,  120,   18,  -83,   67,  -10,    5,    1,  -85,   95,  -35,   70,   -5,   17,   19,   25,   51,  102,   12,  -61,
   71,   12,   35,  -79, -118,  -78,   16, -100,   50,  126,    2,   80,   35,  -68,  -30,   61,  -14,   34,   12, -126,  -60,  -42,   61,  -69,   47,
  110,  -63,  -70, -102,    6,   94,  -65,  126,   61,  -99,   91,   99,   14,    1,   78,   59,  -11,  100,  -74,  -19,  -24,  -30, -107,  -51,   17,
  -42, -104,    1,  -32,   93,  -76,  104,   17,  -77,  -49,  -70, -104,   57,  -13,   62,   62,  -96,  -66,  -50,   49,   10,  101,   97,  -36,  102,
  -53,   54,    4,  -79,  -75,   36,  -62, -111,  -49,   70,   73,  -81,   52,  -83,  -41, -103,   22,   99,   28,  -65,  -13, -106,   76,  -84,  -76,
  124,  -27, -122,  -57,  -71,  -19,   59,  -89,  -80,  -41,    1,  121,  116,  113,    2,  123,  -81,   27,  -58,   43,   27,  -74,  -15,  -46, -114,
   77,  108,   46,  119,   83,   77,   34,   12, -106, -100,    8,  104,  -47,   13, -116,  105,  104,  101,  -54, -120,   97,  -20,   51,  -78, -103,
   17,  -29,  114,   12,   25, -109,   99,  -52,   56,   77,  -21,  -56, -128,   31,   -2,  102,   57,  -41,  -33,  -77,  -52,   51, -109,  111,   95,
   68,  -10,   31,  -39,   68, -128,  116,  -32,  103,  -90,  -70,  -73,    0,   28, -128,  -91,   19,    6,   42,   65,   89,   69,  100,  -36,   36,
 -124,  -28,   93,  106,  -75, -118,    1,  -19,  -75,  -88, -106,   78,   67,   24,  -97, -127,   37,  113,  123,   43,   28,  -88,  -77,   -7,  -62,
 -128,  -83,  -49,   19,  -16,  -93,   47,  -51,  -74,  -54,  -78,  -82,  -22,  -52,   -2,   52,  -55,   35,   50,   46,  127,  -72,   98,  -34,   93,
   59,   -1,  -37,   74,  -26,   30, -119,   44, -125,   12,  -22,   64,  -25,    5,  -87,   33,   93,  105,    2,  -36, -102,  -35,   56,   33,   44,
  -22,    4,  -82,  112,   11,  -16,   43,  111,   35,   11, -124,   79,  -14,  -88, -123, -116,   28,  -80,  -45,  123, -120,   76,   10,  -70,   89,
   93,   46,   62,  105,  125,  108,   90,   88,  -55,  -78, -107,   91, -104,  -66,  -33,  112,   58,   59,   59,  -23,  -20,  -20,  100,  -81,   25,
  -97,  -95,  101,  -44,   76,   12,  -16,  -30,  -86,  -83,   76, -101,  -30,  -52,  -46,   71,   31,  -71,   23, -124,   32,   17,  110, -127,  -37,
   59,  127,  -65, -102,   79,  124,  -30,  112,  -38,  -37,  -37,  -71,  -32,  -62,  -77,   -8,  -45,   -3,  -67,  -52,   56,  104,   50,  -13,  -66,
  120,   52,  -32,    8,  -85,   49,   67,   -9,  -15,  -38,  -70,  -82,    8, -119,  -63,   45,  -92,   88,   95,   42, -119,   37,  -39, -125,  -77,
  121,   -6,  -91,  109,  -20,  -77,  -41,   26,  -10,  -97,   50, -119,   11,   47,   56, -121,   67, -113, -100,  -61,    7,   78,   -7,  100,  -10,
   -5,  -84,   89,  -77,  -40,  101,  -58,  114,  -20,    9,  -89,  112,  -38,  -15,    7,   15,  -72,   86,  -85,    0,   99,   97,  -89,  -75,   24,
  -23,  -64,  107, -115,  117,  -85,  -57,   26, -105,   -5, -100,    0,   70,   90,    7,   96,  -36,  -12,  -62,   63,   59,  -95,  127,  -11,  -74,
   46,  -50,  -66,  -14,  126,   22,  125,  -21,   36,    6,   53,  -27,  105,  107,   83, -116,   27,   95,  100,  -26, -114,   81,  -12,  -11,  -59,
  -12, -106,  -86, -104,  -60,   18,    4, -118, -106,   98,   72,  -82,   73,  -45,  -38,  -90,   24,  -36,  -90,   24,   60,   60,  -96,  109,   84,
 -120,   21, -106,   27,   -2,   99,   21,   87,   47,  122,  -34,  -79, -118,  105,  -27,  -11,  -87,  -46,  117,   -3,   82,  -57,  -32,   15,    0,
   52,  -78, -106,  -47, -108,  -18,   93,   -2,  -84,   68,   42, -127, -113,   -5, -109,  -55,  -61,  -73,   39,   40,   -1,  108,   41,   27,   80,
    6,   26,  -75,   19,   30,  -46,  -42,   45,   86, -128,  -37,   11,  -19,  -83,   14,  -33,  -34,   -6,  119,   87,  -81, -124,   50,   97,   83,
  -25,   70,  -90,   19,    2, -116, -123,   -2,  -40,  -35,  -17,  -35,  -74,  -15,  -65,  -85,  -72,  -66,  117,  -64,    2,  111,  -10,   82,  115,
  107,  -84,  112,   92, -125,    3,  -81,   -9,   85,   -3,  -77,  -92,  126,  112,    6,  116,    0,  -23,   76,  -16,  -44,   47,  -58,  107,  102,
   81,   55,  -17,   56, -117, -101,  -41,   11,   61,  -84,  103,  -87,   73,  -83,  107,  -84, -124,   95,  -36,  -66, -110,  -23,   -5,   13,  103,
  -55, -110,   37,   60,  -69,  -20,   21,  -10,   58,  116,  -74,   51,  117,    4,  -36,  122,  -41,  -54,   12,  -64,  -41,   94,  123,   45,   -5,
   76,   59,   31,   68, -128,  -15,   62,   -6,  -35,  -65,   95,  -59,  -23,  -89,   77,  -31,  115,  -97,  -69, -120, -119,  123,   61,  -50,  -52,
 -103,   51,    0,  -89,  125,   59,   58,   58, -104,  124,  -28,   92,  -12,   30,   47,  102,   -8,   46,  -55,  -28,   77, -110,   87,   70,   15,
  -20,  -89,  -60,   64,   50,   96,  -75,   12,  -88,   26,  -63,  -77,  -85,  123,  -39,  -33,  -81,  -98,   -4, -127, -103,   19, -119,   45,   60,
  -76,  -28,   53, -114,  -97,  -75,   55,  -13,  -25,  -49,  -25,  -32,   -3,  -97,  -32, -112,  -61,  102,   12,  -72,  -18, -110,   75,   46,   97,
  -73, -110,   12,   93,    7,  125,   88,   98,  -84,   -9,  127,  -99,   66,   74,  -92,  116,  -42,  -87,  117,   90,  -40,  -72,   44,   14,   16,
  127,  -58, -124,   78,  -53, -118,  -11,   61, -100,   -6, -107,   63,  -14,  -19,  127,   58, -104,  -29,   15,   29,   75,   -9,  -74, -120,   93,
   93,   33, -107,  -34, -124,  -40,  -25, -127,   10,   37,    8,   67,   40,   52,  -70,   80,   81,  -13,   16,   39,   16,   54,  -68,   94,  -26,
  -58,  123,   94,  -28,  -26,  -59,   43,  -67,  111,  -23,  -18, -109, -111,   96,  117,  -91,   30,  -72,  -75, -127,  -30,  -10,    3,  -76,  113,
  -10,  -62,    5,   82,    6,   78,   11,   74,   32,   17,   84,   77,  -51,  -52,  125,  -73,  102,  105,  -22,  -49, -118,   24,   26,   67,   55,
 -128, -107,    7,  -81,  -15,  126,  -84,  -75,  -75,   -4, -107,    1,  -29,  108,   96,  -77,  107,   41, -126,  -98,  -12,   73,  -37,  109,  -83,
 -101,   54,  -40, -105,  -72,   23,  -13,   -9,  102,   58,  -89,   37,   77,  -26,   72,   53,  108,   10,   72,  -25, -125,  -71,  115,  -92,  -41,
  -96,  117,  -74,  -75,  -21, -101, -108,  124,   18,   46,   99,  -53,   -1,  113,    6,   82,    4,  102,  -62,   55,   37,  -57,   36,  -75, -125,
   64,  -22,  -13,   58,  -95,   32,  -78,    4,   18,  119,  111,   55,  104,   54,  -19,   44,  115,   -9,    3,  -49,  -45,  -47,  -47,   65,  113,
  -16,   62,   20,   91,  -89,  -70,   -5,   11,  -40,  -68,  -77,  -62,  -67,   15,   62,  -49,  -18,   93,  110, -114,  -14,  -72,  -55,  -89, -111,
   -8,    1, -108,   32,   -8,   -9,   95,   45,  -25,  -46,   47,  127, -123,  -45,   78,  -39, -121,  -10,  -10,  118,   22,  119,   44,  -31, -100,
  -77,  -49,  -90,  -81,  127,    7,  -93, -122,   78,  102,   -8,  -72,  -87,  108,  108,   10,   88,  -15,  -52,    6,   94,   94,  -69, -104,  -50,
  -50,   78,  -58,   21,   38,   80,  -75, -110,   73,   27,  -32, -107,  103,   55,  -14,  -62,  -21,   79,    2,   16,   89,  -63,  -74,  -82,   50,
 -113,   45,   89,  -50, -106,   78,  -73,  -16,   65,  108,  -31,  -71, -105,   55,  -79,  -20,  -39,   14,  -38,  -89,   13,  -95,  -67,  -67,  -99,
    7,   59,   86,  112,  -37, -125, -126,   85, -113,   63,  -54,   48,   55,  -59, -117,  -59, -117,   59,   56,   -5, -100,  -77, -103,   58,  109,
   31,  -26,  -49,  119,  -13, -110,  -33,   -8,  -29,  -67,  -28,   -9,   59, -125,  -56,   24,  -65,  100,   20,   36,  -62, -103,  -55,   73,  106,
   58,   91,   75,   34,  -15,   41, -103,  110,  108,   10,   14,  -71,  -15,   63,  -25,   66,   37, -100,  -37,  -66,   55,   95, -102,  119,   16,
   -5,  -20,  -37,  -24,  102,   93,   68, -106,  -92,  -22,   84,  -87,   49, -114,   17, -117,   34,   75,  111, -105,  -31,  -73, -113,  -67,  -54,
   47,   59,   86,  -79,  -91,  -89,  127,   15,   74,   -8,   47,  -36,  -29,  109,   64,   -4,   22,  -16,   -6,  -67, -110, -110,   80,  106,   55,
   87,   89,   68,  -60,   37,   75,  119,   25,   74,   21,  -34,  -11,  -86, -108,  -11,   69,    9,  -89, -123,  -45,   73,   -5, -103,    6,  -90,
    6,  100,   24,  -88,  125,  -77,   38,  -89,  -19,   75,  -39,  -38,  -70,  118,   39,    6,   -6,  -85,  -80,   59,   -2,   -5,   89,  -44,   -3,
   47,   22,  -31,   12,  -34,   -6,  -11,  -67,   82,   95,   87,   10,   -9,   57,  -51,   33, -106,  -98,  -79,  118,  127,  -26,   68, -112,  -90,
   40,   11,  -17,  -29,  -90,   56,  -49,   -2,   22,   18,  117,   22, -117,  116,  -28,   88,  -51,   76,   23,  -23,  -19,  107,   -9,   76,   83,
   84,  -21,   -2,   26,   67,   -1,  -50,  -41,  -40,  -11,  -58,  -67,   20,   91,   -9,  102,  -40,  -34,  -89,  -96,  108,   58,  -99,   16,   -6,
  -69,   94,   99,  -45,  107,   -9,   32, -127,   -3,   15,   -1,   18,   74,  -72,   41, -123,  -39,   30,  -48,   82,  -16,  -38,  -22,  123,  -40,
  -79,  -29,   21,    4,  112,  -62,   81,   95,   70,   75,   65, -117, -126,  -19,   99,  -35,  -35,   59,  126,  121,   13,    9,  -48,  -66,   -1,
   25, -116,   27,   54,  -59,   77,   79,   20, -126,  -33,   60,  -12,   61, -124, -128,   79, -100,  112,   25,   66,   74,  -42,  109,   89,  -59,
   93,  -85,  -17,    0,  -32,  -92,  115,  -66, -122,   16,   48,  121,   35,  -68,  -79,  -23,  101,   -2,  -12,  -62,  -99,    0,   28,   57,  -19,
   76,  -58,   14, -101,  -52,   35,  -53,  -17,  -94,  115,  -53,   42,   12,  110,  -84,   29,  -15,  -79,  -81,  -80,  101,   -3,  106,   86,   63,
  124,   23,  109,   67,   38,   51,  109,  -65,  -45, -119,  -79,   78,  -21,   90, -120, -115,   33,  -74,  110,  -38,   96,    2,  -60,  126, -127,
  119, -105,  -87,  -19,  -93,   27,  -17,    8,  -64, -128,   82,  -96,  -14,  -80,  -41, -112,   38,   78,   -5, -121,  113,  -76,  -74,  -26, -104,
  -42,   56, -110,   65,   58,   96,  117,  -41,   78,  -42,   87,  -70,  120,  -29,  -43,  126,  -18,  127,   97,   45,  125,   73,   45,  102, -106,
   46,  122,   -9,   23,  -63, -101, -106,   61, -128, -101,   29,   18,    3,  127,   15, -123,   34,   12,   53,   58,   52,   40,   19,  -45,  -33,
  103,  -39,  -35,  -17,   66,   54,  -17,  -75,   40,    1,    5,  -19,   72,  -83,   92,  106, -100,  -40,    1,  -69,  108, -126,  127,  125,   25,
    0,  -38,  -70,   15, -107,  -56, -111,   86,  -91,   -1,   95,  -64,   11, -103,  -49,   46,  -22,   82,   31,  -33,   22,  -64, -106,  -70,  -65,
   44,   88,    3,  105,  106,   96,  -55,  122,   64,  -89,  -32,   20,    3,    5,  115,   10,  -46, -102,  -94,  -82,   75,  -73,  -12,  -41,   59,
    2,   80,  102,  125,  -86,  -88,  -69, -105,  112,   66,   64,  121, -127,  -94,  -68,  -55,  -81,  125,   59,  -46,  -17,   74,   56,    0,  106,
 -103,   46,  -42,  -17,  -38,  -82,   -4,   53,    1,    2,   45,    5,   77,   18,  118, -115,  115,  -19, -104,  -80,   17,  122,   18,  119,   63,
  -27,  -37,  -90,  -91,  107,  115,   58,  -57,   91,  -92,   15,   69,  -22,  -66,  -37,  108,  -11,   22,  -64, -103,  -61,  -58,  105,  -51,  -40,
   31,  111, -110,  -80,  102,   20, -128,  101,   72,   39,  -12,   24,    7,  -50,    1,  -32,  -75,  -58,  -19,  -45,   99,  -58,  -19,   99,  -31,
  -10,  -42,   79,  -92,  121,  -57,    0,    6, -105,   51,   17,    4,  -18,  -49,  -85,  -32, -105,  -12,  -88,   95,  -91, -125,  -70,   -3,  -98,
 -107, -102,  119,  112, -105,    1,   81,    5,  -34,   30,  -68,   66,   64,   32,    2,  114, -127,   32,   40,   36, -104,  -35,    9,  -69,   -5,
  -95,  -73,  -62,   59,   19,   18,  -17,  -80, -124,  -46,  -83,   56,   25,   42,  -56,    9,  106,  -93,  -14,  -99,   88,   17,  -58,  -59,  120,
  -85,    9, -108,   34,  -88,  -66,  -81,  107,   94,   -3,   23,  -54,  -98,    0,   22,   53,   22,  -74,   30,  -64, -112,  106,   88, -103,  -59,
  -62,  -83,  -97,  -90,  -23,   38,  -42,  -44,    0, -100,  -26,   65,  -41,    3,   24,   -2, -116,   22,  -10,  -41,  -41,    3,  -40,  -35,  -69,
  -90, -123,  -87,  -45,  -70,    8,    6,  104,   97,  -27, -103,  115,  -19,    1,  -90, -123,   35,  -39,  -36,  -33,   39,  116,  -25,  105,   33,
   92, -126,  -91,   95,  108,  -96,   30,  -60,   13,    2, -122,    5,  -16,   -6,   40,   65,   12, -116,  -37,  -32, -120,  -57,  -76,   78,  -23,
   59,   73,  -43, -115,  -55,  -20,  121,  -46,  104, -123,   39,   79,   12,  100,   38,  113,   81,  -62,  -38,   81,  -34,  -21,  -61,  -46,   -6,
   38, -108,  -20,  -37, -125,  -41,   90, -120,  112,   26,   55,  -78,   54,   91,  -56,   61,  -15,  117,   38,  126,  -55,  -30,   -1,    7,  -37,
  -28,   86,  -79,   76,   15,   60,   18,    0,    0,    0,    0,   73,   69,   78,   68,  -82,   66,   96, -126};

 private static final byte[] imageData_large = {
 -119,   80,   78,   71,   13,   10,   26,   10,    0,    0,    0,   13,   73,   72,   68,   82,    0,    0,    1, -112,    0,    0,    0,   32,    8,
    2,    0,    0,    0,  -77,   96,  -71,   90,    0,    0,    0,    6,   98,   75,   71,   68,    0,    0,    0,   51,    0, -105,  -79,    5,  -26,
   81,    0,    0,    0,    9,  112,   72,   89,  115,    0,    0,   11,   18,    0,    0,   11,   18,    1,  -46,  -35,  126,   -4,    0,    0,    0,
    7,  116,   73,   77,   69,    7,  -46,    1,   21,   10,   53,   23,  -27,  -16,   91,   62,    0,    0,   32,    0,   73,   68,   65,   84,  120,
 -100,  -27,  -67,  121, -100,   94,   69, -107,   62,   -2, -100,   83,   85,   -9,  125,  123,   73,  103,  -33,   23,   18,  -78, -121,  -80,   35,
  -48,   44,   51,  -56,   34,  -69, -126, -104,  -96,  -94,  -30,   50,   40,  -93,   51,   34, -115,   -8,   69,   20,   70,  -65, -118, -118,   14,
   68,   71,   68,   63,  -29,  -82,   48,   74, -126,  -29,   10,   72,   28,   54,   21,   58,   32, -124, -124,   32,   17,   72,   66,   72,   66,
  -56,  -66,  118,  -70,   -5,  125,  -17,  -83,  115,  126,  127,   84,  -43,  125,  -33,  -58,  -96,   -2,  -58,  -15,   51,   95,  -91,  -46,  -23,
  -82,   -9,  -66,   -9,  -42,  -83,   91,  -73,  -22,  -87,  115,  -98,  115,  -22,   20,   25,   12,   33,   56,  -60,  -92,    0,   20, -102,  -14,
  101,  102,  -65,  -33,  -30,  101,   50,   47,  -55,  -65,   36,  -47,   -2,  -14,    3,   50,   20,   63,   18,   17,  -84, -125,  -53,  -40,   89,
  -25,  -67, -109,  -68,  -46,  -97,  -25,  -30,   69,   80,  103,    0,   80,    2,   20,  -71,  -94,   80,  -28, -118,   28, -112, -105,  -65,  -17,
  -97,  -97,   24,  -88,    0,   85,   80,   21,  -88,    2,  121,  106,   34,   29,  -40,   86,   41,  -93,   41,   67,   72,   -7, -105,   79,   68,
   80,  -60,  118,   32,   74,   13,  -14, -110,   76,   -7,  -29, -128,  126,  104,   63,  -48,   15,  -44,   -2,  -62,   79,   -3,  -41, -108, -120,
  -88,  -15,  103,  -32,  -17, -127,    7,  -54,  108,  108,   97,   98,   34,   34,   71,   70, -119, -103,   13,    1,   80,   34,    2,  -89,  -77,
 -108,   72,   13,  -84,   39,  111,   97, -123,  -68,  105,   20,   52,  -80,  124,   34,    2, -127,  -29,   -5,    4, -104,   56,  -68,   -4,  -14,
   55,   19,    3,  -15,  -59, -122,  -68,    2,  -31,   42,   80,   60,   70,   96,   16,  -89,   15,   12, -128, -103,   21,   76,  -60,   76,   12,
  102, -128,    1,   24,   54,   10,   98,   54,   32,    2,   89,   34,   34,   50,   68,    6,  -60,   68,    6,  100, -104,    9,   48,  -52,   12,
   50,   76, -106,   76, -107,  -35,  112,  -78,   35,    9,  117,  -42,  -67,  -34,   -9, -128,   12, -111,   37,  102,  107,   28,  -40,   26,  118,
  108,  -40,  -80,   51,  -42,   24,  -29, -116,  -75,  -58,   58,  107, -116,   53,  -50,   88,  -29,  -84,   99,  -53,  -50,   56,  -74,  108, -115,
  -75,  -58,   48, -125,   13,   51,   25,   98,   50,   68,   74,  -60,    4,   81, -112,  -86,   87,   85,   81,   81,   47,   94,   68,   80,  120,
   95,   -8,   66,  -67,  -26,   69,  -18,  -67,   47,   10,  -15,   62,   23,  -17,  -13,  -62, -117,   47, -118,  -94,   94,  120,  -17, -117,   92,
  -59,   23,  -66,   80,   17,  -15,  117,   21,  -17,  -91, -128,  120,   81,   79,  -36,    6,  -22,   16,   88,  -21, -116,    5,   89,  -88,   15,
  -19,  -90,   80,    2,   41,  -76,   25,   89, -108,   52,  116,    4,   85,    5,    8,  -92,    0,  -91,   65,   24,   62, -122,   55,  -96,    0,
 -122,   15,   31,  121,  -19,   85,   95,   24,   57,  116,   12, -128,   90,  -34,   -1,  -71,  127,   -5,  -48,   83,   43,  -97,  -36,  111,  -17,
   42,   75,   47,  -69,   91,   -7,   49,  116,    3,    5,   12,   19,   51, -116,   81,  102,  120,   66,    6,   83,  -80, -118,   50,  -87,   16,
 -120,  -56,   64,   85,  -96,   32,   17, -111,    2,  -12, -121,  -96,  -14,  -49,   75,  -58,   24,   80,    6,   56,  -64,   17,  114,   64,   21,
   10,   40,  -59,   12,   34,   90,  105, -119,   80,   13,  -88,   -6,  -93,  -43, -118,   95,   83,  -54, -123,   12,   53, -114,   54,  127,    3,
  -28,   10,    7,  120,   64,  -96,  -34,  123,   -1,   63,   -1,  -76,  127,  117,  -87,   25, -110,  -62,   63,   37,  101,   48,   72,    9,   12,
 -112,  -58, -125,  -15,  107, -128,   80,   66,   87,   24,  -25,   12,   98, -112,    9,   93,   29,    4,  112,  -24,  -39,    1,   73,  -56,    8,
 -108,  -55,   10,   20,  100,   37,  -36, -109, -120,  -64,  -60,   10,  -80,  -94,  112,  -79,   34,    1,  -80,  -54,  -60,  -52,  -86,   18,   58,
  -76,  -90,   57,   71,    9, -108,  -70,   60,   69,  -16,   10,   15,   82,   22,   18,   79,  101,  106,   28,  100,    2,  -59,  122,   67,   65,
  -60,  -27,   67,   16,   17, -120,    2,    0,   18,   51, -125,  -96,   68,  -23,   11,  102,   38,   16,  -79,  -83,   58,  -41,   10,   99,   72,
  -14,  -94,  -88,   51,  -77, -126,  -40,   16, -111,    5, -109,   97,  102,   67,  -58,   24,   54,  108, -116,   53,  -42,   24,  107,  -83,   53,
  -50,   90,   54,  -58,  -39, -120,   89,   17,  -67,   44,   25,   99,  -84,   49,  108, -104, -103, -104,  -39,   48,  -89,    7,   80,    0,   94,
   66,   50,  -30,  -91,  -16,  -98,   61,   76,  -63,  -34,    8,   24,   92,   48,   81,  -50,  -20,   60,   41, -128,   28,   98,  -31,    0,   37,
   88,   95,  -88, -123,   41,   32,   12,   35,  -92, -122, -116, -112,   88,  101,   65,  -99,  -71,  -50,  -36,   98,  -99,  -77,  104,  106,  -58,
  -14,  -73, -110, -106,   67,   68,  -91,   28,   47,  105,    0,   34,   34,   87,  -45,   71,    0,  -92,  -94,  103, -100,   -6,  -70,   -9,   92,
  118,  110,   -7,  -74,  -74,  -18,  124,   -9,   85,   87,   95,  -10,  -46,  -34,  -59,    3,   69,   42,   42,   97,  -85,   49,   93, -123, -113,
  -58, -128,   65,   16,   50,   12, -109,    1,   96,   46,  -40,   43, -109, -126,  -96,    4,    5,  -84,   66,   69,  -43,    3,  -98,   68,   -1,
   50,  -46, -122,   33,   56,  103,    0,   11,  -54,    0,  105,  -32, -108,  106,  -62,  -21, -108,  -41,   38,  -16, -118, -103,  -14,   55,    6,
  -26,    9,   96,  -64,    0,  -36, -112,  -86,  -30,  -84,   33, -128,   52,  -63,   22, -107,  -99,   61,  -75,  -75,    0,   25,  -44,    3,   14,
   96,   -1,   23, -125,  -23,  -65, -102,   68,   81,   42,    2,    0,   37,  102,  -88,   18,   49,   20,   17,   60,  -76,    1,   83, -115,  -33,
  -92,    0,   67,   41,  -68,    2,    3,  -61,    0,   43,   71,  -15,   74, -109,  108,   83,  -10,  -57, -112,   97, -128, -104,  -53, -113,   74,
 -116,  -44,   17,   25,   80,   48,    3,   42,   96, -126,   18,   51,    5,  -87,   10,   16,  -55,   45,  -96,   82,  -66,  -57,   40,  106,  -59,
 -126,  -72,   25,  -80,   34,   78,   53,   50,   76,  -52, -118,    6,  120,   49,   55,   50,   20,   48,   49,  -64,  100, -128,  -70,   48, -103,
   19,   64,   12,    2,   49, -127,   12,  -77,  113,  -82,  106,   92, -107,  -87,  -18,  -75,   78,  -88, -127,  -84,   53, -122, -120,  -40,   48,
  -79,   33,   54,  108, -104, -115,  113,  -42,  -78,   53,  -58,   56,  107,   77,  102,   13,   91,   99,  -83,  115,  -50,   88,  -21,  -84,  117,
  -42,   26,  107,   57,  -77, -106,   13,   27,   99,  -84,   53,  -58,  -80,  101,   34,  -90,  -12,   12,   16,   81,   21,   45,   68,  -67, -105,
  -94,  -16,  -58,  -77,  120,   83,   55, -123,  -15,  -60,   76,    5,  -25,  -52,  -56, -117, -126,  -56,   17,  -27,   32, -105,   19,   44,   28,
    8,   10,   15,  -81,   12,   35,    0,   67,    0,   33,   98,   21,   37,  -87,   27,   83,   39,  -82,   89,  -25, -116,  -86,  -88,   32,  -56,
   77,   65, -122,   82,   40,   73,  -44,  -52,   84,  -61,  -21,   32, -123, -106,   61,   34,  -98,  -87,   68,  -44, -128,   42,    0, -125,  -37,
   71,  -67,  -21,   31,  -34,  -39,  -36, -111,   46, -104,  119,  -26,  103,   63,  -11,  -39,   93,   61,   27,    7,  116,  -81,   48,   65, -107,
  -80,   37, -115,  -23,   49, -118,   35,  -27,   52,   40,   68,  -60,  121,  -43,  -76,  120,  -80,   17,  100,   76,    4,  -85,   70,   85, -109,
  -36,  -50,  -34,   11,  121,    5, -108, -116,   -7,   11,    1, -106,  -77,  -80,  -50,   16,   57,  -64,    1,  -11,  -33,  -45,  121, -109,   86,
  -40, -124,   86,   17,  -17,   53,  -86,    2,  -23,  -60, -104,   97,  118, -128,    5,   25,  -64,   16,   88,   97,    0,   16,   88,   33,  104,
   -4,  120,  -96,   72,  -73,  -96, -127,  -73,    3, -112,    1, -123,  -86, -121,  -78,  -49,   -1,    2,  -49,   -4,   87, -106,   26,  -62,   10,
   53,  -53,   45,   81,  -48,   29,  120,   48,   77, -119,  -54,  -31,  116,   11,   98,  -80,    5, -105, -123, -127, -126,  -78,   22,  -75,  -16,
  -88,  -72,    1,    0, -105,  -70,   27,  -48, -116,   92,    0, -120, -125,  114,  -89, -118,   47,  125,   -7, -122,  112,  -53,   36,   40,  -45,
   55,  -66,   -2,  -19,  101,   75, -105,   17,   65,   85,  -68, -115,  119,    9,   37,   54,  -96,   42,   41, -116,  -27,  -12,  -51,   37,  -90,
 -123,   42,   17,   56, -120,  -21, -100,   36,  -69,   32,   85,   81,  -86,  114,  -62,   88,   34,   14, -109,   96,   24,  101,  -60,   96,  -29,
  -40,   84, -116,  -53,  -76,  -40,   39,   90,   59,   -8,  -96,  -55,  -13,  -49,   59,   97,  -47,   79, -106,   -4,  -18,  -39,   45,  -60,   96,
   54,  108,  104,  -50,  -52,  113,   23, -100,  125,   -8,   79,  127,  -15,  -44,  -22,  -25,  119,   90,   75,    1,  -71,  -84,  115,  -50,  -78,
  117,   46,   11,  104,  -27,   56,   72,   89, -103,  -75,  -50,    5,  117,  -47,  -80,   33,  -61,  -60,   20,  103,   84,   81,  -11,  -94,  -30,
   53,   -9,  -34,   23, -100,  -25,   92,   47,   10,   34,   20,  -20,   65,   32,  114,   97,  114,  -55,  -95,   10,  -79, -128, -128,    9,  -84,
   32,  -64,   22,   16,    3,    6,   68,  -63,   66,  -52,  -62,   30,  -62,  100,  -96,   57,  113,  -35, -110, -122,   97,   79, -128,  -86,  106,
 -112, -102,  -30,  109,   53,  106, -120, -128,   42,    5,    8,   43,  -31,   41,  -15,   43,   77,  -30,   66,   95,   43,  -58,  -51, -103,  120,
  -44,   49,  -77, -102,   -5,  -47,  -56,   49,   35,   78,  122,  -19,   41,  -73,   -3,  -16,   -5,  101,  -45, -121,  -44,  -46,   59,   96,  -44,
    7,  122,  -84, -100,  114,  -96,   20,  -92,  121,   82,  -86,  -75,   90,  -53,  -74, -106, -103,  -74, -100,   43,   25,   19,   89, -111,   66,
 -107,   85,   32,  -71,   18,   96,  -56,  -58,  119,   99, -101,   68, -100,   -1,  -71,  100,    8,  -58,  -62,   90,  102,  -74,   64, -119,   13,
   17,  -95, -126,   98,  -88,  -86,    9, -100,   20,    0,   11,    1,  -92,   26,  -96,  -72,  124,   66,   40,  -63,   50,   96,   13,  -63, -126,
   28,  -63,   41, -128,  -58,   64,    1,  -63,   38,   41,  -73,    0,   10,  -64,   70,   84,   42,   37,   45, -108,  127,  114,  -64, -118,   88,
   21,   54, -118,   87,  -78, -112,   69,   65,  -10,    8,  -46,   84,   57,  -85,  -30,   37,  -16,  -44,   36,   91,   37,  -19, -112, -107,   96,
 -116, -110,   97,  102,    2,  -57,   -9,  -96,   12, -122,   42,  -64,   32,  -27,  -96,   58,    6, -116,  -96, -127,   95,   41,  -91,   30,  -37,
   44,   34,    1,   68,   56,  -21,  -20,  -45,   31,  126,   -8, -111, -101,  110,   -6,  114,   56,  114,  -12,  -47,   71,   45,   -6,  -63,   45,
 -105,   94,   -6, -127,  123,  -18,   -7,   37,  -87,   88, -111,  -88,  109,  114,  -28,  -77,   72, -109,  -32,  -90,   32,  -96,  -98, -123,   90,
   51,    7,  -72,  -44,   32,   42,    1,  -60,   65,    9,   45, -111, -117,   25, -119,   55,   75,   66,   25,   83,  -60,  -31,   32,  113,   17,
 -125, -119, -120,  -84,  -83,   88,   87,   37,   18,   65,  -99,  -87,   24,   61,  114,  -16,  -85,   79, -104,  123,  -21,   45,  -33,  -40,  -69,
  -85,  127,  -16,  -56,   41,  -60,  108,   44, -113,   25,  -43,  113,  -62,  -47,    7,  -34,  -14,  -99,  -81,  -18,  -34,  -34,   58,  122,   -4,
   44,  -29,  -40,   90,  -25,   44,  103,   46,   72,   88,  -58,   58,  118,  -42,   57,  103, -100,  115,   89,  102,   50,  107,  -83,   99,  103,
  -40,   88,  -78,  -52, -108,    0,   75,   85,   11,   17,   95,  104,  -18,  -71,  -56, -115,  -79,    5,  -41,   57,  -25, -100,  114,   34,   70,
   78,    0,   57,   42,  -96, -111,  126,   21,  -89,   38,   87,  -79,  -80,    2,  -75,   48,   65,   76,   18,   24,   18,   31, -124,  -56,  -96,
   59,   18,  -68,   37,   16,   41,   41,   41,   64,   81,  106,   66,  -32,  -89,   74,  -76,  106, -102,  -35, -101,   24,  -28,  -98, -114,  112,
  -81,    1,   93,  -25,    3, -105,  126,   48,  -85,  -70,  -26,   35,   89,  -59,  -66,   -9,  -30,  -82,  -37,  126,   -4, -125,  -14,  -60,  -16,
 -114,   -6,  -38, -101,   78,   98,    0,  104,  -33,    3,   72,  -22,   28,  -15,  -51,    7,  -46,  -63, -126,   28,  -77,   37,   95,  -80,   97,
  -25,   32,  -62,  -34,  -77,   -8,   88,   65,   38,    3,   82,   34,   98,    7,    5,  -76,  -64,  127,   39,   49, -109,   73,  -51,  -19,   21,
  -46,  120,   48,   99,   96,   29,  -84,   37,  107, -100,  104,  -67,    9,  -98,   84,    1,  -21,  -87,   96,  -78,   62,  113,   90,   10,    5,
 -124,   65,   10,   33,    4, -119,   47,  -14,   88, -118, -118,    1,  103,   48,   38,   35,  100,  -87,   69,  -53,   70,  -28,  -44,  -96,   33,
   99,    9, -103,  -94,    0,  -22,   34,   57,   21,  -34, -101,  -64,   75,    0,   13,  -42,   35,   43,  124,  -95,  -86,   70,  -15,   74,   38,
  -78,  116,  -96,    0, -107,   68,  -11,    1,    7, -125,   74,  -56,   73,  109,  -44,   -8,  -99,   37,   98,   48,   43,   39,   57,    4,   73,
   83,  -41,    4,   31,   20,  -28,   29,   46,  -79, -119,    0,   40, -109,    9,  -93,   36,  -36,  -95,   33,   22, -107, -118,  -34,  -70,  117,
  -49,  -33,  126,   -5,  -19, -125, -121, -116,    5,  -16, -117,  123,   30, -100,   56,  113,  -46,   87,  -66,  -14,   -7,  113,  -29,  102,  -76,
  -73,   15, -127, -118, -110,   48,  113,  100,  -78, -112,  -28,  -72,   84,  -33,  -86,    7, -125, -119,   80,  -81,  114,  -55,    2,    4, -123,
  -81,  -28,  -20,  -71,  -63,   82,   53,  -28,   47,   34,   34,  -30,  -46, -100,   16,  -78,   76,   12,    6,  -37,   10,   89,  -53,   84,    8,
   10,   37,   31,   36,  -81,   61,   59,   54,  -44,   -6,  -99,  101,   38,    3,  -26, -120,  -71,   59,  -74,  -84,  -18,  -17,   27,  111,  -83,
  -79, -122,  -83,  101,  -25,  -84,  113,  108,  -99,  117, -114,  -83,  -75, -103,   51, -107,   44,  115, -103,  -87,  102,  -42,   57,   83,  113,
  -28, -100,  -77, -119,  -30,   15, -107,   21, -123,  122,   41,   60,  -14,   60,  -81,   89,  -74,   57,   49, -109,   97,   98,  -82,  -93,   94,
  -14,   31,   30,  -80,   10,   81,   88,   85,    5,  -39,    2,   98, -107,  -68,   50,   27,   50,  105,  -74,   17,   16,  -61,    4,  -42,   71,
   81,  -73,  -92,  -44,  -60,  -86,   39,  100,   42,   49,   67,    1,   96,  -49,   16,  116,  -20,  -62,  -98,   33,   24,   55,  100,  -10,  -40,
   33,  -45, -103,  105,  -27, -122,  110,  -12,  109,   41,   95,   88,   72,  -77,   38,   31,  116,  -58,  -87,  -89, -122,   -4,  -38, -107,  -67,
 -109,  103,  -73, -122,   -4,  -52, -125, -121,   77,   63,  -32, -112,   85,  -21,   86,  -58,   34,  -93,   -7,  100,   64, -113,   35,   66,   79,
   71,   36,    8,  -62,   87,   29,  -69,  -29,  125,    5,   86,  -63,   34,  -42,   40, -121,  -26,   96,  -53,   90,    4,   26,   64,   85, -119,
    0,   67,    6,  100, -108,    6,  -44,  -25,   79,   74, -106,   77,  -59,  -63,    5,  -54,   49,  -42,   73,   68,  -44,   11,  114,  -15,  -75,
 -100,  -67,   88,    3,   71,   48,   22, -118, -100, -120,   20,  -28,   10, -120,   70,  -87,   80,    8,   44, -111,   78,   11,    8,   21,   24,
  114,   65,  -28,  -28,  -53,  102,  -52,   50,  100,    6,  -28, -104,  -56,    4, -112, -118,   51,   99,   72,   36,   33,   31, -118,   12,   64,
   70,  -80,    0, -117,  114,   97,  114,   91,   79,   83,   49,  -95,   48,  -95,  103,  -28,  -58,   90,   20,  -56,   13,   10, -127,  -68, -126,
 -123,   44, -115,  -93, -128,  -54,   60,    0,  109,  -12,  -90,   48,   17,  114,   19,   83,   26,  -88,  -83,  -96,    9,  114,  -44,  -26,  -72,
   44, -117,    1, -128,   45,  -40,  -86,    1, -125,  -61,  124,  -61,   82,   68,   -6,   60,   25,  117, -125, -120, -108,  108,   82,   77, -112,
    7,    0,  -80,  -74,  -38,  -34,   62,   42,  124,  -84,   84,  -37,    1,  -68,   -8,  -30,  -77,  -45,  102,  118,   30,  113,  -56, -100,  -85,
   62,  124,  -39,  -31, -121,   31,  -68,  108,  -39, -118,   79,  127,   -6,   11,  -53, -105,   63,  121,  -26, -103,  -89,   46,   88,  -16,  -55,
   43,   46,   -1,  -24,  -30,  -69,  -17,    5,  -80,   -4,  -87, -121,  126,  113,   -9,  125,   87,  117,   93,   83,  -83,  -13,  105,  -81,  121,
  -11,  -89,  111,   -8,   -8,    7,  -81,   -2, -105,  -59,   -9,   60,  112,   -6,  -87,   39,  125,  -18,  -70, -113,    1,   -8,  -18,   -9,  110,
   -1, -113,  -37,  126,  -80,  121,  -21,   54,    0,  -41,  127,  -14,  -22,   83,   95,  125,  -30,  -39,   23,  -68,  -13,   51,   -1,   -9,   -1,
   -4,  -57,  -19,   63,  -69,  -17, -127,   71,   70, -115,   26,  -10,  -90,   55, -100,  -13,  -90,   55, -100,    9,  -32, -102,  -21,  -66,  114,
  -33,  -81, -105,   25, -109,  -79,  -75,  -93,   70,   14,   -7,  -56,   -5,   79,  -99,   51,   99,   44, -128,  -37,  127,  -14,   96,  -84,   37,
   81,   16,   49, -115,   97,   98,    3,  -32,  -40,   99, -113,  -19,  -22,  -22,    2,  -80,  102,  -35,  -50,  111,   44,   90,  -47,  -37,   39,
   71,  -50,   29,  115,  -47,  -21,  102,   46,  -70,  -21,  -71,   85,  -49,  -17,  -55,   50,  -25,   42,  -26,  -97,   47, -102,   14,  -32,  -74,
  -69,   54,   86,   50,   14,  -62, -105,   11,   28,   86,  -24,   -4,  -94,  -71,  104,   81,  120,  107,  -63,   70, -104,   41,   10,   95,  112,
    0,  106,  117,   88,   17,  -88, -123,  -86, -118,   21,   81,  -93, -123,   42, -111,   97,  -93,   86,   85,   89,   11,   40,    3,  -94,  106,
   16,  100,    7,   37,    8,    8,  -34,   90,  -75,  -27,   59,   47,   51,   59, -121,   53, -114,   12,  -86, -114,   24,   70,  -83,  -29,   39,
   78,  -83,  -27,   53,   95,   20, -123,   -6,    9,   67,  103,  -68,  -26,  -24,   -7,  -45,  -90,   79,   93,  -70,   -6,  -82,   71,  -97,   88,
   -6,  -32,   19,   63,   15,   24,  116,  -38,   73,  -81,   25,   49,  -90,    2,  -96,  -73,   39,   -1,  -20, -105,  -66,   -8,  -87, -113,  125,
   96,  -56, -120,   10, -128,   33,   67,   59,  -34,  120,  -34, -123,  -97,  -72,  -23,   58,   42,  111,  -47,  -92, -103, -105,   41,  -46,   99,
   28,   36,  124,  -20,   25,   10,    0,  123, -121, -127,   97, -123,   44,  -79,  -35,   59,   88,    6,  -17,   19,  102,  -88,  -80,   97,  -61,
    4,   65, -102,  -27,  -56, -128,  -64,   46,  114,  -45,   -6,  -89, -120,   27,  -52,  -36,  106,   77,  -59,  -78,  -77,  108,    6,  -24,  -85,
    6,    0,   32,   94,   56,   47,   56,   47,  108,   94,   24,   35,  -42, -127,  -22,   17,  -89,   34,   50,    6,   25,   10,   77,  -78, -107,
  -58,  -55,   89,   57,   34,   87,  -55,   96,   57,   66,  -59,  -94,   98,  -31,  -19,   75,  -22,   33,  104, -104,  -73,   37,  -86,   37,   36,
   77,  -44,    6,   67,  -85,  -52,   80,   69,  -18,  -29,   80,   49,   18,   88,   91,  104,    6,   20,   48,   10,  -93, -112,   87, -122, -112,
   85,   53,  -88,   58,  100,    6,    6,  -16,   64,  -35,  -93,   63,   71,   77, -110,   97, -114, -102,  -90, -127,  -78,    1,  -61,  -33, -127,
   31, -107, -104, -120,   17,    9,  -84,  -14,   92,   38,   64,   77,   70,  -52,  106,   92,   32, -120,  -76,   52,  -52,   41,   65,   11,   21,
 -113,  -94,   96, -126,   54,   46,   75,  -65,   57,  106, -108,  -23,   22,    0,   91,   16,   14,   59,  116,  -50,   97, -121,   30,  -12,  -59,
 -101,  -66,   12,  -32,  -16,  -61,   15,   -5,   -4, -115,   31,   27,   59,  118,  -44,   69,   23,  -67,  -19,  -42,   91,  -65,  -13,  -81,   55,
  124,  -20,  -97,  -33,  127,  -11,  -14,  -27,   43,   31,  123,  108,  -39,   -6,   23,  -98,  -39,  -67,  119,  -41,  -48,  -63,  -93,    0,  -20,
  -38,  -77,  109,  -51,   11,  -49,   28,   56,  -23,   32,  101,  -77,  -12,  -79,  101,  -73,  125,   -9,   27,  -17,  123,  -49,   -5,  -81,   -6,
  -24, -107,   75,   31,   95,  126,   -1,  125,   -9,  116,  117,  117,  -51,  -98,   49,  -27,   93,  -17,  -67,   34,  -53,   76,   16,   22,   63,
   -6,  -63,   75,   14,   62,  104,  -26, -122,  -49,   94,  -65,  115,  -37, -122,  -81,  -35,  -12,  -55,  -71,  115,  -90,   -1,   -5,  -41,  -66,
  125,  -16,   65,   51,   62,  -15, -111,   75,  -81,  -65,  -15,   43,  119,   61,  -16,   44,  -77,  -69,  -26,    3,  -89,  -49, -102,   54,  -22,
  -90, -101,   -1,   -3,  -56,  -61,   15,  126,  -61,  107, -113,   15, -107,  100,   38,  102,   98,  -61,  108,   40, -120,   93,   93,   93,   93,
  -53,  -98,   88,  121,  -17,  127,  -35,  -43,  -43,  -43,  117,  -34,   73,   29,   95,   93,  -76,  122,  -61,  -90,   14,    0,  -21,   87,   -2,
  112,  -27,  -77,  120,  -43,    9,  -25,   84,   43,  -79,   19,  -65,  -15,  -84,  -15,   63,  127,  112,  123,  -59,  -79,   49,  112, -106, -125,
  -72,  -89,   10,   85,  -51,   11,  -15,  -98,  107,  -42,  -80,   41, -104, -119,   24,   74,  -27,   40,   81, -123,   21,   85,  -93,  -74,   16,
   49,   66,  -86,   12,   85,   81, -122, -110,   40,   12,   24,   96,    1, -109,   81,    6,    1,   36,   66,  -60,  -84,   84,   88,   67,    3,
    6,  -48, -114,   17, -115, -119,  -70,  106,   58,   12,  -39,  -34,  124,   95,   33,  -69,  118,  108,  -36,   24,  -80,    0,  -37,  -80,  108,
  -35,  -67,  119,   46,  -75, -109,   71,   30,  118,  -50,  -85,  -34,  -10, -127,  -73,  -98,  -13, -114,  -34,  -73,   95,  -13,  -27,   15,  -11,
  -11,  -26,  111,  126,  -29,   57,  -31,  -62,  -89,   87,  -83,   -6,  -26,  -62,   91,   78,   62,  -27,   85,  111,   56,   -1,  100,    0,  -58,
  -16, -101,  -33,  114,  -58,   39,  110,  -70,   62,  -50,  111,   77,  -44,  102,  -77,  -19,  -66,  -28,  104,  -44,   52,  -27,    1,   37,   11,
  -78,  -62, -106,   24,    6,    5,  -88,   14,  102,   98,   43,    4,  -30, -122,  115,   69,  -80,  -13,  -88,    5,  -55,  -97,  -32,  -10,  -28,
  -84,  105,  -79,  -36, -110,  -79,  -75,   24,  104,  -73,   83, -118,  -54,   32,   25, -104,   44,  -53,   68,   50,   95,   55,  121,  -63,  -66,
   80,    3, -110,   68,  -80,   39,    9,  -85,  -31,  110,   32,    8,    4,   71,   56,  -82,    6,   42,  -15,   25,   45,  -61,   57,   56,    3,
   98,  -40,   -3,  -43, -115,   52, -102,  119, -118,    6,  -13,   43,   32,  -26,   80,   40,  -61,  114,  -90,   42,  -98,   32,  -55,  -58,   20,
  -27,    3,   15,  -74,  -63,  -48,   -8,  123, -106, -128,  -65,  -71, -108,   25,   12,  -83,  -64,  -75, -127,   83,  -89,  117,   64,   21,  104,
  -41,  -94,  -24,  -29,   93,  125,  -88,  107, -102,   75, -126,  108,  -43,   36,   79,   37,  -30,  -87, -127,   41,   68,  -84, -127,  107,   71,
 -110,  -36, -125,   94,  -26,   50,   24, -117,   48, -119,   37,   47,  -85,  -88,  -59,   40,    1, -106,  -84, -126,   11,   72,   61,   86,  -85,
    4,   41,    5,    8,   28,   -4, -124, -128,  121,  -13,  -26,  -51, -101,   55,  -81,  -71,   -2,   55,  -36,  112,  -61,  -48,  -31, -109,  -50,
   60,  -29,  -28,  -79,   99,   71,   77, -103,   50,  117,  -19,  -38,   53,   79,   63,  -69,  -31, -117,   95,  -72,  -18,   -1,   92,   -7,  -34,
   55,   95,  -12,  -66,   45,   91,  119,   44,   92,  -72,  112,  -46,  -92, -125,   79,   59,  -29,  -28,  112,   -7,  -99,  119,   44,   94,  -79,
  108,  -19,  103,   62,  119,  -19,  -99,  119,   44,  -34,  -67,  -77,  -25,  -19,  -17,  -70,  120,  -23,   99,  -53, -114,   60,  -22,  -16,  -74,
 -114,   33,   -9,   60,  -16,  -21,   59,  126,   -4,  -97,  103,  -98,  114,   -4,  -67,  -35,  -53,  -61,   72,  -23,  -22,  -22,   -6,  -19,  -54,
  103,  -10,  -18,  -34,  113,  -31,   91,   46, -103,   59,  103,   -6, -121,   63,  114,  -51,  103,   62,  -11,  -55,  -15,   83,  -26,   94,  117,
  -59,   -5,  -34,   60,  -17,  -20,  -57, -106,   94,   63,  -18,  -64,   73,  -77,  -90, -115,  -70,   -6, -102,   79,  124,   -6, -109,  -41,   78,
 -102,  126,  -52, -107,   31,  120,  -25,   63,  -67,   -9,  -35,    0, -104,  -63,    6,  -58,  112,  112,   33,    2,  112,  -29, -115,   55,   94,
  113,  -59,   21,  -93,  -57,  -49,   82,   59,  -12, -118,   -9,  -65,  -29, -114,   59,   23,   -9,  -12, -115,  127,   96,  -55,  115,   93,   93,
   93,   31,   -1,  -44,  -51,  -43,  -52,  -50, -104,   60,    8,  -64,   -4,   -7,  -13,    1,   84,   43,   92,  -55,   76,  102,  -39,   89,  -26,
  114,   34,   87,  -28, -123,  -44,   11,   33, -114,   20,   51,    5,   34,   69,   85,   69,   84,  109,  -31,  -59,   90,   34,  -80,   23,   86,
  101,   85,   43,  -30,  -83,   49,  -94,  -98, -123,  -96,   12,   67,   26,  -23, -110,   80,   40,  -87, -110,  -86,  -40,  -99,  -93,  -77,  112,
 -125,  -46,   -2,  -98, -108,   -5,   22,    1,   -9,  -43,  123,   20,    0,  101,   13,  -82, -118,   35,   33,  -68,  106,  -53,   19,   55,   -1,
   -4,  -93,  -85,   54,  -66,  -11,   -4,  -45,  -50,   -5,  -18,  -65,  126,  119,   -7,   51,   75,   15,   57,  108,   74,   56,  -27,   71,   63,
  -18,   46,    4,  -65,  -66,   -9,  -39,    0,   88,    0,   38, -114,   31,  117,  -34,  105,  -13,  127,  -12,   95,   63,   10,  -73, -120,   93,
   71,    7,   40, -122,   81,  -70,   72,   56,   85,  126,  -85,   96, -123,   21,   97,   75,  -68,  123,   40,   15,  -39,    5,   37,  -23,  105,
 -105,  -70, -121,   87,  120,  -58, -120,  -51,   32,   96,  -21,  104,    4,  -11,   88,   -8, -113,  -71,   82,   58,   54,  109,   85,  -45, -102,
   17,  113,   82,  -66,   98,  -57,   86,    8,   65,   16,   85,   58,   16,  -63,   86,   80,  -79,   89, -106,  -41, -117,  -34,   58,  -22,   18,
  106,  -98,  -72,  -11,  -56,  -59,   66,   19,   54, -123,    1,   35,  -79, -120,  -64,  -57,   41,   96,   12,   42,    6,  108,   82,   27,  107,
   67,  -76, -116,  -54,   11,  -59,  -31,   80, -118,   78,  -31,   45,   21,  105,   54,  -79,   96,  118,   16,  -96, -106,   15,    0,  119,   16,
  -84, -127, -111,  -64,   76,   -2,   45,   67,   86, -117,  -61,  -88,   22,  -96,  101,   63,   95,   25,  100,  102,   16,  -58,   24,  108,  -83,
  113,   95,   30,   21,  103,    0,   17,  -58,   41,   88,  -33,   16,   44,  124,  -91,  -34,    8,  -78,    0, -125,  -72,  -76,   -6,   41, -128,
  -84,  -86,  -90,   18, -124,   48,  -43,   40,   94,   33, -111,   90,   64,  -92, -118,  -32,   42,  -22,   45, -118,   60,  121,   76,    1,    4,
  102,  107,   76,   69,  -45,   20,  -68,  104,  -47,  -94,   48,  -92, -101,  -45, -108, -103,   39, -115,   25,   51,   26,  -64,  -38,  -75,  107,
  -90,   78,   63,  -95,  103,   31,  -17,  -36,  -35,  127,  -42,   25,  -81,  -34,  -68,  121,  -11,  -30,   -1,  122,  -16,  -52,   51,   78,   94,
  -65,   -2,  -55,   67,   15,   -3,   -8,   29,  119,   44,  -18,  -19,  -35,  125,  -16,   33,  -77,  127,   -7,  -64,  -93,    0,  -66,  -11,  -19,
  -81,  101,   89,  -53, -104,  -79,  -93,  -42,  -82,   93,  -13,  -42,  -73,   92,  124,  -32,   -8,  -39,  -95,   -1,   28,  121,   -8,   97,  -33,
  -65,   -3,   39,  -95,   51,   45,   89,  -78,  -28,  -32,   87,  -99,   52,  104,  -24, -120,  -23,  -77,  -26,    0,   24,   57,  124,  -24,   -5,
   62,  112,  -43,  -16,  -47, -109,   70, -115,  -97,   58,  113,  -30,   68,  -85,  123,  -38,   42,  117,    0,  -85,  -98,   94,   49,   -3, -112,
 -109,   39,  -51,  -20,  -36,   89,   27,   26,  -37,   40,   72,   88,   76,  100, -104,  -54,  -94, -114,  126,  -19, -119,  -89,  -65,  103,  -40,
   -8,    9,    0,  -26,  -52, -102,  -14,  -11,  -17,  -35,   54,  -19, -128,   -9,  -50,  -34,  -70,  -29,   95,  -82,  126,  -17,  123,  -81,  -68,
   -7, -110,  121,  -17,  -35,  -75,  -69,  103,   -3,   -6,  -11,   31,   -5,  -52,   55,   91,   42,  -90, -110,  -39, -118,   35,   99,  -55, -103,
   -8, -104,  -71, -121,   43,  -56,  -28,   81,   75,   16,   88,   17,  -55,   60,  121,  -49, -103,   51,  -34,  -85,   53,   36,  -62,   34,  -54,
 -122, -116,   24,   47,  -34,    8, -117,   72, -112,  -11,    0,  -10,   18,   60,   98,   61, -108,    3,  -19,   67,    2,   65,   97, -107,   44,
  -48,   80,   -6,  -61,   72,   27,  -28,   70,  -18,  -83,  111,   83,  -81,   96,   91,   42,  109,  -47,   25,   43,  113,   52,   96,  -44,  -75,
  -72,  107,  -59,   45,   -3,  121,   -2,   -2,  113,   23,  -99,  115,   90,  -60,  -90,   61,   59,  -21,  -65,   90,  -78,   76, -116,   -3,  -55,
   67,   63,   -5,  -28,  -18,  119,  -76,   15,  -50,    0,   84,   90,  -20,  107,   78,   61,  -18, -121,   -9,   -4,   44,  -68,  -38,  -48, -115,
 -110,   81,  -93,  -15,   58,  -93,   43,    5,    3,    1,   72,   18,  -67,    8,  -78,  -60,   25,   41, -124,  -59,    0,  -69,   58,   10,  -14,
  -62,    2,   79,   80,  -62,  -42,   49,  -47, -106,   76,    6,   36,  127,   76,  -62,   98,   54,  109,   85,  -45,   94,   33,   50,   68,    4,
   50,    0, -110,  -39,    1,  -92,   94,  -93,  107, -107,   15,    2,  -66,   97,  -80, -127,  117,   86,   96,  -67,  -81, -117,   72,   20,  -81,
   18,  -33,   17,  117,   82, -114,  -46,  -94,   42, -126,   32,   38,  -47,  116,    1,   75,  112,   22,  -58,   53,  113,   76,   77,  -56,   82,
  118,   -8,   64,  -74,  -57,  -95, -126,  -40,   68,  -42,  -57,    1,  -58,    4,   50,  112, -128,   87,   20,    2,    9,   -4,  107,   28,   39,
   48,    2,  -10,   32,   -5,   63,  111,   33,   -3,  127,   36,   57,  -62,  -88,   54,   80,   -5,  -53,  -98,  -48,   82,  -29,   -2,  -63,   24,
  -43, -125,   77,   -5,   80,    7,  -62,  -21,  -47,  -28,   22,  -91,  -54,   74,  -51,   64,  -49,   10, -128, -104,  -40, -126,  -93,   47,    2,
    0,   88,   71,  -74,   26, -104,  106,   68,  -21,   92,  -68,   36,   58, -100,  114,   16,   -7,   75,  -41,  115,   11,   41, -126,  -83, -112,
  -40,  -78,  -87,   40,    6,  -48,   28,  -19,   67,   38, -116,  -99,  124,   12, -125,   10,  -97,    7,  -98,   34,   10,  113,    0,    0,   83,
  105,    7, -108,  -39,    2,  -24,  -21,  -35,  -67,  105,  -53,  -82, -115,   27,   55,  -51, -101,   55,  -17,  -83,  111,  125,   -3,  -75,  -41,
  126,  114,  -44,  -24, -111,   93,   93,   93,   43,   87,   62,  -73,  121,  -13,  -74,  -11,  -21,  -41,   15,   31,   54,    1,  -64,   -8,    9,
  -29,  -49,   63,   -1,  -68, -106,  106,   59,    1,  119,  -33,  117,   79,  119,  119,  119,  -49,  -34,   93, -108,   10,  -20,   24,   49,  -70,
   20,   56,   58,   59,   59,  103,  -52,  -38,   99,   93, -123, -120,  126,  118,  -25,  -30,  -25,  -42,   60,   59,   97,  -38,  -79,  -31,  -76,
  106,   91,    7,   53,   57,   76,   32, -112,  -14,  -52, -122,  -71,  -12,   52,   26,   51,   97,   38,    5,  -38,    5,    0,  -80,  -31,   -7,
   39,   55,  109,  -17,  125,  106, -107, -116,   26,   57,  108,  -37,  -13,   -9,    3,  -17,   -3,  -65,   31,  -65,  118,  -51,  115,  -21,   39,
   76, -100,   84,  -55,  108,   37,  -93,  106,   70,  -42,  -78,   53,    0,   24,   16,  -21,   81, -124,  -71,   23,   54, -116,   23,  -15,  -58,
  123,  -28,  -94,  -34,   27,   99,  -67,  -15,  108,  -60,  -88,  120,   17,   35,  -30,  -39,   19,   49,   25,   67,   94,  -56,   48,  -85,   18,
   27, -122,   -6,   66,  -93,  -33,    8, -127,    5,  -34,   40,   89,  112, -122, -110,    8,   87,   64,  -47, -102,  117,  -12,  -27,  125,  -54,
   46, -102,  -52,  -62,  113, -125,   38,   -3,   13,   64,   68,  -82,   66,  -79,  110,  -49,  111,  103,   28,  -36,   94,  -37,  -89,   59,  -73,
  -28,   35,  -57, -102,   21,  -65,   91,  -11,  -64,  -46, -121, -127,  108,  -53, -106,  125,   15,   46,   89,  121,   -6,  -23, -121, -122,  -45,
   95,   -1,  -70,   19,   62,   -2,  -39,  105, -101,  118,  -81, -117, -105,   51,  -92,   65,   43,   36,   89,   61,  125, -107,  -70,   92,   28,
  -37, -118,   76,  -55,   10,   27,  -61,   -7, -114,   81,  108, -124,    9,    2,   29,   80,   29,   50,   73,   65,  -45,    8,  -84,   -5,   77,
  -36, -110, -103,  -74,   22,  102,   11,   50,   32,   11,   34, -118,   62,   80,    0,   60,   84,   73,   11,  -88,    7,  -87,  106,   97,   56,
   56,  -80,  -64,   26,   -8,  -86, -111, -126,  -87,  -81,   40,   69,  -47,  -32,  -45,   14, -122,   72,   20,  -49, -126,  120,  -91,  -44,   84,
   17,   69,   91,    5,   67,   91, -111,   69,   11,    4, -118,   62,  -12,  -42,  -31,   75,  -61,   96,   82,  -15,   74, -122,   62, -114, -109,
 -127,   98,   84,   56,   98,    9,   70,  -31,   11,   16,   74,  -21,   21,   66,   37, -115,    1,   75,  -77,  109,  -13,  111,   42,   13,  107,
   69,  107,    6,  -44,   95,  -10,    4,   98,  -76,  -44,   65,   21,   30,   42,  -40,  -36,   23,   59,   15,   51,  -96,  -63,   17,  -76,  -76,
   76,   69,  -93,   91,   48,   14,   18,  -39, -110,  -32,   98,  -80, -102,   42, -116,    5,  -94,  119,  124,  100,   71,   17,  -51,  109,   64,
  -22,  -81,   73,  -27,  -92,  -52,  104,  -67,    6,   86,   16,   89,  -45,   26,   28,  -30,    7, -102,  125,    8,  -20,   20,  -80, -108,   21,
  -66,   55,   80,   -5,   47,  110,  -34,    1,   96,  -62, -124,    9,   96,   59,  102,  -52, -120,   49,   99,   70,   62,   -4,  -56,  -93,    0,
 -106,   63,  -79,  106,  -39,  -14, -107, -105,   95,  -34,    5,  -32,   19,  -97,  -72,  -26,  -35,  -17,  -23,    2,  -16,  -50,  119,  -66,   -7,
  -79,  -57, -106,   47,   93,  -70,  124,  -58,  -44,   99,  -73,  108,  -34,  -74,   97,  -61, -122,   55,  -67,  -23,  -94, -114,  -10,   97,   68,
  -12,  -38,  -41,  -98,   -3,  -62,  -58, -115,   35,  -85,   35,  -78,   34,  -34,  -46,   88,   11,  -59,  -74,   29,  -69,    0,   44,   90,  -76,
  -24,   27,  -33,  -70,  -91,  -46,  -46,   62,  118,  -20,  -40,  -93, -114,   58,  -94,   38,   45,  -69,  123,  124,  -72,  -17,   51,   27,  -97,
   51,    6,  -77,  103,   76,    8,   87,   49,   33,  112,   76, -100,   84,  -62,   99, -113,   61,  -10,  -18,  -18,  -51,  -42,   96,  104,   71,
   21,   64,  119,  119,   55,   17, -116,  -31,  -19,   61,   45,   91,  -73,  -19,   88,  -72,  112,   33, -128,    5,   11,   22,  -68,   -1, -118,
  -21, -114,   56,  -30, -104, -128,   86,   89,  -58,   89,    0,   44,   34,   40,   23,   62,  -84,  -98,   19,    0,   34,  -20,  -83,   -9,  -50,
  -40,  -62,   59,   67,  -34, -110,  -75,  108,   11,  -29,  -39,   -5,  -64,  -99,  121,   50, -122,   68,   88, -123,  -40,   80,  -92,  -79,   24,
   48,  -60, -128, -128,  -29,  -84,    3,    0,  -80,    4, -117,  -28,  -45,   14,   66, -117,   25,  -44,   94,   25,  -74,  -91,   -2,   60,  -63,
   54,  115,   76,  -94,   24,  -39,   49,  110,  -36,  -32,    9,  -53,  -42,   63, -126,  100,  -26,   11,   95,   13,   27,   93,   25,   59,   57,
   43,  -22,  -40,  -72,  -74,  -65,  125,    8,   -1,  -32,   63, -105,   32,   76,   26,  121,  -19,  -95,   95,  -81,   42,    1,  107,  -16,  -32,
 -106,  -77,  -50,   58,  -26,  -21,  -33,  -33, -120,   38,  -36,   43,    1,   43,  117, -115,  -72,   50,    8,   92,  114,   10,   80,  101,   34,
  -85,  100,  -63,   22,   68,  -32,   34,  -86,   84,   44,  -63,   29,   12,  -87,   71, -122,   85,   86,  -54,   80,   74,  120,   -9, -110,  110,
  109,   45,  -73,  -76,   24,   87,    5,  -79, -110,   37,  -75,  -54,    6,   48,   17,  111,  -32,   73,  -68,  114,   65,   90,   64,    5,  -80,
  100,   60,  -77,   50,  -95,   80, -112,  -77,  -90,  106,  124,  -47,  -81,  -71,   32,   81,  -17,   65, -116,   10,   67,   66, -125,   19,    3,
   71,   97, -119,    4,   74,  104,  -75,   24,  -47, -118,  -86,  107,   -8,  111,   25, -117, -118,  -59,  -34,  126,  -12,   23,   32, -119,  -41,
 -106,   -3,   92,   17,  125,   59,   36,  105, -123, -127, -100,   10,   38,   72,  -61,   48,  -44,   80,   -3,  -62,   35,  -26,    8,   75,   74,
   64,  127,  -93,  104,   85,   53,   24, -108,   13,  -80,  -49,  -20,   39,   17,    7,  -99,  -70,  -93, -126,  -35,   57,   -6,   53,   42, -125,
   13,   -6,  -87, -119,   32,   87,    2,  -63, -126,    9,   48,  -63,  -53, -127,   21,  106,   51,  -78,   89,  -23,  -51,   28,  124,  -94,   41,
 -115,   19,  -43,  -88,   56,   82, -110,  -69,   20,   68,  -58, -110, -123,  -86,  103,  -74,   74, -114,   75,  -67,  -78,  -87,   90,   32,   19,
 -116,  -16,  108,  -86,   34,    5,   24,  -53,   87,  -84,    2,  -80,  112,  -31,  -62,  -73,  -66,  -21,   19,  -97,   -1,  -36,   85, -121,   29,
   50,  -13,  -94,  -73,  -68,  -93,  -91,  109,   24,  -40,  -35,  122,  -37,   47,  110,   -3,  -42,  -89,   30,  126,   -8,   55,    0,  -42,  -82,
  -21,   89,  -70,  116,   69,  103,  103,  -25,  -55,   39,  -97,  -34,  -34,   62,  -94,  125,  -48,  -88,   39,  -97,   92,  117,  -14,   41,  -99,
   93,   93,   93,  -35,   -9,  -81, -102,   -1,  -90,  -77,   -1,  -15,   -3,  111,   89,  124,  -41,  125,   -1,  114,  -43,  -11, -115,  -15,  -60,
 -122,   64,   79,  -83,   94,   11,  -32,  -14,  -53,   47,  -33,  -80,  101,  -49,  -10,  125,   -8,  -30,  103,  -82,  -98,   51,  -13,  -64,   43,
  -81,  -67,  113,   -7,   51,   61,  -31,   -8,  -58,  107,   62,  123,  -14,  -87,  -57,  -65,   -2,  -52,  -50,  116,   21,  -94,   -7, -127,  -93,
   35,   72,   87,   87, -105,  124,  -31,   27,  118,  -56,  -48,   19, -113,  -98,    4,   96,  -63, -126,    5,  103,  -49,  -69,  -62,   48,  111,
  -35,  -47,  -65,  121,   27, -113,   28,   49,  -84,  -69,  -69,   27,  -64, -101,  -33,   -2,   79,  -42,  -78,  -75,  100,   45,  103, -106,   51,
   23,   28,   23,   32,   18,  -35,  -14,   69,   96,   69,  -84,  101,   99,  -39,   20,  106,   12,   91,  -61,   84,  106,  -96,   76,  -60,  100,
   24,   28,  -79,   50,  -70, -110,    9,   19,   49,  -94,  -41, -122,   18,    7,  -62,   10,  113,   61, -127, -123,  -55,  -30,  -76,  -84, -128,
   98,  -46,  -80, -125,   86,  109,   95,  -90,   38, -111,   86, -127,   75,   38, -128,  -80,  117,  -33,  -74,   83,   15,   57,   -9, -102,   75,
   62,   -4,  -12,   11,   43,   54,   -9,  -81,  -75,   25, -100,  -93, -119,   35,  -90, -100,  117,  -54,  -79,  -43,   86,   39,   85,  109,   27,
  -20,  118,  110,  -50,   69,   24,  -63,  -55, -120,  -16,  -99,  -69,  110,  -65,  -22,  -61,  -81,  109,  105,  117,    0,  -84,  -77,  -17,  -69,
  -24,   45,   95,  -65,  -19,   14,   52,  -31,   84,  -28, -128, -112,  116,  -64,   82,   73,  108, -122,   27,    3, -123,   85,  -74,   32,  -85,
    4,   98,   18,   33,   54,   10,    1,  113,  -29, -110,  -24,   42,   16,  -88,  110,  -77,  127,    9, -117,   42,   21,   83,  109,   81,  -29,
 -128,   12,  100,    1,   71,  108,   21, -122,   40,  -72,  -53,  122,   80,    1,  -28,  -86,    5,   80,   39,   41,   64,   22,  -58,   55,    0,
  -83,   98,  -87,    6,  -14, -123, -108,   16,   47,   32, -114,    4,   22,   20,  -60, -112,   32,   27,   18,    4,  -80, -116,  -95,  109,   48,
   14,  -80,   77,  100,   60,    0,  -96,  -83,   21,  -11,  125, -112,  100,  120,   15,    2, -108,   36,   -1,  -57, -122, -119,   62, -115,    1,
  -90,   56,  -30,  -40,   34,  -67,  -62,   -8,   45,   69, -107,    4,  -74,  106,   21,  -92,  -94,  -66,   94,  -68,  100,  -36,   -4,   85,  -89,
 -106,   12, -103,   -5,  -29,  -89,    1,    0,  -63,   90,  -86,  102,  -24,  -81,   71,   69,   66, -109,   51,   86, -125, -118, -118,   11,   14,
   13,  -56,   40,   19,  -61,    4,    1,  -97,  -84,   83,  -50, -104, -110,  -32,  -98,  -60,   94,   42,   95,    3,   39,   81,  -73,  -31,   77,
   74,  -54, -128,  -42, -104,   43,   48,  -90,  -95,   15, -106,    4,   37,   49, -111,   35,    2, -120,   73, -116,    0,    4,  -66,  -21,  -98,
   71,  -41,   92,  112,  -23,  -43,   31,  -68,  120,  -43,   83,  119,   46,  123,  -30, -103,  -65,  127,  -11,  -23,  -65,  -68,  127,  -15,   -8,
   41,  -57, -127,  -20,  -14,   21,  107,    0,  -84,   91,  -73,  -74,  -83,  125,   -8,  -26,  -83,  123,  -74,  110,  -33,    3,  -32,  -66,   -5,
   22,   31,   52,   -9,  116, -112,  -67,  -10,   35,   55,  -35,  122,  -21,  119,  -66,   -2, -115, -101,   67,  -31, -105,   -3,  -45, -121,   -1,
  -19,   75,  -97,   25,   51,   98,  114,   57,   90,   70,  -20,  -74,   59, -122,   19,   17,   46,  124,  -49, -107,  -57,  -52,   28,  -69,  -16,
 -106,  -81,    1,  120,  -22, -103,   53,  -89,  -98,  126,  -10,   61, -117,  -17,  124,  -43,   41, -105,  -68,  -13, -125,   95,  -65,  -28, -126,
   57,  -33,   -1,  -42,   23,    1,  124,  -21,  -42,   31,  -68,   -3,  -94,   11, -128,  -72,   62, -111,  -71,   84, -123,  113,  -29, -115,   55,
  126,  -80,  -85,   11,  -64,  -77,  -49,  109,   58,  -82,  -13,   76,    0,   39,  -98,   -2,   22,   98,   16,  -45,  -29,  -85,   -4,  -36,   89,
  -40,  -80,   97,  -61,   65, -121,  -68, -118, -103,   12,  -61,   50,  -84, -127,   53,   48,   76,  -50,    6,  -60,    1,   10,  -78,   70,  -83,
 -127,  101, -104,   32,  -53,   70,  108,   10,  104,   21,  101, -117,  -24,   65,   22,  -27,   59,   24, -122,   26,  -10,    2,  -61,   16, -122,
 -124,  -27,   77,   28,  -67,   81,   20, -128,   50,  -75,  -50,   62,  -91,  -87,  115, -101,   67,  -58,   28,  -77,  -20, -123, -121,  -30,  -89,
   68,   30, -105,   31,    1,   28,   54,  -11, -120,  -21,   46,  127,  -33, -100, -125,   70,  -73,  -76,   83,   86,  101, -105,   49,   51, -116,
   97,    0,   61,  -69,  -21,  -37,   94,  -88, -115,   61,   48,  123,   -2,  -71,   61,   55,  127,  -21,  -57,  -33,  -71,   -3,  -65,  114,  -23,
   -1,  -47,   87,  -81,   59,  -27,  -44, -125,   66,    1,   -5,  -10,  -11,  -99,  120,  -42,   85,   79,  -84,  126,  -86,  -15,   82,  -87,  -39,
 -108,   19,   91,   62,   54,  127, -109,   49,  -47,  -63,   26,  -78,  -50,   57,  114, -123,  -51,    4,  -66,   96,   47,  -75,   66,  115,   77,
   75,   87,    8,  108,   32,    5,  -92, -114,   34, -121,   20, -112,  -33,  119,   31,   37,  -78, -125,   90,  -35,  -96,   22,   37,   11,  114,
   64,    5,  112,  -32,   10,   96,   35,    7, -122,    2,   82,    3,  114,  -96,    6,  -51,   73, -117,   10,   23,   45, -103,  -74,   90,    9,
  107,  -48,    1,  -44,  123,  -14,   98,  119,  -65,   47,   13, -123,   26, -119,  -10, -128,   89,   34, -111,  -23,   11,  115,  -64,  -96,   42,
   58,   28,   90,   28,  -84, -119,  -25,   75,    2,   32,    2,  106,   57,  122,   -6,   83,   11, -108, -124,   84,  120,  -12,  -33,   63,   18,
  -24,   60,   66,  -18,  -47,  -97,  -93,  -16,    0,   71, -120,  103, -126,    7,   -7,  106, -123,    6,  -75, -117,  -85,  -28,  125,  -75,  125,
   59,  123,  -22,  123,  107,   -1,  -69,  126,   89,  -74,  -75,  -46,   58,  106,   80,  -17, -106,  -67,   69,  111,  -19,  -49,   44,  106,   84,
   11, -122,  -17, -113,  107,   31, -104, -120,  -30,  127,  -34,  -39, -121,   45,   53,  -96,  -12,   27, -115,   70,  -98,  -28,  -22,   16,   77,
 -124,   68,  108,  -93,   35,  120,  -16,  -78,  116,   14,   38, -117,   45,   31,   78,  -42,  -46,   10,   68, -122,   17,   73,  -84,  -72,  104,
   57,  105,   97,  -66,  -48,  -94,  102,  -77,   10, -111,   77,  -54,  101,  -92,   56,   86,   61,   -6,  -11,  -10,   97,   83,  -58,   78,   61,
   57,   45,  -55,  -42,   66,  -14,  112,   45,  -87,  -10,  -17,  -37,  -78,  101,  -35,  -93,   -3,   -5,  -74,  -75,  -76, -115,   24,   53,  -23,
  -56,  -74,  -42,   17,  -87,  100,   60,   -7, -101,   91,    7,   15,   59,   96,  -14,  -12,   19,   89,  -79,  118,  -11,  -81,  119,  110,   95,
  123,   76,  -25,  -59,    4,    4,  -65,  -26,  -19,   59,  -42,  -83,   -4,  -35,  125,  -31,   86, -121,  -52,   62,  121,  -36,  -56,   41, -122,
  -24,  -15,  -89,  -18,  -35,  -72,  117,  -51,  -21,   79,  126,  -73,    5,  109,   27,    9,    2,  106,  -67,  -67,  -21,  -98,  121,   98,  -35,
  -45,  -53,    1,   12,   30,   62,  102,  -18,   49,  -25, -114, -102,  112, -124,   72,  -83,  103,   -9,  -70,  -91,   -9,  -33,  -74,  115,  -21,
   11,    0,  -90,  -50,   61,   97,  -11, -109,  -65,   62,  -26,  -76, -117,   39,   77,   63,  -46,  -40,  -52, -104,  -52,   57,  -69,  -18,  -39,
  -91,   -9,   -2,  -16,  -13, -121,   31,  127,  -34,  -29,   15,   -2,    8,  -64, -124,   41,  115,  -33,  124,  -55,  -89, -121, -115, -102,  -48,
   90,  -51,   42,   85,  119,   -6,  113,  -29,   15,   24,  -37,   74,   68,   39,  -99,  118,  -34,  -25,  -66,  -16,  -99,  -10,   22,   91,  -83,
  112,  107,  -59,   86,   51,  -54,   28,   89,    3,   99,  -56,  123,   45,   60,  -22,  -71,  -10,  -41,  -75,  -73,   86,  -12,  -41,  -92,  -89,
  -81,  -24,  -19,   -9,  -67,  125,  -11,  -66,   -2,  -94,  -89,   47,  -17,  -17,  -81,  -41,  106,   69,  111,  127,   45,  -49,  -13,  -94,  -16,
  -11,  122,   45,   47,  -22,  -11,  122,   94,  -28,  -11,  -68,  -16,  -66,  -88,  -41, -117,  122,    8,  -31,  -32,  125, -120,  -33, -112,   67,
   10,   47,   57,  -87,   80,  -37, -100,   51,  -54, -114,  125,  -54,  -12,   51, -105,  -81,   95,  -66,  -83,  119,   99,   60,   34,   73,  -62,
   66,   83,  -25,   23,  -76,  103,   29,  111,   61,  -25, -100,   75,   47,   62,  123,  -14,  -76,   65,  -58, -127, -119,    3,   96,  -11,  -18,
  -85,  -65,  -16,  108,  109,  -12,  -28,   76,    5,  -42,  -47, -106,  -19,  123,  -82,   -4,  -56,   45, -121,  -52,   62,  -16,  -38, -113,  -66,
  -74,  -20,   86,   55,  -35,  116,  -49,  101,  -41,  -33,   80,  -50,   88,  -91,  120,   21,  -77,    3,  105,   -8, -110, -107,  -73,  -80,   12,
  -21,   50,   75,   78,   50,  -21,   33,    5, -117,  -10,  -25,   82,    4,  -64,  -94,   36,    1,    1,   69,   14,  -87,  -93,   40,  -10,  -25,
  -17,  110, -116,  -21,  104,  -79,  -83,   85,  -91, -116,   52,   83,  -82,   16,   85, -108,  -86,   68, -103,  -62,   16,  -68,  106,  -99,  -76,
   95,  -75,   70,   82,   83,  -86, -109,  -42,   43,   70,   90, -115,  111,  -79,   26,    0,   75, -127,  -68,  -65,  -98,  -17,  -82, -117, -105,
  104,  115, -115, -106,  -41,    8,  -18,    3,   62,   18,   70,  -75,   33,   99,   84,   51,   24, -114, -128, -123,    4,  115,   97,  110,  -39,
  -42, -109,  -76,   97,   70,   80,   64, -126,  -87, -127,  -53,    6,  105,   -6,   42,  -72,   62,  120,   65,   95, -114, -102,   79,   11,   -9,
  -61,  -27,   21,  107, -121,   15,  110,  -99,   60,  -90,  -13, -119, -114,    7,  -26,  -20,  -39,  -74,  106,  -45,  -98,   77,  -69,  125,  -19,
  -65,  -25,  -14,   -1,   63, -109,   58,   14,   24,   -2,  118,   28,   -1,   45,   60,  -72,  -25,   -7,  -19,  127,  102,   81,   99,   90,   48,
  -84,  -14,  -57,   78,   74,   75, -108,   73,  105,  103,   13, -101,   -5,   27,   86,  -26,    6,  -77, -108,   -8,  -86,  -80,  -54, -114, -116,
   33, -104,  -88,   51, -126,  -56,   85,  -64,   54,  -84, -122,  -93,   82,   84, -118,  -31,   99,  -94,  -17,  104, -112,  -65,  -72, -108,  127,
    1,  -93, -123,  -26,  -71,  -75, -114,  -39,   52,    8,  -84,  -12, -105,   75,  -15, -116,   24,  -34,   23, -102,   83,  -84, -116,  -78,   74,
  -30,   65, -119,  -93,   48,    8,   19,   -7,  -75,   16,  115,    1, -122, -104, -128,   96,  -70,   51,   28,   36,  -68,   16,  120,    1,   12,
   98,    6,    3,   22,    8, -114,  -56,   65,  -51,  -78, -128,   81,  -52,   62,  -17,  -16,  107,  -34,   -1,   46,    0,  -89,   95,  116,   89,
   88,  -58,  -19,  -78,   97,   31,   -2,  -89, -117, -114,   59,  114,  -22,  103,  110,   94,  -12,  -32,   35,  -53,   20,  108,  -40,  -98,  112,
  -12,   33,   87,   94,  122,  126,   -9,  -46,  103,   63,   -1,  -51,  -97,   91,  -29,  -40,  100,  -50,   57,   99,  -78,  -52,   57,   99,   93,
 -106,  101,  -50,   85,   42,  -50,   89,  103,  -99,  -85, -116,   26,  -38,  122,  -28,  -36, -111, -121,   76,   31,   -6,  -47, -113,   93,  127,
  -35,  -57,  -81,  -70,  127,  -23,  -50,  -74,  -86,  105,  -85,  -38, -106,   42,   87,   51,   30,  -44,  106,  -38,   90,   24,   64,   81,  -88,
   40,   10, -113,  122,   46,   -3,  117,  -23,  -81,   75,   95,  -65,  -20,  -21,   47,  -10,  -11,   -5,  -66,  -66,  122,  111,   95,  -79,  -81,
   63,  -17,  -81,  -43,  107,   -3,  121,  111,  -83,  -98,  -25,  -11,  -94,  -16,  -11,  122,   61,  -49,  -21,  -11,   60,   47, -118,  122, -111,
   -5,  -62,  -41, -117,  -68,   86,   -8,  -36,   23, -123,   -9,  -71, -120,   -9,   62, -121,   20,   34,   57,   84,   44,  -40, -106,  -31,    6,
   90,  108,   71,   79,  -66,  -73,  113,  -92,   36,  101,   66,  -93,   11,   20, -128,   65,   79,  -34,  123,  -13, -113,   22,  -34,   -4,  -61,
 -123,  115,  -57,  -49,   -3,   -9,   27,   -2,  -15,  -16,   35,  -58, -121,  126,  -16,  -36,  111,   -5,   54,  -81,  -83,   15,   29,   99,  -33,
   -3, -111,    5,   63,   94,  -68,  100,  104,  -37, -112,   75,   46,   60,  123,  104,  -53,  -48,   90,  127,   94,   73,  -21,  117,   46,  120,
   -3,   81,   31,   -8,   92,  -78,  -43, -105,  -18,  -93, -127,  101,  111,  -94,  -34,  -87,  -23,   91,    0,  -96,  -88,   18, -126,   69,   13,
   17,  -64,  -86,  -60,  -46,  -84,   -8, -112, -127,    6,   37,  -47, -128,   -4,  126,   56,   44,  -78,   22,   92,   81,  -54, -120, -100,  -78,
   13,  104,    5,  106,   81,   83,   37,  100, -118,   58,  -68, -119,    1,   23,  -56, -121,   85,  -32,  -52,    5,   12,   17,  107, -104,   51,
 -107,   64,    6,  -28,   52,   88,  124,   52,   57,   94,  -95,  -12,  -70,   98,  -88,   68,   58,   60,  118,   85,   30, -128,  -89, -111,   89,
   47,  -83, -124, -119,  -71,   75,  100,   85,  -54,  104,  -45,  -57,    6,  -31,    7,  112,  -28,  -11,   27,  -28,   98,   24,   78,  -82,  -30,
   90,   91,   78,  124,  122,  -24,  -84,   89,  -57,  -14,  -54,   37,   63,   28,  -68, -101,  108,   63,   -2,   87,   87,   68,   -9,  110,  -17,
   -1,  -42,  -56, -121,  123,  -73,  -10,    7,   66,  -13,  -49,   73,   66,   80,  -13,   71,  -49,   74,   49,   21, -120, -124,    7,  -76,   97,
   73,   63,  105,  105,    3,   35,   82,   16,  -40,   16, -116,   82,   92,  -73,   10,  -78,  -60,   54, -118,   92,  -15,  -84,  -90,   63, -118,
   96,   79,    4,  -95,   41,    4,   86,  -24,  114,   94,  -43, -128,  108,  -29,   40, -110,   46,   63,  -64,   45,   66,  -96, -106,  -94,  -54,
  -87, -128, -121,   34,   -6,   26,  -91,   16,    2,   13,   15,   85,   34,   82,   40, -125, -119,   83,  -24,    8,   72,   84,   71,  -88,   92,
 -127,   24,  113, -114, -126,   -3,   42,   44,  -35, -126, -126,    6,  -17, -114,  -19,  -75,   58,   82,   79,   34,    0,    0,   26,   77,   73,
   68,   65,   84,   -8,   63,   -2,  -19, -116, -117,  -81,    8,  -52,  104,  -88,  -38,  -46,   -5,   23,  -67,  -72,   53,   31,   55,  101,  110,
  -87,    0,  110,   88,  -67,  -84,   -5,  -25,  -33,   60,  -15, -100,  119,    7,   41,  -46,  -80,   38,   16,   71,    8, -118,   18,  -86,  -42,
  -45, -105,  -33,  117,  -33, -118,  -85,   62,  -12,  -35,  -69,  126,  -12,  -75,  -85,   63,  -15,   85,   85,   21,   85,   81, -120,  -64,  107,
   67,  -96,  113, -114,   -9,  -11,   73,  -31,  -43, -121,  -29,    2,   81,   72,   92,  -58,   22, -103,  -34,  -28,   34, -110, -102,   42,  -68,
 -115,  -88,   30,   10,   60, -120,   65,   18,  -52,  -69,   80, -115,   13,  -81,    0,   84,   44,   40,   43,   71,   -3,  -79,   83, -114,   91,
   -4,  -20,   47, -108,  124,   24, -124,  -64,  -17,   25,  -17,   99,  -33, -120,  -33,  -18,  -87,   -9,   12,   25,   82,   13,  -30,   85,   94,
   20,  -21,  -98,  -83,   25, -122,   -9,   58,  -76,  109,    4,   40,  -37,  -79,  -81,   -9,   -6,  -81,   47,  106,  -83,  -76,   28,  116,   -8,
  -88, -109,   79,  -98,   25,   46,   29,   62,  -68,  -19,  -94,   51,  -49,  -67,  -11,  -18,  -69,  -53, -105,   26,  123,   67,   41,   91,   53,
   51, -105,   84,  126,  100,   86,   75,  -56,   64,   94,  -45,  -92,   23,   60, -115, -101,   28,    4,   18,   16, -124,   49,  -65,   31,  -73,
  111,   11,  -74, -128,    3,   89,   80,   69,  -55,    6,  -33,   80,   66,   11,  -52,   32,  -14,  123,  -43,    8,  124,   93,   97, -127,   74,
   96, -101, -126,  -45,   90,  105,   41,  -91,  -80,   32, -115,  124,   48,   44,   70,  -76,   74,  120,   68,  -23,   17,   74,  -82,   42,   46,
  -93,   47, -125,  -57,  -92, -107, -104, -102,   58,   91, -119,   83,  -31,   35,   53, -105, -109,   44,   30,  -15,   83, -110,  -80,   40,   73,
  103,  -51,   99,   82,  -60,  -28,   -3,   -6,  -85,   89,  -67,  -12,  -60,  111,  -18, -101,  -37,   91,  -84,   82,   81, -125,  -76,   92,  -15,
  127,   37,   21,  -67,  -70,  -25,   -7,  -67,    0,   -2,   -4,  106,  -44,  -47,  -36,  -67,   -9,  -97,  -76,  -47,   39, -112,   39,  103, -124,
   38,  -33,   -9,   82,  -73,   35,   68,   12,   49,  -92,  -92,   76,  113, -100,    7,   25, -118,   67,   68,  -85,  -78,  -52,   38,  -87,   31,
   73,    5, -120,  -34,   42,   37, -109,  -95,   74,   89,  -16,  -18,   85,   78,  -80,  -38,  -76,  -34,   42,   94,  -82,   16,   72,   67,   -3,
 -124,  -86,   50,    5,   95,  -60,   36, -123,  105,  122,  -53,  -91,  -58, -111, -108,   88,   66,  -22,   27,   96,  104,  -32,  113,  -45,   97,
  -27,  -12, -120,   41,   60,    3, -108, -120,   26,    0,   31,  -30,   93,  113,   57,  -92,   99, -127,   10,  -46,   70,  -84, -127, -112,   39,
  -60,   62,   74,    1,   68,   34,  -54,    0, -127,   26, -107,  -63,  -61,   70,  -65,  -15,   93,   87,  -65,  -29,  125,  -41,  -74,   85,  -99,
 -118, -118, -105,  -36, -117,   41, -120, -115,  120,  -33,   16,   16,  -22,  -71,  120,   65,   94,   72,   94,   72,  -67,  -48,  -36, -117,  120,
   81,   81,   17,   17,   13, -127,  103,   16,   87,  -29, -122,   17,   22,  110,   17,  -86,   23,  111, -115,  -44,   56,   33,   15,  -91,   24,
  -57,  -55,   42,  -71,  -78,   77,  -37,    7,   87,  -38,   43,  -61,  106,  -67,   91,   75,  -46,  122,   96,  108,   62,   52, -126,  -73,   48,
  -96,   56,  -21,  -17, -113,   59,   96,   74,  116,   63,  -37,  -69,  -61,  111,  -38,   88,   76, -104,  -24,    0,   76,   29,   49,   77,  -23,
  -66,  112,   78,  111,   81,   44,  121,  108,   77,    9,   88,   89,  -59,  -98,  119,   -6,   49,  -73,   44,  -66,  -73,  -47,   51,   74,   59,
   14,   53, -112, -117,  -46, -101,   74,   13,  106,   64,   22,  -20, -108,    8,   76,  -86,    4,  -42,  114,  -88,   75,  -54,  -88, -119,   65,
   89, -108,  -46,   -6, -102,  -90,  -92,   28,   10,   49,   96,   75,  100,   52,  -72,  -31,   40,  -87,   81,  -62,   62,  101,   37,   79,   48,
   86,   37,   39,   53,   80,   27,   58,  -74,   50,   52,   44,  -60,   12,   -2,   10,   76,   10,    7,  -93,  -23,   61,   54,  -40,   -9, -120,
  -74,  -31,   27,    2,   20,  125, -126,  -32,   74,   71,  -55, -116,   88,  -38,  -41,   73,  -47,   95,    0,   38,   65,   18,  -91,  -85,   18,
   18,   49,  -57,   56, -119,  -47,   49,   53,  -11,    7,   97,  -88,   79,   -9, -118, -114, -112,  -16,   57,  -22,  -69, -117,   -2,   53,   59,
  110,   31,  -42, -109,  -81,  -87,   -9,  -19,   44,  -92,   32, -104,   63, -111,  -87,   -2,  127,   61,  -11,  121,  -28, -124,   63,   34,  -89,
  -91,  -66,  -28,    5,  -67,   66,  -47,  -21,   42,  -83,  -20,   75,  -63,  -80, -102,    9,  -45,  -90,   88,  -99,  -31,   28,   50,   96,   67,
   77,   98,   81,  115,  124,   17,  -27,   50,   58,   41, -105,   10,    1,   64,  -22,   61, -111,   37,   34,    1,   71,   -1,   -5,   38,  -55,
   55, -104,   75,   72,   81,  104,   30,   13,  -40,  -95,   76,    1,   72,    8,  -86,   73, -102,   43,  -41, -100,   10,   55,  -80,   16,  -54,
   48,   40,   67,   16,   68,  -60,   52,   72,   75, -126,   26,   48,  -82,   68,   74, -112, -120,   89,  -76,  -28,  -41,  -65,  -99,  -69,   -8,
  -63,   87,  -65,  -26,   -8, -123,   11,   23,  126,  107,  -15,   50,    2,   62,  -12,  -98,  121,  -99,   71,   76,    3,  -80,  112,  -31,  -62,
  -93,   79,  -99,  -57,  -58,    0,  -72,  -30, -110,  -13,    1,  -52,  -97,   63,  127,  -30,  -12,  -61, -120,    0,   82,   38,  -99,   50,  113,
   -8,  -56,   97,   29,  -52,   -4,  -28,  -86,  -19,  -60,   10,  120,  -64,  -88,  -22,  -76,   73,   29, -103,  -75,  -69,  122,   -4,  -72, -111,
   45, -107,  -52,  102, -106,    5,   92,  120,  -15,  -98,  -13,    2,  -71, -105,  114,  -56,  -43,   10, -127, -110,   49,  -88, -128,  -59,   75,
   79,  -97,  -28,   30, -123,   -9, -125,  -38,  -35,  -32,   65,   54,   47,  124,  -98,   23,   59,  118,   -9,  -83,  -35,   88,   67, -112,   14,
    3,   24,  -79,  112,   12,   29,   19,   69,  -68, -112,   97,   18,   79,    0,   69,   39, -107,   38, -107,   16,  104,   27,  100,   38,   12,
 -103,  -68,  -67,  127,  103,  122,   55,  -55,   63,  -85,  -44,  -67,   76,  108,   86,   85,   88,   99,  -25,  -97,  127,  -92,  -77,  -79,   35,
   61,   -9,  -52,  -34,  125,  123,  -10,  -39,  -54,   80,  -21,  104,  123,  -33,   94,   50,   54, -119,  113,   -8,  -54,   15,   22,   95,  121,
   -7,  105,  -27, -103,  -89,  -99,   54,   99,  -22,  -24,  105,  -85,  -73,  -83,   77,  -14,  121,  -71, -100,  -94,   97,  -11, -117,   56,  -39,
   16,   64,  -84, -110, -123,  -79,   68,   68,  108,   32,  -84,   70,   84,   20,  101,   -4,   22, -114,   24,   29,   84,  -62,  -88,   52,  -67,
   52, -103,   80, -120,    6, -106,   61,   44,  -33,   99,   33,  -44,   20,   76,   16,  101,    9,  -21,   87,   67,   92,   30, -104,  -24,   97,
   27,  124,   74,   97,   82,   36,   86,  -14,   96, -115, -117,   10, -109,  -29,   59, -107,  117,   78,  -46,   40,    1,   -5,  114,  -76,  103,
  -16,    4, -109,  124,  -90,  -54,   71,   83,  -96,  -73,    6,   54,    9, -116,   82,  104,  -99,   82,  -98,  -46, -124,  -38, -102, -120,  -83,
   40,   -4,   11, -124,   35,  -28,   69,  -69,   85,  -80,    6,  -12,   22,   -3,  -67,  125,    5,   -6,  -67,   87,  -55,   11, -112,   -3,  -29,
   98,  -55,   95,   73,  -86,    1,  -69,   11,   12,   -1,  -61, -126,   90,   24,  -64, -124,  -35,    5,   -6, -107, -109, -124,   65,    3,  -32,
   35,   30,   43,  -69,   26,   43,   51,   83,  -44,    9,   21,   76,   74,  106,   44, -105,  -45,  103,    2, -100,   88, -110,   54,   65,   85,
  -80,   53,   74,    1,   88,  112, -120,  -51,   70,   33,   34,   41,   74,  -47,   46,   21,   82,   80,    1,   53,  -32,  -24,  -94,   12,    6,
 -111,  -86,  120,   77,   51,  -76, -106,    4,   46,  -89,   24,   90,  -95, -122,    4,    1,   76, -118, -127,  -38,  120,  -94,  -24,  102,   22,
   36,   34,   72,   84,   71,  -47,    8,  -60,   66,  104,  107,   29,   28,   46,   -8,  -35,   29,   63,   58,  -24,  117,  111,    8,   79,  125,
  -29, -115,   55,  118,  117,  117,   -3,  -26,  -98,  -37,  -49,  -97,  -15,  -15,  -26,   70,   57,  -15, -100,  -73,   19,  100,  -38,    1,   35,
  -50,   -8,  -69,   67,  -89,   78,   28,   53,  116,  112,   43, -128,   95,   63,  -68,   98,  119,  -83,  -14,  -56, -118, -115,   74,   10,   -8,
  -77,   79,   60,    0,  -64, -102,  -25,   95,   60,  -16, -128,   17,  -31,  -86,  -35,  123,  122,  -18,  127,  104,  -27, -124,   41,  -77,    1,
 -101,   55,  -15,  -91,  125,   53,   25,   59,  -52,   85,   50,    6,  -16,  -35, -123,  119,  -43,  114,   55,  117,  -26,   17,   19,   70,   87,
   39, -113,  109,  109,  -87,  -60,   23,  -77,  125,  -25,  -98,  111,   44,   92,    9,   30,  -87,  -16,  104,  -24, -119,  -31,  -73,   48,  -87,
 -112,  -80, -127,  -90,  127,   18,   34,    2,  -88,   50,  -79,  -45,  -16,   67,  -82, -106,  -41, -113, -102,  116,    8,  115,    5,   49,   82,
  -99,   83,   56,   85,  -89,  -28,  -64,   14,  -20,   64, -115,  -97,  -29,   14,   59,  -24, -120, -125,   15,    8,  -73,   47,  -14,   98,   -7,
  -93,  -69, -122,   12,  110,  109,   27,  100,   92, -123,  -98,  -37,  -75,   78,  -39,    1,  -15,  -52,   93,  123,  -22,  -65,  121,  -92,   17,
  -64,  -81,  -91,   37,   59,  -21, -116, -125, -120, -100,  -90,  -97,  120,  -69,  -90,  -62,  -61,  -19, -108, -100,  -86,   83,   56,   37, -117,
 -128,   89,  100,   67,  -59,   64,   89,   56, -109,  -40,   17,   59, -126,   83,  118,  -60,   14,  -22,   64,   46,   69,  -38,   27,   -8,   35,
   86,  -59,   16,   44,   81,  -16,   83,   32,   85,   79,   90,  -68,  -28,  119,  -44,  -49,  -56,   16,  -84, -112,   21,  -74,   65, -105, -116,
  -66,   96,  106,   85, -126,  -45, -115,  -93,   24,  -48,  -54,    1,  -87,  113,   82, -122,  -32, -120, -100,   87,  -73,  -77,  -26,    4,   14,
  -95, -110, -108,  126,  -32,  -10,  -43,  -99, -124,   60,  -89,  -33,  112,   76,   14,  -20,   56,   53, -126,  -90,  111, -107,   29,  -91,  -10,
   17,  117,  -94,  -87,  -71,   16,  -49,    9,    7, -117,   58,  -27,  125, -112,   58,   65,   -9,   -9,   -8,  127,  -51,   63,   59,  -21,  110,
  -97,   12,  -20,   33,   -5,   -7,  -55,  122,  125,  -74,  -77, -106,    1,  -79,  -99,   65,   22,  100,   41,   56,   27,   34,   54,   23,   16,
   98,   -5,   56, -112,   33, -108,   97,   20,  109,  -16,   73,  -47,  -16,  -70,  -55,   18, -123, -119,  -51,  -58,  121,   14,   22,  -60,   97,
 -107,   24, -111,   37,   54,   80,   27, -108,   47,  -91,  -16,  118,  -84, -128,   10,  -15,   10,   82,   34,   97,   35,  100,   84,   33,  -30,
  -95,    6,  100,  -95,   54,   86,    9,   86,   99,   63,  -79,  -60,   54,  -60, -124,   72,   10, -124,   85,   88,   98,   75, -100, -123,   -2,
   70,  -28,   66,   53,   68,  -61,   35,  -60,  -34,   40,  -28,   84,  -29,  -45,   53,   13,   13,  -85,  -80,   10,   11,  -75,  -49,  -82,  124,
  -95,   68,  -24,  115,   39,   31,  124,   -4, -111,    7,    1,   88,  -78,  100,    9, -128, -123,   11,   23, -110,  113,  108,   28, -128,  -32,
   81,   21,  -64,  -30,  -93,  -17,  125,  -35,   81,  115,   39,   63,   -9,  -36,  -86,   65,   67,   70,  116,  117,  117,  -99,  112,  -52,  -63,
  103,   -1,  -35,   12,  -41,  -65,   90,   85,   74,  -35,  106,  -13,  -58,  -75,  -45,  103,   31,   65,   68, -117,   22,   45,   26,  -36,  -47,
  -66,  113,  -19,  -78,  -57,   31,  125,  -72,  -81,   94,  -28,   69,   60,   97,  -62, -124,    9,  -93, -121,  -70,   74,  -58,   47,  110,  -38,
  -44,  -39,  -39,   -7,  -74,   11,  -49,  -86,   23,   50,  108,  -80, -103,   61,  121,   80,   75,  -59,   12,   31,   57, -114, -120,  -70,  -69,
  -69, -121,   15,  -19,  -72,  -14,   61,  -25,  -82,   91,   -3,  -92,   66,    1,  -81,   17,  120, -125,   62,   24,  -61,  -19,  -58,  117,   55,
   36,   96,  -80,  113,   48, -114, -116,   99,  -91,   12, -100, -127,   50, -104,  -84,  123,  -13,  111, -114, -103,   61,  123,  116,  -37,  120,
 -124, -125, -100,  -63,  -60,   12,   81,   70, -108,   17,  -57, -109, -103,  -77,  127, -102,  119,   86,  107,   91, -100,  -11,  118,  108,   41,
  122,  -74,   82,  117, -120,  109,  -17,   96,   21,  -12,  -20,  -95,   80,   96,   -8,  -23,  -55,   -3,    3,  -65,  121,  -74,    4,   44,   99,
   -8,  -78,   55,  -97,   13,  -50,  -56,  100,  100,   82, -103,   38,  -29,   80,   13,  -50,   40,  -35,  -82,  -87,   14, -114, -116,   35,  118,
  -32, -116,   40,  -67,   48,  -54,  -64, -103,   82,  -90, -108,  -87,  -55, -120,  -78,   -8,   44,   72,   87,  -67,  -12,  -57,  -62, -109, -105,
   32,  113,    6,   85,   77, -107,    5,   84,    7,  -22,  -96,  -70,  -78,   68,   29, -104,   76,  -12,   89,   99,  -85,  -80,  -62,  -50,   26,
    7,  118,  -86,   22,   98,   40,  -36,  -53,  100,  -60,  -79,   -2,  -60, -103,   82,    6, -109,   50, -100, -111,  -51, -108,   51,  -40,  -84,
  -49,  103,  -37,   -6,  -77,  -66,   34,   35, -109,   49,  103,  -52,   89,   93,  -77,   93,  -11,  -84,  -90,   25,  113,  -90,   38,    3,  103,
   76,   25,  113,  -90, -100,   41,  103, -108,  106,  -50,  -95,  112,  -54, -120,   50,  -26,   88,  -72,  -41,   76,   40,   19,  -54,  -56,  -60,
  107,    9, -103,  -79, -103,  112,   38, -108,   53,  -73,   -7,  -33,  -40,   79, -114,  108,  107,   45,  -21,  -43, -105,   63, -121,  -77,  125,
  -30,   54,  -41,   92,  -99,   28,   25,  -85,   92,   90,  105,   28,  -40,  -63,  -72,  -40, -105,   56,   83,  -74,   96,   71,  108,  -63,   78,
 -115, -123,  113,   48,   14,  -58,   18,  101,   80,   19,   48,   75,  -39,   42,   89,  101,   75,   97, -100,  -80,  -91,  -90,   94,   20,  102,
  -39,   96,  -42,    3,   59,   34,  -85,  108,  -43,   88, -126,   37,  -74,    2,  -14,   10,   81,   17,   21,  101,    3, -109, -127,   45,   25,
   71,  -58, -127,  -84,   26,  -85,    1,  -17,   40,   -8,   21,   90, -126,  -91,   80,  -49, -128, -113, -108,   41,  103,  -32,  -96,   82,   56,
  -80,   85,  -74, -126,  -32, -120,   99,   53,  -52, -102,  -31,   76,  -74,   18,   42,  -55,   38,  -71,   85,   27, -112,   33,   54,   96,  115,
  -41,   79,   30,    5,  112,  -20,  -79,  -57,   30,  122,  -28,  -95,  -32, -105,  -14,   35, -113,  -36,   -3,   -3,  115,   78,   57,   26,  -64,
 -126,    5,   11,   70, -116,  -99,   76, -116,   51,   -2,  -18, -112, -128,   95,  -81,   59,   -9,  -20,  -10, -114,   97,   50,  -14, -124, -113,
   95,   -1,  101,    0,  -89,   28,   55,  -25,  -55,  -57,  -18,   41,  -87,  -96,    5,   11,   22,   40,  112,  -53,  -99,  107,   48,  -20,   88,
    0,   -1,  120,  -23,  -69,   23,   -1,   -4, -121,  -75,  -70,  -81,   23,   81,  -60,   90,  -72,  112,   97,   75, -123, -105,   47,   95,  -15,
   -6,  -13,  -49,   95,  -77,  118,   -3, -105,  -66,  117,   -9,  -76,   89, -121,  109,  -35,   85,   -5,  -10,  109, -117,  -25,  -49,  -97,  -65,
   99,  -37, -117,   95,   94,  -12,  -28,   46,   25,   23,   78,   -2,  -22,  -11, -105,   32,   52,   20,   10,    0,   10,   79,   36,    4,    5,
   73,  -96,  -43,  -62, -102,   33,   50,    6, -122,  -55,   26,   54,  -58,   82,   32,   26,   21,    0,  118,  -17, -112,  113,  -57,  100,  -25,
  -51,  125,  -51,  -41,   30,  -65,   61,   47,   10,  -96,   97,  -74,  107,   44,   82,   83,  -64,  -32, -112,    3,  -90, -100,  -48,   57,  -71,
  124,   -2,  -89, -106,  -17,  109,  -87,  -14,  -56,   17,   60,  116,  -92,   89,  -73,  105,  -25,  -93,  -65,  -35,   72,    3,    9, -108,   91,
  127,   -6,  -40,   85,  -17,   63,  -71,  -12,   94,   25,  119,   64,  -11,  -72,   89,    7,  119,   63,   -5,  -69,   82,  102,    7,    6,   56,
   94,  -59,  -65,  -91,   60,   79,    6,   96,   53,  -42,    4,   43,  -95,  -90,   99,   37,  -53,   22,  -36,  -34,  114,  -96,   84,    9,   -9,
  -25,  -23,  -18,   61,  113,    1,   88,   83,  -82,  107,   33,   18,    5,  107,   36,  -21,   69,   19,  -63,  -92,  106,   64,   16,   85, -127,
 -111,  -88,    9,   66, -118,  -62,   -5,    2,  -28,  -94,  -43, -119,   19,    1,   23, -118, -120,  -70,   69,  -92,  -43, -125,   13,   83,   13,
  106, -126,   29,   53,  -12,   53, -101,   52,  121,  -64,  -77,   54,  -98,  -78,   57,   90,   73,   99,   57,   72,   -6, -106,   80,  104,   90,
  -30,   24,   22,   24,  114,  -76,   93,    2,   80, -122,   80,  -77, -104,   -1,  -73, -106,   -6,   20,   47,  -10,   97,  104,   21,   29,   14,
  110,  -32,  -53,   45,   60,  -10,   20,  -40,   89,   71,   93,   19,  -15,   20,  -56,  117,    5, -104,   27,   61,  -92, -119, -122,   71,   84,
  -36, -116,   18,  -91, -104,   50, -120,   70,   55,    1,  -84, -115,   76,   18,   65, -126, -111, -114,    2,  111, -103,   86,   78, -117,  -88,
   18,  -56,   70,   18,   42,  104,  118, -116,   68, -101,  -91,  -65,   58,  -64,    5,   44,  -36, -127,    4, -128,   39,  -30,  -46,   86,  -84,
  -51,  -34,   15, -108, -126,   20,  -91, -126,   99,   54,  114,  -20, -120,   43,  -29,  -61,    3,   16,   35,   26,   19,    3, -117,   30,   28,
   10,   73,   19, -105,   50,  113,  -30,  -60,  -74,   14,  -69,  -85,  103,    7, -128,   27,  111,  -68,  -79,  -69,  -69,   -5,  -55, -107,  -85,
    0, -128,  105,  -42, -108,  -72,   52,  103,  -42,   81,   39, -127,  121,   -6,  -28,  113,    0,   54,  108,  -40, -112,  -93,  122,  -58,   27,
   47,   83,  -14,  -69,  121,   44, -128,  -50,  -50,  -50,   -7,  111,  -71,  -28,  -48,   87,  -99,   82,  -74,  -10,  -55,  103,  -65,   45,   47,
 -118,   50,  -66,  -22,  -94,   91,  110,  126,  -49,  101,   31,  -85,  -25, -111,  -16,  -23,  -20,  -20,    4,  112,  -23,  -91,  -17,  -34,  -37,
  -21,  -81,   -7,  -44,  -41,   14, -100,  121,   72,  -83,   94,  -44,  114,  -33,   83,  116, -116,   60,  -16, -124,   79,   -1,  -37,    5,  -89,
   30,   51,  -95,  -83,  -91,   65,   72,  -86,   10,   65,   85, -125,   83,   80,  -94,  114, -126,   86,    8, -123,   33,  -80,   33,   77,   62,
   43,   94,  -30,  -30,  -25, -112,  126,  -10,  -60,  -61, -105,   93,  120,  -22,  113,   71,   76,   91,  -67,   -3, -124,   95,   60,   -1, -112,
  -24,  126,  -10,  116,    8, -118,   -9,  107,   79,   62,  100,  -28, -104,  -72,   58,  -82,  103,  111,  125,  -45,  115,   -7,  -88,    9,  -83,
   67,   71,  103, -125,   70,  -70,  127,   -1,  -54, -118,   61,  -75,   62, -118,   27, -117,  -60,  -76,   97,  -57,  -18, -107,  -53,  119,  -50,
   57,   44,  -60,  -39,   66,   37,  115,  -17,  122,  -61,  -15,  -35,  -97,   89,    5,  106,  -78,   63,  -90,  -33,  -44,  124,  -81,  112, -100,
   45,  -63,    0,   86,  -71,   32, -112,    6,   63,  -49,  -76,  100,   47,  -67,  -20,   16,  -28,   11,  -94,   47,   63,  116,    5, -110,   19,
  101,    8,  -92,   35,    0, -123,   80,  -94, -105,  -54,  -64,   50,   10,   67,  -28, -107,  -84,   64, -123,  -30,   18,   28,   41,    4,   57,
 -111,  -40,   72,   57,    1,   34,   17,  -25,   98,   53,   18,  -21,   17,   89,  -86,  -76,  -74,  -79,   80,  -28,    2,  -89,   47,   67,   27,
   55,  -81,   89,   75,  -71,  -26,  -70,  107, -118,   -6,   80,    0, -123,   71, -111,  -88,   43,   74,   38,   81,   47,   40, -126,  -79,  -31,
  111,   23,  -83,   66,  -86,    3, -101,   -5,  -79,   43,   71,   43,   35, -117,   33,   12,   81,   -9,  -24,   -9,  -24,   83, -108,   22,   56,
   32,  -14,  127,  -55, -104, -109,  104,  -11, -108,   85, -115,   75,    7,   82,  -80,  -50,  -24,   16,   16,   97,   35,  -38,  109,   24,   28,
   44,   -4,   77,    4,  119,  120,   73,  -34,   67, -128,   68,  -79, -105,   54,  -55,  -40,    1, -102,  -99,  -35,  -45,   65,  104,  -28,  -16,
 -119,  -96,  -28,  -93,  -53,  -67,  106,  105,   13, -120,  102,  -52,  -26,  -34, -100,  -70, -125,   38,   42,   61,   20,   24,  -65,   84,    4,
 -124,   10,   23,   73,  -80,  -99,  107,  -71,   62, -105,    0,  126,  -28,  -63,  103, -114,   62,  126,   70,   45,  -17,  -69,  -14,  -29,  -17,
    2,  -80,  100,  -55, -110,   65,  -93,  -89,  -17,  -23,  -51,  -25,  -51, -101,   87,  109,   31,  -84,   68,   27,   95,  -36,  -76,  126,   -3,
  122,   59,   -6,   32,   24,   87,  -10,  -71,  -74, -114,  -95,  109,   29,   29,  -86,   49,  108,    9, -128,   13,  107,  126,  -21, -101,  -36,
   26,   69, -118,  -70,   -9,  -51,   78,   51,   -3,   53,  -97,   39,    9,  -85,  -69,  -69,  -69,  -77,  -77,  -13,  -58,   27,  111,  124,  126,
  -85,   12,   26,   62,  -75,  -65,  -98,  -41,  115,  -83,   88, -102,  127,  -42,   97,   35,   47,   58,   22,  -64,  115,  -21,  -73,  108,  -34,
  -72,   97,  -20, -120,  -24,   76,  -18,  -75,   16,  -11,   97, -106,   39,   21,  -96,    0,  121,   16, -108, -103,  -32,  -96,  113,    8,  106,
   32,  123,   53,  104,  -47,   72, -117,  102,   13,  -66,  121,  -49,   47,  -33,  115,  -42,   73,   23,  -10,  119,   26,  -78,   -9,  -82,  127,
  -92,  -49,  -41,   67,  115, -105,  -20,   59,   17,   50,  114,  -77,  -58,   78,  121,   -4,  -41,  -69,  -43,  -61,   11,   -6,  -10,  -28,  -42,
  -16, -104,   73,  110,  -20,  100,  -73,  105,  -37,  -98,  -17,   -2,  124,  105,  112,  -67,  105,  -98,    5,  123,  106,  -11,  -61,  -33,  114,
   61,   53, -119,   22,    2,  -47,   20, -117,   29,   77, -127,  101, -102,  102,   24,   52,   -6,   31,   44,  -56,   68,  -49,   56,   38,   22,
   82,   86,   45,   89,  -10,  -40,   81,   16,   39,   22, -109,   92,   94,   -9, -105,  124,   29, -108, -127,   44,   49,   55, -112,   42,  126,
   71, -127,   89,   39,   85,   40,  108,  112, -110,   17,   10,  -63,  -43,   69,   11,  -15,   57,   41,  -37,  -72,  -42,   15,  -47,   58,   28,
   34,  -35,    7,   -4,   10,  -98,  -97,   18,  -19,  -77,  -48,   36,    0,  122,   70,   77,   65,   22,  -26,   -9,  106,   21,   78,  -32,  -28,
   62,  -46,   44,  124,   53, -125, -105,    0,  -11,    2,  117,   52,   12,   77, -102,   26,  -51,  123,   40,   67,   -4,  -33,  -78,  120,  -43,
 -100,  106, -126, -102,   52,  -19,  -50, -127, -124,   -9,   77,   72,   17, -125,  -81, -121,   47, -101,  -35,  -90, -112, -102,   52,    9,  -79,
   20,  125, -115,   12, -107, -106,  -73,  -48,  -66,    1, -107, -104,  -45,  -82,   15,   20,  -52,   -4,   36,  -63,  -79,  -58,   36,  -39, -118,
   75,   47, -109,    4,   41,  -87,   78,   84, -118,   -1, -115, -118,  -87,   18,  113, -120,  -18, -104, -100,  -53,   74,   56,   76,  -56,  -41,
   36,  126,   83,  -77,  -91,   56,  -64,  -97,   68,  -95,   62,   90,  -44,    4,  -80,  -63,  112,   89,   58,  115,  113,  -12,  -19,  -65,  -23,
 -122,   59,  -65,  115,   -4, -116,  -26,  -90,   27,   49,  126,  -78,  -51,   42,    0,   -6,  123,  118,  111,  127,  -31,  -71,  -19,   47,  -32,
 -119,  -89,  -98,   62,  113,  -58, -119,   97, -125,  -99,   70,   99,  -86,   16,   68, -102,  -62,   96,  106, -109,  -28,   82, -120,  -28,  121,
  -67,  105,   51,   83,  -12,  -43,  -21,  121,   30,   63,  -50,  -97,   63,   -1,  -89,   63,  -67,  -93,  -77,  -77,  115,  -41,  -35,  -65,  120,
  124,  -39,  -61,   83,  103,   30,   90,   20,  -14,  -22,   35,  -58, -115,   28,   90,   13,  -33,  118,  119,  119,  -97,  123,  -34,   -4, -101,
  -65,  120,   67,   44,   89,   84,  -75,    0,   36,   68,   33, -119,  122,    8,  115,  112,   28,   99,   97,   68,   42,   94,   17,   87,   62,
 -109,    3,  -46,   75,   82,  -36,   -6,  -53,   71,  -49,   63,  -31, -120,    3,  -25,  116,  -52,  -93,  -93,  -57,   46,   31,  -15, -117,   53,
   75,   55,  -10,  109,   45,  124,   17, -122,   40,   24,   16,  -44, -128,  -73,   -1,  -21,  119,   70,   84,    6,   31,   52,   98,  106,  -25,
  -44,   25,  -77,  103,   12,   29,  127,   96,  101,  -62,  100,  -41,   54,  -40,   92,   -1,  -71,  -57,  -97,  -33,  -78,   71,   -9,  103,   80,
   15,   29,  -84,  -31,  -35,   14,   67,  -63,  -86,   31,   91,  -71,  -47,   92,  -55,  -55,   41,   94,   16,   54, -107,   36,   50,   97,  103,
 -115,  -80,  -43,  -91, -124,  104,   13,  -27, -116,   68,   17,  -80,   26,   66,  -39,  -53,  -69,   26,   -6,   26,   96,   20,   45,  -90,  -36,
  111,   36,  -35,   58,   68,  -65,   79,   76,   86,   56,   57, -124,  -75,  -20, -121,  -85,  113,  -80,  -58, -124,  -59,  -43,  -31,  -18,  -91,
   72,   22,   -1,   82,  -60,  116,    4,  -55,   63,  116,  117, -123,   87,  -28, -128,   81, -104,   82,  -47,   75,   41,   45,   28,    9,   77,
 -110,   96,  110,   64,   39, -121,    2,  117,   65, -114,   40,   57,  106,  -14,  -58,    2,  -63,   16,  114,   65,  -95,   40,   -2,  -32,   35,
   -1,  -19,   39,   42,  -99,  -58,  -45,  -57,    0,   84,   37,  -95,   17,  -67,  -34, -109,    4, -124,  -40,  -43,   52,   42, -118,  113,  -65,
  -46,  -46,   19,  -80,   97,   -6,   45,  -33,  113, -120,  -14,   78,   28,  -93,  -11,   53,  -76,  -50,  -40,    1,   66,   64,  -73,  -80,  -67,
   88,  -44,  -27,  -76,   33, -100,   37,   60,   83,   13,  103,   52,  124,   50,  -93,  111,   87,  116,  120,    7,   16,  -29,   44,   17,    8,
   18, -107,  -36,  -72,  -16,   58,  -87,  -83,   84, -122,  -64,   73,   39,    7,   80,   13,  -34,   83,   80,  110,   56,   56, -123,  -32,   10,
  -13,  -25,  -49,   95,  -76,  104,  -47, -119,  -25,   -1,  -61, -105,  -66,  119,  -49,  -47,    7,   31,   24,   34,   11,   -2,  -20,  -25,  -65,
   24,   49,  126,  -54,  -16,   73,  -45, -120,  105,  -43, -122,  -83,  -81, -102,  123,  -32, -124,    9,   19,  -58, -116,   30,   37,  -52,   32,
   61,  116,  -26,   56,   36,   86,   94,  -92,   33,   80,  -87,  120,  -17, -117,  -68,  105, -116,  -44,  -21,   69,  -67, -120,  -24,  -74,   97,
  -61, -122,  125,  -38,    1,  -32,  -52,  -45,   79,  -37,  -71,  -21,   63,   -5,  -21,  -71,  120,   95,  -94,  -43,  -94,   69, -117,   46,  122,
  -33,  117,  111,  -70,  -72,   17,   38,   76, -112,   11,   84,  -44,   71, -107,   48,  120,  -58,  -59,  -75,  112,  -94,   28,   66,  -51,   69,
   85, -102,    9,   49,  -68,   76,  -71,  -48,   95, -127,  -85,  -65,  -15,  -13,   -1,  -72,  102,   -2, -108,  -71,   85,  -37,   58,  109,  -38,
  -72,  -79,   79,  -81,  -33,  -76,   98,  -13,  -70,  -51,  125,   59,   -6, -117,  -70,   42,   42,  -58,   13,  -55,    6,   77,   28,   52,  114,
  -10,  -72,  113,   51,  -58,   15,   29,   61,  -79,   50,   98,   66,  101,  -62,   36,   59,  116,  -84,   -5,  -30,   45, -113,  126,  -31, -121,
   15, -127,   51,  110,  -26,  -68, -102,   59,   85,  -77,    8,   80,   -6,  124,   82,  -45,  -15,  -26,  124, -125,  -63,    9,  113,  -12,   13,
 -104,   97,   10,   37,  -46,   16,  -67,  -80,   89, -111,   76,   61,  -84,  113, -121, -105,   23,   55,  -44,  -93,  -24,   35,   40,   80,   97,
  114,    3,  -49,   83, -128,    6, -116,   -5,  -36,   43,  -43,   10,  -28,   97,   51,   40,    0,  113,  -58,   21, -115,   14,    7,   49,   79,
  -47,   49,   87,   76,  124,   46,   54,    1,   -4,  -30,   88,   42,    4,   57,    0,   66,   21,  104,   14,  -56,  -36, -116,  -71,  -55,  -67,
  120,   32,   16,   43,  114,   65,  -35,  -93, -112,    8,  109,   92,  -74,   18,  -59, -115,  -64,   36, -122, -120,  127,  -39,   71,  126,   37,
  -92,  -24,   24,   69,   81,  -14,   45,   61,   93, -108,   26,   61,   47,  -55,  -66,    8,  127,   73,   65,  106,  -30,   98, -104,  -28,   56,
 -110,   38, -115, -122,   98, -106,  -10,  -83,    8,  -46,   15, -107,  -86,   91,  -36,  -11,   15,   17,   79, -112,  116,   52,   66,  116,  -65,
 -118,   -8,  -94,   37,  -93,   21, -122,  -92,   82,   35,   78,  110,  -86,   86,  -55,   80,   53,   42,    7,  -96,  -95,   -8,   43,   65, -108,
   56, -100, -106,  -88,  -74,  -80,  -23,   32,  -72,  -15, -104,  -79,  -86,   81,    2,  -96,   85,   79,  111,  -98,   54,  115,  -12,   -6,  -11,
  -21,  -41,  -81,   95,  127,  122,  -25,   37,   35,  -26,   30,    9,  -63,  -86,  -11,   91,  -90,   77,   28,    5,  -32,  -34,  -59,  119,  -75,
  -17,   99,  -78,   86,   65,  119,   63,  -78,  -14,   77,  103,   28,  -45,  -39,  -39,   -7,  -42,   55,  -66,   -2,  -95,  -43,  -69,  -89,   29,
   52,  -13,  -19,  -81,   59,   30,  -64, -126,    5,   11,  -58,   79, -103,  -27, -101,   36,   44,  -81,   90,   -8,  -94, -103,  -16,  -23,  -81,
  -41,   75, -107,   16,  -64,  -22,   77,   -7,  -88,  -43,  -21,  -90,   79,  -99,  -12,  -26,   11,   95,   -1, -102,  -13,  -34,   -2,   -6, -117,
  -81,    8,  -57, -113,   61,  -10,  -40,  108,  -12,   97,  -81,   57,  -29,  -36,   19, -113, -102,  -42,  120,  107,  -22,   37,   58,  -52,   19,
  -79,   33,  -80,   15,  -31, -109, -116, -127,   70,   95,   -9,    3,   87,  -63, -117,  -84,  -99,   10, -114,   38,  -43,  -78,  -23,    4,    0,
  -98,  -39,  -70,  -19,  109,  -97,   -3,  -63,   45,   31,  -66,   96,   80,   71,  117,   -8,  112,   51,  -23, -128,  -74,   99,  -74,   79,  -38,
  -73,  -81,  -40,  -37,   87,   19,  -81, -103,  -75,   67,  -38,  -78,  -84,  -61,   14,   29,  110, -122,   13,   55,  -61,   70,  -69,  -31,  -29,
   50,   37,   -3,  -46,  -41,   87,  124,   -6,  -42,  -18,   16,   78,   40, -106,   -9,   18,   -2,  -21,   37, -110,   69,  -13,  -24,  -94,  -58,
  -63,   40,  125, -105,  125, -126,  -63,   96,   34,  -93,  -52,   81,  -46,   98,   37,   22, -123, -120,   70, -105,  -47,  120,  -95,   66, -117,
  -72,  -57,  -23,   31,   31,  -70,   30,   69,   63,   68,  -64,  -98,   76, -106,  -30,    0,   97,  -64, -123,   42,  -30,  -21,   34,  117, -107,
    2,  100, -116,   49,  -88,   90,  -28,  105,  -63,  115,  -23,    3,  -91, -120,   43,    9, -108,   81,  -94,  104,  -61,   67,   56, -108,  -86,
   64,  -40,  -50,   80,  -95, -126,  106,  -40,  -20,  -85,  -68,   99, -102,  -38,   95,  -46,   50,    4,  120,   69,   93, -111,  107, -118,   33,
   67,  101,  -41,    4,   17,   44,  -95,  -65,   64,    1,  -28,  -81,  120,  -64,   74,    8,   30,  -99,  -39,   36,  -87,   83,   77,   97,   32,
 -101,  125,  -78, -102, -102, -104,   64, -127,  -85,   74, -117,  -94,   75,   21,  -84,  -92,  -59,   27,   90,  -34,    0,   23,  -88,  -92,  -55,
   53,   28,  -75,    2,  -40,   81,   89,  -78,   82,    3,   41,   21,   12,  120,   50,   26,  119,  -45,  107,  118,  112,  -89, -122,  -72,  -94,
 -119,  124,   75,   42,   36,   51,   65,   40,  -56,  -26,   68,  -96,   24,  -88,   50,  -36,   56,  -15,  -89,   18,   60,   76,   83,  -84, -113,
  -92,  103,  -48,   93,  119,   -2,  -10,  -97,  103, -114,   94,  -78,  100,  -55, -118,   39,  -98,   57,   -3,  -72,   83,   38,  -84,  118,   27,
  103,  -16,   47,  -70,  -97,   10, -128,  -75,   96,  -63, -126,  -41,   28,   -7,  -90,  -23,  -85,  -52,  -22,   89,    4,  -32,  -18, -121,  -97,
   58,   -3, -104,   57, -105,   94,  -14,  -50,    3,  -17,   -5,  -43,  -85, -114,   58,   18,   64,  119,  119,  119,  119,  119,   -9,   81,  -89,
  -67,   73,  -75,   33,   97, -119,   20, -123,  120,  -37,   52,  -68,  -21,  121,   45,   -9,   13,  -64,  -86,  -41,  -21,  -21,  118,   84,  -89,
   79,    5, -128,  -71,    7,   14,  -53,  -13,  124,  -23,  -45, -101, -114, -104,   57,  -90,  -85,  -85,  -21,  -34,   95,   46,   57,  -22,  -56,
 -122, -106,  122,   -7,  -27, -105,    7,   85,   69,   57, -118, -100,   42,   74,  -58, -126,   99,   16,   78, -126,   30,   -8, -116,   10, -112,
  -79,  -79,    6,   44,    8,  -10,   84,   80,   96,   94, -126,   74,   34,  120,  -22, -123,   29,  -25,  127,  -28,   63,   -1,  -27,  -46,  -50,
   87,   31,   53,  101,   -9,  -42,  124,  -25,  -82,  -84,  -74,  -41,   23,  121,   27,    0,   50, -108,  101,  104,  105,  -73,  -19,   29,   60,
  120, -124,    5,  -16,  -62, -102,   -2, -101,  127,   -4, -101,  111,   63,  -16,   56,    0,   50,  -82,   33,   91,  -19,  -49,   78,  -41,  -12,
   -6,  -53,  -49,  -15,  -35,   52,  -11,   10, -108,  -62,   60,   81,  -36,  -23, -117, -115, -127,    1,  -79,   16,   21,   96,   38,   41,   23,
   19,  -60, -109,  -91,   72,   32,   66,   47,  123,  -21,    1,   73,   33,   53, -120,   64,   10,  -80,    1,  -91,    8,   -4,   64,  -36,   82,
   72,   60,  -92,    0,  114,   18,  114,  -34,  -96,    0,  124,   88,   -2, -102,  -30,  -78, -122,  -43,   94,   65,  -19,   14,   22,   67,  110,
   94,   37,  -34,  -28,    7,   95, -122,  -22,  -55,   19,  102,   89, -122,   75,  -80, -107,   34,   48,   53, -124,   44,   37,   -8,   16,    0,
   90,   80,   20,   40,    0,  -94,  -28,   -5,  -50,  -96,  -80,   -7,  112,   18,  -81,   60, -110,  -74,   -2,   10,   78,  -55,  -26,  -37,  -36,
 -109,   74,  -93,   96, -128,  -80,   50,  -46,   85,  -40,   10, -128,  -63,   72,  -34,  -39,   73,  -62,   10,   22,  -59,  -80,    3,  -68,    2,
    8,  -37,   11,   12,  -80,  100, -105,   47,    8,   37,   38,  -90, -101, -107,   52,   86,   20, -104,  -76,   97,  -61,   77,  -99,   60,  -19,
  -14, -108,  106,   21,   -1,   17, -112,  -44,  -64,   82,  -62,   75,  -63,  -32,  -30, -116,   24, -106,  -55,   74,  -24,  -92,   26,   45, -110,
  113,  -97,   37,  -26,   18,  -89,  -62,   46,    7, -127,  -58,   18,  -48,  -10,  109,  125,   -9,  -35,  -69,  -76,  -69,  -69,  123,  -44,  -16,
  -87,   35, -122,   79,  -21,   83, -102,  -66, -102,  118,   76,  -20,  -69,  127,  -55,  -46,  -83,  -21,   87,    3,   56,  106,  -10, -103,   74,
   60,   -3,   89,  -78,  -96, -123,  -10, -119,   15,   94,   -7,  -95,  -45, -114, -102,  -47,  -39,  -39,  -39,  -35,  -35,  125,  -31, -123,  111,
  -20,  -39,  -75,  109,  -20,  -28, -103,   99,  -89,  -52,   86,  -30,   39,   87,  111,   90,  -71,  -12,   87,  -21,  -41,  -81,  -97,   60,   98,
 -114,   -9,  117, -126,  125,  102,  -35,  -10,  -27,   15,  -33,   11,   32,   47, -118,  -35,  123,   -5,  126,  -77,  -20,  119,  107,  -97,   93,
    1,  -96, -106,  -25,  -65,  123,  126,  -37,  -61,  -35,   15,   76,   31,   91,  -23,  -20,  -20,  124,  -12,  -87,   95,  -34, -105,   31,   -5,
  -48,    3, -117,   71,   15,  109,    1,  -16,  -48,   67,  125,   23,   94,  120,  -31, -100, -103,   83,  -61,  -58,   60, -113,   60,  -16,  -45,
  -61,   -1,  -18,   92,   48,   65,  -67, -128,  -29,   86,  -88,  106,    8,   58,  -29,  105,   21,  -47,    2,  -94, -124,   53,  -77,   96,  -62,
   35,  -75,   -3,   -3,   13,  -95, -115,  -53,  -19,  -24,   75,  -23,   64, -127,   11, -114, -100,   -3, -127,   -7,   71,   77, -101,  -47,  -18,
  -67,   -6,   92,  125,   61, -124,   16, -124,   -9, -102,  -25,  -70,  119, -105,   44,  -70,   -1,  119,  -33,   -3,  -43,   19, -101,  118,  -19,
 -117,   47,  -86,  -20,   64,    3,   63,  -90,  -41,   51,  -32,   35, -102,  103,  -69,  116,  119, -108,   75,   82,  -46,   37,  -52,  108,   64,
   21,  -74,  -28, -124,  -99,   50,  -68,   41,  -76,    8,   90,   18,   65,  -46,  -30,  -69,  -62,  -61,   23,  -48,    2,   62,   -1,  111,   69,
   10,   46,  -29,  -88,  -57,   57,  107,   64,   17,   21, -125,   22, -117,   22, -125,  -86,   67,   93,   26,   13,   53,   96,   47,  -62,  -44,
  116,  -95, -128,   70,   16,  -41,  -90, -106,   40,   43,   22,  -73,   14,   11,  -68,   28,  -63,  -90,  -42,    8,  -54,  -99,   72,   92,   53,
 -102,  107,   36,   85,  -62,  -60,   79,   77,  -54,   96,  -58,  -24,  -49,  -47,  -25,  -47,   87,  -96,  -10,  -54,  -40,   47,  -25,   15,  -92,
    8,  -27,  104,  -24,   95,   84,   66,   65,   66,   31,   26,  -40,  -27, -126, -108,   67,  -47,  117, -100,   12,   76,   12,   96,   28, -126,
  -49,   52,   25,   67, -102,   11,   97, -125,  114,  -61,  -33,   96,  -12, -120,   36,  126,   80,  -25, -101,  123,   52,    1,  113,   39,   84,
   41,  -57,    3, -107, -128,   69,   64,  -45, -118,   68,   32,  108,  -83,   25,  -31,  -81,  -36,  -11,   61,  -12,  -54,  -32,    0,   31,   55,
   32, -120,  -79,   25, -120, -126,  -40,  -91,   48,    6,    4,  -80,   18,   19,   44, -125,   20,   22,    8, -101,  -42,   25,   96,  -57, -114,
   85,   43,  -97,  -71,  123,  -12, -120,  105, -121,  -50,   58, -109,   65,   67,   29,   94, -100,   69,  -37,  -41,  -83,   90,  -15,  -32,   29,
  -83,  123,  112,  -34,  -33,  127, -128, -125,   55,   42, -111,   37,   50, -122,  -98,  -99,  -51,   43,  126,  -11,  -77,   45,  107,  -97,   33,
  -62,  -39,  -17,   -8,   16,   17,   25,   54,   22,   48, -122,  127,   -8,  -75,   79,   19,  112,  -36,  107,   46,   56,   96,  -22,   92,  102,
  -61, -122,  -65,   -9,  -91, -113, -127,  -24,  -30,   -9,   95,  -57, -116,  -25,  -97,   -3,  -19,   -3,  119,  126,   15,  -64,  -59, -105,   93,
    7,    0,  -48,  -25,  -98,  121,  -14,   -2,   59,  -65,    7,  -43,  -29,  -49, -104,   63,   97,  -14,  -84,   95,  -33,  -67,  112,  -35,  -86,
  -33, -122,   73,   -4,   -4,  127,   -8,  -16,   -6,  -43,   79,   61,  124,  -49, -113,  -58,   78,  -98,  121,  -40,   73,  -25, -118,  -88,   64,
   11,   81,   64,  -67, -118,   23,  -99,  -79,   82,   11,  -43,    2,  -16,   10,    7,  125,  126,   22,   72,   72,   73,  -87,  -11,  -28,   47,
    0,   73,    6,   40,  -55,  -29, -108,    9,  -94,  -60,  -92,   33,   67,  -50,   57,   -2, -128,   97,  -61,   42, -121,   12,  -98,   48,  -56,
  101,   79,  111,  -33,  -74,  -66,  127,  -57,  115,  -49,  -12,  -34,  -75,  108,  -51,  -66,  -94,   94,   94,   46,   77,  122,  -27,   31,   74,
   65,  -84,  -48,   70,  -66,  124,  -57,  -91,  -76,   85,   42,  -16,    0, -104, -116,    3,   91,  -53,   48,   90,  -51,   84,  -91,  -32,   66,
  -21, -126,   66,   80,   79,  -74,  106,   85,   -8,    2, -110,  -61,  -25,  127, -111,  -35,  -82,   12,  -95,  106,   81,  -75,  104,   49,   64,
 -119,   71,    4, -111,   20,   22,   25,  113,   62, -116,   13, -103,  -52, -108,  105,  -25, -113,   70,   42,  -63, -117, -126,  -35,   33,  -31,
  100,   57,  -15, -122,    0,   91,   94,  -93,   24, -123,   20, -120, -103,    2,   -1, -102,  -10,  -41,    1,  -48,  -25,  -47,   95,  -96,  -65,
  120,   69,  111,   -5,   28,   82,   19,   70,   52, -123,  -63, -118,   13,  -41,  100,  -64,  -47,  -76,  -61,  104,  -78,   32, -106, -112,   97,
  -62,   23,   76,   32, -124,   32,   51,  -27,   86,  -54,  -95,  100,  110,  -78,   54,  114,   83,   33,   17,   89,   82,   76,    8,   68, -106,
  -77,   57,   26, -105, -110,   38,  -45,  124,   66,  -46,  102,  -83,  -94,   44,   57,  126,   68, -108,  -75, -125,  -86,   16,  -41,  -50,    7,
   17,   59,    2,   19,    5,  123,  116, -120,   48,   99,  -46,  -10,  -49,   38,   69,  -95,    9,   59,  127, -103,    0,  109,    0,   19,   12,
  -63,    2, -122,  -88,   74,   24,  -18,  -16,   -4,  108,   86,  -24, -127,   43,  -79,  -41,   19,   51,   12,  -79,   37,   88,  -90,    0,   91,
  -52,  120,  102,   78,   44,  -63,  114,  -40,   22, -101,   12, -127, -103,   12,  -77,    9,   -9,   15,  123,   86,   32,   24,   90,   41,  -74,
   95, -112,   58,   67,   96,    6, -123,  -62,   75, -120,  -22,    0,  120,  -47,   16,  -77,   65,   84,  -62,  106,  104,   85,  120,   40,   68,
   10,  -15,   94,   85,    9,  -86,  -22,    5,  -45,   87,  -86,  -88,   22,  -94, -123,  -62,  -85,   20, -126,   14,  -93,  -85,  102, -127,   65,
   83,  126,   39, -106,   40, -117,   13, -109,  -58,   30, -102,    0,   43,  -68, -110,  -11,  -69,  123,  -65,  124,  -25,   74,    5, -128,  101,
   72,   10,   81,   72,  -54,   89,  108,  -29,  -46,  -24,   17,  122,  -57,  126,   97,  -85,   73,  109,  -47,  -14,  117,  -95, -119,  -79,  106,
   90,   31,  -33,  -24,   43,   76,  -92,   76,  -58, -112,   85,  101,   15,  -91,  -32,  -72,   32,  -55,  113, -108,   12, -126, -118,  -83,  127,
  -70,   62,   -8,   -1,   51,    5,  -27,  -85,    0,  -22, -118,   22, -105,   34,  -78,  107,   34,  -24,    9,   20,  100,   50, -114,   64,  -84,
  -44,   80,    3,  -29,  115,  -17,  -81,   65,   20,  -47,  -81,  -86,   89,  -10, -116,   17,   32, -109,  -72, -105,   60,   33,   75,  -83,    2,
    4,   24,   70,   95,   30,  -85,  -28,    9,    3,  -82,  127,  -59,  -90, -122,   75,   20, -112,  -20,  119,  -63,  108, -105,   66,   26,   64,
   83,  112,  -84,   56,   93,    6,  100,    1,   16,  -12,  -72,   24,  118,   51,  -22,  119,   65,   42,   42,    5,  -94,   50,    2,   32,  -94,
   23,   96, -120,    6,   67,  -63, -127,   75,  -54,  -46,  -76, -116,    3, -125,   20, -114,   22,  -32,  -28,   25,   52,  -64,  119,   53,  -30,
   95,  -71,  -50,  -76,  121,   59,   69,  106,  -14,    2,  -91,  -14,  -23,   98,   24,  102,   34,    0,  105,  -21,  -23,   64,    8,    4,  -31,
   75,   98,    5,   75,   86,   54,  -38,    1, -124,   56,   24,  -37,  106, -124,   94,   64,   45,  -84,   82,  -43,  -95,   78,  -88,   41, -123,
   -6,   -5, -124,  -78,   22,   52,  -13,  119,  -58,   50,   61,   61,    7,   18,  -73, -106, -122,   16,  -91,   29, -122,  -56,   16,   73,   80,
 -121,   41,  -58,   76,   33,   18,   66,   82,   46,   82,  109,   53,   16,   77,   68,  -94,  -86,   68,   26,  -42,  -32,   16,   11,   35,  -72,
  -34,   66,   73,   57,   -6,  -49,  -51,  124,   74,  -67,  120,   81,  -51, -119, -126,   75, -123,   16,   60,   76,  -59,  -23, -102,   57,   48,
    2,   33,  -19, -127,   -7,   -1,    0,  -74,    6,   92,  -46,   -7,   -1,   22,  103,    0,    0,    0,    0,   73,   69,   78,   68,  -82,   66,
   96, -126 };

}

